      subroutine er0090 ( filename, vector, idegfd, kprob, kprobf, &
                          kmesh )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  2.1  date 20-06-2013 New definition file ref numbers
!        version  2.0  date 27-12-2011 New parameter list
!        version  1.0  date 01-01-1998
!
!   copyright (c) 1998-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read point numbers and values from file and store in array vector
!     Real case.
!     At this moment only one degree of freedom per point is permitted
! **********************************************************************
!
!                       KEYWORDS
!
!     read_file
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) filename
      double precision vector(*)
      integer idegfd, kprob(*), kprobf(*), kmesh(*)

!     filename   i    Name of file
!     idegfd     i    Degree of freedom in VECTOR(IVEC) that must be used
!     iwork           integer work array of length NPOINT
!                     The nodal point numbers to be read are stored in iwork
!     kmesh      i    Standard SEPRAN array containing information of the mesh
!     kprob      i    Standard SEPRAN array, containing information of
!                     the problem
!     kprobf     i    KPROB part f: contains the number of unknowns per point
!                     accumulated
!     vector    i/o   Solution array or array of special structure of type 115
!                     in which the values to be read are stored
!     work            real work array of length NPOINT
!                     The values to be read are stored in work
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inunkp, indprf, numnodal, k, inodp, ipos, idum, npoint, &
              error
      integer, allocatable :: iwork(:)
      double precision, allocatable :: work(:)

!     error          Return value of allocate or deallocate
!     idum           General dummy variable
!     indprf         indication if kprob part f has been filled (>0) or not (0)
!     inodp          Nodal point number
!     inunkp         Number of degrees of freedom in first point
!     ipos           Position in array
!     k              Counting variable
!     npoint         Number of nodal points
!     numnodal       Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READFILE       Read information from an ASCII file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1672   Nodal point number out of range
!    1673   Wrong number of nodes found
!    1674   Degree of freedom not present in node
! **********************************************************************
!
!                       PSEUDO CODE
!
!    set constants
!    Read numbers from file
!    Fill values in vector
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'er0090' )

!     --- set constants

      npoint = kmesh(8)
      inunkp = kprob(4)
      indprf = kprob(19)

      allocate ( iwork(npoint), work(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Read numbers from file

      numnodal = 0
      call readfile ( filename, .false., numnodal, &
                      idum, iwork, work, 2 )
      if ( ierror/=0 ) go to 1000
      if ( numnodal>npoint ) then

!     --- Too many numbers in file

         call errint ( numnodal, 1 )
         call errint ( npoint, 2 )
         call errchr ( filename, 1)
         call errsub ( 1673, 2, 0, 1)
         go to 1000

      end if

!     --- Fill values in vector

      do k = 1, numnodal
         inodp = iwork(k)
         if ( inodp<=0 .or. inodp>npoint ) then
            call errint ( k, 1 )
            call errint ( inodp, 2 )
            call errint ( npoint, 3 )
            call errsub ( 1672, 3, 0, 0 )
            go to 1000
         end if
         if ( indprf>0 ) then

!        --- indprf>0  kprob part f contains the number of unknowns

            ipos = kprobf(inodp)
            inunkp = kprobf(inodp+1)-ipos

         else

!        --- indprf = 0

            ipos = (inodp-1)*inunkp

         end if
         if ( idegfd>inunkp ) then
            call errint ( idegfd, 1 )
            call errint ( inodp, 2 )
            call errsub ( 1674, 2, 0 , 0 )
         end if
         vector(ipos+idegfd) = work(k)

      end do

      deallocate ( iwork, work, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'er0090' )
      end
