      function erdistlinepnt ( x1, x2, xp, lambda, inside, ndim, &
                               quaddist )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.0  date 22-08-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the distance between line x1, x2 and point xp
!     Check if the normal onto the line through the point intersects
!     the line x1, x2
!     If not the distance is set to infinity
! **********************************************************************
!
!                       KEYWORDS
!
!     distance
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim
      double precision erdistlinepnt, x1(ndim), x2(ndim), xp(ndim), &
                       lambda
      logical inside, quaddist

!     erdistlinepnt  o    Computed distance (or squared distance)
!                         if intersection not between x1 and x2 infinity
!     inside         o    indicates if the perpendicular intersects
!                         between x1 and x2 (true) or not (false)
!     lambda         o    barycentric coordinate along x1 and x2 of the
!                         intersection point
!     ndim           i    Dimension of the space.
!     quaddist       i    if true the squared distance is computed
!     x1             i    coordinates of first point on line
!     x2             i    coordinates of last point on line
!     xp             i    coordinates of point outseide line from
!                         which the distance must be computed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision dx1(ndim), dxp(ndim)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     dx1            x2-x1
!     dxp            xp-x1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3140   The nodes on the segment on which you want to create a perpendicular
!           have a distance that is (almost) zero
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Line through x1 and x2:  x = x1 + lambda (x2-x1)
!     Let d be on the line x1 x2
!     Since the line through p and d should be perpendicular to x1 - x2:
!     (xp-d,x2-x1) = 0 or
!     (xp-x1 - lambda (x2-x1),x2-x1) = 0
!     So: (xp-x1,x2-x1) = lambda (x2-x1,x2-x1)
!     lambda = (xp-x1,x2-x1) / (x2-x1,x2-x1)
!     ||xp-d|| = || xp -x1 - lambda (x2-x1) ||
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'erdistlinepnt' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erdistlinepnt'
         call prinrl ( x1, ndim, 'x1' )
         call prinrl ( x2, ndim, 'x2' )
         call prinrl ( xp, ndim, 'xp' )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dx1 = x2-x1  ! difference between the two points on the line
      dxp = xp-x1  ! difference between point outside line and begin point

      lambda = dot_product(dx1,dx1)

      if ( lambda<=1d-50 ) then

!     --- Error: x2 and x1 have the same coordinates

         call errsub ( 3140, 0, 0, 0 )
         write(irefwr,2) 'x1:', x1
         write(irefwr,2) 'x2:', x2
         go to 1000

      end if

!     --- compute barycentric coordinate

      lambda = dot_product(dx1,dxp)/lambda

!     --- check if line perpendicular to x1 and x2 through x
!         is between x1 and x2

      inside = lambda>=-1d-12 .and. lambda<=1d0+1d-12

      if ( inside ) then
         dx1 = dxp-lambda*dx1
         erdistlinepnt = dot_product(dx1,dx1)
         if ( .not. quaddist ) erdistlinepnt = sqrt(erdistlinepnt)
      else
         erdistlinepnt = rinfin
      end if  ! ( inside )

1000  call erclos ( 'erdistlinepnt' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'lambda, erdistlinepnt', lambda, erdistlinepnt
         write(irefwr,*) 'inside', inside
         write(irefwr,*) 'End erdistlinepnt'

      end if  ! ( debug )

      end
