      function erdistplanepnt ( xp, x1, x2, x3, inside, quaddist )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.0  date 22-08-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the distance between triangle x1, x2, x3 and point xp
!     Check if the normal onto the triangle through the point intersects
!     the triangle
!     If not the distance is set to infinity
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     distance
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision x1(3), x2(3), x3(3), xp(3), erdistplanepnt
      logical inside, quaddist

!     erdistplanepnt o    Computed distance (or squared distance)
!                         if intersection not in triangle infinity
!     inside         o    indicates if the perpendicular intersects
!                         between x1 and x2 (true) or not (false)
!     quaddist       i    if true the squared distance is computed
!     x1             i    Contains the coordinates of the point x1
!     x2             i    Contains the coordinates of the point x2
!     x3             i    Contains the coordinates of the point x3
!     xp             i    Contains the coordinates of the point xp
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision det, v1(3), v2(3), normal(3), f(3), sum, fact1, &
                       fact2(2), eps
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     det            determinant of system of equations to be solved
!     eps            local accuracy
!     f              Right-hand-side vector of system of equations
!     fact1          Defines the factor for the intersection point
!                    The intersection point on the edgee is defined by
!                    xp1 + fact1 * (xp2-xp1)
!     fact2          Defines the factors for the intersection point
!                    on the surface
!                    The intersection point on the face is defined by
!                    x = x_1 + fact2(1)*(x_2-x_1) + fact2(2)*(x_3-x_1)
!     normal         normal on triangle
!     sum            Help parameter to compute a sum
!     v1             First column of matrix
!     v2             Second column of matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ercdet

!     ELOUTR         Compute outer product of two vectors
!     ERCDET         Compute a 3x3 determinant
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The normal through xp is defined as
!     x = xp1 - fact1 * n, where n is defined by (x2-x1) x (x3-x1)
!     The plane through (x1,x2,x3) is defined as
!     x = x1 + fact2(1) * (x2-x1) + fact2(2) * (x3-x1)
!     If an intersection is present then
!        xp1 - fact1 * n = x1 + fact2(1) * (x2-x1) + fact2(2) * (x3-x1)
!        Hence:
!        [(x2-x1) (x3-x1) n] [fact2_1 fact2_2 fact1]^T = xp1-x1
!     The system of equations is solved by Kramers rule
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'erdistplanepnt' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erdistplanepnt'
         call prinrl ( x1, 3, 'x1' )
         call prinrl ( x2, 3, 'x2' )
         call prinrl ( x3, 3, 'x3' )
         call prinrl ( xp, 3, 'xp' )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      v1 = x2-x1
      v2 = x3-x1
      f  = xp-x1
      call eloutr ( v1, v2, normal )

      eps = 1d-12

      det = ercdet ( v1, v2, normal )

      if ( debug ) then
         call prinrl ( v1, 3, 'v1' )
         call prinrl ( v2, 3, 'v2' )
         call prinrl ( f, 3, 'f' )
         call prinrl ( normal, 3, 'normal' )
         write(irefwr,2) 'det', det
      end if  ! ( debug )

      if ( abs(det)<=1d-50 ) then

!     --- determinant is 0, hence x1, x2 and x3 in a line

         fact1 = -rinfin
         fact2(1) = -rinfin
         fact2(2) = -rinfin
         inside = .false.

      else

!     --- standard case

         det = 1d0/det
         fact2(1) = det*ercdet ( f, v2, normal )
         fact2(2) = det*ercdet ( v1, f, normal )
         sum = fact2(1)+fact2(2)

         inside = fact2(1)>=-eps .and. fact2(1)<=1d0+eps .and. &
                  fact2(2)>=-eps .and. fact2(2)<=1d0+eps .and. &
                  sum>=-eps .and. sum<=1d0+eps

         if ( inside ) then

!        --- intersection point inside triangle

            fact1 = det*ercdet ( v1, v2, f )
            f = fact1*normal  ! difference between xp and intersection

            if ( debug ) then
               write(irefwr,2) 'fact1', fact1
               call prinrl ( f, 3, 'f' )
            end if  ! ( debug )
            erdistplanepnt = dot_product(f,f)
            if ( .not. quaddist ) erdistplanepnt = sqrt(erdistplanepnt)

         else

!        --- intersection point outside triangle

            fact1 = -rinfin
            erdistplanepnt = rinfin

         end if

      end if

1000  call erclos ( 'erdistplanepnt' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'fact2, erdistplanepnt', fact2, erdistplanepnt
         write(irefwr,*) 'inside', inside
         write(irefwr,*) 'End erdistplanepnt'

      end if  ! ( debug )

      end
