      subroutine ernodsurs ( kmesh, nodesurs, kmeshc, kmesht, nodes, &
                             lennodesurs )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  2.0  date 27-09-2010 Make work arrays local
!        version  1.0  date 19-07-2001
!
!   copyright (c) 2001-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Store all surfaces corresponding to a node in array nodesurs
!    Present version is 2d only
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     surface
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), kmeshc(*), kmesht(*), nodes(*), nodesurs(*), &
              lennodesurs

!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmesht         i    kmesh part t, contains element numbers of elements
!                         with highest dimensionality
!     lennodesurs   i/o   Length of array nodesurs
!                         At input estimated length
!                         At output computed length
!     nodes               Work array of length npoint containing the node
!                         numbers of a surface
!     nodesurs       o    Array containing the surfaces corresponding to the
!                         nodes.
!                         This array consists of two parts
!                         Pos. 1 to npoint contain the starting addresses
!                         of the surfaces corresponding to a specific node
!                         nodesurs(1) = npoint+1
!                         nodesurs(npoint+1) contains the length of nodesurs
!                         As a consequence the number of surfaces per node i is
!                         equal to nodesurs(i+1)-nodesurs(i)
!                         The surface numbers corresponding to node i are
!                         stored in nodesurs from positions nodesurs(i)+1 to
!                         nodesurs(i+1)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ndim, npoint, i, nnodes, isum, ih, isurnr, nsurfs, inode

!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     inode          Node sequence number
!     isum           Integer variable to compute a sum
!     isurnr         Surface sequence number
!     ndim           Dimension of the space.
!     nnodes         Number of nodes
!     npoint         Number of nodal points
!     nsurfs         number of surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSRNOD2       Store all nodes in the surfaces ifirst to ilast in array
!                    nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     419   NDIM # 2
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array nodesurs (1:npoint)
!     For isurnr := 1(1) nsurfs do
!        Store all nodes of surface isurnr in array nodes(1:nnodes)
!        For i := 1 (1) nnodes do
!           nodesurs(nodes(i)) := nodesurs(nodes(i))+1
!     isum := npoint+1
!     For i := 1 (1) npoint do
!        ih := nodesurs(i)
!        nodesurs(i) := isum
!        isum := isum+ih
!     nodesurs(npoint+1) := isum
!     Copy nodesurs(1:npoint) in iwork
!     For isurnr := 1(1) nsurfs do
!        Store all nodes of surface isurnr in array nodes(1:nnodes)
!        For i := 1 (1) nnodes do
!           inode := nodes(i)
!           iwork(inode) := iwork(inode)+1
!           nodesurs(iwork(inode)) := isurnr
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'ernodsurs' )
      ndim = kmesh(6)
      npoint = kmesh(8)
      nsurfs = kmesh(12)
      if ( ndim/=2 ) then

!     --- ndim#2

         call errint ( ndim, 1 )
         call errint ( 2, 2 )
         call errint ( 2, 3 )
         call errsub ( 419, 3, 0, 0 )

      end if

!     --- Clear array nodesurs (1:npoint)

      do i = 1, npoint
         nodesurs(i) = 0
      end do

!     --- For isurnr := 1(1) nsurfs do

      do isurnr = 1, nsurfs

!     --- Store all nodes of surface isurnr in array nodes(1:nnodes)

         call ersrnod2 ( kmesh, kmeshc, kmesht, isurnr, isurnr, nodes, &
                         nnodes )

!        --- For i := 1 (1) nnodes do
!               nodesurs(nodes(i)) := nodesurs(nodes(i))+1

         do i = 1, nnodes
            inode = nodes(i)
            nodesurs(inode) = nodesurs(inode)+1
         end do

      end do

!     --- isum := npoint+1

      isum = npoint+1

!     --- Accumulate nodesurs
!         For i := 1 (1) npoint do

      do i = 1, npoint

!     --- ih := nodesurs(i)
!         nodesurs(i) := isum
!         isum := isum+ih

         ih = nodesurs(i)
         nodesurs(i) = isum
         isum = isum+ih

      end do

!     --- nodesurs(npoint+1) := isum

      nodesurs(npoint+1) = isum
      isum = isum-1
      if ( isum>lennodesurs ) then

!     --- estimated length of nodesurs too small

         call errint ( isum, 1 )
         call errint ( lennodesurs, 2 )
         call errchr ( 'nodesurs', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if
      lennodesurs = isum

!     --- For isurnr := 1(1) nsurfs do

      do isurnr = 1, nsurfs

!     --- Store all nodes of surface isurnr in array nodes(1:nnodes)

         call ersrnod2 ( kmesh, kmeshc, kmesht, isurnr, isurnr, nodes, &
                         nnodes )

!        --- For i := 1 (1) nnodes do
!               inode := nodes(i)
!               iwork(inode) := iwork(inode)+1
!               nodesurs(iwork(inode)) := isurnr

         do i = 1, nnodes
            inode = nodes(i)
            nodesurs(inode) = nodesurs(inode)+1
            nodesurs(nodesurs(inode)) = isurnr
         end do

      end do

!     --- Recompute nodesurs positions 1 to npoint

      do i = npoint, 2, -1
         nodesurs(i) = nodesurs(i-1)
      end do
      nodesurs(1) = npoint+1

1000  call erclos ( 'ernodsurs' )
      end
