      subroutine ini086 ( ibuffr, isol, itypvc, ivec, iprob, kmesh, &
                          kprob, indprf, indprp, nphys, namear )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.10 date 04-03-2011 Change of ipkprb
!        version  1.9  date 21-11-2012 Long integers
!        version  1.8  date 28-09-2012 Remove instop
!
!   copyright (c) 1998-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     General memory management subroutine which simplifies the computation
!     of the starting addresses of a new or existing SEPRAN solution array
!     and its corresponding arrays KPROB parts f, h and p.
!     If the solution array does not exist, space is reserved in IBUFFR
!     If these arrays exist, it is checked if they are positioned in
!     in array IBUFFR (Blank common) or not.
!     If one or more of these arrays are not available
!     these array are read from file 3 and space is reserved in array IBUFFR.
!     The priority number is made equal to 10.
!     if no space is available in array IBUFFR, ini086 tries to
!     create space in the following ways:
!       -  by writing low priority arrays to backing storage
!       -  if this is not sufficient by compressing
!          array ibuffr after writing of all arrays with priority
!          smaller than 10 to backing storage
!     ini086 replaces the old subroutine INI103 or INI002
!     If array KPROB part f (or its corresponding counterpart) does exist, &
!     INDPRF gets the correct index number referring to array INFOR in
!     common block CARRAY.
!     Afterwards the starting address of KPROB part f is given by
!     INFOR(1,INDFOR)
!     In the same way INDPRP refers to KPROB part p
!     INI086 is identical to subroutine INI061 as described in the Programmer's
!     Guide, except that the parameter name is not longer used.
!     The name of the calling subroutine is extracted from common blocks
!     CMESSC and CMESSG, so it is supposed that the subroutine EROPEN has been
!     used
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons3f90'
      include 'SPcommon/cbuffrf90'
      include 'SPcommon/carrayf90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cmcdpif90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(5), itypvc, ivec, iprob, kmesh(*), kprob(*), indprf, &
              indprp, nphys, ibuffr(*)
      character *(*) namear

!     ibuffr i/o Integer buffer array in which all large data is stored
!     indprf  o  At output INDPRF contains information concerning the KPROB
!                part f corresponding to ISOL.
!                If no KPROB part f corresponds to ISOL, INDPRF = 0, otherwise
!                (INDPRF>0) the starting address of KPROB part f is given
!                by INFOR(1,INDPRF)
!     indprp  o  At output INDPRP contains information concerning the KPROB
!                part p corresponding to ISOL.
!                If no KPROB part p corresponds to ISOL, INDPRP = 0, otherwise
!                (INDPRP>0) the starting address of KPROB part p is given
!                by INFOR(1,INDPRP)
!     iprob   i  Problem number
!     isol    i  The standard SEPRAN solution array or array of special
!                structure of type 115 or 116 from which the starting positions
!                must be computed and the relevant information found.
!                Other types of arrays are not allowed
!                isol(3) must be filled before
!     itypvc  i  Type number of solution vector (110=solution vector, &
!                115=vector of special structure)
!                If negative the vector may be old and should be read back
!                from backing storage if it is at file 3
!     ivec    i  Indication of the structure of a vector of special structure
!     kmesh   i  Standard SEPRAN array containing information of the mesh
!     kprob   i  Standard SEPRAN array containing information of the problems
!                to be solved.
!     namear  i  Name of array
!     nphys   o  At output NPHYS contains the maximal number of degrees of
!                freedom per point or the number of physical variables if
!                defined
!                If ISOL is a vector of special structure and the number of
!                degrees of freedom per point is not constant then NPHYS = 0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipkprf
      integer i, ipkprb, kpivec, length, ktypvc
      logical new, debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     ipkprb         Starting address of array KPROB for actual problem
!                    number - 1
!     ipkprf         Starting address of array KPROB part f in IBUFFR
!     kpivec         Starting address of information concerning arrays of
!                    special structure in KPROB
!     ktypvc         abs(itypvc )
!     length         Length of the vector in word
!     new            If true the vector is supposed to be new
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
       integer inilen

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INI081         Create permanent space in array IBUFFR
!     INILEN         Compute length of array in IBUFFR
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     699   Problem number incorrect
!     744   Only arrays of type 110 and 115 are permitted
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute problem number (IPROB), type of solution vector (ITYPVC)
!     and for a vector of special structure the vector number (IVEC)
!     Read solution array in IBUFFR (INI070)
!     If ( ISOL is of type 110 ) then
!        if ( KPROB part f has been filled ) then
!           Set INDPRF and read KPROB part f in IBUFFR
!        if ( KPROB part p has been filled ) then
!           Set INDPRP and read KPROB part p in IBUFFR
!     else if ( ISOL is of type 115 ) then
!        if ( KPROB part f has been filled ) then
!           Set INDPRF and read KPROB part f in IBUFFR
!     else if ( ISOL is of type 116 ) then
!        compute length and nphys
!     else if ( ISOL is of type 126 ) then
!        compute length and nphys
!     else
!        wrong type of vector found: error
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ini086' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ini086'
         write(irefwr,1) 'itypvc, ivec, iprob', itypvc, ivec, iprob
         call prinin ( isol, 5, 'isol' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      indprf = 0
      indprp = 0

!     --- Compute IPROB and offset for array KPROB

      if ( iprob<1 .or. iprob>kprob(40) ) then

!     --- iprob out of range

         call errint ( iprob, 1 )
         call errint ( kprob(40), 2 )
         call errsub ( 699, 2, 0, 0 )
         go to 1000

      end if

      ipkprb = 0
      do i = 2, iprob
         ipkprb = ipkprb + kprob ( 3+ipkprb )
      end do

      ktypvc = abs(itypvc)

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ktypvc, iprob, ipkprb, ivec', &
                          ktypvc, iprob, ipkprb, ivec

      end if

      if ( ktypvc==110 ) then

!     --- ISOL is of type 110:  solution vector

         nphys = max ( kprob(ipkprb+4), kprob(ipkprb+33) )
         indprf = kprob(ipkprb+19)
         indprp = kprob(ipkprb+35)

         if ( debug ) then

!        --- Debug information

            write(irefwr,*) 'nphys, indprf, indprp ', &
                             nphys, indprf, indprp

         end if

         if ( indprf>0 ) &
            call ini060text ( ibuffr, -indprf, 'kprobf' )
         if ( indprp>0 ) &
            call ini060text ( ibuffr, -indprp, 'kprobp' )
         length = kprob(ipkprb+5)

      else if ( ktypvc==115 ) then

!     --- ISOL is of type 115:  array of special structure

         kpivec = kprob(ipkprb+18)-1+kprob(ipkprb+39)*kprob(ipkprb+6)+ &
                  ivec + ipkprb
         nphys  = kprob(kpivec)

         if ( debug ) write(irefwr,1) 'nphys', nphys
         if ( nphys<0 ) then

!        --- nphys<0, constant number of degrees of freedom per point

            nphys = -nphys
            length = nphys * kmesh(8)

         else

!        --- nphys>0, number of degrees of freedom per point not constant

            indprf = nphys
            kpivec = kprob(ipkprb+18)-1+(ivec-1)*kprob(ipkprb+39) + &
                     ipkprb
            nphys  = 0
            do i = 1, kprob(ipkprb+39)
               nphys = max ( nphys, kprob(kpivec+i) )
            end do

            if ( debug ) then

!           --- Debug information

               write(irefwr,*) 'kpivec, nphys, indprf ', &
                                kpivec, nphys, indprf

            end if

            call ini060text ( ibuffr, -indprf, 'kprobf' )
            if ( ierror/=0 ) go to 1000
            ipkprf = infor(indprf)
            length = ibuffr(ipkprf+kmesh(8))-ibuffr(ipkprf)

         end if

      else if ( ktypvc==116 ) then

!     --- ktypvc = 116, array of special structure defined per element

         nphys = ivec
         length = nphys * kmesh(9)

      else if ( ktypvc==126 ) then

!     --- ktypvc = 126, array of special structure defined per node per element

         nphys = ivec
         length = nphys * inilen(kmesh(17))

      else if ( ktypvc==127 ) then

!     --- ktypvc = 127, array defined per edge

         nphys = ivec
         length = nphys * kmesh(kmesh(48)-1+4)

      else if ( ktypvc==801 ) then

!     --- ktypvc = 801, capacity array

         length = nphys * (ivec*(ivec-1))/2

      else

!     --- Wrong type of array

         call errint ( ktypvc, 1 )
         call errsub ( 744, 1, 0, 0 )

      end if

!     --- Put ISOL into IBUFFR

      if ( itypvc<0 ) then
         new = ktypvc/=isol(2) .or. isol(5)/=length .or. &
               isol(1)<1 .or. isol(1)>iinfor
      else
         new = .true.
      end if
      isol(5) = length
      length  = length * intlen * (mod(isol(3),10)+1)
      if ( isol(2)/=ktypvc ) isol(2) = 0
      if ( new ) then

!     --- New solution array

         if ( isol(1)<0 .or. isol(1)>iinfor .or. &
              isol(2)<100 .or. isol(2)>130 ) isol(1) = 0
         call ini081 ( ibuffr, isol(1), length, namear )

      else

!     --- Old solution array

         if ( debug ) then

!        --- Debug information

            call prinin ( isol, 5, 'isol' )

         end if

         call ini060text ( ibuffr, isol(1), namear )

      end if
      isol(2) = ktypvc
      if ( ktypvc==110 ) then
         isol(4) = 1000 * (iprob-1)
      else
         isol(4) = ivec + 1000 * (iprob-1)
      end if

1000  call erclos ( 'ini086' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ini086'

      end if

      end
