      subroutine intersect2plan ( coor, nodesp1, nodesp2, t, xi, &
                                  iretur )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-11-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the intersection of two planes each defined by 3 points
!     The intersection line is defined by a point and a tangential vector
! **********************************************************************
!
!                       KEYWORDS
!
!     plane
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cmcdprf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nodesp1(3), nodesp2(3), iretur
      double precision coor(3,*), t(3), xi(3)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     iretur         o    Return value
!                         if an intersection is found 0, otherwise 1
!     nodesp1        i    Contains the node numbers of the first triangle
!     nodesp2        i    Contains the node numbers of the second triangle
!     t              o    tangential vector of intersection
!     xi             o    Coordindates of a point on intersection
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision n1(3), n2(3), lenvec, w(3), u(3), s1
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     lenvec         length of vector
!     n1             Normal to first plane
!     n2             Normal to second plane
!     s1             help parameter
!     u              work array (x2-x1), with xi the coordinates of
!                    the 3 nodes of the first triangle
!     w              work array (y1-x1), with yi the coordinates of
!                    the 3 nodes of the second triangle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELOUTR         Compute outer product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     PRNORMTRIAN    Compute the normal on a triangle
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3153   Both planes are (almost) parallel
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Let the first plane P1 be defined by the nodes nod1, nod2, nod3
!     and normal n1
!     The second plane P2 has normal n2
!     The tangential vector along the intersection line is defined as
!     t = n1 x n2
!     To find a point on the intersection line we compute the intersection
!     of the line L through nodes nod1 and nod2 with plane P2
!     Plane P2 is defined by the first point V0 of the second triangle and
!     its normal n2
!     Line L:  P(s) = N1 + s (N2-N1) (if (N2-N1,t)=0, use N1, N3)
!              P(s) - V0 = N1-V0 + s (N2-N1) must be perpendicalar to n1
!              the intersection point s1 satisfies
!              s1 = (-n2,w)/(n2,u), &
!              with w = N1-V0, u = N2-N1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intersect2plan' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intersect2plan'
         call prinin ( nodesp1, 3, 'nodesp1' )
         call prinin ( nodesp2, 3, 'nodesp2' )
  2      format ( a, 1x, (5e20.12) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute the normal n1 and n2 of both triangles

      call prnormtrian ( coor(1,nodesp1(1)), coor(1,nodesp1(2)), &
                         coor(1,nodesp1(3)), n1 )
      call prnormtrian ( coor(1,nodesp2(1)), coor(1,nodesp2(2)), &
                         coor(1,nodesp2(3)), n2 )

      if ( debug ) then
         write(irefwr,2) 'n1', n1
         write(irefwr,2) 'n2', n2
      end if  ! ( debug )

!     --- Compute the tangential vector t - n1 x n2

      call eloutr ( n1, n2, t )
      lenvec = sqrt(dot_product(t,t))

      if ( lenvec<1d3*epsmac ) then

!     --- Both planes are (almost) parallel

         call errwar ( 3153, 0, 0, 0 )
         iretur = 1  ! no intersection line
         go to 1000

      end if  ! ( abs(t)<1d3*epsmac )

      iretur = 0  ! intersection line exist
      t = t/lenvec  ! normalize t

      if ( debug ) write(irefwr,2) 't', t

!     --- Finally compute the intersection point I = P(s1)
!         let V0 be a point in plane P2
!         P(s) = N1 + s (N2-N1) (if (N2-N1,t)=0, use N1, N3)
!         P(s) - V0 = N1-V0 + s (N2-N1) must be perpendicalar to n1
!         the intersection point s1 satisfies
!         s1 = (-n2,w)/(n2,u), &
!         with w = N1-V0, u = N2-N1

      w = coor(1:3,nodesp1(1))-coor(1:3,nodesp2(1))

      u = coor(1:3,nodesp1(2))-coor(1:3,nodesp1(1))
      lenvec = sqrt(dot_product(u,u))
      u = u/lenvec  ! normalize u

      if ( debug ) then
         write(irefwr,2) 'u', u
         write(irefwr,2) 'w', w
      end if  ! ( debug )

      if ( abs(dot_product(u,t))<sqreps ) then

!     --- n2-n1 almost parallel to t use N3-n1

         u = coor(1:3,nodesp1(3))-coor(1:3,nodesp1(1))
         lenvec = sqrt(dot_product(u,u))
         u = u/lenvec  ! normalize u

      end if

      s1 = -dot_product(n2,w)/dot_product(n2,u)
      xi = coor(1:3,nodesp1(1))+s1*u  ! node on intersection line

1000  call erclos ( 'intersect2plan' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'xi', xi
         write(irefwr,2) 's1', s1
         write(irefwr,*) 'End intersect2plan'

      end if  ! ( debug )

      end
