      subroutine msh025 ( irefc, npoint, lengav, kmeshn, nsurfs, &
                          length )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.7  date 07-11-2013 Maximize number of warnings
!        version  2.6  date 30-03-2004 Debug statements
!        version  2.5  date 12-08-1999 Skip empty surfaces
!
!   copyright (c) 1984-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read kmesh part n
!
! **********************************************************************
!
!                       KEYWORDS
!
!     fill
!     mesh
!     read
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cconstf90'
      include 'SPcommon/cmcdpif90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer irefc, npoint, lengav, kmeshn(*), nsurfs, length

!     irefc    i   dataset reference number for reading (>0 formatted, &
!                  <0 unformatted)
!     kmeshn   o   Array KMESH part n, contains nodal point numbers of the
!                  surfaces
!     lengav   i   Maximal available space for array KMESHM
!     length   o   length of kmesh part n
!     npoint   i   number of nodal points
!     nsurfs   i   Number of surfaces in mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer n, inpelm, isurnr, inpsrf, l, ios, icount
      logical detect, found, debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     detect         If true an negative point has been detected in the surface
!     found          if true an empty surface has been found
!     icount         counter
!     inpelm         number of points per element
!     inpsrf         number of elements in surface
!     ios            IOSTAT variable, returns from reading or writing
!     isurnr         surface number
!     l              Summation variable
!     n              number of nodes in surface (inpelm*inpsrf)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INSTOP         Stop the program
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     258   inpelmsurf has incorrect value
!     259   error found in reading of nodal point numbers of surfaces
!     260   end of file found in reading of nodal point numbers of surfaces
!     261   nodal point number in surface has incorrect value
!     585   Surface has not been filled
!    1381   Too few space available in array KMESHN
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh025' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh025'

      end if
      if ( ierror/=0 ) go to 1000
      length = 2*nsurfs

      found = .false.
      icount = 0
      do isurnr = 1, nsurfs

         detect =.false.
         if ( irefc>=0 ) then
            read(irefc, *, iostat=ios ) inpsrf, inpelm
         else
            read(-irefc, iostat=ios ) inpsrf, inpelm
         end if
         if ( debug ) write(irefwr,*) 'inpsrf, inpelm, ios ', &
            inpsrf, inpelm, ios

         if ( ios<0 ) then
            go to 900
         else if ( ios>0 ) then
            go to 901
         end if

         if ( inpelm<0 .or. inpelm>1000 ) then
            call errint ( inpelm , 1 )
            call errint ( 1000, 2 )
            call errsub ( 258, 2, 0, 0 )
            go to 1000
         end if

         n = inpelm*inpsrf

         if ( length+n>lengav ) then

!        --- Error 1381: Not enough space available

            call errint ( length+n, 1 )
            call errint ( lengav, 2 )
            call errsub ( 1381, 2, 0, 0 )
            go to 1000

         end if

         kmeshn(isurnr) = inpelm
         kmeshn(isurnr+nsurfs) = inpsrf
         if ( n>0 ) then

!        --- N>0, read kmeshn

            if ( irefc>=0 ) then
               read(irefc, *, iostat=ios ) (kmeshn(length+l), &
                    l = 1, n)
            else
               read(-irefc, iostat=ios ) (kmeshn(length+l), &
                    l = 1, n)
            end if

         else

!        --- N = 0, surface has not been filled

            icount = icount+1
            call errint ( isurnr, icount )
            found = .true.

         end if
         if ( debug ) then
            write(irefwr,*) 'isurnr, ios ', isurnr, ios
            call prinin ( kmeshn(length+1), inpelm, 'first element' )
            call prinin ( kmeshn(length+n-inpelm+1), inpelm, &
                         'last element' )
         end if
         if ( ios<0 ) then
            go to 900
         else if ( ios>0 ) then
            go to 901
         end if
         do l = 1, n
            if ( kmeshn(length+l)<=0 .or. kmeshn(length+l)>npoint ) then
            if ( kmeshn(length+l)<=0 ) then
               if ( .not. detect ) then
                  icount = min(1000,icount+1)
                  call errint ( isurnr, icount )
                  detect = .true.
                  found = .true.
               end if
            else
               call errint ( kmeshn(length+l), 1 )
               call errint ( npoint, 2 )
               call errint ( isurnr, 3 )
               call errsub ( 261, 3, 0, 0 )
            end if
         end if

         end do
         length = length+n
      end do
      if ( found ) call errwar ( 585, icount, 0, 0)
      go to 1000

!     --- read errors:

900   call errint ( ios, 1 )
      call errsub ( 259, 1, 0, 0 )
      go to 1000
901   call errsub ( 260, 0, 0, 0 )

1000  call erclos ( 'msh025' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh025'

      end if

      if ( ierror/=0 ) call instop

      end
