      subroutine prextractunk ( ibuffr, buffer, isol, kprob, kmesh, &
                                jdegfd, isolcp, newvec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Extract degree of freedom jdegfd from the vector corresponding
!     to isol and store in the vector corresponding to isolcp
!     if isol has only one unknown per point, isolcp is identical to isol
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     vector
!     extract
!     extract_unknown
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: jdegfd, isol(5), kprob(*), kmesh(*)
      integer, intent(inout) :: ibuffr(*)
      integer, intent(out) :: isolcp(5)
      double precision, intent(inout) :: buffer(*)
      logical, intent(out) :: newvec

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     isolcp         o    Standard SEPRAN array containing information of the
!                         new solution array with only one unknown per point
!     jdegfd         i    Sequence of degree of freedom to be extracted
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     newvec         o    If true isolcp is a new vector otherwise
!                         isolcp contains exactly isol, which means that
!                         removing the array corresponding to isolcp
!                         also implies removing the array corresponding to isol
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: indprf, indprp, nunkp, nphys, itypvc, npoint, ivec, &
                 iprob
      integer (kind=8) :: ipsolcp
      logical :: debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     indprf         indication if kprob part f has been filled (>0)
!                    or not (0)
!     indprp         Indication if array kprob part p has been filled (1)
!                    or not (0)
!     iprob          Actual problem number
!     ipsolcp        Starting address of array usolcp
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     nphys          Number of physical unknowns, or maximal degrees of
!                    freedom per point
!     npoint         Number of nodal points
!     nunkp          Maximum number of unknowns in a point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: inidgttext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI061         compute pointers to degrees of freedom of vectors
!     INI080         Create permanent space in array BUFFER
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INISTRNEW      find structure of vector
!     PRCOPSEPVEC    Copy one degree of freedom of sepran vector in user vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prextractunk' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prextractunk'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First activate isol and compute information

      call ini061 ( ibuffr, isol, kprob, indprf, indprp, &
                    nunkp, nphys )

      if ( nunkp>1 .and. jdegfd>0 ) then

!     --- Create new vector from old vector by extracting degree of freedom

         newvec = .true.

!        --- Find structure of vector with exactly 1 parameters per node

         itypvc = 110
         iprob  = isol(4)/1000+1  ! problem number
         call inistrnew ( 1, kprob, kmesh, itypvc, ivec, iprob )
         if ( ierror/=0 ) go to 1000

!        --- Creat new vector

         npoint = kmesh(8)
         call ini080 ( ibuffr, isolcp(1), npoint, 'isolcp' )

         if ( ivec==0 ) then
            isolcp(2) = 110
         else
            isolcp(2) = 115
         end if  ! ( ivec==0 )
         isolcp(3) = 0
         isolcp(4) = ivec+1000*(iprob-1)
         isolcp(5) = npoint
         ipsolcp = inidgttext ( isolcp(1), 'isolcp' )

!        --- Copy degree of freedom

         call prcopsepvec ( ibuffr, buffer, isol, kprob, kmesh, &
                            jdegfd, buffer(ipsolcp) )

      else

!     --- Copy vector into isolcp

         isolcp(1:5) = isol(1:5)
         newvec = .false.

      end if  ! ( nunkp>1 .and. jdegfd>0 )

1000  call erclos ( 'prextractunk' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prextractunk'

      end if  ! ( debug )

      end subroutine prextractunk
