      subroutine prfillnodelms ( ibuffr, inpsep, kmesh, nodes, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 21-11-2012 Long integers
!        version  1.1  date 10-02-2012 use getnpelmelgrp
!        version  1.0  date 03-01-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes for set of elements
!
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
!     element
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), kmesh(*), nodes(*), nnodes, ibuffr(*)

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine prfillnodes
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     nnodes        i/o   Number of nodes
!     nodes         i/o   Contains the node numbers that must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipkmsc
      integer error, npoint, ifirst, last, numnod, ifirgr, nelgrp, &
              ielgrp, inelem, lastgr, istart, inpelm, jnpelm, &
              ifelem
      integer, allocatable :: iwork(:)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     error          Return value of allocate or deallocate
!     ielgrp         element group number
!     ifelem         First element number
!     ifirgr         First element group number
!     ifirst         First element number
!     inelem         Number of elements in a specific element group
!     inpelm         number of points per element
!     ipkmsc         Starting address of array KMESH part c
!     istart         First position used at a part
!     iwork          integer work array of length npoint
!                    iwork(i) = 0, means node i has not yet been filled
!                    iwork(i)>0, means node i has been filled
!                    The value defines the new node number
!     jnpelm         Number of nodes in an element
!     last           Last number to be read from inpsep
!     lastgr         Last element group number
!     nelgrp         Number of element groups
!     npoint         Number of nodal points
!     numnod         Number of nodes that should be used locally in an element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext
      integer getnumelmelgrp, getnpelmelgrp

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     ERFILLNODSKMC1 Fill array nodes for elements 1 to inelem
!     ERFILLNODSKMC2 Fill array nodes for elements 1 to inelem
!                    Special case oly a restricted number of nodes per element
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     GETNPELMELGRP  Fetch number of points per element in element group ielgrp
!     GETNUMELMELGRP Determine number of elements in element group
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIGETTEXT     Compute pointer of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1123   The elements in which the vector must be created do not all have
!           the same number of nodal points per element, whereas the vector must
!           be created in only one call.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillnodelms' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillnodelms'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      nelgrp = kmesh(5)
      npoint = kmesh(8)

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork = 0

      nnodes = 0

!     --- Get parameters from inpsep

      ifirst = inpsep(1)
      last   = inpsep(2)
      numnod = inpsep(3)

!     --- Set first and last group as well as first element in first group

      ifirgr = 1
      lastgr = nelgrp
      ifelem = ifirst

      if ( nelgrp>1 ) then

!     --- More than one element group, find range of element groups

         inelem = 0

!        --- Loop over element groups

         do ielgrp = 1, nelgrp

            inelem = inelem + getnumelmelgrp ( kmesh, ielgrp )

            if ( inelem<ifirst ) then

!           --- inelem<ifirst, so first group is at least ielgrp + 1

               ifirgr = ielgrp + 1
               ifelem = ifirst - inelem

            end if  ! ( inelem<ifirst )

            if ( inelem<last ) then

!           --- inelem<last, so last group is at least ielgrp + 1

               lastgr = ielgrp + 1

            else if ( inelem==last ) then

!           --- inelem = last, so last group is ielgrp

               lastgr = ielgrp

            end if  ! ( inelem<last )

         end do  ! ielgrp = 1, nelgrp

      end if  ! ( nelgrp>1 )

!     --- Compute starting value istart

      istart = 0
      do ielgrp = 1, ifirgr-1
         inelem = getnumelmelgrp ( kmesh, ielgrp )
         inpelm = getnpelmelgrp ( kmesh, ielgrp )
         istart = istart + inelem * inpelm
      end do
      inpelm = getnumelmelgrp ( kmesh, ifirgr )
      istart = istart + inpelm * (ifelem-1)

!     --- Check inpelm (must be equal for all elment groups to be considered)

      do ielgrp = ifirgr+1, lastgr

         jnpelm = getnumelmelgrp ( kmesh, ielgrp )
         if ( inpelm/=jnpelm ) then
            call errint ( inpelm, 1 )
            call errint ( jnpelm, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 1123, 3, 0, 0 )
         end if

      end do
      if ( ierror/=0 ) go to 1000

      inelem = last - ifirst + 1

!     --- activate kmesh part c

      call ini060text ( ibuffr, -kmesh(17), 'kmeshc' )

!     --- Compute starting address

      ipkmsc = inigettext ( kmesh(17), 'kmeshc' )
      if ( ierror/=0 ) go to 1000

      if ( numnod==0 ) then

!     --- numnod = 0, fill all nodes per element

         call erfillnodskmc1 ( nodes, nnodes, ibuffr(ipkmsc+istart), &
                               inelem, inpelm, iwork )

      else

!     --- numnod>0, fill some nodes per element

         call erfillnodskmc2 ( nodes, nnodes, ibuffr(ipkmsc), inelem, &
                               inpelm, iwork, numnod, inpsep(4), &
                               istart )

      end if  ! ( numnod==0 )

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'prfillnodelms' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfillnodelms'

      end if  ! ( debug )

      end
