      subroutine prfillnodvolms ( ibuffr, kmesh, inum, inpsep, &
                                  nodes, nnodes, iplace )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 21-11-2012 Long integers
!        version  2.0  date 03-04-2012 Extra parameter iplace
!        version  1.0  date 23-12-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes with nodal points as defined in array inpsep
!     Special case: nodes in volumes
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), kmesh(*), inpsep(*), nodes(*), nnodes, inum, &
              iplace

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     inpsep         i    The integer input read is stored in array INPSEP
!                         The input depends in inum
!                         if inum>0, inpsep(1:inum) contains inum
!                         volume numbers
!                         if inum = 0 then
!                         Pos. 1 indicates how array inpsep is filled.
!                         Possible values:
!                          0   only one volume is used. The
!                              volume number is stored in icurvs(2)
!                         -1   The solution is printed along the
!                              volumes icrv1 to icrv2. icrv1 is stored in
!                              icurvs(2) and icrv2 in icurvs(3).
!                         n>0  The volume numbers are stored
!                              in icurvs(2) to icurvs(n+1).
!                              That sequence is used.
!     inum           i    See inpsep
!     iplace         i    Position where nodes must be filled
!                         See subroutine prfillnodes
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     nnodes         o    Number of nodes
!     nodes          o    Contains the node numbers that must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipkmsc
      integer i, error, nvolumes, ivlm1, ivlm2, nvolms, numnod
      integer, allocatable :: volumes(:)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     error          Return value of allocate or deallocate
!     i              Counting variable
!     ipkmsc         Starting address of array KMESH part c
!     ivlm1          First volume number
!     ivlm2          Last volume number
!     numnod         If>0, only numnod nodes per element must be used
!                    In that case ihelp is used
!                    If 0 all nodes are used
!     nvolms         number of volumes
!     nvolumes       Number of volumes in array volumes
!     volumes        Contains volume numbers
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERVOLMSNOD     Store all nodes in the volumes stored in array volumes
!                    in array nodes
!     ERVOLMSNOD1    Store all nodes in the volumes stored in array volumes
!                    in array nodes
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIGETTEXT     Compute pointer of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     296   first volume number has incorrect value
!     297   last volume number has incorrect value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillnodvolms' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillnodvolms'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( kmesh(28)>0 ) &
         call ini060text ( ibuffr, -kmesh(28), 'kmeshn' )
      call ini060text ( ibuffr, -kmesh(17), 'kmeshc' )
      ipkmsc = inigettext ( kmesh(17), 'kmeshc' )

      if ( iplace==25 ) then

!     --- iplace = 25, series of volumes with respect to create

         numnod = inpsep(2)
         if ( numnod==0 ) then

!        --- Standard case, all nodes in volumes

            call ervolmsnod ( kmesh, inpsep(1), nodes, nnodes, &
                              inpsep(3), kmesh(kmesh(36)), &
                              ibuffr(ipkmsc) )

         else

!        --- Special case, restricted set of nodes in volumes

            call ervolmsnod1 ( kmesh, inpsep(1), nodes, nnodes, &
                               inpsep(3), kmesh(kmesh(36)), &
                               ibuffr(ipkmsc), numnod, &
                               inpsep(3+inpsep(1)) )

         end if  ! ( numnod==0 )

      else if ( inum>0 ) then

!     --- inum>0, Use inpsep

         call ervolmsnod ( kmesh, inum, nodes, nnodes, &
                           inpsep, kmesh(kmesh(36)), ibuffr(ipkmsc) )

      else

!     --- inum = 0, inpsep contains old array icurvs

         nvolms = inpsep(1)
         if ( nvolms==0 ) nvolms = 1
         if ( nvolms>0 ) then

!        --- inpsep(1)>= 0, volume numbers stored from position 2

            call ervolmsnod ( kmesh, nvolms, nodes, nnodes, &
                              inpsep(2), kmesh(kmesh(36)), &
                              ibuffr(ipkmsc) )

         else

!        --- inpsep(1)<0, volume numbers from inpsep(2 to 3)

            nvolms = kmesh(13)
            ivlm1 = inpsep(2)
            ivlm2 = inpsep(3)
            if ( ivlm1<=0 .or. ivlm1>nvolms ) then
               call errint ( ivlm1, 1 )
               call errint ( nvolms, 2 )
               call errsub ( 296, 2, 0, 0 )
               ivlm1 = 1
            end if
            if ( ivlm2<=ivlm1 .or. ivlm2>nvolms ) then
               call errint ( ivlm2, 1 )
               call errint ( ivlm1, 2 )
               call errint ( nvolms, 3 )
               call errsub ( 297, 3, 0, 0 )
               ivlm2 = ivlm1
            end if
            nvolumes = ivlm2-ivlm1+1
            allocate ( volumes(nvolumes), stat = error )
            if ( error/=0 ) &
               call eralloc ( error, nvolumes, 'volumes' )
            if ( ierror/=0 ) go to 1000

            do i = 1, nvolumes
               volumes(i) = ivlm1+i-1
            end do  ! i = 1, nvolumes

            call ervolmsnod ( kmesh, nvolumes, nodes, nnodes, &
                              volumes,kmesh(kmesh(36)), ibuffr(ipkmsc) )

            deallocate ( volumes, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'volumes' )

         end if  ! ( ichoice>0 )

      end if  ! ( inum>0 )

1000  call erclos ( 'prfillnodvolms' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfillnodvolms'

      end if  ! ( debug )

      end
