      subroutine prvecfind1 ( ibuffr, kmesh, kprob, iprob, ivec, &
                              itypvc, iseqvec, vecs, isol, numvecrhs, &
                              complexvec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-02-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find the type of vector to be created for the left-hand side
!     This depends on the right-hand side
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     expression
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cpost5f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), kmesh(*), iprob, ivec, itypvc, &
              kprob(*), iseqvec, vecs(*), isol(5,*), numvecrhs
      logical complexvec

!     complexvec     o    If true the vector to be filled is complex
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     iprob          o    Actual problem number
!     iseqvec        i    Starting address of sequence numbers of vectors
!     isol           i    Standard SEPRAN array containing information of the
!                         solution arrays
!     itypvc         o    Indication of the type of solution vector:
!                         110: solution vector
!                         115, 116: vectors of special structure
!     ivec           o    Sequence number of vector of special structure
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     numvecrhs      i    Number of vectors in vecs
!     vecs           i    In this array of size numvecrhs the sequence numbers
!                         of the right-hand side vectors are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ndim, npoint, lenvec, iseqv, length, i, nunkp, lenpcomp, &
              lenpcomp2, nphys, iseqfinal, indprf, indprp, nunkp2
      character (len=32) name1, name2, nameiseqv
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     indprf         indication if kprob part f has been filled (>0)
!                    or not (0)
!     indprp         Indication if array kprob part p has been filled (1)
!                    or not (0)
!     iseqfinal      Sequence number of vector in isol that must be used
!                    to define the type of vector
!     iseqv          Sequence number of vector in isol
!     length         length of array in isol
!     lenpcomp       Length of solution array for one component
!     lenpcomp2      Length of solution array for different array
!                    (one component)
!     lenvec         Length of left-hand side vector in expression
!     name1          Name of first vector in rhs
!     name2          Name of second vector in rhs
!     nameiseqv      Name of vector inlrhs
!     ndim           Dimension of the space.
!     nphys          Number of physical unknowns, or maximal degrees of
!                    freedom per point
!     npoint         Number of nodal points
!     nunkp          Maximum number of unknowns in a point
!     nunkp2         maximum number of unknowns per point for different
!                    vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERROR3161      Print error message 3161
!     INI061         compute pointers to degrees of freedom of vectors
!     INISTREXT      Find the structure of the vector with exactly
!                    NUMBER unknowns per point
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prvecfind1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prvecfind1'
         write(irefwr,1) 'iseqvec', iseqvec
         call prinin1 ( isol, 4, 5, 'isol' )
  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ndim = kmesh(6)    ! dimension of space
      npoint = kmesh(8)  ! number of nodes
      complexvec = .false.

      if ( debug ) &
         write(irefwr,1) 'npoint, ndim', npoint, ndim

!     --- check the vectors in the
!         right-hand side of the expression

      if ( debug ) call prinin ( vecs, numvecrhs, 'vectors' )
      iseqv = vecs(1)
      if ( iseqvec<=maxvc1 ) then
         nameiseqv = names(iseqvec)
      else
         nunkp = 1
         select case (iseqvec-maxvc1)
            case(1)
               nameiseqv = 'coor'
               nunkp = ndim
            case(2)
               nameiseqv = 'x_coor'
            case(3)
               nameiseqv = 'y_coor'
            case(4)
               nameiseqv = 'z_coor'
         end select  ! case (iseqvec)
         iseqfinal = iseqvec
      end if  ! ( iseqvec<=maxvc1 )

      if ( iseqv<=maxvc1 ) then

!     --- The first vector is an existing vector

         call ini061 ( ibuffr, isol(1,iseqv), kprob, indprf, indprp, &
                       nunkp, nphys )
         lenvec = isol(5,iseqv)
         lenpcomp = lenvec/nunkp
         iseqfinal = iseqv
         name1 = names(iseqv)
         complexvec = mod(isol(3,iseqv),10)==1
         itypvc = isol(2,iseqv)
         iprob = isol(4,iseqv)/1000+1

         if ( debug ) &
            write(irefwr,1) 'iseqv, iprob, ivec, itypvc', &
                             iseqv, iprob, ivec, itypvc

      else if ( iseqv==maxvc1+1 ) then

!     --- The first vector is coor

         lenvec = npoint*ndim
         lenpcomp = npoint
         nunkp = ndim
         name1 = 'coor'
         iseqfinal = iseqv

      else if ( iseqv<=maxvc1+1+ndim ) then

!     --- The first vector is x_coor (or y or z)

         lenvec = npoint
         lenpcomp = npoint
         nunkp = 1
         select case(iseqv-maxvc1-1)
         case(1)
            name1 = 'x_coor'
         case(2)
            name1 = 'y_coor'
         case(3)
            name1 = 'z_coor'
         end select  ! case(iseqv-maxvc1-1)
         iseqfinal = iseqv

      end if  ! ( iseqvec<=maxvc1 )

      if ( debug ) then
         write(irefwr,1) 'iseqv, lenvec, lenpcomp, nunkp', &
                          iseqv, lenvec, lenpcomp, nunkp
         write(irefwr,*) 'name1 ', name1
      end if  ! ( debug )

!     --- Check if the rest of the vectors have the same length
!         If necessary define new length

      do i = 2, numvecrhs
         iseqv = vecs(i)
         if ( iseqv<=maxvc1 ) then

!        --- The first vector is an existing vector

            call ini061 ( ibuffr, isol(1,iseqv), kprob, indprf, &
                          indprp, nunkp2, nphys )
            length = isol(5,iseqv)
            lenpcomp2 = length/nunkp2

            if ( debug ) &
               write(irefwr,1) 'i, iseqv, length, lenvec', &
                                i, iseqv, length, lenvec

            if ( length/=lenvec ) then

!           --- The length of both vectors is different

               if ( lenpcomp==lenpcomp2 ) then

!              --- Per component we have the same length

                  if ( nunkp2>nunkp .and. nunkp==1 ) then

!                 --- New length found, previous one was 1

                     nunkp = nunkp2
                     lenvec = length
                     iseqfinal = iseqv
                     name1 = names(iseqv)

                  else if ( nunkp2/=1 ) then

!                 --- The new vector has different length than the previous
!                     one

                     name2 = names(iseqv)
                     call error3161 ( lenvec, length, name1, name2, &
                                      nameiseqv )
                     go to 1000

                  end if  ! ( nunkp2>nunkp .and. nunkp==1 )

               else

!              --- The new vector has different length than the previous
!                  one

                   name2 = names(iseqv)
                   call error3161 ( lenvec, length, name1, name2, &
                                    nameiseqv )
                   go to 1000

               end if  ! ( lenpcomp==lenpcomp2 )

            end if  ! ( length/=lenvec )

            complexvec = mod(isol(3,iseqv),10)==1 .or. complexvec

         else if ( iseqv==maxvc1+1 ) then

!        --- The first vector is coor

            length = npoint*ndim
            name2 = 'coor'

            if ( length/=lenvec ) then

!           --- The length of both vectors is different

               if ( lenpcomp==npoint ) then

!              --- Per component we have the same length

                  if ( ndim>nunkp .and. nunkp==1 ) then

!                 --- New length found, previous one was 1

                     nunkp = ndim
                     lenvec = length
                     iseqfinal = iseqv
                     name1 = 'coor'

                  else if ( ndim/=1 ) then

!                 --- The new vector has different length than the previous
!                     one

                     call error3161 ( lenvec, length, name1, name2, &
                                      nameiseqv )
                     go to 1000

                  end if  ! ( nunkp2>nunkp .and. nunkp==1 )

              else

!             --- The new vector has different length than the previous
!                 one

                  call error3161 ( lenvec, length, name1, name2, &
                                   nameiseqv )
                  go to 1000

               end if  ! ( lenpcomp==lenpcomp )

            end if  ! ( length/=lenvec )

         else if ( iseqv<=maxvc1+1+ndim ) then

!        --- The first vector is x_coor (or y or z)

            length = npoint
            select case(iseqv-maxvc1-1)
            case(1)
               name2 = 'x_coor'
            case(2)
               name2 = 'y_coor'
            case(3)
               name2 = 'z_coor'
            end select  ! case(iseqv-maxvc1-1)

            if ( length/=lenvec ) then

!           --- The length of both vectors is different

               if ( lenpcomp==npoint ) then

!              --- Per component we have the same length

                  if ( ndim>nunkp .and. nunkp==1 ) then

!                 --- New length found, previous one was 1

                     nunkp = 1
                     lenvec = length
                     iseqfinal = iseqv
                     name1 = name2

                  else

!                 --- The new vector has different length than the previous
!                     one

                     call error3161 ( lenvec, length, name1, name2, &
                                      nameiseqv )
                     go to 1000

                  end if  ! ( nunkp2>nunkp .and. nunkp==1 )

               else

!              --- The new vector has different length than the previous
!                  one

                   call error3161 ( lenvec, length, name1, name2, &
                                    nameiseqv )
                   go to 1000

               end if  ! ( lenpcomp==lenpcomp )

            end if  ! ( length/=lenvec )

         end if  ! ( iseqvec<=maxvc1 )

         if ( debug ) write(irefwr,1) 'length', length

      end do  ! i = 2, numvecrhs

!     --- define iprob etc.

      if ( debug ) write(irefwr,1) 'iseqfinal', iseqfinal

      if ( iseqfinal<=maxvc1 ) then

!     --- standard array

         iprob = isol(4,iseqfinal)/1000+1
         ivec = mod(isol(4,iseqfinal),1000)
         itypvc = 110
         if ( ivec>0 ) itypvc = 115

      else

!     --- Coordinates array

         call inistrext ( nunkp, kprob, kmesh, itypvc, ivec, iprob )

      end if  ! ( iseqfinal<=mavc1 )

1000  call erclos ( 'prvecfind1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'complexvec ', complexvec
         write(irefwr,1) 'iprob, ivec, itypvc', iprob, ivec, itypvc
         write(irefwr,*) 'End prvecfind1'

      end if  ! ( debug )

      end
