      subroutine readreal ( openfound, scalpermitted )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 12-02-2015 New call to readexpress
!        version  2.0  date 10-01-2015 Extra parameters scalpermitted
!        version  1.7  date 06-05-2014 jump in case of closing bracket
!
!   copyright (c) 2008-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read a real or real expression from the input file
!     scalars are not permitted
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/cexpressionf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical, intent(in) :: scalpermitted
      logical, intent(inout) :: openfound

!     openfound     i/o   Indicates if an openings bracket is found before
!                         reading the number or expression
!     scalpermitted  i    If true the expression may contain scalars, &
!                         if false not
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lastitemsav, kmesh(9), lastrealexprsv, numexpresssv
      logical debug, closefound, vecfound

!     closefound     If true a closing bracket has been found, which
!                    corresponds to a closing bracket read earlier
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     kmesh          Dummy
!     lastitemsav    Help parameter to save the value of lastitem
!     lastrealexprsv Help parameter to save the value of lastrealexpr
!     numexpresssv   Help parameter to save the value of numexpress
!     vecfound       If true the expression contains vectors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PREVALEXPR     Evaluate expression and fill actual values of scalars
!     PRININ         print 1d integer array
!     READEXPRESS    Interprete part of input as number or expression
!     READREALSCAL   Read integer, real or scalar
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readreal' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readreal'
         write(irefwr,*) 'openfound ', openfound
         write(irefwr,1) 'lastitem', lastitem
         call prinin ( typeitems, nitems, 'typeitems' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kmesh = 0

      if ( lastitem>=nitems ) then

!     --- End of record reached

         ityprd = 1
         go to 1000

      end if  ! ( lastitem>=nitems )

!     --- Skip = or , &

      if ( debug ) &
         write(irefwr,1) 'typeitems(lastitem+1)', typeitems(lastitem+1)

      if ( typeitems(lastitem+1)==6 ) then

!     --- Closing bracket found: return with ityprd = 7

         lastitem = lastitem+1
         ityprd = 7
         go to 1000

      end if  ! ( typeitems(lastitem+1)==6 )

      if ( typeitems(lastitem+1)==13 .or. &
           typeitems(lastitem+1)==26 ) lastitem = lastitem+1

      if ( debug ) &
         write(irefwr,1) 'lastitem, typeitems(lastitem+1)', &
                          lastitem, typeitems(lastitem+1)

!     --- First check if we are dealing with a scalar or a real expression

      if ( typeitems(lastitem+1)==10 ) then

!     --- % found, read scalar

         call asknxt
         if ( debug ) &
            write(irefwr,1) 'after %: ityprd, lastitem', &
                                      ityprd, lastitem
         go to 1000

      else if ( typeitems(lastitem+1)==3 ) then

!     --- String found, check if it is a scalar or a vector

         lastitemsav = lastitem
         call readrealscal
         if ( debug ) write(irefwr,1) &
            'after readstrscals: ityprd, lastitem, lastitemsav', &
                                 ityprd, lastitem, lastitemsav
         if ( ityprd==11 .or. ityprd==14 ) then

!        --- scalar or constant found, jump

            go to 1000

         else if ( ityprd==102 ) then

!        --- vector found

            ityprd = 2
            go to 1000

         else

!        --- no scalar found, reset lastitem

            lastitem = lastitemsav

         end if  ! ( ityprd==11 .or.ityprd==14 )

      end if  ! ( typeitems(lastitem+1)==10 )
      closefound = .false.

      lastrealexprsv = lastrealexpr
      numexpresssv = numexpress
      call readexpress ( openfound, closefound, scalpermitted, .false., &
                         .false., vecfound )

      if ( debug ) then
         write(irefwr,*) 'openfound, closefound ', &
                          openfound, closefound
      end if  ! ( debug )

      if ( ierror/=0 ) go to 1000

      if ( ityprd==19 .and. .not. scalpermitted ) then

!     --- Transform to number and remove information from cexpression

         call prevalexpr ( numexpress, realrd, kmesh, cardim(1:lastp) )
         numexpress = numexpresssv
         lastrealexpr = lastrealexprsv
         ityprd = 3

      end if  ! ( lenstore==1 )
      if ( debug ) write(irefwr,2) 'realrd', realrd

      if ( ityprd==3 .and. abs(realrd)<1d7 ) then

!     --- Real less than 10000000, check if it is an integer

         if ( abs(realrd)<=0d0 ) then

!        --- real = 0

            ityprd = 2

         else if ( abs(realrd)>=0.8d0 ) then

!        --- real = 0

            if ( abs(realrd-nint(realrd))<1d-8*abs(realrd) ) ityprd = 2

         end if  ! ( abs(realrd)<=0d0 )

      end if  ! ( abs(realrd)<1d7 )

1000  call erclos ( 'readreal' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
         write(irefwr,2) 'realrd', realrd
         write(irefwr,*) 'End readreal'

      end if  ! ( debug )

      end
