      subroutine readrecord ( iretur )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 11-07-2012 Use & as continuation
!        version  1.3  date 06-10-2006 Allow empty card if ispecl = 3
!        version  1.2  date 21-03-2005 Allow ! as comment
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read one record from the input file and store in array cardimage
!     If the record is blank or comment only it is skipped
!     If the record ends with // or & the next one is also read
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     record
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cseprdf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/csepchnewf90'
      include 'SPcommon/casciif90'
      include 'SPcommon/cmachtf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iretur

!     iretur         i    Return value of subroutine
!                         Possible values:
!                         <0:  end of file has been reached
!                         >0:  error has been found
!                          0:  normal return
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical finished, varunix, warn2439
      integer ios, j, ifirst, ilast, ih
      character (len=1) tab, controlm

!     controlm       Control m character
!     finished       Indicates if the loop has been finished (true) or not
!                    (false)
!     ifirst         First curve (surface) number read
!     ih             Help variable to store a constant temporarily
!     ilast          Last curve (surface) number read
!     ios            IOSTAT variable, returns from reading or writing
!     j              Counting variable
!     tab            Horizontal tab character
!     varunix        If true the operating system is unix
!     warn2439       If true warning 2439 has already been printed
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    2439   dos input file in unix environment
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Set tab character
!     numrecstored := 0
!     finished := false
!     while ( not finished ) do
!        read record in cards
!        if ( end_of_file ) then
!           iretur := -1
!           finished := true
!        else if ( error ) then
!           iretur := iostatus
!           finished := true
!        else ( regular record found )
!           Replace tab by blanks
!           irecnr := irecnr+1
!           ifirst := first non blank character
!           lastp := last non blank character
!           ilast  := last non-blank for comment character #
!           if ( first character = * ) then
!              print record if necessary
!           else if ( record empty ) then
!              print record if necessary
!           else ( record contains information ) then
!              numrecstored := numrecstored+1
!              cardimage(numrecstored) := cards
!              firstcolumn(numrecstored) := ifirst
!              if ( last two characters are // ) then
!                 lastcolumn(numrecstored) := ilast-2
!                 print record if necessary
!              else ( end record )
!                 lastcolumn(numrecstored) := ilast
!                 finished := true
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data warn2439 /.false./
! ======================================================================
!
      call eropen ( 'readrecord' )
      iretur = 0

!     --- Set tab character

      if ( ascii ) then
         tab = char(9)
      else
         tab = char(5)
      end if
      controlm = char(13)
      varunix = stoper(1:4)=='unix'

!     --- numrecstored := 0
!         finished := false

      numrecstored = 0
      finished = .false.

!     --- while ( not finished ) do

100   do while ( .not. finished )

!     --- read record in cards

         irecnr = irecnr+1
         cards = ' '
         if ( irefre==0 ) then
            read( *, 110, iostat = ios ) cards
         else
            read( irefre, 110, iostat = ios ) cards
         end if
110      format(a)

         if ( ios<0 ) then

!        --- ios<0,  input exhausted, i.e. end of file
!            iretur := -1
!            finished := true

            iretur = -1
            finished = .true.

         else if ( ios>0 ) then

!        --- ios>0,  error in reading record
!            iretur := iostatus
!            finished := true

            iretur = ios
            finished = .true.

         else

!        --- regular record found
!            Replace tab by blanks

            do j = 1, maxcolum
               if ( cards(j:j)==tab ) cards(j:j) = ' '
               if ( varunix .and. cards(j:j)==controlm ) then

!              --- Unix environment and still a control m character found
!                  Replace by space

                  cards(j:j) = ' '
                  if ( .not. warn2439 ) then

!                 --- Warning 2439 not yet found, print warning

                     call errwar ( 2439, 0, 0, 0 )
                     warn2439 = .true.

                  end if
               end if
            end do ! j = 1, maxcolum

!           --- ifirst := first non blank character

            ifirst = maxcolum+1
            do j = 1, maxcolum
               if ( cards(j:j)/=' ' ) then
                  ifirst = j
                  go to 200
               end if
            end do ! j = 1, maxcolum

!           --- If the code arrives at this point, the whole record is empty
!               The next record must be read

            if ( ioutp>=0 ) write(irefwr,*) ' '
            if ( ispecl==3 ) then
               numrecstored = numrecstored+1
               cardimage(numrecstored) = ' '
               firstcolumn(numrecstored) = 1
               lastcolumn(numrecstored) = 1
               go to 1000
            end if  ! ( ispecl==3 )

            go to 100

!           --- lastp := last non blank character

200         lastp = maxcolum
            do j = maxcolum, 1, -1
               if ( cards(j:j)/=' ' ) then
                  lastp = j
                  go to 210
               end if
            end do ! j = maxcolum, 1, -1

!           --- ilast := last non-blank for comment character # or !

210         ilast = lastp+1
            do j = ifirst, lastp
               if ( cards(j:j)=='#' .or. cards(j:j)=='!' ) then
                  ilast = j
                  go to 220
               end if
            end do ! j = ifirst, lastp

220         ilast = ilast-1
            ih = ilast
            do j = ih, ifirst, -1
               if ( cards(j:j)/=' ' ) then
                  ilast = j
                  go to 230
               end if
            end do ! j = ih, ifirst, -1

230         if ( cards(1:1)=='*' .or. &
                 cards(1:1)=='#' .or.cards(1:1)=='!' ) then

!           --- first character = *, #, !, hence comment record
!               print record if necessary

               if ( ioutp>=0 ) write(irefwr,250) cards(1:lastp)
250            format(1x,a)

            else if ( ilast<ifirst ) then

!           --- record is empty
!               print record if necessary

               if ( ioutp>=0 ) write(irefwr,250) cards(1:lastp)

            else

!           --- record contains information
!               numrecstored := numrecstored+1
!               cardimage(numrecstored) := cards
!               firstcolumn(numrecstored) := ifirst

               numrecstored = numrecstored+1
               if ( numrecstored>maxrecords ) then

!              --- numrecstord too large

                  call errint ( numrecstored, 1 )
                  call errint ( maxrecords, 2 )
                  call errchr ( 'cards', 1 )
                  call errsub ( 550, 2, 0, 1 )
                  stop

               end if

               cardimage(numrecstored) = cards
               firstcolumn(numrecstored) = ifirst

               if ( cards(ilast-1:ilast)=='//' ) then

!              --- last two characters are // &
!                  lastcolumn(numrecstored) := ilast-2
!                  print record if necessary

                  lastcolumn(numrecstored) = ilast-2
                  if ( ioutp>=0 ) write(irefwr,250) cards(1:lastp)

               else if ( cards(ilast:ilast)=='&' ) then

!              --- last characters is &
!                  lastcolumn(numrecstored) := ilast-1
!                  print record if necessary

                  lastcolumn(numrecstored) = ilast-1
                  if ( ioutp>=0 ) write(irefwr,250) cards(1:lastp)

               else

!              --- end record
!                  lastcolumn(numrecstored) := ilast
!                  finished := true

                  lastcolumn(numrecstored) = ilast
                  finished = .true.

               end if  ! ( cards(last-1:last)=='//' )

            end if  !  ( cards(1:1)=='*' )

         end if  ! ( ios<0 )

      end do  ! ( .not. finished )

1000  call erclos ( 'readrecord' )

      end
