      subroutine readsrin ( inumbr, maxnum, inum, positive, maxval, &
                            code )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 14-11-2013 New call to prevalexpr
!        version  2.1  date 21-12-2011 New call to prevalexpr
!        version  2.0  date 29-02-2008 Completely rewritten
!
!   copyright (c) 1998-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     subroutine READSRIN is used to read a series of integer numbers
!     The numbers may be alternated with texts, which are skipped
!     The reading of numbers is stopped if
!      a ) sign is found
!      an end of record is found
!      maxnum numbers have been read
!     If a minus sign followed by text followed by a number is read, &
!     the number is made negative
!     After reading the numbers, icolum points to the next position to be read
!     If maxnum<0, texts are used as separater and no numbers are also allowed
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer maxnum, inum, inumbr(*), maxval
      logical positive
      character (len=*) code

!     code           i    String containing the corresponding keyword
!     inum           o    Number of numbers that have been actually read
!     inumbr         o    Array to be filled with the numbers
!     maxnum         i    |maxnum| defines the maximum number of numbers to be
!                         read
!                         If maxnum>0, texts are skipped
!                         If maxnum<0, texts are used as separater
!     maxval         i    Maximum value allowed for the integer read
!     positive       i    If true all numbers must be positive
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kcolsv, ival
      logical debug, finished, minsgn, openfound, closefound

!     closefound     If true a closing bracket has been found, which
!                    corresponds to a closing bracket read earlier
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     ival           Integer value read
!     kcolsv         Help parameter to save the old value of kcolum
!     minsgn         Sign that has been read. If no sign is read minsgn=1, &
!                    if a - has been found minsgn = -1
!     openfound      Indicates if an openings bracket is found before
!                    reading the number or expression
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     READSREXPR     Interprete part of input as number or expression
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1870   Integer number read out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Clear array dnumbr
!   finished  := false
!   inum   := 0
!   ispecl := 4
!   minsgn := false
!   while ( not finished ) do
!      Read next item (ASKNXT)
!      if ( ityprd = 2 or 3 ) then
!         inum := inum+1
!         if ( minsgn ) then realrd := -realrd
!         minsgn := false
!      else if ( ityprd = 6 ) then
!         minsgn := true
!      else if ( ityprd = 7 or 0 or 1 ) then
!         finished := true
!      if ( inum = maxnum ) then
!         finished := true
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Make some initializations

      call eropen ( 'readsrin' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readsrin'
         write(irefwr,1) 'lastitem', lastitem
         call prinin ( typeitems,nitems, 'typeitems' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear array inumbr

      inumbr(1:abs(maxnum)) = 0

      finished  = .false.
      inum   = 0
      ispecl = 4
      minsgn = .false.
      kcolsv = kcolum

      openfound = .false.
      closefound = .false.

      do while ( .not. finished )

!     --- While loop
!         Read next item, skip equals sign

         call readsrexpr ( openfound, closefound, .false., finished )
         if ( debug ) write(irefwr,2) 'realrd', realrd

         if ( ierror/=0 ) go to 1000

         if ( ityprd==2 .or. ityprd==3 ) then

!        --- ityprd = 2 or 3, number found

            inum = inum + 1
            if ( minsgn .and. realrd>0 ) realrd = - realrd
            minsgn = .false.
            inumbr(inum) = nint(realrd)
            ival = nint(realrd)
            if ( positive ) then

!           --- only positive integers

               if ( ival<1 .or. ival>maxval ) then

!              --- Error integer out of range

                  call errint ( ival, 1 )
                  call errint ( 1, 2 )
                  call errint ( maxval, 3 )
                  call errchr ( code, 1 )
                  call errsub ( 1870, 3, 0, 1 )
                  call errcrd

               end if

            else

!           --- positive and negative integers

               if ( abs(ival)>maxval ) then

!              --- Error integer out of range

                  call errint ( ival, 1 )
                  call errint ( -maxval, 2 )
                  call errint ( maxval, 3 )
                  call errchr ( code, 1 )
                  call errsub ( 1870, 3, 0, 1 )
                  call errcrd

               end if

            end if

         else if ( ityprd==6 ) then

!        --- ityprd = 6, minus sign found

            minsgn = .true.

         end if

         if ( inum>=abs(maxnum) ) then

!        --- inum = maxnum, stop reading

            finished = .true.

         end if

      end do  ! while ( .not. finished )

      ispecl = 0
      kcolum = min ( kcolum, kcolsv )

1000  call erclos ( 'readsrin' )
      if ( debug ) then

!     --- Debug information

         call prinin ( inumbr, inum, 'inumbr' )
         write(irefwr,*) 'End readsrin'

      end if  ! ( debug )

      end
