      subroutine msh609 ( coor, i, j, k, e1, e2, e3, xm, ym, zm )
! ======================================================================
!
!        programmer    niek praagman
!        version  4.0  date 03-09-2010 New release
!        version  3.1  date 27-06-2008 Textual adjustements
!        version  3.0  date 28-01-2005 New norms
!        version  2.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!
!   copyright (c) 1987-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to determine the vector (e1,e2,e3) which is perpen- &
!     dicular to the plane of the points i-j-k. Also point (xm,ym,zm)
!     which is a mean value between the origin of the circumcircle of
!     the triangle and the barycenter of the triangle is calculated.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
!   None
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer i, j, k
      double precision e1, e2, e3, xm, ym, zm, coor(3,*)

!     coor           i    array of length 3 x npoint containing the
!                         co-ordinates of the nodal points
!     e1             o    first coordinate vector perp to triangle plane
!     e2             o    second coordinate
!     e3             o    third coordinate unity vector
!     i              i    node of triangle considered
!     j              i    node of triangle considered
!     k              i    node of triangle considered
!     xm             o    x-coordinate centre of circumcircle triangle
!     ym             o    y-coordinate centre of circumcircle triangle
!     zm             o    z-coordinate centre of circumcircle triangle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision xi, xj, xk, yi, yj, yk, zi, zj, zk, &
                       eleng, xm1, ym1, zm1, xm2, ym2, zm2, &
                       a1, a2, a3, a4, a5, a6, a7, a8, a9, &
                       b1, b2, b3, det

!     a1             helpvariable to store matrix element 11
!     a2             helpvariable to store matrix element 12
!     a3             helpvariable to store matrix element 13
!     a4             helpvariable to store matrix element 21
!     a5             helpvariable to store matrix element 22
!     a6             helpvariable to store matrix element 23
!     a7             helpvariable to store matrix element 31
!     a8             helpvariable to store matrix element 32
!     a9             helpvariable to store matrix element 33
!     b1             rhs 1
!     b2             rhs 2
!     b3             rhs 3
!     det            determinant value
!     eleng          length vector
!     xi             x-coordinate node i
!     xj             x-coordinate node j
!     xk             x-coordinate node k
!     xm1            x-coordinate barycentre
!     xm2            x-coordinate centre circumcircle
!     yi             y-coordinate node i
!     yj             y-coordinate node j
!     yk             y-coordinate node k
!     ym1            y-coordinate barycentre
!     ym2            y-coordinate centre circumcircle
!     zi             z-coordinate node i
!     zj             z-coordinate node j
!     zk             z-coordinate node k
!     zm1            z-coordinate barycentre
!     zm2            z-coordinate centre circumcircle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!  None
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine coordinates of three corner nodes
!
!     Compute direction perpendicular to plane of the nodes
!
!     Normalize vector
!
!     Determine Barycenter of triangle
!
!     Compute matrix values
!
!     Compute right-hand-side
!
!     Solve system of equations
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Determine coordinates of three corner nodes

      xi = coor(1,i)
      yi = coor(2,i)
      zi = coor(3,i)

      xj = coor(1,j)
      yj = coor(2,j)
      zj = coor(3,j)

      xk = coor(1,k)
      yk = coor(2,k)
      zk = coor(3,k)

!     --- Compute direction perpendicular to plane of the nodes

      e1 = (yj-yi)*(zk-zi) - (yk-yi)*(zj-zi)
      e2 = (zj-zi)*(xk-xi) - (zk-zi)*(xj-xi)
      e3 = (xj-xi)*(yk-yi) - (xk-xi)*(yj-yi)

!     --- Normalize vector

      eleng = sqrt( e1*e1 + e2*e2 + e3*e3 )

      e1 = e1 / eleng
      e2 = e2 / eleng
      e3 = e3 / eleng

!     --- Determine Barycenter of triangle

      xm1 = ( xi + xj + xk ) / 3d0
      ym1 = ( yi + yj + yk ) / 3d0
      zm1 = ( zi + zj + zk ) / 3d0

!     --- Two possibilities, use the simple one

      if ( 0>1 ) then

!     --- Compute center (xm2,ym2,zm2) of circumcircle:

         a1 = e1
         a2 = e2
         a3 = e3

         a4 = xi-xj
         a5 = yi-yj
         a6 = zi-zj

         a7 = xi-xk
         a8 = yi-yk
         a9 = zi-zk

!        --- Compute right-hand-side

         b1 =   xi*e1 + yi*e2 + zi*e3
         b2 = ( xi*xi - xj*xj + yi*yi - yj*yj + zi*zi - zj*zj ) / 2.d0
         b3 = ( xi*xi - xk*xk + yi*yi - yk*yk + zi*zi - zk*zk ) / 2.d0

!        --- Solve system of equations

         det  =   a1 * ( a5 * a9 - a6 * a8 ) &
                - a2 * ( a4 * a9 - a6 * a7 ) &
                + a3 * ( a4 * a8 - a5 * a7 )

         xm2  =   b1 * ( a5 * a9 - a6 * a8 ) &
                - a2 * ( b2 * a9 - a6 * b3 ) &
                + a3 * ( b2 * a8 - a5 * b3 )

         ym2  =   a1 * ( b2 * a9 - a6 * b3 ) &
                - b1 * ( a4 * a9 - a6 * a7 ) &
                + a3 * ( a4 * b3 - b2 * a7 )

         zm2  =   a1 * ( a5 * b3 - b2 * a8 ) &
                - a2 * ( a4 * b3 - b2 * a7 ) &
                + b1 * ( a4 * a8 - a5 * a7 )

         xm  =  ( xm2 / det + xm1 ) / 2d0
         ym  =  ( ym2 / det + ym1 ) / 2d0
         zm  =  ( zm2 / det + zm1 ) / 2d0

      else

         xm  =  xm1
         ym  =  ym1
         zm  =  zm1

      end if

      end
