      subroutine msh621( coor , npoint, nipnt, iwork, ibuurp, coars, &
                         kelem, nelem )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.2  date 13-04-2012 Change deallocate
!        version  3.1  date 01-12-2010 Textual update
!        version  3.0  date 06-09-2010 New distortion method Laplacian check
!
!   copyright (c) 1988-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to perform a Laplace repositioning of the nodes
!     Check, before accepting the replacement, whether elements are
!     distorted. In that case do not use repos values.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), ibuurp(*), nipnt, npoint, kelem(4,*), nelem
      double precision coor(3,*), coars

!     coars   i    reference length of coarseness
!     coor   i/o   array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     ibuurp  i    array with nodal point numbers of neighbour points
!                  for each point ; also output of msh619
!     iwork   i    helparray output of msh619
!     kelem   i    element array
!     nelem   i    number of elements
!     nipnt   i    number of boundary points
!     npoint  i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2, i3, i4, ibrlen, iel, iend, ikn, iok, iposit, &
              istart, jeind, jkn, jr, jstart, nbuur, npos
      integer istelm(npoint-nipnt+1)
      integer, dimension   (:), allocatable      :: ibrelm
      double precision af, distan, volume, &
                       xh, xz, yh, yz, zh, zz

!     af          distance
!     debug       if true give debug messages
!     distan      reference distance
!     i           loopvariable
!     i1          node number
!     i2          node number
!     i3          node number
!     i4          node number
!     ibrelm      array containing for each internal node the elements
!                 it is a node of
!     ibrlen      computed length of array ibrelm
!     iel         element sequence number
!     iend        last position elements for node
!     ikn         counter
!     iok         helpvariable
!     iposit      position indicator
!     istart      helpvariable
!     istelm      integer helparray with startadresses for each internal
!                 node
!     jeind       helpvariable
!     jkn         counter
!     jr          loop variable
!     jstart      helpvariable
!     nbuur       number of neighbours
!     npos        position in neighbourelements array
!     xh          x-coordinate
!     xz          x-coordinate barycenter
!     yh          y-coordinate
!     yz          y-coordinate barycenter
!     zh          z-coordinate
!     zz          z-coordinate barycenter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     MSH632         compute volume of tetrahedron
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Set value debug

      debug = .false.

!     --- Compute stop value for coarseness loop:

      coars = 0.01 * coars

!     --- Initialize helparray with startaddresses for extra check:

      do i=nipnt+1, npoint + 1
         istelm (i-nipnt) = 0
      end do

!     --- Determine number of neighbours for each internal node:

      do i=1, nelem
         do jkn = 1, 4

!        --- Node number

            ikn    = kelem( jkn, i )
            iposit = ikn - nipnt
            if ( iposit>0 ) then
               npos = npos + 1
               istelm(iposit+1) = istelm(iposit+1)+1
            end if
         end do
      end do

!     --- Number of positions needed is npos, next
!         find startposition for each internal node:

      istelm(1) = 1
      do i= nipnt+2, npoint+1
         istelm(i-nipnt) = istelm(i-nipnt-1)+istelm(i-nipnt)
      end do

!     --- Create and fill ibrelm with numbers:

      ibrlen = istelm(npoint+1-nipnt) - 1

!     --- Allocate space for array ibrelm
!         needed length of array ibrelm is ibrlen

      if ( debug ) then
         write(irefwr,*) 'Allocated length array ibrelm =',ibrlen, &
                   allocated(ibrelm)
      end if
      allocate( ibrelm( ibrlen ), stat = error )
      if ( error>0 ) then
         call eralloc ( error, ibrlen, 'ibrelm' )
         go to 1000
      end if

!     --- Initialize array ibrelm:

      do i = 1, ibrlen
         ibrelm(i) = 0
      end do

!     --- Fill element numbers:

      do i=1, nelem
         do jkn = 1, 4

!        --- Node number

            ikn    = kelem( jkn, i )
            iposit = ikn - nipnt
            if ( iposit>0 ) then

!           --- Find startposition:

               istart = istelm(iposit  )
               iend   = istelm(iposit+1)
               iok    = 0
               do npos = istart, iend - 1
                  if ( ibrelm( npos )==0 .and. iok==0 ) then

!                 --- Place element:

                     ibrelm(npos) = i
                     iok = 1
                  end if
               end do
            end if
         end do
      end do
      if ( debug ) then
         write(irefwr,*) 'msh621: Contents neighbourarray ibrelm is'
         write(irefwr,*) (ibrelm(i),i=1,ibrlen)
      end if

!     --- At most twelve "Laplacian" runs

      do jr = 1, 12

!     --- Set distan

         distan = 0d0

!        --- Run through all (non-fixed, internal) points

         do ikn = nipnt + 1, npoint

!        --- Find first position for all neighbours of
!            node ikn:

            jstart=0
            if ( ikn/=1 ) jstart = iwork(ikn-1)
            jstart = jstart + 1
            jstart = iwork(ikn-1) + 1
            jeind  = iwork(ikn  )

!           --- If there are neighbours:

            if ( jstart<=jeind ) then
                 xz  =  0d0
                 yz  =  0d0
                 zz  =  0d0

!                --- Number of neighbours:

                 nbuur = jeind - jstart + 1

!                --- Run through neighbours:

                 do i = jstart, jeind
                    jkn = ibuurp(i)

!                   --- Existing?

                    if ( jkn>0 ) then
                         xz = xz + coor( 1, jkn )
                         yz = yz + coor( 2, jkn )
                         zz = zz + coor( 3, jkn )
                    else
                         nbuur = nbuur - 1
                    end if
                 end do

!                --- Barycenter of neighbours:

                 xz = xz / nbuur
                 yz = yz / nbuur
                 zz = zz / nbuur

!                --- Present coordinates ikn:

                 xh = coor( 1, ikn )
                 yh = coor( 2, ikn )
                 zh = coor( 3, ikn )

!                --- Place the new values but do distortion checks

                 coor( 1, ikn ) = xz
                 coor( 2, ikn ) = yz
                 coor( 3, ikn ) = zz

!                --- Check sign of all the elements with this new
!                    position of node ikn:

                 iposit = ikn - nipnt

!                --- Find startposition neighbour elements:

                 istart  = istelm(iposit  )
                 iend    = istelm(iposit+1)

!                --- Run through all neighbours

                 do npos = istart, iend - 1

!                --- Consider element with number iel

                    iel = ibrelm(npos)

!                   --- Four nodes:

                    i1 = kelem(1,iel)
                    i2 = kelem(2,iel)
                    i3 = kelem(3,iel)
                    i4 = kelem(4,iel)

!                   --- volume

                    call msh632( coor, i1, i2, i3, i4, volume )

!                   --- check

                    if ( volume<0 ) then

!                   --- Distortion, keep old situation:

                       if ( debug ) &
                          write(irefwr,*) 'Elmnt',ikn,'distorted'
                       coor(1,ikn) = xh
                       coor(2,ikn) = yh
                       coor(3,ikn) = zh
                       goto 100
                    end if
                 end do

!                --- New point accepted:

                 af = sqrt( (xh-xz)*(xh-xz) + (yh-yz)*(yh-yz) + &
                            (zh-zz)*(zh-zz) )
                 if ( af>distan ) distan = af
            end if
100      end do

!        --- Check accuracy:

         if ( jr>2 .and. distan<coars ) goto 1000
      end do
1000  deallocate( ibrelm, stat = error )
      if ( error>0 ) then
         call erdealloc ( error, 'ibrelm' )
      end if

!     --- Return:

      end
