      subroutine sepbufcreate ( mmnum, lenarr, typearr, namear )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.12 date 21-11-2012 Long integers
!        version  1.11 date 28-09-2012 Correction largest integer
!        version  1.10 date 04-06-2007 Fortran 90
!
!   copyright (c) 1997-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute starting position of a new or already existing array in buffer
!     array ibuffr
!     sepbufcreate replaces the old subroutine INI001 (for new arrays)
!     When the array is placed on backing storage, and not in IBUFFR then
!     the array space is reserved without reading from file 3
!     If the array did not exist, or the old array took too few positions to
!     keep the new one, then space in array IBUFFR is reserved
!     if no space is available in array IBUFFR, sepbufcreate tries to
!     create space in the following ways:
!       -  by writing low priority arrays to backing storage
!       -  if this is not sufficient by compressing
!          array ibuffr after writing of all arrays with priority
!          smaller than 10 to backing storage
!     sepbufcreate is identical to subroutine INI051 as described in the Programmer's
!     Guide, except that the parameter name is not longer used.
!     The name of the calling subroutine is extracted from common blocks
!     CMESSC and CMESSG, so it is supposed that the subroutine EROPEN has been
!     used
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuf
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmessgf90'
      include 'SPcommon/cmesscf90'
      include 'SPcommon/cdebugf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: lenarr, typearr
      integer, intent(inout) :: mmnum
      character (len=*) :: namear

!     mmnum        i/o   indexv refers to the array sequence number from which
!                        the starting position must be computed
!     lenarr        o    Length of array
!                        If lenarr<0, read array from backing storage if it
!                        exists
!     typearr       i    indicates type of array
!                        1: integer
!                        2: double precision
!                        3: complex
!     mmnum        i/o   The value of mmnum is the reference number of the
!                        array of which the starting position must be computed
!                        with respect to array INFOR in common block IINFOR
!                        This means that the positions (i,mmnum) (i=1,2,3) in
!                        INFOR contain all the information about the storage
!                        of the array
!     namear        i    Name of array
!                        This name is used in the table if the array
!                        does not yet exist
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) :: indfor, ipstar, i
      integer :: lengol, j, iextra, lenchr, lenabs, lenarray
      logical :: newarr

!     i         General loop variable
!     iextra    Extra space needed at both sides of the array for control
!               integers
!     indfor    Help parameter to store the record number from which the
!               array has been stored in file 3
!     ipstar    computed starting position in array IBUFFR
!     j         General help variable
!     lenabs    |lenarr|
!     lenarray  Lenth of array corrected for typearr
!     lenchr    Number of characters in name of array
!     lengol    Length of the preceding array corresponding to mmnum
!     newarr    If true, the array is new
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI049         Create space in IBUFFR by writing low priority arrays to
!                    file 3, or to compress IBUFFR
!     INICHK1        Check IBUFFR
!     PRINBF         Print and check length of array IBUFFR
!     PRINBUFARS     Print the active arrays present in array buffer
!     SEPACTIVATE    Reactivate existing array in IBUFFR
! **********************************************************************
!
!                       I/O
!
!     If necessary arrays are written to and/or read from file 3 (iref3)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     760  more than numarrs arrays have been created in IBUFFR
!    1765  length of array less than one
! **********************************************************************
!
!                       PSEUDO CODE
!
!     If ( idebug>0 ) then
!        Set iextra = extra space needed
!        check all control integers
!     if ( lenarr<=0 ) then
!        print error
!     If ( Array existed before ) then
!        If ( array in IBUFFR and old length<=new length ) then
!           Compute IPSTAR
!        else ( not enough space at old positions )
!           If ( enough space in IBUFFR after IBFREE ) then
!              IPSTAR = IBFREE
!              Raise IBFREE
!           else
!              Create space by writing low priority arrays to file 3
!              or compressing IBUFFR
!     else (array is new )
!         If ( enough space in IBUFFR after IBFREE ) then
!             IPSTAR = IBFREE
!             Raise IBFREE
!         else
!             Create space by writing low priority arrays to file 3
!             or compressing IBUFFR
!         Raise IINFOR
!         mmnum = IINFOR
!     Fill array INFOR
!     Set INBACK ( mmnum ) negative if nessary
!     If ( idebug>0 ) then
!        place control integers for this array
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepbufcreate' )
      if ( ierror/=0 ) go to 1000

      if ( idebug==0 ) then
         iextra = 0
      else
         iextra = intlen
         ifirws = 0
         if ( calchk==0 ) call inichk1(ibuffr)
      end if
      if ( lenarr==0 ) then

!     --- lenarr = 0, error

         call errint ( lenarr, 1 )
         call errchr ( namear, 1 )
         call errsub ( 1765, 1, 0, 1 )

      end if
      if ( ierror/=0 ) go to 1000

      select case ( typearr)
         case(1)
            lenarray = lenarr          ! integer
         case(2)
            lenarray = lenarr*intlen   ! double
         case(3)
            lenarray = 2*lenarr*intlen ! complex
      end select  ! case ( typearr)

      lenabs = abs(lenarray)
      newarr = .true.

      if ( mmnum>0 .and. mmnum<=iinfor ) then

!     --- 0<mmnum<=iinfor  array has been filled before

         indfor = infor(mmnum)
         lengol = infor1(1,mmnum)
         newarr = .false.
         if ( indfor>0 .and. lenabs<=lengol ) then

!        --- indfor>0  and lenabs<lengol   old positions are used

            ipstar = indfor
            if ( idebug>=2 ) then

               ibuffr(ipstar+lenabs+1:ipstar+lengol) = iinfin

             end if

         else if ( lenarray<0 .and. indfor<0 .and. lenabs==lengol ) then

!        --- indfor<0  and lenabs=lengol   activate array

            call sepactivate ( mmnum, namear )
            go to 1000

         else

!        --- lenabs>lengol or indfor<0
!            new array is bigger than preceding one, &
!            or array is stored in backing storage
!            old positions are deleted, new positions are used
!            check free positions

            if ( calchk==0 .and. idebug>=2 .and. indfor>0 ) then
               ibuffr(indfor-1:indfor+lengol) = iinfin
            end if

            i = ibfree+lenabs+iextra*2
            if ( i>0 .and. i<=3*nbuffr/4 ) then

!           --- enough free space available   ( standard situation )

               ipstar = ibfree
               ibfree = ibfree+lenabs+iextra*2
               if ( intlen==2 .and. mod ( ibfree, 2_8)==0 ) ibfree = ibfree+1

            else

!           --- try to create free space by writing low priority arrays
!               to backing storage   ( file 3 ) or compressing ibuffr

               call ini049 ( ibuffr, lenabs, ipstar )

            end if

         end if

      else

!     --- new array

         if ( ibfree+lenabs+iextra*2-1<=3*nbuffr/4 ) then

!        --- enough free space available   ( standard situation )

            ipstar = ibfree
            ibfree = ibfree+lenabs+iextra*2

!           --- For a 32-bits computer, ibfree must always be odd

            if ( intlen==2 .and. mod(ibfree,2_8)==0 ) ibfree=ibfree+1

         else

!        --- try to create free space by writing low priority arrays
!            to backing storage ( file 3 ) or compressing ibuffr

               call ini049 ( ibuffr, lenabs, ipstar )
               if ( ierror/=0 ) go to 1000

         end if
         iinfor = iinfor+1
         mmnum = iinfor
         if  (  iinfor>numarrs  ) then

!        --- iinfor too large
!            Check if some array has been deleted, if not error

             do j = 1, numarrs
                if ( infor1(1,j)==0 ) then

!               --- infor1(1,j) = 0, means that array i is free

                   mmnum = j
                   iinfor = numarrs
                   go to 150

                end if
             end do

            call errint ( iinfor, 1 )
            call errint ( numarrs, 2 )
            call errsub ( 760, 2, 0, 0 )
            call prinbufars

         end if

      end if

150   if ( ierror/=0 ) go to 1000
      infor(mmnum) = ipstar
      infor1(1,mmnum) = lenabs
      infor1(2,mmnum) = 10
      if ( inback (mmnum)>0 ) inback (mmnum) = -inback (mmnum)
      ibfree = max ( ibfree, ipstar+lenabs+iextra*2)
      call prinbf ( ibfree-1 )
      if ( ierror/=0 ) go to 1000

      if ( idebug>0 ) then

!     --- idebug>0, set control integers and control the buffer array

         ibuffr(ipstar-1) = mmnum
         ibuffr(ipstar+lenabs) = mmnum

      end if
      if ( ioutp>2 ) then
         write(irefwr,*) ' Create array ', namear, &
                         ' with sequence number', mmnum, &
                         ' and starting address', ipstar, &
                         ' and length ', lenarray, &
                         ' in subroutine ', &
                         namsub(levsub-1)//':sepbufcreate'
      end if

      if ( newarr ) then

!     --- Fill names table

         lenchr = min ( 16, len(namear) )
         namarr(mmnum) = namear(1:lenchr)

      end if

1000  call erclos ( 'sepbufcreate' )

      end
