      subroutine er0063 ( npointlc, idegfd, iwork, igathr, nusout, nbnout, &
                          iskip, ioutpoints )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 09-03-2013 Correction declaration
!        version  2.1  date 20-07-2001 Correction iskip=0 and idegfd=0
!        version  2.0  date 02-01-2000 Extra parameters iskip, ioutpoints
!                                      skip element groups
!
!   copyright (c) 1989-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine is used to fill the arrays iwork and igathr with all
!     positions in the solution array that correspond to degree of freedom
!     idegfd (per point)
!     Array IWORK is filled as follows:
!        IWORK(i)>0, gives start position of unknown in point i
!        IWORK(i) = 0, unknown is not available in point i
!        IWORK(i)<0, point is second periodical point
!                      -IWORK(i) refers to first point
!     Array IGATHR contains the same information as IWORK but compressed
!     such that all non-positive information is deleted. Hence:
!     periodical points are counted for only once, and if nboundlc>0, the
!     essential boundary conditions are placed in the rear
!     In combination with subroutines ER0061 and ER0062 this subroutine is
!     used to store the unknowns of a large vector into a smaller one for
!     computational purposes and to place it back into the large vector
! **********************************************************************
!
!                       KEYWORDS
!
!     fill
!     gather
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npointlc, idegfd, iwork(npointlc), igathr(*), &
              nusout, nbnout, iskip, ioutpoints(*)

!     idegfd         i   Actual degree of freedom per point
!     igathr         o   Integer array containing the indices of the positions
!                        in usol corresponding to IDEGFD
!                        If nboundlc>0 the prescribed degrees of freedom are
!                        positioned at the rear. Periodical boundary conditions
!                        are counted for only once
!     ioutpoints     i   Output array of length npointlc
!                        If ioutpoints(i)=0, node i must be skipped, else
!                        ioutpoints(i)>0, node i must not be skipped
!     iskip          i   If>0, element groups must be skipped
!     iwork          o   Work array of length NPOINT to store the degrees of
!                        freedom per point.  See description
!     nbnout         o    Number of prescribed boundary conditions in IGATHR
!     npointlc       i   Number of nodal points
!     nusout         i   Number of degrees of freedom in IGATHR
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inpelm, ikelmc, inunkp, &
              ielgrp, itype, iielem, inodp1, inodp2, ih, j,  jdegfd
      logical presen

!     i        Counting variable
!     ielgrp   Element group number
!     ih       help variable to store a constant
!     iielem   Relative element number in element group
!     ikelmc   Address in KMESH part c
!     inodp1   First node in element for periodical boundary conditions
!     inodp2   Second node in element for periodical boundary conditions
!     inpelm   Number of nodes in the element
!     inunkp         Number of degrees of freedom in point
!     itype    Type number of element
!     j        Counting variable
!     jdegfd   Maximum (1, idegfd)
!     presen   Check if the degree of freedom is present in at least one point
!              (true) or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
!     none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     218   Degree of freedom does not exist
! **********************************************************************
!
!                       PSEUDO CODE
!
!     First fill iwork with the positions of the unknown in all nodes
!     If ( periodical boundary conditions present )
!        Set second point of set of periodical boundary conditions negative
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      kprobc => kprobpart%kprobc

!     --- Fill array IWORK, except for periodical boundary conditions

      jdegfd  = max(idegfd,1)
      if ( iskip>0 .and. idegfd==0 ) then

!     --- idegfd = 0, only possible in combination with skip element group

         nusout = 0
         nbnout = 0
         if ( indprpi>0 ) then

!        --- indprpi>0, i.e. copy iwork from KPROBP

            do i = 1, npointlc
               if ( ioutpoints(i)>0 ) then
                  do j = 1, nphysi
                     ih = kprobpi(i,j)
                     if ( ih>0 ) then
                        if ( nboundi>0 ) then
                           if ( kprobhi(ih)>nusoli-nboundi-npbouni .and. &
                             nboundi>0 ) then
                              nbnout = nbnout + 1
                              igathr(nusoli+1-nbnout) = ih
                           else
                              nusout = nusout + 1
                              igathr(nusout) = ih
                           end if
                        else
                           nusout = nusout + 1
                           igathr(nusout) = ih
                        end if
                     end if
                  end do
               end if
            end do

         else if ( indprfi>0 ) then

!        --- indprfi>0, i.e. no renumbering but the number of degrees of
!                             freedom is variable

            do i = 1, npointlc
               if ( ioutpoints(i)>0 ) then
                  inunkp = kprobfi(i+1) - kprobfi(i)
                  do j = 1, inunkp
                     ih = kprobfi(i)+j
                     if ( ih>0 ) then
                        if ( nboundi>0 ) then
                           if ( kprobhi(ih)>nusoli-nboundi-npbouni .and. &
                             nboundi>0 ) then
                              nbnout = nbnout + 1
                              igathr(nusoli+1-nbnout) = ih
                           else
                              nusout = nusout + 1
                              igathr(nusout) = ih
                           end if
                        else
                           nusout = nusout + 1
                           igathr(nusout) = ih
                        end if
                     end if
                  end do
               end if
            end do

         else

!        --- Constant number of degrees of freedom (nphysi) and no renumbering

            do i = 1, npointlc
               if ( ioutpoints(i)>0 ) then
                  do j = 1, nphysi
                     ih = (i-1)*nphysi+j
                     if ( ih>0 ) then
                        if ( nboundi>0 ) then
                           if ( kprobhi(ih)>nusoli-nboundi-npbouni .and. &
                             nboundi>0 ) then
                              nbnout = nbnout + 1
                              igathr(nusoli+1-nbnout) = ih
                           else
                              nusout = nusout + 1
                              igathr(nusout) = ih
                           end if
                        else
                           nusout = nusout + 1
                           igathr(nusout) = ih
                        end if
                     end if
                  end do
               end if
            end do

         end if
         if ( nbnout>0 ) then

!        --- Boundary conditions

            do i = 1, nbnout
               igathr(nusout+i) = igathr(nusoli-nbnout+i)
            end do
            nusout = nusout + nbnout

         end if
         go to 1000

      else if ( indprpi>0 ) then

!     --- indprpi>0, i.e. copy iwork from kprobpi

         do i = 1, npointlc
            iwork(i) = kprobpi(i,jdegfd)
         end do
         if ( jdegfd>nphysi ) then
            call eropen ( 'er0063' )
            call errint ( jdegfd, 1 )
            call errint ( nphysi, 2 )
            call errsub ( 218, 2, 0, 0 )
            call erclos ( 'er0063' )
         end if

      else if ( indprfi>0 ) then

!     --- indprfi>0, i.e. no renumbering but the number of degrees of freedom
!                          is variable

         presen = .false.
         do i = 1, npointlc
            inunkp = kprobfi(i+1) - kprobfi(i)
            if ( jdegfd<=inunkp ) then
               iwork(i) = kprobfi(i) + jdegfd
               presen = .true.
            else
               iwork(i) = 0
            end if
         end do
         if ( .not. presen ) then
            call eropen ( 'er0063' )
            call errint ( jdegfd, 1 )
            call errint ( inunkp, 2 )
            call errsub ( 218, 2, 0, 0 )
            call erclos ( 'er0063' )
         end if

      else

!     --- Constant number of degrees of freedom (nphysi) and no renumbering

         do i = 1, npointlc
            iwork(i) = nphysi*(i-1) + jdegfd
         end do
         if ( jdegfd>nphysi ) then
            call eropen ( 'er0063' )
            call errint ( jdegfd, 1 )
            call errint ( nphysi, 2 )
            call errsub ( 218, 2, 0, 0 )
            call erclos ( 'er0063' )
         end if

      end if
      if ( ierror/=0 ) go to 1000

      if ( iskip>0 ) then

!     --- Element groups must be skipped

         do i = 1, npointlc
            if ( ioutpoints(i)==0 ) iwork(i) = 0
         end do

      end if

      if ( npbouni>0 ) then

!     --- Periodical boundary conditions
!         Set second point equal to - first one in iwork if iwork(i)#0

         ikelmc = 0

!        --- Loop over element groups

         do ielgrp = 1, nelgrp
            itype  = kprobc(ielgrp)
            inelem = Topgrp(ielgrp)%inelem
            inpelm = Topgrp(ielgrp)%inpelm
            if ( itype/=-1 ) then

!           --- itype # -1, not a periodical boundary condition

               ikelmc = ikelmc + inelem * inpelm

            else

!           --- itype = -1,  periodical boundary condition
!               Loop over elements

               do iielem = 1, inelem
                  inodp1 = kmeshc(ikelmc+1)
                  inodp2 = kmeshc(ikelmc+2)
                  ikelmc = ikelmc + 2
                  if ( iwork(inodp2)>0 ) iwork(inodp2) = - inodp1
               end do

            end if

         end do

      end if

!     --- Reorder iwork and store into IGATHR

      nbnout = 0
      if ( indprfi==0 .and. nboundi==0 .and. iskip==0 ) then

!     --- indprfi=0, nboundi=0, i.e. constant number of degrees of freedom
!                             per point and no boundary conditions

         do i = 1, npointlc
            igathr(i) = iwork(i)
         end do
         nusout = npointlc

      else if ( indprhi==0 ) then

!     --- indprfi#0 and nboundi=0, i.e. no boundary conditions

         nusout = 0
         do i = 1, npointlc
            if ( iwork(i)>0 ) then
               nusout = nusout + 1
               igathr(nusout) = iwork(i)
            end if
         end do

      else if ( indprfi>0 ) then

!     --- indprfi>0 and nboundi>0

         nusout = 0

         do i = 1, npointlc
            inunkp = kprobfi(i+1) - kprobfi(i)
            if ( inunkp>=idegfd ) then
               ih = kprobfi(i)+idegfd
               if ( kprobhi(ih)>nusoli-nboundi-npbouni ) then
                  igathr(npointlc-nbnout) = iwork(i)
                  nbnout = nbnout + 1
               else
                  nusout = nusout + 1
                  igathr(nusout) = iwork(i)
               end if
            end if  ! ( inunkp>=idegfd )
         end do

         do i = 1, nbnout
            igathr(nusout+i) = igathr(npointlc-nbnout+i)
         end do
         nusout = nusout + nbnout

      else

!     --- General situation,  nboundi>0

         nusout = 0

         do i = 1, npointlc
            if ( iwork(kprobhi(i))>nusoli-nboundi-npbouni ) then
               igathr(npointlc-nbnout) = iwork(i)
               nbnout = nbnout + 1
            else if ( iwork(kprobhi(i))>0 ) then
               nusout = nusout + 1
               igathr(nusout) = iwork(i)
            end if
         end do

         do i = 1, nbnout
            igathr(nusout+i) = igathr(npointlc-nbnout+i)
         end do
         nusout = nusout + nbnout

      end if

1000  end
