      subroutine er0076 ( idim, nparts, ipoint, inodes, number, nummax )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 04-10-2000 Correction for nparts>1
!        version  1.4  date 20-10-1998 Extension for number = 0
!        version  1.3  date 18-12-1997 minor corrections
!
!   copyright (c) 1994-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Delete all points in inodes and ipoint not containing the required
!   degree of freedom as given by number
!   This subroutine should only be applied if the number of degrees of
!   freedom per point is not constant and hence kprobfi exists
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer idim, nparts, ipoint(*), inodes(*), number, nummax

!     idim    i    If 2 then curves are required, if 3 it concerns surfaces
!     inodes i/o   Array containing the nodal point numbers along curve or
!                  surface depending on idim.
!                  If idim = 2, the nodes along the curves are ordered in
!                  the sequence indicated by icurvs. Common nodes in consecutive
!                  curves are counted for only once. If a gap occurs, i.e.
!                  if curves are no longer consecutive, the new curve is
!                  considered as start of a new part. Starting addresses of
!                  all parts are stored in array ipoint.
!                  If idim = 3, the nodes in the surfaces are stored only one.
!                  Array ipoint is not used.
!     ipoint i/o   In case of curves this is an array of length npart+1, &
!                  containing starting addresses of the various parts the
!                  curves are subdivided in. The last position contains the
!                  length of array inodes plus one.
!                  In case of surfaces ipoint is not used
!     nparts i/o   If idim = 2, number of parts the curves are subdivided in
!                  if idim = 3 number of different nodes at the surfaces
!     number  i    Sequence number of lowest degree of freedom in node that
!                  must be available
!     nummax  i    Number of subsequently degrees of freedom that must be
!                  available in each point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, lstart, ipart, iend, inodp, inunkp, k, ifound

!     i       Counting variable
!     iend    Last position used in part of inodes
!     ifound  Number of allowed degrees of freedom found in point
!     inodp   Nodal point number
!     inunkp  Number of degrees of freedom in first point
!     ipart   Part sequence number
!     istart  First position of a part in innodes
!     k       Counting variable
!     lstart  New position in array inodes and ipoint
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( idim=2 ) then
!        For ipart := 1 (1) nparts do
!           For all nodal points at part ipart do
!              If ( unknown not available ) then
!                 delete nodal point
!              If ( no points are left at part ) then
!                 delete part
!     else ( idim=3 )
!        Find all nodal points do
!          If ( unknown not available ) then
!             delete nodal point
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( idim.eq.2 ) then

!     --- idim = 2, curves

         istart = 1
         lstart = 1
         do ipart = 1, nparts
            iend = ipoint(ipart+1) - 1
            do i = istart, iend
               inodp = inodes(i)
               if ( indprpi.gt.0 .and. number.gt.0 ) then

!              --- indprpi, check presence of all degrees of freedom

                  ifound = 0
                  do k = 0, nummax-1
                     if ( kprobpi(inodp,number+k).ne.0 ) then
                        ifound = ifound+1
                     end if
                  end do
                  if ( ifound.eq.nummax ) then

!                 --- All degrees of freedom are present in node

                     inodes(lstart) = inodp
                     lstart = lstart+1

                  end if

               else if ( number.eq.0 ) then

!              --- indprf#0

                  inunkp = kprobfi(inodp+1)-kprobfi(inodp)
                  if ( inunkp.gt.0 ) then
                     inodes(lstart) = inodp
                     lstart = lstart+1
                  end if

               else

!              --- indprpi = 0, but indprf#0

                  inunkp = kprobfi(inodp+1)-kprobfi(inodp)
                  if ( inunkp.gt.0 .and. number+nummax-1.le.inunkp ) then
                     inodes(lstart) = inodp
                     lstart = lstart+1
                  end if

               end if

            end do
            ipoint(ipart+1) = lstart
            istart = iend+1

         end do

!        --- Delete empty parts if present

         lstart = 0
         do ipart = 1, nparts
            if ( ipoint(ipart+1)-ipoint(ipart).gt.0 ) then
               lstart = lstart+1
               ipoint(lstart) = ipoint(ipart)
            end if
         end do
         ipoint(lstart+1) = ipoint(nparts+1)
         nparts = lstart

      else

!     --- idim = 3, surfaces

         lstart = 1
         do i = 1, nparts
            inodp = inodes(i)
            if ( indprpi.gt.0 .and. number.gt.0 ) then

!           --- indprpi, check presence of all degrees of freedom

               ifound = 0
               do k = 0, nummax-1
                  if ( kprobpi(inodp,number+k).ne.0 ) then
                     ifound = ifound+1
                  end if
               end do
               if ( ifound.eq.nummax ) then

!              --- All degrees of freedom are present in node

                  inodes(lstart) = inodp
                  lstart = lstart+1

               end if

            else if ( number.eq.0 ) then

!           --- indprf#0

               inunkp = kprobfi(inodp+1)-kprobfi(inodp)
               if ( inunkp.gt.0 ) then
                  inodes(lstart) = inodp
                  lstart = lstart+1
               end if

            else

!           --- indprpi = 0, but indprf#0

               inunkp = kprobfi(inodp+1)-kprobfi(inodp)
               if ( inunkp.gt.0 .and. number+nummax-1.le.inunkp ) then
                  inodes(lstart) = inodp
                  lstart = lstart+1
               end if

            end if

         end do
         nparts = lstart-1

      end if

      end
