      subroutine iinputseq ( number, iseqnr, iinput, &
                             rinput, leninp, lenrnp, name )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 20-08-2016 use sepmoduleinput
!        version  2.3  date 21-11-2012 Long integers
!        version  2.2  date 10-10-2012 Correction empty rinput
!
!   copyright (c) 2004-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read contents of input block with sequence number iseqnr corresponding
!     to main keyword name (sequence number: number)
!     Store results in arrays iinput and rinput
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     input
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer number, iseqnr, iinput(*), leninp, lenrnp
      double precision rinput(*)
      character *(*) name

!     iinput         o    integer input array in which information is stored
!     iseqnr         i    Sequence number with respect to the storage in array
!                         ISEPIN
!                         If negative the routine returns without warning
!                         with leninp = 0 if no input can be found, otherwise
!                         (positive) an error message is given.
!     leninp        i/o   Computed length of integer input array
!                         At input: declared length
!                         At output: actual length
!     lenrnp        i/o   Computed length of real input array
!                         At input: declared length
!                         At output: actual length
!     name           i    Name of specific input block
!     number         i    Sequence number of specific input block in ISEPIN
!     rinput         o    Real input array in which information is stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer leninpact, lenrnpact, lenname, maxcal, ipos
      character (len=80) namecopy

!                    they are not
!     ipos           Position of input arrays in sepmoduleinput
!     leninpact      Computed length of integer input array
!     lenname        Length of name
!     lenrnpact      Computed length of real input array
!     maxcal         Maximum number of coefficient inputs read
!     namecopy       String with copy of name
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHINPSOL    Adapt input array with respect to variables
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     SEPGETINPFINF  Extract infor entries of integer and real input arrays from
!                    ISEPIN
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2494   No input block of specific name found
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'iinputseq' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from iinputseq'
         write(irefwr,1) 'number, iseqnr', number, iseqnr
         write(irefwr,1) 'leninp, lenrnp', leninp, lenrnp
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( iinputarrs(number)%maxcall<1 ) then

!     --- Array iintim has not been filled, i.e. no block name
!         has been found

         if ( iseqnr<0 ) then
            leninp = 0
            go to 1000   ! jump if negative seqnr
         end if  ! ( iseqnr<0 )
         call errint ( iseqnr, 1 )
         call errchr ( name, 1 )
         call errsub ( 2494, 1, 0, 1 )
         go to 1000

      end if

      call sepgetinpfinf ( number, abs(iseqnr), ipos, maxcal )

      if ( .not. in(ipos)%used ) then

!     --- Array has not been filled, i.e. no block name
!         has been found

         if ( iseqnr<0 ) then
            leninp = 0
            go to 1000   ! jump if negative seqnr
         end if  ! ( iseqnr<0 )
         call errint ( iseqnr, 1 )
         call errchr ( name, 1 )
         call errsub ( 2494, 1, 0, 1 )
         go to 1000

      end if  ! ( .not. in(ipos)%used )

      leninpact = size( in(ipos)%iinput )
      if ( debug ) write(irefwr,1) 'leninpact', leninpact

      lenname = min(len(name),80)
      namecopy = name(1:lenname)

      call inichkarlen ( leninpact, leninp, 'iin_'//namecopy )
      leninp = leninpact
      iinput(1:leninp) = in(ipos)%iinput

      if ( associated(re(ipos)%rinput) ) then

!     --- irintm>0, hence array rinput must be filled

         lenrnpact = size(re(ipos)%rinput)
         call inichkarlen ( lenrnpact, lenrnp, 'rin_'//namecopy )
         if ( debug ) write(irefwr,1) 'lenrnpact', lenrnpact
         lenrnp = lenrnpact
         rinput(1:lenrnp) = re(ipos)%rinput

      else

!     --- irintm = 0, hence array rinput must not be filled

         lenrnp = 0

      end if  ! ( irintm>0 )

      if ( ierror/=0 ) go to 1000

!     --- Adapt variables

      call inichinpsol ( iinput, leninp )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'leninp, lenrnp', leninp, lenrnp
         write(irefwr,*) 'End iinputseq'

      end if
      call erclos ( 'iinputseq' )

      end
