      subroutine inicurvuserp ( curvesw0, numcurw0, curvesw1, numcurw1, &
                                userpoint, curveusp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-11-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find user point that is on the interface of two sets of curves
!     In this case one of the sets overlaps the other one
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     user_point
!     interface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numcurw0, numcurw1, curvesw0(numcurw0), &
              curvesw1(numcurw1), userpoint, curveusp(*)

!     curvesw0       i    Contains the curves on which w0 is defined
!     curvesw1       i    Contains the curves on which w1 is defined
!     curveusp       i    array containing the user points corresponding to
!                         the curves in the following way:
!                         curveusp(1)=ncurvs+1
!                         number of user points in curve i:
!                         curveusp(i+1)-curveusp(i)
!                         user points in curve i are stored in curveusp from
!                         position curveusp(i)+1 to curveusp(i+1)
!     numcurw0       i    Number of curves stored in array curvesw0
!     numcurw1       i    Number of curves stored in array curvesw1
!     userpoint      o    User point at interface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, numcurves, icurnr, j
      integer, allocatable, dimension (:) :: iwork, setcurvs

!     i              Counting variable
!     icurnr         Curve sequence number
!     iwork          integer work array
!     j              Counting variable
!     numcurves      Number of curves in array icurves
!     setcurvs       Contains curve numbers of set w1 not in set w0
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2910   Not enough space
!    2911   Not possible to deallocate
!    3039   No common user point of set of curves corresponding to w0 and w1
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Get parameters
!     Allocate arrays
!     Separate sets of curves in two disjoint sets
!     Find common user point of set w0 and setcurvs
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inicurvuserp' )
      debug = .false.
      userpoint = 0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inicurvuserp'
         call prinin ( curvesw0, numcurw0, 'curves for w0' )
         call prinin ( curvesw1, numcurw1, 'curves for w1' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Allocate arrays

      numcurves = max(ncurvs, nuspnt)  ! help parameter
      allocate ( iwork(numcurves), setcurvs(ncurvs), stat = error )
      if ( error/=0 ) then

!     --- error#0, so not enough space

         call errint ( numcurves+ncurvs, 1 )
         call errint ( error, 2 )
         call errchr ( 'iwork', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 900

      end if  ! ( error/= 0 )

!     --- Separate sets of curves in two disjoint sets

      iwork = 0                     ! clear iwork
      do i = 1, numcurw0
         iwork(curvesw0(i)) = i     ! mark that curve belongs to set w0
      end do  ! i = 1, numcurw0

!     --- Store curves of set w1 not in w0 into setcurvs

      numcurves = 0     ! number of curves in set

      do i = 1, numcurw1

         icurnr = curvesw1(i)

         if ( iwork(icurnr)==0 ) then

!        --- iwork(icurnr)=0, hence curve not in set w0

            numcurves = numcurves+1
            setcurvs(numcurves) = icurnr

         end if  ! ( iwork(icurnr)==0 )

      end do  ! i = 1, numcurw1
      if ( debug ) call prinin ( setcurvs, numcurves, 'setcurvs' )

!     --- Find common user point of set w0 and setcurvs

      iwork = 0
      do i = 1, numcurw0

         icurnr = curvesw0(i)

         do j = curveusp(icurnr)+1, curveusp(icurnr+1)
            iwork(curveusp(j)) = 1    ! mark user point
         end do  ! j = curveusp(icurnr)+1, curveusp(icurnr+1)

      end do  ! i = 1, numcurw0

      do i = 1, numcurves

         icurnr = setcurvs(i)
         do j = curveusp(icurnr)+1, curveusp(icurnr+1)

            if ( iwork(curveusp(j))==1 ) then

!           --- Common user point found, return

               userpoint = curveusp(j)
               go to 900

            end if  ! ( iwork(curveusp(j))==0 )

         end do  ! j = curveusp(icurnr)+1, curveusp(icurnr+1)

      end do  ! i = 1, numcurves

!     --- If this stage is reached, no user point has been found
!         Return with error

      call errsub ( 3039, 0, 0, 0 )

900   deallocate ( iwork, setcurvs, stat = error )
      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'iwork', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/= 0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'userpoint', userpoint
         write(irefwr,*) 'End inicurvuserp'

      end if  ! ( debug )
      call erclos ( 'inicurvuserp' )

      end
