      subroutine prevalvecexpr1 ( work, iwork, ncount, result, text, &
                                  lenvec, irefwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-12-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read subexpression not containing any brackets and translate into a
!     vector.
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     vector
!     expression
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), ncount, lenvec, irefwork(ncount)
      double precision work(lenvec,*), result(lenvec)
      character (len=*) text

!     irefwork      i/o   integer work array of length ncount
!                         if irefwork(i) = 0, it concerns a constant
!                         otherwise (1) a vector
!     iwork         i/o   integer work array of length ncount
!                         iwork contains the operators that precede the numbers
!                         according to the following code:
!                          -4:  )
!                          -3:  (
!                          -2:  division
!                          -1:  subtraction
!                           1:  addition
!                           2:  multiplication
!                           3:  raise to the power
!                          10:  sqrt
!                          11:  sin
!                          12:  cos
!                          13:  exp
!                          14:  tan
!                          15:  asin
!                          16:  acos
!                          17:  atan
!     lenvec         i    Length of all vectors
!     ncount         i    Length of arrays iwork and irefwork
!     result         o    Result of the operation according to
!                         result = operator_i work(i) (summation convention)
!                         where operator_i is defined by iwork
!                         The priority rules of Fortran are applied
!     text           i    String variable containing some text for error message
!     work          i/o   work array of length ncount
!                         work contains the numbers that must be used to apply
!                         the operation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      logical extradebug

!     extradebug     If true extra debugging statements are carried out
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PREVALVECEXPRF Evaluate functions in vector expression
!     PREVALVECEXPRM Evaluate multiplications/subdivisions in vector expression
!     PREVALVECEXPRP Evaluate powers in vector expression
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For i := ncount (-1) 2 do
!        if ( iwork(i)=3 ) then
!           work(i-1) := work(i-1)**work(i)
!           shift work and iwork
!     For i := ncount (-1) 2 do
!        if ( iwork(i)=2 ) then
!           work(i-1) := work(i)*work(i-1)
!           ncount := ncount-1
!           shift work and iwork
!        else if ( iwork(i)=-2 ) then
!           work(i-1) := work(i-1)/work(i)
!           ncount := ncount-1
!           shift work and iwork
!
!     result := 0
!     For i := 1 (1) ncount do
!        result := result+iwork(i)*work(i)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prevalvecexpr1' )
      debug = .false. .and. ioutp>=0
      extradebug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prevalvecexpr1'
         call prinin ( iwork, ncount, 'iwork-in' )
         call prinin ( irefwork, ncount, 'irefwork-in' )
         if ( extradebug ) then
            call prinrl2 ( work, lenvec, ncount, 'work_in' )
         else
            do i = 1, ncount
               write(irefwr,4) 'work ', i, work(1:min(5,lenvec),i)
            end do  ! i = 1, ncount
         end if  ! ( extradebug )
  4      format ( a, 1x, i3, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First evaluate all functions

      call prevalvecexprf ( work, iwork, ncount, result, &
                            lenvec, irefwork  )

!     --- Next all powers

      call prevalvecexprp ( work, iwork, ncount, &
                            lenvec, irefwork  )

!     --- Next all multiplications and subdivisions

      call prevalvecexprm ( work, iwork, ncount, text, &
                            lenvec, irefwork  )

!     --- result := 0

      result = 0d0

!     --- For i := 1 (1) ncount do
!           result := result+iwork(i)*work(i)

      if ( debug ) then

         call prinin ( iwork, ncount, 'iwork' )
         call prinin ( irefwork, ncount, 'irefwork' )
      end if  ! ( debug )

      do i = 1, ncount
         if ( iwork(i)==0 ) then
            result = result+work(1:lenvec,i)
         else
            result = result+iwork(i)*work(1:lenvec,i)
         end if
      end do

      irefwork(1) = maxval(irefwork(1:ncount))

1000  call erclos ( 'prevalvecexpr1' )
      if ( debug ) then

         if ( extradebug ) then
            call prinrl ( result, lenvec, 'result' )
         else
            call prinrl ( result, min(5,lenvec), 'result' )
         end if  ! ( extradebug )
         write(irefwr,*) 'End prevalvecexpr1'
      end if  ! ( debug )

      end
