      subroutine prfindelm2obst ( coor, inode, xgauss, igauss, xi, eta, &
                                  found, phi  )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 06-02-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Determine local coordinates of xgauss on curved line
!
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     intersection
!     obstacle
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inode(*), igauss
      double precision  coor(3,*), xi, phi(9), xgauss(3,3), eta
      logical found

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     eta            i    ?
!     found          o    parameter that determines if jnode is in the
!                         element (true) or not (false)
!     igauss         i    Counter for gauss point
!     inode          i    Array of length 3 containing node numbers
!                         of element
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!     xi             o    local coordinate of jnode within the element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  i, j, k, jj, kk, iter, indxi(9), indeta(9)
      double precision mat(2,2), rhs(2), dphi(2,9), eps, delta(2), &
                       xcoor(2), xmin, xmax, ymin, ymax, zmin, zmax, &
                       dx, dy, dz, eps1, thetxi(3), theteta(3), &
                       ddxi(3), ddeta(3), &
                       invmat(2,2), det
      logical debug

!     ddeta          ?
!     ddxi           ?
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     delta          jacobian of element
!     det            ?
!     dphi           Array of length 9 to store the values of dphi/dxi
!                    according to:
!                               k
!                    dphi /dxi(x ) = dpdxi(i,k)
!                        i
!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     dz             length of vector in z-direction
!     eps            Local accuracy
!     eps1           Local accuracy
!     i              Counting variable
!     indeta         eta index for node
!     indxi          ?
!     invmat         ?
!     iter           Iteration sequence number
!     j              Counting variable
!     jj             ?
!     k              Counting variable
!     kk             ?
!     mat            ?
!     rhs            ?
!     theteta        ?
!     thetxi         ?
!     xcoor          x coordinate
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     ymax           Maximum y-value
!     ymin           Minimum y-value
!     zmax           Maximum z-value
!     zmin           Minimum z-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data indxi /1, 2, 3, 3, 3, 2, 1, 1, 2/
      data indeta /1, 1, 1, 2, 3, 3, 3, 2, 2/
! ======================================================================
!
      call eropen ( 'prfindelm2obst')
      debug=.false.

!     --- debug=.true.

!         --- find local coordinates of intersection point using
!            a newton raphson iteration scheme

      found=.false.

      if (debug) then
         write(*,*)'coordinate of node ', xgauss(igauss,1), &
                                          xgauss(igauss,2), &
                                          xgauss(igauss,3)
      end if

!     --- Determine if coordinate xgauss lies in square around
!         the line element. If not, go to end of file.

      eps1=2d-1
      xmin=1d10
      ymin=1d10
      zmin=1d10
      xmax=-1d10
      ymax=-1d10
      zmax=-1d10
      do i=1,9
         xmin=min(coor(1,inode(i)),xmin)
         ymin=min(coor(2,inode(i)),ymin)
         zmin=min(coor(3,inode(i)),zmin)
         xmax=max(coor(1,inode(i)),xmax)
         ymax=max(coor(2,inode(i)),ymax)
         zmax=max(coor(3,inode(i)),zmax)
      end do

      dx=xmax-xmin
      dy=ymax-ymin
      dz=zmax-zmin
      dx=max(dx,dy,dz)
      if (debug) then
            write(*,*)'xmin, xmax', xmin, xmax
            write(*,*)'ymin, ymax', ymin, ymax
            write(*,*)'zmin, zmax', zmin, zmax
      end if
      xmax=xmax+eps1*dx
      ymax=ymax+eps1*dx
      zmax=zmax+eps1*dx
      xmin=xmin-eps1*dx
      ymin=ymin-eps1*dx
      zmin=zmin-eps1*dx

      if (debug) then
            write(*,*)'xmin, xmax', xmin, xmax
            write(*,*)'ymin, ymax', ymin, ymax
            write(*,*)'zmin, zmax', zmin, zmax
      end if

      if ( ( xgauss(igauss,1).gt.xmax.or. &
             xgauss(igauss,1).lt.xmin ) .or. &
           ( xgauss(igauss,2).gt.ymax.or. &
             xgauss(igauss,2).lt.ymin ) .or. &
           ( xgauss(igauss,3).gt.zmax.or. &
             xgauss(igauss,3).lt.zmin ) ) then
         xi=2d0
         if (debug) write(*,*)'POINT OUT OF AREA -> EXIT'
         goto 1000
      end if

!     --- determine if x or y coordinate should be used for finding
!         the local coordinate xi based on the direction of the line
!         element

      if ( dx.le.dy .and. dx.le.dz ) then
         xcoor(1)=xgauss(igauss,2)
         xcoor(2)=xgauss(igauss,3)
         k=1
      else if ( dy.le.dx .and. dy.le.dz ) then
         xcoor(1)=xgauss(igauss,1)
         xcoor(2)=xgauss(igauss,3)
         k=2
      else
         xcoor(1)=xgauss(igauss,1)
         xcoor(2)=xgauss(igauss,2)
         k=3
      end if

      xi=5d-1
      eta=5d-1
      iter=0
      delta(1)=1d0
      delta(2)=1d0
      eps=1d-10
      if (debug) write(*,*)'neglected direction ', k
      do while ( ( abs(delta(1)).gt.eps .or. &
                   abs(delta(2)).gt.eps ) .and. &
                 abs(delta(1)).lt.1d1 .and. &
                 abs(delta(2)).lt.1d1 .and. &
                 iter.le.8 )

         iter = iter+1

!        --- compute some functions for local coordinate xi

         thetxi(1)  =  2d0*(xi-0.5d0)*(xi-1d0)
         thetxi(2)  = -4d0*xi*(xi-1d0)
         thetxi(3)  =  2d0*xi*(xi-0.5d0)
         theteta(1) =  2d0*(eta-0.5d0)*(eta-1d0)
         theteta(2) = -4d0*eta*(eta-1d0)
         theteta(3) =  2d0*eta*(eta-0.5d0)

         ddxi(1) = -(4d0*(1d0-xi)-1d0)
         ddxi(2) =  4d0*(1d0-2d0*xi)
         ddxi(3) =  4d0*xi-1d0
         ddeta(1) = -(4d0*(1d0-eta)-1d0)
         ddeta(2) = 4d0*(1d0-2d0*eta)
         ddeta(3) = 4d0*eta-1d0

         do i = 1, 9
            phi(i) = thetxi(indxi(i))*theteta(indeta(i))
            dphi(1,i) = ddxi(indxi(i))*theteta(indeta(i))
            dphi(2,i) = thetxi(indxi(i))*ddeta(indeta(i))
         end do

!        --- compute error delta

         kk=0
         do i=1,2
            do j=1,2
               mat(i,j)=0d0
            end do
            rhs(i)=0d0
         end do

         do i=1,3
            if (i.ne.k) then
               kk=kk+1
               do j=1,9
                  do jj=1,2
                     mat(kk,jj) = mat(kk,jj) + &
                                      dphi(jj,j)*coor(i,inode(j))
                  end do
                  rhs(kk) = rhs(kk) + phi(j)* coor(i,inode(j))
               end do
               rhs(kk) =  xcoor(kk) - rhs(kk)
            end if
         end do

         det=mat(1,1)*mat(2,2)-mat(1,2)*mat(2,1)
         invmat(1,1)=mat(2,2)/det
         invmat(2,2)=mat(1,1)/det
         invmat(1,2)=-mat(1,2)/det
         invmat(2,1)=-mat(2,1)/det

         do i=1,2
            delta(i)=invmat(i,1)*rhs(1)+invmat(i,2)*rhs(2)
         end do

!        --- update local coordinate

         xi=xi+delta(1)
         eta=eta+delta(2)

         if (debug)write(*,*)'xi  ', xi, '  delta1  ', delta(1)
         if (debug)write(*,*)'eta  ', eta, '  delta2  ', delta(2)

      end do

      if (abs(delta(1)).ge.1d-6 )  then
         xi=2d0
         goto 1000
      end if
      if (abs(delta(2)).ge.1d-6 ) then
         eta=2d0
         goto 1000
      end if

      thetxi(1)  =  2d0*(xi-0.5d0)*(xi-1d0)
      thetxi(2)  = -4d0*xi*(xi-1d0)
      thetxi(3)  =  2d0*xi*(xi-0.5d0)
      theteta(1) =  2d0*(eta-0.5d0)*(eta-1d0)
      theteta(2) = -4d0*eta*(eta-1d0)
      theteta(3) =  2d0*eta*(eta-0.5d0)

      do i = 1, 9
         phi(i)= thetxi(indxi(i))*theteta(indeta(i))
      end do

      if(debug)write(*,*)'phiprfind', (phi(i),i=1,9)

      if (xi.le.1d0+eps  .and. xi.ge.-eps .and. &
          eta.le.1d0+eps .and. eta.ge.-eps  ) then

!     --- local coordinate is found

         found=.true.

      end if

1000  call erclos ( 'prfindelm2obst' )
      end
