      subroutine prmakevector ( isol, iinvec, numvct, ioutvec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 12-02-2015 Extra parameter ioutvec
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 02-03-2011
!
!   copyright (c) 2011-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make new vector out of two old vectors
!     The components of the first vector are the first ones, &
!     the components of the second vector are the last
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     create
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: isol(*), iinvec(*), numvct
      integer, intent(in) :: ioutvec

!     iinvec         i    integer input array,
!                         Contents See subroutine prmanv
!     ioutvec        i    Sequence number of output vector in isol
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution arrays
!     numvct        i/o   Largest sequence number of vector that has been filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivec1, ivec2, indprf1, nunkp1, &
              number, itypvc, ivec, itemp

!     indprf1        indication if kprob1 part f has been filled (>0)
!                    or not (0)
!     itemp          Temporaray array to store information of new vector
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of output vector
!     ivec1          Sequence number of first vector
!     ivec2          Sequence number of second vector
!     number         The number-th unknown in each nodal point of the array
!                    is considered. If the array is complex number=2*i-1
!                    corresponds to the real part of array i and
!                    number=2*i to the imaginary part of array i
!     nunkp1         maximum number of unknowns per point in vector 1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INISTREXT      Find the structure of the vector with exactly
!                    NUMBER unknowns per point
!     PRININ         print 1d integer array
!     PRMAKEVECTOR1  Create new vector from two separate vectors
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3071   Number of degrees of freedom not constant in one of the two vectors
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmakevector' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmakevector'
         call prinin ( iinvec, 13, 'iinvec' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      ivec1 = iinvec(12)
      ivec2 = iinvec(13)
      if ( debug ) then
         write(irefwr,1) 'ioutvec, ivec1, ivec2', ioutvec, ivec1, ivec2
         write(irefwr,1) 'isol(ivec1)', isol(ivec1)
         write(irefwr,1) 'isol(ivec2)', isol(ivec2)
         write(irefwr,1) 'isol(ioutvec)', isol(ioutvec)
      end if  ! ( debug )

!     --- Activate arrays

      call sepactsolbf2 ( isol(ivec1) )
      indprf1 = indprfi
      nunkp1 = nunkpi
      call sepactsolbf2 ( isol(ivec2) )
      if ( ierror/=0 ) go to 1000

      if ( indprf1>0 .or. indprfi>0 ) then

!     --- indprf1>0 or indprfi>0, not yet implemented

         call errsub ( 3071, 0, 0, 0 )
         go to 1000

      end if  ! ( indprf1>0 .or. indprfi>0 )

!     --- Create a new vector with nunkp1+nunkpi degrees of freedom

      number = nunkp1+nunkpi

      if ( debug ) write(irefwr,1) 'number', number

      call inistrext ( number, itypvc, ivec )

      if ( debug ) write(irefwr,1) 'ivec, iprob', ivec, iprob

      if ( ierror/=0 ) go to 1000

!     --- Create output vector

      if ( ioutvec==ivec1 .or. ioutvec==ivec2 ) then

!     --- Create temporary vector in itemp

         itemp = 0
         call sepcreasolbf1 ( itemp, itypvc, ivec, names(ioutvec), &
                              ks(isol(ivec1))%compl )
         vectout => usoltemp
         call sepcreasolbf1 ( isol(ioutvec), itypvc, ivec, names(ioutvec), &
                              ks(isol(ivec1))%compl )

      else

!     --- Create new vector

         call sepcreasolbf1 ( isol(ioutvec), itypvc, ivec, names(ioutvec), &
                              ks(isol(ivec1))%compl )
         vectout => usoltemp

      end if  ! ( ioutvec==ivec1 .or. ioutvec==ivec2 )
      if ( ierror/=0 ) go to 1000

!     --- activate vectors

      vectin => ks(isol(ivec1))%sol
      vectin2 => ks(isol(ivec2))%sol

!     --- Create vector

      call prmakevector1 ( vectin, npoint, nunkp1, &
                           vectin2, nunkpi, vectout )

      if ( ioutvec==ivec1 .or. ioutvec==ivec2 ) then

!     --- Remove array

         call seprmsolinfo ( isol(ioutvec) )
         isol(ioutvec) = itemp

      end if  ! ( ioutvec==ivec1 .or. ioutvec==ivec2 )

      numvct = max(numvct,ioutvec)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmakevector'

      end if  ! ( debug )
      call erclos ( 'prmakevector' )

      end subroutine prmakevector
