      subroutine prvecfill ( iseqvec, vecs, &
                             numvecrhs, vectors, leng, nunkp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-12-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill all vectors of right-hand side of expression in array vectors
!     It is assumed that the complete vectors must be filled
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     expression
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqvec, vecs(*), numvecrhs, leng, nunkp
      double precision vectors(leng,numvecrhs)

!     iseqvec        i    sequence number of vector in left-hand side with
!                         to isol
!     leng           i    Length of the vectors to be filled
!     numvecrhs      i    Number of vectors in right-hand side
!                         Length of array vecs
!     nunkp          i    Maximum number of unknowns in a point
!     vecs           i    In this array of size numvecrhs the sequence numbers
!                         of the right-hand side vectors are stored
!     vectors        o    Array of size length x numvecrhs to store the vectors
!                         of the right-hand side in the nodes required
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iseqv, lenvec, j
      logical extradebug

!     extradebug     If true extra debugging statements are carried out
!     i              Counting variable
!     iseqv          Sequence number of keyword V in array code1
!     j              Counting variable
!     lenvec         Length of left-hand side vector in expression
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3163   Vector has wrong sequence number
!    3165   Not all vectors in the right-hand side of the vector expression
!           have the same length as the left-hand side
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prvecfill' )
      debug = .false. .and. ioutp>=0
      extradebug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prvecfill'
         write(irefwr,1) 'length, nunkp', leng, nunkp
         call prinin ( vecs, numvecrhs, 'vecs' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Loop over vectors

      do i = 1, numvecrhs

         iseqv = vecs(i)

         if ( debug ) write(irefwr,1) 'i, iseqv', i, iseqv
         if ( iseqv<=maxvc1 ) then

!        --- Standard sepran vector

            vectin => ks(isolut2d(iseqv))%sol
            lenvec = size(vectin)
            if ( debug ) write(irefwr,1) 'length, lenvec', leng, lenvec

            if ( lenvec==leng ) then

!           --- Both arrays have same length: copy vector

               call dcopy ( leng, vectin, 1, vectors(1,i), 1 )

            else if ( lenvec*nunkp==leng ) then

!           --- Array to be copied has one degree of freedom
!               Copy each component

               do j = 1, nunkp
                  call dcopy ( lenvec, vectin, 1, vectors(j,i), &
                               nunkp )
               end do  ! j = 1, nunkp

            else

!           --- Error leng not correct

               call errint ( lenvec, 1 )
               call errint ( i, 2 )
               call errint ( leng, 3 )
               call errchr ( names(iseqvec), 1 )
               call errchr ( names(iseqv), 2 )
               call errsub ( 3165, 3, 0, 2 )

            end if  ! ( lenvec==leng )

         else if ( iseqv==maxvc1+1 ) then

!        --- Coordinates
!            Copy vector

            if ( leng==ndim*npoint ) then

!           --- Coor has same length as vector

               call dcopy ( leng, coor, 1, vectors(:,i), 1 )

            else

!           --- Error coor has different length as vector

               call errint ( ndim*npoint, 1 )
               call errint ( i, 2 )
               call errint ( leng, 3 )
               call errchr ( names(iseqvec), 1 )
               call errchr ( 'coor', 2 )
               call errsub ( 3165, 3, 0, 2 )

            end if  ! ( leng==ndim*npoint )

         else if ( iseqv<=maxvc1+1+ndim ) then

!        --- One of the coordinates
!            activate array

            iseqv = iseqv-maxvc1-1

!           --- Copy vector

            if ( npoint==leng ) then

!           --- Both arrays have same length: copy vector

               vectors(1:npoint,i) = coor(iseqv,1:npoint)

            else if ( npoint*nunkp==leng ) then

!           --- Array to be copied has one degree of freedom
!               Copy each component

               do j = 1, nunkp
                  vectors(j:npoint*nunkp:nunkp,i) = coor(iseqv,1:npoint)
               end do  ! i = 1, nunkp

            else

!           --- Error leng not correct

               call errint ( npoint, 1 )
               call errint ( i, 2 )
               call errint ( leng, 3 )
               call errchr ( names(iseqvec), 1 )
               call errchr ( 'coor_x', 2 )
               call errsub ( 3165, 3, 0, 2 )

            end if  ! ( lenvec==leng )

         else

!        --- Unknown vector

            call errint ( iseqv, 1 )
            call errchr ( names(iseqvec), 1 )
            call errsub ( 3163, 1, 0, 1 )
            go to 1000

         end if

      end do  ! i = 1, numvecrhs

1000  if ( debug ) then

!     --- Debug information

         if ( extradebug ) then
            call prinrl2 ( vectors, leng, numvecrhs, 'vectors' )
         else

            do i = 1, numvecrhs
               call prinrl ( vectors(1,i), 5, 'vectors_i' )
            end do  ! i = 1, numvecrhs

         end if  ! ( extradebug )
         write(irefwr,*) 'End prvecfill'

      end if  ! ( debug )
      call erclos ( 'prvecfill' )

      end subroutine prvecfill
