      subroutine prvecfill1 ( isol, iseqvec, vecs, numvecrhs, vectors, &
                              leng, nnodes, nunkploc, idegfd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 17-01-2015 new options implemented
!        version  1.2  date 10-05-2014 correction nunkpi
!        version  1.1  date 11-03-2014 correction maxvc1
!
!   copyright (c) 2013-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill all right-hand side vectors in expression where either
!     idegfd>0 or nunkploc = 1
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     expression
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), iseqvec, vecs(*), numvecrhs, &
              leng, nnodes, nunkploc, idegfd
      double precision  vectors(leng,numvecrhs)

!     idegfd         i    Degree of freedom sequence number
!     iseqvec        i    sequence number of vector in left-hand side with
!                         to isol
!     isol           i    Standard SEPRAN array containing information of the
!                         solution arrays
!     leng           i    Length of the vectors to be filled
!     nnodes         i    Number of nodes in array nodes
!     numvecrhs      i    Number of vectors in right-hand side
!                         Length of array vecs
!     nunkploc        i    Maximum number of unknowns in a point
!     vecs           i    In this array of size numvecrhs the sequence numbers
!                         of the right-hand side vectors are stored
!     vectors        o    Array of size leng x numvecrhs to store the vectors
!                         of the right-hand side in the nodes required
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iseqv, idegfdloc

!     i              Counting variable
!     idegfdloc      local degree of freedom
!     iseqv          Sequence number of vector in isol
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRCOPSEPVEC3   Copy one degree of freedom of sepran vector in user vector
!     PRCOPYVEC2US   Copy part of sepran vector to user vector
!     PRINRL         Print 1d real vector
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3163   Vector has wrong sequence number
!    3166   Number of degrees of freedom in rhs vector less than in rhs vector
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prvecfill1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prvecfill1'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Loop over vectors

      do i = 1, numvecrhs

         iseqv = vecs(i)     ! sequence number of vector
         idegfdloc = idegfd  ! local degree of freedom

         if ( debug ) &
            write(irefwr,1) 'i, iseqv, idegfdloc', i, iseqv, idegfdloc

         if ( iseqv<=maxvc1 ) then

!        --- Standard sepran vector
!            activate array

            call sepactsolbf2 ( isol(iseqv) )

            if ( debug ) &
               write(irefwr,1) 'nunkpi, indprfi, nunkploc', &
                                nunkpi, indprfi, nunkploc

            if ( nunkpi==1 ) then

!           --- Rhs vector contains only 1 component

               idegfdloc = 1

            else if ( nunkpi<nunkploc ) then

!           --- Error nunkpi too small

               call errint ( i, 1 )
               call errint ( nunkpi, 2 )
               call errint ( nunkploc, 3 )
               call errchr ( names(iseqv), 1 )
               call errchr ( names(iseqvec), 2 )
               call errsub ( 3166, 3, 0, 2 )
               go to 1000

           end if  ! ( nunkpi<nunkploc )

!          --- Find starting address

            usoltemp => ks(isol(iseqv))%sol

!           --- Copy vector

            if ( indprfi==0 ) then

!           --- indprfi = 0, number of degrees of freedom fixed
!               Copy first degree of freedom into vectors(i)

               call prcopyvec2us ( usoltemp, vectors(1,i), nodes, &
                                   nnodes, nunkpi, idegfdloc )

            else

!           --- indprfi>0, number of degrees of freedom variable

               call prcopsepvec3 ( usoltemp, vectors(1,i), &
                                   idegfd, nodes, nnodes )

           end if  ! ( indprfi==0 )

         else if ( iseqv==maxvc1+1 ) then

!        --- Coordinates
!            Copy vector

            call prcopyvec2us ( coor, vectors(1,i), nodes, &
                                nnodes, ndim, 1 )

         else if ( iseqv<=maxvc1+1+ndim ) then

!        --- One of the coordinates
!            activate array

            iseqv = iseqv-maxvc1-1

!           --- Copy vector

            call prcopyvec2us ( coor, vectors(1,i), nodes, &
                                nnodes, ndim, iseqv )

         else

!        --- Unknown vector

            call errint ( iseqv, 1 )
            call errchr ( names(iseqvec), 1 )
            call errsub ( 3163, 1, 0, 1 )
            go to 1000

         end if

      end do  ! i = 1, numvecrhs

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( vectors(1,1), nnodes, 'vectors' )
         write(irefwr,*) 'End prvecfill1'

      end if  ! ( debug )
      call erclos ( 'prvecfill1' )

      end subroutine prvecfill1
