      subroutine readcoreal ( iseq, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 11-10-2013 Use readsrnotext
!        version  1.3  date 10-04-2013 Extra debug
!        version  1.2  date 20-04-2012 Indexed reals
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read part reals of input block CONSTANTS
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     constant
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecons
      use sepmoduleread
      use sepmoduleplot
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseq, iwork(*)

!     iseq           i    Sequence numberof vector in VECOLD/ISLOLD
!     iwork         i/o   integer work array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inum, i, iseq1, numbreal

!     i              Counting variable
!     inum           Number of numbers read in a record
!     iseq1          Shifted sequence number
!     numbreal       Number of entries in real array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer readarrindex

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READARRINDEX   Check if the name of a constant vector or varibale is
!                    followed by an index
!     READSRNOTEXT   Read a series of numbers from the input
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1611   Too many reals read
!    2143   Sequence number already in use
!    3100   Name of constant, variable or vector too long
!    3101   Index too large
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readcoreal' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readcoreal'
         write(irefwr,1) 'iseq, numreals, nchar', iseq, numreals, nchar
         write(irefwr,*) 'string ', string(1:nchar)
         call prinin ( typeitems, nitems, 'typeitems' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( nchar>32 ) then

!     --- Too many characters

         call errchr ( string, 1 )
         call errsub ( 3100, 0, 0, 1 )
         go to 1000

      end if  ! ( nchar>32 )

      if ( iseq==0 ) iseq = numreals+1
      iseq1 = iseq+maxints

      if ( iwork(iseq1)/=0 ) then

!     --- error, sequence number already in use

         call errint ( iseq, 1 )
         call errchr ( 'reals' , 1 )
         call errsub ( 2143, 1, 0, 1 )

      end if

      iwork(iseq1) = 1
      numreals = max(numreals,iseq)
      if ( numreals>maxreals ) then

!     --- Too many reals

         call errint ( numreals, 1 )
         call errint ( maxreals, 2 )
         call errsub ( 1611, 2, 0, 0 )
         call errcrd
         go to 1000

      end if  ! ( numreals>maxreals )

!     --- store name into chrlcons

      chrlcons(iseq) = string(1:nchar)

      numbreal = readarrindex()
      if ( debug ) write(irefwr,1) 'numbreal, iseq1', numbreal, iseq1

      dimcons(iseq1) = numbreal  ! dimension of constant
      ncharcons(iseq1) = nchar   ! number of characters of constant

      if ( numbreal>1 ) then

!     --- numbreal>1, hence indexed real

         if ( nchar>29 ) then

!        --- Too many characters

            call errchr ( string, 1 )
            call errsub ( 3100, 0, 0, 1 )
            go to 1000

         end if  ! ( nchar>29 )
         do i = 1, min(numbreal,9)
            write ( chrlcons(iseq+i-1), 100 ) string(1:nchar), i
100         format ( a,'(',i1,')' )
         end do  ! i = 1, min(numbreal,9)

         if ( numbreal>9 ) then

            if ( nchar>28 ) then

!           --- Too many characters

               call errchr ( string, 1 )
               call errsub ( 3100, 0, 0, 1 )
               go to 1000

            end if  ! ( nchar>29 )
            do i = 10, min(numbreal,99)
               write ( chrlcons(iseq+i-1), 110 ) string(1:nchar), i
110            format ( a, '(',i2,')' )
            end do  ! i = 10, min(numbreal,99)

         end if  ! ( numbreal>9 )

         if ( numbreal>=100 ) then

!        --- The upper bound for index is 99 at this moment

            call errint ( numbreal, 1 )
            call errint ( 99, 2 )
            call errsub ( 3101, 2, 0, 0 )
            go to 1000

         end if  ! ( numbreal>100 )

      end if  ! ( numbreal>1 )

!     --- read numbers

      if ( debug ) write(irefwr,1) 'iseq, maxreals', iseq, maxreals
      call readsrnotext ( rlcons(iseq), maxreals-iseq+1, inum )

      if ( debug .and. inum>0 ) &
         call prinrl ( rlcons(iseq), inum, 'reals' )

      numbreal = max(numbreal, inum)
      numreals = max(numreals,iseq+numbreal-1)

      if ( debug ) &
         write(irefwr,1) 'numbreal, numreals', numbreal, numreals

      if ( numreals>maxreals ) then

!     --- Too many reals

         call errint ( numreals, 1 )
         call errint ( maxreals, 2 )
         call errsub ( 1611, 2, 0, 0 )
         call errcrd
         go to 1000

      end if  ! ( numreals>maxreals )

      do i = 1, numbreal-1

         if ( iwork(iseq1+i)/=0 ) then

!        --- error, sequence number already in use

            call errint ( iseq+i, 1 )
            call errchr ( 'reals' , 1 )
            call errsub ( 2143, 1, 0, 1 )

         end if
         iwork(iseq1+i) = 1

      end do  ! i = 1, numbreal-1

1000  call erclos ( 'readcoreal' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readcoreal'

      end if  ! ( debug )

      end
