      subroutine er0034 ( isubr, ictime, isol, iseqnr, islold, maxvecsol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.0  date 26-08-2016 use sepmoduleinput
!        version  5.15 date 24-12-2014 New definition clinsl
!        version  5.14 date 30-12-2013 Jump in case of errors
!
!   copyright (c) 1991-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     ER0034 creates a number of SEPRAN vectors and fills (a part of)
!     these arrays if required
!     ER0034 replaces the call of the SEPRAN subroutine:
!     CREATV: Same meaning as ER0034, but without reading of the
!             standard input file
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduleinput
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: isubr, ictime, isol(*), iseqnr, &
                 islold(*), maxvecsol

!     ictime         i    Indication if information must be read from standard
!                         input file ( 0 ) or that information read before is
!                         used ( 1 )
!     iseqnr         i    Sequence number with respect to the storage in array
!                         ISEPIN
!     islold         i    User input array in which the user puts information
!                         of all preceding solutions
!                         Integer array of length 5 x numold
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution array
!                         Usually isol has the structure of a solution vector,
!                         hence ISOL is used as actual argument.
!                         isol may contain one vector, in which case isol is an
!                         integer array of length 5.
!                         If isol must contain information of n (>1) vectors,
!                         then isol should be declared as:
!                            INTEGER isol(1;5,1:n)
!                         isol(.,i) refers to the i-th vector, its starting
!                         address is given by isol(1,i).
!     isubr         i/o   Indicates the type of calling subroutine according to
!                         isubr = isubrout+100*(ivecsq-1) with
!                         isubrout sequence number of caling subroutine:
!                             1:  CREATE
!                             2:  PRESDF
!                             3:  CREAVC
!                             4:  PRESTM
!                             5:  ERCRET
!                             6:  ERPRES
!                             7:  ERPRES called from structure
!                             8:  ERCRET called from structure
!                             9:  ERCRET called from structure (change vector)
!                         ivecsq sequence number of first vector
!                         At output isubr contains the maximum sequence number
!                         of the array created
!     maxvecsol      i    Largest sequence number of vector in isol
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: leninp, lenrin, i, numveclc, &
                 ivecsq, isubrloc, numarrays, isubrtmp, &
                 iseqarrsav, iseqarrmin, iseqarrmax, maxcal, ipos
      integer, pointer :: inpcre(:), inpcrecop(:)
      double precision, pointer :: rincre(:)
      logical :: inputfound, allocrincre

!     allocrincre    if true rincre has been allocated locally
!     i              Counting variable
!     inpcre         The integer input read is stored in array INPCRE from
!                    position 1. Hence in the call INPCRE(ISTART), with
!                    ISTART the starting position must be used.
!                    INPCRE will be filled in correspondence with array
!                    INPCRE for subroutine CREATV
!     inpcrecop      Copy of inpcre but adapted for actual values of
!                    scalars
!     inputfound     If true an input block for create or presdf has been read
!     ipos           Position of input arrays in sepmoduleinput
!     iseqarrmax     Maximum value of iseqarray in loop
!     iseqarrmin     Minimum value of iseqarray in loop
!     iseqarrsav     Help parameter to store old value of iseqarray
!     isubrloc       Local (original) value of isubr
!     isubrtmp       Temporary value of isubr
!     ivecsq         Sequence number of vector in ivectr
!     leninp         Computed length of integer input array
!     lenrin         Computed length of array user
!     maxcal         Maximum number of coefficient inputs read
!     numarrays      Number of indexed vectors in vector array
!     numveclc         Number of vectors to be created by subroutine CREATE
!     rincre         Double precision array containing the values of the
!                    constant if KFUN < 0
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CREATV01       Create a SEPRAN real or complex array (vector)
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCREA         Read input concerning CREATE VECTOR or boundary conditions
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHINPSOL    Adapt input array with respect to variables
!     INIREDUCEVEC   Compute the reduced value of ivecin which indicates the
!                    first sequence number of a (series of) vectors, as well as
!                    the dimension of the series
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     813   ICTIME out of range
!    2104   Vector number of vector to be created too large
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'er0034' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from er0034'
         write(irefwr,1) 'isubr, iseqnr, ictime', isubr, iseqnr, ictime
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check input

      if ( isubr<3 .and. (ictime<0 .or. ictime>1) ) then
         call errint ( ictime, 1 )
         call errsub ( 813, 1, 0, 0 )
      end if
      allocrincre = .false.

!     --- Read input and store it in arrays iuser and user

      if ( abs(isubr)<=2 ) then

!     --- ISUBR = 1, CREATE or ISUBR = 2, PRESDF/TM

         isubrloc = abs(isubr)+3
         if ( ictime==0 ) then

!        --- Read input from standard input file

            if ( isubr<0 ) then

!           --- isubr < 0, call by stand alone subroutine

               call ercrea ( isubr, .false. )

            else

!           --- isubr > 0, standard case

               call ercrea ( isubr+1, .false. )

            end if  ! ( isubr<0 )
            call sepgetinpfinf ( isubrloc, iseqnr, ipos, maxcal )

         else if ( iinputarrs(5)%maxcall>0 ) then

!        --- input has been read before

            call sepgetinpfinf ( isubrloc, iseqnr, ipos, maxcal )

         else

!        --- no input, use defaults

            ipos = 0

         end if ! ( ictime==0 )

      else if ( isubr<=4 ) then

!     --- ISUBR>2, PRESBC or CREATN

         isubrloc = isubr+1
         call sepgetinpfinf ( isubrloc, iseqnr, ipos, maxcal )
         if ( iseqnr<=iinputarrs(isubrloc)%maxsub ) &
            iinputarrs(isubrloc)%maxcall = &
            max ( iinputarrs(isubrloc)%maxcall, iseqnr )

      else

!     --- ISUBR>4, PRESBC or CREATN
!         create := 4, presbc := 5

         isubrloc = mod(isubr,100)-1
         if ( isubrloc==6 ) then
            isubrloc = 5   ! presbc
         else if ( isubrloc==7 .or. isubrloc==8 ) then
            isubrloc = 4   ! create or change_vector
         end if  ! ( isubrloc==6 )

         if ( iseqnr<=iinputarrs(isubrloc)%maxsub ) &
            iinputarrs(isubrloc)%maxcall = &
            max ( iinputarrs(isubrloc)%maxcall, iseqnr )
         call sepgetinpfinf ( isubrloc, iseqnr, ipos, maxcal )

         if ( debug ) &
            write(irefwr,1) 'isubrloc, iseqnr, ipos, isubr', &
                             isubrloc, iseqnr, ipos, isubr

      end if
      if ( ierror/=0 ) go to 1000

      if ( debug ) &
         write(irefwr,1) 'ictime, isubrloc, iseqnr, ipos', &
                          ictime, isubrloc, iseqnr, ipos

      inputfound = ipos>0
      if ( inputfound ) inputfound = in(ipos)%used

      if ( debug ) write(irefwr,*) 'inputfound ', inputfound

      if ( inputfound ) then

!     --- Input for subroutine read before
!         Copy integer input in inpcrecop and adapt for scalars

         inpcre => in(ipos)%iinput
         leninp = size(in(ipos)%iinput)
         allocate ( inpcrecop(leninp), stat = error )
         if ( error/=0 ) call eralloc ( error, leninp, 'inpcrecop' )
         if ( ierror/=0 ) go to 1000

         inpcrecop = inpcre

!        --- Adapt variables

         call inichinpsol ( inpcrecop, leninp )
         if ( associated(re(ipos)%rinput) ) then
            rincre => re(ipos)%rinput
            lenrin = size(re(ipos)%rinput)
         else
            allocate(rincre(1))
            lenrin = 0
            allocrincre = .true.
         end if  ! ( associated(re(ipos)%rinput) )

      else

!     --- No input for subroutine found, use defaults

         leninp = 7
         allocate ( inpcrecop(leninp), rincre(1), stat = error )
         if ( error/=0 ) call eralloc ( error, leninp, 'inpcrecop' )
         if ( ierror/=0 ) go to 1000
         allocrincre = .true.

         inpcrecop(2) = 1
         inpcrecop(3) = 0
         inpcrecop(4) = 1
         inpcrecop(5) = 1
         inpcrecop(6) = 0
         inpcrecop(7) = 0
         lenrin = 0

      end if  ! ( associated(re(ipos)%rinput) )

      if ( debug ) call prinin ( inpcrecop, leninp, 'inpcrecop' )

!     --- Create vectors

      ivecsq = abs(isubr)/100+1
      numveclc = inpcrecop(2)

      isubrtmp = mod(abs(isubr),100)

      iseqarrsav = iseqarray
      if ( iseqarray==0 ) then

!     --- iseqarray = 0, check if loop is necessary

         call inireducevec ( ivecsq, ivecsq, numarrays )
         iseqarrmin = 1
         iseqarrmax = numarrays

      else

!     --- iseqarray # 0, use iseqarray

         iseqarrmin = iseqarray
         iseqarrmax = iseqarray

      end if  ! ( iseqarray==0 )

      ivecsq = ivecsq-2

!     --- Loop over all vectors

      do iseqarray = iseqarrmin, iseqarrmax

         ivecsq = ivecsq+1
         if ( debug ) &
            write(irefwr,1) 'ivecsq, numveclc, maxvecsol', &
                             ivecsq, numveclc, maxvecsol
         if ( ivecsq+numveclc>maxvecsol ) then

!        --- Too many vectors must be created

            call errint ( ivecsq+1, 1 )
            call errint ( ivecsq+numveclc, 2 )
            call errint ( maxvecsol, 3 )
            call errsub ( 2104, 3, 0, 0 )

         end if
         if ( ierror/=0 ) go to 1000

         inpcrecop(1) = -(isubrtmp+100*ivecsq)

         if ( debug ) write(irefwr,*) 'before creatv01'
         call creatv01 ( isol, inpcrecop, rincre, islold )
         if ( debug ) write(irefwr,*) 'after creatv01'

      end do  ! iseqarray = iseqarrmin, iseqarrmax

      iseqarray = iseqarrsav

      if ( isubr>4 ) isubr = inpcrecop(1) ! maximum sequence number created

!     --- deallocate temporary array and disconnect pointers

      deallocate ( inpcrecop, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'inpcrecop' )
      if ( ierror/=0 ) go to 1000

      if ( associated(inpcre) ) nullify(inpcre)
      if ( allocrincre ) then
         deallocate ( rincre )
      else if ( associated(rincre) ) then
         nullify(rincre)
      end if  ! ( lenrin==0 )

1000  if ( ierror==0 .and. debug ) then

!     --- Debug information

         call prinin1 ( isol(ivecsq+1), numveclc, 5, 'isol' )
         write(irefwr,1) 'isubr ', isubr
         write(irefwr,*) 'End er0034'

      end if  ! ( debug )
      call erclos ( 'er0034' )

      end
