      subroutine ercofs ( kcode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 26-08-2016 use sepmoduleinput
!        version  4.0  date 30-10-2014 Extra parameter kmesh
!        version  3.10 date 09-03-2013 Correction length code
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read input concerning the coefficients from the standard input file
!     Store the result in sepmoduleinput
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulecoeffs
      use sepmoduleinput
      use sepmoduleread1
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: kcode

!     kcode          i    Return code of main keyword in READAL
!                         Possible values:
!                          0   First record not read before
!                          4   COEFFICIENTS found
!                         22   CHANGE found
!                          9   COMPLEX found
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ipos, isubr, maxcall, iseqrdcof
      logical compl

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!                    they are not
!     ipos           Position of input arrays in sepmoduleinput
!     iseqrdcof      Coefficient sequence number to be read in case of kcode=0
!                    That is the case that input has not been read before
!     isubr          Indicates type of calling subroutine for READ02
!                    Possible values:
!                    3  call in case of fill coefficients
!                    4  call in case of change coefficients
!     maxcall        Maximum value of icall in sepmoduleinp
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOPYINPUT    Copy input read by a read subroutine into sepmoduleinput
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPFILLKPROB   Fill parameters in module sepmodulekprob
!     SEPREADCOEFFS  Read input concerning the coefficients
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    Input may be read from the standard input file
!    The first record is interpreted by this subroutine, the reading
!    of the rest of the records is performed by subroutine READ02
!    The first record must have the following shape:
!    [COMPLEX] COEFFICIENTS [,PROBLEM = j], [SEQUENCE_NUMBER = k]
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     628   First record read is not COEFFICIENTS
!     699   Problem number incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Interpret first record
!   Check IPROB and ISEQ
!   reserve space for input arrays
!   Read rest of input until END has been found
!   Make input arrays permanent and store references in isepi3
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data iseqrdcof /0/
! ======================================================================
!
      call eropen ( 'ercofs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercofs'

      end if
      if ( ierror/=0 ) go to 1000
      nncode = 2
      itype(1:2) = 0
      ncode(1) = 4
      ncode(2) = 3
      code(1) = 'PROBLEM'
      code(2) = 'SEQUENCE_NUMBER'
      tabmin(1:2) = 1d0
      ncodmx(1) = 7
      ncodmx(2) = 15
      nmain = 0

      if ( kcode==0 ) then

!     --- First record must be read

         iseqrdcof = iseqrdcof+1   ! raise iseqrdcof in case more coefficient
                                   ! blocks are read
         iseqcofs = iseqrdcof      ! set iseqcofs
         ityprd = 1
         call asknxt
         if ( ityprd/=4 ) then

!        --- First record does not start with a keyword

            call errsub ( 628, 0, 0, 0 )
            call errcrd
            go to 1000

         end if
         if ( string(1:4)=='COMP' ) then

!        --- Complex found, i.e. complex coefficients

            call asknxt
            compl = .true.
            if ( ityprd/=4 ) then

!           --- First record does not start with a keyword

               call errsub ( 628, 0, 0, 0 )
               call errcrd
               go to 1000

            end if
         end if
         if ( string(1:4)/='COEF' ) then
            call errsub ( 628, 0, 0, 0 )
            call errcrd
            go to 1000
         end if

      end if  ! ( kcode==0 )

      if ( kcode==9 ) then

!     --- COMPLEX found, read next keyword (must be COEFFICIENT)

         compl = .true.
         call asknxt
         if ( ityprd/=4 ) then
            call errsub ( 628, 0, 0, 0 )
            call errcrd
         else if ( string(1:4)/='COEF' ) then
            call errsub ( 628, 0, 0, 0 )
            call errcrd
         end if

      else if ( kcode==22 ) then

!     --- CHANGE found, read next keyword (must be [COMPLEX] COEFFICIENT)

         compl = .true.
         call asknxt
         if ( ityprd/=4 ) then
            call errsub ( 628, 0, 0, 0 )
            call errcrd
         else
            if ( string(1:4)=='COMP' ) then
               compl = .true.
               call asknxt
               if ( ityprd/=4 ) then
                  call errsub ( 628, 0, 0, 0 )
                  call errcrd
               end if
            else
               compl = .false.
            end if
            if ( string(1:4)/='COEF' ) then
               call errsub ( 628, 0, 0, 0 )
               call errcrd
            end if

         end if
      else

         compl = .false.

      end if
      if ( kcode==22 ) then
         isubr = 4
      else
         isubr = 3
      end if

!     --- Read rest of record

      if ( iprob>nprob ) iprob = 0
      table(1) = min(iprob+1,nprob)
      if ( kcode==0 ) then
         table(2) = iseqcofs
      else if ( isubr==3 ) then
         table(2) = iseqcofs  + 1
      else
         table(2) = iseqchcf  + 1
      end if
      tabmax(1) = nprob
      tabmax(2) = iinputarrs(isubr+3)%maxsub
      iretur = 0
      call sepreadsetkeyw
      iprob = nint(table(1))
      if ( isubr==3 ) then
         iseqcofs = nint(table(2))
      else
         iseqchcf = nint(table(2))
      end if
      if ( iprob<1 .or. iprob>nprob ) then
         call errint ( iprob, 1 )
         call errint ( nprob, 2 )
         call errsub ( 699, 2, 0, 0)
      end if
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      indprf = kprobloc(19)
      kprobf => kprobpart%kprobf
      call sepfillkprob

!     --- Read input and store in arrays iincof and rincof

      nrcof = ( nelgrp+numnat+nglgrp ) * 100
      nicof = 4 * nrcof

      allocate ( iincof(nicof), rincof(nrcof), stat = error )
      if ( error/=0 ) call eralloc ( error, 5*nrcof, 'iwork' )
      if ( ierror/=0 ) go to 1000

      rincof = 0d0
      iincof = 0

      call sepreadcoeffs ( compl, isubr )

!     --- Make input arrays permanent

      if ( isubr==3 ) then
         call ercopyinput ( isubr+3, iseqcofs, 'COEFFICIENTS', iincof, &
                            rincof, iincof, nicof, nrcof, 0 )
      else
         call ercopyinput ( isubr+3, iseqchcf, 'CHANGE COEFFICIENTS', iincof, &
                            rincof, iincof, nicof, nrcof, 0 )
      end if

      if ( debug ) then

!     --- Debug information

         call prinin ( iincof, nicof, 'iincof' )
         if ( nrcof>0 ) call prinrl ( rincof, nrcof, 'rincof' )

      end if  ! ( debug )

      deallocate ( iincof, rincof, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iincof' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ercofs'

      end if
      call erclos ( 'ercofs' )

      end subroutine ercofs
