      subroutine erderv ( numveclc, iinput, iavs, iseq, &
                          itimelc, time, iter, nameavs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.6  date 24-12-2014 New definition clinsl
!        version  5.5  date 19-04-2014 Extension of iinput
!        version  5.4  date 09-10-2013 Change of dimension of iinput
!
!   copyright (c) 1988-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Help subroutine for subroutine OUTPUT / OUTTIM (PROUTP)
!     This subroutine performs the subsequent calls of subroutine DERIVA.
!     ERDERV calls the following SEPRAN subroutines:
!            DERIVA: Computes derived quantities
!            PRINSL: Write arrays to a file
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     output
!     read
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulevecs
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numveclc, iinput(15,numveclc), &
              itimelc, iseq, iavs, iter, nameavs
      double precision :: time

!     iavs           i    Indicates which kind of output must be created.
!                         Possible values:
!                         0:  Create output for SEPPOST only in the form of file
!                             sepcomp.out
!                         1:  Create output for AVS only in the form of a file
!                             sepavs_xxx.yyy
!                             With erderv a time sequence number of the shape
!                             001, 002 etc and yyy a vector sequence number
!                             of the shape 001, 002 etc
!                         2:  Create output for SEPPOST in the form of file
!                             sepcomp.out
!                             and for AVS in the form of a file sepavs_xxx.yyy
!                         3:  Create output for SEPPOST in the form of file
!                             sepcomp.out and for AVS in the form of a file
!                             sepavs_xxx.yyy
!     iinput         i    Contains the actual integer information from
!                         the standard input file
!                         Length: 8 x numveclc integers
!                         iinput is filled as follows:
!                         iinput(1,i):  Reference number of array for user
!                               (2,i):  icheld
!                               (3,i):  ix
!                               (4,i):  jdegfd
!                               (5,i):  ivec
!                               (6,i):  solution number inpvc0
!                               (7,i):  solution number inpvc1
!                               (8,i):  sequence number coefficients
!                         icheld, ix, jdegfd and ivec are standard
!                         parameters for the SEPRAN subroutine DERIVA (See the
!                         SEPRAN Programmer's Guide).
!                         i refers to ith call of DERIVA
!     iseq           i    Sequence number of vector in VECOLD/ISLOLD
!     iter           i    Iteration sequence number
!     itimelc          i    Integer time sequence number
!     nameavs        i    Sequence number of name of avs file in array
!                         file_names
!     numveclc         i    Number of vectors stored in isol
!     time           i    Actual time
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivec, idummy(1), ichois, i, iprob, iseqcf
      integer, target :: ioutvc

!     i              Counting variable
!     ichois         Standard input parameter for subroutine DERIVA
!     idummy         Dummy parameter
!     ioutvc         SEPRAN array of length 5 in which the output
!                    of DERIVA is stored
!     iprob          Actual problem number
!     iseqcf         Sequence number corresponding to the coefficients
!     ivec           Standard input parameter for subroutine DERIVA, indicating
!                    the type of vector that must be built
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERLINS         Help subroutine to call FILCOF, which fills coefficients in
!                    IUSER and USER
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRIAVS         Write solution array or array of special structure to file
!                    sepavs_xxx.yyy (AVS file)
!     PRININ1        print 2d integer array
!     SEPDERIVA      calculate values derived from solution
!     SEPPRINOUTP    Write solution vectors to sepcomp.out
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     823   The solution vector required for INPVC0 does not exist
!     824   The solution vector required for INPVC1 does not exist
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ioutvc /0/
! ======================================================================
!
      call eropen ('erderv' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erderv'
         write(irefwr,1) 'numveclc', numveclc
         call prinin1 ( iinput, numveclc, 9, 'iinput' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, numveclc
         ivec = iinput(5,i)
         if ( ivec==0 ) then
            ichois = 1
         else
            ichois = 2
         end if
         if ( iinput(6,i)>1 ) then

!        --- iinput(6,i)>1, check if array isol has been filled

            if ( isol2d(iinput(6,i))<=0 .or. isol2d(iinput(6,i)) &
                 >lastsol ) call errsub ( 823, 0, 0, 0 )
         end if
         if ( iinput(7,i)>1 ) then

!        --- iinput(7,i)>1, check if array isol2d has been filled

            if ( isol2d(iinput(7,i))<=0 .or. isol2d(iinput(7,i)) &
                 >lastsol ) call errsub ( 824, 0, 0, 0 )
         end if
         if ( iinput(8,i)>0 ) then

!        --- iinput(8,i)>0, coefficient sequence number has been given

            iseqcf = iinput(8,i)
            iprob = ks(isol2d(iinput(6,i)))%iprob
            if ( iseqcf/=iseqcoef ) then
               call erlins ( 3, iprob, iseqcf )
            end if

!           --- Copy arrays from buffer to IUSER and USER

            iuser => iuser1(iprob)%iuser
            user => iuser1(iprob)%user

            call sepderiva ( ichois, iinput(2,i), &
                             iinput(3,i), iinput(4,i), ivec, ioutvc, &
                             isol2d(iinput(6,i)), isol2d(iinput(7,i)), &
                             idummy )

         else

!        --- iinput(8,i) = 0, use iuser, user

            call sepderiva ( ichois, iinput(2,i), &
                             iinput(3,i), iinput(4,i), ivec, ioutvc, &
                             isol2d(iinput(6,i)), isol2d(iinput(7,i)), &
                             idummy )

         end if

         isolut => ioutvc
         if ( iavs/=2 ) &
            call  sepprinoutp ( isolut, iinput(1,i), iter, time )
         iseq = iseq+1
         if ( iavs/=1 ) call priavs ( itimelc, iseq, time, nameavs )

      end do
      if ( ioutvc>0 ) call seprmsolinfo (ioutvc )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erderv'

      end if  ! ( debug )
      call erclos ( 'erderv' )

      end subroutine erderv
