      subroutine erlinc ( isubr, iproblc, iseqnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.5  date 24-12-2014 New definition clinsl
!        version  2.4  date 13-12-2014 Extra debug
!        version  2.3  date 17-08-2013 Debug statements
!
!   copyright (c) 1993-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   ERLINC performs the actual calling of subroutine CHNCOF for
!   subroutine NLNPRB.
!   It fills the arrays IUSER and USER positions 1 to 5
!   Warning: this subroutine uses memory managment subroutines, so
!            all pointers must be recopied after the call of ERLINC
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isubr, iproblc, iseqnr

!    iproblc     i   Problem sequence number
!    iseqnr    i   Sequence number with respect to the storage in array ISEPIN
!    isubr     i   Indicates by which subroutine ERLINC is called indirectly
!                  Possible values:
!                  1:  LINSOL
!                  2:  LINSTM
!                  3:  LINPRB
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nelgrpkpr, jsubr, lenius, lenusr

!     jsubr          Parameter isubr with respect to TO0093
!     lenius         Computed length of array IUSER
!     lenusr         Computed length of array USER
!     nelgrpkpr      nelgrp+numnat
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPCREATEIBUF  Create permanent space in array IBUFFR
!     SEPCREATERBUF  Create permanent space in array BUFFER
!     SEPFILLKPROB   Fill parameters in module sepmodulekprob
!     TO0093         Body of subroutines FILCOF and CHNCOF
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     699   Problem number incorrect
!    1724   Array IUSER has not been filled correctly
!    1909   Too few space for array IUSER
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Estimate maximal length of arrays IUSER and USER
!   Reserve temporary space for these arrays
!   Copy old arrays IUSER and USER in the temporary positions
!   Fill rest of iuser and user through CHNCOF (TO0093)
!   Copy new arrays IUSER and USER in the final positions
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erlinc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erlinc'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Estimate maximal length of arrays IUSER and USER

      if ( iproblc==0 ) iproblc = iprobls
      iprob = iproblc
      if ( iprob<0 .or. iprob>nprob ) then
         call errint ( iprob, 1 )
         call errint ( nprob, 2 )
         call errsub ( 699, 2, 0, 0 )
      end if
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      call sepfillkprob

!     --- Reserve temporary space for these arrays
!         The arrays IUSER and USER are put at the tail of IBUFFR and
!         nbuffr is decreased.
!         In this way CHNCOF will not overwrite IUSER and USER.
!         Later on new starting positions are computed by INI071
!         and the actual arrays are copied to these new positions.

      nelgrpkpr = nelgrp + numnat
      allocate ( iuser(5+nelgrpkpr*101), user(5+nelgrpkpr*50), stat = error )
      if ( error/=0 ) call eralloc ( error, nelgrpkpr*101, 'iuser' )
      if ( ierror/=0 ) go to 1000

!     --- Copy old arrays iuser and user into these positions

      if ( .not. associated(iuser1(iprob)%iuser) ) then

!     --- The arrays iuser and user have not yet been filled

          call errsub ( 1724, 0, 0, 0)
          go to 1000

      end if
      lenius = size(iuser1(iprob)%iuser)
      iuser(1:lenius) = iuser1(iprob)%iuser
      if ( lenius>5+101*nelgrpkpr ) then

!     --- error too few space for iuser

         call errint ( lenius, 1 )
         call errint ( 5+101*nelgrpkpr, 2 )
         call errsub ( 1909, 2, 0, 0 )

      end if
      lenusr = size(iuser1(iprob)%user)
      user(1:lenusr) = iuser1(iprob)%user
      if ( lenusr>5+50*nelgrpkpr ) then

!     --- error too few space for user

         call errint ( lenusr, 1 )
         call errint ( 5+50*nelgrpkpr, 2 )
         call errsub ( 1909, 2, 0, 0 )

      end if

!     --- Fill rest of iuser and user through CHNCOF (TO0093)

      if ( isubr==3 ) then
         jsubr = 4
      else
         jsubr = 2
      end if
      iuser(1) = 5+101*nelgrpkpr
      user(1) = 5+50*nelgrpkpr

      call to0093 ( jsubr, iprob, iseqnr )

!     --- Make arrays IUSER and USER permanent

      lenius = iuser(4)
      iuser(1) = lenius
      iuser(3) = lenius
      lenusr = nint(user(4))
      user(1) = lenusr
      user(3) = lenusr

      if ( associated(iuser1(iprob)%iuser) ) &
         deallocate ( iuser1(iprob)%iuser, iuser1(iprob)%user )
      allocate ( iuser1(iprob)%iuser(lenius), iuser1(iprob)%user(lenusr) )
      iuser1(iprob)%iuser = iuser(1:lenius)
      iuser1(iprob)%user = user(1:lenusr)
      if ( associated(icofius(iprob)%iuser) ) &
         deallocate ( icofius(iprob)%iuser, icofius(iprob)%user )
      allocate ( icofius(iprob)%iuser(lenius), icofius(iprob)%user(lenusr) )
      icofius(iprob)%iuser = iuser(1:lenius)
      icofius(iprob)%user = user(1:lenusr)

      deallocate ( iuser, user, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iuser' )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iuser1(iprob)%iuser, lenius, 'iuser' )
         call prinrl ( iuser1(iprob)%user, lenusr, 'iuser' )
         write(irefwr,*) 'End erlinc'

      end if  ! ( debug )
      call erclos ( 'erlinc' )

      end subroutine erlinc
