      subroutine ernlnsin ( iseqnr, iinput, rinput, leniinp, lenrinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 26-08-2016 use sepmoduleinput
!        version  1.2  date 22-11-2012 Long integers
!        version  1.1  date 29-10-1999 Extend isepi3
!        version  1.0  date 21-02-1999
!
!   copyright (c) 1999-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill input concerning the non-linear solver from the arrays iinput and
!    rinput
!    Store the result in sepmoduleinput
! **********************************************************************
!
!                       KEYWORDS
!
!     non_linear
!     input
!     fill
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqnr, iinput(*), leniinp, lenrinp
      double precision rinput(*)

!     iinput     i    integer input array for this specific subroutine
!                     For a description of all positions, see subroutine
!                     READNL
!     iseqnr     i    Sequence number of input with respect to the storage in
!                     array ISEPIN for this specific subroutine
!     leniinp    i    Length of array iinput, i.e. number of positions that
!                     have been filled by the user. For all other positions
!                     default values are used.
!     lenrinp    i    Length of array rinput, i.e. number of positions that
!                     have been filled by the user. For all other positions
!                     default values are used.
!     nsubr      i    Number of different subroutines that may store their
!                     information in ISEPIN
!     rinput     i    Real input array for this specific subroutine
!                     For a description of all positions, see subroutine
!                     READNL
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, pointer :: inpsol(:)
      integer i, maxsubr, ninsol, nrisol
      double precision, pointer :: rinsol(:)

!     i              Counting variable
!     inpsol         Integer input array for subroutine solvel
!                    See read03 for a description
!     maxsubr        maximum number of different input arrays for this
!                    subroutine
!     ninsol         Length of array INPSOL
!     nrisol         Length of array RINSOL
!     rinsol         Contains information concerning the linear solver
!                    See the Programmers Guide for a description
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOPYINPUT    Copy input read by a read subroutine into sepmoduleinput
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2043   Sequence number out of range
!    2044   leniinp out of range
!    2046   lenrinp out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Set length of arrays
!     Perform some checks
!     Create input arrays
!     Fill arrays with default values
!     Copy contents of arrays iinput and rinput
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ernlnsin' )

!     --- Set length of arrays

      if ( leniinp>2000 .or. leniinp<0 ) then

!     --- Error leniinp out of range

         call errint ( leniinp, 1 )
         call errint ( 0, 2 )
         call errint ( 2000, 3 )
         call errsub ( 2044, 3, 0, 0 )
         go to 1000

      end if

      if ( lenrinp>100 .or. lenrinp<0 ) then

!     --- Error lenrinp out of range

         call errint ( lenrinp, 1 )
         call errint ( 0, 1 )
         call errint ( 2000, 2 )
         call errsub ( 2046, 2, 1, 0 )
         go to 1000

      end if

      ninsol = max(26,leniinp)
      nrisol = max(3,lenrinp)

      allocate ( inpsol(ninsol), rinsol(nrisol), stat = error )
      if ( error/=0 ) call eralloc ( error, ninsol+nrisol, 'inpsol' )
      if ( ierror/=0 ) go to 1000

      maxsubr = iinputarrs(12)%maxsub
      if ( iseqnr<1 .or. iseqnr>maxsubr ) then

!     --- Sequence number too large

         call errint ( iseqnr, 1 )
         call errint ( maxsubr, 2 )
         call errchr ( 'non-linear solver', 1 )
         call errsub ( 2043, 2, 0, 1 )
         iseqnr = maxsubr

      end if

!     --- Fill arrays with default values

      inpsol = 0
      inpsol(2) = 20
      inpsol(5) = 2
      inpsol(7) = 1
      inpsol(8) = 1
      rinsol = 0d0
      rinsol(1) = 1d-3

!     --- Copy contents of arrays iinput and rinput

      do i = 1, min(ninsol,leniinp)
         inpsol(i) = iinput(i)
      end do
      do i = 1, min(nrisol,lenrinp)
         rinsol(i) = rinput(i)
      end do

      call ercopyinput ( 12, iseqnr, 'ernlnsin', inpsol, rinsol, &
                         inpsol, ninsol, nrisol, 0 )

      deallocate ( inpsol, rinsol, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'inpsol' )

1000  call erclos ( 'ernlnsin' )
      end
