      subroutine instfree ( iread )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.1  date 11-12-2017 New storage of intmat
!        version  4.0  date 20-08-2016 use sepmoduleinput
!        version  3.0  date 30-01-2016 New parameter list
!
!   copyright (c) 1999-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Subroutine to be used for instationary free surfaces
!    It adapts the free boundary
!    This is a very special subroutine in the SEPRAN context, since it
!    assumes that the user has made a loop himself
!    instfree should be called in this loop
!    It returns with a value for ready indicating if the loop may be finished
!    or not.
!    The following tasks are performed:
!    In contrast to adapmesh input is only possible through the input file
!    and not through the parameter list
!    Warning:  Experimental version
!              Both the parameter list as well as the input may change
!              Check this in each new update of SEPRAN until this
!              subroutine is part of the manuals
! **********************************************************************
!
!                       KEYWORDS
!
!     change_boundary
!     change_mesh
!     mesh
!     read
!     time_dependent
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekmesh
      use sepmodulekprob
      use sepmodulemain
      use sepmoduleinput
      use sepmoduletime
      use sepmodulemach
      use sepmodulemesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iread

!     iread   i    Defines how the parameters must be read
!                  Possible values:
!                  0   The input is read from the standard input file
!                  >0  The input as read from the standard input file by
!                      subroutine SEPSTM is used. The value of iread
!                      defines the sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer niinp, nrinp
      parameter ( niinp=80, nrinp=3 )
      integer iinput(niinp), leniinp, iseqprob, &
              ivectr, inpada(1), icurvs(1), &
              iseqnr, iseqmsh, numvecs, istvectr, &
              iseqmsv, maxcal, ipos, &
              lenrinp, jcheck, iwrmesh, iref, ioutpsv
      double precision rinput(nrinp), rinada(1), rinmsh(1)
      double precision :: timebefore(2)

!     icurvs         dummy array
!     iinput         integer input array with the following contents:
!                    1:  LASTIINP  Position of last entry that has been filled
!                    2:  IPRINT    Defines the amount of output produced
!                                  Possible values:
!                                  0:  No special information is printed
!                    3:  ISEQMESH  Sequence number of adapt mesh to be used
!                    4:  IVECTR    Sequence number of vector
!                    5:  INTERPOLATE_SOL: defines the number of solution
!                                  vectors that must be interpolated
!                    6:  INDEX_SOL: defines the starting address in
!                                  IINPUT of the vectors to be interpolated
!                                  In positions IINPUT(INDEX_SOL) to
!                                  IINPUT(INDEX_SOL+INTERPOLATE_SOL-1) the
!                                  sequence numbers of the solution vectors
!                                  to be interpolated are stored.
!                                  If INDEX_SOL = 0, the vectors 1 to
!                                  INTERPOLATE_SOL are interpolated
!                                  If INDEX_SOL<0, the vectors
!                                  -INDEX_SOL to -INDEX_SOL+INTERPOLATE_SOL-1
!                                  are interpolated
!                    7:  IVEC_MESHVEL Consists of two parts iseq and ivec
!                                  according to ISEQMESH = ISEQ + 1000 x IVEC, &
!                                  with
!                           ISEQ   Sequence number of vector in which
!                                  the mesh velocity must be stored
!                                  If 0, no mesh velocity is stored
!                           IVEC   Defines the type of vector to be used for
!                                  the mesh velocity vector
!                                  Possible values:
!                                  0:  The type of vector is found by ADAPMESH
!                                  1:  The vector is of the type solution vector
!                                  2:  The vector is a vector of special
!                                      structure of type IVEC-1
!                    8:  ICHECK_BOUN indication of how the boundary must be
!                                  checked
!                                  Possible values:
!                              0   No checking of the boundary, except in the
!                                  case that general is applied
!                                  If general is applied all boundaries of the
!                                  surfaces generated by general are checked
!                                  with respect to the general requirements
!                              1   The first and last element of each curve
!                                  in array icurves is compared with the
!                                  original length stored in array kmesh p
!                                  If 1/a<dx/dx_orig<a the curve is
!                                  accepted, otherwise jcheck is set equal to 1
!                                  dx is the actual length of the element
!                                  dx_orig is the original length of the element
!                                  a must be stored in rinput(1)
!                                  As soon as one of the curves returns with
!                                  jcheck = 1, checking is stopped
!                    9:  ICHECK_MESH indication of how the internal elements
!                                  must be checked. This is only applied if the
!                                  boundary check returns with jcheck = 0
!                                  Possible values:
!                              0   No checking of the internal elements
!                              1   All angles of the elements are checked
!                                  If the smallest angle of an element<rinput(2)
!                                  or the largest angle of an element >rinput(3)
!                                  jcheck is set equal to 2 and checking is
!                                  stopped
!     inpada         Dummy
!     ioutpsv        Help variable to store the value of ioutp
!     ipos           Position of input arrays in sepmoduleinput
!     iref           File reference number
!     iseqmsh        Sequence number of adap_mesh input
!     iseqmsv        Sequence number of mesh velocity vector with respect to
!                    isol
!     iseqnr         Sequence number with respect to the storage in array ISEPIN
!     iseqprob       Sequence number of problem input
!     istvectr       Starting position of array iseqvecs in iinput
!     ivectr         standard sepran array denoting usol or vector of special
!                    structure defined in nodal points
!     iwrmesh        If 1 the mesh must be written if 0 not
!     jcheck         Indicates if remeshing must be applied or not
!                    Possible values:
!                    0  No remeshing necessary
!                    1  The boundary does not satisfy the requirements
!                    2  The internal elements do not satisfy the requirements
!     leniinp        Length of array iinput
!     lenrinp        Length of array rinput
!     maxcal         Maximum number of coefficient inputs read
!     niinp          Length of array iinput
!     nrinp          Length of array rinput
!     numvecs        Number of vectors in iseqvecs
!     rinada         User input array for subroutine ADAPBOUN
!     rinmsh         User input array for subroutine ADAPMESH
!     rinput         Real input array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLMN         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     ERINSTFR       Read input concerning the instationary free boundaries
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     INCOMGEN       Compute matrix structure for free boundary problem
!     INICHINPSOL    Adapt input array with respect to variables
!     INIFIL         Open files
!     MSH068         write information of the mesh to the file indicated by iref
!     MSHADAP        Adapt mesh in free boundary problem
!     MSHCHECK       Check quality of mesh
!     MSHREMSH       Perform remeshing
!     MSHVELO        Compute the mesh velocity
!     PROBDFEXT      Reads and stores problem definition
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
! **********************************************************************
!
!                       I/O
!
!   Depending on the values in IREAD input may be read from the standard
!   input file by subroutine READFRST
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1948   No input with the main keyword has been read
! **********************************************************************
!
!                       PSEUDO CODE
!
!    If ( mesh_velocity required ) then
!       Store old co-ordinates
!    Adapt mesh
!    If ( mesh_convection required ) then
!       compute mesh velocity
!    If ( mesh or boundary must be checked ) then
!       Check the quality of the mesh
!    If ( remeshing required ) then
!       remesh and interpolate if necessary
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'instfree' )
      if ( itime>0 ) call ersettime ( timebefore )

!     --- Standard tests

      ioutpsv = ioutp
      if ( ierror/=0 ) go to 1000

!     --- Fill arrays iinput and rinput

      if ( iread==0 ) call erinstfr
      iseqnr = max(1,iread)
      call sepgetinpfinf ( 19, iseqnr, ipos, maxcal )
      if ( .not. in(ipos)%used ) then
         call errchr ( 'INSTATIONARY_FREE_BOUNDARY', 1 )
         call errsub ( 1948, 0, 0, 1 )
         go to 1000
      end if
      leniinp = size( in(ipos)%iinput)
      iinput(1:leniinp) = in(ipos)%iinput(1:leniinp)
      length = size( re(ipos)%rinput)
      rinput(1:length) = re(ipos)%rinput(1:length)

!     --- Adapt variables

      call inichinpsol ( iinput, leniinp )
      if ( ierror/=0 ) go to 1000

!     --- Adapt mesh

      istvectr = iinput(4)
      iseqmsh  = iinput(3)
      iseqmsv  = iinput(7)
      numvecs  = iinput(11)
      ivectr   = iinput(istvectr)

      if ( iseqmsv>0 ) then

!     --- mesh_velocity required
!         store old co-ordinates

         allocate ( coorold(ndim,npoint), stat = error )
         if ( error/=0 ) call eralloc ( error, ndim*npoint, 'coorold' )
         if ( ierror/=0 ) go to 1000
         coorold = coor

      end if
      ioutp = -1
      call mshadap ( inpada, rinada, iinput, rinmsh, &
                     icurvs, iseqmsh, numvecs, &
                     iinput(istvectr) )
      if ( ierror/=0 ) go to 1000

      if ( iseqmsv>0 ) then

!     --- mesh_velocity required
!         compute mesh velocity

         iprob = ks(isolut2d(ivectr))%iprob
         call mshvelo ( iseqmsv )

         deallocate ( coorold, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'coorold' )
         if ( ierror/=0 ) go to 1000

      end if
      if ( ierror/=0 ) go to 1000

      jcheck = 0
      if ( (iinput(8)>0 .or. iinput(9)>0) .and. ichanmesh==0 ) then

!     --- The mesh must be checked

         call mshcheck ( iinput(8), rinput, jcheck )

      end if
      if ( ierror/=0 ) go to 1000

      if ( jcheck>0 ) then

!     --- Remeshing is required

         call mshremsh ( isolut2d, iread, iinput, 1 )

      else if ( ichangeobst==1 ) then

!     --- An obstacle has been activated, probdf and so on must be called

         iseqprob = 1
         call probdfext ( 3, iseqprob )
         if ( ierror/=0 ) go to 1000
         call incomgen ( iread, intmat1d )
         if ( ierror/=0 ) go to 1000

      end if

      iwrmesh = iinput(10)
      if ( iwrmesh==1 .and. t+tstep>tend -1d-3*sqreps ) then

!     --- iwrmesh = 1 and last time step, write the mesh for
!                                         post processing purposes

         call inifil ( 10 )
         if ( ierror/=0 ) go to 1000
         call msh068 ( iref10 )
         if ( ierror/=0 ) go to 1000
         iref = abs(iref10)
         close ( iref )

      end if
1000  call erclmn ( 'instfree', timebefore )
      ioutp = ioutpsv

      end subroutine instfree
