      subroutine prbubble ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 20-06-2013 New definition file ref numbers
!        version  1.2  date 25-11-2012 Long integers
!        version  1.1  date 06-02-2012 Allow indexed vectors
!
!   copyright (c) 2010-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute bubbles defined by all regions satisfying function
!     value larger than given threshold
!     Compute areas of all bubbles
! **********************************************************************
!
!                       KEYWORDS
!
!     contour_level
!     area
!     bubble
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmodulemain
      use sepmodulemnsub
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduletime
      use sepmodulevecs
      use sepmoduleplot
      use sepmoduletexts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)
      double precision rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         For a description see readscbubble
!     rinsep         i    Array containing the real input for this subroutine
!                         For a description see readscbubble
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numbubbles, iplots(2), lenarea, idegfd, number, &
              iseqsol, istrin, itextx, itexty, itextf, iref
      parameter (lenarea=1000)
      double precision area(lenarea), value, rplots(3), areatot
      double precision, allocatable :: func(:)
      character (len=120) filename

!     area           Array containing the areas of all the bubbles
!     areatot        Sum oll the areas stored in area
!     filename       Name of file
!     func           Contains the function values
!     idegfd         Degree of freedom sequence number
!     iplots         Integer input array for plot subroutine
!                    See subroutine prcontourcomp
!     iref           File reference number
!     iseqsol        Sequence number of solution array in array ISOL
!     istrin         Starting address of information for this subroutine
!                    in RINSEP
!     itextf         Position of file name in text array in common csepcm
!     itextx         Position of x-text in text array in common csepcm
!     itexty         Position of y-text in text array in common csepcm
!     lenarea        Declared length of array area
!     number         Defines the number of degrees of freedom per point
!                    in array func
!     numbubbles     Number of bubbles found
!     rplots         Real input array for plot subroutine
!                    See subroutine prcontourcomp
!     value          Threshold value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inivectr, inisetnewref

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INISETNEWREF   Create new reference number
!     INIVECTR       Return with sequence number of vector
!     PRCONTOURCOMP  Compute a set of contours corresponding to one function
!                    value and compute the area occupied by each contour for
!                    the part larger than this function
!     PRINRL         Print 1d real vector
!     SEPACTSOLBF1   Activate sepran solution vector and related arrays
!     SEPFILLKMESHO  Fill kmesh part o
!     TOPQU4         Compute vertex mesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     419   NDIM incorrect
!    1382   Value of NPELM is not yet implemented for this subroutine
!    2384   Plane intersects prism in more than 4 edges
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data iref /0/
! ======================================================================
!
      call eropen ( 'prbubble' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prbubble'
         write(irefwr,1) 'nelem, npoint', nelem, npoint
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ndim/=2 ) then

!     --- At this moment only ndim = 2 permitted

         call errint ( ndim, 1 )
         call errint ( 2, 2 )
         call errint ( 2, 3 )
         call errsub ( 419, 3, 0, 0 )

      end if  ! ( ndim/=2 )

      if ( nelgrp/=1 ) then

!     --- At this moment only nelgrp = 1 permitted

         call errint ( nelgrp, 1 )
         call errsub ( 2384, 1, 0, 0 )

      end if  ! ( ndim/=2 )

      if ( npelm/=3 ) then

!     --- At this moment only npelm = 3 permitted

         call errint ( npelm, 1 )
         call errsub ( 1382, 1, 0, 0 )

      end if  ! ( ndim/=2 )

      allocate(func(npoint), stat = error)
      if ( error/= 0 ) then

!     --- error#0, so not enough space

         call errint ( npoint, 1 )
         call errint ( error, 2 )
         call errchr ( 'func', 1 )
         call errsub ( 2910, 2, 0, 1 )

      end if  ! ( error/= 0 )
      if ( ierror/=0 ) go to 1000

      iseqsol = inivectr(inpsep(1))
      istrin = inpsep(10)

!     --- activate arrays

      if ( kelmo==0 ) call sepfillkmesho
      call sepactsolbf1 ( isolut2d(iseqsol) )

      usol => ks(isolut2d(1))%sol
      if ( ierror/=0 ) go to 1000

!     --- Copy solution into array func

      number = 1
      idegfd = 1
      call topqu4 ( number, idegfd, func )
      if ( ierror/=0 ) go to 1000

      value = rinsep(1+istrin)

      iplots(1) = inpsep(3)  ! make plot
      iplots(2) = inpsep(4)  ! color
      rplots(1) = 15d0       ! plotfm
      rplots(2) = rinsep(istrin)  ! yfact
      itextx = inpsep(5)  ! start of textx
      ntextx = inpsep(6)  ! number of characters of textx
      itexty = inpsep(7)  ! start of texty
      ntexty = inpsep(8)  ! number of characters of texty
      itextf = inpsep(9)
      if ( debug ) &
         write(irefwr,1) 'itextx, ntextx, itexty, ntexty', &
                          itextx, ntextx, itexty, ntexty

      if ( itextx>0 ) then

!     --- textx found

         textx = texts(itextx)

      else

!     --- textx not found, use default value

         textx  = 'x'
         ntextx = 1

      end if  ! ( itextx>0 )
      if ( debug ) write(irefwr,*) 'textx ', textx(1:ntextx)

      if ( itexty>0 ) then

!     --- texty found

         texty = texts(itexty)

      else

!     --- texty not found, use default value (name of vector)

         texty  = 'y'
         ntexty = 1

      end if  ! ( itextx>0 )
      if ( debug ) write(irefwr,*) 'texty ', texty(1:ntexty)

      if ( itextf>0 ) then

!     --- itextf>0, use given file name

         filename = texts(itextf)

      else

!     --- itextf = 0, use default file name

         filename = 'bubble.dat'

      end if  ! ( itextf>0 )

!     --- Compute bubbles

      call prcontourcomp ( func, value, numbubbles, area, &
                           lenarea, iplots, rplots )
      if ( ierror/=0 ) go to 1000
      areatot = sum(area(1:numbubbles))

      write(irefwr,100) t, numbubbles, areatot
100   format ( 'At time ', d12.3,' the number of bubbles is ', i3, &
               ' and the area is ', d12.3 )

      call prinrl ( area, numbubbles, 'area of bubbles' )

      istartmn = istartmn+11

      if ( iref==0 ) then
         iref = inisetnewref()
         open ( unit=iref, file = filename )
      end if  ! ( iref==0 )
      write(iref,110) t, numbubbles, areatot
110   format ( d12.5, i5, 3x, d12.5 )

1000  deallocate ( func, stat = error )
      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'func', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/= 0 )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prbubble'

      end if  ! ( debug )
      call erclos ( 'prbubble' )

      end
