      subroutine prcorrinterp1d ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 10-04-2013 Split interpolation concentration
!                                      and levelset function
!        version  1.1  date 25-11-2012 Long integers
!        version  1.0  date 02-11-2011
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Correct interpolation in case nodes of the mesh crossed
!     the old object (1d)
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     level_set
!     node
!     intersection
!     1d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemeshinf
      use sepmodulevecs
      use sepmodulemain
      use sepmoduleplot
      use sepmoduleinput
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)
      double precision rinsep(*)

!     inpsep         i    Integer input array
!                         See subroutine readsclevel for the contents
!                         The position are shifted one down
!     rinsep         i    Array containing the real input for this subroutine
!                         See subroutine readsclevel for the contents
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer mesh1, mesh2, mesh3,  seqphi, npoint, lensto, &
              seqphiold, seqconc, seqmeshvel, iboun, ibounnew, &
              numarray, iseqarrsav, iseqarrmin, iseqarrmax, &
              iseqkm1, iseqkm2, iseqkm3
      double precision dt
      integer, allocatable :: istore(:)

!     dt             Time step
!     iboun          boundary point where level set equals 0 (old mesh)
!     ibounnew       boundary point where level set equals 0 (new mesh)
!     iseqarrmax     Maximum value of iseqarray in loop
!     iseqarrmin     Minimum value of iseqarray in loop
!     iseqarrsav     Help parameter to store old value of iseqarray
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     iseqkm3        Sequence number with respect to km of third mesh
!     istore         work array of length npoint to store nodes for which phi1
!                    and phi2 have different sign
!     lensto         Number of nodes in istore
!     mesh1          Sequence number of basis mesh
!     mesh2          Sequence number of old levelset mesh
!     mesh3          Sequence number of new levelset mesh
!     npoint         Number of nodal points
!     numarray       Number of indexed vectors in vector array
!     seqconc        Sequence number of concentration vector in isol
!     seqmeshvel     Sequence number of new phi vector in isol
!     seqphi         Sequence number of old phi vector in isol
!     seqphiold      Sequence number of mesh velocity vector in isol
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal, sepnodezerlev
      double precision getparm

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     GETPARM        Return with real defined by inpsep/rinsep
!     INIREDUCEVEC   Compute the reduced value of ivecin which indicates the
!                    first sequence number of a (series of) vectors, as well as
!                    the dimension of the series
!     INIREPSCAL     Corrects integer value in case of scalars
!     PRCORRINTERP1  Interpolate special points that where outside the
!                    computational region before move of boundary and are
!                    inside afterwards
!     PRCORRINTERP11 Interpolate special points that where outside the
!                    computational region before move of boundary and are
!                    inside afterwards
!     PRINRL         Print 1d real vector
!     PRLISTPHIPOINT Return with set of nodes with different phi signs
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPNODEZERLEV  Return with node at boundary defined by zero levelset
!     SEPRELKMESH    Make the relation between kmref and the pointers
!                    in sepmodulekmeshp
!     SEPRMSOLARRS   Remove all solution arrays corresponding to a specific mesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcorrinterp1d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcorrinterp1d'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- get parameters from inpsep

      mesh1 = abs(meshseqs(inirepscal(inpsep(1))))
      mesh2 = abs(meshseqs(inirepscal(inpsep(2))))
      mesh3 = abs(meshseqs(inirepscal(inpsep(3))))

      iseqarrsav = iseqarray
      if ( iseqarray==0 ) then

!     --- iseqarray = 0, check if loop is necessary

         call inireducevec ( inpsep(4), seqconc, numarray )
         iseqarrmin = 1
         iseqarrmax = numarray

      else

!     --- iseqarray # 0, use iseqarray

         iseqarrmin = iseqarray
         iseqarrmax = iseqarray
         seqconc = inpsep(4)

      end if  ! ( iseqarray==0 )

      if ( debug ) &
         write(irefwr,1) 'seqconc, numarray', seqconc, numarray

      seqphi = inpsep(5)
      seqphiold = inpsep(6)
      seqmeshvel =inpsep(7)
      if ( debug ) &
         write(irefwr,1) 'mesh1, mesh2, mesh3, seqphi, seqmeshvel', &
                          mesh1, mesh2, mesh3, seqphi, seqmeshvel

!     --- get parameters from rinsep

      dt = getparm ( inpsep, rinsep, 8, 'dt' )

!     --- Get parameters

      iseqkm1 = kmesh2d(mesh1)
      iseqkm2 = kmesh2d(mesh2)
      iseqkm3 = kmesh2d(mesh3)
      npoint = km(iseqkm1)%kmesh(8)

      phi1 => ks(isolut3d(seqphi,mesh2))%sol
      phi2 => ks(isolut3d(seqphiold,mesh2))%sol

      allocate ( istore(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'istore' )
      if ( ierror/=0 ) go to 1000

!     --- Make a list of points for which phi_1 has different sign from phi_2

      kmesh => kmesh2d(mesh2)
      call sepfillkmesh(kmesh)
      iboun =  sepnodezerlev ( 1 )  ! boundary point
      kmesh => kmesh2d(mesh3)
      call sepfillkmesh(kmesh)
      ibounnew = sepnodezerlev ( 1 )  ! new mesh

      call prlistphipoint ( npoint, lensto, istore, ibounnew )

      if ( debug ) write(irefwr,1) 'lensto', lensto

!     --- there are points that must be interpolated
!         At this moment only one level set function
!         Create mesh velocity vector and make equal to zero

      iprob = ks(isolut3d(seqconc,mesh3))%iprob
      iseqkp = ks(isolut3d(seqconc,mesh3))%iseqkp
      call sepcreasolbf1 ( isolut3d(seqmeshvel,mesh3), &
                           ks(isolut3d(seqconc,mesh3))%typevec, &
                           ks(isolut3d(seqconc,mesh3))%ivec, &
                           'meshvel', &
                           ks(isolut3d(seqconc,mesh3))%compl )
      meshvel => usoltemp
      meshvel = 0d0

      coorold => km(iseqkm2)%coor
      coornew => km(iseqkm3)%coor

!     --- compute mesh velocity

      call prcorrinterp1 ( istore, lensto, dt, iboun, ibounnew )
      if ( debug ) &
         call prinrl ( meshvel, km(iseqkm3)%kmesh(8), 'meshvel' )

!     --- Loop over subarrays

      do iseqarray = iseqarrmin, iseqarrmax


         uold => ks(isolut3d(seqconc,mesh2))%sol
         unew => ks(isolut3d(seqconc,mesh3))%sol

!        --- Adapt concentration

         call prcorrinterp11 ( istore, lensto, iboun, ibounnew )

         if ( debug ) &
             call prinrl ( unew, km(iseqkm3)%kmesh(8), 'concnew' )
         seqconc = seqconc+1

      end do  ! iseqarray = iseqarrmin, iseqarrmax
      npoint = km(iseqkm1)%kmesh(8)

      iseqarray = iseqarrsav  ! reset iseqarray

      deallocate ( istore, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'istore' )

      call seprmsolarrs ( mesh2 )  ! remove solution arrays

      call seprelkmesh ( kmesh2d(mesh3) )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcorrinterp1d'

      end if  ! ( debug )
      call erclos ( 'prcorrinterp1d' )

      end
