      subroutine prcorrinterp2d ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.5  date 14-11-2013 Always create mesh_vel
!        version  2.4  date 03-10-2013 Use mshdel to remove kmesh2d
!        version  2.3  date 22-05-2013 Jump in case no interface
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Correct interpolation in case nodes of the mesh crossed
!     the old object (2d and 3d)
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     level_set
!     node
!     intersection
!     2d
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduleinput
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)
      double precision :: rinsep(*)

!     inpsep         i    Integer input array
!                         See subroutine readsclevel for the contents
!                         The position are shifted one down
!     rinsep         i    Array containing the real input for this subroutine
!                         See subroutine readsclevel for the contents
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer mesh1, mesh2, mesh3, seqphi, leng, &
              lensto, seqphiold, seqconc, seqmeshvel, &
              ilev, istart, itypvc, ivec, kelmadm2, &
              nofacenew, nofaceold, &
              numarray, iseqarrsav, iseqarrmin, iseqarrmax, &
              iseqkm1, iseqkm2, iseqkm3
      double precision dt
      integer, allocatable :: istore(:), oldnodes(:)
      integer, pointer :: ipointobm2(:), ipointobm3(:)

!     dt             Time step
!     ilev           Sequence number of level being considered
!     ipointobm2     ipointobst corresponding to second mesh
!     ipointobm3     ipointobst corresponding to third mesh
!     iseqarrmax     Maximum value of iseqarray in loop
!     iseqarrmin     Minimum value of iseqarray in loop
!     iseqarrsav     Help parameter to store old value of iseqarray
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     iseqkm3        Sequence number with respect to km of third mesh
!     istart         First position used at a part
!     istore         work array of length npoint to store nodes for which phi1
!                    and phi2 have different sign
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     kelmadm2       Starting address of kmesh part ad for mesh 2
!     leng           size of array
!     lensto         Number of nodes in istore
!     mesh1          Sequence number of basis mesh
!     mesh2          Sequence number of old levelset mesh
!     mesh3          Sequence number of new levelset mesh
!     nofacenew      Number of nodes in interfacenew
!     nofaceold      Number of nodes in interfaceold
!     numarray       Number of indexed vectors in vector array
!     oldnodes       Contains the set of old nodes corresponding to
!                    the nodes in interfacenew
!     seqconc        Sequence number of concentration vector in isol
!     seqmeshvel     Sequence number of new phi vector in isol
!     seqphi         Sequence number of old phi vector in isol
!     seqphiold      Sequence number of mesh velocity vector in isol
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal
      double precision getparm

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRWAR         Print warnings
!     GETPARM        Return with real defined by inpsep/rinsep
!     INIREDUCEVEC   Compute the reduced value of ivecin which indicates the
!                    first sequence number of a (series of) vectors, as well as
!                    the dimension of the series
!     INIREPSCAL     Corrects integer value in case of scalars
!     INISTR         find structure of vector
!     PRCORRINTERP2  Correct interpolation in case nodes of the mesh crossed
!                    the old object (2d)
!     PRCORRINTERP3  Correct interpolation in case nodes of the mesh crossed
!                    the old object (3d)
!     PRLEVINTERFST  Compute starting address of interface in array ipointobst
!     PRLISTPHIPNT2D Return with set of nodes with different phi signs
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPRELKMESH    Make the relation between kmref and the pointers
!                    in sepmodulekmeshp
!     SEPRMMESHINFO  Remove mesh information
!     SEPRMSOLARRS   Remove all solution arrays corresponding to a specific mesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3142   new interface is empty
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcorrinterp2d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcorrinterp2d'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- get parameters from inpsep

      mesh1 = abs(meshseqs(inirepscal(inpsep(1))))
      mesh2 = abs(meshseqs(inirepscal(inpsep(2))))
      mesh3 = abs(meshseqs(inirepscal(inpsep(3))))
      call sepfillkmesh(kmesh2d(mesh1))
      iseqkm1 =iseqkm
      iseqkm2 = kmesh2d(mesh2)
      iseqkm3 = kmesh2d(mesh3)

!     --- First check if there is an interface

      ipointobm2 => km(iseqkm2)%ipointobst
      ipointobm3 => km(iseqkm3)%ipointobst
      coorold => km(iseqkm2)%coor
      coornew => km(iseqkm3)%coor
      ilev = 1
      call prlevinterfst ( ipointobm3, istart, nofacenew, ilev )
      if ( debug ) write(irefwr,1) 'nofacenew', nofacenew

!     --- Get parameters

      if ( debug ) write(irefwr,1) 'ndim, npoint', ndim, npoint

      seqphi = inpsep(5)
      seqphiold = inpsep(6)
      seqmeshvel = inpsep(7)
      if ( debug ) &
         write(irefwr,1) 'mesh1, mesh2, mesh3, seqphi', &
                          mesh1, mesh2, mesh3, seqphi

!     --- Create mesh velocity vector and make equal to zero

      itypvc = 110
      iprob = 1
      if ( isolut3d(seqmeshvel,mesh3)>0 ) &
         iprob = ks(isolut3d(seqmeshvel,mesh3))%iprob
      kprob => kprob2d(mesh3)
      iseqkp = kprob
      call inistr ( ndim, itypvc, ivec )
      call sepfillkmesh(kmesh2d(mesh3))
      call sepcreasolbf1 ( isolut3d(seqmeshvel,mesh3), itypvc, ivec, &
                           'meshvel', 0 )
      usoltemp = 0d0

      if ( nofacenew==0 ) then

!     --- No new interface, warning and return

         call errwar ( 3142, 0, 0, 0 )
         go to 1000

      end if  ! ( nofacenew==0 )

      iseqarrsav = iseqarray
      if ( iseqarray==0 ) then

!     --- iseqarray = 0, check if loop is necessary

         call inireducevec ( inpsep(4), seqconc, numarray )
         iseqarrmin = 1
         iseqarrmax = numarray

      else

!     --- iseqarray # 0, use iseqarray

         iseqarrmin = iseqarray
         iseqarrmax = iseqarray
         seqconc = inpsep(4)

      end if  ! ( iseqarray==0 )

!     --- get parameters from rinsep

      dt = getparm ( inpsep, rinsep, 8, 'dt' )

!     --- activate arrays

      phi1 => ks(isolut3d(seqphi,mesh2))%sol
      phi2 => ks(isolut3d(seqphiold,mesh2))%sol

      allocate ( istore(npoint), oldnodes(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint, 'istore' )
      if ( ierror/=0 ) go to 1000

!     --- Find old and new interface


      do iseqarray = iseqarrmin, iseqarrmax

         seqconc = seqconc+1

      end do  ! iseqarray = iseqarrmin, iseqarrmax
      seqconc = seqconc-iseqarrmax+iseqarrmin-1

      meshvel => ks(isolut3d(seqmeshvel,mesh3))%sol
      call prlevinterfst ( ipointobm2, istart, nofaceold, ilev )
      leng = size(ipointobm2)
      ipointobm2 => km(iseqkm2)%ipointobst(istart:leng)
      call prlevinterfst ( ipointobm3, istart, nofacenew, ilev )
      leng = size(ipointobm3)
      ipointobm3 => km(iseqkm3)%ipointobst(istart:leng)

!     --- Make a list of points for which phi_1 has different sign from phi_2

      call prlistphipnt2d ( lensto, istore, ipointobm3, nofacenew )

      if ( debug ) write(irefwr,1) 'lensto', lensto

!     --- compute mesh velocity and fill oldnodes

      select case ( ndim )

      case(2)

!     --- ndim = 2

         call prcorrinterp2 ( coorold, ipointobm2, &
                              nofaceold, coornew, &
                              ipointobm3, nofacenew, oldnodes, &
                              meshvel, dt, lensto, istore )

      case(3)

!     --- ndim = 3

         call prcorrinterp3 ( coorold, ipointobm2, &
                              nofaceold, coornew, &
                              ipointobm3, nofacenew, oldnodes, &
                              meshvel, dt, lensto, istore )

      end select  ! case ( ndim )

!     --- Adapt concentration

      do iseqarray = iseqarrmin, iseqarrmax

         uold => ks(isolut3d(seqconc,mesh2))%sol
         unew => ks(isolut3d(seqconc,mesh3))%sol
         seqconc = seqconc+1

         unew(ipointobm3(1:nofacenew)) = uold(oldnodes(1:nofacenew))

         if ( lensto>0 ) &
            unew(istore(1:lensto)) = &
               uold(oldnodes(nofacenew+1:nofacenew+lensto))

      end do  ! iseqarray = iseqarrmin, iseqarrmax

      iseqarray = iseqarrsav  ! reset iseqarray

      deallocate ( istore, oldnodes, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'istore' )

!     --- Remove superfluous parts

      km(iseqkm2)%kmesh(22) = 0
      km(iseqkm2)%kmesh(31:35) = 0
      call seprmmeshinfo ( kmesh2d(mesh2) )

      call seprmsolarrs ( mesh2 )  ! remove solution arrays

      call seprelkmesh ( kmesh2d(mesh3) )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcorrinterp2d'

      end if  ! ( debug )
      call erclos ( 'prcorrinterp2d' )

      end
