      subroutine prenthrefine ( iinput )
! ======================================================================
!
!        programmer    John Brusche
!        version  2.9  date 05-13-2013 Jump in case of errors
!        version  2.8  date 25-11-2012 Long integers
!        version  2.7  date 04-06-2010 New call to prenthalpy
!
!   copyright (c) 2005-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Main routine for adaptive local mesh refinement
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekmesh
      use sepmodulekprob
      use sepmodulecons
      use sepmodulemain
      use sepmodulemnsub
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iinput(*)

!     iinput         i    Integr input array containing information of
!                         the process
!                         Contents
!                         1:  iseqnrenth   sequence number of enthalpy input
!                         2:
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: niinpent=10, nrinpent=10, maxreflvl=3
      integer reflvl, iseqnrenth, iseqtime, iprint, &
              basismesh, iseqtemp, ioutactsav, nelembasis, &
              npointbasis, nodeextrael(0:maxreflvl), &
              meshnew, npointnew, itempold, irewind, i, iseqnr, &
              iinenthalpy(niinpent), leninp, lenrnp, mmiinput, &
              isetctimen
      double precision tm, tstepuser, toutnextsav, rinenthalpy(nrinpent)
      logical first, refine, rewind, &
              reset, firsttimeloop, changestep, timestephalved
      save basismesh, npointbasis, nelembasis, irewind, &
           tstepuser, iinenthalpy, rinenthalpy, first, &
           mmiinput, isetctimen, firsttimeloop

!     basismesh      Mesh sequence number of the basis mesh
!     changestep     If true the time step may be changed
!     first          If true it concerns the first call of a time loop
!     firsttimeloop  If true it concerns the very first time this
!                    subroutine is called
!     i              Counter variable
!     iinenthalpy    Integer input array with respect to enthalpy
!                    computations
!                    For a definition of this array see subroutine
!                    readenthalpy (array iinput)
!     ioutactsav     Help variable to store old value of ioutact
!     iprint         Print level, 0 is no printing
!     irewind        Counting variable that keeps tracks of the number of
!                    time steps that is taken for the current time step size
!     iseqnr         Sequence number with respect to the storage in array
!                    ISEPIN
!     iseqnrenth     sequence number of enthalpy input
!     iseqtemp       Sequence number of temperature in solution vector
!     iseqtime       sequence number of time integration
!     isetctimen     Value of parameter use_ctimen at start of subroutine
!     itempold       Temperature at the previous time level
!     leninp         Computed length of integer input array
!     lenrnp         Computed length of real input array
!     maxreflvl      Maximum value of number of refinements
!     meshnew        Mesh sequence number of the new mesh
!     mmiinput       Memory management sequence number with respect to
!                    iinput
!     nelembasis     Number of elemente of the basis mesh
!     niinpent       Declared length of array iinenthalpy
!     nodeextrael    Array of size maxreflvl that contains the number
!                    of extra nodes per element defined as function
!                    of the refinement level
!     npointbasis    npoint of the basis mesh
!     npointnew      npoint of the new mesh
!     nrinpent       Declared length of array rinenthalpy
!     refine         If true, the mesh is to be refined
!     reflvl         Level of refinement
!     reset          When no convergence is reached for the attempted time step
!                    size, the time level is reset and the time step size is
!                    reduced
!     rewind         When true, the current time level is rewinded
!                    to a previous time level
!     rinenthalpy    Real input array with respect to enthalpy
!                    computations
!                    For a definition of this array see subroutine
!                    readenthalpy (array iinput)
!     timestephalved If true, the the time step size is halved after
!                    an 'interface outside refinement' warning is given
!     tm             Melting temperature
!     toutnextsav    Help varariable to store value of toutnext
!     tstepuser      Initial and maximum time step size as set by the user
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     IINPUTSEQ      Read contents of input block with sequence number iseqnr
!                    corresponding to main keyword name
!     MSHREFINIT     Initialize some vectors for adaptive local mesh refinement?
!     MSHREFMAPPING  Map the solution vector from the old mesh to the new mesh
!     MSHREFUPDATE   Update the mesh as determined based on the previous
!                    solution
!     MSHREFUPDATEND Fill array nodeextrael with number of extra nodes as
!                    function of the refinement level
!     PRENTHPLOTMSH  Plot the mesh. Nodal point numbers are shown.
!     PRENTHREFINE1  Swap meshes and arrays and backup some data.
!                    Remove unnecessary arrays
!     PRENTHREFRDIN  Read some parameters from the input file
!     PRENTHREWIND   Set the time one time step back. Reset the temperature
!                    to the appropriate temperature for the old time level.
!     PRININ         print 1d integer array
!     PRSETREFINE    Set the value of logical refine
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPENTHALPY    Solve the time dependent temperature equation
!                    with  freezing front by a enthalpy method
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPFILLTIMINP  Fill module sepmoduletime with information stored in
!                    inpsep/rinsep and ibuffr
!     SEPMAKEREL     Make relation between local and global arrays
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2791   time step halved>16 times, program halted
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Read input
!     Handle first call of routine
!     Solve 1 time step
!     Determine refinement
!     Refine/no refine
!     Swap meshes
!     Backup and swap some arrays
!     Switch back one time step indeed needed
!     Remove semi-permanent arrays
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data firsttimeloop /.true./
! ======================================================================
!
      call eropen ( 'prenthrefine' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prenthrefine'
         write(irefwr,1) 'meshnr', meshnr
         write(irefwr,*) 'timefinished (prenthrefinebegin)', &
                          timefinished
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )
     !    call prinin ( kprob2d, 60, 'kprob' )
     !    call prinin ( isolut3d, 5, 'isolut' )
     !    call sepprintv ( isolut2d, 'temperature start' )

      end if
      if ( ierror/=0 ) go to 1000
      timestephalved = .false.
      if ( itimloop<=1 ) then

!     --- New time loop, set first

         first = .true.

!        --- Read input for enthalpy problem and store in arrays
!            iinenthalpy, rinenthalpy

         iseqnr = iinput(1)
         leninp = niinpent
         lenrnp = nrinpent
         call iinputseq ( 26, iseqnr, iinenthalpy, &
                          rinenthalpy, leninp, lenrnp, &
                          'ENTHALPY_INTEGRATION' )
         iseqtime = iinenthalpy(4)  ! sequence number time integration

         itiminterval = 1
         call sepfilltiminp ( iseqtime, 3, iinenthalpy(3) )
         tstep = arrtimestep(1)
         usectimen = .true.
         if ( firsttimeloop ) &
            nullify(intnodesnew, intnodesold, refineold, refinenew)

         if ( debug ) &
            write(irefwr,2) 'tinit, tstep, tend', t0, tstep, tend

      else

         first = .false.

      end if

!     --- Save some parameters for the case of rewinding

      ioutactsav = ioutact
      toutnextsav = toutnext

!     --- extract some parameters from input

      call prenthrefrdin ( iseqtemp, tm, reflvl, &
                           maxreflvl, iprint, iinenthalpy, &
                           rinenthalpy )

!     --- Save a copy of the old temperature, so we can revert to it in case
!         we need to go back one time step

      itempold = 0
      call sepcopyvc ( isolut3d(iseqtemp,meshnr), itempold )

!     --- cover some bases in case this routine is called for the first
!         time by creating and filling the appropriate arrays
!         Very first call of this subroutine. Init various quantities

      if ( debug ) write(irefwr,*) 'firsttimeloop', firsttimeloop

      if (firsttimeloop) then

         if (debug) call prenthplotmsh
         tstepuser = tstep

!        --- First call of this subroutine, set basis mesh

         basismesh = meshnr

!        --- Set some global constants

         npointbasis = npoint
         nelembasis = nelem

         if ( debug ) &
            write(irefwr,1) 'meshnr, npoint, nelem', &
                             meshnr, npoint, nelem

!        --- Create 'old' arrays

         allocate ( intnodesold(2*nelembasis), refineold(nelembasis), &
                    refmapold(2*nelembasis) )

!        --- initialize the 'old' arrays

         call mshrefinit ( nelembasis )

!        --- Fill refineold based on the initial temperature
!            Preferably, this is not done.
!            When left out, it is ensured that at t = t_0 the mesh is
!            also refined in case an imposed boundary condition induces
!            a moving boundary...

!            call prfillrefvec ( kmesh2d(meshnr), &
!            +                    isolut(1,iseqtemp,meshnr), tm, &
!            +                    reflvl, mmrefineold )

      end if ! firsttimeloop

      if ( iprint>1 ) &
         write(irefwr,*) 'tstep before time integration', tstep
666   reset = .false.

      if ( iprint>1 ) write(irefwr,*) &
         'Current time level before time integration: ', t

!     --- solve one time step enthalpy equation

      complrun = .false.

      call sepmakerel  ! make relation between global and local arrays

      if ( debug ) write(irefwr,1) 'meshnr', meshnr

      intmatar => intmat1d(iseqtemp:)
      isolut => isolut2d(iseqtemp)
      call sepenthalpy ( timefinished, iinput, reset, iprint, &
                         iinenthalpy, rinenthalpy, changestep )

      if (reset .and. .not. firsttimeloop) then

         if ( iprint>1 ) write(irefwr,*) 'rewinding time at t>0'
         t = t - tstep
         tstep = tstep*0.5d0
         if (tstep<1.525878d-05*tstepuser) then

!        --- time step halved>16 times, program halted

            call errsub ( 2791, 0, 0, 0 )
            go to 1000

         end if
         timefinished = .false.
         reset = .false.

!        --- reset to old values

         ioutact = ioutactsav
         toutnext = toutnextsav

!        --- Copy temperature to temperaturenew

         call sepcopyvc ( itempold, isolut2d(iseqtemp) )

         goto 666

      end if

!     --- Print some useful information concerning time stepping

      if ( iprint>1 ) write(irefwr,*) &
         'Current time level after time integration: ', t

!     --- Based on the computed solution determine the necessity of refining
!         the mesh and of rewinding time

      if ( debug ) write(irefwr,*) 'before prsetrefine'
      call prsetrefine ( meshnr, isolut3d(iseqtemp,meshnr), tm, &
                         reflvl, nelembasis, refine, basismesh, &
                         rewind, firsttimeloop, irewind, iprint )

!     --- Now that we have determined whether to refine the mesh or not,
!         perform either a mesh refinement step, or keep the mesh as it was.
!         In both cases, set a series of arrays as needed for the next
!         time step.

      if (firsttimeloop) nullify(parentnodes)

      if (refine) then
         if (debug) write(irefwr,*) 'Mesh is refined'

!        --- Fill array nodeextrael as function of refinement level
!            and dimension

         call mshrefupdatend ( ndim, nodeextrael, maxreflvl )

         if (debug) then
            call prinin(nodeextrael, maxreflvl, 'nodeextrael')
         end if

!        --- based on refinenew an updated refined mesh can be build

         call mshrefupdate ( reflvl, maxreflvl, basismesh, &
                             nodeextrael, meshnr, meshnew )
         kmesh => kmesh2d(meshnew)
         call sepfillkmesh(kmesh)

         if ( debug ) then
            write(irefwr,1) 'meshnr, meshnew', meshnr, meshnew
  !          if ( kelmm/=0 ) call sepprinkmeshm
         end if  ! ( debug ) then

      else

         if (debug) write(irefwr,*) 'Mesh is not refined'

!        --- Mesh has not changed

         meshnew = meshnr

!        --- Copy intnodesold to intnodesnew, since nothing has changed

         if ( associated(intnodesnew) ) deallocate(intnodesnew)
         allocate ( intnodesnew(size(intnodesold)) )
         intnodesnew = intnodesold

      end if ! refine

!     --- The mapping of the temperature from the old mesh to the new mesh
!         now comes down to the following procedure:
!         mapping of the values on the nodes of the basis mesh
!         for each level for each element of the new mesh, compare with
!         array intnodes of the old mesh. When the refinement level for
!         the element in the old mesh is equal or higher than that of the
!         new mesh, array intnodes contains the corresponding node
!         numbers for both meshes for this level. In case the level for
!         the new mesh is higher than that of the old mesh, interpolate
!         between the nodes of the new mesh.

      npointnew = npoint
      if (rewind .and. .not. firsttimeloop) then
         if ( iprint>1 ) write(irefwr,*) &
            'rewinding time and mapping itempold'
         call mshrefmapping ( nelembasis, &
                           nodeextrael, npointbasis, itempold, &
                           isolut3d(iseqtemp,meshnew), reflvl, &
                           firsttimeloop, &
                           refine, npointnew)

      else

         call mshrefmapping ( nelembasis, &
                           nodeextrael, npointbasis, &
                           isolut3d(iseqtemp,meshnr), &
                           isolut3d(iseqtemp,meshnew), reflvl, &
                           firsttimeloop, &
                           refine, npointnew)

      end if

!     --- Map all remaining vectors in isolut
      i = 1
      do while (i<=numvecmn)
         if (debug) then
            write(irefwr,1) 'iseqtemp, numvecmn', iseqtemp, numvecmn
            write(irefwr,*) 'vector ', iseqtemp, ' will be skipped'
            write(irefwr,1) 'vector to map: i =', i
         end if
         if (i/=iseqtemp) then
            if (debug) write(irefwr,1) 'mapping vector', i
            call mshrefmapping( nelembasis, &
                               nodeextrael, npointbasis, &
                               isolut3d(i,meshnr), &
                               isolut3d(i,meshnew), reflvl, &
                               firsttimeloop, &
                               refine, npointnew)
         end if
         i = i + 1
      end do

      if (debug) then
         write(irefwr,*) 'Plotting new mesh'
         call prenthplotmsh
      end if

!     --- Shift 'new' versions to 'old' versions of the mesh and some
!         arrays. Delete unnecessary arrays.

      call prenthrefine1 ( meshnr, basismesh, meshnew, nelembasis, iseqtemp )

!     --- Switch back to previous time level after first time step

      if ( debug ) &
         write(irefwr,*) 'firsttimeloop, refine, rewind, tstep', &
                          firsttimeloop, refine, rewind, tstep

      if (rewind .and. .not. firsttimeloop) then
         if ( iprint>1 ) write(irefwr,*) 'rewinding time at t>0'
         t = t - tstep
         if (timestephalved) then
            tstep = tstep*0.5d0
            if (tstep<1.525878d-05*tstepuser) then

!           --- time step halved>16 times, program halted

               call errsub ( 2791, 0, 0, 0 )
               go to 1000

            end if ! (tstep<1.525878d-05*tstepuser)
         end if ! (timestephalved)
         timefinished = .false.
         irewind = 0

!        --- reset to old values

         ioutact = ioutactsav
         toutnext = toutnextsav

      else if (irewind>2) then

         if ( iprint>1 ) then
            write(irefwr,*) 'doubling time step size'
            write(irefwr,*) 'but not larger than tstep as set by user'
         end if  ! ( iprint>1 )

!        --- irewind = 0

         if (tstep*2d0<=tstepuser+1d-5) tstep = tstep*2d0

      else

         irewind = irewind + 1

      end if

      if (firsttimeloop .and. refine) then

         iseqnrenth = iinput(1)

         kprob => kprob2d(meshnew)
         isolut2d => isolut3d(:,meshnew)
         call prenthrewind ( npointnew, iseqtemp, iseqnrenth)
         if ( ierror/=0 ) go to 1000

!        --- reset to old values

         ioutact = ioutactsav
         toutnext = toutnextsav

      end if

      ks(isolut3d(iseqtemp,meshnew))%iseqkp = iseqkp

      if ( firsttimeloop ) then
         firsttimeloop = .false.
         irewind = -1
      end if

      if ( debug ) &
         write(irefwr,*) 'firsttimeloop, refine, rewind, tstep', &
                          firsttimeloop, refine, rewind, tstep

!     --- Remove semi-permanent arrays

      if ( associated(refinenew) ) deallocate(refinenew)
      call seprmsolinfo ( itempold )

      if ( timefinished ) usectimen = .false.

      call sepfillkmesh(kmesh2d(meshnr))

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prenthrefine'
         write(irefwr,*) 'timefinished (prenthrefineend)', timefinished

      end if
      call erclos ( 'prenthrefine' )

      end
