      subroutine printimehis ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 28-03-2015 Extension with scalars
!        version  2.0  date 16-04-2014 Extra parameters isolut2d, kprob
!        version  1.4  date 20-06-2013 New definition file ref numbers
!
!   copyright (c) 2010-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print time history for one vector or scalar
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     time_history
!     scalar
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletimhis
      use sepmodulemain
      use sepmodulemnsub
      use sepmodulecpack
      use sepmoduleplot
      use sepmodulekprob
      use sepmoduletexts
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         For a description see subroutine askpritimhis
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivect, ifile, nch1, irefwrsav, idegfd, &
              iscal, num, i, numtim
      integer, allocatable :: iwork(:)
      double precision, allocatable :: work(:,:)
      character (len=80) namear

!                    they are not
!     i              General loop variable
!     idegfd         Degree of freedom sequence number
!     ifile          Sequence number of file name in array texts
!     irefwrsav      Help variable to store the value of irefwr
!     iscal          Sequence number of scalar
!     ivect          Vector sequence number
!     iwork          integer work array to store the sequence numbers of the
!                    scalars
!     namear         Name of array
!     nch1           Number of characters in name
!     num            Number of scalars
!     numtim         number of timesteps
!     work           Work array to store the time history of all scalars
!                    to be printed as well as the time
!                    work(1,:) contains the time
!                    work(1+i,:) contains the i^th scalar
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICLOSEFILE   Close file and reset irefwr
!     INIOPENFILE    Open serial file and set irefwr
!     PRININ         print 1d integer array
!     PRINTIMEHIS1   Write contents of time history array to file
!     PRINTIMEHIS2   Write contents of scalar time history array to file
!     PRINTIMEHIS3   Store contents of time history of one scalar in array work
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3051   Degree of freedom out of range
!    3052   No time history has been stored for vector
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printimehis' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printimehis'
         write(irefwr,1) 'istartmn, inodenr', istartmn, inodenr
         write(irefwr,*) 'parallel ', parallel
         call prinin ( inpsep, 2, 'inpsep' )
  1      format ( a, 1x, (10i9) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ivect = inpsep(1)    ! vector sequence number

!     --- skip in parallel case and not on host

      if ( parallel .and. inodenr/=1 ) then

!     --- Parallel but not on host: jump

         ifile = 0  ! no file
         num = 0
         if ( ivect<-10000 ) num = -ivect-10000
         go to 900

      end if  ! ( parallel .and. inodenr/=1 )

!     --- Get parameters

      ifile = inpsep(2)    ! file sequence number
      idegfd = inpsep(4)   ! degree of freedom
      nphysi = 1            ! number of degrees of freedom per point
      num = 0

      if ( ifile>0 ) &
         call iniopenfile ( ifile, irefwrsav, inpsep(3), ' ' )

      if ( ivect<-10000 ) then

!     --- scalars to be printed

         num = -ivect-10000  ! number of scalars
         numtim = istephiscal-idegfd  ! number of timesteps

         allocate ( work(num+1,numtim), iwork(num), stat = error )
         if ( error/=0 ) call eralloc ( error, num*numtim, 'work' )
         if ( ierror/=0 ) go to 1000

!        --- Store scalars in work array

         do i = 1, num

            iscal = inpsep(4+i)
            iwork(i) = iscal
            rvectim => scaltim(iscal)%arr
            call printimehis3 ( work, i, idegfd, num )

         end do  ! i = 1, num

         call printimehis2 ( work, num, iwork, idegfd )

         deallocate ( iwork, work, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iwork' )
         if ( ierror/=0 ) go to 1000

         go to 900

      else if ( ivect<-1000 ) then

         select case (ivect)
         case(-1001)
            namear = 'x'
         case(-1002)
            namear = 'y'
         case(-1003)
            namear = 'z'
         end select  ! case (ivect)

         ivect = maxvc1-ivect-1000

      else if ( ivect==-1000 ) then

         ivect = maxvc1+1
         namear = 'x'

      else

         nch1  = index(names(ivect),' ')-1
         if ( debug ) write(irefwr,1) 'nch1', nch1
         namear = names(ivect)(1:nch1)//'_timehis'

         call sepactsolbf2 ( isolut2d(ivect) )

      end if  ! ( ivect<-1000 )

      if ( debug ) write(irefwr,1) 'ivect, ifile', ivect, ifile
      if ( debug ) write(irefwr,*) namear

      if ( vecttim(ivect)%npoint_hist<=0 ) then

!     --- Error: There is no time history for this vector

         call errint ( ivect, 1 )
         call errchr ( namear, 1 )
         call errsub ( 3052, 10, 0, 1 )
         go to 1000

      end if  ! ( vecttim(ivect)%npoint_hist<=0 )

      if ( idegfd>nphysi ) then

!     --- Degree of freedom out of range

         call errint ( idegfd, 1 )
         call errint ( nphysi, 2 )
         call errchr ( namear, 1 )
         call errsub ( 3051, 2, 0, 1 )
         go to 1000

      end if  ! ( idegfd>nphysi )

      rvectim => vecttim(ivect)%arr
      npoint_hist = vecttim(ivect)%npoint_hist
      if ( debug ) write(irefwr,1) 'npoint_hist', npoint_hist

      call printimehis1 ( idegfd )

900   if ( ifile>0 ) call iniclosefile ( irefwrsav )

!     --- Raise istartmn

      istartmn = istartmn + 5 + num

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istartmn', istartmn
         write(irefwr,*) 'End printimehis'

      end if  ! ( debug )
      call erclos ( 'printimehis' )

      end
