      subroutine prinversemap2d ( npoint1, npoint2, coor1, coor2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-02-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpolate node in new mesh from nodes in old mesh
!     2d case (linear triangles)
! **********************************************************************
!
!                       KEYWORDS
!
!     mapping_array
!     map
!     2d
!     linear
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint1, npoint2
      double precision coor1(ndim,npoint1), &
                       coor2(ndim,npoint2)

!     coor1          i    Array containing the coordinates of the first mesh
!     coor2          i    Array containing the coordinates of the second mesh
!     npoint1        i    number of points in first mesh
!     npoint2        i    number of points in second mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, inode, nelms, jnode, icount, inpelm, &
              ielem, ikelmc, k, ielgrp
      double precision dphidx(12), dphidy(3), xh(4), yh(4), zh(4)
      integer, allocatable :: ielems(:)
      logical coupled, found, extradebug

!     coupled        If true old node is coupled to itself
!     dphidx         Array of length 3 containing the
!                    x-derivatives of the 3 basis functions
!     dphidy         Array of length 3 containing the
!                    y-derivatives of the 3 basis functions
!     extradebug     If true extra debugging statements are carried out
!     found          If true the node is part of the element
!     i              Counting variable
!     icount         Counter
!     ielem          Element number
!     ielems         Contains all elements a node is lying in
!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     inode          Node sequence number
!     inpelm         number of points per element
!     j              Counting variable
!     jnode          Array containing the node numbers of an element
!     k              Counting variable
!     nelms          Number of elements in ielems
!     xh             contains x coordinates of the vertices of an element
!     yh             contains y coordinates of the vertices of an element
!     zh             contains z coordinates of the vertices of an element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getstartkmeshc, getshapeelgrp
      logical checknodeinelm

!     CHECKNODEINELM Check if node is in element
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     GETELMSNODEH1  Find all elements corresponding to nodal point inode
!     GETSHAPEELGRP  Get the shape of an element in group ielgrp
!     GETSTARTKMESHC Compute starting address in kmeshc and information of
!                    element
!     PRGEOMTETRAHED Compute geometrical quantities of tetrahedron
!     PRGEOMTRIANGLE Compute geometrical quantities of triangle
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINTERPOLNOD2 Check if node is in element, if so interpolate (2d, trian)
!     PRINTERPOLNOD3 Check if node is in element, if so interpolate (3D)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1382   Value of NPELM is not yet implemented for this subroutine
!    3095   Node in old mesh not coupled to same node in new mesh
!    3096   Node in new mesh is interpolated by more than 2 nodes in new mesh
!    3097   Node in old mesh is not interpolated
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinversemap2d' )
      debug = .false. .and. ioutp>=0
      extradebug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinversemap2d'
         write(irefwr,1) 'npoint1, npoint2, ndim', npoint1, npoint2, ndim
  1      format ( a, 1x, (10i6) )
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      ielgrp = sepelgrpelem(1)
      ishape = sepgetshape ( ielgrp )
      if ( ishape/=3 .and. ishape/=11 ) then

!     --- Error: elements of shape/=3 not yet implemented

         call errint ( ishape, 1 )
         call errsub ( 1382, 1, 0, 0 )
         go to 1000

      end if  ! ( ishape/=3 .and. ishape/=11 )

      allocate ( ielems(nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, nelem, 'ielems' )
      if ( ierror/=0 ) go to 1000
      do i = 1, npoint2

         if ( imap2(i,1)==0 ) then

!        --- Node i has not yet been interpolated
!            Check if new node is coupled to old node

            coupled = .false.
            do j = 1, imap(i,1)
               inode = imap(i,j+1)
               if ( inode==i ) then

!              --- new node is coupled to old one

                  coupled = .true.
                  exit

               end if  ! ( inode==node )

            end do  ! j = 1, imap(i,1)

            if ( .not. coupled ) then

!           --- New node not coupled to old one: not yet implemented

               call errint ( inode, 1 )
               call errsub ( 3095, 1, 0, 0 )
               go to 1000

            end if  ! ( .not. coupled )

!           --- Find all elements in new mesh containing node inode
!               Only elements with highest dimension are applicable

            nelms = nelem
            call getelmsnodeh1 ( inode, ielems, nelms )

            if ( extradebug ) then
               write(irefwr,1) 'inode', inode
               call prinin ( ielems, nelms, 'ielems' )
            end if  ! ( debug )

            if ( imap(i,1)==2 ) then

!           --- Remove all elements containing the other node
!               This is the case that the new node is interpolated between
!               2 old nodes.

               if ( imap(i,2)==inode ) then
                  jnode = imap(i,3)
               else
                  jnode = imap(i,2)
               end if  ! ( imap(i,2)==inode )
               if ( extradebug ) write(irefwr,1) 'jnode', jnode

               icount = 0
               do j = 1, nelms

                  ielem = ielems(j)
                  ikelmc = &
                     getstartkmeshc ( inpelm, ielem )
                  found = &
                     checknodeinelm ( kmeshc(ikelmc), inpelm, jnode )
                  if ( extradebug ) then
                     write(irefwr,1) 'ielem', ielem
                     call prinin ( kmeshc(ikelmc), inpelm, 'nodes' )
                  end if  ! ( debug )

                  if ( .not. found ) then

!                 --- Node not in element, add element to list

                     icount = icount+1
                     ielems(icount) = ielem

                  end if  ! ( .not. found )

               end do  ! j = 1, nelms

               nelms = icount
               if ( extradebug ) call prinin ( ielems, nelms, 'ielems/2' )

               do j = 1, nelms

                  ielem = ielems(j)
                  ikelmc = &
                     getstartkmeshc ( inpelm, ielem )

!                 --- Interpolate old coordinate in new elements

                  if ( ndim==2 ) then

!                 --- ndim = 2

                     call prgeomtriangle ( coor1, xh, yh, &
                                           kmeshc(ikelmc), dphidx, &
                                           dphidy )
                     call printerpolnod2 ( inode, coor2, dphidx, dphidy, &
                                           npoint2, xh, &
                                           yh, ielem, kmeshc(ikelmc) )

                  else

!                 --- ndim = 3

                     call prgeomtetrahed ( coor1, xh, yh, zh, &
                                           kmeshc(ikelmc), dphidx )
                     call printerpolnod3 ( inode, coor2, dphidx, &
                                           npoint2, xh, yh, zh, &
                                           ielem, kmeshc(ikelmc) )

                  end if  ! ( ndim==2 )

                  if ( extradebug ) then
                     write(irefwr,1) 'ielem', ielem
                     call prinrl ( xh, ndim+1, 'x_new' )
                     call prinrl ( yh, ndim+1, 'y_new' )
                     if ( ndim==3 ) call prinrl ( zh, ndim+1, 'z_new' )
                     call prinrl ( coor2(1,inode), ndim, 'coor_old' )
                  end if  ! ( debug )

                  if ( imap2(inode,1)>0 ) then

                     if ( extradebug ) then
                        write(irefwr,100) inode, &
                           ( imap2(inode,k+1), rmap2(inode,k), &
                             k=1,imap2(inode,1) )
100                     format ( i6, ':', 20 ( i6, d12.3 ) )
                     end if  ! ( debug )

                     exit

                  end if  ! ( imap2(inode,1)>0 )

               end do  ! j = 1, nelms

            else

!           --- Other situations are not available yet

               call errint ( inode, 1 )
               call errint ( imap(i,1), 2 )
               call errsub ( 3096, 2, 0, 0 )
               go to 1000

            end if  ! ( imap(i,1)==2 )

         end if  ! ( imap(i,1)==1 )

      end do  ! i = 1, npoint2

!     --- Check if there are any nodes that have not been interpolated

      do i = 1, npoint2

         if ( imap2(i,1)==0 ) then

!        --- Node i has not yet been interpolated

            call errint ( i, 1 )
            call errsub ( 3097, 1, 0, 0 )
            go to 1000

         end if  ! ( imap2(i,1)==0 )

      end do  ! i = 1, npoint2

      deallocate ( ielems, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ielems' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinversemap2d'

      end if  ! ( debug )
      call erclos ( 'prinversemap2d' )

      end
