      subroutine proutp ( isubr, timelc, iseqsol, iseqnr, &
                          iter, filusr, ivectr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.6  date 19-04-2014 Extension with append
!        version  6.5  date 09-10-2013 Extension with SEPARATE_ELGRPS
!        version  6.4  date 21-11-2012 Long integers
!
!   copyright (c) 1988-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Subroutine PROUTP is the actual subroutine both for subroutine
!    OUTPUT (stationary case) and subroutine OUTTIM (time-dependent case)
!    PROUTP writes the solution as stored in the solution array
!            corresponding to ISOL to the file sepcomp.out and
!            other necessary information to the file sepcomp.inf
!    PROUTP replaces one or several calls of the SEPRAN subroutine:
!           PRINSL: Write arrays to a file
!           DERIVA: Computes derived quantities
! **********************************************************************
!
!                       KEYWORDS
!
!     output
!     read
!     write
!     tecplot
!     avs
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isubr, iseqnr, iter, ivectr, iseqsol
      double precision timelc
      logical filusr

!    filusr    i   Indication if arrays IUSER and USER have been filled
!                  explictly in the call (true) or that they must be found from
!                  IBUFFR (false)
!    iseqnr    i   Sequence number with respect to the storage in array ISEPIN
!                  In case of TUDFINVOL it is the vector sequence number to
!                  be used counted from 0
!    iseqsol   i   Sequence number of first vector in solut2d
!    isubr     i   Indicates the type of calling subroutine
!                  isubr consists of 2 parts isubr_act and nvect according to
!                  isubr = isubr_act + 10*nvect
!                  Possible values for isubr_act
!                  1:  OUTPUT
!                  2:  OUTTIM
!                  3:  OUTSOL
!                  4:  OUTPOS
!                  5:  TUDFINVOL
!                  nvect is only used if isubr_act=5 and it indicates the number
!                  of vectors to be written
!    iter      i   Iteration number
!    ivectr    i   sequence number of first vector to be used
!                  in the array isolvec(.,j) for output purposes.
!                  IVECTR-1 refers to the vector number in SEPPOST
!    timelc    i   Actual time corresponding to this call of PROUTP
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numveclc, nproblc, icounter, iavs, iseq, &
              isubract, nameavs, namedx, nameparaview, &
              one_file, nametec, appendmode
      logical separate_elgrp
      save nproblc, numveclc, icounter, iavs

!     appendmode     Defines if append mode must be used (1) or ot (0)
!     iavs           Indicates which kind of output must be created.
!                    This parameter consists of three parts:
!                    TOSEPPOST, TOAVS and TODX according to
!                    TOSEPPOST+2*TOAVS+4*TODX
!                    Possible values for TOSEPPOST
!                    1:  Create output for SEPPOST only in the
!                        form of file sepcomp.out
!                    0:  Do not create output for SEPPOST
!                    Possible values for TOAVS
!                    1:  Create output for AVS only in the form
!                        of a file sepavs_xxx.yyy
!                        With xxx a time sequence number of the
!                        shape 001, 002 etc
!                        and yyy a vector sequence number of the
!                        shape 001, 002 etc
!                    0:  Do not create output for avs
!                    Possible values for TODX
!                    1:  Create output for OPEN_DX only in the form
!                        of a file sepodx_xxx.yyy
!                        With xxx a time sequence number of the
!                        shape 001, 002 etc
!                        and yyy a vector sequence number of the
!                        shape 001, 002 etc
!                    0:  Do not create output for open dx
!     icounter       Integer time sequence number
!     iseq           Sequence number for array to be written
!     isubract       Parameter isubr_act, see isubr
!     nameavs        Sequence number of name of avs file in array
!                    file_names
!     namedx         Sequence number of name of open dx file in array
!                    file_names
!     nameparaview   Sequence number of name of paraview file in array
!                    file_names
!     nametec        Sequence number of name of tecplot file in array
!                    file_names
!     nproblc        Number of various problems
!     numveclc       Number of derivative vectors that have been written to file
!                    sepcomp.out
!     one_file       Indicates if all time steps must be stored in one file (1)
!                    or not (0)
!     separate_elgrp If true, each element group is written to a different
!                    output file
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PROUTPMAIN     Main body subroutine with respect to subroutine PROUTP
!     PROUTPRAISE    Raise icounter if files already exist
!     PROUTPSUB1     Set some parameters depending on the input list
!     PROUTPSUB2     Read input and use this to set other parameters
! **********************************************************************
!
!                       I/O
!
!   The solution is written to file "iref74"
!   Information is written to file "iref73"
!   The file sepcomp.inf is filled as follows:
!          First records array IINPUT as output from subroutine PROBDF
!          IINPUT is written by subroutine SEPSTR
!          Next records:
!          Rec. 1:  iversion, isubr
!          if ( isubr = 1 ) then
!             Rec. 2:  numvec
!             Rec. 3...: Sequence numbers of arrays that have been
!                        written to file sepcomp.out
!          else if ( isubr = 2 ) then
!             Rec. 2:  numvec
!             Rec. 3...: Sequence numbers of arrays that have been
!                        written to file sepcomp.out
!             Next records : each containing exactly one real number
!                        representing the time corresponding to the set of
!                        arrays written by one call of PROUTP
!          else if ( isubr = 3 ) then
!             From version 3: followed by
!                Rec. 2: numvec, lenipost, lenrpost
!                Rec. 3...numvec+2: names of the vectors
!                Rec. numvec+3: ipost when lenipost>0
!                Rec. numvec+4: rpost when lenrpost>0
!             Next records : each containing 10 numbers:
!                        sequence_number,iteration_number, time_level
!                        and 7 numbers 0 for future purposes
!          with
!              iversion: Version number of type of file
!                       0: is first version
!                       present version: 3
!              isubr:   type of solution that is written.
!                       Possible values:
!                       1:  stationary case (one call of output only)
!                       2:  Time-dependent  (Information is written only
!                                            once for each time-level t )
!                       3:  general case: There are more calls of output
!                           possible, each with its own sequence number
!                           However, the sequence numbers must be disjunct
!                           Furthermore always a time-level and an iteration
!                           number is available
!              numvec:  number of vectors that have been written to the
!                       file sepcomp.out, for each time-step
!                       It is assumed that in each time-step exactly the
!                       same arrays will be written
!              output_sequence: Sequence number of output input
!              iteration_number: The output vector may be provided by a
!                       iteration number and a time-level
!   The arrays in the file sepcomp.out are written by the subroutine prinsl
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Perform tests
!    if ( first_call and isubr#3 ) then
!       read input
!       write information to file 73
!    else if ( isubr=3 ) then
!       activate input information stored for sequence_number
!    else ( not first_call and isubr#3 )
!       activate previous input information
!    write solution vectors to sepcomp.out
!    numveclc = number of derivative vectors to be computed
!    if ( numveclc>0 ) then
!       Compute derivative vectors and write to sepcomp.out
!    write final information to sepcomp.inf
!             Local arrays that are stored in array IBUFFR
!    iinput        Contains the actual integer information from
!                  the standard input file
!                  Is stored in ibuffr from position ipiinp
!                  See subroutine READ04
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data icounter/0/
! ======================================================================
!
      call eropen ( 'proutp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from proutp'
         write(irefwr,1) 'isubr, iseqnr, iter, ivectr', &
                          isubr, iseqnr, iter, ivectr
         write(irefwr,2) 'timelc', timelc
         write(irefwr,*) 'filusr', filusr
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Set some parameters depending on the input list

      call proutpsub1 ( icounter, isubract, &
                        iseq, isubr, numveclc, iavs, iseqnr )

!     --- Read input and use this to set other parameters

      call proutpsub2 ( isubract, ivectr, nproblc, numveclc, iavs, &
                        nameavs, namedx, one_file, iseq, iseqnr, &
                        nametec, nameparaview, separate_elgrp, &
                        appendmode )

      if ( debug ) &
         write(irefwr,1) 'icounter, appendmode', icounter, appendmode
      if ( icounter==1 .and. appendmode==1 ) then

!     --- Raise icounter depending on type of file and existence of files

         call proutpraise ( icounter, iavs, nameavs, nametec )
         if ( debug ) write(irefwr,1) 'icounter', icounter

      end if  ! ( appendmode==1 )

!     --- Main body of subroutine

      isol2d => isolut2d(iseqsol:size(isolut2d))

      call proutpmain ( iavs, isubract, ivectr, nproblc, &
                        numveclc, filusr, icounter, iseq, iter, nameavs, &
                        namedx, nametec, one_file, timelc, &
                        nameparaview, separate_elgrp )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End proutp'

      end if
      call erclos ( 'proutp' )

      end subroutine proutp
