      subroutine prrefineread ( iinputref, rinputref, &
                                niinpref, nrinpref, inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 20-08-2016 use sepmoduleinput
!        version  1.2  date 24-12-2014 New definition clinsl
!        version  1.1  date 21-11-2012 Long integers
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Combine input  read from structure block and refine block
!     into one input array
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinputref(*), niinpref, nrinpref, inpsep(*)
      double precision rinputref(*)

!     iinputref      o    Contains the integer input for refine extended with
!                         defaults
!     inpsep         i    The integer input read in the structure block
!     niinpref       i    Declared length of array iinputref
!     nrinpref       i    Declared length of array rinputref
!     rinputref      o    Contains the real input for refine extended with
!                         defaults
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqnr, leninp, lenrnp, k, istart, maxcal, ipos
      character (len=8) name(3)

!                    they are not
!     ipos           Position of input arrays in sepmoduleinput
!     iseqnr         Sequence number with respect to the storage in array
!                    ISEPIN
!     istart         First position used at a part (dummy)
!     k              Counting variable
!     leninp         Computed length of integer input array
!     lenrnp         Computed length of real input array
!     maxcal         Maximum value of icall
!     name           Array with names for error messages
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     IINPUTSEQ      Read contents of input block with sequence number iseqnr
!                    corresponding to main keyword name
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     SEPGETINPFINF  Extract infor entries of integer and real input arrays from
!                    ISEPIN
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2756   Parameter instructure block different from refine block
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Process input read in refine block if available
!     First check if input has been read
!     if ( maxcal=0 ) then
!        Copy input from inpsep
!     else
!        merge inpsep and input read in refine block
!     Fill defaults
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data name / 'TIMES', 'MESH_IN', 'MESH_OUT' /
! ======================================================================
!
      call eropen ( 'prrefineread' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prrefineread'

      end if
      if ( ierror/=0 ) go to 1000
      call inichkarlen ( 7, niinpref, 'iinputref' )
      if ( ierror/=0 ) go to 1000

!     --- Process input read in refine block if available
!         First check if input has been read

      call sepgetinpfinf ( 28, 1, ipos, maxcal )

      if ( maxcal==0 ) then

!     --- No input block refine found, use defaults only

         iinputref(1) = inpsep(1)
         iinputref(2) = inpsep(2)
         iinputref(3) = inpsep(3)
         iinputref(4) = 0
         iinputref(5) = 0
         iinputref(6) = 0

      else

!     --- Input block refine found, merge inputs

          iseqnr = inpsep(4)
          leninp = niinpref
          lenrnp = nrinpref
          call iinputseq ( 28, iseqnr, iinputref, &
                           rinputref, leninp, lenrnp, 'REFINE' )
          if ( ierror/=0 ) go to 1000

!         --- Merge inputs

          do k = 1, 3

             if ( iinputref(k)==-1 ) then

!            --- iinputref(k)=-1, not yet filled
!                use inpsep

                if ( inpsep(k)>0 ) then
                   iinputref(k) = inpsep(k)
                else if ( k<3 ) then
                   iinputref(k) = 1          ! default value
                else  ! k=3
                   iinputref(k) =  iinputref(k-1)     ! mesh_out=mesh_in
                end if  ! ( inpsep(k)>0 )

             else if ( inpsep(k)/=-1 ) then

!            --- Special case: both iinputref and iinpsep are filled
!                Must be the same

                if ( inpsep(k)/=iinputref(k) ) then

!               --- error: different values iin inpsep and iinputref

                   call errchr ( name(k), 1 )
                   call errint ( inpsep(k), 2 )
                   call errint ( iinputref(k), 3 )
                   call errsub ( 2756, 2, 0, 1 )

                end if  ! ( inpsep(k)/=iinputref(k) )

             end if  ! ( iinputref(1)==-1 )

          end do  ! k = 1, 3

      end if  ! ( maxcal==0 )

!     --- Fill defaults

      if ( iinputref(1)==-1 ) iinputref(1) = 1            ! nrefine
      if ( iinputref(2)==-1 ) iinputref(2) = 1            ! meshin
      if ( iinputref(3)==-1 ) iinputref(3) = iinputref(1) ! meshout
      if ( iinputref(4)==-1 ) iinputref(4) = 0            ! local refine

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prrefineread'

      end if
      call erclos ( 'prrefineread' )

      end
