      subroutine prrmlevmesh ( inpsep, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.1  date 07-03-2012 Change in interpolation
!        version  3.0  date 10-02-2012 Extra parameter maxmesh
!
!   copyright (c) 2004-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Delete the levelset mesh created for a zero levelset crossing the mesh
!     Also delete the corresponding problem definition arrays and
!     matrix structure arrays.
!     First the solution is copied from the levelset mesh to the original mesh
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     level_set
!     intersection
!     delete
!     matrix_structure
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulemeshinf
      use sepmodulekprob
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), istart

!     inpsep         i    The integer input read is stored in array INPSEP
!                         1:  MESH_ORIG Sequence number of mesh to be adapted
!                         2:  MESH_OBST Sequence number of adapted mesh
!                         3:  SEQ_STRUCTURE (not used)
!     istart        i/o   Starting address in array INPVEC
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer meshorig, meshobst, i, icurvs(1), iseqlc, &
              iseqlevel, numinterpol, inpcop(5), itypvc

!                    they are not
!     i              Counting variable
!     icurvs         Dummy
!     inpcop         Copy of part of integer input inpsep
!     iseqlc         Sequence number
!     iseqlevel      Sequence number of level set function in isol
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     meshobst       Sequence number of levelset mesh to be used
!     meshorig       Sequence number of original mesh to be used
!     numinterpol    number of vectors to be interpolated
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHINPSOL    Adapt input array with respect to variables
!     MSHRMOBSTMSHP  Remove some mesh subarrays that have been created while
!                    making an obstacle mesh
!     PRININ         print 1d integer array
!     PRINMAP        Print the contents of the mapping array map
!     PRINVERSEMAP   Fill reverse mapping array in case mapping array is given
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPFILLKPROB   Fill parameters in module sepmodulekprob
!     SEPINTPOLMS    Interpolate solution from one mesh to another
!     SEPRMINTMT     Remove array with sequence number iseq from intmt
!     SEPRMMESHINFO  Remove array with sequence number iseq from km
!     SEPRMPROBINFO  Remove array with sequence number kproblc from kp
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1055   Array in solution vector has not been filled
!    2951   Sequence number of vectors out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prrmlevmesh' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prrmlevmesh'
         write(irefwr,1) 'istart', istart
         call prinin ( inpsep, 9, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Replace scalars by actual values

      inpcop(1:5) = inpsep(1:5)
      call inichinpsol ( inpsep, 5 )

!     --- Copy solution from new mesh onto old mesh if possible

      meshorig = meshseqs(inpsep(1))
      meshobst = meshseqs(inpsep(2))
      numinterpol = 0

      if ( debug ) then
         write(irefwr,1) 'meshorig, meshobst', meshorig, meshobst
         call prinin ( isolut3d(1,meshorig), numvecmn, 'isol_orig' )
         call prinin ( isolut3d(1,meshobst), numvecmn, 'isol_obst' )
      end if  ! ( debug )

      if ( meshobst<0 ) then

!     --- Special case submesh coincides with original mesh

         meshseqs(inpsep(2)) = -meshseqs(inpsep(2))
         go to 1000

      end if

      if ( map(meshobst,meshorig)%iseqkm1==0 .and. &
           map(meshorig,meshobst)%iseqkm1>0 ) then

!     --- inverse mapping exists, new mapping does not, create new mapping
!         by using the already existing inverse mapping

         if ( debug ) &
            call prinmap ( map(meshorig,meshobst), &
                           kmesh2d(meshobst) )

         call prinversemap ( map(meshorig,meshobst), &
                             map(meshobst,meshorig), &
                             kmesh2d(meshobst), kmesh2d(meshorig) )
         iinmap(2) = 2   ! array map has now been filled
         iinmap(4) = meshobst
         iinmap(5) = meshorig

      end if  ! ( map(meshobst,meshorig)==0 .and. ... )

!     --- Copy solution from old mesh onto new mesh if possible

      iseqlevel = inpsep(4)
      numinterpol = inpsep(5)

      if ( debug ) write(irefwr,1) 'numinterpol', numinterpol

      if ( numinterpol==0 ) then

!     --- numinterpol = 0, interpolate all vectors

         do i = 1, numvecmn
            if ( isolut3d(i,meshobst)>0 ) then
               itypvc = ks(isolut3d(i,meshobst))%typevec
               if (itypvc==110 .or. itypvc==115) then
                  call sepintpolms ( meshobst, meshorig, iinmap, &
                                     isolut3d(i,meshobst), &
                                     isolut3d(i,meshorig), &
                                     map(meshobst,meshorig), icurvs )
                  call seprmsolinfo ( isolut3d(i,meshobst) )
               end if
            end if  ! ( isolut3d(i,meshobst)>0 )
         end do

      else if ( numinterpol>0 ) then

!     --- numinterpol#0, interpolate some vectors

         do i = 0, max(0,numinterpol)

            if ( i==0 ) then

!           --- interpolate phi

               iseqlc = iseqlevel

            else

!           --- interpolate rest

               iseqlc = inpsep(9+i)
               if ( iseqlc<=0 .or. iseqlc>numvecmn ) then

!              --- error: iseqlc out of range

                  call errint ( iseqlc, 1 )
                  call errint ( numvecmn, 2 )
                  call errsub ( 2951, 2, 0, 0 )
                  go to 1000

               end if  ! ( iseqlc<=0 .or. iseqlc>numvecmn )

            end if  ! ( i==0 )
            if ( debug ) write(irefwr,1) 'iseqlc', iseqlc

            if ( isolut3d(iseqlc,meshobst)>0 ) then
               itypvc = ks(isolut3d(iseqlc,meshobst))%typevec
               if (itypvc==110 .or. itypvc==115) then

!              --- vector exists, interpolate

                  call sepintpolms ( meshobst, meshorig, iinmap, &
                                     isolut3d(iseqlc,meshobst), &
                                     isolut3d(iseqlc,meshorig), &
                                     map(meshobst,meshorig), icurvs )

                  if ( inpsep(8)==0 ) &
                     call seprmsolinfo ( isolut3d(i,meshobst) )

               else

!              --- error vector does not exist

                  call errint ( iseqlc, 1 )
                  call errint ( ks(isolut3d(iseqlc,meshobst))%typevec, 2 )
                  call errchr ( names(iseqlc), 1 )
                  call errsub ( 1055, 2, 0, 1 )

               end if  ! ( isolut3d(iseqlc,meshobst)>0 ... )

            end if  ! ( isolut3d(iseqlc,meshobst)>0 )

         end do  ! i = 0, numinterpol

      end if  ! ( numinterpol==0 )
      meshnr = meshorig

!     --- Delete levelset mesh

      if ( inpsep(8)==0 ) then

!     --- inpsep(8)=0, delete complete mesh

         call seprmmeshinfo ( kmesh2d(meshobst) )

      else

!     --- inpsep(8)>0, delete mesh partly

         call mshrmobstmshp ( kmesh2d(meshobst) )

      end if  ! ( inpsep(8)==0 )

!     --- Delete levelset problem definition

      removesol = .false.
      call seprmprobinfo ( kprob2d(meshobst) )
      removesol = .true.

!     --- Delete levelset matrix structure

      do i = 10, 1, -1
         if ( intmat2d(i,meshobst)>0 ) &
            call seprmintmt ( intmat2d(i,meshobst) )
      end do  ! i = 10, 1, -1

!     --- reset inpsep

      inpsep(1:5) = inpcop(1:5)

      kmesh => kmesh2d(meshorig)
      kprob => kprob2d(meshorig)
      iseqkp = kprob
      iprob = 1
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      call sepfillkmesh ( kmesh )
      call sepfillkprob

1000  istart = istart+10+max(0,numinterpol)

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istart/end', istart
         write(irefwr,*) 'End prrmlevmesh'

      end if  ! ( debug )
      call erclos ( 'prrmlevmesh' )

      end subroutine prrmlevmesh
