      subroutine prsclr ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.12 date 28-03-2015 Correction numscl
!        version  2.11 date 24-04-2014 Extra debug
!        version  2.10 date 08-12-2013 Extension with integer scalars
!
!   copyright (c) 1994-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill scalar corresponding to the input part SCALAR i = ...
!
! **********************************************************************
!
!                       KEYWORDS
!
!     fill
!     fill_scalar
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecons
      use sepmoduletexts
      use sepmoduleinput
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(0:*)
      double precision rinsep(*)

!     inpsep  i    The integer input read is stored in array INPSEP
!                  0  Number of values stored in this part of inpsep
!                  1  Sequence number scalar
!                  2  Type of construction
!                     1:  value
!                     2:  func
!                     3:  min
!                     4:  max
!                     5:  integer constant
!                     6:  real constant
!                     8:  expression
!                     9:  variable
!                  3: IPOS1        Position of value in rinsep (>0) or
!                                  parameter IFUNC in
!                                  FUNCSCAL ( IFUNC, SCALARS)
!                                  In case of min/max: number of parameters
!                                  to be compared
!                                  In case of itype = 5 to 7 starting position
!                                  of quantity in incons, rlcons or scalars
!                  4: IPOS2        In case of min/max: information of
!                                  first parameter:
!                                  >0: value in rinsep;
!                                  <0 -Scalar sequence number
!                                  In case of itype = 5 to 7 shift in position
!                                  because an array has been found
!                                  If the shift itself is variable ipos2 = -1
!                                  and the variable sequence number is stored
!                                  in position 5
!                  5...            Rest of parameters in the same way as ipos1
!     rinsep  i    Real input array (Depends on inpsep)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iscal, itype, ipos1, ipos2, maxindex, leninpsep, ivalue, &
              numarrays
      double precision rvalue, value
      logical integerfound

!     integerfound   If true an integer has been found
!     ipos1          See inpsep(3)
!     ipos2          See inpsep(4)
!     iscal          Sequence number of scalar, see inpsep(2)
!     itype          Type of computation, see inpsep(2)
!     ivalue         Integer value
!     leninpsep      Length of array inpsep
!     maxindex       Upper bound of index for an array
!     numarrays      Number of indexed vectors in vector array
!     rvalue         Real value
!     value          Real value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getscalindex
      double precision funcscal, prsclrminmax

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSCALVAL      Fill value of scalar defined by ipos into either rvalue or
!                    ivalue
!     FUNCSCAL       Compute scalar as function of scalars
!     GETSCALINDEX   Return with adapted index of scalar
!     PREVALEXPR     Evaluate expression and fill actual values of scalars
!     PRININLARGE    Print 1d integer vector (large numbers)
!     PRSCLRMINMAX   Evaluate scalar i = min/max ( ... )
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1613   The index number read is smaller than 1 or larger than 1000
! **********************************************************************
!
!                       PSEUDO CODE
!
!     itype := inpsep(2)
!     if ( itype=1 ) then
!        Fill value in scalar
!     else if ( itype=2 ) then
!        Fill scalar with result of funscal
!     else if ( itype=3 ) then
!        Fill scalar with result of minimum
!     else if ( itype=4 ) then
!        Fill scalar with result of maximum
!     else if ( itype>4 ) then
!        Fill value in scalar
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsclr' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsclr'
         call prininlarge ( inpsep(1), inpsep(0), 'inpsep' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      iscal = getscalindex(inpsep(1))
      itype = inpsep(2)
      if ( debug ) write(irefwr,1) 'iscal, itype', iscal, itype

      if ( itype==9 ) then

!     --- get_time

         scalars(iscal) = t

      else if ( itype<8 ) then

!     --- itype<8, standard case

         ipos1 = inpsep(3)
         ipos2 = inpsep(4)
         if ( debug ) write(irefwr,1) 'ipos1,ipos2', ipos1,ipos2
         if ( ipos1>=100000000 ) then

!        --- Scalar found, use value

            call erscalval ( ipos1-100000000, rvalue, ivalue, &
                             integerfound )
            if ( ierror/=0 ) go to 1000
            if ( integerfound ) then
               ipos1 = ivalue
            else
               ipos1 = nint(rvalue)
            end if  ! ( integerfound ) then
            if ( debug ) write(irefwr,1) 'ipos1', ipos1

         end if
         if ( ipos2>100000000 ) then

!        --- Scalar found, use value

            call erscalval ( ipos2-100000000, rvalue, ivalue, &
                             integerfound )
            if ( ierror/=0 ) go to 1000
            if ( integerfound ) then
               ipos2 = ivalue
            else
               ipos2 = nint(rvalue)
            end if  ! ( integerfound ) then
            if ( debug ) write(irefwr,1) 'ipos2', ipos2

         end if
         if ( debug ) &
            write(irefwr,1) 'ipos1, ipos2, itype', ipos1, ipos2, itype

         if ( itype==1 ) then

!        --- value

            value = rinsep(ipos1)

         else if ( itype==2 ) then

!        --- function

            value = funcscal ( ipos1, scalars )

         else if ( itype<5 ) then

!        --- minimum or maximum

            value = &
               prsclrminmax ( inpsep(4), rinsep, itype, ipos1, &
                              chscalars(iscal) )

         else

!        --- itype>4; value

            if ( ipos2==-1 ) then
               ipos2 = nint(scalars(getscalindex(inpsep(5))))
               if ( itype==5 ) then
                  maxindex = maxints
               else if ( itype==6 ) then
                  maxindex = maxreals
               else if ( itype==7 ) then
                  maxindex = maxcls
               end if
               if ( ipos2<1 .or. ipos2+ipos1>maxindex ) then

!              --- ipos2 out of range

                  call errint ( ipos2, 1 )
                  call errint ( 1, 2 )
                  call errint ( maxindex, 3 )
                  call errsub ( 1613, 3, 0, 0 )
                  ipos2 = 1
               end if
               ipos2 = ipos2-1
            end if
            ipos2 = ipos2+ipos1
            if ( itype==5 ) then
               value = incons(ipos2)
            else if ( itype==6 ) then
               value = rlcons(ipos2)
            else if ( itype==7 ) then
               value = scalars(ipos2)
            end if

         end if

         if ( iscal==0 ) then
            t = value
         else
            scalars(iscal) = value
         end if  ! ( iscal==0 )

      else

!     --- itype = 8, evaluate expression

         leninpsep = inpsep(0)-2
         if ( debug ) write(irefwr,1) 'leninpsep', leninpsep
         if ( iscal==0 ) then

!        --- Special case, iscal = 0, hence time = &

            call prevalexpr ( inpsep(3), t, 'time' )

         else if ( iscal<0 ) then

!        --- Scalar array

            iscal = -iscal
            numarrays = dimscals(iscal)

            if ( debug ) &
               write(irefwr,1) 'iscal, numarrays', iscal, numarrays

            do iseqarray = 1, numarrays

               call prevalexpr ( inpsep(3), scalars(iscal), &
                                 chscalars(iscal) )
               iscal = iscal+1

            end do  ! iseqarray = 1, numarrays

            iseqarray = 0  ! reset iseqarray

         else if ( leninpsep==1 .and. inpsep(3)<200000000 ) then

!        --- scalar found

            scalars(iscal) = scalars(inpsep(3)-100000000)

         else

!        --- Standard case

            call prevalexpr ( inpsep(3), scalars(iscal), &
                              chscalars(iscal) )

         end if  ! ( iscal==0 )

      end if  ! ( itype<8 )
      numscals = max(numscals,mod(iscal,1000))

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'numscals', numscals
         write(irefwr,2) 'scalars(iscal)', scalars(iscal)
         write(irefwr,*) 'End prsclr'

      end if
      call erclos ( 'prsclr' )

      end
