      subroutine read03iter ( inpsol, rinsol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.14 date 20-05-2015 Extension with parallel options
!        version  1.13 date 30-07-2014 Extension with options for petsc
!        version  1.12 date 03-04-2013 Extra option parallel_shadow
!
!   copyright (c) 2003-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read all information corresponding to the sub-keyword ITERATION_METHOD
!     corresponding to the main keyword SOLVE
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     solve
!     iteration
!     iterative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulestrings
      use sepmoduleread1
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsol(*)
      double precision rinsol(*)

!     inpsol        i/o   Contains integer information concerning the solution
!                         process.
!                         For a description see subroutine READ03
!     rinsol        i/o   Array containing the real user input if iread=-1
!                         For a description see subroutine READ03
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncod3=64
      integer ncode3(nncod3), itype3(nncod3), ncod3m(nncod3), &
              icgs, iprec, projvect, projmethod, i, m, iseqeps
      double precision :: tabmn3(nncod3), tabmx3(nncod3)
      character (len=28) code3(nncod3)
      logical :: lvalmin, lvalmax

!     code3          Table with the keywords with the options of
!                    iteration_method
!                    they are not
!     i              Counting variable
!     icgs           Indicates the type of non-symmetric solver that is
!                    used.
!     iprec          Type of preconditioning
!                    Possible values:
!                    0   no preconditioning
!                    1   diagonal scaling
!                    2   ILU using Eisenstat implementation
!                    3   ILU
!                    4   Gauss Seidel preconditioning
!                    6   a standard (=default) preconditioner: the choice must
!                        be done in CONGRD, it may depend on the storage scheme
!                    7   MILU using Eisenstat implementation
!                    8   Block SSOR
!                    9   Block ILU
!     iseqeps        Sequence number of epsilon in the set of scalars
!     itype3         Indicates the type of item expected after a keyword in the
!                    options table for code3. See subroutine INTX03
!     lvalmax        Indicates if maximum = has been found (true) or not (false)
!     lvalmin        Indicates if minimum = has been found (true) or not (false)
!     m              number of entries read
!     ncod3m         Table containing the number of maximum characters in
!                    the keyword table code3
!     ncode3         Table containing the number of significant characters in
!                    the keyword table code3
!     nncod3         Number of entries in code3
!     projmethod     Type of projection method
!     projvect       Type of projection vector
!     tabmn3         Contains the minimum values corresponding to code3
!     tabmx3         Contains the maximum values corresponding to code3
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: getscalind, sepitermeth

!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     GETSCALIND     Return with actual scalar number, where the original number
!                    is the modified one stored in the input array
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPITERMETH    Translate icode to value of iteration_method to be stored
!                    in inpsol
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       ITERATION_METHOD = CG, options
!                        = BICGSTAB, options
!                        = CGS, options
!                        = GMRES, options
!                        = GMRESR, options
!                        = GCR, options
!                        = OVERRELAXATION, options
!                        = SIMPLE_GCR, options
!                        = SIMPLER_GCR, options
!                        = HSIMPLER_GCR, options
!                        = MSIMPLER_GCR, options
!                        = LSC_GCR, options
!                        = QLSC_GCR, options
!                        = SYMMLQ, options
!                        = ML_multigrid, options
!                        = IDR, options
!                        = AL_GCR, options
!                        = SCHUR, options
!                        = RICHARDSON, options
!                        = CHEBYSHEV, options
!                        = FGMRES, options
!                        = DGMRES, options
!                        = BICG, options
!                        = TFQMR, options
!                        = TCQMR, options
!                        = CR, options
!                        = LSQR, options
!                        = PREONLY, options
!                        = DEFAULT, options
!        with options empty or a combination of the following possibilities:
!          preconditioning  =  none
!                              diagonal
!                              ilu
!                              eisenstat
!                              Gauss_Seidel
!                              mod_eisenstat
!                              old
!                              block_Gauss_Seidel
!                              block_eisenstat
!                              block_ilu
!                              jacobi
!                              bjacobi
!                              sor
!                              icc
!                              asm
!                              amg
!                              mumps
!                              ml
!                              hypre
!          parallel_preco   = none
!                             diagonal
!          parallel_shadow  = local  (default)
!                             global
!          max_iter         =  m
!          num_subequations =  m
!          accuracy         =  eps  (or alternatively eps=e)
!          In case of cg-type methods also the combination
!          abs_accuracy = eps1, rel_accuracy = eps2 may be used
!
!          print_level      =  p
!          krylov_space     =  k
!          ntrunc           =  n1
!          ninner           =  n2
!          sdimension       =  s
!          termination_crit =  absolute
!                              rel_residual
!                              rel_right_hand_side
!                              rel_solution
!          adapt_diagonal = a
!          laplace_shift = (a,b)
!          type_scaling = none
!                         row
!                         symmetric
!                         column
!          factor = f
!          at_error = stop
!                     resume
!          keep_preconditioning
!          start            =  old_solution
!                              zero
!                              random
!          lambda           =  l
!          iseq_diag        = i
!          iseq_precdiag    = i
!          prec_lumping
!          structure_premat = i
!          parallel_solver = block_jacobi
!                            gcr
!          subsolve_accurracy = eps (default: 1e-2)
!
!          keywords for the deflation type methods
!              projection_method = none
!                                  deflation
!                                  coarse_grid
!              projection_vectors = approximate_eigenvectors
!                                   algebraic
!              projection_accuracy = e
!              projection_ignore = t
!              projection_print_level = i
!              projection_plot_vectors = i
!              projection_keep   = destroy
!                                  keep
!                                  old
!          used to be:
!              proj_method = none
!                            approximate_eigenvectors
!              proj_accuracy = e
!              proj_ignore = t
!              proj_keep   = destroy
!                            keep
!                            old
!              proj_print_level = i
!              proj_plot_vectors = i
!              proj_type = i
!              deflation_method = deflation
!                                 coarse_grid_correction
!          the following options are meant for overrelaxation only
!          alpha      =  a
!          beta       =  b
!          omega      =  w
!          niter1     =  n1
!          minimum    =  m1 or (value=m1) or (func=i1)
!          maximum    =  m2 or (value=m2) or (func=i2)
!          degfd      =  i
!          the following options are meant for gcr_simple only
!          gamma      =  g
!          mu         =  mu
!        the options must be separated by some separator for example ,
!
!      For petsc the following options are available
!      abs_accuracy = eps
!      rel_accuracy = eps
!      div_tolerance = eps
!      damping_factor = d       ! used by richardson
!      emin = e1   emax = e2    ! used by chebyshev
!      ninner = n               ! used by gmres
!      drop_tolerance           ! used by ilu
!      num_levels = i           ! used by ilu
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1588   Two keywords are mutually exclusive
!    2897   keyword structure_premat found, but not preconditioning = ILU
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code3 / 'PRECONDITIONING', 'MAX_ITER_PAR', 'ACCURACY', 'EPS', &
                   'PRINT_LEVEL', 'KRYLOV_SPACE', 'ITER', 'LEVEL', &
                   'SEIDEL', 'TERMINATION_CRITERION', 'START', &
                   'KEEP_PRECONDITIONING', &
                   'NTRUNC', 'NINNER', 'ALPHA', 'BETA', 'OMEGA', &
                   'NITER', 'NITER', 'LAMBDA', 'MINIMUM', 'MAXIMUM', &
                   'DEGFD', 'ADAPT_DIAGONAL', 'FACTOR', 'AT_ERROR', &
                   'PROJ_ACCURACY', 'PROJ_IGNORE', &
                   'PROJ_METHOD', 'PROJ_KEEP', 'PROJ_PRINT_LEVEL', &
                   'PROJ_PLOT_VECTORS', 'REL_ACCURACY', &
                   'ABS_ACCURACY', 'ISEQ_DIAG', 'ISEQ_PRECDIAG', &
                   'PROJ_TYPE', 'PREC_LUMPING', 'TYPE_SCALING', &
                   'DEFLATION_METHOD', 'PROJECTION_ACCURACY', &
                   'PROJECTION_IGNORE', 'PROJECTION_METHOD', &
                   'PROJECTION_KEEP', 'PROJECTION_PRINT_LEVEL', &
                   'PROJECTION_PLOT_VECTORS', 'PROJECTION_VECTORS', &
                   'SDIMENSION', 'STRUCTURE_PREMAT', 'GAMMA', 'MU', &
                   'LDIMENSION', 'LAPLACE_SHIFT', 'PARALLEL_PRECO', &
                   'PARALLEL_SHADOW', 'DIV_TOLERANCE', 'DAMPING_FACTOR', &
                   'EMIN', 'EMAX', 'DROP_TOLERANCE', 'NUM_LEVELS', &
                   'PARALLEL_SOLVER', 'MAX_ITER', 'SUBSOLVE_ACCURRACY' /
      data ncode3 / 6,10, 3, 3, 4,   4, 3, 3, 3, 4, &
                    5, 4, 3, 3, 3,   3, 4, 5, 5, 3, &
                    5, 5, 3, 4, 4,   6, 8, 8, 8, 8, &
                    8, 8, 6, 6, 8,   8, 8, 8, 8, 6, &
                   15,15,15,13,15,  15,15, 4, 5, 5, &
                    2, 4, 4,11,11,   7, 6, 4, 4, 8, &
                    7,11, 3,11 /
      data ncod3m /15,12, 8, 3,11,  12, 4, 5, 5,21, &
                    5,20, 6, 6, 5,   4, 5, 5, 5, 6, &
                    7, 7, 5,14, 6,   8,13,11,11, 9, &
                   16,17,12,12, 9,  13, 9,12,12,16, &
                   19,17,17,15,22,  23,18,10,16, 5, &
                    2,10,13,14,15,  13,14, 4, 4,14, &
                   10,15, 8,28 /
      data itype3 / 5, 0, 1, 1, 0,   0, 0, 0,-1, 5, &
                    5, 2, 0, 0, 1,   1, 1,10, 0, 1, &
                   16,16, 0, 1, 1,   5, 1, 1, 5, 5, &
                    0, 0, 1, 1, 0,   0, 0, 2, 5, 5, &
                    1, 1, 5, 5, 0,   0, 5, 0, 0, 1, &
                    1, 0,19, 5, 5,   1, 1, 1, 1, 1, &
                    0, 5, 0, 1 /
      data tabmn3 / 1d0, 1d0, 0d0, 0d0,-1d0,  1d0, 1d0,  0d0, 0d0, 10d0, &
                    7d0, 0d0, 1d0, 1d0, 0d0,  0d0, 0d0,  1d0,-2d0,  0d0, &
                   -1d20,-1d20,1d0,0d0, 0d0, 14d0,1d-12, 0d0, 16d0,18d0, &
                   -1d0, 0d0, 0d0, 0d0, 1d0,  1d0, 1d0,  0d0, 0d0,  0d0, &
                   1d-12,0d0,16d0,18d0,-1d0,  0d0, 1d0,  1d0, 1d0, 1d-3, &
                   1d-6, 1d0, 2d0,36d0,38d0,  1d0, 1d-2, 1d0,1d-4,1d-12, &
                   1d0, 0d0, 1d0, 1d-20 /
      data tabmx3 / 7d0, 1d6,1d20,1d20, 4d0,200d0, 1d6, 1d20, 0d0, 13d0, &
                    9d0, 0d0,1d3, 1d2,  1d0,  1d0, 2d0,  2d0, 1d5,  1d0, &
                   1d20,1d20,1d2, 1d1,  1d0, 15d0, 1d0,  1d0, 17d0,20d0, &
                   20d0, 1d0,1d20,1d20, 1d2,  1d2, 3d0,  0d0, 0d0,  0d0, &
                    1d0, 1d0,17d0,20d0,20d0,  1d0, 1d0, 20d0, 1d1,  1d3, &
                    1d6,10d0,10d0,37d0,39d0, 1d10, 1d2,  1d2, 1d0,  1d0, &
                    1d2, 0d0, 1d4, 2d-1 /

!     code3   Table with the keywords with the options of iteration_method
!             Values:
!             1:  preconditioning
!             2:  max_iter_par
!             3:  accuracy
!             4:  eps (alternative for accuracy)
!             5:  print_level
!             6:  krylov_space
!             7:  iter (in case max iter is used)
!             8:  level (in case print level is used)
!             9:  seidel (in case Gauss Seidel is used)
!            10:  termination_criterion
!            11:  start
!            12:  keep_preconditioning
!            13:  ntrunc
!            14:  ninner
!            15:  alpha
!            16:  beta
!            17:  omega
!            18:  niter
!            19:  niter
!            20:  lambda
!            21:  minimum
!            22:  maximum
!            23:  degfd
!            24:  adapt_diagonal
!            25:  factor
!            26:  at_error
!            27:  proj_accuracy
!            28:  proj_ignore
!            29:  proj_method
!            30:  proj_keep
!            31:  proj_print_level
!            32:  proj_plot_vectors
!            33:  rel_accuracy
!            34:  abs_accuracy
!            35:  iseq_diag
!            36:  iseq_precdiag
!            37:  proj_type
!            38:  prec_lumping
!            39:  type_scaling
!            40:  deflation_method
!            41:  projection_accuracy
!            42:  projection_ignore
!            43:  projection_method
!            44:  projection_keep
!            45:  projection_print_level
!            46:  projection_plot_vectors
!            47:  projection_vectors
!            48:  sdimension
!            49:  structure_premat
!            50:  gamma
!            51:  mu
!            52:  ldimension
!            53:  laplace_shift
!            54:  parallel_preco
!            55:  parallel_shadow
!            56:  div_tolerance
!            57:  damping_factor
!            58:  emin
!            59:  emax
!            60:  drop_tolerance
!            61:  num_levels
!            62:  parallel_solver
!            63:  max_iter
!            64:  subsolve_accurracy
! ======================================================================
!
      call eropen ( 'read03iter' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read03iter'
         call prinin ( inpsol, 20, 'inpsol' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000
      icgs = 0

!     --- Read first record following iteration method

      call asknxt
      ierpri = 0
      call sepcopycode ( codeitmeth, nncodeitmeth, nscodeitmeth, ncodeitmeth )
      icode = sepsubkeyw()+1
      if ( debug ) write(irefwr,1) 'icode', icode

      icgs = sepitermeth ( icode )
      if ( ierror/=0 ) go to 1000

!     --- Read rest of record (options)

      table(1:nncod3) = 0d0
      table(1) = -1d0
      table(14) = 30d0   ! ninner
      if ( icgs==4 ) table(14) = 5d0
      table(18) = -1d1
      table(19) = -1d1
      table(25) = 0.95d0
      table(27) = 1d-2
      table(48) = 1d0   ! sdimension
      table(52) = 1d0   ! ldimension
      table(56) = 1d5   ! div_tolerance
      table(57) = 1d0   ! damping_factor
      table(58) = 1d2   ! emin
      table(59) = 1d-2  ! emax
      table(60) = 1d-3   ! drop_tolerance
      table(64) = 1d-2   !
      call sepcopytype ( nncod3, itype3, tabmn3, tabmx3 )
      tabmax(1) = nncodeprec
      tabmin(11) = istcodestart
      tabmax(11) = istcodeterm-1
      tabmin(10) = istcodeterm
      tabmax(10) = istcodeerr-1
      tabmin(26) = istcodeerr
      tabmax(26) = istcodeprojm-1
      tabmin(29) = istcodeprojm
      tabmax(29) = istcodeprojk-1
      tabmin(30) = istcodeprojk
      tabmax(30) = istcodetyps-1
      tabmin(39) = istcodetyps
      tabmax(39) = istcodedefl-1
      tabmin(40) = istcodedefl
      tabmax(40) = istcodeprojme-1
      tabmin(43) = istcodeprojme
      tabmax(43) = istcodeprojv-1
      tabmin(47) = istcodeprojv
      tabmax(47) = istcodeparp-1
      tabmax(53) = 10d0
      tabmin(54) = istcodeparp
      tabmax(54) = istcodeparsh-1
      tabmin(55) = istcodeparsh
      tabmax(55) = istcodeparso-1
      tabmin(62) = istcodeparso
      tabmax(62) = nncodepreca
      iretur = 0
      nmain = 0
      call sepcopycode ( code3, nncod3, ncode3, ncod3m )
      call sepcopycodesub ( codeprec, nncodepreca, nscodeprec )
      call sepreadsetkeyw
      if ( iretur==-1 ) go to 1000

      if ( debug ) then
         call prinrl ( table, nncod3, 'table' )
         call prinin ( infotable, nncod3, 'infotable' )
      end if  ! ( debug )

      iprec = nint(table(1))
      if ( debug ) write(irefwr,1) 'iprec', iprec
      if ( iprec>0 .and. iprec<=5 ) then
         iprec = iprec - 1
      else if ( iprec==6 ) then
         iprec = 7
      else if ( iprec==7 ) then
         inpsol(10) = 2
      end if
      if ( iprec>=11 ) iprec = iprec-3
      if ( iprec==-1 ) then
         if ( icgs==9 ) then
            iprec = 0   ! No preconditioner in case of ML-multigrid
         else
            iprec = 6   ! Default preconditioner
         end if  ! ( icgs==9 )
      end if  ! ( iprec==-1 ) then

      if ( iprec==14 ) inpsol(3) = 30  ! PRECONLY in case of mumps

      iprec = iprec+100*nint(table(54))
      inpsol(4) = iprec
      inpsol(5) =  max ( nint(table(63)), nint(table(7)) )  ! max_iter
      inpsol(6) =  max ( nint(table(5)), nint(table(8)) )
      inpsol(8) = nint(table(6))
      if ( inpsol(8)==0 ) then
         if ( icgs==4 .or. icgs==100 ) then
            inpsol(8) = 100
         else if ( icgs==10 ) then
            inpsol(8) = 0   ! cgs, local shadow vector
         else
            inpsol(8) = 20
         end if
      end if
      inpsol(43) = nint(table(2))   ! max_iter_par
      inpsol(13) = nint(table(13))
      if ( inpsol(13)==0 ) inpsol(13) = 20
      if ( icgs==100 ) then

!     --- icgs = 100, i.e. GCR
!         Set method = 3 (GMRESR) and ninner = 1

         inpsol(3) = 4
         inpsol(14) = 1
         if ( nint(table(14))>0 ) then

!        --- ninner may not be used in case of GCR

            call errchr ( 'GCR', 1 )
            call errchr ( 'NINNER', 2 )
            call errwar ( 1588, 0, 0, 2 )

         end if

      else

!     --- standard case

         if ( icgs==10 ) then

!        --- icgs = 10, i.e. IDR. Store sdimension in inpsol(14)

            inpsol(14) = nint(table(48))  ! sdimension
            inpsol(8) = max(nint(table(55))-1,0) ! shadow vec

         else if ( icgs==17 ) then

!        --- icgs = 17, i.e. BiCGSTAB(l). Store ldimension in inpsol(14)

            inpsol(14) = max(1,nint(table(52)))  ! ldimension

         else

!        --- icgs#10, other methods. Store ninner in inpsol(14)

            inpsol(14) = nint(table(14))  ! ninner

         end if  ! ( icgs==10 )

      end if
      iseqeps = 0
      if ( infotable(3)==11 ) then

!     --- accuracy is scalar

          iseqeps = getscalind(nint(table(3)))

      else if ( infotable(4)==11 ) then

!     --- eps is scalar

          iseqeps = getscalind(nint(table(4)))

      else if ( table(3)>0d0 .or. table(4)>0d0 ) then

!     --- accuracy = ... or eps = ... given

         rinsol(1) = max ( table(3), table(4) )

      else if ( table(33)>0d0 ) then

!     --- abs_accuracy = ... given

         rinsol(1) = table(33)

      else if ( table(34)>0d0 ) then

!     --- rel_accuracy = ... given

         rinsol(1) = table(34)

      end if

      if ( table(10)>0d0 )  inpsol(11) = nint(table(10))-1
      inpsol(9) = max(nint(table(11))-1,0)
      if ( table(12)>0d0 ) inpsol(10) = 1
      if ( table(26)>0d0 ) inpsol(22) = nint(table(26))-1
      rinsol(2) = table(24)
      do i = 30, 31
         inpsol(i) = nint(table(5+i))
      end do
      rinsol(3) = table(25)
      if ( table(38)>0d0 ) inpsol(33) = 1
      if ( icgs/=5 .and. table(39)>0d0 )  inpsol(34) = nint(table(39))-1
      if ( table(49)>0d0 ) then

!     --- table(49)>0, hence structure_premat found
!         Store in inpsol and check iprec (must be 3)

         inpsol(40) = nint(table(49)) ! structure_precmat
         if ( iprec/=3 ) then

!        --- iprec/=3, not implemented in this case

            call errsub ( 2897, 0, 0, 0 )

         end if  ! ( iprec/=3 )

      end if  ! ( table(49)>0d0 )

      inpsol(42) = 1
      if ( table(62)>0d0 ) inpsol(42) = nint(table(62))  ! type of par solver

      if ( inpsol(42)==2 ) then

!     --- Parallel solver is gcr, get accuracy

         rinsol(2) = 1d-1  ! Default accuracy
         if ( table(64)>0d0 ) rinsol(2) = table(64)  ! accuracy

      end if  ! ( inpsol(42)==2 )

      if ( icgs/=5 ) then

!     --- Not overrelaxation, store information about termination criterion

         if ( inpsol(11)==10 ) then

!        --- Default termination criterion

            if ( table(34)>0d0 ) then

!           --- abs_accuracy = ... given

               if ( table(33)>0d0 ) then

!              --- both abs_accuracy = ... and rel_accuracy = ... given

                  inpsol(11) = 110
                  rinsol(7) = table(34)

               else

!              --- only abs_accuracy = ... given

                  inpsol(11) = 2

               end if

            end if

         else if ( inpsol(11)/=2 ) then

!        --- Relative termination criterion

            if ( table(34)>0d0 ) then

!           --- abs_accuracy = ... given

               inpsol(11) = inpsol(11)+100
               rinsol(7) = table(34)

            end if

         end if

      end if  ! ( icgs/=5 )

      inpsol(11) = inpsol(11)+ 1000*iseqeps

      if ( icgs<=4 .or. icgs==10 .or. icgs==17 ) then

!     --- icgs = 1-4, 10, 17 cg, cgstab etc.

         rinsol(4) = table(20)

!        --- Information about projection

         rinsol(5) = max(table(27),table(41)) !proj_accur
         rinsol(6) = max(table(28),table(42)) !proj_ignore
         if ( rinsol(4)>0d0 ) inpsol(15) = 1
         inpsol(16) =  max(nint(table(31)),nint(table(45)))  !proj_print_level
         if ( table(30)>0d0 )  inpsol(17) = nint(table(30))-1  !proj_keep
         if ( table(44)>0d0 )  inpsol(17) = nint(table(44))-1  !proj_keep
         if ( table(29)>0d0 )  inpsol(18) = nint(table(29))-1  !proj_method
         inpsol(19) =  max(nint(table(32)),nint(table(46))) !proj_plot_vectors

         if ( table(43)>0d0 ) then

!        --- table(43)>0

            projmethod = nint(table(43))

            if ( projmethod==1 ) then

!           --- projmethod = 1, i.e. none

               inpsol(18) = 0

            else ! ( projmethod>1 )

!           --- projmethod>1, i.e. projection method

               inpsol(18) = 1    ! projection
               inpsol(36) = projmethod-2 ! deflation method

            end if  ! ( projmethod==1 )

         end if  ! ( table(43)>0d0 )

!        --- Deflation method

         if ( table(40)>0d0 ) then

!        --- deflation method found

            inpsol(36) = nint(table(40))-1

         end if
         if ( inpsol(18)==1 ) then

!        --- Projection is used

            if ( table(47)>0d0 ) then

!           --- table(47)>0, i.e. projection vectors

               projvect = nint(table(47))
               if ( projvect==2 ) projvect = 3

            else if ( table(37)>0d0 ) then

!           --- projection type

               projvect = nint(table(37))

            else if ( table(29)>0d0 ) then

!           --- table(29)>0, i.e. proj_method given

               projvect = nint(table(29))-1

            else  ! ( table(47)==0d0 )

!           --- table(47)=0, i.e. projection vectors not given: default

               projvect = 3

            end if  ! ( table(47)>0d0 )
            inpsol(20) = projvect

         end if

      else if ( icgs==5 ) then

!     --- icgs = 5, overrelaxation

         rinsol(3) = table(15)
         rinsol(4) = table(16)
         rinsol(5) = table(17)
         rinsol(6) = table(20)
         rinsol(7) = table(21)
         rinsol(8) = table(22)
         inpsol(20) = nint(table(23))
         i = nint(table(18))
         if ( i==0 ) then
            inpsol(15) = -3
         else if ( i>=-2 ) then
            inpsol(15) = i
         else
            inpsol(15) = 0
         end if
         i = nint(table(19))
         if ( i==0 ) then
            inpsol(16) = -3
         else if ( i>=-2 ) then
            inpsol(16) = i
         else
            inpsol(16) = 0
         end if

         i = nint(table(21))
         if ( i==0 ) then

!        --- not minimum found

            lvalmin = .false.

         else

!        --- minimum found, test if value or func is found

            lvalmin = .true.
            if ( i>0 ) then
               inpsol(18) = -i
            else
               inpsol(18) = -i
               rinsol(7) = tabout(-i)
            end if

         end if

         i = nint(table(22))
         if ( i==0 ) then

!        --- not maximum found

            lvalmax = .false.

         else

!        --- maximum found, test if value or func is found

            lvalmax = .true.
            if ( i>0 ) then
               inpsol(19) = -i
            else
               inpsol(19) = -i
               rinsol(8) = tabout(-i)
            end if

         end if

         if ( lvalmin .and. lvalmax ) then
            inpsol(17) = 3
         else if ( lvalmin ) then
            inpsol(17) = 1
         else if ( lvalmax ) then
            inpsol(17) = 2
         else
            inpsol(17) = 0
         end if

      else if ( icgs==11 ) then

!     --- icgs = 11, al_gcr, read gamma and mu, store in rinsol

         rinsol(2) = 1d0  ! gamma
         rinsol(3) = 1d0  ! mu
         if ( table(50)>0d0 ) rinsol(2) = table(50)  ! gamma
         if ( table(51)>0d0 ) rinsol(3) = table(51)  ! mu

      else if ( icgs==6 .or. icgs==8 .or. icgs==12 .or. icgs==13 ) then

!     --- Simple found, store omega in rinsol(5)

         rinsol(5) = table(17)

      end if  ! ( icgs<=4 .or. icgs==10 )

      inpsol(3) = icgs  ! set method

      if ( table(53)>0d0 ) then

!     --- laplace_shift given

         iposut = nint(table(53))
         m = nint(tabmax(53))
         rinsol(8) = tabout(iposut)
         if ( m==2 ) then

!        --- m = 2, real and imaginary part

            rinsol(9) = tabout(iposut+1)

         else

!        --- m = 1, only real part

            rinsol(9) = 0d0

         end if  ! ( m==2 )

      end if

!     --- Options for petsc

      if ( table(34)<=0d0 ) table(34) = 1d-50
      if ( table(33)<=0d0 ) then
         if ( table(3)<=0d0 .and. table(4)<=0d0 ) then
            table(33) = 1d-3
         else
            table(33) = max(table(3),table(4))
         end if  ! ( table(3)<=0d0 .and. table(4)<=0d0 )
      end if  ! ( table(33)<=0d0 )
      rinsol(11) = table(34) ! abs_accuracy
      rinsol(12) = table(33) ! rel_accuracy
      rinsol(13) = table(56) ! div_accuray
      rinsol(14) = table(57) ! damping_factor
      rinsol(15) = table(58) ! emin
      rinsol(16) = table(59) ! emax
      rinsol(17) = table(60) ! drop_tolerance
      inpsol(41) = nint(table(61)) ! num_levels

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsol, 50, 'inpsol' )
         call prinrl ( rinsol, 20, 'rinsol' )
         write(irefwr,*) 'End read03iter'

      end if
      call erclos ( 'read03iter' )

      end
