      subroutine read05
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.21 date 26-05-2009 Read name of vector immediately
!        version  6.20 date 29-09-2008 New call to readcompute
!        version  6.19 date 12-03-2008 New way of defining scalars
!
!   copyright (c) 1988-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Help subroutine for the program SEPPOST.
!   This subroutine reads the input for one record of the
!   shape prin/plot vi =  ......
!         compute vj = ...
!         time = ...
!         time history
!         define
!         reset
!         fill
!         iteration = ...
!       The part until vi is read by READ05, the rest is read by the
!       under subroutine READ06.
!       Sub parts are read by READ07 and REAd08
!       The contents of the record are filled in the arrays iinputpst, rinputpst
!       and CONTLN according to the description given below.
!       Besides that it is possible that input is stored in common block CPLOT.
!       See subroutine READ06.
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmoduleread1
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivecnm, istrf, iplot, kcode
      double precision value
      logical vecnum

!     iplot          Parameter indicating the type of time-history to be
!                    computed. See iinputpst(20)
!     istrf          Vector number for streamfunction
!     ivecnm         Vector number read
!     kcode          Return code from subroutine READ07. Possible values:
!                    115:  Keyword 3D PLOT
!                    116:  Keyword 3D COLOURED PLOT
!                     40:  Keyword ADD_TO_COOR
!                     -2:  Keyword COMPUTE
!                    -13:  Keyword CLOSE PLOT
!                     -5:  Keyword DEFINE COLOUR TABLE = &
!                    -11:  Keyword DEFINE COLOUR TABLE FROM
!                     -4:  Keyword DEFINE PLOT PARAMETERS
!                      0:  Keyword END
!                     20:  Keyword READ_MESH
!                     21:  Keyword PRESENT_MESH
!                     30:  Keyword INTERPOLATE
!                     81:  Keyword FILL COEFFICIENTS
!                     -6:  Keyword NAME
!                    119:  Keyword PLOT TRACK
!                    120:  Keyword PLOT BOUNDARY
!                    113:  Keyword PLOT COLOURED CONTOUR
!                    112:  Keyword PLOT CONTOUR
!                    103:  Keyword PLOT CURVES
!                    120:  Keyword PLOT FUNCTION
!                    127:  Keyword PLOT IDENTIFICATION
!                    121:  Keyword PLOT INTERSECTION
!                    101:  Keyword PLOT MESH
!                    104:  Keyword PLOT COLOURED_MESH
!                    105:  Keyword PLOT WIRE_FRAME
!                    102:  Keyword PLOT POINTS
!                    126:  Keyword PLOT POLYGON
!                    125:  Keyword PLOT TEXT
!                    128:  Keyword PLOT BOX_TEXT
!                    110:  Keyword PLOT VECTOR
!                    121:  Keyword PLOT VELOCITY PROFILE
!                      2:  Keyword PRINT
!                      3:  Keyword PRINT BOUNDARY FUNCTION
!                      4:  Keyword WRITE_MESH
!                    -10:  Keyword TIME HISTORY
!     value          Value read after POSITION, VALUE = &
!     vecnum         Logical indicating if a vector number must be read (true)
!                    or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCARD         Mark part in a record by underlining with ***
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READ06         Read part options in record for SEPPOST
!     READ07         Interpret first part of the card input
!     READ08         Interpret part of the card input after TIME HISTORY
!     READCOMPUTE    reads the input after the keyword compute until one of
!                    the keywords indicating the type of compute statement is
!                    found
!     READPOSTSCAL   Read information following keyword V.. = &
!     READVECNUM     Read the vector number behind the letter V
! **********************************************************************
!
!                       I/O
!
!     The subroutine reads from the standard input file
!     The following input is recognized:
!
!         compute vj = ...
!         time = ...
!         time history
!         define
!         reset
!         fill
!         iteration = ...
!         read_mesh i, file = '..'
!         present_mesh = j
!         interpolate Vi, mesh_in = i, mesh_out = j
!         add_to_coor vi, factor = f
!
!     The compute statements that are recognized are:
!
!     compute Vi = stream function Vj ....
!     compute Vi = velocity profile Vj ....
!     compute Vi = intersection Vj ....
!     compute Vi = mach_number ....
!     compute Vi = total_enthalpy ....
!     compute Vi = stagnation_pressure ....
!     compute Vi = entropy
!     compute Vi = function_of
!     compute Vi = interpolation
!     compute Vi = normal_stress Vj, curve = Ck
!
!     For a further description see the Users Manual Section 5.2
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     660   Vector number read in input record does not correspond to a
!           predefined array
!     872   Not a letter V found after COMPUTE Vj = ...
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'read05' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read05'
         write(irefwr,1) 'jtimes', jtimes
         write(irefwr,1) 'lastitem/1, ityprd', lastitem, ityprd
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iinputpst(74) = 1

!     --- Initialize some constpstants and arrays

150   if ( ityprd==0 ) go to 1000
      vecnum = .true.

!     --- Read and analyse first word

      call read07 ( kcode )

      if ( debug ) then
         write(irefwr,1) 'lastitem/2, ityprd', lastitem, ityprd
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
      end if  ! ( debug )

      if ( ierror/=0 ) then
         if ( ityprd/=0 ) ityprd = 1
         ierror = 0
         nerror = 0
         go to 150
      end if
      if ( debug ) write(irefwr,1) 'kcode', kcode

      ivecnm = -1

      if ( kcode==0 ) then

!     --- kcode = 0, i.e. END

         iinputpst(1) = 0
         go to 1000

      else if ( kcode<0 ) then

!     --- kcode<0:  special situation

         if ( kcode==-2 ) then

!        --- COMPUTE found, interpret the part [Vj] = keyword

            call readcompute ( istrf, ivecnm )
            if ( debug ) write(irefwr,1) 'istrf', istrf
            if ( ierror/=0 ) then

!           --- Error found, read next record

               if ( ityprd/=0 ) ityprd = 1
               ierror = 0
               nerror = 0
               go to 150

            end if
            if ( iinputpst(1)/=203 ) vecnum = .false.

         else if ( kcode==-3 ) then

!        --- V found, interpret the part [Vj] = alpha Vj + beta

            call readpostscal
            go to 1000

         else if ( kcode==-10 ) then

!        --- kcode = -10, i.e.TIME HISTORY
!            Check next item

            call read08 ( iplot, value )
            if ( iplot==-1 ) go to 150
            if ( ityprd/=4 ) call asknxt
            call readvecnum ( ivecnm )
            if ( ierror/=0 ) then
               if ( ityprd/=0 ) ityprd = 1
               ierror = 0
               nerror = 0
               go to 150
            end if
            iinputpst(2) = ivecnm
            ivecnm = -1
            vecnum = .false.

         end if

      else if ( kcode>0 ) then

!     --- kcode>0, standard situation

         iinputpst(1) = kcode
         if ( kcode==106 ) then

!        --- kcode = 106, i.e. coloured contour plot
!            Set default number of levels

            iinputpst(6) = -21

         else if ( kcode<=113 .or. kcode==115 .or. &
                   kcode>=118 .and. kcode<=121 .or. &
                   kcode>=125 .and. kcode<=128 ) then

!        --- kcode <=113, 115, 118-121, 125-128
!            No vector number should be read

            vecnum = .false.

         else if ( kcode==2 .or. kcode==10 ) then

!        --- print v found

            vecnum = .false.

         end if

      end if
      if ( debug ) write(irefwr,*) 'vecnum', vecnum

!     --- The type of record has been detected
!         Now the rest of the input must be read ( Subroutine READ06 )

      textx = '?'
      texty = '?'
      if ( kcode==112 ) iinputpst(41) = 1
      if ( kcode/=4 .and. kcode/=20 ) then

!     --- kcode # 4, 20, read rest
!         in case of read (write) mesh no further information required

         call read06 ( ivecnm )
         if ( debug ) write(irefwr,1) 'iretur, ivecnm', iretur, ivecnm
         if ( iretur<=-1 ) then
            ierror = 0
            nerror = 0
            if ( ityprd/=0 ) ityprd = 1
         end if

      end if

      if ( ivecnm>1000 .and. ivecnm<=2000 ) then

!     --- Vector corresponding to special function stored in ifuncs

         iinputpst(2) = ivecnm
         textid(1) = namefuncs ( ivecnm )
         if ( iinputpst(1)==120 ) then
            iinputpst(1) = 130
         else
            call errint ( ivecnm, 1 )
            call errsub ( 660, 1, 0, 0 )
            call ercard ( kcolum, icolum )
         end if
         if ( textx(1:1)=='?' ) then

!        --- textx has not been filled, set default

            textx(1:1) = 't'
            iinputpst(37) = 1

         end if
         if ( texty(1:1)=='?' ) then

!        --- texty has not been filled, set default

            nchar = index ( namefuncs(ivecnm-1000), ' ' ) -1
            texty(1:nchar) = namefuncs(ivecnm-1000)(1:nchar)
            iinputpst(38) = nchar

         end if

      else if ( ivecnm>-1 ) then

!     --- Vector number has been found, store in iinputpst

         if ( irefar( ivecnm)==0 ) then

!        --- There is no array corresponding to ivecnm

            call errint ( ivecnm, 1 )
            call errsub ( 660, 1, 0, 0 )
            call ercard ( kcolum, icolum )
            ierror = 0
            nerror = 0
            if ( ityprd/=0 ) ityprd = 1
            goto 150

         end if

!        --- Store the vector number in iinputpst(2)

         iinputpst(2) = irefar(ivecnm)
         if ( iinputpst(39)==0 ) textid(1) = names ( irefar( ivecnm ) )

      else if ( vecnum ) then

!     --- vecnum = true, a vector number should be read but has not been found

         call errsub ( 872, 0, 0, 0 )
         goto 150

      end if
      if ( ivecnm<=1000 .or. ivecnm>2000 ) then

         if ( textx(1:1)=='?' ) then

!        --- textx has not been filled, set default

            textx(1:1) = 'x'
            iinputpst(37) = 1

         end if
         if ( texty(1:1)=='?' ) then

!        --- texty has not been filled, set default

            texty(1:1) = 'y'
            iinputpst(38) = 1

         end if

      end if
      if ( kcode==-10 ) rinputpst(25) = value

!     --- reset the ierror

1000  ierror = 0
      if ( ityprd==0 ) iinputpst(1) = 0

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'jtimes', jtimes
         write(irefwr,*) 'End read05'

      end if
      call erclos ( 'read05' )

      end
