      subroutine readcapacity ( iinput, isubr, iseqnr, niinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-10-1999
!
!   copyright (c) 1999-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  Read the input for the subroutine to compute the capacities
!  The input is stored in the arrays iinput and rinput
!  The input must be enclosed between CAPACITY and END.
! **********************************************************************
!
!                       KEYWORDS
!
!     capacity
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer niinp, iinput(*), iseqnr, isubr

!    iinput  o   The integer input read is stored in array IINPUT
!                This array must be filled as follows:
!                1: Largest entry that has been filled by the user
!                   Default: 8
!                2: iseqsl:  sequence number of input for linear
!                            solver
!                   Default: 1
!                3: iseqcf:  sequence number of input for coefficients
!                   Default: 1
!                4: iveccapacity: sequence number of the capacity
!                                 vector in array isolut
!                   Default: 3
!                5: ibegincv: curve number of first electrode
!                   Default: 1
!                6: iendcv : curve number of last electrode
!                   Default: 2
!                7: ivecsol: sequence number of the solution
!                            vector in array isolut
!                   Default: 1
!                8: ivecreact: sequence number of the reaction
!                              vector in array isolut
!                   Default: 2
!    iseqnr i/o  Sequence number with respect to the storage in array ISEPIN
!    isubr   i   Indicates by which subroutine readeigv is called
!                Possible values:
!                1:  ...      read record with CAPACITY
!                2:  EREIGV,  do not read record with CAPACITY
!    niinp  i/o  Length of integer input array
!                At input available length, at output computed length
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=7
      integer ncode1(nncode1),  itype1(nncode1), ncodmx1(nncode1)
      integer k
      double precision :: tabmin1(nncode1), tabmax1(nncode1)
      character (len=21) :: code1(nncode1)

!     code1      Table with keywords corresponding to options
!     itype1     itype array for code1  (See intx02)
!     k          Counting variable
!     ncode1     Table containing the number of significant characters in
!                the keyword table code1
!     ncodmx1    Table containing the number of maximum characters in
!                the keyword table code1
!     nncode1     Number of entries in table code1
!     tabmax1    Table with maximum values of numbers corresponding to code1
!     tabmin1    Table with minimum values of numbers corresponding to code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSEQNR   Read sequence number iseqnr from record
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
!     SEPSETMAINEND  Fill keyword END as first main keyword and set nmain to 1
! **********************************************************************
!
!                       I/O
!
!     readeigv requires input from the standard input file
!     The following input is recognized:
!     CAPACITY [, SEQUENCE_NUMBER = s] (mandatory):
!         start of input
!      optional records:
!          lin_solver = i          (Default 1)
!          curve_begin = j         (Default 1)
!          curve_end = j           (Default 2)
!          solution_vector = s     (Default 1)
!          capacity_vector = s     (Default 3)
!          reaction_vector = s     (Default 2)
!          seq_coef = m            (Default 1)
!     END   (mandatory):  end of input
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     329   Array to store the input too small
!     650   First record read by subroutine is not equal to CAPACITY
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initialize arrays and parameters with default values
!    If ( isubr=1 ) then
!       Read first record
!       Analyse the first word (CAPACITY)
!    Read rest of first record (sequence number)
!    Read the rest of the records by intx02 until END is found
!    set niinp and nrinp to final values
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'LIN_SOLVER', 'SEQ_COEF', 'CAPACITY_VECTOR', &
                  'CURVE_BEGIN', 'CURVE_END', 'SOLUTION_VECTOR', &
                  'REACTION_VECTOR' /
      data ncode1  / 3, 6, 5, 9, 9, 5, 5 /
      data ncodmx1 /10, 8,15,11, 9,15,15 /
      data itype1  / 0, 0, 0, 0, 0, 0, 0 /
      data tabmin1 / 1d0, 1d0, 1d0, 1d0, 2d0, 1d0, 1d0 /
      data tabmax1 / 1d2, 1d2, 1d2, 1d3, 1d3, 1d2, 1d2 /

!     code1    Table with keywords corresponding to options
!             Values:
!             1:  LIN_SOLVER
!             2:  SEQ_COEF
!             3:  CAPACITY_VECTOR
!             4:  CURVE_BEGIN
!             5:  CURVE_END
!             6:  SOLUTION_VECTOR
!             7:  REACTION_VECTOR
! ======================================================================
!
!     --- Initialize arrays and parameters

      call eropen ( 'readcapacity' )

      if ( niinp<nncode1+1 ) then

!     --- niinp too small

         call errint ( nncode1+1, 1 )
         call errint ( niinp, 2 )
         call errsub ( 329, 2, 0, 0 )
         go to 1000

      end if
      iinput(1) = nncode1+1
      iinput(2:niinp) = 1
      iinput(4) = 3
      iinput(6) = 2
      iinput(8) = 2

      if ( isubr==1 ) then

!     --- Call by xxxxx, read first record  (CAPACITY)

         ityprd = 1
         call asknxt
         if ( ityprd/=4 ) then
            call errchr ( 'CAPACITY', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         else if ( string(1:5)/='CAPAC' ) then
            call errchr ( 'CAPACITY', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         end if

      end if

      call sepreadseqnr ( iseqnr )    ! Read sequence number
      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )
      call sepcopytype ( nncode1, itype1, tabmin1, tabmax1 )
      call sepsetmainend

      table(1:nncode) = -1d20
      istop = 2
      iretur = 0

      call sepreadsetkeyw
      istop = 0
      if ( table(1)>0d0 ) iinput(2) = nint(table(1))
      if ( table(2)>0d0 ) iinput(3) = nint(table(2))
      if ( table(3)>0d0 ) iinput(4) = nint(table(3))
      if ( table(4)>0d0 ) iinput(5) = nint(table(4))
      if ( table(5)>0d0 ) iinput(6) = nint(table(5))
      if ( table(6)>0d0 ) iinput(7) = nint(table(6))
      if ( table(7)>0d0 ) iinput(8) = nint(table(7))

1000  call erclos ( 'readcapacity' )
      end subroutine readcapacity
