      subroutine readcont ( inpconct, isubr, iseqnr, nparm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 10-03-2008 Debug statements
!        version  1.2  date 10-04-1998 Number of significant letters increasd
!        version  1.1  date 14-12-1997 New definition of input
!
!   copyright (c) 1997-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input for the subroutine CONTACT
!     The input is stored in array inpconct
! **********************************************************************
!
!                       KEYWORDS
!
!     contact
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpconct(*), iseqnr, isubr, nparm

!     inpconct  o     In this array it is indicated how the contact surface
!                     must be computed
!                     Contents
!                     1: sequence_number of contact
!                     2: type of contact
!                        1: curves
!                        2: surfaces
!                     3: contact method
!                        1: contact is made if contact_distance<=0
!                     4: contact disable method
!                        1: contact is disabled if CONTACT_FORCE<=0
!                     5: numvec
!                     6: sequence number of contact surface or curve
!                     7: sequence number of vector defining contact distance
!                     8: sequence number of vector defining contact force
!                     9: - &
!     iseqnr   i/o    Sequence number with respect to the storage in array
!                     ISEPIN
!     isubr     i     Indicates by which subroutine READCONT is called
!                     Possible values:
!                     1:  CONTACT,  read record with CONTACT
!                     2:  ERCONTCT,  do not read record with CONTACT
!     nparm     i     Number of entries in inpconct
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=5, nncod2=4
      integer ncode1(nncode1), ncode2(nncod2), ncodmx1(nncode1)
      integer k
      character (len=22) :: code1(nncode1)
      character (len=13) :: code2(nncod2)

!     code1          Table with keywords corresponding to options
!     code2          Table with the main keyword as well as the keywords
!                    following TYPE_OUTPUT
!     k              Counting variable
!     ncode1         Table containing the number of significant characters in
!                    the keyword table code1
!     ncode2         See ncode, now for table code2
!     ncodmx1        Table containing the number of maximum characters in
!                    the keyword table code1
!     nncod2         Number of entries in table code2
!     nncode1        Number of entries in table code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPREADSEQNR   Read sequence number iseqnr from record
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
!     SEPSETMAINEND  Fill keyword END as first main keyword and set nmain to 1
! **********************************************************************
!
!                       I/O
!
!     readcont requires input from the standard input file
!     The following input is recognised:
!     CONTACT [SEQUENCE_NUMBER = k] (mandatory):
!        start of input
!      optional records:
!        CONTACT_SURFACE = S3                     (Default 1)
!        CONTACT_DISTANCE = V5                    (Default 1)
!        CONTACT_FORCE = V1                       (Default 2)
!        CONTACT_METHOD = NEG_DISTANCE            (Default NEG_DISTANCE)
!        CONTACT_DISABLE_METHOD = CONTACT_FORCE   (Default CONTACT_FORCE)
!     END   (mandatory):  end of input
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     650   First record read by subroutine is not equal to SOLVE
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initialize parameters with default values
!    If ( isubr=1 ) then
!       Read first record
!       Analyse the first word (contact)
!    Read rest of first record (sequence number )
!    Read the rest of the records by intx02 until END is found
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'CONTACT_SURFACE', 'CONTACT_DISTANCE', &
                  'CONTACT_DISABLE_METHOD', 'CONTACT_METHOD', &
                  'CONTACT_FORCE' /
      data ncode1 /11,12, 12, 11, 8 /
      data ncodmx1/15,16, 22, 14,15 /
      data code2 / 'S', 'V', 'NEG_DISTANCE', 'CONTACT_FORCE' /
      data ncode2 / 1, 1, 7, 8 /

!     code1   Table with keywords corresponding to options
!             Values:
!             1:  CONTACT_SURFACE
!             2:  CONTACT_DISTANCE
!             3:  CONTACT_DISABLE_METHOD
!             4:  CONTACT_METHOD
!             5:  CONTACT_FORCE
!     code2   Table with the main keyword as well as the keywords
!             following CONTACT_METHOD
!             Values:
!             1: S
!             2: V
!             3: NEG_DISTANCE
!             4: CONTACT_FORCE
! ======================================================================
!
      call eropen ( 'readcont' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readcont'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Initialize parameters with default values

      inpconct(1:nparm) = 0
      inpconct(2) = 2
      inpconct(3:4) = 1
      inpconct(6:7) = 1
      inpconct(8) = 2

      if ( isubr==1 ) then

!     --- Call by CONTACT, read first record  (CONTACT)

         ityprd = 1
         call asknxt
         if ( ityprd/=4 ) then
            call errchr ( 'CONTACT', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         else if ( string(1:6)/='CONTAC' ) then
            call errchr ( 'CONTACT', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         end if

      end if

      call sepreadseqnr ( inpconct(1) )   ! read sequence number
      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 ) ! copy code1 in code
      call sepcopycodesub ( code2, nncod2, ncode2 )
      call sepsetmainend                  ! Fill END as main keyword

      itype(1:nncode) = 18
      itype(3:4) = 5
      tabmin(1:nncode) = 1d0
      tabmax(1:nncode) = 0d0
      table(1:nncode) = -1d0
      table(1)  = -1d0
      table(2)  = -2d0
      table(5)  = -2d0
      tabmin(4) = 3d0
      tabmax(4) = tabmin(4)
      tabmin(3) = tabmax(4)+1d0
      tabmax(3) = tabmin(3)
      tabmax(1) = nsurfs
      tabmax(2) = 1d2
      tabmax(5) = 1d2
      istop = 2
      iretur = 0
      call sepreadsetkeyw
      istop = 0
      if ( nint(table(1))>0d0 ) then

!     --- CONTACT_SURFACE = S.

         iposut = nint(table(1))
         inpconct(6) = nint(tabout(iposut))

      end if
      if ( nint(table(2))>0d0 ) then

!     --- CONTACT_DISTANCE = V.

         iposut = nint(table(2))
         inpconct(7) = nint(tabout(iposut))

      end if
      if ( nint(table(4))>0d0 ) inpconct(3) = nint(table(3))
      if ( nint(table(3))>0d0 ) inpconct(4) = nint(table(4))
      if ( nint(table(5))>0d0 ) then

!     --- CONTACT_FORCE = V.

         iposut = nint(table(5))
         inpconct(8) = nint(tabout(iposut))

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpconct, nparm, 'inpconct' )
         write(irefwr,*) 'End readcont'

      end if  ! ( debug )
      call erclos ( 'readcont' )

      end subroutine readcont
