      subroutine readnleq ( iinsol, rinsol, istart, iposrl, numcop, ihelp, &
                            readkw, niinp, numchn, openfound )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-01-2015
!        version  1.0  date 09-01-2013
!
!   copyright (c) 2013-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read input for subroutine readnl with respect to the part equations
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     non_linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iposrl, iinsol(*), istart, &
                 numcop, ihelp(*), niinp, numchn
      double precision :: rinsol(*)
      logical readkw, openfound

!     ihelp          i    integer work array, see readnl
!     iinsol        i/o   Contains information concerning the non-linear solver
!                         See readnl
!     iposrl        i/o   Position in array rinsol
!     istart        i/o   Starting address in array iinsol
!     niinp         i/o   Length of integer input array
!                         At input available length, at output computed length
!     numchn        i/o   Number of calls of change coefficients
!     numcop        i/o   number of coupled equations
!     openfound     i/o   Indicates if an openings bracket is found before
!                         reading the number or expression
!     readkw         i    Indicates if the next keyword must be read
!                         (true) or not
!     rinsol        i/o   Contains information concerning the non-linear solver
!                         See readnl
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ieq, iseq, iter, numprs
      logical ::  newequation, finished, sbloop

!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     i              Counting variable
!     ieq            Equation number
!     iseq           Sequence number
!     iter           Iteration sequence number
!     newequation    If true a new equation is considered
!     numprs         Number of calls of prescribe essential boundary conditions
!     sbloop         Indicates if the subloop is finished (false) or
!                    not (true)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READNLLOC      Read local options for non-linear equations
!     READREAL       Read a real or real expression from the input file
!     READREALSCAL   Read integer, real or scalar
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     329   Array to store the input too small
!     574   Not an integer number found after the keyword
!    1868   Text not recognized
!    1896   IEQ out of range
!    1897   Not iteration and sequence number found in one record
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!

!     code   Table with the keywords following change_coefficients
!             Values:
!             1:  at_iteration
!             2:  sequence_number
! ======================================================================
!
      call eropen ( 'readnleq' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readnleq'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      newequation = .true.
      icode = 3

      if ( newequation ) then

!     --- New equation found

         newequation = .false.

         call readreal ( openfound, .false. )
         if ( debug ) write(irefwr,*) 'openfound', openfound
         if ( ityprd/=2 ) then
            call errchr ( codemain(icode), 1 )
            call errsub ( 574, 0, 0, 1 )
            call errcrd
         else
            ieq = nint(realrd)
            if ( ieq<1 .or. ieq>numcop ) then
               call errint ( ieq, 1 )
               call errint ( numcop, 2 )
               call errsub ( 1896, 2, 0, 0 )
            end if
            ihelp(ieq) = istart
            do i = 0, 24
               iinsol(istart+i) = 0
            end do
            iinsol(istart) = -1
            iinsol(istart+1) = 1
            iinsol(istart+2) = 2
            iinsol(istart+3) = -1
            iinsol(istart+12) = -1
         end if
         finished = .false.
         readkw = .true.
         numchn = 0
         numprs = 0

!        --- while loop finished

         do while ( .not. finished )

!        --- Read next keyword

            if ( readkw ) then
               ityprd = 1
               call asknxt
            end if
            readkw = .true.

            if ( debug ) write(irefwr,1) 'ityprd', ityprd
            if ( ityprd/=4 ) then

!           --- not a keyword found

               do i = 1, nncodemain
                  call errchr ( codemain(i)(1:ncodmxmain(i)), i )
               end do
               call errsub ( 1868, 0, 0, nncodemain )
               call errcrd

            else

!           --- Check if it is a main keyword

               ierpri = 1
               icode = sepmainkeyw()

               if ( debug ) write(irefwr,1) 'icode', icode

               select case ( icode )

               case(1:4)

!              --- main keyword found

                  readkw = .false.
                  if ( icode/=3 ) finished = .true.
                  iinsol(istart+6) = numchn
                  iinsol(istart+8) = numprs
                  istart = istart+24+2*numchn+2*numprs
                  if ( istart-1>niinp ) then

!                 --- niinp too small

                     call errint ( istart, 1 )
                     call errint ( niinp, 2 )
                     call errsub ( 329, 2, 0, 0 )
                     go to 1000

                  end if
                  if ( icode==3 ) then
                     newequation = .true.
                     finished = .true.
                  end if  ! ( icode==3 )

               case(5)

!              --- keyword = LOCAL

                  call readnlloc ( iinsol, rinsol, istart, iposrl )

               case(6)

!              --- keyword = FILL_COEFFICIENTS

                  call readrealscal   ! Read integer or scalar
                  if ( ityprd==102 ) ityprd = 2  ! vector found

                  if ( ityprd==2 .or. ityprd==11 ) then

!                 --- ityprd = 2, 11, integer or variable found

                     iinsol(istart+4) = nint(realrd)

                  else

!                 --- Wrong type found

                     call errchr ( codemain(icode), 1 )
                     call errsub ( 574, 0, 0, 1 )
                     call errcrd
                  end if  ! ( ityprd==2 )

               case(7)

!              --- keyword = CHANGE_COEFFICIENTS

                  sbloop = .true.

!                 --- while loop subloop

                  do while ( sbloop )

!                 --- read next record and analyze next item

                     ityprd = 1
                     nncode = 2
                     code(1) = 'AT_ITERATION'
                     code(2) = 'SEQUENCE_NUMBER'
                     ncode(1:2) = 2
                     itype(1:2) = 0
                     ncodmx(1) = 12
                     ncodmx(2) = 15
                     tabmin(1:2) = 1d0
                     tabmax(1) = 1d4
                     tabmax(2) = 25d0
                     table(1:2) = 0d0
                     iretur = 0
                     nmain = nncodemain
                     call sepreadsetkeyw

                     if ( iretur/=0 ) then

!                    --- keyword found

                        sbloop = .false.
                        readkw = .false.

                     else

!                    --- new values for change coeff

                        numchn = numchn + 1

                        iter = nint(table(1))
                        iseq = nint(table(2))
                        if ( iter<=0 .or. iseq<=0 ) then
                           call errint ( iter, 1 )
                           call errint ( iseq, 2 )
                           call errsub ( 1897, 2, 0, 0 )
                        end if
                        iinsol(istart+24+2*numchn-1) = iter
                        iinsol(istart+24+2*numchn  ) = iseq
                     end if

                  end do  !  while ( sbloop )

               end select  ! case ( icode )

            end if  ! ( ityprd/=4 )

         end do  ! while ( .not. finished )

      end if  ! ( newequation )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readnleq'

      end if  ! ( debug )
      call erclos ( 'readnleq' )

      end subroutine readnleq
