      subroutine readpresscor ( iinput, rinput, isubr, iseqnr, niinp, &
                                nrinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 07-03-2012 New call to readseqvector
!        version  1.1  date 28-12-2006 Change default for coefficients
!        version  1.0  date 15-06-2005
!
!   copyright (c) 2005-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input for pressure correction
!     The input is stored in array iinput
!     The input must be enclosed between PRESSURE_CORRECTION and END.
! **********************************************************************
!
!                       KEYWORDS
!
!     pressure_correction
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer niinp, iinput(*), iseqnr, isubr, nrinp
      double precision rinput(*)

!     iinput         o    The integer input read is stored in array IINPUT
!                         This array must be filled as follows:
!                          1: iseqveloc, sequence number of velocity
!                             Default value: 1 or if available %velocity
!                          2: iseqpress, sequence number of pressure
!                             Default value: 2 or if available %pressure
!                          3: ireadcf, sequence number of coefficients for
!                                      velocity
!                             Default value: 1
!                          4: ireadsl, sequence number of solution method for
!                                      velocity
!                             Default value: 1
!                          5: ireadtim, sequence number of time integration
!                             Default value: 1
!                          6: ireadcfpres, sequence number of coefficients for
!                                         pressure
!                             Default value: -1
!                          7: ireadslpres, sequence number of solution method
!                                          for pressure
!                             Default value: ireadsl
!                          8: itypeconv, defines how the convective terms must
!                             be treated in case of Navier-Stokes
!                             0: standard, i.e. defined by the system matrix
!                             1: explicit (and operater-time-splitting) using
!                                substeps, extra information is stored in 9-11
!                                iinput(9) and extra n
!                             Default value: 0
!                          9: nsubsteps, number of substeps, see iinput(8)
!                             Default value: 1
!                         10: ireadcfconv, sequence number of coefficients for
!                                         convection
!                             Default value: 3
!                         11: ireadslconv, sequence number of linear solver for
!                                         convection
!                             Default value: ireadsl
!                         12: iseqpressbc, sequence number of pressure boundary
!                             conditions (if 0, all essential pressure boundary
!                             conditions are 0)
!                             Default value: 0
!     iseqnr        i/o   Sequence number with respect to the storage in
!                         array ISEPIN
!     isubr          i    Indicates by which subroutine readpresscor is called
!                         Possible values:
!                         1:  ...      read record with
!                         2:  ERPRESSCOR,  do not read record with
!                                          PRESSURE_CORRECTION
!     niinp          i    Length of integer input array
!     nrinp          i    Length of real input array
!     rinput         o    Real input array
!                         At this moment not actually used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=12
      integer ncode2(nncode2), itype2(nncode2), ncodmx2(nncode2)
      double precision :: tabmn2(nncode2), tabmx2(nncode2)
      character (len=22) code2(nncode2)

!     code2          Table with keywords
!     itype2         Table containing information about the numbers behind
!                    the keywords in the table code2
!     ncode2         Table containing the number of significant characters
!                    in the keyword table code2
!     ncodmx2        Table containing the maximal number of  characters in
!                    the keyword table code2
!     nncode2        Number of entries in code2
!     tabmn2         Contains minimum values allowed in table
!     tabmx2         Contains maximum values allowed in table
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READSEQVECTOR  Put the value of the sequence number read in iinput
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSEQNR   Read sequence number iseqnr from record
!     SEPREADSETKEY1 Interpret a record containing keywords followed by data
!                    Set END as sole main keyword
! **********************************************************************
!
!                       I/O
!
!     readpresscor requires input from the standard input file
!     The following input is recognized:
!     PRESSURE_CORRECTION [ SEQUENCE_NUMBER = s] (mandatory):
!         start of input
!      optional records:
!           SEQ_VELOCITY (Default 1 or %velocity)
!           SEQ_PRESSURE (Default 2 or %pressure)
!           SEQ_VEL_COEFFICIENTS (Default 1)
!           SEQ_PRESS_COEFFICIENTS (Default 3)
!           SEQ_CONV_COEFFICIENTS (Default 2)
!           SEQ_TIME_INTEGRATION (Default 1)
!           SEQ_VEL_SOLVER (Default 1)
!           SEQ_PRESS_SOLVER (Default SEQ_VEL_SOLVER)
!           SEQ_CONV_SOLVER (Default SEQ_VEL_SOLVER)
!           SEQ_PRESS_BOUNCOND (Default 0)
!           NUMBER_OF_SUBSTEPS
!           CONVECTION_TREATMENT (standard/substeps)
!     END   (mandatory):  end of input
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     650   First record read by subroutine is not equal to PRESSURE_CORRECTION
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initialize arrays and parameters with default values
!    If ( isubr=1 ) then
!       Read first record
!       Analyse the first word (PRESSURE_CORRECTION)
!    Read rest of first record (sequence number)
!    Read the rest of the records by intx02 until END is found
!    set niinp and nrinp to final values
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2   / 'SEQ_VELOCITY', 'SEQ_PRESSURE', &
                    'SEQ_VEL_COEFFICIENTS', 'SEQ_PRESS_COEFFICIENTS', &
                    'SEQ_TIME_INTEGRATION', 'SEQ_VEL_SOLVER', &
                    'SEQ_PRESS_SOLVER', 'CONVECTION_TREATMENT', &
                    'NUMBER_OF_SUBSTEPS', 'SEQ_CONV_COEFFICIENTS', &
                    'SEQ_CONV_SOLVER', 'SEQ_PRESS_BOUNCOND' /
      data ncode2  / 10,10,11,11, 8,  11,11, 6,13,13, &
                    13,13 /
      data ncodmx2 / 12,12,20,22,20,  14,16,20,18,21, &
                    15,18 /
      data itype2  /  0, 0, 0, 0, 0,   0, 0, 5, 0, 0, &
                     0, 0 /
      data tabmn2 / 1d0, 1d0, 1d0, 1d0, 1d0,   1d0, 1d0, 1d0, 1d0, 1d0, &
                    1d0, 1d0 /
      data tabmx2 / 1d2, 1d2, 1d1, 1d1, 1d1,   1d1, 1d1, 2d0, 1d5, 1d1, &
                    1d1, 1d1 /

!     code2    Table with keywords corresponding to options
!             Values:
!              1:  SEQ_VELOCITY
!              2:  SEQ_PRESSURE
!              3:  SEQ_VEL_COEFFICIENTS
!              4:  SEQ_PRESS_COEFFICIENTS
!              5:  SEQ_TIME_INTEGRATION
!              6:  SEQ_VEL_SOLVER
!              7:  SEQ_PRESS_SOLVER
!              8:  CONVECTION_TREATMENT
!              9:  NUMBER_OF_SUBSTEPS
!             10:  SEQ_CONV_COEFFICIENTS
!             11:  SEQ_CONV_SOLVER
!             12:  SEQ_PRESS_BOUNCOND
! ======================================================================
!
!     --- Initialize arrays and parameters

      call eropen ( 'readpresscor' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readpresscor'

      end if
      if ( ierror/=0 ) go to 1000

      call inichkarlen ( nncode2, niinp, 'iinput' )
      call inichkarlen ( 1, nrinp, 'rinput' )
      if ( ierror/=0 ) go to 1000

!     --- fill iinput with default values

      rinput(1) = 0d0

      if ( isubr==1 ) then

!     --- Call by readpresscorxx, read first record  (PRESSURE_CORRECTION)

         ityprd = 1
         call asknxt
         if ( ityprd/=4 ) then
            call errchr ( 'PRESSURE_CORRECTION', 1 )
            call errsub ( 650, 0, 0, 1 )
            call errcrd
            go to 1000
         end if

      end if

!     --- Read sequence number

      call sepreadseqnr ( iseqnr )

!     --- Read data records

      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )
      call sepcopytype ( nncode2, itype2, tabmn2, tabmx2 )

      table(1:nncode) = -1d0
      table(3) = 1d0  ! SEQ_VEL_COEFFICIENTS
      table(4) = -1d0 ! SEQ_PRESS_COEFFICIENTS
      table(5) = 1d0  ! SEQ_TIME_INTEGRATION
      table(6) = 1d0  ! SEQ_VEL_SOLVER
      table(8) = 1d0  ! convection
      table(9) = 1d0  ! substeps
      table(10) = 2d0 ! seq_conv_coefficients
      table(12) = 0d0 ! seq_press_bouncond

      codesub(1) = 'STANDARD'
      codesub(2) = 'SUBSTEPS'
      ncodesub(1) = 4
      ncodesub(2) = 5

      call sepreadsetkey1     ! interpret keywords
      if ( debug ) call prinrl ( table, nncode, 'table' )

      call readseqvector ( table(1), 'velocity', iinput(1), 1 )
      call readseqvector ( table(2), 'pressure', iinput(2), 2 )
      iinput(3) = nint(table(3))  ! readcf for velocity
      iinput(4) = nint(table(6))  ! readsl for velocity
      iinput(5) = nint(table(5))  ! seq time integration
      iinput(6) = nint(table(4))  ! seq pressure coefficients
      if ( table(7)>0 ) then
         iinput(7) = nint(table(7))  ! seq pressure solver
      else
         iinput(7) = iinput(4)  ! seq pressure solver
      end if  ! ( table(7)>0 )
      iinput(8) = nint(table(8))-1  ! convection
      iinput(9) = nint(table(9))    ! number of substeps
      iinput(10) = nint(table(10))  ! seq_conv_coefficients
      if ( table(11)>0 ) then
         iinput(11) = nint(table(11))  ! seq convection solver
      else
         iinput(11) = iinput(4)  ! seq convection solver
      end if  ! ( table(11)>0 )
      iinput(12) = nint(table(12))  ! seq_press_bouncond

1000  call erclos ( 'readpresscor' )
      if ( debug ) then

!     --- Debug information

         call prinin ( iinput, 12, 'iinput' )
         write(irefwr,*) 'End readpresscor'

      end if

      end
