      subroutine readscbouncond ( istart, inpsep, nprob, rinsep, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 24-10-2015 Remove ibuffr
!        version  2.6  date 07-01-2015 Extension with existing vector
!        version  2.5  date 26-06-2014 Allow vector = scalar
!
!   copyright (c) 2010-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input following the following standard keywords with respect
!     to subroutine readsc
!             1:  PRESCRIBE_BOUNDARY_CONDITIONS
!     The input is stored in array inpsep and the real array rinsep
!     If this subroutine is adapted also adapt ini016
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     output
!     boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduleplot
      use sepmodulepost
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(0:*), nprob, istrin
      double precision rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         In the standard case this part contains:
!                         1  ISEQNR   Sequence number corresponding to input
!                                     If iseqnr = -10, position 5 ... are used
!                                     See subroutine readscboun1
!                         2  IPROB    Problem sequence number
!                                     If iprob = -1, the next one will be used
!                         3  IVECT    Vector sequence number (first vector)
!                                     If ivector = -1, the same value as iprob
!                                     will be used
!     istart        i/o   Next free position in inpsep
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     nprob          i    Number of various problems
!     rinsep        i/o   Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: istartbef, istrinbef, i, kcode
      logical ::  outfnd, skipread

!     i              Counting variable
!     istartbef      Value of istart at input
!     istrinbef      Value of istrin at input
!     kcode          Contains value of icode at input
!     outfnd         dummy
!     skipread       If true reading must be skipped
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININLARGE    Print 1d integer vector (large numbers)
!     PRINRL         Print 1d real vector
!     READADDVEC     Reread name of new vector and add to list of vectors
!     READSCBOUN1    Read input following prescribe_boundary_conditions vector
!                    No sequence number found
!     READSCSTANDARD Read keywords after a number of standard keywords
! **********************************************************************
!
!                       I/O
!
!       PRESCRIBE_BOUNDARY_CONDITIONS, options (all in one line)
!           where options may have the following contents
!               SEQUENCE_NUMBER = s  (Default the next one)
!               VECTOR = s           (Default the same as problem)
!               PROBLEM = s          (Default 1)
!               VALUE = v            (No default)
!               FUNC = f             (No default)
!               QUADRATIC, MAX = a, MEAN_VALUE = v, BOUNDARY_LAYER_WIDTH = b, &
!                 DIFFERENCE = delta_T
!               HALF_QUADRATIC, MAX = a
!               POINTS = (Pi, Pj, Pk )
!               CURVES = (Ci, Cj, Ck )
!               SURFACES = (Si, Sj, Sk )
!               ZERO_LEVELSET i
!               OLD_VECTOR = m
!               SEQ_VECTORS = V1, V2, ...
!      In case of CREATE_VECTOR, extra
!               OUTPUT_VECTOR = ivec
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readscbouncond' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscbouncond'
         write(irefwr,1) 'istart, istrin, numvec', istart, istrin, numvec
         call prinin ( typeitems, nitems, 'typeitems' )
         write(irefwr,1) 'lastitem, istart', lastitem, istart
         istrinbef = istrin
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istartbef = istart  ! save value of istart
      kcode = icode

!     --- First we read the standard keywords and check if there are
!         no other keywords

      errorcode = 1   ! return when an unknown keyword is found
      call readscstandard ( istart, inpsep(0), outfnd, nprob )
      if ( debug ) then
         write(irefwr,1) 'errorcode, lastitem, numvec', &
                          errorcode, lastitem, numvec
         call prinin ( inpsep(0), 4, 'inpsep' )
         call prinin ( typeitems, nitems, 'typeitems' )
      end if  ! ( debug )

      if ( nitems>1 .and. inpsep(3)<=0 .and. typeitems(2)==3 ) then

!     --- item 3 must be the name of the vector
!         Read name and add to set of vectors

         if ( debug ) write(irefwr,*) 'before readaddvec'
         call readaddvec
         if ( ierror/=0 ) go to 1000

         inpsep(3) = numvec

         if ( debug ) &
            write(irefwr,1) 'numvec, lastitem', numvec, lastitem

         skipread = .false.
         if ( nitems>lastitem ) then

!        --- there are more items to be read

            if ( typeitems(lastitem+1)==13 ) then

!           --- next item is equals sign, do not reread input

               skipread = .true.
               errorcode = -1
               lastitem = lastitem+1       ! reset lastitem

            end if  ! ( typeitems(lastitem+1)==13 )

         end if  ! ( nitems>lastitem )

         if ( debug ) write(irefwr,*) 'skipread, numvec ', skipread, numvec

         if ( .not. skipread ) then

            lastitem = 1        ! reset lastitem
            istart = istartbef  ! reset istart

!           --- Reinterprete input

            errorcode = 1   ! return when an unknown keyword is found
            icode = kcode
            call readscstandard ( istart, inpsep(0), outfnd, nprob )
            if ( debug ) then
               write(irefwr,1) 'errorcode, lastitem, numvec', &
                                errorcode, lastitem, numvec
               call prinin ( inpsep(0), 4, 'inpsep' )
               call prinin ( typeitems, nitems, 'typeitems' )
            end if  ! ( debug )

         end if  ! ( .not. skipread )

      else if ( errorcode<0 .and. lastitem>=4 .and. &
                (typeitems(2)==3 .and. typeitems(3)==13 .or. &
                typeitems(2)==3 .and. typeitems(3)==1 .and. &
                typeitems(4)==13) ) then

!     --- errorcode<0, reset lastitem

         do i = 3, 4

            if ( typeitems(i)==13 ) then

!           --- Equals sign found, set lastitem

               lastitem = i
               exit

            end if  ! ( typeitems(i)==13 )

         end do  ! i = 3, 4
         if ( typeitems(lastitem)==1 ) lastitem = lastitem+1
            ! vector contains number
         if ( debug ) write(irefwr,1) 'lastitem', lastitem

      end if  ! ( inpsep(3)<0 .and. ivec<0 .and. typeitems(2)==3 )

      if ( errorcode>0 ) then

!     --- errorcode>0, might be due to known vector

         if ( nitems>3 .and. inpsep(3)>0 .and. inpsep(2)<0 .and. &
              typeitems(2)==3 .and. typeitems(3)==13 ) then

!        --- Possibly situation prescribe_boundary_conditions vector = found
!            But no sequence number
!            Check if item 2 is sequence_number or problem

            lastitem = 1
            ityprd = 2
            call asknxt
            if ( string(1:8)/='SEQUENCE' .and. &
                 string(1:7)/='PROBLEM' .and. &
                 string(1:nchar)/='VECTOR' ) then

!           --- Not sequence nor problem, reset

               errorcode = -1
               lastitem = 3

            end if  ! ( string(1:8)/='SEQUENCE' .and. string(1:7)/='PROBLEM' )

         end if  ! ( nitems>3 .and. inpsep(3)>0 .and. inpsep(2)<0 .and. ... )

      end if  ! ( errorcode>0 )

      if ( errorcode==1 .and. inpsep(2)>1 ) then

!     --- Known vector found and problem number > 1
!         Reset errorcode and lastitem in order to call readscboun1

         errorcode = -1
         lastitem = 3

      end if  ! ( errorcode==1 .and. inpsep(2)>1 )

      if ( errorcode<0 ) then

!     --- errorcode = -1, hence other keyword found
!         read rest of keywords with standard routine

         if ( debug ) &
            write(irefwr,1) 'lastitem, istart, icode, kcode', &
                             lastitem, istart, icode, kcode

         icode = kcode
         call readscboun1 ( inpsep(1), length, rinsep, istrin, nprob )
         kcode = icode
         if ( ierror/=0 ) go to 1000
         istart = istart+length  ! raise istart

      end if  ! ( errorcode<0 )
      if ( mod(inpsep(3),5000)>0 ) ivectors(mod(inpsep(3),5000)) = 1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istart, numvec', istart, numvec
         call prininlarge ( inpsep(0), istart-istartbef, 'inpsep' )
         if ( istrin>istrinbef ) &
           call prinrl ( rinsep(istrinbef), istrin-istrinbef, 'rinsep' )
         write(irefwr,*) 'End readscbouncond'

      end if  ! ( debug )
      call erclos ( 'readscbouncond' )

      end subroutine readscbouncond
