      subroutine readscmatrix ( inpsep, istart, nprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.5  date 28-03-2015 Correction inpsep(4)
!        version  2.4  date 13-01-2015 Extension with degfd
!        version  2.3  date 06-09-2014 Use mpistart
!
!   copyright (c) 2011-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the options following the keyword MATRIX_STRUCTURE
!     with respect to subroutine readsc
!     The input is stored in array inpsep
!     See subroutine read00 for the description of the parameters
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecpack
      use sepmoduleread1
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nprob
      integer, intent(inout) :: inpsep(0:*), istart

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         Contents
!                         Pos. 0    93 to indicate MATRIX_STRUCTURE
!                              1    sequence number
!                              2    jmethod parameter indicating the structure
!                                   of the large matrix
!                              3:   iprint defines the print level, See read00
!                              4:   invmat_tot
!                              5:   iprob
!                              6:   fillin_extra
!                              7:   iphys1
!                              8:   iphys2
!                              9:   Type of mesh
!                             10:   num_dec
!                             11:   istart_dec
!                             12:   skip_boundary_conditions
!                             13:   use_petsc
!                             14:   parallel_coupling
!                             15:   scheme
!                         istart_dec ... istart_dec+num_dec-1: sequence numbers
!                             of unknowns that are considered to be decoupled
!                         For a description of these parameters, see READ00
!     istart        i/o   Next free position in inpsep
!     nprob          i    Number of various problems
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=31
      character (len=28) :: code1(nncode1)
      integer :: ncode1(nncode1), method, iprob, ncodmx1(nncode1), &
                 lastentryused, numdec, jstart, i, j, icompl, &
                 isymincomp, ishiftlap, isymm

!     code1          Table with keywords
!     i              Counting variable
!     icompl         Multiplication factor for complex numbers
!                    Real: icompl=1,   Complex: icompl=2
!     iprob          Actual problem number
!     ishiftlap      If shifted laplace has been found 1, else 0
!     isymincomp     if incompressibility and gradient each others transpose, 0,
!                    else 1
!     isymm          if symmetrical then 0 else 1
!     j              Counting variable
!     jstart         starting position in inpsep
!     lastentryused  last entry used in inpsep
!     method         Type of method read
!     ncode1         Table containing the number of significant characters
!                    in the keyword table code
!     ncodmx1        Table containing the maximal number of  characters in
!                    the keyword table code1
!                    the extra texts
!     nncode1        Number of entries in code1
!     numdec         Number of unknowns that is considered to be decoupled
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRKEYWEXCLUS  Check if two keywords are mutually exclusive
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPPETSCSTART  Start petsc
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!     The following options are available (all in one line)
!
!       STORAGE_SCHEME = k
!          with k =  COMPACT
!                    PROFILE
!                    SIMPLE
!                    ROW_COMPACT
!                    VECTOR_COMPUTER
!                    AUGMENTED_LAGRANGE
!                    AMG_ROW_COMPACT
!                    NESTED_DISSECTION
!                    MINIMUM_DEGREE
!       SYMMETRIC, UNSYMMETRIC, REAL, COMPLEX
!       INCOMPRESSIBILITY_SYM, INCOMPRESSIBILITY_UNSYM, ASSEMBLE_PRECON
!       REACTION_FORCE, SHIFTED_LAPLACE, PETSC, MUMPS, POSITIVE_DEFINITE
!       NO_SEPRAN_MATRIX
!       DECOUPLED_DEGFD ( d1, d2, .. )
!       SEQUENCE_NUMBER = s
!       PROBLEM = p
!       PARALLEL_COUPLING = dd/nn/row
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'STORAGE_SCHEME', 'SYMMETRIC', 'COMPLEX', 'REAL', &
                  'UNSYMMETRIC', 'INCOMPRESSIBILITY_SYM', &
                  'INCOMPRESSIBILITY_UNSYM', 'ASSEMBLE_PRECON', &
                  'REACTION_FORCE', 'SHIFTED_LAPLACE', &
                  'ASYMMETRIC', &
                  'PETSC', 'MUMPS', 'POSITIVE_DEFINITE', &
                  'SEQUENCE_NUMBER', 'PRINT_LEVEL', 'PROBLEM', &
                  'EXTRA_FILLIN', 'NO_SEPRAN_MATRIX', &
                  'DECOUPLED_DEGFD', 'PARALLEL_COUPLING', &
                  'COMPACT', 'PROFILE', 'SIMPLE', 'ROW_COMPACT', &
                  'VECTOR_COMPUTER', 'AUGMENTED_LAGRANGE', &
                  'AMG_ROW_COMPACT', 'SCHUR_COMPLEMENT', &
                  'NESTED_DISSECTION', 'MINIMUM_DEGREE' /
      data ncode1  /  5, 4, 4, 4,  6,  20, 20,  6, 8, 5, &
                      5, 5, 5, 5,  3,   4,  4,  4, 6, 6, &
                      3, 4, 4, 4,  3,   4,  4,  3, 5, 4, &
                      4 /
      data ncodmx1 / 14, 9, 7, 4, 11,  26, 28, 14,15,15, &
                     10, 5, 5,17, 15,  11,  7, 13,16,15, &
                     17, 7, 7, 6, 11,  15, 18, 15,16,17, &
                     14 /

!     code1   Table with keywords
!             Values:
!              1: STORAGE_SCHEME
!              2: SYMMETRIC
!              3: COMPLEX
!              4: REAL
!              5: UNSYMMETRIC
!              6: INCOMPRESSIBILITY_SYM
!              7: INCOMPRESSIBILITY_UNSYM
!              8: ASSEMBLE_PRECON
!              9: REACTION_FORCE
!             10: SHIFTED_LAPLACE
!             11: ASYMMETRIC
!             12: PETSC
!             13: MUMPS
!             14: POSITIVE_DEFINITE
!             15: SEQUENCE_NUMBER
!             16: PRINT_LEVEL
!             17: PROBLEM
!             18: EXTRA_FILLIN
!             19: NO_SEPRAN_MATRIX
!             20: DECOUPLED_DEGFD
!             21: PARALLEL_COUPLING
!             22: COMPACT
!             23: PROFILE
!             24: SIMPLE
!             25: ROW_COMPACT
!             26: VECTOR_COMPUTER
!             27: AUGMENTED_LAGRANGE
!             28: AMG_ROW_COMPACT
!             29: SCHUR_COMPLEMENT
!             30: NESTED_DISSECTION
!             31: MINIMUM_DEGREE
! ======================================================================
!
      call eropen ( 'readscmatrix' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscmatrix'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )
      codesub(1) = 'NEUMANN_NEUMANN'
      codesub(2) = 'DIRICHLET_DIRICHLET'
      codesub(3) = 'ROW'
      ncodesub(1:3) = 1

      table(1:nncode) = 0d0
      tabmin(1:nncode) = 0d0
      tabmax(1:nncode) = 0d0
      itype(1:nncode) = 2

      itype(1) = -1    ! STORAGE_SCHEME
      itype(15) = 0    ! SEQUENCE_NUMBER
      tabmin(15) = 1d0
      tabmax(15) = 10d0
      itype(16) = 0    ! PRINT_LEVEL
      tabmax(16) = 12d0
      itype(17) = 0    ! PROBLEM
      tabmin(17) = 1d0
      tabmax(17) = nprob
      itype(18) = 0    ! EXTRA_FILLIN
      tabmin(18) = 1d0
      tabmax(18) = 7d0

      itype(20) = 11  ! DECOUPLED_DEGFD
      tabmin(20) = 5d0
      tabmax(20) = 10d0

      itype(21) = 5  ! PARALLEL_COUPLING
      tabmin(21) = 1d0
      tabmax(21) = 3d0

      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      if ( ierror/=0 ) go to 1000

      if ( debug ) call prinrl ( table, nncode, 'table' )

!     --- Fill default values

      lastentryused = 15 ! last entry for standard storage

      inpsep(0) = 93     ! MATRIX_STRUCTURE
      inpsep(1) = -1     ! seq number
      inpsep(2) = 2      ! jmethod
      inpsep(3:lastentryused) = 0   ! rest
      iprob = 0

      if ( table(11)>0d0 ) inpsep(1) = nint(table(11)) ! seq number

      isymm = 1-nint(table(2))
      icompl = nint(table(3))      ! COMPLEX
      isymincomp = nint(table(7))  ! INCOMPRESSIBILITY_UNSYM
      ishiftlap = nint(table(10))   ! SHIFTED_LAPLACE

      if ( table(22)>0d0 ) then  ! COMPACT
         inpsep(2) = 5+isymm+2*icompl+ishiftlap*19
      else if ( table(23)>0d0 ) then  ! PROFILE
         inpsep(2) = 1+isymm+2*icompl
         inpsep(15) = 1
      else if ( table(24)>0d0 .or. table(27)>0d0 .or. table(29)>0d0 ) then
         inpsep(2) = 13+isymm+2*isymincomp  ! 13 to 16 for simple
      else if ( table(25)>0d0 ) then  ! ROW_COMPACT
         inpsep(2) = 9+icompl
      else if ( table(26)>0d0 ) then  ! VECTOR_COMPUTER
         inpsep(2) = 11+icompl
      else if ( table(28)>0d0 ) then  ! AMG_ROW_COMPACT
         inpsep(2) = 51
      else if ( table(30)>0d0 ) then  ! NESTED_DISSECTION
         inpsep(2) = 1+isymm+2*icompl
         inpsep(15) = 2
      else if ( table(31)>0d0 ) then  ! MINIMUM_DEGREE
         inpsep(2) = 1+isymm+2*icompl
         inpsep(15) = 3
      else if ( nint(table(12))==1 ) then

!     --- petsc found

         if ( table(19)>0d0 ) then
            inpsep(2) = 56   ! jmethod (sepran matrix is not made)
         else
            inpsep(2) = 55   ! jmethod (sepran matrix is made)
         end if  ! ( table(19)>0d0 )

         if ( .not. mpistart ) then

!        --- mpi has not started, start petsc and make mpistart true

            call seppetscstart
            mpistart = .true.

         end if  ! ( .not mpistart )

      else if ( nint(table(13))==1 ) then

!     --- mumps found

         inpsep(13) = 2   ! use mumps
         if ( nint(table(2))==1 ) then

!        --- symmetric found

            if ( nint(table(14))==1 ) then
               inpsep(2) = 54   ! symmetric positive definite
            else
               inpsep(2) = 53   ! symmetric
            end if  ! ( nint(table(16))==1 )

         else

!        --- unsymmetric found

            inpsep(2) = 52   ! unsymmetric

         end if  ! ( nint(table(2))==1 )

      else  ! PROFILE
         inpsep(2) = 1+isymm+2*icompl
      end if  ! ( table(22)>0d0 )
      inpsep(3) = nint(table(16))    ! iprint
      inpsep(5) = nint(table(17))    ! problem
      inpsep(6) = nint(table(18))    ! extra_fillin

!     --- Check if the keywords petsc and mumps are found
!         These are mutually exclusive

      call errkeywexclus ( table, code, ncodmx, 12, 13 ) ! petsc/mumps
      call errkeywexclus ( table, code, ncodmx, 2, 5 ) ! (un)symmetric
      call errkeywexclus ( table, code, ncodmx, 2, 11 ) ! (a)symmetric
      call errkeywexclus ( table, code, ncodmx, 5, 11 ) ! (a)symmetric
      do i = 22, 29
         do j = i+1, 29
            call errkeywexclus ( table, code, ncodmx, i, j )
         end do  ! j = i+1, 29
      end do  ! i = 22, 29

      if ( table(20)>0d0 ) then

!     --- DECOUPLE_DEGFD found

         numdec = nint(tabmax(20))  ! number of items
         jstart = lastentryused+1   ! starting address of information
         iposut = nint(table(20))   ! starting address in tabout
         inpsep(10) = numdec
         inpsep(11) = jstart

         inpsep(jstart:jstart+numdec-1) = &
            nint(tabout(iposut:iposut+numdec-1))
         lastentryused = lastentryused+numdec

      end if  ! ( table(20)>0d0 )

      if ( table(9)>0d0 ) then

!     --- not storage scheme found, but reaction_force

         inpsep(4) = nint(table(8)) + 2*nint(table(9))

      end if  ! ( table(2)>0d0 )

      if ( table(21)>0d0 ) then

!     --- PARALLEL_COUPLING found

         inpsep(14) = nint(table(21))-1

      end if  ! ( table(21)>0d0 )

!     --- Raise istart

      istart = istart + lastentryused + 1

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsep(1), lastentryused, 'inpsep' )
         write(irefwr,*) 'End readscmatrix'

      end if  ! ( debug )
      call erclos ( 'readscmatrix' )

      end
