      subroutine readscnlnsol ( istart, inpsep, nprob, rinsep, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.8  date 14-05-2014 Change itype for reaction_force
!        version  1.7  date 23-01-2014 extra call to readscnewvec
!        version  1.6  date 29-06-2011 Extension with criterion
!
!   copyright (c) 2002-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read the input following the following the keyword
!   SOLVE_NONLINEAR_PROBLEM
!   with respect to subroutine readsc
!   The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: istart, inpsep(*), nprob, istrin
      double precision :: rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                          2  ISEQNR  Sequence number corresponding to input
!                                     If iseqnr = -1, the next one will be used
!                          3  IPROB   Problem sequence number
!                                     If iprob = -1, the next one will be used
!                          4  IVECTOR Vector sequence number (first vector)
!                                     If ivector = -1, the same value as iprob
!                                     will be used
!                          5: IVEC_REACT indicates if reaction forces must be
!                             computed (>0) or not (0)
!                             If >0, IVEC_REACT defines the sequence number
!                             of the vector in which the reaction forces
!                             must be stored
!                          6: MAXITER Maximum number of iterations
!                          7: PRINT_LEVEL
!                          8: IRETURN   Indicates if the subroutine must stop if
!                                       no convergence has been found (0) or
!                                       that control must be given back to the
!                                       main program (1)
!                          9: MINITER Minimum number of iterations
!                         10: ICRITER   Indicates termination criterion
!                         11: METHOD_NL Indicates type of iteration method to be
!                                        applied
!                                        Possible values:
!                                        0: Standard method A^k x^(k+1) = f^k
!                                        1: Newton method: dx = 0 at boundary
!                                                          A^k dx = f^k
!                                                          x^(k+1) x^k + dx
!                         12: Position of accuracy in array RINSEP
!                         13: ISEQ_COEF
!     istart        i/o   Next free position in inpsep
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     nprob          i    Number of various problems
!     rinsep        i/o   Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=13, nncode3=11
      integer ncode2(nncode2), ncodmx2(nncode2), &
              ivec, i, ncode3(nncode3)
      double precision :: maxtable
      character (len=20) code2(nncode2), code3(nncode3)

!     code2          Table with keywords
!     code3          Table with subkeywords
!     i              Counting variable
!     ivec           Sequence number of vector with respect to array ISOL
!     maxtable       Maximum value found in table
!     ncode2         Table containing the number of significant letters in
!                    code2
!     ncode3         Table containing the number of significant letters in
!                    code3
!     ncodmx2        Table containing the number of maximum characters in
!                    the keyword table code2
!     nncode2        Number of entries in code2
!     nncode3        Number of entries in code3
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     PRININ         print 1d integer array
!     READSCNEWVEC   Check if text following main keyword is not a keyword
!                    or a known vector
!                    Make a new vector if necessary
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       SOLVE_NONLINEAR_SYSTEM, options (all in one line)
!           where options may have the following contents
!               SEQUENCE_NUMBER = s  (Default the next one)
!               VECTOR = s           (Default the same as problem)
!               PROBLEM = s          (Default 1)
!               REACTION_FORCE = s   (Default 1)
!               MINITER = m          (Default 2)
!               MAXITER = m          (Default 20)
!               ACCURACY = eps       (Default 1d-3)
!               PRINT_LEVEL = p      (Default 0)
!               ITERATION_METHOD = i (Default standard)
!               AT_ERROR = e         (Default stop)
!               SEQ_COEF = i         (Default 0)
!               CRITERION = c        (Default abs)
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'SEQUENCE_NUMBER', 'PROBLEM', 'VECTOR', &
                  'REACTION_FORCE', 'V', 'MAXITER', 'PRINT_LEVEL', &
                  'AT_ERROR', 'MINITER', 'ITERATION_METHOD', &
                  'ACCURACY', 'SEQ_COEF', 'CRITERION' /
      data ncode2 /  6, 4, 6, 5, 1,    3, 5, 5, 3, 4, &
                    4, 6, 4 /
      data ncodmx2 /15, 7, 6,14, 1,    7,11, 9, 7,16, &
                    8, 8, 9 /
      data code3 / 'STANDARD', 'NEWTON', 'STOP', 'RETURN', 'RESUME', &
                   'ABSOLUTE', 'RELATIVE', 'RHS_ABSOLUTE', &
                   'RHS_RELATIVE', 'ENERGY_ABS', 'ENERGY_RELATIVE' /
      data ncode3 / 4, 3, 4, 5, 5,    3, 3, 7, 7, 10, &
                   10 /

!     code2    Table with sub keywords corresponding to keyword SOLVE_NONLIN
!             Values:
!             1:  SEQUENCE_NUMBER
!             2:  PROBLEM
!             3:  VECTOR
!             4:  REACTION_FORCE
!             5:  V
!             6:  MAXITER
!             7:  PRINT_LEVEL
!             8:  AT_ERROR
!             9:  MINITER
!            10:  ITERATION_METHOD
!            11:  ACCURACY
!            12:  SEQ_COEF
!            13:  CRITERION
!     code3   Table with sub keywords corresponding to keywords
!             Values:
!             1:  STANDARD
!             2:  NEWTON
!             3:  STOP
!             4:  RETURN
!             5:  RESUME
!             6:  absolute
!             7:  relative
!             8:  rhs_absolute
!             9:  rhs_relative
!            10:  energy_absolute
!            11:  energy_relative
! ======================================================================
!
      call eropen ( 'readscnlnsol' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscnlnsol'
         write(irefwr,1) 'istart', istart
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )
      call sepcopycodesub ( code3, nncode3, ncode3 )

      inpsep(istart) = icode

!     --- First we check if the second item is an not yet defined vector
!         If so the vector is created

      call readscnewvec

!     --- Next we interpret the complete input

      itype(1:nncode) = 0
      table(1:nncode) = -1d0
      tabmin(1:nncode) = 1d0
      tabmax(1:nncode) = 1d0

      tabmax(1) = 100d0
      tabmax(2) = nprob
      tabmax(3:5) = dble(maxvec)

      itype(4) = 38     ! reaction_force
      table(4) = 0d0

      tabmax(6) = 1d3                ! MAXITER

      tabmin(7) = 0d0                ! PRINT_LEVEL
      tabmax(7) = 3d0

      tabmax(9) = 1d2                ! MINITER

      itype(10) = 5
      tabmax(10) = 2d0

      itype(8)  = 5   !  at_error
      tabmin(8) = tabmax(10)+1d0
      tabmax(8) = tabmin(8)+3d0

      itype(11) = 1   ! accuracy
      tabmin(11) = 1d-12
      tabmax(11) = 1d9

      tabmax(12) = 25d0              ! SEQ_COEF

      itype(13)  = 5   !  criterion
      tabmin(13) = tabmax(8)+1d0
      tabmax(13) = tabmin(8)+5d0

      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      if ( ierror/=0 ) go to 1000

      if ( table(3)>0d0 .and. table(5)>0d0 ) &
         call errkeywexcl ( code2, ncodmx2, 3, 5 )

      table(3) = max(table(3),table(5))

      inpsep(istart+1:istart+4) = nint(table(1:4))

      inpsep(istart+4) = -inpsep(istart+4)

      do i = 3, 4
         ivec = abs(nint(table(i)))
         if ( ivec>0 ) then
            numvec = min ( max(numvec,ivec), maxvec )
            if ( ivec<=maxvec ) ivectors(ivec) = 1
         end if
      end do

      maxtable = maxval(table(6:12))

      if ( inpsep(istart+1)<0 .and. maxtable>=0d0 ) then

!     --- No sequence number found

         inpsep(istart+1) = -10  ! sequence number
         inpsep(istart+5:istart+8) = nint(table(6:9))  ! maxiter .. miniter
         inpsep(istart+9) = max(1,nint(table(13))-1)   ! icrit
         inpsep(istart+10) = max(0,nint(table(10))-1)  ! iteration_method

         inpsep(istart+11) = istrin    ! pos in rinsep
         rinsep(istrin) =  table(11)   ! accuracy
         inpsep(istart+12) = max(0,nint(table(12)))    ! seq_coef

!        --- Fill defaults

         if ( inpsep(istart+5)<0 ) inpsep(istart+5) = 20    ! MAXITER
         if ( inpsep(istart+6)<0 ) inpsep(istart+6) = 0     ! PRINT_LEVEL
         if ( inpsep(istart+7)<0 ) inpsep(istart+7) = 0     ! AT_ERROR
         if ( inpsep(istart+8)<0 ) inpsep(istart+8) = 2     ! MINITER
         if ( inpsep(istart+10)<0 ) inpsep(istart+10) = 1   ! ITERATION_METHOD
         if ( table(11)<0d0 ) rinsep(istrin) = 1d-3         ! ACCURACY

         istart = istart + 13
         istrin =  istrin+1           ! raise istrin

      else

!     --- Sequence number found

         istart = istart+5

      end if  ! ( inpsep(istart+1)<0 ) then

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istart, istrin', istart, istrin
         if ( istrin>1 ) write(irefwr,2) 'eps', rinsep(istrin-1)
         call prinin ( inpsep, istart, 'inpsep' )
         write(irefwr,*) 'End readscnlnsol'

      end if  ! ( debug )
      call erclos ( 'readscnlnsol' )

      end
