      subroutine readscplotmesh ( istart, inpsep, istrin, rinsep, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.4  date 14-11-2013 Shift of inpsep
!        version  2.3  date 12-04-2011 Extra options
!        version  2.2  date 03-02-2009 Correction tabmax
!
!   copyright (c) 2002-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input following the following the keyword
!     PLOT_MESH
!     with respect to subroutine readsc
!     The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     plot
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(0:*), istrin, ndim
      double precision :: rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                         1:  IREGION  defines if region is given
!                             If IREGION#0, 6 positions in rinsep are used
!                             The starting position is rinsep(IREGION)
!                         2:  IYFACT
!                             If IYFACT#0, 1 extra position in rinsep is used
!                             to define the y-scaling factor
!                             The starting position is rinsep(IYFACT)
!                         3:  IEYE_POINT
!                             If IEYE_POINT>0, 3 positions in rinsep are used
!                             The starting position is rinsep(IEYE_POINT)
!                         4:  ORIENTATION
!                             Defines type of orientation:
!                             1: x-y-z   2: z-x-y  3: y-z-x
!                         5:  ELEMENT_NUMBERS
!                             0:  do not plot element numbers
!                             1:  plot element numbers
!                         6:  NODE_NUMBERS
!                             0:  do not plot node numbers
!                             1:  plot node numbers
!                         7:  MARK_NODES
!                             0:  do not mark nodes
!                             1:  not mark nodes
!                         8:  PLOTBOUN
!                             0,1:   only the outer boundaries are plotted
!                             2:     only the inner boundaries are plotted
!                             3:     all boundaries are plotted
!                             4:     no boundaries are plotted
!     istart        i/o   Next free position in inpsep
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     ndim           i    Dimension of the space.
!     rinsep        i/o   Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode1=10
      integer ncode1(nncode1), ncodmx1(nncode1), i
      character (len=20) code1(nncode1)

!     code1         Table with keywords
!     i              Counting variable
!     ncode1         Table containing the number of significant letters in
!                    code1
!     ncodmx1        Table containing the number of maximum characters in
!                    the keyword table code1
!     nncode1        Number of entries in code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     READPLOTBOUN   Read information on how boundary curves must be plotted
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       PLOT_MESH, options
!           where options may have the following contents
!           region = (xmin,xmax,ymin,ymax,zmin,zmax)
!           yfact = y
!           eyepoint = (x_e, y_e, z_e)
!           orientation = 0/1/2
!           mark_nodes
!           node_numbers
!           element_numbers
!           inner_curves
!           outer_curves
!           no_curves
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'REGION', 'YFACT', 'EYEPOINT', 'ORIENTATION', &
                   'ELEMENT_NUMBERS', 'NODE_NUMBERS', 'MARK_NODES', &
                   'INNER_CURVES', 'OUTER_CURVES', 'NO_CURVES' /
      data ncode1 /  4, 3, 5, 5,10,  8,  6, 3, 3, 4 /
      data ncodmx1 /  6, 5, 9,11,15, 12, 10,12,12, 9 /

!     code1   Table with sub keywords corresponding to keyword PLOT_BOUNDARY
!             Values:
!             1:  REGION
!             2:  YFACT
!             3:  EYEPOINT
!             4:  ORIENTATION
!             5:  ELEMENT_NUMBERS
!             6:  NODE_NUMBERS
!             7:  MARK_NODES
!             8:  INNER_CURVES
!             9:  OUTER_CURVES
!            10:  NO_CURVES
! ======================================================================
!
      call eropen ( 'readscplotmesh' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscplotmesh'
         call prinrl ( tabmax, nncode, 'tabmax' )

      end if
      if ( ierror/=0 ) go to 1000

      inpsep(0) = icode
      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )

!     --- Clear table, itype and tabmin

      table(1:nncode) = 0d0

      itype(1) = 19  ! REGION
      itype(2) = 1   ! YFACT
      itype(3) = 19  ! EYEPOINT
      itype(4) = 0   ! ORIENTATION
      itype(5:10) = 2 ! element_numbers ... no_curves

      tabmin(1:nncode) = 0d0
      tabmin(1) = 2d0*ndim  ! number of coordinates in region
      tabmin(3) = 3d0  ! EYEPOINT

      tabmax(1:nncode) = 0d0
      tabmax(1:3) = 1d5     ! maximum eyepoint
      tabmax(4) = 3d0       ! maximum ORIENTATION

!     --- Clear inpsep

      inpsep(1:nncode) = 0

      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      if ( ierror/=0 ) go to 1000
      if ( table(1)>0 ) then

!     --- REGION found, fill value in rinsep and reference in inpsep

         inpsep(1) = istrin
         iposut = nint(table(1))
         do i = 0, 2*ndim-1
            rinsep(istrin+i) = tabout(i+iposut)
         end do
         istrin = istrin+2*ndim

      end if  ! ( table(1)>0 )

      if ( table(2)>0 ) then

!     --- YFACT found, fill value in rinsep and reference in inpsep

         inpsep(2) = istrin
         rinsep(istrin) = table(2)
         istrin = istrin+1

      end if  ! ( table(2)>0 )
      if ( table(3)>0 ) then

!     --- EYE-point found, fill value in rinsep and reference in inpsep

         inpsep(3) = istrin   ! starting address eye-point
         iposut = nint(table(3))    ! position in tabout
         do i = 0, 2
            rinsep(istrin+i) = tabout(i+iposut)
         end do
         istrin = istrin+3

      end if  ! ( table(2)>0 )

      inpsep(4:7) = nint(table(4:7))  ! orientation ... mark_nodes

!     --- Fill plotboun

      call readplotboun ( inpsep(8), code(8), ncodmx(8), table(8) )

!     --- Raise istart

      istart = istart+9

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readscplotmesh'

      end if
      call erclos ( 'readscplotmesh' )

      end subroutine readscplotmesh
