      subroutine readscsoltim ( istart, inpsep, nprob, rinsep, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read input for subroutine soltim in the structure block
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduleplot
      use sepmodulepost
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: istart, inpsep(0:*), istrin
      integer, intent(in) :: nprob
      double precision, intent(inout) :: rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         In the standard case this part contains:
!                         1  ISEQNR   Sequence number corresponding to input
!                                     If iseqnr = -10, position 5 ... are used
!                                     See subroutine readscsoltim
!                         2  IPROB    Problem sequence number
!                                     If iprob = -1, the next one will be used
!                         3  IVECT    Vector sequence number (first vector)
!                                     If ivector = -1, the same value as iprob
!                                     will be used
!     istart        i/o   Next free position in inpsep
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     nprob          i    Number of various problems
!     rinsep        i/o   Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istartbef, istrinbef, i
      logical outfnd, skipread

!     i              Counting variable
!     istartbef      Value of istart at input
!     istrinbef      Value of istrin at input
!     outfnd         dummy
!     skipread       If true reading must be skipped
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     READADDVEC     Reread name of new vector and add to list of vectors
!     READSCSOLTIM1  Read input following solve_time_dependent_problem in the
!                    structure block
!                    No sequence number found
!     READSCSTANDARD Read keywords after a number of standard keywords
! **********************************************************************
!
!                       I/O
!
!       SOLVE_TIME_DEPENDENT_PROBLEM, options (all in one line)
!           where options may have the following contents
!           METHOD = type1
!           TINIT  = t0
!           TEND   = t1
!           THETA  = theta
!           TSTEP  = dt
!           TOUTInit = t
!           TOUTEnd  = t
!           TOUTStep = t
!           STIFFNESS_MATRIX = types
!           RIGHT_HAND_SIDE = typer
!           SEQ_OUTPUT = iseq
!           SEQ_SOLUTION_METHOD = iseq
!           SEQ_COEFFICIENTS = iseq_1
!           SEQ_BOUNDARY_CONDITIONS = iseq
!           DIAGONAL_MASS_MATRIX
!           CONSISTENT_MASS_MATRIX
!           PRINT_LEVEL = p
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readscsoltim' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscsoltim'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istartbef = istart  ! save value of istart

!     --- First we read the standard keywords and check if there are
!         no other keywords

      errorcode = 1   ! return when an unknown keyword is found
      call readscstandard ( istart, inpsep(0), outfnd, nprob )
      if ( debug ) then
         write(irefwr,1) 'errorcode, lastitem', errorcode, lastitem
         call prinin ( inpsep(0), 4, 'inpsep' )
         call prinin ( typeitems, nitems, 'typeitems' )
      end if  ! ( debug )

      if ( nitems>1 .and. inpsep(3)<=0 .and. typeitems(2)==3 ) then

!     --- item 3 must be the name of the vector
!         Read name and add to set of vectors

         if ( debug ) write(irefwr,*) 'before readaddvec'
         call readaddvec
         if ( ierror/=0 ) go to 1000

         inpsep(3) = numvec

         skipread = .false.

         if ( debug ) then
            write(irefwr,*) 'skipread ', skipread
            write(irefwr,1) 'numvec, nitems, lastitem', &
                             numvec, nitems, lastitem
         end if  ! ( debug )

         if ( nitems>lastitem ) then

!        --- there are more items to be read

            skipread = .true.
            errorcode = -1
            lastitem = lastitem+1       ! reset lastitem

         end if  ! ( nitems>lastitem )

         if ( debug ) write(irefwr,*) 'skipread ', skipread

         if ( .not. skipread ) then

            lastitem = 1        ! reset lastitem
            istart = istartbef  ! reset istart

!           --- Reinterprete input

            errorcode = 1   ! return when an unknown keyword is found
            call readscstandard ( istart, inpsep(0), outfnd, nprob )
            if ( debug ) then
               write(irefwr,1) 'errorcode, lastitem', &
                                errorcode, lastitem
               call prinin ( inpsep(0), 4, 'inpsep' )
               call prinin ( typeitems, nitems, 'typeitems' )
            end if  ! ( debug )

         end if  ! ( .not. skipread )

      else if ( errorcode<0 ) then

!     --- errorcode<0, reset lastitem

         lastitem = 3

      end if

      if ( errorcode<0 ) then

!     --- errorcode = -1, hence other keyword found
!         read rest of keywords with standard routine

         if ( debug ) write(irefwr,1) 'lastitem, istart', lastitem, istart

         call readscsoltim1 ( inpsep(1), rinsep, istrin, nprob )
         if ( ierror/=0 ) go to 1000
         istart = istart+length  ! raise istart

      end if  ! ( errorcode<0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readscsoltim'

      end if  ! ( debug )
      call erclos ( 'readscsoltim' )

      end subroutine readscsoltim
