      subroutine readtm ( iintim, rintim, iseqnr, isubr, ndim, &
                          lniintim, leniintim, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.16 date 23-10-2015 Default iseqcoef = 0
!        version  4.15 date 02-12-2014 Extension with add_rhs
!        version  4.14 date 09-03-2013 Correction length code
!
!   copyright (c) 1995-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Reads the standard input file for subroutine TIMPRB and stores the
!     information in the arrays INPTIM and RINTIM
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulestrings
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iintim(*), iseqnr, isubr, ndim, lniintim, leniintim, istrin
      double precision rintim(*)

!     iintim         o    Array INPTIM for subroutine PR1000
!     iseqnr        i/o   Sequence number with respect to the storage in array
!                         ISEPIN
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     isubr          i    Defines the caling subroutine.
!                         Possible values:
!                         1: READAL
!                         2: TIMPRB
!     leniintim      o    computed length of iintim
!     lniintim       i    declared length of iintim
!     ndim           i    Dimension of space
!     rintim         o    Array RINTIM for subroutine PR1000
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncod2=2, nncod3=9, &
                            nncod5=5, maxvct=100, maxscl=100, nncod4=16
      integer :: i, j, ncode2(nncod2), itype2(nncod2), &
                 ncod2m(nncod2), imas, &
                 ncode3(nncod3), iscal, istart, istcof, istieq, &
                 ncode5(nncod5), itype5(nncod5), ieq, ivec, numcop, &
                 ncod5m(nncod5), ncod3m(nncod3), &
                 ncode4(nncod4), itype4(nncod4), ncod4m(nncod4)
      character (len=16) :: code2(nncod2)
      character (len=19) :: code3(nncod3), code5(nncod5)
      character (len=23) :: code4(nncod4)
      double precision :: tabmn4(nncod4), tabmx4(nncod4), &
                          tabmn5(nncod5), tabmx5(nncod5), &
                          tabmn2(nncod2), tabmx2(nncod2)

!     code2          Table with keywords of the first record
!     code3          Table with subkeywords
!     code4          Table with subkeywords
!     code5          Table with subkeywords
!     i              Counting variable
!     ieq            Equation sequence number
!     imas           Indicates the type of mass matrix
!                    (1=diagonal, 2=consistent)
!     iscal          Scalar sequence number
!     istart         First position used at part containing information of
!                    equations
!     istcof         Starting address coefficients
!     istieq         Start position of equation in equation part of iintim
!     itype2         Indicates the type of action corresponding to keywords in
!                    code2
!     itype4         Indicates the type of action corresponding to keywords in
!                    code4
!     itype5         Indicates the type of action corresponding to keywords in
!                    code5
!     ivec           Sequence number of vector of special structure
!     j              Counting variable
!     maxscl         Maximum number of scalars
!     maxvct         Maximum number of vectors per time-step permitted
!     ncod2m         Table containing the maximal number of  characters in
!                    the keyword table code2
!     ncod3m         Table containing the maximal number of  characters in
!                    the keyword table code3
!     ncod4m         Table containing the maximal number of  characters in
!                    the keyword table code4
!     ncod5m         Table containing the maximal number of  characters in
!                    the keyword table code5
!     ncode2         Table containing the number of significant letters in code2
!     ncode3         Table containing the number of significant letters in code3
!     ncode4         Table containing the number of significant letters in code4
!     ncode5         Table containing the number of significant letters in code5
!     nncod2         Number of keywords that are recognized in the first record
!     nncod3         Number of entries in code3
!     nncod4         Number of entries in code4
!     nncod5         Number of entries in code5
!     numcop         number of coupled equations
!     tabmn2         Table containing the minimum values of the quantities
!                    given in code2
!     tabmn4         Table containing the minimum values of the quantities
!                    given in code4
!     tabmn5         Table containing the minimum values of the quantities
!                    given in code5
!     tabmx2         Table containing the maximum values of the quantities
!                    given in code2
!     tabmx4         Table containing the maximum values of the quantities
!                    given in code4
!     tabmx5         Table containing the maximum values of the quantities
!                    given in code5
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READTM1        Read first part of array iintim referring to general items
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODEMN  Copy codemn2 and related arrays into codemain
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!     READTM reads input from the standard input file
!     The input must be of the following shape:
!        TIME_INTEGRATION, SEQUENCE_NUMBER = s  (mandatory)
!           followed by data records of the following shape (optional)
!           METHOD = type1
!           TINIT  = t0
!           TEND   = (t1, t2, t3, ... ,tN)
!           THETA  = (theta1, theta2, ... , thetaM)
!           TSTEP  = (dt1, dt2, dt3, ... ,dtN)
!           BETA =  beta
!           GAMMA = gamma
!           ALPHA_F = alpha_f
!           ALPHA_M = alpha_m
!           TOUTInit = t
!           TOUTEnd  = t
!           TOUTStep = t
!           ACCURACY = eps
!           ABS_STATIONARY_ACCURACY = eps
!           REL_STATIONARY_ACCURACY = eps
!           MASS_MATRIX = typem
!           STIFFNESS_MATRIX = types
!           RIGHT_HAND_SIDE = typer
!           SEQ_OUTPUT = iseq
!           SEQ_SOLUTION_METHOD = iseq
!           SEQ_COEFFICIENTS = iseq_1, iseq_2, ...
!           SEQ_BOUNDARY_CONDITIONS = iseq
!           NUMBER_OF_COUPLED_EQUATIONS = n
!           BOUNDARY_CONDITIONS = typeb
!           LINEAR_SUBELEMENT
!           DIAGONAL_MASS_MATRIX
!           CONSISTENT_MASS_MATRIX
!           PRINT_LEVEL = p
!           THRESHOLD_TIME = t
!           AT_ERROR = type_err
!           USE_CTIMEN
!           KEEP_T
!           REUSE_TIME_PARAMETERS
!           PRINT_TIME_HISTORY = ((x1,y1,z1),(x2,y2,z2),...(xn,yn,zn))
!           NON_LINEAR_ITERATION
!           UPDATED_LAGRANGE
!           UPDATED_LAGRANGE
!           SKIP_MESH_DEFORMATION
!           ABS_ITERATION_ACCURACY = a
!           REL_ITERATION_ACCURACY = a
!           MAX_ITER = m
!           MIN_ITER = m
!           NO_COMPUTATION
!           EQUATION i
!              LOCAL_OPTIONS followed by:
!                 MASS_MATRIX = typem
!                 STIFFNESS_MATRIX = types
!                 RIGHT_HAND_SIDE = typer
!                 SEQ_SOLUTION_METHOD = iseq
!                 SEQ_COEFFICIENTS = iseq
!                 SEQ_LOCAL_TIME_STEP = iseq
!                 SEQ_VELOCITY = iseq
!                 SEQ_ACCELERATION = iseq
!                 LINEAR_SUBELEMENT
!                 DIAGONAL_MASS_MATRIX
!                 CONSISTENT_MASS_MATRIX
!                 VECTOR = i
!                 LIMIT = type_limit
!                 REACTION_FORCE = j
!                 SEQ_MATRIX = i
!              DERIVATIVES, options (all in one line)
!                  where options may have the following contents
!                      SEQ_COEF = s         (Default none)
!                      SEQ_DERIV = s        (Default the next one)
!                      PROBLEM = s          (Default the next one)
!                      VECTOR = s           (Default nprob+1)
!        END (mandatory)
!    type1 may take one of the following values:
!       STATIONARY
!       EULER_TRAPEZOID
!       ROZENBROCK_WANNER
!       RUNGE_KUTTA_12
!       RUNGE_KUTTA_34
!       EULER_IMPLICIT
!       CRANK_NICOLSON
!       EULER_EXPLICIT
!       RUNGE_KUTTA_4
!       THETA
!       FRACTIONAL_STEP
!       CENTRAL_DIFFERENCES
!       GENERALIZED_THETA
!       SECOND_ORDER_GEAR
!       NEWMARK
!       GENERALIZED_ALPHA
!    typem and types may take one of the values:
!       CONSTANT
!       VARIABLE
!    typer may take one of the values
!         CONSTANT
!         VARIABLE
!         ZERO
!    typeb may take one of the values
!         CONSTANT
!         VARIABLE
!         INITIAL_FIELD
!         OLD_VECTOR
!    type_err may take one of the values
!         STOP
!         RETURN
!    type_limit may take one of the values
!         NONE
!         MINMAX
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!     574   Not an integer number found after the keyword
!    1506   Both VECTOR and SCALAR given in DERIVATIVES
!    1595   Main keyword has not been found in first record
!    1896   IEQ out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
!    It is assumed that the first record has already been read, &
!    Only the part after TIME_INTEGRATION must be analyzed
!    The rest of the input is read continuously until the keyword END
!    is found
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'TIME_INTEGRATION', 'SEQUENCE_NUMBER' /
      data ncode2 / 4, 3 /
      data ncod2m / 16, 15 /
      data itype2 / 2, 0 /
      data tabmn2 / 0d0, 1d0 /
      data tabmx2 / 0d0,10d0 /
      data code3 / 'END', 'EQUATION', 'LOCAL_OPTIONS', 'DERIVATIVES', &
                   'VARIABLE', 'CONSTANT', 'ZERO', 'NONE', 'MINMAX' /
      data ncode3 / 3, 5, 5, 5, 4,   4, 4, 4, 4 /
      data ncod3m / 3, 8,13,11, 8,   8, 4, 4, 6 /
      data code4 / 'STIFFNESS_MATRIX',  'MASS_MATRIX', &
                   'RIGHT_HAND_SIDE',   'SEQ_SOLUTION_METHOD', &
                   'SEQ_COEFFICIENTS',  'LINEAR_SUBELEMENT', &
                   'DIAGONAL_MASS_MATRIX', 'CONSISTENT_MASS_MATRIX', &
                   'VECTOR', 'LIMIT', 'SEQ_LOCAL_TIME_STEP', &
                   'REACTION_FORCE', &
                   'SEQ_VELOCITY', 'SEQ_ACCELERATION', 'SEQ_MATRIX', &
                   'SEQ_ADD_RHSD' /
      data ncode4 / 4,   4,  4,  7,  7,   10, 10, 10, 3,  5, &
                    8,   5,  7,  7,  7,    6 /
      data ncod4m / 16, 11, 15, 19, 16,   17, 20, 22, 6,  5, &
                    19, 14, 12, 16, 10,   12 /
      data itype4 / 5,   5,  5,  0,  0,    2,  2,  2, 0,  5, &
                    0,   0,  0,  0,  0,    0 /
      data tabmn4 / 0d0,  0d0,  0d0,  1d0,  1d0, &
                    1d0,  0d0,  0d0,  1d0,  0d0, &
                    0d0,  1d0,  1d0,  1d0, 1d0, 1d0 /
      data tabmx4 / 0d0,  0d0,  0d0, 25d0, 25d0, &
                   25d0,  0d0,  0d0,  1d2,  0d0, &
                    1d2, 25d0,  1d2,  1d2,  1d1,  1d2 /
      data code5 / 'SEQ_COEF', 'PROBLEM', 'VECTOR', 'SEQ_DERIV', &
                   'SCALAR' /
      data ncode5 / 6, 4, 3, 6, 5 /
      data ncod5m / 8, 7, 6, 9, 6 /
      data itype5 / 0, 0, 0, 0, 0 /
      data tabmn5 / 5 * 1d0 /

!     code      Table containing the subkeywords corresponding to this
!               subroutine. Contents:
!               1:  METHOD
!               2:  TINIT
!               3:  TEND
!               4:  THETA
!               5:  TSTEP
!               6:  TOUTINIT
!               7:  TOUTEND
!               8:  TOUTSTEP
!               9:  ABS_STATIONARY_ACCURACY
!              10:  REL_STATIONARY_ACCURACY
!              11:  MASS_MATRIX
!              12:  STIFFNESS_MATRIX
!              13:  RIGHT_HAND_SIDE
!              14:  SEQ_SOLUTION_METHOD
!              15:  SEQ_COEFFICIENTS
!              16:  SEQ_BOUNDARY_CONDITIONS
!              17:  NUMBER_OF_COUPLED_EQUATIONS
!              18:  BOUNDARY_CONDITIONS
!              19:  LINEAR_SUBELEMENT
!              20:  DIAGONAL_MASS_MATRIX
!              21:  CONSISTENT_MASS_MATRIX
!              22:  SEQ_OUTPUT
!              23:  ACCURACY
!              24:  PRINT_LEVEL
!              25:  THRESHOLD_TIME
!              26:  AT_ERROR
!              27:  USE_CTIMEN
!              28:  KEEP_T
!              29:  PRINT_TIME_HISTORY
!              30:  NON_LINEAR_ITERATION
!              31:  ABS_ITERATION_ACCURACY
!              32:  REL_ITERATION_ACCURACY
!              33:  MAX_ITER
!              34:  NO_COMPUTATION
!              35:  REUSE_TIME_PARAMETERS
!              36:  BETA
!              37:  GAMMA
!              38:  ALPHA_F
!              39:  ALPHA_M
!              40:  UPDATED_LAGRANGE
!              41:  SKIP_MESH_DEFORMATION
!              42:  MIN_ITER
!              43:  SEQ_ADD_RHSD
!     code1     Table containing the subsubkeywords corresponding to this
!               subroutine as well as the keyword END. Contents:
!               1:  END
!               2:  EQUATION
!               3:  STATIONARY
!               4:  EULER_TRAPEZOID
!               5:  ROZENBROCK_WANNER
!               6:  RUNGE_KUTTA_12
!               7:  RUNGE_KUTTA_34
!               8:  EULER_IMPLICIT
!               9:  CRANK_NICOLSON
!              10:  EULER_EXPLICIT
!              11:  RUNGE_KUTTA_4
!              12:  THETA
!              13:  FRACTIONAL_STEP
!              14:  CENTRAL_DIFFERENCES
!              15:  GENERALIZED_THETA
!              16:  SECOND_ORDER_GEAR
!              17:  NEWMARK
!              18:  GENERALIZED_ALPHA
!              19:  VARIABLE
!              20:  CONSTANT
!              21:  ZERO
!              22:  VARIABLE
!              23:  CONSTANT
!              24:  INITIAL_FIELD
!              25:  OLD_VECTOR
!              26:  STOP
!              27:  RETURN
!     code2     Table containing the keywords in the first record
!               Contents:
!               1:  TIME_INTEGRATION
!               2:  SEQUENCE_NUMBER
!     code3     Table containing the keywords to be found after EQUATION
!               Contents:
!               1:  END
!               2:  EQUATION
!               3:  LOCAL_OPTIONS
!               4:  DERIVATIVES
!     code4     Table containing the subkeywords corresponding to this
!               subroutine. Contents:
!               1:  STIFFNESS_MATRIX
!               2:  MASS_MATRIX
!               3:  RIGHT_HAND_SIDE
!               4:  SEQ_SOLUTION_METHOD
!               5:  SEQ_COEFFICIENTS
!               6:  LINEAR_SUBELEMENT
!               7:  DIAGONAL_MASS_MATRIX
!               8:  CONSISTENT_MASS_MATRIX
!               9:  VECTOR
!              10:  LIMIT
!              11:  SEQ_LOCAL_TIME_STEP
!              12:  REACTION_FORCE
!              13:  SEQ_VELOCITY
!              14:  SEQ_ACCELERATION
!              15:  SEQ_MATRIX
!              16:  SEQ_ADD_RHSD
!     code5     Table containing the subkeywords corresponding to derivatives
!               Contents:
!               1:  SEQ_COEF
!               2:  PROBLEM
!               3:  VECTOR
!               4:  SEQ_DERIV
!               5:  SCALAR
! ======================================================================
!
!     --- Initialize some parameters

      call eropen ( 'readtm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readtm'
         write(irefwr,1) 'isubr, ityprd', isubr, ityprd
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check if there is any text after TIME_INTEGRATION

      if ( isubr/=1 .or. ityprd/=1 ) then

!     --- read sequence number and possibly time_integration

         table(1) = 0d0
         table(2) = iseqnr
         iretur = 0
         itype(1:nncod2) = itype2
         tabmin(1:nncod2) = tabmn2
         tabmax(1:nncod2) = tabmx2
         call sepcopycode ( code2, nncod2, ncode2, ncod2m )
         call sepreadsetkeyw
         if ( ierror/=0 ) go to 1000
         if ( isubr==2 .and. table(1)<=0d0 ) then

!        --- keyword TIME_INTEGRATION not found

            call errchr ( code2(1), 1 )
            call errsub ( 1595, 0, 0, 1 )

         end if
         iseqnr = nint(table(2))

      end if

      if ( debug ) write(irefwr,1) 'iseqnr', iseqnr

!     --- Read rest of the records until END is found
!         Set istop to continuously reading

      istop = 2

!     --- Read first part of input

      istrin = 1
      call readtm1 ( 1, iintim, rintim, iseqnr, ndim, numcop, &
                     leniintim, istrin )

!     --- Reset istop

      istop = 1
      if ( iretur==2 ) then

!     --- iretur = 2, equation found, fill iintim(38)

         iintim(38) = leniintim+1
         do i = 1, numcop
            iintim(leniintim+1) = iintim(10)  ! iconsstiff
            iintim(leniintim+2) = iintim(11)  ! iconsmass
            iintim(leniintim+3) = iintim(12)  ! iconsrhs
            iintim(leniintim+4) = iintim(13)  ! ireadsl
            iintim(leniintim+5) = 0           ! ireadbc
            iintim(leniintim+6) = iintim(16)  ! isubdivide
            iintim(leniintim+7) = iintim(14)  ! imas
            iintim(leniintim+8) = i           ! sequence number of vector
            iintim(leniintim+9:leniintim+25) = 0
            leniintim = leniintim+25
         end do

      end if

100   if ( iretur==2 ) then

!     --- iretur = 2, equation found, read sequence number

         if ( debug ) write(irefwr,*) 'equation found'
         call asknxt
         if ( ityprd==2 ) then

!        --- integer number found

            ieq = nint(realrd)
            if ( ieq<1 .or. ieq>numcop ) then

!           --- Equation number out of range

               call errint ( ieq, 1 )
               call errint ( numcop, 2 )
               call errsub ( 1896, 2, 0, 0 )
               call errcrd
               ieq = 1

            end if

         else

!        --- Not an integer number found

            call errchr ( 'EQUATION', 1 )
            call errsub ( 574, 0, 0, 1 )
            call errcrd
            ieq = 1

         end if
         if ( debug ) write(irefwr,1) 'ieq', ieq

!        --- Fill information about equation

         istieq = iintim(38)+25*(ieq-1)
         iintim(istieq:istieq+24) = 0
         istcof = iintim(37)-1
         istart = leniintim+1
         iintim(istart) = 0
         iintim(istieq+9) = istart

150      ityprd = 1
         call asknxt
180      ierpri = 1
         call sepcopycode ( code3, nncod3, ncode3, ncod3m )
         iretur = sepsubkeyw()
         if ( iretur==3 ) then

!        --- local options found, read rest of information
!            Read rest of the records until main keyword is found
!            Set istop to continuously reading

            if ( debug ) write(irefwr,*) 'local options found'
            istop = 2

!           --- Fill positions of subkeywords

            nmain = 4
            itype(1:nncod4) = itype4
            tabmin(1:nncod4) = tabmn4
            tabmax(1:nncod4) = tabmx4
            tabmin(1) = nmain+1
            tabmax(1) = tabmin(1)+1d0
            tabmin(2) = tabmin(1)
            tabmax(2) = tabmax(1)
            tabmin(3) = tabmin(1)
            tabmax(3) = tabmin(1)+2d0
            tabmin(10) = tabmax(3)+1d0
            tabmax(10) = tabmin(10)+1d0
            tabmax(4:5) = 25d0
            tabmax(12) = dble(maxvec)

!           --- Put default values into table

            table(1:nncod4) = -1d20
            table(13:14) = 0d0
            table(15) = 2d0
            table(16) = 0d0
            call sepcopycode ( code4, nncod4, ncode4, ncod4m )
            call sepcopycodemn ( code3, nncod3, ncode3 )
            call sepreadsetkeyw
            if ( ierror/=0 ) go to 1000
            if (debug ) call prinrl ( table, nncod4, 'table4' )

!           --- iconsstiff

            if ( table(1)>-1d5 ) iintim(istieq) = nint(table(1))-1

!           --- iconsmass

            if ( table(2)>-1d5 ) iintim(istieq+1) = nint(table(2))-1

!           --- iconsrhs

            if ( table(3)>-1d5 ) iintim(istieq+2) = nint(table(3))-1

!           --- ireadsl

            if ( table(4)>-1d5 ) iintim(istieq+3) = nint(table(4))

!           --- iseq_coefficients

            if ( table(5)>-1d5 ) iintim(ieq+istcof) = nint(table(5))

!           --- isubdivide

            if ( table(6)>-1d5 ) iintim(istieq+5) = nint(table(6))

            iintim(istieq+6) = iintim(14)
            if ( table(7)>-1d5 .and. table(8)>-1d5 ) then

!           --- imas

               if ( table(7)>=0d0 ) then
                  if ( table(8)>=0d0 ) &
                     call errkeywexcl ( code4, ncod4m, 7, 8 )

                  imas = 1
               else if ( table(8)>=0d0 ) then
                  imas = 2
               else
                  imas = 2
               end if
               iintim(istieq+6) = imas

            end if

!           --- seq_vector

            if ( table(9)>-1d5 ) iintim(istieq+7) = nint(table(9))

!           --- limiter

            if ( table(10)>0d0 ) &
               iintim(istieq+10) = nint(table(10))-1

!           --- sequence number of local time step

            if ( table(11)>0d0 ) then
               iintim(istieq+11) = nint(table(11))
            else
               iintim(istieq+11) = 0
            end if  ! ( table(11)>0d0 )

!           --- sequence number of reaction force

            if ( table(12)>0d0 ) then
               iintim(istieq+12) = nint(table(12))
            else
               iintim(istieq+12) = 0
            end if  ! ( table(12)>0d0 )
            iintim(istieq+13) = nint(table(13))  ! iseq_velocity
            iintim(istieq+14) = nint(table(14))  ! iseq_acceleration
            iintim(istieq+15) = nint(table(15))  ! iseq_matrix
            iintim(istieq+16) = nint(table(16))  ! iseq_add_rhs
            istop = 1
            go to 180

         else if ( iretur==4 ) then

!        --- derivatives found, read rest of information

            itype(1:nncod5) = itype5
            tabmin(1:nncod5) = tabmn5
            tabmax(1:nncod5) = tabmx5
            table(1) = 0d0
            table(2) = 1d0
            table(3) = -1d0
            table(4) = -1d0
            table(5) = -1d0
            tabmax(1) = 25d0
            tabmax(2) = numcop
            tabmax(3) = dble(maxvct)
            tabmax(4) = 100d0
            tabmax(5) = dble(maxscl)
            iretur = 0
            nmain = 0
            call sepcopycode ( code5, nncod5, ncode5, ncod5m )
            call sepreadsetkeyw
            if ( ierror/=0 ) go to 1000

!           --- Put information into the arrays iintim and rintim

            iintim(istart) = 8
            iintim(istart+1) = nint(table(1))
            iintim(istart+2) = nint(table(2))
            iintim(istart+4) = nint(table(4))
            ivec  = nint(table(3))
            iscal = nint(table(5))
            if ( ivec==-1 ) then

!           --- Vector not given

               if ( iscal==-1 ) then
                  ivec = numcop+1
               else
                  ivec = -2
               end if

            else

!           --- Vector has been given, so scalar is not allowed

               if ( iscal>0 ) then
                  call errsub ( 1506, 0, 0, 0 )
                  call errcrd
               else
                  iscal = 0
               end if
            end if
            iintim(istart+3) = ivec
            iintim(istart+5) = iscal

            istart = istart + 5
            go to 150

         else

!        --- end of information

            leniintim = istart+1
            iintim(leniintim) = -1

         end if

         goto 100

      end if

!     --- Check leniintim of array iintim

      if ( leniintim>lniintim ) then

!     --- leniintim too small

         call errint ( leniintim, 1 )
         call errint ( lniintim, 2 )
         call errchr ( 'iintim', 1 )
         call errsub ( 550, 2, 0, 1 )

      end if

      istrin = istrin-1

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iintim, leniintim, 'iintim' )
         call prinrl ( rintim, istrin, 'rintim' )
         write(irefwr,*) 'End readtm'

      end if  ! ( debug )
      call erclos ( 'readtm' )

      end subroutine readtm
