      subroutine sepdefmshnew ( isol, scale )
! ======================================================================
!
!        programmer    Marcel Rutten
!        version  1.0  date 28-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Deform mesh for solid computation. The deformed mesh is stored
!     in KMESH! The old KMESH is destroyed.
!     This routine is usually used to:
!     1) plot deformed 3d meshes
!     2) deform meshes for Arbitrary Euler Lagrange computations
!     3) deform meshes for Updated Lagrange computations
!     Old subroutine sepdefmsh
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     coordinate
!     deformation
!     solid
!     solid_mechanics
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*)
      double precision scale

!     isol       i    array of special structure containing information
!                     of the solution
!     scale      i    scaling factor (useful when small displacements are
!                     involved)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lnkmsx, compcurv, ielgrp
      logical adaptcenter
      double precision :: timebefore(2)

!     adaptcenter    If true the center of the element must be adapted
!     compcurv       inidcates if kmeshh has been filled (>0) or not (0)
!     ielgrp         element group number
!     lnkmsx         Length of first dimension of kmeshx
!                    2d: 6,  3d: 12
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DEFM01         actual deformation routine
!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERSETTIME      Get the CPU time at start of subroutine
!     MSHDEFCURVS    Copy coordinates from array coor into array curves
!     MSHRMIOBGRD    Remove all arrays iobgrid from array kmeshx
!     SEPACTSOLBF1   compute pointers to degrees of freedom of vectors
!     SEPADAPCENT3D  Adapt center of elements in case of mesh deformation (3D)
!     SEPADAPCENTER  Adapt center of elements in case of mesh deformation
!     SEPMSHFLKMP    Fill array kmesh part p
!     SEPMSHNORMS    Compute normals on boundary and store in array normal
!     SEPUPDOBS      Fill information in kmesh part x concerning obstacles
!     SEPUPDOBS3D    Fill information in kmesh part x concerning 3d obstacles
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Check input:

      call eropen( 'sepdefmshnew' )
      if ( itime>0 ) call ersettime ( timebefore )
      if ( ierror/=0 ) go to 1000

!     --- Check if there elements with a centroid
!         In that case adaptcenter is set to true

      adaptcenter = .false.   ! do not adapt centroids
      do ielgrp = 1, nelgrp
         ishape = kmeshf(ielgrp)
         if ( ishape>=6 .or. ishape<=10 .or. ishape==14 .or. &
              ishape>=16 .and. ishape<=18 ) adaptcenter = .true.
      end do  ! ielgrp = 0, nelgrp-1
      if ( kelmp==0 .and. ncurvs>0 ) then

!     --- ndim = 2 and ichois<=2
!         Fill array KMESH part P

         allocate ( km(iseqkm)%elsize(2,ncurvs) )
         elsize => km(iseqkm)%elsize
         km(iseqkm)%kmesh(30) = ncurvs
         kelmp = km(iseqkm)%kmesh(30)
         call sepmshflkmp

      end if
      if ( ierror/=0 ) go to 1000

      call sepactsolbf1 ( isol )
      if ( ierror/=0 ) go to 1000
      usoltemp => ks(isol(1))%sol
      if ( ierror/=0 ) go to 1000

!     --- Move coordinates according to their respective
!         degree of freedom in nodal points:

      call defm01 ( coor, kprobfi, &
                    npoint, nphysi, ndim, indprfi, usoltemp, scale )
      if ( ierror/=0 ) go to 1000

      if ( adaptcenter ) then

!     --- There are quadratic elements of which the centroids must be adapted
!         In all other cases nothing happens
!         Only shape 7 and 14 are implemented

         if ( ndim==3 ) then

!        --- Adapt 3d elements

            call sepadapcent3d
         else

!        --- Adapt 2d elements

            call sepadapcenter

         end if  ! ( ndim==3 )

      end if  ! ( adaptcenter ) then

      if ( kelmh>0 ) then

!     --- kelmh> 0, i.e. kmesh part exists
!         Adapt coordinates in curves

         call mshdefcurvs ( icurvsmsh, curvesmsh, ndim, &
                            ncurvs, kmeshm, coor )

      end if  ! ( kelmh>0 )
      if ( ierror/=0 ) go to 1000

      if ( numobst>0 ) then

!     --- numobst>0, there are obstacles
!         These are treated separately
!         Update rmeshx and remove arrays iobgrid

         if ( kelmad/=0 ) iseqobst = kmeshad(5)
         if ( ndim==2 ) then

            call sepupdobs
            lnkmsx=6

         else if ( ndim==3 ) then

            call sepupdobs3d
            lnkmsx=12

         end if

         if ( kelmad/=0 ) kmeshad(5)=iseqobst

!        --- Remove arrays iobgrid

         if ( kelmad/=0 ) call mshrmiobgrd

      end if
      if ( ierror/=0 ) go to 1000

      if ( mmnormal>0 ) then

!     --- mmnormal>0, i.e. normals are available
!         adapt to new mesh

         call sepmshnorms

      end if  ! ( mmnormal>0 )

1000  call erclmn( 'sepdefmshnew', timebefore )
      end subroutine sepdefmshnew
