      subroutine sepfillcoeff2 ( isubr, iseqnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-12-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read input concerning the coefficients from the standard input file
!     or array IBUFFR. Fill coefficients in the arrays IUSER and USER
!     Old subroutine to0093
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     fill
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulekmesh
      use sepmodulekprob
      use sepmoduleinput
      use sepmodulecoeffs
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isubr, iseqnr

!     iseqnr   i   Sequence number with respect to the storage in array ISEPIN
!     isubr    i   Defines from which subroutine sepfillcoeff2 is called
!                  Possible values:
!                  1:  FILCOF
!                  2:  CHNCOF
!                  3:  FILLCF
!                  4:  CHANCF
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxcal, ipos, ngroup
      logical compl

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     ipos           Position of input arrays in sepmoduleinput
!     maxcal         Maximum number of coefficient inputs read
!     ngroup         Total number of element groups
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPFILLIUSER   Fill information of coefficients as stored in arrays
!                    iincof and rincof into iuser/user
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
! **********************************************************************
!
!                       I/O
!
!    Input may be read from the standard input file
!    The actual reading is performed by subroutine sepreadcoeffs
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1290   Sequence number has not been defined
!    1913   Problem number for coefficients differs from the one for the solver
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Check KMESH, KPROB and IPROB
!   Set NELGRP and numnat
!   If ( isubr<3 ) then
!      read first record
!      read input from the standard input file and put in IINCOF, RINCOF
!   else
!      extract input from IBUFFR and put in IINCOF, RINCOF
!   Extract maxprm from IINCOF
!   if ( isubr=1 or 3 ) then
!      Clear IWORK, WORK and NPARM
!   else if ( isubr=2 or 4 ) then
!      Create arrays IWORK, WORK and NPARM from IUSER, USER
!   Expand IINCOF, RINCOF into IWORK, WORK and NPARM
!   Fill arrays IUSER and USER
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillcoeff2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillcoeff2'
         write(irefwr,1) 'isubr', isubr
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Read input and store in arrays iincof and rincof

      ngroup = nelgrp+numnat+nglgrp
      if ( debug ) &
         write(irefwr,1) 'nelgrp, numnat, nglgrp', &
                          nelgrp, numnat, nglgrp

      nrcof = ngroup * 100           ! size of rincof
      nicof = 4 * nrcof              ! size of iincof

!     --- allocate temporary arrays

      allocate ( iincof(nicof), rincof(nrcof), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, nicof+nrcof, 'iincof' )
      if ( ierror/=0 ) go to 1000

!     --- extract from IBUFFR

      call sepgetinpfinf ( isubr+3, iseqnr, ipos, maxcal )

      if ( maxcal==0 ) then

!     --- Coefficients array has not been created

         call errint ( iseqnr, 1 )
         call errchr ( 'coefficients', 1 )
         call errsub ( 1290, 1, 0, 1 )
         go to 1000

      end if

      length = size(in(ipos)%iinput)
      iincof(1:length) = in(ipos)%iinput
      if ( associated(re(ipos)%rinput) ) then
         length = size( re(ipos)%rinput )
         rincof(1:length) = re(ipos)%rinput
      end if  ! ( associated(re(ipos)%rinput)>0 )
      if ( iincof(2)==0 ) then
         compl = .false.
      else
         compl = .true.
      end if
      if ( iincof(3)/=iprob .and. maxcal>1 ) then

!     --- IPROB in linear solver differs from IPROB fill coefficients

         call errint ( iincof(3), 1 )
         call errint ( iprob, 2 )
         call errsub ( 1913, 2, 0, 0 )
         go to 1000

      end if

!     --- Fill arrays iuser/user from iincof and rincof

      call sepfilliuser ( isubr, compl )

!     --- deallocate temporary arrays

      deallocate ( iincof, rincof, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iincof' )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iuser, iuser(3), 'iuser' )
         call prinrl ( user, nint(user(3)), 'user' )
         write(irefwr,*) 'End sepfillcoeff2'

      end if
      call erclos ( 'sepfillcoeff2' )

      end subroutine sepfillcoeff2
