      subroutine sepintbnd ( iintbn, rintbn, &
                             ivectr, icurvs, value, iread )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 31-03-2015 Extra debug
!        version  2.1  date 29-05-2014 Remove time routine
!        version  2.0  date 23-09-2011 Complete new body
!
!   copyright (c) 1997-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute an integral along a boundary
!     In R2 the boundary is supposed to be a curve, in R3 a surface
!     Old subroutine intbnd
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iintbn(*), ivectr, icurvs(*), iread
      double precision rintbn(*), value(*)

!     icurvs         i    Integer array containing the curve numbers for the
!                         computation of the boundary integral. A negative curve
!                         number indicates that the curve must be computed in
!                         the reversed direction.
!                         The curves including their sign must form one
!                         contiguous curve
!                         The number of curves is stored in array iintbn
!                         If the integration must be carried out over surfaces, &
!                         icurvs must contain the surface sequence numbers.
!                         icurvs is only used if iread = -1
!     iintbn         i    Integer input array in which the user indicates what
!                         type of integral must be computed.
!                         IINTBN must be filled as follows:
!                         IINTBN(1)Number of positions filled by user
!                         IINTBN(2) (ICHINT) indicates what type of integral has
!                                   to be computed. Possibible values:
!                                   1          /  f u  ds
!                                   2          /  f  u.n  ds
!                                   3          /  f  u.t  ds
!                                   4          /  n. sigma .n  ds
!                                   5          /  n. sigma .t  ds
!                                   6          /  n. sigma     ds
!                                   7          /  u  n  ds
!                                   8          sum f u
!                                   9          /  f ds
!                                   The possibilities 3 to 6 have only been
!                                   implemented for NDIM = 2, i.e.
!                                   two-dimensional regions
!                                   Default value: 1
!                         IINTBN(3) (ICHFUN) indication of the value of f.
!                                   Possibible values:
!                                   0         f = 1
!                                   >0        f = func(ichfun,x,y,z) (real case)
!                                             or
!                                             f = cfunc(ichfun,x,y,z) (complex
!                                                 case)
!                                             at this moment only the
!                                             combination f#1 and ichint=1
!                                             is permitted
!                                   Default value: 0
!                         IINTBN(4) (IRULE) The numerical integration rule to be
!                                   carried out depends on IRULE.
!                                   Possibible values:
!                                   For curves only (i.e. 2D):
!                                   1  trapezoid rule cartesian co-ordinates
!                                   2  trapezoid rule axisymmetric co-ordinates
!                                   3  simpson's rule cartesian co-ordinates
!                                   4  simpson's rule axisymmetric co-ordinates
!                                   For surfaces only (i.e. 3D):
!                                   1  Newton cotes rule, i.e. integration rule
!                                      based on the type of approximation used
!                                      in the element
!                                   Default value: 1
!                         IINTBN(5) (IDIM) This parameter indicates what type of
!                                   boundary integral must be computed
!                                   Possible values:
!                                   1: curves
!                                   2: surfaces
!                                   3: contact
!                                   4: no_contact
!                                   5: obstacles
!                         IINTBN(6) (NUMBER) This parameter indicates which
!                                   unknown (physical degree of freedom) must
!                                   be used from the (solution) vector u
!                                   (IVECTR)
!                                   If ICHINT = 2, 3 or 7, it is supposed that
!                                   the degrees of freedom are equal to NUMBER, &
!                                   NUMBER+1 and NUMBER+2 (3D)
!                                   If ICHINT = 4, 5 or 6, it is supposed that
!                                   the degrees of freedom are equal to NUMBER, &
!                                   NUMBER+1 and NUMBER+2 (2D) or NUMBER to
!                                   NUMBER+5 (3D)
!                                   Default value: 1
!                         IINTBN(7) (NCURVES) Indicates the number of curves, &
!                                   surfaces or obstacles in array ICURVS that
!                                   must be used for the integration
!                                   Default value: 1
!     iread          i    Defines if parameters should be read or
!                         are given in array IINTBN through the parameter list.
!                         Possible values:
!                         0   The input is read from the standard input file
!                         >0  The input as read from the standard input file by
!                             subroutine SEPSTN is used. The value of iread
!                             defines the sequence number
!                         -1  No input is read, all information from array
!                             IINTBN is used
!     ivectr         i    Standard SEPRAN array containing information of the
!                         vector to be integrated
!     rintbn         i    Real input array (not yet used)
!     value          o    Array containing the computed integrals at output
!                         When ICHINT = 1 to 5, VALUE has length 1, &
!                         when ICHINT = 6 or 7 VALUE has length 2
!                         If the vector to be integrated corresponds to a
!                         complex vector, VALUE is a complex array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nparm
      parameter ( nparm=7)
      integer idim, ichint, inpcop(nparm+100)
      double precision help(2)

!     help           Work array to store some variables to be sent globally
!     ichint         choice parameter for the type of integral
!                    Possibible values:
!                    1          /  f u  ds
!                    2          /  f  u.n  ds
!                    3          /  f  u.t  ds
!                    4          /  f  n. sigma .n  ds
!                    5          /  f  n. sigma .t  ds
!                    6          /  f  n. sigma     ds
!                    7          /  f  u  n  ds
!                    8          sum  f  u
!                    9          /  f  ds
!     idim           This parameter indicates what type of boundary
!                    integral must be computed
!                    Possible values:
!                    1: curves
!                    2: surfaces
!                    3: contact
!                    4: no_contact
!                    5: obstacles
!     inpcop         Copy of integer input array extended with defaults
!     nparm          Number of parameters in array iintbn
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0058         compute a boundary integral over a part of the boundary in
!                    case of curves
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INTBNDINP      Fill array inpcop with input for subroutine intbndbf
!     INTBNDOBST     compute an integral over obstacles
!     INTBNDSUMSURF  compute an integral along a boundary
!                    The integral consists of a summation over surfaces
!     INTBNDSURF     compute an integral along a boundary consisting of surfaces
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRPARGLSUM     This function performs a global sum of a quantitity
!                    or vector across all processes
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPFILLKPROB   Fill parameters in module sepmodulekprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     419   NDIM incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!     perform standard checks
!     initialize array inpcop
!     If ( iread=-1 ) then
!        Fill array inpcop by copying from array iintbn
!     else if ( iread=0 ) then
!        Read array inpcop from the standard input file
!     else (iread>0)
!        Copy array inpcop from IBUFFR
!     Compute integral
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepintbnd' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepintbnd'
         write(irefwr,*) 'parallel ', parallel
  1      format ( a, 1x, (10i6) )

      end if

!     --- Standard checks

      if ( ndim/=2 .and. ndim/=3 ) then
         call errint ( ndim, 1 )
         call errint ( 2, 2 )
         call errint ( 3, 3 )
         call errsub ( 419, 3, 0, 0 )
      end if

!     --- Fill array inpcop at the end of IBUFFR from starting position
!         ipincp

      call intbndinp ( inpcop, nparm, iread, iintbn, icurvs )
      if ( ierror/=0 ) go to 1000

      ichint = inpcop(2)
      idim = inpcop(5)
      if ( debug ) then
         write(irefwr,1) 'ichint, idim', ichint, idim
         call prinin ( inpcop, nparm, 'iinbnd_copy' )
      end if  ! ( debug )

      if ( ichint/=9 ) then
         iprob = ks(ivectr)%iprob
      else
         iprob = 1
      end if  ! ( ichint/=9 )
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      indprf = kprobloc(19)
      kprobf => kprobpart%kprobf
      indprfi = indprf
      kprobfi => kprobf
      call sepfillkprob

      if ( ichint/=9 ) call sepactsolbf2 ( ivectr )

      if ( idim==1 ) then

!     --- idim = 1, integral over curves

         call er0058 ( inpcop, ivectr, inpcop(1+nparm), value )

      else if ( ichint==8 ) then

!     --- idim>=2 and ichint = 8, summation over surfaces

         call intbndsumsurf ( inpcop, ivectr, nparm, value )

      else if ( idim==5 ) then

!     --- idim = 5, integral over obstacles

         call intbndobst ( inpcop, ivectr, nparm, value, ichint )

      else

!     --- idim>=2, integral over surfaces

         call intbndsurf ( inpcop, ivectr, nparm, value, ichint )

      end if

      if ( parallel ) then

!     --- Parallel computing: perform some global operations
!         Sum all local integrals

         help(1) = value(1)
         if ( ichint<=5 .or. ichint==9 ) then
            idim = 1
         else
            idim = 2
            help(2) = value(2)
         end if
         if ( debug ) call prinrl ( help, idim, 'help' )
         call prparglsum ( help, value, idim, SPDOUBLE )

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( value, idim, 'value' )
         write(irefwr,*) 'End sepintbnd'

      end if  ! ( debug )
      call erclos ( 'sepintbnd' )

      end subroutine sepintbnd
