      subroutine sepintp2d ( ichois, ivalue, values, number, &
                             ioutvc, iskip, ihelp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.6  date 14-10-2016 Remove work arrays
!        version  1.5  date 29-05-2014 Extension of timebefore
!        version  1.4  date 15-11-2012 Long integers
!
!   copyright (c) 1999-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Intersects a two-dimensional mesh with a line
!     The intersection is stored in array ioutvc of type 119
!     The corresponding solution in isol is interpolated on this line
!     and also stored in ioutvc
!     Array ioutvc may be plotted with the aid of subroutine PLOTFN
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     interpolation
!     intersection
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekmesh
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, ivalue, number, iskip, ioutvc, ihelp(*)
      double precision values(*)

!     ichois  i   Choice parameter indicating how the line is defined.
!                 Possibilities:
!                 1:  The user defines a line by defining a starting point
!                     and an angle with respect to the positive x-axis under
!                     which the line must be drawn in the positive x-direction.
!                     The values of the starting point resp. the angle must be
!                     stored into array VALUES as follows:
!                     VALUES (1) = xstart
!                     VALUES (2) = ystart
!                     VALUES (3) = angle of rotation with respect to pos. x-axis
!                 2:  See 1, however the solution is multiplied by the factor f
!                     f must be stored into VALUES(4)
!     ihelp   i   Integer input array of length NELGRP that is only used if
!                 ISKIP = 1
!                 It indicates if element group ielgrp must be skipped
!                 ihelp(ielgrp) = 0 or not ihelp(ielgrp) = 1
!     ioutvc  o   Standard SEPRAN array containing information concerning the
!                 interpolated function on the line.
!                 ioutvc is an array of structure 119
!                 Both the one-dimensional co-ordinates as the function values
!                 are stored in the array corresponding to ioutvc
!     iskip   i   Indication if element groups must be skipped (1) or not (0)
!     ivalue      Not yet used
!     number  i   The number-th unknown in each nodal point of the array
!                 is considered.
!     values  i   Double precision or complex array in which the real or
!                 complex information concerning the line is stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nelemt, ncint, special, ilevel, nelgrpsav, nelemsav, &
                 icompl, numseg, nreal, ichoic, i, iresul(5,3), nomber(2)
      integer :: ipoutp, ipxcin, ipfcin
      double precision angle, origx, origy
      logical compl
      integer, allocatable :: kseg(:)
      integer, pointer :: kmeshclc(:)
      double precision, allocatable :: xcint(:), fcint(:)
      double precision :: timebefore(2)

!     angle          angle of rotation
!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     fcint          array of interpolated function values
!     i              Counting variable
!     ichoic         Choice parameter. With this parameter it is defined which
!                    actions will be performed. Possible values:
!                    1:  Create a submesh consisting of 3-point triangles only
!                        Create also a copy of the solution array containing one
!                        unknown only in all points of the new grid
!                        In this case array KPROB and ISOL is also used
!                        If the solution vector is complex only the real or
!                        imaginary part is used
!                    2:  Create a submesh consisting of 3-point triangles only
!                        Create also a copy of the solution array containing one
!                        unknown only in all points of the new grid
!                        In this case array KPROB and ISOL is also used
!                        If the solution vector is complex the copied solution
!                        vector is also complex
!                    3:  Create a submesh consisting of the surface elements
!                        only of each element only the vertices are stored, &
!                        hence the final grid consists of a grid of 3-point
!                        triangles or 4-point quadrilaterals only
!                        This possibility only uses KMESH
!                        It has only been implemented for 3D
!                    4:  Combination of 1 and 3, hence possibly 2 meshes may be
!                        created
!                    5:  Combination of 2 and 3, hence possibly 2 meshes may be
!                        created
!                    6:  Create a submesh consisting of 4-point tetraeders only
!                        Create also a copy of the solution array containing
!                        number(1) unknowns in all points of the new grid
!     icompl         Multiplication factor for complex numbers
!                    Real: icompl=1,   Complex: icompl=2
!     ilevel         dummy
!     ipfcin         Starting address in array FCINT
!     ipoutp         Starting address of array OUTP
!     ipxcin         Starting address of in XCINT
!     iresul         Output array of length 5,1 if ichois=2 and otherwise of
!                    length 5,3
!                    The array is filled as follows:
!                    Pos .,1 corresponds to the "vertex" mesh array (ichois=2,3)
!                    Pos .,2 corresponds to the "triangle" mesh array
!                    (ichois=1,3)
!                    Pos .,3 corresponds to the solution array (ichois=1,3)
!                    Pos. 1,1  INDVER sequence number with respect to INFOR
!                                     for the "vertex" mesh array
!                    Pos. 2,1  NPELM  number of points per element for this mesh
!                    Pos. 3,1  NELEM  number of points per element for this mesh
!                    Pos. 4,1  NEWVER indicates if the mesh is new (1) or is
!                                     in fact the standard mesh (0)
!                    Pos. 5,1  -
!                    Pos. 1,2  INDTRI sequence number with respect to INFOR
!                                     for the "triangle" mesh array
!                    Pos. 2,2  NPELM  number of points per element for this mesh
!                    Pos. 3,2  NELEM  number of points per element for this mesh
!                    Pos. 4,2  NEWTRI indicates if the mesh is new (1) or is
!                                     in fact the standard mesh (0) or the mesh
!                                     created by the "vertex" mesh
!                    Pos. 5,2  NPHYS  Max. number of degrees of freedom
!                    Pos. 1,3  INDSOL sequence number with respect to INFOR
!                                     for the new solution array
!                    Pos. 2,3  NEWSOL indicates if the solution array is new
!                                     (1) or is in fact the standard solution
!                                     array (0)
!                    Pos. 3,3  INDPRF sequence number of KPROBF
!                    Pos. 4,3  INDPRH sequence number of KPROBH
!                    Pos. 5,3  INPPRP sequence number of KPROBP
!     kmeshclc       Local array kmeshc
!     kseg           number of points per segment
!     ncint          Number of interpolation points
!     nelemsav       Help parameter to save the old value of nelem
!     nelemt         Number of elements in the new mesh
!     nelgrpsav      Help parameter to save the old value of nelgrp
!     nomber         Array to store the numbers of the unknowns for TOPQU0
!     nreal          Number of interpolated values at the line of intersection
!     numseg         Number of segments on interpolation line
!     origx          Value of x-coordinate of origin (VALUES(1))
!     origy          Value of y-coordinate of origin (VALUES(2))
!     special        Indicates which special situation is present
!                    Possible values:
!                    0     standard
!                    1     solution vector is of type 116 and a coloured
!                          2d plot is required
!                    2     solution vector is of type 110 or 115 and a coloured
!                          2d plot is required, whereas there are no unknowns
!                          in the vertices
!     xcint          array of interpolated x- values
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DSCAL          Blas routine to scale a vector
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLMN         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     PRLIN1         Interpolate along a line in a triangular mesh (real case)
!     PRLIN2         Interpolate along a line in a triangular mesh
!                    (complex case)
!     SEPCRSOLINFO   Create a new vector in ks
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     TOPQU0         Subdivide mesh if necessary. Map unknowns into an organized
!                    array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       1   ICHOIS is incorrect
!     702   The mesh is not two-dimensional.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepintp2d' )
      if ( itime>0 ) call ersettime ( timebefore )

!     --- Check arrays KMESH, KPROB, ISOL and compute IPKPRB
!         Create submesh of triangles and copy solution

      ichoic = 2
      compl = ks(isolut)%compl==1
      nomber(1) = 1
      nomber(2) = number
      special = 0
      ilevel = 1
      nelgrpsav = nelgrp
      nelemsav = nelem

      call topqu0 ( ichoic, nomber, iskip, ihelp, iresul, &
                    compl, 0, special, ilevel )
      nelemt = iresul(3,2)
      if ( compl ) then
         icompl = 2
      else
         icompl = 1
      end if
      if ( ndim/=2 ) then
         call errint ( ndim, 1 )
         call errsub ( 702, 1, 0, 0 )
      end if
      if(ichois<1 .or. ichois>2) then
         call errint ( ichois, 1 )
         call errsub ( 1, 1, 0, 0 )
      end if
      if(ierror/=0) goto 1000
      iwarn = 0

!     --- Perform the actual cross-section and interpolation
!         Store the arrays kseg, xcint and fcint at the end of ibuffr

      ncint  = npoint * 4

      allocate ( kseg(6), xcint(ncint), fcint(2*ncint*icompl), stat = error )
      if ( error/=0 ) call eralloc ( error, 6+3*ncint, 'kseg' )
      if ( ierror/=0 ) go to 1000

      origx = values(1)
      origy = values(2)
      angle = values(3)

      if ( iresul(1,2)==0 ) then
         kmeshclc => kmeshc
      else
         kmeshclc => kmeshctmp
      end if  ! ( iresul(1,2)==0 )

      if ( compl ) then

!     --- Complex vector

         call prlin2 ( usolt, coor, kmeshclc, &
                       nelemt, npoint, angle, origx, origy, numseg, &
                       kseg, xcint, fcint, ncint )

      else

         call prlin1 ( usolt, coor, kmeshclc, &
                       nelemt, npoint, angle, origx, origy, numseg, &
                       kseg, xcint, fcint, ncint )

      end if
      if ( iwarn/=0 ) then

!     --- Something was wrong, make ioutvc equal to zero

         ioutvc = 0
         go to 1000

      end if

!     --- Fill information in array ioutvc
      nreal = 0
      do i = 1, numseg
         nreal = nreal + kseg(i)
      end do
      call sepcrsolinfo ( ioutvc, nreal*(1+icompl), numseg )
      isoltemp => ks(ioutvc)%isol
      ks(ioutvc)%typevec = 119
      ks(ioutvc)%compl = 0
      ks(ioutvc)%n = numseg
      ks(ioutvc)%ivec = numseg

!     --- Copy kseg into ibuffr from position ipoutp

      isoltemp(1:numseg) = kseg(1:numseg)

      usoltemp => ks(ioutvc)%sol
      ipoutp = 1

!     --- Copy x and f into ibuffr from position ipoutp

      ipxcin = 1
      ipfcin = 1
      do i = 1, numseg

         nreal = kseg(i)
         usoltemp(ipoutp:ipoutp+nreal-1) = xcint(ipxcin:ipxcin+nreal-1)
         ipoutp = ipoutp + nreal
         ipxcin = ipxcin + nreal
         usoltemp(ipoutp:ipoutp+nreal*icompl-1) = &
            fcint(ipfcin:ipfcin+nreal*icompl-1)
         if ( ichois==2 ) call dscal ( nreal*icompl, values(4), &
              usoltemp(ipoutp), 1 )
         ipoutp = ipoutp + nreal * icompl
         ipfcin = ipfcin + nreal * icompl

      end do

      deallocate ( kseg, xcint, fcint, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'kseg' )

1000  if ( iresul(2,3)/=0 ) call seprmsolinfo ( iresul(1,3) )
      if ( iresul(4,2)/=0 ) deallocate ( kmeshctmp )
      if ( iresul(4,1)/=0 ) deallocate ( kmeshcsub )

      kmeshc => km(iseqkm)%kmeshc
      nelgrp = nelgrpsav
      nelem = nelemsav
      inelem = nelem

      call erclmn ( 'sepintp2d', timebefore )

      end subroutine sepintp2d
