      subroutine sepmanmap1 ( jdegfd, vec1, vec2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-09-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy degree of freedom jdegfd from vec1 into vec2
!
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
!     map
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: jdegfd
      double precision, intent(in) :: vec1(*)
      double precision, intent(out) :: vec2(*)

!     jdegfd         i    degree of freedom
!     vec1           i    input vector
!     vec2           o    output vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, i1, i2, inunkp, jdegfd1, jdegfd2

!     i              Counting variable
!     i1             position in vec2
!     i2             position in vec1
!     inunkp         Number of degrees of freedom in point
!     jdegfd1        Degree of freedom corresponding to vec1
!     jdegfd2        Degree of freedom corresponding to vec2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     216   degree of freedom does not exist,
!           parameter idgfd1 or idgfd2 is too large
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmanmap1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmanmap1'
         call ertrace
         write(irefwr,1) 'jdegfd, nphyso, nphysi', jdegfd, nphyso, nphysi
         write(irefwr,1) 'indprpo, indprfo, indprfi, indprpi', &
                          indprpo, indprfo, indprfi, indprpi
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( jdegfd>nphyso ) then

!     --- jdegfd > nphyso, hence store in vec2 position 1

         jdegfd1 = 1
         if ( jdegfd<=nphysi ) then

!        --- jdegfd <= nphysi, hence use position jdegfd in vec1

            jdegfd2 = jdegfd

         else

!        --- jdegfd > nphysi, copying is not possible

            call errsub ( 216, 0, 0, 0 )
            go to 1000

         end if  ! ( jdegfd<=nphysi )

      else if ( jdegfd>=nphysi ) then

!     --- jdegfd > nphysi, hence use position 1 in vec1

         jdegfd1 = jdegfd
         jdegfd2 = 1

      end if  ! ( jdegfd>nphyso )

      if ( debug ) write(irefwr,1) 'jdegfd1, jdegfd2', jdegfd1, jdegfd2

      if ( indprpo/=0 ) then

!     --- indprpo/=0, kprobpo exists

         if ( indprpi/=0 ) then

!        --- indprpi#0, kprobpi exists

            do i = 1, npoint

               i1 = kprobpo(i,jdegfd1)
               if ( i1>0 ) then

!              --- position in ivec1 exists

                  i2 = kprobpi(i,jdegfd2)  ! positions in vec2
                  if ( i2>0 ) vec2(i1) = vec1(i2)

               end if  ! ( i1>0 )

            end do  ! i = 1, npoint

         else if ( indprfi/=0 ) then

!        --- indprfi#0, kprobfi exists

            do i = 1, npoint

               i1 = kprobpo(i,jdegfd1)
               if ( i1>0 ) then

!              --- position in ivec1 exists

                  i2 = kprobfi(i)
                  inunkp = kprobfi(i+1)-i2
                  if ( inunkp>=jdegfd2 ) vec2(i1) = vec1(i2+jdegfd2)

               end if  ! ( i1>0 )

            end do  ! i = 1, npoint

         else

!        --- indprfi=0, constant number of degrees of freedom

            do i = 1, npoint

               i1 = kprobpo(i,jdegfd1)
               if ( i1>0 ) then

!              --- position in ivec1 exists

                  i2 = (i-1)*nphysi
                  if ( nphysi>=jdegfd2 ) vec2(i1) = vec1(i2+jdegfd2)

               end if  ! ( i1>0 )

            end do  ! i = 1, npoint

         end if  ! ( indprpi/=0 )

      else if ( indprfo/=0 ) then

         print *,'indprfo, indprfi, indprpi/1 ', indprfo, indprfi, indprpi
         call instop

      else

!     --- indprpo=0 and inpfo=0

         if ( indprpi/=0 ) then

!        --- indprpi#0, kprobpi exists

            do i = 1, npoint

               i1 = nphyso*(i-1)+jdegfd1
               i2 = kprobpi(i,jdegfd2)  ! positions in vec2
               if ( i2>0 ) vec2(i1) = vec1(i2)

            end do  ! i = 1, npoint

         else if ( indprfi/=0 ) then

!        --- indprfi#0, kprobfi exists

            do i = 1, npoint

               i1 = nphyso*(i-1)+jdegfd1
               i2 = kprobfi(i)
               inunkp = kprobfi(i+1)-i2
               if ( inunkp>=jdegfd2 ) vec2(i1) = vec1(i2+jdegfd2)

            end do  ! i = 1, npoint

         else

!        --- indprfi=0, constant number of degrees of freedom

            do i = 1, npoint

               i1 = nphyso*(i-1)+jdegfd1
               i2 = (i-1)*nphysi
               if ( nphysi>=jdegfd2 ) vec2(i1) = vec1(i2+jdegfd2)

            end do  ! i = 1, npoint

         end if  ! ( indprpi/=0 )

      end if  ! ( indprpi/=0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmanmap1'

      end if  ! ( debug )
      call erclos ( 'sepmanmap1' )

      end subroutine sepmanmap1
