      subroutine seprenumber ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-01-2021
!
!   copyright (c) 2021-2021  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform renumbering of mesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     renumbering
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(0:*)

!     inpsep         i    See subroutine askprivec
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: istart, lenstr, istmet, ifirst
      integer, pointer :: meshsplt(:), levels(:)

!     ifirst         First node
!     istart         Start level
!     istmet         Indicates the method for computing the start for
!                    renumbering
!                    0:  default
!                    1:  user point
!                    2:  curve
!                    3:  surface
!     lenstr         Number of elements in istart
!     levels         Contains the levels in the mesh
!                    Level 1 consists of all nodes from 1 to levels(1)
!                    Level i consists of all nodes from levels(i-1) to
!                    levels(i)  (i>1)
!                    If the mesh is renumbered then
!                    Level 1 consists of all nodes from kmeshj(1) to
!                    kmeshj(levels(1))
!                    Level i consists of all nodes from kmeshj(levels(i-1))
!                    to kmeshj(levels(i))  (i>1)
!     meshsplt       In this array the largest node number of the various
!                    disconnected parts of the mesh are stored
!                    Length: number of parts (<npoint)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHTMCK         Renumber nodes using Cuthill-McKee
!     CHTSLO         Renumber nodes using Sloan
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPNESTEDDIS   Renumbering with nested dissection
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seprenumber' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seprenumber'
         call prinin ( inpsep, 2, 'inpsep' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkm = kmesh

      select case ( inpsep(1) )

      case(1)

!     --- renumber not

         if ( associated(km(iseqkm)%kmeshj) ) deallocate ( km(iseqkm)%kmeshj )
         km(iseqkm)%kmesh(24) = 0

      case(2)

!     --- Cuthill Mckee

         istart = 1
         lenstr = 1
         allocate ( meshsplt(npoint) )
         call chtmck ( istart, lenstr, meshsplt )
         deallocate ( meshsplt )

      case(3)

!     --- Slooan

         istart = 1
         lenstr = 1
         istmet = 0
         ifirst = 1
         allocate ( meshsplt(npoint), levels(npoint) )
         call chtslo ( levels, istart, lenstr, istmet, ifirst, meshsplt )
         deallocate ( meshsplt, levels )

      case(4)

!     --- nested dissection

         call sepnesteddis
         km(iseqkm)%kmeshj => km(iseqkm)%rennested
         km(iseqkm)%kmesh(24) = npoint

      end select  ! case ( inpsep(1) )

!     --- Raise istartmn

      istartmn = istartmn + 2

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kmeshj, npoint, 'kmeshj' )
         write(irefwr,*) 'End seprenumber'

      end if  ! ( debug )
      call erclos ( 'seprenumber' )

      end subroutine seprenumber
