      subroutine sepsymmnodes
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all nodes on symmetry axis and store in listsymm if any
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     mesh
!     2d
!     symmetric
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulesymm
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
     integer :: i
     integer, allocatable :: iwork1(:), iwork2(:)
     double precision, allocatable :: work1(:)

!     i              Counting variable
!     iwork1         integer work array used for ordering of nodes on
!                    symmetry axis
!     iwork2         integer work array for reordering
!     work1          work array to store the z-coordinates of nodes on
!                    the axis
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRSORT         Sort real array for increasing sequence
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iworksymm(1:npoint)
!     Mark all nodes with r-coordinate 0
!     Store information in sepmodulesymm
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsymmnodes' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsymmnodes'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iworksymm(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iworksymm' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iworksymm(1:npoint)

      iworksymm = 0

!     --- Mark all nodes with r-coordinate 0

      numsymm = 0
      do i = 1, npoint

         if ( abs(coor(1,i))<=0d0 ) then

!        --- r-coordinate is 0, hence symmetry point

            iworksymm(i) = i
            numsymm = numsymm+1

         end if  ! ( abs(coor(1,i))<=0d0 )

      end do  ! i = 1, npoint

      if ( debug ) call prinin ( iworksymm, npoint, 'iworksymm' )

      if ( numsymm>0 )then

!     --- numsymm>0, hence there are nodes on the symmetry axis

         allocate ( listsymm(numsymm), work1(numsymm), iwork1(numsymm), &
                    iwork2(numsymm), stat = error )
         if ( error/=0 ) call eralloc ( error, numsymm, 'listsymm' )
         if ( ierror/=0 ) go to 1000

         numsymm = 0
         do i = 1, npoint

            if ( iworksymm(i)>0 ) then

!           --- symmetry point

               numsymm = numsymm+1
               listsymm(numsymm) = i

            end if  ! ( iworksymm(i)==1 )

         end do  ! i = 1, npoint

         if ( debug ) call prinin ( listsymm, numsymm, 'listsymm/1' )

!        --- Sort nodes in increasing z order

         work1 = coor(2,listsymm)
         call prsort ( work1, iwork1, numsymm )
         iwork2 = listsymm
         listsymm(iwork1) = iwork2

         if ( debug ) then
            call prinin ( listsymm, numsymm, 'listsymm/2' )
            call prinin ( iworksymm, npoint, 'iworksymm' )
         end if  ! ( debug )

      end if  ! ( numsymm>0 )

!     --- Fill array nodesnew with node numbers of mirrored mesh

      if ( allocated(nodesnew) ) deallocate(nodesnew)
      allocate ( nodesnew(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'nodesnew' )
      if ( ierror/=0 ) go to 1000

      npointsymm = npoint  ! new npoint
      nodesnew = iworksymm
      do i = 1, npoint

         if ( iworksymm(i)==0 ) then

!        --- Node not on symmetry axis

            npointsymm = npointsymm+1
            nodesnew(i) = npointsymm

         end if  ! ( iworksymm(i)==0 )

      end do  ! i = 1, npoint

      deallocate ( iwork1, iwork2, work1, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork1' )
      if ( ierror/=0 ) go to 1000

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'numsymm', numsymm
         call prinin ( nodesnew, npoint, 'nodesnew' )
         write(irefwr,*) 'End sepsymmnodes'

      end if  ! ( debug )
      call erclos ( 'sepsymmnodes' )

      end subroutine sepsymmnodes
