      subroutine tocomptstforce ( iinsol, ivectr, iinbld, ivecfem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 25-11-2012 Long integers
!        version  1.0  date 21-03-2007
!
!   copyright (c) 2007-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute test force in outer boundary
!     The test force is defined as the force needed to make the
!     solution at the outer boundary equal to 0
!     This is typically used for friction models
! **********************************************************************
!
!                       KEYWORDS
!
!     test_force
!     friction
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinsol(*), ivectr, iinbld(*), ivecfem

!     iinbld         i    User input array for subroutine BUILD
!     iinsol         i    Contains information concerning the linear solver
!                         See the Programmers Guide for a description
!     ivecfem        i    Sequence number of FEM solution vector
!     ivectr         i    Sequence number of the solution vector with respect to
!                         array isolut
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqin, iseqtestforce, isoltmp

!                    they are not
!     iseqin         Sequence number of vector in isolut
!     iseqtestforce  Sequence number of yesy force in isolut
!     isoltmp       Help array to store the solution temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICLEARBNSOL  Set all values of the solution array equal to 0
!                    at the boundary
!     PRININ1        print 2d integer array
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPLOCTRN      Compute (back-)transformation in case of local transform
!     SEPMAVER       perform matrix vector multiplication
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     TO0089REAC     Compute the reaction forces in the linear system of
!                    equations
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Let the solution be divided into an inner part and a part
!     at the outer boundary (u_i and u_b)
!     Then the solution satisfies:
!     A_ii u_i  +  A_ib u_b = f_i
!     A_bi u_i  +  A_bb u_b = f_b
!
!     We set u_b = 0 in order to get the test force tf
!
!     tf_i = A_ii u_i = f_i - A_ib u_b
!     tf_b = A_bi u_i = f_b - A_bb u_b
!
!     The test force is computed by the matrix vector multiplication
!     of A and u_i extended with zeros
!
!     In case of a boundary element matrix (for example due to
!     a slip condition), the test force must be corrected
!     This is done in to0089
!
!     Copy solution vector to temporary vector
!     Replace the solution at the outer boundary by 0
!     Compute the test force by computing matrix times new solution
!     Remove temporary vector
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tocomptstforce' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tocomptstforce'
         call prinin1 ( isolut2d, 2, 5, 'isol' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Copy solution vector to temporary vector

      iseqin = iinsol(6)
      iseqtestforce = iinsol(17)
      if ( debug ) &
         write(irefwr,1) 'iseqin, iseqtestforce', iseqin, iseqtestforce

      isoltmp = 0
      call sepcopyvc ( isolut2d(iseqin), isoltmp )

!     --- Replace the solution at the outer boundary by 0

      call iniclearbnsol ( isoltmp )

!     --- A local transformation is applied to isoltmp to get this vector
!         into the local coordinate system
!         Since the outer boundary is set to zero, this operation usually
!         has no effect at all

      call seploctrn ( 1, isoltmp )

!     --- Compute the test force

      call sepmaver ( stiffmat, isoltmp, &
                      isolut2d(iseqtestforce), 2 )

!     --- The test force is in the local coordinate system and must
!         be transformed back to the standard sequence

      call seploctrn ( 2, isolut2d(iseqtestforce) )

!     --- Remove temporary vector

      call seprmsolinfo ( isoltmp )

!     --- Correct for part due to slip condition

      iinsol(10) = iseqtestforce
      iinsol(13) = 0
      iinsol(14) = 1
      call to0089reac ( iinsol, ivectr, iinbld, iseqtestforce, ivecfem )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End tocomptstforce'

      end if  ! ( debug )
      call erclos ( 'tocomptstforce' )

      end subroutine tocomptstforce
