      subroutine toconct1 ( inpconct, force, distance, nodes, nnodes, &
                            changed )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 27-09-2010 Make work arrays local
!        version  3.3  date 09-03-2007 Extra debug
!        version  3.2  date 20-08-2003 Extension with ccontact
!
!   copyright (c) 1997-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the nodes in the contact surface that make contact and
!     disable the nodes that have a negative force
! **********************************************************************
!
!                       KEYWORDS
!
!     contact
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecontact
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpconct(*), nodes(*), nnodes
      double precision distance(*), force(*)
      logical changed

!     changed    o    The contact points have been changed
!     distance   i    Contains the distance in the nodes of the contact surface
!     force      i    Contains the force component in the nodes of the contact
!                     surface
!     inpconct   i    In this array it is indicated how the contact surface
!                     must be computed
!                     Contents
!                     1: sequence_number of contact
!                     2: type of contact
!                        1: curves
!                        2: surfaces
!                     3: contact method
!                        1: contact is made if contact_distance<=0
!                     4: contact disable method
!                        1: contact is disabled if CONTACT_FORCE<=0
!                     5: numvec
!                     6: sequence number of contact surface or curve
!                     7: sequence number of vector defining contact distance
!                     8: sequence number of vector defining contact force
!                     9: - &
!     nnodes     i    Number of nodes in isolnw
!     nodes      i    Contains the node numbers that must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nconctct, inode, iseqlc, nocontct, istart
      integer, allocatable :: iwork(:), iwork1(:)

!     i              Counting variable
!     inode          Node sequence number
!     iseqlc         Sequence number of contact
!     istart         First position used at a part
!     iwork          Work array of length npoint that is filled only for the
!                    contact points
!                    Let the i^th contact point have node number nodp
!                    then iwork(nodp) = i
!     iwork1         Work array of length npoint that is filled only for the
!                    contact points
!                    Let the i^th contact point have node number nodp
!                    then
!                    iwork1(nodp) =-1 point did make contact, but has been
!                                     disabled
!                    iwork1(nodp) = 0 point makes no contact
!                    iwork1(nodp) = 1 point makes contact
!     nconctct       Number of points with contact
!     nocontct       Number of points that make no contact
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ2        Print an integer two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Contact points are defined depending on the contact method
!    If contact method = 1: distance<0 means contact
!
!    Contact points are removed from the set of contact points depending on the
!    disbale method
!    If contact disable method = 1: force<0 means no contact
!
!    Initialize iwork and iwork1
!    iseqlc := inpconct(1)
!    nconctct := kprobvp1(1,iseqlc)
!    if ( nconctct>0 ) then
!       disable points in contact surface according to method
!       Set iwork1 for disabled and contact points
!    For all points that make no contact yet
!       Set contact points in contact surface according to method
!    Refill kprobv if a change is present
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toconct1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toconct1'

      end if

      allocate ( iwork(npoint), iwork1(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Initialize iwork and iwork1

      do i = 1, nnodes
         iwork(nodes(i)) = i
         iwork1(i) = 0
      end do
      iseqlc = inpconct(1)
      nconctct = kprobvp1(iseqlc,1)
      istart   = kprobvp1(iseqlc,3)
      if ( debug ) write(irefwr,*) 'iseq, nconctct, istart ', &
                                    iseqlc, nconctct, istart

      if ( istart==0 ) then

!     --- istart = 0, hence this surface has not yet been filled
!         Compute istart and store in kprobvp1

         do i = 1, nsurfs
            if ( kprobvp1(i,3)>istart ) &
               istart = istart+kprobvp1(i,1)+kprobvp1(i,2)+kprobvp1(i,3)
         end do
         istart = max(istart,1)
         kprobvp1(iseqlc,3) = istart

      end if
      istart = istart-1
      changed  = .false.

      do i = 1, nconctct

!     --- nconctct>0
!         disable points in contact surface according to method
!         Set iwork1 for disabled and contact points

         inode = iwork(kprobvp2(istart+i))
         if ( force(inode)<0d0 ) then

!        --- force<0, disable point

            iwork1(inode) = -1
            changed = .true.
            nconctct = nconctct-1

            if ( debug ) write(irefwr,*) 'SUB CONTACT NODE: ', &
                nodes(inode), '  REACTION FORCE: ', force(inode)

         else

!        --- force>0, keep point

            iwork1(inode) = 1

         end if

      end do

!     --- For all points that make no contact yet
!         Set contact points in contact surface according to method

      do i = 1, nnodes

         if ( iwork1(i)==0 ) then

!        --- Node is not yet in contact surface

            if ( distance(i)<=0d0 ) then

!           --- distance<0

               nconctct = nconctct+1
               iwork1(i) = 1
               changed = .true.

               if ( debug ) write(irefwr,*) 'ADD CONTACT NODE: ', &
                  nodes(i), '  CONTACT DISTANCE: ', distance(i)

            end if

         end if

      end do

      if ( changed ) then

!     --- Some contact points have been changed, refill kprob part v

         nocontct = nconctct
         nconctct = 0
         do i = 1, nnodes

           if ( iwork1(i)>0 ) then

!          --- Point with contact found

               nconctct = nconctct+1
               kprobvp2(istart+nconctct) = nodes(i)

            else

!           --- Point without contact found

               nocontct = nocontct+1
               kprobvp2(istart+nocontct) = nodes(i)

            end if

         end do
         kprobvp1(iseqlc,1) = nconctct
         kprobvp1(iseqlc,2) = nocontct-nconctct

      else if ( kprobvp1(iseqlc,1)==0 ) then

!     --- There are no contact points in this surface

         kprobvp1(iseqlc,2) = nnodes
         do i = 1, nnodes
            kprobvp2(istart+i) = nodes(i)
         end do

      end if

      contact_changed(iseqlc) = changed
      contact_ncontct(iseqlc) = nconctct

      deallocate ( iwork, iwork1, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         if ( changed ) then
         call prinin2 ( kprobvp1, nsurfs, 3, 'kprobvp1' )
         call prinin ( kprobvp2, nconctct, 'contact points' )
         call prinin ( kprobvp2(1+nconctct), nnodes-nconctct, &
                       'no-contact points' )
         write(irefwr,*) 'NUMBER OF CONTACT POINTS = ', nconctct
         write(irefwr,*) 'NUMBER OF NO-CONTACT POINTS = ', nocontct
         end if  ! ( changed )
         write(irefwr,*) 'End toconct1'

      end if
      call erclos ( 'toconct1' )

      end
