      subroutine todiffnorms ( methodnl, icrit, isol, &
                               isolol, ieqnt, ivectr, &
                               isolit, irhsdlc, diffsl, nparm, icritres, &
                               ivecseq )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 03-01-2011 Extra debug
!        version  1.0  date 29-06-2005
!
!   copyright (c) 2005-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array diffsl with information of norms of differences between
!     succesive iterations in a non-linear iteration process
! **********************************************************************
!
!                       KEYWORDS
!
!     iteration
!     non_linear
!     norm
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer methodnl, icrit, isol(*), &
              isolol(*), ieqnt, ivectr, &
              isolit(*), irhsdlc(*), nparm, icritres, ivecseq
      double precision diffsl(nparm,*)

!     diffsl        i/o   In this array of size nparm x 7
!                         information about the convergence of the
!                         iteration process is stored for each coupled equation
!                         DIFFSL is filled as follows:
!                         diffsl(i,1)   maximum difference between present and
!                                       prior iteration, i.e. ||u(n)-u(n-1)||
!                                       or  || u(n) - u(n-1) || / || u(n) ||
!                         diffsl(i,2)   value of diffsl(i,1) in the prior
!                                       iteration, i.e. || u(n-1) - u(n-2) ||
!                         diffsl(i,3)   value of diffsl(i,2) in the prior
!                                       iteration, i.e. || u(n-2) - u(n-3) ||
!                         diffsl(i,4)   required accuracy
!                         diffsl(i,5)   ||residual||, i.e. ||A(n)u(n-1)-f(n)||
!                         diffsl(i,6)   ||residual|| in the prior iteration
!                         diffsl(i,7)   required residual accuracy
!                         The parameter i indicates the sequence number of the
!                         equation
!     icrit          i    Type of criterion to be used for the termination
!                         Possible values:
!                         0:  Absolute difference between two succeeding
!                             iterations
!                         1:  Relative difference between two succeeding
!                             iterations
!                         2:  Absolute value of right-hand side f
!                         3:  Absolute value of right-hand side f
!                             divided by solution
!                         4:  Absolute value of energy increment (u,f)
!                         5:  Absolute value of energy increment (u,f)
!                             divided by solution
!     icritres       i    Extra type of criterion based on residual.
!                         To reach convergence both the standard criterion as
!                         well as the extra criterion must be satisfied.
!                         Possible values:
!                         0: No residual criterion
!                         1: Absolute residual criterion
!                         2: Relative residual criterion, i.e.
!                            based on ||residual|| / ||solution||
!     ieqnt          i    Equation sequence number
!                         Array diffsl is filled for this specific equation
!     irhsdlc        i    Standard SEPRAN array, containing information of the
!                         right-hand side of the system of equation
!     isol           i    Standard SEPRAN array containing information of the
!                         solution arrays (length 5 x * )
!                         ISOL may refer to more than one solution array.
!                         In fact this subroutine allows for a combination of
!                         solution arrays and derivative arrays
!                         In this subroutine the following entries of the second
!                         index of isol are used depending on criteria and
!                         method:
!                         ieqnt+ivectr
!                         ieqnt+ivecseq
!     isolit         i    Corresponds to the iteration vector. In the iteration
!                         vector the solution of one iteration for one equation
!                         is stored
!     isolol         i    Same as isol but corresponds to the solution of the
!                         prior iteration
!     ivecseq        i    Sequence number of the total solution vector with
!                         respect to array isol
!     ivectr         i    Sequence number of the solution vector with respect to
!                         array isol
!     methodnl       i    Type of nonlinear iteration process:
!                         0: Standard method
!                         1: Newton
!                         2: incremental newton
!     nparm          i    Length of first index of array diffsl
!                         Contains maximum number of coupled equations
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinvec(6)
      double precision rinvec(2)

!     iinvec         User input array for subroutine MANVEC
!                    For a description see subroutine MANVEC
!     rinvec         Real user input array for subroutine MANVEC
!                    The contents depend on iinvec
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPMANVEC      Vector manipulation subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Store the difference of the present and privious iteration into
!     diffsl(ieqnt,1) depending on method and criterion
!     In case of relative criterion fill extra information
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'todiffnorms' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from todiffnorms'
         write(irefwr,1) 'methodnl, icrit', methodnl, icrit
         write(irefwr,1) 'ieqnt, ivectr', ieqnt, ivectr
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Compute norm(s) of difference and store in diffsl(.,1)

      iinvec(1) = 6
      iinvec(2) = 5
      iinvec(3) = 0
      iinvec(4) = 0
      iinvec(6) = 3
      if ( methodnl==0 ) then

!     --- Standard method, difference between 2 successive iterations

         iinvec(5) = 1
         call sepmanvec ( iinvec, rinvec, isol(ieqnt+ivectr), &
                          isolol(ieqnt), isol )

      else if ( icrit<=1 ) then

!     --- Newton method, standard criterion, norm of iteration vector

         iinvec(5) = 0
         call sepmanvec ( iinvec, rinvec, isolit,  isolit, isol )

      else if ( icrit<=3 ) then

!     --- Newton method, norm of right-hand side

         iinvec(5) = 0
         call sepmanvec ( iinvec, rinvec, irhsdlc,  irhsdlc, isol )

      else

!     --- Newton method, inner product iteration vctor and right-hand side

         iinvec(1) = 2
         iinvec(2) = 4
         call sepmanvec ( iinvec, rinvec, isolit, irhsdlc, isol )
         rinvec(1) = abs(rinvec(1))

      end if
      diffsl(ieqnt,1) = rinvec(1)
      if ( ierror/=0 ) go to 1000

      if ( mod(icrit,2)==1 .or. icritres==2 ) then

!     --- Relative criterion, put norm into diffsl(.,1)

         iinvec(5) = 0
         call sepmanvec ( iinvec, rinvec, isol(ieqnt+ivecseq), &
                          isol(ieqnt+ivecseq), isol )
         if ( mod(icrit,2)==1 ) then
            if ( rinvec(1)<=0d0 ) then
               diffsl(ieqnt,1) = diffsl(ieqnt,1)
            else
               diffsl(ieqnt,1) = diffsl(ieqnt,1)/rinvec(1)
            end if
         end if
         if ( icritres==1 ) then
            if ( rinvec(1)<=0d0 ) then
               diffsl(ieqnt,5) = diffsl(ieqnt,5)
            else
               diffsl(ieqnt,5) = diffsl(ieqnt,5)/rinvec(1)
            end if
         end if
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'diffsl(1,1)', diffsl(1,1)
         write(irefwr,*) 'End todiffnorms'

      end if
      call erclos ( 'todiffnorms' )
      end subroutine todiffnorms
