      subroutine checkholes ( iwork, meshsplt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 08-11-2007 correction loop
!        version  1.0  date 03-04-1999
!
!   copyright (c) 1999-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Find and print all user points, curves, surfaces and volumes that
!    have not been connected to the first part of the mesh
!    This subroutine is only called if an incorrect mesh consisting of
!    disconnected parts has been found
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     check
!     error_manipulation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), meshsplt(*)

!     iwork          i    integer work array of length npoint, output of chtmc6
!                         iwork(i)= -1, means that node i is connected to the
!                         first point, iwork(i)=0, means not connected
!     meshsplt       i    In this array the largest node number of the various
!                         disconnected parts of the mesh are stored
!                         Length: number of parts (<npoint)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical first, found
      character *8 text
      integer icount, icurnr, ielgrp, ifelem, ifirst, ikelmc, ilast, &
              inpelm, istar3, isurnr, ivolnr, i

!     first          If true it concerns the first print of a message
!     found          If true already a possibility has been found
!     i              Counting variable
!     icount         number of nodes that are connected to point 1
!     icurnr         Curve sequence number
!     ielgrp         element group number
!     ifelem         First element number
!     ifirst         First curve (surface) number
!     ikelmc         position in kmesh part c
!     ilast          Last curve (surface) number
!     inpelm         number of points per element
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     isurnr         Surface sequence number
!     ivolnr         Volume sequence number
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     initializations
!     Check the user points
!     Check the curves
!     Check the surfaces
!     Check the quantities with highest dimensionality
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- initializations

      icount = meshsplt(nsplit)

!     --- Check the user points

      first = .true.
      do i = 1, nuspnt

         if ( kmeshl(i)>0 .and. kmeshl(i)<=icount ) then

!        --- User point should have been renumbered

            if ( iwork(kmeshl(i))==0 ) then

!           --- user point i has not been renumbered

               if ( first ) write(irefwr,100)
100            format ( ' The following user points are not connected', &
                        ' to the first part of the mesh ' )
               write(irefwr,110) i
110            format( i6 )
               first = .false.

            end if

         end if

      end do

!     --- Check the curves

      first = .true.
      if ( ncurvs>0 ) then
         istar3 = kmeshm(2)-1
         ifirst = istar3+1
      end if
      do icurnr = 1, ncurvs

         ilast = istar3+kmeshm(5+icurnr)-1
         do i = ifirst, ilast

            if ( kmeshm(i)<=0 ) go to 200
            if ( kmeshm(i)<=icount ) then

!           --- Point in curve should have been renumbered

               if ( iwork(kmeshm(i))==0 ) then

!              --- Point in curve has not been renumbered

                  if ( first ) write(irefwr,150)
150               format ( ' The following curves are not connected', &
                           ' to the first part of the mesh ' )
                  write(irefwr,110) icurnr
                  first = .false.
                  go to 200

               end if

            end if

         end do

200      ifirst = ilast+1

      end do

      if ( ndim==3 ) then

!     --- Check the surfaces

         first = .true.
         istar3 = 2*nsurfs
         ifirst = istar3+1
         do isurnr = 1, nsurfs

            ilast = ifirst+kmeshn(isurnr)*kmeshn(isurnr+nsurfs)-1
            do i = ifirst, ilast

               if ( kmeshn(i)<=0 ) go to 300
               if ( kmeshn(i)<=icount ) then

!              --- Point in surface should have been renumbered

                  if ( iwork(kmeshn(i))==0 ) then

!                 --- Point in surface has not been renumbered

                     if ( first ) write(irefwr,250)
250                  format ( ' The following surfaces are not connected', &
                              ' to the first part of the mesh ' )
                     write(irefwr,110) isurnr
                     first = .false.
                     go to 300

                  end if

               end if

            end do

300         ifirst = ilast+1

         end do

      end if  ! ( ndim==3 )

!     --- Finally the quantities with highest dimensionality

      first = .true.
      found = .false.
      ifirst = 1
      if ( nvolms>0 ) then
         text = 'volumes'
      else if ( nsurfs>0 ) then
         text = 'surfaces'
         nvolms = nsurfs
      else
         text = 'curves'
         nvolms = ncurvs
      end if
      ifelem = kmesht(1,1)
      nelem  = 0
      ikelmc = 0
      ivolnr = 1

!     --- Loop over element groups

      do ielgrp = 1, nelgrp

         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm
         nelem = nelem+inelem
         if ( ifelem<nelem ) then

!        --- The first element of kmesht smaller than nelem
!            so element with highest dimensionality found

            if ( kmesht(2,ivolnr)<nelem ) then

!           --- a new volume has been detected

               ivolnr = ivolnr+1
               found = .false.

            end if
            if ( .not. found ) then

!           --- found = false
!               Not yet an incorrect element found for this volume

               do i = ikelmc+1, ikelmc + inelem*inpelm

                  if ( kmeshc(i)<=icount ) then

!                 --- Point in volume should have been renumbered

                     if ( iwork(kmeshc(i))==0 ) then

!                    --- Point in volume has not been renumbered

                        if ( first ) write(irefwr,350) text
350                     format ( ' The following ',a,' are not', &
                                 ' connected to the first part of', &
                                 ' the mesh ' )
                        write(irefwr,110) ivolnr
                        first = .false.
                        found = .true.
                        go to 400

                     end if

                  end if

               end do

            end if

400      end if
         ikelmc = ikelmc + inelem*inpelm

      end do
      end
