      subroutine msh030linesegm ( rinput, iinput, iinp, nelm, rincor, &
                                  np, icurnr, istartcur, curstr, &
                                  irelcurves, istartrel )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-12-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute number of elements along a set of line segments defined
!     by cline
! **********************************************************************
!
!                       KEYWORDS
!
!     create_point
!     curve
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinp, iinput(*), icurnr
      integer, intent(out) :: nelm, np
      integer, intent(inout) :: istartcur, irelcurves(*), istartrel
      double precision, intent(in) :: rinput(*), rincor(*)
      double precision, intent(inout) :: curstr(*)

!     curstr        i/o   Contains real information of the curves
!                         See subroutine mshcrtcurv
!     icurnr         i    Curve sequence number
!     iinp           i    last position used in array iinput
!     iinput         i    array containing the user input concerning the
!                         curve from position iinp
!     irelcurves    i/o   Contains integer information of the curves
!                         See subroutine mshcrtcurv
!     istartcur     i/o   next free position in curstr
!     istartrel     i/o   next free position in irelcurves
!     nelm           o    number of elements along curve
!     np             o    Defines whether the number of points is odd (2) or
!                         even (3)
!     rincor         i    Part of array rinput refering to coarseness
!     rinput         i    array containing on the first nuspnt*ndim positions
!                         coordinates, on position nuspnt+1 the unit edge length
!                         and on the next nuspnt positions the defined
!                         coarseness of the user points
!     sleng          o    length of the curve
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: numuserp, nuspnt, ndim, isegment, nelmloc, j1, j2, &
                 ncurvs
      double precision :: slengloc, x1(3), x2(3), edge1, edge2, edgemn, &
                          edge, sleng

!     edge           Unit length defined for coarseness
!     edge1          Length of first edge
!     edge2          Length of last edge
!     edgemn         Mean length of edges
!     isegment       Loop variable for line segments
!     j1             Index of first point
!     j2             Index of second point
!     ncurvs         Number of curves in mesh
!     ndim           Dimension of the space.
!     nelmloc        Number of elements along line segment
!     numuserp       Number of user points along line
!     nuspnt         Number of user points
!     slengloc       Lenght of line segment
!     x1             coordinates of first point of line segment
!     x2             coordinates of last point of line segment
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH030GEN1     Compute edge1/2 and edgemn for one line segment
!                    corresponding to cline
!     MSH030LINE     determine number of elements to be created along
!                    line with given coarseness
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh030linesegm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh030linesegm'
         write(irefwr,1) 'iinp', iinp
         write(irefwr,1) 'istartcur, istartrel', istartcur, istartrel
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ndim = iinput(1)
      nuspnt = iinput(2)
      ncurvs = iinput(3)

      numuserp = iinput(iinp+3)  ! number of user points
      edge = rincor(1) ! unit length
      sleng = 0d0
      nelm = 0

      if ( debug ) &
         write(irefwr,1) 'ndim, nuspnt, numuserp', &
                          ndim, nuspnt, numuserp

!     --- Loop over the segments

      irelcurves(istartrel) = 0
      do isegment = 2, numuserp

         j1 = iinput(iinp+2+isegment)  ! first user point
         j2 = iinput(iinp+3+isegment)  ! next user point

!        --- Fill x1, x2 and compute edge1, edge2 and edgemn

         call msh030gen1 ( j1, j2, nuspnt, rinput, ndim, rincor, &
                           edge, edge1, edge2, edgemn, icurnr, &
                           x1, x2 )

         if ( debug ) then
            write(irefwr,1) 'isegment, j1, j2', isegment, j1, j2
            write(irefwr,2) 'x1', x1(1:ndim)
            write(irefwr,2) 'x2', x2(1:ndim)
            write(irefwr,2) 'edge1, edge2', edge1, edge2
         end if  ! ( debug )

!        --- Compute local length of line segment

         call msh030line ( iinput, iinp, np, slengloc, x1, x2, ndim, &
                           icurnr )

         sleng = sleng+slengloc

         if (abs(edge1/edge2-1d0)>=epsmac) then
            nelmloc = nint(slengloc/(edge1-edge2)*log(edge1/edge2))
         else
            nelmloc = nint(slengloc/edgemn)
         end if

         curstr(istartcur+isegment-1) = slengloc
         irelcurves(istartrel+isegment-1) = nelmloc

         nelm = nelm+nelmloc

         if ( debug ) then
            write(irefwr,1) 'nelm, nelmloc', nelm, nelmloc
            write(irefwr,2) 'sleng, slengloc', sleng, slengloc
         end if  ! ( debug )

      end do  ! isegment = 2, numuserp

      irelcurves(icurnr) = istartrel
      irelcurves(icurnr+ncurvs) = istartcur
      curstr(istartcur) = sleng
      istartcur = istartcur+numuserp
      istartrel = istartrel+numuserp

      np = iinput(iinp+4+numuserp)

1000  call erclos ( 'msh030linesegm' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'np, nelm, numuserp', np, nelm, numuserp
         call prinrl ( curstr, istartcur-1, 'curstr' )
         write(irefwr,*) 'End msh030linesegm'

      end if  ! ( debug )

      end subroutine msh030linesegm
