      subroutine msh031 ( icurnr, ncurvs, iinput, iinp, curves, rinput, &
                          nelm, ishape, nuspnt, n, ndim, icurve, curstr, &
                          tau, isubd, rinpt1, curvefacts, irelcurves, &
                          istartrel )
! ======================================================================
!
!        programmer    Niek Praagman
!        version 13.2  date 10-02-2015 Extension with ccircle
!        version 13.1  date 08-12-2014 Allow lines with more points
!        version 13.0  date 07-11-2009 Remove some parameters
!
!   copyright (c) 1984-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    fill array curves, in case of a line and user points with
!    given coarseness. Check all lines and give a warning message if
!    coarseness changes too quick (>maxratio) along a line.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     curve
!     point_distribution
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision curves(*), rinput(*), curstr(*), tau(*), &
                       rinpt1(*), curvefacts(*)
      integer icurnr, ncurvs, iinput(*), iinp, nelm, ishape, nuspnt, n, &
              ndim, icurve, isubd, irelcurves(*), istartrel

!     curstr        i/o   Contains real information of the curves
!                         See subroutine mshcrtcurv
!     curvefacts     o    Array containing the factors t along the curves
!     curves         o    array that must be filled with the coordinates
!                         of the curves
!     icurnr         i    Curve sequence number
!     icurve         i    type of curve
!     iinp           i    starting address in iinput
!     iinput         i    standard input arrays for subroutine
!                         mesh , it is supposed that the first five
!                         positions have been deleted
!     irelcurves    i/o   Contains integer information of the curves
!                         See subroutine mshcrtcurv
!     ishape         i    indicator
!                         1  :  linear elements
!                         2  :  quadratic elements
!                         3  :  cubic elements
!     istartrel     i/o   next free position in irelcurves
!     isubd          i    Defines the subdivision of the points along element
!                         boundaries
!                         0:   equidistant
!                         1:   Gauss-Legendre
!     n              o    number of points in curve
!     ncurvs         i    number of curves
!     ndim           i    dimension of space
!     nelm           i    number of elements on curve
!     nuspnt         i    number of user points
!     rinpt1         i    Part of general input array with respect to mesh
!                         generation
!                         The first position must refer to alpha
!     rinput         i    standard input arrays for subroutine
!                         mesh , it is supposed that the first five
!                         positions have been deleted
!     tau                 work array to store the relative arc lengths of the
!                         elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision cunit, sleng, dx, maxratio, work(ishape)
      integer istartcur, i, j1, j2, numusp

!     cunit          global coarseness value
!     dx             difference x-coordinates
!     i              loop variable
!     istartcur      starting position of curves in inpsep
!     j1             First point of curve in KMESH part m
!     j2             Last point of curve in KMESH part m
!     maxratio       help variable to store max allowed coarseness ratio
!     numusp         number of user points
!     sleng          carthesian length of curve
!     work           work array of length ishape
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELLGLP         compute the Legendre Gauss-Lobatto points
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH005         Fill array curves in case of straight line
!     MSH006         Fill array curves in case of circle segment
!     MSH031CIRCSEGM Fill array tau for a circle with more than ndim user points
!     MSH031LINESEGM Fill array tau for a line with more than 2 user points
!     MSH032         Fill array curves in case of parameter function
!     MSHC01         Fill array curves with splines
!     MSHC06         Determine tau for cspline
!     MSHCURCCIRC    ?
!     MSHCURCLINE    Fill array curves for a line with more than 2 user points
!     MSHCURPROF     fill array curves, in case of a profile
!     MSHELLARC      fill part of array curves for an ellips segment
!     MSHFRAMECUR    Fill array curves in case of framecurve
!     MSHLINE1SEGM   Fill array tau and compute sleng for a line with 1 segment
!     MSHTFRAMECUR   determine tau for framecurve
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Fill array WORK with relative lengths along the curve dependent on
!    the value of ishape either equidistant or Gauss-Legendre points
!    Fill array tau dependent on type of curve
!    Make difference between one or more segments
!    Fill array curves
!       Distinguish between
!       -  straight line
!       -  circle segment
!       -  parameter function
!       -  spline
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh031' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh031'
         write(irefwr,1) 'icurnr, ncurvs, iinp, nelm', &
                          icurnr, ncurvs, iinp, nelm
         write(irefwr,1)  'ndim, icurve, isubd', &
                           ndim, icurve, isubd
         call prinin ( irelcurves, istartrel, 'irelcurves' )
  1      format ( a, 1x, (10i6) )

      end if

      istartcur = max(1,irelcurves(ncurvs+icurnr))

      if ( debug ) write(irefwr,1) 'istartcur', istartcur

!     --- First fill array WORK with relative lengths along the sides
!         if ishape>1

      if ( ishape>1 ) then

!     --- ishape>1

         if ( isubd==0 ) then

!        --- isubd = 0, equidistant division

            dx = 1d0 / ishape
            do i = 2, ishape
               work(i) = dx * (i-1)
            end do

         else

!        --- isubd = 1, Gauss-Legendre

            call ellglp ( work, ishape+1 )
            work(2:ishape) = 0.5d0 * ( work(2:ishape) + 1d0 )

         end if

      end if

!     --- Find maxratio

      maxratio = rinput( ndim * nuspnt + 2 + nuspnt )

!     --- Fill array tau

      numusp = iinput(iinp-1)  ! number of user points

      if ( debug ) write(irefwr,1) 'numusp', numusp

      if ( icurve==5 .and. numusp>2 ) then

!     --- Line with more than one segment

         iinp = iinp-4  ! correction for this subroutine
         call msh031linesegm ( rinput, iinput, iinp, nelm, ishape, &
                               icurnr, work, maxratio, tau, n, &
                               irelcurves )
         iinp = iinp+4  ! restore iinp

      else if ( icurve==24 ) then

!     --- circle with more than one segment

         iinp = iinp-4  ! correction for this subroutine
         call msh031circsegm ( rinput, iinput, iinp, nelm, ishape, &
                               icurnr, work, maxratio, tau, n, &
                               irelcurves )
         iinp = iinp+4  ! restore iinp

      else if ( icurve==6 .or. icurve==9 .or. icurve==16 .or. &
                icurve==23 .or. nelm==1 .and. ishape==1 .or. &
                icurve==5 ) then

!     --- Line has only one segment

         if ( icurve==16 ) then
            j1 = iinput(iinp+1)
            j2 = iinput(iinp+2)
         else
            j1 = iinput(iinp)
            j2 = iinput(iinp+1)
         end if
         call mshline1segm ( icurnr, maxratio, rinput, nelm, ishape, &
                             nuspnt, n, ndim, tau, work, j1, j2 )
         if ( icurve==9 .or. icurve==16 ) sleng = curstr(istartcur)

      else if ( icurve==12 ) then

         call mshc06 ( iinput(iinp-4), rinput, nuspnt, ndim, work, tau, &
                       ishape, nelm )
         n = ishape * nelm + 1

      else if ( icurve==21 ) then

!     --- icurve = 21: framecurve

         cunit = rinput(ndim*nuspnt + 1)
         call mshtframecur ( icurnr, cunit, work, tau, ishape, nelm )

!        --- Compute number of nodes on curve:

         n = ishape * nelm + 1

      end if
      if ( debug ) call prinin ( irelcurves, istartrel, 'irelcurves' )

!     --- fill array curves

      numusp = iinput(iinp-1)
      if ( nelm==1 .and. ishape==1 .and. icurve==12 .or. &
           numusp==2 .and. icurve==5 ) then

!     --- straight line

         call msh005 ( ndim, curves, iinput(iinp), rinput, tau, n, &
                       icurnr )
         iinp = iinp + 3

      else if ( icurve==5 ) then

!     --- composite straight line

         call mshcurcline ( ndim, curves, iinput(iinp), rinput, tau, &
                            icurnr, numusp, irelcurves )
         iinp = iinp+1+numusp

      else if ( icurve==6 ) then

!     --- circle segment

         if ( debug ) &
            write(irefwr,1) 'istartcur, icurnr, icurve', &
                             istartcur, icurnr, icurve

         call msh006 ( .true., ndim, curves, iinput(iinp), rinput, &
                       tau, n, nuspnt, curstr(istartcur), icurnr, &
                       icurve )
         iinp = iinp + 4

      else if ( icurve==9 ) then

!     --- parameter function

         call msh032 ( .true., ndim, curves, iinput(iinp), rinput, &
                       tau, n, nuspnt, sleng, icurnr, ncurvs, 1, 0, &
                       curvefacts )
         iinp = iinp + 3

      else if ( icurve==12 ) then

!     --- spline

         call mshc01 ( iinput(iinp), rinput, curves, ndim, n, tau, &
                       rinpt1, icurnr )
         iinp = iinp + 3 + iinput(iinp+1)

      else if ( icurve==16 ) then

!     --- cprofile

         call mshcurprof ( .true., curves, iinput(iinp), rinput, &
                           tau, n, nuspnt, sleng, icurnr, ncurvs, 1, 0, &
                           curvefacts )
         iinp = iinp + 4

      else if ( icurve==21 ) then

!     --- frame curve

         irelcurves(icurnr) = istartrel
         call mshframecur ( iinput(iinp), rinput, curves, ndim, n, tau, &
                            rinpt1, cunit, ishape, icurnr, &
                            irelcurves(istartrel) )
         istartrel = istartrel+iinput(iinp+1)
         iinp = iinp + 3 + iinput(iinp+1)

      else if ( icurve==23 ) then

!     --- ellips segment

         if ( debug ) &
            write(irefwr,1) 'istartcur, icurnr, icurve', &
                             istartcur, icurnr, icurve

         call mshellarc ( .true., ndim, curves, iinput(iinp), rinput, &
                          tau, n, nuspnt, curstr(istartcur) )
         iinp = iinp + 4

      else if ( icurve==24 ) then

!     --- circle with extra points

         if ( debug ) &
            write(irefwr,1) 'istartcur, icurnr, icurve', &
                             istartcur, icurnr, icurve

         call mshcurccirc ( ndim, curves, iinput(iinp), rinput, tau, &
                            icurnr, numusp, irelcurves, &
                            curstr(istartcur) )
         iinp = iinp+1+numusp

      end if

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'n', n
         call prinrl ( curves, 2*n, 'curves' )
         write(irefwr,*) 'End msh031'

      end if
      call erclos ( 'msh031' )

      end
