      subroutine msh041 ( iinput, ikelmc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  7.1  date 28-09-2012 Jump in case of error
!        version  7.0  date 18-06-2012 Make iwork local
!        version  6.3  date 06-11-2008 Jump in case of error
!
!   copyright (c) 1985-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array kmeshc for connected elements
!     The checking of the input is performed in subroutine MSH038
!     That subroutine also computes the length of KMESHC
!     As a consequence a change in MSH041 must always result in a change
!     in MSH038
! **********************************************************************
!
!                       KEYWORDS
!
!     connect
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iinput(*), ikelmc

!     iinput         i    integer input array containing information of the
!                         connected elements
!     ikelmc        i/o   last position used in kmesh part c
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipoint, icurvs, isurfs, iinp, i, ielgrp, &
              ileft, iright, n, istar1, j, iwork(npoint+ncurvs), &
              inpelm, ilefst, irigst, iexclu, k, &
              istoreleft(npoint), istoreright(npoint)
      logical left, right

!     i              Counting variable
!     icurvs         array containing the number of points in the curves
!                    accumulated
!     ielgrp         element group number
!     iexclu         Indicates if begin and end points of curves must be
!                    excluded or not. Possible values in case of curves:
!                    0:  Do not exclude one of these points
!                    1:  Exclude begin point
!                    2:  Exclude end point
!                    3:  Exclude begin and end point
!                    And in case of surfaces:
!                    0:  Do not exclude boundaries
!                    1:  Exclude all external boundaries
!     iinp           last position used in array iinput
!     ilefst         Starting point of "left" surface in KMESH part n
!     ileft          Left curve in special element
!     inpelm         number of points per element
!     ipoint
!     iright         Right curve in special element
!     irigst         Starting point of "right" surface in KMESH part n
!     istar1         Starting position of curves
!     istoreleft     Contains all nodes on left-hand curve
!     istoreright    Contains all nodes on right-hand curve
!     isurfs
!     iwork          Work array of length npoint+ncurvs
!                    This work array is only used in the case of surfaces
!                    to be connected. The first npoint positions are used
!                    to indicate if a nodal point has already been
!                    stored (>0) or not (0) or is excluded (<0)
!                    In this way double elements are avoided
!                    The positions npoint+1 to npoint+ncurvs are used to
!                    store the outer boundary. Inner curves are deleted
!                    from kmeshs
!     j              Counting variable
!     k              Counting variable
!     left           Indicates if left boudary is filled (false) or not (true)
!     n              Number of elements created by mesh connect or twice this
!                    number in the case of surfaces
!     right          Indicates if right boudary is filled (false) or not (true)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHCONNECTCURV Store nodes along left-hand side into istoreleft
!                    and along right-hand side into istoreright
!     MSHCONNECTSURF Compute first part for connection of surfaces
!     MSHCONNECTWORK Fill array iwork for connection of points in surface
!                    elements
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     SEPPRINKMESHC  print kmesh part c
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1243   Surface for mesh connect has not been filled
!    1244   Curve for mesh connect has not been filled
!    1245   User point  for mesh connect has not been filled
!    1442   excluded nodal point not in surface
!    1807   Point on curve is linked with itself
!    1808   Point on surface is linked with itself
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh041' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh041'
         write(irefwr,1) 'npoint', npoint
         call sepprinkmeshc
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      ipoint = iinput(1)
      icurvs = iinput(2)
      isurfs = iinput(3)
      iinp = 4
      if ( debug ) write(irefwr,1) 'ipoint, icurvs, isurfs', &
                                    ipoint, icurvs, isurfs

!     --- Elements connecting points

      do i = 1, ipoint
         ielgrp = iinput(iinp)
         ileft = iinput(iinp+1)
         iright = iinput(iinp+2)
         inpelm = 2
         kmeshg(ielgrp) = kmeshg(ielgrp)+1
         if ( kmeshl(ileft)<=0 ) then
            call errint ( ileft, 1 )
            call errint ( iright, 2 )
            call errint ( ileft, 3 )
            call errint ( ielgrp, 4 )
            call errsub ( 1245, 4, 0, 0 )
         end if
         if ( kmeshl(iright)<=0 ) then
            call errint ( ileft, 1 )
            call errint ( iright, 2 )
            call errint ( iright, 3 )
            call errint ( ielgrp, 4 )
            call errsub ( 1245, 4, 0, 0 )
         end if
         kmeshc(ikelmc+1) = kmeshl(ileft)
         kmeshc(ikelmc+2) = kmeshl(iright)
         ikelmc = ikelmc+2
         iinp = iinp+3
      end do
      nelem = nelem+ipoint

!     --- connection elements defined on curves

      istar1 = 0

!     --- Loop over various curves given

      do i = 1, icurvs

!     --- Extract parameters from iinput

         ielgrp = iinput(iinp)
         iexclu = iinput(iinp+1)/100
         ishape = iinput(iinp+1)-100*iexclu
         ileft = iinput(iinp+2)
         iright = iinput(iinp+3)

!        --- Store nodes along left-hand side into istoreleft
!            and along right-hand side into istoreright

         call mshconnectcurv ( ileft, iright, istoreleft, istoreright, &
                               n, ielgrp, i, iexclu, istar1 )
         if ( ierror/=0 ) go to 1000

!        --- Raise parts in kmesh

         kmeshg(ielgrp) = kmeshg(ielgrp)+n
         nelem = nelem+n
         if ( debug ) then

!        --- Debug information

            write(irefwr,1) 'ielgrp, n, ishape, ileft, iright, istar1', &
                             ielgrp, n, ishape, ileft, iright, istar1
            call prinin ( istoreleft, n, 'istoreleft' )
            call prinin ( istoreright, n, 'istoreright' )

         end if

         if ( ishape==0 ) then

!        --- ishape = 0, points are connected

            left = .false.
            right = .false.
            do j = 1, n
               kmeshc(ikelmc+1) = istoreleft(istar1+j)
               kmeshc(ikelmc+2) = istoreright(istar1+j)
               if ( kmeshc(ikelmc+1)==kmeshc(ikelmc+2) ) then

!              --- Nodes are connected with itself

                  call errint ( j, 1 )
                  call errint ( ileft, 2 )
                  call errint ( iright, 3 )
                  call errsub ( 1807, 3, 0, 0 )

               end if
               if ( kmeshc(ikelmc+1)<=0 ) left = .true.
               if ( kmeshc(ikelmc+2)<=0 ) right = .true.
               ikelmc = ikelmc+2

            end do
            if ( left ) then
               call errint ( ileft, 1 )
               call errint ( iright, 2 )
               call errint ( ileft, 3 )
               call errint ( ielgrp, 4 )
               call errsub ( 1244, 4, 0, 0 )
            end if
            if ( right ) then
               call errint ( ileft, 1 )
               call errint ( iright, 2 )
               call errint ( iright, 3 )
               call errint ( ielgrp, 4 )
               call errsub ( 1244, 4, 0, 0 )
            end if
            if ( ierror/=0 ) go to 1000

            if ( debug ) then

!           --- Debug information

               call prinin1 ( kmeshc(ikelmc-2*n+1), n, 2, 'elements' )

            end if
            inpelm = 2

         else

!        --- ishape>0, elements are connected

            inpelm = ishape + 1
            do j = 1, n
               do k = 1, inpelm
                  kmeshc(ikelmc+k) = istoreleft(istar1+k)
                  kmeshc(ikelmc+k+inpelm) = istoreright(istar1+k)
               end do
               ikelmc = ikelmc+2*inpelm
               istar1 = istar1+ishape

            end do
            inpelm = 2*inpelm
         end if
         iinp = iinp+4

      end do

!     --- Compute surface connections

      do i = 1, isurfs

         ielgrp = iinput(iinp)
         ishape = iinput(iinp+1)
         iexclu = iinput(iinp+4)
         ileft  = iinput(iinp+2)
         iright = iinput(iinp+3)
         if ( debug ) then
            write(irefwr,1) 'i, ielgrp, ishape, iexclu, ileft, iright', &
                             i, ielgrp, ishape, iexclu, ileft, iright
         end if

!        --- Compute parameters (first part)

         call mshconnectsurf ( ileft, iright, inpelm, ilefst, irigst )
         if ( ierror/=0 ) go to 1000

         if ( ishape==0 ) then

!        --- ishape = 0, points are connected
!            Clear array iwork
!            Array IWORK is used as work array
!            It is used to avoid double elements
!            If IWORK(i) = 0, node i has not been found in the left surface yet
!            If IWORK(i)>0, the node has been used and elements containing
!            this node as starting point are skipped

            call mshconnectwork ( iwork, ileft, iexclu, iinput(iinp+5) )

!           --- Loop over all elements in the surface

            n = 0
            left = .false.
            right = .false.
            do j = 1, inpelm*inelem
               ipoint = kmeshn(ilefst+j)
               if ( iwork(ipoint)<0 ) then

!              --- iwork(ipoint)<0, the node is excluded

                  iwork(ipoint) = 1

               else if ( iwork(ipoint)==0 ) then

!              --- iwork(ipoint) = 0, the node has not been used yet

                  iwork(ipoint) = 1
                  kmeshc(ikelmc+n+1) = ipoint
                  kmeshc(ikelmc+n+2) = kmeshn(irigst+j)
                  if ( kmeshc(ikelmc+n+1)==kmeshc(ikelmc+n+2) ) then
                     call errint ( j, 1 )
                     call errint ( ileft, 2 )
                     call errint ( iright, 3 )
                     call errsub ( 1808, 3, 0, 0 )
                  end if
                  if ( kmeshc(ikelmc+n+1)<=0 ) left = .true.
                  if ( kmeshc(ikelmc+n+2)<=0 ) right = .true.
                  n = n + 2

               end if

            end do

            do j = 1, npoint

!           --- check if surface contains all excluded nodal points

               if ( iwork(j)<0 ) then
                  call errint ( j, 1 )
                  call errint ( ileft, 2 )
                  call errsub ( 1442, 2, 0, 0 )
               end if

            end do

!           --- Update KMESH part G

            if ( left ) then
               call errint ( ileft, 1 )
               call errint ( iright, 2 )
               call errint ( ileft, 3 )
               call errint ( ielgrp, 4 )
               call errsub ( 1243, 4, 0, 0 )
            end if
            if ( right ) then
               call errint ( ileft, 1 )
               call errint ( iright, 2 )
               call errint ( iright, 3 )
               call errint ( ielgrp, 4 )
               call errsub ( 1243, 4, 0, 0 )
            end if
            inpelm = 2
            if ( ierror/=0 ) go to 1000

         else

!        --- ishape>0, elements are connected

            n = 0
            do j = 1, inelem
               do k = 1, inpelm
                  kmeshc(ikelmc+n+k) = kmeshn(ilefst+k)
                  kmeshc(ikelmc+n+k+inpelm) = kmeshn(irigst+k)
               end do
               n      = n     +2*inpelm
               ilefst = ilefst+inpelm
               irigst = irigst+inpelm
            end do
            inpelm = 2*inpelm
         end if
         kmeshg(ielgrp) = kmeshg(ielgrp)+n/inpelm
         nelem = nelem+n/inpelm
         ikelmc = ikelmc+n
         iinp = iinp+5+max(0,iexclu)
      end do
      Topgrp(:)%inelem = kmeshg

1000  if ( debug ) then

!     --- Debug information

         call sepprinkmeshc
         write(irefwr,*) 'End msh041'

      end if  ! ( debug )
      call erclos ( 'msh041' )

      end subroutine msh041
