      subroutine msh043 ( iinput, iinpsf, iposi )
! ======================================================================
!
!        programmer    Guus Segal
!        version 11.1  date 22-05-2014 Extension with shape 22
!        version 11.0  date 24-08-2010 Removal of work arrays
!        version 10.0  date 02-07-2008 Extra parameter ibuffr
!
!   copyright (c) 1984-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     initialize some parameters for mshvlm
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinpsf(*), iposi

!     iinpsf         i    array iinput with respect to the surfaces
!     iinput         i    integer input array containing the information of
!                         volumes
!     iposi          i    next free position of iinput
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  ielgeg(8), itetra(8), iirand(8), itrian(8), inplms(8), &
               ndefvl, i, ipossf, j, ishap, ishaps, meshtp, irand, &
               inpelmbot, inpelmsur

!     i              Counting variable
!     ielgeg         array with number of nodes for each element shape
!     iirand         Contains the number of points minus one at an edge of
!                    elements
!     inpelmbot      Number of nodes per element of bottom surface and
!                    top surface of pipe
!     inpelmsur      Number of nodes per element of pipe surface
!     inplms         Number of points in surface element
!     ipossf         ?
!     irand          number of points on the side of an element - 1
!     ishap          Shape number of elements on surface
!     ishaps         Shape number of elements on first surface
!     itetra         Contains the number of elements a hexahedron must be
!                    subdivided into for elements of shape 11 to 18
!                    (hexahedra:1, tetrahedra:6)
!     itrian         Contains the number of triangles or quadrilaterals the
!                    outer boundary can be subdivided into
!     j              Counting variable
!     meshtp         Type number of elements in the mesh or element group
!     ndefvl         Number of surfaces in volume
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH043CASE1    initialize some parameters for mshvlm
!                    Case 1: subroutine brick
!     MSH043CASE4    initialize some parameters for mshvlm
!                    Case 4: subroutine pipe or channel
!     MSH043CASE5    initialize some parameters for mshvlm
!                    Case 5-10: subroutines general, rotate, translate
!     MSH049         Compute starting address of specific surface in array
!                    IINPUT
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     138   Shape numbers of elements along the bottom surface and the top
!           surface of the pipe are not the same
!     292   surface number has incorrect value
!     298   code for generation of volumes incorrect
!     597   Surface elements at boundary of volume do not correspond to volume
!           elements
!     598   Surface elements at the boundary of volume have different shape
!     800   Shape number not available
!    1020   Digit in orientation incorrect
!    1169   Shape number of volume element incorrect
!    2376   Shape number of elements on top and bottom surface does not
!           correspond to prisms used in volume
!    2377   Shape number of pipe surface elements does not correspond to
!           prisms used in volume
!    3189   option not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ielgeg /4, 10, 8, 27, 14, 15, 9, 5/, &
           itetra /6,  6, 1,  1,  6,  6, 1, 6/, &
           iirand /1,  2, 1,  2,  2,  2, 1, 1/, &
           itrian /6, 12, 4,  9, 14, 14, 4, 6/, &
           inplms /3,  6, 4,  9,  7,  7, 4, 3/
! ======================================================================
!
      call eropen ( 'msh043' )
      debug = .false.

      if ( debug ) then

!     --- Check = true, print some information

         write(irefwr,*) 'Debug information in msh043'
         write(irefwr,1) 'ndim, nsurfs, npoint, nelem, ivolnr, nbcrd', &
                          ndim, nsurfs, npointlc, nelemlc, ivolnr, nbcrd
  1      format ( a, 1x, (10i6) )

      end if

      nmes = 0
      mmes = 0
      lmes = 0
      nx = 0
      ny = 0
      nz = 0
      nc = 0
      npointlc = 0
      nbndpt = 0

      ivol = iinput(1)
      if ( ivol==0 ) go to 1000

      if ( ivol<0 .or. ivol>11 ) then
         call errint ( ivol, 1 )
         call errsub ( 298, 1, 0, 0 )
      end if

      ishape = iinput(2)
      if ( ishape<0 .or. ishape>=1 .and. ishape<=10 .or. &
           ishape>18 .and. ishape<=20 .or. &
           ivol==4 .and. ishape>22 .or. &
           ivol/=4 .and. ishape>20 ) then

!     --- shape number out of range

         call errint ( ishape, 1 )
         call errint ( ivolnr, 2 )
         call errsub ( 1169, 2, 0, 0)

      end if
      ndefvl = iinput(3)
      leniwk = 0

      if ( debug ) write(irefwr,1) 'ivol, ishape',ivol, ishape

!     --- Test surfaces

      do i = 1, ndefvl
         isurnr = iinput(3+i)
         if ( ivol==1 ) then

!        --- ivol = 1, check orientation

            iorien = (isurnr-1)/nsurfs
            isurnr = mod(isurnr-1,nsurfs)+1
            if ( iorien<0 .or. iorien>7 ) then

!           --- Error 1020:  digit out of range

               call errint ( i, 1 )
               call errint ( iorien+1, 2 )
               call errint ( 1, 3 )
               call errint ( 8, 1 )
               call errsub ( 1020, 4, 0, 0 )
               call errcrd

            end if

         end if  ! ( ivol==1 )

         if ( isurnr<=0 .or. isurnr>nsurfs ) then
            call errint ( isurnr, 1 )
            call errint ( nsurfs, 2 )
            call errint ( ivolnr, 3 )
            call errsub ( 292, 3, 0, 0)
         end if

!        --- Test corresponding shapes

         ipossf = 1
         do j = 1, isurnr-1
            call msh049(iinpsf, ipossf, iinpsf(ipossf))
         end do
         ishap = iinpsf(ipossf+1)
         if ( ivol/=4 ) then

!        --- All volumes except pipe, the surface elements must all be the same

            if ( i==1 ) then

!           --- first surface, store shape number ishaps

               ishaps = ishap

            else

!           --- next surface, check if the same surface shape number has been
!               used

               if ( ishap/=ishaps ) then

!              --- shape number along surfaces are not the same

                  call errint ( ishap,  1 )
                  call errint ( ishaps, 2 )
                  call errint ( ivolnr, 3 )
                  call errsub ( 598, 3, 0, 0)

               end if

            end if

         else

!        --- ivol = 4, pipe surface

            if ( i==1 ) then

!           --- bottom surface, store shape number in ishaps

               ishaps = ishap

            else if ( i==2 ) then

!           --- top surface, must have the same shape number

               if ( ishap/=ishaps ) then

!              --- shape number along top and bottom surfaces are not the same

                  call errint ( ishap,  1 )
                  call errint ( ishaps, 2 )
                  call errint ( ivolnr, 3 )
                  call errsub ( 138, 3, 0, 0)

               end if

            end if

         end if

      end do

!     --- Test if surface shape number matches the volumes shape number

      if ( ishape<16 .and. ishaps/=ishape-8 .or. &
           ishape==16 .and. ishaps/=7 .or. &
           ishape==17 .and. ishaps/=5 .or. &
           ishape==18 .and. ishaps/=3 ) then

!     --- shape number of surface does not belong to volume elements

         call errint ( ishaps, 1 )
         call errint ( ishape-8, 2 )
         call errint ( ivolnr, 3 )
         call errsub ( 597, 3, 0, 0)

      end if
      if ( ishape>20 ) then

!     --- Volume elements are prisms, at this moment only allowed for pipes
!         Test surface shapes

         if ( ishape==21 .and. ishaps/=3 .or. &
              ishape==22 .and. ishaps/=4 ) then

!        --- shape numbers of bottom and top do not match volume elements

            call errint ( ishape, 1 )
            call errint ( ishaps, 2 )
            call errsub ( 2376, 2, 0, 0 )

         end if
         if ( ishape==21 .and. ishap/=5 .or. ishape==22 .and.ishap/=6 ) then

!        --- shape numbers of pipe surface do not match volume elements

            call errint ( ishape, 1 )
            call errint ( ishap, 2 )
            call errsub ( 2377, 2, 0, 0 )

         end if

      end if
      if ( ierror/=0 ) go to 1000

!     --- Compute inpelm, irand, mtetra, mtrian

      if ( ishape<=20 ) then

!     --- tetrahedrons or hexahedrons

         meshtp = ishape-10
         inpelm = ielgeg(meshtp)
         irand  = iirand(meshtp)
         mtetra = itetra(meshtp)
         mtrian = itrian(meshtp)
         inplmt = inplms(meshtp)
         inpelmbot = inplmt
         inpelmsur = inplmt

      else if ( ishape==21 ) then

!     --- linear prisms

         inpelmbot = 3
         inpelmsur = 4
         inpelm = 6

      else if ( ishape==22 ) then

!     --- quadratic prisms

         inpelmbot = 6
         inpelmsur = 9
         inpelm = 18

      else

!     --- shape number not implemented

         call errint ( ishape, 1 )
         call errsub ( 800, 1, 0, 0 )
         go to 1000

      end if

!     --- Compute some quantities that depend on the type of volume generator

      select case ( ivol )

         case(1)

!        --- ivol = 1     subroutine mshrd3  (brick)

            call msh043case1 ( iinput, ndefvl, nmes, mmes, lmes, nsurfs, &
                               kmeshs, ivolnr, irand, nx, ny, nz, npointlc, &
                               ishape, nelemlc, mtetra, nbcrd, nbndpt, &
                               lenwrk, mtrian )

         case(2,3)

!        --- ivol = 2, 3     subroutine user, input: do nothing

         case(4,9)

!        --- ivol = 4     subroutine MSHPIP  (pipe)
!            ivol = 9     subroutine MSH    (channel)

            call msh043case4 ( iinput, nc, &
                               ivolnr, nx, ny, nz, npointlc, &
                               nelemlc, nbcrd, nbndpt, &
                               lenwrk, inpelmbot, &
                               inpelmsur )

         case ( 5:8,10,11 )

!        --- ivol = 5, 11 subroutine MSHG3d  (General3d)
!            ivol = 6     subroutine MSHTRA  (Translate)
!            ivol = 7     subroutine MSHROT  (Rotate)
!            ivol = 8     subroutine MSHRFL  (Reflect)
!            ivol = 10    subroutine MSHG3d  (General3d, with internal points)

            call msh043case5 ( iinput, iposi )

         case default

!        --- Value of ivol not yet implemented

            call errint ( ivol, 1 )
            call errchr ( 'ivol', 1 )
            call errsub ( 3189, 1, 0, 1 )
            go to 1000

      end select  ! case ( ivol )

      if ( ishape>=16 .and. ishape<=18 ) npointlc = npointlc + nelemlc

      if ( debug ) &
         write(irefwr,1) 'npointlc, nelemlc, npomax, nbnmax, nbndpt, lenwrk', &
                          npointlc, nelemlc, npomax, nbnmax, nbndpt, lenwrk
      npomax = max ( npomax, npointlc )
      nbnmax = max ( nbnmax, nbndpt )
      if ( ivol==5 .or. ivol>=10 ) then
         lnbcor = 0
      else
         lnbcor = ndim*nbcrd
      end if
      lnkbnd = nbndpt
      lenwrk = max( npomax*ndim, lenwrk )+2
      lenwrk = max(lenwrk,783)
      lnkbnd = lnkbnd+nextrapnts+lenextracrvs ! extended with
                           ! extra user points and curves

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'lnbcor, lnkbnd, lenwrk', lnbcor, lnkbnd, lenwrk
         write(irefwr,*) 'End msh043'

      end if  ! ( debug )
      call erclos ( 'msh043' )

      end subroutine msh043
