      subroutine msh066 ( icurvsl, ncurvsl, inpada, iinput, &
                          rinput, ispinp, tau, rinada, icurvar )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.6  date 12-01-2009 New call to mshcursplin
!        version  1.5  date 20-09-2002 Extension for translate/rotate
!        version  1.4  date 06-09-1997 Correction negative curves
!
!   copyright (c) 1996-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Redistribute the nodes at the curves that must be adapted depending
!     on the values of the distribution parameter.
! **********************************************************************
!
!                       KEYWORDS
!
!     adapt_boundary
!     change_boundary
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncurvsl, icurvsl(ncurvsl), inpada(*), iinput(*), &
              ispinp(*), icurvar(*)
      double precision rinput(*), tau(*), rinada(*)

!     icurvar    i    Array of length ncurvs that contains information of
!                     the curves that must be changed. icurvar(i) mus be
!                     filled as follows:
!                     0  curve i is not yet adapted
!                     1  curve i has been adapted and is a single curve
!     icurvsl    i    Array of length ncurvs that is filled with all
!                     stand alone curves that must be adapted in the
!                     correct sequence and provided with a sign
!     iinput     i    integer input array; result of a preceding call of
!                     subroutine mesh
!     inpada     i    User input array for subroutine ADAPBOUN
!     ispinp          Array of length 2 + maximal number of points along
!                     one curve to be used as work array
!     ncurvsl    i    Number of curves stored in icurvsl
!     rinada     i    User input array for subroutine ADAPBOUN
!     rinput     i    real input array; result of subroutine msh047
!     tau             work array of length at most length curves / 2 to store
!                     relative arc lengths along curves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iredist, iposc, irnp, iinp, icurve, n
      integer, allocatable :: iworkcp(:)

!     i          Counting variable
!     icurve     Type of curve
!     iinp       last position used in array iinput
!     iposc      Position in array curves
!     iredist    Defines type of redistribution of nodes
!                Possible values:
!                0: no redistribution
!                1: redistribution of nodes, the number of nodes is kept
!                2: redistribution of nodes, the number of nodes may be changed
!     irnp       position in array rinput
!     iworkcp    integer work array of length ncurvs
!                iworkcp(i) = 1 means curve i is in the boundary
!                iworkcp(i) = 0 means curve i is not in the boundary
!     n          number of points on a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     MSH046         Raise iinp as function of icurve
!     MSHCURSPLIN    Redistribute the coordinates along a curve without changing
!                    the number of nodes along that curve
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     iredist := inpada(15)
!     ncurvs := kmesh(11)
!     Clear array iworkcp
!     For i := 1 (1) ncurvsl do
!        iworkcp(icurvsl(i)) := 1
!     For icurnr := 1 (1) ncurvs do
!        Get icurve from iinput
!        if ( iworkcp(icurnr)=1 ) then
!           if ( iredist=1 ) then
!              Recompute the distribution along the curve and store in array tau
!              Compute a spline through the points on the curve based on
!              the array tau and store the result in curves
!           else if ( iredist=2 ) then
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      iredist = mod(inpada(15),100)

      allocate ( iworkcp(ncurvs), stat = error )
      if ( error/=0 ) call eralloc ( error, ncurvs, 'iworkcp' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iworkcp and mark curves at free boundary

      iworkcp = 0
      do i = 1, ncurvsl
         iworkcp(abs(icurvsl(i))) = 1
      end do

!     --- loop over all curves

      jcoars = iinput(11)
      iposc = 1
      irnp = (ndim+jcoars)*nuspnt+jcoars
      iinp = 16
      do icurnr = 1, ncurvs
         icurve = iinput(iinp)
         if ( iworkcp(icurnr)==1 ) then

!        --- Curve belongs to free boundary, redistribute

            kmeshz(icurnr) = -1
            if ( iredist==1 ) then

!           --- Redistribution of nodes without changing the number of nodes

               call mshcursplin ( iinput, iinp, irnp, icurnr, rinput, &
                                  icurve, ndim, nuspnt, jcoars, curveusrp, &
                                  tau, ispinp, iposc, curvesmsh, &
                                  rinada(5) )

            else

!           --- Redistribution of nodes with changing the number of nodes
!               mark free boundary

                iinput(iinp) = -icurve

            end if

         else if ( kmeshz(icurnr)/=-1 .and. icurvar(icurnr)==1 ) then

            kmeshz(icurnr) = 3

         end if

!        --- raise parameters

         call msh046 ( iinput, iinp, icurve )
         n = kmeshm(5+icurnr)-kmeshm(4+icurnr)
         iposc = iposc+n

      end do

      deallocate ( iworkcp, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iworkcp' )

1000  end
