      subroutine msh073 ( coorol, coornw, icurnr, ipoint, icurvs, &
                          curves )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 06-01-2009 Fortran 90
!        version  1.1  date 22-09-1997 Call to eropen/erclos
!        version  1.0  date 23-08-1996
!
!   copyright (c) 1996-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Correct end points of free boundary
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     free_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icurnr, icurvs(*), ipoint
      double precision coorol(2,*), coornw(2,*), curves(2,*)

!     coornw    i/o   Array containing the new co-ordinates along the boundary
!     coorol     i    Array containing the old co-ordinates along the boundary
!     curves     i    array containing the coordinates of the curves
!     icurnr     i    Curve sequence number
!     icurvs     i    array containing the number of points in the curves
!                     accumulated
!     ipoint     i    Point at boundary that must be adapted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision xp(2), dx1, dx2, alpha
      integer istart, iend, idirec, ih, idist, ifound, i

!     alpha          Multiplication factor for parameter along element
!                    If between 0 and 1 the projection is in the element
!     dx1            distance between point and first point along boundary
!     dx2            distance between point and last point along boundary
!     i              Counting variable
!     idirec         Defines in which direction the curves must be considered
!                    idirec = 1, from low to high, idirec = -1: reversed order
!     idist          If 2 the shortest distance has been found
!     iend           Last point of boundary
!     ifound         Nodal point found
!     ih             Help variable to store a constant temporarily
!     istart         First point of boundary
!     xp             co-ordinates of projection
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     INIPROJ   Compute projection of point onto line
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find initial and end points in array curves
!     Find the direction we have to take
!     Find node along curve that is the closest to new point
!     Compute projection
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh073' )

      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh073'
         write(irefwr,1) 'icurnr, ipoint', icurnr, ipoint
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Find initial and end points in array curves

      if ( icurnr==1 ) then
         istart = 1
      else
         istart = 1 + icurvs(icurnr-1)
      end if
      iend = icurvs(icurnr)
      if ( debug ) write(irefwr,1) 'istart, iend', istart, iend

!     --- Find the direction we have to take

      dx1 = (coorol(1,ipoint)-curves(1,istart))**2 + &
            (coorol(2,ipoint)-curves(2,istart))**2
      dx2 = (coorol(1,ipoint)-curves(1,iend))**2 + &
            (coorol(2,ipoint)-curves(2,iend))**2
      if ( debug ) write(irefwr,2) 'dx1, dx2', dx1, dx2
      if ( dx1<sqreps ) then
         idirec = 1
      else if ( dx2<sqreps ) then
         idirec = -1
         ih = iend
         iend = istart
         istart = ih
      else if ( dx1<=dx2 ) then
         idirec = 1
      else
         idirec = -1
         ih = iend
         iend = istart
         istart = ih
      end if

!     --- Find node along curve that is the closest to new point

      dx1 = (coornw(1,ipoint)-curves(1,istart))**2 + &
            (coornw(2,ipoint)-curves(2,istart))**2
      idist = 0
      ifound = istart
      do i = istart+idirec, iend-idirec, idirec
         dx2 = (coornw(1,ipoint)-curves(1,i))**2 + &
               (coornw(2,ipoint)-curves(2,i))**2
         if ( dx2>dx1 ) then
            idist = idist+1
            if ( idist>=2 ) go to 100
            if ( idist==1 ) ifound = i-idirec
         else
            idist = 0
            dx1 = dx2
            ifound = i
         end if

      end do

100   if ( ifound==istart ) then

!     --- Projection in first element

         call iniproj ( coornw(1,ipoint), curves(1,ifound), &
                        curves(1,ifound+idirec), xp, alpha )

      else

!     --- Projection in intermediate element

         call iniproj ( coornw(1,ipoint), curves(1,ifound-idirec), &
                        curves(1,ifound), xp, alpha )
         if ( alpha>1d0 ) &
            call iniproj ( coornw(1,ipoint), curves(1,ifound), &
                           curves(1,ifound+idirec), xp, alpha )

      end if
      coornw(1,ipoint) = xp(1)
      coornw(2,ipoint) = xp(2)

1000  call erclos ( 'msh073' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh073'

      end if  ! ( debug )

      end
