      subroutine msh475( coor, inode, niedge, nisurf, factor )
! =======================================================================
!
!
!         programmer   niek praagman
!         version 1.2  date  06-12-1995 Use isoparametric transformation
!                                       quadratic and Coons correction
!         version 1.1  date  17-11-1995 Use Coons formula
!         version 1.0  date  07-04-1995
!
!
!
!   copyright (c) 1995 - 1995  "ingenieursbureau sepra"
!   permission to copy or distribute this software or documentation
!   in hard or soft copy granted only by written license obtained
!   from "ingenieursbureau sepra".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system (e.g. , in memory, disk or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Subroutine to place coordinates of internal spectral point(s) in
!     coor for a quadrilateral element while the boundaries have
!     already been augmented with niedge nodes
!
! ***********************************************************************
!
!     KEYWORDS
!
!     mesh_generation
!     quadrilateral
!     spectral
!     coons
! ***********************************************************************
!
!     INPUT / OUTPUT PARAMETERS
!
      implicit none

      integer inode(9), niedge, nisurf
      double precision coor( 3, * ), factor(0:*)

!     coor      i,o  coordinates array
!     factor     i   array containing the distribution of the nodes
!     inode      i   node array quadrilateral
!     niedge     i   number of spectral nodes on line element
!     nisurf     i   number of spectral nodes to be placed in surface element
!
! ***********************************************************************
!
!     COMMON BLOCKS
!
! ***********************************************************************
!
!     LOCAL PARAMETERS

      integer i, j, idim, npt, jnode(9), inp(9), inh(9), &
              ih, ieven, jpl, jph, ikn
      double precision eta, ski, xh, yh, zh, x1(9), x2(9), x3(9), &
                       xn(3), dx1(3), dx2(3), dx3(3), dx4(3)

!     dx1      differences for line 1
!     dx2      differences for line 2
!     dx3      differences for line 3
!     dx4      differences for line 4
!     eta      local coefficient for coordinate computation
!     i        loop variable
!     idim     loop variable
!     ieven    indicator for even or odd points
!     ih       helpvariable
!     ikn      node number
!     inh      helparray for nodes in (even) case
!     inp      helparray for nodes
!     j        loop variable
!     jnode    array with the reordered nodes of the quadrilateral
!     jph      position parameter
!     jpl      position parameter
!     npt      node number of internal point to be placed
!     ski      local coefficient for coordinate computation
!     x1       helparray to store the x-coordinates
!     x2       helparray to store the y-coordinates
!     x3       helparray to store the z-coordinates
!     xh       helpvariable for x-coordinate
!     xn       coordinates of "new" node
!     yh       helpvariable for y-coordinate
!     zh       helpvariable for z-coordinate
!
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
!     INSTOP    Stop execution of SEPRAN programm
!     MSH491    Determine the correct ordering of the nodes
!     MSHG22    Determine the coordinates of a nodal point using
!               the quadratic isoparametric transformation
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
! ***********************************************************************
!
!     METHOD
!
!     PSEUDO CODE
!
!     Determine the quadratic isoparametric transformation
!
!     Run through all interior points and place the coordinates in
!     accordance with the transformation
!
!     Make Coons correction
! =======================================================================

!     Check first the situation

      if ( nisurf.ne.niedge * niedge ) then

         write(*,*) ' Not yet implemented possibility in MSH475 '
         write(*,*) ' Sidepoints = ',niedge
         write(*,*) ' Facepoints = ',nisurf
         write(*,*) ' Please inform SEPRA that you want this pos-'
         write(*,*) ' sibility to be included '
         write(*,*) ' execution terminated '

         call instop

      end if

      npt   = inode(9)

!     Determine the order of the nodes

      call msh491( inode, jnode )

!     Run through all internal points and adjust the coordinates
!     utilizing an isoparametric projection for the corners and
!     the midpoints

      ih = 2 * int( niedge / 2 )

      if ( ih.eq.niedge ) then

         ieven = 2

      else

         ieven = 1

      end if

      if ( ieven.eq.1 ) then

!        Number of sidepoints is odd, simple situation

         ih = int( niedge / 2 ) + 1

         inp(1) = jnode(1)
         inp(2) = jnode(5) + ih - 1
         inp(3) = jnode(2)
         inp(4) = jnode(6) + ih - 1
         inp(5) = jnode(3)
         inp(6) = jnode(7) + ih - 1
         inp(7) = jnode(4)
         inp(8) = jnode(8) + ih - 1

!        Now the coordinates of the basis points can be computed

         do 110 i = 1 , 8

             jpl   =  inp(i)

             x1(i) = coor( 1, jpl )
             x2(i) = coor( 2, jpl )
             x3(i) = coor( 3, jpl )

110      continue

      else

!        Number of sidepoints is even, difficult situation

         ih = int( niedge / 2 )

         inp(1) = jnode(1)
         inh(1) = jnode(1)
         inp(2) = jnode(5) + ih - 1
         inh(2) = jnode(5) + ih
         inp(3) = jnode(2)
         inh(3) = jnode(2)
         inp(4) = jnode(6) + ih - 1
         inh(4) = jnode(6) + ih
         inp(5) = jnode(3)
         inh(5) = jnode(3)
         inp(6) = jnode(7) + ih - 1
         inh(6) = jnode(7) + ih
         inp(7) = jnode(4)
         inh(7) = jnode(4)
         inp(8) = jnode(8) + ih - 1
         inh(8) = jnode(8) + ih

!        Now the coordinates of the basis points can be computed

         do 120 i = 1 , 8

             jpl   =  inp(i)
             jph   =  inh(i)

             x1(i) = (coor( 1, jpl ) + coor( 1, jph )) / 2d0
             x2(i) = (coor( 2, jpl ) + coor( 2, jph )) / 2d0
             x3(i) = (coor( 3, jpl ) + coor( 3, jph )) / 2d0

120      continue

      end if

!     Finally the centroid of the quadrilateral, which is not
!     given a priori, is needed

      x1(9) = 0d0
      x2(9) = 0d0
      x3(9) = 0d0

      do 150 i = 1 , 4

          j = 2 * i - 1

          x1(9) = x1(9) + ( -x1(j) + 2 * x1(j+1) ) / 4d0
          x2(9) = x2(9) + ( -x2(j) + 2 * x2(j+1) ) / 4d0
          x3(9) = x3(9) + ( -x3(j) + 2 * x3(j+1) ) / 4d0

150   continue

!     Next compute the new internal points

      do 230 j = 1 , niedge

          eta = factor(j)

          do 220 i = 1 , niedge

              ski = factor(i)

!             Determine displacements in order to stretch, &
!             compute the corrections

!             Point (i,1) i.e. (ski,-1d0)

              ikn = jnode(5) + i - 1

              xh = coor( 1, ikn )
              yh = coor( 2, ikn )
              zh = coor( 3, ikn )

              call mshg22( ski, -1d0, x1, x2, x3, 3, xn )

              dx1(1) = xn(1) - xh
              dx1(2) = xn(2) - yh
              dx1(3) = xn(3) - zh

!             Point (niedge+2,j) i.e. (1,eta)

              if ( jnode(2).lt.jnode(3) ) then

                   ikn = jnode(6) + j - 1

              else

                   ikn = jnode(6) + niedge - j

              end if

              xh = coor( 1, ikn )
              yh = coor( 2, ikn )
              zh = coor( 3, ikn )

              call mshg22( 1d0, eta, x1, x2, x3, 3, xn )

              dx2(1) = xn(1) - xh
              dx2(2) = xn(2) - yh
              dx2(3) = xn(3) - zh

!             Point (i, niedge+2) i.e. (ski,1d0)

              if ( jnode(4).lt.jnode(3) ) then

                   ikn = jnode(7) + i - 1

              else

                   ikn = jnode(7) + niedge - i

              end if

              xh = coor( 1, ikn )
              yh = coor( 2, ikn )
              zh = coor( 3, ikn )

              call mshg22( ski, 1d0, x1, x2, x3, 3, xn )

              dx3(1) = xn(1) - xh
              dx3(2) = xn(2) - yh
              dx3(3) = xn(3) - zh

!             Point (1,j) i.e. (-1d0,eta)

              ikn = jnode(8) + j - 1

              xh = coor( 1, ikn )
              yh = coor( 2, ikn )
              zh = coor( 3, ikn )

              call mshg22( -1d0, eta, x1, x2, x3, 3, xn )

              dx4(1) = xn(1) - xh
              dx4(2) = xn(2) - yh
              dx4(3) = xn(3) - zh

!             Determine coordinates

              call mshg22( ski, eta, x1, x2, x3, 3, xn )

!             Place new determined coordinates with corrections in
!             array coor

              do 210 idim = 1 , 3

                 coor( idim, npt ) = xn(idim) - &
                        ( 1d0 - eta ) * dx1(idim) / 2d0 - &
                        ( 1d0 + ski ) * dx2(idim) / 2d0 - &
                        ( 1d0 + eta ) * dx3(idim) / 2d0 - &
                        ( 1d0 - ski ) * dx4(idim) / 2d0

210           continue

              npt = npt + 1

220       continue

230   continue

      end
