      subroutine mshc01 ( iinput, coor, curves, ndim, n, tau, &
                          rinput, icurnr )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  5.0  date 12-01-2009 Remove array work
!        version  4.0  date 02-02-1997 Extra parameter icurnr
!        version  3.1  date 18-10-1996 Do not change nodes if user points
!                                      are nearly equal to nodes
!
!   copyright (c) 1989-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Driving subroutine for Splines generation
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     spline
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), ndim, n, icurnr
      double precision coor(ndim,*), curves(ndim,*), tau(*), &
                       rinput(*)

!     coor     i/o   standard coordinate array
!     curves    o    curve array
!     icurnr    i    Curve sequence number
!     iinput    i    array containing the basis nodal point numbers
!     n         i    number of points to be created
!     ndim      i    Euclidian dimension
!     rinput    i    Part of general input array with respect to mesh generation
!                    The first position must refer to alpha
!     tau       i    distribution of points array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ndef, ipsolv, iparc, ipamat, iprhs1, iprhs2, iprhs3, &
              itype, i, j
      double precision alpha, curvelen, diff
      double precision, allocatable :: work(:)

!     alpha          spline parameter
!     curvelen       Length of curve
!     diff           Help parameter to compute the difference of two parameters
!     i              Counting variable
!     ipamat         starting position of matrix
!     iparc          starting position of arclength array
!     iprhs1         starting position of x-rhside
!     iprhs2         starting position of y-rhside
!     iprhs3         starting position of z-rhside
!     ipsolv         starting position solution
!     itype          spline type
!     j              Counting variable
!     ndef           number of (spline) defining points
!     work           work array to store the arrays
!                    SOLVE ( length: 4n * ndim )
!                    ARC   ( length:  n        )
!                    AMAT  ( length: 3n        )
!                    RHS   ( length:  n * ndim )
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHC02         Generate "open" spline
!     MSHC03         Determine coordinates points on generated spline
!     MSHC04         Generate "closed" spline
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine type of spline and make reservations in workarray
!     Dependent on type of spline call generating routine, &
!     if open spline call MSHC02
!     else if closed spline call MSHC04
!     Determine points on the spline generated (MSHC03)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshc01' )

!     --- Read type of Spline and number of defining points

      itype = iinput(1)
      ndef  = iinput(2)

      allocate ( work((5+5*ndim)*max(n,ndef)), stat = error )
      if ( error/=0 ) then

!     --- error#0, so not enough space

         call errint ( (5+5*ndim)*max(n,ndef), 1 )
         call errint ( error, 2 )
         call errchr ( 'work', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 900

      end if  ! ( error/= 0 )

      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshc01'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( itype<9 ) then
         alpha = 1d0
      else if ( itype<19 ) then
         alpha = 0d0
         itype = itype - 10
      else if ( itype<29 ) then
         alpha = 0.5d0
         itype = itype - 20
      else
         alpha = rinput(1)
         itype = itype - 30
      end if

!     --- Make reservations in WORK for the arrays
!         Array SOLVE , length : ndim * 4 * n real positions

      ipsolv = 1

!     --- Array ARC , length : n positions

      iparc = 1 +  4 * ndim * ndef

!     --- Array AMAT , length 3 * n positions

      ipamat = iparc + ndef

!     --- Array RHS , each ndim * n positions

      iprhs1 = ipamat + 3 * ndef
      iprhs2 = iprhs1 + ndef
      iprhs3 = iprhs2 + ndef

!     --- Compute spline coefficients

      if ( itype/=3 ) then
        call mshc02 ( work(ipsolv), ndef, ndim, work(iparc), &
                      work(ipamat), work(iprhs1), work(iprhs2), &
                      work(iprhs3), iinput, coor, alpha, icurnr )
      else
        call mshc04 ( work(ipsolv), ndef, ndim, work(iparc), &
                      work(ipamat), work(iprhs1), work(iprhs2), &
                      work(iprhs3), iinput, coor, alpha )
      end if

!     --- Determine points on curve

      call mshc03( work(ipsolv), ndef, ndim, work(iparc), tau, &
                   curves, n, alpha )
      if ( n==ndef ) then
         curvelen = 0d0
         diff = 0d0
         if ( ndim==1 ) then
            do i = 2, n
               j = iinput(2+i)
               curvelen = abs(curves(1,i) - curves(1,i-1))+curvelen
               diff = max(diff,abs(curves(1,i)-coor(1,j)))
            end do
         else if ( ndim==2 ) then
            do i = 2, n
               j = iinput(2+i)
               curvelen = sqrt((curves(1,i) - curves(1,i-1))**2 + &
                               (curves(2,i) - curves(2,i-1))**2)+ &
                          curvelen
               diff = max(sqrt((curves(1,i) - coor(1,j))**2 + &
                               (curves(2,i) - coor(2,j))**2),diff)
            end do
         else
            do i = 2, n
               j = iinput(2+i)
               curvelen = sqrt((curves(1,i) - curves(1,i-1))**2 + &
                               (curves(2,i) - curves(2,i-1))**2 + &
                               (curves(3,i) - curves(3,i-1))**2)+ &
                          curvelen
               diff = max(sqrt((curves(1,i) - coor(1,j))**2 + &
                               (curves(2,i) - coor(2,j))**2 + &
                               (curves(3,i) - coor(3,j))**2),diff)
            end do
         end if
         if ( diff<=curvelen*1d-4 ) then

!        --- Not much difference between original and new co-ordinates
!            Reset curve

            if ( ndim==1 ) then
               do i = 2, n
                  j = iinput(2+i)
                  curves(1,i)=coor(1,j)
               end do
            else if ( ndim==2 ) then
               do i = 2, n
                  j = iinput(2+i)
                  curves(1,i)=coor(1,j)
                  curves(2,i)=coor(2,j)
               end do
            else
               do i = 2, n
                  j = iinput(2+i)
                  curves(1,i)=coor(1,j)
                  curves(2,i)=coor(2,j)
                  curves(3,i)=coor(3,j)
               end do
            end if
         end if
      end if

900   deallocate ( work, stat = error )
      if ( error/=0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'work', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/=0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshc01'

      end if  ! ( debug )
      call erclos ( 'mshc01' )

      end
