      subroutine mshchkcr ( ndim, npoint, coor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 02-09-2010 Remove work arrays
!        version  1.0  date 05-10-1996
!
!   copyright (c) 1996-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Check uniqueness of co-ordinates in array coor
!    Two co-ordinates are equal if the distance is less than 1d-6*diameter
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, npoint
      double precision coor(ndim,npoint)

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     ndim       i    Dimension of the space.
!     npoint     i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, l, inodp, ifirst, jfirst, kfirst, nx, ny, nz, ih
      integer, allocatable :: iworkx(:), iworky(:), iworkz(:)
      double precision, allocatable :: work(:)
      double precision dist, epsx, epsy, epsz, xmin, xmax, ymin, ymax, &
                       zmin, zmax

!     dist       Distance between two points
!     epsx       Accuracy in x direction
!     epsy       Accuracy in y direction
!     epsz       Accuracy in z direction
!     i          Counting variable
!     ifirst     Starting adress in iworkx (minus 1)
!     ih         Help variable to store a constant temporarily
!     inodp      Nodal point number
!     iworkx     Work array to store the sorted sequence of the
!                x-co-ordinates, length: npoint
!     iworky     Work array to store the sorted sequence of the
!                y-co-ordinates, length: npoint
!     iworkz     Work array to store the sorted sequence of the
!                z-co-ordinates, length: npoint
!     j          Counting variable
!     jfirst     Starting adress in iworky (minus 1)
!     k          Counting variable
!     kfirst     Starting adress in iworkz (minus 1)
!     l          General loop variable
!     nx         number of points with same x co-ordinates
!     ny         number of points with same y co-ordinates
!     nz         number of points with same z co-ordinates
!     work       Work array to store co-ordinates. Length npoint
!     xmax       Maximum x-value
!     xmin       Minimum x-value
!     ymax       Maximum y-value
!     ymin       Minimum y-value
!     zmax       Maximum z-value
!     zmin       Minimum z-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRSORT         Sort real array for increasing sequence
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     982   Nodes have the same co-ordinates
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute xmin, xmax, ymin, ymax, zmin and zmax
!     eps := sqreps * sqrt((xmax-xmin)^2+(ymax-ymin)^2+(zmax-zmin)^2)
!     Sort all co-ordinates with respect to increasing x-co-ordinate
!     nx := 0
!     For all nodes i in new sequence do
!        if ( x_distance between node i and node i+1 )>eps ) then
!           if ( nx>0 ) then
!              If ( ndim=1 ) then
!                 give error message
!              else
!                 Sort array work with respect to y-co-ordinates
!                 ny := 0
!                 For all nodes i in new sequence do
!                 if ( y_distance between node i and node i+1 )>eps ) then
!                    if ( ny>0 ) then
!                       If ( ndim=2 ) then
!                          give error message
!                       else
!                          Sort array work with respect to y-co-ordinates
!                          Check uniqueness of z-co-ordinates
!                    ny := 0
!                 else
!                    ny := ny+1
!           nx := 0
!        else
!           nx := nx+1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshchkcr' )

!     --- Compute xmin, xmax, ymin, ymax, zmin and zmax

      xmin = rinfin
      xmax =-rinfin
      do i = 1, npoint
         xmin = min(xmin,coor(1,i))
         xmax = max(xmax,coor(1,i))
      end do
      epsx = abs(xmax-xmin)*sqreps
      if ( ndim>1 ) then
         ymin = rinfin
         ymax =-rinfin
         do i = 1, npoint
            ymin = min(ymin,coor(2,i))
            ymax = max(ymax,coor(2,i))
         end do
         epsy = abs(ymax-ymin)*sqreps
         if ( ndim>2 ) then
            zmin = rinfin
            zmax =-rinfin
            do i = 1, npoint
               zmin = min(zmin,coor(3,i))
               zmax = max(zmax,coor(3,i))
            end do
            epsz = abs(zmax-zmin)*sqreps
         end if
      end if

      allocate ( iworkx(npoint), iworky(npoint), iworkz(npoint), &
                 work(npoint), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, 4*npoint, 'iworkx' )
      if ( ierror/=0 ) go to 1000

!     --- Sort all co-ordinates with respect to increasing x-co-ordinate

      do i = 1, npoint
         work(i) = coor(1,i)
      end do
      call prsort ( work, iworkx, npoint )
      nx = 0

!     --- For all nodes i in new sequence do

      do i = 1, npoint-1
         inodp = iworkx(i)
         dist = abs(coor(1,inodp)-coor(1,iworkx(i+1)))
         if ( i==npoint-1 .or. dist>epsx ) then
            if ( i==npoint-1 ) then
               if ( dist<=epsx ) then
                  nx = nx+1
                  if ( nx==1 ) ifirst = npoint-2
               end if
            end if

!           --- x_distance between node i and node i+1>eps

            if ( nx>0 ) then

!           --- nx>0

               if ( ndim==1 ) then

!              --- ndim = 1, there are nx+1 points with the same co-ordinates

                  call errint ( nx+1, 1 )
                  do j = 1, nx
                     call errint ( iworkx(ifirst+j), j+1 )
                  end do
                  call erreal ( coor(1,iworkx(ifirst+1)), 1 )
                  call errsub ( 982, nx+2, 1, 0 )
               else

!              --- Sort array work with respect to y-co-ordinates

                  do j = 1, nx+1
                     work(j) = coor(2,iworkx(ifirst+j))
                  end do
                  call prsort ( work, iworky, nx+1 )
                  ny = 0

!                 --- For all nodes i in new sequence do

                  do j = 1, nx
                     inodp = iworkx(ifirst+iworky(j))
                     ih    = iworkx(ifirst+iworky(j+1))
                     dist = abs(coor(2,inodp)-coor(2,ih))
                     if ( j==nx .or. dist>epsy ) then

!                    --- y_distance between node i and node i+1>eps

                        if ( j==nx ) then
                           if ( dist<=epsy ) then
                              ny = ny+1
                              if ( ny==1 ) jfirst = nx-1
                           end if
                        end if
                        if ( ny>0 ) then

!                       --- ny>0

                           if ( ndim==2 ) then

!                          --- ndim = 2, there are ny points with
!                              the same co-ordinates

                              call errint ( ny+1, 1 )
                              do k = 1, ny+1
                                 ih = iworkx(ifirst+iworky(jfirst+k))
                                 call errint ( ih, k+1 )
                              end do
                              ih = iworkx(ifirst+iworky(jfirst+1))
                              call erreal ( coor(1,ih), 1 )
                              call erreal ( coor(2,ih), 2 )
                              call errsub ( 982, ny+2, 2, 0 )
                           else

!                          --- Sort array work with respect to z-co-ordinates
!                              Check uniqueness of z-co-ordinates

                              do k = 1, ny+1
                                 ih = iworkx(ifirst+iworky(jfirst+k))
                                 work(k) = coor(3,ih)
                              end do
                              call prsort ( work, iworkz, ny+1 )
                              nz = 0

!                             --- For all nodes i in new sequence do

                              do k = 1, ny
                                 inodp = iworkx(ifirst+ &
                                         iworky(jfirst+iworkz(k)))
                                 ih    = iworkx(ifirst+ &
                                         iworky(jfirst+iworkz(k+1)))
                                 dist = abs(coor(3,inodp)-coor(3,ih))
                                 if ( k==ny .or. dist>epsz ) then

!                                --- z_distance between node i and node i+1
!                                    >eps

                                    if ( k==ny ) then
                                       if ( dist<=epsz ) then
                                          nz = nz+1
                                          if ( nz==1 ) kfirst = nz-1
                                       end if
                                    end if
                                    if ( nz>0 ) then

!                                   --- nz>0
!                                       there are nz points with
!                                       the same co-ordinates

                                       call errint ( nz+1, 1 )
                                       do l = 1, nz+1
                                          ih = iworkx(ifirst+ &
                                               iworky(jfirst+ &
                                               iworkz(kfirst+l)))
                                          call errint ( ih, l+1 )
                                       end do
                                       ih = iworkx(ifirst+iworky(jfirst+ &
                                            iworkz(kfirst+1)))
                                       call erreal(coor(1,ih),1)
                                       call erreal(coor(2,ih),2)
                                       call erreal(coor(3,ih),3)
                                       call errsub ( 982, nz+2, 3, 0 )
                                       nz = 0
                                    end if
                                 else
                                    nz = nz+1
                                    if ( nz==1 ) kfirst = k-1
                                 end if
                              end do
                           end if
                        end if
                        ny = 0
                     else
                        ny = ny+1
                        if ( ny==1 ) jfirst = j-1
                     end if
                  end do
               end if
            end if
            nx = 0
         else
            nx = nx+1
            if ( nx==1 ) ifirst = i-1
         end if
      end do

      deallocate ( iworkx, iworky, iworkz, work, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iworkx' )

1000  call erclos ( 'mshchkcr' )

      end subroutine mshchkcr
