      subroutine mshchksf ( inpchk, rinchk, icheck )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 13-10-1996 Allow inpchk(2)=2
!        version  1.0  date 11-08-1996
!
!   copyright (c) 1996-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Check if the surface elements of the mesh generated satisfy the conditions
!   as described in the arrays inpchk and rinchk.
!   Return with a value for icheck indicating if remeshing should be applied
! **********************************************************************
!
!                       KEYWORDS
!
!     check
!     surface
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleconsts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: inpchk(*), icheck
      double precision rinchk(*)

!     icheck     o    Indicates if remeshing must be applied or not
!                     Possible values:
!                     0  No remeshing necessary
!                     1  The boundary does not satisfy the requirements
!                     2  The internal elements do not satisfy the requirements
!     inpchk     i    Indicates how the checking must be applied
!                     inpchk must be filled as follows:
!                     1:  indication of how the boundary must be checked
!                         Possible values:
!                         0   No checking of the boundary, except in the
!                             case that general is applied
!                             If general is applied all boundaries of the
!                             surfaces generated by general are checked
!                             with respect to the general requirements
!                         1   The first and last element of each curve
!                             in array icurves is compared with the
!                             original length stored in array kmesh p
!                             If 1/a < dx/dx_orig < a the curve is
!                             accepted, otherwise icheck is set equal to 1
!                             dx is the actual length of the element
!                             dx_orig is the original length of the element
!                             a must be stored in rinchk(1)
!                         As soon as one of the curves returns with icheck = 1, &
!                         checking is stopped
!                     2:  indication of how the internal elements must be
!                         checked. This is only applied if the boundary
!                         check returns with icheck = 0
!                         Possible values:
!                         0   No checking of the internal elements
!                         1   All angles of the elements are checked
!                             If the smallest angle of an element < rinchk(2)
!                             or the largest angle of an element > rinchk(3)
!                             icheck is set equal to 2 and checking is stopped
!                         2   See 1, but now icheck is set to 1, meaning that
!                             the number of nodes along the boundary may be
!                             changed
!                     3:  numcrv, i.e. number of curves in icurves
!     rinchk     i    See inpchk
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, ikelmc, ielgrp, istep, maxnode, irelem, inodp, inext, &
              ibefore
      double precision cosmax, cosmin, x, y, xplus, yplus, xmin, ymin, &
                       dxplus, dxmin, dyplus, dymin, xplnrm, xmnrm, &
                       costh, sinth, cospl, cosmn

!     cosmax     Cosine of minimum angle allowed
!     cosmin     Cosine of maximum angle allowed
!     cosmn      Mimimum cosine found
!     cospl      Maximum cosine found
!     costh      Cosine of angle between two sides
!     dxmin      x-difference between two succeeding points in the backward
!                direction
!     dxplus     x-difference between two succeeding points in the forward
!                direction
!     dymin      y-difference between two succeeding points in the backward
!                direction
!     dyplus     y-difference between two succeeding points in the forward
!                direction
!     ibefore    Relative node number of node along side in backward direction
!     ielgrp     element group number
!     ikelmc     position in kmesh part c
!     inext      Relative node number of node along side in forward direction
!     inodp      Nodal point number
!     inpelm     number of points per element
!     irelem     Relative element sequence number with respect to element group
!     istep      Step size for vertices
!     maxnode    Maximum relative node number along sides of element
!     sinth      Sine of angle between two sides
!     x          x co-ordinate of point
!     xmin       x co-ordinate of point ibefore
!     xmnrm      Norm of dxmin
!     xplnrm     Norm of dxplus
!     xplus      x co-ordinate of point inext
!     y          y co-ordinate of point
!     ymin       y co-ordinate of point ibefore
!     yplus      y co-ordinate of point inext
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Set maximum and minimum value of cosine (function of rinchk)
!     For ielgrp := 1 (1) nelgrp do
!        compute inpelm, ishape, inelem
!        compute max_node and istep
!        For irelem := 1 (1) inelem do
!           Check all angles of the vertices using the cosine rule
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( inpchk(2)>=1 ) then

!     --- the angles of the elements must be checked

         cosmax = cos(rinchk(2)*pi/180d0)
         cosmin = cos(rinchk(3)*pi/180d0)
         cospl = -1d0
         cosmn =  1d0
         ikelmc = 0
         do ielgrp = 1, nelgrp
            ishape = kmeshf(ielgrp)
            inpelm = Topgrp(ielgrp)%inpelm
            inelem = Topgrp(ielgrp)%inelem
            if ( ishape>=3 .and. ishape<=7 ) then

!           --- standard triangles or quadrilaterals

               if ( ishape==4 .or. ishape>=6 ) then
                  istep = 2
               else
                  istep = 1
               end if
               if ( ishape>=6 ) then
                  maxnode = inpelm - 1
               else
                  maxnode = inpelm
               end if

!              --- Loop over elements

               do irelem = 1, inelem

!              --- Loop over vertices

                  do inodp = 1, maxnode, istep
                     if ( inodp==maxnode ) then
                        inext = 1
                     else
                        inext = inodp+1
                     end if
                     if ( inodp==1 ) then
                        ibefore = maxnode
                     else
                        ibefore = inodp-1
                     end if
                     x = coor(1,kmeshc(ikelmc+inodp))
                     y = coor(2,kmeshc(ikelmc+inodp))

                     xplus = coor(1,kmeshc(ikelmc+inext))
                     yplus = coor(2,kmeshc(ikelmc+inext))

                     xmin  = coor(1,kmeshc(ikelmc+ibefore))
                     ymin  = coor(2,kmeshc(ikelmc+ibefore))

                     dxplus = xplus - x
                     dyplus = yplus - y

                     dxmin  = xmin  - x
                     dymin  = ymin  - y

                     xplnrm = sqrt(dxplus**2 + dyplus**2)
                     xmnrm  = sqrt(dxmin**2  + dymin**2)

                     costh = (dxplus*dxmin+dyplus*dymin)/(xplnrm*xmnrm)
                     sinth = (dxplus*dymin-dyplus*dxmin)/(xplnrm*xmnrm)

                     cospl = max(cospl,costh)
                     cosmn = min(cosmn,costh)

                     if ( costh>cosmax .or. costh<cosmin .or. &
                          sinth<0 ) then

!                    --- bad angle

                        icheck = 3 - inpchk(2)
                        go to 1000
                     end if

                  end do
                  ikelmc = ikelmc + inpelm

               end do

            else

!           --- other types of elements are not checked

               ikelmc = ikelmc + inelem*inpelm

            end if

         end do

      end if

1000  continue
      end
