      subroutine mshcortri ( npoint, nelem, inpelm, kmeshc, nbndpt, &
                             kbndpt, coor, nholes, holeinfo )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 18-02-2009 Correction extreme ase
!        version  2.0  date 03-03-2003 Extra parameters nholes, holeinfo
!        version  1.2  date 06-12-2002 Clear arrays
!
!   copyright (c) 1999-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Check for all triangles with 3 vertices on the boundary if there
!    are neigbouring elements connected with the same property
!    If so check if the direction of the diagonal between two neighbouring
!    elements must be switched in order to improve the shape of the elements
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, nelem, inpelm, kmeshc(inpelm,*), nbndpt, &
              kbndpt(*), nholes, holeinfo(2,*)
      double precision coor(2,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     holeinfo       i    Integer array of size 2 x (nholes+2)
!                         Contains information about holes in surfaces and where
!                         a new part of the boundary starts
!                         Pos (1,i-1) contains the local sequence number of the
!                            first curve of part i provided with a sign
!                            If the sign is positive the boundary is
!                            created counter clockwise, otherwise clockwise
!                         Pos (2,i-1) contains the local sequence number of the
!                            first node of of part i
!                         Pos (1,nholes+1) contains the number of curves of the
!                         boundary
!     inpelm         i    number of points per element
!     kbndpt         i    array containing the nodal point numbers of the
!                         boundary
!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nbndpt         i    Number of nodal points in boundary, length of array
!                         kbndpt
!     nelem          i    Number of elements
!     nholes         i    Number of holes in the boundary of a surface
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, inelem, ielem, nodp, istart, ih, inodes, istep, &
              npelm, ipos, iraise(3), ind1(2), ind2(2), node(9), &
              iseq(4), ineighelem, iside, k, nswitch, ind3(5), &
              iboundary, lniwork
      integer, allocatable :: iwork(:), iwork1(:), istartelms(:), &
                              ielements(:), ihelp(:), elmsperpoint(:), &
                              ipoints(:)
      double precision tang(6,2), cosangle1(4), norm, &
                       cosangle2(4), max1, max2, acosangle1(4)
      logical check, standard

!     acosangle1     contains the four angles made by the diagonal
!     check          If true the diagonal must be switched
!     cosangle1      cosine of angle1
!     cosangle2      cosine of angle2
!     elmsperpoint   work array of length npelm*npoint containing all
!                    nodes of the elements in ielements
!     i              Counting variable
!     iboundary      Boundary sequence number
!     ielem          Element number
!     ielements      work array of length nelem that contains  all
!                    elements that have all vertices on the boundary
!     ih             Help variable to store a constant temporarily
!     ihelp          integer work array
!     ind1           Contains 2, 4
!     ind2           Contains 4, 1
!     ind3           Contains 2, 4, 6, 2, 4
!     ineighelem     Neighbour of element in same quadrilateral
!     inelem         Number of elements in a specific element group
!     inodes         Number of nodes
!     ipoints        Work array of length npoint containing a set of nodes
!     ipos           Position in array
!     iraise         Array of length 3 to store the cyclic permutation of
!                    1 - 3
!     iseq           Sequence number
!     iside          edge sequence number
!     istart         First position used at a part
!     istartelms     work array of lengtrh npoint containing the start
!                    positions of elements in nodes
!     istep          step
!     iwork          integer work array of length npoint
!     iwork1         integer work array of length npoint
!     j              Counting variable
!     k              Counting variable
!     lniwork        Declared length of array istartelms
!     max1
!     max2
!     node           Node number
!     nodp           Nodal point sequence number
!     norm
!     npelm          Number of points per element
!     nswitch
!     standard
!     tang           tangential vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHD7          Put coordinates of extra centre point in quadratic triangle
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear arrays iwork and iwork1
!     Fill array iwork with 1 for boundary points
!     Store all elements that have 3 vertices on the boundary into ielements
!     Store the corresponding vertices into ipoints
!     For all points in ipoints store elements of ielements into elmsperpoint
!     For all elements in ielements
!        For all sides with a neighbour with larger number do
!           Check diagonal and if necessary switch
!
!
!     Local numbering applied:
!
!
!          9                                      9
!   1 x---------x 4                        1 x---------x 4
!     |\        |                            |        /|
!     | \       |                            |       / |
!     |  \      |                            |      /  |
!     |   \     |                            |     /   |
!   7 |   5\    | 8         ------>        7 |   5/    | 8
!     |     \   |                            |   /     |
!     |      \  |                            |  /      |
!     |       \ |                            | /       |
!     |        \|                            |/        |
!   3 x---------x 2                        3 x---------x 2
!          6                                      6
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data iraise / 2, 3, 1 /, ind1 / 2, 4 /, ind2 / 4, 1 /, &
           ind3 / 2, 4, 6, 2, 4 /
! ======================================================================
!
      call eropen ( 'mshcortri' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcortri'
         call prinin1 ( kmeshc, nelem, inpelm, 'kmeshc' )
         call prinin ( kbndpt, nbndpt, 'kbndpt' )
         write(irefwr,1) 'nholes', nholes
         call prinin ( kbndpt, nbndpt, 'kbndpt' )
         call prinin1 ( holeinfo, nholes+2, 2, 'holeinfo' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      npelm = min(5,inpelm)
      lniwork = max(npoint,2*nelem)

      allocate ( iwork(lniwork), iwork1(npoint), istartelms(npoint+1), &
                 ielements(nelem), ihelp(nelem), &
                 elmsperpoint(npelm*nelem), ipoints(npoint), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npoint+3*nelem, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Store all elements that have 3 vertices on one boundary into ielements
!         Store the corresponding vertices into ipoints

      if ( inpelm>5 ) then
         istep = 2
      else
         istep = 1
      end if

      do iboundary = 1, nholes+1

!     --- Clear arrays iwork and iwork1

         iwork(1:npoint) = 0
         iwork1(1:npoint) = 0

!        --- Fill array iwork with 1 for boundary points

         if ( nholes==0 ) then

!        --- nholes = 0, no holes

            do i = 1, nbndpt
               iwork(kbndpt(i)) = i
            end do

         else

!        --- nholes>0, there are holes
!            Use boundary part number in iwork

            do i = holeinfo(2,iboundary), holeinfo(2,iboundary+1)-1
               iwork(kbndpt(i)) = i
            end do

         end if
         if ( debug ) call prinin ( iwork, npoint, 'iwork-1' )

         inelem = 0
         inodes = 0
         do ielem = 1, nelem

            do i = 1, npelm, istep

!           --- if one of the nodes not on iboundary skip element

               if ( iwork(kmeshc(i,ielem))==0 ) go to 100

            end do

!           --- element found

            inelem = inelem+1
             ielements(inelem) = ielem
            do i = 1, npelm, istep

               nodp = kmeshc(i,ielem)
               if ( iwork1(nodp)==0 ) then

!              --- Vertex has not yet been stored

                  inodes = inodes+1
                  ipoints(inodes) = nodp
                  iwork1(nodp) = inodes

               end if

            end do

100      end do

         if ( debug ) then
            call prinin ( ielements, inelem, 'ielements' )
            call prinin ( ipoints, inodes, 'ipoints' )
         end if  ! ( debug )

         if ( inelem<=1 ) go to 500

!        --- For all points in ipoints store elements of ielements into
!            elmsperpoint
!            First clear istartelms

         istartelms(1:inodes) = 0

!        --- Next find the number of elements per node and store in istartelms

         do i = 1, inelem
            ielem = ielements(i)
            do j = 1, npelm, istep
               nodp = kmeshc(j,ielem)
               ipos = iwork1(nodp)
               istartelms(ipos) = istartelms(ipos)+1
            end do
         end do

!        --- Accumulate istartelms

         istart = 0
         do i = 1, inodes
            ih = istartelms(i)
            istartelms(i) = istart
            istart = istart+ih
         end do

!        --- Fill array elmsperpoint

         do i = 1, inelem
            ielem = ielements(i)
            do j = 1, npelm, istep
               nodp = kmeshc(j,ielem)
               ipos = iwork1(nodp)
               istartelms(ipos) = istartelms(ipos)+1
               elmsperpoint(istartelms(ipos)) = ielements(i)
            end do
         end do

!        --- Reset istartelms

         istart = 0
         do i = 1, inodes
            ih = istartelms(i)
            istartelms(i) = istart
            istart = ih
         end do
         istartelms(inodes+1) = istart

!        --- For all elements in ielements

         ihelp = 0
         nswitch = 0

         do k = 1, inelem
            ielem = ielements(k)

!           --- For all sides with a neighbour with larger number do

            do iside = 1, 3
               iseq(1) = (iside-1)*istep+1
               node(1) = kmeshc(iseq(1),ielem)

!              --- Set ihelp to 1 for all elements connected to node(1)

               ipos = iwork1(node(1))
               do j = istartelms(ipos)+1, istartelms(ipos+1)
                  ihelp(elmsperpoint(j)) = 1
               end do
               iseq(2) = iseq(1)+istep
               if ( iseq(2)>3*istep ) iseq(2) = 1
               node(2) = kmeshc(iseq(2),ielem)

!              --- Find all elements connected to node(2), that have a larger
!                  number than ielem and also have ihelp(i) = 1

               ipos = iwork1(node(2))
               do i = istartelms(ipos)+1, istartelms(ipos+1)
                  ineighelem = elmsperpoint(i)
                  if ( ineighelem>ielem .and. &
                       ihelp(ineighelem)>0 ) then

!                 --- element found, find two other nodes

                     iseq(3) = iseq(2)+istep
                     if ( iseq(3)>3*istep ) iseq(3) = 1
                     node(3) = kmeshc(iseq(3),ielem)

                     if ( debug ) then
                        write(irefwr,1) 'ielem, ineighelem', &
                                         ielem, ineighelem
                        call prinin ( node, 3, 'node' )
                        call prinin ( iseq, 3, 'iseq' )
                     end if  ! ( debug )

                     do j = 1, 3
                        iseq(4) = (j-1)*istep+1
                        node(4) = kmeshc(iseq(4),ineighelem)
                        if ( debug ) &
                           write(irefwr,1) 'node(4), iseq(4)', &
                                            node(4), iseq(4)
                        if ( node(4)/=node(1) .and. &
                             node(4)/=node(2) ) go to 200
                     end do

!                    --- Check diagonal and if necessary switch

200                  do j = 1, 3
                        tang(j,1) = &
                           coor(1,node(iraise(j)))-coor(1,node(j))
                        tang(j,2) = &
                           coor(2,node(iraise(j)))-coor(2,node(j))
                        norm = 1d0/sqrt(tang(j,1)**2+tang(j,2)**2)
                        tang(j,1) = tang(j,1)*norm
                        tang(j,2) = tang(j,2)*norm
                     end do
                     do j = 2, 3
                        tang(j+2,1) = coor(1,node(ind2(j-1)))- &
                                      coor(1,node(ind1(j-1)))
                        tang(j+2,2) = coor(2,node(ind2(j-1)))- &
                                      coor(2,node(ind1(j-1)))
                        norm = 1d0/sqrt(tang(j+2,1)**2+tang(j+2,2)**2)
                        tang(j+2,1) = tang(j+2,1)*norm
                        tang(j+2,2) = tang(j+2,2)*norm
                     end do

                     cosangle1(1) =-tang(1,1)*tang(2,1)- &
                                    tang(1,2)*tang(2,2)
                     cosangle1(2) =-tang(3,1)*tang(1,1)- &
                                    tang(3,2)*tang(1,2)
                     cosangle1(3) =-tang(1,1)*tang(4,1)- &
                                    tang(1,2)*tang(4,2)
                     cosangle1(4) =-tang(5,1)*tang(1,1)- &
                                    tang(5,2)*tang(1,2)

!                    --- First check if the quadrilateral i convex

                     do j = 1, 4
                        acosangle1(j) = acos(cosangle1(j))
                     end do
                     if ( acosangle1(1)+acosangle1(3)>3d0 .or. &
                          acosangle1(2)+acosangle1(4)>3d0 ) go to 300

!                    --- The quadrilateral is really convex

                     check = .false.
                     do j = 1, 4
                        if ( abs(cosangle1(j))>0.96d0 ) check = .true.
                     end do
                     if ( .not. check ) go to 300

                     tang(6,1) = coor(1,node(4))-coor(1,node(3))
                     tang(6,2) = coor(2,node(4))-coor(2,node(3))
                     norm = 1d0/sqrt(tang(6,1)**2+tang(6,2)**2)
                     tang(6,1) = tang(6,1)*norm
                     tang(6,2) = tang(6,2)*norm
                     cosangle2(1) = tang(6,1)*tang(2,1)+ &
                                    tang(6,2)*tang(2,2)
                     cosangle2(2) = tang(3,1)*tang(6,1)+ &
                                    tang(3,2)*tang(6,2)
                     cosangle2(3) = tang(6,1)*tang(4,1)+ &
                                    tang(6,2)*tang(4,2)
                     cosangle2(4) = tang(5,1)*tang(6,1)+ &
                                    tang(5,2)*tang(6,2)

                     max1 = 0d0
                     max2 = 0d0
                     do j = 1, 4
                        max1 = max(max1,abs(cosangle1(j)))
                        max2 = max(max2,abs(cosangle2(j)))
                     end do

                     if ( debug ) then
                        call prinrl ( cosangle1, 4, 'cosangle1' )
                        call prinrl ( cosangle2, 4, 'cosangle2' )
                        write(irefwr,2) 'max1, max2', max1, max2
                     end if  ! ( debug )

                     if ( max1>max2 .and. max2<0.99d0 ) then

!                    --- Diagonal may be swapped, store both element numbers
!                        into iwork
!                        The check on 0.95 has been used for extreme cases

                        iwork(nswitch+1) = ielem
                        iwork(nswitch+2) = ineighelem
                        nswitch = nswitch+2

                     end if

                  end if

300            end do

!              --- Reset ihelp

               ipos = iwork1(node(1))
               do i = istartelms(ipos)+1, istartelms(ipos+1)
                  ihelp(elmsperpoint(i)) = 0
               end do

            end do

         end do

         if ( debug ) write(irefwr,1) 'nswitch', nswitch
         if ( nswitch>0 ) then

!        --- nswitch>0, diagonals may be switched

            ipoints(1:npoint) = 0

            if ( debug ) call prinin ( iwork, npoint, 'iwork-1' )
            if ( nswitch>2 ) then

!           --- nswitch>2, remove parts if an element appears twice, &
!               This complex situation has not been programmed yet

               ihelp = 0
               do k = 1, nswitch
                  ihelp(iwork(k)) = ihelp(iwork(k))+1
               end do
               do k = 1, nswitch, 2
                  if ( ihelp(iwork(k))>1 .or. &
                       ihelp(iwork(k+1))>1 ) then
                     iwork(k) = 0
                     iwork(k+1) = 0
                  end if
               end do

            end if
            if ( debug ) call prinin ( iwork, npoint, 'iwork-2' )

            do k = 1, nswitch, 2

!           --- Switch elements

               ielem = iwork(k)
               if ( ielem==0 ) go to 400
               ineighelem = iwork(k+1)
               if ( debug ) then

!              --- Debug information

                  write(irefwr,*) 'ielem, ineighelem ', &
                                   ielem, ineighelem
                  call prinin ( kmeshc(1,ielem), inpelm, &
                                'element ielem' )
                  call prinin ( kmeshc(1,ineighelem), inpelm, &
                                'element ineighelem' )

               end if
               do i = 1, 3
                  ipoints(kmeshc((i-1)*istep+1,ielem)) = i
               end do
               ipos = 0
               iseq(1:4) = 0
               do i = 1, 3
                  ih = kmeshc((i-1)*istep+1,ineighelem)
                  j = ipoints(ih)
                  ipoints(ih) = 0
                  if ( j>0 ) then
                     ipos = ipos+1
                     iseq(ipos) = j
                  else
                     iseq(4) = i
                  end if
               end do
               if ( debug ) call prinin ( iseq, 4, 'iseq-1' )

               if ( iseq(1)==1 ) then
                  if ( iseq(2)==2 ) then
                     iseq(3) = 3
                  else
                     iseq(1) = 3
                     iseq(2) = 1
                     iseq(3) = 2
                  end if
               else if ( iseq(1)==2 ) then
                  if ( iseq(2)==1 ) then
                     iseq(1) = 1
                     iseq(2) = 2
                     iseq(3) = 3
                  else
                     iseq(3) = 1
                  end if
               else
                  if ( iseq(2)==1 ) then
                     iseq(3) = 2
                  else
                     iseq(1) = 2
                     iseq(2) = 3
                     iseq(3) = 1
                  end if
               end if
               if ( debug ) call prinin ( iseq, 4, 'iseq-2' )

!              --- Clear array ipoints

               do i = 1, 3
                  ipoints(kmeshc((i-1)*istep+1,ielem)) = 0
               end do

               if ( inpelm==3 ) then

!              --- Linear triangles

                   do i = 1, 3
                      node(i) = kmeshc(iseq(i),ielem)
                   end do
                   node(4) = kmeshc(iseq(4),ineighelem)
                   kmeshc(1,ielem) = node(1)
                   kmeshc(2,ielem) = node(4)
                   kmeshc(3,ielem) = node(3)
                   kmeshc(1,ineighelem) = node(2)
                   kmeshc(2,ineighelem) = node(3)
                   kmeshc(3,ineighelem) = node(4)

               else if ( inpelm<=7 ) then

!              --- Quadratic triangles

                   do i = 1, 3
                      node(i) = kmeshc(2*(iseq(i)-1)+1,ielem)
                   end do
                   node(4) = kmeshc(2*(iseq(4)-1)+1,ineighelem)
                   if ( iseq(1)==3 ) then
                      if ( iseq(2)==1 ) then
                         standard = .true.
                      else
                         standard = .false.
                      end if
                   else
                      if ( iseq(2)==iseq(1)+1 ) then
                         standard = .true.
                      else
                         standard = .false.
                      end if
                   end if
                   if ( standard ) then

!                  --- Standard sequence

                      do i = 1, 3
                         node(i+4) = kmeshc(2*iseq(i),ielem)
                      end do

                   else

!                  --- Reversed sequence

                      do i = 1, 3
                         node(i+4) = kmeshc(ind3(2+iseq(1)-i+1),ielem)
                      end do

                   end if
                   if ( iseq(4)==1 ) then
                      if ( kmeshc(3,ineighelem)==node(1)) then
                         node(9) = kmeshc(2,ineighelem)
                         node(8) = kmeshc(6,ineighelem)
                      else
                         node(8) = kmeshc(2,ineighelem)
                         node(9) = kmeshc(6,ineighelem)
                      end if
                   else if ( iseq(4)==2 ) then
                      if ( kmeshc(3,ineighelem)==node(1)) then
                         node(9) = kmeshc(4,ineighelem)
                         node(8) = kmeshc(2,ineighelem)
                      else
                         node(8) = kmeshc(4,ineighelem)
                         node(9) = kmeshc(2,ineighelem)
                      end if
                   else
                      if ( kmeshc(3,ineighelem)==node(1)) then
                         node(9) = kmeshc(6,ineighelem)
                         node(8) = kmeshc(4,ineighelem)
                      else
                         node(8) = kmeshc(6,ineighelem)
                         node(9) = kmeshc(4,ineighelem)
                      end if
                   end if
                   kmeshc(1,ielem) = node(1)
                   kmeshc(2,ielem) = node(9)
                   kmeshc(3,ielem) = node(4)
                   kmeshc(4,ielem) = node(5)
                   kmeshc(5,ielem) = node(3)
                   kmeshc(6,ielem) = node(7)
                   kmeshc(1,ineighelem) = node(2)
                   kmeshc(2,ineighelem) = node(6)
                   kmeshc(3,ineighelem) = node(3)
                   kmeshc(4,ineighelem) = node(5)
                   kmeshc(5,ineighelem) = node(4)
                   kmeshc(6,ineighelem) = node(8)
                   coor(1,node(5)) = 0.5d0* &
                       (coor(1,node(3))+coor(1,node(4)))
                   coor(2,node(5)) = 0.5d0* &
                       (coor(2,node(3))+coor(2,node(4)))

               end if

               if ( debug ) then

!              --- Debug information

                  write(irefwr,*) 'after switch'
                  call prinin ( kmeshc(1,ielem), inpelm, &
                                'element ielem' )
                  call prinin ( kmeshc(1,ineighelem), inpelm, &
                                'element ineighelem' )

               end if

400         end do
            if ( inpelm==7 ) then

!           --- inpelm = 7, recompute centroids

               call mshd7 ( kmeshc, coor, nelem, 2 )

            end if

         end if  ! ( nswitch>0 )

500   end do  ! iboundary = 1, max(1,nholes)

      deallocate ( iwork, iwork1, istartelms, ielements, ihelp, &
                   elmsperpoint, ipoints, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'mshcortri' )
      if ( debug ) write(irefwr,*) 'End mshcortri'

      end
