      subroutine mshcr06 ( coorol, coornw, usol, npoint, factor, &
                           iellist )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.0    date  08-04-1996
!
!
!
!   copyright (c) 1996-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute the co-ordinates of the new boundary in msh047 in the case
!   iadapt = 6 i.e. x_new = x_old + factor (grad c.n) n
!   grad c must be given per element and the displacement of the mid points
!   of the linear elements is computed
!   The displacement of the vertices is computed by linear interpolation
!   between the displacement of the mid points
! **********************************************************************
!
!                       KEYWORDS
!
!   adapt_boundary
!   free_boundary
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer npoint, iellist(*)
      double precision coorol(2,*), coornw(2,*), usol(2,*), factor

!     coornw     o    Array containing the new co-ordinates along the boundary
!     coorol     i    Array containing the old co-ordinates along the boundary
!     factor     i    Multiplication factor
!     iellist    i    Array of length npoint-1 containing a
!                     list of elements that are adjacent to the boundary
!                     element iellist(i) contains both nodal points
!                     iwork(i) and iwork(i+1)
!     npoint     i    Number of nodal points along the boundary
!     usol       i    Solution vector
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielem
      double precision u, v, alpha, n1, n2, dx1, dx2, h, s, &
                       x1, x2, area, areaold, x1old, x2old

!     alpha      Multiplication factor factor u.n
!     area       Area of displacement times element length / 2
!     areaold    Area of previous element
!     dx1        displacement in x-direction of mid-point
!     dx2        displacement in y-direction of mid-point
!     h          length of an element along the boundary
!     i          Counting variable
!     ielem      Element number
!     n1         x-component of normal with respect to boundary element
!     n2         y-component of normal with respect to boundary element
!     s          Factor by which the normal must be multiplied
!     u          First component of solution
!     v          Second component of solution
!     x1         x-coordinate of mid-point
!     x1old      x-coordinate of mid-point of previous element
!     x2         y-coordinate of mid-point
!     x2old      y-coordinate of mid-point of previous element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The co-ordinates along the boundary are updated according to the
!    formula:
!    x_new(i) = x_old(i) + factor grad(i) . n(i) n(i)
!    For all elements i along the boundary do
!       ielem := element number of adjacent element
!       u := usol(1,ielem)
!       v := usol(2,ielem)
!       compute n
!       alpha := factor u.n
!       dx = alpha * n
!       area = area region spanned by dx and element / 2
!       if ( i=1 ) then
!          unew(i) := uold(i) + dx
!       else
!          Compute n in vertex
!          Compute s such that displacement s n covers (area+areaold)/2
!          unew(i-1) := uold(i-1) + s n
!       dxold := dx
!       areaold := area
!    unew(npoint) := old(npoint) + dx
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      do i = 1, npoint-1

         ielem = iellist(i)
         u = usol(1,ielem)
         v = usol(2,ielem)
         n1 = coorol(2,i+1)-coorol(2,i)
         n2 = coorol(1,i)-coorol(1,i+1)
         h = sqrt(n1**2+n2**2)
         n1 = n1/h
         n2 = n2/h
         alpha = factor*(u*n1+v*n2)
         dx1 = alpha*n1
         dx2 = alpha*n2
         x1 = 0.5d0*(coorol(1,i)+coorol(1,i+1))+dx1
         x2 = 0.5d0*(coorol(2,i)+coorol(2,i+1))+dx2
         area = 0.5d0*sqrt(dx1**2+dx2**2)*h
         if ( i.eq.1 ) then
            coornw(1,i) = coorol(1,i)+dx1
            coornw(2,i) = coorol(2,i)+dx2
         else
            n1 = coorol(2,i+1)-coorol(2,i-1)
            n2 = coorol(1,i-1)-coorol(1,i+1)
            h = sqrt(n1**2+n2**2)
            n1 = n1/h
            n2 = n2/h
            s = (area+areaold)/abs((-n1*(x2old-x2)+n2*(x1old-x1)))
            coornw(1,i) = coorol(1,i)+s*n1
            coornw(2,i) = coorol(2,i)+s*n2
         end if
         x1old = x1
         x2old = x2
         areaold = area

      end do
      coornw(1,npoint) = coorol(1,npoint)+dx1
      coornw(2,npoint) = coorol(2,npoint)+dx2

      end
