      subroutine mshcurccirc ( ndim, curves, iinput, rinput, tau, &
                               icurnr, numuserp, irelcurves, curstr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-12-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array curves for a circle with more than 1 segment
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     curve
!     point_distribution
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), ndim, icurnr, irelcurves(*), numuserp
      double precision curves(ndim,*), rinput(ndim,*), tau(*), curstr(*)

!     curstr        i/o   Contains real information of the curves
!                         See subroutine mshcrtcurv
!     curves         o    array containing the coordinates of the curves
!     icurnr         i    Curve sequence number
!     iinput         i    integer input array   see subroutine mesh
!     irelcurves     i    Contains integer information of the curves
!                         See subroutine mshcrtcurv
!     ndim           i    Dimension of the space.
!     numuserp       i    Number of user points along curve
!     rinput         i    Real input array
!     tau            i    Work array of length n to store the relative distances
!                         between points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: n, ntot, istartrel, i, p1, p2, p3, isegment
      double precision :: x1(ndim), xc(ndim), tang1(ndim), tang2(ndim), &
                          ch, sh, radius, xicoor, etacoor, phi, philoc, &
                          phi0

!     ch             cos(philoc)
!                    they are not
!     etacoor        eta coordinate
!     i              Counting variable
!     isegment       Sequence number of segment
!     istartrel      Starting position in array irelcurves
!     n              number of nodes along curve segment
!     ntot           Accumulated number of nodes
!     p1             user point number of centroid
!     p2             user point number of first point
!     p3             user point number of second point
!     phi            angle
!     phi0           start angle
!     philoc         angle of segment
!     radius         Radius of sphere or circle
!     sh             sin(philoc)
!     tang1          First tangential vector
!     tang2          Second tangential vector
!     x1             coordinates of first point in segment
!     xc             array with the coordinates of the center point of the
!                    element
!     xicoor         xi coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHMAP3D2D     Map plane defined by 3 points onto xi,eta system
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcurccirc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcurccirc'
         write(irefwr,1) 'numuserp', numuserp
         call prinrl ( tau, numuserp, 'tau' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istartrel = irelcurves(icurnr)
      p1 = iinput(1)           ! center of circle
      p2 = iinput(2)           ! first point on circle
      p3 = iinput(3)           ! next point on circle

      radius = curstr(1)
      x1 = rinput(:,p2)
      xc = rinput(:,p1)
      curves(:,1) = x1

      if ( debug ) then
         write(irefwr,1) 'p1, p2, p3', p1, p2, p3
         write(irefwr,2) 'radius', radius
         write(irefwr,2) 'x1', x1
         write(irefwr,2) 'xc', xc
         call prinin ( irelcurves, istartrel+numuserp, 'irelcurves' )
      end if  ! ( debug )

      if ( ndim==3 ) then

!     --- Define mapping from 3d to 2d in case of 3d circle

         call mshmap3d2d ( p1, p2, p3, rinput, tang1, tang2 )

         if ( debug ) then
            write(irefwr,2) 'tang1', tang1
            write(irefwr,2) 'tang2', tang2
         end if  ! ( debug )

      end if  ! ( ndim==3 )

!     --- Loop over the segments

      n = 0
      phi0 = 0d0
      do isegment = 1, numuserp-1

         ntot = irelcurves(istartrel+isegment)

         phi = curstr(isegment+1)
         if ( debug ) then
            write(irefwr,2) 'phi', phi
            write(irefwr,1) 'isegment, ntot, n', isegment, ntot, n
         end if  ! ( debug )

         do i = n+1, ntot

            philoc = phi*tau(i+1)+phi0
            ch = cos(philoc)
            sh = sin(philoc)
            if ( debug ) &
               write(irefwr,2) 'philoc, ch, sh', philoc, ch, sh

            if ( ndim==2 ) then

!           --- ndim = 2

               curves(1,i+1) = xc(1)+ch*(x1(1)-xc(1))-sh*(x1(2)-xc(2))
               curves(2,i+1) = xc(2)+ch*(x1(2)-xc(2))+sh*(x1(1)-xc(1))

            else

!           --- ndim = 3

               xicoor  = radius * ch
               etacoor = radius * sh
               curves(:,i+1) = xc+xicoor*tang1+etacoor*tang2

            end if  ! ( ndim==2 )

         end do  ! i = n+1, ntot

         n = ntot
         phi0 = philoc

      end do  ! isegment = 1, numuserp-1

1000  call erclos ( 'mshcurccirc' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( curves, n, ndim, 'curves' )
         write(irefwr,*) 'End mshcurccirc'

      end if  ! ( debug )

      end subroutine mshcurccirc
