      subroutine mshcurfr ( icurvsold, curvesold, &
                            lencurvs, iinput, rinput, changed )
! ======================================================================
!
!        programmer    Guus Segal
!        version  8.1  date 11-12-2014 Adaptation due to new curve routines
!        version  8.0  date 02-09-2010 Remove intarmsh
!        version  7.4  date 07-11-2009 New call to mshcurcr
!
!   copyright (c) 1996-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute new arrays icurvs and curves in case the number of nodes at
!   one or more of the free boundaries may be changed
! **********************************************************************
!
!                       KEYWORDS
!
!     adapt_boundary
!     change_boundary
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icurvsold(0:*), lencurvs, iinput(*)
      double precision rinput(*), curvesold(2,*)
      logical changed

!     changed        o    If true the number of nodes along the boundary has
!                         been changed
!     curvesold      i    array containing the old coordinates of the curves
!     icurvsold      i    array containing the number of points in the old
!                         curves accumulated
!     iinput         i    integer input array, containing information of the
!                         mesh
!     lencurvs      i/o   At input: declared length of curvesmsh
!                         at output: actual length
!     rinput         i    Real input array containing information of the mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iinp, isum, isumold, ipos, nh, irnp, jinp, &
              icurve, nold, nelm, nsubcv, isub, isubcv, jsubcv, &
              jcurnr, ih, istartrel, istartcur
      integer, allocatable :: irelcurves(:), curveusrp1(:)
      double precision, allocatable :: work(:), curstr(:)

!     curstr         work array of length 10 * ncurvs to store some
!                    geometrical information of the curves temporarily
!     curveusrp1     integer work array of length 2+length curves
!     i              Counting variable
!     icurve         Type of curve
!     ih             Help variable to store a constant temporarily
!     iinp           last position used in array iinput
!     ipos           position used in array curvesmsh with respect to subcurve
!     irelcurves     dummy
!     irnp           position in array rinput
!     istartcur      starting position of in array curstr
!     istartrel      starting position of in array irelcurves
!     isub           Loop parameter
!     isubcv         Subcurve sequence number with direction
!     isum           Number of nodes in the previous old curves
!     isumold        Number of nodes in the previous new curves
!     jcurnr         Curve sequence number
!     jinp           Copy of iinp
!     jsubcv         |isubcv|
!     nelm           number of elements along a curve
!     nh             number of nodes along a subcurve
!     nold           Number of nodes along the old curve
!     nsubcv         Number of subcurves of a curve
!     work           Work array of length 16*number of points in the curves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHCURCR       Create curve
!     MSHCURNE       Compute number of elements along a curve
!     MSHSL2         Compute nelm for a free boundary
!     MSHSL3         Compute coordinates of a curve for a free boundary
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     initialize parameters
!     For icurnr := 1 (1) ncurvs do
!        Get icurve from iinput
!        if ( curve = composite curve ) then
!           Copy information from sub curves
!        else if ( icurve=0 ) then
!           n := 0
!        else if ( icurve<0 ) then
!           n := number of nodes along curve
!           Fill array curvesmsh with spline
!        else
!           n := old number of nodes along curve
!           Copy co-ordinates from curves to curvesmsh
!        Raise parameters
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcurfr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcurfr'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialize parameters

      irnp  = irnp1+(ndim+jcoars)*nuspnt+jcoars-1
      changed = .false.

      istartcur = 1
      istartrel = 2*ncurvs+1

      allocate ( irelcurves(10*ncurvs), work(32*lencurvs), &
                 curstr(10*ncurvs), curveusrp1(ncurvs+2+2*lencurvs), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, 20*ncurvs+32*lencurvs, 'irelcurves' )
      if ( ierror/=0 ) go to 1000

      irelcurves = 0

!     --- First fill number of nodes in icurvsmsh
!         Loop over curves

      do jcurnr = 1, ncurvs

!     --- Get icurve from iinput

         icurnr = iseqcurvs(jcurnr)
         iinp = ipointcurv(icurnr)
         icurve = iinput(iinp)
         if ( icurnr>1 ) then
            nold = icurvsold(icurnr)-icurvsold(icurnr-1)
         else
            nold = icurvsold(icurnr)
         end if
         if ( kelmq>0 ) then
            nsubcv = kmeshq(icurnr+1)-kmeshq(icurnr)
         else
            nsubcv = 1
         end if
         if ( nsubcv>1 ) then

!        --- Curve contains sub curves
!            Copy information from sub curves

            nc = 0
            do isub = 1, nsubcv
               isubcv = kmeshq(kmeshq(icurnr)+isub)
               jsubcv = abs(isubcv)
               nh = icurvsmsh(jsubcv)
               nc = nc+nh
            end do

         else if ( icurve==0 ) then

!        --- empty curve

            nc = 0

         else if ( icurve<0 .or. kmeshz(icurnr)==1 .and. &
                   ( icurve==8 .or. icurve==9 ) ) then

!        --- Compute number of nodes along curve

            icurve = abs(icurve)
            call mshsl2 ( icurnr, icurve, curvesold, iinp, iinput, &
                          rinput(3), nelm, icurvsold, elsize )
            ishape = iinput(iinp+1)
            nc = nelm*ishape+1
            if ( nc/=nold ) changed = .true.

         else if ( kmeshz(icurnr)==1 ) then

!        --- Curve does not belong to free boundary, but number of
!            nodes may have been changed
!            First compute number of nodes along elements

            call mshcurne ( iinput, icurvsmsh, rinput(3), curstr, icurve, &
                            iinp, irelcurves, istartcur, istartrel )
            ishape = iinput(iinp+1)
            nelm = iinput(iinp+2)
            nc = nelm*ishape+1
            if ( nc/=nold ) changed = .true.

         else

!        --- Copy co-ordinates from curvesold to curvesmsh

            nc = nold

         end if

!        --- Fill icurvsmsh

         icurvsmsh(icurnr) = nc

      end do

!     --- Accumulate icurvsmsh

      do icurnr = 2, ncurvs
         icurvsmsh(icurnr) = icurvsmsh(icurnr)+icurvsmsh(icurnr-1)
      end do
      if ( icurvsmsh(ncurvs)>lencurvs ) then
         call errint ( icurvsmsh(ncurvs)+1, 1 )
         call errint ( lencurvs, 2 )
         call errchr ( 'curvesmsh', 1 )
         call errsub ( 550, 2, 0, 1 )
      end if

!     --- Clear curfvefactsnw

      do i = 1, icurvsmsh(ncurvs)
         curvefacts(i) = 0d0
      end do

!     --- Finally fill curvesmsh
!         Loop over curves

      do jcurnr = 1, ncurvs

!     --- Get icurve from iinput

         icurnr = iseqcurvs(jcurnr)
         iinp = ipointcurv(icurnr)
         icurve = iinput(iinp)
         if ( icurnr>1 ) then
            isumold = icurvsold(icurnr-1)
            nold = icurvsold(icurnr)-isumold
            isum = icurvsmsh(icurnr-1)
            nc = icurvsmsh(icurnr)-isum
         else
            nold = icurvsold(icurnr)
            nc = icurvsmsh(icurnr)
            isum = 0
            isumold = 0
         end if
         if ( kelmq>0 ) then
            nsubcv = kmeshq(icurnr+1)-kmeshq(icurnr)
         else
            nsubcv = 1
         end if
         if ( nsubcv>1 ) then

!        --- Curve contains sub curves
!            Copy information from sub curves

            ih = 0
            do isub = 1, nsubcv
               isubcv = kmeshq(kmeshq(icurnr)+isub)
               jsubcv = abs(isubcv)
               if ( jsubcv==1 ) then
                  nh = icurvsmsh(1)
                  ipos = 0
               else
                  ipos = icurvsmsh(jsubcv-1)
                  nh = icurvsmsh(jsubcv)-ipos
               end if
               if ( isubcv>0 ) then
                  do i = 1, nh
                     curvesmsh(1,isum+i+ih) = curvesmsh(1,ipos+i)
                     curvesmsh(2,isum+i+ih) = curvesmsh(2,ipos+i)
                     curvefacts(isum+i+ih) = curvefacts(ipos+i)
                  end do
               else
                  do i = 1, nh
                     curvesmsh(1,isum+i+ih) = curvesmsh(1,ipos+nh+1-i)
                     curvesmsh(2,isum+i+ih) = curvesmsh(2,ipos+nh+1-i)
                     curvefacts(isum+i+ih) = curvefacts(ipos+nh+1-i)
                  end do
               end if
               ih = ih+nh-1
            end do

         else if ( icurve<0 .or. kmeshz(icurnr)==1 .and. &
                   ( icurve==8 .or. icurve==9 ) ) then

!        --- Compute number of nodes along curve

            icurve = abs(icurve)
            ishape = iinput(iinp+1)

!           --- Fill array curvesmsh with spline

            call mshsl3 ( curvesold(1,isumold+1), nold, curvesmsh(1,isum+1), &
                          iinp, iinput, rinput(3), work, &
                          icurve, curveusrp1, ndim, nuspnt, icurnr )
            iinput(iinp) = icurve

         else if ( kmeshz(icurnr)==1 ) then

!        --- Curve does not belong to free boundary, but number of
!            nodes may have been changed

            ishape = iinput(iinp+1)
            nelm = iinput(iinp+2)

!           --- next fill co-ordinates

            jinp   = iinp
            call mshcurcr ( iinput, rinput, curstr, icurve, jinp, &
                            irnp, isum+1, icurnr, nc, jcoars, &
                            ipointcurv, irelcurves, irnp1, istartrel )

         else

!        --- Copy co-ordinates from curves to curvesmsh
!            and from  curvefacts to curvefacts

            do i = 1, nc
               curvesmsh(1,isum+i) = curvesold(1,isumold+i)
               curvesmsh(2,isum+i) = curvesold(2,isumold+i)
               curvefacts(isum+i) = curvefacts(isumold+i)
            end do

         end if

      end do
      lencurvs = icurvsmsh(ncurvs)

      deallocate ( irelcurves, work, curstr, curveusrp1, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'curveusrp' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcurfr'

      end if  ! ( debug )
      call erclos ( 'mshcurfr' )

      end subroutine mshcurfr
