      subroutine mshcurne ( iinput, icurvs, rhinp, curstr, icurve, &
                            iinp, irelcurves, istartcur, istartrel )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.2  date 10-02-2015 Extension with ccircle
!        version  4.1  date 30-07-2012 Extra debug
!        version  4.0  date 02-09-2010 Remove intarmsh
!
!   copyright (c) 1996-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the number of nodes along one specific curve
!
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinp, icurve, icurvs(*)
      integer, intent(inout) :: irelcurves(*), istartcur, &
                                iinput(*), istartrel
      double precision, intent(in) :: rhinp(*)
      double precision, intent(inout) :: curstr(*)

!     curstr        i/o   work array of length 10 * ncurvs to store some
!                         geometrical information of the curves temporarily
!     icurve         i    Type of curve
!     icurvs         o    the number of nodal points in the curves is stored
!                         sequentially in this array
!     iinp           i    last position used in array iinput
!     iinput         i    standard input array for subroutine mesh
!                         it is supposed that the first five positions have been
!                         deleted
!     irelcurves    i/o   Contains integer information of the curves
!                         See subroutine mshcrtcurv
!     istartcur     i/o   next free position in curstr
!     istartrel     i/o   next free position in irelcurves
!     rhinp          i    Part of array rinput containing information about
!                         coarseness and factors
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nelm, ncrvs, j, jcurve, npnts, icount, nelm4, i, &
              icurnrorig, iwork(ncurvs), kcurve, kcount, nsubcv
      save icount, nelm4

!     i              Counting variable
!     icount         Counter
!     icurnrorig     Curve number of originating curve
!     iwork          integer work array of length ncurvs to store all the
!                    subcurve numbers of a composite curve
!     j              Counting variable
!     jcurve         Curve number
!     kcount         Counter
!     kcurve         abs(jcurve)
!     ncrvs          Number of curves
!     nelm           number of elements along a curve
!     nelm4          number of elements subdivided by 4
!     npnts          Number of elements along a curve
!     nsubcv         Number of subcurves of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH030         Fill array with coarseness values
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     124   code for generation of curves (icurve) incorrect
!     498   Curve number of curve to be translated or rotated too large
!     550   array length too short
!     602   curve contains curves with curve numbers larger than itself
!    1489   Empty curve found in curve of curves
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data icount /0/
! ======================================================================
!
      call eropen ( 'mshcurne' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcurne'
         write(irefwr,1) 'icurnr, istartcur, istartrel', &
                          icurnr, istartcur, istartrel
         if ( kelmq>0 ) &
            call prinin ( kmeshq, kmeshq(ncurvs+1), 'kmeshq' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )

      if ( debug ) then
         call prinrl1 ( rhinp, nuspnt, ndim, 'rhinp' )
         write(irefwr,2) 'edge', rhinp(nuspnt*ndim+1)
         call prinrl ( rhinp(nuspnt*ndim+2), nuspnt, 'coarse' )
      end if  ! ( debug )

      if ( icurve>24 ) then
         call errint ( icurve, 1 )
         call errint ( 24, 2 )
         call errsub ( 124, 2, 0, 0 )
         go to 1000
      end if

      if ( icurve==5 .or. icurve==6 .or. icurve==9 .or. &
           icurve==12 .or. icurve==16 .or. icurve==21 .or. &
           icurve==23 .or. icurve==24 ) then

!     --- icurve = 5, 6, 9, 12, 16, 21  Use coarseness

         call msh030 ( icurnr, ncurvs, iinput, iinp, rhinp, icurve, &
                       nelm, curstr, rhinp(ndim*nuspnt+1), ndim, &
                       nuspnt, rhinp(ndim*nuspnt+1), irelcurves, &
                       istartcur, istartrel )

      else if ( icurve==7 .or. icurve==14 ) then

!     --- icurve = 7 or 14, i.e curve of curves or
!         a spline through a number of curves, which are not used themselves

         if ( icurve==7 ) then
            iinput(iinp+1) = 1
            ncrvs = iinput(iinp+3)
         else
            ncrvs = iinput(iinp+6)
         end if
         nelm = 0

!        --- Store all subcurve numbers in iwork

         nsubcv = kelmq
         kcount = 0
         do j = 1, ncrvs
            if ( icurve==7 ) then
               jcurve = iinput(iinp+3+j)
            else
               jcurve = iinput(iinp+6+j)
            end if
            kcurve = abs(jcurve)

            if ( kcurve==0 .or. kcurve>ncurvs ) then

!           --- Subcurve number out of range

               call errint ( icurnr, 1 )
               call errint ( ncurvs, 2 )
               call errint ( jcurve, 3 )
               call errsub ( 602, 3, 0, 0 )
               go to 1000

            end if

            if ( kelmq>0 ) nsubcv = kmeshq(kcurve+1)-kmeshq(kcurve)
            if ( debug ) then

!           --- Debug information

               write(irefwr,1) 'j, ncrvs, icurve, jcurve, nsubcv', &
                                j, ncrvs, icurve, jcurve, nsubcv

            end if

            if ( nsubcv==0 ) then

!           --- No subcurves of subcurve

               kcount = kcount+1
               iwork(kcount) = jcurve

            else

!           --- Subcurve contains subcurves

               if ( jcurve>0 ) then

!              --- Positive direction

                  do i = kmeshq(kcurve)+1, kmeshq(kcurve+1)
                     kcount = kcount+1
                     iwork(kcount) = kmeshq(i)
                  end do

               else

!              --- Negative direction

                  do i = kmeshq(kcurve+1), kmeshq(kcurve)+1,-1
                     kcount = kcount+1
                     iwork(kcount) = -kmeshq(i)
                  end do

               end if

            end if
            if ( debug ) then

!           --- Debug information

               call prinin ( iwork, kcount, 'subcurves' )

            end if

         end do

         if ( kcount>ncurvs ) then

!        --- array length too short

            call errint ( kcount, 1 )
            call errint ( ncurvs, 1 )
            call errchr ( 'iwork', 1 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if

!        --- Loop over all subcurves

         do j = 1, kcount

            jcurve = iwork(j)
            npnts = icurvs(abs(jcurve))-1
            if ( npnts<=0 ) then

!           --- empty curve found

               call errint ( jcurve, 1 )
               call errint ( icurnr, 2 )
               call errsub ( 1489, 2, 0, 0 )
               npnts = 0

            end if

            if (  icurve==7 ) npnts = npnts+1
            nelm = nelm + npnts
            if ( debug ) &
               write(irefwr,1) 'j, jcurve, npnts, nelm', &
                                j, jcurve, npnts, nelm

         end do
         if ( icurve==14 ) then

!        --- icurve = 14, curves approximated by splines

            ishape = iinput(iinp+1)
            nelm = (nelm+ishape-1)/ishape

         else

!        --- icurve = 7, curve of curves

            nelm = nelm-1

         end if
         iinput(iinp+2) = nelm
         if ( debug ) write(irefwr,1) 'nelm', nelm

      else if ( icurve==10 .or. icurve==11 .or. &
                icurve==13 ) then

!     --- icurve = 10, 11, 13 translate, rotate or reflect
!         Find originating curve jcurve

         iinput(iinp+1) = 1
         jcurve = iinput(iinp+4)
         if ( jcurve<=0 ) then
            call errint ( jcurve, 1 )
            call errint ( icurnr, 2 )
            call errsub ( 498, 2, 0, 0)
            go to 1000
         end if
         if ( kelmq==0 ) then

!        --- kelmq, hence there are no composite curves, extract nelm from
!            icurvs

            nelm = icurvs(jcurve)-1

         else

!        --- There are composite curves

            nsubcv = kmeshq(jcurve+1)-kmeshq(jcurve)
            if ( nsubcv<=1 ) then

!           --- at most one subcurve

               nelm = icurvs(jcurve)-1

            else

!           --- more subcurves

               nelm = icurvs(jcurve)-nsubcv

            end if

         end if
         iinput(iinp+2) = nelm

      else if ( icurve==18 .or. icurve==19 ) then

!     --- Special cases of parts of a circle (4 in a row) or
!         constructing curves for sphere (also 4 in a row)
!         It is not possible to give the individual number of elements, &
!         but the sum of 4 can be detected
!         The first estimate must be changed later on

         if ( icount==0 ) then

!        --- icount = 0, first of 4

            ishape = iinput(iinp+1)
            icurnrorig = iinput(iinp+3)
            nelm = icurvs(icurnrorig)
            nelm = (nelm+ishape-2)/ishape
            nelm4 = nelm/4
            iinput(iinp+2) = nelm-3*nelm4
            icount = 1

         else

!        --- icount>0, next of 4

            iinput(iinp+2) = nelm4
            icount = icount+1
            if ( icount==4 ) icount = 0

         end if

      end if
1000  call erclos ( 'mshcurne' )
      end
