      subroutine mshcursphere ( iinput, iwork1, iinp, rinput, curves, &
                                ishape, icurnr, icurvs, inpncv, tau, &
                                nuspnt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-07-2012 Fortran 90
!        version  1.0  date 17-07-1999
!
!   copyright (c) 1999-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Construct 4 generating curves from circle to upper or lower point of
!     sphere
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     curve
!     mesh
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iwork1(*), iinp, ishape, inpncv, icurnr, &
              icurvs(*), nuspnt
      double precision rinput(3,*), curves(3,*), tau(*)

!     curves        i/o   array containing the coordinates of the curves
!     icurnr         i    Curve sequence number
!     icurvs         i    array containing the number of points in the curves
!                         accumulated
!     iinp          i/o   last position used in array iinput
!     iinput         i    integer input array
!     inpncv         i    Number of points in curve
!     ishape         i    Shape number of elements
!     iwork1         i    integer work array
!     nuspnt         i    Number of user points
!     rinput         i    Real input array
!     tau                 Work array of length inpncv to store the relative
!                         distances between points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision  xc(3), tang1(3), tang2(3), curstr(1), lenvec, &
                       radius, radius2, radiusinv, t1(3), t2(3), &
                       deltaphi(0:4), normal(3), dx, sum, alva, s, dtau
      integer i, j, ivert(5), icentre, iinplocal, icurnrpart, &
              ifirst, istartpart, nelm, iinputloc(3), idirec, &
              iinppart, ipart, itop

!     alva           helpvariable for computation of new internal points
!     curstr         Dummy
!     deltaphi       Contains the relative arc length of the elements in the
!                    neighbourhood of the 4 vertices measured in degrees
!                    deltaphi(0) is average
!     dtau           difference of tau values
!     dx             length of vector in x-direction
!     i              Counting variable
!     icentre        User point number of centroid
!     icurnrpart     Curve number of part of circle
!     idirec         Defines where the point not on the circle is positioned
!                    idirec = 1, top point, idirec = -1, lower point
!     ifirst         First curve (surface) number read
!     iinplocal      Starting address of iinput for local curve
!     iinppart       Starting address of iinput for part of circle
!     iinputloc      Local array iinput
!     ipart          Sequence number of part
!     istartpart     Starting position of circle part in curves
!     itop           User point number of top point
!     ivert          Contains the relative numbers of the 4 vertices along the
!                    circle
!     j              Counting variable
!     lenvec         Help parameter to indicate the length of arrays
!     nelm           number of elements along a curve
!     normal         Contains the normal on the plane
!     radius         Radius of sphere or circle
!     radius2        radius^2
!     radiusinv      1/radius
!     s              Help parameter
!     sum            Help parameter to compute a sum
!     t1             First tangential vector
!     t2             Second tangential vector
!     tang1          First tangential vector
!     tang2          Second tangential vector
!     xc             array with the coordinates of the centre point of the
!                    element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELOUTR         Compute outer product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH006         Fill array curves in case of circle segment
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Find 4 points on circle that subdivide the circle approximately into
!    4 equal parts
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcursphere' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcursphere'
         write(irefwr,1) 'ishape, icurnr', ishape, icurnr
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- extract information from iinput

      iinplocal  = iwork1(icurnr)-1

      if ( debug ) write(irefwr,1) 'iinplocal', iinplocal

      icurnrpart = iinput(iinplocal+5)
      idirec     = iinput(iinplocal+6)
      ifirst     = iinput(iinplocal+7)
      itop       = iinput(iinplocal+8)
      icentre    = iinput(iinplocal+9)

      if ( debug ) &
         write(irefwr,1) 'icurnrpart, idirec, ifirst, itop, icentre', &
                          icurnrpart, idirec, ifirst, itop, icentre

!     --- Store the coordinates of the centre in xc and create first tangential
!         vector

      xc = rinput(:,icentre)
      tang1 = rinput(:,ifirst)-xc

      radius2 = dot_product(tang1,tang1)
      radius = sqrt(radius2)
      radiusinv = 1d0/radius

      if ( debug ) write(irefwr,2) 'xc, radius', xc, radius

!     --- Find the average length of the elements in these 4 points
!         Store in deltaphi (radians)

      istartpart = icurvs(icurnrpart-1)
      ivert(1) = istartpart+1
      do ipart = 1, 4
         iinppart = iwork1(icurnrpart+ipart-1)-1
         nelm = iinput(iinppart+3)
         inpncv = nelm*ishape+1
         ivert(ipart+1) = ivert(ipart)+inpncv
      end do

      t1 = curves(:,ivert(5)-1-ishape)-xc
      t2 = curves(:,ivert(1)+ishape)-xc

      deltaphi(1) = acos(dot_product(t1,t2)/radius2)*0.5d0
      do j = 2, 4
         t1 = curves(:,ivert(j)-1-ishape)-xc
         t2 = curves(:,ivert(j)+ishape)-xc
         deltaphi(j) = acos(dot_product(t1,t2)/radius2)*0.5d0
      end do

!     --- The average length of the 4 average lenths of the circle points is
!         the element length in the neighbourhood of the upper and lower point

      deltaphi(0) = 0.25d0 * ( deltaphi(1)+deltaphi(2)+ &
                               deltaphi(3)+deltaphi(4) )

      if ( debug ) call prinrl ( deltaphi(0), 5, 'deltaphi' )

!     --- Construct the upper or lower point

      tang2 = rinput(:,ifirst+1)-xc

      tang1 = tang1*radiusinv
      tang2 = tang2*radiusinv
      call eloutr ( tang1, tang2, normal )

      if ( debug ) then
         call prinrl ( tang1, 3, 'tang1' )
         call prinrl ( tang2, 3, 'tang2' )
         call prinrl ( normal, 3, 'normal' )
      end if  ! ( debug )

      lenvec = 1d0/sqrt(dot_product(normal,normal))
      normal = normal*lenvec

      rinput(:,itop) = xc+radius*normal*idirec

!     --- Fill array tau along the curve

      dx = 1d0 / ishape
      do ipart = 1, 4

         iinppart = iwork1(icurnrpart+ipart-1)-1
         nelm = iinput(iinppart+3)
         inpncv = nelm*ishape+1
         iinput(iinplocal+3+(ipart-1)*9) = nelm

         if ( debug ) &
            write(irefwr,1) 'iinppart, nelm, inpncv, ishape, ipart', &
                             iinppart, nelm, inpncv, ishape, ipart
         tau(1) = 0d0
         tau(nelm) = 1d0
         alva = ( deltaphi(0) / deltaphi(ipart) )**(1d0/(nelm/ishape-1))
         sum = 1d0
         do i = 1, nelm/ishape-1
            sum = alva * sum + 1d0
         end do
         s = 1d0
         do i = 1+ishape, nelm, ishape
            tau(i) = s/sum
            s = alva * s + 1d0
         end do
         if ( ishape>1 ) then

!        --- ishape>1, Fill points in between

            do i = 2, nelm, ishape
               dtau = tau(i+ishape-1)-tau(i-1)
               do j = 1, ishape-1
                  tau(i+j-1) = tau(i-1) + dtau * dx*j
               end do
            end do

         end if
         icurvs(icurnr+ipart-1) = icurvs(icurnr+ipart-2)+inpncv
         iinputloc(1) = iinput(iinplocal+7+(ipart-1)*9)
         iinputloc(2) = itop
         iinputloc(3) = icentre
         call msh006 ( .false., 3, curves(1,icurvs(icurnr+ipart-2)+1), &
                       iinputloc, rinput, tau, inpncv, nuspnt, curstr, &
                       icurnr+ipart-1, 3 )

      end do

      iinp = iinp+5

1000  call erclos ( 'mshcursphere' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcursphere'

      end if  ! ( debug )

      end
