      subroutine mshcut ( kmeshlc, kproblc, intmatlc, isol, coorln, icutcrnr, &
                          iread )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 20-08-2016 use sepmoduleinput
!        version  4.2  date 29-05-2014 Extension of timebefore
!        version  4.1  date 15-11-2016 Long integers
!
!   copyright (c) 1996-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Cut free boundary with given line, and remove all points below
!   that line. The cut curve is reset to the new cutting line
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     free_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduleintmat
      use sepmoduleinput
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshlc, kproblc, isol(*), icutcrnr, iread
      integer, target :: intmatlc(*)
      double precision coorln(2,2)

!     coorln     i    Array containing the co-ordinates of two points
!                     at the cutting line
!                     coorln(1,i) = x_i;  coorln(2,i) = y_i;
!     icutcrnr   i    Contains curve number corresponding to cut line, i.e.
!                     the curve icutcrnr is reset to the cut line
!     intmatlc  i/o   Standard SEPRAN array of length 5 containing information
!                     concerning the structure of the matrix. See the
!                     Programmer's Guide 13.5 for a complete description.
!     iread      i    Defines how the parameters must be read
!                     Possible values:
!                     0   The input is read from the standard input file
!                     >0  The input as read from the standard input file by
!                         subroutine SEPSTM is used. The value of iread
!                         defines the sequence number
!     isol      i/o   Standard SEPRAN array containing information of the
!                     solution array
!     kmeshlc   i/o   Standard SEPRAN array containing information of the mesh
!     kproblc   i/o   Standard SEPRAN array, containing information of
!                     the problem
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer niinp
      parameter ( niinp=40 )
      integer icurvs, iinput(niinp), &
              lencurvs, leniinp, mmiinput, mmrinput, maxcal, ipos
      double precision, allocatable :: curves(:,:), curvesnw(:,:)
      double precision :: timebefore(2)

!     curves         array containing the coordinates of the curves
!     curvesnw       array containing the new coordinates of the curves
!                    The length might be larger than that of curves
!                    Overestimate: 5*length curves
!     icurvs         array containing the number of points in the curves
!                    accumulated
!     iinput         integer input array
!     ipos           Position of input arrays in sepmoduleinput
!     lencurvs       Overestimate of the number of points along the new curves
!     leniinp        Length of array iinput
!     lenprb         Length of array kprob
!     maxcal         Maximum number of coefficient inputs read
!     mmiinput       Memory management sequence number with respect to iinput
!     mmrinput       Memory management sequence number with respect to rinput
!     niinp          Declared length of array iinput
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLMN         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     INICHINPSOL    Adapt input array with respect to variables
!     MSHCUT01       Cut free boundary with given line, and remove all points
!                    below that line.
!     MSHREMSH       Perform remeshing
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1948   No input with the main keyword has been read
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initialize parameters
!    Cut mesh
!    perform remeshing
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcut' )
      if ( itime>0 ) call ersettime ( timebefore )

      intmat1d => intmatlc(1:5)
      kprob2d(1) = kproblc
      kmesh2d(1) = kmeshlc
      kmesh => kmesh2d(1)
      kprob => kprob2d(1)

      call sepfillkmesh(kmesh)

!     --- Initialize parameters

      iinp1 = 16
      irnp1 = 1
      lencrv = (kmeshm(3)-kmeshm(2))
      lencurvs = size ( curvesmsh,2)

!     --- The following temporary arrays are created: curvesnw

      allocate ( curvesnw(ndim,lencurvs*5), stat = error )
      if ( error/=0 ) call eralloc ( error, lencurvs, 'curvesnw' )
      if ( ierror/=0 ) go to 1000

!     --- Permanent arrays

      allocate ( curves(2,lencrv), stat = error )
      if ( error/=0 ) call eralloc ( error, lencrv, 'curves' )
      if ( ierror/=0 ) go to 1000

!     --- Cut mesh
!         Nodes are redistributed, recompute icurvs and curves
!         First copy old array curves to curves and icurvs to icurvscp
!         icurvscp is extended with an extra position containing a 0

      curves(:,1:lencurvs) = curvesmsh
      lencurvs = lencurvs*5
      call mshcut01 ( curves, curvesnw, &
                      lencurvs, coorln, icutcrnr )

!     --- Copy new arrays curves to old ones

      if ( associated(km(iseqkm)%curvesmsh) ) &
         deallocate(km(iseqkm)%curvesmsh)
      allocate( km(iseqkm)%curvesmsh(ndim,lencurvs) )
      curvesmsh =>km(iseqkm)%curvesmsh
      curvesmsh = curvesnw(:,1:lencurvs)
      km(iseqkm)%kmeshh(5,1) = lencurvs

      deallocate ( curves, curvesnw, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'curves' )
      if ( ierror/=0 ) go to 1000

!     --- perform remeshing

      call sepgetinpfinf ( 19, iread, ipos, maxcal )
      if ( maxcal==0 ) then
         call errchr ( 'CUT MESH', 1 )
         call errsub ( 1948, 0, 0, 1 )
         go to 1000
      end if

      leniinp = size( in(ipos)%iinput )
      iinput(1:leniinp) = in(ipos)%iinput(1:leniinp)

!     --- Adapt variables

      call inichinpsol ( iinput, leniinp )
      call mshremsh ( isol, iread, iinput, 1 )

1000  call erclmn ( 'mshcut', timebefore )

      end subroutine mshcut
