      subroutine mshg11 ( bcord, nbcrd, nbndpt, ja )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.1  date 05-02-2003 Correction for holes in boundary
!        version  2.0  date 25-02-2001 extra parameter nbcrd
!                                      complete revision
!        version  1.3  date 21-02-1999 Include common
!        version  1.2  date 10-10-1990 adjusted formulation for xleft
!        version  1.1  date 07-09-1990 renamed version of msho35
!        version  1.0  date 17-06-1990
!
!   copyright (c) 1990-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Subroutine to determine whether the boundary as given by array
!   BCORD is clockwise or counterclockwise.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     check
!     counter_clockwise
!     boundary
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nbndpt, ja, nbcrd
      double precision bcord(nbcrd,*)

!     bcord          i    coordinate array for boundary points
!     ja             o    indication whether boundary is in clockwise (ja=0)
!                         or in counterclockwise direction (ja=1)
!     nbcrd          i    first dimension of bcord
!     nbndpt         i    number of points in boundary, bcord has length
!                         nbndpt + 1 because last point is first point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision eps, xmin, xmax, ymin, ymax, x1, x2, y1, y2, y, &
                       xmid, t1, ds, ycross, alpha, xlow, xhigh
      integer j, iseq, ntop
      logical finished

!     alpha          help parameter to compute cross section
!     ds             Minimum distance between two points along a curve
!     eps            Local accuracy
!     finished       Indicates if the loop has been finished (true) or not
!                    (false)
!     iseq           Sequence number of boundary interval
!     j              Counting variable
!     ntop           Index of last point in outer boundary
!     t1             First component to compute the tangential vector
!     x1             x-coordinate of first point
!     x2             x-coordinate of last point
!     xhigh          min(x1,x2)
!     xlow           max(xx1,x2)
!     xmax           Maximum x-value
!     xmid           Mean value of xmin,xmax
!     xmin           Minimum x-value
!     y              y co-ordinate of point
!     y1             y-coordinate of first point in curve
!     y2             y-coordinate of last point in curve
!     ycross         Lowest y-value of all cross-section with x=xmid
!     ymax           Maximum y-value
!     ymin           Minimum y-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    ds := smallest length of boundary elements
!    xmin,xmax,ymin,ymax := extreme value of boundary
!    xmid, ymid := mean values of (xmin,xmax), (ymin,ymax)
!    eps := 1d-3*ds
!    xmid := xmid + 0.1 ds
!    finished := false
!    While not finished do
!       ycross = ymax+1d3
!       finished := true
!       For i := 1 (1) nbndpt-1 do
!          if ( boundary part i crosses x=xmid in y ) then
!             if ( boundary part i parallel to x=xmid ) then
!                finished := false
!                xmid := xmid+0.1 ds
!             else
!                if ( y < ycross ) then
!                   ycross = y
!                   iseq = i
!    t := tangential vector iseq
!    if ( t1 < 0 ) then
!       counter clockwise
!    else
!       clockwise
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshg11' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshg11'

      end if

!     --- ds := smallest length of boundary elements

      ds = rinfin

      do j = 2, nbndpt
         ds = min(ds,(bcord(1,j)-bcord(1,j-1))**2+ &
                     (bcord(2,j)-bcord(2,j-1))**2)
      end do
      ds = sqrt(ds)

!     --- xmin,xmax,ymin,ymax := extreme value of boundary

      xmin = rinfin
      xmax = -rinfin
      ymin = xmin
      ymax = xmax
      do j = 1, nbndpt
         xmin = min(bcord(1,j),xmin)
         xmax = max(bcord(1,j),xmax)
         ymin = min(bcord(2,j),ymin)
         ymax = max(bcord(2,j),ymax)
      end do

!     --- xmid, ymid := mean values of (xmin,xmax), (ymin,ymax)
!         eps := 1d-3*ds
!         xmid := xmid + 0.1 ds
!         finished := false

      xmid = 0.5d0*(xmin+xmax)
      eps = 1d-3*ds
      xmid = xmid+0.1d0*ds
      finished = .false.
      if ( debug ) then

!     --- Debu information

         write(irefwr,*) 'nbndpt, ds, eps ', nbndpt, ds, eps
         write(irefwr,*) 'xmin, xmax, ymin, ymax, xmid ', &
                          xmin, xmax, ymin, ymax, xmid
         call prinrl1 ( bcord, nbndpt, nbcrd, 'bcord' )

      end if

!     --- Find end of outer boundary

      ntop = nbndpt
      if ( nbcrd.eq.4 ) then

!     --- nbcrd = 4: paver, so perhaps more parts

         do j = 1, nbndpt

            if ( bcord(3,j).lt.0 ) then

!           --- bcord(3,j) < 0, hence new part of boundary detected

               ntop = j-1
               go to 100

            end if

         end do

      end if

!     --- While not finished do

100   do while ( .not. finished )

!     --- ycross := ymax+1d3
!         finished := true

         ycross = ymax+1d3
         finished = .true.

!        --- For i := 1 (1) nbndpt do

         do j = 1, ntop

            x1 = bcord(1,j)
            y1 = bcord(2,j)
            if ( j.lt.ntop ) then
               x2 = bcord(1,j+1)
               y2 = bcord(2,j+1)
            else
               x2 = bcord(1,1)
               y2 = bcord(2,1)
            end if
            xlow = min(x1,x2)-eps
            xhigh = max(x1,x2)+eps

            if ( xmid.ge.xlow .and. xmid.le.xhigh ) then

!           --- boundary part i crosses x=xmid in y

               if ( abs(x2-x1).lt.eps ) then

!              --- boundary part i parallel to x=xmid ) then
!                   finished := false
!                   xmid := xmid+0.1 ds

                  finished = .false.
                  xmid = xmid+0.1*ds
                  if ( debug ) write(irefwr,*) 'j, xmid ',j, xmid
                  go to 100

               else

!              --- intersection possible, compute y

                  alpha = (xmid-x1)/(x2-x1)
                  y = y1+alpha*(y2-y1)

                  if ( y.lt.ycross ) then

!                 --- y < ycross, hence new candidate found
!                     ycross = y
!                     iseq = i

                     ycross = y
                     iseq = j
                     if ( debug ) write(irefwr,*) 'iseq, y ', iseq, y

                  end if

               end if

            end if

         end do

      end do

!     --- t := tangential vector iseq

      if ( iseq.lt.ntop ) then
         t1 = bcord(1,iseq+1) - bcord(1,iseq)
      else
         t1 = bcord(1,1) - bcord(1,iseq)
      end if
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'iseq, x ', iseq, bcord(1,iseq),bcord(1,iseq+1)
         write(irefwr,*) 't1 ', t1

      end if
      if ( t1.gt.0d0 ) then

!     --- t1 > 0, hence counter clockwise

         ja = 1
         if ( debug ) write(irefwr,*) 'counter clockwise'

      else

!     --- clockwise

         ja = 0
         if ( debug ) write(irefwr,*) 'clockwise'

      end if

      call erclos ( 'mshg11' )
      if ( debug ) write(irefwr,*) 'End mshg11'

      end
