      subroutine mshisp ( jnew, bcord, kbndpt, &
                          boundary, numcurvboun, &
                          iinput, rinput, holeinfo, nholes )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  8.2  date 29-05-2014 Extension of timebefore
!        version  8.1  date 22-01-2014 New call to mshoce
!        version  8.0  date 02-11-2010 Dynamical allocation workarrays
!
!   copyright (c) 1994-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Generate a mesh of triangles for a 3D surface area which is
!     given by the line elements of BCORD. Elements are linear or qua- &
!     dratic  (INPELM = 3 , 6 or 7)
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     isoparametric_projection
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kbndpt(*), &
              iinput(*), numcurvboun, boundary(2,numcurvboun), &
              nholes, holeinfo(2,0:nholes+1)
      double precision :: bcord(ndim,*), rinput(*)
      logical jnew

!     bcord          i    coordinates of boundary of mesh
!     boundary            ?
!     holeinfo      i/o   Integer array of size 2 x (nholes+2)
!                         Contains information about holes in surfaces and where
!                         a new part of the boundary starts
!                         Pos (1,i-1) contains the local sequence number of the
!                               first curve of part i provided with a sign
!                               If the sign is positive the boundary is
!                               created counter clockwise, otherwise clockwise
!                         Pos (2,i-1) contains the local sequence number of the
!                               first node of of part i
!                         Pos (1,nholes+1) contains the number of curves of the
!                         boundary
!     iinput         i    array with the numbers of the three constituting
!                         curves as well as some other parameters:
!                         4: type of mapping
!                         5: icentre
!                         6: iregular
!     jnew           i    logical to indicate whether new mesh (true) or old
!                         mesh (false) is considered
!     kbndpt         o    helparray to store the node numbers of the boundary
!     nholes         o    Number of holes in the boundary of a surface
!     numcurvboun    i    number of curves in boundary
!     rinput         i    real input array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, i, ip(6), itimo, ncn(3), nbound, &
              icase, iregular
      double precision, allocatable :: bcornw(:,:), coortmp(:,:)
      double precision xp(3), xst(3), dx(3), ds, &
                       x1(6), x2(6), x3(6), xnew(3), ski, eta, &
                       ez1, ez2, ez3
      double precision :: timebefore(2)

!     bcornw         array containing the coordinates of the projected
!                    boundaries of the triangular region
!     coortmp        Temporary array to store the 2d coordinates
!     ds             distance
!     dx             length of vector in x-direction
!     eta            second coordinate in projection plane
!     ez1            length of side 1
!     ez2            length of side 2
!     ez3            length of side 3
!     i              Counting variable
!     icase          Case indicator
!     ip             help array for the nodes on which the transformation
!                    is based
!     iregular       Defines the type of subdivision
!                    0: default
!                    1: regular
!     itimo          variable to store time indicator temporarily
!     j              Counting variable
!     nbound         number of nodes on boundary of region considered
!     ncn            help array to store the number of nodes in the three
!                    constituting curves
!     ski            first coordinate in projection plane
!     x1             array with the six values of the first coordinates of the
!                    six basis points
!     x2             array with the six values of the second coordinates of the
!                    six basis points
!     x3             array with the six values of the third coordinates of the
!                    six basis points
!     xnew           the coordinates of the new created point
!     xp             x-coordinate of point
!     xst            x-coordinate of first point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHECKRANGE     Check range of integer parameter
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     MSHG23         determine new coordinates in isoparametric bijection
!                    for point ski,eta
!     MSHISPQUAD     Create a quadrilateral mesh in a triangle
!     MSHMAPSUR3D    Map surface in 3D onto prescribed type of surface
!     MSHOCE         Submesh generator TRIANGLE
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
!     PRINTTIME      Print the CPU time at end of subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     First the number of points in the three constituting sides has
!     to be determined
!     Transform points of BCORD to standard triangle
!     Call standard routine for triangulation in 2D, determine
!     internal points and elements
!     Make back transformation for all the points using six points
!     and an isoparametric transformation
!     Compute coordinates for all points, elements do not change
!     Replace all boundary points by the old ones
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('mshisp')
      if ( itime>0 ) call ersettime ( timebefore )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshisp'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icentre = iinput(5)
      mapping3d = iinput(4)
      iregular = iinput(6)
      if ( debug ) &
        write(irefwr,1) 'icentre, isurnr, mapping3d, iregular', &
                         icentre, isurnr, mapping3d, iregular

      call checkrange ( mapping3d, 0, 2, 4, 'mapping3d', 'iinput' )
      call checkrange ( iregular, 0, 1, 6, 'iregular', 'iinput' )
      if ( mapping3d==1 ) then
         call checkrange ( icentre, 1, nuspnt, 5, 'icentre', &
                           'iinput' )
      end if  ! ( mapping3d==1 )
      if ( ierror/=0 ) go to 1000

!     --- Determine the number of points in the three constituting sides

      ncn(1) = nmes
      ncn(2) = mmes
      ncn(3) = lmes
      nbound = ncn(1) + ncn(2) + ncn(3) - 2
      if ( debug ) then
         write(irefwr,1) 'ncn(1-3), nbound', &
                          ncn(1), ncn(2), ncn(3), nbound
         call prinrl1 ( bcord, nbound, ndim, 'bcord' )
      end if  ! ( debug )

!     --- Allocate space for bcornw

      length = size(coor2d,2)
      allocate( bcornw(2,nbound), coortmp(2,length), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*nbound+2*length, 'bcornw' )

!     --- Transform points of BCORD to standard triangle

      do i = 1, ndim
         xst(i) = bcord(i,1)
      end do

      bcornw(1,1) = 0

      dx(1) = 0
      dx(2) = 0
      dx(3) = 0

      do i = 2, nbound

         do j = 1, ndim

            xp(j)  =  bcord(j,i)
            dx(j)  =  xp(j) - xst(j)
            xst(j) =  xp(j)

         end do

         ds = sqrt( dx(1)*dx(1)+dx(2)*dx(2)+dx(3)*dx(3) )

         bcornw(1,i) = bcornw(1,i-1) + ds
         bcornw(2,i) = 0

      end do

!     --- Compute length of each of the sides:

      ez1 = bcornw(1, ncn(1) )
      ez2 = bcornw(1, ncn(1) + ncn(2) - 1 ) - ez1
      ez3 = bcornw(1, nbound) - ez1 - ez2

!     --- Check which mapping3d should be used

      if ( ez1>=ez2 .and. ez1>=ez3 ) then
           icase = 1
      else if ( ez2>=ez1 .and. ez2>=ez3 ) then
           icase = 2
      else
           icase = 3
      end if

      if ( mapping3d==2 ) then

!     --- mapping3d = 2, corresponds to old_sphere, set mapping3d back to 1
!                      and icase back to 1

         mapping3d = 1
         icase = 1

      end if  ! ( mapping3d==1 )

!     --- Place points exactly on the three basis sides
!         The longest side should be corresponding to the hypothenusa

      if ( icase==1 ) then

!     --- First side should be the hypothenusa
!         Start with first side : Points 1 - ncn(1)

         do i = 1, ncn(1)

            bcornw(2,i) = bcornw(1,i) / ez1
            bcornw(1,i) = 1d0 - bcornw(2,i)

         end do

!        --- Second side : Points ncn(1) - (ncn(1)+ncn(2)-1)

         j    = ncn(1) + ncn(2) - 1

         do i = ncn(1)+1 , j

            bcornw(2,i) = 1d0 - ( bcornw(1,i) - ez1 ) / ez2
            bcornw(1,i) = 0d0

         end do

!        --- Third side : Points ncn(1)+ncn(2)-1  -  nbound

         j     =  nbound

         do i = ncn(1)+ncn(2) , j

            bcornw(1,i) = ( bcornw(1,i) - ez1 - ez2 ) / ez3
            bcornw(2,i) = 0d0

         end do

      else if ( icase==2 ) then

!     --- Second side is hypothenusa
!         First side : Points 1 - ncn(1)

         do i = 1 , ncn(1)

            bcornw(1,i) = bcornw(1,i) / ez1
            bcornw(2,i) = 0d0

         end do

!        --- Second side : Points ncn(1) - (ncn(1)+ncn(2)-1)

         j    = ncn(1) + ncn(2) - 1

         do i = ncn(1)+1 , j

            bcornw(2,i) = ( bcornw(1,i) - ez1 ) / ez2
            bcornw(1,i) = 1d0 - bcornw(2,i)

         end do

!        --- Third side : Points ncn(1)+ncn(2)-1  -  nbound

         j     =  nbound

         do i = ncn(1)+ncn(2) , j

            bcornw(2,i) = 1d0 - ( bcornw(1,i) - ez1 - ez2 ) / ez3
            bcornw(1,i) = 0d0

         end do

      else

!     --- Third side is longest one:
!         First side : Points 1 - ncn(1)

         do i = 1 , ncn(1)

            bcornw(2,i) = 1d0 - ( bcornw(1,i) / ez1 )
            bcornw(1,i) = 0d0

         end do

!        --- Second side : Points ncn(1) - (ncn(1)+ncn(2)-1)

         j    = ncn(1) + ncn(2) - 1

         do i = ncn(1)+1 , j

            bcornw(1,i) = ( bcornw(1,i) - ez1 ) / ez2
            bcornw(2,i) = 0d0

         end do

!        --- Third side : Points ncn(1)+ncn(2)-1  -  nbound

         j     =  nbound

         do i = ncn(1)+ncn(2) , j

            bcornw(2,i) = ( bcornw(1,i) - ez1 - ez2 ) / ez3
            bcornw(1,i) = 1d0 - bcornw(2,i)

         end do

      end if

      if ( debug ) then
         call prinrl1 ( bcord, nbound, 2, 'bcornw' )
      end if  ! ( debug )

!     --- Call routine for triangulation

      itimo = itime

      itime = 0
      if ( ierror/=0 ) go to 1000

      if ( iregular==0 .and. &
           (inpelm==3 .or. inpelm==6 .or. inpelm==7) ) then

!     --- triangular elements

         if ( ierror/=0 ) go to 1000
         ncoar = 0        ! no extra points
         nextracrvs = 0   ! no extra curves
         if ( debug ) write(irefwr,*) 'Use triangle'
         call mshoce ( jnew, coortmp, kmeshclc, inpelm, nbound, bcornw, &
                       kbndpt, boundary, numcurvboun, npointlc, nelemlc, &
                       holeinfo, nholes, ncoar, coortmp, ip, &
                       isurnr, nextracrvs, ip, ip, &
                       rinput, nuspnt, ndim, coortmp, coortmp )

      else if ( inpelm==4 .or. inpelm==9 .or. iregular==1 .and. &
               (inpelm==3 .or. inpelm==6) ) then

!     --- quadrilateral elements or regular subdivision

         if ( debug ) write(irefwr,*) 'Use mshispquad'
         call mshispquad ( ncn, bcornw, coortmp, kbndpt )

      else

!     --- shape number not implemented

         call errint ( ishape, 1 )
         call errsub ( 800, 1, 0, 0 )
         go to 1000

      end if
      itime = itimo

!     --- deallocate bcornw

      deallocate ( bcornw, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'bcornw' )

!     --- Make back transformation

      if ( icase==1 ) then

         ip(1) = ncn(1) + ncn(2) - 1
         ip(2) = ip(1) + ncn(3) / 2
         ip(3) = 1
         ip(4) = ip(3) + ncn(1) / 2
         ip(5) = ncn(1)
         ip(6) = ip(5) + ncn(2) / 2

      else if ( icase==2 ) then

         ip(1) = 1
         ip(2) = ip(1) + ncn(1) / 2
         ip(3) = ncn(1)
         ip(4) = ip(3) + ncn(2) / 2
         ip(5) = ncn(1) + ncn(2) - 1
         ip(6) = ip(5) + ncn(3) / 2

      else

         ip(1) = ncn(1)
         ip(2) = ip(1) + ncn(2) / 2
         ip(3) = ncn(1) + ncn(2) - 1
         ip(4) = ip(3) + ncn(3) / 2
         ip(5) = 1
         ip(6) = ip(5) + ncn(1) / 2

      end if

      do i = 1, 6

          x1(i) = bcord(1,ip(i))
          x2(i) = bcord(2,ip(i))

          if ( ndim==3 ) x3(i) = bcord(3,ip(i))

      end do

!     --- Compute coordinates for all points

      do i = npointlc, 1, -1

          ski = coortmp(1,i)
          eta = coortmp(2,i)

          call mshg23( ski, eta, x1, x2, x3, ndim, xnew )
          do j = 1, ndim

              coor2d(j,i) = xnew(j)

          end do

      end do

!     --- deallocate coortmp

      deallocate ( coortmp, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'coortmp' )

!     --- Replace all boundary points

      do j = 1, ndim
         do i = 1, nbound-1
            coor2d(j,kbndpt(i)) = bcord(j,i)
         end do
      end do

      if ( mapping3d>0 ) then

!     --- mapping3d>0, map on 3d surface that is given exactly

         call mshmapsur3d ( bcord, coor2d, nbound-1, npointlc, mapping3d, &
                            isurnr, icentre, rinput )
      end if

      if ( itime==1 ) call printtime('ISOPAR ')

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kbndpt, nbound, 'kbndpt' )
         write(irefwr,*) 'End mshisp'

      end if  ! ( debug )
      call erclos( 'mshisp' )

      end
