      subroutine mshispquad ( ncn, bcord, coorlc, kbndpt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 02-09-2010 Remove intarmsh
!        version  2.2  date 27-10-2003 Debug statements
!        version  2.1  date 20-02-2002 Extra error message
!
!   copyright (c) 1999-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Generate a mesh of quadrilaterals for a 2/3D surface area which is
!     given by the line elements of BCORD. Elements are linear or qua- &
!     dratic  (INPELM = 4 or 9)
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     isoparametric_projection
!     triangle
!     quadrilateral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleplot
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncn(3), kbndpt(*)
      double precision bcord(2,*), coorlc(2,*)

!     bcord          i    array containing the coordinates of the boundaries
!     coorlc        o    array of length ndim x npointlc containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     kbndpt         o    array containing the nodal point numbers of the
!                         boundary
!     ncn            i    Contains the number of nnodes in each of the 3 sides
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, irand, n, istart, ipkbndpt, jtimessav, &
              jdiag, kstart, nelmside, nside, &
              ipcoor1, ipkmsc1, ipos, inode, ihelp(npointlc), &
              iwork(29+2*nbndpt)
      double precision xz, yz, x1, y1, dx, dy, hinv
      double precision, allocatable :: side1(:,:), side2(:,:), &
                                       side3(:,:), bcordloc(:,:)
      logical linearsim, triangle, plotdebug

!     bcordloc       work array in which the coordinates of the local
!                    boundaries must be stored
!                    Its length is 2/3 of bcord
!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     hinv           1/h
!     i              Counting variable
!     ihelp          work array
!     inode          Node sequence number
!     ipcoor1        starting address of co-ordinates array in coorlc
!     ipkbndpt       Starting address of array kbndpt
!     ipkmsc1        Starting address of array KMESH part c in kmeshclc
!     ipos           Position in array
!     irand          number of points on the side of an element - 1
!     istart         First position used at a part
!     iwork          integer work array of length 29 + 2*nbndpt
!     jdiag          Indicates if the basis functions satisfy phii(j = 0, &
!                    if i#j (jdiag=1) or not (jdiag=2)
!     jtimessav      Help variable to save the value of jtimes
!     kstart         Start value of parameter
!     linearsim      Indicates whether the element is a linear simplex
!                    (true) or not (false)
!     n              number of points in "x"-direction with rescpect to
!                    rectangle
!     nelmside       Number of elements along one side of the triangle
!     nside          number of points along side of subregion
!     plotdebug      If true plots must be made for debugging
!     side1          Work array containing the coordinates of the
!                    common boundary of regions 1 and 2
!     side2          Work array containing the coordinates of the
!                    common boundary of regions 2 and 3
!     side3          Work array containing the coordinates of the
!                    common boundary of regions 3 and 1
!     triangle       Indicates if the element is a triangle (true) or not
!     x1             x-coordinate
!     xz             x-coordinate of centroid
!     y1             y-coordinate
!     yz             y-coordinate of centroid
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH2DQ         plot one surface (2D)
!     MSHER1         Submesh generator RECTANGLE
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2141   Number of points not the same on all three sides of "ISOPAR"
!    2142   Number of elements along a side is odd
! **********************************************************************
!
!                       PSEUDO CODE
!
!      The triangle is subdivided into three quads, each of which are
!      treated by rectangle:
!
!
!
!                                /\
!                               /  \
!                              /    \
!                             /      \
!                            /        \
!                           /    III   \
!                          /            \
!                         /              \
!                        / \              \
!                       /     \        /   \
!                      /        \   /       \
!                     /           x          \
!                    /            |           \
!                   /             |            \
!                  /     I        |     II      \
!                 /               |              \
!                /                |               \
!               ----------------------------------- &
!
!
!      Perform some tests
!      Compute centroid of triangle
!      Compute nodes on three extra lines from centroid to midpoints of sides
!      For all three subregions
!         Compute new bcord array for this region
!         Compute coor and kmeshclc using "RECTANGLE", store in coor and kmeshclc
!      Remove double points from coor and kmeshclc
!      Fill array kbndpt
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshispquad' )
      debug = .false.
      plotdebug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshispquad'
         call prinin ( ncn, 3, 'ncn' )
         call prinrl1 ( bcord, ncn(1), 2, 'bcord_1' )
         call prinrl1 ( bcord(1,ncn(1)), ncn(2), 2, 'bcord_2' )
         call prinrl1 ( bcord(1,ncn(1)+ncn(2)), ncn(3), 2, 'bcord_3' )
         write(irefwr,1 ) 'npointlc, nbndpt', npointlc, nbndpt
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Perform some tests

      nelmside = ncn(1)
      if ( ncn(2)/=nelmside .or. ncn(3)/=nelmside ) then

!     --- Number of points not the same on all three sides

         call errint ( ncn(1), 1 )
         call errint ( ncn(2), 2 )
         call errint ( ncn(3), 3 )
         call errsub ( 2141, 3, 0, 0 )
         go to 1000

      end if

      nelmside = nelmside-1
      linearsim = ishape==3 .or. ishape==5
      triangle = ishape<=4 .or. ishape==7
      if ( .not. linearsim ) nelmside = nelmside/2
      if ( mod(nelmside,2)/=0 ) then

!     --- Number of elements along a side is odd, not permitted

         call errint ( nelmside, 1 )
         call errsub ( 2142, 1, 0, 0 )
         go to 1000

      end if
      if ( debug ) &
         write(irefwr,*) 'ishape, linearsim, triangle, nelmside ', &
                          ishape, linearsim, triangle, nelmside

!     --- Compute centroid of triangle

      xz = (bcord(1,1)+bcord(1,ncn(1))+bcord(1,ncn(1)+ncn(2)-1))/3d0
      yz = (bcord(2,1)+bcord(2,ncn(1))+bcord(2,ncn(1)+ncn(2)-1))/3d0
      if ( debug ) write(irefwr,*) 'xz, yz ', xz, yz

!     --- Compute nodes on three extra lines from centroid to midpoints of sides

      nside = ncn(1)/2+1

      allocate ( side1(2,nside), side2(2,nside), side3(2,nside), &
                 bcordloc(2,4*nside), stat = error )
      if ( error/=0 ) call eralloc ( error, 14*nside, 'side' )
      if ( ierror/=0 ) go to 1000

      x1 = bcord(1,nside)
      y1 = bcord(2,nside)
      side1(1,1) = x1
      side1(2,1) = y1
      side1(1,nside) = xz
      side1(2,nside) = yz
      hinv = 1d0/(nside-1)
      dx = (xz-x1)*hinv
      dy = (yz-y1)*hinv
      do i = 2, nside-1
         side1(1,i) = x1 + (i-1)*dx
         side1(2,i) = y1 + (i-1)*dy
      end do
      x1 = bcord(1,ncn(1)-1+nside)
      y1 = bcord(2,ncn(1)-1+nside)
      side2(1,1) = x1
      side2(2,1) = y1
      side2(1,nside) = xz
      side2(2,nside) = yz
      dx = (xz-x1)*hinv
      dy = (yz-y1)*hinv
      do i = 2, nside-1
         side2(1,i) = x1 + (i-1)*dx
         side2(2,i) = y1 + (i-1)*dy
      end do
      x1 = bcord(1,2*ncn(1)-2+nside)
      y1 = bcord(2,2*ncn(1)-2+nside)
      side3(1,1) = x1
      side3(2,1) = y1
      side3(1,nside) = xz
      side3(2,nside) = yz
      dx = (xz-x1)*hinv
      dy = (yz-y1)*hinv
      do i = 2, nside-1
         side3(1,i) = x1 + (i-1)*dx
         side3(2,i) = y1 + (i-1)*dy
      end do
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( side1, nside, 2, 'side1' )
         call prinrl1 ( side2, nside, 2, 'side2' )
         call prinrl1 ( side3, nside, 2, 'side3' )

      end if

!     --- For all three subregions
!         Compute new bcord array for this region
!         Compute coorlc and kmeshclc using "RECTANGLE", store in coorlc and kmeshclc
!         Subregion 1:
!         First lower boundary

      do i = 1, nside
         bcordloc(1,i) = bcord(1,i)
         bcordloc(2,i) = bcord(2,i)
      end do

!     --- Next side1

      istart = nside
      do i = 1, nside-1
         bcordloc(1,i+istart) = side1(1,i+1)
         bcordloc(2,i+istart) = side1(2,i+1)
      end do

!     --- Next side3

      istart = istart+nside-1
      do i = 1, nside-1
         bcordloc(1,i+istart) = side3(1,nside-i)
         bcordloc(2,i+istart) = side3(2,nside-i)
      end do

!     --- Finally left-hand side

      istart = istart+nside-1
      kstart = 2*ncn(1)-2+nside
      do i = 1, nside-1
         bcordloc(1,i+istart) = bcord(1,i+kstart)
         bcordloc(2,i+istart) = bcord(2,i+kstart)
      end do
      n = nelmside/2
      if ( linearsim ) then
         irand = 1
      else
         irand = 2
      end if
      if ( triangle ) then
         mtrian = 2
      else
         mtrian = 1
      end if
      jdiag = 1
      ismoot = 0
      ipkbndpt = 1
      npointlc = (n*irand+1)**2
      nelemlc = n**2*mtrian
      ipcoor1 = 1
      ipkmsc1 = 1
      call msher1 ( .true., irand, iwork(ipkbndpt), bcordloc, n, n, &
                    coorlc(1,ipcoor1), kmeshclc(ipkmsc1), jdiag )

!     --- Subregion 2:
!         First lower boundary

      do i = 1, nside
         bcordloc(1,i) = bcord(1,i+nside-1)
         bcordloc(2,i) = bcord(2,i+nside-1)
      end do

!     --- Next right-hand side

      istart = nside
      kstart = ncn(1)
      do i = 1, nside-1
         bcordloc(1,i+istart) = bcord(1,i+kstart)
         bcordloc(2,i+istart) = bcord(2,i+kstart)
      end do

!     --- Next side2

      istart = istart+nside-1
      do i = 1, nside-1
         bcordloc(1,i+istart) = side2(1,i+1)
         bcordloc(2,i+istart) = side2(2,i+1)
      end do

!     --- Next side1

      istart = istart+nside-1
      do i = 1, nside-1
         bcordloc(1,i+istart) = side1(1,nside-i)
         bcordloc(2,i+istart) = side1(2,nside-i)
      end do
      ipkbndpt = ipkbndpt+4*nside-3
      npointlc = (n*irand+1)**2
      nelemlc = n**2*mtrian
      ipkmsc1 = ipkmsc1+nelemlc*inpelm
      ipcoor1 = ipcoor1+npointlc
      call msher1 ( .true., irand, iwork(ipkbndpt), bcordloc, n, n, &
                    coorlc(1,ipcoor1), kmeshclc(ipkmsc1), jdiag )

!     --- Subregion 3:
!         Right-hand side boundary

      kstart = ncn(1)+nside-2
      do i = 1, nside
         bcordloc(1,i) = bcord(1,i+kstart)
         bcordloc(2,i) = bcord(2,i+kstart)
      end do

!     --- Next left-hand side

      istart = nside
      kstart = kstart+nside
      do i = 1, nside-1
         bcordloc(1,i+istart) = bcord(1,i+kstart)
         bcordloc(2,i+istart) = bcord(2,i+kstart)
      end do

!     --- Next side3

      istart = istart+nside-1
      do i = 1, nside-1
         bcordloc(1,i+istart) = side3(1,i+1)
         bcordloc(2,i+istart) = side3(2,i+1)
      end do

!     --- Next side2

      istart = istart+nside-1
      do i = 1, nside-1
         bcordloc(1,i+istart) = side2(1,nside-i)
         bcordloc(2,i+istart) = side2(2,nside-i)
      end do

      ipkbndpt = ipkbndpt+4*nside-3
      npointlc = (n*irand+1)**2
      nelemlc = n**2*mtrian
      ipkmsc1 = ipkmsc1+nelemlc*inpelm
      ipcoor1 = ipcoor1+npointlc
      call msher1 ( .true., irand, iwork(ipkbndpt), bcordloc, n, n, &
                    coorlc(1,ipcoor1), kmeshclc(ipkmsc1), jdiag )

      deallocate ( side1, side2, side3, bcordloc, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'side' )

!     --- Remove double points from coorlc and kmeshclc and renumber
!         First consider region 2
!         Clear array iwork, it will be filled with the new numbering

      ihelp = 0

!     --- The only common side with region 1 is side1
!         Put the corresponding nodes in iwork

      ipkbndpt = 1+2*(4*nside-3)
      do i = 1, nside
         inode = iwork(nside+i-1)
         ipos = iwork(ipkbndpt-i)
         ihelp(ipos) = inode
      end do

!     --- extend array iwork for points that have not been filled and adjust
!         array coorlc

      nnodes = npointlc
      do i = 1, npointlc
         if ( ihelp(i)==0 ) then
            nnodes = nnodes + 1
            ihelp(i) = nnodes
            coorlc(1,nnodes) = coorlc(1,npointlc+i)
            coorlc(2,nnodes) = coorlc(2,npointlc+i)
         end if
      end do

!     --- renumber kmeshclc

      ipkmsc1 = nelemlc*inpelm
      do i = 1, nelemlc*inpelm
         kmeshclc(i+ipkmsc1) = ihelp(kmeshclc(i+ipkmsc1))
      end do

!     --- renumber kbndpt

      ipkbndpt = 4*nside-3
      do i = 1, 4*nside-3
         iwork(ipkbndpt+i) = ihelp(iwork(i+ipkbndpt))
      end do

!     --- Next consider region 3
!         Clear array iwork, it will be filled with the new numbering

      do i = 1, npointlc
         ihelp(i) = 0
      end do

!     --- The only common side with region 1 is side3 and with region 2: side2
!         Put the corresponding nodes in iwork
!         side2:

      ipkbndpt = 3*(4*nside-3)+1
      istart = 4*nside-3+2*nside-2
      do i = 1, nside
         inode = iwork(istart+i)
         ipos = iwork(ipkbndpt-i)
         ihelp(ipos) = inode
      end do

!     --- side3:

      istart = 3*nside-1
      ipkbndpt = 2*(4*nside-3)+2*nside-2
      do i = 1, nside
         inode = iwork(istart-i)
         ipos = iwork(ipkbndpt+i)
         ihelp(ipos) = inode
      end do

!     --- extend array iwork for points that have not been filled and adjust
!         array coorlc

      do i = 1, npointlc
         if ( ihelp(i)==0 ) then
            nnodes = nnodes + 1
            ihelp(i) = nnodes
            coorlc(1,nnodes) = coorlc(1,2*npointlc+i)
            coorlc(2,nnodes) = coorlc(2,2*npointlc+i)
         end if
      end do

!     --- renumber kmeshclc

      ipkmsc1 = ipkmsc1+nelemlc*inpelm
      do i = 1, nelemlc*inpelm
         kmeshclc(i+ipkmsc1) = ihelp(kmeshclc(i+ipkmsc1))
      end do

!     --- renumber kbndpt

      ipkbndpt = 2*(4*nside-3)
      do i = 1, 4*nside-3
         iwork(ipkbndpt+i) = ihelp(iwork(i+ipkbndpt))
      end do

!     --- Finally fill kbndpt

      npointlc = nnodes
      nelemlc = 3*nelemlc

!     --- First part lower side

      do i = 1, nside
         kbndpt(i) = iwork(i)
      end do

!     --- Second part lower side plus first part right-hand side

      ipkbndpt = nside-1
      istart = 4*nside-3
      do i = 1, nside*2-1
         kbndpt(ipkbndpt+i) = iwork(istart+i)
      end do

!     --- Third part last part right-hand side plus first part left-hand side

      ipkbndpt = ipkbndpt+2*nside-2
      istart = 2*(4*nside-3)
      do i = 1, nside*2-1
         kbndpt(ipkbndpt+i) = iwork(istart+i)
      end do

!     --- Last part: last part left-hand side

      ipkbndpt = ipkbndpt+2*nside-2
      istart = 3*nside-3
      do i = 1, nside
         kbndpt(ipkbndpt+i) = iwork(istart+i)
      end do

1000  if ( plotdebug ) then

!     --- Debug information

         jtimessav = jtimes
         jtimes = 0
         call msh2dq ( bcord, coorlc, kmeshclc, 15d0, 1d0, 0, 0 )
         jtimes = jtimessav

      end if

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshispquad'

      end if
      call erclos ( 'mshispquad' )

      end
