      subroutine mshmapsur3d (  bcord, coor, nbcrd, npoint, mapping, &
                                isurnr, icentre, rinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 24-11-2008 Extra debug
!        version  1.1  date 07-06-2006 Debug statements
!        version  1.0  date 03-04-1999
!
!   copyright (c) 1999-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Map coordinates of surface on a precisely defined surface depending
!   on the parameter mapping
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     map
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer  npoint, mapping, nbcrd ,isurnr, icentre
      double precision  bcord(3,*), coor(3,*), rinput(3,*)

!     bcord          i    array containing the coordinates of the boundaries
!     coor          i/o   array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     icentre        i    User point number of centroid
!     isurnr         i    Surface sequence number
!     mapping        i    defines the type of mapping to a 3d surface
!     nbcrd          i    number of nodal points in boundary of submesh
!     npoint         i    Number of nodal points
!     rinput         i    Real input array
!                         In this array the coordinates of the user points
!                         are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision  radius, centre(3), radiusloc, factor, radius2, &
                        eps, rmax
      integer i
      logical notsphere

!     centre         Contains coordinates of the centre
!     eps            Local accuracy
!     factor         Multiplication factor
!     i              Counting variable
!     notsphere      If true not all points are lying on a sphere
!     radius         Radius of sphere or circle
!     radius2        radius^2
!     radiusloc      Local radius
!     rmax           maximum coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2081   Not all points of surface on a sphere
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( mapping = 1 ) then
!        map all points onto sphere
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmapsur3d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshmapsur3d'
         write(irefwr,1) 'mapping, npoint', mapping, npoint
         write(irefwr,1) 'icentre', icentre
         call prinrl1 ( coor, npoint, 3, 'coor_in' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( mapping==1 ) then

!     --- mapping = 1, hence map onto sphere
!         first find centre of spere

         centre(1) = rinput(1,icentre)
         centre(2) = rinput(2,icentre)
         centre(3) = rinput(3,icentre)
         radius2 = (bcord(1,1)-centre(1))**2 + &
                   (bcord(2,1)-centre(2))**2 + &
                   (bcord(3,1)-centre(3))**2
         radius = sqrt ( radius2 )
         if ( debug ) write(irefwr,2) 'radius, centre', radius, centre

!        --- Check if all points are on the sphere
!            Compute accuracy parameter

         rmax = 0d0
         do i = 1, nbcrd
            rmax = max ( rmax, abs(bcord(1,i)), abs(bcord(2,i)), &
                         abs(bcord(3,i)) )
         end do
         eps = sqreps * rmax
         eps = eps**2
         notsphere = .false.
         do i = 2, nbcrd
            if ( abs( (bcord(1,i)-centre(1))**2 + &
                      (bcord(2,i)-centre(2))**2 + &
                      (bcord(3,i)-centre(3))**2 - radius2 )>eps ) &
               notsphere = .true.
         end do

         if ( notsphere ) then

!        --- Not all points ly on the sphere

            call errint ( isurnr, 1)
            call errsub ( 2081, 1, 0, 0 )

         end if

!        --- map all points onto sphere

         do i = 1, npoint
            radiusloc = sqrt ( (coor(1,i)-centre(1))**2 + &
                               (coor(2,i)-centre(2))**2 + &
                               (coor(3,i)-centre(3))**2 )
            factor = radius/radiusloc
            coor(1,i) = centre(1) + factor*(coor(1,i)-centre(1))
            coor(2,i) = centre(2) + factor*(coor(2,i)-centre(2))
            coor(3,i) = centre(3) + factor*(coor(3,i)-centre(3))

         end do

      end if

1000  call erclos ( 'mshmapsur3d' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( coor, npoint, 3, 'coor_out' )
         write(irefwr,*) 'End mshmapsur3d'

      end if  ! ( debug )

      end
