      subroutine mshob3dintline ( nvert, inodes, rmeshx, iobst, coor, &
                                  nx, ny, nz, iobgrid, iverts, rverts, &
                                  nintersect, iwork1, kmeshc, &
                                  inpelm, coorobst, leniobgrid )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 08-12-2004 Debug information
!        version  2.2  date 12-11-2004 Accuracy from common
!        version  2.1  date 09-11-2002 Warning is printed only once
!        version  2.0  date 14-11-2001 Extra parameter leniobgrid
!        version  1.0  date 29-06-2001
!
!   copyright (c) 2001-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find the intersection of all sides in the element with vertex nodes
!     stored in inodes with an obstacle
!     The number of intersections and the side numbers are returned
!     If a side crosses an obstacle twice a warnning is given and only
!     the first intersection is stored
!     3d version
! **********************************************************************
!
!                       KEYWORDS
!
!     intersection
!     line
!     element
!     obstacle
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulegen
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nvert, inodes(3,*), iobst, nx, ny, nz, iwork1(*), &
              iobgrid(0:nx-1,0:ny-1,0:nz-1,*), iverts(*), nintersect, &
              inpelm, kmeshc(inpelm,*), leniobgrid
      double precision rmeshx(9,*), coor(3,*), rverts(*), coorobst(3,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     coorobst       i    Array of length ndim x npoint_obst containing the
!                         co-ordinates of the nodal points with respect to the
!                         obstacle
!     inodes         i    Array of length 3*nvert containing the node numbers
!                         of the edges as well as the edge number
!                         inodes(1,i) = first node of edge i
!                         inodes(2,i) = last node of edge i
!                         inodes(3,i) = edge number
!     inpelm         i    number of points per element in obstacle
!     iobgrid        i    In R^3:
!                         array of length nx x ny x nz x 6 that is filled
!                         as follows:
!                         The region covered is defined by xmin, xmax, ymin, &
!                         ymax, zmin, zmax
!                         The cell size is dx x dy x dz
!                         Cell i,j,k refers to the coordinates:
!                         (xmin+(i-1)dx,ymin+(j-1)dy,zmin+(k-1)dz) x
!                         (xmin+idx,ymin+jdy,zmin+kdz)
!                         iobgrid(i,j,k,.) = 0: cell outside obstacle
!                         iobgrid(i,j,k,.) = l>0: boundary part l crosses cell
!                         iobgrid(i,j,k,.) = -1: cell completely in obstacle
!                         The last index refers to the various elements in a
!                         cell
!     iobst          i    Obstacle sequence number
!     iverts         o    Array of length nintersect containing the edge
!                         numbers of the edges that interect the obstacle
!     iwork1         i    Integer work array of length npoint
!                         iwork1(i) = 1, means that node i is on the obstacle
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements of the obstacle
!     leniobgrid     i    Declared length of last dimension of iobgrid
!     nintersect     o    Number of intersections found
!     nvert          i    Number of edges to be considered
!     nx             i    number of cells in x direction
!     ny             i    number of cells in y direction
!     nz             i    number of points in z direction
!     rmeshx         i    Real part of kmesh part x (i = obstacle number)
!                         rmeshx(1,i) = xmin
!                         rmeshx(2,i) = xmax
!                         rmeshx(3,i) = ymin
!                         rmeshx(4,i) = ymax
!                         rmeshx(5,i) = dx
!                         rmeshx(6,i) = dy
!                         In R^3:
!                         rmeshx(5,i) = zmin
!                         rmeshx(6,i) = zmax
!                         rmeshx(7,i) = dx
!                         rmeshx(8,i) = dy
!                         rmeshx(9,i) = dz
!     rverts         o    Contains the factors corresponding to the intersection
!                         points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ncell, ivert, icell(8*leniobgrid), l, ielemobs, inode
      double precision x1(3), x2(3), x3(3), xp1(3), xp2(3), xmin, ymin, &
                       zmin, dx, dy, dz, eps, fact1, fact2(2), eps1
      logical found, skip, warning

!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     dz             length of vector in z-direction
!     eps            Local accuracy
!     eps1           Local accuracy
!     fact1          Defines the factor for the intersection point
!                    The intersection point on the face is defined by
!                    x1 + fact1 * (x2-x1)
!     fact2          Defines the factors for the intersection point with respect
!                    to the obstacle surface
!                    There is only an intersection if 0 <= fact2 <= 1
!     found          If true already a possibility has been found
!     i              Counting variable
!     icell          Array containing the surface parts of the obstacle that
!                    must be considered for computing the cross sections
!     ielemobs       Element number in obstacle
!     inode          Node sequence number
!     ivert          Loop variable
!     l              General loop variable
!     ncell          Number of curve parts that must be used to compute
!                    cross sections
!     skip           If true the intersection must be skipped
!     warning        If true already a warning has been printed
!     x1             coordinates of first point in triangle
!     x2             coordinates of second point in triangle
!     x3             coordinates of third point in triangle
!     xmin           Minimum x-value
!     xp1            coordinates of first point of edge
!     xp2            coordinates of second point of edge
!     ymin           Minimum y-value
!     zmin           Minimum z-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     MSHCROSSSURF   Find the intersection of the lines xp1-xp2 and the
!                    triangle x1,x2,x3
!     MSHOB3DINTLIN1 Fill array icell with indices of surface parts of the
!                    obstacle that are possibily intersected by the line x1-x2
!     PRININ1        print 2d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    2211   Obstacle has more than one intersection with a face of the mesh
!    2254   inpelm # 3
! **********************************************************************
!
!                       PSEUDO CODE
!
!     nintersect := 0
!     for ivert := 1 (1) nvert do
!        x1 := coordinates node inodes(i)
!        x2 := coordinates node inodes(i+1)
!        ncell := 0
!        For both points x1 and x2 do
!           Compute corresponding index pairs ip,jp referring to the
!              obstacle cells
!           ind := the corresponding part of the obstacle curve
!           for i := ind-1 (1) ind+1 do
!              if i not in icell then
!                 ncell:= ncell+1
!                 icell(ncell) := i
!           if ( there is a second curve crossing the obstacle cell ) then
!              ind := the corresponding part of the second obstacle curve
!              for i := ind-1 (1) ind+1 do
!                 if i not in icell then
!                    ncell:= ncell+1
!                    icell(ncell) := i
!        found := false
!        For i := 1 (1) ncell do
!           xlow := first point of obstacle part icell(i)
!           xupp := last point of obstacle part icell(i)
!           if ( the line x1-x2 intersects the line xlow-xupp ) then
!              if ( found ) then
!                 warning
!              else ( not yet found )
!                 found := true
!                 nintersect := nintersect+1
!                 Store ivert and coordinates of intersection point in iverts
!                 and rverts
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data warning /.false./
! ======================================================================
!
      call eropen ( 'mshob3dintline' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshob3dintline'
         write(irefwr,100) 'iobst, nvert ', iobst, nvert
100      format ( a, 10i6 )
         call prinin1 ( inodes, nvert, 3, 'inodes' )

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

      if ( inpelm.ne.3 ) then

!     --- Only triangles have been implemented

         call errsub ( 2254, 0, 0, 0 )
         go to 1000

      end if

!     --- nintersect := 0

      nintersect = 0

!     --- Extract xmin,xmax,ymin,ymax from obstacle array

      xmin = rmeshx(1,iobst)
      ymin = rmeshx(3,iobst)
      zmin = rmeshx(5,iobst)
      dx   = rmeshx(7,iobst)
      dy   = rmeshx(8,iobst)
      dz   = rmeshx(9,iobst)
      eps  = 0.01d0
      eps1 = accuracy_obstacle

!     --- for ivert := 1 (1) nvert do

      do ivert = 1, nvert

!     --- xp1 := coordinates node inodes(i)
!         xp2 := coordinates node inodes(i+1)
!         ncell := 0

         do i = 1, 3
            xp1(i) = coor(i,inodes(1,ivert))
            xp2(i) = coor(i,inodes(2,ivert))
         end do
         if ( debug ) then
            write(irefwr,100) 'ivert ', ivert
            call prinrl ( xp1, 3, 'xp1' )
            call prinrl ( xp2, 3, 'xp2' )
         end if  ! ( debug )

!        --- Fill array icell with indices of curve parts of the obstacle that
!            is possibily intersected by the line x1-x2

         call mshob3dintlin1 ( xp1, xp2, xmin, ymin, zmin, nx, ny, nz, &
                               iobgrid, icell, ncell, dx, dy, dz, &
                               leniobgrid )
         if ( debug ) write(irefwr,100) 'ncell ', ncell

         if ( ncell.gt.8*leniobgrid ) then

!        --- ncell too large

            call errint ( ncell, 1 )
            call errint ( 8*leniobgrid, 1 )
            call errchr ( 'icell', 1 )
            call errsub ( 550, 2, 0 , 1 )
            go to 1000

         end if

!        --- found := false

         found = .false.

!        --- For i := 1 (1) ncell do

         do i = 1, ncell

!        --- xlow := first point of obstacle part icell(i)
!            xupp := last point of obstacle part icell(i)

            ielemobs = icell(i)
            inode = kmeshc(1,ielemobs)
            do l = 1, 3
               x1(l) = coorobst(l,inode)
            end do
            inode = kmeshc(2,ielemobs)
            do l = 1, 3
               x2(l) = coorobst(l,inode)
            end do
            inode = kmeshc(3,ielemobs)
            do l = 1, 3
               x3(l) = coorobst(l,inode)
            end do

!           --- Check if line xp1-xp2 intersects triangle and
!               compute intersection

            call mshcrosssurf ( xp1, xp2, x1, x2, x3, fact1, fact2, &
                                eps )
            if ( debug ) then
               write(irefwr,110) 'fact1, fact2 ', fact1, fact2
110            format ( a, 10d12.3 )
            end if  ! ( debug )

            if ( fact1.gt.-eps .and. fact1.lt.1d0+eps ) then

!           --- the line x1-x2 intersects the triangle x1, x2,x3

               skip = .false.
               if ( fact1.le.0.2d0 ) then

!              --- first point may also be a node

                  if ( iwork1(inodes(1,ivert)).ne.0 ) skip = .true.

               end if
               if ( fact1.ge.0.8d0 ) then

!              --- second point may also be a node

                  if ( iwork1(inodes(2,ivert)).ne.0 ) skip = .true.

               end if
               if ( .not. skip .and. found ) then

!              --- warning already an intersection found before

                   if ( .not. warning ) then

!                  --- This warnnig is printed only once

                      call errint ( iobst, 1 )
                      call errwar ( 2211, 1, 0, 0 )
                      warning = .true.

                   end if

!                  --- Compute a mean value except when on of the nodes close
!                      to the end of the edge

                   if ( fact1.lt.eps1 ) then

!                  --- fact1 < eps1, point close to first point
!                      choose point that is at longest distance of end points

                      if ( rverts(nintersect).lt.1d0-eps1 ) then

!                     --- standard case

                         rverts(nintersect) = &
                            max(rverts(nintersect),fact1)

                      else

!                     --- first point close to begin and other point close to
!                         end

                         if ( rverts(nintersect).gt.1d0-eps1+fact1 ) &
                            rverts(nintersect) = fact1

                      end if

                   else if ( fact1.gt.1d0-eps1 ) then

!                  --- fact1 > 1-eps1, point close to second point
!                      choose point that is at longest distance of end points

                      if ( rverts(nintersect).gt.eps1 ) then

!                     --- standard case

                         rverts(nintersect) = &
                            min(rverts(nintersect),fact1)

                      else

!                     --- first point close to begin and other point close to
!                         end

                         if ( rverts(nintersect).lt.1d0-fact1 ) &
                            rverts(nintersect) = fact1

                      end if

                   else if ( rverts(nintersect).lt.eps1 ) then

!                  --- rverts(nintersect) < eps1, point close to first point
!                      New point not, hence use new point

                      rverts(nintersect) = fact1

                   else if ( rverts(nintersect).gt.1d0-eps1 ) then

!                  --- rverts(nintersect) > 1-eps1, point close to end point
!                      New point not, hence use new point

                      rverts(nintersect) = fact1

                   else

!                  --- None of these cases, use mean value

                      rverts(nintersect) = &
                         0.5d0*(fact1+rverts(nintersect))

                   end if

               else if ( .not. skip ) then

!              --- no intersection found before
!                  found := true
!                  nintersect := nintersect+1
!                  Store ivert and coordinates of intersection point in iverts
!                  and rverts

                  found = .true.
                  nintersect = nintersect+1
                  iverts(nintersect) = inodes(3,ivert)
                  rverts(nintersect) = fact1

               end if

            end if

         end do

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshob3dintline'

      end if
      call erclos ( 'mshob3dintline' )

      end
