      subroutine mshobsfl3sf ( nx, ny, nz, iobgrid, ielem, x, y, z, &
                               i1, i2, i3, inpelm, xmin, ymin, zmin, &
                               dx, dy, dz, leniobgrid, leniobgridact )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 14-11-2001 Extra parameters leniobgrid,leniobgridact
!        version  1.0  date 15-06-2001
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill internal part of one element in the obstacle surfaces into iobgrid
!    It is assumed that the edges have been stored already
!    At this moment it assumed that the element is a triangle
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     triangle
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, nz, iobgrid(0:nx-1,0:ny-1,0:nz-1,*), ielem, &
              i1(*), i2(*), i3(*), inpelm, leniobgrid, leniobgridact
      double precision x(*), y(*), z(*), xmin, ymin, zmin, dx, dy, dz

!     dx             i    length of vector in x-direction
!     dy             i    length of vector in y-direction
!     dz             i    length of vector in z-direction
!     i1             i    Array containing the "i" values of the vertices
!     i2             i    Array containing the "j" values of the vertices
!     i3             i    Array containing the "k" values of the vertices
!     ielem          i    Element number
!     inpelm         i    number of points per element
!     iobgrid       i/o   In R^3:
!                         array of length nx x ny x nz x 6 that is filled
!                         as follows:
!                         The region covered is defined by xmin, xmax, ymin, &
!                         ymax, zmin, zmax
!                         The cell size is dx x dy x dz
!                         Cell i,j,k refers to the coordinates:
!                         (xmin+(i-1)dx,ymin+(j-1)dy,zmin+(k-1)dz) x
!                         (xmin+idx,ymin+jdy,zmin+kdz)
!                         iobgrid(i,j,k,.) = 0: cell outside obstacle
!                         iobgrid(i,j,k,.) = l>0: boundary part l crosses cell
!                         iobgrid(i,j,k,.) = -1: cell completely in obstacle
!                         The last index refers to the various elements in a
!                         cell
!     leniobgrid     i    Declared length of last dimension of iobgrid
!     leniobgridact i/o   Actual length of last dimension of iobgrid
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     nz             i    number of points in z direction
!     x              i    Contains the x-coordinates of the vertices
!     xmin           i    Minimum x-value
!     y              i    Contains the y-coordinates of the vertices
!     ymin           i    Minimum y-value
!     z              i    Contains the z-coordinates of the vertices
!     zmin           i    Minimum z-value
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, l, imax, imin, jmax, jmin, kmax, kmin
      double precision alpha, beta, zt, yt, xt, det, detinv

!     alpha          multiplication factor
!     beta           multiplication factor
!     det            determinant
!     detinv         inverse of determinant
!     i              Counting variable
!     imax           Maximum i-value
!     imin           Minimum i-value
!     j              Counting variable
!     jmax           Maximum j-value
!     jmin           Minimum j-value
!     k              Counting variable
!     kmax           Maximum k-value
!     kmin           Minimum k-value
!     l              General loop variable
!     xt             x-coordinate
!     yt             y-coordinate
!     zt             z-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     MSHOBSFL1      Fills array iobsgrid for one cell
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2254   inplem # 3
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The plane defined by the triangle is given by:
!
!       x = x_1 + alpha * (x_2-x_1) + beta * (x_3-x_1)
!
!    Cell ( i,j,k ) is defined by
!
!       xmin + i*dx <= x <= xmin+(i+1)*dx
!       ymin + j*dy <= y <= ymin+(j+1)*dy
!       zmin + k*dz <= z <= zmin+(k+1)*dz
!
!    To check if the triangle is inside the cell, we intersect all
!    boundaries of the cell with the triangle
!
!    For example intersection with the edge x=i, y=j gives
!
!       i = x_1 + alpha * (x_2-x_1) + beta * (x_3-x_1)
!       j = x_1 + alpha * (x_2-x_1) + beta * (x_3-x_1)
!
!    hence
!
!       | alpha | =  1/det | y_3-y_1  x_1-x_3 | | i-x_1 |
!       | beta  |          | y_1-y_2  x_2-x_1 | | j-y_1 |
!
!    If alpha >= 0, beta >= 0 and alpha+beta <= 1 we have an intersection
!    with the triangle
!    The intersection is reached for the corresponding value of z, which defines
!    k
!
!    Code:
!
!    Find imin, imax, jmin, jmax, kmin and kmax for set corresponding to elem.
!    Compute determinant for (x,y) direction
!    If ( determinant# 0 ) then
!       For i := imin (1) imax do
!          For j := jmin (1) jmax do
!             Compute intersection with triangle
!             If intersection exists, store corresponding cell in iobgrid
!    Compute determinant for (y,z) direction
!    If ( determinant# 0 ) then
!       For j := jmin (1) jmax do
!          For k := kmin (1) kmax do
!             Compute intersection with triangle
!             If intersection exists, store corresponding cell in iobgrid
!    Compute determinant for (x,z) direction
!    If ( determinant# 0 ) then
!       For i := imin (1) imax do
!          For k := kmin (1) kmax do
!             Compute intersection with triangle
!             If intersection exists, store corresponding cell in iobgrid
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobsfl3sf' )
      if ( inpelm.ne.3 ) then

!     --- Only triangles have been implemented

         call errsub ( 2254, 0, 0, 0 )
         go to 1000

      end if

!     --- Find imin, imax, jmin, jmax, kmin and kmax for set corresponding to
!         element

      imin = i1(1)
      imax = i1(1)
      jmin = i2(1)
      jmax = i2(1)
      kmin = i3(1)
      kmax = i3(1)
      do l = 2, inpelm
         imin = min(imin,i1(l))
         imax = max(imax,i1(l))
         jmin = min(jmin,i2(l))
         jmax = max(jmax,i2(l))
         kmin = min(kmin,i3(l))
         kmax = max(kmax,i3(l))
      end do

!     --- Compute determinant for (x,y) direction

      det = (x(2)-x(1))*(y(3)-y(1))-(x(3)-x(1))*(y(2)-y(1))
      if ( abs(det).gt.epsmac ) then

!     --- determinant# 0
!         Loop over i and j direction

          detinv = 1d0/det
          do i = imin, imax+1
             do j = jmin, jmax+1

!            --- Compute intersection with triangle

                xt = xmin+i*dx
                yt = ymin+j*dy
                alpha = detinv * ( (y(3)-y(1))*(xt-x(1)) + &
                                   (x(1)-x(3))*(yt-y(1)) )
                beta  = detinv * ( (y(1)-y(2))*(xt-x(1)) + &
                                   (x(2)-x(1))*(yt-y(1)) )

                if ( alpha.gt.-1d-3 .and. beta.gt.-1d-3 .and. &
                     alpha+beta.lt.1.001d0 ) then

!               --- intersection exists, store corresponding cell in iobgrid

                   zt = z(1) + alpha * (z(2)-z(1)) + beta * (z(3)-z(1))
                   zt = (zt-zmin)/dz
                   k = min(max(int(zt),0),nz-1)
                   if ( i.gt.imin ) then

!                  --- cell left of i

                      if ( j.gt.jmin ) then

!                     --- cell left of j

                         call mshobsfl1 ( nx, ny, nz, iobgrid, &
                                          i-1, j-1, k, ielem, &
                                          leniobgrid, leniobgridact )

                      end if
                      if ( j.le.jmax ) then

!                     --- cell j

                         call mshobsfl1 ( nx, ny, nz, iobgrid, &
                                          i-1, j, k, ielem, &
                                          leniobgrid, leniobgridact )

                      end if

                   end if

                   if ( i.le.imax ) then

!                  --- cell i

                      if ( j.gt.jmin ) then

!                     --- cell left of j

                         call mshobsfl1 ( nx, ny, nz, iobgrid, &
                                          i, j-1, k, ielem, &
                                          leniobgrid, leniobgridact )

                      end if
                      if ( j.le.jmax ) then

!                     --- cell j

                         call mshobsfl1 ( nx, ny, nz, iobgrid, &
                                          i, j, k, ielem, &
                                          leniobgrid, leniobgridact )

                      end if

                   end if

                end if

             end do
          end do

      end if

!     --- Compute determinant for (y,z) direction

      det = (y(2)-y(1))*(z(3)-z(1))-(y(3)-y(1))*(z(2)-z(1))
      if ( abs(det).gt.epsmac ) then

!     --- determinant# 0
!         Loop over i and k direction

          detinv = 1d0/det
          do j = jmin, jmax+1
             do k = kmin, kmax+1

!            --- Compute intersection with triangle

                zt = zmin+k*dz
                yt = ymin+j*dy
                alpha = detinv * ( (z(3)-z(1))*(yt-y(1)) + &
                                   (y(1)-y(3))*(zt-z(1)) )
                beta  = detinv * ( (z(1)-z(2))*(yt-y(1)) + &
                                   (y(2)-y(1))*(zt-z(1)) )

                if ( alpha.gt.-1d-3 .and. beta.gt.-1d-3 .and. &
                     alpha+beta.lt.1.001d0 ) then

!               --- intersection exists, store corresponding cell in iobgrid

                   xt = x(1) + alpha * (x(2)-x(1)) + beta * (x(3)-x(1))
                   xt = (xt-xmin)/dx
                   i = min(max(0,int(xt)),nx-1)
                   if ( j.gt.jmin ) then

!                  --- cell left of j

                      if ( k.gt.kmin ) then

!                     --- cell left of k

                         call mshobsfl1 ( nx, ny, nz, iobgrid, &
                                          i, j-1, k-1, ielem, &
                                          leniobgrid, leniobgridact )

                      end if
                      if ( k.le.kmax ) then

!                     --- cell k

                         call mshobsfl1 ( nx, ny, nz, iobgrid, &
                                          i, j-1, k, ielem, &
                                          leniobgrid, leniobgridact )

                      end if

                   end if

                   if ( j.le.jmax ) then

!                  --- cell k

                      if ( k.gt.kmin ) then

!                     --- cell left of k

                         call mshobsfl1 ( nx, ny, nz, iobgrid, &
                                          i, j, k-1, ielem, &
                                          leniobgrid, leniobgridact )

                      end if
                      if ( k.le.kmax ) then

!                     --- cell k

                         call mshobsfl1 ( nx, ny, nz, iobgrid, &
                                          i, j, k, ielem, &
                                          leniobgrid, leniobgridact )

                      end if

                   end if

                end if

!               --- Remark, the x-z direction seems superfluous because
!                           all intersections should be covered now
!                           therefor it is skipped

             end do
          end do

      end if

!     --- Compute determinant for (x,z) direction

      det = (z(2)-z(1))*(x(3)-x(1))-(z(3)-z(1))*(x(2)-x(1))
      if ( abs(det).gt.epsmac ) then

!     --- determinant# 0
!         Loop over k and i direction

          detinv = 1d0/det
          do k = kmin, kmax+1
             do i = imin, imax+1

!            --- Compute intersection with triangle

                zt = zmin+k*dz
                xt = xmin+i*dx
                alpha = detinv * ( (x(3)-x(1))*(zt-z(1)) + &
                                   (z(1)-z(3))*(xt-x(1)) )
                beta  = detinv * ( (x(1)-x(2))*(zt-z(1)) + &
                                   (z(2)-z(1))*(xt-x(1)) )

                if ( alpha.gt.-1d-3 .and. beta.gt.-1d-3 .and. &
                     alpha+beta.lt.1.001d0 ) then

!               --- intersection exists, store corresponding cell in iobgrid

                   yt = y(1) + alpha * (y(2)-y(1)) + beta * (y(3)-y(1))
                   yt = (yt-ymin)/dy
                   j = min(max(0,int(yt)),ny-1)
                   if ( k.gt.kmin ) then

!                  --- cell left of k

                      if ( i.gt.imin ) then

!                     --- cell left of i

                         call mshobsfl1 ( nx, ny, nz, iobgrid, &
                                          i-1, j, k-1, ielem, &
                                          leniobgrid, leniobgridact )

                      end if
                      if ( i.le.imax ) then

!                     --- cell i

                         call mshobsfl1 ( nx, ny, nz, iobgrid, &
                                          i, j, k-1, ielem, &
                                          leniobgrid, leniobgridact )

                      end if

                   end if

                   if ( k.le.kmax ) then

!                  --- cell k

                      if ( i.gt.imin ) then

!                     --- cell left of i

                         call mshobsfl1 ( nx, ny, nz, iobgrid, &
                                          i-1, j, k, ielem, &
                                          leniobgrid, leniobgridact )

                      end if
                      if ( i.le.imax ) then

!                     --- cell i

                         call mshobsfl1 ( nx, ny, nz, iobgrid, &
                                          i, j, k, ielem, &
                                          leniobgrid, leniobgridact )

                      end if

                   end if

                end if

             end do
          end do

      end if

1000  call erclos ( 'mshobsfl3sf' )
      end
