      subroutine mshobsfledge ( curvescp, n, xmin, ymin, dx, dy, iwork, &
                                leniwork, iobgrid, nx, ny, numcells )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-10-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iobgrid with number of edges per cell
!     array numcells with starting addresses and number of cells
!     array iwork with cell pairs
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, leniwork, iwork(2,leniwork), nx, ny, &
              iobgrid(0:nx-1,0:ny-1), numcells(2,n-1)
      double precision curvescp(2,n), xmin, ymin, dx, dy

!     curvescp       i    Contains all coordinates of the obstacle
!     dx             i    step size of grid in x-direction
!     dy             i    step size of grid in y-direction
!     iobgrid        o    Is filled with the number of curve parts that
!                         intersect cell i,j
!     iwork          o    Contains the cell numbers (i,j) in the sequence
!                         of the curve parts
!                         Must be used in connection with numcells
!     leniwork       i    Declared length of array iwork
!     n              i    number of nodes in the obstacle
!     numcells       o    numcells(1,k) contains the number of cells curvepart
!                         k intersects
!                         numcells(1,k) contains the starting address of
!                         the cell numbers in iwork
!     nx             i    number of points of grid in x direction
!     ny             i    number of points of grid in y direction
!     xmin           i    Minimum x-value in grid
!     ymin           i    Minimum y-value in grid
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipiwork, iedge, jpiwork, i, j, jstep, &
              istep, i1, i2, k, iprev, j1, j2, ksign, jprev
      double precision x1, y1, x2, y2, alpha1, alpha2, beta1, beta2, s, &
                       alpha, beta

!     alpha          parameter alpha to define a line
!     alpha1         parameter alpha1 to define a line
!     alpha2         parameter alpha2 to define a line
!     beta           parameter beta to define a line
!     beta1          parameter beta1 to define a line
!     beta2          parameter beta2 to define a line
!     i              Counting variable
!     i1             Index of point with respect to rectangular grid
!     i2             Index of point with respect to rectangular grid
!     iedge          curve part sequence number
!     ipiwork        Starting address in array iwork
!     iprev          Previous value used
!     istep          Step in loop
!     j              Counting variable
!     j1             Index of point with respect to rectangular grid
!     j2             Index of point with respect to rectangular grid
!     jpiwork        Prior value of ipiwork in loop
!     jprev          Previous value of j
!     jstep          Step in loop
!     k              Counting variable
!     ksign          Step in k loop
!     s              Parameter along curve
!     x1             x-coordinate of first point in curve part
!     x2             x-coordinate of last point in curve part
!     y1             y-coordinate of first point in curve part
!     y2             y-coordinate of last point in curve part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSFLEDGE1  Put information of cell i1, j1 in iobgrid, numcells
!                    and iwork
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   For all edges in icurnr do
!      Compute x1,y1, x2,y2
!      alpha1 := (x1-xmin)/dx; alpha2 := (x2-xmin)/dx;
!      beta1 := (y1-ymin)/dy; beta2 := (y2-ymin)/dy;
!      i1 := alpha1; j1:= beta1; i2 := alpha2; j2:= beta2;
!      Mark cells i1,j1 and i2,j2 as boundary cells
!      Mark all cells at the intersection line as boundary cells
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobsfledge' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobsfledge'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ipiwork = 0

!     --- Coordinates of first point in curvescp

      x1 = curvescp(1,1)
      y1 = curvescp(2,1)

      if ( debug ) write(irefwr,2) 'x1, y1', x1, y1

!     --- For all edges in icurnr do

      do iedge = 1, n-1

!     --- Compute x1,y1, x2,y2

         x2 = curvescp(1,1+iedge)
         y2 = curvescp(2,1+iedge)
         if ( debug ) then
            write(irefwr,1) 'iedge', iedge
            write(irefwr,2) 'x2, y2', x2, y2
         end if  ! ( debug )

!        --- alpha1 := (x1-xmin)/dx; alpha2 := (x2-xmin)/dx;
!            beta1 := (y1-ymin)/dy; beta2 := (y2-ymin)/dy;
!            i1 := alpha1; j1:= beta1; i2 := alpha2; j2:= beta2;

         alpha1 = (x1-xmin)/dx
         alpha2 = (x2-xmin)/dx
         beta1  = (y1-ymin)/dy
         beta2  = (y2-ymin)/dy
         i1     = min(int(max(alpha1,0d0)),nx-1)
         i2     = min(int(max(alpha2,0d0)),nx-1)
         j1     = min(int(max(beta1,0d0)),ny-1)
         j2     = min(int(max(beta2,0d0)),ny-1)

         jpiwork = ipiwork+1

         if ( debug ) then

!        --- debug is true, print info

            write(irefwr,2) 'alpha1, alpha2, beta1, beta2', &
                             alpha1, alpha2, beta1, beta2
            write(irefwr,1) 'i1, i2, j1, j2', i1, i2, j1, j2

         end if

!        --- Mark cell i1,j1 as boundary cell

         call mshobsfledge1 ( iobgrid, nx, ny, i1, j1, numcells, &
                              iedge, ipiwork, iwork, leniwork, jpiwork )
         if ( ierror/=0 ) go to 1000

!        --- Mark all cells at the intersection line as boundary cells

         if ( i1==i2 ) then

!        --- i1 = i2

            if ( abs(j2-j1)>0 ) then

!           --- j1 # j2, hence there are more cells

               if ( j1>j2 ) then

!              --- j1>j2, so from j2 to j1-1

                  jstep = -1

               else

!              --- j2>j1, so from j1+1 to j2

                  jstep = 1

               end if  ! ( j1>j2 ) then

               do j = j1+jstep, j2, jstep

!              --- Mark all cells i1,j

                  call mshobsfledge1 ( iobgrid, nx, ny, i1, j, numcells, &
                                       iedge, ipiwork, iwork, leniwork, &
                                       jpiwork )
                  if ( ierror/=0 ) go to 1000

               end do  ! j = jlow, jtop, jstep

            end if

         else

!        --- i1 # i2

            if ( j1==j2 ) then

!           --- j1 = j2 and i1 # i2

               if ( i1>i2 ) then

!              --- i1>i2, so from i2 to i1-1

                  istep = -1

               else

!              --- i2>i1, so from i1+1 to i2

                  istep = 1

               end if  ! ( j1>j2 ) then

               do i = i1+istep, i2, istep

!              --- Mark all cells i,j1

                  call mshobsfledge1 ( iobgrid, nx, ny, i, j1, numcells, &
                                       iedge, ipiwork, iwork, leniwork, &
                                       jpiwork )
                  if ( ierror/=0 ) go to 1000

               end do  ! i = ilow, itop, istep

            else

!           --- j1 # j2 and i1 # i2
!               First compute all cross sections of the edge in the i-direction

               if ( i2>i1 ) then
                  ksign = 1
               else
                  ksign = -1
               end if  ! ( i2>i1 )
               jprev = j1

               do i = i1, i2, ksign

                  alpha = i
                  s = (alpha-alpha1)/(alpha2-alpha1)
                  beta = beta1 + s*(beta2-beta1)
                  j = min(int(max(beta,0d0)),ny-1)

                  do k = min(j,jprev), max(j,jprev)

                     if ( k>min(j1-1,j2-1) .and. &
                          k<max(j1+1,j2+1) ) then

!                    --- Mark all cells i,k

                        call mshobsfledge1 ( iobgrid, nx, ny, i, k, &
                                             numcells, iedge, ipiwork, &
                                             iwork, leniwork, jpiwork )
                        if ( ierror/=0 ) go to 1000

                     end if  ! ( k>min(j1-1,j2-1) ...

                  end do  ! k = min(j,jprev), max(j,jprev)

                  jprev = j

               end do  ! i = i1, i2, ksign

!              --- Next compute all cross sections of the edge in the j-dir

               iprev = i1
               if ( j2>j1 ) then
                  ksign = 1
               else
                  ksign = -1
               end if  ! ( j2>j1 )

               do j = j1, j2, ksign
                  beta = j
                  s = (beta-beta1)/(beta2-beta1)
                  alpha = alpha1 + s*(alpha2-alpha1)
                  i = min(int(max(alpha,0d0)),nx-1)
                  do k = min(i,iprev), max(i,iprev)
                     if ( k>min(i1-1,i2-1) .and. &
                          k<max(i1+1,i2+1) ) then

!                    --- Mark all cells k,j

                        call mshobsfledge1 ( iobgrid, nx, ny, k, j, &
                                             numcells, iedge, ipiwork, &
                                             iwork, leniwork, jpiwork )
                        if ( ierror/=0 ) go to 1000

                     end if  ! ( k>min(i1-1,i2-1) ...

                  end do  ! k = min(i,iprev), max(i,iprev)

                  iprev = i

               end do  ! j = j1, j2, ksign

            end if  ! ( j1==j2 )

         end if  ! ( i1==i2 )

         x1 = x2
         y1 = y2

         if ( debug ) then
            call prinin ( numcells(1,iedge), 2, 'numcells' )
            call prinin1 ( iwork(1,jpiwork), ipiwork-jpiwork+1, 2, &
                           'iwork' )
         end if  ! ( debug )

      end do  ! iedge = 1, n-1

1000  if ( debug ) then

!     --- Debug information

         call prinin1 ( numcells, n-1, 2, 'numcells' )
         call prinin1 ( iwork(1,1), ipiwork, 2, 'iwork' )
         call prinin1 ( iobgrid, ny, nx, 'iobgrid' )
         write(irefwr,*) 'End mshobsfledge'

      end if  ! ( debug )
      call erclos ( 'mshobsfledge' )

      end
