      subroutine mshobstmarkel2 ( numelmobst, ielemobstlc, &
                                  istore, lstore, iobst, curves, nx, ny, &
                                  iwork, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 02-10-2012 New definition of iobgrid2d
!        version  1.2  date 19-02-2003 Enlargement of length icell (RvL)
!        version  1.1  date 28-11-2001 Test on length icell
!
!   copyright (c) 2001-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all elements that intersect the obstacle but do not have a node
!     inside the obstacle
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     element
!     obstacle
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numelmobst, ielemobstlc(*), lstore, &
              istore(*), iobst, nx, ny, &
              iwork(*), nnodes
      double precision curves(2,*)

!     curves         i    array containing the coordinates of the curves
!     ielemobstlc      i    Array containing information about the elements in an
!                         obstacle
!                         The first numobst positions contain the starting
!                         positions of each obstacle sequentially
!                         Per obstacle the first three positions contain
!                         1. The number of elements that are completely inside
!                            the obstacle
!                         2. The number of elements that are not completely
!                            inside the element but with all vertices inside
!                            the element
!                         3. The number of elements that are partly inside
!                            the obstacle
!                            elements that contain only boundary points of the
!                            obstacle are not considered
!                         These positions are followed by the element numbers
!                         in the sequence defined by the first two positions
!     iobst          i    Obstacle sequence number
!     istore         o    In this array all edges are stored that are candidates
!                         for a cross section with the obstacle, although their
!                         end points are not in the obstacle
!     iwork         i/o   integer work array of length nelem
!                         if ( iwork(ielem) = 0 element ielem may be outside
!                         the obstacle
!                         if ( iwork(ielem) = 1 element ielem is inside
!                         the obstacle
!     lstore         o    number of entries in array istore
!     nnodes         i    Number of nodes in obstacle
!     numelmobst     i    Number of entries in array ielemobstlc
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxcells
      parameter (maxcells=60)
      integer i, ind, k, icell(maxcells), ncell, &
              nstore, ielem, ipstart, nedge_elem, j, iedge, jpstart, &
              nelem_edge, jelem, nnode_edge, inode1, inode2
      double precision xmin, ymin, dx, dy, eps, x1(2), &
                       x2(2), xlow(2), xupp(2), fact1, fact2

!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     eps            Local accuracy
!     fact1          Defines the factor for the intersection point
!                    The intersection point on the face is defined by
!                    x1 + fact1 * (x2-x1)
!     fact2          Defines the factor for the intersection point with respect
!                    to the obstacle curve
!                    There is only an intersection if 0<=fact2<=1
!     i              Counting variable
!     icell          Array containing the curve parts that must be considered
!                    for computing the cross sections
!     iedge          edge sequence number
!     ielem          Element number
!     ind            indicates index in curve part
!     inode1         First node in edge
!     inode2         Second node in edge
!     ipstart        Starting address in array
!     j              Counting variable
!     jelem          do loop variable with respect to the elements
!     jpstart        Starting address in array
!     k              Counting variable
!     maxcells       Declared length of array icell
!     ncell          Number of curve parts that must be used to compute
!                    cross sections
!     nedge_elem     number of edges corresponding to element
!     nelem_edge     number of elements corresponding to edge
!     nnode_edge     number of nodes corresponding to edge
!     nstore         Counter with respect to array istore
!     x1             x-coordinate of first point
!     x2             x-coordinate of last point
!     xlow           contains coordinates of first point on curve part
!     xmin           Minimum x-value
!     xupp           contains coordinates of last point on curve part
!     ymin           Minimum y-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCROSSLINE   Computes the intersection of two lines
!     MSHOBSTINTLIN1 Fill array icell with indices of curve parts of the
!                    obstacle that is possibly intersected by the line x1-x2
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Extract xmin,xmax,ymin,ymax from obstacle array
!     for i := 1 (1) numelmobst
!        ielem := next element in ielemobstlc
!        for j := 1 (1) number of edges of the element
!           iedge := next edge
!           for k := 1 (1) number of elements of the edge
!              jelem := next element
!              if (jelem outside the mesh ) then
!                 store edge in istore
!     Remove all edges in istore that do not intersect the obstacle
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstmarkel2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstmarkel2'
         write(irefwr,1) 'nx, ny', nx, ny
         call prinin ( iobgrid2d1, 10, 'iobgrid2d1' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Extract xmin,xmax,ymin,ymax from obstacle array

      xmin = rmeshx(1,iobst)
      ymin = rmeshx(3,iobst)
      dx   = rmeshx(5,iobst)
      dy   = rmeshx(6,iobst)
      eps  = 0.01d0

      nstore = 0

!     --- Loop over the elements in the obstacle

      do i = 1, numelmobst

!     --- element ielem

         ielem = ielemobstlc(i)
         ipstart = nmeshd(ielem)
         nedge_elem = nmeshd(ipstart)

!        --- Loop over the edges of the element

         do j = 1, nedge_elem

!        --- edge iedge

            iedge = abs(nmeshd(ipstart+j))
            jpstart = nmeshh(iedge)
            nelem_edge = nmeshh(jpstart)

!           --- Loop over the elements of the edge

            do k = 1, nelem_edge

!           --- element jelem

               jelem = nmeshh(jpstart+k)
               if ( iwork(jelem)==0 ) then

!              --- element outside the mesh, store edge in istore

                  nstore = nstore+1
                  istore(nstore) = iedge
                  go to 100

               end if

            end do

100      end do

      end do

!     --- Remove all edges in istore that do not intersect the obstacle

      lstore = 0
      do j = 1, nstore

!     --- edge iedge

         iedge = istore(j)
         ipstart = nmeshc(iedge)
         nnode_edge = nmeshc(ipstart)

!        --- store initial and end node in inode1/2

         inode1 = nmeshc(ipstart+1)
         inode2 = nmeshc(ipstart+nnode_edge)
         x1(1) = coor(1,inode1)
         x1(2) = coor(2,inode1)
         x2(1) = coor(1,inode2)
         x2(2) = coor(2,inode2)

!        --- Fill array icell with indices of curve parts of the obstacle that
!            is possibily intersected by the line x1-x2

         call mshobstintlin1 ( x1, x2, xmin, ymin, nx, ny, iobgrid2d, &
                               icell, ncell, dx, dy, nnodes, maxcells, &
                               iobgrid2d1 )
         if ( ierror/=0 ) go to 1000

!        --- For i := 1 (1) ncell do

         do i = 1, ncell

!        --- xlow := first point of obstacle part icell(i)
!            xupp := last point of obstacle part icell(i)

            ind = icell(i)
            xlow(1) = curves(1,ind)
            xlow(2) = curves(2,ind)
            xupp(1) = curves(1,ind+1)
            xupp(2) = curves(2,ind+1)

!           --- Check if lines intersect and compute intersection

            call mshcrossline ( x1, x2, xlow, xupp, fact1, fact2, eps )

            if ( fact2>-eps .and. fact2<1d0+eps ) then

!           --- the line x1-x2 intersects the line xlow-xupp

               lstore = lstore+1
               istore(lstore) = iedge
               go to 200

            end if

         end do

200   end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstmarkel2'

      end if  ! ( debug )
      call erclos ( 'mshobstmarkel2' )

      end subroutine mshobstmarkel2
