      subroutine mshpav ( coor, kbndpt, bcord, npoint, &
                          nelem, maxbounds, maxlen )
! ======================================================================
!
!        programmer    Bas van Rens
!        version  6.1  date 29-05-2014 Extension of timebefore
!        version  6.0  date 02-09-2010 Remove intarmsh
!        version  5.0  date 29-01-2010 Extra parameter intarmsh
!
!   copyright (c) 1997-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mesh generator that is based on paving combined with an
!     advancing front method.  First a layer of a certain thickness
!     is meshed around the boundary of the mesh then an advancing
!     front step is used to tie the meshes together
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(2,*), bcord(4,*), maxlen
      integer kbndpt(*), npoint, nelem, maxbounds

!     bcord          i    array containing the coordinates of the boundaries
!                         as well as information about the thickness of the
!                         layers
!                         bcord(1-2,i) contains the co-ordinates of the i-th
!                         boundary node
!                         bcord(3,i) contains the thickness of the i-th
!                         boundary node.
!                         If negative this means start of a new part
!                         bcord(4,i) contains the factor f of the i-th
!                         boundary node
!     coor           o    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect
!                         to the global numbering
!     kbndpt         o    array containing the nodal point numbers of the
!                         boundary
!     maxbounds      i    Max number of loops in nowbnd
!     maxlen        i/o   Maximum length of edges in the surface
!     nelem          o    Number of elements
!     npoint         o    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncrd, nloop, nel, ndim, maxloop, lnnowbnd, &
              gridinfo(0:10*nbndpt+1), lengridinfo, &
              gridfilled(0:4*nbndpt+1), lengridfilled, numgridfilled, &
              nowbnd(maxbounds,nbndpt+maxbounds)
      integer, allocatable :: grid(:,:)
      double precision rgridinfo(4), crd(4,npoint)
      logical plotdebug
      double precision :: timebefore(2)

!     crd            Work array of length 4*npoint containing the
!                    co-ordinates of the nodes of the actual boundary
!                    Pos. 1-2 contains the coordinates
!                    Pos. 3 contains the thickness of the i-th
!                    boundary node
!                    Pos. 4 contains the factor f of the i-th
!                    boundary node
!     grid           Work array of size at most 101 x 101 to store
!                    information about the boundary in relation with
!                    arrays gridinfo and rgridinfo
!                    The actual size is (0:nx, 0:ny) and (ix,iy) refers
!                    to the cell ( xmin+ix*dx, ymin+iy*dy) x
!                    ( xmin+(ix+1)*dx, ymin+(iy+1)*dy)
!                    if grid(ix,iy) = 0, no boundary node is stored in cell
!                    (ix,iy), if>0 information about the boundary cells
!                    is stored in array grdinfo from position
!                    grid(ix,iy)
!     gridfilled     Work array of size 2 x lengridfilled, indicating which
!                    cells in array grid have value larger than zero
!                    gridfilled(1,i) contains ix and
!                    gridfilled(2,i) contains iy for a cell with
!                    grid(ix,iy)>0
!                    The first and last position are used to store control
!                    integers
!     gridinfo       In this array the boundary points corresponding to a
!                    specific grid cell in grid are stored
!                    istart = grid(ix,iy)
!                    In gridinfo(istart) the number of boundary points of cell
!                    (ix,iy) are stored (if>0).
!                    This is followed by 2 positions for each boundary point
!                    The pair consists of the loop sequence number in the
!                    boundary where loop refers to a closed loop and
!                    a sequence number in the loop
!                    The first and last position are used to store control
!                    integers
!     lengridfilled  Contains the declared length of array gridfilled
!     lengridinfo    Contains the declared length of array gridinfo
!     lnnowbnd       Declared length of second index of array nowbnd
!     maxloop        Max number of loops in nowbnd
!     ncrd           Number of nodes stored in array crd
!     ndim           Dimension of the space.
!     nel            number of elements in the topology
!     nloop          number of boundaries in nowbnd
!     nowbnd         work array containing the boundary nodes for each
!                    closed part
!                    This array is two-dimensional of size (maxbounds,*), &
!                    where implxs defines the maximum number of loops
!                    and * is at most the number of nodes in the boundary+1
!                    Pos. (i,1) contains the number of nodes in the closed
!                       boundary part i
!                    Pos. (i,j+1) contains the j-th node number along this
!                       boundary
!     numgridfilled  Contains the number of gridpoints filled in array
!                    gridfilled
!     plotdebug      If true plots must be made for debugging
!     rgridinfo      Array of size 4 containing the following information
!                    about array grid
!                    1:  xmin
!                    2:  ymin
!                    3:  dx
!                    4:  dy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCONTROL     Check control integers around integer array
!     ERSETTIME      Get the CPU time at start of subroutine
!     MSHPAVCF       copies the relevant nodes from crd to coor
!     MSHPAVCLEARGRD Clear array grid
!     MSHPAVFILLCRD  Fill the arrays crd, kbndpt and nowbnd for the PAVER
!                    algorithm
!     MSHPAVMAXLOOP  Detect the maximum number of layers that may be constructed
!                    before triangle is applied
!     MSHPAVSIZES    Compute minimum and maximum x and y coordinates of region
!                    Compute distances dx and dy for square grid covering region
!     MSHPAVTRLOOP   Create paver mesh
!     PL0001         Compute the parameters xmin, xmax, ymin and ymax
!     PRINRL1        Print 2d real vector
!     PRINTTIME      Print the CPU time at end of subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     fill crd by copying bcord
!     fill array kbndpt
!     Initialize array nowbnd
!     Detect number of loops nloop
!     Compute maxloop = maximum number of layers to be generated before
!     triangle is used
!     treat region by using layers first and triangle later on
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpav' )
      if ( itime>0 ) call ersettime ( timebefore )
      debug = .false.
      plotdebug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpav'
         write(irefwr,1) 'nbndpt, maxbounds', &
                          nbndpt, maxbounds
         write(irefwr,2) 'maxlen', maxlen
         call prinrl1 ( bcord, nbndpt, 4, 'bcord' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( plotdebug ) jtimes = 0

!     --- Set control integers

      gridinfo(0) = 1001
      gridinfo(10*nbndpt+1) = 1001
      gridfilled(0) = 1002
      gridfilled(4*nbndpt+1) = 1002

!     --- ndim is set to two.  Always 2d.

      ndim = 2
      nel = 0
      ncrd = nbndpt-1
      lengridinfo = 10*nbndpt
      lengridfilled = 2*nbndpt
      numgridfilled = 0
      lnnowbnd = nbndpt+maxbounds

!     --- Compute xmin, xmax, ymin, ymax for plotting

      if ( jmax==0 ) call pl0001 ( bcord, 4, nbndpt, xmin, xmax, ymin, ymax )
      if ( ierror/=0 ) go to 1000

!     --- Fill the arrays crd, kbndpt and nowbnd for the PAVER algorithm

      call mshpavfillcrd ( bcord, nbndpt, crd, maxbounds, lnnowbnd, &
                           nloop, kbndpt, nowbnd, maxlen, isurnr )
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,*) 'maxlen ', maxlen

!     --- Detect parameter for covering region with rectangular grid

      call mshpavsizes ( nbndpt, crd, rgridinfo, &
                         nx, ny, nowbnd, maxbounds, nloop )
      if ( ierror/=0 ) go to 1000

      allocate ( grid(0:nx,0:ny), stat = error )
      if ( error/=0 ) call eralloc ( error, (nx+1)*(ny+1), 'grid' )
      if ( ierror/=0 ) go to 1000

!     --- Clear arrays grid and gridfilled

      call mshpavcleargrd ( grid, nx, ny, gridfilled(1), lengridfilled )
      if ( ierror/=0 ) go to 1000

!     --- Compute maxloop depending on nlayers, nloop and nowbnd

      call mshpavmaxloop ( nlayers, nloop, maxloop, maxbounds, &
                           nowbnd, crd )
      if ( debug ) write(irefwr,1) 'nloop, maxloop, maxbounds', &
          nloop, maxloop, maxbounds

!     --- Perform the actual algorithm

      call mshpavtrloop ( nowbnd, nloop, nel, crd, &
                          maxbounds, lnnowbnd, rgridinfo, &
                          grid, gridinfo(1), lengridinfo, &
                          gridfilled(1), lengridfilled, numgridfilled, &
                          maxlen, ncrd, maxloop, npoint, nelem )

      deallocate ( grid, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'grid' )

!     --- Write the final mesh to coor and kmeshc
!         This means that gaps are removed

      call mshpavcf ( coor, npoint, nel, kbndpt, &
                      crd, ncrd, ndim, bcord(ndim,1) )
      if ( ierror/=0 ) go to 1000

!     --- fill nelem

      nelem = nel

      if ( itime==1 ) call printtime('PAVER')

!     --- Check control integers

      call errcontrol ( gridinfo(0), gridinfo(10*nbndpt+1), 1001, &
                        'gridinfo' )
      call errcontrol ( gridfilled(0), gridfilled(4*nbndpt+1), 1002, &
                        'gridfilled' )

 1000 call erclos ( 'mshpav' )
      if ( debug ) write(irefwr,*) 'End mshpav'

      end
