      subroutine mshr14 ( coor, numnp1, numnp2, kmeshc, inpelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.5  date 27-06-2013 Only change coordinates if
!                                      close to the mid point
!        version  3.4  date 06-09-2003 Correction centroid quadrilateral
!        version  3.3  date 07-12-2002 Remove part corresponding to
!                                      centroids of 7-point triangles
!
!   copyright (c) 1982-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    adapt coordinates of midpoints for quadratic elements
!
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     centre
!     mesh
!     mesh_generation
!     rectangle
!     transformation
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnp1, numnp2, inpelm, kmeshc(inpelm,*)
      double precision coor(2,*)

!     coor      i/o   array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     inpelm     i    number of points per element
!     kmeshc     i    kmesh part c, contains the nodal points corresponding
!                     to the elements
!     numnp1     i    number of nodal points along side 1 minus one
!     numnp2     i    number of nodal points along side 2 minus one
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, ij1,ij2, ij, in, n1, num1, num2
      double precision maxcoor, eps, diff(2), lendiff, x(2)

!     diff           Difference between old coordinates and the middle
!                    of the two neighbors
!     eps            Local accuracy
!                    Coordinates are only adapted if they are closer to
!                    the mid point than eps
!     i              Counting variable
!     ij             defines point in which coordinates must be computed
!     ij1            neighbour point
!     ij2            neighbour point
!     in             (i-1) x lrow
!     j              Counting variable
!     lendiff        Squared length of diff
!     maxcoor        Largest x or y coordinate in the surface
!     n1             Help variable
!     num1           Help variable
!     num2           Help variable
!     x              contains co-ordinates of middle of the 2 neighboring nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshr14' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshr14'
         call prinrl1 ( coor, (numnp1+1)*(numnp2+1), 2, 'coor' )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      n1 = numnp1+1
      if ( inpelm==7 ) n1 = n1+numnp1/2
      num1 = numnp1-1
      num2 = numnp2-1

      maxcoor = maxval(abs(coor(:,1:(numnp1+1)*(numnp2+1))))  ! max coordinate
      eps = maxcoor*5d-4  ! accuracy

      if ( debug ) write(irefwr,2) 'maxcoor, eps', maxcoor, eps

      if ( numnp1>3 ) then

!     --- right points of quadrilaterals

         do j = 1, num2, 2
            in = n1*j+1
            do i = 2, num1, 2
               ij = in+i
               x(:) = 0.5d0*(coor(:,ij+n1)+coor(:,ij-n1))
               diff = x-coor(:,ij)
               lendiff = dot_product(diff,diff)
               if ( lendiff<eps ) coor(:,ij) = x
            end do
         end do

      end if

      if ( numnp2>3 ) then

!     --- upper parts of quadrilaterals

         do j = 2, num2, 2
            in = n1*j+1
            do i = 1, num1, 2
               ij = in+i
               x(:) = 0.5d0*(coor(:,ij+1)+coor(:,ij-1))
               diff = x-coor(:,ij)
               lendiff = dot_product(diff,diff)
               if ( lendiff<eps ) coor(:,ij) = x
            end do
         end do

      end if

      if ( debug ) call prinrl1 ( coor, (numnp1+1)*(numnp2+1), 2, 'coor/1' )

      if ( inpelm==9 ) then

!     --- centroids of quadrilaterals, for a quadrilateral
!         Value in centroid is 0.5 ( sum of 4 midside points )
!                             -0.25 ( sum of 4 vertices)
!         This corresponds to the basis functions of the serependity element
!         taken in the centroid of the reference element

         do j = 1, num2, 2
            in = n1*j+1
            do i = 1, num1, 2
               ij = in+i
               coor(1,ij) = 0.5d0*(coor(1,ij+1)+coor(1,ij-1)+ &
                                   coor(1,ij+n1)+coor(1,ij-n1))- &
                            0.25d0*(coor(1,ij+1-n1)+coor(1,ij-1-n1)+ &
                                   coor(1,ij+1+n1)+coor(1,ij-1+n1))
               coor(2,ij) = 0.5d0*(coor(2,ij+1)+coor(2,ij-1)+ &
                                   coor(2,ij+n1)+coor(2,ij-n1))- &
                            0.25d0*(coor(2,ij+1-n1)+coor(2,ij-1-n1)+ &
                                   coor(2,ij+1+n1)+coor(2,ij-1+n1))
            end do
         end do

      else

!     --- centroids of quadrilaterals, in case of triangles

         do i = 1, numnp1*numnp2/2, 2
            ij  = kmeshc(4,i)
            ij1 = kmeshc(3,i)
            ij2 = kmeshc(5,i)
            coor(1,ij) = 0.5d0*(coor(1,ij1)+coor(1,ij2))
            coor(2,ij) = 0.5d0*(coor(2,ij1)+coor(2,ij2))
         end do

      end if

1000  call erclos ( 'mshr14' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( coor, (numnp1+1)*(numnp2+1), 2, 'coor/end' )
         write(irefwr,*) 'End mshr14'

      end if  ! ( debug )

      end
