      subroutine mshrd1curnr ( iinp, inpmax, ipoint, maxcrv, icurnr, kcurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 10-01-2015 New call to readreal
!        version  1.1  date 08-12-2014 Allow lines with more points
!        version  1.0  date 20-11-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read first part of curve record and detect curve number
!
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer,intent(in) :: inpmax, maxcrv
      integer,intent(inout) :: iinp, ipoint(maxcrv), icurnr, kcurvs

!     icurnr        i/o   Curve sequence number
!     iinp          i/o   last position used in array iinput
!     inpmax         i    Available space for array iinput
!     ipoint        i/o   pointer array: contains the starting positions of the
!                         curves in array iinput
!     kcurvs        i/o   Actual number of curves found (<= ncurvs)
!     maxcrv         i    Maximal number of curves allowed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical ::  openfound

!     openfound      Indicates if an openings bracket is found before
!                    reading the number or expression
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     READREAL       Read a real or real expression from the input file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     139   curve number does not exist
!     142   Internal error, too few space for integers
!     276   Curve has already been filled
!     648   string could not be recognized
!    1334   Record does not start with a character
!    2956   No curve number found after the keyword C
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrd1curnr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd1curnr'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read next record

      ityprd = 1    ! indicates that new record must be read
      call asknxt

      if ( debug ) write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem

      if ( iinp>=inpmax ) then

!     --- iinp>=  inpmax

          call errint ( inpmax, 1 )
          call errint ( iinp, 2 )
          call errsub ( 142, 2, 0, 0 )
          go to 1000

      end if

      if ( ityprd/=2 .and. ityprd/=4 ) then

!     --- error 1334: Record does not start with a character
!                     nor an integer

         call errsub ( 1334, 0, 0, 0 )
         call errcrd
         go to 1000

      else if ( ityprd==4 .and. &
                .not. (nchar==1 .and. string(1:1)=='C') ) then

!     --- Detect the type of record
!         Skip this part if the keyword = C
!         If a main keyword has been found, return to calling subroutine

         jcode = sepmainkeyw () - 1
         if ( debug ) then
            write(irefwr,1) 'jcode', jcode
            write(irefwr,1) 'ityprd, lastitem/2', ityprd, lastitem
            write(irefwr,*) 'string ', string(1:nchar)
         end if  ! ( debug )
         if ( jcode>=0 ) go to 1000

      end if

      if ( ityprd==4 ) then

!     --- Record starts with a character, this must be C

         if ( string(1:1)/='C' ) then
            call errchr ( string(1:1), 1 )
            call errchr ( 'C', 2 )
            call errsub ( 648, 0, 0, 2 )
            call errcrd
            go to 1000
         end if

!        --- detect the curve card
!            first the curve number is read

         openfound = .false.
         call readreal ( openfound, .false. )

      end if

      if ( ityprd/=2 ) then

!     --- Next item is not an integer

         call errsub ( 2956, 0, 0, 0 )
         call errcrd
         go to 1000

      end if

      icurnr = nint(realrd)
      ncurvs = max ( ncurvs, icurnr )
      kcurvs = kcurvs + 1
      if ( icurnr<=0 .or. icurnr>maxcrv ) then

!     --- icurnr has wrong value

         call errint ( icurnr, 1 )
         call errint ( maxcrv, 2 )
         call errsub ( 139, 2, 0, 0 )
         call errcrd
         go to 1000

      else if ( ipoint(icurnr)>0 ) then

!     --- Curve icurnr has already been filled

         call errint ( icurnr, 1 )
         call errsub ( 276, 1, 0, 0 )
         call errcrd
         go to 1000

      end if

      ipoint(icurnr) = iinp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'jcode', jcode
         call prinin ( ipoint, ncurvs, 'ipoint' )
         write(irefwr,*) 'End mshrd1curnr'

      end if  ! ( debug )
      call erclos ( 'mshrd1curnr' )

      icode = jcode

      end subroutine mshrd1curnr
