      subroutine mshrdb ( ndim, iinput, rinput, lengrl, lengin, lnrinp, &
                          lniinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.6  date 09-03-2013 Correction length code
!        version  1.5  date 13-11-2010 error 1488 --> errkeywexcl
!        version  1.4  date 12-03-2008 New way of defining scalars
!
!   copyright (c) 1996-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     subroutine MSHRDB reads all records with respect to the option
!     NODAL_POINTS
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduleconsts
      use sepmoduleplot
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, iinput(*), lengrl, lengin, lnrinp, lniinp
      double precision rinput(*)

!     iinput     o    integer input array with the following contents:
!                     1: ndim
!                     2: istruct
!                        0=unstructured
!                        1=structured
!                        2=rectangular
!                     3-5: information of the mesh:
!                        unstructured:
!                        3= number of boundary points
!                        4= number of internal points
!                        structured and rectangular
!                        3= nx
!                        4= ny
!                        5= nz
!                     6: Type of elements (See table 2.2.1 in users manual)
!                     7-10: information of the plots:
!                        7= iplot (0=no plot, 1=plot)
!                        8= jmark
!                        9= orientation (3D mesh)
!                       10= view point given (1) or not (0)
!     lengin     o    Length of integer array iinput
!     lengrl     o    Length of real array rinput
!                     (in double precision variables)
!     lniinp     i    Available space for array IINPUT
!     lnrinp     i    Available space for array RINPUT
!     ndim       i    Dimension of the space.
!     rinput     o    real input array with the following contents:
!                     1: plotfm
!                     2: yfact
!                     3-5: eye point
!                     6-end: information of the co-ordinates:
!                       (un)structured:
!                         co-ordinates in sequence x_1, y_1, z_1, x_2, ..
!                       rectangular:
!                       6: O_1
!                       7: O_2
!                       8: O_3
!                       9: dx
!                      10: dy
!                      11: dz
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode3=11, nncod2=9, nncode1=4
      integer i, ncode3(nncode3), ncodmx3(nncode3), nmcode(1), &
              itype3(nncode3), ncode1(nncode1), ncodmx1(nncode1), &
              itype2(nncod2), ncode2(nncod2), ncodm2(nncod2), inext, m, &
              iorien, firstpos, maxlen
      character (len=14) code3(nncode3)
      character (len=17) code1(nncode1)
      character (len=11) code2(nncod2)
      double precision plotfm, yfact, tabmn2(nncod2), tabmx2(nncod2), &
                       tabmn3(nncode3), tabmx3(nncode3)
      logical finished, internfn, boundrfn

!     boundrfn   If true boundary points have been found
!     code1      Table with keywords
!     code2      Table with keywords
!     code3      Table with keywords
!     finished   Indicates if the loop has been finished (true) or not (false)
!     firstpos   First position in an array
!     i          Counting variable
!     inext      Next point in curve
!     internfn   If true internal points have been found
!     iorien     Defines orientation of the base vectors in 3D plots
!                1 x-y-z
!                2 z-x-y
!                3 y-z-x
!     itype2     Indicates the type of action corresponding to keywords
!                in code2
!     itype3     Table containing information about the numbers behind
!                the keywords in the table code3
!     m          number of items found
!     maxlen     maximum space available
!     ncode1     Table containing the number of significant letters in code1
!     ncode2     Table containing the number of significant letters in code2
!     ncode3     Table containing the number of significant characters in
!                the keyword table code3
!     ncodm2     maximum number of characters in table code2
!     ncodmx1    Table containing the maximal number of  characters in
!                the keyword table code1
!     ncodmx3    Table containing the maximal number of  characters in
!                the keyword table code3
!     nmcode     Table containing the number of significant characters in
!                the main keyword table codemn, including those of
!                the extra texts
!     nncod2     Number of entries in array code2
!     nncode1    Number of entries in array code1
!     nncode3    Number of entries in array code3
!     plotfm     Length of plot
!     tabmn2     Table containing the minimum values of the quantities
!                given in code2
!     tabmn3     Contains minimum values allowed in table
!     tabmx2     Table containing the maximum values of the quantities
!                given in code2
!     tabmx3     Contains maximum values allowed in table
!     yfact      Multiplication for the y-co-ordinates. With this
!                parameter the user can influence the scale in y-direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     READNUMB       Read a series of numbers until end of file or a text is
!                    found
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODEMN1 Copy codemn2 and related arrays into codemain
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    The following input from the standard input file is expected:
!    NODAL_POINTS, options
!       options available:
!            UNSTRUCTURED   (Default)
!            STRUCTURED ( NX=n, NY=m, NZ=k )
!            RECTANGULAR ( NX=n, NY=m, NZ=k, DX=dx, DY=dy, DZ=dz, &
!                          ORIGIN = (Ox,Oy,Oz) )
!            ELEMENT_SHAPE = i
!       BOUNDARY_POINTS
!          co-ordinates of boundary points
!       INTERNAL_POINTS
!          co-ordinates of internal points
!       PLOT, options
!    END
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     142   Internal error, too few space for integers
!     975   Only triangles and tetrahedra allowed for unstructured grids
!     976   Only linear elements are allowed
!    1391   Not enough numbers found after eyepoint.
!    1640   Number of internal points found incorrect
!    1980   keyword may only be used for structured grids
!    1981   keyword not allowed for this dimension
!    1982   keyword may only be used for rectangular grids
!    1984   keyword boundary_points has been found, but the mesh is structured
!    1985   keyword has been found more than once in the file
!    1986   The keyword boundary_points has been found, but alfinished the keyword
!           internal_points has been read
!    1987   keyword internal_points has been found, but the mesh is rectangular
!    1988   Too few reals available in array
!    1990   Number of co-ordinates read for boundary points not a multiple of
!           ndim
!    1991   Number of co-ordinates read for internal points not a multiple of
!           ndim
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Read options
!   While ( next card not END ) do
!      Read next keyword
!      If ( next keyword = BOUNDARY_POINTS and not unstructured grid ) then
!         error
!      If ( next keyword = BOUNDARY_POINTS ) then
!         read and count co-ordinates
!      If ( next keyword = INTERNAL_POINTS and rectagular grid ) then
!         error
!      If ( next keyword = INTERNAL_POINTS ) then
!         read and count co-ordinates
!      If ( next keyword = PLOT ) then
!         read plot options
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code3 / 'UNSTRUCTURED', 'STRUCTURED', 'RECTANGULAR', &
                  'NX', 'NY', 'NZ', 'DX', 'DY', 'DZ', 'ORIGIN', &
                  'ELEMENT_SHAPE' /
      data ncode3  / 5, 5, 4, 2, 2, 2, 2, 2, 2, 3, 4 /
      data ncodmx3 /12,10,11, 2, 2, 2, 2, 2, 2, 6,13 /
      data itype3  / 2, 2, 2, 0, 0, 0, 1, 1, 1,19, 0 /
      data tabmn3 / 0d0, 0d0, 0d0, 1d0, 1d0, 1d0, &
                    1d-18, 1d-18, 1d-18, 3d0, 3d0 /
      data tabmx3 / 0d0, 0d0, 0d0, 1d5, 1d5, 1d5, &
                    1d18,  1d18,  1d18, 1d18, 17d0 /
      data code1 / 'BOUNDARY_POINTS', 'INTERNAL_POINTS', 'PLOT', 'END' /
      data ncode1  / 4, 3, 4, 3 /
      data ncodmx1 /15,15, 4, 3 /
      data code2 / 'LENGTH', 'SCALE', 'YFACT', 'JMARK', &
                   'ROTATE', 'SUPPRESS', 'EYEPOINT', &
                   'PLOTFM', 'ORIENTATIO' /
      data itype2 / 1,  1, 1, 0, 0, 0, 19, 1, 0 /
      data ncode2 / 4,  3, 2, 3, 3, 4,  3, 4, 5 /
      data ncodm2 / 6,  5, 5, 5, 6, 8,  9, 6,11 /
      data tabmn2 / 3d0, 1d-8, 1d-8, 0d0, 0d0, 0d0,  3d0, 3d0, 1d0 /
      data tabmx2 / 1d3,  1d8, 1d8,  5d0, 2d0, 1d0, 1d28, 1d3, 3d0 /

!     code3    Table with keywords corresponding to options
!             1:  UNSTRUCTURED
!             2:  STRUCTURED
!             3:  RECTANGULAR
!             4:  NX
!             5:  NY
!             6:  NZ
!             7:  DX
!             8:  DY
!             9:  DZ
!            10:  ORIGIN
!            11:  ELEMENT_SHAPE
!     code1   Table with keywords
!             1:  BOUNDARY_POINTS
!             2:  INTERNAL_POINTS
!             3:  PLOT
!             4:  END
!    code2     In this table the significant characters of the various
!              texts after plot are stored
!              The following types of texts are recognised:
!              1:   length
!              2:   scale
!              3:   yfact
!              4:   jmark
!              5:   rotate
!              6:   suppress
!              7:   eyepoint
!              8:   plotfm
!              9:   orientation
! ======================================================================
!
      call eropen ( 'mshrdb' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrdb'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set defaults

      iinput(1) = ndim
      iinput(2) = 0
      iinput(3) = 0
      iinput(4) = 0
      if ( ndim==2 ) then
         iinput(5) = 0
         iinput(6) = 3
      else
         iinput(5) = 1
         iinput(6) = 11
      end if
      iinput(7) = 0
      iinput(8) = 5
      iinput(9) = 1
      iinput(10) = 0
      rinput(1) = 0d0
      rinput(2) = 0d0
      rinput(3) = 0d0
      rinput(4) = 0d0
      rinput(5) = 0d0
      yfact = 1d0
      plotfm = defplo
      lengrl = 11
      lengin = 10
      internfn = .false.
      boundrfn = .false.

!     --- Read options

      call sepcopycode ( code3, nncode3, ncode3, ncodmx3 )
      call sepcopytype ( nncode3, itype3, tabmn3, tabmx3 )

      tabmin(10) = ndim
      tabmax(10) = 1d19
      table(1:nncode) = -1d19
      if ( ndim==2 ) then
         tabmin(11) = 3d0
         tabmax(11) = 5d0
      else
         tabmin(11) = 11d0
         tabmax(11) = 13d0
      end if
      iretur = 0
      nmain = 0
      call sepreadsetkeyw

      if ( debug ) then
         write(irefwr,1) 'iretur', iretur
         call prinrl ( table, nncode, 'table' )
      end if  ! ( debug )

      do i = 1, 3
         inext = i+1
         if ( inext==4 ) inext = 1
         if ( table(i)>0d0 .and. table(inext)>0d0 ) &
             call errkeywexcl ( code, ncodmx, i, inext )
      end do

      if ( table(2)>0d0 ) then
         iinput(2) = 1
         iinput(3) = 0
         iinput(4) = 0
      else if ( table(3)>0d0 ) then
         iinput(2) = 2
         iinput(3) = 1
         iinput(4) = 1
      end if

      do i = 1, 3
         if ( table(i+3)>0d0 ) then
            if ( ndim<i ) then

!           --- nz only permitted for three-dimensional grids

               call errchr ( code(i+3), 1 )
               call errint ( ndim, 1 )
               call errsub ( 1981, 1, 0, 1 )
            end if

         if ( iinput(2)==0 ) then

!           --- nx only permitted for structured grids

               call errchr ( code(i+3), 1 )
               call errsub ( 1980, 0, 0, 1 )
            else
               iinput(2+i) = nint(table(3+i))
            end if
         end if
      end do

      do i = 1, 3
         if ( table(i+6)>-1d19 ) then
            if ( ndim<i ) then

!           --- dz only permitted for three-dimensional grids

               call errchr ( code(i+6), 1 )
               call errint ( ndim, 1 )
               call errsub ( 1981, 1, 0, 1 )
            end if
            if ( iinput(2)<=1 ) then

!           --- dx only permitted for rectangular grids

               call errchr ( code(i+6), 1 )
               call errsub ( 1982, 0, 0, 1 )
            else
               rinput(8+i) = table(6+i)
            end if
         else if ( iinput(2)==2 ) then
            rinput(8+i) = 1d0
         end if
         rinput(5+i) = 0d0
      end do

      if ( table(10)>0d0 ) then
         if ( iinput(2)<=1 ) then

!        --- origin only permitted for rectangular grids

            call errchr ( code(10), 1 )
            call errsub ( 1982, 0, 0, 1 )
         else
            iposut = nint(table(10))
            m = nint(tabmax(10))
            do i = 1, m
               rinput(5+i) = tabout(iposut-1+i)
            end do
         end if
      end if
      if ( table(11)>0d0 ) iinput(6) = nint(table(11))

      finished = .false.
100   if ( .not. finished ) then

!     --- read next record

         ityprd = 1
         call asknxt
150      call sepcopycodemn1 ( code1, nncode1, ncode1, ncodmx1 )
         ierpri = 1
         icode = sepmainkeyw()
         if ( debug ) write(irefwr,1) 'icode'
         go to ( 200, 300, 400, 500 ) icode

!        --- icode=1, boundary points

200         if ( iinput(2)/=0 ) call errsub ( 1984, 0, 0, 0 )
            if ( boundrfn ) then
               call errchr ( code1(1), 1 )
               call errsub ( 1985, 0, 0, 1 )
            end if
            boundrfn = .true.
            if ( internfn ) call errsub ( 1986, 0, 0, 0 )
            if ( ndim==3 ) then
               call errint ( ndim, 1 )
               call errchr ( code1(1), 1 )
               call errsub ( 1981, 1, 0, 1 )
            end if
            call readnumb ( rinput(6), lnrinp-5, iinput(3) )
            lengrl = 5+iinput(3)
            go to 150

!           --- icode=2, internal points

300         if ( iinput(2)==2 ) call errsub ( 1987, 0, 0, 0 )
            if ( internfn ) then
               call errchr ( code1(2), 1 )
               call errsub ( 1985, 0, 0, 1 )
            end if
            if ( iinput(2)==0 ) then

!           --- unstructured grid, there might be boundary points

               maxlen = lnrinp-iinput(3)-5
               firstpos = iinput(3)+6

            else

!           --- structured grid, no boundary points

               maxlen = lnrinp-5
               firstpos = 6

            end if
            call readnumb ( rinput(firstpos), maxlen, m )
            internfn = .true.
            if ( iinput(2)==0 ) then
               iinput(4) = m
            else if ( ndim==2 ) then
               if ( m/=2*iinput(3)*iinput(4) ) then
                  call errint ( m, 1 )
                  call errint ( 2*iinput(3)*iinput(4), 2 )
                  call errsub ( 1640, 2, 0, 0 )
               end if
            else
               if ( m/=3*iinput(3)*iinput(4)*iinput(5) ) then
                  call errint ( m, 1 )
                  call errint ( 3*iinput(3)*iinput(4)*iinput(5), 2 )
                  call errsub ( 1640, 2, 0, 0 )
               end if
            end if
            lengrl = firstpos+m-1
            go to 150

!           --- icode=3, plot found, read options

400         codemain(3) = 'mshrdbx'

            call sepcopycode ( code2, nncod2, ncode2, ncodm2 )
            call sepcopytype ( nncod2, itype2, tabmn2, tabmx2 )
            nmain = nncodemain
            table(1:nncode) = -1d19
            call sepreadsetkeyw
            codemain(3) = 'PLOT'
            jmark = 5
            iorien = 1
            if ( jkader<0 .and. jkader>-4 ) then
               jtimes = 1
            else
               jtimes = 0
            end if
            if ( ndim==3 ) then

!           --- NDIM = 3, set eyepoint into rinput

               rinput(3) = 2d0*xmax-xmin
               rinput(4) = 2d0*ymax-ymin
               rinput(5) = 2d0*zmax-zmin

            end if
            if ( table(1)>0d0 ) plotfm = table(1)
            if ( table(8)>0d0 ) plotfm = table(8)
            if ( plotfm<=0d0 ) plotfm = defplo
            if ( table(2)>0d0 ) plotfm = -table(2)
            if ( table(3)>0d0 ) yfact = table(3)
            if ( table(4)>-1d0 ) jmark = nint(table(4))
            if ( table(5)>-1d0 ) irotat = nint(table(5))
            if ( table(6)>-1d0 ) jtimes = nint(table(6))
            if ( table(7)>-1d0 ) then

!           --- eye point found

               if ( nint(tabmx2(7))/=3 ) call errsub ( 1391, 0, 0, 0 )
               do i = 1, 3
                  rinput(2+i) = tabout(i)
               end do
               iinput(10) = 1

            end if
            if ( table(9)>-1d0 ) iorien = nint(table(9))
            iinput(7) = 1
            iinput(8) = jmark
            iinput(9) = iorien
            go to 100

!           --- icode=4, end found

500         finished = .true.
            go to 100
      end if
      rinput(1) = plotfm
      rinput(2) = yfact
      if ( lengin>lniinp ) then
         call errint ( lniinp, 1 )
         call errint ( lengin, 2 )
         call errsub ( 142, 2, 0, 0 )
      end if
      if ( lengin>lnrinp ) then
         call errint ( lnrinp, 1 )
         call errint ( lengrl, 2 )
         call errsub ( 1988, 2, 0, 0 )
      end if
      if ( iinput(2)==0 ) then
         if ( mod(iinput(3),ndim)/=0 ) then
            call errint ( iinput(3), 1 )
            call errint ( ndim, 2 )
            call errsub ( 1990, 2, 0, 0 )
         end if
         iinput(3) = iinput(3)/ndim
         if ( mod(iinput(4),ndim)/=0 ) then
            call errint ( iinput(4), 1 )
            call errint ( ndim, 2 )
            call errsub ( 1991, 2, 0, 0 )
         end if
         iinput(4) = iinput(4)/ndim
      end if

!     --- Test shape numbers

      if ( iinput(2)==0 ) then

!     --- unstructured grid, only triangles and hexhahedrons allowed

         if ( ndim==2 .and. iinput(6)/=3 .or. &
              ndim==3 .and. iinput(6)/=11 ) then
            call errint ( iinput(6), 1 )
            call errint ( ndim, 2 )
            call errsub ( 975, 2, 0, 0 )
         end if

      else

!     --- structured grid, only linear elements allowed

         if ( ndim==2 .and. iinput(6)/=3 .and. iinput(6)/=5 .or. &
              ndim==3 .and. iinput(6)/=11 .and. iinput(6)/=13 ) then
            call errint ( iinput(6), 1 )
            call errint ( ndim, 2 )
            call errsub ( 976, 2, 0, 0 )
         end if
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrdb'

      end if  ! ( debug )
      call erclos ( 'mshrdb' )

      end subroutine mshrdb
