      subroutine mshrep ( coor , npoint, kmeshc, nelem, inpelm, &
                          nbndpt )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.0  date 03-11-2010 Make work arrays local
!        version  1.6  date 05-05-2002 Replace call to mshd7 by two calls
!        version  1.5  date 08-06-1996 New call to mshd6
!
!   copyright (c) 1990-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  Driving subroutine for the repositioning of an arbitrary mesh with linear
!  or quadratic elements of triangular or quadrilateral shape ( all in 2-D )
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(*)
      integer npoint, kmeshc(*), nelem, inpelm, nbndpt

!     coor      i/o   coordinate array
!     inpelm     i    number of points per element
!     kmeshc     i    element array
!     nbndpt     i    Number of nodal points in boundary, length of array kbndpt
!     nelem      i    Number of elements
!     npoint     i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision rim,dx,dy,ds
      integer i,j,npti,ntot,i1,i2,i3,i4,l3,l4,istep,nh,npt
      integer, allocatable :: iarray(:)

!     ds         distance
!     dx         distance in x-direction
!     dy         distance in y-direction
!     i          loop variable
!     i1         pointer to address in helparray
!     i2         pointer to address in helparray
!     i3         pointer to address in helparray
!     i4         pointer to address in helparray
!     iarray     work array
!     istep      indicator whether case is linear (1) or quadratic (2)
!     j          loop variable
!     l3         pointer to helparray
!     l4         pointer to helparray
!     nh         helpvariable for quadratic case
!     npt        highest node number in old mesh
!     npti       start number for new points
!     ntot       total length of helparrays
!     rim        accuracy value for repositioning
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHD06         Determine neighbours and adjacent elements
!     MSHD08         Replace coordinates
!     MSHD09         Make linear elements and determine max corner node
!                    (3-angles)
!     MSHD10         Make linear elements and determine max corner node
!                    (4-angles)
!     MSHD6          Places extra numbers for quadratic case (3-angles)
!     MSHD7          Places coorindates of centroids of 7-point triangles
!                    in array coor
!     MSHTRIANCENTR  Create centre point in 6-point triangles
!     MSHV8          Places extra numbers for quadratic case (4-angles)
!     MSHV9          Places extra centre nodes (4-angles)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     358      Wrong number for inpelm specified
!     359      Not even number of boundary points for quadratic case
! **********************************************************************
!
!                       PSEUDO CODE
!
!     trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrep' )
      ntot = 14 * npoint + 20
      allocate ( iarray(ntot), stat = error )
      if ( error/=0 ) call eralloc ( error, ntot, 'iarray' )
      if ( ierror/=0 ) go to 1000

!     --- Compute help length rim

      j   = 2 * nbndpt  -  1
      dx  =  coor(j)    -  coor(1)
      dy  =  coor(j+1)  -  coor(2)

      rim =  dx*dx + dy*dy

      do i = 2 , nbndpt

          dx = coor( 2*i-1 ) - coor( 2*i-3 )
          dy = coor( 2*i   ) - coor( 2*i-2 )

          ds = dx*dx + dy*dy

          if ( ds<rim ) rim = ds

      end do

!     --- Compute value

      rim = sqrt( rim )

      iarray = 0!     --- Clear help array IARRAY

!     --- First new point has node number npti

      npti = nbndpt + 1

!     --- Set starting positions for helparrays

      i1 =  1
      i2 =  1 +     npoint
      i3 =  1 + 2 * npoint
      i4 =  1 + 8 * npoint
      l3 =  6 * npoint
      l4 =  6 * npoint + 20

!     --- Jump to the right position depending on INPELM

      if ( inpelm==3 ) then

!     --- inpelm = 3 : Linear triangles

         istep=1

!        --- Determine all neighbours and adjacent elements

         call mshd06 ( kmeshc, nelem, npoint, nbndpt, 3, iarray(i1), &
                       iarray(i2), iarray(i3), iarray(i4), l3, l4, &
                       istep )

         if ( npoint>=npti ) then

!        --- Replace coordinates

            call mshd08(coor,npoint,npti,iarray(i1),iarray(i3),rim)

         end if

      else if ( inpelm==6 .or. inpelm==7 ) then

!     --- inpelm = 6 or 7 : Quadratic triangles

         nh  =  nbndpt / 2
         nh  =  2 * nh - nbndpt
         rim =  2 * rim

         if ( nh/=0 ) then

            call errint ( nbndpt, 1 )
            call errsub ( 359, 1, 0, 0 )
            go to 1000

         end if

         call mshd09(kmeshc,npt,nelem,inpelm)

!        --- Determine neighbours and adjacent elements

         istep = 2

         call mshd06 ( kmeshc, nelem, npt, nbndpt, 3, iarray(i1), &
                       iarray(i2), iarray(i3), iarray(i4), l3, l4, &
                       istep )

         if ( npt>=npti) then

              call mshd08 ( coor, npt, npti, iarray(i1), iarray(i3), &
                            rim )

         else

              npt = npti - 1

         end if

!        --- Place extra numbers due to the quadratic case

         npoint = npt

         call mshd6 ( coor, kmeshc, nelem, npoint, npt, npti, &
                      iarray(i1), iarray(i2), iarray(i3), iarray(i4) )

         if ( inpelm==7 ) then

!        --- inpelm = 7, also fill centroid

             call mshtriancentr ( kmeshc, npt, nelem, inpelm )
             call mshd7 ( kmeshc, coor, nelem, 2 )

         end if

         npoint = npt

      else if ( inpelm==4 ) then

!     --- inpelm = 4 :  Linear quadrilaterals

         istep  =  1

         call mshd06 ( kmeshc, nelem, npoint, nbndpt, 4, iarray(i1), &
                      iarray(i2), iarray(i3), iarray(i4), l3, l4, &
                      istep )

         call mshd08( coor, npoint, npti, iarray(i1), iarray(i3), rim )

      else if ( inpelm==8 .or. inpelm==9 ) then

!     --- inpelm = 8 or 9 : Quadratic quadrilaterals

         nh   =     nbndpt / 4
         nh   = 4 * nh - nbndpt

         if ( nh/=0 ) then

              call errint ( nbndpt, 1 )
              call errsub ( 359, 1, 0, 0 )
              go to 1000

         end if

         rim    =  rim * 2d0

         call mshd10(kmeshc,npt,nelem,inpelm)

         istep = 2

         npoint = npt

         call mshd06 ( kmeshc, nelem, npoint, nbndpt, 4, iarray(i1), &
                       iarray(i2), iarray(i3), iarray(i4), l3, l4, &
                       istep )

         if ( npoint>=npti ) then

              call mshd08( coor, npoint, npti, iarray(i1), &
                           iarray(i3), rim )

         else

              npt    = npti - 1
              npoint = npt

         end if

         call mshv8 ( coor, kmeshc, nelem, npoint, npt, npti, nbndpt, &
                      iarray(i1), iarray(i2), iarray(i3), iarray(i4) )

         if ( inpelm==9 ) call mshv9 ( coor, kmeshc, nelem, npt )

         npoint = npt

      else

!     --- Subroutine MSHREP only for inpelm  3,4,6,7,8 or 9

         call errint ( inpelm, 1 )
         call errsub ( 358, 1, 0, 0 )
         go to 1000

      end if

      deallocate ( iarray, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iarray' )

1000  call erclos ( 'mshrep' )

      end
