      subroutine mshrfl ( bcorol, bcornw, nbndpt, ndim, acoor, npoint, &
                          isurf )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.2  date 29-05-2014 Extension of timebefore
!        version  1.1  date 12-08-1999 Replace askclk by printtime
!        version  1.0  date 23-03-1994
!
!   copyright (c) 1994-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute reflected surface in 2d or 3d
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     reflection
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nbndpt, ndim, npoint, isurf
      double precision bcorol(ndim,*), bcornw(ndim,*), &
                       acoor(ndim,npoint)

!     acoor  i/o  at input acoor contains the coordinates of the surface
!                 to be copied
!                 at output acoor contains the coordinates of the new surface
!     bcornw  i   array containing the coordinates of the new boundary
!     bcorol  i   array containing the co-ordinates of boundary of surface
!                 to be copied
!     isurf   i   surface number
!     nbndpt  i   number of points in bcornw
!     ndim    i   dimension of space
!     npoint  i   number of points in acoor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ip, istart
      double precision xp(3), xn(3), xm(3), dx(3), sum1, sum2, e1, &
                       d(3), dxh(3), xmh(3)
      double precision :: timebefore(2)

!     d              help variable for distance
!     dx             direction vector
!     dxh            controle direction vector
!     e1             help variable to store the local magnitude
!     i              loop variable
!     ip             loop variable
!     istart         start position
!     sum1           summation parameter
!     sum2           summation parameter
!     xm             coordinates in reflection line or plane
!     xmh            help-coordinates in line or plane
!     xn             new (reflected) point
!     xp             old (to be reflected) point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Places integer value in error message array
!     ERRSUB         Produces error message
!     ERRWAR         Produces warning
!     ERSETTIME      Get the CPU time at start of subroutine
!     INSTOP         Stops program execution and closes all files
!     MSHSP2         Compute reflection of point submitted
!     PRINTTIME      Print the CPU time at end of subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     961    ndim has wrong value
!     962    new boundary is not reflection of old boundary
!     963    all points of boundary are on reflection line or in
!            reflection plane
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Do some checks on input
!     Check whether mutual distances in old and new boundary
!     are the same
!     Determine reflection line (ndim=2) or plane(ndim=3)
!     (Perpendicular to line old-new !)
!     First check whether all "old" and "new" boundary points are
!     reflections of each other
!     Check for all other boundary points that they have the
!     same reflection line
!     Reflect all co-ordinates of acoor
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrfl' )
      if ( itime>0 ) call ersettime ( timebefore )

      if ( ndim/=2 .and. ndim/=3 ) then
           call errint ( ndim, 1 )
           call errsub ( 961, 1, 0, 0 )
           call instop
      end if

!     --- Check whether mutual distances in bcorol and bcornw are the same

      do 100 i = 2, nbndpt

          sum1 = 0d0

          do 102 ip = 1 , ndim
              d(ip) = bcorol(ip,i) - bcorol(ip,1)
              sum1  = sum1 + d(ip) * d(ip)
102       continue

          sum1 = sqrt( sum1 )

          sum2 = 0d0

          do 104 ip = 1 , ndim
              d(ip) = bcornw(ip,i) - bcornw(ip,1)
              sum2  = sum2 + d(ip) * d(ip)
104       continue

          sum2 = sqrt( sum2 )

          e1   = sqreps * ( abs(sum1) + abs(sum2) )

          if ( e1<epsmac ) e1 = sqreps

          if ( abs(sum1-sum2)>e1 ) then

!         --- Give warning

               call errint( isurf, 1 )
               call errint(  i   , 2 )
               call errwar(  962 , 2 , 0 , 0 )

          end if

100   continue

!     --- Compute coordinates of reflected points, &
!         determine reflection line (ndim=2) or plane(ndim=3)
!         (Perpendicular to line old-new !)

      istart = 0

130   istart = istart + 1

      if ( istart==nbndpt ) then

!     --- error? all points on reflection line, give warning

         call errint( isurf, 1 )
         call errwar(  963 , 1 , 0 , 0 )

!        --- jump to end : all coordinates remain the same

         goto 500

      end if

      sum1 = 0d0

      do 132 i = 1, ndim

          dx(i) = bcorol(i,istart) - bcornw(i,istart)
          sum1  = sum1 + dx(i) * dx(i)

132   continue

      sum1 = sqrt( sum1 )

!     --- Check whether both points are on reflection line or in reflection
!         plane

      if ( sum1<100 * epsmac ) goto 130

!     --- Normalize coefficient of direction

      do 134 i = 1 , ndim
          dx(i) = dx(i) / sum1
134   continue

!     --- Determine point on reflection line or in reflection plane

      do 136 i = 1 , ndim
          xm(i) =  (bcorol(i,1) + bcornw(i,1)) / 2d0
136   continue

!     --- Check for all other boundary points that they have the
!         same reflection line

      do 160 i = istart+1, nbndpt-1

          sum1 = 0d0

          do 138 ip = 1 , ndim
              dxh(ip) =  bcorol(ip,i) - bcornw(ip,i)
              sum1    =  sum1 + dxh(ip) * dxh(ip)
138       continue

          sum1 = sqrt( sum1 )

          if ( sum1>epsmac ) then

!         --- Normalize

               sum2 = 0d0

               do 140 ip = 1 , ndim
                   dxh(ip)  =  dxh(ip) / sum1
                   d(ip)    =  dxh(ip) - dx(ip)
                   sum2     =  sum2 + d(ip) * d(ip)
140            continue

               sum2 = sqrt( sum2 )

               if ( sum2>100 * epsmac ) then

!              --- There should be no difference in the direction

                  call errint( isurf, 1 )
                  call errint(  i   , 2 )
                  call errwar(  962 , 2 , 0 , 0 )

               end if

!              --- Compute point on line or in plane

               do 142 ip = 1 , ndim
                   xmh(ip) =  (bcorol(ip,i) + bcornw(ip,i)) / 2d0
142            continue

!              --- Line xm - xmh should be perpendicular to connection line

               sum2 = 0d0

               do 144 ip = 1 , ndim
                   sum2 = sum2 + ( xmh(ip) - xm(ip) ) * dx(ip)
144            continue

               if ( abs(sum2)>100 * epsmac ) then

                  call errint( isurf, 1 )
                  call errint(  i   , 2 )
                  call errwar(  962 , 2 , 0 , 0 )

               end if

          end if

160   continue

!     --- Reflect all co-ordinates of acoor

      do 400 i = 1, npoint

         do 310 ip = 1 , ndim
             xp(ip) = acoor(ip,i)
310      continue

         call mshsp2( ndim, xp, xm, dx, xn )

         do 320 ip = 1 , ndim
             acoor(ip,i) = xn(ip)
320      continue

400   continue

500   continue

      if ( itime==1 ) call printtime('REFLECT')

      call erclos ( 'mshrfl' )

      end
