      subroutine mshsmh ( nx, ny, coor, eps )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.2  date 07-10-2014 New call to prcgst
!        version  3.1  date 11-12-2012 Long integers
!        version  3.0  date 15-09-2010 Remove work arrays
!
!   copyright (c) 1994-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Smooth a mesh created by rectangle
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     smoothing
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny
      double precision coor(2,nx+1,ny+1), eps

!     coor   i/o   array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     eps     i    Accuracy of smoothing process
!     nx      i    number of points in x direction
!     ny      i    number of points in y direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, iter, j, maxitr
      integer, target :: infmat(9)
      double precision alpha, beta, dp, dq, eps1, eps2, gamma, xeta, &
                       xetaet, xksi, xksiks, yeta, yetaet, yksi, &
                       yksiks, diffx, diffy, axmax, aymax, dummy(1), &
                       matrix(nx-1,ny-1,9), &
                       rhside(nx-1,ny-1,2), x(nx-1,ny-1,2), &
                       p(nx-1,ny-1), q(nx-1,ny-1)
      type (rmatrix) :: A

!     A       Matrix A
!     alpha   Parameter alpha in the pde
!     axmax   Maximal value of x-coordinate
!     aymax   Maximal value of y-coordinate
!     beta    Parameter beta in the pde
!     diffx   Maximal difference in x-direction
!     diffy   Maximal difference in y-direction
!     dp      Step size for p vector
!     dq      Step size for q vector
!     dummy   dummy array
!     eps1    Help variable for the termination criterion of the CG-process
!             If eps1 = 0, the termination criterion is based upon the
!             relative error
!     eps2    Help variable for the termination criterion of the CG-process
!             If eps2 = 0, the termination criterion is based upon the
!             absolute error
!     gamma   Parameter gamma in the pde
!     i       Counting variable
!     infmat         Contains information about the storage of the matrix
!     iter    Iteration variable
!     j       Counting variable
!     matrix  work array to store the discretization matrix
!     maxitr  Maximum number of iterations
!     p       work array to store the function p in the nodes
!     q       work array to store the funcion q in the nodes
!     rhside  work array to store the right-hand side vector
!     x       Work array containing the coordinates during the iteration
!     xeta    dx/deta
!     xetaet  d2x/deta2
!     xksi    dx/dksi
!     xksiks  d2x/dksi2
!     yeta    dy/deta
!     yetaet  d2y/deta2
!     yksi    dy/dksi
!     yksiks  d2y/dksi2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRWAR         Print warnings
!     SEPBICGSTAB    Solve linear system by Bi-CGstab
!     SEPMAT3DTO1D   Map 3d matrix in 1d matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1629   Smoothing process has been stopped, no improvement possible
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsmh' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsmh'
 !        call ertrace

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepmat3dto1d ( matrix, size(matrix), A )

      maxitr = 25

!     --- Copy coor into arrays x and y

      axmax = 0d0
      aymax = 0d0
      do i = 1, nx-1
         do j = 1, ny-1
            x(i,j,1) = coor(1,i+1,j+1)
            x(i,j,2) = coor(2,i+1,j+1)
            axmax= max ( axmax, abs(x(i,j,1)) )
            aymax= max ( aymax, abs(x(i,j,2)) )
         end do
      end do

!     --- Fill arrays p and q for complete inner field
!         First p for lower boundary

      do i = 1, nx-1
         xksi   = 0.5d0*(coor(1,i+2,1) - coor(1,i,1))
         xksiks = coor(1,i+2,1) -2d0*coor(1,i+1,1) + coor(1,i,1)
         yksi   = 0.5d0*(coor(2,i+2,1) - coor(2,i,1))
         yksiks = coor(2,i+2,1) -2d0*coor(2,i+1,1) + coor(2,i,1)
         p(i,1) = (xksi*xksiks+yksi*yksiks)/(xksi**2+yksi**2)
      end do

!     --- Next p for upper boundary

      do i = 1, nx-1
         xksi   = 0.5d0*(coor(1,i+2,ny+1) - coor(1,i,ny+1))
         xksiks = coor(1,i+2,ny+1) -2d0*coor(1,i+1,ny+1) + &
                  coor(1,i,ny+1)
         yksi   = 0.5d0*(coor(2,i+2,ny+1) - coor(2,i,ny+1))
         yksiks = coor(2,i+2,ny+1) -2d0*coor(2,i+1,ny+1) + &
                  coor(2,i,ny+1)
         p(i,ny-1) = (xksi*xksiks+yksi*yksiks)/(xksi**2+yksi**2)
      end do

!     --- Finally p between by linear interpolation

      do i = 1, nx-1
         dp = (p(i,ny-1)-p(i,1))/(ny-2)
         do j = 2, ny-2
            p(i,j) = p(i,j-1) + dp
         end do
      end do

!     --- Next q for left boundary

      do j = 1, ny-1
         xeta   = 0.5d0*(coor(1,1,j+2) - coor(1,1,j))
         xetaet = coor(1,1,j+2) -2d0*coor(1,1,j+1) + coor(1,1,j)
         yeta   = 0.5d0*(coor(2,1,j+2) - coor(2,1,j))
         yetaet = coor(2,1,j+2) -2d0*coor(2,1,j+1) + coor(2,1,j)
         q(1,j) = (xeta*xetaet+yeta*yetaet)/(xeta**2+yeta**2)
      end do

!     --- Finally q for right boundary

      do j = 1, ny-1
         xeta   = 0.5d0*(coor(1,nx+1,j+2) - coor(1,nx+1,j))
         xetaet = coor(1,nx+1,j+2) -2d0*coor(1,nx+1,j+1) + &
                  coor(1,nx+1,j)
         yeta   = 0.5d0*(coor(2,nx+1,j+2) - coor(2,nx+1,j))
         yetaet = coor(2,nx+1,j+2) -2d0*coor(2,nx+1,j+1) + &
                  coor(2,nx+1,j)
         q(nx-1,j) = (xeta*xetaet+yeta*yetaet)/(xeta**2+yeta**2)
      end do

!     --- Finally q between by linear interpolation

      do j = 1, ny-1
         dq = (q(nx-1,j)-q(1,j))/(nx-2)
         do i = 2, nx-2
            q(i,j) = q(i-1,j) + dq
         end do
      end do

      infmat(1) = 0
      infmat(2) = -nx
      infmat(3) = 1-nx
      infmat(4) = 2-nx
      infmat(5) = -1
      infmat(6) = 1
      infmat(7) = -2+nx
      infmat(8) = -1+nx
      infmat(9) = nx

      A%neq = (nx-1)*(ny-1)
      A%rows => infmat
      A%type_prec = 0
      A%scaling = 0
      A%matmultsp = 0
      A%eps1 = 0d0
      A%eps2 = 0.1d0
      A%eps = 0.1d0
      A%nmax = 50
      A%eistat = .false.
      A%precon = .false.
      A%iprint = 0
      A%type_storage = 30
      A%simple = .false.
      A%nproj = 0
      A%startvec = 1
      A%iabserr = 2
      A%irelerr = 1
      A%typesolv = 2

      do iter = 1, maxitr

!     --- Build matrix

         do j = 1, ny-1
            do i = 1, nx-1
               xksi   = 0.5d0*(coor(1,i+2,j+1) - coor(1,i,j+1))
               xksiks = coor(1,i+2,j+1) -2d0*coor(1,i+1,j+1) + &
                        coor(1,i,j+1)
               yksi   = 0.5d0*(coor(2,i+2,j+1) - coor(2,i,j+1))
               yksiks = coor(2,i+2,j+1) -2d0*coor(2,i+1,j+1) + &
                        coor(2,i,j+1)
               xeta   = 0.5d0*(coor(1,i+1,j+2) - coor(1,i+1,j))
               xetaet = coor(1,i+1,j+2) -2d0*coor(1,i+1,j+1) + &
                        coor(1,i+1,j)
               yeta   = 0.5d0*(coor(2,i+1,j+2) - coor(2,i+1,j))
               yetaet = coor(2,i+1,j+2) -2d0*coor(2,i+1,j+1) + &
                        coor(2,i+1,j)
               alpha  = xeta**2+yeta**2
               beta   = 0.5d0*(xksi*xeta+yksi*yeta)
               gamma  = xksi**2+yksi**2
               matrix(i,j,1) = 2d0*(alpha+gamma)
               matrix(i,j,2) = beta
               matrix(i,j,3) = -gamma*(1d0+0.5d0*q(i,j))
               matrix(i,j,4) = -beta
               matrix(i,j,5) = -alpha*(1d0+0.5d0*p(i,j))
               matrix(i,j,6) = -alpha*(1d0-0.5d0*p(i,j))
               matrix(i,j,7) = -beta
               matrix(i,j,8) = -gamma*(1d0-0.5d0*q(i,j))
               matrix(i,j,9) = beta
            end do
         end do

!        --- Fill right-hand side
!            First initialize to zero

         rhside = 0d0

!        --- Next under boundary

         do i = 1, nx-1
            rhside(i,1,1) = rhside(i,1,1) &
                           -matrix(i,1,2)*coor(1,i,1) &
                           -matrix(i,1,3)*coor(1,i+1,1) &
                           -matrix(i,1,4)*coor(1,i+2,1)
            rhside(i,1,2) = rhside(i,1,2) &
                           -matrix(i,1,2)*coor(2,i,1) &
                           -matrix(i,1,3)*coor(2,i+1,1) &
                           -matrix(i,1,4)*coor(2,i+2,1)
            matrix(i,1,2) = 0d0
            matrix(i,1,3) = 0d0
            matrix(i,1,4) = 0d0
         end do

!        --- Next right boundary

         do j = 1, ny-1
            rhside(nx-1,j,1) = rhside(nx-1,j,1) &
                              -matrix(nx-1,j,4)*coor(1,nx+1,j) &
                              -matrix(nx-1,j,6)*coor(1,nx+1,j+1) &
                              -matrix(nx-1,j,9)*coor(1,nx+1,j+2)
            rhside(nx-1,j,2) = rhside(nx-1,j,2) &
                              -matrix(nx-1,j,4)*coor(2,nx+1,j) &
                              -matrix(nx-1,j,6)*coor(2,nx+1,j+1) &
                              -matrix(nx-1,j,9)*coor(2,nx+1,j+2)
            matrix(nx-1,j,4) = 0d0
            matrix(nx-1,j,6) = 0d0
            matrix(nx-1,j,9) = 0d0
         end do

!        --- Next upper boundary

         do i = 1, nx-1
            rhside(i,ny-1,1) = rhside(i,ny-1,1) &
                              -matrix(i,ny-1,7)*coor(1,i,ny+1) &
                              -matrix(i,ny-1,8)*coor(1,i+1,ny+1) &
                              -matrix(i,ny-1,9)*coor(1,i+2,ny+1)
            rhside(i,ny-1,2) = rhside(i,ny-1,2) &
                              -matrix(i,ny-1,7)*coor(2,i,ny+1) &
                              -matrix(i,ny-1,8)*coor(2,i+1,ny+1) &
                              -matrix(i,ny-1,9)*coor(2,i+2,ny+1)
            matrix(i,ny-1,7) = 0d0
            matrix(i,ny-1,8) = 0d0
            matrix(i,ny-1,9) = 0d0
         end do

!        --- Next left boundary

         do j = 1, ny-1
            rhside(1,j,1) = rhside(1,j,1) &
                           -matrix(1,j,2)*coor(1,1,j) &
                           -matrix(1,j,5)*coor(1,1,j+1) &
                           -matrix(1,j,7)*coor(1,1,j+2)
            rhside(1,j,2) = rhside(1,j,2) &
                           -matrix(1,j,2)*coor(2,1,j) &
                           -matrix(1,j,5)*coor(2,1,j+1) &
                           -matrix(1,j,7)*coor(2,1,j+2)
            matrix(1,j,2) = 0d0
            matrix(1,j,5) = 0d0
            matrix(1,j,7) = 0d0
         end do

!        --- Solve system of equations for x and y

         call sepbicgstab ( A, x(1,1,1), rhside(1,1,1) )

         if ( A%numiter>=A%nmax ) then

!        --- No convergence in solution of system of equations

            call errwar ( 1629, 0, 0, 0 )
            go to 1000

         end if
         call sepbicgstab ( A, x(1,1,2), rhside(1,1,2) )
         if ( A%numiter>=A%nmax ) then

!        --- No convergence in solution of system of equations

            call errwar ( 1629, 0, 0, 0 )
            go to 1000

         end if

!        --- Compute difference between two succeeding grids

         diffx = 0d0
         diffy = 0d0
         do i = 1, nx-1
            do j = 1, ny-1
               diffx = max ( diffx, abs(coor(1,i+1,j+1)-x(i,j,1)) )
               diffy = max ( diffy, abs(coor(2,i+1,j+1)-x(i,j,2)) )
            end do
         end do

!        --- Copy solution back into coor

         do i = 1, nx-1
            do j = 1, ny-1
               coor(1,i+1,j+1) = x(i,j,1)
               coor(2,i+1,j+1) = x(i,j,2)
            end do
         end do

         if ( diffx<eps*axmax .and. diffy<eps*aymax ) go to 1000

      end do


1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshsmh'

      end if  ! ( debug )
      call erclos ( 'mshsmh' )

      end
