      subroutine mshz01 ( ielem, nbp, incr, npelm, kMeshE, &
                          npoint, Coor )
! ======================================================================
!
!        programmer    Zdenek Vlasek
!        version  1.2  date 24-02-1991 irefwr instead of * &
!        version  1.1  date 18-06-1990 special case: ielem=0, &
!                                      better error messages
!        version  1.0  date 14-02-1990
!
!   copyright (c) 1990-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Double-faced subroutine:
!    when ielem>=1:
!        To check angles and the orientation of one element
!    when ielem=0:
!        To check the whole boundary and to determine its orientation
!    warning message  - if the angle is out of a reasonable range, &
!    error message    - if the sum of angles is wrong.
!    For each element-boundary point we calculate the inner angle
!    ( on the left ) and we accumulate these angles to recognize a
!    wrong geometry or wrong orientation of the element boundary.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleconsts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ielem, nbp, incr, npelm, kmeshe ( npelm ), npoint
      double precision coor ( 2, npoint )

!     coor     i    Real array (2,mpoint).
!                   Physical coordinates of all meshpoints
!     ielem   i/o   when ielem>=1 on input:
!                                  the element to be checked, &
!                   when ielem=0 on input:
!                                  the whole boundary is to be checked, &
!                        on output ielem=1  ... anticlockwise oriented
!                                  ielem=-1 ... clockwise oriented
!     incr     i    Number of intervals along one element edge
!                   ( ignored when ielem=0 )
!     kmeshe   i    Integer array (npelm).
!                   Global nodenumbers of the element
!     nbp      i    Number of boundary points in the element
!     npelm    i    Number of points per element
!                   ( npelm = nbp  when  ielem=0 )
!     npoint   i    Current number of all meshpoints
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical troubl
      integer ntroub, inode, i1, i3, k1, k2, k3
      double precision alpha, sum, sumerr

!     alpha      An angle in degrees
!     i1         Local node number
!     i3         Local node number
!     inode      Node sequence number
!     k1         Global node number
!     k2         Global node number
!     k3         Global node number
!     ntroub     Number of "trouble" angles found in the element
!     sum        Sum of inner angles, in degrees
!     sumerr     Error in the sum of inner angles
!     troubl     Logical.  Was a trouble angle found ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision mshz00

!     MSHZ00    Real function.  The angle in range [ 0, 2 * pi ), &
!               defined by a triplet of nodenumbers
!     MSHZ04    To display an error message and to stop the run
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     When the sum of inner angles is wrong
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The corner angles are tested for the range  90+-80 degrees, &
!     the other angles ( if any ) for the range  180+-30 degrees.
!     Both ranges may be changed here - thery are voluntary.
!
!     Sum of all inner angles must be   (nbp-2)*180  degrees
!     but for clockwise orientation     (nbp+2)*180  degrees, &
!     ( we accumulate the angles on the left side of the boundary )
!
!     The tolerance is set to 1d-4.
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- initialize sum, ntroub

      sum = 0d0
      ntroub = 0

!     --- visit all element-boundary points

      do 100 inode = 1, nbp

!     --- goal: the inner angle on the left at this point

!           fill i1 by the predecessor of local point i

         if ( inode.eq.1 ) then
            i1 = nbp
         else
            i1 = inode - 1
         end if

         k1 = kMeshE ( i1 )

         k2 = kMeshE ( inode )

!        --- fill i3 by the follower of the local point i

         if ( inode.eq.nbp ) then
            i3 = 1
         else
            i3 = inode + 1
         end if

         k3 = kMeshE ( i3 )

         alpha = mshz00 ( k1, k2, k3, npoint, Coor ) * 180d0 / pi
         sum = sum + alpha

         troubl = .true.

!        --- mes = ' '

         if ( ielem.ge.1 ) then

!        --- an element

            if ( mod ( inode - 1 , incr ).eq.0 ) then

!           --- it is a corner point

               if ( alpha.gt.170d0 ) then

!              --- mes = 'corner > 170'

               else if ( alpha.lt.10d0 ) then

!              --- mes = 'corner <  10'

               else
                  troubl = .false.
               end if

            else

!           --- it is not a corner point

               if ( alpha.gt.210d0 ) then

!              --- mes = 'inner  > 210'

               else if ( alpha.lt.150d0 ) then

!              --- mes = 'inner  < 150'

               else
                  troubl = .false.
               end if

            end if

         end if

         if ( troubl ) then

!        --- the inner angle is out of the range

!              the message is suppressed

!--- write ( irefwr, '( 1x, a, i5, 3x, a, i5, &

!--- &         3x, a, f7.2, 1x, a )' )

!--- &         'mshz01:  ielem=', ielem, 'inode=', inode, &

!--- &         'alpha=', alpha, mes

            ntroub = ntroub + 1

         end if

  100 continue

!     --- check the sum, it is  (nbp-2)*180  ... anticlockwise oriented
!                          or     (nbp+2)*180  ... clockwise oriented

      sumerr = sum - ( nbp - 2 ) * 180d0

      if ( ielem.eq.0 ) then

!     --- check the whole boundary, determine the orientation

         if ( abs(sumerr).le.1d-4 ) then

!        --- anticlockwise oriented

            ielem = 1
         else if ( abs(sumerr-720d0).le.1d-4 ) then

!        --- clockwise oriented

            ielem = -1
         else

!        --- wrong boundary data, issue an error message and stop

            write ( irefwr, '( / 1x, a, f11.4, a / )' ) &
               'mshz01:  error in sum of angles along boundary is ', &
               sumerr, ' deg !!!'
            call mshz04 ( 'mshz01: wrong boundary data' )
         end if

      else

!     --- an usual element

         if ( abs(sumerr).gt.1d-4 ) then

!        --- the sum is wrong, issue an error message and go on

            write ( irefwr, '( / 1x, a, i5, 5x, a, f11.4, a / )' ) &
               'mshz01:  ielem=', ielem, &
               'error in sum of inner angles is ', sumerr, ' deg !!!'
         end if

      end if

!     --- report if some inner angles are out of range

      if  ( ntroub.gt.0 ) then

!     --- the report is suppressed

!--- write ( irefwr, * )

      end if

      end
