      subroutine sepflkmeshad3 ( ichoice )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmesh part ad as well as the subarrays ipointobst and
!     ielemobst (3d only)
!     Old subroutine mshfillkmesha3
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     element
!     node
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ichoice

!     ichoice        i    Choice parameter: defines which part of kmesh part ad
!                         must be filled
!                         Possible values:
!                         0: the complete array
!                         1: part 1 (ipointobst)
!                         2: parts 1,2 (ielemobst)
!                         3: part 1-4 (icrossobst,rcrossobst)
!                         -2: parts 2 only (ielemobst)
!                         -3: part 3,4 only (icrossobst,rcrossobst)
!                         4: part 9 (obstfaces)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iobst, i, lenipointobst, &
              lenicrossobst, lenrcrossobst, &
              lenielemobst, iseqad, lenin, lenrl, nelemobst, &
              leniobgrid, lenobstfaces, ipipoin
      integer, allocatable :: iobgrid(:,:,:,:)
      integer, allocatable :: ielemobstcp(:)

!     i              Counting variable
!     ielemobstcp    Copy of ielemobst extended with extra elements
!     iobgrid        position i,j contains:
!                    -1:  cell inside obstacle
!                     0:  cell outside obstacle
!                    >0:  cell contains part of the obstacle
!                    The value contains a starting address in iobgrid1
!     iobst          Obstacle sequence number
!     ipipoin        Starting address in array ipointobst
!     iseqad           Sequence number
!     lenicrossobst  Computed length of array icrossobst
!     lenielemobst   Computed length of array ielemobst
!     lenin          Number of integers in one row of kmeshx
!     leniobgrid     Declared length of last dimension of iobgrid
!     lenipointobst  length of array ipointobst
!     lenobstfaces   Length of array obstfaces
!     lenrcrossobst  Computed length of array rcrossobst
!     lenrl          Number of reals in one row of rmeshx
!     nelemobst      Number of elements in obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     MSHOBSFL3D     Fill array iobgrid with grid information of one obstacle
!                    (R^3 only)
!     MSHOBSTFACES3D Find all faces corresponding to one obstacle and store
!                    in array obstfaces
!     SEPFILLELOBS   Fill array elemobst for each element with information of
!                    elements in relation to the obstacle
!     SEPFILLKMESHAD Fill array kmesh part d
!     SEPFILLKMESHD  Fill kmesh part d
!     SEPOB3DCROSSEL Mark all elements in the mesh with information if they
!                    are within an obstacle or outside an obstacle
!                    3d version
!     SEPOB3DMARKEL1 Find all elements that intersect the obstacle but do not
!                    have a node inside the obstacle
!                    3d version
!     SEPOB3DMARKPT  Mark all nodal points in the mesh with information if they
!                    are within an obstacle or outside an obstacle
!                    3d version
!     SEPOBSTMARKBT1 Fill ipointobst part 4, i.e. the set of boundary nodes of
!                    an obstacle that are only inside obstacle elements
!     SEPOBSTMARKEL  Mark all elements in the mesh with information if they
!                    are within an obstacle or outside an obstacle
!     SEPPLOTINTERS  plot and print intersection points
!     SEPPLOTOBST    Print and plot information about the obstacle
!     SEPPRINIELOBST Print the contents of array ielemobst
!     SEPPRINIPOBST  Print the contents of array ipointobst
!     SEPPRINKMESHC  print kmesh part c
!     SEPPRINKMESHD  print kmesh part d
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2206   KMESH part x has not been filled
!    2207   There are no obstacles
!    2212   Previous part of kmesh ad has not been filled
!    2213   nmeshc has not been filled
!    2214   kmesh aa has not been filled
!    2288   nmeshh has not been filled
!    2292   More than one obstacle
!    2406   Array has not been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
!     numobst := kmesh(42)
!     for iobst := 1 (1) numobst do
!        Mark all nodes of mesh in obstacle
!        Mark all elements to be in/out or partly in obstacle
!        Mark all nodes that are in obstacle and only in elements that are
!        completely in obstacle
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepflkmeshad3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepflkmeshad3'
         write(irefwr,1) 'ichoice', ichoice
  1      format ( a, 1x, (10i6) )
         call sepprinkmeshc
         call sepprinkmeshd

      end if
      if ( ierror/=0 ) go to 1000

!     --- initializations

      lenrl = 9
      lenin = 12
      if ( numobst==0 ) then

!     --- No obstacles

         call errwar ( 2207, 0, 0, 0 )
         go to 1000

      end if

      if ( debug ) &
         write(irefwr,1) 'numobst, npoint, nelem',numobst, npoint, nelem

      if ( ichoice>=0 ) then

!     --- Initialize kmesh part ad

         call  sepfillkmeshad ( iseqad )
         if ( kelmd==0 ) call sepfillkmeshd
         if ( debug ) write(irefwr,*) 'after initkmeshad'

!        --- Reserve temporary space for array ipointobst

         lenipointobst = 5*numobst+npoint

         if ( associated(ipointobst) ) nullify(ipointobst)
         allocate ( ipointobst(lenipointobst), stat = error )
         if ( error/=0 ) call eralloc ( error, lenipointobst, 'ipointobst' )
         if ( ierror/=0 ) go to 1000

!        --- Set starting address for first obstacle

         ipipoin = numobst+1

!        --- for iobst := 1 (1) numobst do

         do iobst = 1, numobst

!        --- Mark all nodes of mesh in obstacle

            ipointobst(iobst) = ipipoin
            if ( kelmx==0 ) then

!           --- error, kmesh part x has not been filled

               call errsub ( 2206, 0, 0, 0 )
               go to 1000

            end if
            nx = kmeshx(3,iobst)
            ny = kmeshx(4,iobst)
            nz = kmeshx(5,iobst)
            kmobst => km(iseqkm)%kmobst
            kmeshcobst => kmobst(iobst)%kmeshcobst
            coorobst => kmobst(iobst)%coorobst
            if ( ierror/=0 ) go to 1000
            if ( kmeshx(6,iobst)==0 ) then

!           --- Array has not yet been filled

               inpelm = kmeshx(10,iobst)
               nelemobst = kmeshx(11,iobst)

!              --- Create temporary space to store iobgrid;
!                  The actual length is computed later on
!                  The estimated length is 50 nx ny nz

               leniobgrid = 50
               allocate ( iobgrid(nx,ny,nz,leniobgrid), stat = error )
               if ( error/=0 ) &
                  call eralloc ( error, nx*ny*nz*leniobgrid, 'iobgrid' )
               if ( ierror/=0 ) go to 1000

               if ( debug ) write(irefwr,*) 'before mshobsfl3d'
               call mshobsfl3d ( rmeshx, iobgrid, &
                                 kmeshcobst, coorobst, &
                                 nx, ny, nz, iobst, inpelm, nelemobst, &
                                 leniobgrid )

               allocate ( km(iseqkm)%kmobst(iobst)% &
                          iobgrid3d(0:nx-1,0:ny-1,0:nz-1,leniobgrid) )
               iobgrid3d => km(iseqkm)%kmobst(iobst)%iobgrid3d
               iobgrid3d = iobgrid(:,:,:,1:leniobgrid)

               deallocate ( iobgrid, stat = error )
               if ( error/=0 ) call erdealloc ( error, 'iobgrid' )
               if ( ierror/=0 ) go to 1000

               kmeshx(6,iobst) = iseqkm
               kmeshx(7,iobst) = leniobgrid

            end if
            leniobgrid = kmeshx(7,iobst)
            inpelm = kmeshx(10,iobst)

            ipointobstpart => ipointobst(ipipoin:size(ipointobst))
            if ( debug ) write(irefwr,*) 'before sepob3dmarkpt'
            iobgrid3d => kmobst(iobst)%iobgrid3d
            kmeshcobst => kmobst(iobst)%kmeshcobst
            coorobst => kmobst(iobst)%coorobst
            call sepob3dmarkpt ( iobst, nx, ny, nz, inpelm, leniobgrid )

!           --- Fill part 4 of ipointsobst of nodes on the boundary of the
!               obstacle which are only in elements completely inside
!               the obstacle

            if ( debug ) write(irefwr,*) 'before sepobstmarkbt1'
            call sepobstmarkbt1 ( ipointobst(ipipoin) )

!           --- raise ipipointobst

            ipipoin = ipipoin+4+ipointobst(ipipoin)+ipointobst(ipipoin+2)+ &
                      ipointobst(ipipoin+3)

         end do

         if ( debug ) then
            write(irefwr,*) 'ipointobst after mshobstmarkbt'
            call sepprinipobst ( ipointobst )
         end if

!        --- Check array length

         call inichkarlen ( ipipoin, lenipointobst, 'ipointobst' )
         if ( ierror/=0 ) go to 1000

!        --- Make array ipointobst permanent

         lenipointobst = ipipoin-1
         if ( associated(km(iseqkm)%ipointobst) ) &
            deallocate(km(iseqkm)%ipointobst)
         allocate(km(iseqkm)%ipointobst(lenipointobst))
         kmeshad(1) = lenipointobst
         kmeshad(6) = iseqad

!        --- Copy contents of temporary array into permanent array

         km(iseqkm)%ipointobst = ipointobst(1:lenipointobst)

         if ( debug ) write(irefwr,*)'before deallocate'

         deallocate ( ipointobst, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'ipointobst' )
         if ( ierror/=0 ) go to 1000

         ipointobst => km(iseqkm)%ipointobst

      else

         iseqad = kmeshad(5)

      end if

      if ( debug ) write(irefwr,1) 'iseqad', iseqad

!     --- Mark all elements to be in/out or partly in obstacle

      if ( ichoice==0 .or. ichoice>=2 .or. ichoice==-2 ) then

!     --- ichois = 0 or 2, fill array ielemobst
!         Mark all elements to be in/out or partly in obstacle

         if ( kelmad==0 ) then

!        --- kmesh part ad has not been initialized

            call errint ( 2, 1)
            call errint ( 1, 2)
            call errsub ( 2212, 2, 0, 0 )
            go to 1000

         else if ( kmeshad(1)==0 ) then

!        --- kmesh part ad -1 has not been filled

            call errint ( 2, 1)
            call errint ( 1, 2)
            call errsub ( 2212, 2, 0, 0 )
            go to 1000

         end if
         if ( kelmaa==0 ) then

!        --- error, kmesh part aa has not been filled

            call errsub ( 2214, 0, 0, 0 )
            go to 1000

         end if
         if ( nmesh(33)==0 ) then

!        --- error, nmesh part c has not been filled

            call errsub ( 2213, 0, 0, 0 )
            go to 1000

         end if
         if ( nmesh(38)==0 ) then

!        --- error, nmesh part h has not been filled

            call errsub ( 2288, 0, 0, 0 )
            go to 1000

         end if

!        --- Reserve temporary space for arrays ielemobst and ielemobstcp

         length = 6*numobst+nelem
         if ( associated(ielemobst) ) nullify(ielemobst)
         allocate ( ielemobst(length), ielemobstcp(length), stat = error )
         if ( error/=0 ) call eralloc ( error, length, 'ielemobst' )
         if ( ierror/=0 ) go to 1000

!        --- First the elements are created from the nodes only

         call sepobstmarkel ( lenielemobst )

         if ( debug ) then
            write(irefwr,*) 'ipointobst after mshobstmarkel'
            call sepprinipobst ( ipointobst )
            call sepprinielobst ( ielemobst )
         end if

!        --- Next it is checked if there are extra elements that have no node
!            in common with the obstacle but still intersect the obstacle

         lenielemobst = numobst+1
         do iobst = 1, numobst
            leniobgrid = kmeshx(7,iobst)
            nx = kmeshx(3,iobst)
            ny = kmeshx(4,iobst)
            nz = kmeshx(5,iobst)
            inpelm = kmeshx(10,iobst)
            iobgrid3d => kmobst(iobst)%iobgrid3d
            kmeshcobst => kmobst(iobst)%kmeshcobst
            coorobst => kmobst(iobst)%coorobst
            if ( debug ) write(irefwr,*) 'before sepob3dmarkel1'
            call sepob3dmarkel1 ( lenielemobst, ielemobstcp, &
                                  iobst, leniobgrid )
            if ( debug ) write(irefwr,*) 'after sepob3dmarkel1'

         end do
         lenielemobst = lenielemobst-1

!        --- Make array ielemobst permanent

         if ( associated(km(iseqkm)%ielemobst) ) &
            deallocate(km(iseqkm)%ielemobst)
         allocate(km(iseqkm)%ielemobst(lenielemobst))
         kmeshad(2) = lenielemobst
         kmeshad(7) = iseqad

!        --- Copy contents of temporary array into permanent array

         km(iseqkm)%ielemobst = ielemobstcp(1:lenielemobst)

         deallocate ( ielemobst, ielemobstcp, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'ielemobst' )
         if ( ierror/=0 ) go to 1000
         ielemobst => km(iseqkm)%ielemobst

      end if

      if ( debug ) write(irefwr,*) 'after ielemobst'

      if ( ichoice>=-2 .and. (debug .or. plotobst .or. printobst) ) then

!     --- debug = true, plot nodes and elements with a colour

         if ( debug ) write(irefwr,*) 'before sepplotobst'
         call sepplotobst ( debug, ichoice )
         if ( debug ) write(irefwr,*) 'after sepplotobst'

      end if

      if ( ichoice==0 .or. ichoice>=3 .or. ichoice==-3 ) then

!     --- ichois = 0 or 3, fill array icrossobst,rcrossobst

         if ( kelmad==0 ) then

!        --- kmesh part ad has not been initialized

            call errint ( 3, 1)
            call errint ( 2, 2)
            call errsub ( 2212, 2, 0, 0 )
            go to 1000

         else if ( kmeshad(2)==0 ) then

!        --- kmesh part ad -2 has not been filled

            call errint ( 3, 1)
            call errint ( 2, 2)
            call errsub ( 2212, 2, 0, 0 )
            go to 1000

         end if
         if ( kelmx==0 ) then

!        --- error, kmesh part x has not been filled

            call errsub ( 2206, 0, 0, 0 )
            go to 1000

         end if

         if ( kelmaa==0 ) then

!        --- error, kmesh part aa has not been filled

            call errsub ( 2214, 0, 0, 0 )
            go to 1000

         end if
         if ( nmesh(33)==0 ) then

!        --- error, nmesh part c has not been filled

            call errsub ( 2213, 0, 0, 0 )
            go to 1000

         end if

!        --- define work space to store rcrossobst, icrossobst and iedges
!            Length:
!            rcrossobst  at most nedges reals
!            icrossobst  at most 3*nedges+2*numobst
!            iedges      nedges

         if ( associated(icrossobst) ) nullify(icrossobst, rcrossobst)
         allocate ( rcrossobst(nedges), &
                    icrossobst(2*numobst+3*nedges+4*nelem), stat = error )
         if ( error/=0 ) call eralloc ( error, nedges, 'rcrossobst' )
         if ( ierror/=0 ) go to 1000

!        --- compute starting addresses

         lenicrossobst = 0
         lenrcrossobst = 0
         do iobst = 1, numobst
            nx = kmeshx(3,iobst)
            ny = kmeshx(4,iobst)
            nz = kmeshx(5,iobst)
            inpelm = kmeshx(10,iobst)
            leniobgrid = kmeshx(7,iobst)
            iobgrid3d => kmobst(iobst)%iobgrid3d
            kmeshcobst => kmobst(iobst)%kmeshcobst
            coorobst => kmobst(iobst)%coorobst
            call sepob3dcrossel ( lenicrossobst, lenrcrossobst, iobst, &
                                  leniobgrid )

         end do

!        --- Make the arrays rcrossobst, icrossobst permanent

         if ( associated(km(iseqkm)%icrossobst) ) &
            deallocate(km(iseqkm)%icrossobst)
         if ( associated(km(iseqkm)%rcrossobst) ) &
            deallocate(km(iseqkm)%rcrossobst)
         allocate(km(iseqkm)%icrossobst(lenicrossobst))
         km(iseqkm)%icrossobst = icrossobst(1:lenicrossobst)

         if ( lenrcrossobst>0 ) then

!        --- Array may be created

            allocate(km(iseqkm)%rcrossobst(lenrcrossobst))
            kmeshad(4) = lenrcrossobst
            km(iseqkm)%rcrossobst = rcrossobst(1:lenrcrossobst)

         else

!        --- No intersections found, delete previous array if it exists

            kmeshad(4) = 0

         end if
         kmeshad(3) = lenicrossobst
         kmeshad(8) = iseqad

         deallocate ( rcrossobst, icrossobst, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'rcrossobst' )
         if ( ierror/=0 ) go to 1000

         icrossobst => km(iseqkm)%icrossobst
         if ( lenrcrossobst>0 ) rcrossobst => km(iseqkm)%rcrossobst

         if ( lenrcrossobst>0 .and. &
              (debug .or. plotobst .or. printobst) ) then

!        --- debug = true, plot and print intersection points

            call sepplotinters

         end if  ! ( lenrcrossobst>0 .and. ...

      end if

      if ( ichoice==5 ) then

!     --- ichoice = 5, fill array obstfaces
!         Perform standard checks

         if ( numobst==0 ) then

!        --- numobst = 0, filling makes no sense

            call errwar ( 2207, 0, 0, 0 )
            go to 1000

         end if
         if ( numobst>1 ) then

!        --- numobst>1, Not yet implemented

            call errint ( numobst, 1 )
            call errsub ( 2292, 1, 0, 0 )

         end if

!        --- Activate arrays and if necessary create some

         if ( nmesh(36)==0 ) then

!        --- Error nmeshf has not been filled

            call errchr ( 'nmeshf', 1 )
            call errsub ( 2406, 0, 0, 1 )
            go to 1000

         end if
         if ( kmeshad(2)==0 ) then

!        --- Error ielemobst has not been filled

            call errchr ( 'ielemobst', 1 )
            call errsub ( 2406, 0, 0, 1 )
            go to 1000

         end if

!        --- Create temporary arrays

         lenobstfaces = nfaces

         if ( associated(obstfaces) ) nullify(obstfaces)
         allocate ( obstfaces(lenobstfaces), elemobst(nelem), stat = error )
         if ( error/=0 ) &
            call eralloc ( error, lenobstfaces+nelem, 'obstfaces' )
         if ( ierror/=0 ) go to 1000

!        --- Compute starting addresses

         if ( ierror/=0 ) go to 1000

!        --- Fill array elemobst

         call sepfillelobs

!        --- Actual creation of array obstfaces

         length = lenobstfaces-1
         obstfaces(1) = 2
         call mshobstfaces3d ( nfaces, obstfaces(2), &
                               elemobst, nmeshf, length )
         lenobstfaces = length+1

!        --- Make array obstfaces permanent

         if ( associated(km(iseqkm)%obstfaces) ) &
            deallocate(km(iseqkm)%obstfaces)
         allocate( km(iseqkm)%obstfaces(lenobstfaces) )
         km(iseqkm)%obstfaces = obstfaces(1:lenobstfaces)

         deallocate ( obstfaces, elemobst, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'obstfaces' )
         if ( ierror/=0 ) go to 1000

         obstfaces => km(iseqkm)%obstfaces

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepflkmeshad3'

      end if
      call erclos ( 'sepflkmeshad3' )

      end subroutine sepflkmeshad3
