      subroutine sepob3dmarkel1 ( lenielemobst, ielemobstcp, iobst, &
                                  leniobgrid )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all elements that intersect the obstacle but do not have a node
!     inside the obstacle
!     Copy ielemobst into ielemobstcp and extend ielemobstcp with the extra
!     Old subroutine mshob3dmarkel1
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     element
!     obstacle
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ielemobstcp(*), lenielemobst, iobst, leniobgrid

!     ielemobstcp    o    Copy of ielemobst extended with extra elements
!     iobst          i    Obstacle sequence number
!     lenielemobst  i/o   Computed length of array ielemobst
!     leniobgrid     i    Declared length of last dimension of iobgrid3d
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, lstore, nextra, numelmobst, &
              istartelemcp, istartelem, numinelmobst, numbnelmobst

!     i              Counting variable
!     istartelem     Starting address in array ielemobst
!     istartelemcp   Starting address in array ielemobstcp
!     istore         Work array of length nedges
!                    In this array all edges are stored that are candidates
!                    for a cross section with the obstacle, although their
!                    end points are not in the obstacle
!     iwork          integer work array of length nelem
!                    if ( iwork(ielem) = 0 element ielem may be outside
!                    the obstacle
!                    if ( iwork(ielem) = 1 element ielem is inside
!                    the obstacle
!     lstore         Counter with respect to array istore
!     nextra         number of extra elements
!     numbnelmobst   Number of elements that are partly within the obstacle
!     numelmobst     numbnelmobst+numinelmobst
!     numinelmobst   Number of elements that are completely within the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSTMARKEL3 Store all elements that intersect the obstacle but do not
!                    have a node inside the obstacle into ielemobstcp
!     SEPOB3DMARKEL2 Find all elements that intersect the obstacle but do not
!                    have a node inside the obstacle
!                    3d case
!     SEPOB3DMARKEL4 Check if there are any elements with all vertices in the
!                    obstacle, that still crosses the obstacle
!                    3d case
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Store all edges that correspond to elements that are in the obstacle
!     and that also correspond to at least one element outside the obstacle
!     in array istore
!     Remove all edges in istore that do not intersect the obstacle
!     Store all elements corresponding to edges in istore and not already in
!     ielemobst into ielemobstcp
!     Raise lenielemobst
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepob3dmarkel1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepob3dmarkel1'
         write(irefwr,1) 'iobst, nedges, npoint', iobst, nedges, npoint
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inelem = nelem
      allocate ( iwork(nelem), istore(max(nedges,npoint)), stat = error )
      if ( error/=0 ) call eralloc ( error, nelem+max(nedges,npoint), 'iwork' )
      if ( ierror/=0 ) go to 1000


      istartelem = ielemobst(iobst)
      istartelemcp = lenielemobst
      if ( debug ) &
         write(irefwr,*) 'istartelem, istartelemcp, lenielemobst ', &
                          istartelem, istartelemcp, lenielemobst

!     --- Check if there are any elements with all vertices in the obstacle, &
!         that still crosses the obstacle
!         Copy adapted array ielemobst in ielemobstcp

      call sepob3dmarkel4 ( iobst, ielemobst(istartelem), &
                            ielemobstcp(istartelemcp), &
                            nx, ny, nz, inpelm, leniobgrid )
      if ( ierror/=0 ) go to 1000
      numinelmobst = ielemobstcp(istartelemcp)+ &
                     ielemobstcp(istartelemcp+1)
      numbnelmobst = ielemobstcp(istartelemcp+2)

!     --- Clear array iwork

      iwork(1:inelem) = 0

!     --- Mark elements in the obstacle by a 1

      do i = 1, numinelmobst+numbnelmobst
         iwork(ielemobst(istartelem+2+i)) = 1
      end do

!     --- Store all edges that correspond to elements that are in the
!         obstacle and that also correspond to at least one element
!         outside the obstacle in array istore

      numelmobst = numinelmobst+numbnelmobst
      call sepob3dmarkel2 ( numelmobst, ielemobst(istartelem+3), &
                            lstore, iobst, nx, ny, nz, &
                            inpelm, leniobgrid )
      if ( ierror/=0 ) go to 1000

!     --- Store all elements corresponding to edges in istore and not already in
!         ielemobst into ielemobst
!         Raise lenielemobst

      ielemobstcp(iobst) = istartelemcp
      lenielemobst = istartelemcp+2+numbnelmobst+numinelmobst

      call mshobstmarkel3 ( nextra, lstore, istore, nmeshh, iwork, &
                            lenielemobst, ielemobstcp )
      numbnelmobst = numbnelmobst+nextra
      if ( debug ) write(irefwr,*) 'after mshobstmarkel3'

!     --- Repeat the process for all new elements untill no elements are added

300   if ( nextra>0 ) then

!     --- There are extra elements

         if ( debug ) write(irefwr,1) 'nextra', nextra
         call sepob3dmarkel2 ( nextra, &
                               ielemobstcp(lenielemobst-nextra+1), &
                               lstore, iobst, nx, ny, nz, &
                               inpelm, leniobgrid )

!        --- store extra elements in ielemobstcp

         call mshobstmarkel3 ( nextra, lstore, istore, nmeshh, iwork, &
                               lenielemobst, ielemobstcp )
         if ( debug ) write(irefwr,*) 'after mshobstmarkel3/2'
         numbnelmobst = numbnelmobst+nextra
         go to 300

      end if

      deallocate ( iwork, istore, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

      ielemobstcp(istartelemcp+2) = numbnelmobst
      lenielemobst = lenielemobst+1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'lenielemobst, numbnelmobst, nextra ', &
                          lenielemobst, numbnelmobst, nextra
         write(irefwr,*) 'End sepob3dmarkel1'

      end if
      call erclos ( 'sepob3dmarkel1' )

      end
