      subroutine sepobstmarkel1 ( iwork, lenielemobst, istore, &
                                  nx, ny, ielemobstcp, iobst, lencurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all elements that intersect the obstacle but do not have a node
!     inside the obstacle
!     Copy ielemobst into ielemobstcp and extend ielemobstcp with the extra
!     elements
!     Old subroutine mshobstmarkel1
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     element
!     obstacle
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iwork(*), ielemobstcp(*), lenielemobst, istore(*), nx, ny, &
                 iobst, lencurvs

!     ielemobstcp    o    Copy of ielemobst extended with extra elements
!     iobst          i    Obstacle sequence number
!     istore              Work array of length nedges
!                         In this array all edges are stored that are candidates
!                         for a cross section with the obstacle, although their
!                         end points are not in the obstacle
!     iwork               integer work array of length nelem
!                         if ( iwork(ielem) = 0 element ielem may be outside
!                         the obstacle
!                         if ( iwork(ielem) = 1 element ielem is inside
!                         the obstacle
!     lencurvs       i    Overestimate of the number of points along the new
!                         curves
!     lenielemobst  i/o   Computed length of array ielemobst
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, lstore, nextra, nnodes, &
              istartelemcp, istartelem, numinelmobst, numbnelmobst, &
              icurnrobs, numelmobst
      double precision curvescp(2,lencurvs)

!     curvescp       Array to store all coordinates of one curve
!     i              Counting variable
!     icurnrobs      Curve sequence number of obstacle
!     istartelem     Starting address in array ielemobst
!     istartelemcp   Starting address in array ielemobstcp
!     lstore         Counter with respect to array istore
!     nextra         number of extra elements
!     nnodes         Number of nodes
!     numbnelmobst   Number of elements that are partly within the obstacle
!     numelmobst     numbnelmobst+numinelmobst
!     numinelmobst   Number of elements that are completely within the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSTMARKEL2 Find all elements that intersect the obstacle but do not
!                    have a node inside the obstacle
!     MSHOBSTMARKEL3 Store all elements that intersect the obstacle but do not
!                    have a node inside the obstacle into ielemobstcp
!     PRININ         print 1d integer array
!     SEPCOPCURVCOOR Copy coordinates of curve icurnr from array curves into
!                    array curvescp
!     SEPOBSTMARKEL4 Check if there are any elements with all vertices in the
!                    obstacle, that still crosses the obstacle
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Store all edges that correspond to elements that are in the obstacle
!     and that also correspond to at least one element outside the obstacle
!     in array istore
!     Remove all edges in istore that do not intersect the obstacle
!     Store all elements corresponding to edges in istore and not already in
!     ielemobst into ielemobstcp
!     Raise lenielemobst
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepobstmarkel1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepobstmarkel1'
         write(irefwr,*) 'iobst ', iobst
         call prinin ( kmeshx(1,iobst), 6, 'kmeshx' )

      end if
      if ( ierror/=0 ) go to 1000

      inelem = nelem
      istartelem = ielemobst(iobst)
      istartelemcp = lenielemobst

      icurnrobs = kmeshx(2,iobst)

!     --- Copy coordinates of curve into curvescp

      call sepcopcurvcoor ( icurnrobs, curvescp, nnodes )

!     --- Check if there are any elements with all vertices in the obstacle, &
!         that still crosses the obstacle
!         Copy adapted array ielemobst in ielemobstcp

      call  sepobstmarkel4 ( iwork, iobst, ielemobst(istartelem), &
                             ielemobstcp(istartelemcp), &
                             curvescp, nx, ny, nnodes )
      numinelmobst = ielemobstcp(istartelemcp)+ &
                     ielemobstcp(istartelemcp+1)
      numbnelmobst = ielemobstcp(istartelemcp+2)

!     --- Clear array iwork

      iwork(1:nelem) = 0

!     --- Mark elements in the obstacle by a 1

      do i = 1, numinelmobst+numbnelmobst
         iwork(ielemobst(istartelem+2+i)) = 1
      end do

!     --- Store all edges that correspond to elements that are in the
!         obstacle and that also correspond to at least one element
!         outside the obstacle in array istore

      numelmobst = numinelmobst+numbnelmobst
      call mshobstmarkel2 ( numelmobst, ielemobst(istartelem+3), &
                            istore, lstore, iobst, &
                            curvescp, nx, ny, iwork, nnodes )

!     --- Store all elements corresponding to edges in istore and not already in
!         ielemobst into ielemobst
!         Raise lenielemobst

      ielemobstcp(iobst) = istartelemcp
      lenielemobst = istartelemcp+2+numbnelmobst+numinelmobst

      call mshobstmarkel3 ( nextra, lstore, istore, nmeshh, iwork, &
                            lenielemobst, ielemobstcp )
      numbnelmobst = numbnelmobst+nextra

!     --- Repeat the process for all new elements untill no elements are added

300   if ( nextra>0 ) then

!     --- There are extra elements

         call mshobstmarkel2 ( nextra,  ielemobstcp(lenielemobst-nextra+1), &
                               istore, lstore, iobst, curvescp, &
                               nx, ny, iwork, nnodes )
         if ( ierror/=0 ) go to 1000

!        --- store extra elements in ielemobstcp

         call mshobstmarkel3 ( nextra, lstore, istore, nmeshh, iwork, &
                               lenielemobst, ielemobstcp )
         numbnelmobst = numbnelmobst+nextra
         go to 300

      end if

      ielemobstcp(istartelemcp+2) = numbnelmobst
      lenielemobst = lenielemobst+1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepobstmarkel1'

      end if
      call erclos ( 'sepobstmarkel1' )

      end
