      subroutine sepobstmarkel4 ( iwork, iobst, ielemobstlc, ielemobstcp, &
                                  curvescp, nx, ny, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if there are any elements with all vertices in the obstacle, &
!     that still crosses the obstacle
!     Copy adapted array ielemobst in ielemobstcp
!     Old subroutine mshobstmarkel4
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     element
!     obstacle
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), ielemobstcp(*), iobst, nx, ny, &
              nnodes, ielemobstlc(*)
      double precision :: curvescp(2,*)

!     curvescp       i    array containing the coordinates of the curves
!     ielemobstcp    o    Copy of ielemobst rordered
!     ielemobstlc    i    Array containing information about the elements in an
!                         obstacle
!                         The first numobst positions contain the starting
!                         positions of each obstacle sequentially
!                         Per obstacle the first three positions contain
!                         1. The number of elements that are completely inside
!                               the obstacle
!                         2. The number of elements that are not completely
!                               inside the element but with all vertices inside
!                               the element
!                         3. The number of elements that are partly inside
!                               the obstacle
!                               elements that contain only boundary points of
!                               the obstacle are not considered
!                         These positions are followed by the element numbers
!                         in the sequence defined by the first three positions
!     iobst          i    Obstacle sequence number
!     iwork               integer work array of size nelem
!                         if iwork(ielem) = 1 element ielem intersects the
!                         obstacle, otherwise iwork(ielem) = 0, not
!     nnodes         i    Number of nodes
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxcells
      parameter (maxcells=60)
      integer i, ncross, numinelmobst, numbnelmobst, k, j, &
              ielem, ipstart, nedge_elem, iedge, jpstart, nnode_edge, &
              inode1, inode2, icell(maxcells), ncell, ind, istart1, &
              istart2
      double precision xmin, ymin, dx, dy, eps, x1(2), &
                       x2(2), xlow(2), xupp(2), fact1, fact2

!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     eps            Local accuracy
!     fact1          multiplication factor
!     fact2          multiplication factor
!     i              Counting variable
!     icell          Array containing the curve parts that must be considered
!                    for computing the cross sections
!     iedge          edge sequence number
!     ielem          Element number
!     ind            Integer array containing the cyclic permutation of
!                    1,2,3,4
!     inode1         First node in edge
!     inode2         Second node in edge
!     ipstart        Starting address in array
!     istart1        First position used at a part
!     istart2        First position used at a part
!     j              Counting variable
!     jpstart        Starting address in array
!     k              Counting variable
!     maxcells       Declared length of array icell
!     ncell          Number of curve parts that must be used to compute
!                    cross sections
!     ncross         Number of extra cross sections found
!     nedge_elem     number of edges corresponding to element
!     nnode_edge     number of nodes corresponding to edge
!     numbnelmobst   Number of elements that are partly within the obstacle
!     numinelmobst   Number of elements that are completely within the obstacle
!     x1             x-coordinate of first point
!     x2             x-coordinate of last point
!     xlow           contains coordinates of first point on curve part
!     xmin           Minimum x-value
!     xupp           contains coordinates of last point on curve part
!     ymin           Minimum y-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCROSSLINE   Computes the intersection of two lines
!     MSHOBSTINTLIN1 Fill array icell with indices of curve parts of the
!                    obstacle that is possibily intersected by the line x1-x2
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork
!     Extract xmin,xmax,ymin,ymax from obstacle array
!     ncross := 0
!     For all internal elements ielem do
!        Find edges corresponding to element
!        For all edges of element do
!           if ( element intersects obstacle ) then
!              ncross := nross+1
!              iwork(ielem) := 1
!     if ( ncross=0 ) then
!        Copy ielemobst in ielemobstcp
!     else ( ncross>0 )
!        Put elements in ielemobstcp in new sequence
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepobstmarkel4' )

!     --- Clear array iwork

      iwork(1:nelem) = 0

!     --- Extract xmin,xmax,ymin,ymax from obstacle array

      xmin = rmeshx(1,iobst)
      ymin = rmeshx(3,iobst)
      dx   = rmeshx(5,iobst)
      dy   = rmeshx(6,iobst)
      eps  = 0.01d0

!     --- ncross := 0

      ncross = 0
      numinelmobst = ielemobstlc(1)
      numbnelmobst = ielemobstlc(3)

!     --- For all internal elements ielem do

      do k = 1, numinelmobst

         ielem = ielemobstlc(3+k)

!        --- Find edges corresponding to element

         ipstart = nmeshd(ielem)
         nedge_elem = nmeshd(ipstart)

!        --- Loop over the edges of the element

         do j = 1, nedge_elem

!        --- edge iedge

            iedge = abs(nmeshd(ipstart+j))
            jpstart = nmeshc(iedge)
            nnode_edge = nmeshc(jpstart)

!           --- store initial and end node in inode1/2

            inode1 = nmeshc(jpstart+1)
            inode2 = nmeshc(jpstart+nnode_edge)
            x1(1) = coor(1,inode1)
            x1(2) = coor(2,inode1)
            x2(1) = coor(1,inode2)
            x2(2) = coor(2,inode2)

!           --- Fill array icell with indices of curve parts of the obstacle
!               that is possibily intersected by the line x1-x2

            call mshobstintlin1 ( x1, x2, xmin, ymin, nx, ny, iobgrid2d, &
                                  icell, ncell, dx, dy, nnodes, &
                                  maxcells, iobgrid2d1 )
            if ( ierror/=0 ) go to 1000

!           --- For i := 1 (1) ncell do

            do i = 1, ncell

!           --- xlow := first point of obstacle part icell(i)
!               xupp := last point of obstacle part icell(i)

               ind = icell(i)
               xlow(1) = curvescp(1,ind)
               xlow(2) = curvescp(2,ind)
               xupp(1) = curvescp(1,ind+1)
               xupp(2) = curvescp(2,ind+1)

!              --- Check if lines intersect and compute intersection

               call mshcrossline ( x1, x2, xlow, xupp, fact1, fact2, &
                                   eps )

               if ( fact2>-eps .and. fact2<1d0+eps ) then

!              --- the line x1-x2 intersects the line xlow-xupp
!                  ncross := nross+1
!                  iwork(ielem) := 1

                  ncross = ncross+1
                  iwork(ielem) = 1
                  go to 200

               end if

            end do

         end do

200   end do

      if ( ncross==0 ) then

!     --- ncross=0; Copy ielemobst in ielemobstcp

         ielemobstcp(1) = numinelmobst
         ielemobstcp(2) = 0
         ielemobstcp(3) = numbnelmobst
         do i = 1, numinelmobst+numbnelmobst
            ielemobstcp(3+i) = ielemobstlc(3+i)
         end do

      else

!     --- ncross>0; Put elements in ielemobstcp in new sequence

         ielemobstcp(1) = numinelmobst-ncross
         ielemobstcp(2) = ncross
         ielemobstcp(3) = numbnelmobst
         istart1 = 3
         istart2 = 3+numinelmobst-ncross
         do i = 1, numinelmobst
            ielem = ielemobstlc(3+i)
            if ( iwork(ielem)==0 ) then

!           --- internal element

               istart1 = istart1+1
               ielemobstcp(istart1) = ielem

            else

!           --- boundary element

               istart2 = istart2+1
               ielemobstcp(istart2) = ielem

            end if
         end do
         do i = 1, numbnelmobst
            ielemobstcp(3+numinelmobst+i) = ielemobstlc(3+numinelmobst+i)
         end do

      end if

1000  call erclos ( 'sepobstmarkel4' )

      end
