      subroutine msh2iinput ( nbuf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 27-11-2012 Long integers
!        version  2.1  date 30-07-2012 New treatment of iinput/rinput
!        version  2.0  date 02-09-2010 Remove intarmsh
!
!   copyright (c) 2003-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Translate a sepran mesh input file to the arrays iinput and rinput
!     and write to the file iinput.file
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     conversion
!     file
!     input
!     output
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer (kind=8) nbuf

!     nbuf           i    Length of nbuffr as declared by the user.
!                         The user must declare ibuffr himself in the main
!                         program
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, manag(lenmanag), jstart, nextrecord, &
              iref, lengin, lengrl, lniinp, lnrinp, ireadr
      double precision cunit, maxratio
      logical adaptiinput
      integer, allocatable :: iinput(:)
      double precision, allocatable :: rinput(:)

!     adaptiinput    If true iinput must be adapted
!     cunit          Unit length for coarseness
!     i              Counting variable
!     iinput         integer input array
!     ireadr         If 1 the first record has been read by READSP
!     iref           File reference number
!     jstart         starting position of curve i in kmeshm
!     lengin         Length of integer array iinput
!     lengrl         Length of real array rinput
!                    (in double precision variables)
!     lniinp         Available space for array IINPUT
!     lnrinp         Available space for array RINPUT
!     manag          Manager array that sets the parameters in subroutine
!                    SEPSTR
!                    See subroutine readsp
!     maxratio       Maximal ratio between succeeding elements
!     nextrecord     Defines last record read by readsp
!                    Possible values:
!                    0: none
!                    1: PROBLEM
!                    2: POSTPROCESSING
!                    3: MESH
!                    4: READ MESH
!                    5: EXTRA_MESH_INPUT
!     rinput         real input array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INIFIL         Open files
!     INITCB         Initializes SEPRAN machine-independent common blocks
!     INITMD         Set machine-dependent quantities
!     MSHRDA         Read first part of the mesh input and set some constants
!     MSHREA         Read mesh input
!     PRINCONS       Print all constants, variables and vector names
!     READSP         Reads information for subroutine SEPSTL
!     SEPSTART       Standard SEPRAN starting subroutine
!     STARTSEPRAN    Create buffer array and fill part of sepmodulebuf
! **********************************************************************
!
!                       I/O
!
!     The mesh input is read from the standard input file
!     The arrays iinput and rinput are written to the file iinput.file
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Initialize common blocks
!     Open read and write files
!     Open error message file
!     Read debug file if it exists
!     Read array manag (defaults)
!     Print a list of all constants, variables, reals and vector names
!     Start the SEPRAN program
!     read input from standard input file
!     Store arrays IINPUT and RINPUT in IBUFFR
!     Write arrays iinput and rinput to the output file
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh2iinput' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh2iinput'

      end if
      if ( startcalled/=99991 ) call startsepran

!     --- Initialize common blocks

      call initcb
      mainsubr = 1
      call initmd

!     --- Open read and write files

      call inifil(5)
      call inifil(6)

!     --- Open error message file

      call inifil(4)

!     --- Read array manag (defaults)

      if ( nbuf>=0 ) then
         manag(5) = 1000
      else
         manag(5) = -1000
      end if
      call readsp ( manag, lenmanag, nextrecord )

!     --- Print a list of all constants, variables, reals and vector names

      call princons

!     --- Standard SEPRAN start subroutine

      jstart = abs(manag(1))
      if ( mod(jstart/100,10)==0 ) jstart = jstart + 100
      if ( manag(1)<0 ) then
         manag(1) = -jstart
      else
         manag(1) = jstart
      end if
      call sepstart ( manag(1), manag(2), manag(3), manag(4) )
      call eropen ( 'msh2iinput' )

!     --- read input from standard input file
!         Store arrays IINPUT and RINPUT in IBUFFR
!         First compute some constants

      ireadr = 1
      ndim = manag(8)
      call mshrda ( ndim, cunit, maxpnt, maxcrv, maxsrf, &
                    maxvlm, jcoars, ireadr, ispecial, &
                    iscale, itype_scale, maxratio )
      if ( ierror/=0 ) go to 1000

!     --- Reserve temporary space to store  iinput and rinput

      lniinp = 1000 + 10*maxcrv + 15*maxsrf + 10*maxvlm
      lnrinp = 100 + (ndim+1)*maxpnt + 100*maxcrv

      allocate ( iinput(lniinp), rinput(lnrinp), stat = error )
      if ( error/=0 ) call eralloc ( error, lniinp+lnrinp, 'iinput' )
      if ( ierror/=0 ) go to 1000

!     --- Read rest of input file

      call mshrea ( iinput, rinput, lengrl, lengin, &
                    lnrinp, lniinp, cunit, adaptiinput, maxratio )

!     --- Write arrays iinput and rinput to the output file

      iref = 10
      open ( unit = iref, file = 'iinput.file' )
      write(iref,120) '# Array iinput:  preceded by number of entries'
      write(iref,100) lengin
      write(iref,100) (iinput(i), i = 1, lengin)
100   format(10i6)
      write(iref,120) '# Array rinput:  preceded by number of entries'
      write(iref,100) lengrl
      write(iref,110) (rinput(i), i = 1, lengrl)
110   format(4d18.10)
      close(unit=iref)
120   format(a)

      deallocate ( iinput, rinput, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iinput' )

1000  if ( debug ) write(irefwr,*) 'End msh2iinput'
      call erclos ( 'msh2iinput' )

      end subroutine msh2iinput
