      subroutine msh358 ( nelem, coor, kmeshc, inpelm, sameorien )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 04-02-2015 Debug statements
!        version  1.2  date 10-12-2003 Only look to the previous element
!        version  1.1  date 29-12-2001 Adaptation check
!
!   copyright (c) 1998-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Check if all elements in the bottom surface have the same orientation
!
! **********************************************************************
!
!                       KEYWORDS
!
!     check
!     mesh
!     mesh_generation
!     orientation
!     triangle
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, inpelm, kmeshc(inpelm,nelem)
      double precision coor(3,*)
      logical sameorien

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     inpelm     i    number of points per element in bottom surface
!     kmeshc     i    kmesh part c, contains the nodal points corresponding
!                     to the elements in bottom surface
!     nelem      i    Number of elements in bottom surface
!     sameorien  o    If true all elements in the bottom surface have the same
!                     orientation (clockwise or counterclockwise) if false
!                     they have not
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i1,i2, i3, ielem, i
      double precision refnorm(3), norm(3), x1(3), x2(3), inprod, &
                       maxinner

!     i              Counting variable
!     i1             First point in element
!     i2             Second point in element
!     i3             Third point in element
!     ielem          Element number
!     inprod         Inner product
!     maxinner       Maximum value of inner product
!     norm           Contains normal vector of element
!     refnorm        Contains normal vector of first element
!     x1             contains tangential vector along first side
!     x2             contains tangential vector along second side
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ddot

!     DDOT           (CDOT)     Inner product of two vectors
!     ELOUTR         Compute outer product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!      Compute normal on first element and store in refnorm
!      sameorien := true
!      For ielem := 1, nelem do
!         Compute normal on element
!         If (normal,refnorm)<0 then
!            sameorien := false
!         refnorm := norm
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh358' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh358'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute normal on first element and store in refnorm

      i1 = kmeshc(1,1)
      i2 = kmeshc(2,1)
      i3 = kmeshc(3,1)
      do i = 1, 3
         x1(i) = coor(i,i2) - coor(i,i1)
         x2(i) = coor(i,i3) - coor(i,i2)
      end do
      call eloutr ( x1, x2, refnorm )
      sameorien = .true.

!     --- For ielem := 1, nelem do

      maxinner = (refnorm(1)**2+refnorm(2)**2+refnorm(3)**2)*1d-6
      do ielem = 1, nelem

!     --- Compute normal on element

         i1 = kmeshc(1,ielem)
         i2 = kmeshc(2,ielem)
         i3 = kmeshc(3,ielem)
         do i = 1, 3
            x1(i) = coor(i,i2) - coor(i,i1)
            x2(i) = coor(i,i3) - coor(i,i2)
         end do
         call eloutr ( x1, x2, norm )
         inprod = ddot ( 3, refnorm, 1, norm, 1 )

!        --- refnorm := norm, which implies that we look only to the previous
!            element
!            This is necessary to avoid a warning in case of a very curved
!            boundary

         refnorm(1) = norm(1)
         refnorm(2) = norm(2)
         refnorm(3) = norm(3)
         if ( inprod<-maxinner ) sameorien = .false.

         if ( debug ) then
            write(irefwr,100) 'ielem, norm', ielem, norm
100         format ( a, i6, 3d12.3 )
         end if  ! ( debug )

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh358'

      end if  ! ( debug )
      call erclos ( 'msh358' )

      end subroutine msh358
