      subroutine msh3dquadboun1 ( kbndpt, nelboun, iedgenodes, nmeshc, &
                                  ifacedges, npoint, nedges, nfaces, &
                                  nelem, ishape )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-05-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kbndpt for a pipe consisting of tetrahedrons of shape 15 or 16
!     It is supposed that kbndpt has already been filled for linear tetrahedrons
!     kbndpt contains the triangles on the boundary of the pipe in the
!     specific pipe sequence
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     volume
!     pipe
!     tetrahedron
!     boundary
!     quadratic
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kbndpt(*), nelboun, iedgenodes(*), nmeshc(*), &
              ifacedges(2,*), npoint, nedges, nfaces, nelem, ishape

!     iedgenodes     i    Work array in which all the boundary edges
!                         corresponding to the boundary nodes are stored
!                         in the following sequence
!                         Pos. 1 to npoint: starting positions of all nodes i
!                         in iedgenodes
!                         iedgenode(1) = npoint+1
!                         Number of edges coupled to node i:
!                         iedgenode(i+1)-iedgenode(i)
!                         edges coupled to node i are stored in iedgenode
!                         from position iedgenode(i)+1
!     ifacedges      i    Work array in which all the faces corresponding to
!                         the boundary edges i are stored in
!                         ifaceedges(1,i) and ifaceedges(2,i)
!                         if i is not a boundary edge ifaceedges(.,i) = 0
!     ishape         i    Shape number of volume elements
!                         In this case ishape = 15 or 16
!     kbndpt        i/o   array containing the nodal point numbers of the
!                         boundary stored per triangle
!                         At input the linear triangles with corresponding
!                         node numbers
!                         At output the 7 point triangles with the new
!                         numbering
!                         Storage per element as in kmeshn
!     nedges         i    number of edges of volume
!     nelboun        i    Number of elements in the boundary, i.e. in kbndpt
!     nelem          i    Number of elements of volume
!     nfaces         i    Number of faces for volume
!     nmeshc         i    nmesh part c of volume
!     npoint         i    Number of nodal points of volume
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, istartold, istartnew, nodp1, nodp2, nodp3, istart, &
              nedgesconn, i, iedge, iedge1, iedge2, iedge3, icount, &
              ipstart, iface, ncount
      logical edge1found, edge3found

!     edge1found     If true edge 1 has been found
!     edge3found     If true edge 3 has been found
!     i              Counting variable
!     icount         Counter
!     iedge          edge sequence number
!     iedge1         edge sequence number
!     iedge2         edge sequence number
!     iedge3         edge sequence number
!     ielem          Element number
!     iface          Face sequence number
!     ipstart        Starting address in array
!     istart         First position used at a part
!     istartnew      New starting address in array kbndpt
!     istartold      Old starting address in array kbndpt
!     ncount         Counter
!     nedgesconn     Number of edges connected to node
!     nodp1          Node sequence number
!     nodp2          Node sequence number
!     nodp3          Node sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all boundary elements starting from the last one do
!        Compute old and new starting address
!        Extract 3 node numbers
!        Find 3 edges
!        Find face
!        Fill kbndpt
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh3dquadboun1' )
      debug = .false.

      ncount = npoint-nedges-nfaces
      if ( ishape.eq.16 ) ncount = ncount-nelem

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Subroutine msh3dquadboun1:'
         write(irefwr,*) 'npoint,nedges,nfaces,ishape,ncount ', &
                                 npoint,nedges,nfaces,ishape,ncount
         call prinin1 ( ifacedges, nedges, 2, 'ifacedges' )

      end if

!     --- For all boundary elements starting from the last one do

      do ielem = nelboun, 1, -1

!     --- Compute old and new starting address

         istartold = (ielem-1)*3
         istartnew = (ielem-1)*7

!        --- Extract 3 node numbers

         nodp1 = kbndpt(istartold+1)
         nodp2 = kbndpt(istartold+2)
         nodp3 = kbndpt(istartold+3)

!        --- Find 3 edges

         istart = iedgenodes(nodp1)
         nedgesconn = iedgenodes(nodp1+1)-istart
         icount = 0
         edge1found = .false.
         edge3found = .false.
         do i = 1, nedgesconn
            iedge = iedgenodes(istart+i)
            ipstart = nmeshc(iedge)
            if ( .not. edge1found .and. &
                (nmeshc(ipstart+1).eq.nodp2 .or. &
                 nmeshc(ipstart+2).eq.nodp2) ) then

!           --- edge from node 1 to node 2

               iedge1 = iedge
               icount = icount+1
               edge1found = .true.

            else if ( .not. edge3found .and. &
                     (nmeshc(ipstart+1).eq.nodp3 .or. &
                      nmeshc(ipstart+2).eq.nodp3) ) then

!           --- edge from node 1 to node 3

               iedge3 = iedge
               icount = icount+1
               edge3found = .true.

            end if
            if ( icount.eq.2 ) go to 100

         end do

100      istart = iedgenodes(nodp2)
         nedgesconn = iedgenodes(nodp2+1)-istart

         do i = 1, nedgesconn
            iedge = iedgenodes(istart+i)
            ipstart = nmeshc(iedge)
            if ( nmeshc(ipstart+1).eq.nodp3 .or. &
                 nmeshc(ipstart+2).eq.nodp3 ) then

!           --- edge from node 2 to node 3

               iedge2 = iedge
               go to 200

            end if

         end do

!        --- Find face

200      iface = ifacedges(1,iedge1)
         if ( iface.ne.ifacedges(1,iedge2) .and. &
              iface.ne.ifacedges(2,iedge2) ) &
            iface = ifacedges(2,iedge1)

!        --- Fill kbndpt

         kbndpt(istartnew+1 ) = nodp1
         kbndpt(istartnew+2 ) = iedge1+ncount
         kbndpt(istartnew+3 ) = nodp2
         kbndpt(istartnew+4 ) = iedge2+ncount
         kbndpt(istartnew+5 ) = nodp3
         kbndpt(istartnew+6 ) = iedge3+ncount
         kbndpt(istartnew+7 ) = iface+ncount+nedges

      end do

      if ( debug ) then

!     --- Debug information

         call prinin1 ( kbndpt, nelboun, 7, 'kbndpt' )

      end if

      call erclos ( 'msh3dquadboun1' )
      end
