      subroutine msh430 ( crit, coorh, kelemh, kromh , lcoor, &
                          lkelem, lkrom, npold , user, iwork, &
                          irefus, irelat, jrefin )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  4.5  date 28-02-1999 Include commons
!        version  4.4  date 25-02-1996 Adapt error message
!        version  4.3  date 08-11-1993 New call, kpoine added
!
!   copyright (c) 1988-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Refine an existing mesh of linear or quadratic triangles locally
!     utilizing a special criterion as given in array CRIT
!     or as given by the user in a special routine named REFUSE.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), crit(*), coorh(2,*)
      integer kelemh(*), &
              kromh(*), iwork(*), lcoor, lkelem, lkrom, npold, irefus, &
              irelat, jrefin

!     coorh   o    new coor array after the refinement
!     crit    i    criterion array, if irefus = 1 then an INPUT
!                  if irefus = 2 array to be filled by the user
!     irefus  i    indicator whether array crit has already been
!                  filled in REFLOC (=1) or not (=2)
!     irelat  i    relative (=1) or absolute (=2) error
!     iwork   i    array of length ncurvs containing for each
!                  curve a 0 when i has been filled and a 1 when
!                  the curve has not been filled
!     jrefin  i    loop variable for actual number of refinements
!     kelemh  o    new kelem array after the refinement
!     kromh   o    new krom array after the refinement
!                  at input : declared length
!                  at output : used length
!     lcoor  i,o   length of new coor array
!                  at input : declared length
!                  at output : used length
!     lkelem i,o   length of new kelem array
!                  at input : declared length
!                  at output : used length
!     lkrom  i,o   length of new krom array
!                  at input : declared length
!                  at output : used length
!     npold   o    new value of npoint when new points on the
!                  lines have been placed
!     user    i    array which can be used by user to have values
!                  available for criterion array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kwad, i, new, iklemh, ikelem, nelhel, &
              neltal, inpelo, ielgrp, nextra, npunt, ist, ish, &
              ibeg, icurve, iend, ilnew, ilold, inew, inpelm, ipnt, &
              ipos3, npelem
      double precision base1, base2
      integer, allocatable :: kpoine(:)
      double precision, allocatable :: help(:)

!     base1   first help variable to give line unique number
!     base2   second help variable to give line unique number
!     help    help array for new points
!     i       counting variable
!     ibeg    start node of line
!     icurve  curve number
!     ielgrp  element group number
!     iend    end node of line
!     ikelem  pointer in array kelem
!     iklemh  pointer in array kelemh
!     ilnew   new length of curve array
!     ilold   old length of curve array
!     inew    number of new created points
!     inpelm  number of points per element
!     inpelo  old number of points in element (for mshtyp = -9)
!     ipnt    loop variable for points on lines
!     ipos3   position pointer in curves array
!     ish     help variable for nodes
!     ist     help variable for starting position
!     kpoine  Help array (length nelgrp+1) to store kmesh part a
!     kwad    indicator whether linear (=1) or quadratic (=2) mesh
!     nelhel  helpvariable to store number of new elements of one group
!     neltal  helpvariable to store total number of elements in mesh
!     new     indicator whether new point is needed
!     nextra  indicator whether extra internal new points have been
!             created in quadratic case
!     npelem  number of elements in standard group
!     npunt   number of nodes before refinement
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH431         Determine whether new point has been created and if
!                    so which node number
!     MSH433         Run through all lines of linear elements and place
!                    new nodes if criterion indicates so
!     MSH434         Run through all lines of quadratic elements and place
!                    new nodes if criterion indicates so
!     MSH436         In quadratic case : Create new elements and internal
!                    points
!     REFUSE         User routine with refinement criterion
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     475   wrong dimension (#2)
!     550   array length too short
!     800   wrong ishape value
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine the important parameters and do checks
!     Set parameter for linear or quadratic case
!     Local refinement ; User has to start with filling array crit
!     Copy coor to coorh
!     Set variables for unique number system
!     Run through group of elements and check each side
!     if one group of standard elements perform actions
!      else
!     if several groups of standard elements perform actions
!     In quadratic case extra internal points have to be created
!        In that case
!           Run through group of elements and check each element
!           Create internal points and new elements
!     Fill array kmesh and Consider the curves etc
!     Rearrange the criterion array in case of a relative criterion
!     Fill kromh
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh430' )

      allocate ( kpoine(nelgrp+1), help(3*npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, nelgrp+1, 'kpoine' )
      if ( ierror/=0 ) go to 1000

      if ( ndim/=2 ) then
         call errint ( ndim, 1 )
         call errsub ( 475, 1, 0 , 0 )
         go to 1000
      end if

!     --- Set parameter for linear or quadratic case

      kwad  = 1

!     --- Local refinement ; User has to start with filling array crit

      if ( irefus==2 ) call refuse(coor,npoint,crit,user,jrefin)

!     --- Copy coor to coorh

      coorh(:,1:npoint) = coor(:,1:npoint)

!     --- Set variables

      new     =   0
      base1   =   4 * npoint
      base2   =   base1 * base1
      nelhel  =   0
      iklemh  =   1
      ikelem  =   1

!     --- Run through group of elements and check each side

      if ( kelmg==0 ) then

!     --- Only one group of standard elements

           ishape = kmeshf(1)

           if ( ishape==2 .or. ishape==4 .or. &
                ishape==6 ) kwad = 2

           if ( (ishape<1 .or. ishape>4) .and. &
                ishape/=-9 ) then
                call errint ( ishape, 1 )
                call errsub ( 800, 1, 0, 0 )
                go to 1000
           end if

           if ( ishape==-9 .or. ishape==1 .or. ishape.eq.3 ) then

           call msh433(kmeshc,nelem,coor,coorh,npoint,new,crit,help, &
                       base1,base2,ishape,npelm,kelemh,nelhel,irelat)

           iklemh   = nelhel * npelm + 1
           nelem = nelhel

           else if ( ishape==2 .or. ishape==4 ) then

           call msh434(kmeshc,nelem,coor,coorh,npoint,new,crit,help, &
                       base1,base2,ishape,irelat)

           end if

      else

!     --- Several groups of standard elements

           neltal = 0

!          --- Starting address in kmeshc

           ikelem = 1
           iklemh = 1

!          --- Set first position of helparray KPOINE

           kpoine(1) = 0

           do ielgrp=1,nelgrp

!          --- Determine number of points in each standard element

               inpelm = Topgrp(ielgrp)%inpelm
               ishape = kmeshf(ielgrp)

               inpelo = inpelm

               if ( ishape==2 .or. ishape==4 .or. &
                    ishape==6 ) kwad=2

               if ( (ishape<1 .or. ishape>4) .and. &
                    ishape/=-9 ) then
                   call errint ( ishape, 1 )
                   call errsub ( 800, 1, 0, 0 )
                   go to 1000
               end if

!              --- Number of elements of this type

               npelem = Topgrp(ielgrp)%inelem
               nelhel = 0

               if ( ishape==-9.or.ishape==1.or.ishape.eq.3) then

               call msh433(kmeshc(ikelem),npelem,coor,coorh,npoint, &
                           new,crit,help,base1,base2,ishape,inpelm, &
                           kelemh(iklemh),nelhel,irelat)

               else if ( ishape==2 .or. ishape==4 ) then

               call msh434(kmeshc(ikelem),npelem,coor,coorh,npoint, &
                           new,crit,help,base1,base2,ishape,irelat)

               end if

!              --- Increase ikelem

               ikelem = ikelem + npelem * inpelo
               iklemh = iklemh + nelhel * inpelm

               neltal = neltal + nelhel

!              --- Number of elements in new situation

               if ( kwad==1 ) kmeshg(ielgrp) = nelhel

!              --- Set value in helparray

               kpoine( 1 + ielgrp ) = kpoine( ielgrp ) + inpelm

           end do

           if ( kwad==1 ) nelem = neltal

           if ( kelma/=0 ) then

               do ielgrp = 1, nelgrp+1

                   kmesha(ielgrp) = kpoine( ielgrp )

               end do

           else

               npelm = inpelm

           end if

      end if

!     --- In quadratic case extra internal points have to be created

      nextra = 0

      if ( kwad==2 ) then

!     --- Run through group of elements and check each element

         if ( kelmg==0 ) then

!        --- Only one group of standard elements

            ishape = kmeshf(1)

            if ( ishape==4 .or. ishape==6 ) then

!           --- Create internal points and new elements

               call msh436( kmeshc, nelem, coorh, npoint, new, nextra, &
                            crit, help, base1, base2, ishape, kelemh, &
                            nelhel, irelat )

               iklemh   = nelhel * npelm + 1
               nelem = nelhel

            end if

         else

!        --- Several groups of standard elements

            neltal = 0

!           --- Starting address in kmeshc

            ikelem = 1
            iklemh = 1

            do ielgrp = 1 , nelgrp

               inpelm = Topgrp(ielgrp)%inpelm
               ishape = kmeshf(ielgrp)

                inpelo = inpelm

!               --- number of elements of this type

                npelem = Topgrp(ielgrp)%inelem
                nelhel = 0

                if ( ishape==4 .or. ishape==6 ) then

                   call msh436( kmeshc(ikelem), npelem, coorh, npoint, &
                                new, nextra, crit, help, base1, base2, &
                                ishape, kelemh(iklemh), nelhel, irelat)

!                  --- Increase ikelem

                   ikelem = ikelem + npelem * inpelo
                   iklemh = iklemh + nelhel * inpelm

                   neltal = neltal + nelhel

!                  --- Number of elements in new situation

                   kmeshg(ielgrp) = nelhel

                end if

            end do

            if ( ishape==4 .or. ishape==6 ) nelem = neltal

         end if

      end if

!     --- Consider the lines etc

      npunt  = npoint
      npold  = npoint + new + nextra

      npoint   = npold

      if ( npoint>lcoor ) then
          call errint ( npoint, 1 )
          call errint ( lcoor, 2 )
          call errchr ( 'coor', 1 )
          call errsub ( 550, 2, 0, 1 )
          go to 1000
      end if

      lcoor = ndim * npoint

!     --- Rearrange the criterion array in case of a relative criterion

      if ( irelat==1 ) then

           do i = 1 , npoint

               if ( crit(i)>0d0 ) crit(i) = crit(i) - 1d0

           end do

      end if

      if ( iklemh - 1>lkelem ) then
          call errint ( iklemh - 1, 1 )
          call errint ( lkelem, 2 )
          call errchr ( 'kelem', 1 )
          call errsub ( 550, 2, 0, 1 )
          go to 1000
      end if

      lkelem = iklemh - 1

!     --- Fill kromh

      ninner    = kmeshm(1)
      kromh(1)  = kmeshm(1)
      kromh(2)  = kmeshm(2)
      kromh(4)  = kmeshm(4)
      kromh(5)  = 1

      ist       = kmeshm(2)
      ish       = kromh(2)

!     --- Set step-unit

      do icurve = 1 , ncurvs

          ilold = kmeshm(icurve+5)-kmeshm(icurve+4)

          if ( iwork( icurve )==1 ) then

!         --- icurve has not been filled,special situation

               do i = 0 , ilold-1
                   kromh(ish+i) = kmeshm(ist+i)
               end do

               kromh(icurve+5) = kromh(icurve+4) + ilold
               ist = ist + ilold
               ish = ish + ilold

          else

!         --- curve has been filled

              ilnew = ilold

              kromh(ish) = kmeshm(ist)
              ish = ish + 1

              do ipnt = 1 , ilold-1, kwad

                  ibeg = kmeshm(ist+ipnt-1)
                  iend = kmeshm(ist+ipnt+kwad-1)

                  call msh431( ibeg, iend, base1, base2, help, new, &
                               inew, kwad )

                  if ( kwad==1 ) then

                       if ( inew/=0 ) then
                            ilnew        = ilnew + 1
                            kromh(ish)   = npunt + inew
                            ish          = ish + 1
                       end if

                  else if ( kwad==2 ) then

                       if ( inew/=0 ) then
                            ilnew        = ilnew + 2
                            if ( ibeg<iend ) then
                                 kromh(ish  ) = npunt + inew
                                 kromh(ish+1) = kmeshm(ist+ipnt)
                                 kromh(ish+2) = npunt + inew + 1
                            else
                                 kromh(ish  ) = npunt + inew + 1
                                 kromh(ish+1) = kmeshm(ist+ipnt)
                                 kromh(ish+2) = npunt + inew
                            end if

                            ish          = ish + 3

                       else

                            kromh(ish) = kmeshm(ist+ipnt)
                            ish        = ish + 1

                       end if

                  end if

                  kromh(ish)   = iend
                  ish          = ish + 1

              end do

              ist              =  ist + ilold
              kromh(icurve+5)  =  kromh(icurve+4) + ilnew

          end if

      end do

      deallocate ( kpoine, help, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'kpoine' )

      kromh(3) = ish
      ipos3    = ish

      kromh(ipos3)=0

      do i=1,ninner
          kromh(ipos3+i-1)=kmeshm(kmeshm(3)+i-1)
      end do

      if ( kromh(3)+ninner>lkrom ) then
          call errint ( kromh(3)+ninner, 1 )
          call errint ( lkrom, 2 )
          call errchr ( 'kromh', 1 )
          call errsub ( 550, 2, 0, 1 )
          go to 1000
      end if

      lkrom = kromh(3) + ninner

1000  call erclos ( 'msh430' )

      end subroutine msh430
