      subroutine msh647 ( kbound, nbound, coor, number, &
                          nsubsf, ivolnr, inpelm )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.2  date 13-11-2013 Extra information error message
!        version  3.1  date 14-10-2010 New calls msh657 and msh658
!        version  3.0  date 25-03-2009 Change check quadratic elmnts
!
!   copyright (c) 1992-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to determine whether all boundary triangles are mutually
!     disjunct except for edges and points
! **********************************************************************
!
!                       KEYWORDS
!
!     intersection
!     3d
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, kbound(inpelm,*), nbound, nsubsf, &
              number(nsubsf), ivolnr
      double precision coor(3,*)

!     coor           i    standard SEPRAN coordinate array
!     inpelm         i    number of points in boundary element
!     ivolnr         i    Volume sequence number
!     kbound         i    array containing the boundary triangles
!     nbound         i    number of triangles in kbound
!     nsubsf         i    Number of subsurfaces
!     number         i    array containing number of elements of each
!                         subsurface augmented
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i1, i2, i3, i4, i5, i6, ik, il, im, in, io, ip, &
              iprob, iproh, ja, jl1, jl2, itri1, itri2, i, j

!     i              Counting variable
!     i1             node of triangle
!     i2             node of triangle
!     i3             node of triangle
!     i4             node of triangle
!     i5             node of triangle
!     i6             node of triangle
!     ik             node of triangle
!     il             node of triangle
!     im             node of triangle
!     in             node of triangle
!     io             node of triangle
!     ip             node of triangle
!     iprob          indicator for intersections
!     iproh          helpindicator for subintersections
!     itri1          indicator for subsurfacegroup
!     itri2          indicator for subsurfacegroup
!     j              Counting variable
!     ja             indicator whether node is inside
!     jl1            loop variable
!     jl2            loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH657         Subroutine to determine whether the lines of triangle with
!                    nodes (ia, ib, ic) are disjunct with triangle (ik, il, im)
!                    except for edges and nodes
!     MSH658         Check if node i1 is inside "element" ik,il,im,in,io,ip
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     950   There is an intersection of two triangles
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Run through boundary array and check all triangles mutually
!     First check all triangles constructed using the cornernodes only
!     In case of inpelm=6 or =7 and problems check whether the problems
!     are due to the quadratic form. If so change position of nodes
!     temporarily.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh647' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh647'
         write(irefwr,1) 'inpelm, nbound', inpelm, nbound
         call prinin1 ( kbound, nbound, inpelm, 'kbound' )
         call prinrl1 ( coor, 30, 3, 'coor' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Run through array kbound and check all lines and triangles

      itri1 = 1
      itri2 = 1

      if ( inpelm==3 ) then

!     --- Check for all lines and triangles of total boundary:

         do jl1 = 1, nbound
            if ( jl1>number(itri1) ) itri1 = itri1 + 1

!           --- Determine nodes for lines

            i1 = kbound(1,jl1)
            i2 = kbound(2,jl1)
            i3 = kbound(3,jl1)

!           --- Run through boundary and check
!               lines ia - ib, ib - ic, ic - ia

            do jl2 = 1, nbound
               if ( jl2>number(itri2) ) itri2 = itri2 + 1

               ik = kbound( 1,jl2 )
               il = kbound( 2,jl2 )
               im = kbound( 3,jl2 )

               call msh657( ivolnr, coor, itri1, itri2, iprob, &
                            i1, i2, i3, ik, il, im, jl1, jl2 )

               if ( iprob>0 ) then

!              --- Intersection found:

                  call errint(    jl1, 1 )
                  call errint(    jl2, 2 )
                  call errint(  itri1, 3 )
                  call errint(  itri2, 4 )
                  call errint( ivolnr, 5 )
                  do i = 1, 3
                     call errint ( kbound(i,jl1), i+5 )
                     call errint ( kbound(i,jl2), i+8 )
                     do j = 1, 3
                        call erreal ( coor(j,kbound(i,jl1)), 3*(i-1)+j )
                        call erreal ( coor(j,kbound(i,jl2)), 3*(i+2)+j )
                     end do  ! j = 1, 3
                  end do  ! i = 1, 3
                  call errsub ( 950, 11, 18, 0 )

               end if

            end do
            itri2 = 1
         end do
      else if ( inpelm==6.or.inpelm==7 ) then

!     --- Check for all sublines and subtriangles of boundary:

         do jl1 = 1, nbound
            if ( jl1>number(itri1) ) itri1 = itri1 + 1

!           --- Determine nodes for all sublines

            i1 = kbound(1,jl1)
            i2 = kbound(2,jl1)
            i3 = kbound(3,jl1)
            i4 = kbound(4,jl1)
            i5 = kbound(5,jl1)
            i6 = kbound(6,jl1)

            if ( debug ) write(irefwr,'(6i10)') i1,i2,i3,i4,i5,i6

!           --- Check sublines against each boundary triangle:

            do jl2 = 1, nbound
               if ( jl2>number(itri2) ) itri2 = itri2 + 1

               ik = kbound( 1,jl2 )
               il = kbound( 2,jl2 )
               im = kbound( 3,jl2 )
               in = kbound( 4,jl2 )
               io = kbound( 5,jl2 )
               ip = kbound( 6,jl2 )

!              --- Start with checking "only" corner nodes

               call msh657 ( ivolnr, coor, itri1, itri2, iprob, &
                             i1, i3, i5, ik, im, io, jl1, jl2 )

               if ( iprob>0 ) then

!              --- Local problem, due to quadratic form?

                  if ( debug ) &
                  write(irefwr,*) 'local problem due to quadratic form?'
                  iproh = 0

!                 --- Check lines of subtriangles with (sub) ik, il, ip:

                  call msh657 ( ivolnr, coor, itri1, itri2, iprob, &
                                i1, i2, i6, ik, il, ip, jl1, jl2 )
                  iproh = iproh + iprob
                  call msh657 ( ivolnr, coor, itri1, itri2, iprob, &
                                i2, i3, i4, ik, il, ip, jl1, jl2 )
                  iproh = iproh + iprob
                  call msh657 ( ivolnr, coor, itri1, itri2, iprob, &
                                i2, i4, i6, ik, il, ip, jl1, jl2 )
                  iproh = iproh + iprob
                  call msh657 ( ivolnr, coor, itri1, itri2, iprob, &
                                i6, i4, i5, ik, il, ip, jl1, jl2 )
                  iproh = iproh + iprob

!                 --- Check lines of subtriangles with (sub) il, im, in:

                  call msh657 ( ivolnr, coor, itri1, itri2, iprob, &
                                i1, i2, i6, il, im, in, jl1, jl2 )
                  iproh = iproh + iprob
                  call msh657 ( ivolnr, coor, itri1, itri2, iprob, &
                                i2, i3, i4, il, im, in, jl1, jl2 )
                  iproh = iproh + iprob
                  call msh657 ( ivolnr, coor, itri1, itri2, iprob, &
                                i2, i4, i6, il, im, in, jl1, jl2 )
                  iproh = iproh + iprob
                  call msh657 ( ivolnr, coor, itri1, itri2, iprob, &
                                i6, i4, i5, il, im, in, jl1, jl2 )
                  iproh = iproh + iprob

!                 --- Check lines of subtriangles with (sub) il, in, ip:

                  call msh657 ( ivolnr, coor, itri1, itri2, iprob, &
                                i1, i2, i6, il, in, ip, jl1, jl2 )
                  iproh = iproh + iprob
                  call msh657 ( ivolnr, coor, itri1, itri2, iprob, &
                                i2, i3, i4, il, in, ip, jl1, jl2 )
                  iproh = iproh + iprob
                  call msh657 ( ivolnr, coor, itri1, itri2, iprob, &
                                i2, i4, i6, il, in, ip, jl1, jl2 )
                  iproh = iproh + iprob
                  call msh657 ( ivolnr, coor, itri1, itri2, iprob, &
                                i6, i4, i5, il, in, ip, jl1, jl2 )
                  iproh = iproh + iprob

!                 --- Check lines of subtriangles with (sub)  ip, in, io:

                  call msh657 ( ivolnr, coor, itri1, itri2, iprob, &
                                i1, i2, i6, ip, in, io, jl1, jl2 )
                  iproh = iproh + iprob
                  call msh657 ( ivolnr, coor, itri1, itri2, iprob, &
                                i2, i3, i4, ip, in, io, jl1, jl2 )
                  iproh = iproh + iprob
                  call msh657 ( ivolnr, coor, itri1, itri2, iprob, &
                                i2, i4, i6, ip, in, io, jl1, jl2 )
                  iproh = iproh + iprob
                  call msh657 ( ivolnr, coor, itri1, itri2, iprob, &
                                i6, i4, i5, ip, in, io, jl1, jl2 )
                  iproh = iproh + iprob
                  if ( iproh==0 ) then

!                 --- Check for inside nodes

                     if ( debug ) &
                        write(irefwr,*) 'msh647: Problem found?'

!                    --- Check whether i1,i2,i3,i4,i5,i6 are inside
!                        quadratic element ik, il, im, in, io, ip

                     iprob = 0
                     call msh658(i1, coor, ik, il, im, in, io, ip, ja)
                     if ( ja==1 ) iprob = iprob +   1

!                    --- if ( ja==1 ) write(irefwr,*) i1,' is inside'

                     call msh658(i3, coor, ik, il, im, in, io, ip, ja)
                     if ( ja==1 ) iprob = iprob +  10

!                    --- if ( ja==1 ) write(irefwr,*) i3,' is inside'

                     call msh658(i5, coor, ik, il, im, in, io, ip, ja)
                     if ( ja==1 ) iprob = iprob + 100

!                    --- if ( ja==1) write(irefwr,*) i5,' is inside'
!                        write(irefwr,*) i1,i3,i5,'bekeken =',iprob,'?'

                     call msh658(i2, coor, ik, il, im, in, io, ip, ja)
                     if ( ja==1) write(irefwr,*) i2,' is inside'
                     call msh658(i4, coor, ik, il, im, in, io, ip, ja)
                     if ( ja==1) write(irefwr,*) i4,' is inside'
                     call msh658(i6, coor, ik, il, im, in, io, ip, ja)
                     if ( ja==1) write(irefwr,*) i6,' is inside'

!                    --- write(irefwr,*) i2,i4,i6,' not interesting?'

                  else

!                 --- Error in boundary: no solution found

                     write(irefwr,*) 'Real error in boundary volume'
                     call errint(    jl1, 1 )
                     call errint(    jl2, 2 )
                     call errint(  itri1, 3 )
                     call errint(  itri2, 4 )
                     call errint( ivolnr, 5 )
                     do i = 1, 3
                        call errint ( kbound(i,jl1), i+5 )
                        call errint ( kbound(i,jl2), i+8 )
                        do j = 1, 3
                        call erreal ( coor(j,kbound(i,jl1)), 3*(i-1)+j )
                        call erreal ( coor(j,kbound(i,jl2)), 3*(i+2)+j )
                        end do  ! j = 1, 3
                     end do  ! i = 1, 3
                     call errsub ( 950, 11, 18, 0 )
                  end if
               end if

            end do
            itri2 = 1
         end do

      end if

1000  call erclos ( 'msh647' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh647'

      end if  ! ( debug )

      end
