      subroutine mshcentrcircle ( normal, coor, curvenodes, numpoints, &
                                  origin, radius, isurnr, eps )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 15-10-2004 Remove icurnr
!        version  1.0  date 24-03-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute centroid of circle as well as radius
!     Check if all nodes are positioned on circle
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     plane
!     surface
!     centre
!     radius
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision normal(3), coor(3,*), origin(3), radius, eps
      integer curvenodes(*), numpoints, isurnr

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     curvenodes     i    Contains the nodes of the boundary curve
!     eps           i/o   Local accuracy
!     isurnr         i    Surface sequence number
!     normal         i    normal on plane
!     numpoints      i    Number of points in icurnr
!     origin         o    Contains the coordinates of the origin of the circle
!     radius         o    Radius of sphere or circle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision t1(3), t2(3), v1(3), v2(3), xm1(3), xm2(3), &
                       c1, c2, c3, d1, d2, alpha, sum
      integer i, inode1, inode2, inode
      logical positive

!     alpha          constant to compute intersection
!     c1             The matrix to be inverted has elements
!     c2             c1 c2
!     c3             c2 c3
!     d1             first component of right-hand side vector
!     d2             second component of right-hand side vector
!     i              Counting variable
!     inode          Node sequence number
!     inode1         First node in edge
!     inode2         Second node in edge
!     positive       if true the inner product is positive
!     sum            Help parameter to compute a sum
!     t1             First tangential vector
!     t2             Second tangential vector
!     v1             vector perpendicular to t1 in plane
!     v2             vector perpendicular to t2 in plane
!     xm1            Coordinates of mid point of first edge
!     xm2            Coordinates of mid point of second edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELOUTR         Compute outer product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCHECKCIRCLE Check if node is lying on circle defined by origin
!                    and radius
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check if there are sets of consecutive nodes (at least 3) that are on
!     a straight line
!     t1 := first tangential vector
!     Find first tangential vector t2 along curve such that t1 . t2 < 0
!     Use last vector t2
!     C := intersection n x t1 and n x t2
!     R := distance from points on circle to C
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcentrcircle' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcentrcircle'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- t1 := first tangential vector

      inode1 = curvenodes(1)  ! first node on circle
      inode2 = curvenodes(2)  ! second node on circle
      do i = 1, 3
         t1(i) = coor(i,inode2)-coor(i,inode1)
         xm1(i) = 0.5d0*(coor(i,inode2)+coor(i,inode1))
      end do  ! i = 1, 3
      sum = 1d0/sqrt(t1(1)**2+t1(2)**2+t1(3)**2)
      do i = 1, 3
         t1(i) = sum*t1(i)
      end do  ! i = 1, 3
      if ( debug ) &
         write(irefwr,*) 'inode1, inode2, t1 ', inode1, inode2, t1

!     --- Find first tangential vector t2 along curve such that t1 . t2 < 0

      positive = .true.
      inode = 2
      do while ( positive .and. inode.lt.numpoints )

!     --- Raise inode and compute t2

         inode = inode+1
         inode1 = inode2         ! last node used on circle
         inode2 = curvenodes(inode)  ! next node on circle
         do i = 1, 3
            t2(i) = coor(i,inode2)-coor(i,inode1)
         end do  ! i = 1, 3
         sum = 1d0/sqrt(t2(1)**2+t2(2)**2+t2(3)**2)
         do i = 1, 3
            t2(i) = sum*t2(i)
         end do  ! i = 1, 3
         positive = t1(1)*t2(1)+t1(2)*t2(2)+t1(3)*t2(3).gt.1d-12
         if ( debug ) &
            write(irefwr,*) 'inode1, inode2, t2 ', inode1, inode2, t2

      end do  ! while ( positive .and. inode.lt.numpoints )

!     --- Use last vector t2
!         C := intersection n x t1 and n x t2

      do i = 1, 3
         xm2(i) = 0.5d0*(coor(i,inode2)+coor(i,inode1))
      end do  ! i = 1, 3
      call eloutr ( normal, t1, v1 )
      call eloutr ( normal, t2, v2 )

!     --- Compute intersection xm1 + alpha v1 = xm2 + beta v2
!         Hence
!         [ v1  -v2 ] | alpha |  = [xm2-xm1]
!                     | beta  |
!         So
!         | v1^T | [ v1  -v2 ] | alpha |  =  | v1^T | [xm2-xm1]
!         |-v2^T |             | beta  |     |-v2^T |
!         Or
!         | v1^T v1 -v1^T v2 | | alpha | = v1^T [xm2-xm1]
!         |-v2^T v1  v2^T v2 | | beta  | =-v2^T [xm2-xm1]

      do i = 1, 3
         xm2(i) = xm2(i)-xm1(i)
      end do  ! i = 1, 3
      c1 = v1(1)*v1(1)+v1(2)*v1(2)+v1(3)*v1(3)
      c2 = v2(1)*v1(1)+v2(2)*v1(2)+v2(3)*v1(3)
      c3 = v2(1)*v2(1)+v2(2)*v2(2)+v2(3)*v2(3)
      d1 = xm2(1)*v1(1)+xm2(2)*v1(2)+xm2(3)*v1(3)
      d2 = xm2(1)*v2(1)+xm2(2)*v2(2)+xm2(3)*v2(3)
      alpha = 1d0/(c1*c3-c2**2)*(c3*d1-c2*d2)
      if ( debug ) write(irefwr,*) 'c1, c2, c3, d1, d2, alpha ', &
          c1, c2, c3, d1, d2, alpha

      do i = 1, 3
         origin(i) = xm1(i)+alpha*v1(i)
      end do  ! i = 1, 3

      if ( debug ) write(irefwr,*) 'origin ', origin

      if ( debug ) write(irefwr,*) 'origin ', origin

!     --- R := distance from points on circle to C

      radius = (coor(1,inode1)-origin(1))**2+ &
               (coor(2,inode1)-origin(2))**2+ &
               (coor(3,inode1)-origin(3))**2
      eps = radius*eps

      do i = 1, numpoints

         call mshcheckcircle ( curvenodes(i), origin, radius, eps, &
                               coor, isurnr )

      end do  ! i = 1, numpoints
      radius = sqrt(radius)
      if ( debug ) write(irefwr,*) 'radius ', radius

1000  call erclos ( 'mshcentrcircle' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcentrcircle'

      end if

      end
