      subroutine mshconnectcurv ( ileft, iright, istoreleft, istoreright, &
                                  numpoints, ielgrp, iseq, iexclu, istar1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-06-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store nodes along left-hand side into istoreleft
!     and along right-hand side into istoreright
! **********************************************************************
!
!                       KEYWORDS
!
!     connect
!     mesh
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ileft, iright, istoreleft(*), istoreright(*), &
              numpoints, ielgrp, iseq, iexclu, istar1

!     ielgrp         i    element group number
!     iexclu         i    Indicates which points are excluded
!     ileft          i    Left curve in connection element
!     iright         i    Right curve in connection element
!     iseq           i    Sequence number of curve
!     istar1         o    Starting position of first curve in KMESHM
!     istoreleft     o    Contains all periodical right neighbor unknowns
!                         for all left periodical neighbor unknowns
!                         Pos. 1 to nusol+1 contain pointers to the second part
!                         istoreleft(1) contains nusol+1
!                         The number of neighbors of unknown i is stored in
!                         istoreleft(i+1)-istoreleft(i)
!                         The second part contains the neighbors
!     istoreright    o    Contains all periodical left neighbor unknowns
!                         for all right periodical neighbor unknowns
!                         Pos. 1 to nusol+1 contain pointers to the second part
!                         istoreright(1) contains nusol+1
!                         The number of neighbors of unknown i is stored in
!                         istoreright(i+1)-istoreright(i)
!                         The second part contains the neighbors
!     numpoints      o    Number of points in curves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nh

!     nh             number of nodes along a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPER0026      fill nodal points of curves icrv1 to icrv2 in array iwork
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     327   2 curves for connected element have different numbers of nodal
!           points.
!    2369   Left-hand curve is closed but right-hand curve is not
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshconnectcurv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshconnectcurv'
         write(irefwr,1) 'ileft, iright', ileft, iright
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istar1 = 0

!     --- Store nodes along left-hand side into istoreleft

      call seper0026 ( ileft, ileft, 1, numpoints, &
                       istoreleft, .true., kmeshm )
      if ( ierror/=0 ) go to 1000

!     --- Store nodes along right-hand side into istoreright

      call seper0026 ( iright, iright, 1, nh, &
                       istoreright, .true., kmeshm )
      if ( ierror/=0 ) go to 1000

      if ( numpoints/=nh ) then

!     --- Number of nodes along left and right curves not equal

         call errint ( ileft, 1 )
         call errint ( numpoints, 2 )
         call errint ( iright, 3 )
         call errint ( nh, 4 )
         call errint ( ielgrp, 5 )
         call errint ( iseq, 6 )
         call errsub ( 327, 6, 0, 0 )
         go to 1000

      end if

      if ( ishape>=1 ) then

!     --- ishape>0, elements are connected

         numpoints = (numpoints-1)/ishape

      else

!     --- ishape = 0, points are connected

         if ( istoreleft(1)==istoreleft(numpoints) ) then

!        --- left-hand curve is closed

            if (  istoreright(1)/=istoreright(numpoints) ) then

!           --- Right-hand curve is not closed

               call errint ( ileft, 1 )
               call errint ( iright, 2 )
               call errsub ( 2369, 2, 0, 0 )

            end if

            numpoints = numpoints-1

         end if

         if ( iexclu==1 .or. iexclu==2 ) then

!        --- iexclu = 1 or 2, skip initial point or end point

            numpoints = numpoints - 1
            if ( iexclu==1 ) istar1 = istar1+1

         else if ( iexclu==3 ) then

!        --- iexclu = 3 skip initial point and end point

            numpoints = numpoints - 2
            istar1 = istar1+1

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istar1, numpoints', istar1, numpoints
         call prinin ( istoreleft(istar1+1), numpoints, 'istoreleft' )
         call prinin ( istoreright(istar1+1), numpoints, 'istoreright' )
         write(irefwr,*) 'End mshconnectcurv'

      end if  ! ( debug )
      call erclos ( 'mshconnectcurv' )

      end
