      subroutine mshcrtsurf ( holes, jnew, activatm, ipointsurf, iinput, &
                              rinput, lengmx, lenkmeshscp, iseqsurfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.5  date 30-11-2014 Change isurf 23 in 99
!        version  2.4  date 20-06-2013 Extra parameter irefplax
!        version  2.3  date 27-11-2012 Long integers
!
!   copyright (c) 2002-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create surfaces for the mesh generator
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ipointsurf(*), iseqsurfs(*), &
              iinput(*), lengmx, lenkmeshscp
      double precision rinput(*)
      logical holes, jnew, activatm

!     activatm       i    If true kmesh part m must be activated
!     holes         i/o   If true, a hole in the surface has been found
!     iinput         i    integer input array, standard for subroutine MESH
!                         The starting address in MESH is 1
!     ipointsurf     i    Array containing the relative starting addresses
!                         of all surfaces in the part of iinput referring
!                         to surfaces
!     iseqsurfs      i    Contains new sequence of surface numbers
!     jnew           i    indication whether a new mesh must be created
!                         (jnew=true) or an old one must be changed
!                         (coordinates only: jnew=false)
!     lengmx        i/o   Maximum length of local array kmeshc
!     lenkmeshscp    i    Length of kmeshs part 3
!     rinput         i    real input array, standard for subroutine MESH
!                         The starting address in MESH is 1
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxholes
      parameter ( maxholes=100 )
      integer :: jsurnr, lengc, iposi, iposk, irefplax, &
              holeinfo(0:2*(maxholes+2)+1), kmeshscp(0:lenkmeshscp+1), ih

!     holeinfo       Integer array of size 2 x (nholes+2)
!                    Contains information about holes in surfaces and where
!                    a new part of the boundary starts
!                    Pos (1,i-1) contains the local sequence number of the
!                       first curve of part i provided with a sign
!                       If the sign is positive the boundary is
!                       created counter clockwise, otherwise clockwise
!                    Pos (2,i-1) contains the local sequence number of the
!                       first node of of part i
!                    Pos (1,nholes+1) contains the number of curves of the
!                    boundary
!     ih             Help variable to store a constant temporarily
!     iposi          next free position of iinput
!     iposk          Last position in array IINPUT before the information
!                    of the present surface
!     irefplax       File reference number for plaxis
!     jsurnr         Surface sequence number
!     kmeshscp       Copy of kmeshs part 3
!                    The curves may be reversed so that all boundary
!                    parts have curves pointing in the same direction
!     lengc          computed length of kmesh part c
!     maxholes       Maximum number of holes found in a boundary of a
!                    surface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCONTROL     Check control integers around integer array
!     MSH012         write surface isurf to file 3; fill positions of kmesh
!                    part h
!     MSH055         Sort outer boundary of surfaces of surfaces
!     MSH064         initialize some parameters for mshsrf
!     MSHFILCOMPSURF Fill information with respect to composite surfaces
!     MSHKMESHRDIR   Find the direction of the boundary of the elements of each
!                    subsurface of a composite surface
!     MSHSRF         Generate one surface
!     PR0012         Print accumulated array
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcrtsurf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcrtsurf'
         write(irefwr,1) 'lenkmeshscp', lenkmeshscp
  1      format ( a, 1x, (10i6) )

      end if

!     --- Set control integers

      holeinfo(0) = 1001
      holeinfo(2*(maxholes+2)+1) = 1001
      kmeshscp(0) = 1002
      kmeshscp(lenkmeshscp+1) = 1002

!     --- Sort outer boundary of subsurfaces of surfaces in order to get
!         a complete continuous outer boundary

      lnwrk2 = 0
      npomxs = 0
      nelmax = 0
      lengmx = 0
      if ( debug ) &
         write(irefwr,1) 'ndim, ncurvs, nsurfs', ndim, ncurvs, nsurfs

      if ( jnew ) then

!     --- Mesh is created as a new mesh
!         Sort outer boundary of surfaces of surfaces in order to get a
!         complete continuous outer boundary

         call msh055 ( holes, kmeshs(nsurfs+2) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Copy kmeshs part 3 into kmeshscp

      kmeshscp(1:lenkmeshscp) = kmeshs(2+nsurfs*9:1+nsurfs*9+lenkmeshscp)
      if ( debug ) call prinin ( kmeshscp, lenkmeshscp, 'kmeshscp/1' )

      do jsurnr = 1, nsurfs

!     --- (Over) estimate number of points and elements to be created
!         Compute the length of all arrays to be used in MSHSRF

         isurnr = iseqsurfs(jsurnr)
         if ( debug ) write(irefwr,1) 'jsurnr, isurnr', jsurnr, isurnr
         iposi = ipointsurf(isurnr)+iinp3-1
         iposk = iposi

         if ( ierror/=0 ) go to 1000
         call msh064 ( iinput, iposk, maxholes, irefplax )
         if ( ierror/=0 ) go to 1000
         if ( debug ) write(irefwr,1) 'isurf', isurf

         if ( isurf==7 .or. isurf==11 .or. isurf==12 ) then

!        --- isurf = 7 or 11   composite surface of subsurfaces

            if ( .not. jnew ) goto 300

            if ( isurf==7 ) then

!           --- Provide subsurfaces of composite surface with a minus sign if
!               the subsurface has an orientation different from the standard
!               one

               if ( debug ) write(irefwr,*) 'before mshkmeshrdir'
               call mshkmeshrdir ( kmeshr, kmeshs, ncurvs, nsurfs, &
                                   kmeshs(nsurfs+2), isurnr, kmeshscp(1) )
               if ( ierror/=0 ) go to 1000

            end if

            if ( debug ) write(irefwr,*) 'before mshfilcompsurf'
            call mshfilcompsurf ( activatm, iinput, iposi, &
                                  rinput, jnew )
            if ( debug ) write(irefwr,*) 'after mshfilcompsurf'
            if ( ierror/=0 ) go to 1000

         else

!        --- isurf # 7 or 11  Normal surface
!            Activate arrays with respect to previous surfaces
!            This subroutine uses memory management subroutines

            ih = ibcrds
            if ( ndim==3 ) ih = ih-1

            if ( debug ) then
               write(irefwr,1 ) 'ibcrds, ih, lnbcor, inpelm, nelemlc', &
                                 ibcrds, ih, lnbcor, inpelm, nelemlc
               write(irefwr,1 ) 'ndim, npointlc', ndim, npointlc
               write(irefwr,1 ) 'lnrenu, lnkbnd, nextracrvs', &
                                 lnrenu, lnkbnd, nextracrvs
            end if  ! ( debug )
            allocate ( bcord2d(ibcrds,lnbcor), kmeshclc(inpelm*nelemlc),&
                       coor2d(ndim,npointlc), coor3d(ndim,npointlc), &
                       irenum(lnrenu), kbndptlc(lnkbnd+1+nextracrvs), &
                       stat = error )
            if ( error/=0 ) &
               call eralloc ( error, ibcrds*lnbcor+inpelm*nelemlc+ &
                     ndim*npointlc+lnkbnd+nextracrvs, 'bcord2d' )
            if ( ierror/=0 ) go to 1000

            if ( debug ) write(irefwr,*) 'after allocate'

!           --- array kbndpt is extended with nextracrvs positions to
!               store array numnodextcurvs

!           --- Create surface
!               Although ibuffr and buffer are used no memory management
!               routines are called except seppointer
!               Set control integers at the end of iwork and work

            call mshsrf ( iinput, rinput, iposi, jnew, &
                          lengc, holeinfo(1), &
                          maxholes, kmeshscp(1), irefplax )
            if ( ierror/=0 ) go to 1000

            lengmx = max(lengc, lengmx)
            npomxs = max(npomxs, npointlc)
            nelmax = max( nelmax, nelemlc )
            if ( isurf==0 .or. isurf==20 ) go to 300

!           --- Make some local arrays permanent and store in kmeshh

            call msh012 ( jnew )
            deallocate ( bcord2d, kmeshclc, coor2d, coor3d, irenum, &
                         kbndptlc, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'bcord2d' )
            if ( ierror/=0 ) go to 1000

         end if

300   end do

      if ( debug ) then

!     --- debug statements

         if ( kelmr/=0 ) then

!        --- There are subsurfaces

            write(irefwr,*) 'Subsurface numbers of surfaces'
            call pr0012 ( nsurfs, kmeshr, kmeshr )

         end if

      end if

!     --- Check control integers

      call errcontrol ( holeinfo(0), holeinfo(2*(maxholes+2)+1), 1001, &
                        'holeinfo' )
      call errcontrol ( kmeshscp(0), kmeshscp(lenkmeshscp+1), 1002, &
                        'kmeshscp' )

1000  if ( debug ) write(irefwr,*) 'End of mshcrtsurf'
      call erclos ( 'mshcrtsurf' )
      end subroutine mshcrtsurf
