      subroutine mshedgnodes ( iwork, npoint, nmeshc, nedges, &
                               iedgenodes, length, iworkedge )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-05-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iedgenodes with all boundary edges corresponding to the
!     boundary nodes
!     Mark boundary nodes in array iwork
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     volume
!     pipe
!     tetrahedron
!     boundary
!     triangle
!     edge
!     node
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), npoint, nmeshc(*), nedges, &
              iedgenodes(*), iworkedge(nedges), length

!     iedgenodes     o    Work array in which all the boundary edges
!                         corresponding to the boundary nodes are stored
!                         in the following sequence
!                         Pos. 1 to npoint: starting positions of all nodes i
!                         in iedgenodes
!                         iedgenode(1) = npoint+1
!                         Number of edges coupled to node i:
!                         iedgenode(i+1)-iedgenode(i)
!                         edges coupled to node i are stored in iedgenode
!                         from position iedgenode(i)+1
!     iwork          o    integer work array of length npoint
!                         iwork(i) = 0, means point i is an internal point
!                         iwork(i) = 1, means point i is on the boundary
!     iworkedge      o    work array of length nedges, containing information
!                         whether an edge must be subdivided into two parts (>0)
!                         or not (0)
!                         if iworkedge(iedge)>0, the value refers to the node
!                         number of the new node on the edge
!     length        i/o   Help parameter to indicate the length of arrays
!                         At input declared length of array iedgenodes
!                         At output computed length of array iedgenodes
!     nedges         i    number of edges of volume
!     nmeshc         i    nmesh part c of volume
!     npoint         i    Number of nodal points of volume
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iedge, istart, nodp1, nodp2, &
              iworknode(npoint), icount

!     i              Counting variable
!     icount         Counter
!     iedge          edge sequence number
!     istart         First position used at a part
!     iworknode      Work array of length npoint
!                    Is used to store the number of boundary edges connected
!                    to the boundary nodes
!     nodp1          Node sequence number
!     nodp2          Node sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Mark all boundary edges and store number of boundary edges corresponding
!     to nodes
!     Fill iedgenodes
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshedgnodes' )

!     --- Mark all boundary edges and store number of boundary edges
!         corresponding to nodes

      do i = 1, nedges
         iworkedge(i) = 0
      end do

      do i = 1, npoint
         iworknode(i) = 0
      end do

      do iedge = 1, nedges
         istart = nmeshc(iedge)
         nodp1 = nmeshc(istart+1)
         nodp2 = nmeshc(istart+2)
         if ( iwork(nodp1).gt.0 .and. iwork(nodp2).gt.0 ) then

!        --- Both points on the boundary, hence boundary edge found

            iworkedge(iedge) = 1
            iworknode(nodp1) = iworknode(nodp1)+1
            iworknode(nodp2) = iworknode(nodp2)+1

         end if

      end do

!     --- Fill iedgenodes

      icount = npoint+1
      iedgenodes(1) = icount
      do i = 1, npoint
         icount = icount+iworknode(i)
         iedgenodes(i+1) = icount
      end do

      if ( length.lt.icount ) then

!     --- Array length too short

         call errint ( icount, 1 )
         call errint ( length, 2 )
         call errchr ( 'iedgenodes', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if
      length = icount

      do i = 1, npoint
         iworknode(i) = 0
      end do

      do iedge = 1, nedges
         if ( iworkedge(iedge).eq.1 ) then

!        --- iworkedge(iedge) = 1, hence boundary edge

            istart = nmeshc(iedge)
            nodp1 = nmeshc(istart+1)
            iworknode(nodp1) = iworknode(nodp1)+1
            iedgenodes(iedgenodes(nodp1)+iworknode(nodp1)) = iedge
            nodp2 = nmeshc(istart+2)
            iworknode(nodp2) = iworknode(nodp2)+1
            iedgenodes(iedgenodes(nodp2)+iworknode(nodp2)) = iedge

         end if
      end do

1000  call erclos ( 'mshedgnodes' )
      end
