      subroutine mshfillkmeshs1 ( nsurfs, kmeshn, kmeshs, kmeshsp2, &
                                  curvstosurfs, lencurvstosf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 08-04-2005 Extra parameter lencurvstosf
!        version  1.0  date 26-03-2004
!
!   copyright (c) 2004-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kmesh part s from the information in kmesh part n
!     No curves are used and it is assumed that all input has been read
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nsurfs, kmeshn(*), kmeshs(*), kmeshsp2(8,*), &
              curvstosurfs(nsurfs,*), lencurvstosf

!     curvstosurfs   o    Array of size nsurfs x nsurfs
!                         curvstosurfs(i,j) contains the j-th curve
!                         corresponding to surface i
!                         if 0 no more curves can be found
!     kmeshn         i    KMESH part n, contains nodal point numbers of the
!                         surfaces
!     kmeshs         o    Kmesh part s, contains information about the various
!                         surfaces. See the PG 13.1
!     kmeshsp2       o    Second part of kmeshs, referring to surfaces
!     lencurvstosf   i    Length of second dimension of curvstosurfs
!     nsurfs         i    number of surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isurnr, inpelm, inelem, ishape, istart

!     i              Counting variable
!     inelem         Number of elements in a specific element group
!     inpelm         number of points per element
!     ishape         Shape number of elements
!     istart         First position used at a part
!     isurnr         Surface sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PR0012         Print accumulated array
!     PRININ1        print 2d integer array
!     PRININ2        Print an integer two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2174   Wrong shape for surface elements
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill kmeshs part 1 with nsurfs+1+8*nsurfs
!     For each surface do
!        Get information from kmeshn
!        Fill kmeshs part 2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfillkmeshs1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfillkmeshs1'
         call prinin2 ( curvstosurfs, nsurfs, lencurvstosf, &
                        'curvstosurfs' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Fill kmeshs part 1 with information of curves corresponding to
!         surfaces
!         Fill kmeshs part 3

      istart = nsurfs+1+8*nsurfs
      kmeshs(1) = istart
      do isurnr = 1, nsurfs
         do i = 1, lencurvstosf
            if ( curvstosurfs(isurnr,i)==0 ) go to 100
            istart = istart+1
            kmeshs(istart) = curvstosurfs(isurnr,i)
         end do  ! i = 1, nsurfs
100      kmeshs(isurnr+1) = istart
      end do  ! isurnr = 1, nsurfs

!     --- Clear kmeshs part 2

      do isurnr = 1, nsurfs
         do i = 1, 8
            kmeshsp2(i,isurnr) = 0
         end do  ! i = 1, 8
      end do  ! isurnr = 1, nsurfs

!     --- For each surface do

      do isurnr = 1, nsurfs

!     --- Get information from kmeshn

         inpelm = kmeshn(isurnr)
         inelem = kmeshn(isurnr+nsurfs)

!        --- Compute shape number

         ishape = 0
         select case ( inpelm )

            case(:-1, 1, 2, 8, 11:)

!           --- inpelm<=2, 8 or>=10, no shape available

               call errint ( inpelm, 1 )
               call errsub ( 2174, 1, 0, 0 )
               go to 1000

            case(0)

!           --- inpelm = 0

               ishape = 0
               if ( inelem/=0 ) then

!              --- There are elements with 0 nodes per element, impossible

                  call errint ( inpelm, 1 )
                  call errsub ( 2174, 1, 0, 0 )
                  go to 1000

               end if  ! ( inelem/=0 )

            case(3)

!           --- inpelm = 3

               ishape = 3

            case(4)

!           --- inpelm = 4

               ishape = 5

            case(5)

!           --- inpelm = 5

               ishape = 9

            case(6)

!           --- inpelm = 6

               ishape = 4

            case(7)

!           --- inpelm = 7

               ishape = 7

            case(9)

!           --- inpelm = 9

               ishape = 6

            case(10)

!           --- inpelm = 10

               ishape = 10

         end select  ! case ( inpelm )

!        --- Fill kmeshs part 2

         kmeshsp2(1,isurnr) = 5 ! user mesh
         kmeshsp2(2,isurnr) = ishape
         kmeshsp2(3,isurnr) = inelem
         kmeshsp2(4,isurnr) = 0  ! must be npoint but probably not used

      end do  ! isurnr = 1, nsurfs

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1010)
1010     format ( //' Curves corresponding to surfaces'/ )
         call pr0012 ( nsurfs, kmeshs, kmeshs )
         call prinin1 ( kmeshs(nsurfs+2),nsurfs, 8, &
                        'Rest of information per surface' )
         write(irefwr,*) 'End mshfillkmeshs1'

      end if
      call erclos ( 'mshfillkmeshs1' )

      end
