      subroutine mshfilnumusp ( ncurvs, iseqcurvs, ipointcurv, ndim, &
                                jcoars, iwork, iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.6  date 10-02-2015 Extension with ccircle
!        version  1.5  date 14-05-2014 Change of error message
!        version  1.4  date 15-08-2009 Extension with ellips
!
!   copyright (c) 2002-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iwork with number of user points per curve
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     curve
!     user_point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncurvs, iinput(*), iwork(*), ndim, jcoars, &
              iseqcurvs(*), ipointcurv(*)

!     iinput         i    integer input array containing information of the
!                         curves
!     ipointcurv     i    Array containing the relative starting addresses
!                         of all curves in the part of iinput referring
!                         to curves
!     iseqcurvs      i    Contains new sequence of curve numbers
!     iwork          o    work array to be filled in the following way:
!                         iwork(1)=ncurvs+1
!                         number of user points in curve i: iwork(i+1)-iwork(i)
!                         user points in curve i are stored in iwork from
!                         position iwork(i)+1 to iwork(i+1)
!     jcoars         i    If 1 coarseness is given, if 0 not
!     ncurvs         i    Number of curves in mesh
!     ndim           i    Dimension of the space.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iinp, icurnr, nuserp, iprec, nprec, ic, &
              icurve, jcurve, kcurve, ncrvs, nj

!     i              Counting variable
!     ic             Relative subcurve number
!     icurnr         Curve sequence number
!     icurve         Type of curve
!     iinp           last position used in array iinput
!     iprec          Surface number of surface to be copied etc.
!     jcurve         Curve number
!     kcurve         abs(jcurve)
!     ncrvs          Number of curves
!     nj             Help variable to store number of nodes
!     nprec          Number of user points in previous curve
!     nuserp         Number of user points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     124   code for generation of curves (icurve) incorrect
!    1481   More user points on curve to be copied than on preceding curve
!    1616   No coarseness given in relation with carc, cline, cparam or cspline
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfilnumusp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfilnumusp'
         write(irefwr,1) 'ndim', ndim
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, ncurvs
         icurnr = iseqcurvs(i)
         iinp = ipointcurv(icurnr)
         icurve = abs(iinput(iinp))
         nuserp = 0
         if ( debug ) &
            write(irefwr,1) 'i, icurnr, icurve, iinp', &
                             i, icurnr, icurve, iinp

         if ( icurve==0 ) go to 400
         if ( icurve>=10000 .and. icurve<=10003 ) then
            nuserp = 2
            goto 400
         end if

         if ( jcoars==0 .and. (icurve==5 .or. icurve==6 .or. &
                               icurve==9 .or. icurve==12 .or. &
                               icurve==23 .or. icurve==24) ) then
            call errsub ( 1616, 0, 0, 0 )
         end if

         select case ( icurve )

            case(1, 3, 6, 8, 9, 15, 16, 18:20, 22, 23)

!           --- icurve = 1, 3, 5, 6, 8, 9, 15, 16, 18, 19, 20, 22, 23

               nuserp = 2

            case(2)

!           --- icurve = 2

               nuserp = iinput(iinp+2)+1

            case(5)

!           --- icurve = 5

               nuserp = iinput(iinp+3)

            case(4, 12, 21)

!           --- icurve = 4, 12: splines
!               icurve = 21  framecurve

               nuserp = iinput(iinp+5)

            case(7, 14)

!           --- icurve = 7, 14   curve of curves

               if ( icurve==7 ) then
                  ncrvs = iinput(iinp+3)
               else
                  ncrvs = iinput(iinp+6)
               end if
               nuserp = 0
               do ic = 1, ncrvs
                  if ( icurve==7 ) then
                     jcurve = iinput(iinp+ic+3)
                  else
                     jcurve = iinput(iinp+ic+6)
                  end if
                  kcurve = abs(jcurve)
                  nj = iwork(kcurve)
                  nuserp = nuserp+nj-1

               end do
               nuserp = nuserp+1

            case(10, 11, 13)

!           --- icurve = 10, 11 or 13 : translate, rotate or reflect

               nuserp = iinput(iinp+3)
               iprec  = iinput(iinp+4)

!              --- Determine type of preceding curve

               nprec  = iwork(iprec)
               if ( debug ) &
                  write(irefwr,1) 'nuserp, iprec, nprec', &
                                   nuserp, iprec, nprec
               if ( icurve<=11 .and. nuserp>nprec ) then

!              --- icurve = 10 or 11 and nuserp>nprec

                  call errint ( icurnr, 1 )
                  call errint ( nuserp, 2 )
                  call errint ( iprec, 3 )
                  call errint ( nprec, 4 )
                  call errwar ( 1481, 4, 0, 0 )

               else if ( nuserp-ndim>nprec ) then

!              --- icurve = 13 and n-ndim>nprec

                  call errint ( icurnr, 1 )
                  call errint ( nuserp-ndim, 2 )
                  call errint ( iprec, 3 )
                  call errint ( nprec, 4 )
                  call errwar ( 1481, 4, 0, 0 )

               end if

            case(17)

!           --- icurve = 17: circle

               nuserp = 1

            case(24)

!           --- icurve = 24: ccircle

               nuserp = iinput(iinp+3)-1

            case default

!           --- wrong curve type

               call errint ( icurve, 1 )
               call errint ( 23, 2 )
               call errsub ( 124, 2, 0, 0 )

         end select  ! case ( icurve )

400      iwork(icurnr) = nuserp

      end do

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iwork, ncurvs, 'iwork' )
         write(irefwr,*) 'End mshfilnumusp'

      end if  ! ( debug )
      call erclos ( 'mshfilnumusp' )

      end
