      subroutine mshobstms3d01 ( tetnode, tetnodenew, coornew, &
                                 nextranodes, nelemnew, edgenode, &
                                 inpelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 19-05-2005 New calls to subroutines
!        version  2.1  date 07-04-2004 Extension of edgenode (RvL)
!        version  2.0  date 27-09-2002 Extra parameter inpelm
!        version  1.0  date 21-01-2002
!
!   copyright (c) 2002-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Subdivide each tetrahedron into the right number of subtetrahedrons
!    Check volumes of subtetrahedrons if necessary
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     intersection
!     linear
!     tetrahedron
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduledebug
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer edgenode(3,6), inpelm, tetnode(inpelm), tetnodenew(*), &
              nextranodes, nelemnew
      double precision coornew(3,*)

!     coornew       i/o   Array containing the new coordinates of the nodes
!     edgenode       i    Array with all nodes on the crossed edge
!                         edgenode(1,1) first node on crossed edge
!                         edgenode(2,1) last node on crossed edge
!                         edgenode(3,1) new node on crossed edge
!     inpelm         i    number of points per element
!     nelemnew       o    Number of elements in new mesh
!     nextranodes    i    Number of extra nodes in element
!     tetnode        i    Array of nodal point numbers of the corners of
!                         the original tetrahedron.
!     tetnodenew     o    Array of nodal point numbers of the corners of
!                         the new tetrahedrons.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, count(4), isort(4), isorttemp, ledgenode(2,6)
      double precision tetvolold, tetvolnew(1:8), tetvolnewtot

!     count          count(i) is the number of crossed edges on which
!                    tetnode(i) is located
!     i              Counting variable
!     isort          if isort(i)>isort(j) then tetnode(i) is at least on
!                    the same number of crossed edges as tetnode(j)
!     isorttemp      temporary storage for an isort element
!     j              Counting variable
!     k              Counting variable
!     ledgenode      for i=1,2 and 1<j< number of crossed edges
!                    ledgenode(i,j) is the local tetnode of edgenode(i,j), &
!                    in other words:
!                    edgnode(i,j) = tetnode(ledgenode(i,j))
!     tetvolnew      array of volumes of the new tetrahedrons
!     tetvolnewtot   total volume of new tetrahedrons
!     tetvolold      volume of original tetrahedron
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision tetvolume

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHOBSTMS3D04  Subdivide tetrahedron with only one crossed edge into
!                    2 tetrahedrons
!     MSHOBSTMS3D05A Subdivide tetrahedron, with only two crossed edges which
!                    share a node, into three tetrahedrons
!     MSHOBSTMS3D05B Subdivide tetrahedron with only two crossed edges which do
!                    not share a node into four tetrahedrons
!     MSHOBSTMS3D06A Subdivide a tetrahedron with three crossed edges which
!                    have one single node in common into four tetrahedrons
!     MSHOBSTMS3D06B Subdivide tetrahedron with three crossed edges, all in one
!                    plane, into four tetrahedrons
!     MSHOBSTMS3D06C Subdivide a tetrahedron, with three crossed edges which
!                    form a curve that is not closed, into four or five
!                    tetrahedrons
!     MSHOBSTMS3D07A Subdivide a tetrahedron with four crossed edges, of which
!                    3 edges share a single corner node
!     MSHOBSTMS3D07B Subdivide a tetrahedron with four crossed edges which form
!                    a closed curve
!     MSHOBSTMS3D08  Subdivide a tetrahedron with five crossed edges into seven
!                    tetrahedrons
!     MSHOBSTMS3D09  Subdivide a tetrahedron with four crossed edges into eight
!                    tetrahedrons
!     TETVOLUME      Compute the volume of a tetrahedron
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2268   The volume of the original tetraeder is not equal to the sum of
!           the volumes of the new tetraeder
!    2269   Old volume has different orientation than new volume
!    2270   integer incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( nextranodes/=0 ) then
!        count(i) := number of intersected edges which contain tetnode(i)
!        ledgenode(1:2,j) are the local numbers of the corner nodes which lay
!                        on the same edge as intersection node j.
!        isort := the sort index of count, in other words
!                 if ( isort(i)>isort(j) then count(i)>count(j)
!     end if
!     subdivide tetrahedron into the right number of subtetrahedrons, &
!     mark each subtetrahedron as inside or outside
!     if ( checksubtetrahedrons) then
!        check volumes of subtetrahedrons
!     end if
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstms3d01' )

!     --- Determine some topological properties of the new nodes

      if ( nextranodes/=0 ) then

!     --- There are extra nodes

         do i = 1,4
            count(i) = 0
         end do
         do j = 1,nextranodes
            k = 1
            do i = 1, 4
               if ( edgenode(1,j)==tetnode(i) .or. &
                    edgenode(2,j)==tetnode(i) ) then
                  count(i) = count(i)+1
                  ledgenode(k,j) = i
                  k = k+1
               end if
            end do
         end do

!        --- sort count(i)

         do i = 1, 4
            isort(i) = i
         end do
         if ( count(1)>count(2)) then
            isort(1) = 2
            isort(2) = 1
         end if
         if ( count(3)>count(4)) then
            isort(3) = 4
            isort(4) = 3
         end if
         if ( count(isort(1))>count(isort(3)) ) then
            isorttemp = isort(1)
            isort(1) = isort(3)
            isort(3) = isorttemp
         end if
         if ( count(isort(2))>count(isort(4))) then
            isorttemp = isort(2)
            isort(2) = isort(4)
            isort(4) = isorttemp
         end if
         if ( count(isort(2))>count(isort(3))) then
            isorttemp = isort(2)
            isort(2) = isort(3)
            isort(3) = isorttemp
         end if

      end if

      if ( nextranodes==0 ) then

!     --- subtetrahedron = original tetrahedron

         do i = 1, inpelm
            tetnodenew(i) = tetnode(i)
         end do
         nelemnew = 1

      else if ( nextranodes==1 ) then

!     --- nextranodes = 1

         call mshobstms3d04 ( tetnode, tetnodenew, edgenode, &
                              isort, nelemnew )

      else if ( nextranodes==2 ) then

!     --- nextranodes = 2

         if ( count(isort(4))==2 ) then

!        --- the two cut edges share local node isort(4)

            call mshobstms3d05a ( tetnode, tetnodenew, edgenode, &
                                  isort, ledgenode, nelemnew )
         else

!        --- the two cut edges do not share a node

            call mshobstms3d05b ( tetnode, tetnodenew, edgenode, &
                                  ledgenode, nelemnew, 3 )

         end if

      else if ( nextranodes==3 ) then

!     --- nextranodes = 3

         if ( count(isort(4))==3 ) then

!        --- all 3 cut edges share local node isort(4)

            call mshobstms3d06a ( tetnode, tetnodenew, edgenode, &
                                  isort, ledgenode, nelemnew )

         else if ( count(isort(1))==0 ) then

!        --- node isort(1) is not on any of the 3 cut edges

            call mshobstms3d06b ( tetnode, tetnodenew, edgenode, &
                                  isort, ledgenode, nelemnew, 3 )

         else

!        --- the cut edges form a curve
!            edge [isort(3),isort(4)] is the middle segment of the line

            call mshobstms3d06c ( tetnode, tetnodenew, edgenode, &
                                  isort, ledgenode, nelemnew,3  )
         end if

      else if ( nextranodes==4 ) then

!     --- nextranodes = 4

         if ( count(isort(4))==3 ) then

!        --- 3 of the 4 cut edges share local node isort(4)

            call mshobstms3d07a ( tetnode, tetnodenew, edgenode, &
                                  isort, ledgenode, nelemnew, 3 )
         else

!        --- the 4 cut edges form an closed curve

            call mshobstms3d07b ( tetnode, tetnodenew, edgenode, &
                                  ledgenode, nelemnew )
         end if

      else if ( nextranodes==5 ) then

!     --- nextranodes = 5

         call mshobstms3d08 ( tetnode, tetnodenew, edgenode, &
                              isort, ledgenode, nelemnew, 3 )

      else if ( nextranodes==6 ) then

!     --- nextranodes = 6

         call mshobstms3d09 ( tetnode, tetnodenew, edgenode, &
                              ledgenode, nelemnew, 3 )

      else

!     --- nextranodes has wrong value

         call errint ( nextranodes, 1 )
         call errchr ( 'nextranodes', 1 )
         call errsub ( 2270, 1, 0, 1 )

      end if

      if ( checksubtetrahedrons ) then

!     --- check volumes of tetrahedrons

         tetvolold = tetvolume(coornew(1,tetnode(1)), &
                               coornew(1,tetnode(2)), &
                               coornew(1,tetnode(3)), &
                               coornew(1,tetnode(4)) )

         tetvolnewtot = 0.
         do j = 1, nelemnew
            tetvolnew(j) = tetvolume(coornew(1,tetnodenew(1+(j-1)*4)), &
                                     coornew(1,tetnodenew(2+(j-1)*4)), &
                                     coornew(1,tetnodenew(3+(j-1)*4)), &
                                     coornew(1,tetnodenew(4+(j-1)*4)) )

!           --- Test orientation of new tetraeder

            if ( tetvolold*tetvolnew(j)<=0d0 ) then

!           --- New sub tetraeder has different orientation than old one

               call errint ( j, 1 )
               call erreal ( tetvolold, 1 )
               call erreal ( tetvolnew(j), 2 )
               call errsub ( 2269, 1, 2, 0 )

            end if
            tetvolnewtot = tetvolnewtot+ tetvolnew(j)
         end do

!        --- test

         if ( abs((tetvolold-tetvolnewtot)/tetvolold)>1d-12 ) then

!        --- Old volume not equal to sum of new volumes

            call erreal ( tetvolold, 1 )
            call erreal ( tetvolnewtot, 2 )
            call errsub ( 2268, 0, 2, 0 )
         end if

      end if

      call erclos ( 'mshobstms3d01' )
      end
