      subroutine mshouterboun2d
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 27-11-2012 Long integers
!        version  1.4  date 08-12-2011 Extra debug
!        version  1.3  date 25-10-2004 Debug statements
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the outer boundary curves of a region in R^2 and sort this
!     boundary such that it is always counterclockwise
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     boundary
!     outer_boundary
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenoutercurvs, lenkmeshm, nparts

!     lenkmeshm      Length of array kmeshm
!     lenoutercurvs  Length of array outerboun
!     nparts         number of parts the boundary consists of
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOUTERFIND2D Find the outer boundary of a two-dimensional region
!     MSHOUTERORD2D  Order array outerboun containing the outer curves of the
!                    region such that all curves are ordered counter clockwise
!     PR0008         Print information of curves
!     PRINIOUTERCS   Print array outerboun or ioutersurfs,
!                    depending on the dimension
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute all the outer curves of the region and sort them to closed
!     boundaries
!     Make the direction of these boundaries such that they are always
!     counter clockwise
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshouterboun2d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshouterboun2d'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute all the outer curves of the region and sort them to closed
!         boundaries
!         Reserve temporary space

      lenoutercurvs = max(20,4*ncurvs)

      allocate ( outerboun(lenoutercurvs), stat = error )
      if ( error/=0 ) call eralloc ( error, lenoutercurvs, 'outerboun' )
      if ( ierror/=0 ) go to 1000

!     --- Compute starting addresses

      ninner = kmeshm(1)
      lenkmeshm = size ( kmeshm )-ncurvs-5-ninner
      if ( debug ) call pr0008 ( kmeshm, ncurvs )

!     --- Actual computation

      call mshouterfind2d ( ncurvs, kmeshm, outerboun, lenoutercurvs )

      if ( lenoutercurvs>0 ) then

!     --- Make array outerboun permanent

         if ( debug ) call prinioutercs

         allocate ( km(iseqkm)%outerboun(lenoutercurvs) )
         km(iseqkm)%outerboun = outerboun(1:lenoutercurvs)

         deallocate ( outerboun, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'outerboun' )

         outerboun => km(iseqkm)%outerboun
         mmouter = lenoutercurvs

!        --- Make the direction of these boundaries such that they are always
!            counter clockwise

         nparts = outerboun(1)
         if ( debug ) write(irefwr,1) 'nparts', nparts
         call mshouterord2d ( nparts, lenkmeshm )

      else

!        --- Not possible to create outer boundary

         if ( mmouter>0 ) deallocate(km(iseqkm)%outerboun)

         deallocate ( outerboun, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'outerboun' )
         mmouter = 0

      end if  ! ( mmouter>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshouterboun2d'

      end if
      call erclos ( 'mshouterboun2d' )

      end subroutine mshouterboun2d
