      subroutine mshouterfind01 ( nsurfs, ioutersurfs, surface_curvs, &
                                  kmeshm, kmeshn, curve_surfs, kmeshs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 27-11-2011 Fortran 90
!        version  1.4  date 21-09-2007 Correction empty nodes
!        version  1.3  date 25-02-2004 Debug statements
!
!   copyright (c) 2002-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Provide all surfaces in array ioutersurfs with a sign in order
!      to have all outer surfaces directed in the same way
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     boundary
!     outer_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nsurfs, ioutersurfs(*), surface_curvs(*), kmeshm(*), &
              kmeshn(*), curve_surfs(*), kmeshs(8,*)

!     curve_surfs    i    Contains all outer surfaces to the "single" curves
!                         provided the curve contains exactly 2 outer surfaces
!                         Pos. 1 to ncurvs: starting addresses in part 2
!                         Contents of each curve in part 2
!                         Pos. 1 number of outer surfaces coupled to curve
!                         Followed by surface numbers
!     ioutersurfs   i/o   Contains information about the surfaces in the outer
!                         boundary (3D) in the following way
!                         Pos. 1  nparts  Number of parts the outer boundary
!                                         consists of
!                         Pos. 2-1+nparts Starting addresses of each part
!                         Per part: (first address: ioutersurfs(1+i)
!                         Pos. 1: nouter  Number of surfaces in part
!                         followed by the surface numbers (provided with a sign)
!                         The part itself is closed
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     kmeshn         i    KMESH part n, contains nodal point numbers of the
!                         surfaces
!     kmeshs         i    Kmesh part s, contains information about the various
!                         surfaces. See the PG 13.1
!                         In this case it concerns only part 2 containing
!                         shape numbers and so on of the surfaces
!     nsurfs         i    number of surfaces
!     surface_curvs  i    Contains the number of "single" curves coupled
!                         to all outer surfaces in the following way
!                         Pos. 1 to nsurfs: starting addresses for second part
!                         For each surface in second part:
!                         Pos. 1: number of single curves
!                         Followed by curve numbers (all positive)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iwork(nsurfs), i, ipart, npart, nleft, nouter, isurnr1, &
              istart, ipstart, nouterc, icurnr, istar3, k1, node1, &
              node2, isurnr2, numsrf, isign1, isign2, j, jstart, &
              ielem, inelem, inpelm, nfound, inode, icount, ilast, &
              ishape, ifirst, k, k2

!     i              Counting variable
!     icount         Counter
!     icurnr         Curve sequence number
!     ielem          Element number
!     ifirst         Position of first node found in element
!     ilast          Position of last node on the boundary of an element
!     inelem         Number of elements in a specific element group
!     inode          Node sequence number
!     inpelm         number of points per element
!     ipart          Sequence number of part
!     ipstart        Starting address in array
!     ishape         Shape number of elements
!     isign1         sign of first surface
!     isign2         sign of second surface
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         First position used at a part
!     isurnr1        Surface sequence number
!     isurnr2        Surface sequence number
!     iwork          integer work array of length nsurfs
!                    iwork(i) defines sign of surface i
!     j              Counting variable
!     jstart         starting position
!     k              Counting variable
!     k1             First point of curve in KMESH part m
!     k2             Last point of curve in KMESH part m
!     nfound         number of nodes found
!     nleft          Number of surfaces left
!     node1          Node of face
!     node2          Node of face
!     nouter         Number of outer surfaces
!     nouterc        Number of outer curves
!     npart          Number of particles
!     numsrf         number of  surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2414   No element could be found containing the first two nodes of curve
!    2417   Number of outer surfaces coupled to curve>2
!    2418   Inconsistency in outer boundary
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork ( 1..nsurfs )
!     For ipart := 1 (1) npart do
!        While ( there are still surfaces left in part ) do
!           isurnr1 := first free surface in part
!           For all curves icurnr contected to isurnr1 do
!              node1 := first node in curve; node2 := second node in curve
!              ielem1 := element in isurnr1 containing node1 and node2
!              isign1 := direction node1, node2 in ielem1
!              isurnr2 := second surface
!              ielem2 := element in isurnr2 containing node1 and node2
!              isign2 := direction node1, node2 in ielem2
!              if ( isign1 # isign2 ) then
!                 if ( iwork(isurnr1) = 0 and iwork(isurnr2) = 0 ) then
!                    iwork(isurnr1) = 1; iwork(isurnr2) = 1
!                 else if ( iwork(isurnr1) # 0 and iwork(isurnr2) # 0 ) then
!                    if ( iwork(isurnr1) * iwork(isurnr2) # 1 ) then
!                       error: incorrect signs
!                 else if ( iwork(isurnr1) # 0 ) then
!                    iwork(isurnr2) = iwork(isurnr1)
!                 else ( iwork(isurnr2) # 0 )
!                    iwork(isurnr1) = iwork(isurnr2)
!              else ( isign1 = isign2 ) then
!                 if ( iwork(isurnr1) = 0 and iwork(isurnr2) = 0 ) then
!                    iwork(isurnr1) = 1; iwork(isurnr2) = -1
!                 else if ( iwork(isurnr1) # 0 and iwork(isurnr2) # 0 ) then
!                    if ( iwork(isurnr1) * iwork(isurnr2) # -1 ) then
!                       error: incorrect signs
!                 else if ( iwork(isurnr1) # 0 ) then
!                    iwork(isurnr2) = -iwork(isurnr1)
!                 else ( iwork(isurnr2) # 0 )
!                    iwork(isurnr1) = -iwork(isurnr2)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshouterfind01' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshouterfind02'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork ( 1..nsurfs )

      iwork = 0

      npart = ioutersurfs(1)
      istar3 = kmeshm(2)-1

!     --- For ipart := 1 (1) npart do

      do ipart = 1, npart

         istart = ioutersurfs(ipart+1)
         nouter = ioutersurfs(istart)
         icount = 0
         isurnr1 = ioutersurfs(istart+1)
         iwork(isurnr1) = 1
         nleft = nouter-1

!        --- While ( there are still surfaces left in part ) do

         do while ( nleft>0 .and. icount<nouter )

!        --- isurnr1 := first free surface in part

            icount = icount+1
            do i = icount, nouter
               isurnr1 = ioutersurfs(istart+i)
               if ( iwork(isurnr1)/=0 ) go to 100
            end do

!           --- For all curves icurnr connected to isurnr1 do

100         ipstart = surface_curvs(isurnr1)
            nouterc = surface_curvs(ipstart)
            do i = 1, nouterc

               icurnr = surface_curvs(ipstart+i)

!              --- isurnr2 := second surface

               jstart = curve_surfs(icurnr)
               numsrf = curve_surfs(jstart)
               if ( numsrf==2 ) then

!              --- numsrf = 2, find isurnr2

                  isurnr2 = curve_surfs(jstart+1)
                  if ( isurnr1==isurnr2 ) &
                     isurnr2 = curve_surfs(jstart+2)

                  if ( debug ) then

!                 --- debug statements

                     write(irefwr,*) 'ipart, isurnr1, isurnr2, icurnr ', &
                                      ipart, isurnr1, isurnr2, icurnr

                  end if

!                 --- node1 := first node in curve; node2 := second node in
!                     curve

                  k1 = kmeshm(4+icurnr)
                  k2 = kmeshm(5+icurnr)-1
                  do k = k1, k2-1
                     node1 = kmeshm(istar3+k)   ! k-th node
                     node2 = kmeshm(istar3+k+1) ! next node

!                    --- If both nodes are positive 2 nodes have been found

                     if ( node1>0 .and. node2>0 ) go to 110

                  end do  ! k = k1, k2

!                 --- If the program arrives here, no consecutive nodes on the
!                     curve can be found
!                     We have to skip this curve

                  go to 500

110               if ( debug ) then

!                 --- debug statements

                     write(irefwr,*) ' node1, node2 ', node1, node2

                  end if

!                 --- ielem1 := element in isurnr1 containing node1 and node2
!                     isign1 := direction node1, node2 in ielem1

                  jstart = 2*nsurfs
                  do j = 1, isurnr1-1
                     jstart = jstart+kmeshn(j)*kmeshn(j+nsurfs)
                  end do
                  inpelm = kmeshn(isurnr1)
                  inelem = kmeshn(isurnr1+nsurfs)
                  ishape = kmeshs(2,isurnr1)
                  if ( ishape>=6 ) then

!                 --- element contains one internal point, which is the last one

                     ilast = inpelm-1

                  else

!                 --- element contains only boundary points

                     ilast = inpelm

                  end if

                  do ielem = 1, inelem
                     nfound = 0
                     ifirst = 0
                     do j = 1, inpelm
                        inode = kmeshn(jstart+j)
                        if ( inode==node1 .or. inode==node2 ) &
                           nfound = nfound+1
                        if ( ifirst==0 .and. nfound==1 ) ifirst = j
                        if ( nfound==2 )  then

!                       --- Element containg both nodes found

                           if ( inode==node1 ) then

                              if ( ifirst==1 .and. j==ilast ) then

!                             --- node1 is first node, node2 second one
!                                 direction is positive

                                 isign1 = 1

                              else

!                             --- node2 is first node, node1 second one
!                                 direction is negative

                                 isign1 = -1

                              end if

                           else

                              if ( ifirst==1 .and. j==ilast ) then

!                             --- node2 is first node, node1 second one
!                                 direction is negative

                                 isign1 = -1

                              else

!                             --- node1 is first node, node2 second one
!                                 direction is positive

                                 isign1 = 1

                              end if

                           end if  ! ( inode==node1 )
                           go to 150

                        end if  ! ( nfound==2 )

                     end do   ! j = 1, inpelm
                     jstart = jstart+inpelm

                  end do   ! ielem = 1, inelem

!                 --- If the code comes in this line we have an internal error
!                     No element could be found containing node1 and node2

                  call errint ( icurnr, 1 )
                  call errint ( isurnr1, 2 )
                  call errint ( node1, 3 )
                  call errint ( node2, 4 )

!                 --- ielem2 := element in isurnr2 containing node1 and node2
!                     isign2 := direction node1, node2 in ielem2

150               jstart = 2*nsurfs
                  do j = 1, isurnr2-1
                     jstart = jstart+kmeshn(j)*kmeshn(j+nsurfs)
                  end do
                  inpelm = kmeshn(isurnr2)
                  inelem = kmeshn(isurnr2+nsurfs)
                  ishape = kmeshs(2,isurnr2)
                  if ( ishape>=6 ) then

!                 --- element contains one internal point, which is the last one

                     ilast = inpelm-1

                  else

!                 --- element contains only boundary points

                     ilast = inpelm

                  end if

                  do ielem = 1, inelem
                     nfound = 0
                     ifirst = 0
                     do j = 1, inpelm
                        inode = kmeshn(jstart+j)
                        if ( inode==node1 .or. inode==node2 ) &
                           nfound = nfound+1
                        if ( ifirst==0 .and. nfound==1 ) ifirst = j
                        if ( nfound==2 )  then

!                       --- Element containg both nodes found

                           if ( inode==node1 ) then

                              if ( ifirst==1 .and. j==ilast ) then

!                             --- node1 is first node, node2 second one
!                                 direction is positive

                                 isign2 = 1

                              else

!                             --- node2 is first node, node1 second one
!                                 direction is negative

                                 isign2 = -1

                              end if

                           else

                              if ( ifirst==1 .and. j==ilast ) then

!                             --- node2 is first node, node1 second one
!                                 direction is negative

                                 isign2 = -1

                              else

!                             --- node1 is first node, node2 second one
!                                 direction is positive

                                 isign2 = 1

                              end if

                           end if  ! ( inode==node1 )
                           go to 200

                        end if  ! ( nfound==2 )

                     end do   ! j = 1, inpelm
                     jstart = jstart+inpelm

                  end do   ! ielem = 1, inelem

!                 --- If the code comes in this line we have an internal error
!                     No element could be found containing node1 and node2

                  call errint ( icurnr, 1 )
                  call errint ( isurnr2, 2 )
                  call errint ( node1, 3 )
                  call errint ( node2, 4 )
                  call errsub ( 2414, 4, 0, 0 )
                  go to 1000

200               if ( debug ) then

!                 --- debug statements

                     write(irefwr,*) ' isign1, isign2 ', isign1, isign2
                     write(irefwr,*) ' iwork(isurnr1),iwork(isurnr2) ', &
                                       iwork(isurnr1),iwork(isurnr2)

                  end if

                  if ( isign1/=isign2 ) then

!                 --- isign1 # isign2, surfaces are oriented in the same way

                     if ( iwork(isurnr1)/=0 .and. &
                          iwork(isurnr2)/=0 ) then

!                    --- iwork(isurnr1) # 0 and iwork(isurnr2) # 0
!                        Both surfaces have been filled allready

                        if ( iwork(isurnr1)*iwork(isurnr2)/=1 ) then

!                       --- iwork(isurnr1) * iwork(isurnr2) # 1
!                           error: incorrect signs

                           call errint ( ipart, 1 )
                           call errint ( isurnr1, 2 )
                           call errint ( isurnr2, 3 )
                           call errsub ( 2418, 3, 0, 0 )
                           go to 1000

                        end if

                     else if ( iwork(isurnr1)/=0 ) then

!                    --- iwork(isurnr1) # 0 and iwork(isurnr2) = 0

                        iwork(isurnr2) = iwork(isurnr1)
                        nleft = nleft-1

                     else

!                    --- iwork(isurnr2) # 0 and iwork(isurnr1) = 0

                        iwork(isurnr1) = iwork(isurnr2)
                        nleft = nleft-1

                     end if

                  else

!                 --- isign1 = isign2

                     if ( iwork(isurnr1)/=0 .and. &
                          iwork(isurnr2)/=0 ) then

!                    --- iwork(isurnr1) # 0 and iwork(isurnr2) # 0

                        if ( iwork(isurnr1)*iwork(isurnr2)/=-1 ) then

!                       --- iwork(isurnr1) * iwork(isurnr2) # -1
!                           error: incorrect signs

                           call errint ( ipart, 1 )
                           call errint ( isurnr1, 2 )
                           call errint ( isurnr2, 3 )
                           call errsub ( 2418, 3, 0, 0 )
                           go to 1000

                        end if

                     else if ( iwork(isurnr1)/=0 ) then

!                    --- iwork(isurnr1) # 0 and iwork(isurnr2) = 0

                        iwork(isurnr2) = -iwork(isurnr1)
                        nleft = nleft-1

                     else

!                    --- iwork(isurnr2) # 0 and iwork(isurnr1) = 0

                        iwork(isurnr1) = -iwork(isurnr2)
                        nleft = nleft-1

                     end if

                  end if
                  if ( debug ) then

!                 --- debug statements

                     write(irefwr,*) ' iwork(isurnr1),iwork(isurnr2) ', &
                                       iwork(isurnr1),iwork(isurnr2)

                  end if

               else if ( numsrf>2 ) then

!              --- error: numsrf>2, must be a programming error

                  call errint ( icurnr, 1 )
                  call errint ( numsrf, 2 )
                  do j = 1, numsrf
                     call errint ( curve_surfs(jstart+j), 2+j )
                  end do
                  call errwar ( 2417, 2+numsrf, 0, 0 )
                  go to 1000

               end if ! numsrf = 2

500         end do  ! end do  i = 1, nouterc

         end do  ! end do while ( nleft>0 )

      end do  ! end do ipart = 1, npart

!     --- Provide surfaces in ioutersurfs with a sign

      do ipart = 1, npart

         istart = ioutersurfs(ipart+1)
         nouter = ioutersurfs(istart)
         do i = 1, nouter
            isurnr1 = ioutersurfs(istart+i)
            if ( iwork(isurnr1)<0 ) &
               ioutersurfs(istart+i) = -ioutersurfs(istart+i)
         end do

      end do

1000  call erclos ( 'mshouterfind01' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshouterfind02'

      end if

      end
