      subroutine mshparelems4 ( nprocessors, ifirst, ilast, npointproc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 20-03-2015 Use iwork
!        version  1.0  date 19-10-2005 adapted version of mshparelems1
!
!   copyright (c) 2005-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide the region into nprocessors blocks such that all
!     blocks contain connected parts only
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nprocessors, ifirst, ilast, npointproc

!     ifirst        i/o   First node in next block
!     ilast         i/o   Last node in previous block
!     npointproc     i    Number of nodes per processor (approximately)
!     nprocessors    o    Number of processors that is to be used for
!                         parallel processing
!                         Defines also the number of submeshes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iprocessor, numelms, numnodes, nworkprocs
      integer, allocatable :: elementsact(:), pointgrps(:), pointsact(:)
      integer, allocatable :: elements(:), points(:)

!     elements       Array of size nelemloc containing the element
!                    numbers corresponding to the local mesh
!     elementsact    Array of size nelem indicating which elements are
!                    already part of a submesh (>0) or not yet (=0)
!     iprocessor     Processor sequence number
!     numelms        Number of elements in array elements
!     numnodes       Number of nodal points in array points
!     nworkprocs     reserved size of matrix connects
!     pointgrps      for each point contains the number of submesh it
!                    belongs to
!     points         Array of size npointloc containing the node
!                    numbers corresponding to the local mesh
!     pointsact      Array of size npoint indicating which nodes are
!                    already used in a submesh (>0) or not yet (=0)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHMETIS3      Fill array elements with the elements corresponding to one
!                    block and also the corresponding array points
!                    with the point numbers
!                    The elements are subdivided by metis
!     MSHPARELEMS3   Store arrays elements and points into ibuffr and store the
!                    corresponding memory management sequence numbers into
!                    elempointers and nodepointers
!     MSHPARELROWS   Fill array elements with the elements corresponding to one
!                    block and also the corresponding array points with the
!                    point numbers in the case that a layered ordering is chosen
!     MSHPOINTGRPS   Defines submeshes such that each submesh is connected
!     SEPFILLKMESHE  Fill array kmesh part e
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2432   The number of elements in block of the parallel mesh is 0
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparelems4' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparelems4'
         write(irefwr,1) 'nprocessors', nprocessors
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000
      allocate ( elements(nelem), points(npoint) )

!     --- Call subroutine mshpointgrps that will define submeshes
!         First activate arrays and create kmesh e if necessary

      if ( kelme==0 ) call sepfillkmeshe

!     --- Create work space

      nworkprocs = 10*nprocessors

      allocate ( pointgrps(npoint), elementsact(nelem), &
                 pointsact(npoint), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, npoint+nelem, 'iwork' )
      if ( ierror/=0 ) go to 1000
      elementsact = 0
      pointsact = 0

!     --- Create submeshes

      call mshpointgrps ( nprocessors, pointgrps, &
                          npointproc, nworkprocs )

!     --- Loop over the processors

      do iprocessor = 1, nprocessors

!     --- subdivide region in layers of equal size
!         Compute starting addresses

         call mshparelrows ( ifirst, ilast, iprocessor, numelms, &
                             nprocessors, pointsact, elementsact, &
                             npointproc, pointgrps, .true. )
         if ( ierror/=0 ) go to 1000

         if ( numelms==0 ) then

!        --- No elements in block

            call errint ( iprocessor, 1 )
            call errsub ( 2432, 1, 0, 0 )
            go to 1000

         end if

      end do  ! iprocessor = 1, nprocessors

!     --- Loop over the processors in reversed order

      do iprocessor = nprocessors, 1, -1

!     --- subdivide region in layers of equal size

         call mshmetis3 ( iprocessor, nprocessors, elements, points, &
                          numelms, numnodes, elementsact )
         if ( ierror/=0 ) go to 1000

!        --- Copy arrays elements and ipoints in ibuffr

         call mshparelems3 ( elements, iprocessor, numelms, points, &
                             numnodes )
         if ( ierror/=0 ) go to 1000

      end do  ! iprocessor = nprocessors, 1, -1

      deallocate ( pointgrps, elementsact, pointsact, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )
      deallocate ( elements, points )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshparelems4'

      end if
      call erclos ( 'mshparelems4' )

      end
