      subroutine mshparelrows ( ifirst, ilast, iprocessor, numelms, &
                                nprocessors, pointsact, elementsact, &
                                npointproc, pointgrps, usepointgrps )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 20-03-2015 New parameter list
!        version  3.2  date 18-02-2012 Extra debug
!        version  3.1  date 18-12-2007 New definition choice of elements
!
!   copyright (c) 2003-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array elementsact with the elements corresponding to one block
!     In this case the elements are subdivided such that layers of elements
!     arise with approximately the same number of nodes per layer
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     parallel
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ifirst, ilast, iprocessor, nprocessors, &
              pointsact(*), elementsact(*), numelms, &
              npointproc, pointgrps(*)
      logical usepointgrps

!     elementsact   i/o   Array of size nelem indicating which elements are
!                         already part of a submesh (>0) or not yet (=0)
!     ifirst        i/o   First node in next block
!     ilast         i/o   Last node in previous block
!     iprocessor     i    Processor sequence number
!     npointproc     i    Number of nodes per processor (approximately)
!     nprocessors    i    Number of processors that is to be used for
!                         parallel processing
!                         Defines also the number of submeshes
!     numelms        o    Number of elements
!     pointgrps      i    for each point contains the number of submesh it
!                         belongs to
!     pointsact     i/o   Array of size npoint indicating which nodes are
!                         already used in a submesh (>0) or not yet (=0)
!     usepointgrps   i    if true array pointgroups is used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, inpelm, icount, inpelmhalf, &
              ielem, istart, ielgrp, inode, iielem

!     i              Counting variable
!     icount         Counter
!     ielem          Element number
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     inode          Node sequence number
!     inpelm         number of points per element
!     inpelmhalf     inpelm/2
!     istart         First position used at a part
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     ilast := ilast+npointproc
!     if ( iprocessor = nprocessors ) then
!        ilast := npoint
!     if ( nodes are renumbered) then
!        for j := ifirst (1) ilast do
!           pointsact(kmeshj(j)) := 1
!     else ( nodes are not renumbered )
!        for j := ifirst (1) ilast do
!           pointsact(j) := 1
!     numelms := 0
!     For all elements ielem do
!        elementsact(ielem) = 0
!        if ( more than 1/2 of nodes inode in element satisfy
!             pointsact(inode) = 1 ) then
!           elementsact(ielem) := 1
!     ifirst := ilast+1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparelrows' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparelrows'
         write(irefwr,1) 'nprocessors, npoint, npointproc', &
                          nprocessors, npoint, npointproc
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- ilast := ilast+npointproc
!         if ( iprocessor = nprocessors ) then
!            ilast := npoint

      ilast = ilast+npointproc
      if ( iprocessor==nprocessors ) ilast = npoint
      if ( debug ) write(irefwr,*) 'usepointgrps ', usepointgrps

      if ( usepointgrps ) then

!     --- Use array pointgrps that defines the groups of points for each submesh

         do i=1,npoint
            if ( pointgrps(i)==iprocessor ) then
               if ( pointsact(i)==0 ) pointsact(i) = iprocessor
            end if
         end do

      else

         if ( kelmj>0 ) then

!        --- nodes are renumbered
!            for j := ifirst (1) ilast do
!               pointsact(kmeshj(j)) := 1

            do j = ifirst, ilast
               if ( pointsact(kmeshj(j))==0 ) &
                  pointsact(kmeshj(j)) = iprocessor
            end do

         else

!        --- nodes are not renumbered
!            for j := ifirst (1) ilast do
!               pointsact(j) := 1

            do j = ifirst, ilast
               if ( pointsact(j)==0 ) pointsact(j) = iprocessor
            end do

         end if

      end if

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ifirst, ilast', ifirst, ilast
         call prinin ( pointsact, npoint, 'pointsact' )

      end if

!     --- numelms := 0

      numelms = 0

!     --- For all elements ielem do

      ielem  = 0
      istart = 0
      do ielgrp = 1, nelgrp

         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm
         inpelmhalf = inpelm/2  ! 1/2 of number of nodes
         do iielem = 1, inelem
            ielem = ielem + 1
            if ( debug ) then
               write(irefwr,1) 'ielem, elementsact(ielem)', &
                                ielem, elementsact(ielem)
               call prinin (kmeshc(istart+1), inpelm, 'element' )
            end if  ! ( debug )
            if ( elementsact(ielem)==0 ) then

!           --- elementsact(ielem) = 0
!               if ( more than 1/2 of the nodes inode in element
!                    satisfy pointsact(inode) = 1 )
!                  elementsact(ielem) := 1

               icount = 0  ! number of active nodes in element
               do i = 1, inpelm
                  inode = kmeshc(istart+i)
                  if ( pointsact(inode)/=0 ) &
                       icount = icount+1 ! active node
               end do  ! i = 1, inpelm
               if ( debug ) &
                  write(irefwr,1) 'icount, inpelmhalf', &
                                   icount, inpelmhalf

               if ( icount>inpelmhalf ) then

!              --- more than 1/2 of nodes active: i.e. new element found

                  elementsact(ielem) = iprocessor
                  numelms = numelms+1

               end if  ! ( icount>inpelmhalf )

            end if  ! ( elementsact(ielem)==0 )
            istart = istart+inpelm

         end do  ! iielem = 1, inelem

      end do  ! ielgrp = 1, nelgrp

!     --- ifirst := ilast+1

      ifirst = ilast+1

1000  if ( debug ) then
         write(irefwr,*) 'End mshparelrows'
      end if  ! ( debug )
      call erclos ( 'mshparelrows' )

      end
