      subroutine mshparneighbl ( nprocessors, npoint, neighblocks, &
                                 lenneighblock )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 20-03-2015 Extra debug
!        version  1.1  date 27-11-2012 Long integers
!        version  1.0  date 13-03-2003
!
!   copyright (c) 2003-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array neighblocks with information of all blocks coupled to each
!     block in a parallel environment
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     topology
!     neighbor
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nprocessors, npoint, lenneighblock, &
              neighblocks(lenneighblock)

!     lenneighblock i/o   Declared length of array neighblocks
!                         At output: actual length
!     neighblocks    o    Array containing of each block all neighbor blocks
!                         in the following way:
!                         Pos. 1 to nprocesors starting addresses of each block
!                         For each block from start position:
!                         Number of neighbor blocks, followed by
!                         neighbor block sequence numbers
!     npoint         i    Number of nodal points in global mesh
!     nprocessors    i    Number of processors that is to be used for
!                         parallel processing
!                         Defines also the number of submeshes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, iwork(npoint), iblock, blocknr, nneigh
      integer, pointer :: points(:)

!     blocknr        Block sequence number
!     iblock         Block sequence number
!     istart         First position used at a part
!     iwork          integer work array of length npoint
!                    iwork(i) = 0, means that node i is not in blocknr
!                    iwork(i)>0, means that node i is in blocknr
!     nneigh         Number of neighbors of block
!     points         Array of size npointloc containing the node
!                    numbers corresponding to the local mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      logical mshparneighbl1

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHPARINV      Compute the inverse mapping
!                    (i.e. mapping from global to local mesh)
!     MSHPARNEIGHBL1 Returns with value true if there are nodes in array
!                    points with iwork(i)>0
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Reserve space for array neighblocks at the end of ibuffr
!     istart := nprocessors+1
!     For all blocks blocknr do
!        Clear array iwork (1:npoint)
!        Set iwork(i) = i for all points i in blocknr
!        nneigh := 0;  neighblocks(blocknr) := istart
!        For all blocks iblock not equal to blocknr do
!           If ( at least one node j of iblock satisfies iwork(j)>0) then
!              nneigh := nneigh+1
!              neighblocks(istart+nneigh) := iblock
!        neighblocks(istart) := nneigh
!        istart = istart+1+nneigh
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparneighbl' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparneighbl'
         write(irefwr,*) 'nprocessors ', nprocessors

      end if

!     --- istart := nprocessors+1

      istart = nprocessors+1

!     --- For all blocks blocknr do

      do blocknr = 1, nprocessors

!     --- Clear array iwork (1:npoint)
!         Set iwork(i) = i for all points i in blocknr

         points => kmhpar(blocknr)%nodepointers
         nnodes = size (points)  ! number of nodes in points
         call mshparinv ( points, nnodes, iwork, npoint )

         if ( debug ) then

!        --- Debug information

            write(irefwr,*) 'blocknr, nnodes ',blocknr, nnodes
            call prinin ( points, nnodes, 'points' )

         end if

!        --- nneigh := 0;  neighblocks(blocknr) := istart

         nneigh= 0
         neighblocks(blocknr) = istart

!        --- For all blocks iblock not equal to blocknr do

         do iblock = 1, nprocessors

            if ( iblock/=blocknr ) then

!           --- iblock # blocknr
!               If ( at least one node j of iblock satisfies iwork(j)>0) then

               points => kmhpar(iblock)%nodepointers
               nnodes = size(points) ! number of nodes in points

               if ( debug ) then

!              --- Debug information

                  write(irefwr,*) 'iblock, nnodes, istart ', &
                                   iblock, nnodes, istart
                  call prinin ( points, nnodes, 'points' )

               end if

               if ( mshparneighbl1 ( iwork, points, nnodes ) ) &
                  then

!              --- at least one node j of iblock satisfies iwork(j)>0)
!                  nneigh := nneigh+1
!                  neighblocks(istart+nneigh) := iblock

                  nneigh = nneigh+1
                  neighblocks(istart+nneigh) = iblock
                  if ( debug ) &
                    write(irefwr,*) 'iblock, nneigh, istart+nneigh ', &
                                     iblock, nneigh, istart+nneigh

               end if

            end if  ! ( iblock/=blocknr )

         end do  ! iblock = 1, nprocessors

!        --- neighblocks(istart) := nneigh
!            istart = istart+1+nneigh

         neighblocks(istart) = nneigh
         if ( debug ) &
            write(irefwr,*) 'istart, nneigh ',istart, nneigh
         istart = istart+1+nneigh
         if ( istart>lenneighblock ) then

!        --- array lenneighblock too small

            call errint ( istart, 1 )
            call errint ( lenneighblock, 2 )
            call errchr ( 'neighblocks', 1 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if

      end do  ! blocknr = 1, nprocessors

      lenneighblock = istart-1

1000  call erclos ( 'mshparneighbl' )
      if ( debug ) then

!     --- Debug information

         call prinin ( neighblocks, lenneighblock, 'neighblocks' )
         write(irefwr,*) 'End mshparneighbl'

      end if

      end
