      subroutine mshpavfillcrd ( bcord, nbndpt, crd, imxlps, lnnowbnd, &
                                 nloop, kbndpt, nowbnd, maxlen, isurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-06-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the arrays crd, kbndpt and nowbnd for the PAVER algorithm
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     surface
!     paver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer imxlps, nowbnd(imxlps,*), lnnowbnd, nloop, &
              kbndpt(*), nbndpt, isurnr
      double precision bcord(4,*), crd(4,*), maxlen

!     bcord          i    array containing the coordinates of the boundaries
!                         as well as information about the thickness of the
!                         layers
!                         bcord(1-2,i) contains the co-ordinates of the i-th
!                         boundary node
!                         bcord(3,i) contains the thickness of the i-th
!                         boundary node.
!                         If negative this means start of a new part
!                         bcord(4,i) contains the factor f of the i-th
!                         boundary node
!     crd            o    array containing the coordinates of the nodes of
!                         the boundary
!                         Pos. 1-2 contains the coordinates
!                         Pos. 3 contains the thickness of the i-th
!                         boundary node
!                         Pos. 4 contains the factor f of the i-th
!                         boundary node
!     imxlps         i    maximum loops in nowbnd in paver
!     isurnr         i    Surface sequence number
!     kbndpt         o    array containing the nodal point numbers of the
!                         boundary
!     lnnowbnd       i    Declared length of second index of array nowbnd
!     maxlen        i/o   Maximum length of edges in the surface
!     nbndpt         i    Number of nodal points in boundary, length of array
!                         kbndpt
!     nloop          o    number of closed parts the boundary consists of
!     nowbnd         o    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (imxlps,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, lennowbnd, indnr, indnrl, indnrr
      double precision l1, l2, ds, dsmax, dsmin

!     ds             Distance between two points along the boundary
!     dsmax          Maximum of all values ds
!     dsmin          Minimum of all values ds
!     i              Counting variable
!     indnr          node number of the i'th node in ibndnr
!     indnrl         node number of the previous node on the boundary
!     indnrr         node number of the next node on the boundary
!     j              Counting variable
!     l1             edge length to the left of the node
!     l2             edge length to the right of the node
!     lennowbnd      testing variable for the length of nowbnd
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PLOTNOWBND     Plot the actual boundary during mesh generation which is
!                    defined in array nowbnd, used in mshpav
!     PRININ         print 1d integer array
!     PRINNOWBND     Print array nowbnd which is used in mshpav
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    1029   Too much layers generated by PAVER
!    2453   maxlen too small
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Copy array bcord into crd
!     fill nowbnd with node numbers along the boundary
!     Fill array kbndpt
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavfillcrd' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavfillcrd'
  1      format ( a, 1x, (10i6) )

      end if

!     --- fill crd by copying bcord

      do i = 1, nbndpt
         do j = 1, 4
            crd(j,i) = bcord(j,i)
         end do
      end do

!     --- fill nowbnd with node numbers along the boundary

      nloop = 1
      nowbnd(1,1) = 1
      nowbnd(1,2) = 1
      kbndpt(1) = 1
      j=1
      lennowbnd = 1
      do i = 2, nbndpt-1
         if ( bcord(3,i)<0d0 ) then

!        --- start new loop
!            this is used if there are several non connected parts in the
!            boundary
!            Decrease number of nodes in this part of the boundary by 1

            nowbnd(nloop,1) = nowbnd(nloop,1) - 1

!           --- add node to kbndpt and increase boundary loop counter

            kbndpt(j) = nowbnd(nloop,2)
            nloop = nloop+1
            nowbnd(nloop,1) = 0

         end if

!        --- add node to nowbnd(nloop,*)

         nowbnd(nloop,1) = nowbnd(nloop,1) + 1
         nowbnd(nloop,nowbnd(nloop,1)+1) = i
         lennowbnd = max(lennowbnd,nowbnd(nloop,1)+1)
         j = j + 1
         kbndpt(j) = i

      end do

!     --- Add last point of boundary

      j = j + 1
      kbndpt(j) = nowbnd(nloop,2)
      nbndpt = j
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nbndpt, nloop', nbndpt, nloop
         call prinin ( kbndpt, nbndpt, 'kbndpt-1' )
         call prinin ( nowbnd, nloop, 'nowbnd-1' )
         call prinnowbnd ( nowbnd, nloop, imxlps, 0 )

      end if
      if ( nloop>imxlps ) then

!     --- Error nloop too large

         call errint ( nloop, 1 )
         call errint ( imxlps, 2 )
         call errwar ( 1029, 2, 0, 0 )
         go to 1000

      end if
      if ( lennowbnd>lnnowbnd ) then

!     --- Error lennowbnd too large

         call errint ( lennowbnd, 1 )
         call errint ( lnnowbnd, 2 )
         call errchr ( 'nowbnd', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

      dsmax = 0d0
      dsmin = rinfin
      do i = 1, nloop

!     --- add first two nodes at end for looping

         nowbnd(i,nowbnd(i,1)+2) = nowbnd(i,2)
         nowbnd(i,nowbnd(i,1)+3) = nowbnd(i,3)
         lennowbnd = max(lennowbnd,nowbnd(i,1)+3)
         do j=3,nowbnd(i,1)+2

            indnr = nowbnd(i,j)
            indnrl= nowbnd(i,j-1)
            indnrr= nowbnd(i,j+1)
            ds = sqrt( (bcord(1,indnr)-bcord(1,indnrl))**2+ &
                       (bcord(2,indnr)-bcord(2,indnrl))**2 )
            dsmax = max(ds, dsmax)
            dsmin = min(ds, dsmin)

            if ( bcord(4,indnr)>=0d0 ) then

!           --- height is specified in absolute sense

               crd(3,indnr) = abs(bcord(3,indnr))

            else if ( bcord(4,indnr)<0d0 ) then

!           --- height is height * avg edge length
!               compute avg edge length
!               l1, edge length to left of node
!               l2, edge length to right of node

               l1 = sqrt((bcord(1,indnrl) - bcord(1,indnr))**2 + &
                         (bcord(2,indnrl) - bcord(2,indnr))**2)
               l2 = sqrt((bcord(1,indnr) - bcord(1,indnrr))**2 + &
                         (bcord(2,indnr) - bcord(2,indnrr))**2)
               crd(3,indnr) = abs(bcord(3,indnr))* 0.5d0 * (l1+l2)

            end if
            crd(4,indnr) = abs(bcord(4,indnr))

         end do

      end do
      if ( lennowbnd>lnnowbnd ) then

!     --- Error lennowbnd too large

         call errint ( lennowbnd, 1 )
         call errint ( lnnowbnd, 2 )
         call errchr ( 'nowbnd', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

      if ( debug )  then

!     --- Debug information

         call prinnowbnd ( nowbnd, nloop, imxlps, 0 )
         jtimes = 0
         call plotnowbnd ( nowbnd, nloop, imxlps, 0, crd, &
                           'mshpavfillcrd' )

      end if
      if ( maxlen>0d0 .and. maxlen<dsmax*1.4d0 ) then

!     --- Warning maxlen too small

         call erreal ( maxlen, 1 )
         call erreal ( dsmax, 2 )
         call errint ( isurnr, 1 )
         call errwar ( 2453, 1, 2, 0 )
         maxlen = dsmax*1.4d0

      end if

      if ( maxlen.le.0d0 ) maxlen = dsmax*1.4d0

1000  call erclos ( 'mshpavfillcrd' )
      if ( debug ) write(irefwr,*) 'End mshpavfillcrd'

      end
