      subroutine mshpavmndist2 ( maxbounds, nowbnd, ind, nwcr, ibndnr, &
                                 crd, found, factor, jnode1, jnode2, &
                                 layer, intersect )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-09-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     An intersection has been found, check if the edge we are considering is
!     allowed
!     This means that we have to check if at least one, but preferably both
!     nodes of the edge are contained in the angle defined by
!     (ind-1,ind) and (ind,ind+1)
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
!     boundary
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibndnr, ind, maxbounds, nowbnd(maxbounds,*), jnode1, &
              jnode2, layer
      double precision nwcr(2), crd(4,*), factor
      logical intersect, found

!     crd            i    array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     factor         i    Multiplication factor
!     found         i/o   If true an intersection has been found at distance
!                         less than 3h
!     ibndnr         i    i'th boundary in nowbnd
!     ind            i    Sequence number of node along boundary
!     intersect      i    If true an intersection has been found at distance
!                         less than 3h
!     jnode1         i    first node of edge
!     jnode2         i    Last node in edge
!     layer          i    Present layer sequence number
!     maxbounds      i    maximum loops in nowbnd in paver
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (maxbounds,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
!     nwcr           i    coordinates of ind
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer indm1, inode, indp1
      double precision xnext(2), x3(2), x4(2), fact1, fact2, t1(2)
      logical node1, node2

!     fact1          multiplication factor to define intersection
!     fact2          multiplication factor to define intersection
!     indm1          Node on boundary before ind
!     indp1          Node on boundary after ind
!     inode          Node sequence number
!     node1          Node of face
!     node2          Node of face
!     t1             tangential vector
!     x3             coordinates of jnode1
!     x4             coordinates of jnode2
!     xnext          coordinates of line from nwcr in direction of tangent
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCROSSLINE1  Find the intersection of the lines x1-x2 and x3-x4
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The situation we consider is as follows:
!
!
!          inode2                 inode1
!           x --------------------- x
!
!
!
!
!   x                   x
!
!                x
!  indm1       ind     indp1
!
!     If the line ind to indm1 intersects the line inode1 inode2 then
!     inode1 can not be used as new line ind inode1
!     if the intersection is close to inode2 then also inode2 can not be used
!
!     If the line ind to indp1 intersects the line inode1 inode2 then
!     inode2 can not be used as new line ind inode2
!     if the intersection is close to inode1 then also inode1 can not be used
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavmndist2' )
      debug = .false.
      if ( layer.ge.10000 ) debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmndist2'
         write(irefwr,*) 'layer, ibndnr ', layer, ibndnr

      end if

!     --- First we intersect the extension of the line (ind-1, ind) with the
!         edge with nodes inode1, inode2

      indm1 = ind-1
      if ( indm1.eq.0 ) indm1 = nowbnd(ibndnr,1)
      inode = abs(nowbnd(ibndnr,indm1+1))
      t1(1) = crd(1,inode) - nwcr(1)
      t1(2) = crd(2,inode) - nwcr(2)
      xnext(1) = nwcr(1) + 1d3*t1(1)
      xnext(2) = nwcr(2) + 1d3*t1(2)

      x3(1) = crd(1,jnode1)
      x3(2) = crd(2,jnode1)
      x4(1) = crd(1,jnode2)
      x4(2) = crd(2,jnode2)
      node1 = .true.
      node2 = .true.

      call mshcrossline1 ( xnext, nwcr, x3, x4, fact1, fact2, sqreps )

      if ( debug ) then
         write(irefwr,*) 'indm1, ind, inode ', indm1, ind, inode
         write(irefwr,*) 'jnode1, jnode2 ', jnode1, jnode2
         write(irefwr,*) 'nwcr ', nwcr
         write(irefwr,*) 'crd(inode) ', crd(1,inode), crd(2,inode)
         write(irefwr,*) 'xnext ',xnext
         write(irefwr,*) 'x3, x4 ', x3, x4
         write(irefwr,*) 'fact1, fact2 ', fact1, fact2
      end if
      if ( fact2.gt.-0.1d0 .and. fact2.lt.1.1d0 ) then

!     --- An intersection has been found, so we have to do some checking
!         node jnode2 can not be used anymore, because it is too close
!         How about node1?

         node2 = .false.
         found = .false.
         if ( fact2.lt.0.2d0 ) then

!        --- Intersection close to node1, so we can not use this node

            node1 = .false.
            found = .false.

         end if

      end if
      if ( found ) then

!     --- The intersection with line (ind-1, ind) is allowed
!         Now the intersection with the extension of (ind, ind+1)

         indp1 = ind+1
         if ( indp1.gt.nowbnd(ibndnr,1) ) indp1 = 1
         inode = abs(nowbnd(ibndnr,indp1+1))
         t1(1) = crd(1,inode) - nwcr(1)
         t1(2) = crd(2,inode) - nwcr(2)
         xnext(1) = nwcr(1) + 1d3*t1(1)
         xnext(2) = nwcr(2) + 1d3*t1(2)

         call mshcrossline1 ( nwcr, xnext, x3, x4, fact1, fact2, &
                              sqreps )
         if ( ierror.ne.0 ) go to 1000

         if ( debug ) then
            write(irefwr,*) 'indp1, ind, inode ', indp1, ind, inode
            write(irefwr,*) 'nwcr ', nwcr
            write(irefwr,*) 'crd(inode) ', crd(1,inode), crd(2,inode)
            write(irefwr,*) 'xnext ',xnext
            write(irefwr,*) 'x3, x4 ', x3, x4
            write(irefwr,*) 'fact1, fact2 ', fact1, fact2
         end if
         if ( fact2.gt.-0.1d0 .and. fact2.le.1.1d0 ) then

!        --- An intersection has been found, so we have to do some checking
!            node inode1 can not be used anymore, because it is too close
!            How about node2?

            node1 = .false.
            found = .false.
            if ( fact2.gt.0.8d0 .and. factor.ge.0.5d0 ) then

!           --- Intersection close to node2, so we can not use this node

               node2 = .false.
               found = .false.

            end if

         end if

      end if  ! ( found )
      if ( debug ) write(irefwr,*) 'found, node1, node2 ', &
                                    found, node1, node2

      if ( found ) then

!     --- edge is allowed

         if ( .not. node1 ) then

!        --- node1 not allowed

            if ( intersect ) then

!           --- There is an intersection with the normal so connecting is
!               necessary

               found = node2
               factor = 0.9d0

            else if ( factor.ge.0.5d0 ) then

!           --- There is no intersection with the normal so connecting is
!               not necessary

               found = node2

            else

!           --- Reject this node

               found = .false.

            end if

         else if ( .not. node2 ) then

!        --- node2 not allowed

            if ( intersect ) then

!           --- There is an intersection with the normal so connecting is
!               necessary

               found = node1
               factor = 0.1d0

            else if ( factor.le.0.5d0 ) then

!           --- There is no intersection with the normal so connecting is
!               not necessary

               found = node1

            else

!           --- Reject this node

               found = .false.

            end if

         end if

      end if
      if ( debug .and. .not. found ) write(irefwr,*) 'Rejected'

1000  call erclos ( 'mshpavmndist2' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavmndist2'

      end if

      end
