      subroutine mshpip15 ( nsurbt, nelz, npobot, bcord, &
                            ipipetopol, nelbot, kbndpt, &
                            ibottopol, ibotboun )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 24-08-2010 Removal of iwork1
!        version  1.1  date 05-05-2005 Fortran 90
!        version  1.0  date 08-12-2002
!
!   copyright (c) 2002-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Generate volume elements with mesh generator PIPE
!     shape 15 and 15 only
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     volume
!     pipe
!     tetrahedron
!     hexahedron
!     coordinate
!     element
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nsurbt, nelz, npobot, ipipetopol(*), nelbot, &
              kbndpt(*), ibottopol(*), &
              ibotboun(*)
      double precision bcord(*)

!     bcord          i    array containing the coordinates of the boundaries
!     ibotboun       i    Contains node numbers of boundary of bottom surface
!     ibottopol      i    Contains topology of bottom surface
!     ipipetopol     i    Contains topology of pipe surface
!     kbndpt         o    array containing the nodal point numbers of the
!                         boundary
!     nelbot         i    Number of elements in bottom surface
!     nelz           i    Number of line elements in vertical direction
!     npobot         i    Number of points of bottom surface
!     nsurbt         i    Number of elements in boundary of bottom surface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer npointnewbot, m, n, npopip, nelpip, &
              nelboun, npointnew, npointnewpip, &
              lengnmeshd, lengnmesha, lengnmeshb, lengnmeshc, &
              istorebot(nelbot*inplmt), istorepip(nelz*nsurbt*2*inplmt)
      integer, allocatable :: itrianewbot(:), mapbot(:), itrianewpip(:), &
                              mappip(:), kbndbtnew(:)
      double precision, allocatable :: coornewbot(:), coornewtop(:), &
                                       coornewpip(:)

!     coornewbot     work array to store the coordinates of the bottom surface
!     coornewpip     work array to store the coordinates of the pipe surface
!     coornewtop     work array to store the coordinates of the top surface
!     istorebot      Work array to store the nodes of the elements of the
!                    bottom surface at input
!     istorepip      Work array to store the nodes of the elements of the
!                    pipe surface at input
!     itrianewbot    Contains the topology of the new triangle bottom surface
!                    consisting of the vertices only
!     itrianewpip    Contains the topology of the new triangle pipe surface
!                    consisting of the vertices only
!     kbndbtnew      Contains the nodes of the new boundary
!     lengnmesha     Declared length of nmesha
!     lengnmeshb     Declared length of nmeshb
!     lengnmeshc     Declared length of nmeshc
!     lengnmeshd     Declared length of nmeshd
!     m              Help parameter to compute the number of nodes and elements
!     mapbot         Contains the new node numbers of the nodes in the bottom
!                    surface
!     mappip         Contains the new node numbers of the nodes in the pipe
!                    surface
!     n              Help parameter to compute the number of nodes and elements
!     nelboun        Number of elements in the boundary, i.e. in kbndpt
!     nelpip         Number of elements in pipe surface
!     npointnew      Sequence number of last generated node
!     npointnewbot   Number of vertices in bottom surface
!     npointnewpip   Number of vertices in pipe surface
!     npopip         Number of points in pipe surface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH3DLINEARBND Map the boundary consisting of quadratic triangle with 7
!                    points onto a boundary consisting of linear elements only
!     MSH3DLINEARPIP Map the boundary consisting of quadratic triangle with 7
!                    points onto a boundary consisting of linear elements only
!     MSH3DQUADBOUN  Fill array kbndpt for a pipe consisting of tetrahedrons
!                    of shape 15 or 16
!     MSH3DQUADPIP   Fill kmeshc in the specific case of a pipe and quadratic
!                    tetrahedrons of shape 15 or 16
!     MSHCOOR3DQUAD  Fill coor in the specific case of a pipe and quadratic
!                    tetrahedrons of shape 15 or 16
!     MSHEDGFACES    Fill nmesha-d for the specific case of a linear
!                    triangle mesh corresponding to one volume only
!     MSHMAPKBNLIN   Map array kbndbtold onto kbndptnew using the information
!                    stored in array map
!     MSHPIP         Subroutine to create pipe volume elements
!     MSHPIPCOPBOUN  Copy coordinates back from the boundary array bcord into
!                    array coor
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINNMESHX     Print the contents of an integer array with structure of
!                    sub array of nmesh
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpip15' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpip15'

      end if
      if ( ierror/=0 ) go to 1000

!     --- ishape = 15 or 16, special case
!         First copy the boundary from extended quadratic onto a
!         linear set of elements
!         Start with bottom surface

      lengnmesha = 10*nelemlc+1
      lengnmeshb = 18*nelemlc
      lengnmeshc = 12*nelemlc
      lengnmeshd = 8*nelemlc+1
      m = nsurbt*2+1
      n = nelz*2+1

      allocate ( mapbot(npobot), kbndbtnew(m), &
                 mappip((nsurbt*2+1)*(nelz*2+1)+ &
                        nelz*nsurbt*2+13*nelbot), &
                 itrianewpip(3*(nelz*nsurbt*2)), itrianewbot(3*nelbot), &
                 coornewbot(3*npobot), coornewtop(3*npobot), &
                 coornewpip(3*(nsurbt+1)*(nelz+1)), &
                 stat = error )
      if ( error/=0 ) then

!     --- error#0, so not enough space

         call errint ( npobot+nsurbt+(nsurbt*2+1)*(nelz*2+1)+ &
                       nelz*nsurbt*2+13*nelbot+3*(nelz*nsurbt*2)+ &
                       3*nelbot+6*npobot+3*(nsurbt+1)*(nelz+1), 1 )
         call errint ( error, 2 )
         call errchr ( 'mapbot', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 1000

      end if  ! ( error/= 0 )

      call msh3dlinearbnd ( ibottopol, itrianewbot, npobot, &
                            npointnewbot, mapbot, inplmt, &
                            bcord, coornewbot, nelbot )
      if ( ierror/=0 ) go to 1000

      if ( npointnewbot>npobot ) then

!     --- Estimate of size of coornewbot too small

         call errint ( npointnewbot, 1 )
         call errint ( npobot, 2 )
         call errchr ( 'coornewbot', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

!     --- Copy old numbering of elements of bottom surface in istorebot

      istorebot(1:nelbot*inplmt) = ibottopol(1:nelbot*inplmt)

      if ( debug ) then

!     --- Debug statements

         call prinin1 ( ibottopol, nelbot, inplmt, 'itriabottom_old' )
         call prinin1 ( itrianewbot, nelbot, 3, &
                        'itriabottom_new' )
         call prinin ( mapbot, npobot, 'mapping array' )
         call prinrl1 ( bcord, npobot, 3, 'coorbot_old' )
         call prinrl1 ( coornewbot, npointnewbot, 3, &
                        'coorbot_new' )

      end if

!     --- Map array kbndbt to new reduced array

      call mshmapkbnlin ( ibotboun, mapbot, &
                          kbndbtnew, m, npointnew )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then

!     --- Debug statements

         call prinin ( ibotboun, m, &
                       'nodes at boundary of bottom-old' )
         call prinin ( kbndbtnew, npointnew, &
                       'nodes at boundary of bottom-new' )

      end if

      if ( npointnew>nsurbt+1 ) then

!     --- Estimate of size of kbndbtnew too small

         call errint ( npointnew, 1 )
         call errint ( nsurbt, 2 )
         call errchr ( 'kbndbtnew', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

!     --- Next top surface (same as bottom surface)

      call msh3dlinearbnd ( ibottopol, itrianewbot, npobot, &
                            npointnewbot, mapbot, inplmt, &
                            bcord(npobot*3+1), coornewtop, &
                            nelbot )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then

!     --- Debug statements

         call prinin1 ( ibottopol, nelbot, inplmt, 'itriatop_old' )
         call prinin1 ( itrianewbot, nelbot, 3, &
                        'itriatop_new' )
         call prinin ( mapbot, npobot, 'mapping array' )
         call prinrl1 ( bcord(npobot*3+1), npobot, 3, 'coortop_old' )
         call prinrl1 ( coornewtop, npointnewbot, 3, &
                        'coortop_new' )

      end if

!     --- Finally pipe surface

      nelpip = nelz*nsurbt*2
      npopip = m*n+nelpip
      call msh3dlinearpip ( ipipetopol, itrianewpip, &
                            npopip, npointnewpip, mappip, &
                            inplmt, bcord(npobot*6+1), &
                            coornewpip, nelpip, nsurbt, nelz )
      if ( ierror/=0 ) go to 1000
      if ( npointnewpip>(nsurbt+1)*(nelz+1) ) then

!     --- Estimate of size of coornewpip too small

         call errint ( npointnewpip, 1 )
         call errint ( nsurbt*(nelz+1), 2 )
         call errchr ( 'coornewpip', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

!     --- Copy old numbering of elements of pipe surface in istorepip

      istorepip(1:nelpip*inplmt) = ipipetopol(1:nelpip*inplmt)

      if ( debug ) then

!     --- Debug statements

         call prinin1 ( ipipetopol, nelpip, inplmt, &
                        'itriapipsurface_old' )
         call prinin1 ( itrianewpip, nelpip, 3, &
                        'itriapipsurface_new' )
         call prinin ( mappip, npopip, 'mapping array' )
         call prinrl1 ( bcord(npobot*6+1), npopip, 3, 'coorpip_old' )
         call prinrl1 ( coornewpip, npointnewpip, 3, &
                        'coorpip_new' )

      end if

!     --- call to mshpip in order to create the linear elements

      call mshpip ( nsurbt, nelz, npointnewbot, &
                    coornewbot, coornewtop, &
                    coornewpip, coor3d, &
                    itrianewbot, nelbot, &
                    kbndbtnew, npointlc, nelemlc, kmeshc, &
                    ivolnr, kbndpt, nbndpt, &
                    itrianewpip, ivol, 11 )
      if ( ierror/=0 ) go to 1000

      deallocate ( mapbot, mappip, kbndbtnew, itrianewpip, itrianewbot, &
                   coornewbot, coornewtop, coornewpip, stat = error )
      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'mapbot', 1 )
         call errsub ( 2911, 1, 0, 1 )
         go to 1000

      end if  ! ( error/= 0 )

!     --- create edges and faces

      allocate ( nmesha(lengnmesha), nmeshb(lengnmeshb), &
                 nmeshc(lengnmeshd), nmeshd(lengnmeshd), &
                 stat = error )
      if ( error/=0 ) then

!     --- error#0, so not enough space

         call errint ( lengnmesha+lengnmeshb+lengnmeshc+lengnmeshd, 1 )
         call errint ( error, 2 )
         call errchr ( 'nmesha', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 1000

      end if  ! ( error/= 0 )

      call mshedgfaces ( nelemlc, npointlc, kmeshc, nmeshd, &
                         lengnmeshd, nedges, nmesha, &
                         lengnmesha, nfaces, nmeshb, &
                         lengnmeshb, nmeshc, lengnmeshc )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then

!     --- Debug statements

         write(irefwr,*) 'nmesha: faces corresponding to elements'
         call prinnmeshx ( nelemlc, nmesha )
         write(irefwr,*) 'nmeshb: edges corresponding to faces'
         call prinnmeshx ( nfaces, nmeshb )
         write(irefwr,*) 'nmeshc: nodes corresponding to edges'
         call prinnmeshx ( nedges, nmeshc )
         write(irefwr,*) 'nmeshd: edges corresponding to elements'
         call prinnmeshx ( nelemlc, nmeshd )

      end if

!     --- Fill co-ordinates corresponding to edges, faces and elements
!         Quadratic tetrahedrons

      call mshcoor3dquad ( coor3d, npointlc, nedges, nfaces, nelemlc, &
                           ishape, nmeshb, nmeshc, kmeshc )

!     --- Fill kmeshc with node numbers corresponding to final shape
!         Quadratic tetrahedrons

      call msh3dquadpip ( kmeshc, inpelm, nelemlc, nedges, nfaces, &
                          nmesha, nmeshd, npointlc, ishape )

!     --- Finally fill array kbndpt with 7 points per element

      nelboun = 2*nelbot+nsurbt*nelz*2
      call msh3dquadboun ( npointlc, kbndpt, nelboun, nedges, &
                           nfaces, nmeshb, nmeshc, nelemlc, ishape, &
                           nmesha )
      nbndpt = nelboun*7

      deallocate ( nmesha, nmeshb, nmeshc, nmeshd, stat = error )
      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'nmesha', 1 )
         call errsub ( 2911, 1, 0, 1 )
         go to 1000

      end if  ! ( error/= 0 )

!     --- Copy coordinates of boundary back into array coor
!         This is necessary for the non-vertex points, since otherwise
!         curvature is lost
!         First the bottom is copied

      call mshpipcopboun ( nelbot, inplmt, istorebot, kbndpt, &
                           coor3d, bcord )

!     --- Next the top is copied

      call mshpipcopboun ( nelbot, inplmt, istorebot, &
                           kbndpt(nelbot*inplmt+1), coor3d, &
                           bcord(3*npobot+1) )

!     --- Finally the pipe surface

      call mshpipcopboun ( nelpip, inplmt, istorepip, &
                           kbndpt(2*nelbot*inplmt+1), coor3d, &
                           bcord(6*npobot+1) )

      if ( debug ) then

!     --- Debug statements

         call prinrl1 ( coor3d, npointlc, 3, 'coor-volume' )
         call prinin1 ( kmeshc, nelemlc, inpelm, 'kmeshc-volume' )
         call prinin1 ( kbndpt, nelboun, 7, 'kbndpt-volume' )

      end if

1000  if ( debug ) then

         write(irefwr,*) 'End mshpip15'

      end if
      call erclos ( 'mshpip15' )

      end
