      subroutine mshpr1 ( coor, npoint, plane, xsieta, imax, bcord, &
                          kbndbt, alpha, eps, repos, klp, nlp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 29-04-1999 Extra param's for hollow pipes (BvR)
!        version  1.2  date 22-09-1995 New definition of imax
!        version  1.1  date 12-03-1995 Adapt plane if necessary
!        version  1.0  date 12-11-1994
!
!   copyright (c) 1994-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Project co-ordinates stored in bcord onto plane defined by array plane
!   Define surface in this plane using the mapping defined by xsieta
! **********************************************************************
!
!                       KEYWORDS
!
!     plane
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, imax(3), kbndbt(*), klp(2,*), nlp
      double precision coor(3,npoint), plane(4), xsieta(2,npoint,2), &
                       bcord(3,*), alpha, eps
      logical repos

!     alpha   i    multiplication factor to be used between under surface
!                  and upper surface
!                  The ksi-eta co-ordinates in a coross section are computed by
!                  xsieta = (1-alpha)*xsieta_bot _ alpha*xsieta_top
!     bcord   i    array containing the coordinates of the boundaries
!     coor   i/o   array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     eps     o    Smallest distance between two boundary at the pipe surface
!                  eps/3 indicates the accuracy for which it is decided if
!                  repositioning should be applied.
!                  If || x_boundary - x_boundary(computed) || > eps
!                  repositioning must be applied
!     imax    i    In this array the indices with respect to boundary points
!                  in the surfaces are stored
!                  imax(1) stores the node number of the first point (usually 1)
!                  imax(2) stores the node number with respect to the xsi dir.
!                          The xsi direction is defined as P2-P1, where P2 is
!                          given by bcord(kbndbt(imax(2))
!                  imax(3) stores the node number with respect to the first eta
!                          dir.
!                          The eta direction is defined as P3-P1, where P3 is
!                          given by bcord(kbndbt(imax(3))
!     kbndbt  i    Array KBNDPT for bottom surface
!                  Contains the local nodal point numbers of the boundary of
!                  the bottom surface
!     klp     i    In this array the relative node numbers with respect to
!                  array kbndbt of first and last point of a consecutive part
!                  are stored. The number of parts the boundary consists of
!                  is nlp
!                  Hence klp(1,1) = 1, klp(2,1) denotes the end point of the
!                  first part of the boundary
!                  klp(1,2) contains the node number of the first point of
!                  the second part. This must be klp(2,1)+1
!                  klp(2,nlp) contains the length of kbndbt
!     nlp     i    Number of disjoint parts the boundary of the bottom
!                  surface consists of
!     npoint  i    Number of nodal points
!     plane   i    Array of length 4 containing the parameters a, b, c and d
!                  describing the plane: ax=by+cz+d = 0
!                  plane(1) = a; plane(2) = b; plane(3) = c; plane(4) = d;
!     repos   o    If true, repositioning must be applied
!     xsieta  i    Array containing the ksi-eta co-ordinates of the mapping
!                  xsieta(1,i,*) contains the ksi co-ordinate of point i
!                  xsieta(2,i,*) contains the eta co-ordinate of point i
!                  xsieta(*,*,1) corresponds to the under surface
!                  xsieta(*,*,2) corresponds to the upper surface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision x1(3), x2(3), alam, h, t1(3), t2(3), &
                       eta, xsi
      integer i, j, k, ilp, l

!     alam    Factor lambda in method
!     eta     Value of eta-coordinate
!     h       Help variable to store a constant temporarily
!     i       Counting variable
!     ilp     Counting variable
!     j       Counting variable
!     k       Counting variable
!     l       Counting variable
!     t1      First component to compute the tangential vector
!     t2      Second component to compute the tangential vector
!     x1      array to store the co-ordinates of the first point
!     x2      array to store the co-ordinates of the second point
!     xsi     Value of ksi-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELOUTR  Compute outer product of two vectors
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    method:
!    Consider the point P = (x1,y1,z1) and the plane n1 x + n2 y + n3 z + d = 0
!    The line through P with direction (n1,n2,n3) can be written as
!    x = x1 + lambda n
!    Hence for the intersection point we have:
!    n1 (x1+lambda n1) + n2 (x2+lambda n2) + n3 (x3+lambda n3) + d = 0
!    Because ||n||=1, this means
!    lambda = -d-(n1 x1 + n2 x2 + n3 x3 )
!    To define the local co-ordinates we define the first point in the
!    projection as origin O
!    The ksi direction is determined by taking the line through O and the
!    projected point P in the plane with the largest distance to O
!    Define t1 as the line OP, and normalize the length to 1
!    This defines the ksi-axis
!    In order to normalize the normal direction we proceed as follows
!    In the first call we have imax(3) = 0
!    In that case we find the boundary point in the projected plane such
!    that the angle between the xsi-axis and this point is maximal
!    imax is the corresponding index in kbndbt
!    n2 = t1 x (x(1)-x(imax))/||x(1)-x(imax)||
!    if ( n, n2 ) < 0, reverse sign of plane
!    The eta axis is defined by the vector t2 = n x OP / || n x OP ||
!    The mapping is defined as:  x-xo = ksi t1 + eta t2
!          |x-xo|                      |ksi|
!    or    |y-yo|   = |  t1  t2  n  |  |eta|             (1)
!          |z-zo|                      | 0 |
!    dz is the average distance between under and upper surface
!    alpha dz is the average distance between under surface and intersection
!    Then the co-ordinates in the intersection are defined by:
!    ksi = (1-alpha) xsieta(1,.,1) + alpha xsieta(1,.,2)
!    eta = (1-alpha) xsieta(2,.,1) + alpha xsieta(2,.,2)
!    x, y and z may be computed from (1)
!    If the difference between x, y and z computed and x, y and z available
!    at the boundary, repositioning is necessary.
!    pseudo code:
!    lambda = -d-(n1 x1 + n2 x2 + n3 x3 )
!    P1 = first point at boundary
!    P2 = point at boundary with largest distance to P1
!    t1 = P2-P1/||P2-P1||
!    t2 := Pimax - P1 / ||Pimax - P1||
!    n2 := t1 x t2 / ||t1xt2||
!    if ( (n,n2) < 0 ) then
!       n := -n
!    t2 = n x OP / || n x OP ||
!    For i := 1 (1) npoint do
!       ksi = (1-alpha) xsieta(1,.,1) + alpha xsieta(1,.,2)
!       eta = (1-alpha) xsieta(2,.,1) + alpha xsieta(2,.,2)
!       x(.)-xo(.) = t1(.) ksi + t2(.) eta
!    repos = false
!    For i := 1 (1) m do
!       j := kbndbt(i)
!       if ( ||x(j) - x(boundary)|| > eps then
!          repos = true
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- lambda = -d-(n1 x1 + n2 x2 + n3 x3 )

      j = imax(1)
      alam = -plane(4)-plane(1)*bcord(1,j)-plane(2)*bcord(2,j)- &
              plane(3)*bcord(3,j)

!     --- P1 = first point at boundary

      do i = 1, 3
         x1(i) = bcord(i,j) + plane(i)*alam
      end do
      do i = 1, 3
         x2(i) = bcord(i,imax(2)) + plane(i)*alam
      end do

!     --- t1 = P2-P1/||P2-P1||

      h = 1d0/sqrt((x2(1)-x1(1))**2+(x2(2)-x1(2))**2+(x2(3)-x1(3))**2)
      do i = 1, 3
         t1(i) = (x2(i)-x1(i))*h
      end do
      do i = 1, 3
         t2(i) = bcord(i,imax(3))+plane(i)*alam-x1(i)
      end do
      h = 1d0/sqrt(t2(1)**2+t2(2)**2+t2(3)**2)
      do i = 1, 3
         t2(i) = h*t2(i)
      end do

!     --- n2 = t1 x t2

      call eloutr ( t1, t2, x2 )
      if ( plane(1)*x2(1)+plane(2)*x2(2)+plane(3)*x2(3).lt.0d0 ) then

!     --- Reverse sign of plane

         do i = 1, 4
            plane(i) = -plane(i)
         end do
      end if

!     --- t2 = n x OP / || n x OP ||

      call eloutr ( plane, t1, t2 )
      h = 1d0/sqrt(t2(1)**2+t2(2)**2+t2(3)**2)
      do i = 1, 3
         t2(i) = h*t2(i)
      end do

!     --- Compute eps

      eps = rinfin
      do i = klp(1,1), klp(2,nlp)
         h = (bcord(1,i+1)-bcord(1,i))**2 + &
             (bcord(2,i+1)-bcord(2,i))**2 + &
             (bcord(3,i+1)-bcord(3,i))**2
         eps = min ( eps,h )
      end do
      eps = sqrt(eps) * 1d-2

!     --- Compute co-ordinates

      do i = 1, npoint
         xsi = (1d0-alpha) * xsieta(1,i,1) + alpha * xsieta(1,i,2)
         eta = (1d0-alpha) * xsieta(2,i,1) + alpha * xsieta(2,i,2)
         do j = 1, 3
            coor(j,i) = t1(j) * xsi + t2(j) * eta + bcord(j,imax(1))
         end do
      end do
      repos = .false.
      l = 0
      do ilp = 1,nlp
         do i = klp(1,ilp), klp(2,ilp)
            j = kbndbt(i)
            l = l + 1
            h = sqrt ( (coor(1,j)-bcord(1,l))**2 + &
                       (coor(2,j)-bcord(2,l))**2 + &
                       (coor(3,j)-bcord(3,l))**2 )
            if ( h*3.gt.eps ) repos = .true.
            do k = 1, 3
               coor(k,j) = bcord(k,l)
            end do
         end do
      end do
      end
