      subroutine mshrefine ( iinput, rinput, rinputavail, maplc, fillmap )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 22-06-2014 New definition of imap(5)
!        version  3.0  date 06-06-2014 extra parameters
!        version  2.2  date 27-11-2012 Long integers
!
!   copyright (c) 2003-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Refine an existing mesh one times by putting new nodes on all
!     edges in the middle of existing nodes
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*)
      double precision rinput(*)
      logical rinputavail, fillmap
      type (mapping) :: maplc

!     fillmap        i    If true array imap must be filled
!     iinput         i    Output array of subroutine MESH
!     maplc          o    contains information of the mapping
!     rinput         i    Real input array
!     rinputavail    i    If true, the input array rinput is available
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nmapout, nelgrpsav, npoints_new, npoints_old, &
              lenkmeshc, lenkmeshm, lenkmeshn, eldimmax, ielgrp, &
              lenrmap, lenkmesh, ichoice, lencurvs, krenum(10)
      integer, allocatable :: kmeshcnew(:), kmeshmnew(:), &
                              kmeshnnew(:), nodenumbers(:)
      double precision, allocatable :: coorcp(:,:)

!     coorcp         Contains the copy of the coordinates for a set of
!                    nodes
!     eldimmax       Maximum dimension of the elements, i.e.
!                    3 for volume elements
!                    2 for surface elements
!                    1 for line elements
!     ichoice        integer choice parameter for msh000
!     ielgrp         element group number
!     kmeshcnew      New kmesh part c
!     kmeshmnew      New kmesh part m
!     kmeshnnew      New kmesh part n
!     krenum         Array of length 10 with information about the
!                    renumbering, see msh000
!     lencurvs       Overestimate of the number of points along the new
!                    curves
!     lenkmesh       Size of array kmesh2d
!     lenkmeshc      length of array kmeshc
!     lenkmeshm      Length of array kmeshm
!     lenkmeshn      Length of array kmeshn
!     lenrmap        Estimated length of array rmap
!     nelgrpsav      Help parameter to save the old value of nelgrp
!     nmapout        Size of last dimension of rmap
!     nodenumbers    Array containing the node numbers of the newly created
!                    nodes in the following way
!                    Pos. 1 to nedges node number of first node on edge
!                    Next nfaces positions node number of first node on
!                    face not stored before
!                    Next nelem positions node numbers of first extra node
!                    in element not stored before
!                    if elddimmax = 1 only the third part is used
!                    if elddimmax = 2 the second part is not used
!     npoints_new    Number of nodes in new mesh
!     npoints_old    Number of nodes in old mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSH000         Fill some parameters from array IINPUT
!     MSHDELKMESHH   Delete kmesh part h and subarrays
!     MSHDELSEL      Delete a selected set of subarrays of kmesh
!     MSHNORMS       Compute normals on boundary and store in array normal
!     MSHREFCONNECT  Fill connection elements in case of refinement
!     MSHREFCOOR     Create new coodinates in a refine step of the complete mesh
!     MSHREFCOORAD   Adapt coordinates on curves in case of refinement in the
!                    case that a functional description of the curve is
!                    available
!     MSHREFKMESHT   Fill array kmesht in case of a refinement of a
!                    complete mesh
!     MSHREFNUMNODS  Compute the number of nodes in a mesh that is refined
!                    exactly one times
!     MSHREFTOPOL    Fill new topology of mesh into arrays kmeshcnew, kmeshmnew
!                    and kmeshnnew in the case that the mesh is refined
!                    in one global refine step
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLKMESHC  Make relation between kmeshc and Topgr
!     SEPFILLKMESHD  Fill kmesh part d
!     SEPFILLNMESHA  Fill array nmesha
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPFILLNMESHG  Fill array nmeshg
!     SEPFILLNMESHK  Fill array nmesh part k
!     SEPFILLNMESHL  Fill array nmesh part l
!     SEPPOINTKMESHC Fill array pointkmsc with starting addresses of element
!                    groups in kmeshc (minus 1)
!     SEPRELKMESH    Make the relation between kmref and the pointers
!                    in sepmodulekmeshp
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute all edges of the elements
!     Compute the number of new nodes from the number of edges
!     Compute new coordinates in the mesh and store in coor
!     Create new arrays kmeshc, kmeshm and kmeshn and fill these arrays
!        Determine the coordinates ( kmesh part i )
!        Determine the elements ( kmesh part c )
!        Determine the curves ( kmesh part m )
!        Determine the surfaces ( kmesh part n )
!        If it exists determine kmesh part h
!        Determine start positions for new arrays i,c,m,n
!        Make guess of number of neighbours
!        Make reservation for neighbour arrays
!        Set control integers in BUFFER array
!        Set pointers of old arrays
!        Fill array iwork with information of the curves
!        Perform actual refinement
!        Check control integers
!        Refill the parts of KMESH
!        Make arrays istart and ineigh temporarily permanent
!        If 2d then fill array Kmesh part h
!        Delete work arrays for neighbours
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefine' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefine'
         write(irefwr,*) 'original mesh'
         write(irefwr,1) 'npoint, nelem', npoint, nelem
  1      format ( a, 1x, (10i6) )
!        call sepprinkmesh ( 1 )

      end if
      if ( ierror/=0 ) go to 1000

!     --- define starting positions in array iinput and store in intarmsh

      ichoice = 0
      nelgrpsav = nelgrp
      call msh000 ( iinput, krenum, ichoice )

!     --- Check if there surface or volume elements

      nelgrp = nelgrpsav

      eldimmax = 0
      do ielgrp = 1, nelgrp
         ishape = kmeshf(ielgrp)
         if ( ishape>10 .and. ishape<50 ) then
            eldimmax = 3                             ! volume elements
         else if ( ishape>=3 .and. ishape<=10 ) then
            eldimmax = max(eldimmax,2)               ! surface elements
         else if ( ishape>=1 .and. ishape<=2 ) then
            eldimmax = max(eldimmax,1)               ! line elements
         end if
      end do  ! ielgrp = 1, nelgrp

      if ( debug ) write(irefwr,1) 'eldimmax', eldimmax

!     --- Compute all edges of the elements
!         Store all nodes corresponding to the edges in nmeshc
!         and all edge numbers corresponding to the elements in nmeshd
!         In case there all volume elements, also store the faces

      if ( eldimmax>1 ) then

!     --- There are surface elements, hence create edges

         if ( kelmaa==0 ) call sepfillkmeshaa
         if ( nmesh(33)==0 ) call sepfillnmeshc
         if ( nmesh(42)==0 ) call sepfillnmeshl
         if ( eldimmax>2 ) then

!        --- There are volume elements, hence create faces

            if ( nmesh(31)==0 ) call sepfillnmesha
            if ( nmesh(37)==0 ) call sepfillnmeshg
            if ( nmesh(41)==0 ) call sepfillnmeshk

         end if

      else

!     --- elddimmax = 1, activate kmesh part d

         if ( kelmd==0 ) call sepfillkmeshd

      end if  ! ( eldimmax>1 )
      if ( ierror/=0 ) go to 1000

!     --- Compute the number of new nodes from the number of edges,
!         surface elements and volume elements
!         First create space for array nodenumbers (temporarily)

      allocate ( nodenumbers(nedges+nfaces+nelem) )

      if ( debug .and. kelmaa>0 ) then
!         if ( nmesh(33)>0 ) call sepprinnmeshc
      end if  ! ( debug .and. kelmaa>0 )

!     --- actual computation of number of nodes

      call mshrefnumnods ( npoints_new, nodenumbers, eldimmax )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then

!     --- debug information

         write(irefwr,1) 'nedges, nfaces, nelem', nedges, nfaces, nelem
         write(irefwr,1) 'npoints_new', npoints_new
 !        call prinin ( nodenumbers, nedges+nfaces+nelem, 'nodenumbers' )

      end if  ! ( debug )

!     --- Compute new coordinates in the mesh and store in coor
!         First reserve space

      npoints_old = npoint    ! number of old nodes
      npoint = npoints_new    ! number of new nodes

      length = npoints_old*ndim
      allocate ( coorcp(ndim,npoints_old), stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'coorcp' )
      if ( ierror/=0 ) go to 1000

      coorcp(:,1:npoints_old) = coor

      deallocate ( km(iseqkm)%coor )
      allocate ( km(iseqkm)%coor(ndim,npoint) )
      coor => km(iseqkm)%coor
      coor(:,1:npoints_old) = coorcp(:,1:npoints_old)

      deallocate ( coorcp, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'coorcp' )
      if ( ierror/=0 ) go to 1000

      if ( debug .and. kelmaa>0 ) then
!         if ( nmesh(34)>0 ) call sepprinnmeshd
      end if  ! ( debug .and. kelmaa>0 )

!     --- actual computation of coordinates

!      if ( debug ) call prinrl1 ( coor, npoints_old, ndim, 'coorold' )

      if ( fillmap ) then
         allocate ( rmap(npoints_new,4), imap(npoints_new,5), stat = error )
         if ( error/=0 ) call eralloc ( error, npoints_new*9, 'rmap' )
         if ( ierror/=0 ) go to 1000
      end if  ! ( fillmap )


      call mshrefcoor ( nodenumbers, eldimmax, fillmap, nmapout )
      if ( ierror/=0 ) go to 1000

!     --- Create new arrays kmeshc, kmeshm and kmeshn and fill these arrays
!         First overestimate the length of these new arrays

      lenkmeshc = size ( kmeshc )
      if ( ndim==1 ) then
         lenkmeshc = 2*lenkmeshc
      else if ( ndim==2 ) then
         lenkmeshc = 4*lenkmeshc
      else
         lenkmeshc = 8*lenkmeshc
      end if
      if ( kelmm/=0 ) then
         lenkmeshm = 2*size ( kmeshm )
      else
         lenkmeshm = 0
      end if
      if ( kelmn/=0 ) then
         lenkmeshn = 4*size ( kmeshn )
      else
         lenkmeshn = 0
      end if

!     --- Reserve temporary space

      allocate ( kmeshcnew(lenkmeshc), kmeshmnew(lenkmeshm), &
                 kmeshnnew(lenkmeshn), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, lenkmeshc+lenkmeshm+lenkmeshn, 'kmeshcnew' )
      if ( ierror/=0 ) go to 1000

!     --- Fill topology

      call mshreftopol ( kmeshcnew, eldimmax, &
                         nodenumbers, lenkmeshc, &
                         lenkmeshm, lenkmeshn, kmeshmnew, &
                         kmeshnnew, npoints_old, fillmap )
      if ( ierror/=0 ) go to 1000
!      if ( debug ) call prinrl1 ( coor, npoints_new, ndim, 'coornew' )

!     --- Make new arrays permanent

      deallocate ( km(iseqkm)%kmeshc )
      allocate ( km(iseqkm)%kmeshc(lenkmeshc) )
      kmeshc => km(iseqkm)%kmeshc
      kmeshc = kmeshcnew(1:lenkmeshc)
      if ( kelmm/=0 ) then
         deallocate ( km(iseqkm)%kmeshm )
         allocate ( km(iseqkm)%kmeshm(lenkmeshm) )
         kmeshm => km(iseqkm)%kmeshm
         kmeshm = kmeshmnew(1:lenkmeshm)
         km(iseqkm)%kmesh(27) = lenkmeshm
      end if  ! ( kelmm/=0 )
      if ( kelmn/=0 ) then
         deallocate ( km(iseqkm)%kmeshn )
         allocate ( km(iseqkm)%kmeshn(lenkmeshn) )
         kmeshn => km(iseqkm)%kmeshn
         kmeshn = kmeshnnew(1:lenkmeshn)
         km(iseqkm)%kmesh(28) = lenkmeshn
      end if  ! ( kelmn/=0 )

      deallocate ( kmeshcnew, kmeshmnew, kmeshnnew, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'kmeshcnew' )
      if ( ierror/=0 ) go to 1000

      if ( fillmap ) then

!        --- Copy contents of imap and rmap to final positions

         if ( associated(maplc%imap) ) deallocate ( maplc%imap, maplc%rmap )
         allocate ( maplc%imap(npoints_new,nmapout+1), &
                    maplc%rmap(npoints_new,nmapout) )
         maplc%imap = imap(1:npoints_new,1:nmapout+1)
         maplc%rmap = rmap(1:npoints_new,1:nmapout)
         maplc%copy = .true.
         maplc%iseqkm2 = iseqkm
         maplc%equal = .false.

         deallocate ( imap, rmap, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'imap' )
         if ( ierror/=0 ) go to 1000

      end if  ! ( fillmap )
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,*) 'after fillmap'

!     --- refill kmesh part t

      if ( kelmt>0 ) then

!     --- kelmt>0, kmesh part t exists

         call mshrefkmesht
         if ( debug ) write(irefwr,*) 'after mshrefkmesht'

      end if  ! ( kelmt>0 )
      if ( ierror/=0 ) go to 1000

!     --- Adapt coordinates at boundary because of curvature

      if ( kelmm/=0 ) then
         lencurvs = size ( kmeshm ) - ncurvs-5
      else
         lencurvs = 1
      end if

      if ( debug ) write(irefwr,*) 'before mshrefcoorad'
      call mshrefcoorad ( iinput, rinput, rinputavail, lencurvs )

!     --- Remove superfluous arrays and arrays that are not longer valid

      deallocate ( nodenumbers )

!     --- Fill connection elements if necessary

      if ( debug ) write(irefwr,*) 'before mshrefconnect'
      call mshrefconnect ( iinput )
      call sepfillkmeshc   ! Make relation between kmeshc and Topgrp
      call seppointkmeshc  ! Fill array pointkmsc

!     --- Delete arrays in kmesh that are no longer correct

      if ( debug ) write(irefwr,*) 'before mshdelsel'
      call mshdelsel ( kmesh )
      call mshdelkmeshh

      if ( debug ) write(irefwr,*) 'before seprelkmesh'
      call seprelkmesh ( kmesh )
      if ( ierror/=0 ) go to 1000

      if ( mmnormal>0 ) then

!     --- mmnormal>0, recompute normals

         call mshnorms
         if ( ierror/=0 ) go to 1000

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'mesh after refinement'
         write(irefwr,1) 'npoint, nelem', npoint, nelem
!         call sepprinkmesh ( 1 )
         write(irefwr,*) 'End mshrefine'

      end if
      call erclos ( 'mshrefine' )

      end subroutine mshrefine
