      subroutine mshrefkmeshc13 ( kmeshclc, kmeshcnew, nelprev, &
                                  nodenumbers, fillmap )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 24-06-2014 Extra parameters
!        version  1.1  date 10-03-2005 Fortran 90
!        version  1.0  date 29-09-2003
!
!   copyright (c) 2003-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the new array kmeshclc for one element group in case that
!     the complete mesh is refined
!     Elements of shape 13, i.e. trilinear hexahedrons
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     topology
!     linear
!     hexahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshclc(8,*), kmeshcnew(8,*), nodenumbers(*), &
              nelprev
      logical fillmap

!     fillmap        i    If true array imap must be filled
!     kmeshclc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshcnew      o    New kmesh part c
!     nelprev        i    Number of elements stored in kmeshclc at the start
!                         of the subroutine
!     nodenumbers    i    Array containing the node numbers of the newly created
!                         nodes in the following way
!                         Pos. 1 to nedges node number of first node on edge
!                         Next nfaces positions node number of first node on
!                         face not stored before
!                         Next inelem positions node numbers of first extra node
!                         in element not stored before
!                         if elddimmax = 1 only the third part is used
!                         if elddimmax = 2 the second part is not used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielemnew, ielem, ipstart, iedge, edge(12), i, face(6), &
              iface, ivol, inode1, inode2

!     edge           Contains the edge node numbers on the 12 edges
!     face           Contains the face node numbers on the 6 faces
!     i              Counting variable
!     iedge          edge sequence number
!     ielem          Element number
!     ielemnew       Sequence number of element in new mesh
!     iface          Face sequence number
!     inode1         Node number
!     inode2         Node number
!     ipstart        Starting address in array
!     ivol           Contains the volume node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The linear hexahedron has nodes 1 2 3 4   5 6 7 8
!     The edges are defines as:
!     e1:  1 2
!     e2:  2 6
!     e3:  6 5
!     e4:  5 1
!     e5:  4 3
!     e6:  3 7
!     e7:  7 8
!     e8:  8 4
!     e9:  1 4
!    e10:  2 3
!    e11:  6 7
!    e12:  5 8
!
!     Let the corresponding nodes be e1 ... e12
!
!     The faces as
!
!     f1:  1 2 6 5
!     f2:  3 4 8 7
!     f3:  2 1 4 3
!     f4:  5 6 7 8
!     f5:  1 5 8 4
!     f6:  2 3 7 6
!
!     The 8 elements become:
!
!      1  e1  f3 e9      e4  f1  v1  f5
!     e1   2 e10 f3      f1  e2  f6  v1
!     e9  f3  e5  4      f5  v1  f2  e8
!     f3 e10   3 e5      v1  f6  e6  f2
!
!     e4  f1  v1 f5       5  e3  f4 e12
!     f1  e2  f6 v1      e3   6 e11  f4
!     f5  v1  f2 e8     e12  f4  e7   8
!     v1  f6  e6 f2      f4 e11   7  e7
!
!     ielemnew := 0
!     For all elements ielem in old mesh do
!        ielemnew := ielemnew+1
!        fill first element
!        ielemnew := ielemnew+1
!        fill second element
!        ielemnew := ielemnew+1
!        fill third element
!        ielemnew := ielemnew+1
!        fill fourth element
!        ielemnew := ielemnew+1
!        fill fifth element
!        ielemnew := ielemnew+1
!        fill sixth element
!        ielemnew := ielemnew+1
!        fill seventh element
!        ielemnew := ielemnew+1
!        fill element eight
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefkmeshc13' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmeshc13'
         write(irefwr,*) 'inelem, nelprev ', inelem, nelprev
         call prinin1 ( kmeshclc, inelem, 8, 'kmeshclc/volume' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- ielemnew := 0

      ielemnew = 0

!     --- For all elements ielem in old mesh do

      do ielem = 1, inelem

!     --- fill node numbers of edges

         ipstart = nmeshd(nelprev+ielem)
         do i = 1, 12
            iedge = nmeshd(ipstart+i)
            edge(i) = nodenumbers(abs(iedge))
         end do

!        --- fill node numbers of faces

         ipstart = nmesha(nelprev+ielem)
         do i = 1, 6
            iface = abs(nmesha(ipstart+i))
            face(i) = nodenumbers(iface+nedges)
         end do
         ivol = nodenumbers(nedges+nfaces+nelprev+ielem)

         inode1 = face(1)
         inode2 = face(2)
         coor(1:3,ivol) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))

         if ( fillmap ) then
            imap(ivol,1) = 2
            imap(ivol,2) = inode1
            imap(ivol,3) = inode2
            rmap(ivol,1:2) = 0.5d0
         end if  ! ( fillmap )

!        --- ielemnew := ielemnew+1
!            fill first element
!            1  e1  f3 e9      e4  f1  v1  f5

         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = kmeshclc(1,ielem)
         kmeshcnew(2,ielemnew) = edge(1)
         kmeshcnew(3,ielemnew) = face(3)
         kmeshcnew(4,ielemnew) = edge(9)
         kmeshcnew(5,ielemnew) = edge(4)
         kmeshcnew(6,ielemnew) = face(1)
         kmeshcnew(7,ielemnew) = ivol
         kmeshcnew(8,ielemnew) = face(5)

!        --- ielemnew := ielemnew+1
!             fill second element
!             e1   2 e10 f3      f1  e2  f6  v1

         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = edge(1)
         kmeshcnew(2,ielemnew) = kmeshclc(2,ielem)
         kmeshcnew(3,ielemnew) = edge(10)
         kmeshcnew(4,ielemnew) = face(3)
         kmeshcnew(5,ielemnew) = face(1)
         kmeshcnew(6,ielemnew) = edge(2)
         kmeshcnew(7,ielemnew) = face(6)
         kmeshcnew(8,ielemnew) = ivol

!        --- ielemnew := ielemnew+1
!             fill third element
!             e9  f3  e5  4      f5  v1  f2  e8

         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = edge(9)
         kmeshcnew(2,ielemnew) = face(3)
         kmeshcnew(3,ielemnew) = edge(5)
         kmeshcnew(4,ielemnew) = kmeshclc(4,ielem)
         kmeshcnew(5,ielemnew) = face(5)
         kmeshcnew(6,ielemnew) = ivol
         kmeshcnew(7,ielemnew) = face(2)
         kmeshcnew(8,ielemnew) = edge(8)

!        --- ielemnew := ielemnew+1
!             fill fourth element
!             f3 e10   3 e5      v1  f6  e6  f2

         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = face(3)
         kmeshcnew(2,ielemnew) = edge(10)
         kmeshcnew(3,ielemnew) = kmeshclc(3,ielem)
         kmeshcnew(4,ielemnew) = edge(5)
         kmeshcnew(5,ielemnew) = ivol
         kmeshcnew(6,ielemnew) = face(6)
         kmeshcnew(7,ielemnew) = edge(6)
         kmeshcnew(8,ielemnew) = face(2)

!        --- ielemnew := ielemnew+1
!             fill fifth element
!             e4  f1  v1 f5       5  e3  f4 e12

         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = edge(4)
         kmeshcnew(2,ielemnew) = face(1)
         kmeshcnew(3,ielemnew) = ivol
         kmeshcnew(4,ielemnew) = face(5)
         kmeshcnew(5,ielemnew) = kmeshclc(5,ielem)
         kmeshcnew(6,ielemnew) = edge(3)
         kmeshcnew(7,ielemnew) = face(4)
         kmeshcnew(8,ielemnew) = edge(12)

!        --- ielemnew := ielemnew+1
!             fill sixth element
!             f1  e2  f6 v1      e3   6 e11  f4

         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = face(1)
         kmeshcnew(2,ielemnew) = edge(2)
         kmeshcnew(3,ielemnew) = face(6)
         kmeshcnew(4,ielemnew) = ivol
         kmeshcnew(5,ielemnew) = edge(3)
         kmeshcnew(6,ielemnew) = kmeshclc(6,ielem)
         kmeshcnew(7,ielemnew) = edge(11)
         kmeshcnew(8,ielemnew) = face(4)

!        --- ielemnew := ielemnew+1
!             fill seventh element
!             f5  v1  f2 e8     e12  f4  e7   8

         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = face(5)
         kmeshcnew(2,ielemnew) = ivol
         kmeshcnew(3,ielemnew) = face(2)
         kmeshcnew(4,ielemnew) = edge(8)
         kmeshcnew(5,ielemnew) = edge(12)
         kmeshcnew(6,ielemnew) = face(4)
         kmeshcnew(7,ielemnew) = edge(7)
         kmeshcnew(8,ielemnew) = kmeshclc(8,ielem)

!        --- ielemnew := ielemnew+1
!             fill element eight
!             v1  f6  e6 f2      f4 e11   7  e7

         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = ivol
         kmeshcnew(2,ielemnew) = face(6)
         kmeshcnew(3,ielemnew) = edge(6)
         kmeshcnew(4,ielemnew) = face(2)
         kmeshcnew(5,ielemnew) = face(4)
         kmeshcnew(6,ielemnew) = edge(11)
         kmeshcnew(7,ielemnew) = kmeshclc(7,ielem)
         kmeshcnew(8,ielemnew) = edge(7)

      end do  ! ielem = 1, inelem

1000  call erclos ( 'mshrefkmeshc13' )

      if ( debug ) then

!     --- Debug information

         call prinin1 ( kmeshcnew, ielemnew, 8, 'kmeshcnew/volume' )
         write(irefwr,*) 'End mshrefkmeshc13'

      end if

      end
