      subroutine mshrefkmeshc14 ( kmeshclc, kmeshcnew, nelprev, &
                                  nodenumbers, fillmap )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 18-06-2014 Extra parameters
!        version  1.1  date 10-03-2005 Fortran 90
!        version  1.0  date 29-09-2003
!
!   copyright (c) 2003-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the new array kmeshclc for one element group in case that
!     the complete mesh is refined
!     Elements of shape 14, i.e. quadratic hexahedrons
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     topology
!     quadratic
!     hexahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshclc(27,*), kmeshcnew(27,*), nodenumbers(*), &
              nelprev
      logical fillmap

!     fillmap        i    If true array imap must be filled
!     kmeshclc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshcnew      o    New kmesh part c
!     nelprev        i    Number of elements stored in kmeshclc at the start
!                         of the subroutine
!     nodenumbers    i    Array containing the node numbers of the newly created
!                         nodes in the following way
!                         Pos. 1 to nedges node number of first node on edge
!                         Next nfaces positions node number of first node on
!                         face not stored before
!                         Next inelem positions node numbers of first extra node
!                         in element not stored before
!                         if elddimmax = 1 only the third part is used
!                         if elddimmax = 2 the second part is not used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielemnew, ielem, ipstart, iedge, edge(12,2), i, face(6,8), &
              iface, nrotat, ind(8), ivol, inode1, inode2

!     edge           Contains the edge node numbers on the 12 edges (2 per edge)
!     face           Contains the face node numbers on the 6 faces (8 per face)
!     i              Counting variable
!     iedge          edge sequence number
!     ielem          Element number
!     ielemnew       Sequence number of element in new mesh
!     iface          Face sequence number
!     ind            Integer array containing the cyclic permutation of
!                    1,2,3,4
!     inode1         Node number
!     inode2         Node number
!     ipstart        Starting address in array
!     ivol           Contains the volume node number
!     nrotat         Number of rotations necessary to map both elements
!                    onto each other
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The quadratic hexahedron has nodes
!      1  2  3    4  5  6    7  8  9
!     10 11 12   13 14 15   16 17 18
!     19 20 21   22 23 24   25 26 27
!     The edges are defines as:
!     e1:  1  2  3
!     e2:  3 12 21
!     e3: 21 20 19
!     e4: 19 10  1
!     e5:  7  8  9
!     e6:  9 18 27
!     e7: 27 26 25
!     e8: 25 16  7
!     e9:  1  4  7
!    e10:  3  6  9
!    e11: 21 24 27
!    e12: 19 22 25
!
!     Let the corresponding nodes be e1a, e1b, e2a, e2b, e3a, ... e12b
!
!     The faces as
!
!     f1:  1  2  3 12 21 20 19 10 11
!     f2:  9  8  7 16 25 26 27 18 17
!     f3:  3  2  1  4  7  8  9  6  5
!     f4: 19 20 21 24 27 26 25 22 23
!     f5:  1 10 19 22 25 16  7  4 13
!     f6:  3  6  9 18 27 24 21 12 15
!
!     The nodes on each face is positioned as follows:
!     a between 2 and 9
!     b between 4 and 9
!     c between 6 and 9
!     d between 8 and 9
!     The other 4 in the centers of the 4 quadrants:
!     e in  1 - 2 - 9 - 8
!     f in  2 - 3 - 9 - 2
!     g in  4 - 5 - 6 - 9
!     h in  6 - 7 - 8 - 9
!
!     We need 26 extra volume nodes
!     The 8 elements consist of 5 layers of faces with the following numbering
!     (top to bottom)
!
!       25  e7b   26  e7a   27
!     e12b  f4f  f4c  f4g e11b
!       22  f4d   23  f4b   24
!     e12a  f4e  f4a  f4f e11a
!       19  e3b   20  e3a   21
!
!      e8a  f2g  f2c  f2h  e6b
!      f5g  v25  v17  v23  f6g
!      f5b  v24  v6   v22  f6c
!      f5f  v21  v18  v26  f6h
!      e4a  f1h  f1c  f1g  e2b
!
!       16  f2b   17  f2d   18
!      f5c  v15  v5   v16  f6b
!       13  v3    14  v4    15
!      f5a  v8   v2   v9   f6d
!       10  f1d   11  f1b   12
!
!      e8b  f2f  f2a  f2e  e6a
!      f5h   v19  v14  v20 f6f
!      f5d   v10  v1   v11 f6a
!      f5e   v13  v7   v12 f6e
!      e4b  f1e  f1a  f1f  e2a
!
!        7  e5a    8  e5b    9
!      e9b  f3g  f3c  f3h e10b
!        4  f3b    5  f3d    6
!      e9a  f3f  f3a  f3e e10a
!        1  e1a    2  e1b    3
!
!     The 8 elements become:
!
!        1  e1a    2     e9a  f3f  f3a      4   f3b    5
!      e4b  f1e  f1a     f5e  v13   v7    f5d   v10   v1
!       10  f1d   11     f5a   v8  v2      13    v3   14
!
!        2  e1b    3     f3a  f3e e10a       5  f3d    6
!      f1a  f1f  e2a      v7  v12  f6e      v1  v11  f6a
!       11  f1b   12      v2   v9  f6d      14   v4   15
!
!        4  f3b    5     e9b  f3g  f3c       7  e5a    8
!      f5d  v10   v1     f5h  v19  v14     e8b  f2f  f2a
!       13   v3   14     f5c  v15   v5      16  f2b   17
!
!        5  f3d    6     f3c  f3h e10b       8  e5b    9
!       v1  v11  f6a     v14  v20  f6f     f2a  f2e  e6a
!       14   v4   15      v5  v16  f6b      17  f2d   18
!
!       10  f1d   11     f5a   v8   v2      13   v3   14
!      e4a  f1h  f1c     f5f  v21  v18     f5b  v24  v6
!       19  e3b   20    e12a  f4e  f4a      22  f4d   23
!
!       11  f1b   12      v2   v9  f6d      14   v4   15
!      f1c  f1g  e2b     v18  v26  f6h      v6  v22  f6c
!       20  e3a   21     f4a  f4f e11a      23  f4b   24
!
!       13   v3   14     f5c  v15   v5      16  f2b   17
!      f5b  v24   v6     f5g  v25  v17     e8a  f2g  f2c
!       22  f4d   23    e12b  f4h  f4c      25  e7b   26
!
!       14   v4   15      v5  v16  f6b      17  f2d   18
!       v6  v22  f6c     v17  v23  f6g     f2c  f2h  e6b
!       23  f4b   24     f4c  f4g e11b      26  e7a   27
!
!     ielemnew := 0
!     For all elements ielem in old mesh do
!        ielemnew := ielemnew+1
!        fill first element
!        ielemnew := ielemnew+1
!        fill second element
!        ielemnew := ielemnew+1
!        fill third element
!        ielemnew := ielemnew+1
!        fill fourth element
!        ielemnew := ielemnew+1
!        fill fifth element
!        ielemnew := ielemnew+1
!        fill sixth element
!        ielemnew := ielemnew+1
!        fill seventh element
!        ielemnew := ielemnew+1
!        fill element eight
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ind / 1, 2, 3, 4, 1, 2, 3, 4 /
! ======================================================================
!
      call eropen ( 'mshrefkmeshc14' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmeshc14'
         write(irefwr,*) 'inelem, nelprev ', inelem, nelprev
         call prinin1 ( kmeshclc, 3, 27, 'kmeshclc_old' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- ielemnew := 0

      ielemnew = 0

!     --- For all elements ielem in old mesh do

      do ielem = 1, inelem

!     --- fill node numbers of edges

         ipstart = nmeshd(nelprev+ielem)
         do i = 1, 12
            iedge = nmeshd(ipstart+i)
            if ( iedge>0 ) then
               edge(i,1) = nodenumbers(iedge)       ! 12 edge numbers / a
               edge(i,2) = nodenumbers(iedge)+1     ! 12 edge numbers / b
            else  ! ( iedge<0 )
               edge(i,1) = nodenumbers(-iedge)+1    ! 12 edge numbers / a
               edge(i,2) = nodenumbers(-iedge)      ! 12 edge numbers / b
            end if  ! ( iedge>0 )
         end do

!        --- fill node numbers of faces

         ipstart = nmesha(nelprev+ielem)
         do i = 1, 6
            iface = nmesha(ipstart+i)
            nrotat =  nmesha(ipstart+6+i)
            if ( iface>0 ) then

!           --- Standard direction

               iface = nodenumbers(iface+nedges)
               face(i,ind(nrotat+1)) = iface
               face(i,ind(nrotat+2)) = iface+1
               face(i,ind(nrotat+3)) = iface+2
               face(i,ind(nrotat+4)) = iface+3
               face(i,ind(nrotat+1)+4) = iface+4
               face(i,ind(nrotat+2)+4) = iface+5
               face(i,ind(nrotat+3)+4) = iface+6
               face(i,ind(nrotat+4)+4) = iface+7

            else

!           --- Reversed direction

               iface = nodenumbers(-iface+nedges)
               face(i,1) = iface-1+ind(nrotat+1)
               face(i,2) = iface-1+ind(nrotat+4)
               face(i,3) = iface-1+ind(nrotat+3)
               face(i,4) = iface-1+ind(nrotat+2)
               face(i,5) = iface+ind(nrotat+2)+3
               face(i,6) = iface+ind(nrotat+1)+3
               face(i,7) = iface+ind(nrotat+4)+3
               face(i,8) = iface+ind(nrotat+3)+3

            end if

         end do
         ivol = nodenumbers(nedges+nfaces+nelprev+ielem)-1

!        --- coor(ivol+1) := 1/2 ( x_5 + x_14 )
!            coor(ivol+2) := 1/2 ( x_11 + x_14 )
!            coor(ivol+3) := 1/2 ( x_13 + x_14 )
!            coor(ivol+4) := 1/2 ( x_14 + x_15 )
!            coor(ivol+5) := 1/2 ( x_14 + x_17 )
!            coor(ivol+6) := 1/2 ( x_14 + x_23 )

         inode1 = kmeshclc(5,ielem)
         inode2 = kmeshclc(14,ielem)
         coor(1:3,ivol+1) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))
         inode1 = kmeshclc(11,ielem)
         coor(1:3,ivol+2) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))
         inode1 = kmeshclc(13,ielem)
         coor(1:3,ivol+3) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))
         inode1 = kmeshclc(15,ielem)
         coor(1:3,ivol+4) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))
         inode1 = kmeshclc(17,ielem)
         coor(1:3,ivol+5) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))
         inode1 = kmeshclc(23,ielem)
         coor(1:3,ivol+6) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))

!        --- coor(ivol+7) := 1/2 ( v2 + f3a )
!            coor(ivol+8) := 1/2 ( v2 + f5a )
!            coor(ivol+9) := 1/2 ( v2 + f6d )
!            coor(ivol+10) := 1/2 ( v1 + f5d )
!            coor(ivol+11) := 1/2 ( v1 + f6a )
!            coor(ivol+12) := 1/2 ( v7 + f6e )
!            coor(ivol+13) := 1/2 ( v7 + f5e )

         inode1 = ivol+2
         inode2 = face(3,1)
         coor(1:3,ivol+7) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))
         inode2 = face(5,1)
         coor(1:3,ivol+8) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))
         inode2 = face(6,4)
         coor(1:3,ivol+9) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))

         inode1 = ivol+1
         inode2 = face(5,4)
         coor(1:3,ivol+10) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))
         inode2 = face(6,1)
         coor(1:3,ivol+11) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))

         inode1 = ivol+7
         inode2 = face(6,5)
         coor(1:3,ivol+12) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))
         inode2 = face(5,5)
         coor(1:3,ivol+13) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))

!        --- coor(ivol+14) := 1/2 ( v5 + f3c )
!            coor(ivol+15) := 1/2 ( v5 + f5c )
!            coor(ivol+16) := 1/2 ( v5 + f6b )
!            coor(ivol+17) := 1/2 ( v5 + f4c )
!            coor(ivol+18) := 1/2 ( v2 + f4a )
!            coor(ivol+19) := 1/2 ( v14 + f5h )
!            coor(ivol+20) := 1/2 ( v14 + f6f )
!            coor(ivol+21) := 1/2 ( v18 + f5f )
!            coor(ivol+22) := 1/2 ( v4 + f4b )
!            coor(ivol+23) := 1/2 ( v16 + f4g )
!            coor(ivol+24) := 1/2 ( v6 + f5b )
!            coor(ivol+25) := 1/2 ( v17 + f5g )
!            coor(ivol+26) := 1/2 ( v18 + f6h )

         inode1 = ivol+5
         inode2 = face(3,3)
         coor(1:3,ivol+14) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))
         inode2 = face(5,3)
         coor(1:3,ivol+15) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))
         inode2 = face(6,2)
         coor(1:3,ivol+16) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))
         inode2 = face(4,3)
         coor(1:3,ivol+17) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))
         inode1 = ivol+2
         inode2 = face(4,1)
         coor(1:3,ivol+18) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))
         inode1 = ivol+14
         inode2 = face(5,8)
         coor(1:3,ivol+19) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))
         inode2 = face(6,6)
         coor(1:3,ivol+20) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))
         inode1 = ivol+18
         inode2 = face(5,6)
         coor(1:3,ivol+21) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))
         inode2 = face(6,8)
         coor(1:3,ivol+26) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))
         inode1 = ivol+4
         inode2 = face(4,2)
         coor(1:3,ivol+22) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))
         inode1 = ivol+16
         inode2 = face(4,7)
         coor(1:3,ivol+23) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))
         inode1 = ivol+6
         inode2 = face(5,2)
         coor(1:3,ivol+24) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))
         inode1 = ivol+17
         inode2 = face(5,7)
         coor(1:3,ivol+25) = 0.5d0*(coor(1:3,inode1)+coor(1:3,inode2))

         if ( fillmap ) then
            imap(ivol+1:ivol+26,1) = 2
            imap(ivol+1:ivol+6,3) = kmeshclc(14,ielem)
            imap(ivol+1,2) = kmeshclc(5,ielem)
            imap(ivol+2,2) = kmeshclc(11,ielem)
            imap(ivol+3,2) = kmeshclc(13,ielem)
            imap(ivol+4,2) = kmeshclc(15,ielem)
            imap(ivol+5,2) = kmeshclc(17,ielem)
            imap(ivol+6,2) = kmeshclc(23,ielem)
            imap(ivol+7:ivol+9,3) = ivol+2
            imap(ivol+7,2) = face(3,1)
            imap(ivol+8,2) = face(5,1)
            imap(ivol+9,2) = face(6,4)
            imap(ivol+10:ivol+11,3) = ivol+1
            imap(ivol+10,2) = face(5,4)
            imap(ivol+11,2) = face(6,1)
            imap(ivol+12:ivol+13,3) = ivol+7
            imap(ivol+12,2) = face(6,5)
            imap(ivol+13,2) = face(5,5)
            imap(ivol+14:ivol+17,3) = ivol+5
            imap(ivol+14,2) = face(3,3)
            imap(ivol+15,2) = face(5,3)
            imap(ivol+16,2) = face(6,2)
            imap(ivol+17,2) = face(4,3)
            imap(ivol+18,3) = ivol+2
            imap(ivol+18,2) = face(4,1)
            imap(ivol+19:ivol+20,3) = ivol+14
            imap(ivol+19,2) = face(5,8)
            imap(ivol+20,2) = face(6,6)
            imap(ivol+21,3) = ivol+18
            imap(ivol+21,2) = face(5,6)
            imap(ivol+26,2) = face(6,8)
            imap(ivol+26,3) = ivol+18
            imap(ivol+22,3) = ivol+4
            imap(ivol+22,2) = face(4,2)
            imap(ivol+23,3) = ivol+16
            imap(ivol+23,2) = face(4,7)
            imap(ivol+24,3) = ivol+6
            imap(ivol+24,2) = face(5,2)
            imap(ivol+25,3) = ivol+17
            imap(ivol+25,2) = face(5,7)
            rmap(ivol+1:ivol+26,1:2) = 0.5d0
         end if  ! ( fillmap )

!        --- ielemnew := ielemnew+1
!            fill first element
!              1  e1a    2     e9a  f3f  f3a      4  f3b    5
!            e4b  f1e  f1a     f5e   v13   v7    f5d   v10   v1
!             10  f1d   11     f5a  v8  v2     13  v3   14

         ielemnew = ielemnew+1
         kmeshcnew( 1,ielemnew) = kmeshclc(1,ielem)
         kmeshcnew( 2,ielemnew) = edge(1,1)
         kmeshcnew( 3,ielemnew) = kmeshclc(2,ielem)
         kmeshcnew( 4,ielemnew) = edge(9,1)
         kmeshcnew( 5,ielemnew) = face(3,6)
         kmeshcnew( 6,ielemnew) = face(3,1)
         kmeshcnew( 7,ielemnew) = kmeshclc(4,ielem)
         kmeshcnew( 8,ielemnew) = face(3,2)
         kmeshcnew( 9,ielemnew) = kmeshclc(5,ielem)
         kmeshcnew(10,ielemnew) = edge(4,2)
         kmeshcnew(11,ielemnew) = face(1,5)
         kmeshcnew(12,ielemnew) = face(1,1)
         kmeshcnew(13,ielemnew) = face(5,5)
         kmeshcnew(14,ielemnew) = ivol+13
         kmeshcnew(15,ielemnew) = ivol+7
         kmeshcnew(16,ielemnew) = face(5,4)
         kmeshcnew(17,ielemnew) = ivol+10
         kmeshcnew(18,ielemnew) = ivol+1
         kmeshcnew(19,ielemnew) = kmeshclc(10,ielem)
         kmeshcnew(20,ielemnew) = face(1,4)
         kmeshcnew(21,ielemnew) = kmeshclc(11,ielem)
         kmeshcnew(22,ielemnew) = face(5,1)
         kmeshcnew(23,ielemnew) = ivol+8
         kmeshcnew(24,ielemnew) = ivol+2
         kmeshcnew(25,ielemnew) = kmeshclc(13,ielem)
         kmeshcnew(26,ielemnew) = ivol+3
         kmeshcnew(27,ielemnew) = kmeshclc(14,ielem)

!        --- ielemnew := ielemnew+1
!             fill second element
!               2  e1b    3     f3a  f3e e10a      5  f3d    6
!             f1a  f1f  e2a      v7   v12  f6e     v1   v11  f6a
!              11  f1b   12     v2  v9  f6d     14  v4   15

         ielemnew = ielemnew+1
         kmeshcnew( 1,ielemnew) = kmeshclc(2,ielem)
         kmeshcnew( 2,ielemnew) = edge(1,2)
         kmeshcnew( 3,ielemnew) = kmeshclc(3,ielem)
         kmeshcnew( 4,ielemnew) = face(3,1)
         kmeshcnew( 5,ielemnew) = face(3,5)
         kmeshcnew( 6,ielemnew) = edge(10,1)
         kmeshcnew( 7,ielemnew) = kmeshclc(5,ielem)
         kmeshcnew( 8,ielemnew) = face(3,4)
         kmeshcnew( 9,ielemnew) = kmeshclc(6,ielem)
         kmeshcnew(10,ielemnew) = face(1,1)
         kmeshcnew(11,ielemnew) = face(1,6)
         kmeshcnew(12,ielemnew) = edge(2,1)
         kmeshcnew(13,ielemnew) = ivol+7
         kmeshcnew(14,ielemnew) = ivol+12
         kmeshcnew(15,ielemnew) = face(6,5)
         kmeshcnew(16,ielemnew) = ivol+1
         kmeshcnew(17,ielemnew) = ivol+11
         kmeshcnew(18,ielemnew) = face(6,1)
         kmeshcnew(19,ielemnew) = kmeshclc(11,ielem)
         kmeshcnew(20,ielemnew) = face(1,2)
         kmeshcnew(21,ielemnew) = kmeshclc(12,ielem)
         kmeshcnew(22,ielemnew) = ivol+2
         kmeshcnew(23,ielemnew) = ivol+9
         kmeshcnew(24,ielemnew) = face(6,4)
         kmeshcnew(25,ielemnew) = kmeshclc(14,ielem)
         kmeshcnew(26,ielemnew) = ivol+4
         kmeshcnew(27,ielemnew) = kmeshclc(15,ielem)

!        --- ielemnew := ielemnew+1
!             fill third element
!               4  f3b    5     e9b  f3g  f3c      7  e5a    8
!             f5d   v10   v1     f5h   v19   v14    e8b  f2f  f2a
!              13  v3   14     f5c  v15  v5     16  f2b   17

         ielemnew = ielemnew+1
         kmeshcnew( 1,ielemnew) = kmeshclc(4,ielem)
         kmeshcnew( 2,ielemnew) = face(3,2)
         kmeshcnew( 3,ielemnew) = kmeshclc(5,ielem)
         kmeshcnew( 4,ielemnew) = edge(9,2)
         kmeshcnew( 5,ielemnew) = face(3,7)
         kmeshcnew( 6,ielemnew) = face(3,3)
         kmeshcnew( 7,ielemnew) = kmeshclc(7,ielem)
         kmeshcnew( 8,ielemnew) = edge(5,1)
         kmeshcnew( 9,ielemnew) = kmeshclc(8,ielem)
         kmeshcnew(10,ielemnew) = face(5,4)
         kmeshcnew(11,ielemnew) = ivol+10
         kmeshcnew(12,ielemnew) = ivol+1
         kmeshcnew(13,ielemnew) = face(5,8)
         kmeshcnew(14,ielemnew) = ivol+19
         kmeshcnew(15,ielemnew) = ivol+14
         kmeshcnew(16,ielemnew) = edge(8,2)
         kmeshcnew(17,ielemnew) = face(2,6)
         kmeshcnew(18,ielemnew) = face(2,1)
         kmeshcnew(19,ielemnew) = kmeshclc(13,ielem)
         kmeshcnew(20,ielemnew) = ivol+3
         kmeshcnew(21,ielemnew) = kmeshclc(14,ielem)
         kmeshcnew(22,ielemnew) = face(5,3)
         kmeshcnew(23,ielemnew) = ivol+15
         kmeshcnew(24,ielemnew) = ivol+5
         kmeshcnew(25,ielemnew) = kmeshclc(16,ielem)
         kmeshcnew(26,ielemnew) = face(2,2)
         kmeshcnew(27,ielemnew) = kmeshclc(17,ielem)

!        --- ielemnew := ielemnew+1
!             fill fourth element
!              5  f3d    6     f3c  f3h e10b      8  e5b    9
!             v1   v11  f6a      v14   v20  f6f    f2a  f2e  e6a
!             14  v4   15     v5  v16  f6b     17  f2d   18

         ielemnew = ielemnew+1
         kmeshcnew( 1,ielemnew) = kmeshclc(5,ielem)
         kmeshcnew( 2,ielemnew) = face(3,4)
         kmeshcnew( 3,ielemnew) = kmeshclc(6,ielem)
         kmeshcnew( 4,ielemnew) = face(3,3)
         kmeshcnew( 5,ielemnew) = face(3,8)
         kmeshcnew( 6,ielemnew) = edge(10,2)
         kmeshcnew( 7,ielemnew) = kmeshclc(8,ielem)
         kmeshcnew( 8,ielemnew) = edge(5,2)
         kmeshcnew( 9,ielemnew) = kmeshclc(9,ielem)
         kmeshcnew(10,ielemnew) = ivol+1
         kmeshcnew(11,ielemnew) = ivol+11
         kmeshcnew(12,ielemnew) = face(6,1)
         kmeshcnew(13,ielemnew) = ivol+14
         kmeshcnew(14,ielemnew) = ivol+20
         kmeshcnew(15,ielemnew) = face(6,6)
         kmeshcnew(16,ielemnew) = face(2,1)
         kmeshcnew(17,ielemnew) = face(2,5)
         kmeshcnew(18,ielemnew) = edge(6,1)
         kmeshcnew(19,ielemnew) = kmeshclc(14,ielem)
         kmeshcnew(20,ielemnew) = ivol+4
         kmeshcnew(21,ielemnew) = kmeshclc(15,ielem)
         kmeshcnew(22,ielemnew) = ivol+5
         kmeshcnew(23,ielemnew) = ivol+16
         kmeshcnew(24,ielemnew) = face(6,2)
         kmeshcnew(25,ielemnew) = kmeshclc(17,ielem)
         kmeshcnew(26,ielemnew) = face(2,4)
         kmeshcnew(27,ielemnew) = kmeshclc(18,ielem)

!        --- ielemnew := ielemnew+1
!             fill fifth element
!              10  f1d   11     f5a  v8  v2     13  v3   14
!             e4a  f1h  f1c     f5f  v21  v18    f5b  v24  v6
!              19  e3b   20    e12a  f4e  f4a     22  f4d   23

         ielemnew = ielemnew+1
         kmeshcnew( 1,ielemnew) = kmeshclc(10,ielem)
         kmeshcnew( 2,ielemnew) = face(1,4)
         kmeshcnew( 3,ielemnew) = kmeshclc(11,ielem)
         kmeshcnew( 4,ielemnew) = face(5,1)
         kmeshcnew( 5,ielemnew) = ivol+8
         kmeshcnew( 6,ielemnew) = ivol+2
         kmeshcnew( 7,ielemnew) = kmeshclc(13,ielem)
         kmeshcnew( 8,ielemnew) = ivol+3
         kmeshcnew( 9,ielemnew) = kmeshclc(14,ielem)
         kmeshcnew(10,ielemnew) = edge(4,1)
         kmeshcnew(11,ielemnew) = face(1,8)
         kmeshcnew(12,ielemnew) = face(1,3)
         kmeshcnew(13,ielemnew) = face(5,6)
         kmeshcnew(14,ielemnew) = ivol+21
         kmeshcnew(15,ielemnew) = ivol+18
         kmeshcnew(16,ielemnew) = face(5,2)
         kmeshcnew(17,ielemnew) = ivol+24
         kmeshcnew(18,ielemnew) = ivol+6
         kmeshcnew(19,ielemnew) = kmeshclc(19,ielem)
         kmeshcnew(20,ielemnew) = edge(3,2)
         kmeshcnew(21,ielemnew) = kmeshclc(20,ielem)
         kmeshcnew(22,ielemnew) = edge(12,1)
         kmeshcnew(23,ielemnew) = face(4,5)
         kmeshcnew(24,ielemnew) = face(4,1)
         kmeshcnew(25,ielemnew) = kmeshclc(22,ielem)
         kmeshcnew(26,ielemnew) = face(4,4)
         kmeshcnew(27,ielemnew) = kmeshclc(23,ielem)

!        --- ielemnew := ielemnew+1
!             fill sixth element
!              11  f1b   12     v2  v9  f6d     14  v4   15
!             f1c  f1g  e2b     v18  v26  f6h    v6  v22  f6c
!              20  e3a   21     f4a  f4f e11a     23  f4b   24

         ielemnew = ielemnew+1
         kmeshcnew( 1,ielemnew) = kmeshclc(11,ielem)
         kmeshcnew( 2,ielemnew) = face(1,2)
         kmeshcnew( 3,ielemnew) = kmeshclc(12,ielem)
         kmeshcnew( 4,ielemnew) = ivol+2
         kmeshcnew( 5,ielemnew) = ivol+9
         kmeshcnew( 6,ielemnew) = face(6,4)
         kmeshcnew( 7,ielemnew) = kmeshclc(14,ielem)
         kmeshcnew( 8,ielemnew) = ivol+4
         kmeshcnew( 9,ielemnew) = kmeshclc(15,ielem)
         kmeshcnew(10,ielemnew) = face(1,3)
         kmeshcnew(11,ielemnew) = face(1,7)
         kmeshcnew(12,ielemnew) = edge(2,2)
         kmeshcnew(13,ielemnew) = ivol+18
         kmeshcnew(14,ielemnew) = ivol+26
         kmeshcnew(15,ielemnew) = face(6,8)
         kmeshcnew(16,ielemnew) = ivol+6
         kmeshcnew(17,ielemnew) = ivol+22
         kmeshcnew(18,ielemnew) = face(6,3)
         kmeshcnew(19,ielemnew) = kmeshclc(20,ielem)
         kmeshcnew(20,ielemnew) = edge(3,1)
         kmeshcnew(21,ielemnew) = kmeshclc(21,ielem)
         kmeshcnew(22,ielemnew) = face(4,1)
         kmeshcnew(23,ielemnew) = face(4,6)
         kmeshcnew(24,ielemnew) = edge(11,1)
         kmeshcnew(25,ielemnew) = kmeshclc(23,ielem)
         kmeshcnew(26,ielemnew) = face(4,2)
         kmeshcnew(27,ielemnew) = kmeshclc(24,ielem)

!        --- ielemnew := ielemnew+1
!             fill seventh element
!              13  v3   14     f5c  v15  v5     16  f2b   17
!             f5b  v24  v6     f5g  v25  v17     e8a  f2g  f2c
!              22  f4d   23    e12b  f4h  f4c      25  e7b   26

         ielemnew = ielemnew+1
         kmeshcnew( 1,ielemnew) = kmeshclc(13,ielem)
         kmeshcnew( 2,ielemnew) = ivol+3
         kmeshcnew( 3,ielemnew) = kmeshclc(14,ielem)
         kmeshcnew( 4,ielemnew) = face(5,3)
         kmeshcnew( 5,ielemnew) = ivol+15
         kmeshcnew( 6,ielemnew) = ivol+5
         kmeshcnew( 7,ielemnew) = kmeshclc(16,ielem)
         kmeshcnew( 8,ielemnew) = face(2,2)
         kmeshcnew( 9,ielemnew) = kmeshclc(17,ielem)
         kmeshcnew(10,ielemnew) = face(5,2)
         kmeshcnew(11,ielemnew) = ivol+24
         kmeshcnew(12,ielemnew) = ivol+6
         kmeshcnew(13,ielemnew) = face(5,7)
         kmeshcnew(14,ielemnew) = ivol+25
         kmeshcnew(15,ielemnew) = ivol+17
         kmeshcnew(16,ielemnew) = edge(8,1)
         kmeshcnew(17,ielemnew) = face(2,7)
         kmeshcnew(18,ielemnew) = face(2,3)
         kmeshcnew(19,ielemnew) = kmeshclc(22,ielem)
         kmeshcnew(20,ielemnew) = face(4,4)
         kmeshcnew(21,ielemnew) = kmeshclc(23,ielem)
         kmeshcnew(22,ielemnew) = edge(12,2)
         kmeshcnew(23,ielemnew) = face(4,8)
         kmeshcnew(24,ielemnew) = face(4,3)
         kmeshcnew(25,ielemnew) = kmeshclc(25,ielem)
         kmeshcnew(26,ielemnew) = edge(7,2)
         kmeshcnew(27,ielemnew) = kmeshclc(26,ielem)

!        --- ielemnew := ielemnew+1
!             fill element eight
!              14  v4   15     v5  v16  f6b     17  f2d   18
!             v6  v22  f6c     v17  v23  f6g    f2c  f2h  e6b
!              23  f4b   24     f4c  f4g e11b     26  e7a   27

         ielemnew = ielemnew+1
         kmeshcnew( 1,ielemnew) = kmeshclc(14,ielem)
         kmeshcnew( 2,ielemnew) = ivol+4
         kmeshcnew( 3,ielemnew) = kmeshclc(15,ielem)
         kmeshcnew( 4,ielemnew) = ivol+5
         kmeshcnew( 5,ielemnew) = ivol+16
         kmeshcnew( 6,ielemnew) = face(6,2)
         kmeshcnew( 7,ielemnew) = kmeshclc(17,ielem)
         kmeshcnew( 8,ielemnew) = face(2,4)
         kmeshcnew( 9,ielemnew) = kmeshclc(18,ielem)
         kmeshcnew(10,ielemnew) = ivol+6
         kmeshcnew(11,ielemnew) = ivol+22
         kmeshcnew(12,ielemnew) = face(6,3)
         kmeshcnew(13,ielemnew) = ivol+17
         kmeshcnew(14,ielemnew) = ivol+23
         kmeshcnew(15,ielemnew) = face(6,7)
         kmeshcnew(16,ielemnew) = face(2,3)
         kmeshcnew(17,ielemnew) = face(2,8)
         kmeshcnew(18,ielemnew) = edge(6,2)
         kmeshcnew(19,ielemnew) = kmeshclc(23,ielem)
         kmeshcnew(20,ielemnew) = face(4,2)
         kmeshcnew(21,ielemnew) = kmeshclc(24,ielem)
         kmeshcnew(22,ielemnew) = face(4,3)
         kmeshcnew(23,ielemnew) = face(4,7)
         kmeshcnew(24,ielemnew) = edge(11,2)
         kmeshcnew(25,ielemnew) = kmeshclc(26,ielem)
         kmeshcnew(26,ielemnew) = edge(7,1)
         kmeshcnew(27,ielemnew) = kmeshclc(27,ielem)

      end do  ! ielem = 1, inelem

1000  call erclos ( 'mshrefkmeshc14' )

      if ( debug ) then

!     --- Debug information

!         call prinin1 ( kmeshcnew, ielemnew, 27, 'kmeshcnew/surface' )

         call prinin1 ( kmeshcnew, 24, 27, 'kmeshcnew/surface' )
         write(irefwr,*) 'End mshrefkmeshc14'

      end if

      end
