      subroutine mshrefkmeshc21 ( kmeshc, kmeshcnew, nelem, nelprev, &
                                  nmeshd, nodenumbers, nmesha, nedges )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-06-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the new array kmeshc for one element group in case that
!     the complete mesh is refined
!     Elements of shape 21, i.e. linear prisms
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     topology
!     linear
!     prism
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(6,*), kmeshcnew(6,*), nelem, nodenumbers(*), &
              nelprev, nmeshd(*), nmesha(*), nedges

!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshcnew      o    New kmesh part c
!     nedges         i    number of edges
!     nelem          i    Number of elements
!     nelprev        i    Number of elements stored in kmeshc at the start
!                         of the subroutine
!     nmesha         i    nmesh part a
!     nmeshd         i    nmesh part d
!     nodenumbers    i    Array containing the node numbers of the newly created
!                         nodes in the following way
!                         Pos. 1 to nedges node number of first node on edge
!                         Next nfaces positions node number of first node on
!                         face not stored before
!                         Next nelem positions node numbers of first extra node
!                         in element not stored before
!                         if elddimmax = 1 only the third part is used
!                         if elddimmax = 2 the second part is not used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielemnew, ielem, ipstart, iedge, edge(9), i, face(5), &
              iface

!     edge           Contains the edge node numbers on the edges
!     face           Contains the face node numbers on the faces
!     i              Counting variable
!     iedge          edge sequence number
!     ielem          Element number
!     ielemnew       Sequence number of element in new mesh
!     iface          Face sequence number
!     ipstart        Starting address in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The linear prism has nodes 1 2 3 and 4 5 6
!     The edges are defined as: 1-2 2-3 3-1
!                               4-5 5-6 6-1
!                               1-4 2-5 3-6
!     Let the corresponding nodes be e1, e2 ... e9
!
!     The faces as 1-2-3  4-5-6   1-2-5-4 2-3-6-5 1-3-6-4
!     Let the nodes of the faces 3, 4, 5 be f3, f4 f5
!
!     The 8 elements become:
!
!     1   e1  e3  e7 f3 f5
!     2   e2  e1  e8 f4 f3
!     3   e3  e2  e9 f5 f4
!     e1  e2  e3  f3 f4 f5
!
!     e7 f3 f5   4  e4  e6
!     e8 f4 f3   5  e5  e4
!     e9 f5 f4   6  e6  e5
!     f3 f4 f5   e4 e5  e6
!
!     ielemnew := 0
!     For all elements ielem in old mesh do
!        ielemnew := ielemnew+1
!        fill first element
!        ielemnew := ielemnew+1
!        fill second element
!        ielemnew := ielemnew+1
!        fill third element
!        ielemnew := ielemnew+1
!        fill fourth element
!        ielemnew := ielemnew+1
!        fill fifth element
!        ielemnew := ielemnew+1
!        fill sixth element
!        ielemnew := ielemnew+1
!        fill seventh element
!        ielemnew := ielemnew+1
!        fill eigth element
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefkmeshc21' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmeshc21'
         write(irefwr,*) 'nelem, nelprev ', nelem, nelprev
         call prinin1 ( kmeshc, nelem, 6, 'kmeshc/volume' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- ielemnew := 0

      ielemnew = 0

!     --- For all elements ielem in old mesh do

      do ielem = 1, nelem

!     --- fill node numbers of edges

         ipstart = nmeshd(nelprev+ielem)
         do i = 1, 9
            iedge = abs(nmeshd(ipstart+i))
            edge(i) = nodenumbers(iedge)
         end do

!        --- fill node numbers of faces

         ipstart = nmesha(nelprev+ielem)
         do i = 3, 5
            iface = abs(nmesha(ipstart+i))
            face(i) = nodenumbers(iface+nedges)
         end do

!        --- 1   e1  e3  e7 f3 f5

         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = kmeshc(1,ielem)
         kmeshcnew(2,ielemnew) = edge(1)
         kmeshcnew(3,ielemnew) = edge(3)
         kmeshcnew(4,ielemnew) = edge(7)
         kmeshcnew(5,ielemnew) = face(3)
         kmeshcnew(6,ielemnew) = face(5)

!        --- 2   e2  e1  e8 f4 f3

         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = kmeshc(2,ielem)
         kmeshcnew(2,ielemnew) = edge(2)
         kmeshcnew(3,ielemnew) = edge(1)
         kmeshcnew(4,ielemnew) = edge(8)
         kmeshcnew(5,ielemnew) = face(4)
         kmeshcnew(6,ielemnew) = face(3)

!        --- 3   e3  e2  e9 f5 f4

         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = kmeshc(3,ielem)
         kmeshcnew(2,ielemnew) = edge(3)
         kmeshcnew(3,ielemnew) = edge(2)
         kmeshcnew(4,ielemnew) = edge(9)
         kmeshcnew(5,ielemnew) = face(5)
         kmeshcnew(6,ielemnew) = face(4)

!        --- e1  e2  e3  f3 f4 f5

         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = edge(1)
         kmeshcnew(2,ielemnew) = edge(2)
         kmeshcnew(3,ielemnew) = edge(3)
         kmeshcnew(4,ielemnew) = face(3)
         kmeshcnew(5,ielemnew) = face(4)
         kmeshcnew(6,ielemnew) = face(5)

!        --- e7 f3 f5   4  e4  e6

         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = edge(7)
         kmeshcnew(2,ielemnew) = face(3)
         kmeshcnew(3,ielemnew) = face(5)
         kmeshcnew(4,ielemnew) = kmeshc(4,ielem)
         kmeshcnew(5,ielemnew) = edge(4)
         kmeshcnew(6,ielemnew) = edge(6)

!        --- e8 f4 f3   5  e5  e4

         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = edge(8)
         kmeshcnew(2,ielemnew) = face(4)
         kmeshcnew(3,ielemnew) = face(3)
         kmeshcnew(4,ielemnew) = kmeshc(5,ielem)
         kmeshcnew(5,ielemnew) = edge(5)
         kmeshcnew(6,ielemnew) = edge(4)

!        --- e9 f5 f4   6  e6  e5

         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = edge(9)
         kmeshcnew(2,ielemnew) = face(5)
         kmeshcnew(3,ielemnew) = face(4)
         kmeshcnew(4,ielemnew) = kmeshc(6,ielem)
         kmeshcnew(5,ielemnew) = edge(6)
         kmeshcnew(6,ielemnew) = edge(5)

!        --- f3 f4 f5   e4 e5  e6

         ielemnew = ielemnew+1
         kmeshcnew(1,ielemnew) = face(3)
         kmeshcnew(2,ielemnew) = face(4)
         kmeshcnew(3,ielemnew) = face(5)
         kmeshcnew(4,ielemnew) = edge(4)
         kmeshcnew(5,ielemnew) = edge(5)
         kmeshcnew(6,ielemnew) = edge(6)

      end do  ! ielem = 1, nelem

1000  call erclos ( 'mshrefkmeshc21' )

      if ( debug ) then

!     --- Debug information

         call prinin1 ( kmeshcnew, ielemnew, 6, 'kmeshcnew/volume' )
         write(irefwr,*) 'End mshrefkmeshc21'

      end if

      end
