      subroutine mshrefkmeshc7 ( kmeshc, kmeshcnew, nelem, nelprev, &
                                 nmeshd, nodenumbers, nedges )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-03-2005 Fortran 90
!        version  1.0  date 26-09-2003
!
!   copyright (c) 2003-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the new array kmeshc for one element group in case that
!     the complete mesh is refined
!     Elements of shape 7, i.e. extended quadratic triangles
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(7,*), kmeshcnew(7,*), nelem, nodenumbers(*), &
              nelprev, nmeshd(*), nedges

!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshcnew      o    New kmesh part c
!     nedges         i    number of edges
!     nelem          i    Number of elements
!     nelprev        i    Number of elements stored in kmeshc at the start
!                         of the subroutine
!     nmeshd         i    nmesh part d
!     nodenumbers    i    Array containing the node numbers of the newly created
!                         nodes in the following way
!                         Pos. 1 to nedges node number of first node on edge
!                         Next nfaces positions node number of first node on
!                         face not stored before
!                         Next nelem positions node numbers of first extra node
!                         in element not stored before
!                         if elddimmax = 1 only the third part is used
!                         if elddimmax = 2 the second part is not used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielemnew, ielem, ipstart, iedge, edge(3,2), i, iface

!     edge           Contains the edge node numbers on the 3 edges (2 per edge)
!     i              Counting variable
!     iedge          edge sequence number
!     ielem          Element number
!     ielemnew       Sequence number of element in new mesh
!     iface          Face sequence number
!     ipstart        Starting address in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The extended quadratic triangle has 7 nodes according to
!
!                  5
!                  x
!                 / \
!                /   \
!             6 x     x 4
!              /   x   \
!             /     7   \
!            x-----x-----x
!           1      2     3
!
!     The six face nodes are defined as follows
!     f_1 between 6 and 2
!     f_2 between 2 and 4
!     f_3 between 4 and 6
!     The other 3 in the centers of the 3 triangles:
!     f_4 in  1 - 2 - 6
!     f_5 in  2 - 3 - 4
!     f_6 in  4 - 5 - 6
!
!     The edges are defines as: 1-3,  3-5,  5-1
!     Let the corresponding nodes be e1, e2 and e3
!
!            5
!            x
!           / \
!       e3 0   0  e2
!         /     \
!        x---0---x
!        1   e1  3
!
!     On each edge we have two extra nodes: e1a/b etc
!     Then the 4 new triangles become:
!     1 e1a 2 f_1 6 e3b f_4
!     3 e2a 4 f_2 2 e1b f_5
!     5 e3a 6 f_3 4 e2b f_6
!     2 f_2 4 f_3 6 f_1 7
!
!     ielemnew := 0
!     For all elements ielem in old mesh do
!        ielemnew := ielemnew+1
!        fill first element
!        ielemnew := ielemnew+1
!        fill second element
!        ielemnew := ielemnew+1
!        fill third element
!        ielemnew := ielemnew+1
!        fill fourth element
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefkmeshc7' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmeshc7'
         write(irefwr,*) 'nelem, nelprev ', nelem, nelprev
         call prinin1 ( kmeshc, nelem, 3, 'kmeshc/surface' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- ielemnew := 0

      ielemnew = 0

!     --- For all elements ielem in old mesh do

      do ielem = 1, nelem

!     --- ielemnew := ielemnew+1
!         fill first element
!         ielemnew := ielemnew+1
!         fill second element
!         ielemnew := ielemnew+1
!         fill third element
!         ielemnew := ielemnew+1
!         fill fourth element

         ipstart = nmeshd(nelprev+ielem)
         do i = 1, 3
            iedge = nmeshd(ipstart+i)
            if ( iedge>0 ) then
               edge(i,1) = nodenumbers(iedge)       ! 3 edge numbers / a
               edge(i,2) = nodenumbers(iedge)+1     ! 3 edge numbers / b
            else  ! ( iedge<0 )
               edge(i,1) = nodenumbers(-iedge)+1    ! 3 edge numbers / a
               edge(i,2) = nodenumbers(-iedge)      ! 3 edge numbers / b
            end if  ! ( iedge>0 )
         end do
         iface = nodenumbers(nedges+ielem+nelprev)  ! node number of first face
         ielemnew = ielemnew+1

!        --- 1 e1a 2 f_1 6 e3b f_4

         kmeshcnew(1,ielemnew) = kmeshc(1,ielem)
         kmeshcnew(2,ielemnew) = edge(1,1)
         kmeshcnew(3,ielemnew) = kmeshc(2,ielem)
         kmeshcnew(4,ielemnew) = iface
         kmeshcnew(5,ielemnew) = kmeshc(6,ielem)
         kmeshcnew(6,ielemnew) = edge(3,2)
         kmeshcnew(7,ielemnew) = iface+3
         ielemnew = ielemnew+1

!        --- 3 e2a 4 f_2 2 e1b f_5

         kmeshcnew(1,ielemnew) = kmeshc(3,ielem)
         kmeshcnew(2,ielemnew) = edge(2,1)
         kmeshcnew(3,ielemnew) = kmeshc(4,ielem)
         kmeshcnew(4,ielemnew) = iface+1
         kmeshcnew(5,ielemnew) = kmeshc(2,ielem)
         kmeshcnew(6,ielemnew) = edge(1,2)
         kmeshcnew(7,ielemnew) = iface+4
         ielemnew = ielemnew+1

!        --- 5 e3a 6 f_3 4 e2b f_6

         kmeshcnew(1,ielemnew) = kmeshc(5,ielem)
         kmeshcnew(2,ielemnew) = edge(3,1)
         kmeshcnew(3,ielemnew) = kmeshc(6,ielem)
         kmeshcnew(4,ielemnew) = iface+2
         kmeshcnew(5,ielemnew) = kmeshc(4,ielem)
         kmeshcnew(6,ielemnew) = edge(2,2)
         kmeshcnew(7,ielemnew) = iface+5
         ielemnew = ielemnew+1

!        --- 2 f_2 4 f_3 6 f_1 7

         kmeshcnew(1,ielemnew) = kmeshc(2,ielem)
         kmeshcnew(2,ielemnew) = iface+1
         kmeshcnew(3,ielemnew) = kmeshc(4,ielem)
         kmeshcnew(4,ielemnew) = iface+2
         kmeshcnew(5,ielemnew) = kmeshc(6,ielem)
         kmeshcnew(6,ielemnew) = iface
         kmeshcnew(7,ielemnew) = kmeshc(7,ielem)

      end do  ! ielem = 1, nelem

1000  call erclos ( 'mshrefkmeshc7' )

      if ( debug ) then

!     --- Debug information

         call prinin1 ( kmeshcnew, ielemnew, 7, 'kmeshcnew/surface' )
         write(irefwr,*) 'End mshrefkmeshc7'

      end if

      end
