      subroutine mshsmooth2d1 ( npoint, ndim, npelm, nelem, coor, dtdx, &
                                ang, compgrad, knode, kmeshc, kmeshd, &
                                length, indelm )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 26-09-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!         Compute the angles of the elements and their derivatives
!         in case of 2D triangles
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     triangle
!     subdivision
!     smoothing
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, npoint, npelm, compgrad, kmeshd(*), nelem, &
              knode, length, indelm(nelem), kmeshc(*)
      double precision  coor(ndim,*), ang(length), dtdx(length,ndim)

!     ang            o    Angle
!     compgrad       i    Parameter for computing derivatives of angle (1)
!                         or not (0)
!     coor           i    Array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     dtdx           o    Array containing the derivatives of the angles
!     indelm        i/o   Array of length nelem in which elements are marked
!                         of which at least one node is shifted
!     kmeshc         i    kmesh part c, contains the nodal points corresponding
!                         to the elements
!     kmeshd         i    KMESH part d, contains the elements corresponding to
!                         the nodal points
!     knode          i    Shifting node
!     length         i    Help parameter to indicate the length of arrays
!     ndim           i    Dimension of the space.
!     nelem          i    Number of elements
!     npelm          i    Number of points per element
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  i,j,k, ielem, kk, idx(3,3), mark, jnode(3), nrelem, &
               idx1(3)
      double precision l(3), dldx(3,ndim), temp, u(3), &
                       dudx(3), xp1(ndim), xp2(ndim)

!     dldx           ?
!     dudx           Real m x ndim  array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du / dx  (x ) = dudx ( k, j )
!                      j     k
!                    If jdercn = 0 in common block celint, then only
!                    dudx (1,*) is filled because dudx is then a constant
!                    per element
!                    dudx is only filled if jderiv > 0
!     i              Counting variable
!     idx            Array used for roulating the indices
!     idx1           Array for finding the vertices
!     ielem          Element number
!     j              Counting variable
!     jnode          Array containing the node numbers of the vertices
!     k              Counting variable
!     kk             Counting variable
!     l              General loop variable
!     mark           Parameter for marking node
!     nrelem         Number of elements around node
!     temp           Work parameter
!     u              Work parameter
!     xp1            Coordinates of first point of edge
!     xp2            Coordinates of second point of edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data idx / 1,2,3, 2,3,1, 3,1,2  /
      data idx1 / 1,3,5  /
! ======================================================================
!
      call eropen ( 'mshsmooth2d1' )

      kk=1

!     --- determine the nr of elements around knode

      nrelem=kmeshd(knode+1)-kmeshd(knode)

!     --- loop over the elements

      do k=1,nrelem
         ielem=kmeshd(npoint+1+kmeshd(knode)+k)
         indelm(ielem)=1

!        --- determine which nodes are involved in element
!            do j=1,npelm

         do j=1,3
            if (npelm.eq.3) then
               jnode(j)=kmeshc((ielem-1)*npelm+j)
            else if (npelm.eq.7) then
               jnode(j)=kmeshc((ielem-1)*npelm+idx1(j))
            end if
            if (jnode(j).eq.knode) mark=j
         end do

!        --- determine the lengths of the elementsides
!            do j=1,npelm

         do j=1,3

            do i=1,ndim
               if (mark.eq.1) then
                  xp1(i) = coor(i,jnode(idx(j,2)))
                  xp2(i) = coor(i,jnode(idx(j,3)))
               else if (mark.eq.2) then
                  xp1(i) = coor(i,jnode(idx(j,3)))
                  xp2(i) = coor(i,jnode(idx(j,1)))
               else if (mark.eq.3) then
                  xp1(i) = coor(i,jnode(idx(j,1)))
                  xp2(i) = coor(i,jnode(idx(j,2)))
               end if
            end do

            if ( ndim.eq.2 ) then

!           --- determine the lengths

               l(j) =  sqrt( ((xp1(1)-xp2(1))**2d0)+ &
                             ((xp1(2)-xp2(2))**2d0) )

!              --- determine the derivatives of the lengths

               do i=1,ndim
                  dldx(j,i)=(xp1(i)-xp2(i))/l(j)
                  if (mark.eq.1) then
                     if (jnode(idx(j,3)).eq.knode) dldx(j,i)=-dldx(j,i)
                  else if (mark.eq.2) then
                     if (jnode(idx(j,1)).eq.knode) dldx(j,i)=-dldx(j,i)
                  else if (mark.eq.3) then
                     if (jnode(idx(j,2)).eq.knode) dldx(j,i)=-dldx(j,i)
                  end if
               end do
            end if

         end do

!        --- determine the angles of the element
!            do j=1,npelm

         do j=1,3
            u(j)= -(l(j)**2d0 &
                  - l(idx(j,2))**2d0 &
                  - l(idx(j,3))**2d0) &
                  / ( 2d0*l(idx(j,2))*l(idx(j,3)) )
            ang(kk-1+j)=acos(u(j))
         end do

         if ( compgrad.eq.1 ) then

!        --- compute the derivatives of the angles

            do i=1,ndim
               temp=-2d0*( l(2)*dldx(3,i)+dldx(2,i)*l(3) )/ &
                            ((2d0*l(2)*l(3))**2d0)
               dudx(1)=(l(1)**2d0)*temp
               dudx(2)=(2d0*l(2)*dldx(2,i)/ &
                       (2d0*l(2)*l(3)))+l(2)**2d0*temp
               dudx(3)=(2d0*l(3)*dldx(3,i)/ &
                       (2d0*l(2)*l(3)))+l(3)**2d0*temp
               dtdx(kk,i)=-(-dudx(1)+dudx(2)+dudx(3)) / &
                          sqrt(1d0-(u(1)**2d0))
            end do

            do i=1,ndim
               temp=2d0*l(1)*dldx(3,i)/((2d0*l(1)*l(3))**(2d0))
               dudx(1)=2d0*l(2)*dldx(2,i)/((2d0*l(1)*l(3)))- &
                       l(2)**2d0*temp
               dudx(2)=-l(1)**2d0*temp
               dudx(3)=2d0*l(3)*dldx(3,i)/((2d0*l(1)*l(3)))- &
                       l(3)**2d0*temp
               dtdx(kk+1,i)=-((-dudx(1)+dudx(2)+dudx(3))) / &
                            sqrt(1d0-(u(2)**2d0))
            end do

            do i=1,ndim
               temp=2d0*l(1)*dldx(2,i)/((2d0*l(1)*l(2))**(2d0))
               dudx(1)=2d0*l(3)*dldx(3,i)/((2d0*l(1)*l(2)))- &
                       l(3)**2d0*temp
               dudx(2)=-l(1)**2d0*temp
               dudx(3)=2d0*l(2)*dldx(2,i)/((2d0*l(1)*l(2)))- &
                       l(2)**2d0*temp
               dtdx(kk+2,i)=-(-dudx(1)+dudx(2)+dudx(3)) / &
                            sqrt(1d0-(u(3)**2d0))
            end do

         end if
         kk=kk+3

      end do  ! end element loop

      call erclos ( 'mshsmooth2d1' )
      end
