      subroutine mshsmoothing ( skipnod, coornew, &
                                shifted, indelm, map, smth )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 26-09-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Smooth the mesh around the obstacle using Laplacian smoothing
!     or an angle optimization algorithm
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     triangle
!     subdivision
!     smoothing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleconsts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer skipnod(*), shifted(*), indelm(nelem), &
              map(*), smth
      double precision  coornew(ndim,*)

!     coornew       i/o   Array containing the new coordinates of the nodes
!     indelm        i/o   Array of length nelem in which elements are marked
!                         of which at least one node is shifted
!     map            i    Mapping array that is filled if ichoice = 2 or must
!                         be filled if ichoice = 3
!                         map is an integer array of length 5 with the following
!                         contents:
!                         1: mmimap  memory management reference to array imap
!                         2: 130 to indicate the type of array
!                         3: mmrmap  memory management reference to array rmap
!                         4: mmcoor1  memory management reference to array coor1
!                         5: mmcoor2  memory management reference to array coor2
!     shifted        i    Array of npoint long in which the nodes are marked
!                         which are shifted onto the obstacle
!     skipnod       i/o   Array of npoint long in which the nodes are marked
!                         which are not allowed to be moved
!     smth           i    Parameter for Laplacian smoothing (1), angle
!                         optimization (2), both (3) or none (0)
!                         Is only allowed if shft=1
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  inode, i, j, k, smooth(npoint), nrnodes, layer, ielem, &
               angidx, idx(4), compgrad , iter1, iter2, iter3, &
               niter1, niter2, niter3, nlayer, crit, idx1(11), &
               inpelm, istart, ielgrp, sumnod, nrelem
      double precision sum(ndim), dx(ndim), mindx(ndim), ang(length), &
                       dtdx(length,ndim), minang, ang1(length), &
                       markelm(nelem), maxdtdx(3), eps

!     ang            Angle
!     ang1           Angle
!     angidx         Index variable
!     compgrad       Parameter for computing derivatives of angle (1)
!                    or not (0)
!     crit           Counting variable
!     dtdx           Array containing the derivatives of the angles
!     dx             Length of vector in x-direction
!     eps            Local accuracy
!     i              Counting variable
!     idx            Array containing the indices of the non-vertices 2D
!     idx1            Array containing the indices of the non-vertices 3D
!     ielem          Element number
!     ielgrp         element group number
!     inode          Node sequence number
!     inpelm         number of points per element
!     istart         First position used at a part
!     iter1          Counting variable
!     iter2          Counting variable
!     iter3          Counting variable
!     j              Counting variable
!     k              Counting variable
!     layer          Number of layers around the obstacle for smoothing
!     markelm        Array of nelem long containing the minimal angles
!     maxdtdx        Maximal derivative of angle
!     minang         Minimal angle
!     mindx          Minimal displacement
!     niter1         Counting variable
!     niter2         Counting variable
!     niter3         Counting variable
!     nlayer         Number of layers of nodes that should be considered
!                    for smoothing
!     nrelem         Number of elements connected to a node
!     nrnodes        Number of surrounding nodes
!     smooth         array of npoint long which indicates if nodes
!                    should be considered for smoothing (1) or not (0)
!     sum            Help parameter to compute a sum
!     sumnod         Help parameter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHASPRAT2     Check for ill-shaped elements
!     MSHSMOOTH2D1   Find minimal angles and derivatives 2D
!     MSHSMOOTH3D1   Find minimal angles and derivatives 3D
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data idx / 2,4,6,7 /
      data idx1 / 2,4,6,7,8,9,11,12,13,14,15 /
! ======================================================================
!
      call eropen ( 'mshsmoothing' )

      eps=1d-13

!     --- clear arrays

      do i = 1, npoint
         smooth(i)=0
      end do

      ishape=kmeshf(1)
      if ( ishape==7 ) then

!     --- ishape = 7
!         loop over elements

         do i=1, nelem

!        --- loop over non-vertices
!            in case of quadratic elements only the
!            vertices are considered in the smoothing
!            subroutines. The midside nodes and
!            centroid are positioned in mshshiftnod5.f

            do j=1,4
               skipnod(kmeshc((i-1)*7+idx(j)))=3
            end do

         end do

      else if ( ishape==16 ) then

!     --- ishape = 16
!         loop over elements

         do i = 1, nelem

!        --- loop over non-vertices
!            in case of quadratic elements only the
!            vertices are considered in the smoothing
!            subroutines. The midside, midplane nodes and
!            centroid are positioned in mshshiftnod5.f

            do j = 1, 11
               skipnod(kmeshc((i-1)*15+idx1(j)))=3
            end do

         end do

      end if

!     --- get nodes around the shifted nodes
!         loop over nodes

      do i = 1, npoint
         if (shifted(i)==1) then

!        --- find number of nodal points connected to node i

            nrnodes=kmeshe(i+1)-kmeshe(i)

!           --- loop over nodes connected to node i

            do j=1,nrnodes
               inode=kmeshe(npoint+1+kmeshe(i)+j)

!              --- if node is allowed to be shifted then

               if ( skipnod(inode)/=1 .and. skipnod(inode).ne.3 ) then

!              --- node is marked for shifting

                  smooth(inode)=1
                  map(inode)=0

               end if
            end do
         end if
      end do

      nlayer=1

!     --- get several layers of nodes around the obstacle

      do layer=1,nlayer

!     --- loop over nodes

         do i=1,npoint

!        --- if node is not considered earlier in the routine then

            if (smooth(i)==layer) then

!           --- loop over nodal points connected to node i

               nrnodes=kmeshe(i+1)-kmeshe(i)
               do j=1,nrnodes
                  inode=kmeshe(npoint+1+kmeshe(i)+j)
                  if ( skipnod(inode)/=1 .and. &
                       smooth(inode)<=layer.and. &
                       skipnod(inode)/=3  ) then

!                 --- node is marked for shifting

                     smooth(inode)=layer+1
                     map(inode)=0

                  end if
               end do

            end if

         end do

      end do

      if ( smth==1 .or. smth.eq.3 ) then

!     --- apply a laplacian smoothing algorithm on the nodes around the obstacle
!         iteration loop
!         do number of iterations

         do iter1 = 1,1

!        --- loop over nodes

            do i=1,npoint
              if (smooth(i)/=0) then
                 nrnodes=kmeshe(i+1)-kmeshe(i)
                 do j=1,ndim
                    sum(j)=0d0
                 end do

!                --- loop over surrounding nodes

                 sumnod=nrnodes
                 do j=1,nrnodes
                    inode=kmeshe(npoint+1+kmeshe(i)+j)

!                   --- find the geografical center of the surrounding nodes

                     if (skipnod(inode)/=3) then

!                    --- vertex node is found

                        do k=1,ndim
                           sum(k)=sum(k)+coornew(k,inode)
                        end do
                     else

!                    --- midpoint or centroid is found
!                        number of surrounding nodes is reduced by 1

                        sumnod=sumnod-1
                     end if

                 end do

!                --- shift the node to the center of the surrounding nodes

                 do k=1,ndim
                    coornew(k,i)=sum(k)/sumnod
                 end do

!                --- loop over the elements

                 nrelem=kmeshd(i+1)-kmeshd(i)
                 do k=1,nrelem
                    ielem=kmeshd(npoint+1+kmeshd(i)+k)
                    indelm(ielem)=1
                 end do

              end if
            end do

         end do

      end if

      if ( smth==2 .or. smth.eq.3 ) then

!     --- use an angle optimization algorithm

         istart=0
         do ielgrp = 1, nelgrp

            inpelm = Topgrp(ielgrp)%inpelm
            inelem = Topgrp(ielgrp)%inelem
            ishape = kmeshf(ielgrp)
            if ( ishape==3 .or. ishape.eq.7 .or. &
                 ishape==11 .or. ishape.eq.16 ) then

               if ( ndim==3 ) then

!              --- find the angles of the elements around the obstacle

                  call mshasprat2 ( npoint, ndim, nelem, kmeshc(istart), &
                                    kmeshd, coornew, markelm, smooth, &
                                    indelm, inpelm )

!                 --- loop over elements

                  do ielem=1,inelem

!                 --- if element is a boundary element and if the minimal angle
!                     is less than 0.2 then

                     if (markelm(ielem)>0d0 .and. &
                         markelm(ielem)<0.2d0) then

!                    --- loop over nodes per element

                        do j=1,inpelm
                           inode=kmeshc(istart+(ielem-1)*inpelm+j)

!                          --- node is considered in smoothing algorithm

                           if ( smooth(inode)/=0 ) &
                                             smooth(inode)=nlayer+2
                        end do

                     end if

                  end do

                  crit=nlayer+2

               else if ( ndim==2 ) then

                  crit=1

               end if

!              --- set maximum number of iterations

               niter1=4
               niter2=2
               niter3=5

!              --- apply an angle optimization algorithm to the nodes around the
!                  obstacle
!                  start first iteration loop

               do iter1 = 1, niter1

!              --- loop over nodes

                  do inode=1,npoint

                     if (smooth(inode)>=crit) then

!                    --- start second iteration loop

                        do iter2=1, niter2

!                       --- compute derivatives (compgrad=1) or not (compgrad=0)

                           compgrad=1

!                          --- compute angles and their derivatives

                           if (ndim==2) then

                              call mshsmooth2d1 ( npoint, ndim, inpelm, &
                                                  nelem, coornew, dtdx, &
                                                  ang, compgrad, inode, &
                                                  kmeshc(istart+1), &
                                                  kmeshd, length, &
                                                  indelm )

                           else if (ndim==3) then

                              call mshsmooth3d1 ( npoint, ndim, inpelm, &
                                                  coornew, dtdx, ang, &
                                                  compgrad, inode, &
                                                  kmeshc(istart+1), &
                                                  kmeshd, length )

                           end if

!                          --- compute the minimal angle
!                              clear array

                           minang=pi
                           do j=1,ndim
                              maxdtdx(j)=0d0
                           end do

!                          --- loop over number of angles

                           do i=1,length
                              if( ang(i)<minang ) then

!                             --- minimal angle and its index are stored

                                 angidx=i
                                 minang=ang(i)
                              end if
                              do j=1,ndim
                                 if ( abs(maxdtdx(j))< &
                                      abs(dtdx(i,j)) ) then

!                                --- find steepest descent

                                   maxdtdx(j)=dtdx(i,j)
                                 end if
                              end do
                           end do

!                          --- check sign

                           do j=1,ndim
                              if(dtdx(angidx,j)<0d0 .and. &
                                 maxdtdx(j)>0d0) &
                                          maxdtdx(j)=-maxdtdx(j)
                              if(dtdx(angidx,j)>0d0 .and. &
                                 maxdtdx(j)<0d0) &
                                          maxdtdx(j)=-maxdtdx(j)
                           end do

!                          --- compute the minimal step size

                           do i=1,ndim
                              mindx(i)=1d0
                           end do

!                          --- loop over angles

                           do i=1,length

!                          --- loop over dimensions

                              do j=1,ndim

!                             --- find minimal step size

                                 if ( i/=angidx .and. &
                                      abs(maxdtdx(j)-dtdx(i,j))> &
                                      eps ) then
                                    dx(j)=-( ang(angidx)-ang(i) )/ &
                                          ( maxdtdx(j)- dtdx(i,j) )
                                    if ( abs(dx(j))<= &
                                         abs(mindx(j))) then
                                        mindx(j)=dx(j)
                                    end if
                                 end if
                              end do
                           end do

!                          --- start third iteration loop

                           do iter3=1,niter3

!                          --- shift the node

                              do i=1,ndim
                                 coornew(i,inode)=coornew(i,inode)+mindx(i)
                              end do

!                             --- check if solution satisfies the criteria

                              compgrad=0

!                             --- compute nr of elements around node to be
!                             --- shifted

                              if (ndim==2) then
                                 call mshsmooth2d1 ( npoint, ndim, &
                                                     inpelm, nelem, &
                                                     coornew, dtdx, ang1, &
                                                     compgrad, inode, &
                                                     kmeshc(istart+1), &
                                                     kmeshd, length, &
                                                     indelm )
                              else if (ndim==3) then
                                 call mshsmooth3d1 ( npoint, ndim, &
                                                     inpelm, coornew, dtdx, &
                                                     ang1, compgrad, &
                                                     inode, &
                                                     kmeshc(istart+1), &
                                                     kmeshd, length )
                              end if

!                             --- loop over angles

                              do i=1,length

!                             --- if angle is smaller then the smallest angle

                                 if ( ang1(i)<minang ) then

!                                --- undo shifting of node

                                    do j=1,ndim
                                       coornew(j,inode)= &
                                               coornew(j,inode)-mindx(j)
                                    end do

                                    if ( iter3==niter3 )  then

!                                   --- if maximum nr of iterations is reached
!                                   --- then
!                                       stop trying and move on to next node

                                       goto 300

                                    else

!                                   --- else take step = 0.5*step and try again

                                       do j=1,ndim
                                          mindx(j)=mindx(j)*0.5d0
                                       end do
                                       goto 100

                                    end if
                                 end if

                              end do   ! end of loop over angles

!                             --- shifting is accepted, continue the second
!                             --- iteration
!                                 loop

                              goto 200

100                        end do  ! end of third iteration loop

200                     end do  ! end of second iteration loop

                     end if

300               end do  ! end of nodal loop

               end do  ! end of first iteration loop

            end if  ! end check the angles

            istart=istart+inelem*inpelm

         end do ! end element groups

      end if  ! end check the angles

      call erclos ( 'mshsmoothing' )
      end
