      subroutine mshsurfaces
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 15-11-2012 Long integers
!        version  2.0  date 26-01-2009 Remove mmsurffaces
!        version  1.4  date 24-11-2004 Fortran 90
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create all faces corresponding to a three-dimensional mesh with volumes
!     Check if all outer faces correspond to surfaces, if not create extra
!     surfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     face
!     mesh
!     outer_boundary
!     surface
!     compound_surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulework
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nouterfaces, &
              nsurfsnew, leniwork2, lenextsurffac, leniworksurfs, &
              nextra, inpelm, lenkmeshn, &
              nsurfsold, ifile, lenkmeshs
      integer, allocatable :: outerfaces(:), kmeshsold(:), kmeshnold(:)
      integer, allocatable :: extsurfface(:), ipoint(:), iworksurfs(:)

!     extsurfface    Array of length lenextsurffac containing the
!                    face sequence numbers of the new surfaces to be
!                    created
!     ifile          input parameter for subroutine prinsurfaces
!     inpelm         number of points per element
!     ipoint         Array of estimated length 10*nsurfsold that is used to
!                    store the end addresses of each new surface in array
!                    extsurffaces
!                    The positions 1 to nsurfs get the value 0
!     iworksurfs     array of length nsurfs x nsurfs= &
!                    Is initially filled with zeros
!                    If a surface contains subsurfaces (i.e. the extra
!                    surfaces) then the sequence numbers of these
!                    subsurfaces are stored in iworksurfs(surface,.)
!                    sequentially
!     kmeshnold      Old array kmeshn
!     kmeshsold      Old array kmeshs
!     lenextsurffac  Length of array extsurfface
!     leniwork2      Length of array iwork2
!     leniworksurfs  Length of array iworksurfs
!     lenkmeshn      Length of array kmeshn
!     lenkmeshs      Length of array kmeshs
!     nextra         Number of faces that are outer faces, but do not
!                    correspond to a surface
!     nouterfaces    Number of outer faces (length of outerfaces)
!     nsurfsnew      Last surface sequence number used
!     nsurfsold      Number of surfaces in the mesh read
!     outerfaces     Array containing the face numbers of the outer faces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     MSHSURFACES1   If there are faces that belong to more surfaces then
!                    split surfaces in subsurfaces and make connected surfaces
!     MSHSURFACES11  Extend array kmeshn with extra surfaces created in case of
!                    double surfaces
!     MSHSURFACES12  Fill array kmeshr for composite surfaces
!     MSHSURFACES14  Refill kmesh part s in case new surfaces have been created
!     MSHSURFACES2   If there are outer faces not belonging to a surface then
!                    create new surface
!     MSHSURFACES3   Extend array kmeshn with extra surface consisting of all
!                    outer faces that are not yet in a surface
!     PR0012         Print accumulated array
!     PRINSURFACES   Print the contents of array kmeshn either to the standard
!                    output file a specified file or a set of specified files
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLKMESHS  Make relation between kmeshs and kmeshs2
!     SEPFILLNMESHF  Fill array nmeshf
!     SEPFILLNMESHG  Fill array nmeshg
!     SEPFILLNMESHJ  Fill array nmesh part j
!     SEPFILLNMESHK  Fill array nmesh part k
!     TOPOUTERFAC    Find all outer faces of a volume by storing only those
!                    faces corresponding to one element
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2371   No volumes in the mesh
!    2630   Outer faces found that are not yet connected to a surface
!    2646   Compound surfaces have been filled before (not implemented yet)
!    2685   Surfaces overlap
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Create all faces
!     Detect which faces are outer faces
!     Compute face numbers corresponding to given surfaces
!     If ( there are faces that belong to more surfaces ) then
!        split surfaces in subsurfaces and make connected surfaces
!     If ( there are outer faces not belonging to a surface ) then
!        create new surface
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsurfaces' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsurfaces'
         write(irefwr,1) 'nsurfs ', nsurfs
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( nvolms==0 ) then

!     --- No volumes found

         call errsub ( 2371, 0, 0, 0 )
         go to 1000

      end if

!     --- Create all faces and find all elements connected to the faces

      if ( kelmaa==0 ) call sepfillkmeshaa
      if ( nmesh(36)==0 ) call sepfillnmeshf

!     --- Detect which faces are outer faces
!         First reserve temporary space for array outerfaces

      allocate ( outerfaces(nfaces), stat = error )
      if ( error/=0 ) call eralloc ( error, nfaces, 'outerfaces' )
      if ( ierror/=0 ) go to 1000

!     --- actual call

      call topouterfac ( nmeshf, nfaces, nouterfaces, outerfaces )
      if ( ierror/=0 ) go to 1000

!     --- Compute face numbers corresponding to given surfaces
!         First create arrays nmeshj and k and activate array kmeshn

      if ( nmesh(40)==0 ) call sepfillnmeshj
      if ( nmesh(41)==0 ) call sepfillnmeshk

!     --- If ( there are outer faces not belonging to a surface ) then
!            create new surface

      call mshsurfaces2 ( nouterfaces, outerfaces, &
                          nmeshj, nextra, inpelm )
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,*) 'nextra ', nextra

      if ( nextra>0 ) then

!     --- nextra>0, there are outer surfaces not yet created

         call errwar ( 2630, 0, 0, 0 )

      end if  ! ( nextra>0 )
      nsurfsold = nsurfs

      if ( nextra>0 ) then

!     --- nextra>0, extend kmeshn with one extra surface containing all
!                     outer faces
!                     At this moment we assume that all faces have the same
!                     number of nodes (inpelm)
!                     kmeshn is extended with 2 + nextra*inpelm positions
!                     First the old kmeshn is copied to the end of ibuffr

         lenkmeshn = size ( kmeshn )

         allocate ( kmeshnold(lenkmeshn), stat = error )
         if ( error/=0 ) call eralloc ( error, lenkmeshn, 'kmeshnold' )
         if ( ierror/=0 ) go to 1000

!        --- Copy old kmeshn to kmeshnold

         kmeshnold(1:lenkmeshn) = kmeshn

!        --- Reserve space for new array kmeshn

         deallocate ( km(iseqkm)%kmeshn )
         length = lenkmeshn+2+nextra*inpelm
         allocate ( km(iseqkm)%kmeshn(length) )
         km(iseqkm)%kmesh(28) = length
         kmeshn => km(iseqkm)%kmeshn

!        --- Fill nmeshg if necessary

         if ( nmesh(37)==0 ) call sepfillnmeshg

!        --- Fill new array kmeshn

         call mshsurfaces3 ( nouterfaces, outerfaces, nmeshj, &
                             kmeshnold, nextra )
         km(iseqkm)%kmesh(12) = nsurfs

!        --- Remove temporary array at end of ibuffr

         deallocate ( kmeshnold, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshnold' )
         if ( ierror/=0 ) go to 1000

      end if  ! ( nextra>0 )

      deallocate ( outerfaces, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'outerfaces' )
      if ( ierror/=0 ) go to 1000

!     --- If ( there are faces that belong to more surfaces ) then
!            split surfaces in subsurfaces and make connected surfaces
!         The next temporary arrays are created
!         extsurfface    lenextsurffac
!         iworksurfs     nsurfs*nsurfs
!         ipoint         10*nsurfs

      leniwork2 = 5*nfaces
      lenextsurffac = 2*nfaces
      leniworksurfs = nsurfs*nsurfs

      allocate ( iwork(nfaces), iwork1(nfaces), iwork2(leniwork2), &
                 ihelp(nsurfs), extsurfface(lenextsurffac), &
                 iworksurfs(leniworksurfs), ipoint(10*nsurfs), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, 2*nfaces+leniworksurfs+nsurfs+lenextsurffac, &
                       'iwork' )
      if ( ierror/=0 ) go to 1000

      call mshsurfaces1 ( nmeshj, extsurfface, &
                          leniwork2, lenextsurffac, nsurfsold, &
                          nsurfsnew, iworksurfs, &
                          ipoint, leniworksurfs )
      if ( ierror/=0 ) go to 1000

      if ( nsurfsnew>nsurfs ) then

!     --- Extra surfaces have been created, give warning

         call errwar ( 2685, 0, 0, 0 )

!        --- Fill new array kmeshn
!            At this moment we assume that all faces have the same
!            number of nodes (inpelm)

         lenkmeshn = size ( kmeshn )

         allocate ( kmeshnold(lenkmeshn), stat = error )
         if ( error/=0 ) call eralloc ( error, lenkmeshn, 'kmeshnold' )
         if ( ierror/=0 ) go to 1000

!        --- Copy old kmeshn to kmeshnold

         kmeshnold(1:lenkmeshn) = kmeshn

!        --- Reserve space for new array kmeshn

         nextra = nsurfsnew-nsurfs
         length = lenkmeshn+2*nextra+ipoint(nsurfsnew)*inpelm
         deallocate ( km(iseqkm)%kmeshn )
         allocate ( km(iseqkm)%kmeshn(length) )
         km(iseqkm)%kmesh(28) = length
         kmeshn => km(iseqkm)%kmeshn
         kmeshnglob => kmeshn

!        --- Fill nmeshg if necessary

         if ( nmesh(37)==0 ) call sepfillnmeshg

         call mshsurfaces11 ( kmeshnold, nsurfsnew, ipoint, extsurfface )
         km(iseqkm)%kmesh(12) = nsurfs

!        --- Remove temporary array kmeshnold

         deallocate ( kmeshnold, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshnold' )
         if ( ierror/=0 ) go to 1000

      end if

      if ( nsurfsnew>nsurfs .or. lenextsurffac>0 ) then

!     --- leniworksurfs>0, there are compound curves
!         Fill kmesh part r

         if ( kelmr/=0 ) then

!        --- Error: compound surfaces have already been filled

            call errsub ( 2646, 0, 0, 0 )
            go to 1000

         end if  ! ( kelmr/=0 )

!        --- Create space for permanent array kmeshr

         length = 2*nsurfs+1+leniworksurfs
         allocate ( km(iseqkm)%kmeshr(length) )
         km(iseqkm)%kmesh(32) =  length
         kelmr =>km(iseqkm)%kmesh(32)
         kmeshr => km(iseqkm)%kmeshr

!        --- Create array kmeshr

         call mshsurfaces12 ( ihelp, nsurfs, nsurfsold, &
                              kmeshr, iworksurfs, leniworksurfs )

      end if  ! ( leniworksurfs>0 )

      deallocate ( iwork, iwork1, iwork2, ihelp, extsurfface, &
                   ipoint, iworksurfs, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

      if ( nextra>0 .or. nsurfs>nsurfsold ) then

!     --- nextra>0 or leniworksurfs>0, i.e. extra surfaces have been
!         created
!         Compute face numbers corresponding to given surfaces
!         First delete old array nmeshj, then create again

         deallocate ( km(iseqkm)%nmeshj )
         nullify(nmeshj)
         call sepfillnmeshj

         if ( kelms/=0 ) then

!        --- kelms/=0, i.e. kmeshs exists, refill kmeshs
!            First copy the old kmeshs to kmeshsold

            lenkmeshs = size ( kmeshs )
            allocate ( kmeshsold(lenkmeshs), stat = error )
            if ( error/=0 ) call eralloc ( error, lenkmeshs, 'kmeshsold' )
            if ( ierror/=0 ) go to 1000

            kmeshsold(1:lenkmeshs) = kmeshs

!           --- Next create new array kmeshs

            lenkmeshs = lenkmeshs+9*(nsurfs-nsurfsold) ! new length
            deallocate ( km(iseqkm)%kmeshs )
            allocate ( km(iseqkm)%kmeshs(lenkmeshs) )
            km(iseqkm)%kmesh(33) = lenkmeshs
            kmeshs => km(iseqkm)%kmeshs
            call sepfillkmeshs ( km(iseqkm)%kmeshs(nsurfs+2) )

            call mshsurfaces14 ( nsurfsold, nsurfs, kmeshsold, kmeshs )

            deallocate ( kmeshsold, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'kmeshsold' )
            if ( ierror/=0 ) go to 1000

         end if  ! ( kelms/=0 )

      end if  ! ( nextra>0 .or. leniworksurfs>0 )

1000  if ( debug ) then

!     --- Debug information

         if ( kelmr/=0 ) then

!        --- compound surfaces

            write(irefwr,*)' Subsurface numbers of surfaces'
            call pr0012 ( nsurfs, kmeshr, kmeshr )

         end if  ! ( kelmr/=0 )

         ifile = 2
         call prinsurfaces ( nsurfs, kmeshn, ifile, 'sn' )
         write(irefwr,*) 'End mshsurfaces'

      end if
      call erclos ( 'mshsurfaces' )

      end subroutine mshsurfaces
