      subroutine mshsurfcurvs ( surface_curvs, lensurfcur, nsurfs, &
                                ncurvs, kmeshs, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-03-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store all "single" curves corresponding to outer surfaces in array
!     surface_curvs
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     surface
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer surface_curvs(*), lensurfcur, nsurfs, ncurvs, &
              kmeshs(*), iwork(*)

!     iwork          i    integer work array of length nsurfs
!                         if iwork(i) < 0, surface i is not an outer surface
!     kmeshs         i    Kmesh part s, contains information about the various
!                         surfaces. See the PG 13.1
!     lensurfcur     i    Declared length of array surface_curvs
!     ncurvs         i    Number of curves in mesh
!     nsurfs         i    number of surfaces
!     surface_curvs  o    Contains the number of "single" curves coupled
!                         to all outer surfaces in the following way
!                         Pos. 1 to nsurfs: starting addresses for second part
!                         For each surface in second part:
!                         Pos. 1: number of single curves
!                         Followed by curve numbers (all positive)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, isurnr, icurnr, nouterc, i, iwork1(ncurvs), &
              ipstart, numcrv

!     i              Counting variable
!     icurnr         Curve sequence number
!     ipstart        Starting address in array
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     iwork1         integer work array of length ncurvs
!                    iwork1(i) contains the number of times curve i is present
!                    in the surface
!     nouterc        Number of outer curves
!     numcrv         number of parts where curves are given for generation
!                    of boundary elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     istart := nsurfs+1
!     for all surfaces isurnr do
!        surface_curvs(isurnr) := istart
!        if ( isurnr is inner surface ) then
!            surface_curvs(istart) := 0
!            istart := istart+1
!        else ( isurnr is outer surface )
!           Clear array iwork1 (1:ncurvs)
!           Store number of times a curve is coupled to the surface in iwork1
!           nouterc := 0
!           For all curves icurnr do
!              if ( single curve found ) then
!                 nouterc :=  nouterc+1
!                 surface_curvs(istart+nouterc) := icurnr
!           surface_curvs(istart) := nouterc
!           istart := istart+1+nouterc
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsurfcurvs' )

!     --- istart := nsurfs+1

      istart = nsurfs+1

!     --- for all surfaces isurnr do

      do isurnr = 1, nsurfs

!     --- surface_curvs(isurnr) := istart

         surface_curvs(isurnr) = istart
         if ( iwork(isurnr).lt.0 ) then

!        --- isurnr is inner surface
!            surface_curvs(istart) := 0
!            istart := istart+1

            surface_curvs(istart) = 0
            istart = istart+1

         else

!        --- isurnr is outer surface
!            Clear array iwork1 (1:ncurvs)

            do i = 1, ncurvs
               iwork1(i) = 0
            end do

!           --- Store number of times a curve is coupled to the surface in
!               iwork1

            ipstart = kmeshs(isurnr)
            numcrv = kmeshs(isurnr+1)-ipstart
            do i = 1, numcrv
               icurnr = abs(kmeshs(ipstart+i))
               iwork1(icurnr) = iwork1(icurnr)+1
            end do

!           --- nouterc := 0
!               For all curves icurnr do

            nouterc = 0
            do icurnr = 1, ncurvs

               if ( iwork1(icurnr).eq.1 ) then

!              --- iwork1(icurnr) = 1, single curve found
!                  nouterc :=  nouterc+1
!                  surface_curvs(istart+nouterc) := icurnr

                  nouterc =  nouterc+1
                  surface_curvs(istart+nouterc) = icurnr

               end if

            end do

!           --- surface_curvs(istart) := nouterc
!               istart := istart+1+nouterc

            surface_curvs(istart) = nouterc
            istart = istart+1+nouterc

         end if
         if ( istart-1.gt.lensurfcur ) then

!        --- estimated length of array surface_curvs too small

            call errint ( istart-1, 1 )
            call errint ( lensurfcur, 2 )
            call errchr ( 'surface_curvs', 1 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if

      end do

1000  call erclos ( 'mshsurfcurvs' )
      end
