      subroutine mshusermesh
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.8  date 27-11-2012 Long integers
!        version  1.7  date 26-01-2009 New calls to mshsurfaces and mshcurves
!        version  1.6  date 29-01-2005 Skip creation of curves in quadratic case
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create extra surfaces, curves and user points in a user mesh if necessary
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     outer_boundary
!     surface
!     curve
!     user_point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lncurvstosurfs
      logical fillcurves
      integer, allocatable :: curvstosurfs(:)

!     curvstosurfs   Array of size nsurfs x nsurfs
!                    curvstosurfs(i,j) contains the j-th curve
!                    corresponding to surface i
!                    if 0 no more curves can be found
!     fillcurves     If true curves must be filled
!     lncurvstosurfs Length of array curvstosurfs
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCURVES      Create all curves that can be seen as intersection of
!                    more surfaces
!     MSHFILLKMESHQ  Check if surfaces have more than one intersecting curve
!                    with other surfaces.
!                    If so create compound curves by combination of such curves
!     MSHFILLKMESHS  Fill kmesh part s from the information in kmesh part n
!     MSHSURFACES    Create all faces corresponding to a three-dimensional mesh
!                    with volumes. Check if all outer faces correspond to
!                    surfaces, if not create extra surfaces
!     PR0008         Print information of curves
!     SEPPRINKMESHS  Print the contents of array kmeshs
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshusermesh' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshusermesh'
         write(irefwr,*) 'nsurfs, nvolms ', nsurfs, nvolms

      end if
      if ( ierror/=0 ) go to 1000

      if ( nvolms>0 ) then

!     --- nvolms>0, so there are volumes
!         Extend kmeshn if necessary

         call mshsurfaces
         if ( debug ) write(irefwr,*) 'nsurfs ', nsurfs
         if ( ierror/=0 ) go to 1000

      end if

      fillcurves = nsurfs>0
      if ( fillcurves ) then

!     --- nsurfs>0, check if linear elements are used

         if ( ndim==2 ) then

!        --- ndim = 2: Linear elements, implies 3 or 4 points

            fillcurves = npelm<=4

         else

!        --- ndim = 3:Linear elements, implies 4 or 8 points

            fillcurves = npelm<=8

         end if  ! ( ndim==2 )

      end if  ! ( fillcurves )

      if ( fillcurves ) then

!     --- nsurfs>0, so there are surfaces
!         Extend kmeshm if necessary
!         At this moment only in case of linear elements
!         The next array is made semi-permanent, it is removed later on
!         curvstosurfs  4 nsurfs^2     ipcurvstosurfs
!         At this moment we restrict ourselves to linear elements

         lncurvstosurfs = max(4*nsurfs**2,4000)

         allocate ( curvstosurfs(lncurvstosurfs), stat = error )
         if ( error/=0 ) call eralloc ( error, lncurvstosurfs, 'curvstosurfs' )
         if ( ierror/=0 ) go to 1000

         call mshcurves ( curvstosurfs, lncurvstosurfs/nsurfs )

      end if  ! ( fillcurves )

      if ( fillcurves .and. kelms==0 ) then

!     --- There are surfaces but kmesh part s has not been filled
!         Fill kmeshs from information in kmeshn

         call mshfillkmeshs ( curvstosurfs, lncurvstosurfs )
         if ( ierror/=0 ) go to 1000

      end if

!     --- Remove temporary arrays

      deallocate ( curvstosurfs, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'curvstosurfs' )

      if ( fillcurves ) then

!     --- Fill kmeshq

         call mshfillkmeshq

      end if  ! ( fillcurves )

1000  if ( debug ) then

!     --- Debug information

         if ( kelmm/=0 ) call pr0008 ( kmeshm, ncurvs )
         call sepprinkmeshs
         write(irefwr,*) 'End mshusermesh'

      end if
      call erclos ( 'mshusermesh' )

      end subroutine mshusermesh
