      subroutine sepob3dmarkel2 ( numelmobst, ielemobstlc, lstore, &
                                  iobst, nx, ny, nz, inpelm, leniobgrid )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all elements that intersect the obstacle but do not have a node
!     inside the obstacle
!     3d case
!     Old subroutine mshob3dmarkel2
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     element
!     obstacle
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      use sepmodulegen
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numelmobst, ielemobstlc(*), lstore, iobst, &
              nx, ny, nz, inpelm, leniobgrid

!     ielemobstlc      i  Array containing information about the elements in an
!                         obstacle
!                         The first numobst positions contain the starting
!                         positions of each obstacle sequentially
!                         Per obstacle the first three positions contain
!                         1. The number of elements that are completely inside
!                               the obstacle
!                         2. The number of elements that are not completely
!                               inside the element but with all vertices inside
!                               the element
!                         3. The number of elements that are partly inside
!                               the obstacle
!                               elements that contain only boundary points of
!                               the obstacle are not considered
!                         These positions are followed by the element numbers
!                         in the sequence defined by the first three positions
!     inpelm         i    number of points per element
!     iobst          i    Obstacle sequence number
!     leniobgrid     i    Declared length of last dimension of iobgrid3d
!     lstore         o    number of entries in array istore
!     numelmobst     i    Number of entries in array ielemobst
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     nz             i    number of points in z direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ncell, l, k, ielemobs, inode, &
              nstore, ielem, ipstart, nedge_elem, j, iedge, jpstart, &
              nelem_edge, jelem, nnode_edge, inode1, inode2, &
              icell(8*leniobgrid)
      double precision xmin, ymin, zmin, dx, dy, dz, eps, &
                       x1(3), x2(3), x3(3), xp1(3), xp2(3), &
                       fact1, fact2(2)

!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     dz             length of vector in z-direction
!     eps            Local accuracy
!     fact1          multiplication factor
!     fact2          multiplication factors for triangle
!     i              Counting variable
!     icell          Array containing the surface parts of the obstacle that
!                    must be considered for computing the cross sections
!     iedge          edge sequence number
!     ielem          Element number
!     ielemobs       Element number in obstacle
!     inode          Node sequence number
!     inode1         First node in edge
!     inode2         Second node in edge
!     ipstart        Starting address in array
!     j              Counting variable
!     jelem          do loop variable with respect to the elements
!     jpstart        Starting address in array
!     k              Counting variable
!     l              Counting variable
!     ncell          Number of curve parts that must be used to compute
!                    cross sections
!     nedge_elem     number of edges corresponding to element
!     nelem_edge     number of elements corresponding to edge
!     nnode_edge     number of nodes corresponding to edge
!     nstore         Counter with respect to array istore
!     x1             coordinates of first point in triangle
!     x2             coordinates of second point in triangle
!     x3             coordinates of third point in triangle
!     xmin           Minimum x-value
!     xp1            coordinates of first point of edge
!     xp2            coordinates of second point of edge
!     ymin           Minimum y-value
!     zmin           Minimum z-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHCROSSSURF   Find the intersection of the lines xp1-xp2 and the
!                    triangle x1,x2,x3
!     MSHOB3DINTLIN1 Fill array icell with indices of surface parts of the
!                    obstacle that are possibly intersected by the line x1-x2
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    2254   inpelm # 3
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Extract xmin,xmax,ymin,ymax from obstacle array
!     for i := 1 (1) numelmobst
!        ielem := next element in ielemobst
!        for j := 1 (1) number of edges of the element
!           iedge := next edge
!           for k := 1 (1) number of elements of the edge
!              jelem := next element
!              if (jelem outside the mesh ) then
!                 store edge in istore
!     Remove all edges in istore that do not intersect the obstacle
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepob3dmarkel2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepob3dmarkel2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( inpelm/=3 ) then

!     --- Only triangles have been implemented

         call errsub ( 2254, 0, 0, 0 )
         go to 1000

      end if

!     --- Extract xmin,xmax,ymin,ymax from obstacle array

      xmin = rmeshx(1,iobst)
      ymin = rmeshx(3,iobst)
      zmin = rmeshx(5,iobst)
      dx   = rmeshx(7,iobst)
      dy   = rmeshx(8,iobst)
      dz   = rmeshx(9,iobst)
      eps  = accuracy_obstacle

      nstore = 0

!     --- Loop over the elements in the obstacle

      do i = 1, numelmobst

!     --- element ielem

         ielem = ielemobstlc(i)
         ipstart = nmeshd(ielem)
         nedge_elem = nmeshd(ipstart)

!        --- Loop over the edges of the element

         do j = 1, nedge_elem

!        --- edge iedge

            iedge = abs(nmeshd(ipstart+j))
            jpstart = nmeshh(iedge)
            nelem_edge = nmeshh(jpstart)

!           --- Loop over the elements of the edge

            do k = 1, nelem_edge

!           --- element jelem

               jelem = nmeshh(jpstart+k)
               if ( iwork(jelem)==0 ) then

!              --- element outside the mesh, store edge in istore

                  nstore = nstore+1
                  istore(nstore) = iedge
                  go to 100

               end if

            end do

100      end do

      end do

!     --- Remove all edges in istore that do not intersect the obstacle

      lstore = 0
      do j = 1, nstore

!     --- edge iedge

         iedge = istore(j)
         ipstart = nmeshc(iedge)
         nnode_edge = nmeshc(ipstart)

!        --- store initial and end node in inode1/2

         inode1 = nmeshc(ipstart+1)
         inode2 = nmeshc(ipstart+nnode_edge)
         xp1(1) = coor(1,inode1)
         xp1(2) = coor(2,inode1)
         xp1(3) = coor(3,inode1)
         xp2(1) = coor(1,inode2)
         xp2(2) = coor(2,inode2)
         xp2(3) = coor(3,inode2)

!        --- Fill array icell with indices of surface parts of the obstacle
!            that are possibily intersected by the line x1-x2

         call mshob3dintlin1 ( xp1, xp2, xmin, ymin, zmin, &
                               nx, ny, nz, iobgrid3d, icell, ncell, &
                               dx, dy, dz, leniobgrid )
         if ( ierror/=0 ) go to 1000

         if ( ncell>8*leniobgrid ) then

!        --- ncell too large

            call errint ( ncell, 1 )
            call errint ( 8*leniobgrid, 1 )
            call errchr ( 'icell', 1 )
            call errsub ( 550, 2, 0 , 1 )
            go to 1000

         end if

!        --- For i := 1 (1) ncell do

         do i = 1, ncell

!        --- Store coordinates in x, y and z

            ielemobs = icell(i)
            inode = kmeshcobst(1,ielemobs)
            do l = 1, 3
               x1(l) = coorobst(l,inode)
            end do
            inode = kmeshcobst(2,ielemobs)
            do l = 1, 3
               x2(l) = coorobst(l,inode)
            end do
            inode = kmeshcobst(3,ielemobs)
            do l = 1, 3
               x3(l) = coorobst(l,inode)
            end do

!           --- Check if line xp1-xp2 intersects triangle and
!               compute intersection

            call mshcrosssurf ( xp1, xp2, x1, x2, x3, fact1, fact2, &
                                eps )
            if ( ierror/=0 ) go to 1000

            if ( fact1>-eps .and. fact1<1d0+eps ) then

!           --- the line x1-x2 intersects the triangle (x1,x2,x3)

               lstore = lstore+1
               istore(lstore) = iedge
               go to 200

            end if

         end do

200   end do

1000  if ( debug ) then

!     --- Debug information

         call prinin ( istore, lstore, 'edges' )
         write(irefwr,*) 'End sepob3dmarkel2'

      end if
      call erclos ( 'sepob3dmarkel2' )

      end
