      subroutine sepobsfl ( iobgrid, nx, ny, iobst, lencurvs, &
                            iobgrid1, leniobgrid1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iobgrid with grid information of one obstacle
!     Old subroutine mshobsfl
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmoduledebug
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, iobgrid(0:nx-1,0:ny-1), &
              iobst, lencurvs, leniobgrid1, iobgrid1(leniobgrid1)

!     iobgrid        o    array of length nx x ny that is filled as follows:
!                         The region covered is defined by xmin,xmax, ymin,ymax
!                         The cell size is dx x dy
!                         Cell i,j refers to the coordinates:
!                         (xmin+(i-1)dx,ymin+(j-1)dy) x (xmin+idx,ymin+jdy)
!                         iobgrid(i,j) = -1: cell completely in obstacle
!                         iobgrid(i,j) = 0: cell outside obstacle
!                         iobgrid(i,j) = k>0: boundary part crosses cell
!                         The value of k indicates starting address in iobgrid2
!     iobgrid1       o    Contains for all cells referenced by iobgrid:
!                         number of edges crossing the cell, followed by edge
!                         numbers
!     iobst          i    sequence number of obstacle
!     lencurvs       i    Overestimate of the number of points along the new
!                         curves
!     leniobgrid1    i    Declared length of array iobgrid1
!     nx             i    number of cells in x-direction
!     ny             i    number of cells in y-direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, n, icurnr, leniwork
      double precision xmin, ymin, dx, dy, ymax, curvescp(2,lencurvs)
      integer, allocatable :: iwork(:,:), numcells(:,:)
      logical closed

!     closed         If true the boundary is closed
!     curvescp       Array to store all coordinates of one curve
!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     i              Counting variable
!     icurnr         Curve sequence number
!     iwork          Contains the cell numbers (i,j) in the sequence
!                    of the curve parts
!                    Must be used in connection with numcells
!     leniwork       Declared length of array iwork
!     n              number of nodes in curve
!     numcells       numcells(1,k) contains the number of cells curvepart
!                    k intersects
!                    numcells(1,k) contains the starting address of
!                    the cell numbers in iwork
!     xmin           Minimum x-value
!     ymax           Maximum y-value
!     ymin           Minimum y-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSFLEDGE   Fill array iobgrid with number of edges per cell
!                    array numcells with starting addresses and number of cells
!                    array iwork with cell pairs
!     MSHOBSFLINSIDE Fill cells inside and outside the obstacle
!     MSHOBSFLIOBGRD Fill iobgrid from information stored in iobgrid,
!                    iwork and numcells
!     PRINIOBGRID    Print the contents of array iobgrid
!     PRINRL1        Print 2d real vector
!     SEPCOPCURVCOOR Copy coordinates of curve icurnr from array curves into
!                    array curvescp
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Initializations
!   Clear array iobgrid
!   icurnr := boundary curve of obstacle
!   For all edges in icurnr do
!      Compute x1,y1, x2,y2
!      alpha1 := (x1-xmin)/dx; alpha2 := (x2-xmin)/dx;
!      beta1 := (y1-ymin)/dy; beta2 := (y2-ymin)/dy;
!      i1 := alpha1; j1:= beta1; i2 := alpha2; j2:= beta2;
!      Mark cells i1,j1 and i2,j2 as boundary cells
!      Mark all cells at the intersection line as boundary cells
!   For each column in the grid do
!      Store all parts that are possibly in this column into array
!         istorecurves(1:nparts)
!      Clear column in iobgrid
!      For each curve part do
!         Find (x1,y1), (x2,y2)
!         Compute intersections with lines x=xmin, x=xmid and x=xmax
!         Fill iobgrid for elements that are crossed by boundary
!         Store ymid in storecurves(ipart)
!      Sort storecurves in increasing sequence
!      For each row in column do
!         Colour row as internal or external depending on number of cross
!         sections with line xmid
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      debug = .false. .and. ioutp>=0
      call eropen ( 'sepobsfl' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepobsfl'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initializations

      xmin = rmeshx(1,iobst)
      ymin = rmeshx(3,iobst)
      ymax = rmeshx(4,iobst)
      dx   = rmeshx(5,iobst)
      dy   = rmeshx(6,iobst)

!     --- icurnr := boundary curve of obstacle

      icurnr = kmeshx(2,iobst)
      closed = kmeshx(1,iobst)>0

      if ( debug ) then

!     --- debug is true, print info

         write(irefwr,*) 'Data with respect to obstacle'
         write(irefwr,2) 'xmin, ymin, dx, dy', xmin, ymin, dx, dy
         write(irefwr,1) 'nx, ny', nx, ny
         write(irefwr,1) 'iobst, icurnr', iobst, icurnr
         write(irefwr,2) 'x_coor cells', (xmin+i*dx,i=0,nx)
         write(irefwr,2) 'y_coor cells', (ymin+i*dy,i=0,ny)

      end if

!     --- Clear array iobgrid

      iobgrid = 0

!     --- Store all coordinates of the curve in curvescp
!         and compute number of nodes

      call sepcopcurvcoor ( icurnr, curvescp, n )

      if ( debug ) call prinrl1 ( curvescp, n, 2, 'obst_curv' )

      leniwork = max(nx*ny,10*n)

      allocate ( iwork(2,leniwork), numcells(2,n-1), stat = error )
      if ( error/=0 ) call eralloc ( error, leniwork, 'iwork' )
      if ( ierror/=0 ) go to 1000

      numcells = 0

!     --- Fill array iobgrid with number of edges per cell
!              array numcells with starting addresses and number of cells
!              array iwork with cell pairs

      call mshobsfledge ( curvescp, n, xmin, ymin, dx, dy, iwork, &
                          leniwork, iobgrid, nx, ny, numcells )
      if ( ierror/=0 ) go to 1000

!     --- Refill iobgrid with starting addresses to iobgrid1
!         Fill array iobgrid1

      call mshobsfliobgrd ( iobgrid, nx, ny, iwork, numcells, n, &
                            iobgrid1, leniobgrid1 )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,*) 'closed ', closed

      if ( closed ) then

!     --- The obstacle is closed
!         Fill cells inside and outside the obstacle

         call mshobsflinside ( iobgrid, nx, ny, iobgrid1, numcells, &
                               xmin, ymin, ymax, iwork, n, &
                               leniwork, dy, closed, dx, curvescp )

!        --- Refill iobgrid with starting addresses to iobgrid1
!            Fill array iobgrid1

         call mshobsfliobgrd ( iobgrid, nx, ny, iwork, numcells, n, &
                               iobgrid1, leniobgrid1 )

      end if  ! ( closed )

      deallocate ( iwork, numcells, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         call priniobgrid ( iobgrid, nx, ny, iobgrid1 )
         write(irefwr,*) 'End sepobsfl'

      end if  ! ( debug )
      call erclos ( 'sepobsfl' )

      end
