      subroutine septransf ( iintrn )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.0  date 22-02-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Transform an existing Sepran mesh of triangles and tetrahedrons
!     into a mesh of quadrilaterals and hexahedrons
!     All elements must be linear
!     Old subroutine transf
! **********************************************************************
!
!                       KEYWORDS
!
!     transformation
!     mesh
!     linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iintrn(*)

!     iintrn   i   integer input array in which information about the process
!                  must be stored by the user.
!                  iintrn must be filled as follows:
!                  Pos. 1:  Highest entry number of iintrn that has been filled
!                           by the user. The psoitions 1 to iintrn(1) must
!                           have been filled. For all other positions default
!                           values are used.
!                           So iintrn(1) = 0 or 1 means only defaults
!                  Pos. 2:  itopol, indicates if subroutine topol must be called
!                           Possible values:
!                           0  subroutine topol is called (standard situation)
!                           1  subroutine topol is skipped (must only be used
!                              when the mesh will be written by meshwr)
!                           Default value: 0
!                  Pos. 3:  jrenum   Indicates the type of renumbering to be
!                                    performed. see subroutine TOPOL
!                           Default value: 9
!                  Pos. 4:  ishape   Indicates the type of mesh that must be
!                           constructed. Possible values:
!                           4    Construct quadratic triangles and/or tetra-
!                                hedrons
!                           5    Construct linear quadrilaterals and/or hexa-
!                                hedrons from the linear triangles and tetra-
!                                hedrons
!                           6    Construct quadratic quadrilaterals and/or
!                                hexahedrons
!                           Default value: 5
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jrenum(1), itopol, ishapelc
      double precision :: timebefore(2)

!     ishapelc       Local shape number
!     itopol         Indicates if array TOPOL must be called, see ichois
!     jrenum         Parameter jrenum for TOPOL
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     MSHDELSEL      Delete a selected set of subarrays of kmesh
!     MSHTRANSKMESHS Store new shape numbers in kmeshs in case of transformation
!     PRININ         print 1d integer array
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPPRINKMESH   Print array KMESH
!     TOPOL          Fill integer information corresponding to the mesh in KMESH
!     TRANSFBFLIN    Transform an existing Sepran mesh of triangles and
!                    tetrahedrons into a mesh of quadrilaterals and hexahedrons
!                    All elements must be linear
!     TRANSFBFQUAD   Transform an existing Sepran mesh of linear elements
!                    into a mesh of quadratic elements
!     TRANSFTRIA7    Extend mesh consisting of triangles with extra node in
!                    center
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1182   ITOPOL has incorrect value
!    1763   ishape has incorrect value
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine whether there are surfaces and volumes or only surfaces
!     Check whether mesh has to be transformed (ishape = 5 or 6) or
!     that only a quadratic mesh has to be constructed (ishape = 4)
!     If transform then
!       To transform Coordinates, Elements and Curves are needed
!       Be sure that all are in IBUFFR - array
!       Determine present length of the arrays
!       Determine startpositions for help-arrays , first the total
!       length needed has to be computed. Next divide over the arrays.
!       Set check variables length for new arrays
!       Perform actual transformation into quadrilaterals  (MSH440)
!       Store new arrays on a permanent position
!     Endif
!     If (ishape=4 or ishape=6) then
!       Make linear mesh quadratic
!       Do the same steps as above for the neighbours
!       Make mesh quadratic (MSH450)
!       Store new arrays on a permanent position
!     Endif
!     If needed call topol for topology
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septransf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septransf'
         write(irefwr,*) 'old mesh'
         call prinin ( iintrn, 4, 'iintrn' )
         call sepprinkmesh ( 1 )
         call sepprinkmesh ( 38 )

      end if  ! ( debug )
      if ( itime>0 ) call ersettime ( timebefore )
      if ( ierror/=0 ) go to 1000

!     --- determine whether there are surfaces and volumes or only surfaces

      if ( iintrn(1)>=2 ) then
         itopol = iintrn(2)
      else
         itopol = 0
      end if

      if ( iintrn(1)>=3 ) then
         jrenum(1) = iintrn(3)
      else
         jrenum(1) = 9
      end if

      if ( iintrn(1)>=4 ) then
         ishapelc = iintrn(4)
      else
         ishapelc = 5
      end if

      if ( ishapelc<4 .or. ishapelc>7 ) then
         call errint ( ishapelc, 1 )
         call errsub ( 1763, 1, 0, 0 )
      end if

      if ( itopol<0 .or. itopol>1 ) then
         call errint ( itopol, 1 )
         call errsub ( 1182, 1, 0, 0 )
      end if

      if ( ierror/=0 ) go to 1000

!     --- check whether endmesh contains triangles or quadrilaterals
!         (in case of triangles jump to second part of routine)

      if ( ishapelc==5 .or. ishapelc==6 ) then

!     --- Quadrilateral must be created from triangle

         call transfbflin

      end if  ! ( ishapelc==5 .or. ishapelc==6 )

!     --- Second part of routine, quadratic or linear mesh

      if ( ishapelc/=5 ) then

!     --- Make mesh quadratic
!         (coordinates and elements are needed)

         call transfbfquad

      end if  ! ( ishapelc/=5 )

      if ( ishapelc==7 ) then

!     --- Shape = 7 (2d only)
!         Add extra point to triangles

         call transftria7

      end if  ! ( ishapelc==7 )

      if ( kelms/=0 ) then

!     --- kmeshs exists, adapt shape numbers

         call mshtranskmeshs ( kmeshs(nsurfs+2), nsurfs, ishapelc )

      end if

!     --- Remove some subarrays of kmesh that are no longer valid

      call mshdelsel ( kmesh )

      call sepfillkmesh(kmesh)

      if ( itopol==0 ) call topol ( jrenum )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'new mesh'
         call sepprinkmesh ( 1 )
         call sepprinkmesh ( 38 )
         call sepprinkmesh ( 63 )
         write(irefwr,*) 'End septransf'

      end if  ! ( debug )
      call erclmn ( 'septransf', timebefore )

      end subroutine septransf
