      subroutine transfbfquad
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.1  date 27-11-2012 Long integers
!        version  1.0  date 15-11-2006
!
!   copyright (c) 2006-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Transform an existing Sepran mesh of linear elements
!    into a mesh of quadratic elements
! **********************************************************************
!
!                       KEYWORDS
!
!     transformation
!     mesh
!     linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lncoor, lnkmsc, lnkmsm, lnkmsn, ibrp
      integer, allocatable :: kmeshnnew(:), kmeshmnew(:), kmeshcnew(:)
      double precision, allocatable :: coornew(:)

!     coornew        Array containing the new coordinates of the nodes
!     ibrp           Estimated number of neighbours of one point
!     kmeshcnew      New kmesh part c
!     kmeshmnew      New array kmeshm
!     kmeshnnew      New array kmeshn
!     lncoor         declared length for coor, resp coorh
!     lnkmsc         length of kelem array (kmesh part c)
!     lnkmsm         length of curves array (kmesh part m)
!     lnkmsn         length of surfaces array (kmesh part n)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSH450         Make linear mesh quadratic
!     SEPTRANSF1     Copy new kmeshc, coor, kmeshm and kmeshn to old arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       Make linear mesh quadratic
!       Do the same steps as above for the neighbours
!       Make mesh quadratic (MSH450)
!       Store new arrays on a permanent position
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'transfbfquad' )
      if ( ierror/=0 ) go to 1000

!     --- Make mesh quadratic
!         (coordinates and elements are needed)
!          determine present length elements

      lnkmsc = size ( kmeshc )

!     --- determine present length curves

      lnkmsc = size ( kmeshc )
      lnkmsm = size ( kmeshm )
      lnkmsn = max(1,size ( kmeshn ))

!     --- determine startpositions for help-arrays , first the total
!         length needed has to be computed

      if ( ndim==2 ) then

!     --- array coorh, new length is determined
!         extra points on all lines and in case of quadrilaterals in
!         all barycenters, hence length is at most

         lncoor = ( nelem + 6 * npoint ) * ndim

!        --- array kelemh
!            nelem triangles or quadrilaterals are transformed, &
!            length<=3 * length kelem integer positions

         lnkmsc = 3 * lnkmsc

!        --- array kromh
!            length<=2 * length kromme integer positions

         lnkmsm = 2 * lnkmsm

!        --- array help, to store temporarily the nodenumbers of the extra
!            points on the lines (for each line a position)

         ibrp   = 10

!        --- in 2D there is no space needed for triangles and surfaces

         lnkmsn = 0

      else if ( ndim==3 ) then

!     --- array coorh, new length is determined
!         extra points on all lines, in all barycenters and in all elements

         lncoor = ndim * ( 3 * nelem + 5 * npoint )

!        --- array kelemh
!            number of elements is not changing only length (i.e. number
!            of nodes)
!            tetrahedrons factor = 2.5 ( 10 / 4 )
!            hexahedrons factor  = 3.4 ( 27 / 8 )

         lnkmsc = 7 * lnkmsc / 2

!        --- array kromh
!            length<2 * length kromme integer positions

         lnkmsm = 2 * lnkmsm

!        --- array surfh
!            length<3 * length surf integer positions

         lnkmsn = 3 * lnkmsn

!        --- arrays ibrpnt and istart, to store temporarily the nodenumbers
!            of the extra points on the lines (for each line one position)
!            length =  41 * npoint  integers

         ibrp   = 40

      end if

!     --- Make reservation

      allocate ( kmeshnnew(lnkmsn), kmeshmnew(lnkmsm), coornew(lncoor), &
                 kmeshcnew(lnkmsc), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, lnkmsn+lnkmsm+lncoor+lnkmsc, 'kmeshnnew' )
      if ( ierror/=0 ) go to 1000

!     --- Perform actual transformation into quadratic elements

      call msh450( coornew, kmeshcnew, kmeshmnew, kmeshnnew, &
                   lncoor, lnkmsc, lnkmsm, lnkmsn, ibrp   )

!     --- Copy to old arrays and recreate these arrays

      call septransf1 ( lnkmsc, lncoor, lnkmsm, lnkmsn, kmeshcnew, &
                        kmeshmnew, kmeshnnew, coornew )

      deallocate ( kmeshnnew, kmeshmnew, kmeshcnew, coornew, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'kmeshnnew' )

1000  call erclos ( 'transfbfquad' )

      end subroutine transfbfquad
