      subroutine msh075curvs ( iinput, iinp, icurvs, inodp, lengc, iselem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-03-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine checks the input for interface elements. It computes the
!     required length of these elements with respect to KMESH part c
!     Furthermore inodp is raised if necessary
!     Curves only
! **********************************************************************
!
!                       KEYWORDS
!
!     interface_element
!     mesh
!     mesh_generation
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, icurvs, inodp, lengc, iselem

!     icurvs         i    Number of parts where curve interface elements are given
!     iinp          i/o   last position used in array iinput with respect to
!                         interface elements
!     iinput         i    integer input array with respect to the mesh
!     inodp         i/o   Next nodal point number
!     iselem        i/o   Next free element number
!     lengc         i/o   computed length of kmesh part c
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp, i, ileft, idir1, iright, idir2, nsubcv, n, &
              istar3, inpelm, ipos, j, k, istar1, istar2, nodalp

!     i              Counting variable
!     idir1          Direction of first curve (1=positive, -1=negative)
!     idir2          Direction of second curve (1=positive, -1=negative)
!     ielgrp         element group number
!     ileft          Left curve in special element
!     inpelm         number of points per element
!     ipos           Position in array
!     iright         Right curve in special element
!     istar1         Starting position of first curve in KMESHM
!     istar2         Starting address of KMESHM part 2 in KMESHM
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     j              Counting variable
!     k              Counting variable
!     n              Number of elements created by mesh interface
!     nodalp         Nodal point sequence number
!     nsubcv         Number of subcurves of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH044         fill negative values in kmeshm if possible
!     MSH044SURF     fill negative values in kmeshm and kmeshn if possible
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     325   curve number for connected element is incorrect
!     327   2 curves for connected element have different numbers of nodal
!           points.
!    1002   Shape number out of range
!    1003   Shape number does not match number of nodes along curve
!    1805   Curve is linked with itself
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh075curvs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh075curvs'

      end if
      if ( ierror/=0 ) go to 1000

      istar3 = kmeshm(2)-1

      do i = 1, icurvs

         ielgrp = iinput(iinp)
         ishape = iinput(iinp+1)
         if ( ishape<50 .or. ishape>59 ) then

!        --- error 1002: ishape out of range

            call errint ( ishape, 1 )
            call errint ( 50, 2 )
            call errint ( 59, 3 )
            call errsub ( 1002, 3, 0, 0 )

         end if
         ileft = iinput(iinp+2)
         idir1 = sign(1, ileft)
         ileft = abs(ileft)
         if ( ileft==0 .or. ileft>ncurvs ) then
            call errint(idir1*ileft,1)
            call errint(ncurvs,2)
            call errint(ielgrp,3)
            call errint(i,4)
            call errsub ( 325, 4, 0, 0 )
         end if
         iright = iinput(iinp+3)
         idir2 = sign(1,iright)
         iright = abs(iright)
         if ( iright==0 .or. iright>ncurvs ) then
            call errint(idir2*iright,1)
            call errint(ncurvs,2)
            call errint(ielgrp,3)
            call errint(i,4)
            call errsub ( 325, 4, 0, 0 )
         end if
         if ( ileft==iright ) then
            call errint ( ileft, 1 )
            call errsub ( 1805, 1, 0, 0 )
         end if
         nsubcv = 1
         if ( kelmq>0 ) nsubcv = kmeshq(ileft+1)-kmeshq(ileft)
         nsubcv = max(1,nsubcv)
         n = kmeshm(ileft+5)-kmeshm(ileft+4)+1-nsubcv

         nsubcv = 1
         if ( kelmq>0 ) nsubcv = kmeshq(iright+1)-kmeshq(iright)
         nsubcv = max(1,nsubcv)

         if ( n/=(kmeshm(iright+5)-kmeshm(iright+4)+1-nsubcv) ) then
            call errint(ileft,1)
            call errint(n,2)
            call errint(iright,3)
            call errint(kmeshm(iright+5)-kmeshm(iright+4)+1-nsubcv,4)
            call errint(ielgrp,5)
            call errint(i,6)
            call errsub ( 327, 6, 0, 0 )
         end if
         if ( ileft==iright ) then
            call errint ( ileft, 1 )
            call errsub ( 1805, 1, 0, 0 )
         end if
         if ( ierror==0 ) then

!        --- Not an error continue

            ishape = ishape-49
            if ( mod(n-1,ishape)/=0 ) then
               call errint ( n, 1 )
               call errint ( ishape+49, 2 )
               call errsub ( 1003, 2, 0, 0 )
            end if
            n = (n-1)/ishape
            iselem = iselem+n
            if ( idir1>0) then
               istar1 = istar3+kmeshm(ileft+4)
            else
               istar1 = istar3-1+kmeshm(ileft+5)
            end if
            if ( idir2>0) then
               istar2 = istar3+kmeshm(iright+4)
            else
               istar2 = istar3-1+kmeshm(iright+5)
            end if

            inpelm = ishape + 1
            do j = 1, n
               do k = 1, inpelm
                  ipos = istar1+(k-1)*idir1
                  nodalp = kmeshm(ipos)
                  if ( nodalp<0 ) then
                     if ( kmeshl(-nodalp)>0 ) then
                        kmeshm(ipos) = kmeshl(-nodalp)
                     else
                        kmeshl(-nodalp) = inodp
                        kmeshm(ipos) = inodp
                        inodp = inodp+1
                     end if
                  else if ( nodalp==0 ) then
                     kmeshm(ipos) = inodp
                     inodp = inodp+1
                  end if
                  ipos = istar2+(k-1)*idir2
                  nodalp = kmeshm(ipos)
                  if ( nodalp<0 ) then
                     if ( kmeshl(-nodalp)>0 ) then
                        kmeshm(ipos) = kmeshl(-nodalp)
                     else
                        kmeshl(-nodalp) = inodp
                        kmeshm(ipos) = inodp
                        inodp = inodp+1
                     end if
                  else if ( nodalp==0 ) then
                     kmeshm(ipos) = inodp
                     inodp = inodp+1
                  end if
               end do
               istar1 = istar1+ishape*idir1
               istar2 = istar2+ishape*idir2
            end do

            lengc = lengc + 2*n*(ishape+1)

         end if

         iinp = iinp+4

      end do

!     --- fill array kmeshm if necessary

      if ( kelmn==0 ) then

!     --- kmeshn has not been filled

         call msh044 ( kmeshm, kmeshl )

      else

!     --- kmeshn has been filled

         call msh044surf ( kmeshm, kmeshl, kmeshn, nsurfs )

      end if  ! ( kmesh(28)==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh075curvs'

      end if
      call erclos ( 'msh075curvs' )

      end
