      subroutine mshcopboun2pl ( bcord, nbndpt, trans, n, t1, t2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-09-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy 3d boundary onto plane
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     mesh
!     boundary
!     3d
!     plane
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nbndpt
      double precision bcord(3,nbndpt), trans(*), t1(3), t2(3), n(3)

!     bcord         i/o   array containing the coordinates of the boundaries
!                         At input the original boundary
!                         At output mapped onto a plane
!     n             i/o   normal vector
!                         At input the first estimate
!                         At output the corrected one
!     nbndpt         i    Number of nodal points in boundary, length of array
!                         kbndpt
!     t1             o    First tangential vector
!     t2             o    Second tangential vector
!     trans          o    array containing the inverse of the transformation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2, i3
      double precision det, norm, p(3), g, mmat(3,3), rhs(3), sol(3)
      logical singular

!     det            determinant
!     g              parameter for correction of coordinates
!     i              Counting variable
!     i1             i3+1
!     i2             i3+2
!     i3             index of largest element in normal
!     mmat           Matrix M defined in method
!     norm           Norm of vector
!     p              Contains coordinates of node
!     rhs            Right-hand side vector
!     singular       if true the matrix is singular
!     sol            Solution of system of equations
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ercdet

!     ERCDET         Compute a 3x3 determinant
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOMPNORMVEC  Compute the normal vector with respect to a plane, given
!                    two orthogonal tangential vectors in this plane
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     MSHTRANSMAT    Compute the inverse transformation matrix to be used to
!                    map a 3d plane onto 2d coordinates, given 2 tangential
!                    vectors and the normal
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     PRSOLMAT3      Solve system of 3 linear equations S x = f
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3150   Mapping matrix is singular
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The plane is given by ax + by + c = z, or ax + by - z + c = 0
!     The boundary nodes are given by (x_i, y_i, z_i), &
!     hence we want to solve
!     M a = z, with a the vector (a,b,c), z the vector (z_1, z_2,...z_n)
!     and M the matrix:
!
!           |  x_1  y_1  1  |
!           |  x_2  y_2  1  |
!           |      .        |
!     M =   |      .        |
!           |      .        |
!           |  x_n  y_n  1  |
!     Solution in least square sense is:  M^t M a = M^t z, yielding the
!     vector a
!     The normal n to the plane is defined by (a, b, -1) / norm, &
!     where norm is the norm of (a, b, -1)
!     The plane is defined by n . x = -c/norm, with x = (x,y,z)
!
!     The projection of a node P to the plane is defined by x = P + g n, &
!     hence n . x = n . P + g = -c/norm, or g = -c/norm -n . P
!             | sum x_i^2   sum x_i y_i   sum x_i  |
!     M^t M = | sum x_i y_i sum y_i^2     sum y_i  |
!             | sum x_i     sum y_i         n      |
!     and
!             | sum x_i z_i |
!     M^t z = | sum y_i z_i |
!             | sum z_i     |
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcopboun2pl' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcopboun2pl'
         call prinrl1 ( bcord, nbndpt, 3, 'bcord')
         call prinrl ( n, 3, 'n' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Find largest element of n, store index in i3

      g = abs(n(1))
      i3 = 1
      do i = 2, 3
         if ( abs(n(i))>g ) then
            g = abs(n(i))
            i3 = i
         end if  ! ( abs(n(i))>g )
      end do  ! i = 2, 3
      i1 = i3+1
      i2 = i3+2
      if ( i1>3 ) i1 = i1-3
      if ( i2>3 ) i2 = i2-3

      if ( debug ) write(irefwr,1) 'i1, i2, i3', i1, i2, i3

!     --- Create matrix M

      mmat(1,1) = dot_product ( bcord(i1,1:nbndpt), bcord(i1,1:nbndpt) )
      mmat(1,2) = dot_product ( bcord(i1,1:nbndpt), bcord(i2,1:nbndpt) )
      mmat(1,3) = sum ( bcord(i1,1:nbndpt) )
      mmat(2,1) = mmat(1,2)
      mmat(2,2) = dot_product ( bcord(i2,1:nbndpt), bcord(i2,1:nbndpt) )
      mmat(2,3) = sum ( bcord(i2,1:nbndpt) )
      mmat(3,1) = mmat(1,3)
      mmat(3,2) = mmat(2,3)
      mmat(3,3) = nbndpt

!     --- Create right-hand side M^t z

      rhs(1) = dot_product ( bcord(i1,1:nbndpt), bcord(i3,1:nbndpt) )
      rhs(2) = dot_product ( bcord(i2,1:nbndpt), bcord(i3,1:nbndpt) )
      rhs(3) = sum ( bcord(i3,1:nbndpt) )

!     --- Solve system of equations

      call prsolmat3 ( mmat, rhs, sol, singular )
      if ( debug ) call prinrl ( sol, 3, 'sol' )

      if ( singular ) then

!     --- Matrix is singular

         call errsub ( 3150, 0, 0, 0 )

      end if  ! ( singular )

!     --- Fill normal vector

      n(i1) = sol(1)
      n(i2) = sol(2)
      n(i3) = -1d0

!     --- Orthogonolize

      norm = sqrt(dot_product(n,n))
      n = n/norm
      sol(3) = sol(3)/norm

      if ( debug) then
         call prinrl ( n, 3, 'n')
         write(irefwr,2) 'norm, g', norm, sol(3)
      end if  ! ( debug)

!     --- Project boundary onto plane
!         The projection of a node P to the plane is defined by x = P + g n, &
!         hence n . x = n . P + g = -c/norm, or g = -c/norm -n . P

      do i = 1, nbndpt
         p = bcord(:,i)
         g = -sol(3) - dot_product ( n, p )
         bcord(:,i) = p + g * n
      end do  ! i = 1, nbndpt

!     --- Compute tangential vectors in plane

      t1 = bcord(:,2) - bcord(:,1)       ! first tangential vector
      norm = sqrt(dot_product(t1,t1))
      t1 = t1/norm
      call ercompnormvec ( n, t1, t2 )   ! second one by outer product

      det = ercdet ( t1, t2, n )  ! determinant

      if ( det<0 ) n = -n

!     --- compute inverse of transformation

      call mshtransmat ( t1, t2, n, trans )

1000  call erclos ( 'mshcopboun2pl' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( t1, 3, 't1' )
         call prinrl ( t2, 3, 't2' )
         call prinrl ( n, 3, 't3' )
         call prinrl1 ( bcord, nbndpt, 3, 'bcord')
         call prinrl1 ( trans, 3, 3, 'trans1' )
         write(irefwr,*) 'End mshcopboun2pl'

      end if  ! ( debug )

      end
