      subroutine mshfindadsurf ( marksurf, marknodes, &
                                 numsurfs, adapsurfs, markcurv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-01-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all surfaces that have at least one node that has been moved
!     during a free boundary problem
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     mesh
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer marksurf(*), marknodes(*), &
              numsurfs, adapsurfs(*), markcurv(*)
!     adapsurfs      o    Array of size numsurfs containing the surfaces that
!                         must be adapted
!     markcurv       i    Array of size ncurvs that marks all curves that
!                         have been adapted in the moving boundary
!                         markcurv(icurnr) = 0: curve has not been moved
!                         markcurv(icurnr) = 1: curve has been moved
!     marknodes      i    Array of size npoint that marks all nodes that
!                         have been adapted in the moving boundary
!                         marknodes(node) = 0: node has not been moved
!                         marknodes(node) = 1: node has been moved
!     marksurf       i    Array of size nsurfs that marks all surfaces that
!                         have been adapted in the moving boundary
!                         marksurf(isurnr) = 0: surface has not been moved
!                         marksurf(isurnr) = 1: surface has been moved
!     numsurfs       o    Number of surfaces that must be adapted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isurnr, icurnr, istart, numcurvs
      logical changed

!     changed        If true the surface is a candidate for adpatation
!     i              Counting variable
!     icurnr         Curve sequence number
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     numcurvs       Number of curves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     numsurfs := 0
!     For all single surfaces isurnr do
!        if ( surface is not changed before ) then
!           if ( one of the end points has been moved ) then
!              numsurfs := numsurfs+1;  adapsurfs(numsurfs) := isurnr
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfindadsurf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfindadsurf'
         call prinin ( marknodes, npoint, 'marknodes' )
         call prinin ( markcurv, ncurvs, 'markcurv' )
         call prinin ( marksurf, nsurfs, 'marksurf' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- numsurfs := 0

      numsurfs = 0

!     --- For all single surfaces isurnr do

      do isurnr = 1, nsurfs

         changed = .false.  ! start value of changed
         if ( kelmr>0 ) then

!        --- There are composite surfaces
!            Set changed true in case of compound surface

            if ( kmeshr(isurnr+1)-kmeshr(isurnr)>0 ) changed = .true.

         end if  ! ( kelmq>0 ) then

         changed = changed .or. marksurf(isurnr)>0  ! surface is changed
         if ( debug ) then
            write(irefwr,1) 'isurnr', isurnr
            write(irefwr,*) 'changed ', changed
         end if  ! ( debug )

         if ( .not. changed ) then

!        --- surface is not changed before
!            Check boundary

            istart = kmeshs(isurnr)
            numcurvs = kmeshs(isurnr+1)-istart
            do i = 1, numcurvs

               icurnr = abs(kmeshs(istart+i))
               if ( markcurv(icurnr)>0 ) then

!              --- Curve has been moved
!                  numsurfs := numsurfs+1;  adapsurfs(numsurfs) := isurnr

                  numsurfs = numsurfs+1
                  adapsurfs(numsurfs) = isurnr
                  marksurf(isurnr) = 1
                  go to 100   ! jump since surface found

               end if  ! ( markcurv(icurnr)>0 )

            end do  ! i = 1, numcurvs

         end if  ! ( .not. changed )
100      istart = istart+numcurvs

      end do  ! isurnr = 1, nsurfs

1000  if ( debug ) then

!     --- Debug information

         call prinin ( adapsurfs, numsurfs, 'adapsurfs' )
         write(irefwr,*) 'End mshfindadsurf'

      end if  ! ( debug )
      call erclos ( 'mshfindadsurf' )

      end
