      subroutine mshintcurvline ( xstart, tangent, coor, kmeshm, &
                                  icurnr, coors )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-11-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     intersect curve icurnr with a straight line
!     Store the coordinates and distance from origin into coors
! **********************************************************************
!
!                       KEYWORDS
!
!     intersection
!     curves
!     line
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision xstart(2), tangent(2), coors(3), coor(2,*)
      integer kmeshm(*), icurnr

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     coors          o    Array containing the coordinates and distance
!                         of the intersection from origin
!     icurnr         i    Curve sequence number
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     tangent        i    Contains the tangential vector
!     xstart         i    Starting point of line
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nodp1, nodp2, nstar3, ilast, ifirst, i
      double precision x1(2), x2(2), x4(2), eps, distance, fact1, fact2
      logical intersect, closed, beginpoint, endpoint

!     beginpoint     if true the intersection coincides with the starting point
!                    of the curve
!     closed         If true the curve is closed
!     distance       distance between origin and intersection
!     endpoint       if true the intersection coincides with the starting point
!                    of the curve
!     eps            Local accuracy
!     fact1          multiplication factor related to interval on curve
!     fact2          multiplication factor related to line
!     i              Counting variable
!     ifirst         First position in kmeshm corresponding to curve
!     ilast          Last position in kmeshm corresponding to curve
!     intersect      if true an intersection has been found
!     nodp1          Node sequence number
!     nodp2          Node sequence number
!     nstar3         Starting address of KMESHM part 3 in KMESHM containing the
!                    node numbers of the curves
!     x1             coordinates of first point of interval on curve
!     x2             coordinates of last point of interval on curve
!     x4             coordinates of "end" point of line
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     MSHCROSSLINE   Computes the intersection of two lines
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2741   more than 1 intersection of line with curve
!    2742   no intersection of line with curve
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute some constants
!     intersect := false
!     For all intervals along the curve do
!        if ( interval intersects line ) then
!           compute intersection
!           if ( intersect ) then
!              warning: more than one intersection
!           intersect := true
!     If ( not intersect ) then
!        Warning: no intersection found
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshintcurvline' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshintcurvline'
         write(irefwr,100) 'xstart ', xstart
         write(irefwr,100) 'tangent ', tangent
100      format ( a, 2d13.5 )

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

!     --- Compute some constants

      nstar3 = kmeshm(2)-1
      ifirst = kmeshm(4+icurnr)
      ilast = kmeshm(5+icurnr)-1

!     --- The end point of the line is positioned at sufficient distance of
!         the starting point (hopefully)

      x4(1) = xstart(1) + 1d4*tangent(1)
      x4(2) = xstart(2) + 1d4*tangent(2)
      if ( debug ) write(irefwr,*) 'x4 ', x4

!     --- intersect := false

      intersect = .false.
      endpoint = .false.
      beginpoint = .false.
      closed = kmeshm(nstar3+ifirst).eq.kmeshm(nstar3+ilast)
      eps = sqreps
      distance = rinfin

!     --- For all intervals along the curve do

      do i = ifirst, ilast-1

         nodp1 = kmeshm(nstar3+i)
         nodp2 = kmeshm(nstar3+i+1)
         if ( debug ) &
            write(irefwr,110) 'i, nodp1, nodp2 ', i, nodp1, nodp2
110      format ( a, 10i6 )
         if ( nodp1.gt.0 .and. nodp2.gt.0 ) then

!        --- Both nodes exist
!            compute intersection

            x1(1) = coor(1,nodp1)
            x1(2) = coor(2,nodp1)
            x2(1) = coor(1,nodp2)
            x2(2) = coor(2,nodp2)
            call mshcrossline ( x1, x2, xstart, x4, fact1, fact2, eps )
            if ( debug ) then
               write(irefwr,100) 'x1 ', x1
               write(irefwr,100) 'x2 ', x2
               write(irefwr,100) 'fact1, fact2 ', fact1, fact2
            end if  ! ( debug )
            if ( fact2.ge.-eps ) then

!           --- interval intersects line

               if ( fact1.le.eps ) then

!              --- point close to begin point of interval

                  if ( endpoint ) then

!                 --- Point has been used in previous interval
!                     Set logicals and do nothing

                     endpoint = .false.
                     intersect = .false.

                  else

!                 --- New point

                     if ( i.eq.ifirst ) beginpoint = .true.

                  end if  ! ( endpoint )

               else if ( fact1.ge.1d0-eps ) then

!              --- point close to end point of interval

                  if ( beginpoint .and. closed .and. i.eq.ilast-1 ) then

!                 --- Point has been used in first interval
!                     Set logicals and do nothing

                     intersect = .false.

                  end if  ! ( beginpoint .and. closed .and. i.eq.ilast-1 )

               end if  ! ( fact1.le.eps )
               if ( intersect ) then

!              --- intersect = true
!                  warning: more than one intersection
!                  Compute smallest distance

                  call erreal ( xstart(1), 1 )
                  call erreal ( xstart(2), 2 )
                  call erreal ( tangent(1), 3 )
                  call erreal ( tangent(2), 4 )
                  call errint ( icurnr, 1 )
                  call errwar ( 2741, 1, 4, 0 )

                  if ( fact2.lt.distance ) then

!                 --- New point found

                     distance = fact2

                  end if  ! ( fact1.lt.distance )

               end if  ! ( intersect )

!              --- intersect := true

               intersect = .true.

            end if  ! ( fact2.ge.-eps )

         end if  ! ( nodp1.gt.0 .and. nodp2.gt.0 )

      end do  ! i = ifirst, ilast-1

      if ( .not. intersect ) then

!     --- Warning: no intersection found

         call erreal ( xstart(1), 1 )
         call erreal ( xstart(2), 2 )
         call erreal ( tangent(1), 3 )
         call erreal ( tangent(2), 4 )
         call errint ( icurnr, 1 )
         call errwar ( 2742, 1, 4, 0 )

      else

!     --- intersection found, fill coors

         coors(1) = fact2*1d4
         coors(2) = xstart(1)+fact2*tangent(1)*1d4
         coors(3) = xstart(2)+fact2*tangent(2)*1d4

      end if  ! ( .not. intersect )

1000  call erclos ( 'mshintcurvline' )

      if ( debug ) then

!     --- Debug information

         call prinrl ( coors, 3, 'coors' )
         write(irefwr,*) 'End mshintcurvline'

      end if

      end
