      subroutine mshmapcurvs ( nextracrvs, curvenumbers, kmeshm, kmeshl, &
                               kbndpt, mapar, isurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-07-2008
!
!   copyright (c) 2008-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Put nodal point numbers of extra curves into array kmeshm
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     topology
!     node
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nextracrvs, curvenumbers(*), kmeshm(*), kmeshl(*), &
              kbndpt(*), mapar(*), isurnr

!     curvenumbers   i    Contains the extra curve numbers
!     isurnr         i    Surface sequence number
!     kbndpt         i    array containing the nodal point numbers of the
!                         boundary
!                         In this case only the part corresponding to
!                         the extra curves
!     kmeshl        i/o   KMESH part l, contains nodal point numbers of the
!                         user points
!     kmeshm        i/o   KMESH part m, contains nodal point numbers of the
!                         curves
!     mapar          i    mapping array: contains the mapping from local to
!                         global numbering of the nodal points
!     nextracrvs     i    Number of extra (internal) curves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, istar3, i, icurnr, k1, k2, numnodes, j, nodp, ipos

!     i              Counting variable
!     icurnr         Curve sequence number
!     ipos           Position in array
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         First position used at a part
!     j              Counting variable
!     k1             First point of curve in KMESH part m
!     k2             Last point of curve in KMESH part m
!     nodp           Nodal point sequence number
!     numnodes       Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     617   User point corresponds to two different nodal point numbers.
!    2969   Different node numbers found for same node
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all internal curves in surface do
!        get nodal point numbers
!        Find positions in array kmeshm
!        Put nodal point numbers into kmeshm
!
!     The boundary points are all stored in kbndpt with length nbndpt
!     The next nextracrvs points contain the nodes of the extra curves
!     Finally the next extra nextrapnts points contain the nodes of the extra
!     user points
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmapcurvs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshmapcurvs'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istart = 1            ! starting address in kbndpt
      istar3 = kmeshm(2)-1  ! starting address in kmeshm part 3

!     --- For all internal curves in surface do

      do i = 1, nextracrvs

         icurnr = abs(curvenumbers(i))

!        --- get nodal point numbers
!            Find positions in array kmeshm
!            Put nodal point numbers into kmeshm

         k1 = kmeshm(4+icurnr)      ! first position of icurnr in kmeshm 3
         k2 = kmeshm(5+icurnr)      ! last position of icurnr in kmeshm 3 (+1)
         numnodes = k2-k1           ! number of nodes along curve
         if ( debug ) then
            write(irefwr,1) 'icurnr, k1, k2, numnodes', &
                             icurnr, k1, k2, numnodes
            call prinin ( kbndpt(istart), numnodes, 'kbndpt' )
         end if  ! ( debug )

         do j = 0, numnodes-1

            nodp = mapar(kbndpt(istart+j)) ! node number
            ipos = istar3+k1+j             ! position in kmeshm
            if ( debug ) then
               write(irefwr,1) 'j, nodp, ipos', j, nodp, ipos
               write(irefwr,1) 'kmeshm(ipos)', kmeshm(ipos)
            end if  ! ( debug )

            if ( kmeshm(ipos)<0 ) then

!           --- Position in kmeshm negative, hence also kmeshl must be filled

               if ( kmeshl(-kmeshm(ipos))==0 ) then

!              --- kmeshl has not yet been filled

                  kmeshl(-kmeshm(ipos)) = nodp
                  if ( debug ) &
                     write(irefwr,1) 'kmeshl:', -kmeshm(ipos), nodp

               else if ( kmeshl(-kmeshm(ipos))/=nodp ) then

!              --- kmeshl has been filled with other value
!                  give warning

                  call errint( -kmeshm(ipos), 1 )
                  call errint( nodp, 2 )
                  call errint( kmeshl(-kmeshm(ipos)), 3 )
                  call errwar ( 617, 3, 0, 0 )

               end if  ! ( kmeshl(-kmeshm(ipos))==0 )
               kmeshm(ipos) = nodp

            else if ( kmeshm(ipos)>0 ) then

!           --- kmeshm has been filled before

               if ( kmeshm(ipos)/=nodp .and. nodp/=0 ) then

!              --- Position in curve has already been filled with
!                  different nodal point number

                  call errint ( j+1, 1 )
                  call errint ( icurnr, 2 )
                  call errint ( kmeshm(ipos), 3 )
                  call errint ( isurnr, 4 )
                  call errint ( nodp, 5 )
                  call errsub ( 2969, 5, 0, 0 )

               end if  ! ( kmeshm(ipos)/=nodp )

            else

!           --- kmeshm(ipos) = 0, standard situation

               kmeshm(ipos) = nodp

            end if  ! ( kmeshm(ipos)<0 )

         end do  ! j = 1, numnodes
         istart = istart+numnodes   ! raise starting address

      end do  ! i = 1, nextracrvs

1000  call erclos ( 'mshmapcurvs' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshmapcurvs'

      end if  ! ( debug )

      end
