      subroutine mshmarkcurv ( markcurvs, kmeshs, isurfsl, kmeshm, &
                               coornew, coorold )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-02-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark all curves in free surface that have been changed
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     curve
!     free_surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer markcurvs(*), kmeshs(*), isurfsl(0:*), kmeshm(*)
      double precision coornew(3,*), coorold(3,*)

!     coornew        i    Array containing the new coordinates of the nodes
!     coorold        i    Array containing the old coordinates of the nodes
!     isurfsl        i    Array of length nsurfsl+1 that is filled with all
!                         stand alone surfaces that must be adapted
!                         isurfsl(0) contains the number of surfaces in isurfsl
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     kmeshs         i    Kmesh part s, contains information about the various
!                         surfaces. See the PG 13.1
!     markcurvs     i/o   Indicates which curves are adapted
!                         markcurvs(i) = 0, curve is not changed
!                         markcurvs(i) = 1, curve is changed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istar3, numsurfs, i, isurnr, istart, numcurvs, j, icurnr, &
              k1, k2, icount, nodp
      double precision dist
      logical changed

!     changed        If true the curve has been changed
!     dist           Distance between two points (squared)
!     i              Counting variable
!     icount         Counter
!     icurnr         Curve sequence number
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         First position for surface in kmeshs
!     isurnr         Surface sequence number
!     j              Counting variable
!     k1             First point of curve in KMESH part m
!     k2             Last point of curve in KMESH part m
!     nodp           Nodal point sequence number
!     numcurvs       Number of curves in surface
!     numsurfs       number of surfaces in nsurfsl
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmarkcurv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshmarkcurv'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill array markcurvs for all curves that have been changed

      istar3 = kmeshm(2)-1   ! starting address of node numbers
      numsurfs = isurfsl(0)
      do i = 1, numsurfs
         isurnr = abs ( isurfsl(i) )
         istart = kmeshs(isurnr)
         numcurvs = kmeshs(isurnr+1)-istart
         do j = 1, numcurvs
            icurnr = abs(kmeshs(istart+j))
            k1 = kmeshm(4+icurnr)
            k2 = kmeshm(5+icurnr)-1
            changed = .false.
            icount = k1

!           --- Loop over all nodes

            do while ( .not. changed .and. icount<=k2 )

!           --- Changed node yet found

               icount = icount+1
               nodp = kmeshm(istar3+icount)  ! node on curve
               dist = (coornew(1,nodp)-coorold(1,nodp))**2+ &
                      (coornew(2,nodp)-coorold(2,nodp))**2+ &
                      (coornew(3,nodp)-coorold(3,nodp))**2
               if ( dist>epsmac ) then

 !             --- node has been changed

                  changed = .true.
                  markcurvs(icurnr) = 1

               end if  ! ( dist>epsmac )

            end do  ! while ( .not. changed .and. icount<nnodes )

         end do  ! j = 1, numcurvs

         if ( debug ) then
            write(irefwr,1) 'i, isurnr, numcurvs', i, isurnr, numcurvs
            call prinin ( kmeshs(istart+1), numcurvs, 'kmeshs' )
         end if  ! ( debug )

      end do  ! i = 1, numsurfs

1000  call erclos ( 'mshmarkcurv' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshmarkcurv'

      end if  ! ( debug )

      end
