      subroutine mshmarkedgpar ( refedges, nsubdiv, &
                                 iedges, newtransinfo, newblockinfo, &
                                 lennewtrans, istorest, leng2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     make sure that edges on common sides of blocks in parallel case
!     are subdivided in both blocks
! **********************************************************************
!
!                       KEYWORDS
!
!     edge
!     refine
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer refedges(*), nsubdiv, iedges(*), &
              leng2, newblockinfo(leng2), lennewtrans, &
              istorest(4,*), newtransinfo(lennewtrans)

!     iedges        i/o   Work array of size nedges
!                         If iedges(i)>0, edge i must be splitted and
!                         the node number will be iedges(i)+npoint
!     istorest       o    array of length (4,iacnodes) containing information
!                         for the various blocks
!                         istorest(1,iblock):  starting address of block iblock
!                         istorest(2,iblock):  number of edges of block iblock
!                         istorest(3,iblock):  starting address in transinfo
!                         istorest(4,iblock):  sequence number of neighbor
!     leng2          i    Declared length of array newblockinfo
!     lennewtrans   i/o   Length of new array transinfo
!                         At input declared length
!                         At output actual length
!     newblockinfo  i/o   New array blockinfo
!     newtransinfo   o    New array transinfo
!     nsubdiv        i    Number of edges that are subdivided, hence number
!                         of extra nodes to be created
!     refedges      i/o   Contains the edge numbers of the edges that
!                         must be subdivided
!                         Length: nsubdiv
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numneigh, i, iblock, istartinfo, nneigh, j, &
              inode, ipstart, nnode_edge, inode1, inode2, icontext, &
              icount, iedge, ipstore, nstore, &
              lenrefedges, istartnew, nelems
      integer, allocatable :: imark(:), istore(:,:), istorerec(:,:)
      integer, pointer :: send_buffer(:)
      integer, save :: send_buffer_size

!     i              Counting variable
!     iblock         Block sequence number
!     icontext       Tag for sending and receiving, is used as check
!     icount         Counter
!     iedge          edge sequence number
!     imark          Work array of length npoint
!                    if k = imark(inode)>0, k defines the sequence number
!                    of node inode in transinfo
!     inode          Node sequence number
!     inode1         First node in edge
!     inode2         Second node in edge
!     ipstart        Starting address in array
!     ipstore        Starting address of array store
!     istartinfo     Starting address in array transinfo
!     istartnew      Starting address in array newtransinfo
!     istore         Work array of length (2,nsubdiv) to store the local
!                    node numbers of the edges at the interface that are
!                    intersected
!     istorerec      Work array of length (2,nsubdiv) to store the local
!                    node numbers of the edges at the interface that are
!                    intersected in the neighbor block
!     j              Counting variable
!     lenrefedges    Computed length of array refedges
!     nelems         number of elements connected to edge
!     nneigh         Number of nodes on common side of two blocks
!     nnode_edge     number of nodes corresponding to edge
!     nstore         number of edges in istore
!     numneigh       number of neighbor blocks
!     send_buffer    send buffer
!     send_buffer_size  Length of the send buffer
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INIBSENDMPI    Send information to other node (mpi buffered)
!     INIBUFATTMPI   Attach mpi buffer if it exists
!     INIBUFDETMPI   Detach mpi buffer if it exists
!     INIBUFSIZEMPI  Extend send_buffer_size with mpi overhead
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INIRECEIVEMPI  Receive information from other node (mpi)
!     MSHMARKEDGLEN  Compute length of array istore
!     MSHMARKEDGPAR1 Compare both sets of nodes on common interface of two
!                    blocks and if there are differences mark extra edges
!     MSHTRANSINFONW Fill new array transinfo from old array transinfo
!                    and array iedges
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINKMESHPAR   Print the contents of the parallel arrays blockinfo and
!                    transinfo
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array imark(npoint)
!     For all neighbor blocks do
!        For all nodes inode on common side of present block and iblock do
!          imark(inode) := sequence number in transinfo
!        For all edges that must be refined do
!           If both nodes satisfy imark(inode)>0 then
!              Store local node numbers in istore
!        Send istore to neighbor
!        Receive istore_neigh from neighbor
!        Compare both sets and if there are differences mark extra edges
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmarkedgpar' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshmarkedgpar'
         write(irefwr,1) 'nsubdiv, lennewtrans', nsubdiv, lennewtrans
         call prinin ( refedges, nsubdiv, 'refedges' )
         call prinkmeshpar ( blockinfo, transinfo )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numneigh = blockinfo(1)  ! number of neighbors of block

!     --- allocate work arrays

      i = max(1,nsubdiv)
      allocate ( imark(npoint), istore(2,i), &
                 istorerec(2,npoint), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, 3*npoint+2*nsubdiv+6*iacnodes+nedges, &
                        'imark' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array imark(npoint)

      imark = 0

      call mshmarkedglen ( blockinfo, transinfo, &
                           refedges, nsubdiv, nmeshc, nmeshh, &
                           send_buffer_size, imark )

      send_buffer_size = 4*send_buffer_size

!     --- Compute actual buffer size using mpi information
!         allocate array and attach buffer

      call inibufsizempi ( send_buffer_size, numneigh )

      allocate(send_buffer(send_buffer_size))

      call inibufattmpi ( send_buffer, send_buffer_size )

      imark = 0

      ipstore = 0  ! starting address in istore (-1)
      length = lennewtrans   ! declared length of newtransinfo

!     --- For all neighbour blocks do

      call inichkarlen ( 2*numneigh, leng2, 'newblockinfo' )
      if ( ierror/=0 ) go to 1000

      do i = 1, numneigh

         iblock =  blockinfo(2*i)        ! neighbor block
         newblockinfo(2*i) = iblock      ! store block number
         istorest(1,iblock) = ipstore+1  ! starting address of block iblock
         istartinfo = blockinfo(2*i+1)   ! starting address in transinfo
         nneigh = transinfo(istartinfo)  ! number of nodes on common side
         istorest(3,iblock) = istartinfo
         istorest(4,iblock) = i

         if ( debug ) then
            write(irefwr,1) 'i, iblock, istartinfo, nneigh, ipstore', &
                             i, iblock, istartinfo, nneigh, ipstore
            call prinin ( transinfo(istartinfo+1), nneigh, &
                          'nodes on common side' )
         end if  ! ( debug )

!        --- For all nodes inode on common side of present block and iblock do
!              imark(inode) := sequence number in transinfo

         do j = 1, nneigh
            inode = transinfo(istartinfo+j)  ! node on common side
            imark(inode) = j
         end do  ! j = 1, nneigh

!        --- For all edges that must be refined do
!              If both nodes satisfy imark(inode)>0 then
!                 Store local node numbers in istore

         nstore = 0  ! number of edges in istore
         do j = 1, nsubdiv

            iedge = refedges(j)  ! edge sequence number
            ipstart = nmeshh(iedge)  ! starting address
            nelems = nmeshh(ipstart)  ! number of elements connected to edge

            if ( debug ) &
               write(irefwr,1) 'j, iedge, nelems', j, iedge, nelems
            if ( nelems==1 ) then

!           --- only one element connected to edge, &
!               hence edge may be on interface

               ipstart = nmeshc(iedge)  ! starting address
               nnode_edge = nmeshc(ipstart)  ! number of nodes on edge
               inode1 = nmeshc(ipstart+1)    ! first node
               inode2 = nmeshc(ipstart+nnode_edge) ! last node

               if ( debug ) then
                  write(irefwr,1) 'iedge, inode1, inode2', &
                                   iedge, inode1, inode2
                  write(irefwr,1) 'imark(inode1), imark(inode2)', &
                                   imark(inode1), imark(inode2)
               end if  ! ( debug )

               if ( imark(inode1)>0 .and. imark(inode2)>0 ) then

!              --- edge found, store nodes

                  nstore = nstore+1
                  if ( imark(inode1)<imark(inode2) ) then

!                 --- imark(node1)<imark(node2), use this sequence

                     istore(1,nstore+ipstore) = imark(inode1)
                     istore(2,nstore+ipstore) = imark(inode2)

                  else

!                 --- imark(node1)>imark(node2), use this reversed sequence

                     istore(1,nstore+ipstore) = imark(inode2)
                     istore(2,nstore+ipstore) = imark(inode1)

                  end if  ! ( imark(inode1)<imark(inode2) )

               end if  ! ( imark(inode1)>0 .and. imark(inode1)>0 )

            end if  ! ( nelems==1 )

         end do  ! j = 1, nsubdiv

!        --- Send istore to neighbor

         if ( nstore==0 ) then

!        --- Nothing to be sent, but that gives problems since there is nothing
!            to receive in thate case
!            Therefore we send two zeros

             istore(1:2,1+ipstore) = 0
             nstore = 1

         end if  ! ( nstore==0 )

         if ( debug ) then
            write(irefwr,1) 'i, iblock', i, iblock
            call prinin1 ( istore(1,1+ipstore), nstore, 2, 'istore' )
         end if  ! ( debug )

         icontext = 23
         call inibsendmpi ( istore(1,1+ipstore), 2*nstore, SPINT, &
                            iblock, icontext )

         ipstore = ipstore+nstore
         istorest(2,iblock) = nstore    ! number of edges of block iblock

      end do  ! i = 1, numneigh

      if ( debug ) call prinin1 ( istorest, iacnodes, 2, 'istorest' )

!     --- For all neighbour blocks do

      istartnew = 1
      do i = 1, numneigh

!     --- Receive istore_neigh from neighbor iblock

         icount = 2*npoint
         call inireceivempi ( istorerec, icount, SPINT, iblock, &
                              icontext )

         icount = icount/2  ! number of entries for second dimension

         if ( debug ) then
            write(irefwr,1) 'i, iblock, istartnew', i, iblock, istartnew
            call prinin1 ( istorerec, icount, 2, 'istorerec' )
         end if  ! ( debug )

!        --- Compare both sets and if there are differences mark extra edges

         ipstore = istorest(1,iblock)
         nstore = istorest(2,iblock)
         istartinfo = istorest(3,iblock)
         if ( debug ) write(irefwr,1) 'ipstore, nstore', ipstore, nstore
         call mshmarkedgpar1 ( istore(1,ipstore), istorerec, nstore, &
                               icount, iedges, nsubdiv, &
                               refedges, lenrefedges, &
                               transinfo(istartinfo+1) )
         if ( debug ) write(irefwr,1) 'lenrefedges/1', lenrefedges

!        --- Compute new array transinfo

         call inichkarlen ( 2*istorest(4,iblock)+1, leng2, &
                            'newblockinfo' )
         if ( ierror/=0 ) go to 1000

         newblockinfo(2*istorest(4,iblock)+1) = istartnew
         if ( debug ) write(irefwr,1) 'istartnew/1', istartnew
         call mshtransinfonw ( refedges, lenrefedges, iedges, &
                               transinfo(istartinfo), &
                               newtransinfo(istartnew) )
         istartnew = istartnew+newtransinfo(istartnew)+1
         if ( debug ) write(irefwr,1) 'istartnew/2', istartnew
         lennewtrans = istartnew-1

!        --- Check array length

         call inichkarlen ( lennewtrans, length, 'newtransinfo' )
         if ( ierror/=0 ) go to 1000

      end do  ! i = 1, numneigh

      if (associated(send_buffer)) then

!     --- deattach send_buffer and deallocate

         call inibufdetmpi ( send_buffer, send_buffer_size )
         deallocate(send_buffer)

      end if  ! (associated(send_buffer))

      deallocate ( imark, istore, istorerec, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'imark' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nsubdiv, lennewtrans', nsubdiv, lennewtrans
         call prinin ( iedges, nedges, 'iedges' )
         call prinkmeshpar ( newblockinfo, newtransinfo )
         write(irefwr,*) 'End mshmarkedgpar'

      end if  ! ( debug )
      call erclos ( 'mshmarkedgpar' )

      end
