      subroutine mshnormalplane ( nodes, nnodes, coor, normal, isurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-02-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the normal on a set of coordinates in a plane
!
! **********************************************************************
!
!                       KEYWORDS
!
!     normal_vector
!     plane
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nodes(*), nnodes, isurnr
      double precision coor(3,*), normal(3)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     isurnr         i    Surface sequence number
!     nnodes         i    Number of nodes in nodes
!     nodes          i    Contains the node numbers in the plane
!     normal         o    Contains the normal on the plane
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inode1, inode2, i, nodp, nodsav
      double precision sum, x1(3), x2(3), t1(3), t2(3), prodsv, &
                       produc, sumsav

!     i              Counting variable
!     inode1         First node in edge
!     inode2         Second node in edge
!     nodp           Nodal point sequence number
!     nodsav         Nodal point number to be saved
!     prodsv         Inner product between two tangential vectors to be
!                    saved
!     produc         Inner product between two tangential vectors
!     sum            Help parameter to compute a sum
!     sumsav         help variable to store the norm of the vector t2
!     t1             First tangential vector
!     t2             Second tangential vector
!     x1             first position vector
!     x2             second position vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1788   The angle between the first point at the boundary and all other
!           points at the boundary is too small
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Define t1 = (x2-x1) / ||x2-x1|| (First tangential vector)
!     Find i such that |(t2*, t1)| < 0.5  with t2* = (xi-x1) / ||xi-x1||, i.e.
!     Each vector x in the plane may be written as:
!      x = lambda t1 + mu t2* &
!     Define second tangential vector orthogonal to t1:
!     t2 = lambda t1 + mu t2*, such that (t2, t1) = 0:
!     lambda (t1, t1) + mu (t2*, t1) = 0, hence
!     mu=1, lambda=-(t2*, t1)
!     t2:=t2/||t2||
!                                           | t12 t23 - t13 t22 |
!     n = t1 x t2 / ||t1 x t2||;  t1 x t2 = | t13 t21 - t11 t23 |
!                                           | t11 t22 - t12 t21 |
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshnormalplane' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshnormalplane'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- Define t1 = (x2-x1) / ||x2-x1|| (First tangential vector)

      inode1 = nodes(1)
      inode2 = nodes(2)
      sum = 0d0
      do i = 1, 3
         x1(i) = coor(i,inode1)
         x2(i) = coor(i,inode2)
         t1(i) = x2(i)-x1(i)
         sum = sum+t1(i)*t1(i)
      end do  ! i = 1, 3
      sum = sqrt(sum)
      do i = 1, 3
         t1(i) = t1(i)/sum
      end do

!     --- Find i such that |(t2*, t1)| < 0.5  with t2* = (xi-x1) / ||xi-x1||, &
!         i.e. Each vector x in the plane may be written as:
!         x = lambda t1 + mu t2* &

      prodsv = rinfin
      do nodp = 2, nnodes
         sum = 0d0
         inode2 = nodes(nodp)
         do i = 1, 3
            t2(i) = coor(i,inode2)-coor(i,inode1)
            sum = sum+t2(i)*t2(i)
        end do
        if ( sum.le.0d0 ) go to 160
        sum = sqrt(sum)
        do i = 1, 3
           t2(i) = t2(i)/sum
        end do

!       --- check angle between t1 and t2* &

         produc = 0d0
         do i = 1, 3
            produc = produc+t1(i)*t2(i)
         end do
         if ( abs(prodsv).gt.abs(produc) ) then
            prodsv = produc
            nodsav = inode2
            sumsav = sum
         end if

         if ( abs(produc).lt.0.5d0 ) goto 210

160   end do

!     --- No point found such that the angle is large enough

      if ( abs(prodsv).lt.0.99d0 ) then

!     --- Inner product less than .99
!         use nodsav as nodal point to compute second tangential vector

         do i = 1, 3
            t2(i) = coor(i,nodsav) -coor(i,inode1)
         end do

         do i = 1, 3
            t2(i) = t2(i) / sumsav
         end do
         produc = prodsv

      else

!     --- All inner products are larger than .99, so all angles are too small

         call errint ( isurnr, 1 )
         call errsub ( 1788, 1, 0, 0 )

      end if

!     --- Define second tangential vector orthogonal to t1:
!         t2 = lambda t1 + mu t2*, such that (t2, t1) = 0:
!         lambda (t1, t1) + mu (t2*, t1) = 0, hence
!         mu=1, lambda=-(t2*, t1)
!         t2:=t2/||t2||

210   sum = 0d0

      do i = 1, 3
         t2(i) = t2(i)-produc*t1(i)
         sum = sum+t2(i)*t2(i)
      end do

      sum = sqrt(sum)
      do i = 1, 3
         t2(i) = t2(i)/sum
      end do

!     --- | t12 t23 - t13 t22 |
!         n = t1 x t2 / ||t1 x t2||;  t1 x t2 = | t13 t21 - t11 t23 |
!                                               | t11 t22 - t12 t21 |
!         The general formula for a plane is defined by
!            (x, y, z) n = d

      normal(1) = t1(2)*t2(3) - t1(3)*t2(2)
      normal(2) = t1(3)*t2(1) - t1(1)*t2(3)
      normal(3) = t1(1)*t2(2) - t1(2)*t2(1)
      sum = 0d0
      do i = 1, 3
         sum = sum + normal(i)*normal(i)
      end do
      sum = sqrt(sum)
      do i = 1, 3
         normal(i) = normal(i)/sum
      end do

1000  call erclos ( 'mshnormalplane' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshnormalplane'

      end if

      end
