      subroutine mshnumstraight ( numpoints, angles, numstraight, &
                                  startpoints, endpoints, isurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 06-04-2005 Correction for inaccurate data
!        version  1.0  date 13-10-2004
!
!   copyright (c) 2004-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Find the number of straight lines as well as the start and end points
!      of these lines in a pie (at most 2)
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     line
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numpoints, numstraight, startpoints(*), endpoints(*), &
              isurnr
      double precision angles(*)

!     angles         i    Array containing the angles along the closed curve in
!                         the sequence defined by curvenodes
!     endpoints      o    Array containing the relative end points of the
!                         straight lines on the part of the circle (if any)
!                         relative with respect to array curvenodes
!     isurnr         i    Surface sequence number
!     numpoints      i    Number of points in array angles
!     numstraight    o    Number of straight lines on the part of the circle
!     startpoints    o    Array containing the relative start points of the
!                         straight lines on the part of the circle (if any)
!                         relative with respect to array curvenodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ifirst, ilast, inext
      double precision eps
      logical finished, commonpoint

!     commonpoint    If true a common point has been found
!     eps            Local accuracy
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     i              Counting variable
!     ifirst         First node in a row
!     ilast          Last node in a row
!     inext          Next point in  a row
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2638   Boudary of circle (part) is straight line
!    2735   the circular surface contains 2 straight lines that are not
!           connected
!    2736   the circular surface contains more than 2 straight lines
! **********************************************************************
!
!                       PSEUDO CODE
!
!     numstraight := 0
!     ifirst := first point with angle = 1
!     ilast := numpoints
!     if ( ifirst>0 ) then
!        inext := next point with angle # 1
!        numstraight := 1
!        if ( ifirst = 1 ) then
!           Find last node with angle # 1
!           ifirst := this last node
!           ilast := ifirst
!           ifirst := ifirst+1
!        startpoints(1) := ifirst-1
!        endpoints(1) := inext
!        if ( inext>=ilast ) then
!           error: the whole line is straight
!        else
!           ifirst := next point with angle = 1
!           if ( ifirst>0 ) then
!              inext := next point with angle # 1
!              numstraight := 2
!              startpoints(2) := ifirst-1
!              endpoints(2) := inext
!              if ( inext>=ilast ) then
!                 error: the boundary consists of 2 straight lines
!              if ( startpoints(2) # endpoints(1) and
!                   endpoints(2) # startpoints(1) ) then
!                 error: the two straight lines have not a common point
!              if ( inext<ilast ) then
!                 if ( there are still straight parts left ) then
!                    error
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshnumstraight' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshnumstraight'
         write(irefwr,100) 'numpoints ', numpoints
100      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set accuracy

      eps = sqreps

!     --- numstraight := 0
!         ilast := numpoints

      numstraight = 0
      ilast = numpoints

!     --- ifirst := first point with angle = 1

      finished = .false.
      ifirst = 0
      do while ( .not. finished )

         ifirst = ifirst+1
         if ( abs(angles(ifirst))>1d0-eps .and. &
              abs(angles(ifirst+1))>1d0-eps ) then

!        --- two consecutive lines with angle 1 found, i.e. straight line

            finished = .true.

         else if ( ifirst>=ilast-1 ) then

!        --- last point has been reached, withoud finding a straight line

            finished = .true.
            ifirst = 0

         end if  ! ( abs(angle-1d0)<eps )

      end do  ! while ( .not. finished )
      if ( debug ) write(irefwr,100) 'ifirst, ilast', ifirst, ilast

      if ( ifirst>0 ) then

!     --- ifirst>0, i.e. there is at least one straight line
!         inext := next point with angle # 1

         inext = ifirst
         finished = .false.
         do while ( .not. finished )

            inext = inext+1
            if ( debug ) &
               write(irefwr,*) 'inext, angle ', inext, angles(inext)
            if ( abs(angles(inext))<1d0-eps ) then

!           --- angle unequal to 1 found, i.e. end of straight line

               finished = .true.

            else if ( inext>=ilast ) then

!           --- last point has been reached

               finished = .true.
               numstraight = 1
               startpoints(1) = max(1,ifirst-1)
               endpoints(1) = 1
               go to 1000

            end if  ! ( abs(angle-1d0)<eps )

         end do  ! while ( .not. finished )

!        --- numstraight := 1

         numstraight = 1
         if ( debug ) &
            write(irefwr,*) 'numstraight, ifirst ', numstraight, ifirst
         if ( ifirst==1 ) then

!        --- ifirst = 1, so first node is on straight line
!            Find last node with angle # 1
!            ifirst := this last node
!            ilast := ifirst
!            ifirst := ifirst+1

            ilast = numpoints+1
            finished = .false.
            do while ( .not. finished )

               ilast = ilast-1
               if ( abs(angles(ilast))<1d0-eps ) then

!              --- angle unequal to 1 found, i.e. end of straight line

                  finished = .true.
                  startpoints(1) = ilast
                  endpoints(1) = inext

               else if ( inext>=ilast ) then

!              --- last point has been reached, withoud finding end of straight
!                  line (error)

                  finished = .true.
                  call errint ( isurnr, 1 )
                  call errsub ( 2638, 1, 0, 0 )
                  go to 1000

               end if  ! ( abs(angle-1d0)<eps )

            end do  ! while ( .not. finished )

         else

!        --- ifirst>1, so first node is not on straight line
!            startpoints(1) := ifirst-1
!            endpoints(1) := inext

            startpoints(1) = ifirst-1
            endpoints(1) = inext

         end if  ! ( ifirst==1 )
         if ( debug ) &
            write(irefwr,*) 'Set 1: ', startpoints(1), endpoints(1)

!        --- ifirst := next point with angle = 1

         ifirst = inext
         finished = .false.
         do while ( .not. finished )

            ifirst = ifirst+1
            if ( abs(angles(ifirst))>1d0-eps ) then

!           --- angle 1 found, i.e. straight line

               finished = .true.

            else if ( ifirst>=ilast ) then

!           --- last point has been reached, withoud finding a straight line

               finished = .true.
               ifirst = 0

            end if  ! ( abs(angle-1d0)<eps )

         end do  ! while ( .not. finished )

         if ( ifirst>0 ) then

!        --- ifirst>0, i.e. there is another straight line
!            inext := next point with angle # 1

            inext = ifirst
            finished = .false.
            if ( debug ) write(irefwr,*) 'ifirst ', ifirst
            do while ( .not. finished )

               inext = inext+1
               if ( debug ) &
                  write(irefwr,*) 'inext, angle ', inext, angles(inext)
               if ( abs(angles(inext))<1d0-eps ) then

!              --- angle unequal to 1 found, i.e. end of straight line

                  finished = .true.

               else if ( inext>=ilast ) then

!              --- last point has been reached, withoud finding end of straight
!                  line

                  finished = .true.
                  if ( debug ) &
                     write(irefwr,*) 'inext, ilast ', inext, ilast
                  if ( ilast==numpoints .and. startpoints(1)/=1 ) then
                     inext = 1
                  end if  ! ( ilast==numpoints .and. startpoints(1)/=1 )

               end if  ! ( abs(angle-1d0)<eps )

            end do  ! while ( .not. finished )

!           --- numstraight := 2
!               startpoints(2) := ifirst-1
!               endpoints(2) := inext

            numstraight = 2
            startpoints(2) = ifirst-1
            endpoints(2) = inext
            commonpoint = startpoints(2)==endpoints(1) .or. &
                          startpoints(1)==endpoints(2) .or. &
                          startpoints(1)==1 .and. &
                          endpoints(2)==numpoints

            if ( .not. commonpoint ) then

!           --- not commonpoint
!               error: the two straight lines do not have a common point

               call errint ( isurnr, 1 )
               call errsub ( 2735, 1, 0, 0 )
               go to 1000

            end if  ! ( startpoints(2)/=endpoints(1) ....

            if ( debug ) then
               write(irefwr,*) 'Set 2: ', startpoints(2), endpoints(2)
               write(irefwr,*) 'inext, ilast ', inext, ilast
            end if  ! ( debug )
            if ( inext<ilast ) then

!           --- inext<ilast, check if there are still straight parts left

               do i = inext+1, ilast

                  if ( abs(angles(inext))>1d0-eps ) then

!                 --- error: more than 2 straight parts

                     call errint ( isurnr, 1 )
                     call errsub ( 2736, 1, 0, 0 )
                     go to 1000

                  end if  ! ( abs(angles(inext))<1d0-eps )

               end do  ! i = inext+1, ilast

            end if  ! ( inext<ilast )

         end if  ! ( ifirst>0 )

      end if  ! ( ifirst>0 )

1000  call erclos ( 'mshnumstraight' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'numstraight ', numstraight
         if ( numstraight>0 ) then
            call prinin ( startpoints, numstraight, 'startpoints' )
            call prinin ( endpoints, numstraight, 'endpoints' )
         end if  ! ( numstraight>0 )
         write(irefwr,*) 'End mshnumstraight'

      end if

      end
