      subroutine msho33 ( coor, ratio, i1, i2, i3, isurnr, ndim, t1, t2, &
                          bcord3d, xmint, ymint, tran, itrans )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.0  date 22-01-2014 Extra parameters
!        version  1.0  date 17-01-2007
!
!   copyright (c) 2007-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Determine the ratio (2*Rout/Rin) for all triangles of the mesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     radius
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer i1, i2, i3, isurnr, ndim, itrans
      double precision ratio, coor(2,*), t1(3), t2(3), bcord3d(3), &
                       xmint, ymint, tran

!     bcord3d        i    Contains the coordinates of the 3d boundary
!                         Is only used for error messages
!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     i1             i    first node of triangle
!     i2             i    second node of triangle
!     i3             i    third node of triangle
!     isurnr         i    Surface sequence number
!                         Is only used for error messages
!     itrans         i    Indicates if the coordinates must be transformed
!                         in case of an error message using xmint, ymint, tran
!                         If 0, no transformation if 1: transformation
!     ndim           i    Dimension of the space.
!                         Is only used for error messages
!     ratio          o    2 * radius divided by radius outer circle
!     t1             i    First tangential vector
!                         Is only used for error messages
!     t2             i    Second tangential vector
!                         Is only used for error messages
!     tran           i    scaling parameter (itrans=1)
!                         Is only used for error messages
!     xmint          i    min x for transformation (itrans=1)
!                         Is only used for error messages
!     ymint          i    min y for transformation (itrans=1)
!                         Is only used for error messages
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision opp, ri, ro, s, s1, s2, s3, &
                       x1, x2, x3, y1, y2, y3, temp(3)

!     i              Counting variable
!     opp            area
!     ri             radius inner circle
!     ro             radius outer circle
!     s              helpvariable to store sum of length of sides
!     s1             euclidian distance side 1
!     s2             euclidian distance side 2
!     s3             euclidian distance side 3
!     temp           Work array to store transformed coordinates
!     x1             x-coordinate node 1
!     x2             x-coordinate node 2
!     x3             x-coordinate node 3
!     y1             y-coordinate node 1
!     y2             y-coordinate node 2
!     y3             y-coordinate node 3
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3168   area of triangle is zero
!    3169   area of triangle is zero
! **********************************************************************
!
!                       PSEUDO CODE
!
!     trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msho33' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msho33'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Determine coordinates

      x1 = coor(1,i1)
      y1 = coor(2,i1)

      x2 = coor(1,i2)
      y2 = coor(2,i2)

      x3 = coor(1,i3)
      y3 = coor(2,i3)

!     --- Determine length of the three sides:

      s1 = sqrt( (x2-x1)*(x2-x1) + (y2-y1)*(y2-y1) )
      s2 = sqrt( (x3-x2)*(x3-x2) + (y3-y2)*(y3-y2) )
      s3 = sqrt( (x1-x3)*(x1-x3) + (y1-y3)*(y1-y3) )

      s = 0.5 * ( s1 + s2 + s3 )

!     --- Calculate the area of the triangle:

      opp = sqrt( s * ( s - s1 ) * ( s - s2 ) * ( s - s3 ) )

      if ( abs(opp)<=0d0 ) then

!     --- area is zero

         if ( itrans==1 ) then
            x1 = (x1-1d0)*tran + xmint
            y1 = (y1-1d0)*tran + ymint
            x2 = (x2-1d0)*tran + xmint
            y2 = (y2-1d0)*tran + ymint
            x3 = (x3-1d0)*tran + xmint
            y3 = (y3-1d0)*tran + ymint
         end if  ! ( itrans==1 )

         call errint ( isurnr, 1 )
         call errint ( i1, 2 )
         call errint ( i2, 3 )
         call errint ( i3, 4 )
         if ( ndim<=2 ) then

!        --- 2-dimensional case

            call erreal ( x1, 1 )
            call erreal ( y1, 2 )
            call erreal ( x2, 3 )
            call erreal ( y2, 4 )
            call erreal ( x3, 5 )
            call erreal ( y3, 6 )
            call errsub ( 3168, 4, 6, 0 )

         else

!        --- 3-dimensional case

            do i = 1, 3
               temp = x1*t1+y1*t2+bcord3d
               call erreal ( temp(i), i )
               temp = x2*t1+y2*t2+bcord3d
               call erreal ( temp(i), i+3 )
               temp = x3*t1+y3*t2+bcord3d
               call erreal ( temp(i), i+6 )
            end do  ! i = 1, 3
            call errsub ( 3169, 4, 9, 0 )

         end if  ! ( ndim==2 )
         go to 1000

      end if  ! ( abs(opp)<=0d0 )

!     --- Calculate the values of the radii of outer and inner circle:

      ro = ( s1 * s2 * s3 ) / ( 4 * opp )

      ri = opp / s

!     --- Finally determine the ratio value:

      ratio = 2 * ri / ro
1000  call erclos ( 'msho33' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msho33'

      end if  ! ( debug )

      end
