      subroutine mshobstcheck1 ( imarknode, iobst, ipointobst, npoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-12-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check consistency of nodes in array ipointobst and elements in array
!     ielemobst
! **********************************************************************
!
!                       KEYWORDS
!
!     obstacle
!     check
!     node
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer imarknode(*), iobst, ipointobst(*), npoint

!     imarknode     i/o   Marks nodes in obstacle
!                         if imarknode(i) = 1, node i is inside the obstacle
!                         and only in elements that are in the obstacle
!                         otherwise imarknode(i) = 0
!     iobst          i    Obstacle sequence number
!     ipointobst     i    Array containing information about the nodes in an
!                         obstacle
!                         See PG Section 24.2 for a description
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inode, iwork(100), icount, istart, numpntobst, i, &
              numinnobst, numbndobst, numinonobst

!     i              Counting variable
!     icount         Counter
!     inode          Node sequence number
!     istart         First position used at a part
!     iwork          integer work array to store nodes for the error message
!     numbndobst     Number of nodal points on the boundary of the obstacle
!     numinnobst     Number of points that are near the boundary of the obstacle
!     numinonobst    Number of nodal points on the boundary of the obstacle
!                    which are only in elements completely inside the obstacle
!     numpntobst     Number of nodal points inside the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2747   nodes found in obstacle that are not in imarknode
!    2748   nodes found in imarknode that are not in obstacle
! **********************************************************************
!
!                       PSEUDO CODE
!
!     icount := 0
!     For all nodes inode in the obstacle do
!        if ( imarknode(inode) # 1 ) then
!           icount := icount+1; iwork(icount) := inode
!        imarknode(inode) := 0
!     if ( icount > 0 ) then
!        error nodes found in obstacle that are not in imarknode
!     icount := 0
!     For all nodes inode in the mesh do
!        if ( imarknode(inode) = 1 ) then
!           icount := icount+1; iwork(icount) := inode
!     if ( icount > 0 ) then
!        error nodes found in imarknode that are not in obstacle
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstcheck1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstcheck1'

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

!     --- Extract parameters

      istart = ipointobst(iobst)
      numpntobst = ipointobst(istart)
      numinnobst = ipointobst(istart+1)
      numbndobst = ipointobst(istart+2)
      numinonobst = ipointobst(istart+3)
      istart = istart+3

!     --- icount := 0

      icount = 0

!     --- For all nodes inode in the obstacle do
!            if ( imarknode(inode) # 1 ) then
!               icount := icount+1; iwork(icount) := inode
!            imarknode(inode) := 0

      do i = 1, numpntobst

         inode = ipointobst(istart+i)   ! node inside obstacle
         if ( imarknode(inode).ne.1 .and. icount.lt.100 ) then

!        --- imarknode(iniode)#1, so node not marked

             icount = icount+1
             iwork(icount) = inode

         end if  ! ( imarknode(inode).ne.1 ) then
         imarknode(inode) = 0

      end do  ! i = 1, numpntobst

      istart = istart+numpntobst+numinnobst+numbndobst
      do i = 1, numinonobst

         inode = ipointobst(istart+i)   ! node inside obstacle
         if ( imarknode(inode).ne.1 .and. icount.lt.100 ) then

!        --- imarknode(iniode)#1, so node not marked

             icount = icount+1
             iwork(icount) = inode

         end if  ! ( imarknode(inode).ne.1 ) then
         imarknode(inode) = 0

      end do  ! i = 1, numinonobst

      if ( icount.gt.0 ) then

!     --- icount > 0
!         error nodes found in obstacle that are not in imarknode

         call errint ( iobst, 1 )
         call errsub ( 2747, 1, 0, 0 )
         call prinin ( iwork, icount, 'node numbers' )

      end if  ! ( icount.gt.0 )

!     --- icount := 0

      icount = 0

!     --- For all nodes inode in the mesh do
!            if ( imarknode(inode) = 1 ) then
!               icount := icount+1; iwork(icount) := inode

      do i = 1, npoint
         if ( imarknode(i).eq.1 .and. icount.lt.100 ) then

!        --- imarknode(inode) = 1, hence node has not been detected in obstacle

            icount = icount+1
            iwork(icount) = i

         end if  ! ( imarknode(inode).eq.1 )

      end do  ! i = 1, npoint

      if ( icount.gt.0 ) then

!     --- icount > 0
!         error nodes found in imarknode that are not in obstacle

         call errint ( iobst, 1 )
         call errsub ( 2748, 1, 0, 0 )
         call prinin ( iwork, icount, 'node numbers' )

      end if  ! ( icount.gt.0 )

1000  call erclos ( 'mshobstcheck1' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstcheck1'

      end if

      end
