      subroutine mshoutercurves ( iinput, ioutercurvs, lenoutercurvs, &
                                  iseq )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-01-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the outer boundary of the set of 2d elements defined by
!     a set of element groups in array iinput
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     boundary
!     outer_boundary
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), ioutercurvs(*), lenoutercurvs, iseq

!     iinput         i    Integer input array describing which element
!                         groups must be coupled to a sequence number
!                         iinput must be filled as follows:
!                         Pos. 1  numelgrps: number of element groups
!                              2-numelgrps+1  element group sequence numbers
!     ioutercurvs    i    Contains information about the curves in the outer
!                         boundary (2D) in the following way
!                         Pos. 1  nparts  Number of parts the outer boundary
!                                         consists of
!                         Pos. 2-1+nparts Starting addresses of each part
!                         Per part: (first address: ioutercurvs(1+i)
!                         Pos. 1: nouter  Number of curves in part
!                         followed by the curve numbers (provided with a sign)
!                         These curve numbers are consecutive, which means
!                         the end of one curve is the start of the new one
!                         The part itself is closed
!     iseq           i    Sequence number of input (used for error messages)
!     lenoutercurvs i/o   Length of array ioutercurvs
!                         At input: declared length
!                         At input: computed length
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer listgrp(nelgrp), i, ngroups, ielgrp, &
              ielem, k1, k2, &
              istore(ncurvs), k, ksave, iedge, numedges, inode, &
              ipstart, istar3, nouter, numelems, numsurelms
      integer :: iworkelem(nelem), iwork(nedges)
      logical single

!     i              Counting variable
!     iedge          edge sequence number
!     ielem          Element number
!     ielgrp         element group number
!     inode          Node sequence number
!     ipstart        Starting address in array
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istore         Work array of length ncurvs used to store the set of outer
!                    curves
!     iwork          integer work array of length nedges
!                    During the computation the number of nodes of a curve
!                    coupled to edge i is stored in iwork(i)
!     iworkelem      integer work array of length nelem
!                    iworkelem(i) = 1, if element i belong to set of
!                    elements in active element groups, otherwise
!                    iworkelem(i) = 0
!     k              Counting variable
!     k1             First point of curve in KMESH part m
!     k2             Last point of curve in KMESH part m
!     ksave          Help variable to store the value of k
!     listgrp        Array of size nelgrp indicating if an element
!                    group must be used (1) or not (0)
!     ngroups        number of groups
!     nouter         Number of points outside the obstacle
!     numedges       number of edges
!     numelems       Number of elements
!     numsurelms     Number of surface elements
!     single         If true the curve is not compound
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHOUTERCURVS1 Organize the curves stored in array istore such that
!                    connected curves are stored in parts
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2484   No outer surface found for this mesh
!    2485   Element group sequence number out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill array listgrp for all element groups in list
!     Clear array iworkelem
!     Make iworkelem(i) = 1 if i is surface element corresponding to list
!     of element groups in iinput
!     For all curves icurnr do
!        find edge number iedge corresponding to first curve element in icurnr
!        find the element numbers corresponding to iedge
!        if ( number of surface elements connected to iedge = 1 ) then
!           curve icurnr is outer curve
!           find direction of normal on edge
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshoutercurves' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshoutercurves'
         write(irefwr,*) 'ncurvs, nedges, nelem ', ncurvs, nedges, nelem
         call prinin ( iinput, iinput(1), 'iinput' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      do i = 1, nedges
         iwork(i) = 0
      end do

!     --- Fill array listgrp for all element groups in list

      do i = 1, nelgrp
         listgrp(i) = 0
      end do
      ngroups = iinput(1)

      do i = 1, ngroups

         ielgrp = iinput(1+i)
         if ( ielgrp<1 .or. ielgrp>nelgrp ) then

!        --- element group out of range

            call errint ( ielgrp, 1 )
            call errint ( nelgrp, 2 )
            call errint ( i, 3 )
            call errint ( iseq, 4 )
            call errsub ( 2485, 4, 0, 0 )
            go to 1000

         end if  ! ( ielgrp<1 .or. ielgrp>nelgrp )
         listgrp(ielgrp) = 1

      end do  ! i = 1, ngroups

!     --- Clear array iworkelem

      do i = 1, nelem
         iworkelem(i) = 0
      end do

!     --- Make iworkelem(i) = 1 if i is surface element and in list

      ielem = 1

      do ielgrp = 1, nelgrp

         ishape = kmeshf(ielgrp)
         inelem = Topgrp(ielgrp)%inelem
         if ( ishape>2 .and. ishape<=10 .and. &
              listgrp(ielgrp)>0 ) then

!        --- 3 <= ishape <= 10, hence surface element

            do i = ielem, ielem+inelem-1
               iworkelem(i) = 1
            end do  ! i = ielem, ielem+inelem-1

         end if  ! ( ishape>10 )
         ielem = ielem+inelem

      end do  ! ielgrp = 1, nelgrp

!     --- For all curves icurnr do

      nouter = 0
      istar3 = kmeshm(2)-1
      do icurnr = 1, ncurvs

!     --- find edge number iedge corresponding to first curve element in icurnr
!         first find first node with positive node number on curve

         if ( kelmq>0 ) then

!        --- There are composite surfaces

            single = kmeshq(icurnr+1)-kmeshq(icurnr)<=1

         else

!        --- No composite surfaces

            single = .true.

         end if  ! ( kelmq>0 )

         if ( .not. single ) go to 200  ! jump in case of compount curve

         k1 = kmeshm(4+icurnr)
         k2 = kmeshm(5+icurnr)-1
         do k = k1, k2-1
            if ( kmeshm(k+istar3)>0 .and. kmeshm(k+1+istar3).gt.0 ) then

!           --- inodp > 0, first node on curve found, store k

               ksave = k
               go to 100

            end if

         end do  ! k = k1, k2

!        --- if the routine comes here, no node has been found
!            we skip this curve
!            This is for example the case if the curve is not coupled
!            to surface elements and the mesh
!            An example is the obstacle surface

         if ( debug ) write(irefwr,*) 'Curve ', icurnr, &
            ' is not connected '
         go to 200

!        --- At least two neighbour nodes found on curve
!            Find all edges connected to these nodes

100      do k = ksave, ksave+1
            inode = kmeshm(k+istar3)
            ipstart = nmeshl(inode)
            numedges = nmeshl(ipstart)  ! number of edges connected to inode
            do i = 1, numedges
               iedge = nmeshl(ipstart+i) !edge connected to inode
               iwork(iedge) = iwork(iedge)+1

!              --- As soon as iwork(iedge) = 2, we have found the edge
!                  number and we leave the loop

               if ( iwork(iedge)==2 ) go to 150

            end do  ! i = 1, numedges

         end do  ! k = ksave, ksave+1

!        --- Clear array iwork

150      if ( debug ) write(irefwr,*) 'iedge ', iedge
         do k = ksave, ksave+1

            inode = kmeshm(k+istar3)
            ipstart = nmeshl(inode)
            numedges = nmeshl(ipstart)  ! number of edges connected to inode

            do i = 1, numedges

               iwork(nmeshl(ipstart+i)) = 0 !edge connected to inode

            end do  ! i = 1, numedges

         end do  ! k = ksave, ksave+1

!        --- find the element numbers corresponding to iedge

         ipstart = nmeshh(iedge)
         numelems = nmeshh(ipstart)
         numsurelms = 0   ! number of surface elements corresponding to iedge
         do i = 1, numelems

            if ( iworkelem(nmeshh(ipstart+i))>0 ) then

!           --- element is surface element

               ielem = nmeshh(ipstart+i) ! surface element corresponding
                                         ! to iedge
               numsurelms = numsurelms+1

            end if

         end do  ! i = 1, numelems
         if ( debug ) then
            write(irefwr,*) 'numsurelms ', numsurelms
            call prinin ( nmeshh(ipstart+1), numelems, 'elements' )
         end if

         if ( numsurelms==1 ) then

!        --- number of surface elements connected to iedge = 1
!            curve icurnr is outer surface
!            find direction of normal on edge

            ipstart = nmeshd(ielem)
            numedges = nmeshd(ipstart)  ! number of edges corresponding to ielem
            do i = 1, numedges

               if ( abs(nmeshd(ipstart+i))==iedge ) then

!              --- edge found

                  nouter = nouter+1
                  if ( nmeshd(ipstart+i)>0 ) then

!                 --- direction is positive

                     istore(nouter) = icurnr

                  else

!                 --- direction is negative

                     istore(nouter) = -icurnr

                  end if

               end if  ! ( abs(nmeshd(ipstart+i))==iedge )

            end do  ! i = 1, numedges

         end if  ! ( numsurelms==1 )

200   end do  ! icurnr = 1, ncurvs
      if ( debug ) call prinin ( istore, nouter, 'istore' )

      if ( nouter==1 ) then

!     --- simple case, only one outer curve

          ioutercurvs(1) = 1           ! nparts
          ioutercurvs(2) = 3           ! starting address
          ioutercurvs(3) = 1           ! nouter
          ioutercurvs(4) = istore(1)   ! curve number with sign
          lenoutercurvs = 4

      else if ( nouter>1 ) then

!     --- complex case, more than one outer curve
!         Now we have to check which curves form a part

         call mshoutercurvs1 ( istore, nouter, ncurvs, ioutercurvs, &
                               lenoutercurvs, npoint, kmeshm )

      else

!     --- error: no outer surfaces found

         call errsub ( 2484, 0, 0, 0 )

      end if  ! ( nouter==1 )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( ioutercurvs, lenoutercurvs, 'ioutercurvs' )
         write(irefwr,*) 'End mshoutercurves'

      end if
      call erclos ( 'mshoutercurves' )

      end
