      subroutine mshowncurve ( ndim, curves, iinput, rinput, npoints, &
                               nuspnt, icurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 23-06-2005 Correction eps
!        version  1.0  date 24-06-2004
!
!   copyright (c) 2004-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill part of array curves for a user curve
!     npoints positions are filled
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), ndim, npoints, nuspnt, icurnr
      double precision curves(ndim,*), rinput(ndim,*)

!     curves         o    array containing the coordinates of the curves
!     icurnr         i    Curve sequence number
!     iinput         i    integer input array
!                         Position 1 contains ifunc, positions 2 en 3
!                         the user point numbers of begin and end point
!     ndim           i    Dimension of the space.
!     npoints        i    Number of points
!     nuspnt         i    Number of user points
!     rinput        i/o   array containing the coordinates of the user points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision x1(3), x2(3), amax, eps, diff
      integer i, i1, i2, ifunc

!     amax           Maximum value of coordinates in a point
!     diff           difference between coordinates
!     eps            Local accuracy
!     i              Counting variable
!     i1             First user point of curve
!     i2             Last user point of curve
!     ifunc          Choice parameter with respect to function calls
!     x1             coordinates of first point
!     x2             coordinates of last point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     OWN_CURVE      User written subroutine that fills the coordinates along
!                    a curve
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     127   user point number out of range
!    2656   The co-ordinates of the first point of the user curve incorrect
!    2657   The co-ordinates of the last point of the user curve incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshowncurve' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshowncurve'
         write(irefwr,*) 'npoints, ndim, icurnr ', npoints, ndim, icurnr

      end if
      if ( ierror/=0 ) go to 1000

!     --- Get begin and end user points

      i1 = iinput(2)
      i2 = iinput(3)
      if ( i1<=0 .or. i1>nuspnt ) then
         call errint ( i1, 1 )
         call errint ( nuspnt, 2 )
         call errsub ( 127, 2, 0, 0 )
      end if
      if ( i2<=0 .or. i2>nuspnt ) then
         call errint ( i2, 1 )
         call errint ( nuspnt, 2 )
         call errsub ( 127, 2, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000

      ifunc = iinput(1)
      call own_curve ( ifunc, icurnr, curves, npoints, ndim )
      if ( debug ) call prinrl1 ( curves, npoints, 2, 'curves' )

!     --- Check initial and end points

      do i = 1, ndim
         x1(i) = rinput(i,i1)
         x2(i) = rinput(i,i2)
      end do
      do i = ndim+1, 3
         x1(i) = 0d0
         x2(i) = 0d0
      end do

!     --- Find maximum coordinates

      amax = max(abs(x1(1)),abs(x1(2)),abs(x1(3)))

      if ( amax<=0d0 ) then

!     --- amax = 0, i.e. either both nodes have not been filled yet, &
!         or they are 0 by coincidence
!         find maximum in

         do i = 1, ndim
            rinput(i,i1) = curves(i,1)
         end do

      else

!     --- amax>0, check coordinates of points

         eps = 1d-5*amax
         diff = 0d0
         do i = 1, ndim
            diff = diff+abs(x1(i)-curves(i,1))
         end do
         if ( diff>eps ) then

!        --- Distance too large

            call erreal ( x1(1), 1 )
            call erreal ( x1(2), 2 )
            call erreal ( x1(3), 3 )
            call erreal ( curves(1,1), 4 )
            if ( ndim>=2 ) then
               call erreal ( curves(2,1), 5 )
            else
               call erreal ( 0d0, 5 )
            end if
            if ( ndim>=3 ) then
               call erreal ( curves(3,1), 6 )
            else
               call erreal ( 0d0, 6 )
            end if
            call errint ( icurnr, 1 )
            call errsub ( 2656, 1, 6, 0 )

         end if  ! ( diff>eps )

      end if  ! ( amax<=0d0 )

      amax = max(abs(x2(1)),abs(x2(2)),abs(x2(3)))

      if ( amax<=0d0 ) then

!     --- amax = 0, i.e. either both nodes have not been filled yet, &
!         or they are 0 by coincidence

         do i = 1, ndim
            rinput(i,i2) = curves(i,npoints)
         end do

      else

!     --- amax>0, check coordinates of points

         eps = 1d-5*amax
         diff = 0d0
         do i = 1, ndim
            diff = diff+abs(x2(i)-curves(i,npoints))
         end do
         if ( diff>eps ) then

!        --- Distance too large

            call erreal ( x2(1), 1 )
            call erreal ( x2(2), 2 )
            call erreal ( x2(3), 3 )
            call erreal ( curves(1,npoints), 4 )
            if ( ndim>=2 ) then
               call erreal ( curves(2,npoints), 5 )
            else
               call erreal ( 0d0, 5 )
            end if
            if ( ndim>=3 ) then
               call erreal ( curves(3,npoints), 6 )
            else
               call erreal ( 0d0, 6 )
            end if
            call errint ( icurnr, 1 )
            call errsub ( 2657, 1, 6, 0 )

         end if  ! ( diff>eps )

      end if  ! ( amax<=0d0 )

1000  call erclos ( 'mshowncurve' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshowncurve'

      end if

      end
