      subroutine mshpar3dblock ( iproci, iprocj, iprock, &
                                 nprocessors, elements, points, &
                                 numelms, numnodes, kmeshclc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-11-2003
!
!   copyright (c) 2003-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array elements with the elements corresponding to one block
!     and also the corresponding array points with the point numbers
!     In this case we assume a block with the same number of elements in
!     all direction and a identical subdivision for all blocks
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     parallel
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iproci, iprocj, iprock, nprocessors, &
              elements(*), points(*), numelms, &
              numnodes, kmeshclc(8,*)

!     elements       o    Array of size nelemloc containing the element
!                         numbers corresponding to the local mesh
!     iproci         i    Loop counter for sub block renumbering in "i"-dir
!     iprocj         i    Loop counter for sub block renumbering in "j"-dir
!     iprock         i    Loop counter for sub block renumbering in "k"-dir
!     kmeshclc       i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nprocessors    i    Number of processors that is to be used for
!                         parallel processing
!                         Defines also the number of submeshes
!     numelms        o    Number of elements
!     numnodes       o    Number of nodal points in array nodes
!     points         o    Array of size npointloc containing the node
!                         numbers corresponding to the local mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iwork(npoint), i, nhalf, nsubdiv, nelem1, &
              ielem, icount, j, k
      double precision h

!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     icount         Counter
!     ielem          Element number
!     iwork          integer work array of size npoint
!                    iwork(i) = 0 means node is not present in this block
!                    iwork(i) = 1 means node is present in this block
!     j              Counting variable
!     k              Counting variable
!     nelem1         number of elements in one direction for the whole block
!     nhalf          number of elements in one direction for one block
!     nsubdiv        number of processors ^ (1/3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2652   nelements is not third power
!    2653   nelements1 is not multiple of nhalf
! **********************************************************************
!
!                       PSEUDO CODE
!
!     clear array iwork(1:npoint)
!     Fill array elements depending on iproci/j/k
!     Set iwork(i) = 1, for all nodes i in elements
!     numnodes := 0
!     For i := 1 (1) npoint do
!        if ( iwork(i)= 1 ) then
!           numnodes := numnodes+1;  points(numnnodes) := i
!     ifirst := ilast+1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpar3dblock' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpar3dblock'
         write(irefwr,*) 'nprocessors, npoint ', &
                          nprocessors, npoint
         write(irefwr,*) 'iproci, iprocj, iprock ', &
                          iproci, iprocj, iprock

      end if

!     --- Initializations

      inelem = nelem
      h = nprocessors
      nsubdiv = nint(h**(1d0/3d0))
      h = inelem
      nelem1  = nint(h**(1d0/3d0))
      if ( nelem1**3/=inelem ) then

!     --- nelements is not third power

         call errint ( inelem, 1 )
         call errint ( nelem1, 2 )
         call errint ( nelem1**3, 3 )
         call errsub ( 2652, 3, 0, 0 )
         go to 1000

      end if
      nhalf = nelem1/nsubdiv
      if ( nhalf*nsubdiv/=nelem1 ) then

!     --- nelem1 is not mutiple of nhalf

         call errint ( nhalf, 1 )
         call errint ( nsubdiv, 2 )
         call errint ( nelem1, 3 )
         call errint ( nhalf*nsubdiv, 4 )
         call errsub ( 2653, 4, 0, 0 )
         go to 1000

      end if
      if ( debug ) write(irefwr,*) 'nsubdiv, nelem1, nhalf ', &
         nsubdiv, nelem1, nhalf

!     --- Clear array iwork (1:npoint)

      iwork = 0

!     --- Fill array elements depending on iproci, iprocj and iprock

      icount = 0
      do k = 1, nhalf

         do j = 1, nhalf

            do i = 1, nhalf

               ielem = (iproci-1)*nhalf+i + &
                       nelem1*((iprocj-1)*nhalf+j-1) + &
                       nelem1**2*((iprock-1)*nhalf+k-1)
               icount = icount+1
               elements(icount) = ielem

            end do  ! i = 1, nhalf

         end do  ! j = 1, nhalf

      end do  ! k = 1, nhalf
      numelms = icount

!     --- Set iwork(i) = 1, for all nodes i in elements

      do i = 1, numelms
         ielem = elements(i)
         do j = 1, 8
            iwork(kmeshclc(j,ielem)) = 1
         end do  ! j = 1, 8
      end do  ! i = 1, numelms

!     --- Fill array points using iwork

      numnodes = 0
      do i = 1, npoint

         if ( iwork(i)==1 ) then

!        --- iwork(i) = 1, hence node belongs to block

            numnodes = numnodes+1
            points(numnodes) = i

         end if

      end do  ! i = 1, npoint

1000  if ( debug ) then
         call prinin ( points, numnodes, 'points' )
         call prinin ( elements, numelms, 'elements' )
         write(irefwr,*) 'End mshpar3dblock'
      end if
      call erclos ( 'mshpar3dblock' )

      end
