      subroutine mshparelems ( methodpar, nprocessors )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.3  date 05-03-2015 Extension with metis
!        version  3.2  date 27-11-2012 Long integers
!        version  3.1  date 18-02-2012 Extra debug
!
!   copyright (c) 2003-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide the region into nprocessors blocks depending on the parameter
!     methodpar
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer methodpar, nprocessors

!     methodpar      i    Defines the type of subdivision of the mesh to be made
!                         for parallel computing
!                         Possible values:
!                         1:  If the mesh contains volumes the number of
!                                subparts is equal to the number of volumes
!                                otherwise equal to the number of surfaces
!                         2:  the mesh is subdivided in layers
!                         3:  the mesh is subdivided in blocks
!                         4:  specially meant for testpurposes:
!                             the number of elements in each region is
!                             approximately the same
!                             elements are not renumbered and are subdivided
!                             in natural sequence
!                         5:  specially meant for testpurposes:
!                             a 3d block of equal number of elements
!                             (hexahedrons) in each direction is subdivided into
!                             subblocks of equal size ( block subdivision)
!                             The number of blocks must be a power of 3
!                         6:  subdivision by metis
!     nprocessors    o    Number of processors that is to be used for
!                         parallel processing
!                         Defines also the number of submeshes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer npointproc, ilast, ifirst, &
              nelements(nprocessors), nelemproc, nsubdiv
      double precision h

!     h              Help variable to store a constant temporarily
!     ifirst         First node in next block
!     ilast          Last node in previous block
!     nelements      Array of size nprocessors containing for each block
!                    the end pointer in array elements
!                    Hence the elements stored in array elements
!                    pos. 1 to nelements(1) belong to block 1, &
!                    the elements from pos.nelements(1)+1 to nelements(2)
!                    belong to block 2 etc
!     nelemproc      Number of elements per processor (approximately)
!     npointproc     Number of nodes per processor (approximately)
!     nsubdiv        number of processors ^ (1/3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHPARELBLOCKS Fill arrays elements and nelements in order to get a
!                    subdivision of the region in blocks for parallel computing
!     MSHPARELEMS1   Subdivide the region into nprocessors blocks depending on
!                    the parameter methodpar
!     MSHPARELEMS2   Subdivide the region into nprocessors blocks depending on
!                    the parameter methodpar
!     MSHPARELEMS4   Subdivide the region into nprocessors blocks such that all
!                    blocks contain connected parts only
!     MSHPARELEMSMET Subdivide the region into nprocessors blocks using
!                    the metis package
!     PRININ         print 1d integer array
!     SEPFILLKMESHE  Fill array kmesh part e
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( methodpar=4 ) then
!        make a subdivision based on elements
!     if ( methodpar=3 ) then
!        make a subdivision in block form
!     if ( methodpar = 1,2 ) then
!        if ( nvolms>0 ) then
!           Not yet implemented
!        else (nvolms=0, nsurfs>0 ) then
!           For all processors do
!              Store all elements and nodes of processor
!     else
!        Not yet implemented
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparelems' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparelems'
         write(irefwr,1) 'methodpar, kelmj, nprocessors', &
                          methodpar, kelmj, nprocessors
  1      format ( a, 1x, (10i6) )

      end if

      if ( methodpar==3 ) then

!     --- methodpar = 3, block oriented subdivision
!         Activate arrays

         if ( kelme==0 ) call sepfillkmeshe

!        --- Compute subdivision in elements

         call mshparelblocks ( nprocessors, nelements )
         if ( debug ) then

!        --- Debug information

            call prinin ( nelements, nprocessors, 'nelements' )

         end if
         if ( ierror/=0 ) go to 1000


      end if

      if ( methodpar==4 ) then

!     --- methodpar = 4, subdivision based on elements (test only)

         nelemproc = nelem/nprocessors
         ilast = 0
         ifirst = 1

      else if ( methodpar==5 ) then

!     --- methodpar = 5, subdivision of a 3d block into subblocks

         h = nprocessors
         nsubdiv = nint(h**(1d0/3d0))

      end if

      if ( methodpar==2 ) then

!     --- Compute some specific quantities

         npointproc = npoint/nprocessors
         ilast = 0
         ifirst = 1

      end if

      if ( methodpar==5 ) then

!     --- methodpar = 5, special case of equal sized blocks, subdivided into
!         equal sized subblocks, just for test purposes

         call mshparelems2 ( nprocessors, nsubdiv )

      else if ( methodpar==2 .and. kelmj/=0 ) then

!     --- methodpar = 2 and renumbering
!         a set of layers is made such that disconnected parts per
!         submesh are impossible

         call mshparelems4 ( nprocessors, ifirst, ilast, npointproc )

      else if ( methodpar==6 ) then

!     --- metis subdivision

         if ( debug ) write(irefwr,*) 'metis subdivision'

         call mshparelemsmet ( nprocessors )

      else

!     --- methodpar = 1 to 4, standard cases

         call mshparelems1 ( methodpar, nprocessors, ifirst, &
                             ilast, nelemproc, npointproc, nelements )

      end if  ! ( methodpar==5 )

1000  if ( debug ) then
         write(irefwr,*) 'End mshparelems'
      end if  ! ( debug )
      call erclos ( 'mshparelems' )

      end
