      subroutine mshpavchkquad ( inode1, inode2, inode3, inode4, &
                                 acceptquad, coor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-07-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if quadrilateral to be subdivided is acceptable
!     Necessary is that all angles are smaller than pi
!     Also the line from mid point of boundary edge must intersect other
!     boundary edge
! **********************************************************************
!
!                       KEYWORDS
!
!     paver
!     mesh
!     mesh_generation
!     quadrilateral
!     angle
!     boundary
!     edge
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inode1, inode2, inode3, inode4
      double precision coor(2,*)
      logical acceptquad

!     acceptquad     o    Check if quadrilateral is acceptable (true) or not
!                         (false)
!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     inode1         i    First node in quadrilateral
!     inode2         i    Second node in quadrilateral
!     inode3         i    Third node in quadrilateral
!     inode4         i    Fourth node in quadrilateral
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ih
      double precision x1(2), x2(2), x3(2), x4(2), w, n(2), x(2), xn(2), &
                       fact1, fact2, area

!     area           Area of element
!     fact1          multiplication factor
!     fact2          multiplication factor
!     i              Counting variable
!     ih             If 1, the quadrilateral is concave and hence not acceptable
!     n              normal vector
!     w              help variable result of MSHG02
!     x              co-ordinates of point
!     x1             co-ordinates of inode1
!     x2             co-ordinates of inode2
!     x3             co-ordinates of inode3
!     x4             co-ordinates of inode4
!     xn             coordinates of next point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERNORMAL2D     Compute normal vector perpendicular on the line through the
!                    points with coordinates (x1,y1) and (x2, y2)
!     EROPEN         Produces concatenated name of local subroutine
!     MSHAREA        Compute the area of the triangle spanned by x1-x2-x3
!                    The sign of the are defines the direction
!     MSHCROSSLINE2  Computes the intersection of two lines
!     MSHG02         Check whether angle is greater or smaller than 180 degrees
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     acceptquad := true
!     For all 4 angles in quadrilateral, while acceptquad do
!        if ( angle > pi ) acceptquad := false
!     if ( acceptquad ) then
!        intersect normal on edge1 through mid point with edge3
!        if ( intersection not in edge3 ) then
!           acceptquad := false
!        intersect normal on edge3 through mid point with edge1
!        if ( intersection not in edge1 ) then
!           acceptquad := false
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavchkquad' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavchkquad'
         write(irefwr,*) 'inode1, inode2, inode3, inode4 ', &
                          inode1, inode2, inode3, inode4

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

!     --- acceptquad := true

      acceptquad = .true.

!     --- Store coordinates of quadrilateral in x1 ... x4

      do i = 1, 2

         x1(i) = coor(i,inode1)
         x2(i) = coor(i,inode2)
         x3(i) = coor(i,inode3)
         x4(i) = coor(i,inode4)

      end do  ! i = 1, 2

!     --- Check direction (must be counter clockwise for mshg02)

      call msharea ( x1, x2, x3, area )
      if ( area.lt.0d0 ) then

!     --- Area < 0, wrong direction, 4 --> 2, 2 --> 4

         do i = 1, 2

            x(i) = x2(i)
            x2(i) = x4(i)
            x4(i) = x(i)

         end do  ! i = 1, 2

      end if  ! ( area.lt.0d0 )

!     --- For all 4 angles in quadrilateral, while acceptquad do
!            if ( angle > pi ) acceptquad := false
!         First angle 1-2-3

      call mshg02 ( x1(1), x1(2), x2(1), x2(2), x3(1), x3(2), ih, w )
      if ( ih.eq.1 ) then

!     --- Concave angle found, jump

         acceptquad = .false.
         if ( debug ) write(irefwr,*) 'angle 1 > pi'
         go to 1000

      end if  ! ( ih.eq.1 )

!     --- Next angle 2-3-4

      call mshg02 ( x2(1), x2(2), x3(1), x3(2), x4(1), x4(2), ih, w )
      if ( ih.eq.1 ) then

!     --- Concave angle found, jump

         acceptquad = .false.
         if ( debug ) write(irefwr,*) 'angle 2 > pi'
         go to 1000

      end if  ! ( ih.eq.1 )

!     --- Next angle 3-4-1

      call mshg02 ( x3(1), x3(2), x4(1), x4(2), x1(1), x1(2), ih, w )
      if ( ih.eq.1 ) then

!     --- Concave angle found, jump

         acceptquad = .false.
         if ( debug ) write(irefwr,*) 'angle 3 > pi'
         go to 1000

      end if  ! ( ih.eq.1 )

!     --- Finally angle 4-1-2

      call mshg02 ( x4(1), x4(2), x1(1), x1(2), x2(1), x2(2), ih, w )
      if ( ih.eq.1 ) then

!     --- Concave angle found, jump

         acceptquad = .false.
         if ( debug ) write(irefwr,*) 'angle 4 > pi'
         go to 1000

      end if  ! ( ih.eq.1 )

      if ( area.lt.0d0 ) then

!     --- Area < 0, reset coordinates

         do i = 1, 2

            x(i) = x2(i)
            x2(i) = x4(i)
            x4(i) = x(i)

         end do  ! i = 1, 2

      end if  ! ( area.lt.0d0 )

!     --- intersect normal on edge1 through mid point with edge3
!          n = normal on (x1-x2)

      call ernormal2d ( x1, x2, n )

!     --- x = mid point on of x1 x2

      x(1) = 0.5d0*(x1(1)+x2(1))
      x(2) = 0.5d0*(x1(2)+x2(2))

!     --- compute intersection x - x+n, x3 - x4

      xn(1) = x(1)+n(1)
      xn(2) = x(2)+n(2)
      call mshcrossline2 ( x, xn, x3, x4, fact1, fact2 )

      if ( fact2.le.1d-2 .or. fact2.gt.1.01d0 ) then

!     --- intersection not in edge3
!         acceptquad := false

         acceptquad = .false.
         if ( debug ) write(irefwr,*) 'no intersection with edge3 ', &
            'fact1 = ', fact1, '  fact2 = ', fact2
         go to 1000

      end if  ! ( fact2.le.1d-2 .or. fact2.gt.1.01d0 )

!     --- intersect normal on edge3 through mid point with edge1
!          n = normal on (x3-x4)

      call ernormal2d ( x3, x4, n )

!     --- x = mid point on of x3 x4

      x(1) = 0.5d0*(x3(1)+x4(1))
      x(2) = 0.5d0*(x3(2)+x4(2))

!     --- compute intersection x - x+n, x1 - x2

      xn(1) = x(1)+n(1)
      xn(2) = x(2)+n(2)
      call mshcrossline2 ( x, xn, x1, x2, fact1, fact2 )

      if ( fact2.le.1d-2 .or. fact2.gt.1.01d0 ) then

!     --- intersection not in edge1
!          acceptquad := false

         acceptquad = .false.
         if ( debug ) write(irefwr,*) 'no intersection with edge1 ', &
            'fact1 = ', fact1, '  fact2 = ', fact2
         go to 1000

      end if  ! ( fact2.le.1d-2 .or. fact2.gt.1.01d0 )


1000  call erclos ( 'mshpavchkquad' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavchkquad'

      end if

      end
