      subroutine mshpavquads ( nmeshl, numbounnodes, nedges, &
                               iworkedge, markedges, nmeshc, nmeshh, &
                               iworkelem, nquads, iworkquads, nmeshd, &
                               nelem, coor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 18-02-2009 Extra debug
!        version  1.0  date 02-07-2004
!
!   copyright (c) 2004-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Combine triangles with one edge on the boundary and the other edges
!     connected with the opposite boundary into quadrilaterals with two edges
!     on opposite boundaries
!     Mark remaining triangles if one edge is on the boundary and the other
!     2 edges are on opposite boundaries
! **********************************************************************
!
!                       KEYWORDS
!
!     triangle
!     edge
!     boundary
!     quadrilateral
!     mesh_generation
!     paver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nmeshl(*), nedges, numbounnodes, iworkedge(nedges), &
              markedges(*), nmeshc(*), nmeshh(*), nquads, &
              iworkquads(2,*), nmeshd(*), nelem, iworkelem(nelem)
      double precision coor(2,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     iworkedge      o    work array of length nedges to be used to mark
!                         edges
!                          0:  not marked yet
!                         -1:  boundary edge that has been used already
!                         -2:  diagonal of quadrilateral that may be removed
!     iworkelem      o    Work array of length nelem to be used to mark
!                         elements
!                         0:  element is not special
!                         1:  element is triangle with all nodes on the boundary
!                         2:  elements is triangle, that is part of a
!                             quadrilateral with all 4 points on the boundary
!     iworkquads     o    work array of size nelem, containing sets of
!                         triangles that may be considered as quads
!                         (1,.) first triangle
!                         (2,.) second triangle
!     markedges      i    array of length nedges indicating whether a
!                         edge is internal or on the boundary
!                         -2:  internal edge between 2 boundary nodes
!                         -1:  boundary edge
!                          0:  standard internal edge
!     nedges         i    number of edges
!     nelem          i    number of elements
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
!     nmeshd         i    nmesh part d, contains edge numbers of elements
!     nmeshh         i    nmesh part h, contains element numbers corresponding
!                         to edges
!     nmeshl         i    nmesh part l, contains edge numbers corresponding
!                         to nodes
!     nquads         o    number of quads stored in iworkquads
!     numbounnodes   i    Number of boundary nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inode, inode1, inode2, iwork(nedges), startnode, &
              numconedge1, numconedge2, iedge1, iedge2, iedge4

!     iedge1         edge sequence number
!     iedge2         edge sequence number
!     iedge4         edge sequence number
!     inode          Node sequence number
!     inode1         First node in edge
!     inode2         Second node in edge
!     iwork          integer work array of length nedges
!                    At start iwork must be set to zero
!                    Is used by lower subroutines
!     numconedge1    number of boundary connecting edges connected to node 1
!     numconedge2    number of boundary connecting edges connected to node 2
!     startnode      Starting node number of boundary part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer iniedge, mshpavnumedges

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIEDGE        Returns with edge number corresponding to 2 nodes
!     MSHPAVBNQUAD1  Find quadrilateral with all 4 nodes on the boundary in
!                    case one of the nodes on one boundary is connected to
!                    only one node on oppsoite boundary
!     MSHPAVBNQUAD2  Find quadrilateral with all 4 nodes on the boundary in
!                    case both nodes on one boundary are connected to more
!                    nodes on opposite boundary
!     MSHPAVBNTRIAN  Check if a triangle may be formed from three given points
!                    If so return with mshpavbntrian true
!     MSHPAVNUMEDGES Return with number of boundary connecting edges in node
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     We are looking for sets of triangles that can be clustered into one
!     quadrilateral (1,2,3,4) with 2 edges on the boundary
!     Situation 1:      Situation 2:
!
!      4      3            4      3
!      x------x            x------x
!      |    / |\           |    / |
!      |   /  |            |   /  |
!      |  /   |            |  /   |
!      | /    |            | /    |/
!      x------x-           x------x- &
!      1      2            1      2
!
!     For all boundary nodes inode do
!        inode1 := inode; inode2 := next node
!        Find boundary edge iedge1 from inode1 to inode2
!        found := false
!        if ( iedge1 not used before ) then
!           Mark iedge1 as used
!           if ( one of the nodes has one boundary connecting edge ) then
!              iedge2 := boundary connecting edge
!              inode3 := node at opposite boundary in iedge3
!              Find non-used boundary edges in inode3
!              if ( only one non-used boundary edge in inode3 ) then
!                 iedge3 := non-used boundary edge
!                 inode4 := new point of iedge3
!                 Mark iedge3 as used
!                 if ( there is an edge from inode4 to inode1 ) then
!                    iedge4 := that edge
!                    Mark diagonal edge
!                    found := true
!              else if ( there are 2 non-used boundary edge in inode3 ) then
!                 if ( one of these edges can be used to define iedge3 and
!                      inode4 ) then
!                    Mark iedge3 as used
!                    if ( there is an edge from inode4 to inode1 ) then
!                       iedge4 := that edge
!                    Mark diagonal edge
!                       found := true
!           else if ( both nodes have more boundary connecting edges ) then
!              compute intersection of normal from midpoint to opposite
!              boundary and find corresponding edge
!              Mark quadrilateral if possible
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavquads' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavquads'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Clear arrays iwork, iworkedge and iworkelem

      iwork = 0
      iworkedge = 0
      iworkelem = 0
      nquads = 0

!     --- For all boundary nodes inode do

      startnode = 1
      do inode = 1, numbounnodes

!     --- inode1 := inode; inode2 := next node
!         Find boundary edge iedge1 from inode1 to inode2
!         found := false

         inode1 = inode
         inode2 = inode+1
         if ( inode2>numbounnodes ) inode2 = startnode
         iedge1 = iniedge ( inode1, inode2, nmeshl, iwork )
         if ( debug ) &
            write(irefwr,1) 'inode, inode1, inode2, iedge1', &
                             inode, inode1, inode2, iedge1
         if ( iedge1==0 ) then

!        --- No edge between inode1 and inode2
!            This is clearly the end of a boundary, try inode2 = startnode

            inode2 = startnode
            startnode = inode+1
            iedge1 = iniedge ( inode1, inode2, nmeshl, iwork )
            if ( iedge1==0 ) go to 100
            if ( debug ) &
               write(irefwr,1) 'inode2, iedge1', inode2, iedge1

         end if ! ( iedge1==0 )

!        --- Check if edge found is indeed a boundary edge

         if ( debug ) &
            write(irefwr,1) 'markedges(iedge1)', markedges(iedge1)
         if ( markedges(iedge1)/=-1 ) then

!        --- edge is not a boundary edge, so inode2 is incorrect

            inode2 = startnode
            startnode = inode+1
            iedge1 = iniedge ( inode1, inode2, nmeshl, iwork )
            if ( iedge1==0 ) go to 100
            if ( markedges(iedge1)/=-1 ) go to 100
            if ( debug ) &
               write(irefwr,1) 'inode2, iedge1', inode2, iedge1

         end if  ! ( markedges(iedge1)==-1 )

         if ( debug ) &
           write(irefwr,1) 'inode1, inode2, iedge1, iworkedge(iedge1)', &
                            inode1, inode2, iedge1, iworkedge(iedge1)
         if ( iworkedge(iedge1)==0 ) then

!        --- iedge1 has not been used before
!            Mark iedge1 as used

            iworkedge(iedge1) = -1

!           --- Compute number of boundary connecting edges connected to nodes

            numconedge1 = &
               mshpavnumedges ( inode1, markedges, nmeshl, iedge4 )
            numconedge2 = &
               mshpavnumedges ( inode2, markedges, nmeshl, iedge2 )
            if ( debug ) then
               write(irefwr,1) 'numconedge1, numconedge2', &
                               numconedge1, numconedge2
               write(irefwr,1) 'iedge4, iedge2', iedge4, iedge2
            end if  ! ( debug )

            if ( numconedge1==1 .and. numconedge2==1 ) then

!           --- There is one boundary connecting edge coupled to node1
!               and one to node 2, so a triangle might have been found

               call mshpavbntrian ( iedge1, iedge2, iedge4, inode1, &
                                    inode2, nmeshc, nmeshh, iworkelem )

            else if ( numconedge1==1 .and. numconedge2>1 ) then

!           --- There is one boundary connecting edge coupled to node1
!               so a quadrilateral has been found

               call mshpavbnquad1 ( iedge1, iedge4, inode1, inode2, &
                                    nmeshc, nmeshl, iwork, nmeshh, &
                                    nmeshd, markedges, iworkelem, &
                                    iworkedge, nquads, iworkquads, &
                                    coor )

            else if ( numconedge2==1 .and. numconedge1>1 ) then

!           --- There is one boundary connecting edge coupled to node2
!               so a quadrilateral has been found

               call mshpavbnquad1 ( iedge1, iedge2, inode2, inode1, &
                                    nmeshc, nmeshl, iwork, nmeshh, &
                                    nmeshd, markedges, iworkelem, &
                                    iworkedge, nquads, iworkquads, &
                                    coor )

            else if ( numconedge1>1 .and. numconedge2>1 ) then

!           --- Complex situation, both nodes have more than one connection
!               to opposite boundary

               call mshpavbnquad2 ( iedge1, inode1, inode2, nmeshl, &
                                    nmeshc, markedges, iwork, nmeshh, &
                                    nmeshd, iworkedge, iworkelem, &
                                    nquads, iworkquads, coor )

            end if  ! ( numconedge1==1 ... )
            if ( ierror/=0 ) go to 1000

         end if  ! ( iworkedge(iedge1)==0 )

100   end do  ! inode = 1, numbounnodes

1000  call erclos ( 'mshpavquads' )

      if ( debug ) then

!     --- Debug information

         call prinin ( iworkedge, nedges, 'iworkedge' )
         call prinin ( iworkelem, nelem, 'iworkelem' )
         call prinin1 ( iworkquads, nquads, 2, 'iworkquads' )
         write(irefwr,*) 'End mshpavquads'

      end if

      end
