      subroutine mshpavtriaadap ( kmeshcloc, nelemloc, npointloc, &
                                  nbounpoints, thickness, &
                                  coorloc, maxnpoint, maxnelem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-06-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt mesh created by subroutine mshpavtrian to satisfy the thickness
!     requirements of paver
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     paver
!     mesh_generation
!     triangle
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshcloc(*), nelemloc, npointloc, nbounpoints, &
              maxnpoint, maxnelem
      double precision thickness(*), coorloc(2,*)

!     coorloc       i/o   array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     kmeshcloc     i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!     maxnelem       i    Maximum number of elements allowed
!     maxnpoint      i    Maximum number of nodal points allowed
!     nbounpoints    i    Number of nodes on the boundary
!     nelemloc      i/o   Number of elements
!     npointloc     i/o   Number of nodal points
!     thickness      i    Array containing the thickness in the boundary nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kmeshlc, nquads, iseqkm1
      logical found, checkarea
      integer, allocatable :: markedges(:), iworkedge(:), iworkelem(:), &
                              iworkquads(:)

!     checkarea      If true the area must be checked
!     found          If true already a possibility has been found
!     iseqkm1        Sequence number with respect to km of first mesh
!     iworkedge      work array of length nedges to be used to mark
!                    edges
!                     0:  not marked yet
!                    -1:  boundary edge that has been used already
!                    -2:  diagonal of quadrilateral that may be removed
!     iworkelem      Work array of length nelem to be used to mark
!                    elements
!                    0:  element is not special
!                    1:  element is triangle with all nodes on the boundary
!                    2:  elements is triangle, that is part of a
!                        quadrilateral with all 4 points on the boundary
!     iworkquads     work array of size nelem, containing sets of
!                    triangles that may be considered as quads
!                    (1,.) first triangle
!                    (2,.) second triangle
!     kmeshlc        Output of the mesh generator
!     markedges      array of length nedges indicating whether a
!                    edge is internal or on the boundary
!     nquads         number of quads stored in iworkquads
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHKSBM         Check volume of submesh
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHPAVEDGES    Fill edges of the mesh and compute edges corresponding to
!                    elements and nodes
!     MSHPAVINTEDGES Mark all internal edges  of the mesh that go from one
!                    boundary point to another boundary point by a -2
!     MSHPAVMARKBEDG Mark all boundary edges by a -1 and all internal edges by
!                    a 0
!     MSHPAVMARKEDGE Fill edges of the mesh and compute edges corresponding to
!                    elements and nodes
!     MSHPAVQUADS    Combine triangles with one edge on the boundary and the
!                    other edges connected with the opposite boundary into
!                    quadrlaterals with two edges on opposite boundaries
!     MSHPAVTRIANEXT Subdivide triangles created by msho2d into smaller
!                    triangles based on the information stored in iworkedges
!     PLOTMESHINTER  Plot actual mesh during the process of creation of a mesh
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
!     SEPCRMESHINFO  Create permanent space for mesh
!     SEPRELKMESH1   Make the relation between kmref and the pointers
!     SEPRMMESHINFO  Remove array with sequence number iseq from km
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Copy old kmeshcloc into kmeshcloccopy
!     Compute all edges in kmeshcloc
!     Detect edges corresponding elements and elements corresponding to edges
!     Detect edges corresponding to nodes
!     Mark boundary edges
!     Find all quadrilaterals consisting of 2 boundary edges and 2 edges
!     connecting both boundary edges
!     Mark edges that must be subdivided
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavtriaadap' )
      debug = .false.
      checkarea = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavtriaadap'
         call plotmeshinter ( nelemloc, kmeshcloc, coorloc, 6, 2, &
                              npointloc, 'triangle' )

      end if  ! ( debug )

      if ( checkarea ) then

!      --- Compute area of region

         call chksbm ( kmeshcloc, nelemloc, 3, 3, isurnr, coorloc, &
                       2, 2, npointloc )

      end if
      if ( ierror.ne.0 ) go to 1000

      iseqkm1 = iseqkm

      kmeshlc = 0
      call sepcrmeshinfo ( kmeshlc )

!     --- Compute all edges in kmeshcloc
!         Detect edges corresponding elements
!         Detect elements corresponding to edges
!         Detect edges corresponding to nodes

      call mshpavedges ( kmeshcloc, npointloc, nelemloc )
      if ( ierror.ne.0 ) go to 1000

!     --- Mark boundary edges

      allocate ( markedges(nedges), iworkedge(nedges), &
                 iworkelem(nelemloc), iworkquads(nelemloc) )

      call mshpavmarkbedg ( nedges, markedges, nmeshh )
      if ( ierror.ne.0 ) go to 1000

!     --- Mark all internal edges from boundary to boundary

      call mshpavintedges ( nedges, markedges, nmeshc, &
                            nbounpoints, found )
      if ( ierror.ne.0 ) go to 1000
      if ( debug ) &
         call prinin ( markedges, nedges, 'markedges' )

!     --- Find all quadrilaterals consisting of 2 boundary edges and 2 edges
!         connecting both boundary edges


      call mshpavquads ( nmeshl, nbounpoints, nedges, &
                         iworkedge, markedges, nmeshc, nmeshh, &
                         iworkelem, nquads, &
                         iworkquads, nmeshd, nelemloc, &
                         coorloc )
      if ( ierror.ne.0 ) go to 1000
      if ( debug ) &
         call prinin ( iworkedge, nedges, 'iworkedge/1' )

!     --- Check if these edges must be subdivided (mark them if so)

      call mshpavmarkedge ( nquads, iworkquads, &
                            nmeshd, markedges, &
                            nmeshc, thickness, coorloc, &
                            iworkedge, nedges, nelemloc, &
                            iworkelem )
      if ( debug ) &
         call prinin ( iworkedge, nedges, 'iworkedge/2' )

!     --- Create new nodes and coorlocdinates and subdivide triangles

      call mshpavtrianext ( npointloc, nedges, iworkedge, &
                            nmeshc, coorloc, nquads, &
                            iworkquads, nmeshd, &
                            nelemloc, iworkelem, maxnpoint, &
                            maxnelem, kmeshcloc )

      if ( checkarea ) then

!      --- Compute area of region

         call chksbm ( kmeshcloc, nelemloc, 3, 3, isurnr, &
                       coorloc, 2, 2, npointloc )

      end if

      deallocate ( markedges, iworkedge, iworkelem, iworkquads )

      call seprmmeshinfo ( kmeshlc )
      iseqkm = iseqkm1
      call seprelkmesh1

      deallocate ( pointkmsc )
      allocate ( pointkmsc(nelgrp+1) )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( coorloc, npointloc, 2, 'coorlocloc-end' )
         call prinin1 ( kmeshcloc, nelemloc, 3, 'kmeshclocloc-end' )
         call plotmeshinter ( nelemloc, kmeshcloc, coorloc, 6, 2, &
                              npointloc, 'triangle' )
         write(irefwr,*) 'End mshpavtriaadap'

      end if  ! ( debug )

      if ( checkarea ) then

!      --- Compute area of region

         call chksbm ( kmeshcloc, nelemloc, 3, 3, isurnr, &
                       coorloc, 2, 2, npointloc )

      end if
      call erclos ( 'mshpavtriaadap' )

      end
