      subroutine mshpavtrianext ( npoint, nedges, iworkedge, nmeshc, &
                                  coor, nquads, iworkquads, nmeshd, &
                                  nelem, iworkelem, maxnpoint, maxnelem, &
                                  kmeshc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-07-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide triangles created by msho2d into smaller triangles based
!     on the information stored in iworkedges
! **********************************************************************
!
!                       KEYWORDS
!
!     triangle
!     edge
!     boundary
!     quadrilateral
!     mesh_generation
!     paver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, nedges, iworkedge(*), nmeshc(*), nquads, &
              iworkquads(2,*), nmeshd(*), nelem, iworkelem(*), &
              maxnpoint, maxnelem, kmeshc(3,*)
      double precision coor(2,*)

!     coor          i/o   array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     iworkedge     i/o   work array of length nedges to be used to mark
!                         edges
!                         At input:
!                          0:  not marked yet
!                         -1:  boundary edge that has been used already
!                         -2:  diagonal of quadrilateral that may be removed
!                         At output:
!                          0:  No extra node on edge
!                         >0:  Node number of extra node on edge
!     iworkelem      i    Work array of length nelem to be used to mark
!                         elements
!                         0:  element is not special
!                         1:  element is triangle with all nodes on the boundary
!                         2:  elements is triangle, that is part of a
!                             quadrilateral with all 4 points on the boundary
!     iworkquads     i    work array of size nelem, containing sets of
!                         triangles that may be considered as quads
!                         (1,.) first triangle
!                         (2,.) second triangle
!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!     maxnelem       i    Maximum number of elements allowed
!     maxnpoint      i    Maximum number of nodes allowed
!     nedges         i    number of edges
!     nelem         i/o   Number of elements
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
!     nmeshd         i    nmesh part d, contains edge numbers of elements
!     npoint        i/o   Number of nodal points
!     nquads         i    number of quads stored in iworkquads
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer npointprev, iedge, ipstart, inode1, inode2, iquad, &
              itrian1, itrian2, itrian, nelemprev
      double precision area
      logical checkarea

!     area           Area of element
!     checkarea      If true the area must be checked
!     iedge          edge sequence number
!     inode1         First node in edge
!     inode2         Second node in edge
!     ipstart        Starting address in array
!     iquad          Sequence number of quadrilateral
!     itrian         Sequence number of triangle
!     itrian1        Sequence number of first triangle
!     itrian2        Sequence number of second triangle
!     nelemprev      Number of elements at input
!     npointprev     Number of nodes at input
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     MSHAREASUR     Compute the area of a region divided into linear triangles
!     MSHPAVSUB2TRIA Subdivide a set of 2 triangles that form a quadrilateral if
!                    necessary into more triangles
!     MSHPAVSUBTRIAN Check if triangle is candidate for subdivision into more
!                    triangles and if so subdivide this triangle
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     npointprev := npoint
!     For iedge := 1 (1) nedges do
!        if ( iworkedge(iedge) = 1 ) then
!           npoint := npoint+1; iworkedge(iedge) := npoint
!           coor(.,npoint) := coordinates of mid point of edge
!     if ( npoint > npointprev ) then
!        For all quadrilaterals stored in workquads do
!           Subdivide triangles in quadrilateral if necessary
!        For all triangles with iworkelem(.) = 1 do
!           Subdivide triangle if necessary
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavtrianext' )
      debug = .false.
      checkarea = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavtrianext'

      end if
      if ( ierror.ne.0 ) go to 1000

      if ( checkarea ) then

!     --- Compute area of triangles

         call mshareasur ( kmeshc, nelem, coor, area, 1 )

      end if  ! ( checkarea )

!     --- npointprev := npoint

      npointprev = npoint
      nelemprev = nelem

!     --- Check if there are any edges to be subdivided
!         For iedge := 1 (1) nedges do

      do iedge = 1, nedges

         if ( iworkedge(iedge).eq.1 ) then

!        --- iworkedge(iedge) = 1, hence edge must be subdivided into 2 parts
!            the new nodal point gets number npoint+1
!            npoint := npoint+1; iworkedge(iedge) := npoint
!            coor(.,npoint) := coordinates of mid point of edge

            npoint = npoint+1
            call inichkarlen ( npoint, maxnpoint, 'crd' )
            if ( ierror.ne.0 ) go to 1000
            iworkedge(iedge) = npoint
            ipstart = nmeshc(iedge)
            inode1 = nmeshc(ipstart+1)
            inode2 = nmeshc(ipstart+2)
            coor(1,npoint) = 0.5d0*(coor(1,inode1)+coor(1,inode2))
            coor(2,npoint) = 0.5d0*(coor(2,inode1)+coor(2,inode2))

         end if  ! ( iworkedge(iedge).eq.1 )

      end do  ! iedge = 1, nedges
      if ( debug ) then
         call prinin ( iworkedge, nedges, 'iworkedge' )
         call prinrl1 ( coor, npoint, 2, 'coor' )
      end if  ! ( debug )

      if ( npoint.gt.npointprev ) then

!     --- npoint > npointprev, so some edges have been subdivided
!         subdivide elements
!         For all quadrilaterals stored in workquads do
!            Subdivide triangles in quadrilateral if necessary

         do iquad = 1, nquads

            itrian1 = iworkquads(1,iquad)
            itrian2 = iworkquads(2,iquad)
            call mshpavsub2tria ( nmeshd, itrian1, itrian2, nelem, &
                                  kmeshc, nmeshc, coor, iworkedge )
            call inichkarlen ( nelem, maxnelem, 'kmeshc' )
            if ( ierror.ne.0 ) go to 1000

         end do  ! iquad = 1, nquads

!        --- For all triangles with iworkelem(.) <= 1 do
!              Subdivide triangle if necessary

         do itrian = 1, nelemprev

            if ( iworkelem(itrian).le.1 ) then

!           --- Triangle is a candidate for subdivision

               call mshpavsubtrian ( itrian, nmeshc, iworkedge, kmeshc, &
                                     nelem, coor, nmeshd )
               call inichkarlen ( nelem, maxnelem, 'kmeshc' )
               if ( ierror.ne.0 ) go to 1000

            end if  ! ( iworkelem(itrian).eq.1 )

         end do  ! itrian = 1, nelem

      end if  ! ( npoint.gt.npointprev )

1000  call erclos ( 'mshpavtrianext' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavtrianext'

      end if

      if ( checkarea ) then

!     --- Compute area of triangles

         call mshareasur ( kmeshc, nelem, coor, area, 1 )

      end if  ! ( checkarea )

      end
