      subroutine mshplaxis ( isurnr, coor  , kmeshc, inpelm, nbndpt, &
                             kbndpt, bcord , npoint, nelem , boundary, &
                             numcurvboun   , ntria , itria , cord  , &
                             nnodes, holeinfo, nholes, ncoar , coar  , &
                             userpoints, numextcurves, &
                             numnodextcurvs, curvenumbers, numextrnods, &
                             rinput, ndim, nuspnt )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.3  date 29-05-2014 Extension of timebefore
!        version  3.2  date 22-01-2014 New call to msho2d
!        version  3.1  date 02-11-2010 Dynamical allocation workarrays
!
!   copyright (c) 2007-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make a mesh of triangles in a 3D surface for an area
!     which is given by the line elements of BCORD. A description of the
!     surface is given by a rough triangulation as given by the points
!     and triangles in the arrays itria and cord.
!     Use a least squares method (kkm) to map the 3D surface to a 2D
!     surface and utilize the 2D meshgenerator msho2d.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isurnr, inpelm, nelem, kmeshc(*), nbndpt, &
              kbndpt(*), npoint, nholes, holeinfo(2,0:nholes+1), &
              ntria, itria(3,ntria), ndim, nuspnt, &
              numcurvboun, boundary(2,numcurvboun), &
              nnodes, ncoar, userpoints(*), numextcurves, &
              numnodextcurvs(*), curvenumbers(*), numextrnods
      double precision coor(3,npoint), bcord(3,*), cord(3,nnodes), &
                       coar(4,ncoar), rinput(*)

!     bcord          i    3D array containing the coordinates of the boundaries
!     boundary       i    Array of size 2 x numcurvboun containing
!                         information of the boundary of the surface
!                         (1,i) contains the i-th curve number (including sign)
!                         (2,i) contains the starting address in kbndpt
!     coar           i    array containing the coordinates as well as the
!                         local coarseness of internal points to be placed
!                         in the surface
!                         Length 3 x ncoar
!                         (1,i):  x-coordinate of point i
!                         (2,i):  y-coordinate of point i
!                         (3,i):  z-coordinate of point i
!                         (4,i):  coarseness of point i
!     coor           o    3D array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     cord           i    3D array contains the coordinates of the nodes in the
!                         triangle mesh (nnodes nodes generator framesurf)
!     curvenumbers   i    Contains the curve numbers of the extra internal
!                         curves
!     holeinfo       i    Integer array of size 2 x (nholes+2)
!                         Contains information about holes in surfaces and where
!                         a new part of the boundary starts
!                         Pos (1,i-1) contains the local sequence number of the
!                               first curve of part i provided with a sign
!                               If the sign is positive the boundary is
!                               created counter clockwise, otherwise clockwise
!                         Pos (2,i-1) contains the local sequence number of the
!                               first node of of part i
!                         Pos (1,nholes+1) contains the number of curves of the
!                         boundary
!     inpelm         i    number of points per element
!     isurnr         i    Surface sequence number
!     itria          i    Contains the topology of the triangle mesh (frame)
!     kbndpt         o    array containing the nodal point numbers of the
!                         boundary
!     kmeshc         o    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nbndpt         i    Number of nodal points in boundary, length of array
!                         kbndpt
!     ncoar          i    Number of internal points in surface
!     ndim           i    Dimension of original space
!     nelem         i/o   Number of elements
!                         (at input estimated, at output found)
!     nholes         i    Number of holes in surface and hence in boundary
!     nnodes         i    Number of nodes
!                         (at input estimated, at output found)
!     npoint        i/o   Number of nodal points
!     ntria          i    number of triangles in the triangle mesh (frame)
!     numcurvboun    i    Number of curves in the boundary
!     numextcurves   i    Number of extra (internal) curves
!     numextrnods    i    Sum of number of nodes on all extra internal curves
!     numnodextcurvs i    Contains the number of nodes for each internal
!                         curve in the sequence of these curves
!     nuspnt         i    Submitted number of user points
!     rinput         i    Real input array, see subroutine MESHBODY
!     userpoints     i    integer array of length ncoar containing the
!                         user point numbers of the extra points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2, i3, nbound, j, ibound(2,nbndpt), &
              nelemest, npunt, nipnt, nnodcrs, nnodds, id, idrie, ib, &
              iprodbest, inside, istep, nextra
      double precision dismin, dist1, dist2, dist3, &
                       dif, opp1, opp2, opp3, opp4, &
                       x1, xi, y1, yi, z1, zi, x2, y2, z2, &
                       ski, eta, zta, ak, &
                       xp, yp, zp, bnorm, dx, dy, dz, a, b, c, val, &
                       base1x, base1y, base1z, base2x, base2y, base2z, &
                       ski1, ski2, ski3, eta1, eta2, eta3, zta1, zta2, &
                       zta3, prpnt(2,nnodes+2), xn(3), xnsep(3), &
                       prodbest, prodh, prcord(3,nnodes), &
                       caor( 3, max(1, ncoar) ), &
                       cocurvs( 3, max(1,numextrnods) ), &
                       cacurvs( 2, max(1,numextrnods) ), &
                       xtran, ytran, ztran, bval
      integer, allocatable, dimension (:)             :: extquanodes
      double precision, allocatable, dimension(:,:)   :: hcoor
      double precision, allocatable, dimension(:,:)   :: hpnt
      double precision :: timebefore(2)

!     a              help variable to store aberration
!     ak             helpvariable for norm of normal vector
!     b              helpvariable to store aberration
!     base1x         first component first base vector in new plane
!     base1y         second component first base vector in new plane
!     base1z         third component first base vector in new plane
!     base2x         first component second base vector in new plane
!     base2y         second component second base vector in new plane
!     base2z         third component second base vector in new plane
!     bnorm          variable to be used during normcomputation
!     bval           helpvariable for norm computation
!     c              helpvariable to store aberration
!     cacurvs        projections of the fixed line coordinates
!     caor           projections array of the internal nodes
!     cocurvs        coordinates of internal curves
!     debug          if true debug statements are carried out otherwise
!     dif            helpvariable
!     dismin         smallest element in boundary
!     dist1          distance
!     dist2          distance
!     dist3          distance
!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     dz             length of vector in z-direction
!     error          Return value of allocate or deallocate
!     eta            second coefficient final kkm plane
!     eta1           second coefficient first experimental kkm plane
!     eta2           second coefficient second experimental kkm plane
!     eta3           second coefficient third experimental kkm plane
!     extquanodes    helparray for quadratic elements case
!     hcoor          3D helparray to store during the process the projections
!                    of all the nodes in the 3d kkm plane (length 3 * npunt)
!     hpnt           2D helparray to store projection of all nodes. At the end
!                    array contains npoint nodes in the local plane
!     i              counting variable
!     i1             node number
!     i2             node number
!     i3             node number
!     ib             helpvariable for boundary pieces
!     ibound         help array boundary segments triangles
!     id             indicator for finding location
!     idrie          loopvariable for do loop
!     inside         indicator whether there are internal surfaces in
!                    the surface submitted
!     iprodbest      indicator which inner product is the best
!     istep          indicator (1=linear, 2=quadratic)
!     j              counting variable
!     nbound         number of elements in ibound
!     nelemest       estimate of the number of elements
!     nextra         helpcounter
!     nipnt          number of boundary points
!     nnodcrs        number of nodes on internal curves
!     nnodds         helpvariable
!     npunt          estimate of the number of nodes
!     opp1           helpvariable to store surface of helptriangle 1
!     opp2           helpvariable to store surface of helptriangle 2
!     opp3           helpvariable to store surface of helptriangle 3
!     opp4           helpvariable to store surface triangle 1-2-3
!     prcord         projection array submitted triangle nodes array
!     prodbest       best choice for innnerproduct direction
!     prodh          helpvariable to store inner product value
!     prpnt          helparray to store projections of internal nodes and
!                    triangle nodes
!     ski            first coefficient final kkm plane
!     ski1           first coefficient first experimental kkm plane
!     ski2           first coefficient second experimental kkm plane
!     ski3           first coefficient third experimental kkm plane
!     val            helpvariable to store temporarily helpvalue
!     x1             x-coordinate 3D node 1
!     x2             x-coordinate 3D node 2
!     xi             x-coordinate node i
!     xn             avarage normal for all nodes
!     xnsep          avarage normal for special plane
!     xp             x-coordinate node p
!     xtran          x-coordinate translation
!     y1             y-coordinate 3D node 1
!     y2             y-coordinate 3D node 2
!     yi             y-coordinate node i
!     yp             y-coordinate node p
!     ytran          y-coordinate translation
!     z1             z-coordinate 3D node 1
!     z2             z-coordinate 3D node 2
!     zi             z-coordinate node i
!     zp             z-coordinate node p
!     zta            third coefficient final kkm plane
!     zta1           third coefficient first experimental kkm plane
!     zta2           third coefficient second experimental kkm plane
!     zta3           third coefficient third experimental kkm plane
!     ztran          z-coordinate translation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRORMSHSUR1   Compare estimated size of npoint and nelem with actual size
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     ERSETTIME      Get the CPU time at start of subroutine
!     INICHKARLEN    Check length of array and give error message 550 if
!                    array is too small
!     MSH603         Compute the distance between two points i and j
!     MSHD7          Place coordinates of centroids of 7-point triangles
!     MSHO19         Compute area of triangle
!     MSHO2D         driving subroutine to triangulate region
!     MSHO31         place extra nodes in case of quadratic elements
!     MSHPLAXIS30    Improve Delaunaymesh in projectionplane and next go back
!                    to the original space
!     MSHPLAXIS31    Determine the avarage value of the normals on a surface
!                    of triangles in a 3D-space
!     MSHTRIANCENTR  Create centre point in 6-point triangles
!     PRININ1        Print 2d integer array
!     PRINRL1        Print 2d real vector
!     PRINTTIME      Print the CPU time at end of subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
!    2916   Large difference in maximal and minimal element along boundary
!    2979   Internal error in mshplaxis
!    2980   Essential error in mshplaxis
!    2981   Internal error in mshplaxis
!    3018   npoint too large
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Start initializing the various arrays
!   Translate all nodes to the first octant.
!   Initialize number of nodes on curves
!     Compute number of nodes on extra curves
!   Check shape number asked for
!   First step determine the (SEPRAN) boundary nodes from the
!         given array bcord and fill array kbndpt. Store the 3D coordinates
!         of these nodes in array coor.
!         Fill array ibound with the boundary pieces.
!     Set number of boundary points
!     Check coarseness
!     Compute mean coarseness value
!     Second step, determine the kkm-plane:
!             ski * x + eta * y + zta * z = d
!     Determine average normal of all triangles in framesurf plane
!     and determine the best choice for the reference plane kkm
!     Compute for all boundary Sepran (coor) points projections in
!     the now chosen "best normal" kkm plane
!     Do the same for all submitted framesurf nodes, &
!     and place projections of the framesurf nodes in prcord
!       Map all the projected (boundary)nodes to the 2D plane
!         Determine base vectors
!       Find elements in the 2D plane:
!       Call 2d generator msho2d (i.e. triangle)
!       Next compute projections on the original framesurf triangles
!     Perform backtranslation for all translated nodes
!     and all new nodes
!   Return to main program with the resulting triangulation.
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshplaxis' )

      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshplaxis'
         write(irefwr,1) 'isurnr, nbndpt', isurnr, nbndpt
         call prinrl1 ( bcord, nbndpt, 3, 'Boundary nodes' )
  1      format ( a, 1x, (10i6) )
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Find estimates for number of nodes and number of elements given
!         the number of boundary points nbndpt
!         (npunt parameter is a tuning parameter!)

      npunt    = 105+(nbndpt*nbndpt)/4  ! estimate number of nodes in surface
      nelemest = 2 * npunt - nbndpt     ! estimate number of elmnts in surface

      if ( itime>0 ) call ersettime ( timebefore )

!     --- Allocate space for helparrays hcoor and hpnt

      allocate( hcoor ( 3, npunt ), stat = error )
      if ( error/=0 ) call eralloc ( error, npunt, 'hcoor' )
      allocate( hpnt  ( 2, npunt ), stat = error )
      if ( error/=0 ) call eralloc ( error, npunt, 'hpnt'  )

!     --- Initialize all points both in 3D and in 2D:

      do i = 1, npunt
         coor(1,i) = -99999d0
         coor(2,i) = -99999d0
         coor(3,i) = -99999d0

         hpnt(1,i) = -99999d0
         hpnt(2,i) = -99999d0
      end do

      do i = 1, nnodes
         prpnt(1,i) = -99999d0
         prpnt(2,i) = -99999d0
      end do

      debug = .false.
      if ( debug ) then
         write(irefwr,*) 'Surface ',isurnr
         write(irefwr,*) 'bcord, 3D array boundary, holes permitted'
         do i=1,nbndpt
            write(irefwr,*) i,(bcord(j,i),j=1,3)
         end do

         write(irefwr,*) 'Input coordinates Frame, nodes = ',nnodes
         do i=1,nnodes
            write(irefwr,*) i,(cord(j,i),j=1,3)
         end do
      end if

!     --- Replace all nodes temporarily to positive octant:
!         (Determine translation parameters x-,y-,z-tran)

      xtran = 1D10
      ytran = 1D10
      ztran = 1D10

      do i = 1, nbndpt
         if ( xtran>bcord(1,i) ) xtran = bcord(1,i)
         if ( ytran>bcord(2,i) ) ytran = bcord(2,i)
         if ( ztran>bcord(3,i) ) ztran = bcord(3,i)
      end do

      do i = 1, nbndpt
         bcord(1,i) = bcord(1,i) + 1d0 - xtran
         bcord(2,i) = bcord(2,i) + 1d0 - ytran
         bcord(3,i) = bcord(3,i) + 1d0 - ztran
      end do

      do i = 1, nnodes
         cord(1,i) = cord(1,i) + 1d0 - xtran
         cord(2,i) = cord(2,i) + 1d0 - ytran
         cord(3,i) = cord(3,i) + 1d0 - ztran
      end do

      if ( ncoar>0 ) then
         do i = 1, ncoar
            coar(1,i) = coar(1,i) + 1d0 - xtran
            coar(2,i) = coar(2,i) + 1d0 - ytran
            coar(3,i) = coar(3,i) + 1d0 - ztran
         end do
      end if

!     --- Initialize number of nodes on curves:

      nnodcrs = 0

      if ( numextcurves>0 ) then

!     --- Quadratic elements not yet implemented for quadratic curves:

         if ( inpelm==6 .or. inpelm==7 ) then

!        --- Give message and stop execution:

            write(irefwr,*) 'Not yet implemented:'
            write(irefwr,*) 'Quadratic internal curves for'
            write(irefwr,*) 'meshgenerator Framesurf'
            write(irefwr,*) 'Adjust your input!'
            write(irefwr,*) 'Execution stopped!'
            go to 1000
         end if

!        --- Compute number of nodes on extra curves:

         do i=1, numextcurves

!        --- Run through line i of the internal curves:

            do j= nnodcrs + 1, nnodcrs + numnodextcurvs(i)

!           --- Determine coordinates node and translate

               cocurvs(1,j) = bcord(1,nbndpt+j) + 1d0 - xtran
               cocurvs(2,j) = bcord(2,nbndpt+j) + 1d0 - ytran
               cocurvs(3,j) = bcord(3,nbndpt+j) + 1d0 - ztran
               if ( debug) write(irefwr,*) j,(cocurvs(i1,j),i1=1,3)
            end do
            nnodcrs = nnodcrs + numnodextcurvs(i)
         end do
      end if

!     --- Use Least Squares Method (KKM method) with generator Triangle

      if ( inpelm/=3 .and. inpelm/=6 .and. inpelm/=7 ) then

!     --- Shape number not yet implemented

         call errint ( inpelm, 1 )
         call errsub ( 800, 1, 0, 0 )
         goto 1000
      end if

!     --- First step:
!         determine the (SEPRAN) boundary nodes from the given array
!         bcord and fill array kbndpt. Store the 3D coordinates of these
!         nodes in array coor.
!         Fill array ibound with the boundary pieces.

      npoint = 0

      j  =  0
      ib = -1

      do i = 1, nbndpt
         if ( j==0 ) then

!        --- First node of this boundary block:

            ib = ib + 1

            x1 = bcord(1,i)
            y1 = bcord(2,i)
            z1 = bcord(3,i)

            if ( debug ) then
               write(irefwr,*) 'Boundary block starts at',x1,y1,z1
            end if

!           --- Store node:

            npoint = npoint + 1

            coor(1,npoint) = x1
            coor(2,npoint) = y1
            coor(3,npoint) = z1

            kbndpt(i) = npoint

            j = npoint

            ibound(1,i-ib) = j
            ibound(2,i-ib) = j + 1

         else

!        --- Check for next coordinates:

            x2 = bcord(1,i)
            y2 = bcord(2,i)
            z2 = bcord(3,i)

            dist1 = (x2-x1)*(x2-x1)+(y2-y1)*(y2-y1)+(z2-z1)*(z2-z1)

            dist1 = sqrt(dist1)

            if ( dist1<1d-8 ) then

!           --- Closed region, skip this node, but store in kbndpt:

               kbndpt(i)        = j
               ibound(2,i-ib-1) = j

               j = 0

            else

!           --- New permanent point

               npoint = npoint + 1

               coor(1,npoint) = x2
               coor(2,npoint) = y2
               coor(3,npoint) = z2

               if ( debug ) then
                  write(irefwr,*) 'Next node',x2,y2,z2
               end if

               kbndpt(i) = npoint

               ibound(1,i-ib) = ibound(2,i-ib-1)
               ibound(2,i-ib) = npoint + 1

            end if
         end if
      end do

!     --- Number of boundary points:

      nipnt  = npoint
      nbound = npoint

      call inichkarlen ( nbound, nbndpt, 'ibound' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,*) 'Number of boundary line pieces: = ',nbound
         do i=1,nbound
            i1 = ibound(1,i)
            i2 = ibound(2,i)
            write(irefwr,*) 'Bound',i,'from ',i1,' to ',i2
         end do

         do i=1,npoint

!        --- Print coordinates of boundary:

            write(irefwr,*) 'Node ',i,(coor(j,i),j=1,3)
         end do

!        --- Print contents array kbndpt

         write(irefwr,*) 'kbndpt array'
         write(irefwr,*) (kbndpt(i),i=1,nbndpt)
         write(irefwr,*) 'array ok?'
      end if

!     --- Determine coarseness

      dist1  = 0d0
      dismin = 1d20
      dist3  = 0d0

      do i = 1, nbound
         call msh603 ( ibound(1,i), ibound(2,i), coor, dist2 )
         if ( dist2<dismin ) dismin = dist2
         if ( dist2>dist3  ) dist3  = dist2
         dist1 = dist1 + dist2
      end do

      if ( dist3/dismin>1d1 ) then

!     --- Large difference in minimal and maximal elementlength
!         along boundary

         if ( inpelm==6 .or. inpelm==7 ) then

!        --- Adjust the values (quadratic elements: double)

            dist3  = 2 * dist3
            dismin = 2 * dismin
         end if

         call erreal( dist3 , 1 )
         call erreal( dismin, 2 )
         call errwar( 2916, 0, 2, 0 )
      end if

!     --- Compute mean coarseness value:
!         Step 2: Determine the best kkm-plane:
!                 ski * x + eta * y + zta * z = d
!         Start with determination of the average normal of all
!         triangles in framesurf plane

      call mshplaxis31( ntria, itria, cord, xn )

      if ( debug ) &
         write(irefwr,*) 'Avarage normal has values',(xn(j),j=1,3)

!     --- Determine the best choice for reference plane kkm:

      iprodbest = 0
      prodbest  = 0d0

!     --- 1) Compute values for z = ski1 + eta1 * x + zta1 * y:

      call mshplaxis30( 1, nnodes, cord, ski1, eta1, zta1, base1z )

!     --- Normal on this plane is: ( eta1, zta1, -1 )

      xnsep(1) = eta1
      xnsep(2) = zta1
      xnsep(3) = -1d0

!     --- Normalize:

      a = dot_product( xnsep, xnsep )

      if ( a>0 ) xnsep = xnsep / sqrt(a)

!     --- Find value of dot product xn and xnsep:

      prodh   = dot_product ( xn, xnsep )
      prodh   = abs( prodh )

      if ( prodh>prodbest ) then
           iprodbest = 1
           prodbest  = prodh
      end if

!     --- 2) Compute values for x = ski2 + eta2 * y + zta2 * z

      call mshplaxis30( 2, nnodes, cord, ski2, eta2, zta2, base1x )

!     --- Normal on this plane is: ( -1,  eta2, zta2 )

      xnsep(1) = -1d0
      xnsep(2) = eta2
      xnsep(3) = zta2

!     --- Normalize:

      a = dot_product( xnsep, xnsep )

      if ( a>0 ) xnsep = xnsep / sqrt(a)

!     --- Find largest value of dot product xn and xnsep:

      prodh    = dot_product ( xn, xnsep )
      prodh    = abs( prodh )

!     --- Compare to prodbest:

      if ( prodh>prodbest ) then
         iprodbest = 2
         prodbest  = prodh
      end if

!     --- 3) Compute values for y = ski3 + eta3 * z + zta3 * x

      call mshplaxis30( 3, nnodes, cord, ski3, eta3, zta3, base1y )

!     --- Normal on this plane is: ( zta3, -1, eta3 )

      xnsep(1) = zta3
      xnsep(2) = -1d0
      xnsep(3) = eta3

!     --- Normalize:

      a = dot_product( xnsep, xnsep )
      if ( a>0 ) xnsep = xnsep / sqrt(a)

!     --- Find largest value of dot product xn and xnsep:

      prodh    = dot_product ( xn, xnsep )
      prodh    = abs( prodh )

!     --- Compare to prodbest:

      if ( prodh>prodbest ) then
         iprodbest = 3
         prodbest  = prodh
      end if

!     --- Use plane that gives best dot product with avarage normal
!         of triangles:

      if ( iprodbest==1 ) then

!     --- First plane is best:

         zta =    1d0 / ski1
         ski =  -eta1 * zta
         eta =  -zta1 * zta
      else if ( iprodbest==2 ) then

!     --- Second plane is best:

         ski =    1d0 / ski2
         eta =  -eta2 * ski
         zta =  -zta2 * ski
      else if ( iprodbest==3 ) then

!     --- Third plane is best:

         eta =    1d0 / ski3
         ski =  -zta3 * eta
         zta =  -eta3 * eta
      else

!     --- Not expected situation, internal error give message:

         write(irefwr,*) 'Error in mshplaxis while choosing kkm plane'
         write(irefwr,*) 'abcd1:',eta1, zta1, -1d0, ski1, base1z
         write(irefwr,*) 'abcd2:',-1d0, eta2, zta2, ski2, base1x
         write(irefwr,*) 'abcd3:',zta3, -1d0, eta3, ski3, base1y
         write(irefwr,*) 'xn:',xn
         call errsub ( 2979, 0, 0, 0 )
         go to 1000
      end if
      if ( debug ) then
         write(irefwr,*) ' KKM plane found '
         write(irefwr,*) ' a = ',ski,' b = ',eta,' c = ',zta,' d = 1.0'
         if ( ski>0 ) then
            write(irefwr,*) 'Different formulation'
            write(irefwr,*) 'a=1 b=',eta/ski,'c=',zta/ski,'d=',1/ski
         end if
      end if

!     --- Compute for all boundary Sepran (coor) points projections in
!         the now chosen "best normal" kkm plane:

      ak = ski*ski + eta*eta + zta*zta

      do i=1,npoint
         xi = coor(1,i)
         yi = coor(2,i)
         zi = coor(3,i)

         dif = 1d0 - ski*xi - eta*yi - zta*zi

         xp = xi + ski * dif / ak
         yp = yi + eta * dif / ak
         zp = zi + zta * dif / ak

         if ( debug ) then
            write(irefwr,*) 'Sepran point',i,xi,yi,zi
            write(irefwr,*) 'has projection in 3D',xp,yp,zp
         end if

         hcoor(1,i) = xp
         hcoor(2,i) = yp
         hcoor(3,i) = zp
      end do

!     --- Do the same for all submitted framesurf nodes!
!         Place projections of the framesurf nodes in prcord:

      do i=1,nnodes
         xi = cord(1,i)
         yi = cord(2,i)
         zi = cord(3,i)

         dif = 1d0 - ski*xi - eta*yi - zta*zi

         xp = xi + ski * dif / ak
         yp = yi + eta * dif / ak
         zp = zi + zta * dif / ak

         if ( debug ) then
            write(irefwr,*) 'Framepunt',i,xi,yi,zi
            write(irefwr,*) 'Projection in 3D',xp,yp,zp
         end if

         prcord(1,i) = xp
         prcord(2,i) = yp
         prcord(3,i) = zp
      end do

!     --- Map all the projected (boundary)nodes to the 2D plane
!         Start with determination of the base vectors
!         First base vector:

      base1x = hcoor(1,2) - hcoor(1,1)
      base1y = hcoor(2,2) - hcoor(2,1)
      base1z = hcoor(3,2) - hcoor(3,1)

      bnorm = base1x*base1x + base1y*base1y + base1z*base1z
      bnorm = sqrt ( bnorm )

      base1x = base1x / bnorm
      base1y = base1y / bnorm
      base1z = base1z / bnorm

!     --- Second base vector:

      i = 1
      j = 1

      do while ( j/=0 )
         i = i+1

         if ( i>npoint ) then
            call errsub ( 2980, 0, 0, 0 )
            go to 1000
         end if

         dx = hcoor(1,i+1) - hcoor(1,1)
         dy = hcoor(2,i+1) - hcoor(2,1)
         dz = hcoor(3,i+1) - hcoor(3,1)

         a = dy*base1z-dz*base1y
         b = dz*base1x-dx*base1z
         c = dx*base1y-dy*base1x

         if ( a*a+b*b+c*c>1d-6 ) j=0
      end do

!     --- Compute basevector components:

      bnorm = sqrt(a*a + b*b + c*c)

      a = a / bnorm
      b = b / bnorm
      c = c / bnorm

      base2x = base1y*c - base1z*b
      base2y = base1z*a - base1x*c
      base2z = base1x*b - base1y*a

      bnorm = base2x*base2x + base2y*base2y + base2z*base2z
      bnorm = sqrt( bnorm )

      base2x = base2x / bnorm
      base2y = base2y / bnorm
      base2z = base2z / bnorm

!     --- Next determine 2D coordinates for all boundary nodes, &
!         Start with defition of origin:

      hpnt(1,1) = 0
      hpnt(2,1) = 0

      a = base1x * base2y - base1y * base2x
      b = base1y * base2z - base1z * base2y
      c = base1z * base2x - base1x * base2z

      val  = max( abs(a)     , abs(b)     , abs(c)      )
      bval = max( abs(base2x), abs(base2y), abs(base2z) )

      if ( abs(a)>val-1d-6 ) then
         do i=2, npoint
            xp = hcoor(1,i) - hcoor(1,1)
            yp = hcoor(2,i) - hcoor(2,1)
            zp = hcoor(3,i) - hcoor(3,1)

!           --- Computation:

            ski = (xp * base2y - yp * base2x)/val
            if ( abs(base2x)>bval-1d-6 ) then
               eta = ( xp - ski*base1x ) / base2x
            else if ( abs(base2y)>bval-1d-6 ) then
               eta = ( yp - ski*base1y ) / base2y
            else if ( abs(base2z)>bval-1d-6 ) then
               eta = ( zp - ski*base1z ) / base2z
            end if
            hpnt(1,i) = ski
            hpnt(2,i) = eta
         end do

!        --- Extra for ncoar

         do i=1, ncoar
            xp = coar(1,i) - hcoor(1,1)
            yp = coar(2,i) - hcoor(2,1)
            zp = coar(3,i) - hcoor(3,1)

!           --- Computation:

            ski = (xp * base2y - yp * base2x)/val
            if ( abs(base2x)>bval-1d-6 ) then
               eta = ( xp - ski*base1x ) / base2x
            else if ( abs(base2y)>bval-1d-6 ) then
               eta = ( yp - ski*base1y ) / base2y
            else if ( abs(base2z)>bval-1d-6 ) then
               eta = ( zp - ski*base1z ) / base2z
            end if
            caor(1,i) = ski
            caor(2,i) = eta

!           --- Coarseness:

            caor(3,i) = coar(4,i)
         end do

!        --- Extra for internal curves:

         do i=1, nnodcrs
            xp = cocurvs(1,i) - hcoor(1,1)
            yp = cocurvs(2,i) - hcoor(2,1)
            zp = cocurvs(3,i) - hcoor(3,1)

!           --- Computation:

            ski = (xp * base2y - yp * base2x)/val
            if ( abs(base2x)>bval-1d-6 ) then
               eta = ( xp - ski*base1x ) / base2x
            else if ( abs(base2y)>bval-1d-6 ) then
               eta = ( yp - ski*base1y ) / base2y
            else if ( abs(base2z)>bval-1d-6 ) then
               eta = ( zp - ski*base1z ) / base2z
            end if
            cacurvs(1,i) = ski
            cacurvs(2,i) = eta
         end do
      else if ( abs(b)>val-1d-6 ) then
         do i=2, npoint
            xp = hcoor(1,i) - hcoor(1,1)
            yp = hcoor(2,i) - hcoor(2,1)
            zp = hcoor(3,i) - hcoor(3,1)

!           --- Computation:

            ski = (yp * base2z - zp * base2y)/val
            if ( abs(base2x)>bval-1d-6 ) then
               eta = ( xp - ski*base1x ) / base2x
            else if ( abs(base2y)>bval-1d-6 ) then
               eta = ( yp - ski*base1y ) / base2y
            else if ( abs(base2z)>bval-1d-6 ) then
               eta = ( zp - ski*base1z ) / base2z
            end if
            hpnt(1,i) = ski
            hpnt(2,i) = eta
         end do
         do i=1, ncoar
            xp = coar(1,i) - hcoor(1,1)
            yp = coar(2,i) - hcoor(2,1)
            zp = coar(3,i) - hcoor(3,1)

!           --- Computation:

            ski = (yp * base2z - zp * base2y)/val
            if ( abs(base2x)>bval-1d-6 ) then
               eta = ( xp - ski*base1x ) / base2x
            else if ( abs(base2y)>bval-1d-6 ) then
               eta = ( yp - ski*base1y ) / base2y
            else if ( abs(base2z)>bval-1d-6 ) then
               eta = ( zp - ski*base1z ) / base2z
            end if

            caor(1,i) = ski
            caor(2,i) = eta

!           --- Coarseness:

            caor(3,i) = coar(4,i)
         end do

         do i=1, nnodcrs
            xp = cocurvs(1,i) - hcoor(1,1)
            yp = cocurvs(2,i) - hcoor(2,1)
            zp = cocurvs(3,i) - hcoor(3,1)

!           --- Computation:

            ski = (yp * base2z - zp * base2y)/val
            if ( abs(base2x)>bval-1d-6 ) then
               eta = ( xp - ski*base1x ) / base2x
            else if ( abs(base2y)>bval-1d-6 ) then
               eta = ( yp - ski*base1y ) / base2y
            else if ( abs(base2z)>bval-1d-6 ) then
               eta = ( zp - ski*base1z ) / base2z
            end if

            cacurvs(1,i) = ski
            cacurvs(2,i) = eta
         end do
      else if ( abs(c)>val-1d-6 ) then
         do i=2, npoint
            xp = hcoor(1,i) - hcoor(1,1)
            yp = hcoor(2,i) - hcoor(2,1)
            zp = hcoor(3,i) - hcoor(3,1)

!           --- Computation:

            ski = (zp * base2x - xp * base2z)/val
            if ( abs(base2x)>bval-1d-6 ) then
               eta = ( xp - ski*base1x ) / base2x
            else if ( abs(base2y)>bval-1d-6 ) then
               eta = ( yp - ski*base1y ) / base2y
            else if ( abs(base2z)>bval-1d-6 ) then
               eta = ( zp - ski*base1z ) / base2z
            end if

            hpnt(1,i) = ski
            hpnt(2,i) = eta
         end do

         do i=1, ncoar
            xp = coar(1,i) - hcoor(1,1)
            yp = coar(2,i) - hcoor(2,1)
            zp = coar(3,i) - hcoor(3,1)

!           --- Computation:

            ski = (zp * base2x - xp * base2z)/val
            if ( abs(base2x)>bval-1d-6 ) then
               eta = ( xp - ski*base1x ) / base2x
            else if ( abs(base2y)>bval-1d-6 ) then
               eta = ( yp - ski*base1y ) / base2y
            else if ( abs(base2z)>bval-1d-6 ) then
               eta = ( zp - ski*base1z ) / base2z
            end if
            caor(1,i) = ski
            caor(2,i) = eta

!           --- Coarseness:

            caor(3,i) = coar(4,i)
         end do

         do i=1, nnodcrs
            xp = cocurvs(1,i) - hcoor(1,1)
            yp = cocurvs(2,i) - hcoor(2,1)
            zp = cocurvs(3,i) - hcoor(3,1)

!           --- Computation:

            ski = (zp * base2x - xp * base2z)/val
            if ( abs(base2x)>bval-1d-6 ) then
               eta = ( xp - ski*base1x ) / base2x
            else if ( abs(base2y)>bval-1d-6 ) then
               eta = ( yp - ski*base1y ) / base2y
            else if ( abs(base2z)>bval-1d-6 ) then
               eta = ( zp - ski*base1z ) / base2z
            end if
            cacurvs(1,i) = ski
            cacurvs(2,i) = eta
         end do
      end if

      if ( debug ) then
         write(irefwr,*) 'Projections in mshplaxis Sepran boundarynodes'
         do i=1,npoint
            write(irefwr,*) 'Point ',i,' crd ',hpnt(1,i),hpnt(2,i)
         end do
      end if

!     --- Find elements in the 2D plane:

      inside = 0
      if (nnodcrs>0) inside = 1

!     --- Set default istep:

      istep = 1

!     --- Check for other situation:

      if ( inpelm==6 .or. inpelm==7 ) then

!     --- Adjust istep and boundary array in case of quadratic elements:

         istep  = 2
         nextra = nipnt

!        --- Check all pieces:

         nipnt = 1
         i = 1
         i1 = ibound(1,1)
         do while ( i<nextra + 1 )
            i = i + 2
            ibound( 2, nipnt ) = ibound( 2, i-1 )
            if ( ibound( 2, nipnt )==i1 ) then

!           --- Boundary part is closed, next part:

               if ( i<nextra + 1 ) then
                  nipnt = nipnt + 1
                  ibound( 1, nipnt ) = ibound( 1, i )
                  i1    = ibound(1,i)
               end if
            else

!           --- Not yet closed:

               nipnt = nipnt + 1
               ibound( 1, nipnt ) = ibound( 1, i )
            end if
         end do
      end if

      if ( debug ) then

!     --- Print pieces of boundary:

         write(irefwr,*) 'Number of boundary points:',nipnt
         do i = 1, nipnt
            write(irefwr,*) 'Piece',ibound(1,i),ibound(2,i)
         end do

!        --- Write coordinates of boundary points:

         write(irefwr,*) 'Number of projection points',npoint
         do i = 1, npoint
            write(irefwr,*) i,(hpnt(j,i),j=1,2)
         end do
         write(irefwr,*) 'Projections ok?'
      end if

!     --- Allocate space for helparray:
!         (Only in case of extra quadratic curves?

      nnodds = 1
      if ( istep==2 .and. numextcurves>0 ) then

!     --- Determine the length of extra array for curves:

         nnodds = 0
         do i=1, numextcurves

!        --- Run through line i of the internal curves:

            nnodds = nnodds + numnodextcurvs(i)
         end do

!        --- Extra array needed to place temporarily the quadratic
!            elements of the internal lines:
!            Length needed: 2 * nnodes

      end if

      allocate( extquanodes( 2 * nnodds ), stat = ierror)
      if ( ierror/= 0 ) then

!     --- error#0, so not enough space

         write(irefwr,*) 'Extquanodes:Not enough space available'
         write(irefwr,*) 'Local allocation in mshplaxis'
         write(irefwr,*) 'Execution stopped'
         go to 1000
      end if  ! ( error/= 0 )

      extquanodes(1) = 0

!     --- Call Triangulation routine for plane:

      call msho2d ( hpnt, npoint, ibound, nipnt, kmeshc, nelem, &
                    boundary, numcurvboun, npunt, inside, holeinfo, &
                    nholes, .true., kbndpt, nbndpt, &
                    caor, ncoar, userpoints, cacurvs, &
                    numextcurves,  numnodextcurvs, &
                    istep, extquanodes, isurnr, &
                    rinput, nuspnt, ndim, cacurvs, bcord )

      if ( debug ) then
         write(irefwr,*) 'Contents array kbndpt after calling msho2d'
         write(irefwr,*) (kbndpt(i),i=1,nbndpt)
         write(irefwr,*) 'Array coor (hpnt) after calling msho2d'
         write(irefwr,*) 'number of nodes is',npoint
         do i=1, npoint
            write(irefwr,*) i,hpnt(1,i),hpnt(2,i)
         end do
         write(irefwr,*) 'end coordinates array coor'
         write(irefwr,*) 'number of elements = ',nelem
         do i = 1, nelem
            write(irefwr,*) i,(kmeshc(j),j=3*i-2,3*i)
         end do
         write(irefwr,*) 'number of extra curves = ',numextcurves
         write(irefwr,*) 'number of extra nodes on curves'
         write(irefwr,*) (numnodextcurvs(i),i=1,numextcurves)
         write(irefwr,*) 'it concerns curves with numbers:'
         write(irefwr,*) (curvenumbers(i),i=1,numextcurves)
      end if

      if ( ierror/=0 ) go to 1000

!     --- Special treatment if elements are quadratic

      if ( inpelm==6 .or. inpelm==7 ) then

!     --- inpelm = 6 or 7, extra nodes in elements

         call msho31 ( hpnt, npoint, kmeshc, nelem, ibound, nipnt, &
                       extquanodes )
         if ( inpelm==7 ) then

!        --- inpelm = 7, also fill centroid

            call mshtriancentr ( kmeshc, npoint, nelem, inpelm )
            if ( ierror/=0 ) go to 1000
            call mshd7 ( kmeshc, coor, nelem, 2 )
         end if

         if ( debug ) then
            write(irefwr,*) 'After adding extra nodes, npoint =',npoint
            do i=1, npoint
               write(irefwr,*) i,hpnt(1,i),hpnt(2,i)
            end do
            write(irefwr,*) 'Coordinates Ok?'
         end if
      end if

!     --- Check whether number of nodes found is not too large:

      if ( npoint>npunt ) then

!     --- npoint too large

         call errsub ( 3018, 0, 0, 0 )
         go to 1000
      end if

!     --- Now compute projections on the original framesurf triangles
!         First the two-dimensional projection coordinates for the
!         Framesurf nodes (all with respect to the origin):

      if ( abs(a)>val-1d-6 ) then
         do i=1, nnodes
            xp = prcord(1,i) - hcoor(1,1)
            yp = prcord(2,i) - hcoor(2,1)
            zp = prcord(3,i) - hcoor(3,1)

!           --- Computation:

            ski = (xp * base2y - yp * base2x)/val
            if ( abs(base2x)>bval-1d-6 ) then
               eta = ( xp - ski*base1x ) / base2x
            else if ( abs(base2y)>bval-1d-6 ) then
               eta = ( yp - ski*base1y ) / base2y
            else if ( abs(base2z)>bval-1d-6 ) then
               eta = ( zp - ski*base1z ) / base2z
            end if
            prpnt(1,i) = ski
            prpnt(2,i) = eta
         end do
      else if ( abs(b)>val-1d-6 ) then
         do i=1, nnodes
            xp = prcord(1,i) - hcoor(1,1)
            yp = prcord(2,i) - hcoor(2,1)
            zp = prcord(3,i) - hcoor(3,1)

!           --- Computation:

            ski = (yp * base2z - zp * base2y)/val
            if ( abs(base2x)>bval-1d-6 ) then
               eta = ( xp - ski*base1x ) / base2x
            else if ( abs(base2y)>bval-1d-6 ) then
               eta = ( yp - ski*base1y ) / base2y
            else if ( abs(base2z)>bval-1d-6 ) then
               eta = ( zp - ski*base1z ) / base2z
            end if
            prpnt(1,i) = ski
            prpnt(2,i) = eta
         end do
      else if ( abs(c)>val-1d-6 ) then
         do i=1, nnodes
            xp = prcord(1,i) - hcoor(1,1)
            yp = prcord(2,i) - hcoor(2,1)
            zp = prcord(3,i) - hcoor(3,1)

!           --- Computation:

            ski = (zp * base2x - xp * base2z)/val
            if ( abs(base2x)>bval-1d-6 ) then
               eta = ( xp - ski*base1x ) / base2x
            else if ( abs(base2y)>bval-1d-6 ) then
               eta = ( yp - ski*base1y ) / base2y
            else if ( abs(base2z)>bval-1d-6 ) then
               eta = ( zp - ski*base1z ) / base2z
            end if
            prpnt(1,i) = ski
            prpnt(2,i) = eta
         end do
      end if

      if ( debug ) then
         write(irefwr,*) 'Projections in mshplaxis all framesurfnodes'
         do i=1,nnodes
            write(irefwr,*) 'Punt ',i,' crd ',prpnt(1,i),prpnt(2,i)
         end do
      end if

!     --- In quadratic case: boundary nodes are already ok:

      if ( inpelm==6 .or. inpelm==7 ) nipnt = nextra

!     --- Next find framesurf triangle for the new created nodes:

      do i = nipnt + 1, npoint

!     --- Run through all original framesurf triangles and
!         place point to be considered in array prpnt:

         prpnt(1,nnodes+1) = hpnt(1,i)
         prpnt(2,nnodes+1) = hpnt(2,i)

         id = 0

         do idrie=1, ntria
            if ( id==0 ) then
               i1 = itria(1,idrie)
               i2 = itria(2,idrie)
               i3 = itria(3,idrie)

!              --- Check whether node i belongs to this triangle:
!                  Compute ski, eta, zta

               call msho19( prpnt, i1, i2,       i3, opp4 )
               call msho19( prpnt, i1, i2, nnodes+1, opp3 )
               call msho19( prpnt, i2, i3, nnodes+1, opp1 )
               call msho19( prpnt, i3, i1, nnodes+1, opp2 )

               ski = opp1 / opp4
               eta = opp2 / opp4
               zta = opp3 / opp4

               if ( ski>=0D0.and.ski<=1d0 .and. &
                    eta>=0D0.and.eta<=1d0 .and. &
                    zta>=0D0.and.zta<=1d0 ) id = idrie
            end if
         end do

         if ( debug .and. id>0 ) then
            write(irefwr,*) 'Node ',i,'in triangle',i1,i2,i3
            write(irefwr,*) '2d node x = ',hpnt(1,i),' y = ',hpnt(2,i)
            write(irefwr,*) ' x1 = ',hpnt(1,i1),' y1 = ',hpnt(2,i1)
            write(irefwr,*) ' x2 = ',hpnt(1,i2),' y2 = ',hpnt(2,i2)
            write(irefwr,*) ' x3 = ',hpnt(1,i3),' y3 = ',hpnt(2,i3)
            write(irefwr,*) 'ski, eta, zta',ski,eta, zta
         end if
         if ( id==0 ) then
            call errint ( i, 1 )
            call erreal ( hpnt(1,i), 1 )
            call erreal ( hpnt(2,i), 2 )
            call errsub ( 2981, 1, 2, 0 )
            go to 1000
         end if

!        --- Compute "original" coordinates for the new created nodes:

         do j=1,3
            coor(j,i) = ski*cord(j,i1)+eta*cord(j,i2)+zta*cord(j,i3)
         end do

         if ( debug ) write(irefwr,*) 'In 3D',(coor(j,i),j=1,3)
      end do

!     --- Backtranslation:

      do i = 1, nbndpt - ncoar
         bcord(1,i) = bcord(1,i) - 1d0 + xtran
         bcord(2,i) = bcord(2,i) - 1d0 + ytran
         bcord(3,i) = bcord(3,i) - 1d0 + ztran
      end do

      do i = 1, nnodes
         cord(1,i) = cord(1,i) - 1d0 + xtran
         cord(2,i) = cord(2,i) - 1d0 + ytran
         cord(3,i) = cord(3,i) - 1d0 + ztran
      end do

      do i = 1, ncoar
         coar(1,i) = coar(1,i) - 1d0 + xtran
         coar(2,i) = coar(2,i) - 1d0 + ytran
         coar(3,i) = coar(3,i) - 1d0 + ztran
      end do

      do i = 1, npoint
         coor(1,i) = coor(1,i) - 1d0 + xtran
         coor(2,i) = coor(2,i) - 1d0 + ytran
         coor(3,i) = coor(3,i) - 1d0 + ztran
      end do

      call errormshsur1 ( nelemest, npunt, nelem, npoint, &
                          isurnr )

!     --- Deallocate

      deallocate( extquanodes, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'extquanodes'  )
      deallocate( hcoor, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'hcoor' )
      deallocate( hpnt , stat = error )
      if ( error/=0 ) call erdealloc ( error, 'hpnt'  )

      if ( itime==1 ) call printtime('mshplaxis')

1000  call erclos ( 'mshplaxis' )

      if ( debug ) then

!     --- Debug information:

         write(irefwr,*) 'End mshplaxis debug information'
         write(irefwr,*) 'Surface',isurnr,' has ',nelem,' elmt'
         call prinin1 ( kmeshc, nelem, inpelm, 'triangles' )
         call prinrl1 ( coor, npoint, 3, 'coordinates' )
         write(irefwr,*) 'End mshplaxis'
      end if  ! ( debug )

      end
