      subroutine mshrefpar6 ( kmeshmold, icurnr, istartnode, &
                              kmeshmcopy, kmeshmglobnew, kmeshmglobold, &
                              npointold, lencurvenew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-05-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make a new curve from the old global curve and all changes
!     that have been read from the various nodes
! **********************************************************************
!
!                       KEYWORDS
!
!     refine
!     parallel
!     curve
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshmold(*), icurnr, istartnode(*), &
              kmeshmcopy(iacnodes,*), kmeshmglobnew(*), &
              kmeshmglobold(*), npointold, lencurvenew

!     icurnr         i    Curve sequence number
!     istartnode    i/o   Array of size iacnodes containing the next position
!                         in kmeshmcopy to be used for each parallel block
!     kmeshmcopy     i    Work array of length (iacnodes,lenkmeshm) to store
!                         all curves that have new nodal point numbers per
!                         parallel node
!                         (.,1) contains the used length for node .
!                         The next positions contain for each curve stored:
!                         Pos. i: curve number
!                              i+1: number of nodal points in curve
!                              followed by nodal point numbers
!     kmeshmglobnew i/o   Array kmeshmglob corresponding to new mesh
!     kmeshmglobold  i    Array kmeshmglob corresponding to old mesh
!     kmeshmold      i    Old array kmeshm
!     lencurvenew    i    Number of nodes in new curve
!     npointold      i    Number of nodes in old mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lencurveold, numnodes, iblock, ipstart, jcurnr, &
              storenodes(iacnodes)

!     iblock         Block sequence number
!     ipstart        Starting address in array
!     jcurnr         curve sequence number
!     lencurveold    Number of nodes in old curve
!     numnodes       Number of parallel blocks stored in storenodes
!     storenodes     Contains the parallel blocks that contain a new
!                    description of curve icurnr
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHREFPAR9     Make a new curve from the old global curve and all changes
!                   that have been read from the various nodes
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefpar6' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefpar6'
         write(irefwr,1) 'icurnr', icurnr
         call prinin ( istartnode, iacnodes, 'istartnode' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get length of old curve

      lencurveold = kmeshmold(5+icurnr)-kmeshmold(4+icurnr)

      if ( debug ) write(irefwr,1) 'lencurveold', lencurveold

!     --- Check which nodes contain an extension of icurnr

      numnodes = 0
      do iblock = 1, iacnodes

         ipstart = istartnode(iblock)  ! next position in kmeshmcopy

         if ( ipstart<kmeshmcopy(iblock,1) ) then

!        --- The starting address in kmeshmcopy is less than the length
!            hence there is information to be read

            jcurnr = kmeshmcopy(iblock,ipstart)
            if ( debug ) &
               write(irefwr,1) 'iblock, jcurnr', iblock, jcurnr

            if ( jcurnr==icurnr ) then

!           --- curve icurnr is stored in kmeshmcopy

               ipstart = ipstart+1
               numnodes = numnodes+1
               storenodes(numnodes) = iblock
               istartnode(iblock) = ipstart

            end if  ! ( jcurnr==icurnr )

         end if  ! ( ipstart<kmeshmcopy(iblock,1) then

      end do  ! iblock = 1, iacnodes

!     --- Copy information

      if ( numnodes==0 ) then

!     --- numnodes = 0, hence the curve has not been changed
!         Just make a copy of kmeshmglob

         kmeshmglobnew(1:lencurveold) = kmeshmglobold(1:lencurveold)
         lencurvenew = lencurveold

      else

!     --- numnodes>0, hence the curve has been changed
!         We have to merge the curves

         call mshrefpar9 ( storenodes, istartnode, kmeshmcopy, &
                           kmeshmglobnew, kmeshmglobold, numnodes, &
                           npointold, icurnr, lencurvenew, &
                           lencurveold )

      end if  ! ( numnodes==0 )

1000  call erclos ( 'mshrefpar6' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefpar6'

      end if  ! ( debug )

      end
