      subroutine mshshift2d4b ( nodeend, nodebegin, nmeshc, nmeshl, &
                                coor, fact, markedge, icrossobst, &
                                rcrossobst, jump, iedge, ndim, eps, &
                                skipnod )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.1  date 21-11-2012 Correction nodes near curve
!        version  4.0  date 19-09-2012 Extra parameter skipnod
!        version  3.0  date 03-07-2012 Extra parameter eps
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Find edges that are intersected by obstacle and contain nodeend
!      Compute edge with minimum distance between obstacle and nodeend
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     triangle
!     subdivision
!     adaptation
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nodeend, nmeshc(*), nmeshl(*), markedge(*), jump, &
              icrossobst(*), nodebegin, iedge, ndim, skipnod(*)
      double precision coor(ndim,*), fact, rcrossobst(*), eps

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     eps            i    Defines when the intersection is close to a node
!     fact          i/o   factor defining the distance between obstacle and node
!     icrossobst     i    array containing information about the cross section
!                         of obstacles with edges
!                         See subroutine mshobstms2d
!     iedge          i    edge sequence number
!     jump           i    2 in case linear elements are used, 3 if quadratic
!     markedge       i    Array of size nedges
!                         See subroutine mshshift2d
!     ndim           i    Dimension of the space.
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
!     nmeshl         i    nmesh part l, contains edge numbers corresponding
!                         to nodes
!     nodebegin     i/o   Node at the begin of a crossed edge
!     nodeend       i/o   Node at the end of a crossed edge
!     rcrossobst     i    Real array corresponding to icrossobst
!                         See subroutine mshobstms2d
!     skipnod        i    Array of npoint long indicating if a node has been
!                         moved or not
!                         See subroutine mshshift2d
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, edgenr, ipstart, nnode_edge, icross, jpstart, &
              inode1, inode2
      double precision diff(ndim), h, distmin, fact1, dist

!     diff           Difference vector
!     dist           Distance between two points
!     distmin        Minimal distance
!     edgenr         Edge sequence number
!     fact1          See fact
!     h              length of edge
!     i              Counting variable
!     icross         Position of iedge in array icrossobst
!     inode1         First node in edge
!     inode2         Second node in edge
!     ipstart        Starting address in array
!     jpstart        Starting address in array
!     nnode_edge     number of nodes corresponding to edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshshift2d4b' )
      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshshift2d4b'
         write(irefwr,1) 'nodeend, nodebegin, iedge, jump', &
                          nodeend, nodebegin, iedge, jump
         write(irefwr,2) 'fact', fact
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute distance between obstacle and nodeend

      diff = coor(:,nodeend) - coor(:,nodebegin)  ! difference vector
      h = dot_product(diff,diff)                  ! length of edge ^2
      distmin = fact*h                            ! distance
      if ( debug ) write(irefwr,2) 'distmin', distmin

!     --- Find all edges corresponding to nodeend
!         Choose the one with shortest distance to obstacle

      ipstart = nmeshl(nodeend)
      nnode_edge = nmeshl(ipstart)   ! number of edges connected to nodeend

      if ( debug ) &
         write(irefwr,1) 'ipstart, nnode_edge', ipstart, nnode_edge

      do i = 1, nnode_edge

         edgenr = nmeshl(ipstart+i)  ! edge number
         icross = markedge(edgenr)   ! position in icrossobst

         if ( debug ) &
            write(irefwr,1) 'i, edgenr, icross', i, edgenr, icross

         if ( edgenr/=iedge .and. icross>0 ) then

!        --- Other edge found, check new distance

            jpstart = nmeshc(edgenr)      ! starting address
            inode1 = nmeshc(jpstart+1)     ! first node on edge
            inode2 = nmeshc(jpstart+jump)  ! last node on edge

            if ( skipnod(inode1)==0 .and. skipnod(inode2)==0 .or. &
                 skipnod(inode1)==0 .and. skipnod(inode2)==1.or. &
                 skipnod(inode1)==1 .and. skipnod(inode2)==0 ) then

!           --- None of the nodes has been moved
!               If the new cross section is not close to the end nodes of
!               the edge, the edge is not considered as applicable
!               Therefore we jump in that case

              fact1 = rcrossobst(icrossobst(icross+1)) ! factor
              if ( fact1>eps .and. fact1<1d0-eps ) go to 100

              if ( debug ) then
                 write(irefwr,1) 'inode1, inode2, icross', &
                                  inode1, inode2, icross
                 write(irefwr,2) 'fact1', fact1
              end if  ! ( debug )

              if ( inode1/=nodeend ) then

!             --- inode2 = nodeend; interchange inode1 and inode2
!                 change fact1

                 fact1 = 1d0-fact1
                 inode2 = inode1

              end if  ! ( inode1/=nodeend )

              if ( fact1>eps ) go to 100  ! wrong node

              diff = coor(:,inode2) - coor(:,nodeend) ! difference vector
              h = dot_product(diff,diff)              ! length of edge ^ 2
              dist = h*fact1   ! distance

              if ( debug ) then
                 write(irefwr,1) 'inode1, inode2/2', inode1, inode2
                 write(irefwr,2) 'fact1, h, dist', fact1, h, dist
              end if  ! ( debug )

              if ( dist<distmin ) then

!             --- New distance is shorter than old one

                 distmin = dist
                 fact = fact1
                 nodebegin = inode2

              end if  ! ( dist<distmin )

            end if  ! ( skipnod(inode1)==0 .and. skipnod(inode2)==0 )

         end if  ! ( edgenr/=iedge .and. markedge(edgenr)>0 )

100   end do  ! i = 1, nnode_edge

1000  call erclos ( 'mshshift2d4b' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nodeend, nodebegin, iedge', &
                          nodeend, nodebegin, iedge
         write(irefwr,2) 'fact', fact
         write(irefwr,*) 'End mshshift2d4b'

      end if  ! ( debug )

      end
