      subroutine mshshift2dcr1 ( numnodesb, nodesb, numelmsT, &
                                 setT, imarkelem, imarknodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all triangles with all points inside the obstacle
!     or in set B
!     Triangles with all nodes in B are not skipped
!     Store in setT
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     triangle
!     adaptation
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnodesb, nodesb(*), numelmsT, setT(*), &
              imarkelem(*), imarknodes(*)

!     imarkelem     i/o   See subroutine mshshift2dcor
!     imarknodes    i/o   See subroutine mshshift2dcor
!     nodesb         i    Set of nodes B of nodes on edges that are intersected
!                         where the node closest to the boundary is used
!     numelmst       o    number of elements in set T
!     numnodesb      i    Number of nodal points in array nodesb
!     sett           o    triangles with all points outside the obstacle
!                         or in set B
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, node, istart, numels, j, ielem, ikelmc, k, inpelm, &
              node1
      integer, allocatable :: iwork(:)
      logical extradebug, markelement

!     extradebug     If true extra debugging statements are carried out
!     i              Counting variable
!     ielem          Element number
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     istart         First position used at a part
!     iwork          integer work array of length nelem to mark elements that
!                    are treated (1). Not treated is 0
!     j              Counting variable
!     k              Counting variable
!     markelement    if true the element is marked
!     node           Node number
!     node1          Node of face
!     numels         number of elements connected to node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPFILLKMESHD  Fill kmesh part d
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshshift2dcr1' )
      debug = .false. .and. ioutp>=0
      extradebug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshshift2dcr1'
         call prinin ( nodesb, numnodesb, 'nodesb' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate or create arrays

      if ( kelmd==0 ) call sepfillkmeshd

!     --- allocate work space

      allocate ( iwork(nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, nelem, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- clear iwork

      iwork = 0

      numelmst = 0

!     --- Loop pver nodes in set B

      do i = 1, numnodesb

         node = nodesb(i)  ! node in set B

!        --- Find all elements containing node

         istart = kmshd1(node)
         numels = kmshd1(node+1)-istart  ! number of elements connected

         if ( extradebug ) then
            write(irefwr,1) 'i, node', i, node
            call prinin ( kmshd2(istart+1), numels, 'elements' )
            write(irefwr,1) 'iwork', &
                             iwork(kmshd2(istart+1:istart+numels))
         end if  ! ( extradebug )

         do j = 1, numels

            ielem = kmshd2(istart+j)
            if ( iwork(ielem)==0 ) then

!           --- element not treated yet

               iwork(ielem) = 1  ! mark element as treated

!              --- Find starting address in kmeshc

               ikelmc = sepstkmeshc ( ielem, inpelm )
               markelement = imarkelem(ielem)<2  ! element not yet in setT

               if ( extradebug ) &
                  write(irefwr,1) 'j, ielem, mark', &
                                   j, ielem, imarkelem(ielem)
               if ( imarkelem(ielem)==0 ) then

!              --- element has not been marked yet

                  do k = 0, inpelm-1
                     node1 = kmeshc(ikelmc+k)
                     if ( imarknodes(node1)<0 ) then

!                    --- Node inside obstacle and not in B

                        markelement = .false. ! element does not belong to setT
                        exit

                     end if  ! ( imarknodes(nodes1)<0 )

                  end do   ! k = 1, inpelm

               end if  ! ( imarkelem(ielem)==0 )

               if ( extradebug ) &
                  write(irefwr,*) 'markelement/1 ', markelement

               if ( markelement ) then

!              --- all nodes of element are either outside or belong to B

                  imarkelem(ielem) = 2

!                 --- element belongs to setT

                  numelmst = numelmst+1
                  setT(numelmst) = ielem

               end if  ! ( markelement )

            end if   ! ( iwork(ielem)==0 )

         end do  ! j = 1, numels

      end do  ! i = 1, numnodesb

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )


1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshshift2dcr1'

      end if  ! ( debug )
      call erclos ( 'mshshift2dcr1' )

      end
