      subroutine mshshift2dcr2 ( numelmsT, setT, imarknodes, &
                                 setE, numedges, lensetE, &
                                 removenodes, nodesb, numnodesb )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find the set of edges that have all nodes in B and are at the
!     boundary of setT
!     Only those edges that must be removed of the set B are stored
!     It concerns the edges with both corresponding triangles
!     with the third node outside the obstacle
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     triangle
!     adaptation
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numelmsT, setT(*), lensetE, &
              imarknodes(*), setE(lensetE), numedges, &
              numnodesb, nodesb(numnodesb)
      logical removenodes

!     imarknodes     i    See subroutine mshshift2dcor
!     lensete        i    Declared length of array setE
!     nodesb        i/o   Set of nodes B of nodes on edges that are intersected
!                         where the node closest to the boundary is used
!                         At output some nodes may be removed
!     numedges       i    number of edges
!     numelmst       i    number of elements in set T
!     numnodesb      i    Number of nodal points in array nodesb
!     removenodes   i/o   if true nodes  must be removed from nodesb
!     sete           o    Set of edges that have all nodes in B and are at the
!                         boundary of setT
!                         Only edges inside triangles with nodes outside
!                         the obstacle and in setB are stored
!     sett           i    triangles with all points outside the obstacle
!                         or in set B
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ipstart, nedge_elem, nodp, &
              ielem, j, k, iedge, jpstart, nnode_edge, icount, numnodes
      integer, allocatable :: markedges(:)
      logical store, nodeinsetB, extradebug

!     extradebug     If true extra debugging statements are carried out
!     i              Counting variable
!     icount         Counter
!     iedge          edge sequence number
!     ielem          Element number
!     ipstart        Starting address in array
!     j              Counting variable
!     jpstart        Starting address in array
!     k              Counting variable
!     markedges      array of length nedges indicating whether a
!                    edge is internal or on the boundary
!     nedge_elem     number of edges corresponding to element
!     nnode_edge     number of nodes corresponding to edge
!     nodeinsetb     if true both nodes of edge are in setB
!     nodp           Nodal point sequence number
!     numnodes       Number of nodes in an edge
!     store          If true edges must be stored
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLNMESHC  Fill array nmeshc
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     markedges(1:nedges) := 0
!     numedges := 0
!     For all elements ielem in setT do
!        for all edges iedge in ielem do
!           if ( all nodes of iedge are in setB ) then
!              if ( markedges(iedge)=0 ) then
!                 markedges(iedge) := 1
!                 numedges := numedges+1
!                 setE(numedges) := iedge
!              else ( markedges(iedge)#0 )
!                 markedges(iedge) := 2
!     Remove all edges with markedges(iedge)>1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshshift2dcr2' )
      debug = .false. .and. ioutp>=0
      extradebug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshshift2dcr2'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate or create arrays

      if ( kelmaa==0 ) call sepfillkmeshaa
      if ( nmesh(33)==0 ) call sepfillnmeshc

!     --- Actual call
!         allocate work space

      allocate ( markedges(nedges), stat = error )
      if ( error/=0 ) call eralloc ( error, nedges, 'markedges' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array markedges

      markedges = 0

      numedges = 0  ! number of edges in setE
      store = .false.  ! no edges to be stored

!     --- For all elements ielem in setT do

      do i = 1, numelmsT

         ielem = setT(i)
         ipstart = nmeshd(ielem)
         nedge_elem = nmeshd(ipstart)  ! number of edges in element

         if ( extradebug ) &
            write(irefwr,1) 'i, ielem, nedge_elem', &
                             i, ielem, nedge_elem

!        --- for all edges iedge in ielem do

         do j = 1, nedge_elem

            iedge = abs(nmeshd(ipstart+j))  ! edge number
            jpstart = nmeshc(iedge)
            nnode_edge = nmeshc(jpstart)  ! number of nodes in edge

            if ( extradebug ) then
               write(irefwr,1) 'j, iedge, nnode_edge', &
                                j, iedge, nnode_edge
               write(irefwr,1) 'nodes', &
                  nmeshc(jpstart+1:jpstart+nnode_edge)
               write(irefwr,1) 'marknodes', &
                  imarknodes(nmeshc(jpstart+1:jpstart+nnode_edge))
            end if  ! ( extradebug )

            nodeinsetB = .true.  ! all nodes in set B
            do k = 1, nnode_edge

               nodp = nmeshc(jpstart+k)
               if ( imarknodes(nodp)/=1 ) then

!              --- Node not in setB

                  nodeinsetB = .false.  ! not all nodes in set B
                  exit

               end if  ! ( imarknodes(nodp)/=1 )

            end do  ! k = 1, nnode_edge

!           --- if ( all nodes of iedge are in setB ) then

            if ( nodeinsetB ) then

!           --- All nodes in set B
!               if ( markedges(iedge)=0 ) then
!                  markedges(iedge) := 1
!                  numedges := numedges+1
!                  setE(numedges) := iedge
!               else ( markedges(iedge)#0 )
!                  markedges(iedge) := 2

               if ( markedges(iedge)==0 ) then

!              --- edge has not been found yet, store in setE

                  markedges(iedge) = 1      ! mark edge as used
                  numedges = numedges+1     ! raise number of edges
                  setE(numedges) = iedge    ! add edge to setE

               else

!              --- double edge, must be stored

                  markedges(iedge) = 2     ! mark edge as double
                  store = .true.           ! at least one edge must be stored

               end if  ! ( markedges(iedge)==0 )

            end if  ! ( nodeinsetB )

         end do  ! j = 1, nedge_elem

      end do  ! i = 1, numelmsT

      call inichkarlen ( numedges, lensetE, 'setE' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,*) 'store ', store
         call prinin ( setE, numedges, 'setE/1' )
         call prinin ( nodesb, numnodesb, 'nodesb' )
      end if  ! ( debug )

!     --- If a node in nodesb is not part of the set of edges setE then
!         the node is surrounded by triangles of which all nodes except the
!         specific node are outside the obstacle
!         This means that the node may not be moved and hence must be
!         removed from the set nodesB
!         This is marked by setting removenodes true and setting
!         imarknodes(node) to 2

      removenodes = .false.   ! no nodes to be removed

!     --- First check if all nodes in setB belong to edges in setE
!         Those not in setE are skipped

      imarknodes(nodesb(1:numnodesb)) = 2  ! mark all nodes as to be removed

!     --- Loop over edges

      icount = 0  ! number of nodes to be used

      do i = 1, numedges

         iedge =  setE(i)            ! next edge
         jpstart = nmeshc(iedge)     ! start in edge
         numnodes = nmeshc(jpstart)  ! number of nodes in edge

         if ( debug ) then
            write(irefwr,1) 'i, iedge, nodes ', &
                            i, iedge, nmeshc(jpstart+1:jpstart+numnodes)
         end if  ! ( debug )

         do j = 1, numnodes

            nodp = nmeshc(jpstart+j)  ! node in edge
            if ( imarknodes(nodp)==2 ) then

!           --- node has not been marked yet

               icount = icount+1
               imarknodes(nodp) = 1  ! mark node as not to be removed

            end if  ! ( imarknodes(nodp)==2 )

         end do  ! j = 1, numnodes

      end do  ! i = 1, numedges

      if ( icount<numnodesb ) removenodes = .true.

      if ( debug ) &
         write(irefwr,1) 'numnodesb, icount', numnodesb, icount

!     --- Remove all edges with markedges(iedge)=1

      if ( store ) then

!     --- store is true, so there are edges to be stored

         icount = 0             ! counter
         do i = 1, numedges

            iedge = setE(i)
            if ( markedges(iedge)==2 ) then

!           --- edge is single

               icount = icount+1
               setE(icount) = iedge

            end if  ! ( markedges(iedge)==2 )

         end do  ! i = 1, numedges

         numedges = icount  ! new value of numedges

      end if  ! ( store )

      deallocate ( markedges, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'markedges' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshshift2dcr2'

      end if  ! ( debug )
      call erclos ( 'mshshift2dcr2' )

      end
