      subroutine mshsinglesurf ( isurfs, nsurfaces, isurfar, &
                                 isurfsl, nsurfsl, second )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 13-05-2009 Extra debug
!        version  1.0  date 21-01-2009 Fortran 90
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy contents of array isurfs into temporary array isurfsl
!     Compound surfaces are splitted into single surfaces
!     Array isurfar is updated with respect to all surfaces that must be
!     changed
! **********************************************************************
!
!                       KEYWORDS
!
!     fill
!     surface
!     adapt_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nsurfaces, isurfs(nsurfaces), isurfar(*), &
              isurfsl(*), nsurfsl
      logical second

!     isurfar        i    Array of length nsurfs that contains information of
!                         the surfaces that must be changed. isurfar(i) mus be
!                         filled as follows:
!                         0  surfe i is not yet adapted
!                         1  surfe i has been adapted and is a single surfe
!     isurfs         i    Integer array containing the surface numbers that
!                         must be adapted. It is assumed that these surfaces
!                         form one continuous surface.
!     isurfsl        o    Array of length nsurfsl that is filled with all
!                         stand alone surfaces that must be adapted
!     nsurfaces      i    Number of surfaces in array isurfs
!     nsurfsl        o    Number of surfaces stored in isurfsl
!     second         i    If true this is the second call in a row and only
!                         isurfsl and nsurfsl must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, jsurnr, jsubsur
      logical compound

!     compound       If true the surface is a compound surface
!     i              Counting variable
!     jsubsur        |isubsur|
!     jsurnr         |isurnr|
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3010   Surface has been used more than once to be updated
! **********************************************************************
!
!                       PSEUDO CODE
!
!     nsurfsl = 0
!     For i := 1 (1) nsurfaces do
!        isurnr := isurfs(i)
!        if ( isurnr = compound surface ) then
!           For all subsurves isubsur of isurnr do
!              if ( isurfar(isubsur)=1 ) then
!                 error surface has already been updated
!              isurfar(isubsur) := 1
!              nsurfsl := nsurfsl+1
!              isurfsl := isubsur
!        else
!           if ( isurfar(isurnr)=1 ) then
!              error surface has already been updated
!           isurfar(isurnr) := 1
!           nsurfsl := nsurfsl+1
!           isurfsl := isurnr
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsinglesurf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information in mshsinglesurf'
         write(irefwr,*) 'second ', second
         call prinin ( isurfar, nsurfs, 'isurfar' )
         call prinin ( isurfs, nsurfaces, 'isurfs' )

      end if

      nsurfsl = 0

      do i = 1, nsurfaces

         jsurnr = isurfs(i)
         compound = .false.
         if ( kelmr>0 ) then
            if ( kmeshr(jsurnr+1)-kmeshr(jsurnr)>0 ) compound = .true.
         end if

         if ( debug ) then

!        --- Debug information

            write(irefwr,*) 'i, jsurnr, jsurnr, compound ', &
                             i, jsurnr, jsurnr, compound
         end if

         if ( compound ) then

!        --- surface is a composite surface

            do k = kmeshr(jsurnr)+1, kmeshr(jsurnr+1)

               jsubsur = kmeshr(k)

               if ( .not. second .and. isurfar(jsubsur)==1 ) then

!              --- surface has already been used

                  call errint ( jsubsur, 1 )
                  call errsub ( 3010, 1, 0, 0 )
                  go to 1000

               else

!              --- surface has not yet been used

                  isurfar(jsubsur) = 1
                  nsurfsl = nsurfsl + 1
                  isurfsl(nsurfsl) = jsubsur

               end if

            end do

         else

!        --- surface is a stand alone surface

            if ( .not. second .and. isurfar(jsurnr)==1 ) then

!           --- surface has already been used

               call errint ( jsurnr, 1 )
               call errsub ( 3010, 1, 0, 0 )
               go to 1000

            else

!           --- surface has not yet been used

               isurfar(jsurnr) = 1
               nsurfsl = nsurfsl + 1
               isurfsl(nsurfsl) = jsurnr

            end if

         end if

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshsinglesurf'

      end if
      call erclos ( 'mshsinglesurf' )

      end
