      subroutine mshsurfgen ( jnew, npoh, nelh, trans, check, &
                              rinput, iposi, iinput, irinpu, holeinfo, &
                              nholes, doublecurvs, boundary, &
                              numcurvboun, irefplax, bcord3d )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.3  date 12-03-2015 Extra debug
!        version  4.2  date 30-11-2014 Change isurf 23 in 99
!        version  4.1  date 22-05-2014 Extra error message
!
!   copyright (c) 2008-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create subsurfaces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoh, nelh, iposi, &
              iinput(*), irinpu, holeinfo(2,0:*), &
              nholes, numcurvboun, irefplax, &
              boundary(2,numcurvboun)
      double precision :: rinput(*), trans(*), bcord3d(*)
      logical jnew, check, doublecurvs

!     bcord3d        i    Contains the coordinates of the 3d boundary
!                         Is only used for error messages
!     boundary       i    Array of size 2 x numcurvboun containing
!                         information of the boundary of the surface
!                         (1,i) contains the i-th curve number (including sign)
!                         (2,i) contains the starting address in kbndpt
!     check          i    Indicates if the surface is old or a copy
!                         of a prior surface (true) or not (false)
!     doublecurvs    i    If true the boundary contains double curves
!     holeinfo       o    Integer array of size 2 x (nholes+2)
!                         Contains information about holes in surfaces and where
!                         a new part of the boundary starts
!                         Pos (1,i-1) contains the local sequence number of the
!                            first curve of part i provided with a sign
!                            If the sign is positive the boundary is
!                            created counter clockwise, otherwise clockwise
!                         Pos (2,i-1) contains the local sequence number of the
!                            first node of of part i
!                         Pos (1,nholes+1) contains the number of curves of the
!                         boundary
!     iinput         i    integer input array, see subroutine mesh
!     iposi         i/o   next free position of iinput
!     irefplax       i    File reference number for plaxis
!     irinpu         i    starting address in RINPUT local 3D surface parameters
!                         (u,v)
!     jnew           i    indication whether the mesh is new (true) or old
!     nelh          i/o   Estimate of the number of elements
!     nholes         i    Number of holes in the boundary of a surface
!     npoh          i/o   Estimate of the number of points
!     numcurvboun    i    Number of curves in the boundary
!     rinput         i    Real input array
!     trans          i    Contains information of the tranformation 3D-->2D
!                         Pos. 1-3:  t1
!                         Pos. 4-6:  t2
!                         Pos. 7-9:  t3
!                         Pos.10-18:  trans (inverse of transformation)
!                         Pos.19-21:  bcorut (coordinates of first point)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jdiag, ja, nhelp
      double precision umin, umax, vmin, vmax, maxlen
      logical trian

!     ja
!     jdiag          Defines diagonal direction for triangles in quadrilateral
!     maxlen         Maximum length of edges in the surface
!     nhelp
!     trian          Indication whether triangles (true) or quadrilaterals
!                    are created (false)
!     umax           largest value first coordinate parameter surface
!     umin           smallest value first coordinate parameter surface
!     vmax           largest value second coordinate parameter surface
!     vmin           smallest value second coordinate parameter surface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRORMSHSUR1   Compare estimated size of npointlc and nelemlc with actual size
!     ERRSUB         Error messages
!     MESHUS         User submesh generator
!     MSHCHECKKBN    Perform some checks on the mesh created in meshus
!     MSHDOUBLECURVS Map nodes of double curves onto each other
!     MSHER1         Submesh generator RECTANGLE
!     MSHG           Submesh generator GENERAL
!     MSHG11         Determines whether the boundary is clockwise or
!                    counterclockwise
!     MSHG12         Reorder the points of a boundary by making transformation
!                    X => -X
!     MSHISP         create a mesh in 2-D or 3-D surrounded by three curves
!                    as given by array BCORD using an isoparametric
!                    transformation for a triangle based on six points
!     MSHO50         create a mesh in 3-D space surrounded by four lines
!                    as given by array BCORD by the COONS method with help of
!                    MSHZDE
!     MSHO60         Submesh generator Parameter Surface
!     MSHPAV         Submesh generator PAVER
!     MSHPLAXSURF    Create surface mesh using surface generator framesurf
!     MSHPSR         Submesh generator PIPE SURFACE
!     MSHRFL         Submesh generator REFLECT
!     MSHROT         Submesh generator ROTATE
!     MSHSIM         Submesh generator SIMILAR
!     MSHSRFTRIA     Create surface mesh using surface geneartor mshoce
!     MSHTRA         Fill array coor by translation with respect to preceding
!                    array
!     MSHZDE         Submesh generator QUADRILATERAL
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     971   Mesh generator only available for fixed number of constituting
!           curves
!    1025   shape number has not yet been implemented for this subroutine
!    3189   option not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsurfgen' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsurfgen'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nhelp  =  npointlc

      select case ( isurf )

         case ( 1, 17 )

!        --- isurf = 1, 17   msher1

            if ( nvolms>0 .and. idiag==0 ) then
               jdiag = 1
            else
               jdiag = idiag
            end if
            if ( ishape==10 ) then
               inpelm = inpelm-1
               if ( jnew ) npointlc = npointlc-nelemlc
            end if

            call msher1 ( jnew, nboune, kbndptlc, &
                          bcord2d, nmes, mmes, coor2d, kmeshclc, jdiag )

         case (2)

!        --- isurf = 2   mshg

            npoh = npointlc
            nelh = nelemlc
            if ( ishape>=9 ) inpelm = inpelm-1

!           --- Check whether boundary is given counter clockwise

            call mshg11( bcord2d, 2, nbndpt, ja )

!           --- if ja=0 , the boundary is created clockwise
!               This is corrected by multiplying the bcord2d(1,.) by -1
!               hence x --> -x

            if ( ja==0 ) call mshg12 ( bcord2d, 2, nbndpt )
            call mshg ( jnew, coor2d, kmeshclc, nbndpt-1, kbndptlc, &
                        bcord2d, nhelp, irenum )
            call errormshsur1 ( nelh, npoh, nelemlc, npointlc, isurnr )

            if ( ja==0 ) then

!           --- Boundary has been created clock wise, reset coor2d(1,.) and
!               bcord2d(1,.)

               call mshg12 ( coor2d, 2, npointlc )
               call mshg12 ( bcord2d, 2, nbndpt )

            end if

         case (3, 99)

!        --- isurf = 3, 99: triangle

            call mshsrftria ( jnew, bcord2d, &
                              kbndptlc, boundary, numcurvboun, &
                              holeinfo, nholes, trans, iinput, &
                              rinput, bcord3d )

            if ( doublecurvs ) then

!           --- doublecurvs true, hence there are double curves in the boundary
!               To avoid problems with the volume generator, we map these points
!                on the same node numbers and reduce npointlc

               call mshdoublecurvs ( kmeshclc, nelemlc, inpelm, coor2d, 2, &
                                     npointlc, kbndptlc, nbndpt, ncurvs, &
                                     boundary, numcurvboun, isurnr )

            end if  ! ( doublecurvs ... )

         case(4)

!        --- isurf = 4, QUADRILATERAL

            if ( .not. check ) then
               npoh  = npointlc
               nelh  = nelemlc
            end if
            trian = mtrian==2
            if ( ishape==10 ) inpelm = inpelm-1

!           --- Check whether boundary is given counterclockwise

            call mshg11( bcord2d, 2, nbndpt, ja )

!           --- if ja=0 , the boundary is created clockwise
!               This is corrected by multiplying the bcord(1,.) by -1
!               hence x --> -x

            if ( ja==0 ) call mshg12 ( bcord2d, 2, nbndpt )

            call mshzde ( jnew, trian, inpelm, icurvt, iblend, nx, ny, &
                          nz, nc, nbndpt, bcord2d, npoh, nelh, &
                          kbndptlc, npointlc, nelemlc, coor2d, kmeshclc )
            call errormshsur1 ( nelh, npoh, nelemlc, npointlc, isurnr )

            if ( ja==0 ) then

!           --- Boundary has been created clock wise, reset coor2d(1,.) and
!               bcord2d(1,.)

               call mshg12 ( coor2d, 2, npointlc )
               call mshg12 ( bcord2d, 2, nbndpt )

            end if

         case (5)

!        --- isurf = 5
!            Clear arrays kmeshclc and kbndptlc before the call

            npoh = npointlc
            nelh = nelemlc
            kbndptlc(1:nbndpt) = 0
            kmeshclc(1:inpelm*nelemlc) = 0
            call meshus ( isurnr, jnew, coor2d, kmeshclc, inpelm, nbndpt-1, &
                          kbndptlc, bcord2d, npointlc, nelemlc )
            kbndptlc(nbndpt) = kbndptlc(1)
            call mshcheckkbn ( kbndptlc, kmeshclc, npoh, npointlc, nelh, &
                               nelemlc, isurnr, nbndpt, inpelm )

         case(8)

!        --- isurf = 8    translate

            call mshtra ( bcorold, bcord2d, nbndpt, ndim, coor2d, npointlc, &
                          isurnr )

         case(9)

!        --- isurf = 9    rotate

            call mshrot ( bcorold, bcord2d, nbndpt, ndim, coor2d, npointlc, &
                          isurnr )

         case(10)

!        --- isurf = 10   pipe surface

            if ( ishape>=9 ) inpelm = inpelm-1
            if ( debug ) then
               write(irefwr,1) 'ndim, mmes, nboune, inpelm', &
                                ndim, mmes, nboune, inpelm
               write(irefwr,1) 'startc3', ndim*(2*(mmes*nboune+1)+1)+1
            end if  ! ( debug )
            call mshpsr ( bcord2d, bcord2d(1,mmes*nboune+2), &
                          bcord2d(1,2*(mmes*nboune+1)+1), kbndptlc )

         case(13)

!        --- isurf = 13  Coons3D

            if ( .not. check ) then
               npoh  = npointlc
               nelh  = nelemlc
            end if
            trian = mtrian==2
            if ( check ) npointlc = npointlc + 1
            if ( ishape==10 ) inpelm = inpelm-1
            idiag = max(1,idiag)

            call msho50 ( jnew, bcord2d, &
                          kbndptlc, nboune, rinput(3), idiag )

            kmeshs(nsurfs+1+8*(isurnr-1)+3) = nx
            kmeshs(nsurfs+1+8*(isurnr-1)+4) = ny
            kmeshs(nsurfs+1+8*(isurnr-1)+5) = nz
            kmeshs(nsurfs+1+8*(isurnr-1)+6) = nc
            call errormshsur1 ( nelh, npoh, nelemlc, npointlc, isurnr )

         case(14)

!        --- isurf = 14    SIMILAR

            call mshsim ( kmeshclc, nelemlc, inpelm, bcord2d, nbndpt, ndim, &
                          coor2d, npointlc, isurnr, kbndptlc )

         case(15)

!        --- isurf = 15   PARSURF : User prescribed surface, use FUNCSF, &

            if ( .not. check ) then
               npoh  = npointlc
               nelh  = nelemlc
            end if
            trian = mtrian==2
            if ( check ) npointlc = npointlc + 1
            if ( ishape==10 ) inpelm = inpelm-1

!           --- Determine umin, umax, vmin, vmax

            umin = rinput(irinpu+4*isurnr-3)
            umax = rinput(irinpu+4*isurnr-2)
            vmin = rinput(irinpu+4*isurnr-1)
            vmax = rinput(irinpu+4*isurnr  )

            call msho60 ( jnew, kmeshclc, &
                          kbndptlc, bcord2d, nx, ny, nz, nc, &
                          umin, umax, vmin, vmax, nboune )

            kmeshs(nsurfs+1+8*(isurnr-1)+3) = nx
            kmeshs(nsurfs+1+8*(isurnr-1)+4) = ny
            kmeshs(nsurfs+1+8*(isurnr-1)+5) = nz
            kmeshs(nsurfs+1+8*(isurnr-1)+6) = nc
            call errormshsur1 ( nelh, npoh, nelemlc, npointlc, isurnr )

         case(16)

!        --- isurf = 16    REFLECT

            call mshrfl ( bcorold, bcord2d, nbndpt, ndim, coor2d, npointlc, &
                          isurnr )
         case(18)

!        --- isurf = 18    ISOPAR

            npoh = npointlc
            nelh = nelemlc
            if ( ishape==10 ) inpelm = inpelm-1

            if ( iinput(2+iposi)/=3 ) then

!           --- Number of constituting curves should be 3

               call errint ( 3, 1 )
               call errint ( iinput(2+iposi), 2 )
               call errint ( isurnr, 3 )
               call errchr ( 'isopar', 1 )
               call errsub ( 971, 3, 0, 1 )
               go to 1000

            end if

            call mshisp ( jnew, bcord2d, &
                          kbndptlc, boundary, numcurvboun, iinput(3+iposi), &
                          rinput(3), holeinfo, nholes )
            call errormshsur1 ( nelh, npoh, nelemlc, npointlc, isurnr )

         case(19)

!        --- isurf = 19 PAVER

            if ( ishape/=3 ) then

               call errint ( ishape, 1 )
               call errchr ( 'paver', 1 )
               call errsub ( 1025, 1, 0, 1 )

            else

!           --- Check if boundary is created counter clockwise

               call mshg11 ( bcord2d, 4, nbndpt, ja )

!              --- if ja=0 , the boundary is created clockwise
!                  This is corrected by multiplying the bcord2d(1,.) by -1
!                  hence x --> -x

               if ( ja==0 ) call mshg12 ( bcord2d, 4, nbndpt )
               maxlen = rinput(irinpu+4*isurnr-3)
               call mshpav ( coor2d, kbndptlc, bcord2d, &
                             npointlc, nelemlc, maxloops, maxlen )

               if ( ja==0 ) then

!              --- Boundary has been created clock wise, reset coor2d(1,.) and
!                  bcord2d(1,.)

                  call mshg12 ( coor2d, 2, npointlc )
                  call mshg12 ( bcord2d, 4, nbndpt )

               end if

            end if

         case(21)

!        --- In this case the surface is a precise copy
!            Nothing has to be done

         case (22)

!        --- isurf = 22: framesurf

            call mshplaxsurf ( bcord2d, &
                               kbndptlc, holeinfo, nholes, &
                               iinput, rinput, &
                               nextrapnts, doublecurvs, &
                               numcurvboun, boundary, irefplax )

         case default

!        --- Value of isurf not yet implemented

            call errint ( isurf, 1 )
            call errchr ( 'isurf', 1 )
            call errsub ( 3189, 1, 0, 1 )
            go to 1000

      end select  ! case(isurf)

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kbndptlc, nbndpt, 'kbndptlc' )
         write(irefwr,*) 'End mshsurfgen'

      end if  ! ( debug )
      call erclos ( 'mshsurfgen' )
      end subroutine mshsurfgen
