      subroutine seplevelcrel1d ( levelset, lenicrossobst, lenrcrossobst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute all information concerning kmesh ad parts 3 and 4, i.e.
!     information of the cross sections of elements with zero level set and
!     store in arrays icrossobst and rcrossobst
!     Old subroutine mshlevelcrel1d
! **********************************************************************
!
!                       KEYWORDS
!
!     1d
!     element
!     level_set
!     crossing
!     cross_section
!     crossing_point
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lenicrossobst, lenrcrossobst
      double precision levelset(*)

!     lenicrossobst  o    Computed length of array icrossobst
!     lenrcrossobst  o    Computed length of array rcrossobst
!     levelset       i    Array of length npoint containing the value of the
!                         level set function in each node
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielem, iobst, ikelmc, inode1, inode2, &
              numintersect, inpelm, istartrcross, istart, istarticr, &
              istarticross, numinelmobst, numbnelmobst
      double precision fact1

!     fact1          multiplication factor
!     i              Counting variable
!     ielem          Element number
!     ikelmc         position in kmesh part c
!     inode1         First node in edge
!     inode2         Second node in edge
!     inpelm         number of points per element
!     iobst          Obstacle sequence number
!     istart         Starting address
!     istarticr      Starting address of cross section in array icrossobst
!     istarticross   Starting address of obstacle in array icrossobst
!     istartrcross   Starting address of cross section in array rcrossobst
!     numbnelmobst   Number of elements that are partly within the obstacle
!     numinelmobst   Number of elements that are completely within the obstacle
!     numintersect   Number of intersections
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array icrossobst
!     For iobst := 1 (1) numobst
!        For all elements that are partly in obstacle
!           ikelmc := starting address of element in kmeshc
!           inpelm := number of nodes in element
!           Compute the intersections of the element with the
!           obstacles and store information in icrossobst and rcrossobst
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seplevelcrel1d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seplevelcrel1d'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear array icrossobst

      icrossobst(1:numobst) = 0
      istarticross = numobst+1
      istartrcross = 0

      do iobst = 1, numobst

         icrossobst(iobst) = istarticross
         istart = ielemobst(iobst)
         numinelmobst = ielemobst(istart)
         numbnelmobst = ielemobst(istart+1)+ielemobst(istart+2)
         istart = istart+2+numinelmobst
         numintersect = 0

         istarticr = istarticross

!        --- For all elements that are partly in obstacle
!              ikelmc := starting address of element in kmeshc
!              inpelm := number of nodes in element

         do i = 1, numbnelmobst

            ielem = ielemobst(istart+i)
            ikelmc = sepstkmeshc ( ielem, inpelm )
            if ( debug ) &
               write(irefwr,1) 'i, ielem, ikelmc, inpelm', &
                                i, ielem, ikelmc, inpelm

!           --- Compute the intersections of the element with the
!               obstacle and store information in icrossobst and rcrossobst

            inode1 = kmeshc(ikelmc)
            inode2 = kmeshc(ikelmc+inpelm-1)
            fact1 = levelset(inode1)/(levelset(inode1)-levelset(inode2))
            if ( fact1>1d-6 .and. fact1<1d0-1d-6 ) then

!           --- Node not on intersection

               numintersect = numintersect+1
               istartrcross = istartrcross+1
               rcrossobst(istartrcross) = fact1
               icrossobst(istarticr+1) = ielem
               icrossobst(istarticr+2) = istartrcross
               istarticr = istarticr+2

               if ( debug ) then
                  write(irefwr,1) 'inode1, inode2', inode1, inode2
                  write(irefwr,2) 'fact1', fact1
               end if  ! ( debug )

            end if  ! ( fact1>1d-6*dx .and. fact1<1d0-1d-6*dx )

         end do  ! i = 1, numbnelmobst
         icrossobst(istarticross) = numintersect
         istarticross = istarticross+2*numintersect+1

      end do  ! iobst = 1, numobst

      lenrcrossobst = istartrcross
      lenicrossobst = istarticross-1

1000  if ( debug ) then

!     --- Debug information

         call prinin ( icrossobst, lenicrossobst, 'icrossobst' )
         call prinrl ( rcrossobst, lenrcrossobst, 'rcrossobst' )
         write(irefwr,*) 'End seplevelcrel1d'

      end if  ! ( debug )
      call erclos ( 'seplevelcrel1d' )

      end
