      subroutine er0070 ( iintbn, usol, isurfs, value, compl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.5  date 11-03-2013 Correction formats
!        version  3.4  date 07-10-2011 Extra debug
!
!   copyright (c) 1991-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     subroutine er0070 computes the boundary integral over a set of
!     subsequent surfaces
!     ER0070 is a help subroutine for subroutine INTBOU
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iintbn(*), isurfs(*)
      double precision usol(*), value(*)
      logical compl

!     compl      i    Indication if the solution vector is real (false) or
!                     complex (true)
!     iintbn     i    Integer input array in which the user indicates what type
!                     of integral must be computed.
!                     IINTBN must be filled as follows:
!                     IINTBN(1) Type of calling subroutine
!                               1:  BOUNIN
!                               2:  BOUNIV
!                               3:  INTBOU
!                               4:  INTBND
!                     IINTBN(2) (ICHINT) indicates what type of integral has to
!                               be computed. Possibible values:
!                               1          /  f u  ds
!                               2          /  f  u.n  ds
!                               3          /  f  u.t  ds
!                               4          /  f  n. sigma .n  ds
!                               5          /  f  n. sigma .t  ds
!                               6          /  f  n. sigma     ds
!                               7          /  f  u  n  ds
!                               8          sum f u
!                               9          /  f ds
!                               The possibilities 3 to 6 have only been
!                               implemented for NDIM = 2, i.e. two-dimensional
!                               regions
!                               Default value: 1
!                     IINTBN(3) (ICHFUN) indication of the value of f.
!                               Possibible values:
!                               0         f = 1
!                               >0        f = func(ichfun,x,y,z) (real case)
!                                         or
!                                         f = cfunc(ichfun,x,y,z) (complex
!                                             case)
!                                         at this moment only the combination
!                                         f#1 and ichint=1 is permitted
!                               Default value: 0
!                     IINTBN(4) (IRULE) The numerical integration rule to be
!                               carried out depends on IRULE. Possibilities:
!                               Possibible values:
!                               For curves only (i.e. 2D):
!                               1  trapezoid rule,  cartesian co-ordinates
!                               2  trapezoid rule,  axisymmetric co-ordinates
!                               3  simpson's rule   cartesian co-ordinates
!                               4  simpson's rule   axisymmetric co-ordinates
!                               For surfaces only (i.e. 3D):
!                               1  Newton cotes rule, i.e. integration rule
!                                  based on the type of approximation used
!                                  in the element
!                               Default value: 1
!                     IINTBN(5) (IDIM) This parameter indicates what type of
!                               boundary integral must be computed
!                               Possible values:
!                               1: curves
!                               2: surfaces
!                               3: contact
!                               4: no_contact
!                     IINTBN(6) (NUMBER) This parameter indicates which unknown
!                               (physical degree of freedom) must be used from
!                               the (solution) vector u (IVECTR)
!                               If ICHINT = 2, 3 or 7, it is supposed that the
!                               degrees of freedom are equal to NUMBER, &
!                               NUMBER+1 and NUMBER+2 (3D)
!                               If ICHINT = 4, 5 or 6, it is supposed that the
!                               degrees of freedom are equal to NUMBER, &
!                               NUMBER+1 and NUMBER+2 (2D) or NUMBER to
!                               NUMBER+5 (3D)
!                               Default value: 1
!                     IINTBN(7) (NCURVES) Indicates the number of curves or
!                               surfaces in array ICURVS that must be used for
!                               the integration
!                               Default value: 1
!     isurfs     i    Contains surf numbers
!     usol       i    Solution vector
!     value      o    array containing the computed value of the integral
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isurf, inpelm, i, idim, &
              k, numsrf, ichint, ichfun, number, iconct, nelcon, &
              nelnocon, istart, iposkmeshn

!     i              Counting variable
!     ichfun         choice parameter for the computation of f
!                    Possibible values:
!                    0         f = 1
!                    >0        f = func ( ichfun, x, y, z ) (real case)
!                                  or
!                              f = cfunc ( ichfun, x, y, z ) (complex case)
!                              at this moment only the combination f#1
!                              and ichint=1 is permitted
!     ichint         choice parameter for the type of integral
!                    Possibible values:
!                    1          /  f u  ds
!                    2          /  f  u.n  ds
!                    3          /  f  u.t  ds
!                    4          /  f  n. sigma .n  ds
!                    5          /  f  n. sigma .t  ds
!                    6          /  f  n. sigma     ds
!                    7          /  f  u  n  ds
!                    9          /  f ds
!     iconct         Sequence number of contact
!     idim           This parameter indicates what type of boundary
!                    integral must be computed
!                    Possible values:
!                    1: curves
!                    2: surfaces
!                    3: contact
!                    4: no_contact
!     inpelm         number of points per element
!     iposkmeshn     Start address in kmeshn
!     istart         First position used at a part
!     isurf          surface sequence number
!     k              Counting variable
!     nelcon         Number of contact elements
!     nelnocon       Number of no_contact elements
!     number         The number-th unknown in each nodal point of the array
!                    is considered. If the array is complex number=2*i-1
!                    corresponds to the real part of array i and number=2*i
!                    to the imaginary part of array i
!     numsrf         Number of surfaces in array isurfs, see iintbn
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0071  Actual computation of boundary integral for one surface
!             Real case
!     ER0072  Actual computation of boundary integral for one surface
!             Complex case
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     146   surface number has incorrect value
!     451   ichois has incorrect value
!    1513   Too much surfaces in isurfs
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'er0070' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from er0070'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if

!     --- checks of the parameters

      ichint = iintbn(2)
      ichfun = iintbn(3)
      idim   = iintbn(5)
      number = iintbn(6)
      if ( debug ) &
         write(irefwr,1) 'ichint, ichfun, idim, number', &
                          ichint, ichfun, idim, number
      if ( idim>2 ) then
         numsrf = 1
         iconct = iintbn(7)
      else
         numsrf = iintbn(7)
         iconct = 0
      end if
      if ( ichint/=1 .and. ichint/=9 ) then
         call errint ( ichint, 1 )
         call errint ( ichfun, 2 )
         call errint ( ichint, 3 )
         call errsub ( 451, 3, 0, 0 )
      end if
      if ( ichfun<0 .or. ichfun>1000 ) then
         call errint ( ichfun, 1 )
         call errint ( ichfun, 2 )
         call errint ( ichint, 3 )
         call errsub ( 451, 3, 0, 0 )
      end if
      if ( numsrf>100 ) then
         call errint ( numsrf, 1 )
         call errint ( 100, 2 )
         call errsub ( 1513, 2, 0, 0 )
      end if
      if ( iconct>maxcontc ) go to 1000
      if ( ierror/=0 ) go to 1000

      value(1) = 0d0
      if ( compl ) value(2) = 0d0

      if ( debug ) &
         write(irefwr,1) 'npoint, nsurfs, numsrf', npoint, nsurfs, numsrf

      do i = 1, numsrf

         if ( idim==2 ) then

!        --- Standard case (surfaces)

            isurf = isurfs(i)
            if ( isurf<=0 .or. isurf>nsurfs ) then
               call errint ( isurf, 1 )
               call errint ( nsurfs, 2 )
               call errsub ( 146, 2, 0, 0 )
            end if
            inpelm = kmeshn(isurf)
            inelem  = kmeshn(isurf+nsurfs)
            iposkmeshn = 1+2*nsurfs
            do k = 1, isurf-1
               iposkmeshn = iposkmeshn + kmeshn(k)*kmeshn(k+nsurfs)
            end do
            istart = 1
            if ( debug ) &
               write(irefwr,1) 'isurf, inpelm, inelem, iposkmeshn', &
                                isurf, inpelm, inelem, iposkmeshn

         else if ( idim==3 ) then

!        --- Contact elements

            nelcon = kprobwp1(iconct,1)
            if ( nelcon==0 ) go to 1000
            isurf = kprobwp1(iconct,3)
            istart = kprobwp1(iconct,4)
            iposkmeshn = 1+2*nsurfs
            do k = 1, isurf-1
               iposkmeshn = iposkmeshn + kmeshn(k)*kmeshn(k+nsurfs)
            end do
            inpelm = kmeshn(isurf)
            inelem = nelcon

         else

!        --- No_contact elements

            nelcon = kprobwp1(iconct,1)
            nelnocon = kprobwp1(iconct,2)
            if ( nelnocon==0 ) go to 1000
            isurf = kprobwp1(iconct,3)
            istart = kprobwp1(iconct,4)+nelcon
            iposkmeshn = 1+2*nsurfs
            do k = 1, isurf-1
               iposkmeshn = iposkmeshn + kmeshn(k)*kmeshn(k+nsurfs)
            end do
            inpelm = kmeshn(isurf)
            inelem = nelnocon

         end if
         if ( compl ) then

!        --- Complex case

            call er0072 ( kmeshn(iposkmeshn), inpelm, inelem, usol, &
                          ichint, ichfun, number, value, isurf, idim, &
                          istart )
         else

!        --- Real case

            call er0071 ( kmeshn(iposkmeshn), inpelm, inelem, usol, &
                          ichint, ichfun, number, value, isurf, idim, &
                          istart )

         end if

      end do

1000  if ( debug ) then

         write(irefwr,2) 'value', value(1)
         write(irefwr,*) 'End er0070'

      end if
      call erclos ( 'er0070' )

      end
