      subroutine erintpol ( iinput, isol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 30-11-2012 Long integers
!        version  1.2  date 30-08-2006 New definition of cpost5
!        version  1.1  date 22-11-2004 Remove chvectors
!
!   copyright (c) 2001-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Interpolate element values to nodal points using a type of interpolation
!    defined by the user
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     node
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulevecs
      use sepmoduleplot
      use sepmoduletexts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iinput(*), isol(*)

!     iinput         i    integer input array that must be filled as follows
!                         1:  ichoice, defines type of interpolation
!                             Possible values
!                             1: linear interpolation based on three points
!                         2:  iseq_input, defines sequence number of input
!                             array with respect to isol
!                         3:  iseq_output, defines sequence number of output
!                             array with respect to isol
!                         4:  iseq_start, defines sequence number of start
!                             array with respect to isol
!     isol           i    Standard SEPRAN array containing information of the
!                         solution arrays
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseq_input, iseq_output, iseq_start, ndegfd
      integer, allocatable :: iwork1(:), iwork2(:)

!     iseq_input     sequence number of input array with respect to isol
!     iseq_output    sequence number of output array with respect to isol
!     iseq_start     sequence number of start array with respect to isol
!     iwork1         integer work array of size npoint
!                    iwork1 is filled in the following way:
!                    iwork1(i) = 0:  node i is internal
!                    iwork1(i) = 1:  node i is on the boundary
!     iwork2         integer work array of size 2 x nelem
!                    iwork2 is filled in the following way:
!                    iwork2(1,ielem) contains number of nodes of element
!                    iwork2(2,ielem) contains number of starting position
!                                    of element ielem in array kmeshc
!     ndegfd         Number of degrees of freedom per point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     ERFILLBOUN     Fill array iwork with a 1 for all boundary nodes and
!                    a 0 for all internal nodes
!     ERFILLSTRTKMC  Fill array iwork with number of nodes per element as well
!                    as starting addresses in array kmeshc
!     ERINTPTHREEINT Interpolate values defined per element to nodal points
!                    Interpolation based on three points
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCOPYVC      Copy one SEPRAN vector into another one
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      60   Input solution array has not been filled or is of the wrong type
!    1059   number of degrees of freedom incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Get parameters from iinput
!     Test input
!     Activate arrays
!     Create solution array
!     Define work arrays
!     Mark all boundary points in iwork1
!     Fill iwork2 with information concerning the elements
!     Interpolate the vector
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erintpol' )

!     --- Get parameters from iinput

      iseq_input = iinput(2)
      iseq_output = iinput(3)
      iseq_start = iinput(4)

!     --- Test input

      if ( ks(isol(iseq_input))%typevec/=116 ) then

!     --- input vector is not defined per element

         call errint ( ks(isol(iseq_input))%typevec, 1 )
         call errsub ( 60, 1, 0, 0 )

      end if
      ndegfd = ks(isol(iseq_input))%n
      if ( ndegfd/=1 ) then

!     --- ndegfd # 1

         call errint ( ndegfd, 1 )
         call errint ( 1, 2 )
         call errchr ( names(iseq_input), 1 )
         call errsub ( 1059, 2, 0, 1 )

      end if
      if ( ks(isol(iseq_start))%typevec/=115 .and. &
           ks(isol(iseq_start))%typevec/=110 ) then

!     --- start vector is not of the correct type

         call errint ( ks(isol(iseq_start))%typevec, 1 )
         call errsub ( 60, 1, 0, 0 )

      end if

!     --- Create solution array

      call sepcopyvc ( isol(iseq_start), isol(iseq_output) )

!     --- Activate arrays

      call sepactsolbf2 ( isol(iseq_start) )
      if ( nphyslc/=1 ) then

!     --- nphyslc # 1

         call errint ( nphyslc, 1 )
         call errint ( 1, 2 )
         call errchr ( names(iseq_start), 1 )
         call errsub ( 1059, 2, 0, 1 )

      end if

!     --- Define work arrays

      allocate ( iwork1(npoint), iwork2(2*nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint+2*nelem, 'iwork1' )
      if ( ierror/=0 ) go to 1000

      vectin => ks(isol(iseq_input))%sol
      vectout => ks(isol(iseq_output))%sol

!     --- Mark all boundary points in iwork1

      call erfillboun ( iwork1, iwork2 )

!     --- Fill iwork2 with information concerning the elements

      call erfillstrtkmc ( iwork2 )

!     --- Interpolate the vector

      call erintpthreeint ( vectin, vectout, iwork1, iwork2 )


      deallocate ( iwork1, iwork2, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork1' )

1000  call erclos ( 'erintpol' )
      end subroutine erintpol
