      subroutine erintpthreeint ( value, result, iwork1, iwork2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-02-2001
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Interpolate values defined per element to nodal points
!    Three surrounding elements are used and a linear interpolation is applied
!    This subroutine is applied only for internal points
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     node
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork1(*), iwork2(2,*)
      double precision value(*), result(*)

!     iwork1         i    integer work array of size npoint
!                         iwork1 must have been filled before in the following
!                         way:
!                         iwork1(i) = 0:  node i is internal
!                         iwork1(i) = 1:  node i is on the boundary
!     iwork2         i    integer work array of size 2 x nelem
!                         iwork2 must have been filled before in the following
!                         way:
!                         iwork2(1,ielem) contains number of nodes of element
!                         iwork2(2,ielem) contains number of starting position
!                                         of element ielem in array kmeshc
!     result        i/o   Result of the interpolation in the vertices
!     value          i    Array of size nelem containing the values to be
!                         interpolated in the elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxlocal
      parameter ( maxlocal=10 )
      integer i, j, k, l, nlocal, ind(3), ielem, imin, irep, &
              inodp, ifirst, istart, iwork(maxlocal), iquadrant, &
              locquad(maxlocal), listquad(7), inewquadrant
      double precision locval(maxlocal), loccoor(2,maxlocal), &
                       locdist(maxlocal), x, y, ds, delta, &
                       lambda(3), e(3,2), xcentr, ycentr, &
                       minloc, dphidx(3,0:2)
      logical found

!     delta          jacobian of element
!     dphidx         Array of length n x ndim  containing the
!                    derivatives
!                    of the basis functions in the sequence:
!                    d phi_i/ dx  = dphidx (i,1);
!                    d phi_i/ dy  = dphidx (i,2);
!     ds             Distance between first two points
!     e              Work array of length 3 x 2, to store the factors
!                    e(i,j) to compute the derivatives, see als elp620
!     found          If true a point has been found
!     i              Counting variable
!     ielem          Element number
!     ifirst         First point that maes a triangle complete
!     imin           Position in array for which minimum is reached
!     ind            Integer array containing the positions of the three nodes
!                    of a triangle
!     inewquadrant   New quadrant
!     inodp          Nodal point number
!     iquadrant      quadrant sequence number
!     irep           Loop variable
!     istart         First position used at a part
!     iwork          Work array of size maxlocal that is filled as follows
!                    iwork(i) = 0 if local node i has not yet been used
!                    iwork(i) = 1 if local node i has been used
!     j              Counting variable
!     k              Counting variable
!     l              General loop variable
!     lambda         Array containing the linear basis functions in a point
!     listquad       List of opposite quadrants (cyclic)
!     loccoor        array containing the coordinates of the centroids of
!                    surrounding elements
!     locdist        Array containing the distances of nodal point to centroids
!                    of surrounding elements
!     locquad        Contains the quadrants corresponding to the points
!     locval         Array containing the values of the funcion to be
!                    interpolated in surrounding elements
!     maxlocal       Maximum number of surrounding elements
!     minloc         Minimum local distance
!     nlocal         Actual number of surrounding elements
!     x              x co-ordinate of point
!     xcentr         x co-ordinate of centroid
!     y              y co-ordinate of point
!     ycentr         y co-ordinate of centroid
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!     2296           ?
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all nodes i not on the boundary do
!        nlocal := 0
!        For all real elements ielem containing node i do
!           nlocal := nlocal+1
!           locval(nlocal) := ielem
!           loccoor(.,nlocal) := coordinates centroid
!           locdist(nlocal) := dist(centroid, node_i)
!        Clear array iwork(1...nlocal)
!        ind(1) := index minimum locdist
!        iwork(ind(1)) := 1
!        ind(2) := index minimum locdist with iwork(ind(2))#0
!        iwork(ind(2)) := 1
!        found := false
!        While not found do
!           For all points with index = 0 do
!              ind(3) := index minimum locdist with iwork(ind(3))#0
!              iwork(ind(3)) := 1
!              If ( ind(1-3) a real triangle and i in triangle then
!                 found := true
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data listquad / 3, 4, 1, 2, 3, 4, 1 /
! ======================================================================
!
      call eropen ( 'erintpthreeint' )

!     --- For all nodes i not on the boundary do

      do i = 1, npoint

         if ( iwork1(i)==0 ) then

!        --- iwork1(i) = 0, hence point not on boundary
!            nlocal := 0

            nlocal = 0
            x = coor(1,i)
            y = coor(2,i)

!           --- For all real elements ielem containing node i do

            do j = kmshd1(i)+1, kmshd1(i+1)

               ielem = kmshd2(j)

               if ( iwork2(1,ielem)/=0 ) then

!              --- nlocal := nlocal+1
!                  locval(nlocal) := ielem
!                  loccoor(.,nlocal) := coordinates centroid
!                  locdist(nlocal) := dist(centroid, node_i)

                  if ( nlocal>=maxlocal ) then

!                 --- maxlocal too small

                     call errint ( nlocal+1, 1 )
                     call errint ( maxlocal, 2 )
                     call errchr ( 'locval', 1 )
                     call errsub ( 550, 2, 0, 1 )
                     go to 1000

                  end if
                  nlocal = nlocal+1
                  locval(nlocal) = value(ielem)
                  xcentr = 0
                  ycentr = 0
                  npelm = iwork2(1,ielem)
                  istart = iwork2(2,ielem)
                  do k = 0, npelm-1
                     inodp = kmeshc(istart+k)
                     xcentr = xcentr+coor(1,inodp)
                     ycentr = ycentr+coor(2,inodp)
                  end do
                  xcentr = xcentr/npelm
                  ycentr = ycentr/npelm
                  loccoor(1,nlocal) = xcentr
                  loccoor(2,nlocal) = ycentr
                  locdist(nlocal) = sqrt((xcentr-x)**2+(ycentr-y)**2)

!                 --- compute quadrant and store in locquad

                  if ( x-xcentr>0 ) then
                     if ( y-ycentr>0 ) then
                        iquadrant = 3
                     else
                        iquadrant = 4
                     end if
                  else
                     if ( y-ycentr>0 ) then
                        iquadrant = 2
                     else
                        iquadrant = 1
                     end if
                  end if
                  locquad(nlocal) = iquadrant

               end if

            end do

!           --- Clear array iwork(1...nlocal)

            do k = 1, nlocal
               iwork(k) = 0
            end do

!           --- ind(1) := index minimum locdist
!               iwork(ind(1)) := 1

            minloc = rinfin
            do k = 1, nlocal
               if ( locdist(k)<minloc ) then
                  imin = k
                  minloc = locdist(k)
               end if
            end do
            ind(1) = imin
            iwork(imin) = 1

!           --- ind(2) := index minimum locdist with iwork(ind(2))#0
!               iwork(ind(2)) := 1
!               First we are looking in other quadrants

            found = .false.
            iquadrant = locquad(ind(1))
            do l = 1, 4
               inewquadrant = listquad(iquadrant+l-1)
               if ( inewquadrant/=iquadrant ) then

!              --- new quadrant found

                  minloc = rinfin
                  do k = 1, nlocal
                     if ( iwork(k)==0 .and. &
                          locquad(k)==inewquadrant .and. &
                          locdist(k)<minloc ) then
                        imin = k
                        minloc = locdist(k)
                        found = .true.
                     end if
                  end do
                  if ( found ) then

!                 --- Point with minimal distance in other quadrant found

                     ind(2) = imin
                     iwork(imin) = 1
                     go to 100

                  end if

               end if

            end do

100         if ( .not. found ) then

!           --- No point found in other quadrant (hardly possible)

               minloc = rinfin
               do k = 1, nlocal
                  if ( iwork(k)==0 .and. locdist(k)<minloc ) then
                     imin = k
                     minloc = locdist(k)
                  end if
               end do
               ind(2) = imin
               iwork(imin) = 1

            end if

            ds = sqrt((loccoor(1,ind(2))-loccoor(1,ind(1)))**2 + &
                      (loccoor(2,ind(2))-loccoor(2,ind(1)))**2 )

!           --- found := false

            ifirst = 0

!           --- While not found do
!               For all points with index = 0 do
!                  ind(3) := index minimum locdist with iwork(ind(3))#0
!                  iwork(ind(3)) := 1

            do irep = 1, nlocal-2
               minloc = rinfin
               do k = 1, nlocal
                  if ( iwork(k)==0 .and. locdist(k)<minloc ) then
                     imin = k
                     minloc = locdist(k)
                  end if
               end do
               ind(3) = imin
               iwork(imin) = 1

!              --- If ( ind(1-3) a real triangle and i in triangle then
!                     found := true

               e(1,1) = loccoor(1,ind(3)) - loccoor(1,ind(2))
               e(2,1) = loccoor(1,ind(1)) - loccoor(1,ind(3))
               e(3,1) = loccoor(1,ind(2)) - loccoor(1,ind(1))
               e(1,2) = loccoor(2,ind(2)) - loccoor(2,ind(3))
               e(2,2) = loccoor(2,ind(3)) - loccoor(2,ind(1))
               e(3,2) = loccoor(2,ind(1)) - loccoor(2,ind(2))
               delta = e(3,2)*e(1,1) - e(1,2)*e(3,1)
               if ( abs(delta)>0.2d0*ds**2 ) then

!              --- Real triangle found

                  if ( ifirst==0 ) ifirst = ind(3)
                  do k = 1, 3
                     dphidx(k,1) = e(k,1)/delta
                     dphidx(k,2) = e(k,2)/delta
                     dphidx(k,0) = 1d0-dphidx(k,2)*loccoor(1,ind(k))- &
                                       dphidx(k,1)*loccoor(2,ind(k))
                     lambda(k) = dphidx(k,0)+dphidx(k,2)*x+dphidx(k,1)*y
                  end do

                  if ( lambda(1)>-0.05d0 .and. &
                       lambda(2)< 1.05d0 .and. &
                       lambda(2)>-0.05d0 .and. &
                       lambda(1)< 1.05d0 .and. &
                       lambda(3)>-0.05d0 .and. &
                       lambda(3)< 1.05d0 ) then
                     result(i) = lambda(1)*locval(ind(1)) + &
                                 lambda(2)*locval(ind(2)) + &
                                 lambda(3)*locval(ind(3))
                     go to 200

                  end if

               end if

            end do
            if ( ifirst>0 ) then

!           --- Not a third point found, use ifirst

               ind(3) = ifirst
               do k = 1, 3
                  dphidx(k,1) = e(k,1)/delta
                  dphidx(k,2) = e(k,2)/delta
                  dphidx(k,0) = 1d0-dphidx(k,2)*loccoor(1,ind(k))- &
                                    dphidx(k,1)*loccoor(2,ind(k))
                  lambda(k) = dphidx(k,0)+dphidx(k,2)*x+dphidx(k,1)*y
               end do
               result(i) = lambda(1)*locval(ind(1)) + &
                           lambda(2)*locval(ind(2)) + &
                           lambda(3)*locval(ind(3))

            else

!           --- There is no real triangle, give error message

               call errint ( i, 1 )
               call errsub ( 2296, 1, 0, 0 )

            end if

200      end if

      end do

1000  call erclos ( 'erintpthreeint' )
      end
