      subroutine intersect3dch1 ( coor, plane, trans, ichoice_tang, &
                                  values, npoint, kdir )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-12-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill origin in trans
!
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     interpolation
!     intersection
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichoice_tang, kdir
      double precision coor(*), trans(3,3), plane(*), values(*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     ichoice_tang   i    indicates how the tangential vectors must be computed
!                         See subroutine intp3dbf
!     kdir           o    Coordinate direction of largest component
!     npoint         i    Number of nodal points
!     plane          i    Array of length 4 containing the parameters a, b, c
!                         and d describing the plane: ax=by+cz+d = 0
!                         plane(1) = a; plane(2) = b; plane(3) = c; plane(4) = d
!     trans          o    In this array the Transformation matrix from x,y,z
!                         co-ordinates to ksi-eta co-ordinates is stored as
!                         well as the origin in trans(.,3)
!                         The transformation of x,y,z to ksi,eta is defined by
!                         ksi = sum_i trans(i,1) ( x(i) - trans(i,3) )
!                         eta = sum_i trans(i,2) ( x(i) - trans(i,3) )
!     values         i    Array containing information concerning the plane, &
!                         tangential vectors and origin
!                         See subroutine intp3dbf
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ind(4), i, npoint
      double precision anorm, alpha, d, xmin, xmax, ymin, ymax, &
                       zmin, zmax, t1(3), t2(3), epsacc

!     alpha          factor to be used in computation
!     anorm          norm of vecor
!     d              max ( xmax-xmin, ymax-ymin, zmax-zmin )
!     epsacc         Accuracy depending on the coordinates
!     i              Counting variable
!     ind            Integer array containing the cyclic permutation of
!                    1,2,3,4
!     t1             First tangential vector
!     t2             Second tangential vector
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     ymax           Maximum y-value
!     ymin           Minimum y-value
!     zmax           Maximum z-value
!     zmin           Minimum z-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer idamax
      double precision dnrm2

!     DAXPY          Overwrite double precision dy with double precision
!                    da*dx + dy
!     DNRM2          Blas routine to compute the 2-norm of a vector
!     DSCAL          Blas routine to scale a vector
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     IDAMAX         Blas subroutine defining the index of the maximum
!                    element in an array
!     PR2013         Compute minimum and maximum co-ordinates
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2147   Parameter out of range
!    2350   Plane is not defined by z = constant
!    2351   Plane is not defined by x = constant
!    2352   Plane is not defined by y = constant
!    2353   Plane is perpendicular to x-y plane
!    2354   Plane is perpendicular to y-z plane
!    2355   Plane is perpendicular to x-z plane
!    2356   Norm of tangential vector is 0
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ind /2,3,1,2/
! ======================================================================
!
      call eropen ( 'intersect3dch1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intersect3dch1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute xmin, xmax, . . . , zmax and epsacc

      call pr2013 ( coor, npoint, 3, xmin, xmax, ymin, ymax, &
                    zmin, zmax, d )

      epsacc = epsmac * d * 100

!     --- Compute tangential vectors to plane
!         These vectors are stored temporarily in the following arrays t1, t2

      if ( ichoice_tang==0 ) then

!     --- ichoice_tang = 0, compute in default way
!         Find k corresponding to largest |plane(kdir)|

         kdir = idamax(3,plane,1)

!        --- Fill normed vector t1

         anorm = sqrt( plane(kdir)**2 + plane(ind(kdir))**2 )
         t1(kdir) = -plane(ind(kdir))/anorm
         t1(ind(kdir)) = plane(kdir)/anorm
         t1(ind(kdir+1)) = 0d0

!        --- Fill pre-tangential vector t2

         t2(kdir) = -plane(ind(kdir+1))
         t2(ind(kdir)) = 0d0
         t2(ind(kdir+1)) = plane(kdir)

!        --- Compute factor alpha such that t2 = alpha t1 + t2 is perpendicular
!            to t1, and recompute t2

         alpha = -t1(kdir) * t2(kdir)
         call daxpy ( 3, alpha, t1, 1, t2, 1 )

!        --- Normalize t2

         anorm = dnrm2 ( 3, t2, 1 )
         call dscal ( 3, 1d0/anorm, t2, 1 )

!        --- Fill array trans with transformation factors for ksi and eta

         do i = 1,3
            trans(i,1) = t1(i)
            trans(i,2) = t2(i)
         end do

      else if ( ichoice_tang==1 ) then

!     --- ichoice_tang = 1, tangential vectors along x and y-axis

         trans(1,1) =  1d0
         trans(2,1) =  0d0
         trans(3,1) =  0d0
         trans(1,2) =  0d0
         trans(2,2) =  1d0
         trans(3,2) =  0d0
         if ( abs(plane(1))>epsacc .or. abs(plane(2))>epsacc ) then

!        --- This is not the plane z = constant

            call erreal ( plane(1), 1 )
            call erreal ( plane(2), 2 )
            call errsub ( 2350, 0, 2, 0 )

         end if
         kdir = 3

      else if ( ichoice_tang==2 ) then

!     --- ichoice_tang = 2, tangential vectors along y and z-axis

         trans(1,1) =  0d0
         trans(2,1) =  1d0
         trans(3,1) =  0d0
         trans(1,2) =  0d0
         trans(2,2) =  0d0
         trans(3,2) =  1d0
         if ( abs(plane(2))>epsacc .or. abs(plane(3))>epsacc ) then

!        --- This is not the plane x = constant

            call erreal ( plane(2), 1 )
            call erreal ( plane(3), 2 )
            call errsub ( 2351, 0, 2, 0 )

         end if
         kdir = 1

      else if ( ichoice_tang==3 ) then

!     --- ichoice_tang = 3, tangential vectors along x and z-axis

         trans(1,1) =  1d0
         trans(2,1) =  0d0
         trans(3,1) =  0d0
         trans(1,2) =  0d0
         trans(2,2) =  0d0
         trans(3,2) =  1d0
         if ( abs(plane(1))>epsacc .or. abs(plane(3))>epsacc ) then

!        --- This is not the plane y = constant

            call erreal ( plane(1), 1 )
            call erreal ( plane(3), 2 )
            call errsub ( 2352, 0, 2, 0 )

         end if
         kdir = 2

      else if ( ichoice_tang==4 ) then

!     --- ichoice_tang = 4, tangential vectors in the direction of x and y-axis
!         mapped onto plane

         if ( abs(plane(3))<=epsmac ) then

!        --- There is no z-component in the plane

            call erreal ( plane(3), 1 )
            call errsub ( 2353, 0, 1, 0 )

         else

!        --- Map onto plane
!            First store normal in t2

            anorm = 1d0/sqrt(plane(1)**2+plane(2)**2+plane(3)**2)
            t2(1) = plane(1)*anorm
            t2(2) = plane(2)*anorm
            t2(3) = plane(3)*anorm

!           --- Map first tangential vector t1 = e_1 - (e_1,n ) n

            t1(1) =  1d0-t2(1)*plane(1)
            t1(2) =  0d0-t2(1)*plane(2)
            t1(3) =  0d0-t2(1)*plane(3)
            anorm = 1d0/sqrt(t1(1)**2+t1(2)**2+t1(3)**2)
            trans(1,1) = t1(1)*anorm
            trans(2,1) = t1(2)*anorm
            trans(3,1) = t1(3)*anorm

!           --- Map second tangential vector t2 = e_2 - (e_2,n ) n

            t1(1) =  0d0-t2(2)*t2(1)
            t1(2) =  1d0-t2(2)*t2(2)
            t1(3) =  0d0-t2(2)*t2(3)
            anorm = 1d0/sqrt(t1(1)**2+t1(2)**2+t1(3)**2)
            trans(1,2) = t1(1)*anorm
            trans(2,2) = t1(2)*anorm
            trans(3,2) = t1(3)*anorm

         end if
         kdir = 3

      else if ( ichoice_tang==5 ) then

!     --- ichoice_tang = 5, tangential vectors in the direction of y and z-axis
!         mapped onto plane

         if ( abs(plane(1))<=epsmac ) then

!        --- There is no z-component in the plane

            call erreal ( plane(1), 1 )
            call errsub ( 2354, 0, 1, 0 )

         else

!        --- Map onto plane
!            First store normal in t2

            anorm = 1d0/sqrt(plane(1)**2+plane(2)**2+plane(3)**2)
            t2(1) = plane(1)*anorm
            t2(2) = plane(2)*anorm
            t2(3) = plane(3)*anorm

!           --- Map first tangential vector t1 = e_2 - (e_2,n ) n

            t1(1) =  0d0-t2(2)*plane(1)
            t1(2) =  1d0-t2(2)*plane(2)
            t1(3) =  0d0-t2(2)*plane(3)
            anorm = 1d0/sqrt(t1(1)**2+t1(2)**2+t1(3)**2)
            trans(1,1) = t1(1)*anorm
            trans(2,1) = t1(2)*anorm
            trans(3,1) = t1(3)*anorm

!           --- Map second tangential vector t2 = e_3 - (e_3,n ) n

            t1(1) =  0d0-t2(3)*t2(1)
            t1(2) =  0d0-t2(3)*t2(2)
            t1(3) =  1d0-t2(3)*t2(3)
            anorm = 1d0/sqrt(t1(1)**2+t1(2)**2+t1(3)**2)
            trans(1,2) = t1(1)*anorm
            trans(2,2) = t1(2)*anorm
            trans(3,2) = t1(3)*anorm

         end if
         kdir = 1

      else if ( ichoice_tang==6 ) then

!     --- ichoice_tang = 6, tangential vectors in the direction of x and z-axis
!         mapped onto plane

         if ( abs(plane(2))<=epsmac ) then

!        --- There is no y-component in the plane

            call erreal ( plane(2), 1 )
            call errsub ( 2355, 0, 1, 0 )

         else

!        --- Map onto plane
!            First store normal in t2

            anorm = 1d0/sqrt(plane(1)**2+plane(2)**2+plane(3)**2)
            t2(1) = plane(1)*anorm
            t2(2) = plane(2)*anorm
            t2(3) = plane(3)*anorm

!           --- Map first tangential vector t1 = e_1 - (e_1,n ) n

            t1(1) =  1d0-t2(1)*plane(1)
            t1(2) =  0d0-t2(1)*plane(2)
            t1(3) =  0d0-t2(1)*plane(3)
            anorm = 1d0/sqrt(t1(1)**2+t1(2)**2+t1(3)**2)
            trans(1,1) = t1(1)*anorm
            trans(2,1) = t1(2)*anorm
            trans(3,1) = t1(3)*anorm

!           --- Map second tangential vector t2 = e_3 - (e_3,n ) n

            t1(1) =  0d0-t2(3)*t2(1)
            t1(2) =  0d0-t2(3)*t2(2)
            t1(3) =  1d0-t2(3)*t2(3)
            anorm = 1d0/sqrt(t1(1)**2+t1(2)**2+t1(3)**2)
            trans(1,2) = t1(1)*anorm
            trans(2,2) = t1(2)*anorm
            trans(3,2) = t1(3)*anorm

         end if
         kdir = 2

      else if ( ichoice_tang==7 ) then

!     --- ichoice_tang = 7, tangential vectors are explicitly given by the user

         anorm = sqrt(values(8)**2+values(9)**2+values(10)**2)
         if ( anorm<=epsacc ) then

!        --- norm equal to zero

            anorm = 1d0
            call errint ( 1, 1 )
            call erreal ( values(8), 1 )
            call erreal ( values(9), 2 )
            call erreal ( values(10), 3 )
            call errsub ( 2356, 1, 3, 0 )

         else

!        --- norm not equal to zero

            anorm = 1d0/anorm

         end if
         trans(1,1) = values(8)*anorm
         trans(2,1) = values(9)*anorm
         trans(3,1) = values(10)*anorm

         anorm = sqrt(values(11)**2+values(12)**2+values(13)**2)
         if ( anorm<=epsacc ) then

!        --- norm equal to zero

            anorm = 1d0
            call errint ( 2, 1 )
            call erreal ( values(11), 1 )
            call erreal ( values(12), 2 )
            call erreal ( values(13), 3 )
            call errsub ( 2356, 1, 3, 0 )

         else

!        --- norm not equal to zero

            anorm = 1d0/anorm

         end if
         trans(1,2) = values(11)*anorm
         trans(2,2) = values(12)*anorm
         trans(3,2) = values(13)*anorm
         kdir = idamax(3,plane,1)

      else

!     --- ichoice_tang has wrong value

         call errint ( ichoice_tang, 1 )
         call errint ( 0, 2 )
         call errint ( 7, 3 )
         call errchr ( 'ichoice_tang', 1 )
         call errsub ( 2147, 3, 0, 1 )
         kdir = 1

      end if

1000  call erclos ( 'intersect3dch1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intersect3dch1'

      end if  ! ( debug )

      end
