      subroutine intmap2d1c ( nnodes, nunkp, usol2d, usol3d, &
                              itrans, plane, trans, iplane )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-12-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map solution defined in 3d mesh onto 2d intersection
!     The solution in the 3d mesh has a constant number of degrees
!     of freedom per point
!     The mapping of nodes in the intersection is already computed
!     Complex version
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     interpolation
!     intersection
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, nunkp, itrans, iplane
      double precision plane(4), trans(3,3)
      double complex usol2d(nunkp,*), usol3d(nunkp,*)

!     iplane         i    Defines direction of plane
!                         1:  plane x = constant
!                         2:  plane y = constant
!                         3:  plane z = constant
!                         4:  general plane
!     itrans         i    indicates if the quantity to be interpolated
!                         must be transformed or not.
!                         Possible values:
!                         0   No special transformation
!                         1   The first two unknowns are projected in the
!                             plane in the direction of the tangential
!                             vectors, the third one perpendicular to the
!                             plane
!     nnodes         i    Number of nodes in intersection
!     nunkp          i    Number of unknowns in a point (3d mesh)
!     plane          i    Array of length 4 containing the parameters a, b, c
!                         and d describing the plane: ax=by+cz+d = 0
!                         plane(1) = a; plane(2) = b; plane(3) = c; plane(4) = d
!     trans          i    In this array the Transformation matrix from x,y,z
!                         co-ordinates to ksi-eta co-ordinates is stored as
!                         well as the origin in trans(.,3)
!                         The transformation of x,y,z to ksi,eta is defined by
!                         ksi = sum_i trans(i,1) ( x(i) - trans(i,3) )
!                         eta = sum_i trans(i,2) ( x(i) - trans(i,3) )
!     usol2d         o    Solution array corresponding to 2d mesh
!     usol3d         i    Solution array corresponding to 3d mesh
!                         Constant number of degrees of freedom per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, node1, node2
      double complex f1, f2, f

!     f              help variable to store quantity
!     f1             multiplication factor
!     f2             multiplication factor
!     i              Counting variable
!     j              Counting variable
!     node1          first node for interpolation
!     node2          second node for interpolation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intmap2d1c' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intmap2d1c'

      end if  ! ( debug )

      if ( ierror/=0 ) go to 1000

      do i = 1, nnodes
         node1 = imap(i,1)
         node2 = imap(i,2)
         f1 = rmap(i,1)
         f2 = 1d0-f1
         do j = 1, nunkp
            usol2d(j,i) = f1*usol3d(j,node1)+f2*usol3d(j,node2)
         end do  ! j = 1, nunkp
      end do  ! i = 1, nnodes

!     --- transform solution if necessary

      if ( itrans==1 .and. nunkp>=3 ) then

!     --- Perform transformation to tangential and normal components

         if ( iplane==1 ) then

!        --- plane x = constant, sequence 2, 3, 1

            do i = 1, nnodes
               f  = usol2d(1,i)
               usol2d(1,i) = usol2d(2,i)
               usol2d(2,i) = usol2d(3,i)
               usol2d(3,i) = f
            end do  ! i = 1, nnodes

         else if ( iplane==2 ) then

!        --- plane y = constant, sequence 1, 3, 2

            do i = 1, nnodes
               f  = usol2d(2,i)
               usol2d(2,i) = usol2d(3,i)
               usol2d(3,i) = f
            end do  ! i = 1, nnodes

         else if ( iplane>3 ) then

!        --- General case

            do i = 1, nnodes
               f  = usol2d(1,i)*plane(1) + usol2d(2,i)*plane(2) + &
                    usol2d(3,i)*plane(3)
               f1 = usol2d(1,i)*trans(1,1)+usol2d(2,i)*trans(2,1)+ &
                    usol2d(3,i)*trans(3,1)
               f2 = usol2d(1,i)*trans(1,2)+usol2d(2,i)*trans(2,2)+ &
                    usol2d(3,i)*trans(3,2)
               usol2d(1,i) = f1
               usol2d(2,i) = f2
               usol2d(3,i) = f

            end do  ! i = 1, nnodes

         end if  ! ( iplane==1 )

      end if  ! ( itrans==1 .and. nunkp>=3 )

1000  call erclos ( 'intmap2d1c' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intmap2d1c'

      end if  ! ( debug )

      end
