      subroutine prdeldoubleelm ( kmesh2dc, nnodes, nel2d )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-07-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Replace double elements by single elements
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh2dc(*), nnodes, nel2d

!     kmesh2dc      i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!                         At input including double elements
!                         At output with double elements removed
!     nel2d         i/o   Number of elements in 2d mesh
!                         At input including double elements
!                         At output with double elements removed
!     nnodes         i    Number of nodes in mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, ih, icount, istart, iend, ist1, iend1, ih1
      integer, allocatable :: iwork(:), iwork1(:), iwork2(:), iwork3(:)

!     i              Counting variable
!     icount         Counter
!     iend           Last position used at a part
!     iend1
!     ih             Help variable to store a constant temporarily
!     ih1            Help variable to store a constant temporarily
!     ist1
!     istart         First position used at a part
!     iwork          integer work array of length max(nodp, nel2d)
!     iwork1         integer work array of length max(nodp, 3*nel2d)
!     iwork2         integer work array of length max(nodp, nel2d)
!     iwork3         integer work array of length nel2d
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHSORT         Sort integer array for increasing sequence
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3088   option not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     In order to delete double elements array kmesh2dc is numbered in the
!     following sequence
!     Per triangle nodes are numbered from small to large
!     Then the triangles are ordened with increasing first node
!     Triangles with equal first nodal point number are sorted with
!     increasing second node and finally this procedure is repeated
!     for the last node
!     First each triangle is sorted
!     The new numbering is stored in iwork1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prdeldoubleelm' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prdeldoubleelm'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ih = max(nnodes,nel2d)
      ih1 = max(nnodes,3*nel2d)
      allocate ( iwork(ih), iwork1(ih1), iwork2(ih), iwork3(ih), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, ih, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- In order to delete double elements array kmesh2dc is numbered in the
!         following sequence
!         Per triangle nodes are numbered from small to large
!         Then the triangles are ordened with increasing first node
!         Triangles with equal first nodal point number are sorted with
!         increasing second node and finally this procedure is repeated
!         for the last node
!         First each triangle is sorted
!         The new numbering is stored in iwork1

      do i = 1, nel2d
         ih = 3*(i-1)
         call chsort ( kmesh2dc(ih+1), iwork, 3 )
         do k = 1, 3
            iwork1(ih+k) = kmesh2dc(ih+iwork(k))
         end do
      end do

!     --- Sort elements for increasing first node number
!         The new numbering is stored in iwork

      do i = 1, nel2d
         iwork3(i) = iwork1(3*i-2)
      end do
      call chsort ( iwork3, iwork, nel2d )

!     --- Next sort for increasing second node corresponding to constant
!         first node number
!         Store sequence in iwork2

      icount = 0
      istart = 1

!     --- Repeat until end

400      iend = istart
         ih = iwork1(iwork(istart)*3-2)
         do i = istart+1, nel2d
            if ( iwork1(iwork(i)*3-2)/=ih ) goto 420

!           --- iwork1(iwork(i)*3-2) = ih   same first nodal point number

               iend = i

         end do

!        --- The first point differs from the preceding element
!            The elements istart to iend have the same first point, sort
!            the row of second points

420      do i = istart, iend
            iwork3(i) = iwork1(iwork(i)*3-1)
         end do
         call chsort ( iwork3(istart), iwork2, iend-istart+1 )
         do i = 1, iend-istart+1
            iwork3(i+istart-1) = iwork(istart-1+iwork2(i))
         end do
         do i = istart, iend
            iwork(i) = iwork3(i)
         end do

!        --- Finally sort last row of points

         ist1 = istart

!        --- Repeat until end

500         iend1 = ist1
            ih = iwork1(iwork(ist1)*3-1)
            do i = ist1+1, iend
               if ( ih/=iwork1(iwork(i)*3-1) ) goto 520

!              --- ih = iwork1(iwork(i)*3-1)   same second nodal point number

                  iend1 = i

            end do

!           --- The second point differs from the preceding element
!               The elements ist1 to iend1 have the same first and second point, &
!               sort the row of third points

520         do i = ist1, iend1
               iwork3(i) = iwork1(iwork(i)*3)
            end do
            call chsort ( iwork3(ist1), iwork2, iend1-ist1+1 )
            do i = 1, iend1-ist1+1
               iwork3(i+ist1-1) = iwork(ist1-1+iwork2(i))
            end do
            do i = ist1, iend1
               iwork(i) = iwork3(i)
            end do

!           --- Fill the final sequence in array iwork3

            icount = icount + 1
            iwork3(icount) = iwork(ist1)
            ih = iwork1(iwork(ist1)*3)
            do i = ist1+1, iend1
               if ( ih/=iwork1(iwork(i)*3) ) then

!              --- ih # iwork1(iwork(i)*3)   New element found

                  ih = iwork1(iwork(i)*3)
                  icount = icount + 1
                  iwork3(icount) = iwork(i)
               end if
            end do
            ist1 = iend1 + 1
            if ( ist1<=iend ) goto 500

!           --- End of inner loop

         istart = iend + 1
         if ( istart<=nel2d ) goto 400

!        --- End of outer loop
!            Copy kmesh2dc into iwork1

      do i = 1, 3*nel2d
         iwork1(i) = kmesh2dc(i)
      end do

!     --- Finally fill kmesh2dc with the new numbering

      nelgrp => km(iseqkm)%kmesh(5)
      if ( nelgrp==1 ) then

!     --- Standard case: nelgrp = 1

         nel2d = icount
         do i = 1, nel2d
            ih = (i-1)*3
            ih1 = (iwork3(i)-1)*3
            do k = 1, 3
               kmesh2dc(ih+k) = iwork1(ih1+k)
            end do
         end do

      else

!     --- Special case: more element groups

  !       call pr2007ren ( iwork1, kmesh2dc, icount, iwork3 )
   !      nel2d = icount

         call errsub (3088, 0, 0, 0)
         call instop

      end if

      deallocate ( iwork, iwork1, iwork2, iwork3, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prdeldoubleelm'

      end if  ! ( debug )
      call erclos ( 'prdeldoubleelm' )

      end
