      subroutine printerstrian ( nelem2d, markedge, marknode, &
                                 kmeshc2dl, lenkmeshc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 17-11-2006 Make all triangles counterclockwise
!        version  1.0  date 29-06-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create triangular mesh, constructed by intersection of 3d mesh
!     by a plane
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     plane
!     intersection
!     topology
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem2d, markedge(nedges), marknode(npoint), lenkmeshc, &
              kmeshc2dl(3,lenkmeshc)

!     kmeshc2dl      o    kmeshc for 2d mesh, contains nodes of trinagles
!     lenkmeshc      i    length of array kmeshc
!     markedge       i    Array of size nedges
!                         If edge i is not intersected markedge(i) = 0, &
!                         otherwise markedge(i)>0 contains the sequence number
!                         of the new node in the local mesh
!     marknode       i    Array of size npoint
!                         If node i is not on plane marknode(i) = 0, &
!                         otherwise marknode(i)>0 contains the sequence number
!                         of the new node in the local mesh
!     nelem2d        o    Number of elements in local 2d mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, ipstart, ncomplete, nintersect, i, iedge, jpstart, &
              node1, node2, nodes(5), nnodes, j, node3, numedges
      double precision area
      logical found

!     area           Area of element
!     found          If true already a possibility has been found
!     i              Counting variable
!     iedge          edge sequence number
!     ielem          Element number
!     ipstart        Starting address in array
!     j              Counting variable
!     jpstart        Starting address in array
!     ncomplete      Number of edges that re completely in plane
!     nintersect     Number of edges that are intersected by plane
!     nnodes         Number of nodes
!     node1          Node of tetrahedron
!     node2          Node of tetrahedron
!     node3          Node of tetrahedron
!     nodes          Contains the node numbers of the intersections
!     numedges       number of edges
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHAREA        Compute the area of the triangle spanned by x1-x2-x3
!                    The sign of the are defines the direction
!     PRININ         print 1d integer array
!     PRINTERSTRIAN1 Create two triangles from 4 nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2837   The number of intersection points with plane found in tetrahedron
!           is more than 4
!    2838   The number of intersection points with plane found # 3
! **********************************************************************
!
!                       PSEUDO CODE
!
!     nelem2d := 0
!     For all elements ielem in 3d mesh
!        ncomplete := number of edges that are in plane
!        nintersect := number of edges that are intersected by plane
!        if ( ncomplete>=2 ) then
!           nelem2d := nelem2d+1
!           create triangle nelem2d
!        else if ( ncomplete=1 and nintersect=1 ) then
!           nelem2d := nelem2d+1
!           create triangle nelem2d
!        else if ( ncomplete=0 ) then
!           if ( nintersect=3 ) then
!              nelem2d := nelem2d+1
!              create triangle nelem2d
!           else if ( nintersect=4 ) then
!              nelem2d := nelem2d+2
!              create triangles nelem2d and nelem2d-1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printerstrian' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printerstrian'
  1      format ( a, 1x, (10i9) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- nelem2d := 0

      nelem2d = 0

!     --- For all elements ielem in 3d mesh

      do ielem = 1, nelem

         ipstart = nmeshd(ielem)   ! starting address of edges

!        --- ncomplete := number of edges that are in plane
!            nintersect := number of edges that are intersected by plane

         ncomplete = 0
         nintersect = 0
         nnodes = 0
         numedges = nmeshd(ipstart)
         if ( debug ) &
            write(irefwr,1) 'ielem, numedges', ielem, numedges
         do i = 1, numedges

            iedge = abs(nmeshd(ipstart+i))
            if ( debug ) &
               write(irefwr,1) 'i, iedge, markedge(iedge)', &
                                i, iedge, markedge(iedge)
            if ( markedge(iedge)>0 ) then

!           --- edge is intersected by plane, check nodes

               jpstart = nmeshc(iedge)
               node1 = nmeshc(jpstart+1)
               node2 = nmeshc(jpstart+2)
               if ( debug ) then
                  write(irefwr,1) 'node1, node2', node1, node2
                  write(irefwr,1) 'marknode(..) ', &
                                   marknode(node1), marknode(node2)
               end if  ! ( debug )
               if ( marknode(node1)>0 .and. marknode(node2)>0 ) then

!              --- Both nodes are marked, edge is in plane

                  ncomplete = ncomplete+1
                  node1 = marknode(node1)
                  node2 = marknode(node2)
                  found = .false.
                  do j = 1, nnodes
                     if ( node1==nodes(j) ) found = .true.
                  end do  ! j = 1, nnodes
                  if ( .not. found ) then
                     nnodes = nnodes+1
                     nodes(nnodes) = node1
                  end if  ! ( .not. found )
                  found = .false.
                  do j = 1, nnodes
                     if ( node2==nodes(j) ) found = .true.
                  end do  ! j = 1, nnodes
                  if ( .not. found ) then
                     nnodes = nnodes+1
                     nodes(nnodes) = node2
                  end if  ! ( .not. found )

               else

!              --- At most one node is marked, edge is not in plane

                  if ( marknode(node1)>0 ) then

!                 --- node is vertex, check if already in nodes

                     node1 = marknode(node1)
                     found = .false.
                     do j = 1, nnodes
                        if ( node1==nodes(j) ) found = .true.
                     end do  ! j = 1, nnodes
                     if ( .not. found ) then
                        nnodes = nnodes+1
                        nintersect = nintersect+1
                        nodes(nnodes) = node1
                     end if  ! ( .not. found )
                     if ( debug ) &
                        write(irefwr,1) 'node1, nnodes', node1, nnodes

                  else if ( marknode(node2)>0 ) then

!                 --- node is vertex, check if already in nodes

                     node2 = marknode(node2)
                     found = .false.
                     do j = 1, nnodes
                        if ( node2==nodes(j) ) found = .true.
                     end do  ! j = 2, nnodes
                     if ( .not. found ) then
                        nnodes = nnodes+1
                        nintersect = nintersect+1
                        nodes(nnodes) = node2
                     end if  ! ( .not. found )
                     if ( debug ) &
                        write(irefwr,1) 'node2, nnodes', node2, nnodes

                  else

!                 --- node is intersection

                     nnodes = nnodes+1
                     nintersect = nintersect+1
                     nodes(nnodes) = markedge(iedge)

                  end if

               end if  ! ( marknode(node1)>0 .and. marknode(node2)>0 )

            end if  ! ( markedge(iege)>0 )

         end do  ! i = 1, 3

         if ( debug ) then
            if ( nnodes>0 ) call prinin ( nodes, nnodes, 'nodes' )
            write(irefwr,1) 'ncomplete, nintersect, nnodes', &
                             ncomplete, nintersect, nnodes
         end if  ! ( debug )

         if ( nnodes>4 ) then

!        --- Error more than 4 nodes in tetrahedron (impossible)

            call errint ( nnodes, 1 )
            call errsub ( 2837, 1, 0, 0 )
            go to 1000

         end if  ! ( nnodes>4 )

         if ( nnodes==4 ) then

!        --- 4 nodes found
!            nelem2d := nelem2d+2
!            create triangles nelem2d and nelem2d-1

            call printerstrian1 ( nelem2d, nodes, kmeshc2dl, lenkmeshc )

         else if ( ncomplete>=2 .or. ncomplete==1 .and. nnodes==3 ) then

!        --- ncomplete>=2, so we must have three edges in plane
!            or ncomplete=1 and nnodes = 3
!            nelem2d := nelem2d+1
!            create triangle nelem2d

            nelem2d = nelem2d+1
            do j = 1, 3
               kmeshc2dl(j,nelem2d) = nodes(j)
            end do  ! j = 1, 3
            if ( debug ) then
               write(irefwr,1) 'nelem2d', nelem2d
               call prinin ( kmeshc2dl(1,nelem2d), 3, 'element/1' )
            end if  ! ( debug )

         else if ( ncomplete==0 ) then

!        --- ncomplete=0

            if ( nintersect==3 ) then

!           --- nintersect=3
!               nelem2d := nelem2d+1
!               create triangle nelem2d

               if ( nnodes/=3 ) then

!              --- Error not 3 nodes in tetrahedron (impossible)

                  call errint ( nnodes, 1 )
                  call errint ( 3, 2 )
                  call errsub ( 2838, 2, 0, 0 )
                  go to 1000

               end if  ! ( nnodes/=3 )

               nelem2d = nelem2d+1
               do j = 1, 3
                  kmeshc2dl(j,nelem2d) = nodes(j)
               end do  ! j = 1, 3
               if ( debug ) then
                  write(irefwr,1) 'nelem2d', nelem2d
                  call prinin ( kmeshc2dl(1,nelem2d), 3, 'element/2' )
               end if  ! ( debug )

            else if ( nintersect==4 ) then

!           --- nintersect=4
!               nelem2d := nelem2d+2
!               create triangles nelem2d and nelem2d-1

               if ( nnodes/=4 ) then

!              --- Error not 4 nodes in tetrahedron (impossible)

                  call errint ( nnodes, 1 )
                  call errint ( 4, 2 )
                  call errsub ( 2838, 2, 0, 0 )
                  go to 1000

               end if  ! ( nnodes/=3 )

               call printerstrian1 ( nelem2d, nodes, kmeshc2dl, lenkmeshc )

            end if  ! ( nintersect==3 )

         end if  ! ( ncomplete>=3 ... )

      end do  ! ielem = 1, nelem

!     --- Make all triangles counterclockwise

      do ielem = 1, nelem2d

!     --- Find three nodes

         node1 = kmeshc2dl(1,ielem)
         node2 = kmeshc2dl(2,ielem)
         node3 = kmeshc2dl(3,ielem)

!        --- Compute direction

         call msharea ( coor(1,node1), coor(1,node2), &
                        coor(1,node3), area )

         if ( area<0d0 ) then

!        --- direction negative, reverse sequence

            kmeshc2dl(2,ielem) = node3
            kmeshc2dl(3,ielem) = node2

         end if  ! ( area<0d0 ) then

      end do  ! ielem = 1, nelem2d

1000  call erclos ( 'printerstrian' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printerstrian'

      end if  ! ( debug )

      end
