      subroutine printerstrian1 ( nelem2d, nodes, kmeshc2dl, lenkmeshc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-12-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create two triangles from 4 nodes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     plane
!     intersection
!     topology
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem2d, nodes(*), lenkmeshc, kmeshc2dl(3,lenkmeshc)

!     kmeshc2dl      i/o   Array kmesh part c for new 2d mesh
!                         Contains node numbers of elements (topology)
!     lenkmeshc      i    length of array kmeshc
!     nelem2d       i/o   Number of elements in 2d local mesh
!     nodes          i    Contains the 4 node numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer node1, node2, node3, node4, j, nodsort(4), ihelp(4)
      double precision eps, fact1, fact2

!     eps            Local accuracy
!     fact1          multiplication factor w.r.t. intersection
!     fact2          multiplication factor w.r.t. intersection
!     ihelp          integer work array to be used in reordering
!     j              Counting variable
!     node1          First node
!     node2          Second node
!     node3          Third node
!     node4          Fourth node
!     nodsort        integer work array to be used in reordering
!                    Contains new sequence of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHSORT         Sort integer array for increasing sequence
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCROSSLINE   Computes the intersection of two lines
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printerstrian1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printerstrian1'
         call prinin ( nodes, 4, 'nodes' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Sort nodes in ascending order

      call chsort ( nodes, nodsort, 4 )
      do j = 1, 4
         ihelp(j) = nodes(nodsort(j))
      end do  ! j = 1, 4
      do j = 1, 4
         nodes(j) = ihelp(j)
      end do  ! j = 1, 4

      if ( debug ) then
         call prinin ( nodes, 4, 'nodes/reordered' )
         call prinrl ( coor(1,nodes(1)), 2, 'coor/1' )
         call prinrl ( coor(1,nodes(2)), 2, 'coor/2' )
         call prinrl ( coor(1,nodes(3)), 2, 'coor/3' )
         call prinrl ( coor(1,nodes(4)), 2, 'coor/4' )
      end if  ! ( debug )

!     --- The first triangle is simple (1-2-3)

      nelem2d = nelem2d+1
      do j = 1, 3
         kmeshc2dl(j,nelem2d) = nodes(j)
      end do  ! j = 1, 3
      if ( debug ) then
         write(irefwr,1) 'nelem2d', nelem2d
         call prinin ( kmeshc2dl(1,nelem2d), 3, 'element/3' )
      end if  ! ( debug )

!     --- The next triangle may be (1-3-4) or (2-3-4)
!         This depends on whether 1-4 crosses 2-3
!         or 1-3 crosses 2-4 or 3-4 crosses 1-2

      node1 = nodes(1)
      node2 = nodes(2)
      node3 = nodes(3)
      node4 = nodes(4)

      eps = 1d-5
      call mshcrossline ( coor(1,node1), coor(1,node4), &
                          coor(1,node3), coor(1,node2), &
                          fact1, fact2, eps )
      if ( debug ) write(irefwr,2) 'fact1, fact2 ', fact1, fact2
      if ( fact1>0d0 .and. fact1<1d0 ) then

!     --- 0<fact1<1, hence 1-4 crosses 2-3

         nelem2d = nelem2d+1
         kmeshc2dl(1,nelem2d) = node2
         kmeshc2dl(2,nelem2d) = node4
         kmeshc2dl(3,nelem2d) = node3

      else

!     --- 1-4 does not cross 2-3, check if 2-4 crosses 1-3

         call mshcrossline ( coor(1,node2), coor(1,node4), &
                             coor(1,node3), coor(1,node1), &
                             fact1, fact2, eps )
         if ( debug ) write(irefwr,2) 'fact1, fact2 ', fact1, fact2
         if ( fact1>0d0 .and. fact1<1d0 ) then

!        --- 0<fact1<1, hence 2-4 crosses 1-3

            nelem2d = nelem2d+1
            kmeshc2dl(1,nelem2d) = node1
            kmeshc2dl(2,nelem2d) = node3
            kmeshc2dl(3,nelem2d) = node4

         else

!        --- no intersection, hence 1-2 crosses 4-3

            nelem2d = nelem2d+1
            kmeshc2dl(1,nelem2d) = node1
            kmeshc2dl(2,nelem2d) = node2
            kmeshc2dl(3,nelem2d) = node4

         end if  ! ( fact1>0d0 .and. fact1<1d0 )

      end if  ! ( fact1>0d0 .and. fact1<1d0 )
      if ( debug ) then
         write(irefwr,1) 'nelem2d', nelem2d
         call prinin ( kmeshc2dl(1,nelem2d), 3, 'element/4' )
      end if  ! ( debug )

1000  call erclos ( 'printerstrian1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printerstrian1'

      end if  ! ( debug )

      end
