      subroutine printvect ( inppri, ivectr, icurvs, text, rinpri )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 30-11-2012 Long integers
!        version  1.1  date 21-09-2012 Allow nnodes = 0
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the values of a SEPRAN solution vector or vector of special
!     structure
!     The printing may be reduced to printing along curves or surfaces
!     If necessary the tangential or normal component is printed
!     This subroutine replaces and extends the old SEPRAN subroutines PRINCR,
!     PRINOV, PRINTV, printvect
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inppri(*), ivectr, icurvs(*)
      double precision rinpri(*)
      character (len=*) text

!     icurvs   i  Integer array containing the user points, curve or
!                 surface numbers that must be printed.
!                 Pos. 1 indicates how array icurvs is filled.
!                 Possible values:
!                  0   only one curve is used. The curve number is stored in
!                      icurvs(2)
!                 -1   The solution is printed along the curves
!                      icrv1 to icrv2. icrv1 is stored in icurvs(2) and
!                      icrv2 in icurvs(3).
!                 -2   See -1, however, the sequence of the printing is
!                      reversed. In case of curves this means that the
!                      curves are printed from the end point of icrv2 to
!                      the initial point of icrv2, then the end point of
!                      icrv2-1 to the begin point of icrv2-1 and so on.
!                      In case of user points or surfaces ichois=-2 is
!                      treated as ichois=-1
!                 n>0  The curve numbers numbers are stored in
!                      icurvs(2) to icurvs(n+1). That sequence is used.
!                 Remark: a negative curve number means that the curve is
!                      printed from end point to initial point, hence in
!                      reversed order. Negative user point or surface numbers
!                      are not yet available.
!                      Everywhere where curves are mentioned, either user
!                      points, curves or surfaces are meant.
!     inppri  i   Integer array in which the user indicates which quantities
!                 must be printed by subroutine printvect
!                 The entries of INPPRI must be filled (by the user) as follows
!                 If the user sets a value equal to 0, the default value is used
!                 1:  Number of entries filled by the user
!                 2:  ICHOICE, indicates the type of print
!                     Choice parameter consisting of 3 parts:
!                     ichoice_print, isuppress and suppress_zeros according to
!                     ichoice = ichoice_print+10*isuppress+100*suppress_zeros
!                     with:
!                     ichoice_print
!                     0:   The nodal point number is printed followed by the
!                          values in the nodal point
!                     1:   The nodal point number is printed followed by the
!                          coordinates and then the values in the nodal point
!                     2:   The nodal point number is printed followed by the
!                          coordinates
!                     3:   See 2, however, preceded by the number of nodal
!                          points to be printed
!                     isuppress:
!                     if 1 the nodal point number is suppressed if 0 not
!                     suppress_zeros
!                     if 1 points in which the solution is zero are skipped,
!                     if 0 not
!                     Default value: 0
!                 3:  IDIM, indicates which quantities must be printed.
!                     Possible values:
!                     0: The complete vector is printed
!                     1: The vector is only printed in user points.
!                        These user points must be defined in array ICURVS
!                     2: The vector is only printed in user curves.
!                        These curves must be defined in array ICURVS
!                     3: The vector is only printed in user surfaces.
!                        These surfaces must be defined in array ICURVS
!                     4: The vector is only printed in user volumes.
!                        These volumes must be defined in array ICURVS
!                     Default value: 0
!                 4:  IQUANTITY, indicates which quantity must be printed
!                     Possible values:
!                     0:  The contents of the vector are printed as stored in
!                         the vector corresponding to IVECTR i.e. without
!                         transformation.
!                     1:  The normal component of a vector along the curve
!                         or surface is printed. (only if IDIM>1)
!                     2:  The tangential component(s) of a vector along the
!                         curve or surface is printed. (only if IDIM>1)
!                     Default value: 0
!                 5:  NUMBER, indicates which degree of freedom in each point
!                     must be printed. This parameter must be seen in connection
!                     with IQUANTITY.
!                     Possible values:
!                     0:  If iquantity=0, all degrees of freedom are printed
!                         in the nodes defined by icurvs, &
!                         if iquantity>0, the vector from which the normal or
!                         tangential components must be computed is supposed
!                         to be a n-dimensional vector with first degree of
!                         freedom 1, second degree of freedom 2 and third
!                         degree of freedom 3. In this case it is necessary
!                         that IDIM = NDIM (dimension of space)
!                     i>0: If iquantity=0, the degree of freedom i is printed
!                         if iquantity>0, the vector from which the normal or
!                         tangential components must be computed is supposed
!                         to be a n-dimensional vector with first degree of
!                         freedom i, second degree of freedom i+1 and third
!                         degree of freedom i+2. In this case it is necessary
!                         that IDIM = NDIM (dimension of space)
!                     Default value: 0
!                 6:  ISEQ, indicates the sequence in which the coordinates
!                     along the surface must be ordered in case idim=3
!                     Possible values:
!                     1:  increasing x, for fixed x increasing y,
!                         for fixed y increasing z
!                     2:  increasing y, for fixed y increasing z,
!                         for fixed z increasing x
!                     3:  increasing z, for fixed z increasing x,
!                         for fixed x increasing y
!                     4:  sequence of nodal point numbers
!                     Default value: 1
!                 7:  IREGION, indicates if the complete region must be printed
!                     (0) or that the printing is restricted to a sub region,
!                     defined by the parameters xmin, xmax, ymin, ymax, zmin,
!                     zmax in the common block CPLOT.
!                     Possible values:
!                     0:  The complete region is printed
!                     1:  The region is restricted to xmin<=x<=xmax,
!                         ymin<=y<=ymax, zmin<=z<=zmax
!                    -1:  The region is restricted to x=xmin
!                    -2:  The region is restricted to y=ymin
!                    -3:  The region is restricted to z=zmin
!                    -11: The region is restricted to x>=xmin
!                    -12: The region is restricted to y>=ymin
!                    -13: The region is restricted to z>=zmin
!                    -21: The region is restricted to x<=xmin
!                    -22: The region is restricted to y<=ymin
!                    -23: The region is restricted to z<=zmin
!                     Default value: 0
!                 8:  IHEADER, indicates if the header must be printed (0) or
!                     not (1)
!                     Default value: 0
!                 9:  ICOOR, type of coordinate system
!                     1: Cartesian (x,y)
!                     3: Polar (r, phi)
!                     Default value: 1
!                 10: IFORMAT  Format used for printing
!                     1: e13.5
!                     2: e16.8
!                     3: e13.5, one node per line
!                     4: e16.8, one node per line
!                     Default value: 1
!                 11: MINVAL (if 0 no minimum value, if>0 value in rinpri )
!                 12: MAXVAL (if 0 no maximum value, if>0 value in rinpri )
!     ivectr  i   Standard SEPRAN array referring to the vector to be printed
!     rinpri  i    See inppri
!     text    i   Text to be printed as header for the output
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nparm
      parameter (nparm=12)
      integer inpcop(nparm), i, n, dummy(1), npointsav, &
              nunkp, nelectrodes, nchar
      logical compl
      character (len=80) text1
      double precision, pointer :: coornw(:,:)

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!                    they are not
!     coornw         Array containing the co-ordinates of the centroids
!     dummy          Dummy parameter
!     i              Counting variable
!     inpcop         Copy of integer input array extended with defaults
!     n              Number of parts for vectors of type 119
!     nchar          Number of characters in text1
!     nelectrodes    Number of electrodes
!     nparm          Number of parameters for a specific element group
!     npointsav      Help parameter to save the old value of npoint
!     nunkp          Maximum number of unknowns in a point
!     text1          Copy of text with trailing spaces removed
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0087         Fill array with co-ordinates of centroids of elements
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRINCAPVEC     print values of capacity vector (real case)
!     PRININ         print 1d integer array
!     PRSW01         Actual print subroutine for arrays of type 119
!     PRVEC0C        Print vector of special structure defined per element per
!                    integration point (real case)
!     PRVEC0R        Print vector of special structure defined per element per
!                    integration point (complex case)
!     PRVECSPC       Print complex vector of special structure of type 126
!     PRVECSPR       Print real vector of special structure of type 126
!     SEPER0075      Print the values of a SEPRAN array along curves or surfaces
! **********************************************************************
!
!                       I/O
!
!    Print of vector
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2490   Printing of vector defined per element along a set of user points
!           curves of surfaces not possible
! **********************************************************************
!
!                       PSEUDO CODE
!
!   initialize array inpcop
!   Fill array inpcop by copying from array inppri
!   if ( vector is of type 119 ) then
!      Call special print subroutine
!   else if ( vector is of type 116 ) then
!      Make help array containing the coordinates of the centroids in
!      the points
!      Fake kmesh by overwriting coordinate array by temporary array
!      and making npoint equal to nelem
!      Call standard print subroutine
!      Reset kmesh
!   else
!      Standard print
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printvect' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printvect'
         call prinin ( ivectr, 5, 'ivectr' )
         call prinin ( inppri, inppri(1), 'inppri' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Put string in text1

      text1 = adjustl(text)   ! remove leading spaces

!     --- Compute last non blank character

      nchar = len_trim(text)

!     --- Fill array inpcop with defaults

      inpcop = 0
      inpcop(1) = 1
      do i = 1, min(inppri(1),nparm)
         inpcop(i) = inppri(i)
      end do
      if ( inpcop(6)==0 ) inpcop(6) = 1
      if ( inpcop(9)==0 ) inpcop(9) = 1
      if ( inpcop(10)==0 ) inpcop(10) = 1

      if ( ks(ivectr)%typevec==116 ) then

!     --- Vectors of special structure defined per element
!         In order to apply the standard case the subroutine is faked
!         npoint is made equal to nelem
!         coor is replaced by an array containing the coordinates
!         of the centroids of the elements

         if ( inpcop(3)>0 ) then

!        --- idim>0, i.e. print user points, curves or surfaces
!            This is not possible for element-wise storage

            call errsub (2490, 0, 0, 0 )
            go to 1000

         end if

         allocate ( coornw(ndim,nelem), stat = error )
         if ( error/=0 ) call eralloc ( error, ndim*nelem, 'coornw' )
         if ( ierror/=0 ) go to 1000

!        --- Store coordinates of centroid in coor_new

         call er0087 ( coornw )

!        --- Actual printing

         coor => coornw
         npointsav = npoint
         npoint = nelem
         call seper0075 ( inpcop, ivectr, icurvs, text1(1:nchar), rinpri )

!        --- Remove temporary array and reset kmesh

         deallocate ( coornw, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'coornw' )
         if ( ierror/=0 ) go to 1000
         npoint = npointsav
         coor => km(iseqkm)%coor

      else if ( ks(ivectr)%typevec==119 ) then

!     --- Vectors of type 119 require a special treatment

         n = ks(ivectr)%n
         isoltemp => ks(ivectr)%isol
         usoltemp => ks(ivectr)%sol
         compl = ks(ivectr)%compl==1
         call prsw01 ( n, isoltemp, usoltemp, &
                       text1(1:nchar), compl )

      else if ( ks(ivectr)%typevec==126 ) then

!     --- Vectors of special structure defined per element per node

         nunkp = ks(ivectr)%n
         usoltemp => ks(ivectr)%sol
         if ( ks(ivectr)%compl==0 ) then

!        --- real case

            call prvecspr ( usoltemp, nunkp, inpcop, &
                            text1(1:nchar) )

         else

!        --- complex case

            call prvecspc ( usoltemp, nunkp, inpcop, &
                            text1(1:nchar) )

         end if

      else if ( ks(ivectr)%typevec==129 ) then

!     --- Vectors of special structure defined per element per integration point

         isoltemp => ks(ivectr)%isol
         usoltemp => ks(ivectr)%sol
         if ( ks(ivectr)%compl==0 ) then

!        --- real case

            call prvec0r ( usoltemp, inpcop, &
                           text1(1:nchar), isoltemp )

         else

!        --- complex case

            call prvec0c ( usoltemp, inpcop, &
                           text1(1:nchar), isoltemp )

         end if

      else if ( ks(ivectr)%typevec==801 ) then

!     --- Vectors of special structure defined for a series of electrodes

         nelectrodes = ks(ivectr)%ivec
         usoltemp => ks(ivectr)%sol
         nunkp = ks(ivectr)%nusol*2/(nelectrodes*(nelectrodes-1))
         call princapvec ( inpcop, nelectrodes, usoltemp, nunkp, &
                           text1(1:nchar) )

      else

!     --- Standard case

         if ( debug ) write(irefwr,*) 'before er0075'
         call seper0075 ( inpcop, ivectr, icurvs, text1(1:nchar), rinpri )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printvect'

      end if  ! ( debug )
      call erclos ( 'printvect' )

      end subroutine printvect
