      subroutine prinvcbf119 ( isol, numsol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-04-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     print vector of type 119
!
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: numsol, isol(numsol)

!     isol           i    Standard SEPRAN array containing information of the
!                         solution arrays
!     numsol         i    Total number of solution vectors in isol
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, numnodes, numsubcurvs, j, leng1
      double precision, allocatable :: work(:,:)

!     i              Counting variable
!     j              Counting variable
!     leng1          Help parameter to indicate the length of arrays
!     numnodes       Number of nodal points in solution arrays
!     numsubcurvs    Number of subcurves
!     work           Work array to store the solution and coordinates
!                    temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     ERFILL119      Fill array outvec with the information of array ivectr
!                    of type 119
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3179   Not the same length for all vectors
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinvcbf119' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinvcbf119'
         write(irefwr,1) 'numsol', numsol
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate arrays

      do i = 1, numsol
      end do  ! i = 1, numsol
      if ( ierror/=0 ) go to 1000

!     --- Check if they all have the same length

      isoltemp => ks(isol(1))%isol
      leng1 = size ( isoltemp )  ! length of array

      do i = 2, numsol
         isoltemp => ks(isol(i))%isol
         if ( leng1/=size ( isoltemp ) ) then

!        --- Error not the same number of nodes in the various arrays

            call errsub ( 3179, 0, 0, 0 )
            go to 1000

         end if  ! ( leng1/=size ( isoltemp) )
      end do  ! i = 1, numsol

!     --- Compute starting address as well as some extra information

      numsubcurvs = ks(isol(1))%n   ! number of subcurves
      isoltemp => ks(isol(1))%isol
      numnodes = sum(isoltemp(1:numsubcurvs)) ! number of nodes

!     --- Copy vectors in temporary vector work

      allocate ( work(numnodes,numsol+1), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, numnodes*(numsol+1), 'work' )
      if ( ierror/=0 ) go to 1000

      call erfill119 ( isol, numsol, numsubcurvs, work, numnodes )

!     --- Print vector

      write(irefwr,*) 'coordinate        values'
      do i = 1, numnodes
         write(irefwr,100) (work(i,j), j=1,numsol+1)
      end do  ! i = 1, numnodes
100   format ( 40d13.5 )

!     --- remove temporary array

      deallocate ( work, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'work' )
      if ( ierror/=0 ) go to 1000

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinvcbf119'

      end if  ! ( debug )
      call erclos ( 'prinvcbf119' )

      end subroutine prinvcbf119
