      subroutine prlin1 ( fval, coor, knodp, nelem, npoint, angdeg, &
                          orgx, orgy, nseg, kseg, xcint, fcint, ncint )
! ======================================================================
!
!        programmer    Jos van Kan
!        version  3.0  date 14-10-2016 New parameter list
!        version  2.5  date 09-06-2010 Adapt coorindates
!        version  2.4  date 09-03-1995 Restore coordinates after a warning
!
!   copyright (c) 1988-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  prlin1 is a SEPRAN subroutine to do the following:
!  Given a mesh consisting of triangles with only tree nodal points
!  and function values (one only) in those nodal points, to calculate
!  a set of interpolated values along a given line.
!  The angle of that line with respect to the x-axis is given
!  by angdeg in degrees. The new origin by (orgx, orgy).
!  The output is organized as follows:
!  any given line may consist of a number of separate SEGMENTS, for
!  example if an L-shaped region is cut like this:
!             |---\-----
!             |    \   |
!             |     \  |
!             |      \ |
!             |       \|
!             |        \
!             |        |\
!             |        | \
!             |        ---\----|
!             |            \   |
!             |             \  |
!             |______________\__
!  the line consists of two segments. This number of segments is stored
!  in nseg. The number of interpolated points per segment is stored in
!  kseg (j), in which j is increased by one for each new segment. The
!  coordinate system is rotated over angdeg degrees, and shifted, so that
!  intersection line is y = 0 in the new coordinates.
!  The (rotated and shifted) x-coordinates of interpolation are stored in
!  xcint, the corresponding interpolated function values in fcint.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nseg, nelem, npoint, ncint, &
              knodp(3,nelem), kseg(*)
      double precision fval(npoint), coor(2,npoint), xcint(ncint), &
             fcint(ncint),  orgx, orgy, angdeg

!     angdeg   i      angle of rotation in degrees
!     coor     i      nodal coordinates
!     fcint    o      array of interpolated function values
!     fval     i      function values in nodes
!     knodp    i      nodal points of elements
!     kseg     o      number of points per segment
!     ncint    o      declared length of fcint and xcint (must be
!                     larger than sum kseg (j))
!     nelem    i      number of elements
!     npoint   i      number of nodal points
!     nseg     o      number of segments per line
!     orgx     i      x-coordinate of new origin
!     orgy     i      y-coordinate of new origin
!     xcint    o      array of interpolated x- values
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, niout, nhelp, iout(nelem)
      double precision slevel, eps, shift

!     eps            Local accuracy
!     i              Counting variable
!     iout           Integer work array of length nelem
!                    array containing element numbers of elements that
!                    intersect this line.
!     nhelp          Help variable to store a constant
!     niout          number of elements in iout.
!     shift          shift in coordinate in xcint
!     slevel         value of y coordinate to intersect the elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     ERRWAR    Print warnings
!     PRELI1    calculates which elements intersect this line. (unsorted
!               element field)
!     PRGATH    removes doubles and sorts.
!     PRROTA    rotate coordinates
!     PRSEGI    what are the intersection points?
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     752
!     1827
! **********************************************************************
!
!                       PSEUDO CODE
!
!       Algorithm
!     The coordinate system is first shifted to orgx, orgy and rotated
!     over angdeg degrees.
!     1. The intersection points (if any) of the elements with the given
!        line are calculated, together with the interpolated function
!        values. All internal points now will be double, boundary
!        points single.
!     2. These points are sorted and the double points removed. The number
!        of segments per line and points per segment are stored in nseg and
!        kseg respectively.
!     The coordinate system is rotated back to clean up the mess.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prlin1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prlin1'
         write(irefwr,2) 'orgx, orgy, angdeg', orgx, orgy, angdeg
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- rotate over angdeg degrees
!         Shift origin

      coor ( 1,1:npoint ) = coor ( 1,1:npoint ) - orgx
      coor ( 2,1:npoint ) = coor ( 2,1:npoint ) - orgy

      if ( angdeg/=0d0 ) call prrota (coor, angdeg, npoint)
      slevel = 0d0

!     --- calculate which elements intersect this line

      call preli1 ( slevel, knodp, nelem, coor, iout, niout )
      if ( niout==0 ) then

!     --- No intersection found, warning and return

         call errwar ( 752, 0, 0, 0 )
         nseg = 0
         go to 200

      else if ( 2*niout>ncint ) then

!     --- 2*niout>ncint

         call errint ( 2*niout, 1 )
         call errint ( ncint, 2 )
         call errsub ( 1827, 2, 0, 0 )
         go to 200

      end if

!     --- calculate in which points this line intersects and interpolate
!         function values

      call prsegi ( slevel, coor, fval, knodp, iout, niout, &
                    xcint, fcint )

!     --- sort the points, remove the doubles and partition into segments

      nhelp = 2*niout
      call prgath ( xcint, fcint, nseg, kseg, nhelp )

!     --- now rotate back the coordinate system

200   if ( angdeg/=0d0 )call prrota (coor, -angdeg, npoint)

!     --- And back shift the origin

      coor ( 1,1:npoint ) = coor ( 1,1:npoint ) + orgx
      coor ( 2,1:npoint ) = coor ( 2,1:npoint ) + orgy

      eps = 1d-3
      if ( abs(angdeg)<eps ) then

!     --- Angle = 0, use x coordinates, i.e. add orgx to array xcint

         shift = orgx

      else if ( abs(angdeg-90d0)<eps ) then

!     --- Angle = 90, use y coordinates, i.e. add orgy to array xcint

         shift = orgy

      else

!     --- Other angle, no shift

         shift = 0d0

      end if  ! ( abs(angdeg)<eps )

      if ( debug ) write(irefwr,2) 'shift', shift

      if ( abs(shift)>0d0 ) then

!     --- Shift array xcint

         nhelp = 0
         do i = 1, nseg
            nhelp = nhelp+kseg(i)
         end do  ! i = 1, nseg
         xcint(1:nhelp) = xcint(1:nhelp)+shift

      end if  ! ( abs(shift)>0d0 )

1000  call erclos ( 'prlin1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prlin1'

      end if  ! ( debug )

      end
