      subroutine prmatlabsol ( isol, filename )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 20-06-2013 New definition file ref numbers
!        version  1.2  date 30-11-2012 Long integers
!        version  1.1  date 15-02-2010 Extension with complex vectors
!
!   copyright (c) 2008-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print solution vector in a form suitable for matlab
!     The vector is written with a constant number of unknowns per node
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     real
!     print
!     matlab
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol
      character (len=*) filename

!     filename       i    Name of file to which the solution must be written
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array that must be written
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iref
      logical compl

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     iref           File reference number
!     usol           solution array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inisetnewref

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
!     PRCOPYVECS2    Copy the contents of vector uold into vector unew
!                    with structure ivec
!     PRTOMATLAB     Write solution stored in columns per degree of freedom
!                    to Matlab input file
!     PRTOMATLABC    Write complex solution stored in columns per degree of
!                    freedom to Matlab input file
!     SEPACTSOLBF2   compute pointers to degrees of freedom of vectors
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Open file
!     Detect number of unknowns per point
!     If ( number of unknowns per point constant ) then
!        print vector to file
!     else
!        Copy vector to new vector with constant number of unknowns per point
!        print new vector to file
!        delete new vector
!     Close file
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmatlabsol' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmatlabsol'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get constants

      compl = ks(isol)%compl==1

!     --- Open file

      iref = inisetnewref()
      open ( unit=iref, file = filename, form = 'formatted')

!     --- Detect number of unknowns per point
!         Activate arrays

      call sepactsolbf2 ( isol )

      if ( indprfi==0 ) then

!     --- number of unknowns per point constant
!         print vector to file

         usol => ks(isol)%sol
         if ( compl ) then

!        --- complex case

            call prtomatlab ( usol, npoint, nunkpi, iref )

         else

!        --- real case

            call prtomatlabc ( usol, npoint, nunkpi, iref )

         end if  ! ( compl )

      else

!     --- number of unknowns per point not constant
!         Copy vector to new vector with constant number of unknowns per point
!         print new vector to file
!         delete new vector
!         First the new vector is defined

         length = nunkpi*npoint

         allocate ( usol(length), stat = error )
         if ( error/=0 ) call eralloc ( error, length, 'usol' )
         if ( ierror/=0 ) go to 1000

         usoltemp => ks(isol)%sol  ! old vector

         call prcopyvecs2 ( usoltemp, usol, nunkpi, npoint, kprobfi )  ! copy

         if ( compl ) then

!        --- complex case

            call prtomatlab ( usol, npoint, nunkpi, iref )  ! write

         else

!        --- real case

            call prtomatlabc ( usol, npoint, nunkpi, iref )  ! write

         end if  ! ( compl )


         deallocate ( usol, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'usol' )
         if ( ierror/=0 ) go to 1000

      end if  ! ( indprf==0 )

!     --- Close file

      close ( iref )
      call inirmref ( iref )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmatlabsol'

      end if  ! ( debug )
      call erclos ( 'prmatlabsol' )

      end subroutine prmatlabsol
