      subroutine prreducework ( work, numnodes, numsol, reduction, &
                                inpsep, rinsep, values, ientry )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-05-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if end point of line is given and if so reduce array work
!     to part defined by this length if length smaller than original length
! **********************************************************************
!
!                       KEYWORDS
!
!     reduction
!     length
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: numnodes, inpsep(*), numsol
      integer, intent(out) :: ientry
      double precision, intent(in) :: rinsep(*), values(*)
      double precision, intent(inout) :: work(numnodes,numsol+1)
      logical, intent(out) :: reduction

!     ientry         o    The last entry of work that is used
!     inpsep         i    The integer input read is stored in array INPSEP
!                         Only positions 8-10 are used in this subroutine
!                         See subroutine readscintersin for a description
!     numnodes       i    Number of nodes in work
!     numsol         i    Total number of components stored in owrk
!     reduction      o    If true array work is reduced
!     rinsep         i    Array containing the real input for this subroutine
!                         See subroutine readscintersin for a description
!     values         i    Double precision or complex array in which the real
!                         or complex information concerning the line is stored
!                         See subroutine readscprinint (rinsep)
!     work          i/o   Array to be adapted
!                         (.,1) the x-coordinates
!                         (.,2..) the values
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: lenline, dx, eps, x1, x2, fact1, fact2

!     dx             locla distance
!     eps            Local accuracy
!     fact1          multiplication factor
!     fact2          multiplication factor
!     lenline         Help parameter to indicate the length of arrays
!     x1             x-coordinate of point
!     x2             x-coordinate of next point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision :: prlengthline

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRSUB         Error messages
!     PRINRL2        Print a real two-dimensional array
!     PRLENGTHLINE   Compute length of line with origin (o_x,o_y), angle and
!                    end point(e_x,e_y)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3185   length less than zero
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prreducework' )
      debug = .true. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prreducework'
         write(irefwr,1) 'numnodes', numnodes
         call prinrl2 ( work, numnodes, numsol, 'work' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      lenline = prlengthline ( inpsep, rinsep, values )
      if ( ierror/=0 ) go to 1000

      dx = (work(numnodes,1)-work(1,1))/numnodes
      eps = 1d-3*dx

      ientry = numnodes

      if ( debug ) write(irefwr,2) 'lenline, dx, eps', lenline, dx, eps

      if ( work(numnodes,1)<=lenline ) then

!     --- lenline is larger than interval, no reduction

          reduction = .false.

      else

!     --- lenline is smaller than interval, reduction
!         Find first entry larger than lenline

         ientry = 1
         do while ( work(ientry,1)<=lenline )
            ientry = ientry+1
         end do  ! while ( work(ientry,1)<=lenline )
         if ( ientry==1 ) then

!        --- Error ientry = 1, hence lenline must be zero

            call erreal ( lenline, 1 )
            call errsub ( 3185, 0, 1, 0 )
            reduction = .false.
            go to 1000

         else if ( (abs(work(ientry,1)-lenline))<eps ) then

!        --- Use ientry as end point and do not interpolate

            reduction = .true.

         else if ( (abs(work(ientry-1,1)-lenline))<eps ) then

!        --- Use ientry-1 as end point and do not interpolate

            ientry = ientry-1
            reduction = .true.

         else

!        --- Use ientry as end point and interpolate

            x1 = work(ientry-1,1)
            x2 = work(ientry,1)
            dx = x2-x1
            fact2 = (lenline-x1)/dx
            fact1 = (x2-lenline)/dx
            work(ientry,1) = lenline
            work(ientry,2:numsol+1) = fact1*work(ientry-1,2:numsol+1)+ &
                                      fact2*work(ientry,2:numsol+1)
            reduction = .true.

         end if  ! ( ientry==1 )

      end if  ! ( work(numnodes)<=lenline )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ientry', ientry
         write(irefwr,*) 'End prreducework'

      end if  ! ( debug )
      call erclos ( 'prreducework' )

      end
