      subroutine sepcompsr ( ichoice, &
                             ivectr, number, isurfs, funcx, funcy, nodeslc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-06-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute degrees of freedom along surfaces and store in array funcy
!     the corresponding co-ordinates are stored in funcx
!     The corresponding nodal point numbers are stored in array inodes
!     Old subroutine compsr
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     function_value
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichoice, ivectr, isurfs(*), number, &
              nodeslc(*)
      double precision funcx(*), funcy(*)

!    funcx    o    user array of type 117, where the co-ordinates of
!                  the boundary points with a degree of freedom are stored
!    funcy    o    user array of type 117, where the degrees of freedom
!                  of the boundary points are stored
!    ichoice  i    choice parameter. Possibilities:
!                   0 standard situation
!                  -1 only the co-ordinates of the surfaces are stored in funcx
!    isurfs   i    definition of the surfaces by the user.
!                  position 1 contains the number of surfaces
!                  the next positions contain the surface numbers
!    ivectr   i    array containing information of the function to be
!                  printed
!    nodeslc  o     integer array containing the node numbers corresponding to
!                  the arrays funcx and funcy in the same sequence from
!                  position 6
!                  Pos. 1 must contain the declared length of inodes
!                  Pos. 2 is filled with the number 128
!                  Pos. 3 gets the computed length of this array
!                         Initialize it with 0 at the first call
!                  Pos. 4 - &
!                  Pos. 5 number of nodes in this array
!    number   i    the number th degree of freedom in each nodal point
!                  is stored
!                  when number = 0 all degrees of freedom in the boundary
!                  points are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical compl
      integer m, n, nparts, ipoint(1)
      integer, allocatable :: inodes(:)
      double precision, allocatable :: funcxtemp(:), funcytemp(:)

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     funcxtemp      Work array to store the new contents of funcx
!     funcytemp      Work array to store the new contents of funcy
!     inodes         Work space to store boundaries
!     ipoint         dummy
!     m              number of degrees of freedom per point
!     n              number of nodes
!     nparts         dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ER0040         compute values of vector in nodal points of curves
!                    (real case)
!     ER0041         compute values of vector in nodal points of curves;
!                    complex case
!     ER0042         store co-ordinates of the nodal points stored in icurvs in
!                    array funcx
!     ER00741        Compute number of degrees of freedom per point in a set of
!                    nodes, stored in array inodes
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINLG         Print and checks length of user arrays
!     SEPACTSOLBF2   compute pointers to degrees of freedom of vectors
!     SEPFILLNODES   Fill array nodes with nodal points as defined in array
!                    inpsep
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       1   ICHOIS is incorrect
!     102   NUMBER has wrong value
!     396   number = 0, whereas the number of degrees of freedom in the
!           nodal points is not constant
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Perform some general tests
!     reserve space for arrays
!     Store the nodes along the surface in array work
!     if ( ichoice = -1 ) then
!        fill coordinates in funcx
!     else
!        fill arrays funcx and funcy
!     Put node numbers into inodes
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Perform some general tests

      call eropen ( 'sepcompsr' )
      if ( ichoice<-1 .or. ichoice>0 ) then
         call errint ( ichoice, 1 )
         call errsub ( 1, 1, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000

      if ( ichoice>=0 ) then

         call sepactsolbf2 ( ivectr )
         compl = ks(ivectr)%compl==1
         if ( number<0 .or. number>nphysi )then
            call errint ( number, 1 )
            call errint ( nphysi, 2 )
            call errsub ( 102, 2, 0, 0 )
         end if
      end if
      if ( ierror/=0 ) go to 1000

!     --- reserve space for arrays
!         Define a semi-permanent work array

      allocate ( inodes(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'inodes' )
      if ( ierror/=0 ) go to 1000

!     --- Store the nodes along the surface in array work

      call sepfillnodes ( 3, isurfs(1), isurfs(2), &
                          inodes, n, ipoint, nparts )

      if ( ichoice==-1 ) then

!     --- ichoice = -1  fill co-ordinates

         call prinlg ( 'funcx', -n*ndim-5, funcx, inodes )
         call er0042 (inodes, n, ndim, coor, funcx(6))
         funcx(2) = 117d0
         funcx(4) = ndim
         funcx(5) = n*ndim

      else

!     --- ichoice = 0  fill co-ordinates and function values

         m = 1
         if ( number==0 ) then

!        --- Find and check number of unknowns per point for the surface

            call er00741 ( n, inodes, m )
            nphysi = m
            if ( m<0 ) then
               call errsub ( 396, 0, 0, 0 )
               go to 1000
            end if

         end if
         if ( compl ) m = 2*m

         allocate ( funcxtemp(ndim*n), funcytemp(m*n), stat = error )
         if ( error/=0 ) call eralloc ( error, (ndim+m)*n, 'funcxtemp' )
         if ( ierror/=0 ) go to 1000

         usoltemp => ks(ivectr)%sol

         if ( compl ) then

!        --- complex output vector

            call er0041 ( inodes, n, number, usoltemp, &
                          funcxtemp, funcytemp )

         else

!        --- real output vector

            call er0040 ( inodes, n, number, usoltemp, &
                          funcxtemp, funcytemp )
         end if

!        --- Check length of funcx and funcy

         call prinlg ( 'funcx', -n*ndim-5, funcx, inodes )
         call prinlg ( 'funcy', -n*m-5, funcy, inodes )

!        --- Fill funcx and funcy

         funcx(2) = 117d0
         funcx(4) = ndim
         funcx(5) = n*ndim
         funcy(2) = 117d0
         funcy(4) = m
         funcy(5) = n*m
         funcx(6:5+n*ndim) = funcxtemp(1:n*ndim)
         funcy(6:5+n*m) = funcytemp(1:n*m)

         deallocate ( funcxtemp, funcytemp, stat = error )

      end if

!     --- Copy node numbers into nodes

      call prinlg ( 'inodes', n+5, funcx, nodeslc )
      nodeslc(2) = 128
      nodeslc(5) = n
      nodeslc(6:5+n) = inodes(1:n)

!     --- Delete inodes

      deallocate ( inodes, stat = error )

1000  call erclos ( 'sepcompsr' )
      end subroutine sepcompsr
