      subroutine sepderintcurvs ( list, icurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array list with information about elements for
!     boundary integrals
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: list(3,npoint), icurvs(*)

!     icurvs         i    Integer array containing the curve numbers for the
!                         computation of the boundary integral.
!                         A negative curve number indicates that the curve
!                         must be computed in the reversed direction.
!                         The curves including their sign must form one
!                         contiguous curve
!                         The number of curves is stored in array iinder
!     list           o    Array of length 3*nnodes-1, containing the list of
!                         all admissible elements that have two points with the
!                         curves to be considered in common. For each two points
!                         only one element is found.
!                         The sequence of the elements is list is the same as
!                         the one stored in skipelgrps at output.
!                         list(1,i) contains the i-th element sequence number
!                         list(2,i) contains the first node at the curve for
!                                   this element
!                         list(3,i) contains the second node at the curve for
!                                   this element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nnodes, nparts, inumcv
      integer, allocatable :: ipoint(:)

!     inumcv         Number of curves to be used
!     ipoint         Array of length nparts+1 containing the starting
!                    addresses in array inodes of the various parts
!                    the boundary consists of.
!                    ipoint(1) = 1,
!                    number of nodes at part i = ipoint(i+1)-ipoint(i)
!     nnodes         Number of nodes in array nodes
!     nparts         number of parts the boundary consists of
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLNODES   Fill array nodes with nodal points as defined in array
!                    inpsep
!     TO0101         Make list of element groups and elements at subsequent
!                    curves
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepderintcurvs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepderintcurvs'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( ipoint(npoint), nodes(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'ipoint' )
      if ( ierror/=0 ) go to 1000

!     --- Fill array nodes

      inumcv = iinder(15)   ! number of user points or curves in boundary
      nnodes = inumcv
      call sepfillnodes ( 2, inumcv, icurvs, &
                          nodes, nnodes, ipoint, nparts )

!     --- Construct list_of_element_groups and per element group
!         list_of_elements allowed

      call to0101 ( list, nnodes )

1000  call erclos ( 'sepderintcurvs' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepderintcurvs'

      end if  ! ( debug )

      end subroutine sepderintcurvs
