      subroutine sepintersecpla ( meshin, meshout, plane, maplc, &
                                  ichoicearr, values, iplane )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-05-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Intersect 3d mesh by plane
!     Compute mapping
!     Old subroutine intersectplane
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     plane
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemeshinf
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer meshin, ichoicearr(3), iplane, meshout
      double precision plane(*), values(*)
      type (mapping) :: maplc

!     ichoicearr     i    Array of size 3 containing the three parts of ichoice
!                         1:  ichoice_plane
!                         2:  ichoice_tang
!                         3:  ichoice_orig
!     iplane         i    Defines direction of plane
!                         1:  plane x = constant
!                         2:  plane y = constant
!                         3:  plane z = constant
!                         4:  general plane
!     maplc         i/o   map information
!     meshin         i    Sequence number of 3d mesh
!     meshout        i    Sequence number of 2d mesh
!     plane          i    Array of length 4 containing the parameters a, b, c
!                         and d describing the plane: ax=by+cz+d = 0
!                         plane(1) = a; plane(2) = b; plane(3) = c; plane(4) = d
!     values         i    Array containing information concerning the plane, &
!                         tangential vectors and origin
!                         See subroutine intp3dbf
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseqkm1, iseqkm2
      integer, target :: kmeshnew
      logical subdiv

!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     kmeshnew       New array kmesh (temporary)
!                    Contains a mesh consisting of tetrahedrons only
!     subdiv         If true the elements must be subdivided
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCLEAR       clear array map
!     SEPCRMESHINFO  Create permanent space for mesh
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPINTSEC3DMSH Intersect 3d mesh with a plane
!                    create new 2d mesh consisting of triangles
!                    Compute mapping array from 3d mesh to 2d mesh
!     SEPRMMESHINFO  Remove array with sequence number iseq from km
!     TOPMAPTETRA    Map 3d mesh onto a mesh consisting of linear tetrahedrons
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Map 3d mesh onto a mesh consisting of linear tetrahedrons only
!     Intersect tetrahedron mesh by plane
!     Remove superfluous arrays
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepintersecpla' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepintersecpla'
         write(irefwr,1) 'meshin, meshout', meshin, meshout
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepclear ( maplc )

      kmeshnew = 0
      call sepcrmeshinfo ( kmeshnew )
      iseqkm2 = iseqkm

!     --- Map 3d mesh onto a mesh consisting of linear tetrahedrons only

      call sepfillkmesh(kmesh)
      iseqkm1 = iseqkm
      iseqkm = iseqkm2
      call topmaptetra ( kmeshnew, subdiv )
      if ( debug ) then
         write(irefwr,*) 'subdiv', subdiv
         write(irefwr,1) 'iseqkm1, iseqkm2', iseqkm1, iseqkm2
      end if  ! ( debug )

!     --- Intersect tetrahedron mesh by plane

      if ( subdiv ) then

!     --- subdiv true, i.e. a new mesh has been made
!         Use the new mesh

         kmesh => kmeshnew
         call sepfillkmesh(kmesh)

         call sepintsec3dmsh ( kmesh2d(meshout), plane, maplc, &
                               ichoicearr, values, iplane )

!        --- Remove superfluous arrays
!            Since the coordinates in kmeshnew are the ones used in kmesh
!            that array may not be removed
         call seprmmeshinfo ( kmeshnew )  ! remove kmeshnew
         kmesh => kmesh2d(meshin)
         call sepfillkmesh(kmesh)

      else

!     --- subdiv false, use standard mesh

         call seprmmeshinfo ( kmeshnew )  ! remove kmeshnew
         kmesh => kmesh2d(meshin)
         call sepfillkmesh(kmesh)
         call sepintsec3dmsh ( kmesh2d(meshout), &
                               plane, maplc, ichoicearr, values, iplane )

      end if  ! ( subdiv )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepintersecpla'

      end if  ! ( debug )
      call erclos ( 'sepintersecpla' )

      end subroutine sepintersecpla
