      subroutine sepintmap3d4 ( usol, usolcop, kprobf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-05-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy solution in mesh with non-constant number of unknowns
!     per point to solution array with constant number of unknowns
!     The solution to be copied is not present in all nodes
!     Center nodes of elements and faces are skipped
!     Old subroutine intmap3d4
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     interpolation
!     intersection
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kprobf(*)
      double precision usol(*), usolcop(*)

!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     usol           i    Solution vector to be copied and interpolated
!     usolcop        o    Result vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inpelm, istart, ielgrp

!     i              Counting variable
!     ielgrp         element group number
!     inpelm         number of points per element
!     istart         First position used at a part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INTMAP3D411    Copy solution in mesh with non-constant number of unknowns
!                    per point to solution array with one degree of freedom
!                    The solution to be copied is not present in all nodes
!                    Center nodes of elements and faces are skipped
!                    Elements of shape 11, 13, 17, 18 or 21
!     INTMAP3D412    Copy solution in mesh with non-constant number of unknowns
!                    per point to solution array withone degree of freedom
!                    The solution to be copied is not present in all nodes
!                    Center nodes of elements and faces are skipped
!                    Elements of shape 12, 15 or 16
!     INTMAP3D414    Copy solution in mesh with non-constant number of unknowns
!                    per point to solution array withone degree of freedom
!                    The solution to be copied is not present in all nodes
!                    Center nodes of elements and faces are skipped
!                    Elements of shape 14
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Set output array to zero
!     For all elements do
!        For all unknowns do
!           for all non-center points do
!              if ( unknown available ) then
!                 copy
!              else
!                 interpolate
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepintmap3d4' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepintmap3d4'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set output array to zero

      do i = 1, npoint
         usolcop(i) = 0d0
      end do  ! i = 1, npoint

!     --- For all elements do

      istart = 1

      do ielgrp = 1, nelgrp
         ishape = kmeshf(ielgrp)
         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm

         select case ( ishape )

            case(11, 18)

!           --- Linear tetrahedron

               call intmap3d411 ( kmeshc(istart), inelem, kprobf, &
                                  usol, usolcop, ielgrp, inpelm, 4 )

            case(12, 15, 16)

!           --- Quadratic tetrahedron

               call intmap3d412 ( kmeshc(istart), inelem, kprobf, &
                                  usol, usolcop, ielgrp, inpelm )

            case(13, 17)

!           --- Trilinear hexahedron

               call intmap3d411 ( kmeshc(istart), inelem, kprobf, &
                                  usol, usolcop, ielgrp, inpelm, 8 )

            case(14)

!           --- Quadratic hexahedron

               call intmap3d414 ( kmeshc(istart), inelem, kprobf, &
                                  usol, usolcop, ielgrp, inpelm )

            case(21)

!           --- Linear prism

               call intmap3d411 ( kmeshc(istart), inelem, kprobf, &
                                  usol, usolcop, ielgrp, inpelm, 6 )

            case default

!           --- Shape number not available

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               go to 1000

         end select  ! case ( ishape )
         istart = istart+inelem*inpelm    ! raise starting address

      end do  ! ielgrp = 1, nelgrp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepintmap3d4'

      end if  ! ( debug )
      call erclos ( 'sepintmap3d4' )

      end
