      subroutine sepintmapplane ( imesh, isol, number, nunkploc, interp, maplc, &
                                  isol2, iplane, itrans )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-05-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map solution in 3d region onto plane
!     The intersection of the mesh is already available
!     Old subroutine intmapplane
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     interpolation
!     intersection
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol2, nunkploc, imesh, &
              itrans, isol(*), iplane, interp, number
      type ( mapping) :: maplc

!     imesh          i    Sequence number of 2d mesh
!     interp         i    Indication of what variables must be interpolated
!                         See subroutine intp3dbf
!     iplane         i    Defines direction of plane
!                         1:  plane x = constant
!                         2:  plane y = constant
!                         3:  plane z = constant
!                         4:  general plane
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     isol2          o    Standard SEPRAN array containing information of the
!                         2d interpolated vector stored as solution vector
!     itrans         i    indicates if the quantity to be interpolated
!                         must be transformed or not.
!                         Possible values:
!                         0   No special transformation
!                         1   The first two unknowns are projected in the
!                             plane in the direction of the tangential
!                             vectors, the third one perpendicular to the
!                             plane
!     maplc          o    Mapping array
!     number         i    indicates which is the first degree of freedom
!                         that must be interpolated in each point
!     nunkploc       i    indicates the number of unknowns that must be
!                         interpolated in each point containing this
!                         number of unknowns counted from number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision plane(4), trans(3,3)

!     plane          Array of length 4 containing the parameters a, b, c
!                    and d describing the plane: ax=by+cz+d = 0
!                    plane(1) = a; plane(2) = b; plane(3) = c; plane(4) = d
!     trans          In this array the Transformation matrix from x,y,z
!                    co-ordinates to ksi-eta co-ordinates is stored as
!                    well as the origin in trans(.,3)
!                    The transformation of x,y,z to ksi,eta is defined by
!                    ksi = sum_i trans(i,1) ( x(i) - trans(i,3) )
!                    eta = sum_i trans(i,2) ( x(i) - trans(i,3) )
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INTMAP2D       Map solution defined in 3d mesh onto 2d intersection
!     PRININ         print 1d integer array
!     SEPINTMAP3D    Map solution in 3d region onto regular 3d solution defined
!                    in all nodes of the mesh (except centroids)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Make temporary solution array containing all unknowns in 3d mesh
!     if ( transformation required ) then
!        map and transform solution onto 2 dmesh
!     else ( not transformation required )
!        map solution onto 2 dmesh
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepintmapplane' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepintmapplane'
         call prinin ( isol, 5, 'isol' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Make temporary solution array usolt containing all unknowns
!         in 3d mesh

      call sepintmap3d ( isol, number, nunkploc, interp )

!     --- map solution onto 2d mesh
!         if ( transformation required ) then
!            transform solution

      call intmap2d ( isol2, nunkploc, maplc, &
                      imesh, itrans, plane, trans, iplane )

!     --- Remove superfluous arrays

      deallocate(usolt)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepintmapplane'

      end if  ! ( debug )
      call erclos ( 'sepintmapplane' )

      end
