      subroutine sepintp3d ( ichoice, values, jndsol, indsol, interpol, imesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-05-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine intersects a three-dimensional (volume) mesh with
!     a plane
!     The intersection is represented by a two-dimensional triangular mesh
!     The corresponding solution stored in isol is interpolated onto this
!     triangular linear mesh
!     The aim of this subroutine is to create three new arrays kmsh2d, kprb2d
!     and isol2d, that can be used to represent the solution in an
!     intersection by using the standard two-dimensional print and plot
!     subroutines
!     Old subroutine intp3d
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     interpolation
!     intersection
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulemain
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichoice, interpol, imesh, jndsol, indsol
      double precision values(*)

!     ichoice        i    Choice parameter consisting of three parts:
!                         ichoice_plane, ichoice_tang and ichoice_orig
!                         according to ichoice = ichoice_plane+
!                                      10*ichoice_tang+100*ichoice_orig
!                         Meaning of the various parts:
!                         ichoice_plane indicates how the plane is defined by
!                         the user with the aid of array values. Possibilities:
!                         1 Interpolate plane x = xc, with xc stored in
!                           values(1)
!                         2 Interpolate plane y = yc, with yc stored in
!                           values(1)
!                         3 Interpolate plane z = zc, with zc stored in
!                           values(1)
!                         4 Interpolate plane ax + by + cz = d
!                           Array values must be stored with a, b, c and d in
!                           that sequence
!                         ichoice_tang indicates how the tangential vectors
!                         must be computed, that define the axis in the plane
!                         Possible values:
!                         0 the tangential vectors are computed automatically
!                         1 the tangential vectors are defined by the x and y
!                           axis, hence the plane must be z = constant
!                         2 the tangential vectors are defined by the y and z
!                           axis, hence the plane must be x = constant
!                         3 the tangential vectors are defined by the x and z
!                           axis, hence the plane must be y = constant
!                         4 the tangential vectors are defined by the mapping
!                           of the x and y axis onto the plane.
!                           This is only allowed if the coefficient for z in the
!                           plane is unequal to 0
!                         5 the tangential vectors are defined by the mapping
!                           of the y and z axis onto the plane.
!                           This is only allowed if the coefficient for x in the
!                           plane is unequal to 0
!                         6 the tangential vectors are defined by the mapping
!                           of the x and z axis onto the plane.
!                           This is only allowed if the coefficient for y in the
!                           plane is unequal to 0
!                         7 the tangential vectors are given by the user in
!                           array values pos. 8-10 and 11-13
!                         ichoice_orig indicates how the origin must be computed
!                         Possible values:
!                         0 Computed by the program
!                         1 Given by the user in values(5 to 7)
!     imesh          i    Sequence number of 2d mesh
!     indsol         i    Sequence number of output vector in isol
!     interpol       i    Indication of what variables must be interpolated
!                         interpol consists of two parts nunkp and number
!                         according to
!                         interpol = nunkp + 100 * number + 10000*itrans
!                         number  indicates which is the first degree of freedom
!                                 that must be interpolated in each point
!                                 When number = 0 is given, it is interpreted as
!                                 number = 1
!                         nunkp   indicates the number of unknowns that must be
!                                 interpolated in each point containing this
!                                 number of unknowns counted from number
!                         itrans  indicates if the quantity to be interpolated
!                                 must be transformed or not.
!                                 Possible values:
!                                 0   No special transformation
!                                 1   The first two unknowns are projected in
!                                     the plane in the direction of the
!                                     tangential vectors, the third one
!                                     perpendicalar to the plane
!                                     This may only be used in combination
!                                     with nunkp=3
!                         The resulting array isol2d corresponds to an array
!                         with a fixed (nunkp) number of unknowns in each point.
!                         Hence when an unknown with number in the range number
!                         to number+nunkp-1 does not exists, this point is
!                         not used in the output
!                         It is necessary that the unknowns to be interpolated
!                         are at least present in vertex nodal points
!     jndsol         i    Sequence number of input vector in isol
!     values         i    Array containing information concerning the plane,
!                         tangential vectors and origin
!                         in accordance to ichoice
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer itrans, number, ichoicearr(3), iplane, nunkp
      double precision plane(4)
      integer, pointer :: isol3d, isol2dlc
      type (mapping) :: maplc
      double precision :: timebefore(2)

!     ichoicearr     Array of size 3 containing the three parts of ichoice
!                    1:  ichoice_plane
!                    2:  ichoice_tang
!                    3:  ichoice_orig
!     iplane         Defines direction of plane
!                    1:  plane x = constant
!                    2:  plane y = constant
!                    3:  plane z = constant
!                    4:  general plane
!     isol2dlc       isol for 2d problem
!     isol3d         isol for 3d problem
!     itrans         indicates if the quantity to be interpolated
!                    must be transformed or not.
!                    Possible values:
!                    0   No special transformation
!                    1   The first two unknowns are projected in the
!                        plane in the direction of the tangential
!                        vectors, the third one perpendicular to the
!                        plane
!     maplc          Contains the memory management numbers of arrays
!                    imap (1) and rmap (2)
!     number         The unknowns from position number must be interpolated
!     nunkp          Maximum number of unknowns in a point
!     plane          Array of length 4 containing the parameters a, b, c
!                    and d describing the plane: ax=by+cz+d = 0
!                    plane(1) = a; plane(2) = b; plane(3) = c; plane(4) = d
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERSETTIME      Get the CPU time at start of subroutine
!     SEPCLEAR       clear array map
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPINTERSECPLA Intersect 3d mesh by plane
!     SEPINTMAPLANEC Map solution in 3d region onto plane
!                    Complex version
!     SEPINTMAPPLANE Map solution in 3d region onto plane
!     SEPINTP3DCHK   Perform some checks for sepintp3d
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Perform some checks
!     Intersect 3d mesh by plane
!     Map solution on plane
!     Remove superfluous arrays
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Perform some tests

      call eropen ( 'sepintp3d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepintp3d'

      end if

      if ( itime>0 ) call ersettime ( timebefore )

      call sepclear ( maplc )

      isol3d => isolut2d(jndsol)
      isol2dlc => isolut2d(indsol)

      kmesh => kmesh2d(1)
      call sepfillkmesh(kmesh)

!     --- Perform some checks

      call sepintp3dchk ( isol3d, ichoice, ichoicearr, icompl, &
                          interpol, itrans, number, nunkp, plane, &
                          values )
      compl = icompl==2
      if ( ierror/=0 ) goto 1000

!     --- Intersect 3d mesh by plane

      call sepintersecpla ( 1, imesh, plane, maplc, ichoicearr, &
                            values, iplane )
      if ( ierror==2836 ) then

!     --- No intersection found, skip

         ierror = 0
         go to 1000

      end if  ! ( ierror==2836 )
      if ( ierror/=0 ) go to 1000

!     --- Map solution onto plane

      if ( compl ) then

!     --- Complex case

         call sepintmaplanec ( imesh, isol3d, number, nunkp, interpol, maplc, &
                               isol2dlc, iplane, itrans )

      else

!     --- Real case

         call sepintmapplane ( imesh, isol3d, number, nunkp, interpol, maplc, &
                               isol2dlc, iplane, itrans )

      end if  ! ( compl )

      ks(isol2dlc)%typevec = 110
      if ( compl ) ks(isol2dlc)%compl = ks(isol2dlc)%compl+1
      if ( associated(maplc%imap) ) deallocate(maplc%imap, maplc%rmap)

1000  if ( debug ) write(irefwr,*) 'End sepintp3d'
      call erclmn ( 'sepintp3d', timebefore )

      ierror = 0
      end subroutine sepintp3d
