      subroutine sepintsec3dmsh ( kmesh2do, plane, maplc, &
                                  ichoicearr, values, iplane )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-05-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Intersect 3d mesh with a plane
!      create new 2d mesh consisting of triangles
!      Compute mapping array from 3d mesh to 2d mesh
!      Old subroutine intersect3dmsh
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     plane
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: kmesh2do, ichoicearr(3), iplane
      double precision plane(4), values(*)
      type (mapping) :: maplc

!     ichoicearr     i    Array of size 3 containing the three parts of ichoice
!                         1:  ichoice_plane
!                         2:  ichoice_tang
!                         3:  ichoice_orig
!     iplane         i    Defines direction of plane
!                         1:  plane x = constant
!                         2:  plane y = constant
!                         3:  plane z = constant
!                         4:  general plane
!     kmesh2do        o    Standard SEPRAN array containing information of the
!                         2d mesh
!     maplc         i/o   Mapping information
!     plane          i    Array of length 4 containing the parameters a, b, c
!                         and d describing the plane: ax=by+cz+d = 0
!                         plane(1) = a; plane(2) = b; plane(3) = c; plane(4) = d
!     values         i    Array containing information concerning the plane,
!                         tangential vectors and origin
!                         See subroutine intp3dbf
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nelem2d, nnodes, iseqkm1, iseqkm2, i, lenkmeshc
      integer, allocatable :: marknode(:), markedge(:), &
                              kmeshc2dl(:,:)
      double precision trans(3,3)

!     i              Counting variable
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     kmeshc2dl      kmesh part c for 2d intersection
!     lenkmeshc      length of array kmeshc
!     markedge       Array of size nedges
!                    If edge i is not intersected markedge(i) = 0,
!                    otherwise markedge(i)>0 contains the sequence number
!                    of the new node in the local mesh
!     marknode       Array of size npoint
!                    If node i is not on plane marknode(i) = 0,
!                    otherwise marknode(i)>0 contains the sequence number
!                    of the new node in the local mesh
!     nelem2d        Number of elements in 2d local mesh
!     nnodes         Number of nodes
!     trans          In this array the Transformation matrix from x,y,z
!                    co-ordinates to ksi-eta co-ordinates is stored as
!                    well as the origin in trans(.,3)
!                    The transformation of x,y,z to ksi,eta is defined by
!                    ksi = sum_i trans(i,1) ( x(i) - trans(i,3) )
!                    eta = sum_i trans(i,2) ( x(i) - trans(i,3) )
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRDELDOUBLEELM Replace double elements by single elements
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL         Print 1d real vector
!     PRINTERSCOOR2D Fill coordinates array for intersection of 3d mesh
!                    with plane
!     PRINTERSEDG3D  Mark edges that are intersected (3d) by plane
!     PRINTERSTRIAN  Create triangular mesh, constructed by intersection of
!                    3d mesh by a plane
!     SEPCRMESHINFO  Create permanent space for mesh
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLKMESHMN Fill array kmesh part m with information of all the curves
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPINTSECT3DCH Perform checks for subroutine sepintsec3dmsh
!     SEPPRINKMESH   Print array KMESH
!     SEPRELKMESH    Make the relation between kmref and the pointers
!                    in sepmodulekmeshp
!     SEPRELKMESH1   Make the relation between kmref and the pointers
!                    in sepmodulekmeshp
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Perform some checks
!     Activate or create edges corresponding to mesh
!     Clear array markedge(1:nedges)
!     Mark edges that are intersected
!     Store coordinates and mapping array
!     Create 2d triangular mesh
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepintsec3dmsh' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepintsec3dmsh'
  1      format ( a, 1x, (10i6) )
         call sepprinkmesh ( 1 )  ! kmeshc3d
         call sepprinkmesh ( 38 ) ! coor3d

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
!     --- Perform some checks

      iseqkm1 = iseqkm
      call sepintsect3dch ( plane, iplane, ichoicearr, trans, values )
      if ( ierror/=0 ) go to 1000

!     --- Fill first part of kmesh2d

      call sepcrmeshinfo ( kmesh2do )
      km(iseqkm)%kmesh(4) = 3
      km(iseqkm)%kmesh(5) = 1
      km(iseqkm)%kmesh(6) = 2
      km(iseqkm)%kmesh(14) = 3
      km(iseqkm)%kmesh(20) = 1
      allocate (km(iseqkm)%kmeshf(1))
      km(iseqkm)%kmeshf(1) = 3
      iseqkm2 = iseqkm

!     --- Activate or create edges corresponding to mesh

      iseqkm = iseqkm1
      call seprelkmesh1
      if ( kelmaa==0 ) call sepfillkmeshaa
      if ( nmesh(33)==0 ) call sepfillnmeshc
      if ( ierror/=0 ) go to 1000


!     --- Create temporary (permanent) arrays and clear

      allocate ( markedge(nedges), marknode(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint+nedges, '' )
      if ( ierror/=0 ) go to 1000

      markedge = 0
      marknode = 0

!     --- Create temporary arrays

      nnodes = max(nedges,npoint)+100

      allocate ( imap(nnodes,2), rmap(nnodes,1), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'imap' )
      if ( ierror/=0 ) go to 1000

!     --- Mark edges that are intersected
!         Store mapping array

      call printersedg3d ( nmeshc, nnodes, iplane, &
                           coor, plane, marknode, &
                           markedge, npoint, nedges )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,1) 'nnodes, iplane, npoint, nedges', &
                          nnodes, iplane, npoint, nedges
         call prinin ( marknode, npoint, 'marknode' )
         call prinin ( markedge, nedges, 'markedge' )
         call prinin1 ( imap, nnodes, 2, 'imap' )
         call prinrl ( rmap, nnodes, 'rmap' )
      end if  ! ( debug )

!     --- Make arrays imap and rmap permanent

      if ( associated(maplc%imap) ) deallocate ( maplc%imap )
      allocate ( maplc%imap(nnodes,2), maplc%rmap(nnodes,1) )
      maplc%imap = imap(1:nnodes,1:2)
      maplc%rmap = rmap(1:nnodes,1:1)

!     --- Create coordinates array

      iseqkm = iseqkm2
      allocate( km(iseqkm)%coor(2,nnodes) )

!     --- Fill coordinates array

      call printerscoor2d ( coor, km(iseqkm)%coor, nnodes, iplane, trans )
      if ( ierror/=0 ) go to 1000

      coor => km(iseqkm)%coor

      lenkmeshc = 2*nelem+100
      nelem2d = 0

!     --- Create temporary array kmesh2d

      allocate ( kmeshc2dl(3,lenkmeshc), stat = error )
      if ( error/=0 ) call eralloc ( error, 3*lenkmeshc, 'kmesh2dc' )
      if ( ierror/=0 ) go to 1000

!     --- Create 2d triangular mesh

      if ( ierror/=0 ) go to 1000
      call printerstrian ( nelem2d, markedge, marknode, kmeshc2dl, lenkmeshc )
      call inichkarlen ( 3*nelem2d, 3*(2*nelem+100), 'kmesh2dc' )
      if ( ierror/=0 ) go to 1000

!     --- Remove double elements

      call prdeldoubleelm ( kmeshc2dl, nnodes, nelem2d )

!     --- make kmesh2dc permanent

      call seprelkmesh(kmesh2do)
      allocate ( km(iseqkm)%kmeshc(3*nelem2d) )
      do i = 1, 3
         km(iseqkm)%kmeshc(i:3*nelem2d:3) = kmeshc2dl(i,1:nelem2d)
      end do  ! i = 1, 3
      km(iseqkm)%kmesh(17) = 3*nelem2d
      km(iseqkm)%kmesh(5) = 1

!     --- Fill rest of kmesh 2d

      km(iseqkm)%kmesh(8) = nnodes
      km(iseqkm)%kmesh(9) = nelem2d

!     --- Fill kmesh 2d part m

      call sepfillkmesh(kmesh2do)

      call sepfillkmeshmn

      deallocate ( imap, rmap, marknode, markedge, kmeshc2dl, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'imap' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ierror', ierror
         call sepprinkmesh ( 1 )
         call sepprinkmesh ( 38 )
         call sepprinkmesh ( 6 )
         write(irefwr,*) 'End sepintsec3dmsh'

      end if  ! ( debug )

      call sepfillkmesh(kmesh)

      call erclos ( 'sepintsec3dmsh' )

      end subroutine sepintsec3dmsh
