      subroutine sepscatter ( isol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-11-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Distribute global vector over various nodes in parallel case
!     The global vector is stored on the host (node 1)
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     solution_vector
!     vector
!     global
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulevecs
      use sepmodulecpack
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isol

!     isol           i    Sequence number of vector in array ks
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ivec, typevec, leng, icount, icontext, iblock, &
                 npointsav
      integer, allocatable :: ibuffer(:)

!     i              Counting variable
!     iblock         Block sequence number
!     ibuffer        work array to store information to be sent to nodes
!     icontext       Tag for sending and receiving, is used as check
!     icount         Counter
!     ivec           Sequence number of vector of special structure
!     leng           Help variable to compute the length of some arrays
!     npointsav      Help parameter to save the old value of npoint
!     typevec        Type of vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: seplenglocvec

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INCOPY         Copy one integer array into another
!     INIBARRIERMPI  Synchronize parallel sending and receiving (mpi)
!     INIRECEIVEMPI  Receive information from other node (mpi)
!     INISENDMPI     Send information to other node (mpi)
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPLENGLOCVEC  Return with length of local vector corresponding to isol
!     SEPMAPLOCVEC1  Map global vector referred to by isol into local vector
!     SEPPROBLOOP2   information depending on node numberin with respect to
!                    the problem is computed and filled in array kprobg
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     If ( nodenr==1 ) then
!        compute length of local vector
!        allocate vector if it does not exist
!        copy part of global vector into local vector
!        For i := 2 (1) iacnodes do
!           compute length of local vector
!           allocate vector if it does not exist
!           copy part of global vector into local vector
!           send local vector to nodenr i
!     else
!        receive local vector from nodenr 1
!        create new local vector
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepscatter' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepscatter'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iprob = ks(isol)%iprob
      iseqkp = ks(isol)%iseqkp
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      if ( .not. associated(kprobpart%kprobfglobal) ) &
         call sepprobloop2
      kprobpar => kprobpart%kprobpar
      kprobfglobal => kprobpart%kprobfglobal
      nusolglob = kprobpar(1)
      iseqkm = kp(iseqkp)%iseqkm
      npoint = km(iseqkm)%kmesh(8)
      npointsav = npoint
      kmeshpar => km(iseqkm)%kmeshpar
      npointglob = kmeshpar(1)
      typevec = ks(isol)%typevec
      if ( typevec==110 ) then
         kprobloc => kprobpart%kprobloc
         indprf = kprobloc(19)
         nunkp = kprobloc(4)
         nusol = kprobloc(5)
         ivec = 0
      else
         ivec = ks(isol)%ivec
         kprobe2 => kprobpart%kprobe2
         nunkp = -kprobe2(ivec)
         indprf = min(nunkp,0)
      end if  ! ( typevec==110 )

      allocate ( ibuffer(8+2*nusolglob) )

      call sepcreasolbf1 ( isol, typevec, ivec, 'vector', 0 )
      usol => ks(isol)%sol
      usolglob => ks(isol)%solglob
      nodes => km(iseqkm)%mappar(inodenr)%arr
      if ( inodenr==1 ) then

!     --- nodenr=1

         call sepmaplocvec1 ( ivec )
         do i = 2, iacnodes

!        --- compute length of local vector

            nodes => km(iseqkm)%mappar(i)%arr
            leng = seplenglocvec ( i, ivec )

!           --- allocate vector if it does not exist

            nullify(usol)
            allocate ( usol(leng) )

!           --- copy part of global vector into local vector

            npoint = size(nodes)
            call sepmaplocvec1 ( ivec )
            ibuffer(1) = ks(isol)%compl
            ibuffer(2) = ks(isol)%typevec
            ibuffer(3) = ks(isol)%ivec
            ibuffer(4) = ks(isol)%jcart
            ibuffer(5) = ks(isol)%nphys
            ibuffer(6) = leng
            ibuffer(7) = ks(isol)%iprob
            ibuffer(8) = ks(isol)%iseqkp
            call incopy ( usol, ibuffer(9), 2*leng )

!           --- send local vector to nodenr i

            icontext = 45
            call inisendmpi ( ibuffer, 2*leng+8, SPINT, i, icontext )
            deallocate ( usol )

         end do  ! i = 2, iacnodes

      else

!     --- receive local vector from nodenr 1
!         create new local vector

         icontext = 45
         icount = npointglob+2*nusolglob+2
         call inireceivempi ( ibuffer, icount, SPINT, iblock, icontext )
         leng = ibuffer(6)

         call sepcreasolbf1 ( isol, ibuffer(2), ibuffer(3), 'vector', &
                              ibuffer(1) )
         call incopy ( ibuffer(9), ks(isol)%sol, leng*2 )

      end if  ! ( inodenr==1 )
      deallocate ( ibuffer )

!     --- Synchronize communication by calling mpi_barrier
!         This avoids that something is written to other nodes, &
!         before reading by those other nodes has been done

      call inibarriermpi
      npoint = npointsav

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepscatter'

      end if  ! ( debug )
      call erclos ( 'sepscatter' )

      end subroutine sepscatter
