      subroutine to0088
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.2  date 30-11-2012 Long integers
!        version  3.1  date 14-12-2001 Allow vectors of type 116
!        version  3.0  date 14-03-2001 Extra parameter skip
!
!   copyright (c) 1991-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   subroutine to0088 is used to copy the input vector, which is a vector
!   corresponding to the vertices of quadratic elements onto the output
!   vector. All midside points are filled by linear interpolation.
!   Restriction:  At this moment only the shape numbers: 2, 4, 6, 14
!                 are available
!                 Both the input vector and output vector must contain
!                 one unknown per point (in vertices or all points)
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulesmf
      use sepmodulemain
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer itypvc, i, ikelmc, kpivec, ivec, nodp, inunkp
      integer, pointer :: elnodes(:)

!     elnodes        Contains nodes of element
!     i              General loop variable
!     ikelmc         position in kmesh part c
!     inunkp         Number of degrees of freedom in point
!     itypvc         Type number of input or output vector
!     ivec           Sequence number for array of special structure
!     kpivec         Starting address in array KPROB with respect to the ivec'th
!                    vector of special structure
!     nodp           Nodal point number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPELLINQUAD   Fill midpoints in array
!     SEPFINDIVEC    Find the structure of the vector with exactly NUMBER
!                    unknowns per point
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      60   Input solution array has not been filled or is of the wrong type
!     800   Shape number not available
!    1731   The number of unknowns per point in the input vector is not 1
!    1733   Number of unknowns in vertex not 1
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Activate input vector
!   Find type of output vector
!   Activate output vector or create space for this vector
!
!   For element group := 1 (1) number of element groups do
!      if ( element type is quadratic ) then
!         For element := 1 (1) number of elements do
!             Fill midpoints
!      else if ( element type is linear ) then
!         skip element group
!      else ( element type is linear nor qudratic ) then
!         error
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0088' )

!     --- Activate input array

      itypvc = ks(isolut)%typevec
      if ( itypvc/=110 .and. itypvc/=115 .and. itypvc/=116 ) then
         call errint ( itypvc, 1 )
         call errsub ( 60, 1, 0, 0 )
         go to 1000
      end if
      call sepactsolbf2 ( isolut )
      if ( nphysi>1 ) then

!     --- error 1731: The input vector contains more than one unknown per point

         call errint ( nphysi, 1 )
         call errsub ( 1731, 1, 0, 0 )
         go to 1000

      end if

!     --- Find type of output vector

      call sepfindivec ( 1, itypvc, ivec )

!     --- Activate output vector

      call sepcreasolbf1 ( SMF%ioutvc, itypvc, ivec, 'vector', &
                           ks(isolut)%compl )

!     --- Compute some constants

      inpelm = npelm

!     --- Compute starting positions in array IBUFFR

      vectin => ks(isolut)%sol
      vectout => ks(SMF%ioutvc)%sol

!     --- Loop over element groups

      ikelmc = 0
      do ielgrp = 1, nelgrp

         inelem = sepgetinelem ( ielgrp )  ! number of elements per group
         inpelm = sepgetinpelm ( ielgrp )  ! number of nodes per group

         if ( skipelgrps(ielgrp)==0 ) then

!        --- Element group must be skipped

            ikelmc = ikelmc+inelem*inpelm

         else

!        --- Use element group

            ishape = sepgetshape ( ielgrp )   ! shape number of group
            elnodes => kmeshc(ikelmc+1:ikelmc+inpelm)

            if ( ishape==2 .or. ishape==4 .or. ishape==6 &
                .or. ishape==7 .or. ishape==14 ) then

!           --- Quadratic element, fill midpoints
!               Loop over all elements

               if ( ishape==2 ) then
                  nvert = 2
               else if( ishape==6 )then
                  nvert = 4
               else if( ishape==14 )then
                  nvert = 8
               else
                  nvert = 3
               end if

               if ( indprfi>0 ) then

!              --- indprfi>0, Check if vertices are filled

                  do i = 1, nvert

                     nodp = elnodes(2*i-1)
                     inunkp = kprobfi(nodp+1) - kprobfi(nodp)
                     if ( inunkp/=1 ) then
                        call errint ( nodp, 1 )
                        call errint ( inunkp, 2 )
                        call errsub ( 1733, 2, 0, 0 )
                        go to 1000
                     end if
                  end do

               end if

               do ielem = 1, inelem

                  call sepellinquad ( elnodes, inpelm, &
                                      vectin, vectout, nvert )

               end do

            else

!           --- This type of elements has not yet been implemented

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               go to 1000

            end if

         end if  ! ( skipelgrps(ielgrp)==0 )

      end do  ! ielgrp = 1, nelgrp

1000  call erclos ( 'to0088' )
      end
