      subroutine pl0014 ( coor, v, jv, procor, npoint, xmean, perspc, &
                          nprcor, vact, ichois, plotfm, yfact, iorien )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  5.2  date 17-10-2000 Correction xmean
!        version  5.1  date 21-06-1997 Layout
!        version  5.0  date 21-04-1995 Introduction of basic coordinate
!                                      system, extra parameter iorien
!
!   copyright (c) 1990-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Computes the projections of the three-dimensional points onto a
!   two-dimensional face, using a given view point
!   also the projections of the unit vectors are computed
!   at this moment only one projection is available
!   With the parameter jmax the dimensions can be kept or reused
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     plot
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, jv, nprcor, ichois, iorien
      double precision coor(3,*), v(3), procor(nprcor,*), xmean(3), &
                       vact(3), plotfm, yfact
      logical perspc

!     coor     i   array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     ichois   i   Choice parameter indicating if the plot must be opened or
!                  not. Possible values:
!                  0: Do not open plot
!                  1: Open plot and plot axis
!                  2: Open plot but do not plot axis
!     iorien   i   indicator what basic system is used
!                  1 : x - y - z system (also default)
!                  2 : z - x - y system
!                  3 : y - z - x system
!     jv       i   indicator whether v has been filled (>1) or not (0)
!                  The value of jv indicates how v has been filled.
!                  Possibilities:
!                  0:    v has not been filled
!                  1:    v has been filled with the coordinates of the
!                          view point
!     npoint   i   number of nodal points to be considered
!     nprcor   i   Dimension of first index of procor
!                  If nprcor = 3, pl0014 is called by a colour subroutine, &
!                  which implies that also the distance to the view-point is
!                  computed, if nprcor = 2, only the projection is computed
!     perspc   i   Indication if a perspective (true) or a parallel
!                  projection
!                  must be carried out (false)
!     plotfm   i   Plotting factor. When PLOTFM < 0, then the length of
!                  the plot is computed by - (xmax-xmin) * PLOTFM
!                  and the width by - (ymax-ymin) * YFACT * PLOTFM
!                  else the length of the plot is equal to PLOTFM cm
!                  When by some mistake the user has given | PLOTFM | a value
!                  smaller than 1 cm, then the length is chosen equal to
!                  the default length stored in common block consta
!     procor   o   coordinate array for projected points
!                  (1,i) contains the x-coordinate of the projected point i
!                  (2,i) contains the y-coordinate of the projected point i
!                  (3,i) contains the distance of point i to
!                        the view-point
!     v        i   co-ordinates of view point (3D)
!     vact     o   computed (corrected) view point
!     xmean    o   Array containing the mean values of the co-ordinates
!     yfact    i   Multiplication for the y-co-ordinates. With this
!                  parameter the user can influence the scale in y-direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, in(3)
      double precision eps, xdef, ydef, zdef, relafs, drel, dunity, &
                       xa, ya, za, ext, ski(3), eta(3), xao, yao, zao, &
                       skia, etaa, xminp, xmaxp, yminp, ymaxp, dunitx
      double precision axis(2,4)
      save xminp, xmaxp, yminp, ymaxp

!     axis    Co-ordinates of axis in projected plane
!             axis(.,1)  Co-ordinates of centre of axis
!             axis(.,2)  Co-ordinates of end point of x-axis
!             axis(.,3)  Co-ordinates of end point of y-axis
!             axis(.,4)  Co-ordinates of end point of z-axis
!     drel    help factor for correction of coordinates in case of hidden line
!             plot
!     dunitx  Unit length in x-direction
!     dunity  Unit length in y-direction
!     eps     Corrected machine accuracy
!     eta     second unit vector in projection plane
!     etaa    help variable to store second local plane coordinate
!     ext     help variable for displacement
!     i       Counting variable
!     in      Helparray for ordering
!     j       Counting variable
!     relafs  Correction factor for coordinates in case of hdden line plot
!     ski     first unit vector in projection plane
!     skia    help variable to store first local plane coordinate
!     xa      x-distance between two points
!     xao     x-coordinate of projected point in 3-d
!     xdef    Default x-component of view parameter
!     xmaxp   Maximum x value of the projected region to be considered
!             See xminp
!     xminp   Minimum x value of the projected region to be considered
!             If jmax=1 the values of the preceding call are used
!             This is meant for the case where you plot more items
!             in one plot, for example mesh and contour plot
!             First plot for example the mesh and store xmin, xmax, ...
!             in common /cplot/, next set jmax=1 and then plot the contours
!     ya      y-distance between two points
!     yao     y-coordinate of projected point in 3-d
!     ydef    Default y-component of view parameter
!     ymaxp   Maximum y value of the projected region to be considered
!             See xminp
!     yminp   Minimum y value of the projected region to be considered
!             See xminp
!     za      z-distance between two points
!     zao     z-coordinate of projected point in 3-d
!     zdef    Default z-component of view parameter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERREAL  Put real in error message
!     ERRWAR  Print warnings
!     PL0000  Compute length and width of the plot
!     PL0015  Plot 3D axis
!     PL0016  Check plotfm and compute width
!     PLAFEP  General SEPRAN low level plotting subroutine
!     PLT351  Compute new coordinates
!     PLT358  Compute eta and ski
! **********************************************************************
!
!                       I/O
!
!     none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     1708   View point within region to be plotted
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute extreme coordinate values
!     Compute centre of projection plane ("barycenter of object")
!     Compute default eye-position value
!        If eye-position is given then
!           Check whether given values are allowed
!     If parallel projection enlarge distance eye point to projection
!     plane
!     Compute projection of all nodal points
!     (Start with vectors of unity ski and eta in projection plane)
!     If needed compute also the distance from point to view point
!     Correct co-ordinates with mean value
!     Compute projection of origin and the three axes
!     Open plot
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pl0014' )

!     --- Compute basic coordinate system

      if ( iorien.eq.2 ) then

!     --- z - x - y system

         in(1) = 3
         in(2) = 1
         in(3) = 2

      else if ( iorien.eq.3 ) then

!     --- y - z - x system

         in(1) = 2
         in(2) = 3
         in(3) = 1

      else

!     --- x - y - z system

         in(1) = 1
         in(2) = 2
         in(3) = 3

      end if

!     --- Compute extreme coordinate values

      if ( jmax.eq.0 )  then

!     --- jmax = 0

         xmin =  rinfin
         ymin =  rinfin
         zmin =  rinfin

         xmax = -rinfin
         ymax = -rinfin
         zmax = -rinfin

         do i = 1, npoint

            xmin = min ( xmin, coor( in(1) ,i ) )
            ymin = min ( ymin, coor( in(2) ,i ) )
            zmin = min ( zmin, coor( in(3) ,i ) )

            xmax = max ( xmax, coor( in(1) ,i ) )
            ymax = max ( ymax, coor( in(2) ,i ) )
            zmax = max ( zmax, coor( in(3) ,i ) )

         end do

      end if

!     --- Compute centre of projection plane and default eye - position

      xmean( in(1) ) = 0.5d0 * ( xmax + xmin )
      xmean( in(2) ) = 0.5d0 * ( ymax + ymin )
      xmean( in(3) ) = 0.5d0 * ( zmax + zmin )

      xdef = xmean(1) + ( xmax - xmin )
      ydef = xmean(2) - ( ymax - ymin )
      zdef = xmean(3) + ( zmax - zmin )

      if ( jv.eq.0 ) then

         vact(1) = xdef
         vact(2) = ydef
         vact(3) = zdef
         v(1) = vact(1)
         v(2) = vact(2)
         v(3) = vact(3)

      else

!     --- Check whether given values are allowed

         vact(1) = v( in(1) )
         vact(2) = v( in(2) )
         vact(3) = v( in(3) )

         if ( vact(1).lt.xmax .and. vact(1).gt.xmin .and. &
              vact(2).lt.ymax .and. vact(2).gt.ymin .and. &
              vact(3).lt.zmax .and. vact(3).gt.zmin ) then

!        --- Submitted values are not good , use default values

            call erreal ( vact(1), 1 )
            call erreal ( vact(2), 2 )
            call erreal ( vact(3), 3 )

            vact(1) = xdef
            vact(2) = ydef
            vact(3) = zdef

            call erreal ( vact(1), 4 )
            call erreal ( vact(2), 5 )
            call erreal ( vact(3), 6 )

            call errwar ( 1708, 0, 6, 0 )

         end if

      end if

      if ( .not. perspc ) then

!     --- Parallel projection, enlarge the distance from view point to centroid

         vact(1) = 11d0 * vact(1) - 10d0 * xmean(1)
         vact(2) = 11d0 * vact(2) - 10d0 * xmean(2)
         vact(3) = 11d0 * vact(3) - 10d0 * xmean(3)

      end if

!     --- Compute projection of points

      eps = 10 * epsmac

!     --- (Start with vectors of unity ski and eta )

      vact(1) = vact(1) - xmean(1) + 203.7 * eps
      vact(2) = vact(2) - xmean(2) - 201.0 * eps
      vact(3) = vact(3) - xmean(3) + 100.5 * eps

      call plt358( vact, ski, eta )

      if ( jmax.eq.0 )  then

         xmin =  rinfin
         ymin =  rinfin
         xmax = -rinfin
         ymax = -rinfin

      else

         xmin = xminp
         ymin = yminp
         xmax = xmaxp
         ymax = ymaxp

      end if

      do i = 1 , npoint

           xa = coor( in(1) , i ) - xmean(1)
           ya = coor( in(2) , i ) - xmean(2)
           za = coor( in(3) , i ) - xmean(3)

           call plt351( vact, xa, ya, za, ski, eta, xao, yao, zao, &
                        skia, etaa )

           procor(1,i) = skia
           procor(2,i) = etaa

           if ( nprcor.eq.3 ) then

!          --- nprcor = 3, compute the distance from point to view point

              xa = vact(1) - xa
              ya = vact(2) - ya
              za = vact(3) - za

              procor(3,i) = sqrt( xa*xa + ya*ya +za*za )

           end if

           if ( jmax.eq.0 )  then

              xmin = min ( skia, xmin )
              xmax = max ( skia, xmax )

              ymin = min ( etaa, ymin )
              ymax = max ( etaa, ymax )

           end if

      end do

      if ( jmax.eq.0 )  then

         xminp = xmin
         yminp = ymin
         xmaxp = xmax
         ymaxp = ymax

      end if

      if ( nprcor.eq.2 ) then

!     --- Make a small correction for all points in the case of a hidden line
!         plot. In this way some possible errors due to double projected
!         points may be avoided

         relafs = eps * sqrt ( (xmax-xmin)**2 + (ymax-ymin)**2 )
         drel   = 0.5d0 * npoint * relafs

         do i = 1 , npoint

            procor(1,i) = procor(1,i) + i * relafs - drel
            procor(2,i) = procor(2,i) + i * relafs - drel

         end do

!        --- Correct co-ordinates with mean value

         do j = 1, 3
            do i = 1, npoint
               coor(j,i) = coor(j,i) - xmean(j)
            end do
        end do

      end if

      ext  = ( xmax - xmin ) / 20d0

      xmin = xmin - ext
      xmax = xmax + ext
      ymin = ymin - ext
      ymax = ymax + ext

!     --- Compute projection of axis
!         Origin

      xa = 0d0
      ya = 0d0
      za = 0d0

      call plt351( vact, xa, ya, za, ski, eta, xao, yao, zao, &
                   axis(1,1), axis(2,1) )

!     --- x-axis

      xa = xa + 1d0

      call plt351( vact, xa, ya, za, ski, eta, xao, yao, zao, &
                   axis(1,2), axis(2,2) )

!     --- y-axis

      xa = xa - 1d0
      ya = ya + 1d0

      call plt351( vact, xa, ya, za, ski, eta, xao, yao, zao, &
                   axis(1,3), axis(2,3) )

!     --- z-axis

      ya = ya - 1d0
      za = za + 1d0

      call plt351( vact, xa, ya, za, ski, eta, xao, yao, zao, &
                   axis(1,4), axis(2,4) )

!     --- Open plot

      call pl0016 ( plotfm, width, yfact )
      call pl0000 ( plotfm, yfact )
      if ( ichois.ge.1 .and. mod(jtimes,10).le.1 ) &
           call plafep ( 1, aleng, width, 0d0, 0d0, 0 )

      if ( ichois.eq.1 ) then

!     --- Plot cross of axis

         call pl0015 ( axis, plotfm, iorien )

      else

!     --- Axis is not plotted but the values of xmin and ymin are adapted
!         in order to get the same pictures with or without axis

         dunitx = ( xmax - xmin ) / ( plotfm - 2d0 )
         dunity = ( ymax - ymin ) / ( width  - 2d0 )

         xmin = xmin - 2d0 * dunitx
         ymin = ymin - 2d0 * dunity

      end if

      call erclos ( 'pl0014' )

      end
