      subroutine pl0017 ( ivectr, ipict, npict, isymbl, ichfun, funcx, &
                          funcy, number, icurvs, valext, ileft, modifys )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.8  date 27-11-2013 Allow disjoint curves
!        version  4.7  date 15-11-2012 Long integers
!        version  4.6  date 13-05-2010 Change error messages
!
!   copyright (c) 1992-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the x-values as well as the function values to be plotted by
!     PLOTFN in idummy and compute extreme values
!     Help subroutine for plotfn
!     Warning: this subroutine uses memory managment subroutines, so
!     pointers must be recomputed after this call
! **********************************************************************
!
!                       KEYWORDS
!
!     function_plot
!     fill_function
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplotfun
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulevecs
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichfun, ivectr, ipict, &
              npict, number, isymbl, icurvs(*), ileft, modifys
      double precision funcx(*), funcy(*), valext(4)

!     funcx     i   User array of type 117, which is only used if ichfun = 1
!                   funcx contains the x-coordinates to be plotted
!                   funcx is filled as follows:
!                   1:  Declared length of the array
!                   4:  1
!                   5:  Number of nodal points
!                   6 ...:  actual data
!     funcy     i   User array of type 117, which is only used if ichfun = 1
!                   funcy contains the function values to be plotted
!                   funcy is filled as follows:
!                   1:  Declared length of the array
!                   4:  number of degrees of freedom per point
!                   5:  Number of nodal points * number of degrees of freedom
!                   6 ...:  actual data
!     ichfun    i   Indication how the function is given. Possibilities:
!                   0:  standard situation
!                       the input vector may be of type 110, 115 or 119
!                       when itype=110 or 115 array icurvs is used
!                   1:  the input is stored in the arrays funcx and funcy
!                       of type 117 (user arrays)
!                   2:  see ichfun=0, however, the minimum and maximum x and y
!                       values xmin,xmax,ymin and ymax are not computed
!                       from the functions to be plotted, but from the mesh
!                       boundaries    (itype=119 only)
!                   3:  see ichfun=2, boundaries of the mesh are also plotted
!                   4:  see ichfun=3, the initial- and end-points of curves
!                       to be plotted are connected by a straight line.
!     icurvs    i   definition of the curves by the user.
!                   position 1 gives the type of indication
!                   the next positions contain the curve numbers
!                   values for icurvs(1):
!                     0       only one curve is used
!                    -1       the function is plotted along curves icrv1 to
!                             icrv2
!                    -2       see -1, however in reversed order
!                    >0       the function is plotted along the curves icrv1,
!                             icrv2, . . .  , icrvn
!     ileft     i   indication if left (0) or right y-scale (1) should be used
!     ipict     i   sequence number of the function to be plotted
!                   Must be between 1 and npict (inclusive end-points)
!     isymbl    i   Choice parameter indicating with what symbols the
!                   functions are plotted. Isymbl may be different for every
!                   call of plotfn. Possibilities are:
!                   0      Only straight lines are plotted
!                   1-10   Points will be connected with a line and marked with
!                          a symbol, system depended
!                   11-20  Only the same symbols are used, no lines
!     ivectr    i   array containing information of the function to be
!                   plotted
!     modifys    i  If 1 the minimum and maximum s coordinates are stored in
!                   funcx
!     npict      i  number of functions to be plotted in one picture
!     number     i  the number th degree of freedom in each nodal point
!     valext     o  In this array the extreme values of function and
!                   co-ordinates are stored:
!                   Pos. 1 xmin
!                   Pos. 2 xmax
!                   Pos. 3 ymin
!                   Pos. 4 ymax
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical compl
      integer kpict, n, indexl, itypvc, icompl, nh, m, idummy, &
              i, icrvcp(2), nparts, ipoint(100), ipos, ipvecr
      double precision multfact, amin, amax, smin, smax, ds
      integer, allocatable :: iwork(:)
      double precision, allocatable :: work(:)
      save kpict

!     amax           Maximum value in an array
!     amin           Minimum value in an array
!     compl          Indicates if the vector is complex (true) or not
!     ds             Multipication factor for transforamtion of co-ordinates
!     i              General loop variable
!     icompl         Multiplication factor for reals (1=real, 2=complex)
!     icrvcp         Help array to store initial end and curve numbers
!     idummy         Dummy parameter
!     indexl         Infor entry with respect to present FUNCTION array
!     ipoint         Array of length nparts+1 containing the starting
!                    addresses in array inodes of the various parts
!                    the boundary consists of.
!                    ipoint(1) = 1,
!                    number of nodes at part i = ipoint(i+1)-ipoint(i)
!     ipos           Position in array
!     ipvecr         Position of subcurve in usoltemp
!     itypvc         Type number of solution vector
!     iwork          integer work array containing number of nodes per segment
!     kpict          Preceding picture number
!     m              Number of subcurves
!     multfact       multiplication factor for function (at this moment only 1)
!     n              Number of points of the function to be plotted
!     nh             Overestimation of n
!     nparts         number of parts the boundary consists of
!     smax           Maximum s-value
!     smin           Minimum s-value
!     work           Work array containing x-coordinates and function
!                    values for each segment
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     ER0028         Fill function values and co-ordinates from curves
!     ER0060         Fill function values and co-ordinates in 1D case
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
!     PRINLG         Print and checks length of user arrays
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCURVENODES  Store all nodes in one curves in array inodes and gaps in
!                    these curves in array ipoint
!     SEPER0027      Fill nodal points of curves as indicated by ichois and
!                    icurvs in array iwork
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      60   Input solution array has not been filled or is of the wrong type
!     102   NUMBER has wrong value
!     369   Not the next picture
!     370   NPICT has wrong value
!     393   Too few points in FUNCX/Y
!     394   Not the same number of points in FUNCX and FUNCY
!    1825   ICHFUN incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!   if ( ipict=1 ) then
!      initialize and create array ILOCAL
!   else ( ipict>1)
!      activate array ILOCAL
!   Perform checks
!   itypvc = "type of vector"
!   if ( ichfun=1 ) then
!      Create "function" array and store sequence number in ILOCAL
!      Copy contents of arrays FUNCX and FUNCY into "function" array
!   else ( ichfun#1 )
!      ndim   := kmesh(6)
!      if ( itypvc<116 ) then
!         if ( ndim=1 ) then
!            Fill array iwork with nodal points of the curves (er0027)
!            Fill function array and compute number of points (er0060)
!         else ( ndim>1 )
!            Fill array iwork with nodal points of the curves (er0027)
!            Fill function array and compute number of points (er0028)
!         Remove possible gap between x and y values by joining x and y values
!         Copy contents of arrays FUNCX and FUNCY into "function" array
!      else ( itypvc=119 )
!         m := ivectr(4)
!         for subcurv := 1 (1) m do
!            fill function array for subcurv
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data kpict /0/
! ======================================================================
!
      call eropen ( 'pl0017' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pl0017'
         write(irefwr,1) 'ichfun, ipict, number, npict', &
                          ichfun, ipict, number, npict
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( npict<ipict .or. npict>10000 ) then
         call errint ( npict, 1 )
         call errint ( ipict, 2 )
         call errsub ( 370, 2, 0, 0 )
      end if

      if ( ipict==1 ) then

!     --- First picture
!         reserve space for array ILOCAL, and clear this array

         do i = 1, npict
            ilocal(i)%n = 0
            ilocal(i)%isymbl = 0
            ilocal(i)%dcolor = 0
            ilocal(i)%ileft = 0
            nullify ( ilocal(i)%work, ilocal(i)%iwork )
         end do  ! i = 1, npict

      else

!     --- ipict>1, activate array ILOCAL

         if ( ipict/=kpict+1 ) then
            call errint ( ipict, 1 )
            call errint ( kpict, 2 )
            call errsub ( 369, 2, 0, 0 )
         end if

      end if
      ilocal(ipict)%isymbl = isymbl
      ilocal(ipict)%dcolor = dcolor
      ilocal(ipict)%ileft = ileft
      multfact = 1d0

!     --- Fill function arrays

      if ( ichfun==1 ) then

!     --- ichfun = 1, function values in funcx and funcy

         n = nint(funcx(5))
         if ( n<2 ) then
            call errint ( n, 1 )
            call errsub ( 393, 1, 0, 0 )
         end if
         nphysi = nint(funcy(4))
         if ( number<=0 .or. number>nphysi ) then
            call errint ( number, 1 )
            call errint ( nphysi, 2 )
            call errsub ( 102, 2, 0, 0 )
         end if
         if ( ierror/=0 ) go to 1000
         if ( nint(funcy(5))/=n*nphysi ) then
            call errint ( n, 1 )
            call errint ( nint(funcy(5))/nphysi, 2 )
            call errsub ( 394, 2, 0, 0 )
         end if

         call prinlg ( 'funcx', -n-5, funcx, idummy )
         call prinlg ( 'funcy', -n*nphysi-5, funcy, idummy )

!        --- Create FUNCTION array

         indexl = 0
         allocate ( ilocal(ipict)%work(2*n) )
         ilocal(ipict)%n = n

         call dcopy ( n, funcx(6), 1, ilocal(ipict)%work, 1 )
         call dcopy ( n, funcy(5+number), nphysi, &
                      ilocal(ipict)%work(1+n), 1 )
         valext(1) = minval(ilocal(ipict)%work(1:n))
         valext(2) = maxval(ilocal(ipict)%work(1:n))
         valext(3) = minval(ilocal(ipict)%work(n+1:2*n))
         valext(4) = maxval(ilocal(ipict)%work(n+1:2*n))

      else

!     --- ichfun # 1, standard SEPRAN arrays, types must be 110, 115 or 119

         compl = ks(ivectr)%compl==1
         icompl = ks(ivectr)%compl+1
         itypvc = ks(ivectr)%typevec
         if ( itypvc/=110 .and. itypvc/=115 .and. itypvc/=119 ) &
              then
            call errint ( itypvc, 1 )
            call errsub ( 60, 1, 0, 0 )
            go to 1000
         end if
         if ( debug ) write(irefwr,1) 'itypvc', itypvc

         if ( itypvc==110 .or. itypvc==115 ) then

!        --- itypvc  =  110 or 115: standard SEPRAN arrays

            call sepactsolbf2 ( ivectr )
            if ( number<=0 .or. number>nphysi*icompl ) then
               call errint ( number, 1 )
               call errint ( nphysi*icompl, 2 )
               call errsub ( 102, 2, 0, 0 )
            end if
            if ( ichfun/=0 ) &
               call errsub ( 1825, 0, 0, 0 )

            if ( debug ) write(irefwr,1) 'ndim', ndim

            if ( ndim==1 ) then

!           --- ndim  =  1, One-dimensional case

               nh = npoint
               if ( debug ) write(irefwr,*) 'npoints ', nh

!              --- Reserve work space to store the values and the node
!                  numbers

               allocate ( iwork(nh), work(2*nh), stat = error )
               if ( error/=0 ) call eralloc ( error, 3*nh, 'iwork' )
               if ( ierror/=0 ) go to 1000

               usoltemp => ks(ivectr)%sol

!              --- Determine the node-numbers on the curve

               if ( debug ) call prinin ( icurvs, 2, 'icurvs' )

               if ( icurvs(1)==0 .and. icurvs(2)==0 ) then

                  icrvcp(1) = 1
                  icrvcp(2) = ncurvs
                  if ( debug ) &
                     write(irefwr,1) 'npoints before er0027', nh
                  call seper0027 ( -1, icrvcp,  nh, iwork )
                  if ( debug ) then
                     write(irefwr,1) 'npoints after er0027', nh
                     call prinin ( iwork, nh, 'nodes' )
                  end if

               else

                  nparts = 0
                  call sepcurvenodes ( icurvs, ipoint, iwork, nparts, kmeshm )

                  call inichkarlen ( nparts+1, 100, 'ipoint' )
                  if ( ierror/=0 ) go to 1000

                  nh = ipoint(nparts+1)-1
                  if ( debug ) then
                     write(irefwr,1) 'npoints after ercurvenodes', nh
                     call prinin ( iwork, nh, 'nodes' )
                  end if

               end if

!              --- Fill co-ordinates and function values in work array work
!                  Compute number of points

               call er0060 ( number, work, work(1+nh), nh, n, coor, iwork, &
                             usoltemp, kprobfi, indprfi, &
                             nphysi, multfact )
               if ( debug ) write(irefwr,1) 'n', n

            else

!           --- ndim>1, curves available

               ncurvs = ncurvs
               n = kmeshm(3)-kmeshm(2)

               allocate ( iwork(n), work(2*n), stat = error )
               if ( error/=0 ) call eralloc ( error, 3*n, 'iwork' )
               if ( ierror/=0 ) go to 1000

               usoltemp => ks(ivectr)%sol

!              --- Compute number of points on curves

               call seper0027 ( icurvs(1), icurvs(2), n, iwork )
               if ( ierror/=0 ) go to 1000

               nh = n
               call er0028 ( iwork, n, number, ndim, &
                             coor, usoltemp, work, work(1+n), &
                             kprobfi, indprfi, nphysi, multfact, compl )
               if ( modifys==1 ) then

!              --- The x-coordinates must be modified

                  smin = funcx(1)
                  smax = funcx(2)
                  ds = (smax-smin)/work(n)
                  do i = 1, n
                     work(i) = smin + ds*work(i)
                  end do

               end if

            end if

            if ( nh>n ) work(n+1:2*n) = work(nh+1:nh+n)

            allocate ( ilocal(ipict)%work(2*n) )
            ilocal(ipict)%work = work(1:2*n)
            ilocal(ipict)%n = n
            valext(1) = minval(work(1:n))
            valext(2) = maxval(work(1:n))
            valext(3) = minval(work(n+1:2*n))
            valext(4) = maxval(work(n+1:2*n))

            deallocate ( iwork, work, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'iwork' )
            if ( ierror/=0 ) go to 1000

         else

!        --- itypvc  =  119, special SEPRAN array

            if ( debug ) call prinin ( ivectr, 5, 'ivectr' )

            nphysi = icompl
            if ( number<=0 .or. number>nphysi*icompl ) then
               call errint ( number, 1 )
               call errint ( nphysi*icompl, 2 )
               call errsub ( 102, 2, 0, 0 )
            end if
            m = ks(ivectr)%n

            if ( debug ) write(irefwr,*) 'compl ', compl
            if ( compl ) then

!           --- complex case
!               reserve space for the vector

               isoltemp => ks(ivectr)%isol
               n = size(isoltemp)
               usoltemp => ks(ivectr)%sol
               length = size(usoltemp)

               allocate ( iwork(n), work(length), stat = error )
               if ( error/=0 ) call eralloc ( error, n+length, 'iwork' )
               if ( ierror/=0 ) go to 1000

!              --- Copy integer information

               iwork(1:n) = isoltemp

!              --- Loop over subcurves

               ipos = 1
               ipvecr = 1
               do i = 1, m
                  n = iwork(i)

!                 --- Copy x-co-ordinates

                  work(1:n) = usoltemp(ipvecr:ipvecr+n-1)
                  ipos = ipos + n
                  ipvecr = ipvecr + n

!                 --- Copy function values

                  work(ipos:ipos+n-1) = &
                     usoltemp(ipvecr+number:ipvecr+number+2*n-1:2)
                  ipvecr = ipvecr + 2*n
                  ipos = ipos + n

               end do
               ipos = ipos-1

            else

!           --- real case
!               Create FUNCTION array and copy contents

               isoltemp => ks(ivectr)%isol
               n = size ( isoltemp )
               usoltemp => ks(ivectr)%sol
               length = size(usoltemp)

               allocate ( iwork(n), work(length), stat = error )
               if ( error/=0 ) call eralloc ( error, n+length, 'iwork' )
               if ( ierror/=0 ) go to 1000

               iwork(1:n) = isoltemp
               work(1:length) = usoltemp
               ipos = length

            end if

!           --- Create final function array

            allocate ( ilocal(ipict)%iwork(n) )
            ilocal(ipict)%iwork = iwork(1:n)
            ilocal(ipict)%n = -m
            indexl = 0
            allocate ( ilocal(ipict)%work(ipos) )
            ilocal(ipict)%work = work(1:ipos)

!           --- Compute extreme values

            valext(1) = rinfin
            valext(2) = -rinfin
            valext(3) = rinfin
            valext(4) = -rinfin
            ipos = 1
            do i = 1, m
               n = iwork(i)
               valext(1) = min ( valext(1), minval(work(ipos:ipos+n-1)) )
               valext(2) = max ( valext(2), maxval(work(ipos:ipos+n-1)) )
               ipos = ipos + n
               valext(3) = min ( valext(3), minval(work(ipos:ipos+n-1)) )
               valext(4) = max ( valext(4), maxval(work(ipos:ipos+n-1)) )
               ipos = ipos + n
            end do
            deallocate ( iwork, work, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'iwork' )
            if ( ierror/=0 ) go to 1000

         end if

      end if

!     --- Raise kpict

1000  kpict = kpict + 1
      if ( ipict==npict ) kpict = 0

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pl0017'

      end if
      call erclos ( 'pl0017' )

      end subroutine pl0017
