      subroutine plafps ( ipict, xminrd, xmaxrd, yminrd, ymaxrd, &
                          numdigits )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 02-09-2013 Extra parameter numdigits
!        version  1.1  date 31-05-2004 Correction for colours
!        version  1.0  date 17-05-2003
!
!   copyright (c) 2003-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read a sepran neutral plot file and translate the contents to
!     postscript format.
!     Print to a postscript file
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     read
!     postscript
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulemach
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ipict, numdigits
      real xminrd, xmaxrd, yminrd, ymaxrd

!     ipict          i    Picture number (also file number)
!     numdigits      i    Number of digits in plot file
!     xmaxrd         i    Maximum x-value read in file
!     xminrd         i    Minimum x-value read in file
!     ymaxrd         i    Maximum y-value read in file
!     yminrd         i    Minimum y-value read in file
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      character * 60 namefl
      integer maxpol, maxchr, ios, ichois, j, nchar, icolor, npolyg, &
              iref, lastps, ierrtn
      real size, xoffset, yoffset
      parameter ( maxpol = 100, maxchr = 100, size = 72/3.0, &
                  xoffset = 15.0, yoffset = 20.0 )
      real x, y, angle, height, polyg(2*maxpol)
      character string*(maxchr)
      logical finished

!     angle          angle of rotation
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     height         Height of a plot, when <0 then in cm, else
!                    dimensionless
!     ichois         Choice parameter, see plposx
!     icolor         Sequence number of colour
!     ierrtn         Error return code with the following meaning:
!                    0: No error has been found
!                    1: The file with reference number |irefpl| is empty
!                    2: An incorrect record has been read in the plot file
!                    3: The plot file does not end with the correct record
!                    4: The plot file contains a string with too much characters
!                    5: The plot file contains a polygon consisting of too many
!                       points
!     ios            IOSTAT variable, returns from reading or writing
!     iref           File reference number
!     j              Counting variable
!     lastps         Last non-blank position in namefl
!     maxchr         Maximum number of characters allowed in a string
!                    Length of string
!     maxpol         Maximum number of polygon points allowed
!                    Dimension for array POLYG
!     namefl         Name of default SEPRAN neutral plotting file
!     nchar          Number of characters in a word
!     npolyg         Number of points in a polygon
!     polyg          Array containing the co-ordinates of the point in the
!                    polygon in the following way:
!                    polyg(1,i) contains the x-coordinate of the first
!                    point
!                    polyg(2,i) contains the y-coordinate of the first
!                    point
!     size           integer denoting the size of the matrix
!     string         Word to be analyzed
!     x              x co-ordinate of point
!     xoffset        offset in x direction
!     y              y co-ordinate of point
!     yoffset        offset in y direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PLCOL0         Routine to look at color and set it if changed since last
!                    call
!     PLCOLI         sets the machine independent color table
!     PLPOS3         Plot symbols in postscript
!     PLPOS6         General plot commands in postscript, except special ones
!     PLPOS7         Plot text in postscript
!     PLPOS8         Plot coloured lines or filled regions in postscript
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     184   Error in opening of plot file
!    1709   Plot file is empty
!    1710   Plot file is incorrect
!    1711   Plot file end with incorrect record
!    1712   Too much characters in a string
!    1713   Too many nodes in a polygon
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plafps' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plafps'

      end if

!     --- construct name of plot file by concatenation of namepl and picture
!         number

      lastps = index ( namepl, ' ' ) - 1
      namefl = ' '
      namefl(1:lastps) = namepl(1:lastps)
      namefl(lastps+1:lastps+1) = '.'

      if ( numdigits==3 ) then

!     --- file name is of the form  name.xxx

         write ( namefl(lastps+2:lastps+4), 100 ) ipict
100      format ( i3.3 )
         lastps = lastps + 4

      else

!     --- file name is of the form  name.xxxx

         write ( namefl(lastps+2:lastps+5), 101 ) ipict
101      format ( i4.4 )
         lastps = lastps + 5

      end if  ! ( numdigits==3 )

!     --- Open plot file

      iref = abs(irefpl)
      if ( irefpl<0 ) then

!     --- irefpl<0, file is unformatted

         open ( unit=iref, file=namefl(1:lastps), iostat=ios, &
                status='old', form='unformatted' )

      else

!     --- irefpl>0, file is formatted

         open ( unit=iref, file=namefl(1:lastps), iostat=ios, &
                status='old' )

      end if

      if ( ios/=0 ) then

!     --- Error on opening of file
!         Presumably the file does not exist

         call errchr ( namefl(1:lastps), 1 )
         call errint ( ios, 1 )
         call errsub ( 184, 1, 0, 1 )
         go to 1000

      end if

!     --- Read and plot PLOT file

      ierrtn = 0

!     --- Read first record

      if ( irefpl<0 ) then
         read (iref,iostat=ios) ichois,x,y,angle,height,nchar
      else
         read (iref,*,iostat=ios) ichois,x,y,angle,height,nchar
      end if

!     --- Check first record

      if ( ios<0 ) then

!     --- ios<0, end of file has been reached

         ierrtn = 1
         go to 1000

      else if ( ios>0 .or. ichois/=1 ) then

!     --- ios>0,or ichois # 1, contents of file incorrect

         ierrtn = 2
         go to 1000

      end if

1000  if ( ierrtn>0 ) then

!     --- Error in PLOT file

         call errchr ( namefl(1:lastps), 1 )
         call errsub ( 1708+ierrtn, 0, 0, 1 )

      end if

!     --- Open plot
!         First set colour table

      call plcoli
      call plpos6 ( ichois, x, y, xminrd*size, yminrd*size, &
                    xmaxrd*size+2*xoffset, ymaxrd*size+3*yoffset )

!     --- Read rest of records until end record or end of file is found

      finished = .false.
      do while ( .not. finished )

!     --- Read next record

         if ( irefpl<0 ) then
            read (iref,iostat=ios) ichois,x,y,angle,height,nchar
         else
            read (iref,*,iostat=ios) ichois,x,y,angle,height,nchar
         end if
         if ( ios<0 ) then

!        --- end of record found before ichois = 5

            ierrtn = 3
            call plpos6 ( 5, x, y, xminrd, yminrd, xmaxrd, ymaxrd )
            go to 1000

         else if ( ios>0 ) then

!        --- wrong record found

            ierrtn = 2
            go to 1000

         end if

         if ( ichois==2 .or. ichois==3 ) then

!        --- ichois = 2 or 3, move with pen down or up

            if ( nchar>0 ) dcolor = nchar
            call plcol0 ( dcolor )
            call plpos6 ( ichois, x, y, xminrd, yminrd, xmaxrd, ymaxrd )

         else if ( ichois==6 ) then

!        --- ichois = 6, plot text

            if ( nchar>100 ) then
               dcolor = nchar/100
               nchar = mod ( nchar,100 )
            end if
            if ( nchar>maxchr ) then
               ierrtn = 4
               nchar = maxchr
            end if
            if ( irefpl<0 ) then
               read (iref,iostat=ios) string(1:nchar)
            else
               read (iref,110,iostat=ios) string(1:nchar)
110            format(a)
            end if

            if ( ios<0 ) then

!           --- end of record found before ichois = 5

               ierrtn = 3
               call plpos6 ( 5, x, y, xminrd, yminrd, xmaxrd, ymaxrd )
               go to 1000

            else if ( ios>0 ) then

!           --- wrong record found

               ierrtn = 2
               go to 1000

            end if
            nchar = nchar+100*dcolor
            call plpos7 ( x, y, string(1:nchar), angle, height, nchar )

         else if ( ichois==7 ) then

!        --- ichois = 7, fill polygon with colour

            icolor = nint(x)
            if (y>0.0) dcolor = nint(y)
            if ( nchar>maxpol ) then
               ierrtn = 5
               nchar = maxpol
            end if
            npolyg = 2*nchar
            if ( irefpl<0 ) then
               read (iref,iostat=ios) (polyg(j),j=1,npolyg)
            else
               read (iref,*,iostat=ios) (polyg(j),j=1,npolyg)
            end if
            if ( ios<0 ) then

!           --- end of record found before ichois = 5

               ierrtn = 3
               call plpos6 ( 5, x, y, xminrd, yminrd, xmaxrd, ymaxrd )
               go to 1000

            else if ( ios>0 ) then

!           --- wrong record found

               ierrtn = 2
               go to 1000

            end if
            call plpos8 ( icolor, nchar, polyg )

         else if ( ichois==8 ) then

!        --- ichois = 8, plot symbol

            if ( nchar>100 ) then
               dcolor = nchar/100
               nchar = mod ( nchar,100 )
            end if
            call plcol0 ( dcolor )
            call plpos3 ( nchar, x, y )

         else if ( ichois==5 ) then

!        --- ichois = 5, close plot

            call plpos6 ( ichois, x, y, xminrd, yminrd, xmaxrd, ymaxrd )
            finished = .true.

         else

!        --- wrong record found (ichois incorrect)

            ierrtn = 2
            go to 1000

         end if

      end do  ! while ( .not. finished )

!     --- Close plot file

      close ( unit = iref )

      call erclos ( 'plafps' )
      if ( debug ) write(irefwr,*) 'End plafps'

      end
