      subroutine plbc01 ( coor, npoint, kmeshc, nelemt, funk, contln, &
                          ncntln, jsmoot, ichois, icolor )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  5.0  date 21-11-2012 Extra parameter icolor
!        version  4.1  date 27-09-2005 New call to plbc03
!        version  4.0  date 22-01-1997 Extra parameter ichois
!
!   copyright (c) 1986-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    driving subroutine for the plotting of smoothed contours
!
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     plot
!     plot_contour
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, nelemt, ncntln, jsmoot, kmeshc(3,*), &
              ichois, icolor
      double precision coor(*), funk(*), contln(*)

!     contln     i    User array of length ncntln + 5 (if used) in which the
!                     user is supposed to have stored the contour levels from
!                     position 6
!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     funk       i    array with function values in the nodes
!     ichois     i    if ichois=4, the labels are not plotted
!     icolor     i    Sequence number of colour
!     jsmoot     i    indicator which type of smoothing is required
!     kmeshc     i    kmesh part c, contains the nodal points corresponding
!                     to the elements
!     ncntln     i    number of contour levels
!     nelemt      i    Number of elements
!     npoint     i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision valmax, q, w, val, f1, f2, f3
      integer i, iel, i1, i2, i3, itel, itelh, lbrpnt, istep, icr3, ik, &
              ikn, icont, ir, ielem1, ielem2, ielem, ja, icr1, icr2
      integer, allocatable :: inodes(:), ihelp(:,:), nbndpt(:), &
                              itria(:), ibrelm(:), ibrpnt(:)
      double precision, allocatable :: coorp(:), coorn(:)

!     coorn          help array for smoothing,
!                    length: 4*npoint
!     coorp          help array for coordinates of final curve
!                    length: 4*npoint
!     f1
!     f2
!     f3
!     i              Counting variable
!     i1
!     i2
!     i3
!     ibrelm         help array for the neighbor elements of the nodes
!                    length: 3*nelemt
!     ibrpnt         help array with pointers for ibrelm
!                    length: npoint+1
!     icont
!     icr1
!     icr2
!     icr3
!     iel
!     ielem          Element number
!     ielem1
!     ielem2
!     ihelp          help array for final curve, &
!                    length: 4*npoint+100
!     ik
!     ikn
!     inodes         help array
!                    length: 2*nelemtt
!     ir
!     istep
!     itel
!     itelh
!     itria          help array to indicate whether a contour goes
!                    through an element (itria(ielem)=1) or not (=0)
!                    length: nelemtt
!     ja
!     lbrpnt
!     nbndpt         help array for all boundary elements
!                    length: 2*npoint
!     q
!     val
!     valmax
!     w
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     PLBC02         fill the element number in the right position of array
!                    ibrelm for point i
!     PLBC03         fill array nbndpt with all the boundary elements
!     PLBC04         fill array itria
!     PLBC05         determine the, at most 2, elements where both nodes i1 and
!                    i2 belong to
!     PLBC06         determine third point of triangle with corners i1 and i2
!     PLBC10         determine the coordinates of the contour lines
!     PLBC11         plot contour with points coorp
!     PLBC12         smooth contour lines using the Shuman technique
!     PLBC13         smooth contour lines using the spline technique
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     471   function array is empty
!    1699   Array too small
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plbc01' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plbc01'
         call ertrace
         write(irefwr,1) 'jsmoot', jsmoot
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      allocate ( inodes(max(10,nelemt)), ihelp(2,2*npoint+50), &
                 nbndpt(2*npoint), itria(nelemt), ibrelm(3*nelemt), &
                 ibrpnt(npoint+1), coorp(4*npoint), coorn(8*npoint), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, nelemt+npoint, 'inodes' )
      if ( ierror/=0 ) go to 1000

!     --- determine max value of contour values

      valmax = 0d0
      do i = 1, npoint
         q = abs(funk(i))
         if ( q>valmax ) goto 110
      end do

      call errsub ( 471, 0, 0, 0 )
      go to 1000

110   do i = 1, ncntln
         valmax = max(abs(contln(i)), valmax)
      end do
      if ( valmax<=0d0 ) valmax = 1d0

!     --- scale funk array

      do i=1, npoint
         if ( funk(i)<rinfin ) funk(i)=funk(i)/valmax
      end do
      do i = 1, ncntln
         contln(i) = contln(i)/valmax
      end do

!     --- fill neighbor arrays of points

      ibrpnt = 0
      do iel=1,nelemt
         i1=kmeshc(1,iel)
         i2=kmeshc(2,iel)
         i3=kmeshc(3,iel)
         ibrpnt(i1)=ibrpnt(i1)+1
         ibrpnt(i2)=ibrpnt(i2)+1
         ibrpnt(i3)=ibrpnt(i3)+1
      end do

!     --- rearrange

      itel=0
      do i=1,npoint
         itelh=itel+1
         itel=itel+ibrpnt(i)
         ibrpnt(i)=itelh
      end do
      ibrpnt(npoint+1)=itel+1
      do i=1,itel
         ibrelm(i)=0
      end do
      do iel=1,nelemt
         i1=kmeshc(1,iel)
         i2=kmeshc(2,iel)
         i3=kmeshc(3,iel)
         call plbc02(ibrelm,ibrpnt,i1,iel)
         call plbc02(ibrelm,ibrpnt,i2,iel)
         call plbc02(ibrelm,ibrpnt,i3,iel)
      end do

!     --- fill nbndpt

      call plbc03 ( ibrpnt, ibrelm, npoint, kmeshc, &
                    lbrpnt, nbndpt )

!     --- run through all contour values

      do istep=1,ncntln
         w=contln(istep)
         do ikn=1,npoint
            val=abs(funk(ikn)-w)
            if ( val<10*epsmac ) then
               if ( w>0.5d0 ) then
                  funk(ikn)=w+10*epsmac
               else
                  funk(ikn)=w-10*epsmac
               end if
            end if
         end do
         call plbc04 ( kmeshc, nelemt, funk, w, itria )

!        --- arrange ihelp

         icont=0
         do ir=1,lbrpnt
            i1=nbndpt(2*ir-1)
            i2=nbndpt(2*ir  )
            f1=funk(i1)-w
            f1=f1/abs(f1)
            f2=funk(i2)-w
            f2=f2/abs(f2)
            if ( f1*f2<0d0 ) then

!           --- follow until next boundary

               call plbc05 ( ibrelm, ibrpnt, i1, i2, ielem1, ielem2 )
               if ( itria(ielem1)==1 ) then

!              --- start

                  ihelp(1,icont+1)=0
                  ihelp(2,icont+1)=0
                  ihelp(1,icont+2)=i1
                  ihelp(2,icont+2)=i2
                  icont=icont+2
                  itria(ielem1)=0

!                 --- fill array ihelp

                  call plbc06 ( ihelp, icont, ibrelm, ibrpnt, itria, &
                                kmeshc, ielem1, i1, i2, funk, w, &
                                npoint )
                  if ( ierror/=0 ) go to 1000
               end if
            end if
         end do

!        --- determine whether array itria is empty or not
!            ( look for internal contours )

         do ielem = 1, nelemt
            ja=itria(ielem)
            ielem1=ielem
            if ( ja==1 ) then
               ihelp(1,icont+1)=-10
               ihelp(2,icont+1)=-10
               icont=icont+1
               i1=kmeshc(1,ielem1)
               i2=kmeshc(2,ielem1)
               i3=kmeshc(3,ielem1)
               f1=funk(i1)-w
               f1=f1/abs(f1)
               f2=funk(i2)-w
               f2=f2/abs(f2)
               if ( f1*f2<0d0 ) then
                  icont=icont+1
                  ihelp(1,icont)=i1
                  ihelp(2,icont)=i2
               else
                  icont=icont+1
                  ihelp(1,icont)=i1
                  ihelp(2,icont)=i3
               end if
               if ( icont>2*npoint+50 ) then
                  call errint ( 2*icont, 1 )
                  call errint ( 4*npoint+100, 2 )
                  call errchr ( 'ihelp', 1 )
                  call errsub ( 1699, 2, 0, 1 )
                  go to 1000
               end if

290            itria(ielem1)=0
               i1=ihelp(1,icont)
               i2=ihelp(2,icont)
               icr1=kmeshc(1,ielem1)
               icr2=kmeshc(2,ielem1)
               icr3=kmeshc(3,ielem1)
               if ( icr1/=i1 .and. icr1/=i2 ) i3=icr1
               if ( icr2/=i1 .and. icr2/=i2 ) i3=icr2
               if ( icr3/=i1 .and. icr3/=i2 ) i3=icr3
               f1=funk(i1)-w
               f1=f1/abs(f1)
               f3=funk(i3)-w
               f3=f3/abs(f3)
               if ( f1*f3<0d0 ) then
                  i2=i3
               else
                  i1=i3
               end if

!              --- fill ihelp

               icont=icont+1
               if ( icont>2*npoint+50 ) then
                  call errint ( 2*icont, 1 )
                  call errint ( 4*npoint+100, 2 )
                  call errchr ( 'ihelp', 1 )
                  call errsub ( 1699, 2, 0, 1 )
                  go to 1000
               end if
               ihelp(1,icont)=i1
               ihelp(2,icont)=i2
               call plbc05(ibrelm,ibrpnt,i1,i2,ielem1,ielem2)
               if ( itria(ielem1)==1 ) goto 290
               if ( itria(ielem2)==1 ) then
                  ielem1=ielem2
                  goto 290
               end if
            end if
         end do
         do ik=1,10
            inodes(ik)=0
         end do

!        --- fill arrays coorp and inodes

         call plbc10 ( funk, ihelp, icont, coorp, inodes, w, coor, &
                       npoint )
         if ( ierror/=0 ) go to 1000

!        --- smooth results

         if ( jsmoot==1 ) then
            call plbc12 ( inodes, coorp, coorn, npoint )
            if ( ierror/=0 ) go to 1000
         end if
         if ( jsmoot>=2 ) then
            call plbc13 ( jsmoot, inodes, coorp, coorn, npoint, istep )
            if ( ierror/=0 ) go to 1000
         end if

!        --- plot results

         if ( jsmoot<2 ) then
            call plbc11 ( inodes, coorp, istep, npoint, ichois, icolor )
            if ( ierror/=0 ) go to 1000
         end if

      end do

!     --- rescaling of funk and contln

      do i=1,npoint
         if (funk(i)<rinfin) funk(i)=funk(i)*valmax
      end do
      do i=1,ncntln
         contln(i)=contln(i)*valmax
      end do

      deallocate ( inodes, ihelp, nbndpt, itria, ibrelm, ibrpnt, &
                   coorp, coorn, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'inodes' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plbc01'

      end if  ! ( debug )
      call erclos ( 'plbc01' )

      end
