      subroutine plcn01 ( iplots, ndim, ncont )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 24-01-2015 Use prcompncntln
!        version  2.1  date 05-12-1999 Extension of levels
!        version  2.0  date 04-03-1999 New parameter list
!
!   copyright (c) 1992-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Put the parameters from array IPLOTS into common celint
!   Compute number of contour lines
! **********************************************************************
!
!                       KEYWORDS
!
!     plot_contour
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iplots(*), ncont, ndim

!     iplots  i     Integer control array in which the user defines which
!                   choices he wants for this subroutine
!                   IPLOTS must be filled  by the user in the following way:
!                   Position:
!                    1: Not used
!                    2: (CHOICE) defines the type of contour lines or levels
!                       required. Possible values for CHOICE are:
!                       1:  Contour lines will be drawn. These lines are
!                           provided with a numbered label.
!                           In the case of a 3D grid, contour lines are plotted
!                           on the outer surface only
!                       2:  Contour levels will be coloured, i.e. the interior
!                           space between contour levels as well as the region
!                           smaller than the smallest contour level and the
!                           region larger than the largest contour level is
!                           filled with a specific colour for that level.
!                       3:  See 1, however, not only are the contour levels
!                           plotted, but also the co-ordinates of these contour
!                           lines are outputed in the arrays COORCN and NCOORC
!                           This possibility is only available in 2D
!                    3: (NUMBER) the number'th physical unknown in each node is
!                       considered When this unknown does not appear in a nodal
!                       point, the point is skipped
!                       When array ISOL corresponds to a complex array, &
!                       NUMBER=2*i-1 corresponds to the real part of unknown i
!                       and NUMBER=2*i to the imaginary part of unknown i
!                    4: (NCNTLN) Parameter indicating the number of contour
!                       levels to be used. Possible values of ncntln are:
!                      >0:  There are ncntln contour levels that must be
!                             stored in array contln from position 6
!                       = 0:  The program computes the contour levels by
!                             dividing the range of the function between maximum
!                             and minimum value into 10 parts in the case of
!                             CHOICE = 1 or 3 and in 20 parts if CHOICE=2
!                      <0>-1000: The program computes the contour levels by
!                             dividing the range of the function between maximum
!                             and minimum value into |NCNTLN| - 1 parts. The
!                             values are chosen such that rounded values of the
!                             contour levels appear
!                      <-1000: The program computes the contour levels by
!                             dividing the range of the function between maximum
!                             and minimum value into |NCNTLN + 1000| - 1 parts.
!                       -10000: separate levels are used for the positive values
!                               as well as for the negative values
!                               The zero level is always plotted
!                             The number of positive levels is stored in pos. 23
!                             The number of negative levels is stored in pos. 24
!                       -10001: see -10000, however, now the values are rounded
!                               as well as for the negative values
!                    5: (JINDCOL) Parameter indicating the type of colouring to
!                       be used for the contour lines or levels:
!                       Possible values:
!                       0:  The standard colours are used, i.e. standard colour
!                           contour lines in the case of CHOICE=1 or CHOICE=3
!                           and colour numbers 1 to number_of_contour_levels in
!                           the case of CHOICE=2.
!                       1:  The user defines the colour numbers explicitly
!                           through array INDCOL. This possibility is
!                           moment only available for CHOICE=2
!                       2:  The user defines the minimum colour number
!                           explicitly through array INDCOL. This possibility
!                           is only available for CHOICE=2
!                       3:  The user defines the minimum and maximum colour
!                           number explicitly through array INDCOL. This
!                           possibility is only available for CHOICE=2
!                       >0:  The visible parts of the contour lines get the
!                           colour sequence number JINDCOL CHOICE=1 or 3
!                    6: (PROJECT) choice parameter for type of projection
!                       (3D only)
!                       0:  parallel projection plot (View point automatic)
!                       1:  perspective projection plot (View point automatic)
!                       10: parallel projection plot (view point given by the
!                           user in RPLOTS(4-6) )
!                       11: perspective projection plot (view point given by the
!                           user in RPLOTS(4-6) )
!                    7: (JSMOOTH) choice parameter for type of smoothing to be
!                       used for the contour lines. At this moment JSMOOTH is
!                       only implemented for CHOICE=1 or 3 in 2-dimensional
!                       grids.
!                       Possible values:
!                       0:  no smoothing
!                       1:  shuman smoothing
!                       2:  |
!                       3:  |  all spline smoothing : the higher the value the
!                       4:  |  more smoothing and the less interpolation of
!                       5:  |  the values.
!                    8: (IDOUBLE) indication whether the outer boundary is
!                       plotted as a single line (0) or as double line (1)
!                       (2D only)
!                    9: (INNER) indication whether inner boundaries must be
!                       plotted (1) or not (0)
!                       (2D only)
!                   10: (PLOT_CONTOUR) indication if the boundaries of the
!                       contourlevels must be plotted (>0) or not (0). The
!                       value of PLOT_CONTOUR defines the colour.
!                       This possibility is only available in the case CHOICE=2.
!                   11: (TYPE_LEGENDA) defines the type of legenda to be plotted
!                       Possible values:
!                       1:  No legenda
!                       2:  Standard legenda; the text is plotted at the right
!                           side of the picture.
!                       3:  Horizontal legenda; the text plotted below the
!                           picture. Only minimum and maximum values are given.
!                           Only available for CHOICE=2
!                   12: (TYPE_SCALES) defines whether the scaling factors are
!                       plotted (1) or not (2)
!                   13: (ISKIP) indicates if element groups must be excluded
!                       (>0) or not (0)
!                       If ISKIP>0, the value of ISKIP defines the first
!                       position in INPUT where information about the element
!                       groups to be excluded is stored
!                   14: IDEFFM   Indicates if default sizes of the plot are used
!                       (IDEFFM=0) or that the user gives these values
!                       in RPLOTS(1) and RPLOTS(2) (IDEFFM=1)
!                       If IDEFFM=2 the user must also fill RPLOTS(3) with a
!                       premultiplication factor for the solution
!                   15: (IMESH) indicates if the mesh must be plotted in the
!                       picture (>0) or not (0)
!                       If IMESH>0, the value of IMESH defines the colour
!                       sequence number with which the mesh must be plotted.
!                       In 3D only the visible parts of the mesh are plotted
!                       with colour IMESH
!                   16: (INVIS_MESH) indicates if the invisible part of the
!                       mesh must be plotted in the picture (>0) or not (0)
!                       If INVIS_MESH>0, the value of INVIS_MESH defines the
!                       colour sequence number with which the invisible parts
!                       of the mesh must be plotted. (CHOICE=1 and 3D only)
!                   17: (INVIS_CONT) indicates if the invisible part of the
!                       contour lines must be plotted in the picture (>0) or not
!                       (0). If INVIS_CONT>0, the value of INVIS_CONT defines
!                       the colour sequence number with which the invisible
!                       parts of the contours must be plotted.
!                       (CHOICE=1 and 3D only)
!                   18: (BACK_COLOUR) indicates if back ground lines must
!                       plotted in the picture (>0) or not (0)
!                       If BACK_COLOUR>0, the value of BACK_COLOUR defines the
!                       colour sequence number with which the back ground lines
!                       must be plotted. (CHOICE=1 and 3D only)
!                   19: (CONS_COLOUR) indicates if constituting lines must be
!                       plotted in the picture (>0) or not (0)
!                       If CONS_COLOUR>0 and<100, the value of CONS_COLOUR
!                       defines the colour sequence number with which the
!                       constituting lines must be plotted.
!                       If CONS_COLOUR>100 only the visible part of the
!                       constituting lines is plotted, the value of
!                       CONS_COLOUR-100 defines the colour sequence number
!                       with which the constituting lines must be plotted.
!                       This possibility is only available for CHOICE=1 and 3D
!                   20: (ITEXT) indicates if text must be plotted in the
!                       picture.
!                       Possible values:
!                       0:  No text at all
!                       1:  Extra text at bottom
!                       2:  Extra text at bottom and top
!                   21: (ICOMPT) indicates how the contour levels must be
!                       computed from NCNTLN in the case NCNTLN<=0
!                       0:  The minimum and maximum level are computed
!                       1:  The minimum level is given in contln(6), the
!                           maximum level must be computed
!                       2:  The maximum level is given in contln(7), the
!                           minimum level must be computed
!                       3:  The minimum level is given in contln(6), the
!                           maximum level in contln(7)
!                   23: (NLEV_POS) indicates the number of positive levels
!                       for ncntln = -10000 or -10001
!                   24: (NLEV_NEG) indicates the number of negative levels
!                       for ncntln = -10000 or -10001
!     ncont   o     Actual number of contour levels
!     ndim    i     Dimension of the space.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ichois, iconcl, idoubl, iinvcn, iinvms, inner, iplotc, &
              iprojt, ityplg, jindcl, jsmoot, ncntln

!     ichois     Choice parameter See IPLOTS(2)
!     iconcl     See IPLOTS(19)
!     idoubl     See IPLOTS(8)
!     iinvcn     See IPLOTS(17)
!     iinvms     See IPLOTS(16)
!     inner      See IPLOTS(9)
!     iplotc     See IPLOTS(10)
!     iprojt     See IPLOTS(6)
!     ityplg     See IPLOTS(11)
!     jindcl     See IPLOTS(5)
!     jsmoot     Parameter indicating the amount of smoothing
!                Minimum value 0, maximum value 5
!     ncntln     Parameter to compute the number of contour levels
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRCOMPNCNTLN   Compute number of contour lines
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     106   jsmoot has wrong value
!    1830   JINDCOL has wrong value for coloured levels
!    1838   PROJECT has wrong value in 3D
!    1839   IDOUBLE has wrong value in 3D
!    1840   IINNER has wrong value in 3D
!    1841   PLOT_CONTOUR has wrong value in case of contour lines to be plotted
!    1842   TYPE_LEGENDA has wrong value in case of contour lines to be plotted
!    1843   INVIS_MESH has wrong value in case of contour levels to be plotted
!    1844   INVIS_CONT has wrong value in case of contour levels to be plotted
!    1845   CONS_COLOUR has wrong value in case of 3D
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Put parameters in common CELINT
!   Check parameters
!   Compute NCONT
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plcn01' )

!     --- Fill parameters from IPLOTS

      ichois = iplots(2)
      ncntln = iplots(4)
      jindcl = iplots(5)
      iprojt = iplots(6)
      jsmoot = iplots(7)
      idoubl = iplots(8)
      inner  = iplots(9)
      iplotc = iplots(10)
      ityplg = iplots(11)
      iinvms = iplots(16)
      iinvcn = iplots(17)
      iconcl = iplots(19)

!     --- Check parameters

      if ( ichois==2 ) then

!     --- Colour levels

         if ( jindcl>3 ) then
            call errint ( jindcl, 1 )
            call errsub ( 1830, 1, 0, 0 )
         end if
         if ( jsmoot>0 ) then
            call errint ( jsmoot, 1 )
            call errint ( 0, 2 )
            call errwar ( 106, 2, 0, 0 )
            jsmoot = 0
         end if
         if ( iinvms>0 ) then
            call errint ( iinvms, 1 )
            call errwar ( 1843, 1, 0, 0 )
            iinvms = 0
         end if
         if ( iinvcn>0 ) then
            call errint ( iinvcn, 1 )
            call errwar ( 1844, 1, 0, 0 )
            iinvcn = 0
         end if

      else

!     --- Contour lines

         if ( iplotc>0 ) then
            call errint ( iplotc, 1 )
            call errwar (1841, 1, 0, 0 )
            iplotc = 0
         end if
         if ( ityplg==3 ) then
            call errint ( ityplg, 1 )
            call errwar (1842, 1, 0, 0 )
            ityplg = 2
         end if
      end if
      if ( ndim==3 ) then

!     --- NDIM = 3

         if ( iprojt>1 .and. iprojt<10 ) then
            call errint ( iprojt, 1 )
            call errsub ( 1838, 1, 0, 0 )
         end if
         if ( jsmoot>0 ) then
            call errint ( jsmoot, 1 )
            call errint ( 0, 2 )
            call errwar ( 106, 2, 0, 0 )
            jsmoot = 0
         end if
         if ( idoubl>0 ) then
            call errint ( idoubl, 1 )
            call errwar ( 1839, 1, 0, 0 )
            idoubl = 0
         end if
         if ( inner>1 ) then
            call errint ( 1, 1 )
            call errwar ( 1840, 1, 0, 0 )
            inner = 0
         end if

      else

!     --- NDIM = 2

         if ( iinvms>0 ) then
            call errint ( iinvms, 1 )
            call errwar ( 1843, 1, 0, 0 )
            iinvms = 0
         end if
         if ( iinvcn>0 ) then
            call errint ( iinvcn, 1 )
            call errwar ( 1844, 1, 0, 0 )
            iinvcn = 0
         end if
         if ( iconcl>0 ) then
            call errint ( iconcl, 1 )
            call errwar ( 1845, 1, 0, 0 )
            iconcl = 0
         end if
      end if

!     --- Compute ncont

      call prcompncntln ( ncntln, ncont, ichois, iplots )

      iplots(7)  = jsmoot
      iplots(8)  = idoubl
      iplots(9)  = inner
      iplots(16) = iinvms
      iplots(17) = iinvcn
      iplots(10) = iplotc
      iplots(11) = ityplg
      iplots(19) = iconcl
      call erclos ( 'plcn01' )
      end
