      subroutine plfl02 ( xin, yin, in, esol, nelem, kelem, coor, itel, &
                          totl, partout, maxsteps )
! ======================================================================
!
!        programmer    Arno van Duin
!        version  2.2  date 25-04-2010 New call to plaf05
!        version  2.1  date 06-10-2001 New call to pltr02
!        version  2.0  date 06-04-1996 Extra parameters partout and maxsteps
!
!   copyright (c) 1991-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Calculate field line starting in point (xin,yin) in electric field
!    given by esol.
! **********************************************************************
!
!                       KEYWORDS
!
!     particle_tracking
!     print
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(*), esol(*), totl, xin, yin, partout(3,*)
      integer in, itel(*), kelem(*), nelem, maxsteps

!     coor       i    KMESH part i, see P.G. 13.1.6.
!     esol       i    electric field
!     in         i    indicator checking on inward(2) or outward(3) flux
!     itel       o    work array
!     kelem      i    KMESH part m, see P.G. 13.1.8.
!     maxsteps   i    Maximum number of positions that may be stored in partout
!     nelem      i    number of elements
!     partout    o    Array containing the actual information concerning the
!                     particle trajectories
!                     partout(1,i) contains t_i
!                     partout(2-3,i) contains x_i and y_i
!     totl       o    total length of field line
!     xin        i    x-coordinate of starting point
!     yin        i    y-coordinate of starting point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision afst, det, dref, dx, dy, eta, ski, t, tend, u(3), &
                       up(3), v(3), vp(3), x, xl(3), y, yl(3)
      integer i, i1, i2, i3, ielem, io1, io2, io3, &
              iresul, ital, itrap, istag, nsteps, icallfn
      logical repeat

!     afst       length of field line in element
!     det        determinant of transformation to local coordinates
!     dref       reference length ( || (dx,dy) || / 200 )
!     dx         xmax - xmin
!     dy         ymax - ymin
!     eta        local y-coordinate
!     i          loop variable
!     i1         nodal point number of first nodal point of element
!     i2         nodal point number of second nodal point of element
!     i3         nodal point number of third nodal point of element
!     icallfn    Indicates if for each position in the particle trace
!                a user subroutine FNPARTIC must be called (1) or not (0)
!     ielem      element number
!     io1        nodal point number of first nodal point of previous element
!     io2        nodal point number of second nodal point of previous element
!     io3        nodal point number of third nodal point of previous element
!     iresul     checks whether point lies in element ( 0 : no, 1 : yes )
!     istag      indicator for stagnation
!     ital       number of points that this element has in common with
!                previous one
!     itrap      trap parameter, number of times it is allowed to enter the same
!                element
!     nsteps     number of points in a particle trajectory
!     repeat     while-loop variable
!     ski        local x-coordinate
!     t          timevariable, dummy for pltr02
!     tend       timeout variable
!     u          array with velocities in x-direction in nodes of element
!     up         local velocities in x-direction in nodes of element
!     v          array with velocities in y-direction in nodes of element
!     vp         local velocities in y-direction in nodes of element
!     x          x-coordinate of point of track
!     xl         array of x-coordinates of nodes of element
!     y          y-coordinate of point of track
!     yl         array of y-coordinates of nodes of element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    error subroutine, deletes routine name from string
!     EROPEN    error subroutine, concatenates routine name to string
!     ERREAL    error subroutine, fills real into array for error message
!     ERRWAR    error subroutine, writes warning to standard output file
!     PLAF05    Plot polygon
!     PLTR01    calculate local coordinates
!     PLTR02    calculate track through this element
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     936     particle trapped in element : same element entered for the
!                                           20-th time
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Initialise some parameters
!     While not end of track do
!        Determine element that contains the point
!        If no point found then end of track
!        If same element is entered too many times then end of track
!        Get global velocities
!        Calculate local velocities
!        Calculate track within this element
!     Endwhile
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plfl02' )

!     --- initialise some parameters, go to starting point for plotting

      totl   = 0d0
      icallfn = 0
      itrap  = 20
      istag  = 0
      tend   = rinfin
      t  = 0d0
      dx = xmax - xmin
      dy = ymax - ymin
      x = xin
      y = yin
      dref = sqrt( dx * dx + dy * dy ) / 2d2
      do 10 i = 1 , nelem
         itel( i ) = 0
10    continue
      io1 = 0
      io2 = 0
      io3 = 0
      nsteps = 1
      partout(1,nsteps) = t
      partout(2,nsteps) = x
      partout(3,nsteps) = y
      repeat = .true.

!     --- while not end of track do

20    if ( repeat ) then

!     --- determine elementnumber of triangle

         ielem = 0

!        --- check all neighbour elements

         do 30 i = 1, nelem
            i1 = kelem(3*i-2)
            i2 = kelem(3*i-1)
            i3 = kelem(3*i  )

!           --- Check whether element is neighbour element

            ital = 0
            if ( i1==io1 .or. i1==io2 .or. i1==io3 ) &
               ital = ital + 1
            if ( i2==io1 .or. i2==io2 .or. i2==io3 ) &
               ital = ital + 1
            if ( i3==io1 .or. i3==io2 .or. i3==io3 ) &
               ital = ital + 1
            if ( ital==2 ) then

!           --- if element is neighbour element then calculate local coordinates

               xl(1) = coor(2*i1-1)
               xl(2) = coor(2*i2-1)
               xl(3) = coor(2*i3-1)
               yl(1) = coor(2*i1)
               yl(2) = coor(2*i2)
               yl(3) = coor(2*i3)
               iresul=0
               call pltr01 ( x, y, iresul, xl, yl, ski, eta, det )
               if ( iresul==1 ) then

!              --- if point lies in this element than store this element

                  ielem = i
                  io1   = i1
                  io2   = i2
                  io3   = i3

!                 --- Element found, jump to next section

                  goto 50
               end if
            end if
30       continue

!        --- Element not yet found, &
!            Second action, run through all elements

         ielem = 0
         do 40 i = 1, nelem
            i1 = kelem(3*i-2)
            i2 = kelem(3*i-1)
            i3 = kelem(3*i)
            xl(1) = coor(2*i1-1)
            xl(2) = coor(2*i2-1)
            xl(3) = coor(2*i3-1)
            yl(1) = coor(2*i1)
            yl(2) = coor(2*i2)
            yl(3) = coor(2*i3)
            iresul = 0

!           --- calculate local coordinates

            call pltr01 ( x, y, iresul, xl, yl, ski, eta, det )
            if ( iresul==1 ) then

!           --- if point lies in this element than store this element

               ielem = i
               io1   = i1
               io2   = i2
               io3   = i3

!              --- Element found, jump to next section

               goto 50
            end if
40       continue

!        --- No element found, point outside, track ready

         repeat = .false.
         goto 20
50       continue

!        --- Check whether particle is trapped in element

         itel(ielem) = itel(ielem) + 1
         if ( itel(ielem)>itrap .or. istag==1 ) then
            call erreal ( xl(1), 1 )
            call erreal ( yl(1), 2 )
            call erreal ( xl(2), 3 )
            call erreal ( yl(2), 4 )
            call erreal ( xl(3), 5 )
            call erreal ( yl(3), 6 )
            call errwar ( 936, 0, 6, 0 )
            repeat = .false.
            goto 20
         end if

!        --- Determine global velocities

         if ( in==2 ) then

!        --- positive velocity has been given per point

            u(1) = esol(2*i1-1)
            u(2) = esol(2*i2-1)
            u(3) = esol(2*i3-1)
            v(1) = esol(2*i1)
            v(2) = esol(2*i2)
            v(3) = esol(2*i3)
         else if ( in==3 ) then

!        --- negative velocity has been given per point

            u(1) = -esol(2*i1-1)
            u(2) = -esol(2*i2-1)
            u(3) = -esol(2*i3-1)
            v(1) = -esol(2*i1)
            v(2) = -esol(2*i2)
            v(3) = -esol(2*i3)
         end if

!        --- Compute local velocities

         up(1) =   ( (yl(3)-yl(1))*u(1) - (xl(3)-xl(1))*v(1) ) / det
         up(2) =   ( (yl(3)-yl(1))*u(2) - (xl(3)-xl(1))*v(2) ) / det
         up(3) =   ( (yl(3)-yl(1))*u(3) - (xl(3)-xl(1))*v(3) ) / det
         vp(1) = - ( (yl(2)-yl(1))*u(1) - (xl(2)-xl(1))*v(1) ) / det
         vp(2) = - ( (yl(2)-yl(1))*u(2) - (xl(2)-xl(1))*v(2) ) / det
         vp(3) = - ( (yl(2)-yl(1))*u(3) - (xl(2)-xl(1))*v(3) ) / det

!        --- determine track of particle in this element

         istag = 0
         call pltr02 ( up, vp, ski, eta, t, tend, x, y, xl, yl, &
                       dref, afst, istag, nsteps, partout, maxsteps, &
                       1, 1d-2, icallfn, repeat )
         totl = afst + totl

!        --- endwhile

         goto 20
      end if
      call plaf05 ( 2, dcolor, nsteps, partout, 3 )
      call erclos ( 'plfl02' )
      end
