      subroutine plotm2par ( plotfm, yfact, nprocessors )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 29-05-2014 Extension of timebefore
!        version  1.1  date 25-11-2012 Long integers
!        version  1.0  date 14-02-2003
!
!   copyright (c) 2003-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot two-dimensional mesh and give each parallel block a different colour
!     If this subroutine is changed, also chane PLOTM2BF
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     plot_mesh
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nprocessors
      double precision plotfm, yfact

!     nprocessors    i    Number of processors that is to be used for
!                         parallel processing
!                         Defines also the number of submeshes
!     plotfm         i    Plotting factor. When PLOTFM<0, then the length of
!                         the plot is computed by - (xmax-xmin) * PLOTFM
!                         and the width by - (ymax-ymin) * YFACT * PLOTFM
!                         else the length of the plot is equal to PLOTFM cm
!                         When by some mistake the user has given | PLOTFM | a
!                         value smaller than 1 cm, then the length is chosen
!                         equal to the default length stored in common block
!                         consta
!     yfact          i    Multiplication for the y-co-ordinates. With this
!                         parameter the user can influence the scale in
!                         y-direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical ncontr, part, plboun, plrenm, fillcl, bouncurv
      integer ichois, ikader, imark, i, iscolr, nelemloc, &
              iuser(1), elements(nelem)
      integer, pointer :: elementsp(:), points(:)
      double precision :: timebefore(2)

!     bouncurv       if true the 2D boundaries are plotted using icurvs/curves
!                    otherwise using kmeshm and coor
!     elements       Array of length nelem indicating if element ielem
!                    must be plotted (>0) or not (0)
!     elementsp      Array of size nelemloc containing the element
!                    numbers corresponding to the local mesh
!     fillcl         Indicates if the elements must be filled with a colour
!                    (true) or that only the boundaries must be plotted (false)
!                    If fillcl = true, the colour is given by dcolor, the
!                    boundaries get the default colour. Otherwise the colour of
!                    the boundaries is defined by dcolor
!     i              Counting variable
!     ichois         dummy
!     ikader         Help variable to store jkader temporarily
!     imark          jmark mod 10
!     iscolr         Help variable to store the value of dcolor temporarily
!     iuser          dummy
!     ncontr         indication whether the elements are contracted (false)
!                    or not (true)
!     nelemloc       Number of new elements in the original one
!     part           indication whether all elements are plotted (true)
!                    or that some are skipped (false)
!     plboun         Indicates if the boundaries must be plotted (true) or not
!     plrenm         Indicates if the renumbered nodes must be plotted (true)
!                    or not
!     points         Array of size npointloc containing the node
!                    numbers corresponding to the local mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERSETTIME      Get the CPU time at start of subroutine
!     INITS5         Compute parameters for the plotting subroutine
!     MSHPARINV      Compute the inverse mapping
!                    (i.e. mapping from global to local mesh)
!     PL0003         Open plot, plots frame work and computes plotting
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLELM1PAR      Plot elements in a one or two-dimensional mesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotm2par' )
      if ( itime>0 ) call ersettime ( timebefore )
      iscolr = dcolor
      ichois = 0
      call inits5 ( iuser, ncontr, jmark, imark, &
                    part, plboun, ichois, plrenm, fillcl )

      bouncurv = .false.

      if ( part .and. .not. plboun .and. kelmh>0 ) then

!     --- kmesh part h has been filled
!         Check if icurvs and curves have been filled

         if ( kmeshh(3,1)>0 .and. kmeshh(5,1)>0 ) bouncurv = .true.

      end if

!     --- Set starting addresses

      if ( ierror/=0 ) go to 1000
      textx = 'x'
      texty = 'y'
      if ( bouncurv ) then

!     --- Use arrays curves and icurvs

         call pl0003 ( ikader, 1, plotfm, yfact, 10 )

      else

!     --- Use arrays coor and kmeshm

         call pl0003 ( ikader, 2,  plotfm, yfact, 0 )

      end if

!     --- plot elements

      do i = 1, nprocessors

!     --- Fill array elements for one processor and set color

         elementsp => kmhpar(i)%elempointers

!        --- Store inverse mapping in elements

         nelemloc = size ( elementsp )
         call mshparinv ( elementsp, nelemloc, elements, nelem )
         if ( ierror>0 ) go to 1000

         dcolor = iscolr + 3*i
         call plelm1par ( elements )

      end do

!     --- close plot dataset

      call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )

1000  call erclmn ( 'plotm2par', timebefore )

      jkader = ikader
      dcolor = iscolr

      end subroutine plotm2par
