      subroutine plotm3obst ( iplots, plotfm, yfact, rplots )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.2  date 29-05-2014 Extension of timebefore
!        version  1.1  date 15-11-2012 Long integers
!        version  1.0  date 18-04-2002
!
!   copyright (c) 2002-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to plot a 3-D obstacle mesh with hidden lines removed
!
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     plot_mesh
!     hidden_line
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iplots(*)
      double precision plotfm, yfact, rplots(3)

!     iplots         i    Integer input array for plot subroutine
!                          1:  Position of last item that has been filled by
!                              the user
!                          6:  Plot outer boundaries double (1) or not (0)
!                              In case of coloured 3D plot: plot boundaries of
!                              elements
!                          7:  Skip element groups (>0) or not (0)
!                          8:  Plot renumbered node numbers (1) or original
!                              ones (0)
!                              Default: 0
!                          9:  Make a filled coloured mesh plot (1) or a line
!                              plot (0)
!                              Default: 0
!                         10:  Plot axis (1) or not (0)
!                              Default: 0
!                         11:  Type of projection. Possible values:
!                               0 : perspective projection plot
!                               1 : parallel projection plot
!                              10 : perspective projection plot
!                                   (View point automatic)
!                              11 : parallel projection plot
!                                   (View point automatic)
!                              Default: 10
!                         12:  Colour number of farthest elements
!                              Default: 1
!                         13:  Colour number of nearest elements
!                              Default: 16
!                         14:  Basic coordinate system for plots
!                                1 :  x - y - z
!                                2 :  z - x - y
!                                3 :  y - z - x
!                         15:  numrotations:  Number of rotations
!                              Default value: 1
!                         16:  Use volumes (>0) or not (0)
!                              Default: 0
!                         17:  Plot elements per layer (1) or not (0)
!                         18:  idefstart, defines how the start level must
!                              be computed
!                              1: Start with set of nodes with largest
!                                 distance to view point
!                              2: Start with set of surfaces indicated in
!                                 positions 12 and 13
!                         19:  ifirst, first surface number
!                         20:  ilast, last surface number
!     plotfm         i    Plotting factor. When PLOTFM<0, then the length of
!                         the plot is computed by - (xmax-xmin) * PLOTFM
!                         and the width by - (ymax-ymin) * YFACT * PLOTFM
!                         else the length of the plot is equal to PLOTFM cm
!                         When by some mistake the user has given | PLOTFM | a
!                         value smaller than 1 cm, then the length is chosen
!                         equal to the default length stored in common block
!                         consta
!     rplots         i    Real input array for plot subroutine
!     yfact          i    Multiplication for the y-co-ordinates. With this
!                         parameter the user can influence the scale in
!                         y-direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer npointlc, nelemlc, inpelm, iorien, &
              neleml, ichrec, jtimessav, &
              irotation, jv, i, numrotations, lenrl, iobst
      double precision viewpoint(3), viewp(3)
      integer, allocatable :: itria(:,:)
      double precision, pointer :: procor(:,:)
      double precision :: timebefore(2)

!     i              Counting variable
!     ichrec         Reduced value of ichois, i.e. without the part iorient
!     inpelm         number of points per element
!     iobst          Obstacle sequence number
!     iorien         Indicator for coordinate basis
!                    1 :  x - y - z
!                    2 :  z - x - y
!                    3 :  y - z - x
!     irotation      Loop variable for rotation
!     itria          Contains the triangles
!     jtimessav      Help variable to save the value of jtimes
!     jv             Indicates if the view point has been filled (1)
!                    or not (0)
!     lenrl          Number of reals in one row of rmeshx
!     neleml         Number of line elements
!     nelemlc        Number of elements
!     npointlc       Number of nodal points
!     numrotations   Number of rotations
!     procor         coordinate array for projected points
!                    procor(1,i) contains the x-coordinate of the
!                    projected point
!                    procor(2,i) contains the y-coordinate of the
!                    projected point
!                    procor(3,i) contains the distance of point to the
!                    view-point i is the point number
!     viewp          Array containing the first view point
!     viewpoint      Array containing actual view point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLMN         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     PLFILLVIEWP    Fill array viewpoint depending on the rotation
!     PLTHID         Plot 3D mesh with hidden lines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     581   The mesh is not three-dimensional.
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Subdivide mesh into surface mesh consisting of triangles and
!     quadrilaterals only
!     Compute number of points for hidden lines routine
!     Check whether line elements are present
!     Make space reservation for the work arrays of PLTHID
!     Copy array coor in case of quadrilaterals to help array
!     Set pointers in ibuffr
!     Copy mesh
!     Make plot with hidden lines
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotm3obst' )
      if ( itime>0 ) call ersettime ( timebefore )

!     --- Subdivide mesh into surface mesh consisting of triangles and
!         quadrilaterals only

      ichrec = iplots(14)
      iorien = iplots(11)
      numrotations = iplots(15)

      jtimessav = jtimes

      if ( ndim/=3 ) then
         call errint ( ndim, 1 )
         call errsub ( 581, 1, 0, 0 )
      end if

      lenrl = 3*ndim
      inpelm = 3
      iobst = 1

      nelemlc  = kmeshx(11,iobst)
      npointlc = kmeshx(12,iobst)

      neleml = 0

      allocate ( itria(inpelm,nelemlc), procor(3,npointlc), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, nelemlc*inpelm+3*npointlc, 'itria' )
      if ( ierror/=0 ) go to 1000

!     --- Copy mesh

      itria = kmeshcobst

!     --- Make plot

      jv = 1 - ichrec / 10

!     --- Loop over the rotations

      if ( numrotations>1 ) then

!     --- More than one rotation, separate pictures required

         if ( jtimes>10 ) jtimes = jtimes-10
         if ( jtimes==2 .or. jtimes==3 ) then
            jtimes = 3
         else
            jtimes = 0
         end if

      end if

      do irotation = 1, numrotations

         if ( numrotations==1 ) then
            jtimes = jtimessav
         else if ( irotation==numrotations .and. &
                   jtimessav>=10 ) then
            jtimes = 11
         end if
         if ( irotation==1 .and. jv==1 ) then

!        --- First rotation, set viewpoint

            do i = 1, 3
               viewpoint(i) = rplots(i)
            end do

         end if

         call plfillviewp ( irotation, jv, viewp, viewpoint, iorien, &
                            numrotations )

         if ( irotation>1 ) then

!        --- Next rotation, adapt viewp

            if ( ichrec>=10 ) ichrec = ichrec-10

!           --- Copy mesh

            length = inpelm*nelemlc
            itria = kmeshcobst

         end if

         call plthid ( coorobst, npointlc, nelemlc, &
                       inpelm, itria, plotfm, yfact, viewpoint, &
                       procor, ichrec, neleml, itria, iorien )
         if ( irotation==1 .and. jv==0 ) then

!        --- First step and view point has not yet been filled

            do i = 1, 3
               viewp(i) = viewpoint(i)
            end do

         end if

      end do
      jtimes = jtimessav

      deallocate ( itria, procor, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'itria' )
      if ( ierror/=0 ) go to 1000

1000  call erclmn ( 'plotm3obst', timebefore )

      end subroutine plotm3obst
