      subroutine plotsfobst ( coornw, npoint, kmesha, kmeshg, nelgrp, &
                              kmeshc, iplots, neleml, keleml, itricol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 29-05-2014 Extension of timebefore
!        version  1.1  date 25-04-2010 New call to plaf05
!        version  1.0  date 20-07-2001 Copied from plotsf
!
!   copyright (c) 2001-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       PLOTSF plots a 3-D image on a 2-D color-terminal
!       It has been assumed that the projection has been computed before
!       and the the opening of the plot has also been performed
!       This subroutine does not close the plot
!       Dependent on the value of iplots(1) the color of a polygon is
!       determined by
!       (iplots(1)=1) : The distance of barycenter and viewpoint
!       (iplots(1)=2) : The local value of the solution
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     color
!     contour_level
!     plot
!     plot_mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, nelgrp, kmesha(nelgrp), kmeshg(nelgrp), &
              kmeshc(*), iplots(5), neleml, keleml(*), itricol(*)
      double precision coornw(3,npoint)

!     coornw        i     Array of length 3*npoint in which the co-ordinates of
!                         the projected points and the distances to the viewer
!                         are stored
!                         coornw(1,i):  x-coor point i
!                         coornw(2,i):  y-coor point i
!                         coornw(3,i):  distance point i
!     iplots        i     Input array of length 4
!                         This array must be filled as follows:
!                         Pos. 1:  kind of plot
!                                  Possible values:
!                                  1: color is determined by distance, 2-D mesh
!                                     with function or a 3-D mesh
!                                     without function
!                                  2: color is determined by function value, &
!                                     3-D mesh and function
!                              2:  choice parameter for type of projection
!                                  0 : parallel projection plot
!                                  1 : perspective projection plot
!                                  10: parallel projection plot
!                                      (View point automatic)
!                                  11: perspective projection plot (View point
!                                      automatic)
!                              3:  color number of farthest triangle
!                                  (1<=x<=256)
!                              4:  color number of nearest triangle
!                                  (x<=y<=256)
!     itricol        i    Indicates if an elements is internal (3), external (1)
!                         or intersecting the obstacle (2)
!     keleml         i    Line elements array
!     kmesha         i    Contains the number of nodes in each element of
!                         an element group
!                         In the case iplots(1)=2, only linear triangles are
!                         allowed
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshg         i    number of elements in element group
!     neleml         i    Number of line elements
!     nelgrp         i    Number of element groups
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision polyg(2,10), weight(20), dum, factor
      integer          i, j, icolor, nelem, ielgrp, ikelmc, ielem, &
                       inpelm, iielem, iwork1(nelgrp+1,2)
      double precision, allocatable :: work(:)
      integer, allocatable :: iwork(:)
      double precision :: timebefore(2)

!     dum            General double precision dummy variable
!     factor         Help factor for the computation of the colors
!     i              Counting variable
!     icolor         color of the polygon to be plotted
!     ielem          Element number
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     iwork          Work array to store the sorted array after a call of
!                    chsort() at least nelem elements
!     iwork1         Work array to store the accumulated number of elements
!                    in an element group, such that:  iwork1(1,1) = 0, &
!                    number of elements in
!                    group i = iwork1(i+1,1) - iwork1(i,1)
!                    In iwork1(i,2) the starting address of an element
!                    group in kmeshc is stored in exactly the same way
!     j              Counting variable
!     nelem          Number of elements
!     polyg          array with x,y-coordinates for call to plaf05()
!     weight         array to sort vertices from polygon
!     work           a work-array of length nelem (=number of elements)
!                    to make it possible to determine which polygon to
!                    plot first
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLMN         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERSETTIME      Get the CPU time at start of subroutine
!     PLAF05         Plot polygon
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PRSORT         Sort real array for increasing sequence
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       Number of elements in kmeshc
!               nelgrp
!               sum     kmesha(k) * kmeshg(k)
!               k=1
!       A polygon N belongs to element group k, so:
!               N<=iwork1(k+1,2)     and     k<=nelgrp
!       The vertices for this polygon are
!                k-1
!       kmeshc(  sum kmesha(i) * kmeshg(i) + &
!                i=1
!                    ( N - iwork1(k,2) - 1 ) * kmesha(k)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'plotsfobst' )
      if ( itime>0 ) call ersettime ( timebefore )

!     --- Fill array IWORK1

      iwork1(1,1) = 0
      iwork1(1,2) = 0
      dum = 0d0
      do ielgrp = 1, nelgrp
         iwork1(ielgrp+1,1) = iwork1(ielgrp,1) + kmeshg(ielgrp)
         iwork1(ielgrp+1,2) = iwork1(ielgrp,2) + kmeshg(ielgrp) &
                              * kmesha(ielgrp)
      end do

!     --- Determine the number of polygons to plot

      nelem = iwork1(nelgrp+1,1)

      allocate ( iwork(nelem), work(nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, nelem, 'work' )
      if ( ierror/=0 ) go to 1000

!     --- ikelmc = index in kmeshc
!         ielem  = element number

      ikelmc = 0
      ielem = 0

!     --- determine the weight for every polygon in the plot

      do ielgrp = 1, nelgrp
         inpelm = kmesha(ielgrp)
         do iielem = 1, kmeshg(ielgrp)
            ielem = ielem + 1

!           --- maximum of 10 points in one polygon because of weight()

            do i = 1, inpelm
               weight(i) = coornw(3,kmeshc(ikelmc+i))
            end do
            ikelmc = ikelmc + inpelm
            call prsort ( weight, iwork, inpelm )

!           --- determine the weight of a polygon, in case that all vertices
!               are so far away that they will be within the plot

            if ( weight(iwork(1))>0.0d0 ) then
               work(ielem) = weight(iwork(inpelm)) + &
                             0.01d0*weight(iwork(inpelm-1)) + &
                             0.0001d0*weight(iwork(inpelm-2))
            else

!           --- Element will not be plotted

               work(ielem) = 0.0d0
            end if
         end do
      end do

!     --- Add extra values to work in case of line elements

      if ( neleml>0 ) then
         do ielem = 1, neleml

!        --- maximum of 10 points in one polygon because of weight()

            ikelmc = 2 * ielem - 2
            do i = 1, 2
               weight(i) = coornw( 3, keleml( ikelmc+i ) )
            end do
            call prsort ( weight, iwork, 2 )

!           --- determine the weight of a polygon, in case that all vertices
!               are so far away that they will be within the plot

            if ( weight(iwork(1))>0.0d0 ) then
               work(nelem+ielem) = weight(iwork(2)) + &
                            0.01d0*weight(iwork(1))
            else

!           --- Element will not be plotted

               work(nelem+ielem) = 0.0d0
            end if
         end do
      end if

!     --- Sort the work-array to determine which polygon
!         should be plotted first

      call prsort ( work, iwork, nelem+neleml )

!     --- Make plot depending on value of IPLOTS(1)
!         Colour is a function of the distance of the elements

      factor = (iplots(3) - iplots(4)) / &
               ( work(iwork(nelem+neleml))-work(iwork(1)) )

!     --- Plot every polygon with a color depending on its distance

      do j = nelem+neleml, 1, -1
          ielem = iwork(j)
          if ( ielem<=nelem ) then

!         --- Plot polygon

             if ( work(ielem)>0.0d0 ) then

!            --- Determine to which element group polygon iwork(i) belongs

                ielgrp = 1
510             if ( iwork1(ielgrp+1,1)<ielem ) then
                   ielgrp = ielgrp + 1
                   goto 510
                end if

!               --- Determine the offset in kmeshc for the vertices
!                   of polygon ielem

                ikelmc = iwork1(ielgrp,2)+(ielem-iwork1(ielgrp,1)-1) &
                       * kmesha(ielgrp)

!               --- At this moment : ielem is the polygon we want to plot
!                                    ielgrp is the element group to which
!                                    ielem belongs
!                                    ikelmc is the offset in kmeshc for ielem
!                   Fill polygon with x and y values for call to plaf05()

                do i = 1, kmesha(ielgrp)
                    polyg(1,i) = coornw(1,kmeshc(ikelmc+i))
                    polyg(2,i) = coornw(2,kmeshc(ikelmc+i))
                end do

!               --- Determine the color in which the polygon has
!                   to be plotted

                icolor = 20
                if ( itricol(ielem)==2 ) then
                   icolor = 63
                else if ( itricol(ielem)==3 ) then
                   icolor = 15
                end if

!               --- Plot the polygon

                call plaf05 ( 1,icolor,kmesha(ielgrp),polyg,2 )

!               --- Plot the contour (i.e. the boundary of the elements)
!                   if required

                if ( iplots(5)==1 ) then
                   call plafep ( 4, polyg(1,1), polyg(2,1), &
                                 dum ,dum, 0)
                   do i = 2, kmesha(ielgrp)
                      call plafep ( 5, polyg(1,i), polyg(2,i), &
                                    dum, dum, 0 )
                   end do
                   call plafep ( 5, polyg(1,1), polyg(2,1), &
                                 dum, dum, 0)
                end if
             end if
          else

!         --- Make plot of extra line element
!             (plot every line with a color depending on its distance)
!             Fill polygon with x and y values for call to plaf05()

             polyg(1,1) = coornw(1,keleml(2*ielem-1))
             polyg(2,1) = coornw(2,keleml(2*ielem-1))
             polyg(1,2) = coornw(1,keleml(2*ielem))
             polyg(2,2) = coornw(2,keleml(2*ielem))

!            --- Determine the color in which the line has to be plotted

             icolor = nint ( iplots(4) + &
                      factor * ( work(ielem) - work(iwork(1)) ) )

!            --- Plot the line

             call plaf05 ( 1, icolor, 2, polyg, 2 )
          end if
      end do

      deallocate ( work, iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'work' )

1000  call erclmn( 'plotsfobst', timebefore )
      end
