      function pltjbl ( iseg, ilines, ncomp, nextra, kextra, acnseg, &
                        inc )
! ======================================================================
!
!
!        programmer     Jos van Kan
!        version 1.0    date   20-01-1995
!
!
!
!   copyright (c) 1995-1995 "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!   pltbjl checks the ancestors of segment iseg, an ancestor being
!   1. A segment in  the same component with lower intersection number.
!   (not important here. pltbjl is called by pltjnx, and whenever a
!   segment is checked all segments in the same component with lower
!   intersection number have already been plotted)
!   2. If this segment has a high left point
!   (i.e. nextra (1, iseg) = 1 & kextra (1, 2, iseg) = 1
!   or nextra (1, iseg) = 3 or 6)
!   the highest upper component of the segment to the left of this
!   segment (BUT only if this has a low right point i.e. nextra (2, &
!   jseg) = 1 & kextra (2, 2, jseg) = -1/0 or nextra (2, jseg) = 2 or 3.
!   3. If this segment has a high right point:
!   the right successor of the highest uppercomponent of this segment
!   (BUT only if it has a low left point)
!   If ALL ancestors have been plotted (nextra(2, jseg) > 10) then this
!   segment is plottable and pltjbl gets the value true.
!   A forked segment is marked plottable only if its high end points
!   are plottable. This means effectively, that plotb not only flags
!   plottability, but also removability from the cmplist. (In the
!   generic case this makes no difference, but for forked endings it does.)
! ********************************************************************
!
!                      KEYWORDS
!
!
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer iseg, ilines, ncomp(2, *), nextra(2, *), kextra(2, 2, *), &
              acnseg(0:*), inc
      logical pltjbl

!       acnseg i    array containing accumulated number of segments per line.
!       ilines i    linenumber of this segment
!       inc    i    increment : +1 for ordinary projection, -1 for reversed
!                   projection.
!       iseg   i    segment number of this segment
!       kextra i    array containing information about forked endings of the
!                   segments.
!       ncomp  i    array containing the component structure of the segments
!       nextra i    array containing information about forked endings ofthe
!                   segments and information whether the segments have been
!                   plotted and loaded in the complist (See Anlsnc & Plnxlr)
!       pltjbl o    result
! ********************************************************************
!
!                      COMMON BLOCKS
!
! ********************************************************************
!
!                      LOCAL PARAMETERS
!
      integer lnbinx, ucpinx, ancinx, remanc, modseg, mod1an, mod2an

!     ancinx  ?
!     lnbinx  ?
!     mod1an  ?
!     mod2an  ?
!     modseg  ?
!     remanc  ?
!     ucpinx  ?
! ********************************************************************
!
!                      SUBROUTINES CALLED
!
! ********************************************************************
!
!                      I/O
!
! ********************************************************************
!
!                      ERROR MESSAGES
!
! ********************************************************************
!
!                      PSEUDO CODE
!
!  For clarity we first describe the generic case, which will cover about
!  99% of the segments checked by pltjbl. The generic case is a segment
!  without forked endings (nextra (*, iseg) = 0 or 1). This is simply
!  checked if the ancestors are also generic. If the ancestor is generic it
!  has been plotted if nextra (2, ancseg) > 10. If the ancestor is forked
!  the low part also has been plotted if nextra (2, ancseg) > 10.
!  Whether the low part has been plotted or not is critical for any
!  ancestor. True enough, forked segments
!  are plotted twice: once for the low, once for the high endings. Every
!  time a segment is plotted nextra (2, *) is increased by 10.
!  This however is of no concern regarding the plottability. The rest is
!  pretty straightforward. There are a few traps: we talk about left and
!  right ancestors, which is OK as far as the projection is concerned, but
!  in real life this may be reversed. If that is the case inc = -1, &
!  indicating that to go from left to right in the (projected) segments, &
!  the segment index has to be DECREASED.
! ======================================================================
!
      modseg = mod (nextra (1, iseg), 10)
!
!     First we check if the left end is forked and if so, if the lower
!     part has been plotted already. If it hasn't been the segment is
!     marked unplottable. pltjnx takes care of the rest.
!
      if ((modseg.ge.2) .and. (nextra (2, iseg).lt.10)) then
!
!     forked and not yet plotted: segment marked unplottable
!
          pltjbl = .false.
          return
      end if
!       Check left ancestor

!       There is no left ancestor if
!       0. This segment is the first segment of the line.
!       1. this segment has an unforked low left extra point.
!           this is the generic case.
!       2. This segment has no left extra point. (beginnings & ends of
!          components.)
!       3. This Segment has a forked low and semilow extra point.
!          very rare indeed! (Component splittings)

!       Otherwise there is an ancestor.

      if ((acnseg (ilines - inc).eq.iseg - inc)  .or. &
       ((modseg.eq.1) .and. (kextra (1, 2, iseg).eq.-1)) .or. &
       (modseg.eq.0) ) then
          goto 100
!       100 Next Checkpoint
      else if (modseg.eq.2) then

!       Exception condition: low & semi low forked endpoint.
!       Lower part has been plotted

         goto 100
      end if

!       There is a left ancestor. It is the right most upper component
!       of the left neighbour of this segment. Exception condition:
!       if the left neighbor has no uppercomponent the left neighbor
!       itself is ancestor whether it has a low right point or not.
!       We calculate the LeftNeighBorINdeX and the ANCestorINdeX.

      lnbinx = iseg - inc
      ancinx = ncomp (2, lnbinx)
      if (ancinx.eq.0) then
         ancinx = lnbinx
      end if
      remanc = nextra (2, ancinx) / 10
      mod1an = mod (nextra (1, ancinx), 10)
      mod2an = mod (nextra (2, ancinx), 10)
!       Has it a low right point?
      if (ancinx.ne.lnbinx) then
         if  ((mod2an.eq.0) .or. (mod2an.eq.1) .and. (kextra (2, 2, &
           ancinx).ge.0)) then
           goto 100
!       No low right point. Goto next Checkpoint
         end if
      end if

!       Has it been plotted?

!       if remanc = 0 : no
!       else : yes

      if (remanc.eq.0) then
         pltjbl = .false.
         return
      end if

!       This ends the left ancestor.

100   continue

!       The check for the right ancestor goes in much the same way.
!       We give it without commentary.
      modseg = mod (nextra (2, iseg), 10)
      if ((modseg.ge.2) .and. (nextra (2, iseg).lt.10)) then
         pltjbl = .false.
         return
      end if
      if ((acnseg (ilines).eq.iseg) .or. &
       ((modseg.eq.1) .and. (kextra (2, 2, iseg).eq.-1)) .or. &
       (modseg.eq.0)) then
         pltjbl = .true.
         return
      else if (modseg.eq.2) then
         pltjbl = .true.
         return
      end if

!       There is a right ancestor. It is the right neighbor of the leftmost
!       upper component of this segment.
!       We calculate the UpperComPonentINdeX and the ANCestorINdeX.

      ucpinx = ncomp (2, iseg)

!       Seldom occurring exception : Iseg has a right neighbor, but
!       its upper component has not. Isegs right neighbor is now its
!       ancestor (also when it does not have a low left point).

      if (acnseg (ilines + inc).eq.ucpinx) then
        ancinx = iseg + inc
        remanc = nextra (2, ancinx) / 10
        mod1an = mod (nextra (1, ancinx), 10)
        mod2an = mod (nextra (2, ancinx), 10)
!       skip the low left point test
      else
         ancinx = ucpinx + inc
         remanc = nextra (2, ancinx) / 10
         mod1an = mod (nextra (1, ancinx), 10)
         mod2an = mod (nextra (2, ancinx), 10)
!       Has it a low left point?
        if  ((mod1an.eq.0) .or. (mod1an.eq.1) .and. &
          (kextra (1, 2, ancinx).ge.0)) then
          pltjbl = .true.
          return
!       No low left point. Segment is plottable.
        end if
      end if

!       Has it been plotted?

!       if remanc = 0 : no
!       else : yes

      if (remanc.eq.0) then
         pltjbl = .false.
      else
         pltjbl = .true.
      end if

!       This ends the right ancestor.

      end
