      subroutine pltjre ( cmplst, iseg, acnseg, ncomp, nextra, inc )
! ======================================================================
!
!        programmer    Jos van Kan
!        version  1.2  date 09-03-2013 Correction index = 0
!        version  1.1  date 22-11-1995 Error corrected
!        version  1.0  date 20-01-1995
!
!   copyright (c) 1995-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   pltjre replaces segment iseg in the cmplist of possible candidates to be
!   plotted. The generic case is that iseg is replaced by its upper
!   component. There are a few exception handlings: iseg may have more than
!   one upper component, in which case all upper components are loaded into
!   cmplist. (component split). The upper component of iseg may have more
!   than one lower component. Then iseg is just removed from the cmplist
!   unless it is the last member of that lower component, in which case
!   we're back to generic. (Component confluence).
!   Iseg may not have an upper component: it is removed from the cmplst.
!   (Component Death).
!   The neighbor of a freshly loaded segment has no lower component:this
!   neighbor is also loaded into the complist. (component birth).
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer cmplst (0:*), iseg, acnseg (0:*), ncomp (2,*), &
              nextra(2,*), inc

!     acnseg  i    accumulated segment numbers
!     cmplst i/o   array containing segment numbers and line numbers of
!                  candidates to be plotted.
!     inc     i    1 = forward loading, -1 = backward loading (reverse = true)
!     iseg   i/o   index in cmplst of segment about to be replaced. Cmplst
!                  (iseg) contains the segment number, cmplst (iseg + 1)
!                  the line number on which this segment lies.
!     ncomp   i    arry containing lower (1, segnum) and upper (2, segnum)
!                  components of segment segnum.
!     nextra  i    array containing information about segment endings and
!                  whether segments have already been loaded into the
!                  complist(1, segnum)>10 or have already been plotted
!                  (2, segnum)>10.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer segnum, ilines, segnm0, segnm1
      integer cmpinx, cnt, segcnt, upseghi, upseglo
      logical check

!     check   Indicates if a condition is satisfied
!     cmpinx  ?
!     cnt     ?
!     ilines  Counting variable
!     segcnt  ?
!     segnm0  ?
!     segnm1  ?
!     segnum  ?
!     upseghi  ?
!     upseglo  ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PLTJMO  ?
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Since the new segments are loaded consecutively it is only necessary to
!    establish the number of new segments to be loaded and the offset. If the
!    number is not equal to one there is an exception condition: data in the
!    cmplst have to be moved so as to make room to accomodate the new
!    segments.
!    We proceed in two stages:
!    First we establish if a new segment must be loaded at all. If not, we
!    shrink the cmplist by two places, starting at iseg.
!    If so, we establish an offset by moving down and counting how many
!    segments to the left have to be loaded. These segments have the
!    property, that they have lower component either equal to the segment
!    about to be replaced, or 0 (that is a new component). This establishes
!    an offset. Then we move to the right counting segments with the same
!    property. We have to take care of two things: we must not drop off the
!    line (which can not happen to genuine upper components, but might happen
!    to newly born components) and we must not load a segment more than once.
!    This also only can happen to newly born components.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      segnum = cmplst (iseg)
      ilines = cmplst (iseg + 1)
      upseghi = ncomp (2, segnum)
      upseglo = ncomp (2, segnum)
      if (ncomp (2, segnum)/=0) then

!     --- No component death

         segnm0 = segnum
100      if ((inc * (segnm0 - acnseg (ilines))<0) .and. &
            (ncomp (2, segnm0 + inc)==ncomp (2, segnum))) then

!        --- Component confluence
!            Check if the neighbouring segment has been plotted.
!            We do this by looking if it has been loaded into the complst
!            and if so if it is still there.
!            If not, break. The cmplist must be reduced.
!            If yes, increase segnum0 & repeat

            segnm1 = segnm0 + inc
            cmpinx = abs (segnm1 - segnum)
            if ((nextra (1, segnm1)>=10) .and. (((iseg + 2 * cmpinx) &
            >cmplst (0)) .or. (cmplst (iseg + 2 * cmpinx) /= &
             segnm1))) then

!           --- This neighbour has been plotted
!               Check next neighbour

               segnm0 = segnm1
               goto 100
            else

!           --- This neighbour has not been plotted
!               So this component cannot be loaded

            upseghi = upseghi - inc
            ilines = ilines + inc
            goto 250
            end if
         end if

!        --- All neighbours to the right have been plotted
!            Now check the neighbours to the left.

         segnm0 = segnum
150      check = segnm0 -inc>0
         if ( check ) check = inc * (segnm0 - acnseg (ilines - inc))>1
         if ( check ) check = ncomp(2,segnm0-inc)==ncomp(2,segnum)
         if ( check ) then
            segnm1 = segnm0 - inc
            cmpinx = abs (segnum - segnm1)
            check = iseg-2*cmpinx>0
            if ( check ) check = nextra (1, segnm1)>=10 .and. &
                                (iseg - 2 * cmpinx<1 .or. &
                                 cmplst(iseg-2*cmpinx)/=segnm1)
            if ( check ) then
              segnm0 = segnm1
              goto 150
            else
              call pltjmo (cmplst, iseg, -2)
              return
            end if
         end if  ! ( check )

!        --- All neighbours have been plotted, so component confluence has
!            been reduced to the generic case. The only trouble that still
!            awaits us
!            is component splitting and component birth. From our vantage point
!            we
!            step first to the right along upper components that have our
!            segment
!            as lower component. After that we check for component birth to the
!            right
!            and add segments with lower component 0, but only if they have
!            not been loaded yet. (nextra (1,*)<10).
!            After that we do the same thing to the left.
!            That about will do the trick. Here goes.

         ilines = ilines + inc
         check = upseghi+inc>0
         if ( check ) check = inc*(upseghi-acnseg(ilines))<0 .and. &
                              ncomp(1,upseghi+inc)==segnum .and. &
                              nextra(1,upseghi+inc)<10
200      if ( check ) then
            upseghi = upseghi + inc
            goto 200
         end if
225      check = upseghi+inc>0
         if ( check ) check = inc * (upseghi - acnseg (ilines))<0 .and. &
                              ncomp(1, upseghi + inc)==0 .and. &
                              nextra (1, upseghi+ inc)<10
         if ( check ) then
            upseghi = upseghi + inc
            goto 225
         end if
250      check = upseglo - inc>0
         if ( check ) check = inc*(upseglo-acnseg(ilines-inc))>1 .and. &
                              ncomp(1,upseglo-inc)==segnum .and. &
                              nextra (1,upseglo - inc)<10
         if ( check ) then
            upseglo = upseglo - inc
            goto 250
         end if
275      if ((inc * (upseglo - acnseg (ilines - inc))>1) .and. &
            (ncomp (1, upseglo - inc)==0) .and. (nextra (1, upseglo &
            - inc)<10)) then
            upseglo = upseglo - inc
            if ( upseglo - inc>0 )goto 275
         end if

!        --- upseghi & upseglo contain indices of segments about to be loaded
!            there is still a catch: the highest may have an overlap
!            with a lower component that has not been plotted yet.

         if ((ncomp (1, upseghi)/=0) .and. (ncomp (1, upseghi) &
           /=segnum) .and. (nextra (2, segnum + inc)<10) .and. &
            (inc * (upseghi - upseglo)>=0)) then
            upseghi = upseghi - inc
         end if

!        --- same thing for upseglo

         check = ncomp(1,upseglo)/=0
         if ( check ) check = segnm0-inc>0
         if ( check ) check = ncomp(2,segnm0-inc)==upseglo .and. &
                              nextra(2,segnm0-inc)<10 .and. &
                              inc*(upseghi-upseglo)>=0
         if ( check ) then
            upseglo = upseglo + inc
         end if

!        --- so now upseglo and upseghi contain left & right indices of the
!            --- number
!            of segments about to be loaded. To avoid unnecessary data moves we
!            make
!            an exception for the case that they are equal.

         if (upseghi==upseglo) then
            cmplst (iseg) = upseglo
            cmplst (iseg + 1) = ilines
            nextra (1, upseglo) = nextra (1, upseglo) + 10
            iseg = iseg + 2
         else
            call pltjmo (cmplst, iseg, 2 * inc * (upseghi - upseglo))
            cnt = 0
            do 300 segcnt = upseglo, upseghi, inc
               cmplst (iseg + cnt) = segcnt
               cmplst (iseg + cnt + 1) = ilines
               nextra (1, segcnt) = nextra (1, segcnt) + 10
               cnt = cnt + 2
300         continue
            iseg = iseg + cnt
         end if
         return
      else

!     --- component death. Just remove two places from the cmplst.

         call pltjmo (cmplst, iseg, -2)
      end if
      end
