      subroutine prcontourcomp2 ( markelems, area, nelem, kmeshc, &
                                  coor, kmesho, plotpos, numelms, &
                                  activeelems, icolor, zeroval, func, &
                                  marknodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-04-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all neighboring elements of element corresponding to the
!     same region satisfying function value>specified value
!     Compute area and plot elements if necessary
!     The mesh must consist of linear trangles only
!     Special case of 2 nodes completely in region
! **********************************************************************
!
!                       KEYWORDS
!
!     contour_level
!     area
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, markelems(nelem), kmeshc(3), kmesho(3), &
              numelms, activeelems(nelem), marknodes(*)
      double precision area, coor(2,*), func(*)
      logical plotpos, zeroval

!     activeelems   i/o   Contains the list of still active elements
!                         These elements are neighbors of the elements in the
!                         bubble
!     area          i/o   Area of bubble
!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     func           i    Contains the scaled and shifted function values
!     icolor         i    Sequence number of colour
!     kmeshc         i    kmesh part c, contains the nodal point numers of the
!                         element
!     kmesho         i    kmesh part o, contains the neighbor elements of the
!                         element
!     markelems     i/o   See subroutine prcontourcomp
!     marknodes     i/o   See subroutine prcontourcomp
!     nelem          i    Number of elements
!     numelms       i/o   Number of elements in activeelems
!     plotpos        i    If true the bubble must be plotted
!     zeroval        i    If true one of the points has value 0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ineigh, icolor, icount, i1, i2, ineg, node, node1, &
              node2, nodeneg, nodepos, npolyg, nodezero
      double precision polyg(2,4), areatrian, arealoc, x1(2), x2(2), &
                       x(2)

!     arealoc        Area of small triangle
!     areatrian      Area of complete triangle
!     i              Counting variable
!     i1             First point with positive function value
!     i2             Second point with positive function value
!     icount         Counter
!     ineg           Sequence number of edge from with non-positive function
!     ineigh         neighbor
!     node           Node number
!     node1          Node number
!     node2          Node number
!     nodeneg        Node number of node with negative funtion value
!     nodepos        Node number of node with positive funtion value
!     nodezero       Node number of node with zero funtion value
!     npolyg         Number of points in a polygon
!     polyg          Array containing the co-ordinates of the point in the
!                    polygon in the following way:
!                    polyg(1,i) contains the x-coordinate of the first
!                    point
!                    polyg(2,i) contains the y-coordinate of the first
!                    point
!     x              contains coordinates of intersection point
!     x1             contains coordinates of first intersection point
!     x2             contains coordinates of second intersection point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision mshareatria

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHAREA        Compute the area of the triangle spanned by x1-x2-x3
!                    The sign of the are defines the direction
!     MSHAREATRIA    Return with area of triangle, with nodes given in kmeshc
!     PLAF05         Plot polygon
!     PRROOT         Find coordinates where function value is equal to 0
!                    function value on end points is given
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Mark element and all its non-marked neighbors with num_bubbles
!     Compute area
!     Plot element if necessary
!     The situation of two nodes that belong to the bubble can be split into
!     a:  Both nodes have positive value: 2 edges are intersected
!              + &
!            /  \
!           /\  \
!          - ---+ &
!
!     b:  One node is positive the other one 0: one edge is intersected
!              + &
!            /  \
!           / / \
!          - ---0
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcontourcomp2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcontourcomp2'
         write(irefwr,2) 'area', area
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( zeroval ) then

!     --- One of the points is zero the other one positive
!         Detect the three points

         do i = 1, 3

            node = kmeshc(i)
            if ( marknodes(node)<0 ) then
               nodeneg = node
               i1 = i
            else if ( marknodes(node)>0 ) then
               nodepos = node
            else
               nodezero = node
               i2 = i
            end if  ! ( marknodes(node)<0 )

         end do  ! i = 1, 3

!        --- Find side zero to min

         if ( i1*i2==1 ) then
            ineg = 1      ! side 1-2
         else if ( i1*i2==6 ) then
            ineg = 2      ! side 2-3
         else
            ineg = 3      ! side 1-3
         end if  ! ( i1*i2==1 )

!        --- Find non-marked neighbors

         do i = 1, 3

            if ( ineg/=i ) then

!           --- Side contains positive point

               ineigh = mod(kmesho(i),nelem+1)
               if ( ineigh>0 ) then

!              --- neighbor found

                  if ( markelems(ineigh)==0 ) then

!                 --- new neighbor, add to list

                     numelms = numelms+1
                     activeelems(numelms) = ineigh
                     markelems(ineigh) = -2

                  end if  ! ( markelems(ineigh)==0 )

               end if  ! ( ineigh>0 )

            end if  ! ( ineg/=i )

         end do  ! i = 1, 3

!        --- Compute coordinates of intersection point, store in x

         call prroot ( x, coor(1,nodepos), coor(1,nodezero), &
                       func(nodepos), func(nodezero), 2 )
         call msharea ( coor(1,nodepos), coor(1,nodezero), &
                        x, arealoc )

         area = area+abs(arealoc)
         if ( debug ) write(irefwr,2) 'area, arealoc', area, arealoc

         if ( plotpos ) then

!        --- Plot small triangle

            polyg(1:2,1) = coor(1:2,nodepos)
            polyg(1:2,2) = x
            polyg(1:2,3) = coor(1:2,nodezero)
            npolyg = 3

         end if  ! ( plotpos )

      else

!     --- one of the points is negative, two are positive
!         Detect the three points

         icount = 0
         do i = 1, 3

            node = kmeshc(i)
            if ( marknodes(node)<0 ) then
               nodeneg = node
            else if ( marknodes(node)>0 ) then
               if ( icount==0 ) then
                  node1 = node
                  icount = 1
               else
                  node2 = node
               end if  ! ( icount==0 )
            end if  ! ( marknodes(node)<0 )

         end do  ! i = 1, 3

!        --- Find non-marked neighbors

         do i = 1, 3

            ineigh = mod(kmesho(i),nelem+1)
            if ( ineigh>0 ) then

!           --- neighbor found

               if ( markelems(ineigh)==0 ) then

!              --- new neighbor, add to list

                  numelms = numelms+1
                  activeelems(numelms) = ineigh
                  markelems(ineigh) = -2

               end if  ! ( markelems(ineigh)==0 )

            end if  ! ( ineigh>0 )

         end do  ! i = 1, 3

!        --- Compute the coordinates of the tow intersection points

         call prroot ( x1, coor(1,node1), coor(1,nodeneg), func(node1), &
                       func(nodeneg), 2 )
         call prroot ( x2, coor(1,node2), coor(1,nodeneg), func(node2), &
                       func(nodeneg), 2 )

!        --- Compute area by subtracting small triangle from complete triangle

         areatrian = mshareatria ( kmeshc, coor )  ! area of complete triangle
         call msharea ( coor(1,nodeneg), x1, x2, arealoc )

         area = area + areatrian-abs(arealoc)
         if ( debug ) &
            write(irefwr,2) 'area, areatrian, arealoc', &
                             area, areatrian, arealoc

         if ( plotpos ) then

!        --- Plot quadrilateral

            polyg(1:2,1) = coor(1:2,node1)
            polyg(1:2,2) = x1
            polyg(1:2,3) = x2
            polyg(1:2,4) = coor(1:2,node2)
            npolyg = 4

         end if  ! ( plotpos )

      end if  ! ( zeroval )

      if ( plotpos ) then

!     --- Plot element with one colour

         call plaf05 ( 1, icolor, npolyg, polyg, 2 )

      end if  ! ( plotpos )

1000  call erclos ( 'prcontourcomp2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'area', area
         write(irefwr,*) 'End prcontourcomp2'

      end if  ! ( debug )

      end
