      subroutine sepplot3m ( iplots, rplots )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.9  date 29-05-2014 Extension of timebefore
!        version  1.8  date 25-11-2012 Long integers
!        version  1.7  date 27-09-2010 Make work arrays local
!
!   copyright (c) 1999-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  subroutine sepplot3m is used to plot a three-dimensional mesh with
!  colors. In fact this subroutine creates a hidden line plot on a screen.
!  Remark:  this subroutine is only fitted for display plotting, not
!           for a pen plotter.
!           Subroutine PLAFP8 (fill colored surfaces) must have been
!           implemented for the plot package
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     color
!     mesh
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulemesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iplots(*)
      double precision rplots(*)

!     iplots  i   Integer array in which the following quantities must be
!                 stored:
!                  1:  Position of last item that has been filled by the user
!                  2:  choice parameter for type of projection
!                      Possible values:
!                       0 : parallel projection plot
!                       1 : perspective projection plot
!                      10 : parallel projection plot (View point automatic)
!                      11 : perspective projection plot (View point automatic)
!                      Default value: 1
!                  3:  color number of farthest triangle
!                      Default value: 1
!                  4:  color number of nearest triangle
!                      Default value: 16
!                  5:  Indication if boundaries must be plotted (1) or not (0)
!                      Default value: 1
!                  6:  Indication whether x-y is basic plane (1), z-x is basic
!                      plane (2) or y-z is basic plane (3)
!                      Default value: 1
!                  7:  iskip:  Indication if element groups must be skipped (1)
!                              or not (0)
!                      Default value: 0
!                  8:  numrotations:  Number of rotations
!                      Default value: 1
!                  9:  Use specific volumes stored from position 15 (>0) or not
!                      (0)
!                 10:  Plot elements per layer (1) or not (0)
!                 11:  idefstart, defines how the start level must
!                      be computed
!                      1: Start with set of nodes with largest
!                         distance to view point
!                      2: Start with set of surfaces indicated in
!                         positions 12 and 13
!                 12:  ifirst, first surface number
!                 13:  ilast, last surface number
!                 15...14+nelgrp:  ihelp  Integer input array of length NELGRP
!                                       that is only used if ISKIP = 1
!                                       It indicates if element group ielgrp
!                                       must be skipped ihelp(ielgrp) = 0
!                                       or not ihelp(ielgrp) = 1
!     rplots  i   Double precision input array that must be filled as follows:
!                 1:    plotfm
!                 2:    yfact
!                 3-5:  eye-point
!                 6-8:  projection point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ilplot(13), idum(1), jv, ichois, iresul(5), iskip, &
              nomber(2), neleml, special, i, irotation, &
              numrotations, jtimessav, iinput(4), ilevel, &
              nlevels, npelmlc, nelgrplc, nelgrpsav, nelemsav
      integer, allocatable :: elements(:), elmlevels(:)
      integer :: kmeshalc(1), kmeshglc(1)
      double precision, allocatable :: coornw(:)
      logical perspc, compl, uselevels
      double precision dum(3), xmean(3), viewpoint(3), &
                       viewp(3)
      double precision :: timebefore(2)

!     compl          dummy logical
!     coornw         coordinate array for projected points
!                    (1,i) contains the x-coordinate of the projected point i
!                    (2,i) contains the y-coordinate of the projected point i
!                    (3,i) contains the distance of point i to
!                          the view-point
!     dum            General dummy double precision array
!     elements       Array of length levels(nlevels) (<=nelem)
!                    containing the reordered element sequence numbers
!                    So elements(1) contains the first element number to
!                    be used, elements(2) the second one and so on
!     elmlevels      Array of length nlevels (<nelem) containing the
!                    levels in array elements
!                    The first level consists of the elements 1 to
!                    elmlevels(1) of array elements
!                    The second level contains the elements  elmlevels(1)+1
!                    to elmlevels(2) and so on
!     i              Counting variable
!     ichois         Choice parameter. With this parameter it is defined which
!                    actions will be performed. Possible values:
!                    1:  Create a submesh consisting of 3-point triangles only
!                        Create also a copy of the solution array containing one
!                        unknown only in all points of the new grid
!                        In this case array KPROB and ISOL is also used
!                        If the solution vector is complex only the real or
!                        imaginary part is used
!                    2:  Create a submesh consisting of 3-point triangles only
!                        Create also a copy of the solution array containing one
!                        unknown only in all points of the new grid
!                        In this case array KPROB and ISOL is also used
!                        If the solution vector is complex the copied solution
!                        vector is also complex
!                    3:  Create a submesh consisting of the surface elements
!                        only of each element only the vertices are stored, &
!                        hence the final grid consists of a grid of 3-point
!                        triangles or 4-point quadrilaterals only
!                        This possibility only uses KMESH
!                        It has only been implemented for 3D
!                    4:  Combination of 1 and 3, hence possibly 2 meshes may be
!                        created
!                    5:  Combination of 2 and 3, hence possibly 2 meshes may be
!                        created
!     idum           General integer dummy variable
!     iinput         integer input array for subroutine TOPLEVELS
!     ilevel         Defines level sequence number for plotting
!     ilplot         Copy of iplots, if necessary extended with defaults
!     iresul         Output array of length 5,1 if ichois=2 and otherwise of
!                    length 5,3
!                    The array is filled as follows:
!                    Pos .,1 corresponds to the "vertex" mesh array (ichois=2,3)
!                    Pos .,2 corresponds to the "triangle" mesh array
!                            (ichois=1,3)
!                    Pos .,3 corresponds to the solution array (ichois=1,3)
!                    Pos. 1,1 INDVER  sequence number with respect to INFOR
!                                     for the "vertex" mesh array
!                    Pos. 2,1 NPELM   number of points per element for this mesh
!                    Pos. 3,1 NELEM   number of points per element for this mesh
!                    Pos. 4,1 NEWVER  indicates if the mesh is new (1) or is
!                                     in fact the standard mesh (0)
!                    Pos. 5,1 INDLIN  sequence number with respect to INFOR
!                                      for the "line" elements array
!     irotation      Loop variable for rotation
!     iskip          dummy
!     jtimessav      Help variable to save the value of jtimes
!     jv             Indicates if the view point has been filled (1) or not (0)
!     kmeshalc       kmesh part a
!     kmeshglc       kmesh part g
!     neleml         Number of "line"-elements in mesh
!     nelemsav       Help parameter to save the old value of nelem
!     nelgrplc       Number of element groups
!     nelgrpsav      Help parameter to save the old value of nelgrp
!     nlevels        Number of entries in array elmlevels
!     nomber         Help array to store relative number of unknown considered
!     npelmlc        Maximum number of nodes in an element
!     numrotations   Number of rotations
!     perspc         indicates if a perspective (true) or a parallel projection
!                    (false) must be applied
!     special        Indicates which special situation is present
!                    Possible values:
!                    0     standard
!                    1     solution vector is of type 116 and a colored
!                          2d plot is required
!                    2     solution vector is of type 110 or 115 and a colored
!                          2d plot is required, whereas there are no unknowns
!                          in the vertices
!     uselevels      If true plotting per level must be performed
!     viewp          Array containing the first view point
!     viewpoint      Array containing actual view point
!     xmean          Array containing the mean values of the co-ordinates
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLMN         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     PL0011         Fill copy of IPLOTS and extend with defaults
!     PL0014         Compute projection of points onto projection plane
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLFILLVIEWP    Fill array viewpoint depending on the rotation
!     PLMORE         User subroutine to plot extra information in the plot
!     PLOTSF         Make a three-dimensional hidden line plot of a 3D surface,
!                    using colored surfaces. (Display only)
!     TOPELEMSKIP    Fill array elemskip with information about elements to be
!                    skipped and elements to be plotted
!     TOPLEVELS      Reorder a restricted set of elements in an ordered
!                    level set
!     TOPQU0         Subdivide mesh if necessary. Map unknowns into an organized
!                    array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     581   The mesh is not three-dimensional
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Check input
!   Copy iplots into ilplot and fill defaults (PL0011)
!   Activate permanent arrays
!   Create surface vertex elements only
!   Perform actual procedure (PLOTSF)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepplot3m' )
      if ( itime>0 ) call ersettime ( timebefore )
      if ( ndim/=3 ) then
         call errint ( ndim, 1 )
         call errsub ( 581, 1, 0, 0 )
      end if
      nelgrpsav = nelgrp
      nelemsav = nelem

!     --- Fill local array ilplot and check rplots

      call pl0011 ( iplots, rplots, ilplot )

!     --- Define work space and activate permanent arrays

      ichois = 3
      compl = .false.
      nomber(1) = 1
      nomber(2) = 1
      special = 0
      if ( ilplot(7)>0 ) then
         iskip = 1
      else if ( ilplot(9)>0 ) then
         iskip = -1
      else
         iskip = 0
      end if
      uselevels = ilplot(10)>0
      nlevels = 1
      nullify(elemskip)

      if ( uselevels ) then

!     --- Plot per level
!         Call subroutine toplevels to compute the levels
!         First activate arrays

         allocate ( elemskip(nelem), elmlevels(nelem), &
                    elements(nelem), stat = error )
         if ( error/=0 ) call eralloc ( error, 3*nelem, 'elemskip' )
         if ( ierror/=0 ) go to 1000

!        --- Actual call

         iinput(1) = ilplot(7)
         iinput(2) = ilplot(11)
         iinput(3) = ilplot(12)
         iinput(4) = ilplot(13)
         call toplevels ( rplots(3), iskip, elements, elmlevels, &
                          nlevels, iinput )
         length = elmlevels(1)+nlevels

!        --- Create array elemskip

         call topelemskip ( nlevels, elmlevels, elements )

         deallocate ( elmlevels, elements, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'elmlevels' )

      end if
      jtimessav = jtimes

      allocate ( coornw(3*npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'coornw' )
      if ( ierror/=0 ) go to 1000

!     --- Loop over levels

      do ilevel = 1, nlevels

         call topqu0 ( ichois, nomber, iskip, iplots(15), iresul, &
                       compl, 0, special, ilevel )
         if ( ierror/=0 ) go to 1000

!        --- Compute actual plot

         npelmlc = iresul(2)
         nelemlc = iresul(3)
         nelgrplc = 1
         kmeshalc(1) = npelmlc
         kmeshglc(1) = nelemlc
         jv = 1 - iplots(2)/10
         perspc = mod(iplots(2),10)==1
         iorien = ilplot(6)
         numrotations = ilplot(8)

!        --- Loop over the rotations

         if ( numrotations>1 .or. nlevels>1 ) then

!        --- More than one rotation, separate pictures required

            if ( jtimes>10 ) jtimes = jtimes-10
            if ( jtimes==2 .or. jtimes==3 ) then
               jtimes = 3
            else
               jtimes = 0
            end if

         end if

         do irotation = 1, numrotations

            if ( numrotations==1 .and. ilevel==nlevels ) then
               jtimes = jtimessav
            else if ( irotation==numrotations .and. &
                      jtimessav>=10 .and. ilevel==nlevels ) then
               jtimes = 11
            end if
            if ( irotation==1 .and. jv==1 ) then

!           --- First rotation, set viewpoint

               do i = 1, 3
                  viewpoint(i) = rplots(2+i)
               end do

            end if

            call plfillviewp ( irotation, jv, viewp, viewpoint, iorien, &
                               numrotations )

            call pl0014 ( coor, viewpoint, jv, coornw, &
                          npoint, xmean, perspc, 3, dum, 1, rplots(1), &
                          rplots(2), iorien )
            if ( irotation==1 .and. jv==0 ) then

!           --- First step and view point has not yet been filled

               do i = 1, 3
                  viewp(i) = viewpoint(i)
               end do

            end if

            if ( iresul(5)/=0 ) then

!           --- line elements

               neleml = size ( kmeshcline ) / 2

            else

!           --- no line elements

               neleml = 0

            end if
            call plotsf ( coornw, npoint, kmeshalc, &
                          kmeshglc, nelgrplc, kmeshcsub, &
                          ilplot, dum, dum, idum(1), &
                          idum, 0, 0, neleml, kmeshcline )

            if ( mod(jtimes,10)<1 .or. mod(jtimes,10)>2 ) then
               call plmore
               call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )
            end if

         end do

      end do

      deallocate ( coornw, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'coornw' )

1000  if ( ierror/=0 ) then

!     --- ierror # 0  Reset ierror and nerror
!                     Return to main program

         ierror = 0
         nerror = 0
         if ( jtimes==11 ) jtimes = 1
         go to 1010

      end if

      if ( iwarn/=0 ) then

!     --- iwarn # 0  Reset iwarn and nwarn
!                    Return to main program

         iwarn = 0
         nwarn = 0

      end if

      kmeshc => km(iseqkm)%kmeshc
      nelgrp = nelgrpsav
      nelem = nelemsav
      inelem = nelem

1010  if ( iresul(5)/=0 ) deallocate ( kmeshcline )
      if ( iresul(4)==1 ) deallocate ( kmeshcsub )
      if ( uselevels ) deallocate ( elemskip )
      call erclmn ( 'sepplot3m', timebefore )

      end subroutine sepplot3m
