      subroutine sepplotms ( iinplt, rinplt, iskip )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.8  date 15-11-2013 Use local arrays
!        version  1.7  date 25-11-2012 Long integers
!        version  1.6  date 04-06-2007 Correction "iuser"
!
!   copyright (c) 1999-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot a SEPRAN mesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     plot_mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iinplt(*), iskip(*)
      double precision rinplt(*)

!     iinplt  i    Integer input array indicating how the mesh plot must be
!                  made. Contents:
!                  1:  Last entry that has been filled by the user.
!                      For all other entries defaults are used
!                      Hence if iinplt(1) = 0 or 1 only defaults are used
!                  2:  Plot element numbers (1) or not (0)
!                      Default: 0
!                  3:  Plot nodal point numbers (1) or not (0)
!                      Default: 0
!                  4:  Mark nodal point numbers (1) or not (0)
!                      Default: 0
!                  5:  Contract elements (1) or not (0) (2D only)
!                      Default: 0
!                  6:  Plot outer boundaries double (1) or not (0)
!                      In case of coloured 3D plot: plot boundaries of
!                      elements
!                      Default: 0
!                  7:  Skip element groups (>0) or not (0)
!                      Default: 0
!                  8:  Plot renumbered node numbers (1) or original ones (0)
!                      Default: 0
!                  9:  Make a filled coloured mesh plot (1) or a line plot (0)
!                      Default: 0
!                 10:  Plot axis (1) or not (0)
!                      Default: 0
!                 11:  Type of projection. Possible values:
!                       0 : perspective projection plot
!                       1 : parallel projection plot
!                      10 : perspective projection plot (View point automatic)
!                      11 : parallel projection plot (View point automatic)
!                      Default: 10
!                 12:  Colour number of farthest elements
!                      Default: 1
!                 13:  Colour number of nearest elements
!                      Default: 16
!                 14:  Basic coordinate system for plots
!                        1 :  x - y - z
!                        2 :  z - x - y
!                        3 :  y - z - x
!                 15:  numrotations:  Number of rotations
!                      Default value: 1
!                 16:  Use volumes (>0) or not (0)
!                      Default: 0
!                 17:  Plot elements per layer (1) or not (0)
!                 18:  idefstart, defines how the start level must
!                      be computed
!                      1: Start with set of nodes with largest
!                         distance to view point
!                      2: Start with set of surfaces indicated in
!                         positions 12 and 13
!                 19:  ifirst, first surface number
!                 20:  ilast, last surface number
!                 21:  plotboun, see subroutine plotm2bf
!     iskip   i    In this array it is stored which element groups must
!                  be skipped
!                  iskip(ielgrp) = 0: skip element group ielgrp, otherwise (1)
!                                     do not skip
!     rinplt  i    Double precision input array that must be filled as follows:
!                  1:    plotfm
!                  2:    yfact
!                  3-5:  eye-point
!                  6-8:  projection point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nincop
      parameter ( nincop=21 )
      integer iincop(nincop), imknod, icontr, iouter, iprojt, &
              jskip, irenpl, icolor, jaxis, icolfr, icolnr, &
              jmarks, i, ichois, jkadsv, iplelm, iplnod, &
              iorien, numrotations, jmarksav
      integer, allocatable :: iuser(:)
      double precision plotfm, yfact

!     i              Counting variable
!     ichois         Choice parameter
!     icolfr         Colour number of farthest element
!     icolnr         Colour number of nearest element
!     icolor         Make a filled coloured mesh plot (1) or a line plot (0)
!     icontr         Contract elements (1) or not (0) (2D only)
!     iincop         Copy of array iinplt, extended with defaults
!     imknod         Mark nodal point numbers (1) or not (0)
!     iorien         Indicator for basis coordinate system
!                      1 :  x - y - z
!                      2 :  z - x - y
!                      3 :  y - z - x
!     iouter         Plot outer boundaries double (1) or not (0)
!                    In case of coloured 3D plot: plot boundaries of
!                    elements
!     iplelm         Plot element numbers (1) or not (0)
!     iplnod         Plot nodal point numbers (1) or not (0)
!     iprojt         Type of projection. Possible values:
!                      0 : perspective projection plot
!                      1 : parallel projection plot
!                     10 : perspective projection plot (View point automatic)
!                     11 : parallel projection plot (View point automatic)
!     irenpl         If 1 renumbered nodes must be plotted
!     iuser          Integer work array to pass information to subroutines
!     jaxis          Plot axis (1) or not (0)
!     jkadsv         help variable to save the old value of jkader
!     jmarks         help variable to save the old value of jmark
!     jmarksav       help variable to save the old value of jmark
!     jskip          Skip element groups (>0) or not (0) or volumes (<0)
!     nincop         length of array iincop
!     numrotations   Number of rotations
!     plotfm         Plotting factor. When PLOTFM<0, then the length of
!                    the plot is computed by - (xmax-xmin) * PLOTFM
!                    and the width by - (ymax-ymin) * YFACT * PLOTFM
!                    else the length of the plot is equal to PLOTFM cm
!                    When by some mistake the user has given | PLOTFM | a value
!                    smaller than 1 cm, then the length is chosen equal to
!                    the default length stored in common block consta
!     yfact          Multiplication for the y-co-ordinates. With this
!                    parameter the user can influence the scale in y-direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PLOT3MOBST     plot obstacle in 3d mesh with colors
!     PLOTM3OBST     plot obstacle in 3d mesh with hidden lines
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPPLOT3M      plot 3d mesh by coloring all outer boundary elements from
!                    infinity to nearby
!     SEPPLOTM1      plot 1D mesh
!     SEPPLOTM2      plot 2D mesh
!     SEPPLOTM3      plot 3D mesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepplotms' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepplotms'
         call prinin ( iinplt, iinplt(1), 'iinplt' )
         call prinrl ( rinplt, 5, 'rinplt' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Set parameters

      iincop = 0
      iincop(11) = 10
      iincop(12) = 1
      iincop(13) = 16
      iincop(14) = 1
      iincop(15) = 1
      iincop(18) = 1
      iincop(19) = 1
      do i = 1, min(iinplt(1),nincop)
         iincop(i) = iinplt(i)
      end do
      if ( iincop(20)==0 ) iincop(20) = iincop(19)

      if ( debug) call prinin ( iincop, 20, 'iincop' )

      iplelm = iincop(2)
      iplnod = iincop(3)
      imknod = iincop(4)
      icontr = iincop(5)
      iouter = iincop(6)
      jskip  = iincop(7)
      irenpl = iincop(8)
      icolor = iincop(9)
      jaxis  = iincop(10)
      iprojt = iincop(11)
      icolfr = iincop(12)
      icolnr = iincop(13)
      iorien = iincop(14)
      numrotations = iincop(15)
      plotfm = rinplt(1)
      yfact  = rinplt(2)

!     --- Set jmark

      jmarks = jmark
      jkadsv = jkader
      if ( iplnod==0 ) then

!     --- Do not plot nodal point numbers

         if ( imknod==0 ) then
            jmark = 2
         else
            jmark = 1
         end if

      else

!     --- Plot nodal point numbers

         if ( imknod==0 ) then
            jmark = 6
         else
            jmark = 0
         end if

      end if
      if ( iplelm==0 ) then

!     --- Do not plot element numbers

         if ( jmark<5 ) then
            jmark = jmark + 3
         else
            jmark = 7
         end if
      end if
      jmark = jmark + 10 * irenpl
      if ( jaxis>0 ) jkader = -3

!     --- Set ichois

      if ( ndim<=2 ) then

!     --- ndim = or 2

         ichois = 0
         if ( icontr==1 ) ichois = ichois + 2
         if ( jskip>0 ) ichois = ichois + 1
         if ( iouter==1 ) ichois = ichois + 10
         if ( icolor==1 ) ichois = ichois + 5
         ichois = ichois+100*iincop(21)

      end if

!     --- Copy iskip if necessary

      if ( jskip>=1 ) then

!     --- jskip>0

         allocate ( iuser(nelgrp+5), stat = error )
         if ( error/=0 ) call eralloc ( error, nelgrp+5, 'iuser' )
         if ( ierror/=0 ) go to 1000

         iuser(1) = nelgrp+5
         iuser(3) = nelgrp+5
         iuser(6:5+nelgrp) = iskip(1:nelgrp)

      else if ( iincop(16)>0 ) then

!     --- iincop(16)>0, restricted number of volumes

         allocate ( iuser(nvolms+5), stat = error )
         if ( error/=0 ) call eralloc ( error, nvolms+5, 'iuser' )
         if ( ierror/=0 ) go to 1000

         iuser(1) = nvolms+5
         iuser(3) = nvolms+5
         iuser(6:5+nvolms) = iskip(1:nvolms)

      else

!     --- Initialize iuser

         allocate ( iuser(1), stat = error )
         if ( error/=0 ) call eralloc ( error, 1, 'iuser' )
         if ( ierror/=0 ) go to 1000
         iuser(1) = 0

      end if
      if ( ndim==1 ) then

!     --- ndim = 1, 1D mesh

         call sepplotm1 ( ichois, iuser, plotfm )

      else if ( ndim==2 ) then

!     --- ndim = 2, 2D mesh

         call sepplotm2 ( ichois, iuser, plotfm, yfact )

      else if ( ndim==3 ) then

!     --- ndim = 3, 3D mesh

         if ( debug ) write(irefwr,1) 'icolor', icolor
         if ( icolor==0 ) then

!        --- icolor = 0, hidden line plot

            jmarksav = jmark
            if ( debug ) write(irefwr,*) 'before sepplotm3'
            call sepplotm3 ( iincop, iuser, plotfm, yfact, rinplt(3) )
            if ( debug ) write(irefwr,*) 'after sepplotm3'

            if ( numobst>0 ) then

!           --- numobst>0, there are obstacles
!               PLot these obstacles separately

               jmark = jmarksav
               call plotm3obst ( iincop, plotfm, yfact, rinplt(3) )

            end if

        else

!       --- icolor = 1, colored plot

            deallocate ( iuser, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'iuser' )
            allocate ( iuser(nelgrp+14), stat = error )
            if ( error/=0 ) call eralloc ( error, nelgrp+14, 'iuser' )
            if ( ierror/=0 ) go to 1000

            iuser(1) = 14+nelgrp
            iuser(2) = iprojt
            iuser(3) = icolfr
            iuser(4) = icolnr
            iuser(5) = iouter
            iuser(6) = iorien
            iuser(7) = jskip
            iuser(8) = numrotations
            iuser(9) = iincop(16)
            iuser(10) = iincop(17)
            iuser(11) = iincop(18)
            iuser(12) = iincop(19)
            iuser(13) = iincop(20)
            if ( jskip>0 ) then

!           --- jskip>0, skip element groups

               iuser(14:13+nelgrp) = iskip(1:nelgrp)

            else if ( iincop(16)>0 ) then

!           --- iincop(16)>0, restricted number of volumes

               iuser(14:13+nvolms) = iskip(1:nvolms)

            end if
            jmarksav = jmark
            call sepplot3m ( iuser, rinplt )

            if ( numobst>0 ) then

!           --- numobst>0, there are obstacles
!               PLot these obstacles separately

               jmark = jmarksav
               call plot3mobst ( iuser, rinplt )

            end if

         end if

      end if

      jmark  = jmarks
      jkader = jkadsv

      deallocate ( iuser, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iuser' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepplotms'

      end if
      call erclos ( 'sepplotms' )

      end subroutine sepplotms
