      subroutine topsubdiv2d1 ( ishape, mult, typesubdiv, idegfd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if element must be subdivided into triangles
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     subdivision
!     surface
!     2d
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ishape, idegfd
      integer, intent(out) :: mult, typesubdiv

!     idegfd         i    Degree of freedom sequence number
!     ishape         i    Shape number of elements
!     mult           o    Number of triangles an element is subdivided in
!     typesubdiv     o    Defines type of subdivision and interpolation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topsubdiv2d1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topsubdiv2d1'
         call ertrace
         write(irefwr,1) 'ishape', ishape
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      mult = 0
      typesubdiv = 0

      select case ( ishape )
      case (4)

!     --- quadratic triangle

         if ( debug ) call prinin ( kprobatmp, 6, 'kprobatmp' )
         if ( kprobatmp(1)>=idegfd .and. kprobatmp(2)>=idegfd ) then

!        --- unknown in all nodes

            mult = 4  ! number of elements

         else if ( kprobatmp(1)>=idegfd .and. kprobatmp(2)<idegfd ) then

!        --- unknown in vertices

            mult = 1  ! number of elements

         else if ( kprobatmp(1)<idegfd .and. kprobatmp(2)>=idegfd ) then

!        --- unknown in mid-side points

            mult = 4  ! number of elements
            typesubdiv = 1

         end if  ! ( kprobatmp(1)>=idegfd .and. kprobatmp(2)>=idegfd )

      case (5)

!     --- bilinear quadrilateral

         if ( debug ) call prinin ( kprobatmp, 4, 'kprobatmp' )
         if ( kprobatmp(1)>=idegfd ) then

!        --- unknown in all nodes

            mult = 2  ! number of elements

         end if  ! ( kprobatmp(1)>=idegfd )

      case (6)

!     --- bi-quadratic quadrilateral

         if ( debug ) call prinin ( kprobatmp, 9, 'kprobatmp' )
         if ( kprobatmp(1)>=idegfd .and. kprobatmp(2)>=idegfd .and. &
              kprobatmp(9)>=idegfd ) then

!        --- unknown in all nodes

            mult = 8  ! number of elements

         else if ( kprobatmp(1)>=idegfd .and. kprobatmp(2)>=idegfd .and. &
                   kprobatmp(9)<idegfd ) then

!        --- unknown in all nodes but not in center

            mult = 8  ! number of elements
            typesubdiv = 1

         else if ( kprobatmp(1)>=idegfd .and. kprobatmp(2)<idegfd ) then

!        --- unknown in vertices

            mult = 2  ! number of elements

         else if ( kprobatmp(1)<idegfd .and. kprobatmp(2)>=idegfd .and. &
                   kprobatmp(9)>=idegfd ) then

!        --- unknown in mid-side points and center

            mult = 8  ! number of elements
            typesubdiv = 2

         else if ( kprobatmp(1)<idegfd .and. kprobatmp(2)<idegfd .and. &
                   kprobatmp(9)>=idegfd ) then

!        --- unknown in center

            mult = 2  ! number of elements
            typesubdiv = 3

         else if ( kprobatmp(1)<idegfd .and. kprobatmp(2)>=idegfd ) then

!        --- unknown in mid-side points

            mult = 6  ! number of elements
            typesubdiv = 4

         end if  ! ( kprobatmp(1)>=idegfd .and. kprobatmp(2)>=idegfd )

      case (7)

!     --- quadratic triangle with extra point

         if ( debug ) call prinin ( kprobatmp, 7, 'kprobatmp' )
         if ( kprobatmp(1)>=idegfd .and. kprobatmp(2)>=idegfd ) then

!        --- unknown in all nodes

            mult = 4  ! number of elements

         else if ( kprobatmp(1)>=idegfd .and. kprobatmp(2)<idegfd ) then

!        --- unknown in vertices

            mult = 1  ! number of elements

         else if ( kprobatmp(1)<idegfd .and. kprobatmp(2)>=idegfd ) then

!        --- unknown in mid-side points

            mult = 4  ! number of elements
            typesubdiv = 1

         else if ( kprobatmp(1)<idegfd .and. kprobatmp(2)<idegfd .and. &
                   kprobatmp(7)>=idegfd ) then

!        --- unknown in centroid

            mult = 1  ! number of elements
            typesubdiv = 2

         end if  ! ( kprobatmp(1)>=idegfd .and. kprobatmp(2)>=idegfd )

      case (9)

!     --- bilinear quadrilateral with extra point

         if ( debug ) call prinin ( kprobatmp, 5, 'kprobatmp' )
         if ( kprobatmp(1)>=idegfd ) then

!        --- unknown in all vertices

            mult = 2  ! number of elements

         else if ( kprobatmp(5)>=idegfd ) then

!        --- unknown in centroid

            mult = 2  ! number of elements
            typesubdiv = 1

         end if  ! ( kprobatmp(1)>=idegfd )

      case (10)

!     --- linear triangle with extra point

         if ( debug ) call prinin ( kprobatmp, 4, 'kprobatmp' )
         if ( kprobatmp(1)>=idegfd ) then

!        --- unknown in all vertices

            mult = 1  ! number of elements

         else if ( kprobatmp(4)>=idegfd ) then

!        --- unknown in centroid

            mult = 1  ! number of elements
            typesubdiv = 1

         end if  ! ( kprobatmp(1)>=idegfd )

      end select  ! case ( ishape )

1000  call erclos ( 'topsubdiv2d1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'mult, typesubdiv', mult, typesubdiv
         write(irefwr,*) 'End topsubdiv2d1'

      end if  ! ( debug )

      end subroutine topsubdiv2d1
