      subroutine topsubdivmesh ( subver, ichois, &
                                 neleml, iresul, subtri, idegfd, &
                                 itrian, iskip, iskpar, &
                                 special, isol, centroid )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     subdivide volume into tetrahedrons or hexahedrons
!     or check if mesh must be subdivided into triangles
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     subdivision
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ichois, idegfd, iskip, iskpar(*), isol(*)
      integer, intent(inout) :: neleml, iresul(5,*), &
                                itrian(*), special, centroid
      logical, intent(in) :: subver
      logical, intent(inout) :: subtri

!     centroid      i/o   If 1 a 9 point element has been found of which the
!                         only 8 points have been filled
!     ichois         i    Choice parameter
!                         See subroutine topqu0
!     idegfd         i    Degree of freedom sequence number
!                         In the real case idegfd is number in the complex case
!                         it corresponds to the degree of freedom number
!                         refers to
!     iresul        i/o   See subroutine topqu0
!     iskip          i    Indicates if element groups must be skipped (>0)
!                         or not (0)
!     iskpar         i    array of length NELGRP that is only used if ISKIP>0
!                         It indicates if element group ielgrp must be skipped
!                         iskpar(ielgrp) = 0 or not iskpar(ielgrp) = 1
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     itrian        i/o   work array of length nelgrp, see topqu2
!     neleml        i/o   Number of "line"-elements
!     special       i/o   Standard situation: special=0 at input
!                         If special=2 and the unknown is present in the
!                         vertices, special is set to 0, else if the unknown is
!                         only present in the midside points, special
!                         remains unchanged
!     subtri         o    Indicates if the mesh must be subdivided into
!                         triangles and also the solution copied (true) or not
!     subver        i/o   Indicates if a "vertex" mesh must be created (true)
!                         or not
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nplsub, lensub, nelemt
      logical :: triangul

!     lensub         If zero the original mesh is used, otherwise
!                    lensub>0 gives the length of the KMESH part c
!     nelemt         Number of elements in the new mesh
!     nplsub         Number of points in the submesh consisting of elements
!                    with vertex nodes only
!     triangul       if true hexahedrons must be subdivided into tetrahedrons,
!                    if false in hexahedrons
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPQU2         Check if mesh must be subdivided and compute length of
!                    "KMESHC"
!     SEPQU3         Subdivide mesh into triangles
!     TO0096         Store unknown positioned in center of two-dimensional
!                    element only into array uout organized per element
!     TOPQ02         Create a submesh consisting of triangles or quadrilaterals
!                    where only the vertices are used.
!     TOPQ03         Check if vertex mesh must be subdivided and compute length
!                    of "KMESHC"
!     TOPQ04         Construct a submesh of vertices only in case of surface
!                    elements
!     TOPQ13         Fill extra line elements array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topsubdivmesh' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topsubdivmesh'
         write(irefwr,*) 'subver, subtri', subver, subtri
         write(irefwr,1) 'ichois, npelm', ichois, npelm
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( subver .and. ichois<6 ) then

!     --- In this case it is necessary to make a mesh of the vertices only
!         Reserve space for the submesh array
!         Restriction: this possibility has only been programmed when all
!         elements have the same shape

         if ( debug ) write(irefwr,1) 'kelma', kelma
         if ( kelma/=0 ) then

!        --- Not all elements have the same shape

            call topq03 ( nplsub, nelemlc, neleml )
            if ( ierror/=0 ) go to 1000

            if ( nelemlc>0 ) then

               allocate ( kmeshcsub(nelemlc*nplsub) )
               iresul(1,1) = nelemlc*nplsub
               if ( ierror/=0 ) go to 1000
               call topq04 ( nplsub, kmeshcsub )
               iresul(2,1) = nplsub
               iresul(4,1) = 1

            end if

            if ( neleml>0 ) then

!           --- Line elements encountered, make reservation
!               for these line elements

                allocate ( kmeshcline(2*neleml) )
                iresul(5,1) = 2*neleml
                if ( ierror/=0 ) go to 1000

!               --- Fill extra line elements array

                if ( ierror/=0 ) go to 1000
                call topq13 ( neleml, kmeshcline )

            end if

         else if ( npelm>4 ) then

            if ( npelm==9 ) then
               nplsub = 4
            else
               nplsub = 3
            end if
            allocate ( kmeshcsub(nelem*nplsub) )
            iresul(1,1) = nelem*nplsub
            call topq02 ( nelem, kmeshc, kmeshcsub, npelm, nplsub )
            iresul(2,1) = nplsub
            iresul(4,1) = 1

         else

            iresul(2,1) = npelm

         end if

         iresul(3,1) = nelem

      end if
      if ( ierror/=0 ) go to 1000

      if ( subtri ) then

!     --- Make a submesh consisting of triangles/tetrahedrons only
!         Make a fake array kmeshl in order to apply topqu2/3

         if ( ierror/=0 ) go to 1000

!        --- Check if mesh must be subdivided into subtriangles or not

         neleml = 0
         triangul = ichois/=7

         call sepqu2 ( idegfd, lensub, itrian, nelemt, iskip, iskpar, &
                       neleml, special, centroid, triangul )
         if ( ierror/=0 ) go to 1000
         if ( .not. triangul ) iresul(2,2) = 8
         if ( debug ) &
            write(irefwr,1) 'lensub, neleml, nelemt', lensub, neleml, nelemt

         if ( lensub>0 ) then

!        --- lensub>0, so mesh must be subdivided

            allocate ( kmeshctmp(lensub) )
            iresul(1,2)= lensub

!           --- If needed make reservation for "line-elements" array

            if ( neleml/=0 ) then
               allocate ( kmeshcline(2*neleml) )
               iresul(5,1) = 2*neleml
            end if

            if ( ierror/=0 ) go to 1000
            call sepqu3 ( itrian, neleml, kmeshcline )
            iresul(4,2) = 1

            if ( debug ) write(irefwr,1) 'neleml', neleml

         else if ( special==2 ) then

!        --- lensub = 0 and special = 2, unknown only present in centroid
!            of elements

            allocate ( kmeshcline(nelgrp), usolt(nelem) )
            iresul(5,1) = nelgrp
            iresul(2,3) = 1
            usoltemp => ks(isol(1))%sol
            if ( ierror/=0 ) go to 1000
            call to0096 ( usoltemp, idegfd, usolt, iskip, &
                          iskpar, kmeshcline )
            subtri = .false.

         else

!        --- lensub = 0, no subdivision

            if ( nelemt>0 .and. neleml==0 ) then

!           --- Standard situation

                 nelemt = nelem
                 iresul(1,2) = 0

            else

!           --- Special situation, there are line elements

                 if ( nelemt/=0 ) then
                      nelemt = nelem
                      iresul(1,2) = 0
                 end if

!                --- Make reservation for these line elements

                 allocate ( kmeshcline(2*neleml) )
                 iresul(5,1) = 2*neleml

!                --- Fill extra line elements array

                 if ( ierror/=0 ) go to 1000
                 call topq13 ( neleml, kmeshcline )

            end if

         end if
         iresul(3,2) = nelemt

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nelemt', nelemt
         write(irefwr,*) 'End topsubdivmesh'

      end if  ! ( debug )
      call erclos ( 'topsubdivmesh' )

      end subroutine topsubdivmesh
