      subroutine er0048 ( vector, nodes, n, kdeg, iardeg, kfun, &
                          rincre, bndcon )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.5  date 14-07-2011 Extra debug
!        version  4.4  date 26-11-2010 correction for scalars
!        version  4.3  date 17-11-2000 Correction error message
!
!   copyright (c) 1989-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array vector in the nodal points stored in array NODES
!     Real case
!     The vector is only filled for the degrees of freedom required
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     real
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulecons
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, nodes(n), kdeg, iardeg(*), kfun
      double precision vector(*), rincre(*)
      logical bndcon

!    bndcon  i   If true boundary conditions must be filled, otherwise
!                a vector must be created
!    iardeg  i   If KDEG<0, the positions 1 to -KDEG of IARDEG contain
!                the degrees of freedom to be filled
!    kdeg    i   Indication which degrees of freedom must be filled
!                Possibilities:
!                0:    All degrees of freedom
!                >0:   Only the KDEGth degree of freedom
!                <0:   The -KDEG degrees of freedom stored in array IARDEG
!                      must be filled
!    kfun    i   Indication of how the degrees of freedom must be filled.
!                Possibilities:
!                0:  Not filled
!               <0:  Constant value stored in RINCRE(-KFUN)
!               >0:  Function subroutine
!    n       i   Number of nodes stored in array NODES
!    nodes   i   Contains the node numbers that must be filled
!    rincre  i   Double precision array containing the values of the constant
!                if KFUN<0
!    vector i/o  Array of length NUSOL to be (partly) filled by this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, node, kdegun, kdegup, jdeg, k, kdim, inusol, inunkp
      double precision consta

!     consta         The value of the constant to be used from RINCRE
!                    if KFUN<0
!     i              General loop variable
!     inunkp         Number of unknowns in a point
!     inusol         Positions of degree of freedom in array IVECTR
!     jdeg           Actual degree of freedom
!     k              General loop variable
!     kdegun         Under boundary for loop over degrees of freedom
!     kdegup         Upper boundary for loop over degrees of freedom
!     kdim           Help parameter that is equal to two if NDIM >=2 and
!                    1 otherwise
!     node           Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision func, funcbc, funcol

!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     FUNC    Computes the value of the vector in a point as function of the
!             co-ordinates
!     FUNCBC  Computes the value of the boundary conditions in a point as
!             function of the co-ordinates
!     FUNCOL  Computes the value of the vector in a point as function of the
!             co-ordinates and the previous value
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1071   Special_function has not been implemented
!    1131   Degree of freedom required is larger than number of unknowns
!           per point
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'er0048' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from er0048'
         write(irefwr,1) 'n, npoint, kdeg', n, npoint, kdeg
         write(irefwr,1) 'kfun, indprfo, indprpo, ndim, nphyso', &
                          kfun, indprfo, indprpo, ndim, nphyso
         call prinin ( nodes, n, 'nodes' )
         write(irefwr,*) 'bndcon ', bndcon
  1      format ( a, 1x, (10i9) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute kdegun and kdegup

      if ( kdeg==0 ) then

!     --- kdeg  0, all degrees of freedom

         kdegun = 1
         kdegup = nphyso

      else if ( kdeg>0 ) then

!     --- kdeg>0, only degree of freedom kdeg

         kdegun = kdeg
         kdegup = kdeg
         if ( kdeg>nphyso ) then
            call errint ( kdeg, 1 )
            call errint ( nphyso, 2 )
            call errsub ( 1131, 2, 0, 0 )
         end if

      else

!     --- kdeg<0, only degrees of freedom stored in iardeg

         kdegun = 1
         kdegup = -kdeg

      end if
      if ( debug ) write(irefwr,1) 'kdegun, kdegup', kdegun, kdegup
      if ( kfun>1000 .and. kfun<2000 ) then

!     --- kfun<2000 is not implemented

         call errint ( kfun-1000, 1 )
         call errsub ( 1071, 1, 0, 0 )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Compute consta and kdim

      if ( kfun<0 ) consta = rincre(-kfun)
      if ( ndim==1 ) then
         kdim = 1
      else
         kdim = 2
      end if

!     --- Loop over degrees of freedom

      do k = kdegun, kdegup

!     --- Compute jdeg

         if ( kdeg<0 ) then

!        --- kdeg<0, find degree of freedom from IARDEG

            jdeg = iardeg(k)

         else

!        --- kdeg>=0, degree of freedom is equal to k

            jdeg = k

         end if
         if ( debug ) write(irefwr,1) 'jdeg, indprpo', jdeg, indprpo

!        --- Distinguish between the various cases

         if ( indprpo>0 ) then

!        --- indprpo>0, degrees of freedom are stored in kprobpo
!            Loop over nodes

            do i = 1, n
               node   = nodes(i)
               inusol = kprobpo(node,jdeg)
               if ( inusol>0 ) then

!              --- kprobpo (NODE,JDEG)>0, fill degree of freedom

                  if ( kfun<0 ) then

!                 --- kfun<0, constant value

                    vector(inusol) = consta

                  else if ( kfun>100000000 ) then

!                 --- scalar found

                     vector(inusol) = scalars(kfun-100000000)

                  else if ( kfun>2000 ) then

!                 --- kfun>2000, function of old solution

                    vector(inusol) = funcol ( kfun-2000, coor(1,node), &
                      coor(kdim,node), coor(ndim,node), vector(inusol) )

                  else if ( kfun>0 ) then

!                 --- kfun>0, function

                    if ( bndcon ) then

                       vector(inusol) = funcbc ( kfun, coor(1,node), &
                             coor(kdim,node), coor(ndim,node) )

                    else

                       vector(inusol) = func ( kfun, coor(1,node), &
                             coor(kdim,node), coor(ndim,node) )

                    end if

                  end if

               end if

            end do

         else if ( indprfo>0 ) then

!        --- indprfo>0, Non-constant number of degrees of freedom per point
!             Loop over nodes

            do i = 1, n
               node   = nodes(i)
               inusol = kprobfo(node)
               inunkp = kprobfo(node+1) - inusol
               if ( inunkp>=jdeg ) then

!              --- inunkp>=jdeg, fill degree of freedom

                  inusol = inusol + jdeg
                  if ( kfun<0 ) then

!                 --- kfun<0, constant value

                    vector(inusol) = consta

                  else if ( kfun>2000 ) then

!                 --- kfun>2000, function of old solution

                    vector(inusol) = funcol ( kfun-2000, coor(1,node), &
                      coor(kdim,node), coor(ndim,node), vector(inusol) )

                  else if ( kfun>0 ) then

!                 --- kfun>0, function

                    if ( bndcon ) then

                       vector(inusol) = funcbc ( kfun, coor(1,node), &
                             coor(kdim,node), coor(ndim,node) )

                    else

                       vector(inusol) = func ( kfun, coor(1,node), &
                             coor(kdim,node), coor(ndim,node) )

                    end if

                  end if

               end if

            end do

         else if ( jdeg<=nphyso ) then

!        --- Constant number of degrees of freedom per point

!            --- Loop over nodes

            do i = 1, n
               node   = nodes(i)
               inusol = (node-1) * nphyso + jdeg
               if ( kfun<0 ) then

!              --- kfun<0, constant value

                  vector(inusol) = consta

               else if ( kfun>2000 ) then

!              --- kfun>2000, function of old solution

                 vector(inusol) = funcol ( kfun-2000, coor(1,node), &
                   coor(kdim,node), coor(ndim,node), vector(inusol) )

               else if ( kfun>0 ) then

!              --- kfun>0, function

                 if ( bndcon ) then

                    vector(inusol) = funcbc ( kfun, coor(1,node), &
                          coor(kdim,node), coor(ndim,node) )

                 else

                    vector(inusol) = func ( kfun, coor(1,node), &
                          coor(kdim,node), coor(ndim,node) )

                 end if

               end if

            end do

         end if

      end do

1000  call erclos ( 'er0048' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End er0048'

      end if  ! ( debug )

      end
