      subroutine ercreatsub4 ( isol, itypvc, ivec, &
                               icompl, kvec, new, ifill, bndcon, &
                               isubr, inpcre, kfirst, ipstin, &
                               obstaclefound )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 05-13-2013 Extension with change_vector
!        version  1.1  date 05-12-2012 Long integers
!        version  1.0  date 27-12-2011
!
!   copyright (c) 2011-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute some parameters concerning the vector to be filled
!     If vector is new create space and clear vector
!     In case of boundary conditions, clear last part of vector
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), itypvc, ivec, isubr, kvec, ifill, &
              icompl, inpcre(*), kfirst, ipstin
      logical new, bndcon, obstaclefound

!     bndcon         i    If true boundary conditions must be filled, otherwise
!                         a vector must be created
!     icompl         i    Indication if the vector to be created is real (0) or
!                         complex
!                         May be changed if old vector is used
!     ifill          i    Indicates how the vector must be filled.
!                         Possible values:
!                         -1  The corresponding vector is not filled, &
!                             only created
!                          0  The corresponding vector is created and made
!                             equal to zero
!                         m>0 The filling of the vector is defined by m commands
!     inpcre         i    The integer input read is stored in array INPCRE from
!                         position 1. Hence in the call INPCRE(ISTART), with
!                         ISTART the starting position must be used.
!                         INPCRE will be filled in correspondence with array
!                         INPCRE for subroutine CREATV
!     ipstin         i    Starting address in array INPCRE
!     isol          i/o   Standard SEPRAN array containing information of
!                         the solution vectors (same as isol)
!     isubr          i    Indicates by what subroutine creatv01 has been called.
!                         Possible values:
!                         1:  By the user or CREATE
!                         2:  By PRESDF or PRESTM
!                         3:  CREAVC
!                         4:  PRESTM
!                         5:  ERCRET
!                         6:  ERPRES
!                         7:  ERPRES called from structure
!                         8:  ERCRET called from structure
!                         9:  ERCRET called from structure (change vector)
!     itypvc         i    Indication of the type of solution vector:
!                         110: solution vector
!                         115, 116: vectors of special structure
!     ivec           i    Sequence number of vector of special structure
!     kfirst         i    Value of first position of array INPCRE
!     kvec           i    Sequence number over the vectors
!                         Loop parameter in call ing subroutine
!     new            i    If true the vector to be created is new,
!                         otherwise an old vector is adapted
!     obstaclefound  i    If true values must be given in points in an obstacle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERGETIFILL     Check if values must be given in obstacles
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPFLKMESHAD2  Fill array kmesh part ad as well as the subarrays
!                    ipointobst and ielemobst (R^2 only)
!     SEPFLKMESHAD3  Fill array kmesh part ad as well as the subarrays
!                    ipointobst and ielemobst (R^3 only)
!     TOBVALZC       Clear part of usol corresponding to boundary conditions
!                    (comp)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     206   Input vector with illegal sequence number ivec
!    1127   The parameter ICOMPL has wrong value
!    2208   There are no obstacles, whereas essential boundary conditions
!           for the obstacle are required
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercreatsub4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercreatsub4'
         write(irefwr,1) 'ivec, kvec, isubr', ivec, kvec, isubr
         write(irefwr,*) 'bndcon ', bndcon
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check some parameters

      if ( icompl<0 .or. icompl>1 ) then
         call errint ( icompl, 1 )
         call errsub ( 1127, 1, 0, 0 )
      end if
      if ( itypvc==115 ) then
         if ( ivec<=0 .or. ivec>ndtvec ) then
            call errint ( ivec, 1 )
            call errint ( ndtvec, 2 )
            call errsub ( 206, 2, 0, 0 )
         end if
      end if

!     --- Check if information of obstacles must be generated

      call ergetifill ( inpcre, ifill, ipstin, kfirst, obstaclefound )

      if ( debug ) write(irefwr,*) 'obstaclefound ', obstaclefound
      if ( obstaclefound ) then

!     --- Information of obstacles must be generated
!         Activate or create ipointobst

         if ( numobst==0 ) then

!        --- Error no obstacles present

            call errsub ( 2208, 0, 0, 0 )
            go to 1000

         end if

         if ( debug ) write(irefwr,1) 'kelmad', kelmad
         if ( kelmad==0 ) then

!        --- kelmad = 0, kmesh part ad does not exist; create

            if ( ndim==2 ) then

!           --- ndim = 2

               call sepflkmeshad2 ( 2 )

            else

!           --- ndim = 3

               call sepflkmeshad3 ( 2 )

            end if  ! ( ndim==2 )

         end if  ! ( kelmad==0 )

      end if  ! ( obstaclefound )

!     --- Create space for solution array and compute indprf, indprh and indprp

      if ( debug ) &
         write(irefwr,1) 'kvec, itypvc, ivec, iprob', &
                          kvec, itypvc, ivec, iprob
      call sepcreasolbf1 ( isol(kvec), -itypvc, ivec, 'vector', icompl )
      ks(isol(kvec))%jcart = jcart
      usol => usoltemp

      if ( debug ) then
         write(irefwr,1) 'nphys, indprf, nphyso', nphys, indprf, nphyso
         call prinin ( isol(kvec), 5, 'isol' )
      end if  ! ( debug )

      if ( isubr==9 ) then

!     --- isubr =9, do not clear

      else if ( new .or. ifill==0 .and. .not. bndcon ) then

!     --- ifill = 0, or new array, clear array

         usol = 0d0

      else if ( bndcon .and. isubr/=7 ) then

!     --- isubr = 2, PRESDF/TM; set last part of array equal to zero

         if ( nbound>0 ) then

!        --- There are essential boundary conditions

            if ( icompl==0 ) then
               usol(kprobhinvo(nrusol+1:nrusol+nbound)) = 0d0
            else
               call tobvalzc ( usol )
            end if

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ercreatsub4'

      end if  ! ( debug )
      call erclos ( 'ercreatsub4' )

      end subroutine ercreatsub4
