      subroutine erfillcopvec ( nodes, kdeg, iardeg, uold, unew, &
                                indprfold, indprfnew, kprobfold, &
                                kprobfnew, nphysold, nphysnew, &
                                numnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-01-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy the contents of array uold into array unew for
!     all nodes stored in array nodes and the degrees of freedom indicated
!     in kdeg/iardeg
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     vector
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nodes(*), kdeg, iardeg(*), indprfold, indprfnew, &
              kprobfold(*), kprobfnew(*), nphysold, nphysnew, &
              numnodes
      double precision uold(*), unew(*)

!     iardeg         i    If KDEG<0, the positions 1 to -KDEG of IARDEG
!                         contain the degrees of freedom to be filled
!     indprfnew      i    indication if kprob part f has been filled (>0)
!                         or not (0) for the new vector
!     indprfold      i    indication if kprob part f has been filled (>0)
!                         or not (0) for the old vector
!     kdeg           i    Indication which degrees of freedom must be filled
!                         Possibilities:
!                         0:    All degrees of freedom
!                         >0:   Only the KDEGth degree of freedom
!                         <0:   The -KDEG degrees of freedom stored in array
!                               IARDEG must be filled
!     kprobfnew      i    KPROB part f: contains the number of unknowns per
!                         point accumulated with respect to the new vector
!     kprobfold      i    KPROB part f for old vector
!     nodes          i    Contains the node numbers that must be filled
!     nphysnew       i    Number of physical unknowns, or maximal degrees of
!                         freedom per point with respect to the new vector
!     nphysold       i    Number of physical unknowns, or maximal degrees of
!                         freedom per point with respect to the old vector
!     numnodes       i    Number of nodal points in array nodes
!     unew          i/o   New solution vector
!     uold           i    Array containing the old vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kdegun, kdegup, k, jdeg, node, i

!     i              Counting variable
!     jdeg           Actual degree of freedom
!     k              Counting variable
!     kdegun         Under boundary for loop over degrees of freedom
!     kdegup         Upper boundary for loop over degrees of freedom
!     node           Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1131   Degree of freedom required is larger than number of unknowns
!           per point
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfillcopvec' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillcopvec'
         call ertrace

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( kdeg==0 ) then

!     --- kdeg  0, all degrees of freedom

         kdegun = 1
         kdegup = min(nphysnew,nphysold)

      else if ( kdeg>0 ) then

!     --- kdeg>0, only degree of freedom kdeg

         kdegun = kdeg
         kdegup = kdeg
         if ( kdeg>nphysnew ) then
            call errint ( kdeg, 1 )
            call errint ( nphysnew, 2 )
            call errsub ( 1131, 2, 0, 0 )
            go to 1000
         end if  ! ( kdeg>nphysnew )
         if ( kdeg>nphysold ) then
            call errint ( kdeg, 1 )
            call errint ( nphysold, 2 )
            call errsub ( 1131, 2, 0, 0 )
            go to 1000
         end if  ! ( kdeg>nphysold )

      else

!     --- kdeg<0, only degrees of freedom stored in iardeg

         kdegun = 1
         kdegup = -kdeg

      end if

!     --- Loop over degrees of freedom

      do k = 1, kdegup-kdegun+1

!     --- Compute jdeg

         if ( kdeg<0 ) then

!        --- kdeg<0, find degree of freedom from IARDEG

            jdeg = iardeg(k+kdegun-1)

         else

!        --- kdeg>=0, degree of freedom is equal to k

            jdeg = k+kdegun-1

         end if

!        --- Distinguish between the various cases

         if ( indprfnew==0 ) then

!        --- indprfnew = 0, hence constant number of degrees of freedom in unew

            if ( indprfold==0 ) then

!           --- constant number of degrees of freedom in uold

               do i = 1, numnodes
                  node = nodes(i)
                  unew((node-1)*nphysnew+jdeg) = &
                     uold((node-1)*nphysold+jdeg)
               end do  ! i = 1, numnodes

            else

!           --- variable number of degrees of freedom in uold

               do i = 1, numnodes

                  node = nodes(i)
                  if ( kprobfold(node+1)>=kprobfold(node)+jdeg ) then

!                 --- Degree of freedom available in uold

                     unew((node-1)*nphysnew+jdeg) = &
                        uold(kprobfold(node)+jdeg)

                  end if  ! ( kprobfold(node+1)>=kprobfold(node)+jdeg )

               end do  ! i = 1, numnodes

            end if  ! ( indprfold==0 )

         else

!        --- indprfnew>0, hence variable number of degrees of freedom in unew

            if ( indprfold==0 ) then

!           --- constant number of degrees of freedom in uold

               do i = 1, numnodes

                  node = nodes(i)
                  if ( kprobfnew(node+1)>=kprobfnew(node)+jdeg ) then

!                 --- Degree of freedom available in unew

                     unew(kprobfnew(node)+jdeg) = &
                        uold((node-1)*nphysold+jdeg)

                  end if  ! ( kprobfnew(node+1)>=kprobfnew(node)+jdeg )

               end do  ! i = 1, numnodes

            else

!           --- variable number of degrees of freedom in uold

               do i = 1, numnodes

                  node = nodes(i)
                  if ( kprobfnew(node+1)>=kprobfnew(node)+jdeg .and. &
                       kprobfold(node+1)>=kprobfold(node)+jdeg ) then

!                 --- Degree of freedom available in unew and uold

                     unew(kprobfnew(node)+jdeg) = &
                        uold(kprobfold(node)+jdeg)

                  end if  ! ( kprobfnew(node+1)>=kprobfnew(node)+jdeg ... )

               end do  ! i = 1, numnodes

            end if  ! ( indprfold==0 )

         end if  ! ( indprfnew==0 )

      end do  ! k = 1, kdegup-kdegun+1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erfillcopvec'

      end if  ! ( debug )
      call erclos ( 'erfillcopvec' )

      end
