      subroutine erfillquadsur1 ( nnodes, nodes, origin, radius, &
                                  mean_val, delta, kdeg, vector )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-03-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill boundary condition of the shape quadratic function with
!     given mean value and boundary layer width along a surface
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     vector
!     quadratic
!     polynomial
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, nodes(*), kdeg
      double precision origin(3), radius, delta, mean_val, &
                       vector(*)

!     delta          i    boundary layer thickness
!     kdeg           i    Indication which degrees of freedom must be filled
!                         Possibilities:
!                         0:    All degrees of freedom
!                         >0:   Only the KDEGth degree of freedom
!                         <0:   The -KDEG degrees of freedom stored in array
!                               IARDEG must be filled
!     mean_val       i    mean value of the boundary condition
!     nnodes         i    Number of nodes in nodes
!     nodes          i    Contains the node numbers that must be filled
!     origin         i    Contains the coordinates of the origin of the circle
!     radius         i    Radius of sphere or circle
!     vector        i/o   Array to be filled with the boundary condition
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inode, i, inusol, inunkp
      double precision value, h, r, rhat, uc

!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     inode          Node sequence number
!     inunkp         Number of degrees of freedom in point
!     inusol         Position of unknown in array USOL
!     r              distance to centroid
!     rhat           Relative distance to centroid
!     uc             Computed maximum value in middle of circle
!     value          Value of boundary condition
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The vector is filled in the following way:
!
!     r = ||x-o||
!     r^ = (R-r)/delta
!     uc = mean_val/(1-delta/R(4-delta/R)/6)
!     if ( r^ < 1 ) then
!        u = uc r^ ( 2-r^)
!     else ( r^ > 1 )
!        u = uc
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfillquadsur1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillquadsur1'
         write(irefwr,*) 'delta, radius, mean_val ', &
                          delta, radius, mean_val
         call prinrl ( origin, 3, 'origin' )

      end if
      if ( ierror.ne.0 ) go to 1000

      if ( delta.gt.0d0 ) then

!     --- delta > 0, uc = mean_val/(1-delta/R(4-delta/R)/6)

         h = delta/radius
         uc = mean_val/(1d0-h*(4d0-h)/6d0)
         if ( debug ) write(irefwr,*) 'h, uc ', h, uc

      end if

!     --- For all nodes do

      do i = 1, nnodes

         inode = nodes(i)


         if ( delta.le.0d0 ) then

!        --- delta = 0, value = mean_val

            value = mean_val

         else

!        --- delta > 0, r = ||x-o||

            r = sqrt ( (coor(1,inode)-origin(1))**2 + &
                       (coor(2,inode)-origin(2))**2 + &
                       (coor(3,inode)-origin(3))**2 )

!           --- r^ = (R-r)/delta

            rhat = (radius-r)/delta
            if ( rhat.lt.1d0 ) then

!           --- r^ < 1 ) then
!               u = uc r^ ( 2-r^)

               value = uc*rhat*(2d0-rhat)

            else

!           --- r^ >= 1
!               u = uc

               value = uc

            end if  ! ( rhat.lt.1d0 )
            if ( debug ) &
               write(irefwr,*) 'r, rhat,  value ', r, rhat, value

         end if  ! ( delta.le.0d0 ) then

         if ( indprpo.gt.0 ) then

!        --- indprpo>0, degrees of freedom are stored in kprobpo

            inusol = kprobpo(inode,kdeg)
            if ( inusol.gt.0 ) vector(inusol) = value

         else if ( indprfo.gt.0 ) then

!        --- indprfo>0, Non-constant number of degrees of freedom per point

            inusol = kprobfo(inode)
            inunkp = kprobfo(inode+1) - inusol
            if ( kdeg.le.inunkp ) vector(inusol) = value

         else

!        --- indprfo=0, Constant number of degrees of freedom per point

            inusol = (inode-1) * nphyso + kdeg
            vector(inusol) = value

         end if  ! ( indprpo.gt.0 )

      end do  ! i = 1, nnodes

1000  call erclos ( 'erfillquadsur1' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erfillquadsur1'

      end if

      end
