      subroutine ervl15 ( nr, m, maxlen, eps, alpha, beta, tk, w, z, &
                          minit, multnr, maxtk )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.1  date 05-12-2012 Long integers
!        version  4.0  date 05-11-1992 Z.Vlasek: new parameter list: maxtk
!                                      avoid undef when e.v.=0)
!        version  3.1  date 04-11-1992 Z.Vlasek: missing ierr when e.v.=0
!
!   copyright (c) 1986-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Computing of the eigenvectors of the tridiagonal matrix
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nr, m, maxlen, multnr, minit(nr), maxtk
      double precision alpha(m), beta(m), tk(maxtk,2), w(maxlen), &
                       z(m,maxlen), eps

!     alpha      i    Diagonal of tridiagonal matrix
!     beta       i    Subdiagonal of tridiagonal matrix
!     eps        i    accuracy of eigenvalues
!     m          i    format of tridiagonal matrix
!     maxlen     i    nr x multm
!     maxtk      i    format of the matrix tk(maxtk,2), it must be nr<=maxtk
!     minit      i    for each eigenvalue: number of iterations to get it, &
!                     it must be minit(*)<=m
!     multnr     o    Number of eigenvalues of the tridiagonal matrix including
!                     double ones
!     nr         i    number of eigenvalues that are computed
!     tk         i    Array containing the lower resp upper bounds of the
!                     eigenvalues
!     w          o    Array containing the computed eigenvalues of the
!                     tridiagonal matrix
!     z          o    Array of length m x maxlen containing the maximal maxlen
!                     eigenvectors of the tridiagonal matrix of length m each
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision bl, bu, eps1, e2(m), work(m,6)
      integer i, ierr, j, mult, multm

!     bl
!     bu
!     e2              Work array of length m
!     eps1
!     i
!     ierr
!     j
!     mult
!     multm
!     work            Work array of length 6 x m
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     ERVL09    finds those eigenvalues of a tridiagonal
!               symmetric matrix which lie in a specified interval and their
!               associated eigenvectors, using bisection and inverse iteration
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     467   multiplicity of eigenvalue larger than 10
!     468   eigenvector  fails to converge in 20 iterations
!     708   Too much eigenvectors of tridiagonal matrix
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ervl15' )

!     --- preparing input for tsturm

      do j=1,m-1
         beta(m+1-j)=beta(m-j)
      end do
      do j=1,m
         e2(j)=beta(j)**2
      end do

!     --- loop for eigenvectors of tridiagonal matrix

      multnr = 1
      do j=1,nr

         if ( tk(j,1)<=1d-9 * tk(nr,2) ) then

!        --- Eigenvalue approximately equal to 0

!              It may happen even tk(j,1)<0

!              When called from ereig1 or from ereig0, &
!              the values w(*) will be discarded anyway

!              Skip the computation of the eigenvector, &
!              fill it temporary, to avoid undefined values in ervl16

!              This eigenvector is filled later by a constant in ervl12

            mult = 1
            w(multnr) = tk(j,1)

            z(1,multnr) = 1d0
            do i=2,m
              z(i,multnr) = 0d0
            end do

            ierr = 0

         else

!        --- Eigen values larger than 0
!            slight enlargement of interval

            bl=tk(j,1)-eps
            bu=tk(j,2)+eps

!           --- error set for tsturm

            eps1=0d0

!           --- call for computation of eigenvalue and eigenvector in interval
!               The maximal multiplicity is set equal to nr+10-j, which
!               implies that the maximal multiplicity of the last eigenvector is
!               equal to 10, and of the other ones larger than 10

            multm = nr+10-j
            call ervl09(m,minit(j),eps1,alpha,beta,e2,bl,bu,multm,mult, &
                        w(multnr),z(1,multnr),ierr,work(1,1),work(1,2), &
                        work(1,3), work(1,4), work(1,5), work(1,6) )
            do i=minit(j)+1,m
              z(i,multnr)=0d0
            end do

         end if
         multnr = multnr + 1
         if ( multnr+mult-1>maxlen+1 ) then
            call errint ( multnr+mult-1, 1 )
            call errint ( maxlen+1, 2 )
            call errsub ( 708, 2, 0, 0 )
            go to 1000
         end if

         if ( ierr>0 ) then

            call errint ( j, 1 )
            if ( ierr>m*4 ) then

               call errsub ( 468, 1, 0, 0 )

            else

               call errsub ( 467, 1, 0, 0 )

            end if

         end if
         if ( ierror/=0 ) go to 1000
      end do
      multnr = multnr - 1
1000  call erclos ( 'ervl15' )

      end
