      subroutine sepactbuild
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-02-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Activate and create arrays for module sepmodulebuild
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mass_matrix
!     right_hand_side
!     stiffness_matrix
!     assembly
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecpack
      use sepmodulevecs
      use sepmodulematstr
      use sepmodulematr
      use sepmodulekprob
      use sepmodulemain
      use sepmoduleintmat
      use sepmodulesmf
      use sepmoduleoldvecs
      use sepmodulebuild
      use sepmodulecons
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k, ih, nch
      integer, pointer :: islold(:)
      logical :: complexmat
      type (intmatrix), pointer :: IM
      type (globmat), pointer :: A

!     A              Refers to realmt(iseqmt)
!     IM             Refers to intmt(intmat)
!     complexmat     If true the matrix is complex
!     ih             Help variable to store a constant temporarily
!     islold         Reference to all SEPRAN solution vectors
!     k              Counting variable
!     nch            Number of characters in a string
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPCRDIAGMATR  create a new diagonal matrix in realmt
!     SEPCREASOL110  Activate or create arrays of type 110
!     SEPCRMATRIX    Create permanent space for matrix
!     SEPFILLOLDSOLS Fill array istold with information of old solution vectors.
!                    Reserve space for the necessary arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     720   Array ISLOLD has not been filled.
!     723   Array ISLOLD does not correspond to problem number IPROB.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepactbuild' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepactbuild'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(iseqim)

      islold => SMF%islold

      complexmat = IM%complexm
      icompl = 1
      if ( complexmat ) icompl = 2

!     --- MATRIX

      if ( clrmat .or. addmat ) then

!     --- matrix must be computed or changed

         if ( IM%nosepmat ) then

!        --- Part 1 is not used (petsc matrix)

            call sepcrmatrix ( stiffmat(1) )
            A => realmt(stiffmat(1))

         else

!        --- Standard case

            call sepcrmatrix ( stiffmat(1) )
            A => realmt(stiffmat(1))
            if ( IM%lapshift ) then
               icompl = 1
               if ( complexmat ) icompl = 2
               allocate ( A%Dlapshift(nrusol*icompl) )
               if ( clrmat ) A%Dlapshift = 0d0
            end if  ! ( IM%lapshift )

         end if  ! ( IM%nosepmat )
         iseqmt = stiffmat(1)
         A%complexmat = complexmat

         if ( idefcr==1 ) then
            call sepcrmatrix ( stiffmat(2) )
            iseqmt = stiffmat(2)
            A => realmt(iseqmt)
            A%complexmat = complexmat

         end if  ! ( idefcr==1 )

      end if  ! ( clrmat .or. addmat )

!     --- RIGHT-HAND SIDE

      if ( clrvec.or. addvec .or. effbc.and.nbound>0 ) then

         if ( idefcr==1 ) numrhs = 2
         icompl = 0
         if ( complexmat ) icompl = 1
         do k = 1, numrhs
            call sepcreasol110 ( SMF%irhsd(k), 'rhsd', icompl )
         end do
         icompl = icompl+1

      end if  ! ( clrvec.or. addvec .or. effbc.and.nbound>0 )

!     --- Mass matrix

      if ( imas==1 ) then

!     --- Diagonal mass matrix

         call sepcrdiagmatr ( massmat )

      else if ( imas>=2 ) then

!     --- Non-diagonal mass matrix

         call sepcrmatrix ( massmat )

      end if  ! ( imas==1 )

      ih = nelem
      if ( isubdv>0 ) then
         ih = ih*2**ndim          ! standard subdivision of elements
      else
         ih = ih*6                ! sometimes subdivision of elements
      end if  ! ( isubdv>0 )

      if ( debug ) &
         write(irefwr,1) 'nelem, ndim, numnat, ih', nelem, ndim, numnat, ih

      if ( associated(flipflop) ) then

!     --- flip flop array has been created before, check length

         if ( size(flipflop)<ih ) then
            deallocate ( flipflop )
            allocate ( flipflop(ih) )
         end if  ! ( size(flipflop)<ih )

      else

!     --- flip flop array has not been created before

         allocate ( flipflop(ih) )

      end if  ! ( associated(flipflop)

!     --- Activate solution arrays
!         This part may be changed in the future
!         Depends on subroutines elem
!         Compute pointers for array islold

      numtypesols = 1  ! number of different types of old vectors (at least 1)
      oldsols(1)%nunkp = nunkp

      if ( debug ) then
         write(irefwr,1) 'numtypesols, numold', numtypesols, numold
         write(irefwr,*) 'oldsol ', oldsol
      end if  ! ( debug )

      if ( numold>0 ) then

!     --- numold>0, hence store information concerning the arrays uold into
!                   array istold

         call sepfilloldsols ( islold, 1 )

      else if ( oldsol ) then

!     --- numold = 0, islold contains at most one solution vector.
!                     which must be of the same structure as the solution
!                     vector
!         oldsol, so array islold must be used

         if ( ks(islold(1))%typevec/=110 ) then

!        --- Error 720: array islold has not been filled

            call errint ( 1, 1 )
            call errint ( ks(islold(1))%typevec, 2 )
            call errsub ( 720, 2, 0, 0 )

         end if

         if ( ks(islold(1))%iprob/=iprob ) then
            nch = index(names(1),' ')-1
            if ( nch<=0 ) nch = len(names(1))
            call errchr ( names(1)(1:nch), 1 )
            call errint ( 1, 1 )
            call errint ( 1, 2 )
            call errint ( iprob, 3 )
            call errint ( ks(islold(1))%iprob, 4 )
            call errsub ( 723, 4, 0, 1 )
         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepactbuild'

      end if  ! ( debug )
      call erclos ( 'sepactbuild' )

      end subroutine sepactbuild
