      subroutine sepaddmat ( matr1, matr2, &
                             alpha1, alpha2, beta1, beta2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-01-2018
!
!   copyright (c) 1999-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    add alpha times matrix amat1 to beta times matrix amat2 and
!    store in amat1
!    amat1 must be a large matrix
!    amat2 may be either a large matrix or a diagonal matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     multiplication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      use sepmodulematr
      use sepmodulemain
      use sepmoduleintmat
      use sepmodulematstr
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: matr1, matr2
      double precision :: alpha1, alpha2, beta1, beta2

!     alpha1  i    multiplication factor:  alpha = alpha1 + i alpha2
!     alpha2  i    multiplication factor:  alpha = alpha1 + i alpha2
!     beta1   i    multiplication factor:  beta = beta1 + i beta2
!     beta2   i    multiplication factor:  beta = beta1 + i beta2
!     matr1   i    integer array containing information concerning the
!                  large matrix amat1
!     matr2   i    integer array containing information concerning the
!                  matrix amat2
!                  amat2 may be a large matrix or a diagonal matrix
!                  stored as right-hand-side vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double complex :: alpha, beta
      double precision, pointer :: M2(:), S3(:)
      double complex, pointer :: S1(:), S2(:)
      type (intmatrix), pointer :: IM
      double precision :: timebefore(2)

!     IM             Refers to intmt(intmat)
!     M2             Refers to real matrix corresponding to matr2
!     S1             Refers to complex matrix corresponding to matr1
!     S2             Refers to complex matrix corresponding to matr2
!     S3             Refers to real matrix corresponding to matr1
!     alpha          complex number alpha1+i alpha2
!     beta           complex number beta1+i beta2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERSETTIME      Get the CPU time at start of subroutine
!     SEPPRINTMAT    Print matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepaddmat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepaddmat'
         write(irefwr,2) 'alpha1, alpha2, beta1, beta2', &
                          alpha1, alpha2, beta1, beta2
         write(irefwr,1) 'matr1, matr2', matr1, matr2
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- tests

      intmat = realmt(matr1)%intmatref

      IM => intmt(intmat)
      if ( itime>0 ) call ersettime ( timebefore )

      if ( realmt(matr1)%complexmat ) then

!     --- First matrix is complex

         alpha = cmplx(alpha1,alpha2, kind=8)
         beta = cmplx(beta1,beta2, kind=8)
         S1 => realmt(matr1)%Sz_ii
         if ( realmt(matr2)%diagonal ) then

!        --- Second matrix diagonal

            S1 = alpha*S1
            M2 => realmt(matr2)%S_ii
            S1(IM%diag) = S_ii(IM%diag)+beta*M2

         else if ( realmt(matr2)%complexmat ) then

!        --- Both matrices complex

            S2 => realmt(matr2)%Sz_ii
            S1 = alpha*S1+beta*S2

         else

!        --- Second matrix real

            M2 => realmt(matr2)%S_ii
            S1 = alpha1*S1+beta1*M2

         end if

      else

!     --- Both matrices real

         S3 => realmt(matr1)%S_ii
         M2 => realmt(matr2)%S_ii
         if ( realmt(matr2)%diagonal ) then

!        --- Second matrix diagonal

            S3 = alpha1*S3
            S3(IM%diag) = S_ii(IM%diag)+beta1*M2

         else

!        --- Both matrix not diagonal

            S3 = alpha1*S3+beta1*M2

         end if
      end if

      realmt(matr1)%decomp = .true.

1000  if ( debug ) then

!     --- Debug information

         call sepprintmat ( matr1 )
         write(irefwr,*) 'End sepaddmat'

      end if  ! ( debug )
      call erclmn ( 'sepaddmat', timebefore )

      end subroutine sepaddmat
