      subroutine sepalgcr ( normrhsd, schurmat, A, usol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-05-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve a system of linear equations of special form using the
!     Augmented Lagrange appoach of Benzi et al.
!     The method is accelerated by the GCR process
!     Old subroutine pralgcr
! **********************************************************************
!
!                       KEYWORDS
!
!     simple
!     linear_solver
!     matrix
!     real
!     krylov
!     conjugate_gradients
!     augmented_lagrange
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecpu
      use sepmodulesimple
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: normrhsd, schurmat(*)
      type (rmatrix) :: A
      double precision :: usol(A%neq)

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     normrhsd       i    Norm of right-hand side vector
!     schurmat       i    Contains the approximate inverse Schur complement
!                         matrix
!     usol           o    solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, i, iter, niteru, &
              maxiter, itimesav, niteruacc, ngcr, iref, &
              niteracc, niterutot
      double precision, allocatable :: svec(:,:), vvec(:,:), work(:), rud(:)
      double precision rnorm, normu
      logical matlab
      double precision :: timebefore(2)

!     i              Counting variable
!     iref           File reference number
!     iter           Iteration sequence number
!     itimesav       Help parameter to save the parameter itime
!     k              Counting variable
!     matlab         If true matlab input is generated
!     maxiter        Maximum number of iterations
!     ngcr           maximum number of GCR iterations
!     niteracc       Accumulated number of GCR iterations
!     niteru         Number of iterations to solve the u-equations
!     niteruacc      Accumulated number of iterations to solve the u-equations
!     niterutot      Total number of u iterations in all GCR steps
!     normu          norm of u-vector
!     rnorm          norm of vector
!     rud            Work array of length nu+np to store right-hand side
!                    of intermediate equation
!     svec           Work array of size ngcr x nu to store the vectors
!                    s during the iteration
!     vvec           Work array of size ngcr x nu to store the vectors
!                    v during the iteration
!     work           Work array of length nu to store intermediate vectors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inisetnewref

!     ASKCLK         print elapsed time
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     ERSETTIME      Get the CPU time at start of subroutine
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
!     PRINRL         Print 1d real vector
!     SEPALITER      Perform the preconditioning step in the GCR part of the
!                    Augmented Lagrange Method
!     SEPGCRCYCLE    Perform one step of the gcr process
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     676   The CG-process has terminated because the number of iterations
!           exceeds the maximum value
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Start with The matrices Q, R, D and the vectors u, p, ru and r_p
!     For k := 0 (1) ngcr do
!        Perform one step of simple to compute du and dp
!        s^(k+1) := (du,dp), r = (ru,r_p)
!        v^(k+1) := A s^(k+1)
!        For i := 1 (1) k do
!           alpha := ( v^(k+1), v^i)
!           v^(k+1) := v^(k+1)- alpha v^i
!           s^(k+1) := s^(k+1)- alpha s^i
!        v^(k+1) := v^(k+1)/||v^(k+1)||
!        s^(k+1) := s^(k+1)/||v^(k+1)||
!        alpha := ( r, v^(k+1))
!        (u,p) := (u,p) + alpha s^(k+1)
!        r := r - alpha v^(k+1)
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data niteruacc /0/, niteracc /0/
! ======================================================================
!
      call eropen ( 'sepalgcr' )
      call ersettime ( timebefore )

      debug = .false.
      matlab = .false.
      itimesav = itime
      itime = 0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepalgcr'
         write(irefwr,1) 'nu, np ', nu, np
         call prinrl ( resup, nu+np, 'ru' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if

      if ( matlab ) then
         iref = inisetnewref()
         open ( unit=iref, file = 'fu.dat' )
         do i = 1, nu
            write(iref,10) ru(i)
 10         format ( e15.8 )
         end do
         close ( unit=iref )
         open ( unit=iref, file = 'fp.dat' )
         do i = 1, np
            write(iref,10) rp(i)
         end do
         close ( unit=iref )
         call inirmref ( iref )

      end if

      ngcr = inpsol(13,1)   ! (maximum) number of gcr iterations
      allocate( svec(nu+np,0:ngcr+1) , vvec(nu+np,0:ngcr+1), work(nu), &
                rhsu(nu), rud(nu+np), stat = error)
      if ( error/=0 ) call eralloc ( error, (nu+np)*(ngcr+2), 'svec' )
      if ( ierror/=0 ) go to 1000

      rnorm = norm (  ru, A )
      if ( debug ) write(irefwr,2) 'norm', rnorm

!     --- Check if accuracy has been reached

      if ( A%irelerr==10 ) A%irelerr = 1
      if ( debug ) write(irefwr,*) 'irelerr, eps', A%irelerr, A%eps
      if ( A%irelerr==-1 ) then

!     --- Relative error compared to rhs

         A%eps = A%eps * normrhsd

      else if ( A%irelerr==1 ) then

!     --- Relative error compared to initial residue

         A%eps = A%eps * rnorm

      end if
      if ( A%irelerr/=0 .and. rnorm<=A%eps ) then

!     --- Initial residual already small

         go to 1000

      end if
      niterutot = 0

!     --- Print heading if iprint>0

      if ( A%iprint>0 ) write ( irefwr, 100 )

100   format(//' Output for the AL method'//)

      if ( A%iprint==1 ) then

!     --- iprint = 1,  Print information

         write ( irefwr, 110 ) rnorm
110      format ( ' The 2-norm of the initial residual', ss, 1pe17.2e2 )

      else if ( A%iprint>=2 ) then

!     --- iprint>=2,  Maximal output

         normu = norm (  ru, Q )
         write ( irefwr, 120 ) 0, rnorm, normu, 0
120      format ( '  Iter    residual  res_u       iter_u'/ &
                  ' number   |r_k|_2   |ru_k|_2' // &
                    i5, 3x, ss, 2(1pe10.2e2), i8 )

      end if
      iter = 0

!     --- For k := 0 (1) maxiter-1 do

      do k = 0, A%nmax-1

!     --- Perform one step of AL to compute du and dp
!         s^(k+1) := (du,dp), r = (ru,r_p)

         iter = iter+1
         call sepaliter ( svec(1,0), schurmat, niteru, work )
         if ( ierror/=0 ) go to 1000

!        --- Perform one step of the gcr process.


!        --- v^(k+1) := A s^(k+1)

         vvec(:,0) = A*svec(:,0)
         if ( ierror/=0 ) go to 1000

         call sepgcrcycle ( svec, vvec, ngcr, k, rnorm, resup, usol, A )
         if ( ierror/=0 ) go to 1000

         if ( A%iprint>=2 ) then

!        --- iprint>=2,  Maximal output

            normu = norm ( ru, Q )
            write ( irefwr, 400 ) iter, rnorm, normu, niteru
400         format ( i5, 3x, ss, 2(1pe10.2e2), i8 )

         end if

         niterutot = niterutot+niteru

         if ( rnorm<=A%eps ) go to 500

      end do  ! k = 1, ngcr

!     --- If the subroutine reaches this stage, too much iterations have
!         been carried out

      call errint ( iter, 1 )
      if ( inpsol(22,1)==1) then
         inpsol(1,1) = iter
         call errwar ( 676, 1, 0, 0 )
      else
         call errsub ( 676, 1, 0, 0 )
      end if

500   niteracc = niteracc + iter
      niteruacc = niteruacc + niterutot
      if ( A%iprint>=1 ) then

!     --- iprint = 1  Print information

         write ( irefwr, 510 ) iter, rnorm, niterutot, &
                               niteracc, niteruacc
510      format('The number of executed iterations in the', &
                ' GCR-process is', i5/ &
                'The 2-norm of the residual is:', ss,1pe12.2e2/ &
                'The number of u iterations is ', i6/ &
                'The accumulated number of GCR-iterations is', i8/ &
                'The accumulated number of u-iterations is', i8 )

      end if

!     --- Fill information in citerinfo
!         This is only used for performance checking

      niterout = niteracc
      niteruout = niteruacc
      niterpout = 0
      call askclk
      timesubr = tim3-timebefore(1)

1000  deallocate ( svec, vvec, work, rhsu, rud, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'svec' )

      if ( debug ) write(irefwr,*) 'End sepalgcr'
      itime = itimesav
      call erclos ( 'sepalgcr' )

      end
