      subroutine sepbuildstart
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 13-12-2017 New storage of intmat
!        version  1.0  date 01-02-2016
!
!   copyright (c) 2016-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform some tests and fill intmat with defaults in case
!     intmat has not been filled.
!     First routine to be called in sepbuild
!     If changed, also change sepreacstart
! **********************************************************************
!
!                       KEYWORDS
!
!     diagonal_matrix
!     mass_matrix
!     right_hand_side
!     stiffness_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulexxx
      use sepmodulekprob
      use sepmodulemain
      use sepmoduleintmat
      use sepmodulesmf
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: lniincommat=25
      integer :: iincommat(lniincommat)

!     iincommat      Input array for subroutine incommat
!                    See subroutine read00 for a description
!     lniincommat    Length of iincommat
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     INITST         Standard initializations for TO0050
!     PRININ         print 1d integer array
!     SEPACTBUILD    Activate and create arrays for module sepmodulebuild
!     SEPBUILDACTSOL Activate array usol and renumber if necessary
!     SEPBUILDCHINTM Check second array intmat
!     SEPBUILDFICT   Store the fluid elements corresponding to each lambda point
!                    in array kprobfict
!     SEPBUILDINTMAT Fill array intmat with default values
!     SEPFILLBUILD   Fill pointers in sepmodulebuild for subroutine build
!                    Create relation between arrays and buffer
!     SEPFILLKPROB   Fill parameters in module sepmodulekprob
!     SEPFILLKPROBN  Fill parameters in module sepmodulekprob
!     SEPFILLKPROBP  Make a reference between ibuffr and parinfo
!     SEPFILLMATSTR  Fill parameters in module sepmodulematstr
!     SEPFILLMATSTR2 Fill parameters in module sepmodulematstr
!     SEPFILLMDBUILD initialize some parameters for subroutine BUILD
!     SEPINITBUILD   Allocate some arrays for subroutine to0050
!                    Initialize some parameters
!     TO0005         Standard initializations for TO0050
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepbuildstart' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepbuildstart'
         write(irefwr,1) 'intmat', intmat
         call prinin ( iinbld, 9, 'iinbld' )
         call ertrace
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      bufchanged = .true.

!     --- Detect iprob from iinbld or intmat

      if ( iinbld(9)>0 ) iprob = iinbld(9)

      if ( intmat<=0 .or. intmat>lastintmat ) then

!     --- Array intmat has not been filled, fill with default values

         call sepbuildintmat ( iincommat, lniincommat )
         if ( ierror/=0 ) go to 1000

      end if  ! ( SMF%intmat(2,1)/=102 )

!     --- Array intmat has been filled

      call sepfillmatstr
      if ( iinbld(9)==0 ) then
         iprob = intmt(intmat)%iprob
      else
         iprob = iinbld(9)
      end if  ! ( iinbld(9)==0 )
      if ( debug ) write(irefwr,1) 'iprob', iprob

!     --- Compute some parameters

      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      kmeshloc => kmesh

      call sepfillkprob
      nrusol = kprobloc(66)
      call sepfillkprobp
      bufchanged = .false.

      call sepfillkprobn
      indprz = kprobloc(52)
      iskipelem = kprobloc(53)

      if ( nusol-nbound-npboun==0 ) go to 1000  ! there are no unknowns

!     --- if usol exits, activate array and renumber if necessary

      if ( SMF%iusol/=0 .and. nbound+npboun>0 ) call sepbuildactsol
      if ( ierror/=0 ) go to 1000

!     --- Check second array intmat

      if ( debug ) write(irefwr,*) 'before sepbuildchintm'
      call  sepbuildchintm
      if ( debug ) write(irefwr,*) 'after sepbuildchintm'

!     --- Change matrix structure for moving fictitious elements

      if ( indpry>0 ) call sepbuildfict ( iincommat, lniincommat )
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,*) 'after sepbuildfict'

      addvec = .false.  ! no right hand side
      if ( imas>0 .or. iinbld(2)/=4 ) then

!     --- store parameters from iinbld into sepmodulebuild

         call sepfillmdbuild

      end if  ! ( imas>0 .or. iinbld(2)/=4 )

!     --- initialize some parameters (used by several subroutines)

      call initst

      if ( imas>0 .or. iinbld(2)/=4 ) then

!     --- At least a mass matrix, stiffness matrix or right-hand side
!         must be built
!         Set positions 1 to 5 of arrays IUSOL, IRHSD, MATR and MASSMT
!         and compute length of arrays

         call to0005

!        --- Allocate some arrays for subroutine sepbuild
!            Initialize some parameters
!            These parameters and arrays are all used by element subroutines

         call sepinitbuild
         if ( ierror/=0 ) go to 1000

!        --- Activate and create arrays for module sepmodulebuild

         call sepactbuild

!        --- Fill some parameters in sepmodulematstr

         call sepfillmatstr2
         ndimlc = ndim

!        --- Fill pointers in sepmodulebuild for subroutine build
!            Create relation between arrays and buffer

         call sepfillbuild

      end if  ! ( imas>0 .or. iinbld(2)/=4 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'addmat, clrmat, addvec, clrvec, compl', &
                          addmat, clrmat, addvec, clrvec, compl
         write(irefwr,*) 'End sepbuildstart'

      end if  ! ( debug )
      call erclos ( 'sepbuildstart' )

      end subroutine sepbuildstart
