      subroutine sepcheckaccurp ( A, x, b, res, iter, ifiter, &
                                  rnorm, repeat1, rferr, &
                                  imin, ireturn, nmax, rmin, priter, &
                                  residu, xmin, work, vecxp, workglob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-06-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check on accuracy reached in step iter-1 of bi-cgstab
!
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      use sepmoduleread
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(inout) :: A
      integer, intent(in) :: iter, nmax, priter(*)
      integer, intent(out) :: ireturn
      integer, intent(inout) :: ifiter, imin
      double precision, intent(inout) :: x(A%neq)
      double precision, intent(in) :: b(A%neq), residu(*), xmin(A%neq)
      double precision, intent(inout), optional :: work(A%neq)
      double precision, intent(inout), optional :: workglob(nglob)
      double precision, intent(in), optional ::  vecxp(A%neq)
      double precision, intent(inout) :: rnorm, rferr, rmin
      double precision, intent(out) :: res(A%neq)
      logical, intent(inout) :: repeat1

!     A             i/o   Matrix A
!     b              i    Right-hand-side vector
!     ifiter        i/o   Starting value of parameter iter
!     imin           i    Iteration with minimal norm
!     ireturn        o    Return code:
!                         0: continue process
!                         1: restart process
!                         2: end of process
!     iter           i    Iteration sequence number
!     nmax           i    maximum number of iterations that may be performed by
!                         the CG-process.
!     priter         i    Output array to print residuals
!     repeat1        i    Logical variable indicating if the accuracy check
!                         must be carried out twice (true) or once (false), &
!                         because the norm of the solution could not be computed
!     res            o    residual vector
!     residu         i    Computed residual
!     rferr          i    Maximal accuracy possible for this matrix, due to
!                         round-off
!     rmin           i    minimal norm of residual
!     rnorm          i    Norm of residual
!     vecxp          i    Work array to store the initial value of x
!     work                Work array
!     workglob            Extension of work to global vector
!     x              i    Solution vector
!     xmin           i    Solution vector with minimal residual
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: j, ldim

!     j              Counting variable
!     ldim           Parameter l in bicgstab(l)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     677   The maximal accuracy with respect to the condition of the
!           matrix is reached
!    1063   Bi-cgstab looks divergent
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcheckaccurp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcheckaccurp'
         write(irefwr,1) 'iter, ifiter', iter, ifiter
         write(irefwr,2) 'eps, rnorm, rferr', A%eps, rnorm, rferr
         write(irefwr,2) 'eps1, eps2', A%eps1, A%eps2
         write(irefwr,*) 'repeat1 ', repeat1
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ldim = 1
      if ( A%typesolv>10 ) ldim = A%ninner

      ireturn = 0
140   if ( iter>ifiter .and. rnorm<=A%eps .or. rnorm<=0d0 ) then

!     --- The accuracy of the process seems to be reached
!         Check if eps must be recomputed

         if ( repeat1 ) then

!        --- repeat1 is true, recompute eps

            repeat1 = .false.
            A%eps = A%eps1+norm(x, A)*A%eps2

!           --- Check error level.

            if ( A%eps<rferr ) then

!           --- eps is smaller than maximal accuracy possible
!               It is useless to continue the process after rnorm
!               is less than rferr

               A%eps = rferr

               if ( A%iprint>=0 ) then
                  call erreal ( rferr, 1 )
                  call errwar ( 677, 0, 1, 0 )
               end if

            end if

!           --- return to the check if rnorm is less than

            A%eps = rferr

            goto 140

         end if

!        --- Due to rounding errors it is possible that the updated
!            residual differs from the true residual, for this
!            reason the norm of the true residual is computed, &
!            if it does not satisfy the termination criterion the
!            method is restarted.
!            Set residual r = b

         if ( A%typesolv>10 ) then

!        --- icgs=17, in case of postconditionering it is necessary
!                     to back substitute x
!                     x is stored in work

            work = x/A

            work = work+vecxp
            res = residual ( A, work, b )

         else

!        --- ldim = 1, standard

            res = residual ( A, x, b )

         end if  ! ( inpsol(14)>1 )

!        --- Check if the norm of the true residual satisfies the
!            termination criterion

         rnorm = norm(res, A)

         if ( rnorm>A%eps .and. iter<=nmax ) then

!        --- rnorm>eps and iter<nmax

            ifiter = iter
            rmin   = rnorm
            imin   = iter

!           --- the method is restarted since the norm of
!               the true residual is not less than eps

            ireturn = 1
            go to 1000

         end if  ! ( rnorm>A%eps .and. iter<=nmax )

         if ( A%iprint==2 .and. iter>ldim*20 ) then

!        --- iprint = 2 Print final residuals

            do j = max(1,ldim*20+12-iter), 10
               write ( irefwr, 450 ) priter(j), residu(j)
450            format( i10, ss,1pe15.2e2 )
            end do

         end if

         if ( A%iprint>=1 ) then

!        --- iprint>=1  Print information

            write ( irefwr,150 ) iter-1, rnorm
150         format(' The number of executed iterations in the', &
                   ' Bi-CGSTAB process is', i5/ &
                   ' The 2-norm of the residual is:', ss,1pe12.2e2 )

         end if

         A%numiter = iter-1
         A%itererr = 0

!        --- this is a regular termination

         ireturn = 2
         if ( A%typesolv>10 ) x = work   ! ldim>1, copy work in x
         go to 1000

      end if

!     --- Accuracy not yet reached

      if ( rnorm>A%eps/epsmac .and. ifiter<iter ) then

!     --- rnorm  overflow, Process restartes or terminates

         call errint ( iter, 1 )

         if ( A%aterror==1 .and. imin>1 ) then

            call errwar ( 1063, 1, 0, 0 )
            x = xmin

            if ( imin>ifiter ) then

               ifiter = iter

!              --- overflow occurs, however the norm is reduced since
!                  the last restart so it is useful to restart the
!                  method

               ireturn = 1
               go to 1000

            else

               A%numiter = iter-1
               A%itererr = 0

            end if

         else

            call errsub ( 1063, 1, 0, 0 )

         end if

         if ( A%iprint>=1 ) then

!        --- iprint>=1  Print information

            write ( irefwr,150 ) iter-1, rmin
            write ( irefwr,160 ) imin-1
160         format (' The minimal residual is obtained at', &
                    ' iteration', i5 )

         end if

!        --- overflow occurs and the norm is not reduced since
!            the last restart so it is not useful to restart the
!            method, or it is not requested to restart

         ireturn = 2
         go to 1000

      end if

1000  call erclos ( 'sepcheckaccurp' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ireturn', ireturn
         write(irefwr,2) 'eps, rnorm', A%eps, rnorm
         write(irefwr,*) 'End sepcheckaccurp'

      end if  ! ( debug )

      end subroutine sepcheckaccurp
