      subroutine sepcprojUT ( A, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-12-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute y = U^T x as part of the projection algorithm
!     U is the matrix of projection vectors
!     Complex version of sepprojUT
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleproj
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(in) :: A
      double complex, intent(in) :: x(A%neq)
      double complex, intent(out) :: y(A%nproj)

!     A             i/o   Matrix A
!     x              i    Vector to be multiplied
!     y              o    result of multiplication
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double complex :: h, usend(A%nproj)
      integer :: i, j, idegfd, iblock, numneigh, icontext, nsend, j1, j2

!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     iblock         Block sequence number
!     icontext       Tag for sending and receiving, is used as check
!     idegfd         Degree of freedom sequence number
!     j              Counting variable
!     j1             Lower bound of loop
!     j2             Upper bound of loop
!     nsend          Help variable for receiving
!     numneigh       number of neighbors of a point
!     usend          Help array to be used to send reals to other processor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIRECEIVEMPI  Receive information from other node (mpi)
!     INISENDMPI     Send information to other node (mpi)
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcprojUT' )
      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcprojUT'
         call prinin ( A%projrows, nproj+1, 'A%projrows' )
         call prinin ( A%projcols, A%projrows(nproj+1), 'A%projcols' )
         call prinrl ( A%projU, A%neq+A%projrows(nproj+1), 'A%projU' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( parallel ) then

!     --- Parallel case
!         y = U^T x

         y(1) = dot_product ( A%projU(1:A%neq), x )

!        --- send all contributions to the first node

         icontext = 11

         if ( inodenr==1 ) then

!        --- Receive next contributions from the other nodes and sum up

            do j = 2, A%nproj
               nsend = A%nproj*2
               call inireceivempi ( usend, nsend, SPDOUBLE, iblock, &
                                    icontext )
               y(iblock) = usend(1)
            end do

         else

!        --- Send contributions to the first node

            call inisendmpi ( y, 1, SPDOUBLE, 1, icontext )

         end if

         if ( debug ) call prinrl ( y, A%nproj*2, 'y/1' )

      else

!     --- Serial case
!         y = U^T x


         do i = 1, A%nproj

            h = 0d0
            j1 = A%projrows(i)+1
            j2 = A%projrows(i+1)
            h = h + dot_product(A%projU(j1:j2), x(A%projcols(j1:j2)) )
            y(i) = h

         end do

      end if  ! ( parallel )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcprojUT'

      end if  ! ( debug )
      call erclos ( 'sepcprojUT' )

      end subroutine sepcprojUT
