      subroutine sepcreaprec ( A )
! ======================================================================
!
!        version  1.0  date 05-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     The routine sepcreaprec computes the preconditioning matrix for the CG-
!     CGS- and Bi-CGSTAB process, for real case
!     Old subroutine precmk
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulemat
      use sepmodulematr
      use sepmodulesolve
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: n
      double precision, allocatable :: work(:)
      type (intmatrix), pointer :: IM
      double precision :: timebefore(2)

!     IM             Refers to intmt(intmat)
!     n              number of rows in matrix
!     work           Work array of length 2n to store intermediate vectors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     ERSETTIME      Get the CPU time at start of subroutine
!     PRINRL         Print 1d real vector
!     PRINTTIME      Print the CPU time at end of subroutine
!     SEPCREAPRECSYM computes the preconditioning matrix for the CG-process
!                    complex matrix
!     SEPCREAPRECUSY computes the preconditioning matrix for the CGS-process
!                    complex matrix
!     SEPPRECPRIN1   Print minimum and maximum element of diagonal of matrix
!     SEPPRECPRIN2   Print minimum and maximum element of diagonal
!                    of preconditioning matrix
!     SEPPRECPRIN3   Print diagonal of precondtioning matrix
!     SEPPRINMAT6    Print matrix that is stored as row-compact real matrix
!     SEPPRINMATDC6  Print preconditioning matrix that is stored as
!                    row-compact real matrix
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2234   For this preconditioner (A%type_prec ), lumping of the positive
!           off-diagonal elements has not been implemented.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ('sepcreaprec')

      if ( itime>0 ) call ersettime ( timebefore )

      debug = .false.

      IM => intmt(A%iseqim)
      n = A%neq

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcreaprec'
         write(irefwr,1) 'n',  n
         call sepprinmat6 ( A )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( A%type_prec==0 ) go to 1000  ! jump in case of no preconditioning
      if ( debug ) write(irefwr,1) 'A%type_prec', A%type_prec

      length = n

      allocate(work(length), stat = error)
      if ( error/=0 ) call eralloc ( error, length, 'work' )
      if ( ierror/=0 ) go to 1000

      if ( A%prlump==1 .and. &
         ( A%type_prec/=2 .and. A%type_prec/=3 .and. A%type_prec/=7 ) ) then
         call errint ( A%type_prec, 1 )
         call errwar ( 2234, 1, 0, 0 )
         A%prlump = 0
      end if

      if ( A%iprint>=2 ) then

!     --- print the maximal and minimal diagonal element of the matrix

         call sepprecprin1 ( A )

      end if

      if ( A%symmetric ) then

!     --- Real symmetric matrix

         call sepcreaprecsym ( A, work )

      else

!        --- Real non-symmetric matrix

         call sepcreaprecusy ( A, work )

      end if  ! ( A%symmetric )

      if ( A%iprint>=2 ) then

!     --- A%iprint>=2, extra output

         call sepprecprin2 ( A )

      end if

      if ( A%iprint>=4 ) then

!     --- A%iprint>=4, print diagonal of preco matrix

         call sepprecprin3 ( A )

      end if

      deallocate ( work, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'work' )
      if ( ierror/=0 ) go to 1000

1000  if ( itime==1 ) call printtime ( 'sepcreaprec')

      if ( debug ) then

!     --- Debug information

         if ( A%type_prec==1 .or. A%type_prec==2 .or. A%type_prec==7 ) then
            call prinrl ( A%MD, n, 'preconditioning matrix')
         else
            call sepprinmat6p ( A )
         end if  ! ( A%type_prec==1 .or. A%type_prec==2 .or. A%type_prec==7 )

         write(irefwr,*) 'End sepcreaprec'

      end if  ! ( debug )
      call erclos ('sepcreaprec')

      end
