      subroutine sepcreaprecsym ( A, work )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     The routine sepcreaprecsym computes the preconditioning matrix for
!     the CG-process
!     Old subroutine prcg01
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulematstr
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision, intent(inout) :: work(A%neq)

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     work                Work array that is used to store the row sum of the
!                         matrix U in the eisentat MILU preconditioning
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i
      double precision :: amax, amin

!     amax           maximal element of a matrix
!     amin           Minimum element of a matrix
!                    they are not
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     SEPPRECEISSYM  compute the preconditioning matrix for the CG-process
!                    iprec = 2,  Eisenstat preconditioning
!     SEPPRECIGGT    compute the preconditioning matrix for the CG-process
!                    iprec = 3,  Incomplete LU-decomposition
!     SEPPRECILDLT   compute the preconditioning matrix for the SYMMLQ-process
!                    iprec = 3,  Incomplete LDL^T-decomposition
!     SEPPRECMEISSYM compute the preconditioning matrix for the CG-process
!                    iprec = 7,  Modified Eisenstat preconditioning
!     SEPPRECSYMGS   compute the preconditioning matrix for the CG-process
!                    iprec = 4,  Symmetric Gauss-Seidel
!     SEPPRINMAT6    print compact matrix
!     SEPPRINMAT6P   print compact preconditioning matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     674   The desired preconditioning matrix does not exist
!    1405   This combination of the storage scheme and the preconditioner
!           is not implemented.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcreaprecsym' )
      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcreaprecsym'
         write(irefwr,1) 'A%neq, typesolv', A%neq, typesolv
         write(irefwr,1) 'ipreco', A%type_prec
         write(irefwr,2) 'adapt', A%adapt
         call sepprinmat6 ( A )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Determine the maximal diagonal element of the matrix and
!         multiply it by 10^10. This value is used to predict
!         overflow.

      amax = 10d10*maxval(A%S)

      select case ( A%type_prec )

         case(1,6)

!        --- ipreco = 1, 6   Diagonal scaling

            if ( A%adapt>0d0 ) then

!           --- adapt diagonal

               A%MD = max(A%adapt,A%S(A%diag))
               A%MD = sqrt(A%MD)

            else

!           --- Check if all values are positive

               amin = minval(A%S(A%diag))
               if ( amin<=0d0 ) then

!              --- Negative element found: error

                  call errint ( minloc(A%S), 1 )
                  call erreal ( amin, 1 )
                  call errsub ( 674, 1, 1, 0)
                  go to 1000

               end if  ! ( A%adapt<0d0 .and. amin<0d0 )

               A%MD = sqrt(A%S(A%diag))

            end if  ! ( A%adapt>0d0 )

         case(2)

!        --- ipreco = 2,  Eisenstat preconditioning

            call seppreceissym ( amax, A )

         case(3)

!        --- ipreco = 3,  Incomplete LU-decomposition

            if ( typesolv==7 ) then

!           --- typesolv = 7, i.e. symmlq, use ldl^t

               call sepprecildlt ( amax, A )

            else

!           --- typesolv # 7, i.e. cg, use gg^t

               call seppreciggt ( amax, A )

            end if  ! ( typesolv==7 )

         case(4)

!        --- ipreco = 4,  Symmetric Gauss-Seidel

            call sepprecsymgs ( A )

         case(7)

!        --- ipreco = 7,  Modified Eisenstat preconditioning

            call sepprecmeissym ( amax, A, work )

         case default

!        --- preconditioning not available

            call errint ( A%type_storage, 1 )
            call errint ( A%type_prec, 2 )
            call errsub ( 1405, 2, 0, 0 )

      end select  ! case ( A%type_prec )

1000  if ( debug ) then

!     --- Debug information

         if ( A%type_prec==1 .or. A%type_prec==2 .or. A%type_prec==7 ) then
            call prinrl ( A%MD, A%neq, 'preconditioning matrix')
         else
            call sepprinmat6p ( A )
         end if  ! ( A%type_prec==1 .or. A%type_prec==2 .or. A%type_prec==7 )
         write(irefwr,*) 'End sepcreaprecsym'

      end if  ! ( debug )
      call erclos ( 'sepcreaprecsym' )

      end subroutine sepcreaprecsym
