      subroutine sepcreaprecusy ( A, work )
! ======================================================================
!
!        version  1.0  date 05-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     The routine sepcreaprecusy computes the preconditioning matrix
!     for the CGS-process
!     Old subroutine prcg02
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulematstr
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision, intent(inout) :: work(A%neq)

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     work                Work array that is used to store the row sum of the
!                         matrix U in the eisentat MILU preconditioning
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, n, nolump
      double precision :: amax

!     amax           maximal element of a matrix
!                    they are not
!     i              Counting variable
!     n              number of rows in matrix
!     nolump         No lumping should be used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     SEPPRECD9      computes the preconditioning matrix for the CGS-process
!                    ipreco = 1, diagonal scaling, method = 9
!     SEPPRECEIS9    computes the preconditioning matrix for the CGS-process
!                    ipreco = 2, Eisenstat preconditioning
!     SEPPRECILU9    computes the preconditioning matrix for the CGS-process
!                    ipreco = 3, Incomplete LU-decomposition
!                    row-compact storage
!     SEPPRECMODEIS9 computes the preconditioning matrix for the CGS-process
!                    ipreco = 7,  Eisenstat preconditioning (MILU)
!     SEPPRECUSYMGS9 computes the preconditioning matrix for the CGS-process
!                    ipreco = 4,  Symmetric Gauss-Seidel
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2243   The elements of the main diagonal have different signs
!    2764   Integer parameter has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
!       Formulas for ILU decomposition:
!       A%type_prec = 3,  Incomplete LU-decomposition
!          first step ( explained on complete decomposition ):
!            goal: A = (L+I)*(D+U), diag(L)=diag(U)=0
!            do i=1,n
!              do j=1,i-1
!                 L(i,j) = [ A(i,j) - sum L(i,k)*U(k,j) ] / D(j)
!                                     k<j
!                 U(j,i) = [ A(j,i) - sum U(k,i)*L(j,k) ]
!                                     k<j
!              end do
!              D(i) = [ A(i,i) - sum L(i,k)*U(k,i) ]
!                                k<i
!            end do
!          now ( explained on complete decomposition):
!             A  =  ( L + I ) * ( D + U ), diag(L)=diag(U)=0
!          let  L = low,  U = upp * D, then
!             A  =  ( low + I ) * ( I + upp ) * D ,
!          which is suitable for later calculations
!          D should be stored as it is,  strictly triangular
!          matrices  low = L,  upp = U * inv(D),
!          will be passed to the calling process:
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcreaprecusy' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcreaprecusy'
         write(irefwr,1) 'type_prec', A%type_prec
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      n = A%neq

!     --- If nolump is 1 no lumping is tried when a zero element of
!         A%MD occurs. This is only used for A%type_prec = 3

      nolump = 0

!     --- Determine the maximal diagonal element of the matrix and
!         multiply it by 10^10. This value is used to predict
!         overflow.
!         Determine if the elements of the main diagonal have the same
!         signs.

      amax = maxval(A%S(A%diag))

      i = 2
      do while ( A%S(A%diag(i-1))*A%S(A%diag(i))>=0 .and. i<n )
         i = i+1
      end do

      if ( i<n .or. A%S(A%diag(n-1))*A%S(A%diag(n))<0 ) then
         nolump = 1
         if ( A%prlump==1 ) then
            call errwar ( 2243, 0, 0, 0 )
         end if

      end if
      amax = 10d10*amax

      if ( debug ) then
         write(irefwr,1) 'nolump, prlump', nolump, A%prlump
         write(irefwr,2) 'amax', amax
      end if  ! ( debug )

      select case ( A%type_prec )

         case(1, 6)

!        --- A%type_prec = 1   Diagonal scaling

            call sepprecd9 ( A )

         case(2)

!        --- A%type_prec = 2,  Eisenstat preconditioning

            call seppreceis9 ( A, amax )

         case(3)

!        --- A%type_prec = 3,  Incomplete LU-decomposition

            call sepprecilu9 ( A, amax, nolump )

         case(4)

!        --- A%type_prec = 4,  Symmetric Gauss-Seidel

            call sepprecusymgs9 ( A )

         case(7)

!        --- A%type_prec = 7,  Eisenstat preconditioning (MILU)

            call sepprecmodeis9 ( A, amax, work )

         case default

            call errchr ( 'A%type_prec', 1 )
            call errint ( A%type_prec, 1 )
            call errsub ( 2764, 1, 0, 1 )

      end select  ! case ( A%type_prec )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcreaprecusy'

      end if  ! ( debug )
      call erclos ( 'sepcreaprecusy' )

      end subroutine sepcreaprecusy
