      subroutine sepcreapreczus ( A )
! ======================================================================
!
!        programmer    Zdenek Vlasek
!        version  2.1  date 06-02-2013 New definition inpsol(4)
!        version  2.0  date 10-11-2010 Complete new version
!        version  1.3  date 08-01-1999 Adaptation of error message
!
!   copyright (c) 1990-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     The routine sepcreapreczus computes the complex preconditioning matrix
!     for the CGS- or CGSTAB-process
! **********************************************************************
!
!                       KEYWORDS
!
!     complex
!     conjugate_gradients
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulecmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
!
      type (cmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPPRECD10     computes the preconditioning matrix for the CGS-process
!                    ipreco = 1, diagonal scaling, jmethod = 10
!     SEPPRECILU10   computes the preconditioning matrix for the CGS-process
!                    ipreco = 3, Incomplete LU-decomposition
!                    row-compact storage (complex)
!     SEPPRECUSYMGS1 computes the preconditioning matrix for the CGS-process
!                    ipreco = 4,  Symmetric Gauss-Seidel, complex
!     SEPPRECZEIS10  computes the preconditioning matrix for the CGS-process
!                    ipreco = 3, Incomplete LU-decomposition, complex
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2764   Integer parameter has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
!    In case of ipreco = 5 we have:
!    The preconditioning matrix must be transformed from the form
!       -1
!    L D  U to L U, where L and U have the square root of the
!    diagonal on their main diagonal
!    So from:
!                                          -1
!                     | d    0    0 |   | d     0     0 |  | d    u    u   |
!                     |  11         |   |  11           |  |  11   12   13 |
!                     |             |   |       -1      |  |               |
!          A  =       | l    d    0 |   | 0    d      0 |  | 0    d    u   |
!                     |  21   22    |   |       22      |  |       22   23 |
!                     |             |   |             -1|  |               |
!                     | l    l   d  |   | 0     0    d  |  | 0    0    d   |
!                        31   32  33                  33                33
!            to
!                                                               _    _
!                     | sqrt(d  )  0       0    |   | sqrt(d )  u    u      |
!                     |       11                |   |       11   12   13    |
!                     |   _                     |   |                 _     |
!          A  =       |   l    sqrt(d  )   0    |   | 0    sqrt(d )   u     |
!                     |    21        22         |   |            22   23    |
!                     |   _      _              |   |                       |
!                     |   l      l    sqrt(d  ) |   | 0       0    sqrt(d ) |
!                          31     32        33                           33
!                                  _
!       It is easily verified that l   = l  / sqrt(d  )
!                                   ik    ik        kk
!                                  _
!                                  u   = u  / sqrt(d  )
!                                   kj    kj        kk
!       Afterwards the matrix is scaled by pre- and post multiplying by
!             -1
!       sqrt(D  ) as for the other preconditioning matrices
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcreapreczus' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcreapreczus'
         write(irefwr,1) 'ipreco', A%type_prec
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case ( A%type_prec )

         case(1)

!        --- ipreco = 1   Diagonal scaling

            call sepprecd10 ( A )

         case(2)

!        --- ipreco = 2,  Eisenstat preconditioning

            call seppreczeis10 ( A )

         case(3)

!        --- ipreco = 3,  Incomplete LU-decomposition

            call sepprecilu10 ( A )

         case(4)

!        --- ipreco = 4,  Symmetric Gauss-Seidel

            call sepprecusymgs1 ( A )

         case default

            call errchr ( 'ipreco', 1 )
            call errint ( A%type_prec, 1 )
            call errsub ( 2764, 1, 0, 1 )

      end select  ! case ( A%type_prec )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcreapreczus'

      end if  ! ( debug )
      call erclos ( 'sepcreapreczus' )

      end subroutine sepcreapreczus
