      subroutine sepcscalmat ( A, diag, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Scale the original large sparse matrix,
!     the right-hand side and if necessary the solution vector
!     with the inverse of the diagonal of the preconditioning matrix.
!     complex case
!     Old routine: pzcg04
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(inout) :: A
      double complex, intent(inout) :: y(A%neq), x(A%neq)
      double complex, intent(in) :: diag(A%neq)

!     A             i/o   Matrix A
!     diag           i    Diagonal of the scaling matrix
!     x             i/o   Solution vector
!     y             i/o   Right-hand side
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iprec, n, i, j, idegfd
      double complex :: h

!     h              Help variable to store a constant temporarily
!     i              General loop variable
!     idegfd         Position in vector
!     iprec          Type of preconditioning used. Possible values:
!                    0    No preconditioning
!                    1    Diagonal scaling
!                    2    Eisenstat preconditioning
!                    3    Incomplete LU decomposition
!                    4    Symmetric Gauss-Seidel preconditioning
!                    5    Pre-assembled preconditioning according to
!                         Kaasschieter
!                    7    Eisenstat preconditioning (MILU variant)
!     j              General loop variable
!     n              number of equations
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINCOMPL      Print 1d complex vector
!     PRINRL         Print 1d real vector
!     SEPPRINMAT6    Print matrix that is stored as row-compact real matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2917   Option not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcscalmat' )
      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcscalmat'
         write(irefwr,1) 'scaling', A%scaling
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iprec = A%type_prec
      n = A%neq  ! number of rows

      if ( debug ) then
         write(irefwr,1) 'iprec, n', iprec, n
         write(irefwr,*) 'amat'
         call prinrl ( diag, n, 'diag_pre' )
      end if  ! ( debug )

!     --- Scale diagonal of A, y and if necessary x

      if ( A%scaling==2 ) then

!     --- iscaling = 2, symmetric scaling

         do i = 1, A%neq
            j = A%diag(i)
            A%S(j) = A%S(j)/diag(i)**2
         end do  ! i = 1, A%neq

      else

!     --- iscaling = 1, 3, row or column scaling

         do i = 1, A%neq
            j = A%diag(i)
            A%S(j) = A%S(j)/diag(i)
         end do  ! i = 1, A%neq

      end if

      if ( A%scaling<=2 ) y = y / diag
      if ( A%startvec==1 .and. A%scaling>=2 ) x = x * diag

!     --- Scale rest of matrix

         select case ( A%scaling)

         case(1)

!         --- iscaling = 1, row scaling

            do i = 1, n

               h = diag(i)
               do j = A%rows(i)+1,A%diag(i)-1
                  A%S(j) = A%S(j) / h
               end do  ! j = A%rows(i)+1,A%diag(i)-1
               do j = A%diag(i)+1,A%rows(i+1)
                  A%S(j) = A%S(j) / h !diag(A%cols(j))
               end do  ! A%diag(i)+1,A%rows(i+1)

            end do  ! i = 1, n

         case(2)

!         --- iscaling = 2, symmetric scaling

            do i = 1, n

               h = diag(i)
               do j = A%rows(i)+1,A%diag(i)-1
                  A%S(j) = A%S(j) / (h*diag(A%cols(j)))
               end do  ! j = A%rows(i)+1,A%diag(i)-1
               do j = A%diag(i)+1,A%rows(i+1)
                  A%S(j) = A%S(j) / (h*diag(A%cols(j)))
               end do  ! A%diag(i)+1,A%rows(i+1)

            end do  ! i = 1, n

         case(3)

!         --- iscaling = 3, column scaling

            do i = 1, n

               do j = A%rows(i)+1,A%diag(i)-1
                  A%S(j) = A%S(j) / diag(A%cols(j))
               end do  ! j = A%rows(i)+1,A%diag(i)-1
               do j = A%diag(i)+1,A%rows(i+1)
                  A%S(j) = A%S(j) / diag(A%cols(j))
               end do  ! A%diag(i)+1,A%rows(i+1)

            end do  ! i = 1, n

         case default

            call errint ( A%scaling, 1 )
            call errsub ( 2917, 1, 0, 0 )

         end select  ! case ( A%scaling)

      if ( iprec==2 .or. iprec==4 .or. iprec==7 ) then

!     --- IPREC =2,  Eisenstat   Change diagonal of the large matrix

         do i = 1, A%neq
            j = A%diag(i)
            A%S(j) = A%S(j)-2d0
         end do  ! i = 1, A%neq

      end if

      if ( A%nproj>0 ) then

!     --- There are projection vectors, scale both the vectors z and Az

         if ( A%scaling>=2 ) then

!        --- column or symmetric scaling, pre-multiply vector z

            do i = 1, A%nproj
               do j = A%projrows(i)+1, A%projrows(i+1)
                  idegfd = A%projcols(j)
                  A%projU(j) = A%projU(j) * diag(idegfd)
               end do  ! j = A%projrows(i)+1, A%projrows(i+1)
            end do  ! i = 1, A%nproj

         end if  ! ( A%scaling>=2 )

         if ( A%scaling<=2 ) then

!        --- row or symmetric scaling, pre-multiply vector Az

            do i = 1, A%nproj
               do j = A%projrows(i)+1, A%projrows(i+1)
                  idegfd = A%projcols(j)
                  A%projAu(j) = A%projAu(j)/diag(idegfd)
               end do  ! j = A%projrows(i)+1, A%projrows(i+1)
            end do  ! i = 1, A%nproj

         end if  ! ( iscaling<=2 )

      end if  ! ( amat%nproj>0 )

1000  if ( debug ) then

!     --- Debug information

         call princompl ( y, n, 'y' )
         write(irefwr,*) 'amat after scaling'
         call sepprinmat6 ( A )
         write(irefwr,*) 'End sepcscalmat'

      end if  ! ( debug )
      call erclos ( 'sepcscalmat' )

      end subroutine sepcscalmat
