      subroutine sepdecllt ( amat, n )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-02-2021
!
!   copyright (c) 2021-2021  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute LL t decomposition for a full symmetric positive
!     definite real matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     positive_definite
!     real
!     symmetric
!     full_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: n
      double precision, intent(inout) :: amat((n*n+n)/2)

!     amat          i/o   Matrix amat. Only the lower part is stored
!                         At input: original matrix
!                         At output: decomposition matrix L
!     n              i    number of unknowns
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ii, i, j, jj, ij, ih1, ih

!     i              General loop variable
!     ih             Help variable to store a constant temporarily
!     ih1            Help variable to store a constant temporarily
!     ii             Position of diagonal element i
!     ij             Position of matrix element ij
!     j              General loop variable
!     jj             Position of diagonal element j
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision :: ddot

!     DDOT           (CDOT)     Inner product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!      The L L'   decomposition of the matrix A is based on the following
!      formulas:
!              n          T
!      a   =  sum   l    l
!       ji    k=1    ik   kj
!      this implies:
!
!      l_ij   = ( a_ij - sum_k=1^j-1  l_ik l_jk  )/l_jj   ( j=1 (1) i-1 )
!
!      l_ii   = sqrt(a_ii - sum_k=1^i-1  l_ik l_ik)

! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!debug      call eropen ( 'sepdecllt' )
!debug      debug = .false. .and. ioutp>=0
!debug      if ( debug ) then
!debug
!debug!     --- Debug information
!debug
!debug         write(irefwr,*) 'Debug information from sepdecllt'
!debug         call prinrl ( amat, (n+1)*n/2, 'amat' )
!debug
!debug      end if  ! ( debug )
!debug      if ( ierror/=0 ) go to 1000

      ii = 1

      do i = 1, n
         jj = 1
         ih1 = ii-i
         ih = ih1+1
         do j = 1, i-1
            ij = ih1+j
            amat(ij) = (amat(ij)-ddot(j-1,amat(ih),1,amat(jj-j+1),1))/amat(jj)
            jj = jj+j+1
         end do  ! j = 1, i-1
         amat(ii) = sqrt(amat(ii)-ddot(i-1, amat(ih), 1, amat(ih), 1) )
         ii = ii+i+1
      end do  ! i = n

!debug1000  if ( debug ) then
!debug
!debug!     --- Debug information
!debug
!debug         write(irefwr,*) 'End sepdecllt'
!debug
!debug      end if  ! ( debug )
!debug      call erclos ( 'sepdecllt' )

      end subroutine sepdecllt
