      subroutine sepeigval ( eval, nr, ivec, istiff, imas, icheig, eveps )
! ======================================================================
!
!        programmer    Guus Segal, Zdenek Vlasek
!        version  1.0  date 29-03-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find eigenvalues and if necessary eigenvectors of the generalised
!     eigenvalue problem
!             stiff * x = lambda amas * x
!     The matrix stiff must be symmetric and
!     amas must be a mass matrix, symmetric positive definite.
!     At this moment only diagonal mass matrices are permitted.
!     Old subroutine ereig1
! **********************************************************************
!
!                       KEYWORDS
!
!     eigenvalue
!     eigenvector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemat
      use sepmodulematr
      use sepmodulekprob
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision eval(*), eveps
      integer nr, ivec(*), istiff, imas, icheig(*)

!     eval       o    array of length nr, containing the nr lowest
!                     eigenvalues of the problem
!                             stiff * x = lambda amass * x
!                     in the positions 1 to nr in increasing order at output
!     eveps      i    Accracy to compute the eigenvectors
!     icheig     i    input array of length 8 containing some
!                     information for the subroutine,
!                     all positions that are not used must be filled with zeros
!                     icheig(1): indication whether the eigenvectors
!                                must be computed (1) or not (0)
!                     icheig(2): indication whether an information about the
!                                iteration process must be printed (1) or not
!                                (0) or debug prints (2)
!                     icheig(3): indication if the old method must be used (1)
!                                or the new method of Parlett and Reid (0)
!                     icheig(4): indication how to fill the initial vector
!                                           in the Lanczos process,
!                                 =0 :       vj(i) = i
!                                 otherwise: random vj(*),
!                                            abs(icheig(4)) becomes a seed
!                     icheig(5): order of the relative precision ( with respect
!                                of 2-norm of the matrix A ) of the eigenvalues,
!                                rel.precision is then   eveps = 10**icheig(5),
!                                ( when 0 a default value is used ), see eveps
!                     icheig(6): upper bound for number of Lanczos iterations
!                                ( when 0 a default value is used ), see mmax
!                     icheig(7): upper bound for number of e.v. intervals
!                                temporary stored in an irregular order, they
!                                cover both ends of the spectrum
!                                ( when 0 a default value is used ), see maxnev
!                     icheig(8): upper bound for number of points x(*) used
!                                in P+R process to catch the eigenvalues
!                                ( when 0 a default value is used ), see maxnx
!     imas       i    standard sepran array containing information
!                     concerning the mass matrix,
!                     at this moment the mass matrix must be a diagonal matrix
!                     stored as right-hand-side vector and imas is not used
!     istiff     i    standard sepran array containing information concerning
!                     the solution stiffness matrix stiff,
!                     at this moment only jmetod=5 is permitted for the matrix
!     ivec       o    contains information of the eigenvectors at output,
!                     length: 5*nr
!                     pos (1-5,i) corresponds to the i-th eigenvalue
!                     ivec is only filled when the eigenvectors must be computed
!     nr         i    number of lowest eigenvalues to be computed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer n, jevec, jprint, &
              lentmp, maxlen, multnr, m, mmax, maxnev, maxnx, ieraut, &
              nev, nx, niter, nlow, j
      integer, allocatable :: mini(:)
      double precision, pointer :: alpha(:), beta(:), ev(:,:)
      double precision, allocatable :: msin(:), w(:), z(:), y(:)
      double precision eps
      logical restrt, low
      type (globmat), pointer :: S
      type (rmatrix), pointer :: Amat

!     Amat           pointer to matrix
!     S              pointer to realmt(stiff)
!     alpha          array for the lanczos process
!     beta           array for the lanczos process
!     eps            Local accuracy
!     ev             array for the lanczos process
!     ieraut         Error indication from ervl20
!                    ieraut = 0  Problem was solved:  at least
!                                nr lowest e.v. intervals found.
!                                The restart is possible.
!                    ieraut = 1  Only  nlow  of  nr  lowest e.v. was found.
!                                It is possible to restart this subroutine.
!                                Enlarge    icheig(6)=mmax
!                                or enlarge icheig(5) ( ==> greater eveps )
!                                or reduce  nr.
!                    ieraut = 2  Overflow of  maxnev.  List of the
!                                eigenvalue intervals in not complete.
!                                Restart is possible. Enlarge icheig(7)=maxnev.
!                    ieraut = 3  Too small  eveps  with respect to
!                                round-off error estimation.
!                                Restart is possible. Enlarge icheig(5).
!                    ieraut = 4  Overflow of  maxnx.
!                                Restart is possible. Enlarge icheig(8).
!                    ieraut = 5  Attempt to restart, but the restart
!                                is not possible.
!                                Probably the first call of ervl20.
!                                No results.
!                    ieraut = 6  the Lanczos process has returned
!                                beta(j) = 0.0, it cannot proceed.
!                                If the matrix is not singular, change
!                                the initial vector, by icheig(4).
!                                No results.
!                    ieraut = 7  nr>n or nr<1 . No results.
!     j              Counting variable
!     jevec          ?
!     jprint         ?
!     lentmp         ?
!     low            ?
!     m              number of integration points
!     maxlen         Maximum amount of contiguous space in IBUFFR after a
!                    starting address
!     maxnev         ?
!     maxnx          ?
!     mini           for each eigenvalue: number of iterations to get it, &
!                    it must be mini(*)<=m
!     mmax           ?
!     msin           sqrt of the mass matrix which is diagonal
!     multnr         ?
!     n
!     nev            ?
!     niter          Iteration sequence number
!     nlow           ?
!     nx             number of points in x direction
!     restrt         ?
!     w               Array containing the computed eigenvalues of the
!                     tridiagonal matrix
!     y               ?
!     z               Array of length m x maxlen containing the maximal maxlen
!                     eigenvectors of the tridiagonal matrix of length m each
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERVL01         sqrt of amas
!     ERVL12         reorder and normalize eigenvectors of the original matrix
!     ERVL15         eigenvectors of the tridiagonal matrix
!     ERVL16         back transformation of e.vectors of the tridiagonal matrix,
!                    second pass of the Lanczos method
!     ERVL20         Parlett+Reid control of the Lanczos process,
!                    it controls the first pass of the Lanczos method
!     SEPCREASOL110  Activate or create arrays of type 110
!     TOREORDR       Map array of solution type from matrix sequence to vector
!                    sequence. Real version
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     1408    Too few eigenvalues found
!     1409    Overflow of  maxnev
!     1410    Overflow of  maxnx
!     1411    Attempt to restart, but the restart is not possible.
!     1412    Lanczos process has returned  beta(j) = 0.0, it cannot proceed.
!     1413    Wrong number of e.values
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The generalized e.v.problem is transformed to normal e.v.
!     problem, let
!                    smas = sqrt(amas)
!     then
!             inv(smas) * stiff * inv(smas) * y = lambda y, &
!     where
!             y = smas * x
!     The new (normal) eigenvalue problem is now
!             A * y = lambda y
!     with
!             A = inv(smas) * stiff * inv(smas)
!     First, the lowest eigenvalue intervals are determined by
!     the Parlett+Reid's method of the control of the Lanczos process, &
!     and a tridiagonal matrix is prepared, for which
!              A S = S T, where S is a regular matrix (full)
!     Then, if necessery:
!     The eigenvectors of the tridiagonal matrix are calculated.
!     These eigenvectors are transformed to eigenvectors y of A.
!     To avoid problems with storing the huge matrix S, the Lanczos process
!     is performed again  in a simple version ) and columns of S are
!     used as soon as they appear.
!     Then, the back transformation from y to x is performed.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeigval' )
      n = nusol-nbound-npboun
      jevec = icheig(1)
      jprint = icheig(2)

!     --- initializations;   computing starting positions

      S => realmt(istiff)
      S1_ii => realmt(imas)%S_ii
      Amat => S%S_i

      if ( jprint==1 ) write ( irefwr, '(/1x,a)' ) &
         'Lanczos method with Parlett+Reid eigenvalue localization'

!     --- initializations for Lanczos

      if ( icheig(6)>0 ) then
         mmax = icheig(6)
      else

!     --- default

         mmax = max(3*n,n+10)
      end if

!     --- ev(maxnev,2) keeps all e.v. intervals found, not only the smallest,
!         it can contain:  nr lowest e.v., &
!                        approx. nr greatest e.v.
!                        some neigbouring e.v.
!         arrays jit(*), xit(*) keep chronological track of e.v.intervals,
!         ( it can happen, e.g. for large eveps, that the interval is later
!           dropped, which can caused the overflow of maxnev )

      if ( icheig(7)>0 ) then
         maxnev = icheig(7)
      else

!     --- default

         maxnev = max(n/4,100)
      end if

!     --- array x(maxnx) separates and locates the e.v.

      if ( icheig(8)>0 ) then
         maxnx = icheig(8)
      else

!     --- default

         maxnx = 6 * min(n,maxnev)
      end if

      restrt = .false.

!     --- if any of the values n, mmax, maxnev, maxnx is changed later, &
!         the control must come here again !!!
!         ( otherwise all scratch arrays must be re-allocated and shifted )

      if ( jprint==1 ) &
         write ( irefwr, '( /1x, a, d12.3, 3(3x,a,i5) )' ) &
         'eveps=',eveps, 'mmax=',mmax, 'maxnev=',maxnev, 'maxnx=',maxnx

!     --- calculate the total length of temporary arrays for ervl20

      allocate ( alpha(mmax), beta(mmax), ev(maxnev,2), &
                 msin(n), mini(maxnev), stat = error )
      if ( error/=0 ) call eralloc ( error, 6*mmax+n, 'alpha' )
      if ( ierror/=0 ) go to 1000

!     --- store from the position ipmsin the sqrt of the mass matrix

      call ervl01 ( S1_ii, msin, n )

!     --- loop point when the precision gets changed
!         lowest eigenvalues, Parlett+Reid's control of the Lanczos process

 20   call ervl20 ( n, icheig, eval,  &
                    Amat, msin, mmax, restrt, &
                    alpha, beta, maxnev, nev, ev, maxnx, nx, nr, &
                    low, nlow, mini, eveps, niter, ieraut )

      if ( ieraut/=0 ) write(irefwr, '(/1x,a,i3)' ) &
         'warning: sepeigval   ervl20 has returned ieraut=', ieraut

      if ( ieraut==1 ) then

         call errint ( nlow, 1 )
         call errint ( nr, 2 )
         call errsub ( 1408, 2, 0, 0 )

      else if ( ieraut==2 ) then

         call errint ( maxnev, 1 )
         call errsub ( 1409, 1, 0, 0 )

      else if ( ieraut==3 ) then

!     --- Too small  eveps  with respect to round-off error estimation.
!         --- Restart is possible. Enlarge eveps.

         write (irefwr, '(/1x,a,d12.3,a,a)') &
         'warning: sepeigval   rel.precision eveps=', eveps, ' too high', &
         '    multiplied by 10'
         eveps = 10 * eveps
         restrt = .true.
         goto 20

      else if ( ieraut==4 ) then

         call errint ( maxnx, 1 )
         call errsub ( 1410, 1, 0, 0 )

      else if ( ieraut==5 ) then

         call errsub ( 1411, 0, 0, 0 )

      else if ( ieraut==6 ) then

         call errsub ( 1412, 0, 0, 0 )

      else if ( ieraut==7 ) then

         call errint ( nr, 1 )
         call errint ( n, 2 )
         call errsub ( 1413, 2, 0, 0 )

      end if

      if ( ieraut/=0 ) go to 1000
      if ( ierror/=0 ) go to 1000

!     --- e.values are in      eval(1..nr)

      if ( jevec==1 ) then

!     --- calculate the nr eigenvectors
!         release all temporary arrays but alpha(*), beta(*), ev(*,*) and
!         msin. These arrays are made semi-permanent
!         They are deleted at the end of the job
!         fill m by minimum number of iterations needed to get
!         the first nr eigenvalues

         m = maxval(mini(1:nr))

         if ( jprint==1 ) write ( irefwr, '(/1x,a,i4)' ) &
            'for the eigenvectors: the number of iterations can be ', m

!        --- now the dimensions are:
!            sqrt_of_mass(n), alpha(m), beta(m), ev(maxnev,2), mini(nr)

         maxlen = nr + 10
         allocate ( w(maxlen), z(maxlen*m), y(maxlen*n), stat = error )
         if ( error/=0 ) call eralloc ( error, maxlen*m, 'w' )
         if ( ierror/=0 ) go to 1000

!        --- Built nr eigenvectors of the tridiagonal matrix, &
!            corresponding to nr selected eigenvalues

         eps = eveps

         call ervl15 ( nr, m, maxlen, eps, alpha, &
                       beta, ev, w, z, mini, multnr, maxnev )

         if ( jprint==1 ) write ( irefwr, '(/1x,a,i4)' ) &
            'number of lowest e.v. including multiple, multnr=', multnr

!        --- built eigenvectors of the original matrix by backtransformation,
!            performing again the Lanczos process, uncontrolled

         call ervl16 ( n, m, Amat, msin, z, y, nr, 0 )

!        --- reorder and normalize the eigenvectors, include b.c
!            Loop over eigenvectors

         do j = 1, nr

!        --- Create the next eigenvector

            call sepcreasol110 ( ivec(j), 'eigenvector', 0 )
            eigvec => usoltemp

            call ervl12 ( j, n, multnr,  w, &
                          y, Amat, msin, eigvec )

            if ( indprh>0 ) call toreordr ( kprobh, eigvec, &
                               nusol ) ! reorder eigenvector

         end do

!        --- Delete semi-permanent arrays

         deallocate ( w, z, y )

      end if

      deallocate ( alpha, beta, ev, msin, mini, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'alpha' )

1000  call erclos ( 'sepeigval' )

      end subroutine sepeigval
