      subroutine sepfillparmeul ( first, isol, isolprec, ieqn, irhsdlc, &
                                  islold )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-12-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill some parameters in modules for subroutine septimeulstep
!     Create solution array at new time level if it does not exist
!     Create S+M/dt, store in Mattot
!     Fill arrays usol, rhsd and uold
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
use sepmoduledebug
      use sepmodulesolve
      use sepmoduletime
      use sepmodulekprob
      use sepmodulematr
      use sepmoduleintmat
      use sepmodulecactl
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isolprec, ieqn, isol(*), irhsdlc, islold(*)
      logical, intent(in) :: first

!     first          i    If true it concerns the first step
!     ieqn           i    Equation sequence number
!     irhsdlc        i    sequence number of right-hand side
!     islold         i    User input array in which the user puts information
!                         of all preceding solutions
!     isol           i    Contains sequence numbers of solution arrays
!     isolprec       i    Solution at preceding level
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (globmat), pointer :: S, MASS
      logical :: extradebug
      type (intmatrix), pointer :: IM
      integer, pointer :: diag(:)
      integer :: n1

!     IM             Refers to intmt(intmat)
!     MASS           Refers to mass matrix
!     S              Refers to stiffness + dt * mass matrix
!     S1             Refers to stiffness matrix or
!     diag           positions of the diagonal of the large matrix
!     extradebug     If true extra debugging statements are carried out
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPADAPINPSOL  Adapt array inpsoltim. Fill array inpsol from inpsoltim
!     SEPADDMT       Sum of two matrices
!     SEPCOPYINPSOL  Put information of inpsol and rinsol in matrix structure
!     SEPCOPYMT      Copy matrix
!     SEPCREASOL110  Activate or create arrays of type 110
!     SEPFILLSOLMOD2 Fill several modules for subroutine seplinsol
!     SEPPARALLOC    Allocate some global arrays for parallel computing
!     SEPPRINTMAT    Print matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillparmeul' )
      debug = .false. .and. ioutp>=0
      extradebug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillparmeul'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iprob = ks(isolprec)%iprob
      iseqkp = ks(isolprec)%iseqkp
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      kprobhinv => kprobpart%kprobhinv
      nusol = kprobloc(5)
      nbound = kprobloc(10)
      npltra = kprobloc(13)
      indprh = kprobloc(50)
      if ( parallel ) call sepparalloc
      Smat => realmt(stiffmat(1))
      iseqim = Smat%intmatref

      IM => intmt(iseqim)
      typecoupl = IM%typecoupl
      if ( parallel .and. typecoupl==1 ) then
         kprobhinv => kprobpart%kprobhddinv
      else
         kprobhinv => kprobpart%kprobhinv
      end if  ! ( parallel .and. typecoupl==1 )
      nrusol = IM%nrusol

!     --- Create array isol if necessary, store contents in usol

      if ( isol(ieqn)==0 ) call sepcreasol110 ( isol(ieqn), 'isol', 0 )
      ks(isol(ieqn))%jcart = jcart-1

      iseqreactim = Teq(ieqn)%iseqreac
      if ( iseqreactim/=0 ) then
         call sepcreasol110 ( islold(iseqreactim), 'reac', 0 )
         reac => ks(islold(iseqreactim))%sol
      end if  ! ( iseqreactim/=0 )

      if ( first .or. Teq(ieqn)%imasstot==0 ) then

!     --- create   :  Mattot = M + S

         MASS => realmt(massmat)  ! refers to mass matrix
         call sepcopymt ( stiffmat, totmat )  ! Copy stiffness matrix
         S => realmt(totmat)

         if ( MASS%diagonal ) then

!        --- M is diagonal

            n1 = nrusol
            diag => IM%diag
            S%S_ii(diag(1:n1)) = S%S_ii(diag(1:n1))+MASS%S_ii(1:n1)

         else

!        --- M is not diagonal

            S%S_ii = S%S_ii + MASS%S_ii

            if ( extradebug ) then
               write(irefwr,*) 'totmat'
               call sepprintmat ( totmat )
            end if  ! ( debug )

         end if  ! ( MASS%diagonal )

      end if
      nrusol = IM%nrusol

!     --- Store solution at previous time time level in uold

      if ( debug ) call prinrl ( ks(isolprec)%sol, nusol, 'uold/1' )

      if ( indprh>0 ) then
         allocate ( usol(nusol), uold(nusol) )
         uold = ks(isolprec)%sol(kprobhinv)
         usol = ks(isol(ieqn))%sol(kprobhinv)
      else
         uold => ks(isolprec)%sol
         usol => ks(isol(ieqn))%sol
      end if  ! ( indprh>0 )

      if ( Teq(ieqn)%iconsrhs/=2 ) then

!     --- f # 0, help := help + f

         if ( indprh>0 ) then
            allocate ( rhsd(nusol) )
            rhsd = ks(irhsdlc)%sol(kprobhinv)
         else
            rhsd => ks(irhsdlc)%sol
         end if  ! ( indprh>0 )

      end if

!     --- Adapt array inpsoltim
!         Fill array inpsol from inpsoltim

      S => realmt(totmat)
      iseqmt = totmat
      call sepadapinpsol ( ieqn, first) ! Adapt inpsoltim and fill inpsol/rinsol
      call sepcopyinpsol ( S%S_i, 1 )  ! Put information in matrix structure
      call sepfillsolmod2 ( usol )! Fill several modules for subroutine seplinsol

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillparmeul'

      end if  ! ( debug )
      call erclos ( 'sepfillparmeul' )

      end subroutine sepfillparmeul
