      subroutine sepgmres ( A, x, b )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-06-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve linear system of equation by the preconditioned GMRES method
!
! **********************************************************************
!
!                       KEYWORDS
!
!     iterative
!     linear_solver
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(inout) :: A
      double precision, intent(inout) :: x(A%neq), b(A%neq)

!     A              i    Matrix A
!     b              i    Right-hand side
!     x              o    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: imax, nmax, iter, inner, j, k
      double precision, allocatable :: work(:,:), hh(:,:), c(:), s(:), &
                                       rs(:), help(:), diag(:)
      double precision :: bnorm, rnorm, rferr, t
      logical :: repeat1, finished

!     bnorm          Norm of the right-hand side (b,b)
!     c              Array determining the plane rotations which transform the
!                    upper Hessenberg-matrix into an upper triangular matrix.
!     diag           Diagonal of the matrix
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     help           help array to store a vector temporarily
!     hh             Array containing an upper Hessenberg-matrix of dimension
!                    (inner+1).
!     imax           Parameter controlling the size of the subspaces, thus
!                    determining the restarts of the outer iteration
!                    process.
!                    The iterations restart after imax iterations, &
!                    with the latest iterate as new starting vector. The
!                    parameter should be larger than zero.
!     inner          Loop variable for the inner iteration.
!     iter           Iteration sequence number
!     j              Counting variable
!     k              Counting variable
!     nmax           maximum number of iterations that may be performed by
!                    the CG-process. In any way no more than the number of
!                    degrees of freedom iterations are carried out.
!     repeat1        Logical variable indicating if the accuracy check must be
!                    carried out twice (true) or once (false), because the
!                    norm of the solution could not be computed
!     rferr          Maximal accuracy possible for this matrix, due to
!                    round-off
!     rnorm          Norm of residual
!     rs             Array containing the norms of the residuals from the inner
!                    iterations.
!     s              Array determining the plane rotations which transform the
!                    upper Hessenberg-matrix into an upper triangular matrix.
!     t              Auxiliary variable.
!     work           Work array with maximum length (maxiter-ifiter)*n
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     ERTRACE        Prints trace back of subroutines
!     SEPCGESTEPS    Print first line of iteration output
!                    Compute first estimate of accuracy  eps
!     SEPCGSTART     Fill starting value in array x and compute initial residual
!     SEPMAXACC      Compute maximum accuracy possible in iterative solver
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     676   The CG-process has terminated because the number of iterations
!           exceeds the maximum value
!     677   The maximal accuracy with respect to the condition of the
!           matrix is reached
!    2168   Right-hand side equal to 0
! **********************************************************************
!
!                       PSEUDO CODE
!
!     We try to solve the system of equation A x = b, which A a mxm matrix
!
!     The Krylov space K_n = K_N(A,b) = span ( b, Ab, A^2b, .. , A^(n-1)b )
!     In GMRES we search for the vector x_n in K_n which minimizes r_n
!     with r_n = A x_n - b
!     With the Arnoldi process a basis q_1, q_2, .. q_n of K_n is found
!     Hence the vector x_n can be written as x_n = Q_n y_n with y_n in R^n
!     and Q_n the matrix ( q_1 q_2 .. q_n )
!     The Arnoldi process also produces an (n+1)-by-n upper Hessenberg matrix
!     H_n with A Q_n = Q_(n+1) H_n
!     Because columns of Q_n are orthogonal, we have
!     || A x_n -b || = || H_n y_n - beta e_1 || with e_1 the first unit vector
!     and beta = || b - A x_0 ||
!     Hence, x_n can be found by minimizing the Euclidean norm of the residual
!     r_n = H_n y_n - beta e_1
!     This is a linear least squares problem of size n.
!
!     This yields the GMRES method. On the n-th iteration:
!
!        calculate q_n with the Arnoldi method;
!        find the y_n which minimizes ||r_n||;
!        compute x_n = Q_n y_n ;
!        repeat if the residual is not yet small enough.

! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepgmres' )
      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepgmres'
         call ertrace
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      bnorm = norm(b, A)
      if ( debug ) write(irefwr,2) 'bnorm', bnorm

      if ( bnorm<=0d0 ) then

!     --- Right-hand side is equal to zero

         x = 0d0
         call errwar ( 2168, 0, 0, 0 )
         go to 1000  ! rhs is equal to 0

      end if  ! ( bnorm<=0d0 )

      if ( .not. parallel ) nglob  = A%neq   ! number of global unknowns
      imax = A%imax                  ! Maximal dimension of Krylov space
      nmax = A%nmax                  ! maximum number of iterations
      if ( nmax==0 ) nmax=10*nglob

      if ( debug ) &
         write(irefwr,1) 'iprec, iprint', A%type_prec, A%iprint

      allocate ( work(A%neq,imax+2), hh(imax+1,imax), c(imax), s(imax), &
                 rs(imax+1), help(A%neq), stat = error )
      if ( error/=0 ) call eralloc ( error, A%neq*imax, 'work' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,1) 'imax, iprec, nmax, iprint', &
                          imax, A%type_prec, nmax, A%iprint
         write(irefwr,*) 'eistat, precon ', A%eistat, A%precon
      end if  ! ( debug )

!     --- Compute maximum accuracy possible

      call sepmaxacc ( A, b, work, bnorm, rferr )
      if ( debug ) write(irefwr,2) 'rferr', rferr

      if ( A%eistat ) then

!     --- Eisenstat preconditioning, fill diag

         allocate ( diag(A%neq) )

         do j = 1, A%neq
            diag(j) = A%S(A%diag(j))
         end do  ! j = 1, A%neq

      end if  ! ( A%eistat )

!     --- Print heading if iprint>0

      if ( A%iprint>0 ) write ( irefwr,100 ) imax
100   format(//' Output for the GMRES method  (non-symmetric matrix)',/ &
               ' GMRES is restarted after ', i4, ' iterations'//)

!     --- Set start vector and initial residu (stored in work(1))

      call sepcgstart ( A, x, b, work(1,1), work(1,2) )

      rnorm = norm(work(:,1), A)

!     --- Print first line of iteration output
!         Compute first estimate of accuracy  eps

      iter = 1
      inner = 0
      call sepcgesteps ( A, b, help, rnorm, rferr, repeat1, iter )
      if ( debug ) write(irefwr,2) 'rnorm, eps', rnorm, A%eps

      finished = .false.

      do while ( .not. finished .and. iter<nmax )

         if ( debug ) write(irefwr,1) 'iter', iter

         finished = iter>1 .and. rnorm<=A%eps .or. rnorm<=0d0

         do while ( finished )

!        --- The accuracy of the process seems to be reached
!            Check if eps must be recomputed

            if ( repeat1 ) then

!           --- repeat is true, recompute eps

               repeat1 = .false.
               A%eps = A%eps1 + norm(x, A) * A%eps2

!              --- Check error level.

               if ( A%eps<rferr ) then

!              --- eps is smaller than maximal accuracy possible
!                  It is useless to continue the process after rnorm
!                  is less than rferr

                  A%eps = rferr

                  if ( A%iprint>=0 ) then
                     call erreal ( rferr, 1 )
                     call errwar ( 677, 0, 1, 0 )
                  end if

               end if
               if ( debug ) write(irefwr,2) 'eps', A%eps

               finished = rnorm<=A%eps

            end if

            if ( finished .and. .not. repeat1 ) then

!           --- Process is ended

               if ( A%iprint>=1 ) then

!              --- iprint>=1  Print information

                  write ( irefwr,150 ) iter-1, rnorm
150               format(' The number of executed iterations in the', &
                         ' GMRES process is', i5/ &
                         ' The 2-norm of the residual is:', &
                         ss,1pe12.2e2 )

               end if

               A%numiter = iter-1
               A%itererr = 0

               go to 500

            end if  ! ( finished .and. .not. repeat1 ) then

         end do  ! while ( finished )

!        --- Accuracy not yet reached
!            do not start (nmax+1)-th iteration

         if ( iter>nmax ) then
            finished = .true.
            exit
         end if  ! ( iter>nmax )

         if ( inner==0 ) then

!        --- First inner iteration
!            Subdivide residual by norm and store norm in rs(1)

            work(:,1) = work(:,1)/rnorm
            rs(1)  = rnorm

         end if  ! ( inner==0 )

         if ( debug ) write(irefwr,2) 'rnorm/1', rnorm

!        --- Inner iteration step, apply modified Gram-Schmidt to form a
!            new column of hh.

         inner = inner+1

!        --- Compute Ap, store in work
!            In case of Eisenstat Ap = E^(-T) p + E^(-1) p + K E^(-T) p
!            If necessary projection is carried out

         help = work(:,inner)/A

         if ( A%eistat ) then
            work(:,inner+1) = work(:,inner) + diag * help
            work(:,inner+1) = A/work(:,inner+1)
            work(:,inner+1) = work(:,inner+1)+help
         else
            work(:,inner+1) = A*help
         end if  ! ( A%eistat )

         do j = 1, inner
            t = dotprod ( work(:,j), work(:,inner+1), A )
            hh(j,inner) = t
            work(:,inner+1) = work(:,inner+1) -t*work(:,j)
         end do  ! j = 1, inner
         hh(inner+1,inner) = norm(work(:,inner+1), A)

!        --- Perform plane rotations on new column.

         do k = 2, inner
            t=hh(k-1,inner)
            hh(k-1,inner) =  c(k-1)*t + s(k-1)*hh(k,inner)
            hh(k,inner)   = -s(k-1)*t + c(k-1)*hh(k,inner)
         end do  ! k = 2, inner

!        --- Calculate new plane rotation.

         t = sqrt(hh(inner,inner)**2 + hh(inner+1,inner)**2)
         if ( t<=0d0 ) t=epsmac
         c(inner) = hh(inner,inner)/t
         s(inner) = hh(inner+1,inner)/t
         rs(inner+1) = -s(inner)*rs(inner)
         rs(inner) = c(inner)*rs(inner)
         hh(inner,inner) = c(inner)*hh(inner,inner) + &
                           s(inner)*hh(inner+1,inner)

!        --- Determine residual norm.

         rnorm = abs(rs(inner+1))
         if ( debug ) write(irefwr,2) 'rnorm/2', rnorm

!        --- Check for termination of inner iterations.

         if ( inner==imax .or. rnorm<A%eps .or. iter==nmax ) then

!        --- Compute new solution. First solve upper triangular system.

            rs(inner) = rs(inner)/hh(inner,inner)
            do k=inner-1,1,-1
               t=rs(k)
               do j=k+1,inner
                  t=t-hh(k,j)*rs(j)
               end do
               rs(k)=t/hh(k,k)
            end do

!           --- Form linear combinations of residuals to get solution.

            work(:,inner+2) = rs(1)*work(:,1)
            do j = 2, inner
               work(:,inner+2) = work(:,inner+2)+rs(j)*work(:,j)
            end do  ! j = 2, inner
            if ( A%eistat .or. A%precon ) then
               help = work(:,inner+2)/A
            else
               help = work(:,inner+2)
            end if  ! ( A%eistat .or. A%precon )

            x = x + help

            inner = 0
            work(:,1) = b
            work(:,2) = A*x

            if ( A%eistat ) then

!           --- In the case of Eisenstat there is  amat = A - 2 * I, &
!               so t(*) must be corrected

               work(:,2) = work(:,2)+2d0*x

            end if

!           --- Update r

            work(:,1) = work(:,1) - work(:,2)

!           --- Premultiply residual by L^(-1) in the case of Eisenstat

            if ( A%eistat ) work(:,1) = A/work(:,1)
            rnorm = norm(work(:,1), A)

         else
            work(:,inner+1) = work(:,inner+1)/hh(inner+1,inner)
         end if

         if ( A%iprint>=2 ) then

!        --- iprint>=2,  Maximal output

            write ( irefwr, 350 ) iter, rnorm
350         format( i10, ss,1pe15.2e2 )

         end if

         iter = iter+1

      end do  ! while ( .not. finished .and. iter<nmax )

      if ( .not. finished ) then

!     --- No convergence

         call errint ( nmax, 1 )
         if ( A%aterror==1 ) then
            A%numiter=nmax
            call errwar ( 676, 1, 0, 0 )
         else
            call errsub ( 676, 1, 0, 0 )
         end if  ! ( A%aterror==1 )

      end if  ! ( .not. finished )

500   deallocate ( work, hh, c, s, rs, help, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'work' )
      if ( A%eistat ) deallocate ( diag )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepgmres'

      end if  ! ( debug )
      call erclos ( 'sepgmres' )

      end subroutine sepgmres
