      subroutine sepidrshadowg ( A, sparam, pshadow )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-03-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute shadow vectors p for IDR process
!     Store in pshadow
!     Parallel case, global shadow vectors are created at node 1
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
!     parallel
!     idr
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(inout) :: A
      integer, intent(in) :: sparam
      double precision, intent(out) :: pshadow(nrusol,sparam)

!     A              i    Matrix A
!     pshadow        o    Array of size n x s
!                         Contains all shadow vectors of size n each
!                         So pshadow(i,j) contains i^th component of p_j
!     sparam         i    Defines the parameters s in IDR(s)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenpshadow, istart, kstart, icontext, iblock, i
      double precision, allocatable :: work(:,:), pshadowsend(:)

!     i              Counting variable
!     iblock         Block sequence number
!     icontext       Tag for sending and receiving, is used as check
!     istart         Starting address in kprobhsend
!     kstart         Starting address in pshadowsend
!     lenpshadow     Estimated length of pshadowsend
!     pshadowsend    Is used to store the shadow vectors that must be sent
!                    to other nodes
!     work           Work array to store the global shadow vectors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INIRECEIVEMPI  Receive information from other node (mpi)
!     INISENDMPI     Send information to other node (mpi)
!     PRINRL1        Print 2d real vector
!     SEPIDRSHADOW   Compute shadow vectors p for IDR process
!                    Store in pshadow
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepidrshadowg' )
      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepidrshadowg'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icontext = 43
      if ( inodenr==1 ) then

!     --- Node number 1, create global shadow vectors

         if ( debug ) write(irefwr,1) 'nglob', A%nglob

         allocate ( work(A%nglob,sparam), stat = error )
         if ( error/=0 ) &
            call eralloc ( error, A%nglob*sparam, 'work' )
         if ( ierror/=0 ) go to 1000

!        --- create global shadow vectors

         call sepidrshadow ( sparam, A%nglob, work )

         if ( debug ) call prinrl1 ( work, sparam, A%nglob, 'work' )

!        --- map shadow vectors to all nodes


         do i = 1, sparam
            pshadow(1:nrusol,i) = work(1:nrusol,i)
         end do  ! i = 1, sparam
         if ( debug ) &
            call prinrl1 ( pshadow, sparam, nrusol, 'pshadow' )

         lenpshadow = 0
         do iblock = 2, iacnodes
            nrusolloc = nrusollocs(iblock)-nrusollocs(iblock-1)
            lenpshadow = max(lenpshadow,nrusolloc*sparam)
         end do  ! iblock = 2, iacnodes
         if ( debug ) write(irefwr,1) 'lenpshadow', lenpshadow

         allocate ( pshadowsend(lenpshadow), stat = error )
         if ( error/=0 ) &
            call eralloc ( error, lenpshadow, 'pshadowsend' )
         if ( ierror/=0 ) go to 1000

         do iblock = 2, iacnodes

            istart = nrusollocs(iblock-1)
            nrusolloc = nrusollocs(iblock)-istart
            kstart = 0
            do i = 1, sparam
               pshadowsend(kstart+1:kstart+nrusolloc) = &
                  work(istart+1:istart+nrusolloc,i)
               kstart = kstart+nrusolloc
            end do  ! i = 1, sparam

            if ( debug ) then
               write(irefwr,1) 'iblock, kstart', iblock, kstart
               call prinrl1 ( pshadowsend, sparam, nrusolloc, &
                              'pshadowsend' )
            end if  ! ( debug )

            call inisendmpi ( pshadowsend, kstart, SPDOUBLE, iblock, &
                              icontext )

         end do  ! iblock = 2, iacnodes

         deallocate ( work, pshadowsend, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'work' )

      else

!     --- Node number>1, receive shadow vectors

         lenpshadow = nrusol*sparam
         call inireceivempi ( pshadow, lenpshadow, SPDOUBLE, iblock, &
                              icontext )

      end if  ! ( inodenr==1 )

1000  call erclos ( 'sepidrshadowg' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( pshadow, sparam, nrusol, 'Pshadow' )
         write(irefwr,*) 'End sepidrshadowg'

      end if  ! ( debug )

      end
