      subroutine sepmatdiv ( A, x, b )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-06-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute b = A^(-1) x, where A is a preconditioning matrix
!     a symmetric compact storage is used, hence only D and L
!     The preconditioning matix has been created before
!     In other words the ILU inverse of A is stored
!     Solve A b = x, where A is an ILU preconditioning matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr ! definition of matrix structure
      use sepmodulemat    ! definition of matrices
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(in) :: A
      double precision, intent(in) :: x(A%neq)
      double precision :: b(A%neq)

!     A              i    Matrix A
!     b              o    result Ax
!     x              i    vector to be multiplied
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iprec

!     iprec          type of preconditioning
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRPARPREC2     Perform first parallel part of matrix vector multiplication
!     SEPPARPREC1    Perform parallel part of matrix vector multiplication
!     SEPRCMATDIV    Compute y = A^(-1) x, where A is a preconditioning matrix
!                    a row-compact storage is used
!     SEPRCMATDIVEU  Compute y = A^(-1) x, where A is a preconditioning matrix
!                    a symmetric compact storage is used, hence only D and L
!                    Eisenstat preconditioning
!     SEPRCMATDIVNS  Compute y = A^(-1) x, where A is a preconditioning matrix
!                    a row-compact storage is used
!                    Matrix is not scaled
!     SEPSCMATDIV    Compute y = A^(-1) x, where A is a preconditioning matrix
!                    a symmetric compact storage is used, hence only D and L
!     SEPSCMATDIVU   Compute y = A^(-1) x, where A is a preconditioning matrix
!                    a symmetric compact storage is used, hence only D and L
!                    Eisenstat preconditioning
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmatdiv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmatdiv'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iprec = A%type_prec

      if ( debug ) &
         write(irefwr,1) 'iprec, scaling ', iprec, A%scaling

      b = x
      if ( iprec<=1 ) go to 1000

      if ( parallel .and. typecoupl==0 ) then

!     --- Parallel case with Neumann-Neumann coupling
!         Adapt solution

         call prparprec2 ( b, A%innercontr(A%neq+1), A%innercontr(A%neq+2) )
         if ( debug ) call prinrl ( b, A%neq, 'b/1' )

      end if

      if ( iprec==2 .or. iprec==4 .or. iprec==7 ) then

!     --- Eisenstat preconditioning, only D U y = x

         if ( A%symmetric ) then

!        --- symmetric compact matrix

            call sepscmatdivU ( A, b )

         else

!        --- (non-)symmetric compact matrix

            call seprcmatdiveU ( A, b )

        end if  ! ( A%symmetric )

      else

!     --- Standard preconditioning, only A y = x

         if ( A%symmetric ) then

!        --- symmetric compact matrix

            if ( A%scaling==0 .or. iprec<8 ) then

!           --- Standard preconditioner with scaling

               call sepscmatdiv ( A, b )

            end if  ! ( A%scaling==0 .or. iprec<8 )

         else

!        --- row-compact matrix

            if ( A%scaling==0 ) then
               call seprcmatdivns ( A, b )
            else
               call seprcmatdiv ( A, b )
            end if  ! ( A%scaling==0 )

         end if  ! ( A%symmetric )

      end if  ! ( iprec==2 .or. iprec==4 .or. iprec==7 )

      if ( parallel .and. typecoupl==0 ) then

!     --- Parallel case with Neumann-Neumann coupling
!         Adapt solution and make equal for all common nodal points in blocks

         call sepparprec1 ( A, b, A%innercontr(A%neq+1), A%innercontr(A%neq+2) )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmatdiv'

      end if  ! ( debug )
      call erclos ( 'sepmatdiv' )

      end subroutine sepmatdiv
