      subroutine sepmatvec30 ( A, x, y, nusol, infshi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-03-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Matrix vector multiplication for serial matrices
!     Special case: finite difference matrix
!     Old subroutine prmv01
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     matrix_vector_multiplication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nusol, infshi(9)
      double precision :: A(nusol,9), y(nusol), x(nusol)

!     A              i    Two-dimensional double precision array of size
!                         NUSOL * 9 in which the matrix is stored.
!                         v are the basis functions
!     infshi         i    Integer information array of size 9
!                         INFSHI(j)  contains the distance of the
!                                    j-th diagonal to the main diagonal.
!                                    If the diagonal belongs to the lower
!                                    diagonal matrix then this distance is
!                                    assumed to be negative.
!     nusol          i    Total number of unknowns
!     x              i    vector to be multiplied
!     y              o    result vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, ishift

!     i              Counting variable
!     ishift         Shift to compute address of column in matrix
!                    The matrix column in row i corresponding to element (i)
!                    is equal to i+ishift, where ishift = infshi(j)
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The matrix vector multiplication is defined by:  a   x  = f
!                                                       ij  j    i
!
!     Since the matrix has a different structure for both degrees of freedom
!     we have to perform different loops
!
!     In order to get maximum vector length in the multiplication, we
!     multiply over the diagonals, rather than in the row direction
!
!     So:
!
!            For j = 1 (1) ndegfd
!
!                (j)    (j)   (j)
!               f    = d     u          (d is diagonal)
!                i      ii    i
!
!
!            For j = 1 (1) ndegfd
!
!               For k = 2 (1) nconct
!
!                  (j)     (j)    (j)   (j)
!                 f    =  f    + a     u
!                  i       i      i l   l
!                                    k   k
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmatvec30' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmatvec30'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
!     ---- Multiply by diagonal
!
      y = A(:,1) * x

!     --- Multiply rest of matrix
!         Loop over the connections

      do k = 2, 9

         ishift = infshi(k)

!        *  From the full range i=1,nusol we should deal only the cases when
!        *      1 <= i+ishift <= nusol*1

!        *  Loop over the unknowns

         do i = max (     1,     1-ishift ), min ( nusol, nusol-ishift )

!           *  i+ishift  can be fare out of the range  1..nusol
!           *  and  A(i,k)  nonzero,
!           *  to access the other columns of the x(*,*) array

            y(i) = y(i) + A(i,k) * x(i+ishift)

         end do  ! i = max (     1,     1-ishift ), min ( nusol, nusol-ishift )

      end do  ! k = 2, 9

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmatvec30'

      end if  ! ( debug )
      call erclos ( 'sepmatvec30' )

      end subroutine sepmatvec30
