      subroutine sepnestsollu ( A, y, rhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-12-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute y = A^(-1) y, where A is a preconditioning matrix
!     a row-compact storage is used
!     The preconditioning matrix has been created before
!     In other words the ILU inverse of A is stored
!     In this case the matrix is not scaled
! **********************************************************************
!
!                       KEYWORDS
!
!     iterative
!     linear_solver
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat   ! definition of matrices
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(in) :: A
      double precision, intent(out) :: y(A%neq)
      double precision, intent(in) :: rhsd(A%neq)

!     A              i    Matrix A
!     rhsd           i    right-hand-side vector
!     y              o    solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, ilow, itop
      integer, pointer :: diag(:), row(:), col(:)
      double precision, pointer :: mat(:)
      double precision :: sum1

!     col            Column numbers of matrix
!     diag           Positions of diagonal elements of the large matrix
!     i              Counting variable
!     ilow           lower bound in loop
!     itop           Upper bound in loop
!     j              Counting variable
!     mat            elements of matrix
!     row            row numbers of matrix
!     sum1           Parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepnestsollu' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepnestsollu'
         call prinrl ( y, A%neq, 'x' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      diag => A%Mdiag
      row => A%Mrows
      col => A%Mcols
      mat => A%MD

      y = rhsd

!     --- First forward substitution L y = x

      do i = 2, A%neq

         ilow = row(i)+1
         itop = diag(i)-1
         y(i) = y(i)-sum(mat(ilow:itop)*y(col(ilow:itop)))

      end do  ! i = 1, A%neq
      if ( debug ) call prinrl ( y, A%neq, 'y/1' )

!     --- Finally backward substitution U y = y

      do i = A%neq, 1, -1
         ilow = diag(i)+1
         itop = row(i+1)
         y(i) = (y(i)-sum(mat(ilow:itop)*y(col(ilow:itop)))) / mat(diag(i))
      end do  ! i = A%neq, 2, -1

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( y, A%neq, 'y' )
         write(irefwr,*) 'End sepnestsollu'

      end if  ! ( debug )
      call erclos ( 'sepnestsollu' )

      end subroutine sepnestsollu
