      subroutine seppreceissym ( amax, A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the preconditioning matrix for the CG-process
!     iprec = 2,  Eisenstat preconditioning
!     Old subroutine prcg01prec2
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulemat
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision, intent(in) :: amax

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     amax           i    maximal element of a matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, k, kpo, k1
      double precision :: sum1
      logical :: again
      integer, pointer :: point(:), rowhor(:), colhor(:)

!     again          Indicates if the loop must be repeated (true) or not
!                    they are not
!     colhor         contains column numbers per horizontal row
!     i              Counting variable
!     k              Counting variable
!     k1             column number
!     kpo            position of element k1 in preconditioning matrix
!     point          pointer array referring to next element in a row
!     rowhor         contains starting addresses (-1) of horizontal rows in
!                    colhor
!     sum1           Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     SEPCOMMATREV   Fill arrays irowsnew and icolsnew with upper triangular
!                    part if icolsii and irowsii contain the lower triangular
!                    part and vice versa
!     SEPPRINMAT6    print compact matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     674   The desired preconditioning matrix does not exist
!    2236   The desired preconditioning matrix does not exist, lumping is tried
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppreceissym' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppreceissym'
         write(irefwr,1) 'prlump', A%prlump
         write(irefwr,2) 'adapt', A%adapt
         call sepprinmat6 ( A )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000
      again = .true.

      allocate ( point(A%neq+1), rowhor(A%neq+1), colhor(size(A%cols)) )

!     --- Store row structure of matrix in row/colhor

      call sepcommatrev ( rowhor, colhor, A%rows, A%cols, A%neq )

      do while ( again )

         if ( A%adapt>0d0 ) then

!        --- adapt diagonal

            A%MD = max(A%adapt,A%S(A%diag))

         else

!        --- No adaptation

            A%MD = A%S(A%diag)

         end if  ! ( A%adapt>0d0 )

         if ( A%prlump==1) then

!        --- Positive off-diagonal elements are lumped to the main
!            diagonal

            do i = 1, A%neq
               do k = A%rows(i)+2, A%rows(i+1)
                  if ( A%S(k)>0d0 ) then
                     A%MD(i) = A%MD(i) + A%S(k)
                     A%MD(A%cols(k)) = A%MD(A%cols(k)) + A%S(k)
                  end if
               end do
            end do

         end if
         point = A%rows+2

!        --- end of lumping

         do i = 1, nrusol

            sum1 = A%MD(i)

            do k = rowhor(i)+1, rowhor(i+1)-1

               k1 = colhor(k)

!              --- The sum is not adapted when prlump = 1 and the
!                  off-diagonal is positive

               kpo = point(k1)
               if ( A%S(kpo)<=0d0 .or. A%prlump/=1 ) then
                  sum1 = sum1 - ( A%S(kpo) / A%MD( k1 ) ) ** 2
               end if
               point(k1) = point(k1)+1

            end do  ! k = rowhor(i)+1, rowhor(i+1)-1

            if ( A%prlump/=1 .and. &
                 ( sum1<=0d0 .or. sum1>=amax ) ) then

!           --- If sum becomes too small or too big make prlump = 1
!               and start again.

                call errwar ( 2236, 0, 0, 0)
                A%prlump = 1
                again = .true.
                exit

            else

!           --- If sum becomes too small and prlump = 1 give an error
!               message and stop

               if ( sum1<=0d0 ) then
                  call errint ( i, 1 )
                  call erreal ( sum1, 1 )
                  call errsub ( 674, 1, 1, 0)
                  go to 1000
               end if

               A%MD (i) = sqrt ( sum1 )

               again = .false.

            end if  ! ( A%prlump/=1 .and. ... )

         end do  ! i = 1, nrusol

      end do  ! while ( again )

      deallocate ( point, rowhor, colhor )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seppreceissym'

      end if
      call erclos ( 'seppreceissym' )

      end subroutine seppreceissym
