      subroutine seppreciggt ( amax, A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-06-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute GG^T decomposition for system of real linear equations
!     (nested dissection)
!     Symmetric positive definite matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     real
!     direct_method
!     nested_dissection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulekprob
      use sepmoduleintmat
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision, intent(in) :: amax

!     A             i/o   Matrix A
!     amax           i    maximal element of a matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k, kk, ip, jp, ii
      integer, pointer :: point(:), point1(:), row(:), col(:), &
                          rowhor(:), colhor(:), diag(:)
      double precision, pointer :: mat(:), c(:)
      double precision :: Lik, matinv
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     Lik            Element L_ik of decomposed matrix
!     c              Work array to store one column of decomposition matrix
!     col            Column numbers of matrix
!     colhor         contains column numbers per horizontal row
!     diag           Positions of the diagonal elements of the matrix
!     i              Row index
!     ii             Position of element i,i
!     ip             Loop variable
!     j              Column index
!     jp             Loop variable
!     k              Column index
!     kk             position of element D(k)
!     mat            elements of matrix
!     matinv         inverse of  element D(k)
!     point          pointer array referring to next element in a row
!     point1         pointer array referring to next element to be used
!                    in last summation over columns
!     row            row numbers of matrix
!     rowhor         contains starting addresses (-1) of horizontal rows in
!                    colhor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRINRL         Print 1d real vector
!     SEPCOMMATREV   Fill arrays irowsnew and icolsnew with upper triangular
!                    part if icolsii and irowsii contain the lower triangular
!                    part and vice versa
!     SEPCOPYTOPREC5 Copy matrix in preconditioning matrix
!     SEPPRINCOMPSTR print part of array intmat corresponding to internal
!                    unknowns. Compact storage
!     SEPPRINMAT6    print compact matrix
!     SEPPRINMAT6P   print compact preconditioning matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2236   The desired preconditioning matrix does not exist, lumping is tried
!    3229   Internal error in SEPRAN: col(point1(i))<=k.
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For i = 1 to n
!        l_ii = sqrt(a_ii - sum_k=1^k=i-1 l_ik^2)
!        For j = i+1 to n do
!           l_ji = ( a_ji-sum_k=1^k=i-1 l_ik l_jk )
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppreciggt' )
      debug = .false. .and. ioutp>=0

      IM => intmt(iseqim)

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppreciggt'
         write(irefwr,1) 'A%prlump', A%prlump
         write(irefwr,2) 'A%adapt', A%adapt
         write(irefwr,*) 'IM%lapshift ', IM%lapshift
         call sepprinmat6 ( A )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )
  4      format ( a, 1x, i6, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      row => A%Mrows
      col => A%Mcols
      mat => A%MD
      diag => A%Mdiag

      allocate ( point(A%neq+1), point1(A%neq+1), c(A%neq), &
                 rowhor(A%neq+1), colhor(size(col)) )

!     --- Store row structure of matrix in row/colhor

      call sepcommatrev ( rowhor, colhor, row, col, A%neq )
      if ( debug ) call sepprincompstr ( A%neq, rowhor, colhor )

!     --- Copy matrix into preconditioning matrix

300   if ( copyprec ) call sepcopytoprec5 ( A )
      point = row+2
      point1 = point
      c = 0d0

!     --- Adapt diagonal in case of shifted laplace

      if ( IM%lapshift ) mat(A%diag) = mat(A%diag)-(1d0+lapshift)*A%DLshift

      if ( A%adapt>0d0 ) mat(A%Mdiag) = max(mat(A%Mdiag),A%adapt)

      if ( A%prlump==1 ) then

!     --- Positive off-diagonal elements are lumped to the main
!         diagonal

         do i = 1, A%neq
               ii = A%Mdiag(i)
               do k = A%Mrows(i)+2, A%Mrows(i+1)
                  if ( mat(k)>0d0 ) then
                     mat(ii) = mat(ii) + mat(k)
                     kk = A%Mdiag(A%Mcols(k))
                     mat(kk) = mat(kk) + mat(k)
                     mat(k) = 0d0
                  end if
               end do
         end do

      end if

!     --- end of lumping

!     --- Loop over rows

      do k = 1, A%neq

         kk = row(k)+1      ! position of L(k,k)

!        --- Copy column k of mat in c

         do i = row(k)+1, row(k+1)
            c(col(i)) = mat(i)
         end do  ! i = row(k)+1, row(k+1)

!        --- For all non-zero rows k do

         do ip = rowhor(k)+1, rowhor(k+1)-1

            i = colhor(ip)  ! row number

            Lik = mat(point(i))  ! element L(i,k)
            mat(kk) = mat(kk)-Lik*Lik

!           --- For each i < k: all columns j>k for which L(i,j)#0

            if ( col(point1(i))<=k ) point1(i) = point1(i)+1
            if ( point1(i)<=row(i+1) ) then
               if ( col(point1(i))<=k ) then
                  call errsub ( 3229, 0, 0, 0 )
                  go to 1000
               end if  ! ( col(point1(i))<=k )
            end if  ! ( point1(i)<=row(i+1) )

            do jp = point1(i), row(i+1)
               j = col(jp)
               c(j) = c(j)-Lik*mat(jp)   ! c_j = c_j - L(k,i)*L(k,j)
            end do  ! i = point(j), row(j+1)
            point(i) = point(i)+1

         end do  ! j = 1, k

         if ( A%prlump/=1 .and. &
              ( mat(kk)<=0d0 .or. mat(kk)>=amax ) ) then

!        --- If mat(kk) becomes too small or too big make prlump = 1
!            and start again.

             call errwar ( 2236, 0, 0, 0)
             A%prlump = 1
             goto 300

         end if

         mat(kk) = sqrt(mat(kk))
         matinv = 1d0/mat(kk)

         if ( debug ) write(irefwr,4) 'k, mat(kk)', k, mat(kk)

!        --- Store c in mat and reset c

         do ip = row(k)+2, row(k+1)
            i = col(ip)
            mat(ip) = c(i)*matinv
            c(i) = 0d0
         end do  ! i = row(k)+2, row(k+1)
         c(k) = 0d0

         if ( debug .and. k<A%neq ) &
            call prinrl ( mat(row(k)+2), row(k+1)-row(k)-1, 'col k' )

      end do  ! j = 1, A%neq

      if ( A%scaling/=10 ) then

!     --- Scale matrix using diagonal

         do i = 1, A%neq
            do k = row(i)+2, row(i+1)
               mat(k) = mat(k) / mat(diag(col(k)))
            end do
         end do

      end if  ! ( A%scaling/=10 )

      deallocate ( point, point1, c, rowhor, colhor )

1000  if ( debug ) then

!     --- Debug information

         call sepprinmat6p ( A )
         write(irefwr,*) 'End seppreciggt'

      end if  ! ( debug )
      call erclos ( 'seppreciggt' )

      end subroutine seppreciggt
