      subroutine sepprecilu9 ( A, amax, nolump )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-12-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     computes the preconditioning matrix in case of row-compact storage
!     ipreco = 3, Incomplete LU-decomposition
!     Real matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulesolve
      use sepmodulematstr
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      integer, intent(in) :: nolump
      double precision, intent(in) :: amax

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     amax           i    maximal element of a matrix
!     nolump         i    If 1, no lumping should be used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k, jp, kp, idiag, ii
      integer, pointer :: point(:), diag(:), row(:), col(:)
      double precision, pointer :: mat(:)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     col            Column numbers of matrix
!     diag           Positions of diagonal elements of the large matrix
!     i              Counting variable
!     idiag          Position of diagonal element in matrix
!     ii             Position of diagonal element i in matrix
!     j              Counting variable
!     jp             position of element i,j in matrix
!     k              Counting variable
!     kp             Position of element j,k in matrix
!     mat            elements of matrix
!     point          Array with positions of specific matrix elements
!     row            row numbers of matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     SEPCOPYTOPREC9 Copy matrix in preconditioning matrix
!                    Row compact storage
!     SEPPRINMAT6P   print compact preconditioning matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     674   The desired preconditioning matrix does not exist
!    2236   The desired preconditioning matrix does not exist, lumping is tried
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprecilu9' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprecilu9'
         write(irefwr,1) 'n', A%neq
         write(irefwr,1) 'iseqpre, iseqintmat', iseqpre, iseqintmat
         write(irefwr,1) 'prlump', A%prlump
         write(irefwr,2) 'lapshift, adapt', lapshift, A%adapt
         call sepprinmat6p ( A )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Copy matrix into preconditioning matrix

      if ( copyprec ) call sepcopytoprec9 ( A )

      diag => A%Mdiag
      row => A%Mrows
      col => A%Mcols
      mat => A%MD

      IM => intmt(intmat)

!     --- Adapt diagonal in case of shifted laplace

      if ( IM%lapshift ) then
         do i = 1, A%neq
            idiag = diag(i)
            mat(idiag) = mat(idiag)-lapshift*A%DLshift(i)
         end do  ! i = 1, A%neq
      end if  ! ( IM%lapshift )

      if ( A%adapt>0d0 ) then

!     --- adapt diagonal

         do i = 1, A%neq
            idiag = diag(i)
            mat(idiag) = max(mat(idiag),A%adapt)
         end do

      end if

100   if ( A%prlump==1 ) then

!     --- Off-diagonal elements with the same sign as the diagonal
!         element are lumped to the main diagonal, except when the
!         diagonal element in the same row or column is equal to zero.

         do i = 1, A%neq

!        --- Loop over columns in lower triangular part

            ii = diag(i)
            do jp = row(i)+1, diag(i)-1

               j = col(jp)  ! column number
               if ( mat(jp)*mat(ii)>0d0 .and. &
                    abs(mat(diag(j)))>0d0 ) then
                  mat(ii) = mat(ii)+mat(jp)
                  mat(jp) = 0d0
               end if  ! ( mat(jp)*mat(ii)>0d0 .and. .. )

            end do  ! jp = row(i)+1, diag(i)-1

!           --- Loop over columns in upper triangular part

            do jp = diag(i)+1, row(i+1)

               j = col(jp)
               if ( mat(jp)*mat(ii)>0d0 .and. &
                    abs(mat(diag(j)))>0d0 ) then
                  mat(ii) = mat(ii)+mat(jp)
                  mat(jp) = 0d0
               end if  ! ( mat(jp)*mat(ii)>0d0 .and. .. )

            end do  ! k = diag(j)+1, row(j+1)-1

         end do  ! i = 1, A%neq

      end if  ! ( A%prlump==1 )

      allocate ( point(A%neq) )
      point = 0

!     --- Loop over rows

      do i = 2, A%neq

!     --- Loop over columns to fill start in point

         do j = row(i)+1, row(i+1)
            point(col(j)) = j
         end do  ! j = row(i)+1, row(i+1)
         ii = diag(i)

!        --- Loop over columns in lower triangular part

         do jp = row(i)+1, diag(i)-1

            j = col(jp)  ! column number
            mat(jp) = mat(jp)/mat(diag(j))

!           --- Loop over columns in upper triangular part

            do kp = diag(j)+1, row(j+1)

               k = point(col(kp))
               if ( k>0 ) then

!              --- k> 0, update element k

                  mat(k) = mat(k)-mat(jp)*mat(kp)

               end if  ! ( k>0 )

            end do  ! k = diag(j)+1, row(j+1)-1

         end do  ! j = row(i)+1, diag(i)-1

!        --- Reset point

         do j = row(i)+1, row(i+1)
            point(col(j)) = 0
         end do  ! j = row(i)+1, row(i+1)-1

         if ( skipzeros .and. abs(mat(ii))<=0d0 ) A%MD(ii) = epsmac

         if ( A%prlump/=1 .and. nolump/=1 .and. &
              ( abs(A%MD(ii))<=0d0 .or. &
                abs(A%MD(ii))>=amax ) ) then

!        --- If |A%MD(ii)| becomes zero or too big make prlump = 1
!            and start again.

             call errwar ( 2236, 0, 0, 0)
             A%prlump = 1
             goto 100

         end if

!        --- If |A%MD(ii)| becomes zero try the Meijerink approach

         if ( abs(A%MD(ii))<=0d0 ) A%MD(ii) = 0.01d0*A%S(ii)

!        --- If |A%MD(ii)| remains zero give an error message
!            and stop

         if ( abs(A%MD(ii))<=0d0 ) then
            call errint ( i, 1 )
            call erreal ( A%MD(ii), 1 )
            call errsub ( 674, 1, 1, 0)
            go to 1000
         end if

      end do  ! i = 2, A%neq

      do  i = 1, A%neq
         do k = diag(i)+1, A%Mrows(i+1)
            mat(k) = mat(k) / mat(diag(col(k)))
         end do
      end do

      deallocate ( point )

1000  if ( debug ) then

!     --- Debug information

         call sepprinmat6p ( A )
         write(irefwr,*) 'End sepprecilu9'

      end if  ! ( debug )
      call erclos ( 'sepprecilu9' )

      end subroutine sepprecilu9
