      subroutine sepprojAUT ( A, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-07-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute y = AU^T x as part of the projection algorithm
!     U is the matrix of projection vectors
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleproj
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(in) :: A
      double precision, intent(in) :: x(A%neq)
      double precision, intent(out) :: y(A%nproj)

!     A             i/o   Matrix A
!     x              i    Vector to be multiplied
!     y              o    result of multiplication
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: h, usend(A%nproj)
      integer :: numneigh, i, j, idegfd, nsend, icontext, iblock

!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     iblock         Block sequence number
!     icontext       Tag for sending and receiving, is used as check
!     idegfd         Degree of freedom sequence number
!     j              Counting variable
!     nsend          Help variable for receiving
!     numneigh       number of neighbors of a point
!     usend          Help array to be used to send reals to other processor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIRECEIVEMPI  Receive information from other node (mpi)
!     INISENDMPI     Send information to other node (mpi)
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprojAUT' )
      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprojAUT'
         call prinin ( A%projrows, nproj+1, 'A%projrows' )
         call prinin ( A%projcols, A%projrows(nproj+1), 'A%projcols' )
         call prinrl ( A%projU, A%projrows(nproj+1), 'A%projU' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( parallel ) then

!     --- Parallel case
!         y = AU^T x

         numneigh = blockunkinfo(1)
         y = 0
         do j = 1, A%neq
            y(inodenr) = y(inodenr)+A%innercontr(j)*A%projAu(j)*x(j)
         end do  ! j = 1, A%neq

!        --- Now the contributions of the neighboring nodes

         do i = 1, numneigh

            iblock =  blockunkinfo(2*i)

            do j = A%projrowsa(i)+1, A%projrowsa(i+1)

               idegfd = A%projcolsa(j)
               y(iblock) = y(iblock) + &
                         innercontr(idegfd)*A%projAu(A%neq+j)*x(idegfd)

            end do

         end do

!        --- send all contributions to the first node

         icontext = 13

         if ( inodenr==1 ) then

!        --- Receive next contributions from the other nodes and sum up

            do j = 2, A%nproj
               nsend = A%neq
               call inireceivempi ( usend, nsend, SPDOUBLE, iblock, &
                                    icontext )
               do i = 1, nproj
                  y(i) = y(i) + usend(i)
               end do

            end do

         else

!        --- Send contributions to the first node

            call inisendmpi ( y, A%nproj, SPDOUBLE, 1, icontext )

         end if

         if ( debug ) call prinrl ( y, A%nproj, 'y/1' )

      else

!     --- Serial case
!         y = AU^T x

         do i = 1, A%nproj

            h = 0d0

            do j = A%projrows(i)+1, A%projrows(i+1)
               idegfd = A%projcols(j)
               h = h + A%projAU(j)*x(idegfd)
            end do

            y(i) = h

         end do

      end if  ! ( parallel )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprojAUT'

      end if  ! ( debug )
      call erclos ( 'sepprojAUT' )

      end subroutine sepprojAUT
