      subroutine sepsimple ( A, usol, rhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-04-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve system of linear equations of special form using the SIMPLE method, &
!     the LSC method or some other iteartive methods with variable
!     preconditioners
!     All variants are included
!     Old subroutine prsimple
! **********************************************************************
!
!                       KEYWORDS
!
!     simple
!     linear_solver
!     matrix
!     real
!     krylov
!     conjugate_gradients
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesimple
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision :: usol(A%neq)
      double precision, intent(in) :: rhsd(A%neq)

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     rhsd           i    right-hand-side vector
!     usol           o    solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: normrhsd
      double precision, allocatable :: work(:)

!     normrhsd       Norm of right-hand side vector
!     work           Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRWAR         Print warnings
!     SEPSIMPLEDEL   Delete all temporary created arrays in the simple process
!     SEPSIMPLEGCR   Solve a system of linear equations of special form using
!                    the SIMPLE method
!                    The method is accelerated by the GCR process
!     SEPSIMPLEMAT   Create the matrices as well as the preconditioning matrices
!                    with respect to the SIMPLE algorithm
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2446   Right-hand side identical 0
!    2447   Norm of right-hand side is small
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The simple algorithm can be formulated as follows:
!
!     Let the system of equations be:
!
!     | Q    G | |u|  = |f_u|
!     | D    C | |p|    |f_p|
!
!     input:  Q, G, D, C, f_u, f_p
!     output: du, dp
!
!     To solve this system we use a preconditioner that depends on the
!     method used
!     All these preconditioners have in common that they are solved
!     inaccurately. As a consequence the preconditioners must be considered
!     as changing per iteration. Hence we have to use the GCR method
!
!     The systems of equations are computed approximately
!
!     The other versions as well as LSC can be found in lower level subroutines
!     Code
!
!     Fill constants
!     Compute or activate start vector
!     Renumber right-hand side and solution arrays
!     Compute the residual r_u = f_u - Q u - G p
!     Compute the residual r_p = f_p - D u
!     Create the matrices as well as the preconditioning matrices with respect
!     to the SIMPLE algorithm
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsimple' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsimple'
  2      format ( a, 1x, (5d12.4) )

      end if

      allocate(worky1(np), worky2(nu), work(nu+np), stat = error)
      if ( error/=0 ) call eralloc ( error, nrusol+nu+np, 'worksim' )
      if ( ierror/=0 ) go to 1000

      if ( A%irelerr==10 ) A%irelerr = 1

!     --- Compute the residual r_u = f_u - Q u - G p
!         Compute the residual r_p = f_p - D u
!         The arrays dp and du are created and cleared

      work = A*usol
      resup = rhsd(1:nu+np)-work
      normrhsd = norm ( rhsd, A )
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,2) 'normrhsd', normrhsd

      if ( normrhsd<=0d0 ) then

!     --- Right-hand side is identical 0, return

         call errwar ( 2446, 0, 0, 0 )
         go to 500

      else if ( normrhsd<=1d-10 ) then

!     --- Right-hand side is small: give warning

         call erreal ( normrhsd, 1 )
         call errwar ( 2447, 0, 1, 0 )

      end if

!     --- Create the matrices as well as the preconditioning matrices with
!         respect to the SIMPLE algorithm

      if ( debug ) write(irefwr,*) 'before sepsimplemat'
      call sepsimplemat ( A )
      if ( ierror/=0 ) go to 1000

!     --- Perform the GCRsimple algorithm

      if ( debug ) write(irefwr,*) 'before sepsimplegcr'
      call sepsimplegcr ( normrhsd, A, usol )

!     --- Fill the output values into inpsol(2)

      if ( inpsol(22,1)==1 .and. ierror/=0 ) inpsol(2,1) = 1

!     --- Delete temporary created arrays in reversed sequence

500   call sepsimpledel ( A )
      deallocate ( work )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepsimple'

      end if  ! ( debug )
      call erclos ( 'sepsimple' )

      end subroutine sepsimple
