      subroutine sepsolcopsol ( usollc, usolglob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-05-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Send parts of local solution array to neighbor blocks
!     Receive these solution arrays and store in usolglob
! **********************************************************************
!
!                       KEYWORDS
!
!     iterative
!     linear_solver
!     preconditioning
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulemat
      use sepmodulekprob
      use sepmodulesolve
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: usollc(*)
      double precision, intent(inout) :: usolglob(*)

!     usolglob      i/o   Structure of type vector containing the necessary
!                         information of the global solution vector
!                         i.e. the part that is not positioned in this cpu
!                         but is necessary for the solution at this cpu
!     usollc         i    Solution vector (local for this cpu)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iblock, istart, nunks, icontext, i, nblocks
      integer, pointer :: send_buffer(:)
      integer, save :: send_buffer_s

!     i              Counting variable
!     iblock         Block sequence number
!     icontext       Tag for sending and receiving, is used as check
!     istart         First position used at a part
!     nblocks        Number of blocks
!     nunks          Number of unknowns to be send
!     send_buffer    Buffer for sending messages
!     send_buffer_s  Length of array send_buffer
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIBARRIERMPI  Synchronize parallel sending and receiving (mpi)
!     INIBSENDMPI    Send information to other node (mpi buffered)
!     INIBUFATTMPI   Attach mpi buffer if it exists
!     INIBUFDETMPI   Detach mpi buffer if it exists
!     INIBUFSIZEMPI  Extend send_buffer_size with mpi overhead
!     INIRECEIVEMPI  Receive information from other node (mpi)
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsolcopsol' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsolcopsol'
         call prinin ( sendunknowns, iacnodes+1, 'sendunknowns' )
         call prinrl ( usollc, nusol, 'usollc' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Synchronize communication by calling mpi_barrier
!         This avoids that something is written to other nodes, &
!         before reading by those other nodes has been done

      call inibarriermpi

!     --- Compute length of send_buffer

      send_buffer_s = max(1,sendunknowns(iacnodes+1))

!     --- Multiply by 8 (doubles) and add 100 for safety

      send_buffer_s = send_buffer_s*8+100

!     --- Compute actual buffer size using mpi information
!         allocate array and attach buffer

      call inibufsizempi ( send_buffer_s, iacnodes )

      allocate(send_buffer(send_buffer_s))

      call inibufattmpi ( send_buffer, send_buffer_s )

      icontext = 57

!     --- Send solution from usollc to neighboring blocks

      istart = sendunknowns(1)
      do iblock = 1, iacnodes

         nunks = sendunknowns(iblock+1)-istart

         if ( nunks>0 ) then

!        --- Some unknowns must be sent to iblock
!            Store in array sendwork

            if ( debug ) write(irefwr,1) 'block number, nunks', iblock, nunks
            istart = istart+1

            sendwork(1:nunks) = usollc(sendunknowns(istart+1:istart+nunks))
            call inibsendmpi ( sendwork, nunks, SPDOUBLE, iblock, icontext )
            istart = istart+nunks-1

         end if  ! ( nunks>0 )

      end do  ! iblock = 1, iacnodes

      call inibarriermpi

!     --- Receive solution from neighboring blocks
!         Store in usolglob

      nblocks = recunknowns(1)

      if ( debug ) write(irefwr,1) 'nblocks, nglob', nblocks, nglob

      do i = 1, nblocks

         nunks = size(sendwork)
         call inireceivempi ( sendwork, nunks, SPDOUBLE, iblock, icontext )
         istart = recunknowns(iblock+1)

         usolglob(recunknowns(istart+1:istart+nunks)) = sendwork(1:nunks)

      end do  ! i = 1, nblocks

      if (associated(send_buffer)) then

!     --- deattach send_buffer and deallocate

         call inibufdetmpi ( send_buffer, send_buffer_s )
         deallocate(send_buffer)

      end if  ! (associated(send_buffer))

      call inibarriermpi

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( usolglob, nusolglob, 'usolglob' )
         write(irefwr,*) 'End sepsolcopsol'

      end if  ! ( debug )
      call erclos ( 'sepsolcopsol' )

      end subroutine sepsolcopsol
