      subroutine sepsoldir ( matr, isol, irhsdlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-12-2017 New storage of intmat
!        version  1.0  date 21-03-2016
!
!   copyright (c) 2016-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     solve a system of linear equations by
!     gaussian elimination. For a description see the Programmer's Guide 6.8
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     solve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulemain
      use sepmodulesolve
      use sepmodulematr
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: irhsdlc
      integer, intent(inout) :: matr, isol

!     irhsdlc          i    Standard SEPRAN array of length 5 containing
!                         information concerning the right-hand-side vector.
!     isol          i/o   Standard SEPRAN array of length 5 containing
!                         the problem information concerning the solution vector
!     matr          i/o   Standard SEPRAN array of length 5 containing
!                         information concerning the large matrix.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical :: decomp, compl

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     decomp         Logical parameter indicating if the decomposition of
!                    the matrix must be computed (true) or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     PRSOLCHECK     Perform some checks with respect to the input for
!                    linear solvers
!     SEPCREASOL110  Activate or create arrays of type 110
!     SEPFILLKPROB   Fill parameters in module sepmodulekprob
!     SEPFILLMATR    Fill parameters in module sepmodulematr
!     SEPFILLSOLMOD1 Create arrays usol and rhsd
!     SEPFILLSOLVED  Fill parameters in sepmodulesolve that are used by
!                    the direct solver
!     SEPLOCTRNCM    Compute (back-)transformation in case of local
!                    transformations ( complex case )
!     SEPLOCTRNRL    Compute (back-)transformation in case of local
!                    transformations ( real case )
!     SEPRMMATSTR2   nullify arrays in module sepmodulematstr
!     SEPSOLDIAGMAT  solve system of equations amat u = f, where amat is a
!                    diagonal matrix
!     SEPSOLDIRCOMPL solve a system of linear equations by gaussian elimination.
!                    Complex matrix
!     SEPSOLDIRREAL  solve a system of linear equations by gaussian elimination.
!                    Real matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsoldir' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsoldir'
         call ertrace
         write(irefwr,1) 'matr, isol, irhsdlc', matr, isol, irhsdlc
         write(irefwr,1) 'itrans', itrans
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Put parameters in sepmodulesolve

      call sepfillsolved
      if ( ierror/=0 ) go to 1000

!     --- Compute iprob and fill sepmodulekprob

      if ( itrans==2 ) then
         iprob = ks(isol)%iprob
      else
         iprob = ks(irhsdlc)%iprob
      end if  ! ( itrans==2 )
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      kprobh => kprobpart%kprobh
      kprobhinv => kprobpart%kprobhinv
      kprobp => kprobpart%kprobp
      kprobr => kprobpart%kprobr
      indprh = kprobloc(50)
      indprp = kprobloc(35)

      if ( itrans==2 ) then

!     --- ITRANS = 2, only the back transformation is carried out


!        --- Fill sepmodulekprob

         call sepfillkprob
         usol => ks(isol)%sol

         compl = ks(isol)%compl==1

         if ( compl ) then

!        --- Vector is complex  call seploctrncm

            call seploctrncm ( itrans, usol )

         else

!        --- Vector is real  call seploctrnrl

            call seploctrnrl ( itrans, usol )

         end if

      else

!     --- ITRANS = 0 or 1, the system of equations must be solved
!         First check input and compute some relevant parameters
!         ( jmethod, iprob, compl, jcart)

         call prsolcheck ( matr, irhsdlc, compl, jcart )
         if ( ierror/=0 ) go to 1000

!        --- Create solution array

         call sepcreasol110 ( isol, 'solution', ks(irhsdlc)%compl )
         usolbuf => usoltemp

!        --- Fill parameters in sepmodulekprob and so on

         call sepfillkprob
         rhsdbuf => ks(irhsdlc)%sol

         if ( ierror/=0 .or. nrusol==0 ) go to 1000

!        --- Decomposition and solution of system of equations:

         S_ii => realmt(iseqmt)%S_ii

         if ( realmt(iseqmt)%diagonal ) then

!        --- matrix is diagonal
!            Solve system of equations

            call sepsoldiagmat ( S_ii, rhsdbuf, usolbuf )

         else

!        --- ks(matr)%typevec # 110,  Standard matrix
!            Fill parameters in sepmodulematstr and sepmodulematr

            call sepfillmatr ( matr )

            compl = realmt(iseqmt)%complexmat

            call sepfillsolmod1 ( compl )  ! create irhsdlc and usol

            decomp = realmt(iseqmt)%decomp

            if ( compl ) then

!           --- Complex case

               if ( debug ) write(irefwr,*) 'before sepsoldircompl'
               call sepsoldircompl ( decomp )

            else

!           --- Real case

               call sepsoldirreal ( decomp )

            end if  ! ( compl )
            realmt(iseqmt)%decomp = .false.

         end if  ! ( ks(matr)%typevec==110 )

      end if  ! ( itrans==2 )

      call seprmmatstr2  ! remove associations with arrays in sepmodulematstr
      if ( matr>0 ) then
         if ( associated(realmt(iseqmt)%matinv) ) &
            deallocate(realmt(iseqmt)%matinv)
         nullify (realmt(iseqmt)%S_i%MD )
      end if  ! ( matr>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepsoldir'

      end if  ! ( debug )
      call erclos ( 'sepsoldir' )

      end subroutine sepsoldir
