      subroutine sepsolvsm ( n, amat, usol, rhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-12-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     solve system of linear equations by gaussian elimination
!     where the matrix has been decomposed by subroutine prdecosm
!     This implies that the matrix is supposed to be a real symmetric
!     full matrix
!     Old subroutine prsolvsm
! **********************************************************************
!
!                       KEYWORDS
!
!    linear_solver
!    real
!    symmetric
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n
      double precision amat(n,n), usol(n), rhsd(n)

!     amat       i    Matrix in which the LDL^T decomposition has been stored
!     n          i    size of matrix
!     rhsd       i    Right-hand-side vector
!     usol       o    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i       Counting variable
!     j       Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!               T
!      The L D L   decomposition of the matrix A has been computed by
!      subroutine PRDECOSM
!      Now the system of equations:   A x = f must be solved, hence:
!
!                                                    T
!         L D U  x = f   or:     U  x  =  y     U = L
!                                D  y  =  v
!                                L  v  =  f
!
!
!      First:  L v = f
!
!             i                                   i-1
!      f  =  sum   l    v      or:    v   = f   - sum   l   v
!       i    k=1    ik   k             i     i    k=1    ik  k
!
!      Next:  D y = v          or:    y   =  v  / d
!                                      i      i    ii
!
!      Finally  U x = y
!
!             n                                    n
!      y  =  sum   u    x      or:    x   = y   - sum   u   x
!       i    k=i    ik   k             i     i    k=i+1  ik  k
!
!
!      Because of the profile storage it is not easy to get u   so the
!                                                            ik
!      order must be reversed:   x:=y
!
!      i = n (-1) 1
!         j = 1  (1) i-1
!
!             x  = x  - u   x                u   = l
!              j    j    ji  i                ji    ij
!
!
!
!      Hence the final algorithm becomes:
!
!
!      For i := 1 (1) n
!
!                   i-1
!         x  = f  - sum  l   x
!          i    i   k=1   ij  j
!
!      end for i
!
!      For i := 1 (1) n :    x  = x  / d
!                             i    i    ii
!
!      For i := n (-1) 1
!
!         For j := 1  (1) i-1
!
!            x  = x  - l   x
!             j    j    ij  i
!
!         end for j
!
!      end for i
!
!
!      In the case of super elements, it is not possible to store the
!      complete matrix into memory, and therefore each time a part of
!      the matrix is read, the corresponding part of the algorithm carried out
!      and the next part read
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsolvsm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsolvsm'
         call prinrl ( rhsd, n, 'rhsd' )
         call prinrl1 ( amat, n, n, 'amat' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set minimum and maximum for row index and boundaries for matrix
!         to be read  (At most two parts of the matrix may be put into core)
!         Cary out the part L v = f  and then D y = v
!         Loop over rows

      usol = rhsd
      do i = 1, n

         do j = 1, i-1
            usol(i) = usol(i) - amat(i,j)*usol(j)
         end do

      end do
      if ( debug ) call prinrl ( usol, n, 'usol/1' )

!     --- Divide usol by diagonal

      do i = 1, n
         usol(i) = usol(i) / amat(i,i)
      end do
      if ( debug ) call prinrl ( usol, n, 'usol/2' )

!     --- Finally the back substitution
!         Hence
!                         Ux = y

      do i = n, 1, -1
         do j = 1, i-1
            usol(j) = usol(j) - amat(i,j)*usol(i)
         end do
      end do

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( usol, n, 'usol' )
         write(irefwr,*) 'End sepsolvsm'

      end if  ! ( debug )
      call erclos ( 'sepsolvsm' )

      end subroutine sepsolvsm
