      subroutine sepupdaterhsal ( rhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-05-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Update right-hand side of Stokes equation with contribution
!     due to boundary conditions in case of augmented lagrangian method
!     Old subroutine prupdaterhsal
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     right_hand_side
!     real
!     augmented_lagrange
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesimple
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision :: rhsd(nu+np)

!     rhsd           i    right-hand-side vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, allocatable ::  work(:), work1(:)
      logical :: matlab

!                    they are not
!     matlab         If true matlab input is generated
!     work           Work array
!     work1          Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRVEC2MATLAB   Print a sepran vector in a form suitable for reading
!                    in matlab
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     In the augmented Lagrange method the velocity part of the
!     (Navier-)Stokes matrix is updated by gamma G M^-1 L, &
!     with L the divergence matrix M the diagonal of the pressure
!     mass matrix and G the gradient matrix
!     This is allowed since Lu = 0.
!     However, since we may have essential boundary conditions, &
!     Lu does not have to be zero.
!     In fact we have L u = f_p, with f_p the contribution of Diriclet
!     boundary conditions for the velocity.
!     Therefore the right-hand side must be corrected with the term:
!     G M^-1 f_p
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepupdaterhsal' )
      debug = .false.
      matlab = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepupdaterhsal'
         write(irefwr,2) 'gamma', algamma
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( matlab ) call prvec2matlab ( nu+np, rhsd, 'rold.dat' )

      allocate(work(np), work1(nu), stat = error)
      if ( error/=0 ) call eralloc ( error, np+nu, 'work' )
      if ( ierror/=0 ) go to 1000

!     --- First we invert the pressure mass matrix
!         It is inverted back at the end of pral

      S_PMD = 1d0/S_PMD

!     --- Compute gamma G M^-1 f_p
!         First step copy f_p to work

      work = rhsd(nu+1:nu+np)
      if ( debug ) call prinrl ( work, np, 'f_p' )

!     --- Next step M^-1 f_p; input: work, result work

      work = work*S_PMD
      if ( debug ) call prinrl ( work, np, 'M-1 f_p' )

!     --- Next y := G M-1 Du; input: work, result work1

      work1 = G*work
      if ( debug ) call prinrl ( work1, nu, 'G M-1 f_p' )

!     --- Finally, add gamma work1 to rhsd and store in rhsd

      rhsd(1:nu) = rhsd(1:nu) + algamma * work1
      if ( matlab ) call prvec2matlab ( nu+np, rhsd, 'rnew.dat' )

1000  deallocate ( work, work1, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'work' )

      call erclos ( 'sepupdaterhsal' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( rhsd, nu+np, 'rhsd_updated' )
         write(irefwr,*) 'End sepupdaterhsal'

      end if  ! ( debug )

      end
