      subroutine to0099 ( ivec1, ivec2, iconst, idegfd, iskip, indelgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.4  date 05-12-2012 Long integers
!        version  3.3  date 26-01-2011 New call to to0082
!        version  3.2  date 12-09-2008 Extra debug
!
!   copyright (c) 1995-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    TO0099 is a vector manipulation subroutine
!    It maps vector of one structure onto a vector of another structure
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivec1, ivec2, iconst(*), idegfd(*), iskip, indelgrp(*)

!     iconst        i/o   In- and output integer array containing constants for
!                         the manipulation depending on ichois
!     idegfd         i    Array with degrees of freedom.
!                         Only the first one is used.
!                         It indicates that the mapping must be stored from the
!                         idegfd-th unknown per point in the array vecout
!     indelgrp       i    If indelgrp(i) = 1, then element group i must be
!                         skipped, otherwise it must be taken into account
!     iskip          i    Indicates if element groups must be skipped (>0)
!                         or not (0)
!     ivec1          i    First input vector to be manipulated
!                         ivec1 must be a SEPRAN array of length 5, which may be
!                         of type 110, 115, 116 or 119
!     ivec2          o    Output vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, itypvc, ivec, jdegfd, inunkp, idummy
      integer, allocatable :: iskippoints(:)
      logical compli

!     compli         Indication if IVEC1 corresponds to a complex array (true)
!                    or to a real one (false)
!     i              Counting variable
!     idummy         Dummy parameter
!     inunkp         Number of degrees of freedom in point
!     iskippoints    If iskippoints(i)=0, node i must be skipped, else
!                    iskippoints(i)>0, node i must not be skipped
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     jdegfd         First degree of freedom in output vector to which
!                    the vector must be copied
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSKIPELGRP    Fill array ioutpoints for all points that are not lying in
!                    element groups that must be skipped
!     PRININ         print 1d integer array
!     SEPACTSOLBF2   compute pointers to degrees of freedom of vectors
!     SEPCOPYDF      copies the contents of one degree of freedom in array ivec1
!                    into another degree of freedom in array ivec2
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     TO0082         Map one vector onto another (real case)
!     TO0087         Map one vector onto another (complex case)
!     TOMAPTOVEC116  Map vector defined per node onto a vector defined per
!                    element
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     386   complex quantities are not permitted in this subroutine
!     699   Problem number incorrect
!    1742   The mapping of the input array to the output array is not possible
!           because the number of degrees of freedom per point, or the total
!           number of degrees of freedom do not match.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0099' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0099'
         call prinin ( ivec1, 5, 'ivec1' )
         call prinin ( ivec2, 5, 'ivec2' )
         call prinin ( iconst, 2, 'iconst' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check type of ivec1

      if ( ks(ivec1)%compl==0 ) then
         compli = .false.
      else
         compli = .true.
      end if

!     --- Reserve space

      itypvc = ks(ivec1)%typevec
      call sepactsolbf2 ( ivec1 )
      ivec  = iconst(1)
      iprob = iconst(2)
      if ( ivec==0 ) then
         itypvc = 110
      else if ( ivec<0 ) then
         itypvc = 116
         ivec = -ivec
      else
         itypvc = 115
      end if
      if ( iprob<1 .or. iprob>nprob )  then
         call errint ( iprob, 1 )
         call errint ( nprob, 2 )
         call errsub ( 699, 2, 0, 0)
      end if

!     --- First reserve space for the output vector and activate the
!         corresponding arrays KPROBF and KPROBP

      call sepcreasolbf1 ( ivec2, itypvc, ivec, 'vector', &
                           ks(ivec1)%compl )
      vectout => usoltemp
      if ( indprfi==0 .and. indprfo==0 .and. nphysi/=nphyso ) then
         call errint ( nphysi, 1 )
         call errint ( nphyso, 2 )
         call errint ( ks(ivec1)%nusol, 3 )
         call errint ( ks(ivec2)%nusol, 4 )
         call errsub ( 1742, 4, 0, 0 )
      end if

      if ( debug ) call prinin ( ivec2, 5, 'ivec2' )

!     --- Carry out the actual mapping

      vectin => ks(ivec1)%sol

      if ( iskip>0 ) then

!     --- iskip>0, create and fill array iskippoints

         allocate ( iskippoints(npoint), stat = error )
         if ( error/=0 ) call eralloc ( error, npoint, 'iskippoints' )
         if ( ierror/=0 ) go to 1000

         call erskipelgrp ( indelgrp, iskippoints )

      end if  ! ( iskip>0 )

      if ( compli ) then

!     --- Complex case

         if ( ks(ivec2)%nusol/=ks(ivec1)%nusol ) then

!        --- New vector larger than old one
!            clear this vector first

            vectout(1:ks(ivec2)%nusol*2) = 0d0

         end if
         if ( itypvc==116 ) then

!        --- itypvc = 116, array defined per element
!            Not yet implemented for type 116

            call errsub ( 386, 0, 0, 0 )
            go to 1000

         else

!        --- itypvc = 110 or 115, standard arrays

            call to0087 ( vectin, vectout, indprfi, &
                          kprobfi, indprfo, kprobfo, npoint, nphysi )

         end if  ! ( ivec==116 )

      else

!     --- Real case

         if ( ks(ivec2)%nusol/=ks(ivec1)%nusol ) then

!        --- New vector larger than old one
!            clear this vector first

            vectout(1:ks(ivec2)%nusol) = 0d0

         end if

         if ( itypvc==116 ) then

!        --- itypvc = 116, array defined per element

            call tomaptovec116 ( vectin, vectout, nphysi, &
                                 ivec, idegfd(1), indprfi )

         else

!        --- itypvc = 110 or 115, standard arrays

            if ( indprfo/=0 .and. indprfi==0 .or. &
                 indprfo==0 .and. indprfi/=0 ) then

!           --- Both vectors have different structure
!               Copy component-wise

               jdegfd = max(1,idegfd(1))           ! first degree of freedom
               inunkp = min(nphysi,nphyso-jdegfd+1) ! number to be copied
               do i = jdegfd, inunkp+jdegfd-1
                  call sepcopydf ( ivec1, ivec2, kprob, &
                                   kprob, i+1-jdegfd, i )
               end do  ! i = jdegfd, inunkp+jdegfd-1

            else if ( iskip>0 ) then

!           --- Both vectors have the same structure; iksip>0

               call to0082 ( vectin, vectout,&
                             idegfd(1), iskip, iskippoints )

            else

!           --- Both vectors have the same structure; iksip==0

               call to0082 ( vectin, vectout, &
                             idegfd(1), iskip, idummy )

            end if  ! ( indfpo/=0 .and. indfpi==0 ... )

         end if  ! ( ivec==116 )

      end if

      if ( iskip>0 ) then

!     --- iskip>0, deallocate iskippoints

         deallocate ( iskippoints, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iskippoints' )

      end if  ! ( iskip>0 ) then

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End to0099'

      end if  ! ( debug )
      call erclos ( 'to0099' )

      end subroutine to0099
