      subroutine tolincomb ( vec1, vec2, vec3, a, b, c, nusol, kprobh, &
                             indprh, vector2, ichois )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-03-2013 Correction declaration
!        version  1.0  date 17-05-2008
!
!   copyright (c) 2008-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store linear combination a vec1 + b vec2 into vec3
!     if vector2 is false a vec1 is computed, except when b #0 then
!     vec3 = a vec1 + b
! **********************************************************************
!
!                       KEYWORDS
!
!     algebraic_manipulation
!     manipulation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nusol, kprobh(*), indprh, ichois
      double precision vec1(*), vec2(*), vec3(*), a, b, c
      logical vector2

!     a              i    Multiplication factor for vec1
!     b              i    Multiplication factor for vec2
!     c              i    if ichois = 24, maximum value of component
!     ichois         i    Choice parameter, see to0035
!     indprh         i    indication if kprob part h has been filled (>0)
!                         or not (0)
!     kprobh         i    KPROB part h: contains the new numbering of the
!                         unknowns
!     nusol          i    Total number of unknowns
!     vec1           i    first input vector
!     vec2           i    second input vector
!     vec3           o    ?
!     vector2        i    If true the second vector is not 0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ichoicea, ichoiceb, i, j

!     i              Counting variable
!     ichoicea       Indicates value of a
!                     0:  a = arbitrary
!                     1:  a = 1
!                    -1:  a = -1
!     ichoiceb       Indicates value of b
!                     0:  b = 0
!                     1:  b = 1
!                    -1:  b = -1
!                     2:  b = arbitrary
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tolincomb' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tolincomb'
         write(irefwr,1) 'indprh, ichois, nusol', indprh, ichois, nusol
         write(irefwr,2) 'a, b, c', a, b, c
         write(irefwr,*) 'vector2 ', vector2
         call prinrl ( vec1, 10, 'vec1' )
         if ( vector2 ) call prinrl ( vec2, 10, 'vec2' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Determine range of a

      ichoicea = 0       ! a = arbitrary
      if ( abs(a-1d0)<=epsmac ) then
         ichoicea = 1       ! a = 1
      else if ( abs(a+1d0)<=epsmac ) then
         ichoicea = -1      ! a = -1
      end if  ! ( abs(a-1d0)<=epsmac )

!     --- Determine range of b

      ichoiceb = 2       ! b = arbitrary
      if ( abs(b)<=0d0 ) then
         ichoiceb = 0       ! b = 0
      else if ( abs(b-1d0)<=epsmac ) then
         ichoiceb = 1       ! b = 1
      else if ( abs(b+1d0)<=epsmac ) then
         ichoiceb = -1      ! b = -1
      end if  ! ( abs(b)<=0d0 )

      if ( debug ) &
         write(irefwr,1) 'ichoicea, ichoiceb', ichoicea, ichoiceb

      if ( indprh==0 ) then

!     --- No renumbering

         if ( vector2 .and. ichoiceb/=0 ) then

!        --- vector2 exists

            if ( ichoicea==1 .and. ichoiceb==1 ) then

!           --- a=1 and b=1

               vec3(1:nusol) = vec1(1:nusol)+vec2(1:nusol)

            else if ( ichoicea==-1 .and. ichoiceb==1 ) then

!           --- a=-1 and b=1

               vec3(1:nusol) = -vec1(1:nusol)+vec2(1:nusol)

            else if ( ichoicea==1 .and. ichoiceb==-1 ) then

!           --- a=1 and b=-1

               vec3(1:nusol) = vec1(1:nusol)-vec2(1:nusol)

            else

!           --- general case

               vec3(1:nusol) = a*vec1(1:nusol)+b*vec2(1:nusol)

            end if  ! ( ichoicea==1 .and. ichoiceb==1 )

         else if ( ichoiceb==0 ) then

!        --- vector2 does not exist

            if ( ichoicea==1 ) then

!           --- a=1

               vec3(1:nusol) = vec1(1:nusol)

            else if ( ichoicea==-1 ) then

!           --- a=-1

               vec3(1:nusol) = -vec1(1:nusol)

            else

!           --- general case

               vec3(1:nusol) = a*vec1(1:nusol)

            end if  ! ( ichoicea==1 )

         else

!        --- vector2 does not exist, but a constant must be added

            if ( ichoicea==1 ) then

!           --- a=1

               vec3(1:nusol) = vec1(1:nusol) + b

            else if ( ichoicea==-1 ) then

!           --- a=-1

               vec3(1:nusol) = -vec1(1:nusol) + b

            else

!           --- general case

               if ( debug ) write(irefwr,*) 'a*vec1+b'
               vec3(1:nusol) = a*vec1(1:nusol) + b

            end if  ! ( ichoicea==1 )

         end if  ! ( vector2 )

         if ( ichois==24 ) then

!        --- ichois = 24, fullfil constraint

            do i = 1, nusol
               if ( vec3(i)<c ) vec3(i) = c
            end do

         end if

      else

!     --- Renumbering required

         if ( vector2 .and. ichoiceb/=0 ) then

!        --- vector2 exists

            if ( ichoicea==1 .and. ichoiceb==1 ) then

!           --- a=1 and b=1

               do i = 1, nusol
                  j = kprobh(i)
                  vec3(j) = vec1(j)+vec2(j)
               end do

            else if ( ichoicea==-1 .and. ichoiceb==1 ) then

!           --- a=-1 and b=1

               do i = 1, nusol
                  j = kprobh(i)
                  vec3(j) = -vec1(j)+vec2(j)
               end do

            else if ( ichoicea==1 .and. ichoiceb==-1 ) then

!           --- a=1 and b=-1

               do i = 1, nusol
                  j = kprobh(i)
                  vec3(j) = vec1(j)-vec2(j)
               end do

            else

!           --- general case

               do i = 1, nusol
                  j = kprobh(i)
                  vec3(j) = a*vec1(j)+b*vec2(j)
               end do

            end if  ! ( ichoicea==1 .and. ichoiceb==1 )

         else if ( ichoiceb==0 ) then

!        --- vector2 does not exist

            if ( ichoicea==1 ) then

!           --- a=1

               do i = 1, nusol
                  j = kprobh(i)
                  vec3(j) = vec1(j)
               end do

            else if ( ichoicea==-1 ) then

!           --- a=-1

               do i = 1, nusol
                  j = kprobh(i)
                  vec3(j) = -vec1(j)
               end do

            else

!           --- general case

               do i = 1, nusol
                  j = kprobh(i)
                  vec3(j) = a*vec1(j)
               end do

            end if  ! ( ichoicea==1 )

         else

!        --- vector2 does not exist, but a constant must be added

            if ( ichoicea==1 ) then

!           --- a=1

               do i = 1, nusol
                  j = kprobh(i)
                  vec3(j) = vec1(j)+b
               end do

            else if ( ichoicea==-1 ) then

!           --- a=-1

               do i = 1, nusol
                  j = kprobh(i)
                  vec3(j) = -vec1(j)+b
               end do

            else

!           --- general case

               do i = 1, nusol
                  j = kprobh(i)
                  vec3(j) = a*vec1(j)+b
               end do

            end if  ! ( ichoicea==1 )

         end if  ! ( vector2 )

         if ( ichois==24 ) then

!        --- ichois = 24, fullfil constraint

            do i = 1, nusol
               j = kprobh(i)
               if ( vec3(j)<c ) vec3(j) = c
            end do

         end if

      end if  ! ( indprh==0 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( vec3, nusol, 'vec3' )
         write(irefwr,*) 'End tolincomb'

      end if  ! ( debug )
      call erclos ( 'tolincomb' )

      end
