      subroutine tolincombinat1 ( vec1, vec2, vec3, indprf1, indprf2, &
                                  indprf3, kprobf1, kprobf2, kprobf3, &
                                  npoint, idegfd, const1, const2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-12-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute linear combination of 2 vectors and store in third vector
!     The vectors may have different length
!     vec3 = const1 vec1 + const2 vec2
! **********************************************************************
!
!                       KEYWORDS
!
!     algebraic_manipulation
!     manipulation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer indprf1, indprf2, indprf3, npoint, idegfd, &
              kprobf1(*), kprobf2(*), kprobf3(*)
      double precision vec1(*), vec2(*), vec3(*), const1, const2

!     const1         i    multiplication constant for vec1
!     const2         i    multiplication constant for vec2
!     idegfd         i    Degree of freedom sequence number
!                         If 0 all degrees of freedom
!     indprf1        i    indication if kprob part f has been filled (>0)
!                         or not (0) for vec1
!     indprf2        i    indication if kprob part f has been filled (>0)
!                         or not (0) for vec2
!     indprf3        i    indication if kprob part f has been filled (>0)
!                         or not (0) for vec3
!     kprobf1        i    kprob part f corresponding to vec1
!     kprobf2        i    kprob part f corresponding to vec2
!     kprobf3        i    kprob part f corresponding to vec3
!     npoint         i    Number of nodal points
!     vec1           i    first input vector
!     vec2           i    second input vector
!     vec3           o    result vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart1, istart2, istart3, nunkp, nunkp1, nunkp2, nunkp3, &
              node

!     istart1        Last position used in vec1
!     istart2        Last position used in vec2
!     istart3        Last position used in vec3
!     node           Node number
!     nunkp          Maximum number of unknowns in a point
!     nunkp1         number of unknowns in point in vec1
!     nunkp2         number of unknowns in point in vec2
!     nunkp3         number of unknowns in point in vec3
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tolincombinat1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tolincombinat1'
         write(irefwr,1) 'idegfd, npoint, indprf1, indprf2, indprf3', &
                          idegfd, npoint, indprf1, indprf2, indprf3
         write(irefwr,2) 'const1, const2', const1, const2
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )
         nunkp1 = 0
         nunkp2 = 0
         nunkp3 = 0

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Loop over nodes

      istart1 = 0  ! last position used in vec1
      istart2 = 0  ! last position used in vec2
      istart3 = 0  ! last position used in vec3

      if ( indprf1<0 ) nunkp1 = -indprf1
      if ( indprf2<0 ) nunkp2 = -indprf2
      if ( indprf3<0 ) nunkp3 = -indprf3

      if ( debug ) &
         write(irefwr,1) 'nunkp1, nunkp2, nunkp3', &
                          nunkp1, nunkp2, nunkp3

      do node = 1, npoint

         if ( indprf1>0 ) nunkp1 = kprobf1(node+1)-istart1
         if ( indprf2>0 ) nunkp2 = kprobf2(node+1)-istart2
         if ( indprf3>0 ) nunkp3 = kprobf3(node+1)-istart3

         nunkp = min(nunkp1,nunkp2,nunkp3)
         if ( nunkp>idegfd-1 ) then

!        --- There are unknowns to be filled

            if ( idegfd==0 ) then

!           --- idegfd = 0, all degrees of freedom must be filled

               vec3(istart3+1:istart3+nunkp) = &
                  const1*vec1(istart1+1:istart1+nunkp) + &
                  const2*vec2(istart2+1:istart2+nunkp)

            else

!           --- idegfd>0, one degree of freedom must be filled

               vec3(istart3+idegfd) = const1*vec1(istart1+idegfd) + &
                                      const2*vec2(istart2+idegfd)

            end if  ! ( idegfd==0 )

         end if  ! ( nunkp>0 )

         istart1 = istart1+nunkp1  ! raise istart1
         istart2 = istart2+nunkp2  ! raise istart2
         istart3 = istart3+nunkp3  ! raise istart3

      end do  ! node = 1, npoint

1000  call erclos ( 'tolincombinat1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End tolincombinat1'

      end if  ! ( debug )

      end
