      subroutine tomanvec48 ( ivec1, ivec2, ivecout, iconst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 05-12-2012 Long integers
!        version  1.0  date 21-09-2002
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Multiply the idegfd-th component of vector ivec1 by the idegfd-th
!     component of vector ivec1 and the unit vector in idegfd direction
!     The output vector has the same structure as ivec1 and ivec2 must be
!     defined in at least the same points as ivec1
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
!     2d
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivec1, ivec2, iconst(*), ivecout

!     iconst         i    Contains some constants required to compute the vector
!                         1:  idegfd
!     ivec1          i    input vector to be manipulated
!                         ivec1 must be a SEPRAN array of length 5, which may be
!                         of type 110, 115, 116 or 119
!     ivec2          i    Second input vector to be manipulated
!     ivecout       i/o   Output vector, may be the same as input vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nphys2, ivec, idegfd, indprf2
      integer, pointer :: kprobf2(:)

!     idegfd         Degree of freedom sequence number
!     indprf2        indication if kprob2 part f has been filled (>0)
!                    or not (0)
!     ivec           Sequence number of vector of special structure
!     kprobf2        kprobf for second vector
!     nphys2         Number of physical unknowns, or maximal degrees of
!                    freedom per point in second mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPACTSOLBF2   compute pointers to degrees of freedom of vectors
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     TOMANVEC48_1   Multiply the idegfd-th component of vector ivec1 by the
!                    idegfd-th component of vector ivec1 and the unit vector
!                    in idegfd direction
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     218   Degree of freedom does not exist
!     222   input vector is complex:  not yet allowed.
!     419   NDIM incorrect
!    2276   nphys in output vector<ndim
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check type of ivec1
!     Activate input vector and corresponding arrays
!     Perform some checks
!     Activate or create output vector and corresponding arrays
!     Compute normals on surface isurnr (array defined in all points)
!     Clear output vector
!     Perform the actual action on the solution vector
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tomanvec48' )

!     --- Check type of ivec1

      if ( ks(ivec1)%compl/=0 ) then
         call errsub ( 222, 0, 0, 0 )
         go to 1000
      end if

!     --- Activate input vectors and corresponding arrays

      call sepactsolbf2 ( ivec2 )
      indprf2 = indprfi
      nphys2 = nphysi
      kprobf2 => kprobfi

      call sepactsolbf2 ( ivec1 )
      if ( nphysi<ndim ) then

!     --- Number of degrees of freedom incorrect

         call errint ( nphysi, 1 )
         call errint ( ndim, 2 )
         call errsub ( 2276, 2, 0, 0 )

      end if
      if ( ndim/=3 ) then

!     --- ndim # 3

         call errint ( ndim, 1 )
         call errint ( 3, 2 )
         call errint ( 3, 3 )
         call errsub ( 419, 3, 0, 0 )

      end if
      if ( nphys2<ndim ) then

!     --- Number of degrees of freedom incorrect

         call errint ( nphys2, 1 )
         call errint ( ndim, 2 )
         call errsub ( 2276, 2, 0, 0 )

      end if

!     --- Activate or create output vector and corresponding arrays

      iprob = ks(ivec1)%iprob
      ivec = ks(ivec1)%ivec
      call sepcreasolbf1 ( ivecout, ks(ivec1)%typevec, ivec, 'vector', &
                           ks(ivec1)%compl )
      vectout => usoltemp

      idegfd = iconst(1)
      if ( idegfd<1 .or. idegfd>ndim ) then

!     --- degree of freedom out of range

         call errint ( idegfd, 1 )
         call errint ( ndim, 2 )
         call errsub ( 218, 2, 0, 0 )

      end if

!     --- Compute starting addresses

      vectin => ks(ivec1)%sol
      vectin2 => ks(ivec2)%sol
      if ( ivec1==ivec2 ) then
         vectout => vectin
         nphyso = nphysi
         indprfo = indprfi
         kprobfo => kprobfi
      end if
      if ( ierror/=0 ) go to 1000

!     --- Clear output vector

      vectout(1:size(vectin)) = 0d0

!     --- Perform the actual action on the solution vector

      call tomanvec48_1 ( vectin, vectin2, vectout, indprfo, kprobfo, &
                          nphyso, npoint, idegfd, indprf2, &
                          kprobf2, nphys2 )

1000  call erclos ( 'tomanvec48' )
      end subroutine tomanvec48
