      subroutine erstressvec ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 30-12-2013 New storage of information
!        version  1.1  date 11-12-2012 Long integers
!        version  1.0  date 03-05-2010
!
!   copyright (c) 2010-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the normal stress vector along a set of curves and store
!     in a solution vector
!     All components outside the set of curves are set to 0
! **********************************************************************
!
!                       KEYWORDS
!
!     stress
!     normal_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: inpsep(*)

!     inpsep         i    integer input array, &
!                         Contents See subroutine askvec
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqstress, iseqnormstress, itypvc, ivec, &
              istart, ncurvslc

!     ipnormstress   Starting address of array normstress
!     ipstress       Starting address of array stress
!     iseqnormstress Sequence number of array normstressin isol
!     iseqstress     Sequence number of array stress in isol
!     istart         Starting address in inpsep with information of curves
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     ncurvslc       Number of curves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOMPVECTOR2  Store inner product of normal and tensor in all nodes of a
!                    set of curves into solution vector defined per node
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPCREASTRUC   find structure of vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1059   number of degrees of freedom incorrect
!    1653   Number of unknowns not constant in node
!    3049   Stress tensor and vector have the same sequence number
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erstressvec' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erstressvec'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      iseqstress = inpsep(9)
      iseqnormstress = inpsep(3)
      if ( inpsep(15)>0 ) then
         istart = inpsep(15)
         ncurvslc = inpsep(istart)
         istart = istart+1
      else
         istart = 1
         ncurvslc = 0
      end if  ! ( inpsep(15)>0 )
      if ( ierror/=0 ) go to 1000
      if ( debug ) &
         write(irefwr,1) 'iseqstress, iseqnormstress, ndim, ncurvslc', &
                          iseqstress, iseqnormstress, ndim, ncurvslc

      if ( iseqstress==iseqnormstress ) then

!     --- Error: both stress and stress vector must be stored in the
!         same vector

         call errint ( iseqstress, 1 )
         call errint ( iseqnormstress, 2 )
         call errchr ( names(iseqstress), 1 )
         call errchr ( names(iseqnormstress), 2 )
         call errsub ( 3049, 2, 0, 2 )
         go to 1000

      end if  ! ( iseqstress==iseqnormstress )

!     --- solution vector and corresponding arrays

      call sepactsolbf2 ( isolut2d(iseqstress) )
      if ( ierror/=0 ) go to 1000
      iprob  = ks(isolut2d(iseqstress))%iprob
      if ( nphysi/=6 ) then

!     --- nphysi # 6

         call errint ( nphysi, 1 )
         call errint ( 6, 2 )
         call errchr ( names(iseqstress), 1 )
         call errsub ( 1059, 2, 0, 1)

      end if  ! ( nphysi/=6 )

      if ( indprfi/=0 ) then

!     --- indprfi # 0, not implemented

         call errsub ( 1653, 0, 0, 0 )

      end if  ! ( indprfi/=0 )
      if ( ierror/=0 ) go to 1000

!     --- Find structure of vector with exactly ndim parameters per node

      itypvc = 110
      call sepcreastruc ( ndim, itypvc, ivec )
      if ( ierror/=0 ) go to 1000

!     --- Create normal stress vector

      call sepcreasolbf1 ( isolut2d(iseqnormstress), itypvc, ivec, &
                           names(iseqnormstress), 0 )
      if ( ierror/=0 ) go to 1000

!     --- Clear normal stress array

      usoltemp => ks(isolut2d(iseqnormstress))%sol
      call sep2dvec ( usoltemp, 2, size(usoltemp)/2 )

      normalstress => usoltemp2d
      normalstress = 0d0

      usoltemp => ks(isolut2d(iseqstress))%sol
      call sep2dvec ( usoltemp, 6, size(usoltemp)/6 )
      stress => usoltemp2d


!     --- Compute the inner product of stress tensor and normals and store in
!         usol

      call ercompvector2 ( ncurvslc, inpsep(istart) )
      if ( ierror/=0 ) go to 1000

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erstressvec'

      end if  ! ( debug )
      call erclos ( 'erstressvec' )

      end subroutine erstressvec
