      subroutine prcompcapacity ( iinputcap )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 11-12-2012 Long integers
!        version  1.1  date 30-08-2006 New definition of cpost5
!        version  1.0  date 18-10-1999
!
!   copyright (c) 1999-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       Compute the capacity of all electrodes and store in array c
!
! **********************************************************************
!
!                       KEYWORDS
!
!     capacity
!     electrode
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduletime
      use sepmodulevecs
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinputcap(*)

!     iinputcap      i    Integer input array for subroutine compcapacity
!                         Contents
!                         1: Largest entry that has been filled by the user
!                         2: iseqsl:  sequence number of input for linear
!                                     solver
!                         3: iseqcf:  sequence number of input for coefficients
!                         4: iveccapacity: sequence number of the capacity
!                                          vector in array isolut
!                         5: ibegincv: curve number of first electrode
!                         6: iendcv : curve number of last electrode
!                         7: ivecsol: sequence number of the solution
!                                     vector in array isolut
!                         8: ivecreact: sequence number of the reaction
!                                       vector in array isolut
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  niinsol, niintbn
      parameter ( niinsol=13, niintbn=7 )
      integer iinsol(niinsol), iintbn(niintbn), icurvsloc(1), i, &
              ibegincv, iendcv, icount, icurnr, &
              iseqcf, iseqsl, iveccapacity, ivecreact, ivecsol, j, &
              iread, jcurnr, jdimpr, nelectrodes, itimloopsave
      double precision rintbn(1), valint

!     i              Counting variable
!     ibegincv       First curve number
!     icount         Counter
!     icurnr         Curve sequence number
!     icurvsloc      Local array icurvs
!     iendcv         Last curve number
!     iinsol         Contains information concerning the linear solver
!                    See the Programmers Guide for a description
!     iintbn         Contains information for the subroutine to compute the
!                    boundary integrals
!                    See the Programmers Guide for a description
!     iread          Defines how the input must be read, see intbnd
!     iseqcf         Sequence number corresponding to the coefficients
!     iseqsl         Sequence number corresponding to the linear solver
!     itimloopsave   Help variable to store the value of itimloop
!     iveccapacity   Sequence number of capacity vector in isolut
!     ivecreact      Sequence number of reaction vector in isolut
!     ivecsol        Sequence number of solution vector in isolut
!     j              Counting variable
!     jcurnr         |icurnr| i.e. absolute value of curve sequence number
!     jdimpr         Parameter for bvalue
!     nelectrodes    Number of electrodes
!     niinsol        Length of array iinsol
!     niintbn        Length of array iintbn
!     rintbn         Double precision input array for intbnd
!     valint         Value of the integral
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     LINPRBNEW      Build system of equations and solves linear system of
!                    equations
!     SEPBVALUE      Fill boundary conditions into solution array
!     SEPCREASOLVEC  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPINTBND      compute an integral along a boundary
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2128   array isol has not been filled, or the boundary conditions have
!           not been stored in the solution vector
!    2133   Sequence number of vector too large
! **********************************************************************
!
!                       PSEUDO CODE
!
!     N := number of electrodes
!     icount := 0
!     For i := 1 (1) N-1 do
!        icurnr := ibegincv+i-1
!        Set all electrodes at potential 0
!        Set Electrode icurnr at potential 1
!        Solve linear problem
!        For j := i+1 (1) N do
!           Compute capacity c_ij
!           icount := icount+1
!           c(icount) := c_ij
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcompcapacity' )

!     --- Perform some initializations

      iseqsl = 1
      iseqcf = 1
      ibegincv = 1
      iendcv   = 2
      iveccapacity = 3
      ivecsol = 1
      ivecreact = 2
      if ( iinputcap(1)>1 ) iseqsl = iinputcap(2)
      if ( iinputcap(1)>2 ) iseqcf = iinputcap(3)
      if ( iinputcap(1)>3 ) iveccapacity = iinputcap(4)
      if ( iinputcap(1)>4 ) ibegincv = iinputcap(5)
      if ( iinputcap(1)>5 ) iendcv = iinputcap(6)
      if ( iinputcap(1)>6 ) ivecsol = iinputcap(7)
      if ( iinputcap(1)>7 ) ivecreact = iinputcap(8)
      if ( iveccapacity>maxvc1 ) then

!     --- error: iveccapacity too large

         call errint ( iveccapacity, 1 )
         call errint ( maxvc1, 2 )
         call errchr ( 'capacity', 1 )
         call errsub ( 2133, 2, 0, 1 )

      end if
      if ( ivecsol>maxvc1 ) then

!     --- error: ivecsol too large

         call errint ( ivecsol, 1 )
         call errint ( maxvc1, 2 )
         call errchr ( 'solution', 1 )
         call errsub ( 2133, 2, 0, 1 )

      end if
      if ( ivecreact>maxvc1 ) then

!     --- error: ivecreact too large

         call errint ( ivecreact, 1 )
         call errint ( maxvc1, 2 )
         call errchr ( 'reaction vector', 1 )
         call errsub ( 2133, 2, 0, 1 )

      end if

!     --- number of electrodes

      nelectrodes = iendcv-ibegincv+1
      iprob = ks(isolut2d(ivecsol))%iprob
      icount = 1

!     --- array iinsol

      iinsol(1) = 13
      iinsol(2) = 0
      iinsol(3) = iseqcf
      iinsol(4) = iseqsl
      iinsol(5) = iprob
      iinsol(6) = ivecsol
      iinsol(7) = numvecmn
      iinsol(8) = 0
      iinsol(9) = 0
      iinsol(10) = ivecreact
      iinsol(11) = 0
      iinsol(12) = 0

!     --- array iinsol

      iintbn(1) = 7
      iintbn(2) = 8
      iintbn(3) = 0
      iintbn(4) = 1
      iintbn(5) = 1
      iintbn(6) = 1
      iintbn(7) = 1

!     --- Perform checks

      if ( ks(isolut2d(ivecsol))%typevec/=110 ) then

!     --- No boundary conditions in solution vector

         call errint ( ivecsol, 1 )
         call errint ( ks(isolut2d(ivecsol))%typevec, 2 )
         call errchr ( names(ivecsol), 1 )
         call errsub ( 2128, 2, 0, 1 )

      end if

      nphyso = 1
      call sepcreasolbf1 ( isolut2d(iveccapacity), 801, nelectrodes, &
                           'capacity', 0 )
      capacity => usoltemp
      itimloopsave = itimloop

!     --- Loop over the electrodes

      do i = 1, nelectrodes-1
         icurnr = ibegincv+i-1

!        --- Set Electrode icurnr at potential 1

         jdimpr = 2+1000*(iprob-1)
         valint = 1d0
         call sepbvalue ( 0, jdimpr, isolut2d(ivecsol), &
                          valint, icurnr, icurnr, 1, 0 )

!        --- Solve linear problem

         if ( i==1 ) then
            iinsol(13) = 1
         else if ( i<nelectrodes-1 ) then
            iinsol(13) = 3
         else
            iinsol(13) = 2
         end if

         itimloop = max(i,itimloop)
         intmat => intmat1d(iprob)
         call linprbnew ( iinsol )

!        --- Loop over all electrodes larger than i

         do j = i+1, nelectrodes

            jcurnr = ibegincv+j-1

!           --- Compute capacity c_ij

            icurvsloc(1) = jcurnr
            iread = -1
            call sepintbnd ( iintbn, rintbn, &
                             isolut2d(ivecreact), icurvsloc, valint, iread )

!           --- c(icount) := c_ij

            capacity(icount) = -valint
            icount = icount+1

         end do

!        --- Reset boundary value to 0

         valint = 0d0
         call sepbvalue ( 0, jdimpr, isolut2d(ivecsol), &
                          valint, icurnr, icurnr, 1, 0 )

      end do
      itimloop = itimloopsave
      numvecmn = max(numvecmn,iveccapacity,ivecsol,ivecreact)

      call erclos ( 'prcompcapacity' )
      end
