      subroutine preccopy ( nrusol, imata, imatpre, amat, &
                            premat, icola, icolpre )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-10-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy matrix amat into matrix premat, where premat may have
!     a different structure.
!     Both matrices are stored as compact matrix
!     If an entry of amat is not present in premat this entry is skipped
!     This concerns the lower and upper part of the matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     matrix
!     preconditioning
!     compact
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nrusol, imata(*), imatpre(*), icola(*), icolpre(*)
      double precision amat(*), premat(*)

!     amat           i     the large matrix
!     icola          i    Integer array containing the column numbers of the
!                         non-zero elements in the lower triangular matrix
!                         amata sequentially.
!     icolpre        i    Integer array containing the column numbers of the
!                         non-zero elements in the lower triangular matrix
!                         premata sequentially.
!     imata          i    Integer array of length n+1, containing the number of
!                         elements in each row of the lower triangular matrix
!                         accumulated. The first position contains zero, the
!                         last the number of non-zero elements in the
!                         lower-triangular matrix
!                         Corresponds to the matrix amat
!     imatpre        i    Integer array of length n+1, containing the number of
!                         elements in each row of the lower triangular matrix
!                         accumulated. The first position contains zero, the
!                         last the number of non-zero elements in the
!                         lower-triangular matrix
!                         Corresponds to the peconditioning matrix premat
!     nrusol         i    Number of free unknowns
!     premat         o    preconditioning matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iwork(nrusol), i, istarta, istartpre, irow, numcola, &
              numcolpre, j, icolumn, ipos

!     i              Counting variable
!     icolumn        Column sequence number
!     ipos           Position in array
!     irow           Row sequence number
!     istarta        Last entry used in array amatl
!     istartpre      Last entry used in aray prematl
!     iwork          integer work array of length nrusol
!                    This array is used to mark non-zero columns in each row
!                    of the preconditioning matrix
!                    iwork(i) = 0: entry not in premat
!                    iwork(i)>0 then column i is in premat in position iwork(i)
!     j              Counting variable
!     numcola        Number of non-zero columns in row of matrix amata
!     numcolpre      Number of non-zero columns in row of matrix amatpre
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     iwork(1-nrusol) := 0
!     istarta := imata(1);  istartpre := imatpre(1)
!     For irow := 1 (1) nrusol
!        numcola := imata(irow+1)-istarta
!        numcolpre := imatpre(irow+1)-istartpre
!        Mark positions of columns in premat in array iwork:
!        For j := 1, numcolpre
!           iwork(icolpre(istartpre+j)) := istartpre+j
!        For j := 1, numcola
!           istarta := istarta+1
!           icolumn := icola(istarta), ipos := iwork(icolumn)
!           if ( ipos>0 ) then
!              prematl(ipos) := amatl(istarta)
!              prematu(ipos) := amatu(istarta)
!        Clear array iwork
!        For j := 1, numcolpre
!           iwork(icolpre(istartpre+j)) := 0
!        istartpre := istartpre+numcolpre
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'preccopy' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from preccopy'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First clear array iwork

      iwork = 0
      premat(1:imatpre(nrusol+1)) = 0d0

!     --- istarta := imata(1);  istartpre := imatpre(1)

      istarta = imata(1)
      istartpre = imatpre(1)

!     --- For irow := 1 (1) nrusol

      do irow = 1, nrusol

!     --- numcola := imata(irow+1)-istarta
!         numcolpre := imatpre(irow+1)-istartpre

         numcola = imata(irow+1)-istarta
         numcolpre = imatpre(irow+1)-istartpre

!        --- Mark positions of columns in premat in array iwork:
!            For j := 1, numcolpre
!               iwork(icolpre(istartpre+j)) := istartpre+j

         do j = 1, numcolpre
            iwork(icolpre(istartpre+j)) = istartpre+j
         end do  ! j = 1, numcolpre

!        --- For j := 1, numcola

         do j = 1, numcola

!        --- istarta := istarta+1
!            icolumn := icola(istarta), ipos := iwork(icolumn)

            istarta = istarta+1
            icolumn = icola(istarta)
            ipos = iwork(icolumn)
            if ( ipos>0 ) then

!           --- ipos>0, hence entry in both matrices
!               prematl(ipos) := amatl(istarta)
!               prematu(ipos) := amatu(istarta)

               premat(ipos) = amat(istarta)

            end if  ! ( ipos>0 )

         end do  ! j = 1, numcola

!        --- Clear array iwork
!            For j := 1, numcolpre
!               iwork(icolpre(istartpre+j)) := 0

         do j = 1, numcolpre
            iwork(icolpre(istartpre+j)) = 0
         end do  ! j = 1, numcolpre

!        --- istartpre := istartpre+numcolpre

         istartpre = istartpre+numcolpre

      end do  ! irow = 1, nrusol

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End preccopy'

      end if  ! ( debug )
      call erclos ( 'preccopy' )

      end
