      subroutine prfillintfnmsc ( nmeshctmp, lennmeshc, iobst, &
                                  numedgeintf, numnodeintf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-05-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill nmeshctmp for interface (Node numbers of the edges)
!     The local numbering of the interface is used
! **********************************************************************
!
!                       KEYWORDS
!
!     interface
!     level_set
!     node
!     edge
!     2d
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lennmeshc, nmeshctmp(lennmeshc), iobst, &
              numedgeintf, numnodeintf

!     iobst          i    Obstacle sequence number
!     lennmeshc     i/o   Declared length of array nmeshc
!                         At output the computed length
!     nmeshctmp      o    nmesh part c, contains local node numbers
!                         corresponding to local edges of interface
!     numedgeintf    o    Number of edges at the interface
!     numnodeintf    o    Number of nodes at the interface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istarticross, numintersect, istart, i, numelms, jstart, &
              numbndobst, numedges, lenistore
      integer, allocatable :: iwork(:), istore(:,:), imark(:)

!     i              Counting variable
!     imark          Integer array to mark edges that have been used
!     istart         Starting address
!     istarticross   Starting address array icrossobst
!     istore         Work array to store nodes of global edges that are
!                    completely inside the interface, i.e. both nodes
!                    are global nodes on the interface
!                    It concerns the relative node numbers with respect to the
!                    interface nodes as stored in ipointobst + numintersect
!     iwork          integer work array to mark all global nodes on the
!                    interface
!                    iwork(i) = 0: node not on interface
!                    iwork(i) > 0: the node is on the interface.
!                    The value indicates the sequence number in array ipointobst
!     jstart         starting address
!     lenistore      Computed length of array istore
!     numbndobst     Number of nodal points on the boundary of the obstacle
!     numedges       number of edges
!     numelms        Number of elements that are intersected by the interface
!     numintersect   Number of intersections with edges
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INIZEROLEV     Compute number of nodes in zero level set and starting
!                    position in ipointobst
!     INSTOP         Stop the program
!     PRFILLINTFNMC1 Mark all local edges that have a global node on
!                    the interface
!     PRFILLINTFNMC2 Fill nmeshc for interface (Node numbers of the edges)
!     PRINNMESHC     print nmesh part c
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillintfnmsc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillintfnmsc'
         write(irefwr,1) 'iobst, lennmeshc', iobst, lennmeshc
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istarticross = icrossobst(iobst)

      numintersect = icrossobst(istarticross)     ! number of local nodes
      istarticross = istarticross+numintersect*2

!     --- Get number of nodes at boundary of interface

      call inizerolev ( ipointobst, istart, numbndobst, iobst )

      lenistore = max(1,numbndobst)
      allocate ( iwork(npoint), istore(2,lenistore), imark(nedges), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, npoint+nedges+lenistore, 'iwork' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'numbndobst', numbndobst

      numedges = 0  ! number of edges on interface
      imark = 0     ! mark all global edges as not used
      iwork = 0     ! mark all nodes as not in interface

      if ( numbndobst>0 ) then

!     --- numbndobst>0, hence there are global nodes in the interface
!         Mark all local interface edges that have at least one global node
!         on the interface in imark
!         Store all local edges that have two global nodes on the interface
!         Hence all local edges on the interface that are also global edges
!         are stored
!         These numedges edges are stored in istore

         call prfillintfnmc1 ( iwork, npoint, numbndobst, &
                               ipointobst(istart+1), nmeshl, imark, &
                               nmeshc, istore, numedges, &
                               numintersect, lenistore )

      end if  ! ( numbndobst>0 )

      numelms = icrossobst(istarticross+1)

!     --- Check array length

      call inichkarlen ( 4*(numelms+numedges), lennmeshc, 'nmeshc' )

      if ( debug ) &
         write(irefwr,1) 'numintersect, numelms, numedges', &
                          numintersect, numelms, numedges

      istart = istarticross+1
      if ( debug ) write(irefwr,1) 'istart', istart

      jstart = numelms+1+numedges
      if ( ndim==2 ) then

!     --- ndim = 2, elements shoud be triangles

         call prfillintfnmc2 ( numelms, nmeshctmp, istart, numbndobst, &
                               iwork, numintersect, lennmeshc, jstart )

      else

!     --- ndim = 2, elements shoud be tetrahedrons

         print *,'ndim = 3 not yet implemented in prfillintfnmsc'
         call instop

      end if  ! ( ndim==2 )

!     --- Extend nmeshctmp with edges that are completely on the interface

      do i = 1, numedges
         nmeshctmp(i+numelms) = jstart
         nmeshctmp(jstart) = 2
         nmeshctmp(jstart+1:jstart+2) = istore(1:2,i)
         jstart = jstart+3
      end do  ! i = 1, numedges

      lennmeshc = jstart-1
      numedgeintf = numelms+numedges
      numnodeintf = numintersect+numbndobst

      deallocate ( iwork, istore, imark, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         call prinnmeshc ( numedgeintf, nmeshctmp )
         write(irefwr,*) 'End prfillintfnmsc'

      end if  ! ( debug )
      call erclos ( 'prfillintfnmsc' )

      end
