      subroutine prfillmasslev3 ( massmt, coor, nmeshg, nfaces, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-02-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill mass matrix for all faces zero level set
!     3d version
! **********************************************************************
!
!                       KEYWORDS
!
!     mass_matrix
!     level_set
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nmeshg(*), nfaces, iwork(*)
      double precision massmt(*), coor(3,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     iwork          i    integer work array of size npoint
!                         if iwork(i) = 0 point is not on the interface, &
!                         if 1 it is
!     massmt         o    mass matrix defined over the interface
!     nfaces         i    number of faces
!     nmeshg         i    nmesh part g, contains node numbers corresponding to
!                         faces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iface, ipstart, numnodes, inode1, inode2, inode3, k, &
              ndim, iqmat
      double precision x(3,3), xloc(3,2), e(3,2), h, delta, qmat(3)

!     delta          jacobian of element
!     e              Work array of length 3 x 2, to store the factors e^ij
!     h              area of face / 3
!     iface          face sequence number
!     inode1         First node in face
!     inode2         Second node in face
!     inode3         Third node in face
!     ipstart        Starting address in array
!     iqmat          if 1 the qmat matrix must be filled
!     k              Counting variable
!     ndim           Dimension of the space.
!     numnodes       Number of nodal points in face
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     x              Contains co-ordinates of vertices
!     xloc           co-ordinates of vertices in local (xi-zeta) coordinate
!                    system
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELP620JACOB    Compute the Jacobian of a triangle
!     ELP620XLOC     Fill local coordinates for triangle in 3d
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2813   more than 3 nodes in face
! **********************************************************************
!
!                       PSEUDO CODE
!
!     for all faces in mesh do
!        if ( all nodes have iwork = 1 ) then
!           create element mass matrix for face
!           Add to large matrix
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillmasslev3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillmasslev3'
100      format ( a, 1x, (10i6) )
110      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- for all faces in mesh do

      do iface = 1, nfaces

         ipstart = nmeshg(iface)
         numnodes = nmeshg(ipstart)
         if ( numnodes>3 ) then

!        --- Not all faces are linear

            call errint ( numnodes, 1 )
            call errint ( iface, 2 )
            call errint ( 3, 3 )
            call errsub ( 2813, 3, 0, 0 )
            go to 1000

         end if  ! ( numnodes>2 )

         inode1 = nmeshg(ipstart+1)
         inode2 = nmeshg(ipstart+2)
         inode3 = nmeshg(ipstart+3)
         if ( iwork(inode1)>0 .and. iwork(inode2)>0 .and. &
              iwork(inode3)>0 ) then

!        --- all nodes have iwork = 1
!            create element mass matrix for face
!            Add to large matrix

            do k = 1, 3
               x(1,k) = coor(k,inode1)
               x(2,k) = coor(k,inode2)
               x(3,k) = coor(k,inode3)
            end do

            if ( debug ) then

!           --- Debug information

               write(irefwr,100) 'inode1, inode2, inode3', &
                                  inode1, inode2, inode3
               write(irefwr,110) 'coordinates node1 ', (x(1,k),k=1,3)
               write(irefwr,110) 'coordinates node2 ', (x(2,k),k=1,3)
               write(irefwr,110) 'coordinates node3 ', (x(3,k),k=1,3)

            end if  ! ( debug )

!           --- Fill local coordinates of element

            ndim = 3
            iqmat = 0
            call elp620xloc ( x, ndim, iqmat, qmat, xloc )

            if ( debug ) then

!           --- Debug information

               write(irefwr,110) &
                  'local coordinates node1 ', (xloc(1,k),k=1,2)
               write(irefwr,110) &
                  'local coordinates node2 ', (xloc(2,k),k=1,2)
               write(irefwr,110) &
                  'local coordinates node3 ', (xloc(3,k),k=1,2)

            end if  ! ( debug )

!           --- Compute Jacobian

            call elp620jacob ( e, xloc, delta, 0, nmeshg(ipstart+1), &
                               0, 0, 0 )
            if ( ierror/=0 ) go to 1000

            h = delta/6d0
            massmt(inode1) = massmt(inode1)+h
            massmt(inode2) = massmt(inode2)+h
            massmt(inode3) = massmt(inode3)+h

         end if  ! ( iworkinode1()>0 .and. iwork(inode2)>0 )

      end do  ! iface = 1, nfaces

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfillmasslev3'

      end if
      call erclos ( 'prfillmasslev3' )

      end
