      subroutine principstr ( isol, iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 11-12-2012 Long integers
!        version  1.1  date 06-02-2012 Allow indexed vectors
!        version  1.0  date 15-06-2000
!
!   copyright (c) 2000-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Computes the principal stresses from a stress tensor
!
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     stress
!     principal_stress
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), iinput(*)

!     iinput         i    integer input array with the following contents
!                         1: iseqstress Sequence number of array eigval in isol
!                         2: iseqeigval Sequence number of array eigvec in isol
!                         3: iseqeigvec Sequence number of array stress in isol
!                         4: ichoice Integer choice parameter with the
!                            following meaning
!                            1: Compute the eigenvalues only
!                            2: Compute the eigenvectors only
!                            3: Compute the scaled eigenvectors only
!                            4: Compute the eigenvectors  and eigenvalues
!                            5: Compute the scaled eigenvectors and eigenvalues
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution arrays
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ichoice, ivec, iseqeigval, iseqeigvec, iseqstress, &
              ivecstruc, nnodes

!     ichoice        Integer choice parameter with the following meaning
!                    1: Compute the eigenvalues only
!                    2: Compute the eigenvectors only
!                    3: Compute the scaled eigenvectors only
!                    4: Compute the eigenvectors  and eigenvalues
!                    5: Compute the scaled eigenvectors and eigenvalues
!     iseqeigval     Sequence number of array eigval in isol
!     iseqeigvec     Sequence number of array eigvec in isol
!     iseqstress     Sequence number of array stress in isol
!     ivec           Sequence number of vector of special structure
!     ivecstruc      Defines type o vetcor of special structure
!     nnodes         Number of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inivectr

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIVECTR       Return with sequence number of vector
!     PRINCIP02      Computes the principle stresses from a 2d stress tensor
!     PRINCIP03      Computes the principle stresses from a 3d stress tensor
!     sepcreasolbf1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2179   Input vector is of the wrong type
!    2180   Input vector is of the wrong type
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Extract information from array iinput
!     perform some checks
!     Create output vectors depending on ivecstruc
!     Compute starting addresses of arrays
!     Call subroutines that compute eigenvectors and eigenvalues
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'principstr' )

!     --- Extract information from array iinput

      iseqstress = inivectr(iinput(1))
      iseqeigval = inivectr(iinput(2))
      iseqeigvec = inivectr(iinput(3))
      ichoice = iinput(4)

!     --- perform some checks

      ivecstruc = ks(isol(iseqstress))%typevec
      if ( ivecstruc/=115 .and. ivecstruc/=116 ) then

!     --- error: input vector is of the wrong type

         call errint ( ivecstruc, 1 )
         call errsub ( 2179, 1, 0, 0 )

      end if
      ivec = ks(isol(iseqstress))%ivec
      if ( ivecstruc==115 ) then

!     --- vector is defined per node

         if ( ivec/=4 .and. ivec/=9 ) then

!        --- Vector does not correspond to stress tensor

            call errint ( ivec, 1 )
            call errint ( 4, 2 )
            call errchr ( 'stress', 1 )
            call errsub ( 2180, 2, 0, 1 )

         end if

      else

!     --- vector is defined per element

         if ( ivec/=6 ) then

!        --- Vector does not correspond to stress tensor

            call errint ( ivec, 1 )
            call errint ( 6, 2 )
            call errchr ( 'stress', 1 )
            call errsub ( 2180, 2, 0, 1 )

         end if

      end if
      if ( ierror/=0 ) go to 1000

!     --- Create output vectors depending on ivecstruc

      nnodes = ks(isol(iseqstress))%nusol/6
      if ( ivecstruc==115 ) then

!     --- vector is defined per node or vertex

         if ( ichoice==1 .or. ichoice>=4 ) then

!        --- store the eigenvalues

            call sepcreasolbf1 ( isol(iseqeigval), 115, ivec+1, &
                                 'stress_eigenvalues', 0 )

         end if

         if ( ichoice>1 ) then

!        --- store the eigenvectors

            if ( ivec==4 ) then
               ivec = ivec+9
            else
               ivec = ivec+7
            end if
            call sepcreasolbf1 ( isol(iseqeigvec), 115, ivec, &
                                 'stress_eigenvectors', 0 )

         end if

      else

!     --- vector is defined per element

         if ( ichoice==1 .or. ichoice>=4 ) then

!        --- store the eigenvalues

            call sepcreasolbf1 ( isol(iseqeigval), 116, ivec+ndim-6, &
                                 'stress_eigenvalues', 0 )

         end if

         if ( ichoice>1 ) then

!        --- store the eigenvectors

            call sepcreasolbf1 ( isol(iseqeigvec), 116, ivec-6+ndim**2, &
                                  'stress_eigenvectors', 0 )

         end if

      end if

!     --- Compute starting addresses of arrays

      stress1d => ks(isol(iseqstress))%sol
      if ( ichoice==1 .or. ichoice>=4 ) eigval => ks(isol(iseqeigval))%sol
      if ( ichoice>1 ) eigvec => ks(isol(iseqeigvec))%sol

      if ( ierror/=0 ) go to 1000

!     --- Call subroutines that compute eigenvectors and eigenvalues

      if ( ndim==2 ) then

!     --- ndim = 2

         call princip02 ( ichoice, stress1d, eigval, eigvec, nnodes )

      else

!     --- ndim = 3

         call princip03 ( ichoice, stress1d, eigval, eigvec, nnodes )

      end if
1000  call erclos ( 'principstr' )
      end
