      subroutine prmakedist ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.6  date 22-05-2013 Jump in case no interface
!        version  1.5  date 25-04-2013 Use getparm
!        version  1.4  date 11-12-2012 Long integers
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make the level set function phi a distance function
!
! **********************************************************************
!
!                       KEYWORDS
!
!     time_integration
!     level_set
!     distance
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)
      double precision rinsep(*)

!     inpsep         i    Contains the integer input
!                         See subroutine readscmakedist for a description
!     rinsep         i    Contains the real input for this subroutine
!                         See subroutine readscmakedist for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqphi, nnodes, typedist, &
              nusol, iprint, istop, igrad(5), ninterf
      double precision phibarrier, eps, err, err0, h
      integer, allocatable :: iwork(:), interf(:)
      double precision, allocatable :: phiold(:)
      logical :: finished

!                    they are not
!     eps            Local accuracy with respect to the distance function
!     err            Parameter to be kept for prmakedisterr
!     err0           Parameter to be kept for prmakedisterr
!     finished       Indicates if accuracy has been reached (true) or not
!                    (false)
!     h              Step size for rectangular grid
!     igrad          Sepran array containing information of the gradient
!     interf         Contains the node numbers of the interface
!     iprint         Defines the amount of printing (0 none, 2 maximal)
!     iseqphi        Sequence number of level set function phi in isolut2d
!     istop          Indicates if program must stop in case of error (0,1)
!                    or must continue (2,3)
!     iwork          integer work array to mark the nodes
!                    -10:  node with phi<0 outside the band
!                     -1:  node with phi<0 inside the band
!                      0:  node on interface
!                      1:  node with phi>0 inside the band
!                     10:  node with phi>0 outside the band
!     ninterf        number of nodes in array interf
!     nnodes         Number of nodes in array nodes
!     nusol          Total number of unknowns
!     phibarrier     Defines the region where the accuracy is tested
!                    Only nodes close to the interface defined by
!                    |phi|<phibarrier are used
!     phiold         Copy of phi from the previous iteration (time step)
!     typedist       Defines the type of method used to make a distance function
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inivectr
      double precision getparm

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     GETPARM        Return with real defined by inpsep/rinsep
!     INIVECTR       Return with sequence number of vector
!     PRCOMPGRAD     Compute gradient of a vector with one component per point
!     PRININ         print 1d integer array
!     PRMAKEDIST1D1  Make exact distance function in 1d
!     PRMAKEDISTERR  Compute difference between phi and phiold and check
!                    accuracy
!     PRMAKEDISTEXAC Make phi a distance function for all nodes next to the
!                    interface and all of its neighbors
!     PRMAKEDISTEXNW Make phi a distance function for all nodes next to the
!                    interface and all of its neighbors
!                    Non adapted mesh
!     PRMAKEDISTMARK Mark nodes in interface and outside interface
!                    Fill arrays nodes, interf and array phiold
!     PRMAKEDISTPSEU Make the level set function phi a distance function
!                    Use pseudo time integration
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1576   Value of integer parameter incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The signed distance function phi should satisfy ||grad(phi)|| = 1 to
!     make it a distance function
!     Here we start with the computed levelset function phi.
!     To make it a distance function we solve the time dependent PDE:
!     d phi/dt + sign(phi) grad(phi)/||grad(phi)|| grad (phi) = sign(phi)
!     with phi at the interface equal to 0
!     The equation is linearized to the convection equation
!     d phi/dt + w grad (phi) = sign(phi)
!     with w equal to sign(phi) grad(phi)/||grad(phi)|| at the previous
!     time step
!     If t goes to infinity and the equation becomes stationary
!     we get:
!     sign(phi) grad(phi)/||grad(phi)|| grad (phi) = sign(phi)
!     or grad(phi) . grad (phi) / ||grad(phi)|| = 1, hence ||grad(phi)|| = 1
!     This PDE is solved by Euler implicit.
!     In fact this may be considered as kind of Picard iteration.
!     The pseudo time step may be relatively large to improve convergence
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmakedist' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmakedist'
         call prinin ( inpsep,15, 'inpsep' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      iprint = inpsep(3)
      iseqphi = inivectr(inpsep(6))
      phibarrier= getparm ( inpsep, rinsep, 8, 'phibarrier' )
      eps = getparm ( inpsep, rinsep, 9, 'eps' )
      h = getparm ( inpsep, rinsep, 15, 'h' )
      istop = inpsep(10)
      typedist = inpsep(14)
      if ( debug ) then
         write(irefwr,1) 'iseqphi, istop, typedist', &
                          iseqphi, istop, typedist
         write(irefwr,2) 'phibarrier, eps, h', phibarrier, eps, h
      end if  ! ( debug )
      nusol = ks(isolut2d(iseqphi))%nusol

!     --- In case the mesh has not been adapted (meshadapt=0)
!         and typedist=2 we set typedist to 3 to indicate that the interface
!         has to be computed

      if ( typedist==2 .and. meshadapt==0 ) typedist=3  ! mesh has not been

      igrad = 0   ! gradient of phi

!     --- Allocate temporary arrays

      allocate ( nodes(nusol), phiold(nusol), iwork(nusol), &
                 interf(nusol), stat = error )
      if ( error/=0 ) call eralloc ( error, 3*nusol, 'nodes' )
      if ( ierror/=0 ) go to 1000

      phi1 => ks(isolut2d(iseqphi))%sol

!     --- Mark all nodes (interface 0, positive 1 and negative -1)
!         Store all nodes at interface in interf
!         Find all nodes with phi<phibarrier and store in array nodes
!         Copy contents of phi into phiold

      call prmakedistmark ( nnodes, nodes, ninterf, interf, iwork, &
                            nusol, phi1, phibarrier, phiold )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,1) 'ninterf', ninterf
         call prinin ( nodes, nnodes, 'nodes' )
      end if  ! ( debug )

!     --- compute gradient of phi

      call prcompgrad ( igrad, iseqphi )
      grad => ks(igrad(1))%sol

!     --- Check if max | ||grad phi||-1 |<eps
!         if so skip the process (finished = true)

      call prmakedisterr ( phi1, phiold, nodes, nnodes, &
                           nusol, eps, iprint, 0, finished, 1, &
                           istop, err, err0, grad, 2, ndim )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,2) 'err, err0', err, err0
         write(irefwr,*) 'finished', finished
      end if  ! ( debug )

      if ( .not. finished ) then

         select case ( typedist )

         case(1)

!        --- typedist = 1: solve by distance function

            call prmakedistpseu ( inpsep, rinsep, igrad, &
                                  eps, err, err0, nnodes, phiold )

         case(2)

!        --- typedist = 2: compute "exact" distance

            if ( ndim==1 .and. ninterf>0 ) then

!           --- ndim = 1, simple case

               call prmakedist1d1 ( interf, ninterf, coor, npoint, &
                                    phi1, iwork )

            else if ( ninterf>0 ) then

!           --- ndim>1, more complex

               call prmakedistexac ( interf, ninterf, iwork, eps, nnodes, h )

            end if  ! ( ndim==1 )

         case(3)

!        --- typedist = 3: compute "exact" distance
!            The mesh has not been adapted

            if ( ndim==1 ) then

!           --- ndim = 1, simple case, not yet implemented


               call errchr ( 'typedist', 1 )
               call errchr ( 'inpsep', 2 )
               call errint ( typedist, 1 )
               call errint ( 14, 2 )
               call errsub ( 1576, 2, 0, 2 )

            else

!           --- ndim>1, more complex

               call prmakedistexnw ( iwork, eps, nnodes, h, &
                                     isolut2d(iseqphi), kprob )
            end if  ! ( ndim==1 )

         case default

!        --- Error parameter has wrong value

            call errchr ( 'typedist', 1 )
            call errchr ( 'inpsep', 2 )
            call errint ( typedist, 1 )
            call errint ( 14, 2 )
            call errsub ( 1576, 2, 0, 2 )

         end select  ! case ( typedist )

      end if  ! ( .not. finished )

!     --- Remove igrad

      if ( igrad(1)>0 ) call seprmsolinfo ( igrad(1) )

!     --- raise istartmn

      istartmn = istartmn+16

!     --- Remove temporary array

      deallocate ( nodes, phiold, iwork, interf, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nodes' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmakedist'

      end if  ! ( debug )
      call erclos ( 'prmakedist' )

      end subroutine prmakedist
