      subroutine prmakedistex3 ( ninterf, iwork, grid, isigphi, &
                                 origin, nx, dx, coorinter, nedginterf, &
                                 nmeshcloc, nmeshlloc, numnodeintf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-05-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make phi a distance function for all nodes next to the interface
!     and all of its neighbors
!     Points that satisfy | ||grad phi||-1 |<eps are excluded!
!     Only points with phi>0 or phi<0 are considered
!     It concerns the case of a non-adapted mesh
!     This subroutine is the equivalent of prmakedistex1
! **********************************************************************
!
!                       KEYWORDS
!
!     level_set
!     distance
!     interface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), ninterf, &
              grid(*), isigphi, nx(ndim), nedginterf, &
              nmeshcloc(*), nmeshlloc(*), numnodeintf
      double precision origin(ndim), dx(ndim), &
                       coorinter(ndim,ninterf)

!     coorinter      i    Contains coordinates of the interface nodes
!                         in the local numbering of the interface
!     dx             i    contains the difference between xmax, xmin etc
!     grid           i    The domain is subdivided in nx(1) x nx(2) cells
!                         of size dx(1) x dx(2)
!                         In grid the number of interface nodes per cell is
!                         stored as well as the node numbers in the folwoing way
!                         Pos. 1 starting address of node numbers
!                         (nx(1)*nx(2)+1)))
!                         The number of nodes in cell ij is defined by
!                         grid(start+1)-grid(start). where
!                         start = (j)-1)*nx(1)+i
!                         The corresponding node numbers are stored in grid
!                         from position start
!     isigphi        i    sign of phi (1 if phi>0), (-1 if phi<0)
!     iwork         i/o   integer work array to mark the nodes
!                         -10:  node with phi<0 outside the band
!                          -1:  node with phi<0 inside the band
!                           0:  node on interface
!                           1:  node with phi>0 inside the band
!                          10:  node with phi>0 outside the band
!     nedginterf     i    Number of edges in interface
!     ninterf        i    number of nodes in array interf
!     nmeshcloc      o    Local array nmeshc
!     nmeshlloc      o    Local array nmeshl
!     numnodeintf    i    Number of nodes at the interface
!     nx             i    number of points in ndim directions of the grid
!     origin         i    Contains the coordinates of the origin of the region
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nset1, nset2, nsete, nset0, iobst, numbndobst, iseq
      integer, allocatable :: set1(:), set2(:), markintf(:), setwk(:), &
                              set0(:)

!     iobst          Obstacle sequence number
!     iseq           Dummy
!     markintf       Array of size nedges or nfaces to mark the edges or
!                    faces of the interface (1) else 0
!     nset0          number of nodes in set0
!     nset1          number of nodes in set1
!     nset2          Number of nodes in set 2
!     nsete          Number of nodes in set0 extended with interface points
!     numbndobst     Number of nodal points on the boundary of the obstacle
!     set0           Contains all direct neighbors of the interface
!                    (one side only)
!     set1           Contains all neighbors of the interface (one side only)
!     set2           Contains all neighbors of set1 not in the interface
!     setwk          Help array to store the values of iwork in set1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ININEIGHMARK   Find all marked neighbors of a set of nodes and store
!                    in array neighbors
!     ININEIGHMARK1  Find all marked neighbors of the interface and store in
!                    array neighbors
!     ININEIGHMARK2  Reduce set of nodes to those nodes that satisfy
!                    grad(phi) not close to 1
!     PRCOMPDIST1    Compute distance of set of nodes to interface
!     PRININ         print 1d integer array
!     SEPFILLKMAD3   Fill array kmesh part ad as well as the subarrays
!                    ipointobst and ielemobst (R^3 only)
!     SEPFILLKMESHAD Fill array kmesh part d
!     SEPFILLKMESHE  Fill array kmesh part e
!     SEPFILLOBFACES Fill kmesh part ad subpart 9: iobstfaces
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmakedistex3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmakedistex3'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate arrays

      if ( kelme==0 ) call sepfillkmeshe
      if ( kelmad==0 ) call sepfillkmeshad ( iseq )
      if ( kmeshad(3)==0 .or. kmeshad(5)/=kmeshad(8) ) call sepfillkmad3
      if ( kmeshad(9)==0 ) call sepfillobfaces

!     --- allocate arrays

      allocate ( set1(npoint), set2(npoint), set0(npoint), &
                 setwk(npoint), markintf(nedginterf), &
                 stat = error )
      length = 4*npoint+nedginterf
      if ( error/=0 ) call eralloc ( error, length, 'set1' )
      if ( ierror/=0 ) go to 1000

      markintf = 1  ! clear array

      iobst = 1  ! default obstacle number

!     --- Set 0 = all immediate neighbors in nodes of interface

      nset0 = npoint

!     --- Fill array set0 with the nodes of the intersected set of edges

      call inineighmark1 ( npoint, nset0, set0, iwork, &
                           icrossobst, nmeshc, &
                           ninterf, iobst, numbndobst, ipointobst )

      nsete = nset0+numbndobst
      setwk(1:nsete) = iwork(set0(1:nsete))  ! store values of iwork in setwk
      iwork(set0(1:nsete)) = 2   ! Mark nodes in set0 in iwork with a 2

      if ( debug ) call prinin ( set0, nsete, 'set0' )

!     --- Set 1 = all neighbors of set0

      nset1 = npoint
      call inineighmark ( set0, nsete, npoint, kmeshe, &
                          nset1, set1, iwork )
      if ( ierror/=0 ) go to 1000

      if ( debug ) call prinin ( set1, nset1, 'set1' )

      setwk(nsete+1:nsete+nset1) = iwork(set1(1:nset1)) ! store values of iwork
      iwork(set1(1:nset1)) = 2   ! Mark nodes in set1 in iwork with a 2

!     --- Set 2 = all direct neighbors of the previous set not already marked

      nset2 = npoint
      call inineighmark ( set1, nset1, npoint, kmeshe, nset2, &
                          set2, iwork )
      if ( ierror/=0 ) go to 1000

      if ( debug ) call prinin ( set2, nset2, 'set2' )

!     --- Reset iwork
!         Reduce all sets to those nodes that satisfy grad(phi) not close to 1

      iwork(set0(1:nsete)) = setwk(1:nsete)
      iwork(set1(1:nset1)) = setwk(nsete+1:nsete+nset1)

!     --- Reduce all sets to those nodes that satisfy grad(phi) not close to 1

      call inineighmark2 ( nset0, set0, iwork )
      call inineighmark2 ( nset1, set1, iwork )
      call inineighmark2 ( nset2, set2, iwork )

      if ( debug ) then
         call prinin ( set0, nset0, 'set0/2' )
         call prinin ( set1, nset1, 'set1/2' )
         call prinin ( set2, nset2, 'set2/2' )
      end if  ! ( debug )

      deallocate ( setwk, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'setwk' )
      if ( ierror/=0 ) go to 1000

!     --- Compute distance for all nodes in sets 0/1/2

      if ( nset0>0 ) then

!     --- nset0>0, there are nodes to be adapted

         call prcompdist1 ( set0, nset0, coor, ndim, nx, &
                            origin, dx, grid, markintf, &
                            nmeshcloc, nmeshlloc, numnodeintf, &
                            phi1, isigphi, nedginterf, coorinter )

      end if  ! ( nset1>0 )
      if ( ierror/=0 ) go to 1000

      if ( nset1>0 ) then

!     --- nset1>0, there are nodes to be adapted

         call prcompdist1 ( set1, nset1, coor, ndim, nx, &
                            origin, dx, grid, markintf, &
                            nmeshcloc, nmeshlloc, numnodeintf, &
                            phi1, isigphi, nedginterf, coorinter )

      end if  ! ( nset1>0 )
      if ( ierror/=0 ) go to 1000

!     --- Compute distance for all nodes in set 2

      if ( nset2>0 ) then

!     --- nset2>0, there are nodes to be adapted

         call prcompdist1 ( set2, nset2, coor, ndim, nx, &
                            origin, dx, grid, markintf, &
                            nmeshcloc, nmeshlloc, numnodeintf, &
                            phi1, isigphi, nedginterf, &
                            coorinter )

      end if  ! ( nset2>0 )

      deallocate ( set0, set1, set2, markintf, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'set1' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmakedistex3'

      end if  ! ( debug )
      call erclos ( 'prmakedistex3' )

      end subroutine prmakedistex3
