      subroutine prmassmat
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-12-2012 Long integers
!        version  1.0  date 03-05-2010
!
!   copyright (c) 2010-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill mass matrix for all nodes that are on the outer boundary
!     The mass matrix is defined for the elements on this set only
!     It is zero everywhere else
! **********************************************************************
!
!                       KEYWORDS
!
!     mass_matrix
!     outer_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp
      logical linear, linfound

!     ielgrp         element group number
!     linear         Indicates whether the elements are linear
!                    (true) or not (false)
!     linfound       If true linear elements have been found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRFILLMASS2D   Fill 2d mass matrix for outer boundary
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     419   NDIM incorrect
!     800   Shape number not available
!    3047   Both linear and quadratic surface elements found in mesh
! **********************************************************************
!
!                       PSEUDO CODE
!
!     linear := linear elements are used
!     set (diagonal) mass matrix to zero
!     if ( mesh is 2d ) then
!        for all edges on outer boundary do
!           if ( both nodes have iwork = 1 ) then
!              create element mass matrix for edge
!              Add to large matrix
!     else
!        for all faces in outer boundary do
!           if ( all faces have iwork = 1 ) then
!              create element mass matrix for face
!              Add to large matrix
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmassmat' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmassmat'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ndim/=2 ) then

!     --- ndim # 2, not yet implemented

         call errint ( ndim, 1 )
         call errint ( 2, 2 )
         call errint ( 2, 3 )
         call errsub ( 419, 3, 0, 0 )
         go to 1000

      end if  ! ( ndim/=2 )

      if ( ndim==2 ) then

!     --- ndim = 2, compute linear

         linear = .true.
         linfound = .false.
         do ielgrp = 1, nelgrp

            ishape = kmeshf(ielgrp)
            select case ( ishape )
            case(3, 5, 9, 10 )

!           --- linear element

               linfound = .true.

            case ( 4, 6, 7 )

!           --- quadratic element

               linear = .false.

            case (8)

!           --- element with three nodes on side (not permitted)

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               go to 1000

            end select  ! case ( ishape )

         end do  ! ielgrp = 1, nelgrp

         if ( linfound .and. .not. linear ) then

!        --- Linear and quadratic elements, mix is not allowed

            call errsub ( 3047, 0, 0, 0 )
            go to 1000

         end if  ! ( linfound .and. .not. linear )

      end if  ! ( ndim==2 )

!     --- Create mass matrix

      allocate ( matrixm(npoint) )

      if ( ndim==2 ) then

!     --- mesh is 2d
!         for all edges in outer boundary do
!            if ( both nodes have iwork = 1 ) then
!               create element mass matrix for edge
!               Add to large matrix

         call prfillmass2d ( matrixm, linear, kmeshm, &
                             ncurvs, coor, npoint )

      else

!     --- mesh is 3d
!         for all faces in outer boundary do
!            if ( all nodes have iwork = 1 ) then
!               create element mass matrix for face
!               Add to large matrix
!         Not yet implemented

      end if  ! ( ndim==2 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmassmat'

      end if  ! ( debug )
      call erclos ( 'prmassmat' )

      end subroutine prmassmat
