      subroutine prsimplepar1 ( nu, np, leninnercontrp, &
                                leninnercontru )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 07-10-2014 New parameter list
!        version  2.0  date 06-11-2008 Extra parameters
!        version  1.0  date 09-05-2007
!
!   copyright (c) 2007-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy contents of global sendinfo into sendinfo for pressure
!     and velocity
!     Adapt the results
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     simple
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nu, np
      integer, intent(out) :: leninnercontrp, leninnercontru

!     leninnercontrp o    Length of array innercontrp
!     leninnercontru o    Length of array innercontru
!     np             i    Number of pressure unknowns
!     nu             i    Number of velocity unknowns
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numneigh, ineigh, ipstart, numunk, numunku, numunkp, i, &
              iunk, m, mu, lensendinfou, lensendinfop, lensend

!     i              Counting variable
!     ineigh         Loop variable over neighbor blocks
!     ipstart        Starting address in array
!     iunk           Position of degree of freedom in node
!     lensend        Length of array sendinfo
!     lensendinfop   Length of array sendinfop
!     lensendinfou   Length of array sendinfou
!     m              number of unknowns at interfaces
!     mu             number of velocity unknowns at interfaces
!     numneigh       number of neighbours blocks of present block
!     numunk         Number of unknowns at interface of 2 blocks
!     numunkp        Number of pressure unknowns at interface of 2 blocks
!     numunku        Number of velocity unknowns at interface of 2 blocks
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRSIMPLEPAR2   Fill arrays inumblocks for u and p unknowns
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     numneigh := number of neighbor blocks
!     for i := 1 (1) numneigh do
!        ipstart := starting address in sendinfo
!        Copy information from sendinfo into sendinfou and sendinfop
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsimplepar1' )
      debug = .false.
      innercontr => kprobpart%innercontr
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsimplepar1'
         write(irefwr,1) 'nu, np', nu, np
         call prinin ( innercontr, nu+np, 'innercontr/1' )
         m =  innercontr(nu+np+1)
         write(irefwr,1) 'm', m
         call prinin1 ( innercontr(nu+np+2), m, 2, 'inumblocks' )
         call prinin ( sendinfo, 10, 'sendinfo' )

  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      lensend = size(sendinfo)
      allocate ( sendinfou(lensend), sendinfop(lensend) )

!     --- numneigh := number of neighbor blocks

      numneigh = blockunkinfo(1)
      if ( debug ) write(irefwr,1) 'numneigh', numneigh

!     --- for i := 1 (1) numneigh do

      do ineigh = 1, numneigh

!     --- ipstart := starting address in sendinfo
!         Copy information from sendinfo into sendinfou and sendinfop

         ipstart = blockunkinfo(2*ineigh+1) ! starting address
         numunk = sendinfo(ipstart)    ! number of unknowns in interface
         numunku = 0                   ! number of vel unknowns in interface
         numunkp = 0                   ! number of press unknowns in interface
         if ( debug ) &
            write(irefwr,1) 'ineigh, ipstart, numunk', &
                             ineigh, ipstart, numunk

         do i = 1, numunk

            iunk = sendinfo(ipstart+i) ! unknown sequence number
            if ( iunk>nu ) then

!           --- Unknown has number larger than nu, hence it is pressure

               numunkp = numunkp + 1
               sendinfop(ipstart+numunkp) = iunk-nu

            else

!           --- Velocity unknown

               numunku = numunku + 1
               sendinfou(ipstart+numunku) = iunk

            end if  ! ( sendinfo(ipstart+i)>nu )

         end do  ! i = 1, numunk

         sendinfou(ipstart) = numunku
         sendinfop(ipstart) = numunkp
         if ( debug ) &
            write(irefwr,1) 'numunku, numunkp', numunku, numunkp

      end do  ! ineigh = 1, numneigh

!     --- Fill arrays innercontr

      innercontru(1:nu) = innercontr(1:nu)
      innercontrp(1:np) = innercontr(nu+1:nu+np)

      m = innercontr(nu+np+1)
      mu = m
      do i = 1, m

         if ( innercontr(nu+np+2*i)>nu ) then

!        --- Pressure unknown found

            mu = i-1
            go to 100

         end if  ! ( innercontr(nu+np+1+i)>nu )

      end do  ! i = 1, m

100   innercontru(nu+1) = mu
      innercontrp(np+1) = m-mu
      call prsimplepar2 ( innercontr(nu+np+2), innercontru(nu+2), &
                          innercontrp(np+2), mu, m-mu, nu )
      if ( debug ) write(irefwr,1) 'mu', mu

      leninnercontru = nu+2+2*mu
      leninnercontrp = np+2+2*(m-mu)
      lensendinfou = ipstart+numunku
      lensendinfop = ipstart+numunkp
      if ( associated(kprobpart%sendinfou) ) &
           deallocate(kprobpart%sendinfou)
      allocate ( kprobpart%sendinfou(lensendinfou) )
      kprobpart%sendinfou = sendinfou(1:lensendinfou)
      if ( associated(kprobpart%sendinfop) ) &
           deallocate(kprobpart%sendinfop)
      allocate ( kprobpart%sendinfop(lensendinfop) )
      kprobpart%sendinfop = sendinfop(1:lensendinfop)
      deallocate ( sendinfou, sendinfop )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( innercontru, nu, 'innercontr_u' )
         m =  innercontru(nu+1)
         call prinin1 ( innercontru(nu+2), m, 2, 'inumblocks_u' )

         call prinin ( innercontrp, np, 'innercontr_p' )
         m =  innercontrp(np+1)
         call prinin1 ( innercontrp(np+2), m, 2, 'inumblocks_p' )
         call prinin ( sendinfou, lensendinfou, 'sendinfou' )
         call prinin ( sendinfop, lensendinfop, 'sendinfop' )
         write(irefwr,*) 'End prsimplepar1'

      end if  ! ( debug )
      call erclos ( 'prsimplepar1' )

      end
