      subroutine prvecconst ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-02-2015 Allow ivec = 0
!        version  1.0  date 28-02-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create vector in isol and give constant value
!
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
!     constant
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekmesh
      use sepmodulemain
      use sepmodulemnsub
      use sepmodulekprob
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*)
      double precision, intent(in) :: rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine ASKVEC
!                         inpsep(7) contains ivec
!     rinsep         i    Real input array
!                         See subroutine ASKVEC
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: itypvc, ivec, ipos, nparts, idegfd, i, j, inodp, &
                 iseqvec, nnodes, ipoint(1)
      double precision :: const
      logical :: new

!     const          Value of constant
!                    they are not
!     i              Counting variable
!     idegfd         Degree of freedom sequence number
!     inodp          Nodal point number
!     ipoint         dummy
!     ipos           Position in array
!     iseqvec        Starting address of sequence numbers of vectors
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     j              Counting variable
!     new            if true the vector is new
!     nnodes         Number of nodes
!     nparts         number of parts the boundary consists of
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPFILLNODES   Fill array nodes with nodal points as defined in array
!                    inpsep
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     973   Required option has not yet been implemented
!    3088   option not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prvecconst' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prvecconst'
         call prinin ( inpsep, 20, 'inpsep' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get some parameters from inpsep

      iseqvec = inpsep(3)       ! sequence number of vector in left-hand side
      iprob = max(1,inpsep(2))  ! problem number
      idegfd = inpsep(5)        ! Degree of freedom

      if ( inpsep(14)>0 ) then

!     --- Option not yet implemented

         call errsub ( 973, 0, 0, 0 )
         go to 1000

      end if  ! ( inpsep(14)>0 )

!     --- Create vector

      itypvc = 110
      ivec = inpsep(7)
      new = isolut2d(iseqvec)<=0 .or. isolut2d(iseqvec)>lastsol
      if ( .not. new ) new = solarr(isolut2d(iseqvec))==0
      if ( ivec==0 .and. .not. new ) ivec = ks(isolut2d(iseqvec))%ivec
      if ( ivec>0 ) itypvc = 115
      call sepcreasolbf1 ( isolut2d(iseqvec), itypvc, ivec, &
                           names(iseqvec), 0 )

!     --- get starting address

      usol => ks(isolut2d(iseqvec))%sol

!     --- Make equal to constant stored in rinsep

      const = rinsep(inpsep(13))

      if ( inpsep(16)==0 ) then

!     --- The whole domain must be filled

         if ( idegfd==0 .or. nphyso==1 .and. indprfo==0 ) then

!        --- One unknown in each node

            usol = const

         else if ( indprp>0 ) then

!        --- More unknowns in each node. kprobp has been filled

            do i = 1, npoint

               ipos = kprobpo(i,idegfd)
               if ( ipos>0 ) usol(ipos) = const

            end do  ! i = 1, npoint

         end if  ! ( idegfd==0 .or. nphyso==1 .and. indprfo==0 )

      else

!     --- Only a part must be filled
!         Store node numbers in array nodes

         allocate ( nodes(npoint), stat = error )
         if ( error/=0 ) call eralloc ( error, npoint, 'nodes' )
         if ( ierror/=0 ) go to 1000

         call sepfillnodes ( inpsep(16), 0, inpsep(inpsep(15)), &
                             nodes, nnodes, ipoint, nparts )

         if ( new ) usol = 0d0
         if ( nphyso==1 .and. indprfo==0 ) then

!        --- One unknown in each node

            usol(nodes(1:nnodes)) = const

         else if ( indprp>0 ) then

!        --- More unknowns in each node. kprobp has been filled

            if ( idegfd==0 ) then

!           --- All degrees of freedom

               do i = 1, nnodes

                  inodp = nodes(i)

                  do j = 1, nphyso

                     ipos = kprobpo(inodp,j)
                     if ( ipos>0 ) usol(ipos) = const

                  end do  ! j = 1, nphys

               end do  ! i = 1, nnodes

            else

!           --- Only one degree of freedom

               do i = 1, nnodes

                  inodp = nodes(i)

                  ipos = kprobpo(inodp,idegfd)
                  if ( ipos>0 ) usol(ipos) = const

               end do  ! i = 1, nnodes

            end if  ! ( idegfd==0 )

         else

!        --- Option not yet available

            call errsub ( 3088, 0, 0, 0 )
            go to 1000

         end if  ! ( nphys==1 .and. indprf==0 )n

         deallocate ( nodes, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'nodes' )
         if ( ierror/=0 ) go to 1000

      end if  ! ( inpsep(16)==0 )

      numvecmn = max(numvecmn, iseqvec)

      nullify (usol)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prvecconst'

      end if  ! ( debug )
      call erclos ( 'prvecconst' )

      end
