      subroutine sepcnvchk ( isolprec, niter, finished, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Checks the convergence of an iteration process and prints information
!     about the convergence if required
!     Old subroutine prcnvchk
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical, intent(inout) :: finished
      integer, intent(in) :: isolprec(5,*), niter
      character (len=*), intent(inout) :: text

!     finished      i/o   Indicates if the loop has been finished (true) or not
!                         (false)
!     isolprec       i    Solution at preceding level
!     niter          i    Iteration sequence number
!     text          i/o   String variable containing some text
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iinvec(6), ieqn, i
      double precision :: rinvec(2), alam, eps
      logical :: diver

!     alam           Eigenvalue lambda
!     diver          Indicates if the iteration process diverges (true) or not
!     eps            Local accuracy
!     i              Counting variable
!     ieqn           Equation sequence number
!     iinvec         User input array for subroutine MANVEC
!     rinvec         Real user input array for subroutine MANVEC
!                    The contents depend on iinvec
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     SEPMANVEC      Vector manipulation subroutine
!     SEPPRINTV      Print solution vector or vector of special structure
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1906   Divergence of iteration process
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcnvchk' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcnvchk'
         write(irefwr,*) 'finished', finished
         write(irefwr,1) 'lineartim', lineartim
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'ncopeqtim, iprinttim', ncopeqtim, iprinttim
      iinvec(1) = 6   ! number of entries filled by user
      iinvec(2) = 5   ! ||u1|| (ispec=0) or ||u1-u2|| (ispec=1)
      iinvec(3) = 0   ! idegfd
      iinvec(4) = 0   ! ipoint
      iinvec(5) = 1   ! ispec
      iinvec(6) = 3   ! ichnrm (max norm)
      if ( .not. stationarysol .or. niter>1 ) finished = .true.
      do ieqn = 1, ncopeqtim

!     --- || u^(k+1)-u^k||

         if ( lineartim==3 ) then

!        --- Updated Lagrangian, use du stored in isolprec

            iinvec(5) = 0   ! ispec
            call sepmanvec ( iinvec, rinvec, isolprec, isolprec, isolprec )

!           --- Clear array isolprec

            usoltemp => ks(isolprec(1,ieqn))%sol
            usoltemp = 0

         else

!        --- Standard case

            call sepmanvec ( iinvec, rinvec, isolut2dred(Teq(ieqn)%iseqfr), &
                             isolprec(1,ieqn), isolut2dred )

         end if  ! ( lineartim==3 )
         diffsl(ieqn,1) = rinvec(1)
         if ( .not. stationarysol .or. niter>1 ) then

!        --- niter>1, a check on convergence is possible

            if ( niter>1 ) alam = diffsl(ieqn,1)/diffsl(ieqn,2)
            eps = absaccurtim
            if ( debug ) write(irefwr,2) 'eps, alam', eps, alam
            if ( relaccurtim>0d0 ) then

!           --- Relative accuracy required, compute ||u^(k+1)||

               iinvec(5) = 0
               call sepmanvec ( iinvec, rinvec, &
                                isolut2dred(Teq(ieqn)%iseqfr), &
                                isolut2dred, isolut2dred )
               eps = eps+relaccurtim*rinvec(1)

            end if
            if ( niter>1 .and. alam>0d0 ) eps = eps*(1d0-alam)/alam
            finished = finished .and. diffsl(ieqn,1)<eps .and. &
                       niter>=minitertim
            if ( debug ) write(irefwr,2) 'eps', eps

         end if
         write(text(23:25),230) niter
230      format(i3)
         if ( iprinttim>=100 ) &
            call sepprintv ( isolut2dred(Teq(ieqn)%iseqfr), text )
      end do

      if ( .not. finished .and. t>=tthreshold .and. niter>=minitertim ) then

!     --- Check divergence

         diver = .false.
         do i = 1, ncopeqtim
            if ( diffsl(i,2)<diffsl(i,1)*.99d0 .and. &
                 diffsl(i,3)<diffsl(i,2)*.99d0 ) diver = .true.
         end do
         if ( diver ) then
            call errint ( niter, 1 )
            if ( ireturtim==0 ) then
               call errsub ( 1906, 1, 0, 0 )
            else
               call errwar ( 1906, 1, 0, 0 )
            end if
            if ( ireturtim==0 .or. nwarn<=mxwarn ) then

!           --- Print extra information

               do i = 1, ncopeqtim
                  write ( irefwr, 380 ) i, diffsl(i,1), diffsl(i,2)
380               format ( i5, 10x, ss, 1pe13.2e2, 10x, 1pe13.2e2 )
               end do
            end if

         end if

      end if
      if ( abs(iprinttim)>=1 ) then

!     --- Print output required

         if ( abs(iprinttim)>=2 ) then

!        --- iprinttim>=2, print extra information

            if ( niter==1 ) then
               if ( .not. stationarysol ) write(irefwr,390) t
390            format ( 'Time = ', f12.4 )
               write(irefwr,400)
400            format ( /' Information about the iteration process'/ &
                         ' Iteration  Equation', &
                         '    || u(n) - u(n-1) || ')
            end if
            write (irefwr,410) niter, 1, diffsl(1,1)
410         format ( 2x, i5, 5x, i5, 7x, ss, 1pe13.2e2 )
            do i = 2, ncopeqtim
               write (irefwr,420) i, diffsl(i,1)
420            format ( 12x, i5, 7x, ss, 1pe13.2e2 )
            end do

         end if

         if ( finished ) then
            write(irefwr,440) niter
440         format ( ' Convergence has been reached after ', i3, &
                     ' steps' )
         end if

      end if

!     --- Copy diffsl(1) into 2

      do i = 1, ncopeqtim
         diffsl(i,3) = diffsl(i,2)
         diffsl(i,2) = diffsl(i,1)
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'finished', finished
         write(irefwr,*) 'End sepcnvchk'

      end if  ! ( debug )
      call erclos ( 'sepcnvchk' )

      end
