      subroutine sepcompreaccur ( icurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-11-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute reaction force along curves
!
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     reaction_force
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulevecs
      use sepmodulesmf
      use sepmodulemain
      use sepmodulekprob
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelm
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: icurvs(*)

!     icurvs         i    Curve numbers along which the reaction force
!                         must be computed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, allocatable :: points(:), listelms(:)
      integer :: i, nnodes
      logical :: clear
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     clear          If true the output vector must be cleared before
!     i              Counting variable
!     listelms       Array of length nelem, containing information about the
!                    elements
!                    if listelms(i) = 0, the element must be skipped,
!                    if listelms(i) = 1, the element must be taken into account
!     nnodes         Number of nodes
!     points         Contains nodes of user points/curves/surfaces
!                    to be considered
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     SEPASSEMSTREAC Reserve space in array IBUFFR with respect to
!                    subroutine sepcompreaccur
!                    Fill array ipoint in common CSUBWK
!     SEPREACBODY    Compute the reaction force along a number of given curves
!     SEPREACSTART   Perform some tests
!                    First routine to be called in sepcompreaccur
!     SEPREACSTART1  Start of assembly process
!                    Make all initializations, fill modules and declare arrays
!     SEPRMMATSTR2   nullify arrays in module sepmodulematstr
!     TO0100REAC1    Store nodes in which reaction force has to be computed
!                    in iwork
!     TO0112CLEAR    Clear part of output array not corresponding to nodes
!                    in inodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcompreaccur' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcompreaccur'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

!     --- Perform some checks
!         Make initializations
!         Allocate all arrays related to ibuffr
!         Create output array
!         Allocate flipflop array
!         Fill information concerning old solutions
!         Fill modules

      if ( debug ) write(irefwr,*) 'before sepreacstart'
      call sepreacstart
      if ( ierror/=0 ) go to 1000

!     --- If there are no unknowns the matrix must not be filled and
!         everything is skipped

      if ( nusol-nbound-npboun==0 ) go to 1000

!     ---  Start of assembly process
!          Make some initializations, fill modules and declare arrays

      if ( debug ) write(irefwr,*) 'before sepreacstart1'
      call sepreacstart1
      if ( ierror/=0 ) go to 1000

      numold = max(1,numold)
      allocate ( points(npoint), listelms(nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint+nelem, 'points' )

!     --- reaction forces over curves
!         First find all points at the curves in icurvs
!         Store in array iwork

      if ( debug ) write(irefwr,*) 'before to0100reac1'
      call to0100reac1 ( clear, iinder, icurvs, &
                         skipelgrps, points, nnodes, listelms )
      if ( ierror/=0 ) go to 1000

!     --- Initialize some parameters for the element groups

      if ( debug ) write(irefwr,*) 'before sepassemstreac'
      call sepassemstreac

!     --- compute reaction force

      if ( debug ) write(irefwr,*) 'before sepreacbody'
      call sepreacbody ( listelms )

      if ( debug ) write(irefwr,*) 'before to0112clear'
      call to0112clear ( vectout, points, indprf, &
                         kprobf, nunkp, nnodes, npoint )

      do i = 1, numtypesols
         if ( associated(indoldsols(i)%index3) ) &
            nullify (indoldsols(i)%index3)
         if ( associated(indoldsols(i)%index4) ) &
            nullify (indoldsols(i)%index4)
      end do  ! i = 2, numtypesols

      deallocate ( istold, istold1, oldsols, indoldsols, points, &
                   listelms, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'istoldlc' )

      ks(SMF%ioutvc)%jcart = jcart

      if ( debug ) write(irefwr,*) 'before seprmmatstr2'
      call seprmmatstr2  ! nullify arrays in modulematstr

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcompreaccur'

      end if  ! ( debug )
      call erclos ( 'sepcompreaccur' )

      end subroutine sepcompreaccur
