      subroutine sepenthlinsear ( fold, f, stpmax, check, tolx, &
                                  iseqtemp, ireadcf, iprint )
! ======================================================================
!
!        programmer    John Brusche
!        version  1.0  date 12-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform a line search in a Newton iteration to prevent
!     unwanted divergence
!     Old subroutine prenthlinesear
! **********************************************************************
!
!                       KEYWORDS
!
!     line_search
!     newton
!     phase_change
!     convergence
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemain
      use sepmodulevecs
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iseqtemp, ireadcf
      integer, intent(inout) :: iprint
      double precision, intent(in) :: fold, stpmax, tolx
      double precision, intent(inout) :: f
      logical, intent(out) :: check

!     check          o    If true, the calling routine needs to check on
!                         convergence on deltatemp
!     f             i/o   0.5d0*|F|^2
!     fold           i    Old value of f
!     iprint        i/o   Print level, 0 is no printing
!     ireadcf        i    Sequence number of input for coefficients
!     iseqtemp       i    Sequence number of temperature in solution vector
!     stpmax         i    Maximum step size allowed
!     tolx           i    Maximum tolerated error on the temperature
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, parameter :: alf=1d-4
      integer :: iincopy(2), iter, i, normtype, lsmaxiter, irhsdlc(2)
      integer, target :: tempoldcp
      integer :: iprintsav

      double precision :: a, alam, alam2, alamin, b, disc, f2, rhs1, rhs2, &
                          slope, tmplam, norm, f1, f3, tmpalam1, &
                          tmpalam2, tmpalam1tmp, tmpalam2tmp
      logical :: finished, bisection, cubic, quadratic, linear, &
                 linesearch, halving
      double precision, pointer :: tempoldcop(:), rhsdlc1(:), rhsdlc2(:)

!     a              Help variable
!     alam           Scaling parameter
!     alam2          Previous value of scaling parameter
!     alamin         Minimum value of scaling parameter
!     alf            Parameter for determining suffient function decrease
!     b              Help variable
!     bisection      A bisection is used to determine parameter alpha
!                    during line search
!     cubic          A cubic approximation polynomial is used to
!                    determine parameter alpha during line search
!     disc           Discriminant
!     f1             previous value of f
!     f2             previous value of f
!     f3             previous value of f
!     finished       If true, the linesearch loop is terminated
!     halving        A halving of alpha is used for consecutive
!                    iterations of the Newton Raphson iterative scheme
!     i              Counting variable
!     iincopy        defines which part of the array is copied
!                    iincopy must be filled as follows:
!                    1:  Number of entries filled by the user
!                        If 0 or 1 only default values are used
!                    2:  copy_part defines whether the complete array is
!                        copied or not. Possible values:
!                        0  The complete array is copied
!                        1  Only the part not corresponding to essential
!                           boundary conditions is copied
!                        2  Only the part corresponding to essential boundary
!                           conditions is copied
!                        3  Copy the coordinates of the mesh as stored in IOLD
!                           (=kmesh) into INEW
!                       Default value: 0
!                    3:  IPROB, problem number
!                        Only used if copy_part = 3
!                    Default value: 1
!     iprintsav      Saves value of iprint
!     irhsdlc        Right-hand side vectors R1 and R2
!     iter           Counting variable that keeps track of the number of
!                    iterations
!     linear         A linear approximation polynomial is used to
!                    determine parameter alpha during line search
!     linesearch     Line search is used to improve/ensure converge
!                    of the Newton Raphson iterative scheme
!     lsmaxiter      Maximum number of linesearch iterations
!     norm           Norm of a vector
!     normtype       Type of norm used. 1: infinity norm 2: l2-norm
!     quadratic      A quadratic approximation polynomial is used to
!                    determine parameter alpha during line search
!     rhs1           Help variable
!     rhs2           Help variable
!     rhsdlc1        First right-hand side
!     rhsdlc2        Second right-hand side
!     slope          Help variable
!     tempoldcop     Copy of tempold
!     tempoldcp      Copy of tempold
!     tmpalam1       Intermediate value of parameter alam
!     tmpalam1tmp    Intermediate value of parameter alam
!     tmpalam2       Intermediate value of parameter alam
!     tmpalam2tmp    Intermediate value of parameter alam
!     tmplam         Intermediate value of parameter alam
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision prenthlinesea1

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INSTOP         Stop the program
!     PRENTHLINESEA1 Compute the minimum value of scaling parameter alam
!     PRINRL         Print 1d real vector
!     sepcreasolbf1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPENTHBLDRHS  Build the right-hand side vector R2 for line search
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2790   roundoff problem in line search
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepenthlinsear' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepenthlinsear'
         iprintsav = iprint
         iprint = 2
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      bisection = .false.  ! not yet used
      linear = .false.
      quadratic = .true.
      cubic = .false.
      normtype = 1
      linesearch = .true.
      halving = .false.
      lsmaxiter = 10

      irhsdlc = 0

!     --- create some arrays

      call sepcreasolbf1 ( tempoldcp, ks(isolut2d(iseqtemp))%typevec, &
                           ks(isolut2d(iseqtemp))%ivec, 'tempoldcp', 0 )
      tempoldcop => usoltemp

      call sepcreasolbf1 ( irhsdlc(1), ks(isolut2d(iseqtemp))%typevec, &
                           ks(isolut2d(iseqtemp))%ivec, 'irhsdlc1', 0 )
      rhsdlc1 => usoltemp
      call sepcreasolbf1 ( irhsdlc(2), ks(isolut2d(iseqtemp))%typevec, &
                           ks(isolut2d(iseqtemp))%ivec, 'irhsdlc2', 0 )
      rhsdlc2 => usoltemp

      check = .false.
      iter = 0

      if (linesearch .and. .not. linear .and. .not. quadratic &
                     .and. .not. cubic ) then
         write(irefwr,*) &
                      'error: no linesearch interpolation type selected'
         call instop

      end if

!     --- scale if attempted step is too big

      if (normtype==2) then

         norm = norm2 ( deltatemp )/size(deltatemp)

      else

         norm = max ( maxval(deltatemp), -minval(deltatemp) )

      end if

      norm = npoint*norm
      if ( debug ) write(irefwr,2) 'norm, stpmax', norm, stpmax

      if (norm>stpmax) then
         a = stpmax/norm
         deltatemp = deltatemp*a
      end if

      slope = dot_product( deltatemp, grad )

      if ( debug ) write(irefwr,2) 'norm, slope', norm, slope

      if (slope>=0d0) then

!     --- slope non-negative, roundoff problem in linesearc

         call errsub ( 2790, 0, 0, 0 )
         call prinrl ( deltatemp, nusol, 'deltatemp' )
         call prinrl ( grad, nusol, 'gradf')
         go to 1000

      end if

!     --- Compute minimum value of alam

      alamin = prenthlinesea1 ( deltatemp, tempold, npoint, TOLX )

      if ( debug ) write(irefwr,2) 'alamin', alamin

!     --- Always attempt full Newton step first

      alam = 1d0

      finished = .false.
      f1 = fold

      do while (.not. finished .and. iter<lsmaxiter)
         iter = iter + 1

!        --- compute tempnew = tempold + lambda*deltatemp

         temperature = deltatemp*alam+tempold

!        --- Build right-hand side

         isolut => isolut2d(iseqtemp)
         call sepenthbldrhs ( iprob, ireadcf, irhsdlc )

!        --- retrieve saved R2 and store in irhds(2)

         iincopy(1) = 0
         rhsdlc2 = res2

!        --- add both right-hand sides and store in irhsdlc(1)

         rhsdlc1 = rhsdlc1+rhsdlc2

!        --- compute f = 0.5d0*residual*residual for initial T
!            we use a copy of irhsdlc(1,1)
!            Keep in mind that irhsdlc(1,1) contains -1d0*residual !!!

         rhsdcp = rhsdlc1*0.25d0

         f = dot_product( rhsdcp, rhsdcp )

         if ( debug ) write(irefwr,2) 'f, alam', f, alam

!        --- compute underrelaxation parameter alam (lambda) when needed
!            in order to ensure sufficient function decrease for the
!            Newton step
!            Possible methods:
!            1) linesearch (linear, quadratic and cubic approximation)
!            2) bisection (partly implemented)
!            3) halving (simply halving alam each linesearch iteration)

         if (alam<alamin) then

!        --- convergence on deltatemp, calling routine should check for
!            convergence
!            set isolut2d equal to tempold.

            temperature = tempold
            if ( iprint>2 ) write(irefwr,*) 'convergence on deltatemp'
            finished = .true.
            check = .true.

         else if (f<=fold+ALF*alam*slope) then

!        --- Sufficient function decrease

            if ( iprint>2 ) &
               write(irefwr,*) 'Sufficient function decrease'
            finished = .true.

         else if (linesearch) then  ! Backtrack

            if ( iprint>2 ) write(irefwr,*) 'Backtracking'

            if (abs(alam - 1d0) <=1d-5) then

!           --- First time

               if (linear) then
                  tmplam = alam - f/slope
               else
                  tmplam = -slope/(2d0*(f-fold-slope))
               end if

            else

!           --- subsequent backtracks

               if (linear) then
                  tmplam = alam - f/slope
               else if (quadratic) then
                  a = (f - f2 - slope*(alam-alam2))/(alam2 - alam)**2
                  b = slope - 2*a*alam2
                  tmplam = alam2 - slope/(2d0*a)
               else ! cubic
                  rhs1=f-fold-alam*slope
                  rhs2=f2-fold-alam2*slope
                  a=(rhs1/alam**2-rhs2/alam2**2)/(alam-alam2)
                  b=(-alam2*rhs1/alam**2+alam*rhs2/alam2**2)/ &
                           (alam-alam2)

                  if (abs(a)<=1d-5) then

                     tmplam=-slope/(2d0*b)

                  else

                     disc = b*b-3d0*a*slope

                     if (disc<0d0) then

                        tmplam = 0.5d0*alam

                     else if (b<=0d0) then

                        tmplam = (-b+sqrt(disc))/(3d0*a)

                     else

                        tmplam=-slope/(b+sqrt(disc))

                     end if ! (disc<0d0)

                  end if ! (linear)
               end if ! (a==0d0)

            end if ! (alam==1d0)

!           --- lambda<=0.5*lambda_1

            if (tmplam>0.5d0*alam) tmplam = 0.5d0*alam

         else if (bisection) then

            if (abs(alam - 1d0)<1d-6) then

               write(irefwr,*) 'bisection, first time'
               tmplam = 0.5d0
               tmpalam1 = 0d0
               tmpalam2 = 1d0
               f3 = f

            else

               write(irefwr,*) 'bisection, subsequent times'

               tmpalam1tmp = 0.5d0*(tmpalam1 + tmplam)
               tempoldcop = tempold+deltatemp*tmpalam1tmp

!              --- Build right-hand side

               isolut => tempoldcp
               call sepenthbldrhs ( iprob, ireadcf, irhsdlc )

!              --- retrieve saved R2 and store in irhds(1,2)

               rhsdlc2 = res2+rhsdlc1

!              --- compute f = 0.5d0*residual*residual for initial T
!                  we use a copy of irhsdlc(1,1)
!                  Keep in mind that irhsdlc(1,1) contains -1d0*residual !!!

               rhsdcp = rhsdlc1*0.25d0

               tmpalam2tmp = 0.5d0*(tmpalam2 + tmplam)
               tempoldcop = tempold+deltatemp*tmpalam2tmp

!              --- Build right-hand side

               isolut => tempoldcp
               call sepenthbldrhs ( iprob, ireadcf,irhsdlc )

!              --- retrieve saved R2 and store in irhds(1,2)

               rhsdlc2 = res2+rhsdlc1

!              --- compute f = 0.5d0*residual*residual for initial T
!                  we use a copy of irhsdlc(1,1)
!                  Keep in mind that irhsdlc(1,1) contains -1d0*residual !!!

               rhsdlc2 = rhsdcp*0.25d0

               if ( debug ) write(irefwr,2) 'f, f1, f3', f,  f1, f3

               if (f1<f3) then
                  if (f1<f) then
                     tmpalam2 = tmplam
                     tmplam = 0.5d0*(tmpalam1 + tmplam)
                     f3 = f
                  else
                     write(irefwr,*) 'divergence in bisection'
                     call instop
                  end if
               else
                  write(irefwr,*) 'divergence in bisection'
                  call instop
               end if

            end if

         else if (halving) then

             tmplam = alam/2d0

         else

!        --- Continue without linesearching

             if ( iprint>2 ) write(irefwr,*) &
                'Insufficient function decrease, no linesearch'
             finished = .true.

         end if ! (alam<alamin)

         if (linesearch .or. bisection .or. halving) then
            alam2 = alam
            f2 = f
            alam = max(tmplam, 0.1d0*alam) ! lambda>=0.1*lambda_1

!           --- Finish when alam is up to 2 significant digits accurate

            if (.not. finished .and. &
                         abs(alam - alam2)/max(alam,alam2)<1d-3) then

               finished = .true.
               if ( iprint>2 ) write(irefwr,*) 'Alam accurate enough'

            end if

            if ( .not. finished .and. iprint>0 ) &
               write(irefwr,*) 'f, alam, iter' , f, alam, iter

         end if ! (linesearch)
      end do ! (.not. finished)

!     --- Remove some arrays

      call seprmsolinfo ( tempoldcp )
      call seprmsolinfo ( irhsdlc(1) )
      call seprmsolinfo ( irhsdlc(2) )

1000  if ( debug ) then

!     --- Debug information

         iprint = iprintsav
         write(irefwr,*) 'End sepenthlinsear'

      end if
      call erclos ( 'sepenthlinsear' )

      end subroutine sepenthlinsear
