      subroutine sepnewmarkst ( isol, irhsdlc, ieqn, irhsdold )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute starting value for Newmark and generalized
!     alpha scheme
!     Old subroutine pr1114start
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
!     newmark
!     generalized_alpha
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      use sepmodulematr
      use sepmoduleintmat
      use sepmoduleuser
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol, irhsdlc, ieqn, irhsdold(2)

!     ieqn           i    Equation sequence number
!     irhsdlc       i/o   Standard SEPRAN array, containing information of the
!                         large vector
!     irhsdold       o    Sepran array to store the vector
!                         alpha_f F^n - alpha_m M a^n  - alpha_f S u^n
!     isol           o    Standard SEPRAN array containing information of the
!                         solution array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iwork, ichois, ihelp, ihelp1, i, iproblc
      double precision factor

!     factor         Multiplication factor
!     i              Counting variable
!     ichois         Choice parameter
!     ihelp          integer work array of length 5 to store sepran vector
!     ihelp1         integer work array of length 5 to store sepran vector
!     iproblc        Local value of iprob
!     iwork          integer work array of length 5 to store sepran vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRSUB         Error messages
!     PRCHANGEDIAGMT Change zero diagonal elements of matrix by epsilon
!     SEPCOPYMT      Copy matrix
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPCREAVEC     Create a new sepran vector inew with the same properties
!                    as the old vector iold
!     SEPMAVER       perform matrix vector multiplication
!     SEPPRINTV      Print solution vector or vector of special structure
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     SEPTIMBUILDMAT Build mass matrix, stiffness matrix and right-hand side
!     SEPTIMSOLVE    Solve system of linear equations
!     SEPTIMUSER     Fill arrays iuser and user for a specific
!                    problem in a sequence of problems
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2891   Local transformations have not been implemented
!    2892   beta_g = 0, not allowed in Newmark or generalizd alpha
! **********************************************************************
!
!                       PSEUDO CODE
!
!      M  a^0 + S u^0 = F^0, store in third vector
!      The boundary condition for a^0 is computed from:
!      u^(n+1)=u^n+v^ndt+a^n dt^2/2
!      The vector a^0 is filled as:
!      2/dt^2 ( u^(n+1)-u^n-v^n dt)
!
!      In case of Newton the matrix contains J = dS/du (the jacobian)
!      and F contains f - S u, in other words in that case we have to
!      solve M a^0 = F^0
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepnewmarkst' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepnewmarkst'
         write(irefwr,2) 't_begin', t
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( abs(beta_g)<1d-10 ) then

!     --- beta_g = 0 not allowed

         call erreal ( beta_g, 1 )
         call errsub ( 2892, 0, 1, 0 )
         go to 1000

      end if  ! ( abs(beta_g)<1d-10 )

!     --- Set first positions of work arrays to zero to avoid
!         problems with memory management

      ihelp = 0
      ihelp1 = 0
      iwork = 0

!     --- Newmark or generalized alpha
!         M  a^0 + S u^0 = F^0, store in third vector
!         Compute starting addresses for the arrays iuser and user

      iproblc = iprob
      iprobtim = ks(isolut2dred(iseqfrtim))%iprob
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprobtim)
      kprobloc => kprobpart%kprobloc
      if ( kprobloc(13)>0 ) then

!     --- Local transformations have not been implemented yet

         call errsub ( 2891, 0, 0, 0 )
         go to 1000

      end if  ! ( kprobloc(13)>0 )

!     --- Fill information about the coefficients
!         Either in iuser/user or in ibuffr/buffer from ipiusr/ipuser

      call septimuser ( .true., ieqn )
      if ( ierror/=0 ) go to 1000

!     --- Build matrices at old time level

      call septimbuildmat ( isol, irhsdlc, isolut2dred, .true., ieqn )
      if ( ierror/=0 ) go to 1000

      if ( lineartim<3 ) then

!     --- Compute S u^0,  store in iwork (including boundary conditions)

         ichois = 5
         call sepmaver ( stiffmat, isolut2dred(iseqfrtim), &
                         iwork, ichois )
         if ( ierror/=0 ) go to 1000
         if ( debug ) then
            call sepprintv ( iwork, 'S u^0' )
            call sepprintv ( irhsdlc, 'F' )
         end if  ! ( debug )

!        --- Copy right-hand side in ihelp

         call sepcopyvc ( irhsdlc, ihelp )
         if ( ierror/=0 ) go to 1000

!        --- compute right-hand side, i.e. F^0 - S u^0, store in ihelp

         workv => ks(iwork)%sol
         help => ks(ihelp)%sol
         help = help-workv
         if ( debug ) call sepprintv ( ihelp, 'F-S u^0' )

      else

!     --- Copy F^0 into ihelp

         call sepcopyvc ( irhsdlc, ihelp )

      end if  ! ( lineartim<3 )
      if ( ierror/=0 ) go to 1000

      if ( abs(alpha_f)>0d0 .or. abs(alpha_m)>0d0 ) then

!     --- alpha_f and/or alpha_m unequal to 0, Fill ihelp1

         call sepcopyvc ( ihelp, ihelp1 )

      end if  ! ( abs(alpha_f)>0d0 .or. abs(alpha_m)>0d0 )
      if ( ierror/=0 ) go to 1000

!     --- Create start vector a^0 (in fact the boundary conditions)
!         a^0 i = 2/dt^2 ( u^(n+1)-u^n-v^n dt)

      factor = 2d0/tstep**2
      call sepcreavec ( isol, isolut2dred(Teq(3)%iseqfr) )
      if ( ierror/=0 ) go to 1000
      u1 => ks(isol)%sol
      u2 => ks(isolut2dred(iseqfrtim))%sol
      u3 => ks(isolut2dred(Teq(2)%iseqfr))%sol
      unew => ks(isolut2dred(Teq(3)%iseqfr))%sol
      unew = factor*(u1-u2-tstep*u3)

      if ( itimbc/=1 ) then

!     --- Compute effect of boundary conditions and add to right-hand side F^0
!         This is only necessary if the boundary conditions are time-dependent
!         Otherwise the accelaration is 0 at the boundary

         call sepmaver ( massmat, isolut2dred(Teq(3)%iseqfr), &
                         ihelp, 7 )
         if ( debug ) call sepprintv ( ihelp, 'rhs' )

      end if  ! ( itimbc/=1 )
      if ( ierror/=0 ) go to 1000

!     --- Solve system of equations M a^0 = F^0 - S u^0
!         To prevent overwriting of massmat, we copy the matrix into
!         totmat, before solving

      call sepcopymt ( massmat, totmat )
      if ( ierror/=0 ) go to 1000

!     --- Next zero diagonals are replaced by eps
!         This is meant for incompressible materials, where the
!         pressure part is completely zero

      iseqmt = totmat
      if ( .not. realmt(iseqmt)%diagonal ) call prchangediagmt

!     --- Solve system of equations

      call septimsolve ( totmat, isolut2dred, ihelp, 3, irhsdlc, .true. )
      if ( ierror/=0 ) go to 1000

      if ( abs(alpha_f)>0d0 .or. abs(alpha_m)>0d0 ) then

!     --- alpha_f and/or alpha_m unequal to 0, Fill irhsdold with
!         alpha_f (F^0 - S u^0) - alpha_m M a^0
!         First compute M a^0 including boundary conditions
!         Store in ihelp

         call sepmaver ( massmat, isolut2dred(Teq(3)%iseqfr), &
                         ihelp, 5 )

         help => ks(ihelp)%sol
         help1 => ks(ihelp1)%sol
         call sepcreavec ( ihelp, irhsdold(1) )
         rhsdcp => ks(irhsdold(1))%sol
         rhsdcp = alpha_f*help1-alpha_m*help
         if ( debug ) call sepprintv ( irhsdold(1), 'F*^0' )

      end if  ! ( abs(alpha_f)>0d0 .or. abs(alpha_m)>0d0 )
      if ( ierror/=0 ) go to 1000

!     --- remove iwork

      if ( iwork>0 ) call seprmsolinfo ( iwork )
      if ( ihelp>0 ) call seprmsolinfo ( ihelp )
      if ( ihelp1>0 ) call seprmsolinfo ( ihelp1 )
      iprob = iproblc

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,2) 't_end start', t
         call sepprintv ( isolut2dred(iseqfrtim), 'u^0' )
         call sepprintv ( isolut2dred(Teq(2)%iseqfr), 'du/dt^0' )
         call sepprintv ( isolut2dred(Teq(3)%iseqfr), 'd^2u/dt^2^0' )
         if ( Teq(4)%iseqfr>0 ) &
            call sepprintv ( isolut2dred(Teq(4)%iseqfr), 'u_tot^0' )
         write(irefwr,*) 'End sepnewmarkst'

      end if  ! ( debug )
      call erclos ( 'sepnewmarkst' )

      end subroutine sepnewmarkst
