      subroutine sepnewmarkstep ( isol, irhsdlc, first, ieqn, irhsdold )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 28-05-2014 New call to prtimsolve
!        version  1.0  date 08-07-2007
!
!   copyright (c) 2007-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform one step in the solution of the wave equation
!     Either Newmark or generalized alpha is used
!     Old subroutine prtm00_14
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulematr
      use sepmodulevecs
      use sepmodulemain
      use sepmoduleintmat
      use sepmoduleuser
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol, irhsdlc, ieqn, irhsdold(2)
      logical first

!     first         i/o   If true it concerns the first step
!     ieqn           i    Equation sequence number
!     irhsdlc       i/o   Standard SEPRAN array, containing information of the
!                         large vector
!     irhsdold       i    Sepran array to store the vector
!                         alpha_f F^n - alpha_m M a^n  - alpha_f S u^n
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ichois, iwork, ihelp, iincopy(2)
      double precision :: f1
      double precision, pointer :: STIFF(:), TOTM(:), MASS(:)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     MASS           Refers to mass matrix
!     STIFF          Refers to stifness matrix
!     TOTM           Refers to combination of mass and stiffness matrix
!     f1             Multiplication factor
!     ichois         Choice parameter for subroutine MAVER
!     ihelp          Standard SEPRAN array of length 5 containing information
!                    concerning a work vector which must of the type of the
!                    solution vector
!     iincopy        User input array for subroutine COPYVEC
!     iwork          Standard SEPRAN array of length 5 containing information
!                    concerning a work vector which must of the type of the
!                    solution vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCOPYMT      Copy matrix
!     SEPCOPYVEC     copy Sepran array
!     SEPMAVER       perform matrix vector multiplication
!     SEPPRINTV      Print solution vector or vector of special structure
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     SEPTIMBUILDMAT Build mass matrix, stiffness matrix and right-hand side
!     SEPTIMSOLVE    Solve system of linear equations
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!      The Newmark methods is given by:
!
!      u^n+1 = u^n + dt v^n + dt^2/2 [ ( 1 - 2 beta_g) a^n + 2 beta_g a^n+1 ] (1)
!      v^n+1 = v^n + dt [ (1-gamma) a^n + gamma a^n+1 ]                   (2)
!      M a^n+1 + S u^n+1 = F^n+1                                          (3)
!
!      with u the displacement, v the velocity and a the acceleration
!      At t=0: a^n+1 is computed from (3)
!      (1) is rewritten as u^n+1 = h^n + dt^2 beta_g a^n+1
!      with h^n = dt v^n + dt^2/2 [ ( 1 - 2 beta_g) a^n
!
!      In the next steps (1) is substituted in (3) to compute u^n+1
!      After that a^n+1 and v^n+1 are computed from (1) and (2)
!      The Newmark method can also be written as:
!      (M + beta_g S dt^2) u^n+1 = -[M + (1/2-gamma+beta_g) S dt^2] u^n-1 + &
!      [2M - (1/2+gamma-2 beta_g) S dt^2] u^n + dt^2 F* &
!      with F* = (1/2-gamma+beta_g)F^n-1 + (1/2+gamma-2 beta_g)F^n + beta_g F^n+1
!
!      beta_g = 0, gamma = 1/2 gives the central difference scheme
!      beta_g = 1/4, gamma = 1/2 is second order accurate
!
!      u^n+1 follows from:
!      ( 1/(beta_g dt^2) M  + S ) u^n+1 = F^n+1 + ( 1/(beta_g dt^2) M h^n     (4)
!      The boundary conditions for a^n+1 follow from (1), hence
!
!      Computation is as follows:
!
!      Compute matrices and right-hand side at new time level
!      Compute vector h = u^n + dt v^n + dt^2/2 ( 1 - 2 beta_g) a^n
!      Update v^n by v^n := v^n + dt(1-gamma) a^n
!      Solve u^n+1 from  (4)
!      a^n+1 = (u^n+1 - h^n ) / ( beta_g dt^2 )
!      v^n+1 = v^n + gamma dt a^n+1
!
!      The generalized alpha scheme replaces (3) by:
!
!      M (1-alpha_m) a^n+1 + S (1-alpha_f) u^n+1 = (1-alpha_f) F^n+1 + F*, &
!      with F* = alpha_f F^n - M alpha_m a^n - S alpha_f u^n
!      The vector F* is stored in irhsdold
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepnewmarkstep' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepnewmarkstep'
         call sepprintv ( isol, 'u^n/bc' )
         write(irefwr,*) 'first', first
         write(irefwr,2) 't', t
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,1) 'ichtim', ichtim
         write(irefwr,2) 'beta_g, alpha_f, alpha_m', &
                          beta_g, alpha_f, alpha_m
      end if  ! ( debug )

!     --- Build matrices and right-hand side at t^n+1

      call septimbuildmat ( isol, irhsdlc, isolut2dred, first, ieqn )
      if ( ierror/=0 ) go to 1000

      IM => intmt(realmt(stiffmat(1))%intmatref)
      iprob = IM%iprob

      STIFF => realmt(stiffmat(1))%S_ii
      MASS => realmt(massmat)%S_ii

      rhsdcp => ks(irhsdlc)%sol

!     --- create   :  Mattot = (1-alpha_m)/(beta_g dt^2) M + (1-alpha_f) S

      f1 = (1d0-alpha_m)/(beta_g*tstep**2)
      if ( debug ) write(irefwr,2) 'f1', f1
      if ( first .or. Teq(ieqn)%imasstot==0 ) then

!     --- First step, or matrices time-dependent
!         create   :  Mattot

         call sepcopymt ( stiffmat, totmat )
         TOTM => realmt(totmat)%S_ii
         TOTM = (1d0-alpha_f)*TOTM
         if ( realmt(massmat)%diagonal ) then
            TOTM(IM%diag) = TOTM(IM%diag)+f1*MASS
         else
            TOTM = TOTM+f1*MASS
         end if  ! ( realmt(massmat)%diagonal )

      else

         TOTM => realmt(totmat)%S_ii

      end if  ! ( first .or. iintlc(8)==0 )
      if ( debug ) call sepprintv ( irhsdlc, 'f^n' )
      if ( ierror/=0 ) go to 1000

!     --- Compute M h,  store in iwork (including boundary conditions)

      ichois = 5
      iwork = 0
      call sepmaver ( massmat, irhsdold(2), iwork, ichois )
      if ( debug ) call sepprintv ( iwork, 'M h' )
      if ( ierror/=0 ) go to 1000

      workv => ks(iwork)%sol

!     --- Store (1-alpha_f) f + (1-alpha )/ (beta_g dt^2) M h + F*  into iwork
!         (including boundary conditions)

      if ( irhsdold(1)>0 ) then

!     --- irhsdold(1)>0, hence Generalized alpha method

         if ( debug ) call sepprintv ( irhsdold(1), 'F^*' )
         u1 => ks(irhsdold(1))%sol
         workv = (1d0-alpha_f)*rhsdcp+f1*workv+u1

      else

!     --- irhsdold(1)=0, hence Newmark method

         workv = rhsdcp+f1*workv

      end if  ! ( irhsdold(1)>0 )
      if ( debug )  call sepprintv ( iwork, 'rhs' )
      if ( ierror/=0 ) go to 1000

!     --- Compute effect of boundary conditions and add to right-hand side

      call sepmaver ( totmat, isol, iwork, 7 )

      if ( debug ) call sepprintv ( iwork, 'rhsd' )
      if ( ierror/=0 ) go to 1000

      if ( lineartim==3 .or. lineartim==4 ) then

!     --- Updated Lagrange formulation
!         Subtract (1-alpha_m)/(beta_g dt^2) M un from the rhs
!         First compute M un and store in ihelp

         ihelp = 0
         call sepmaver ( massmat, isolut2dred(iseqfrtim), ihelp, 5 )

!        --- Next subtract

         help => ks(ihelp)%sol
         workv = workv - f1*help

!        --- Delete array ihelp

         call seprmsolinfo ( ihelp )

!        --- Solve system of equations and store in isol (du)

         call septimsolve ( totmat, isol, iwork, ieqn, irhsdlc, first )

!        --- un := du + un, store result in isolut2dred(.,1)

         u1 => ks(isol)%sol
         u2 => ks(isolut2dred(iseqfrtim))%sol
         u2 = u1+u2

         if ( debug ) then
            call sepprintv ( isol, 'du' )
            call sepprintv ( isolut2dred(iseqfrtim), 'u' )
         end if  ! ( debug )

      else

!     --- Standard case
!         Multiply by inverse of Mattot and store in isolut2dred(.,1)
!         First copy boundary conditions from isol into isolut2dred(1)

         iincopy(1) = 2   ! number of entries in iincopy
         iincopy(2) = 2   ! only part corresponding to boundary conds is copied
         call sepcopyvec ( isol, isolut2dred(iseqfrtim), iincopy )
         call septimsolve ( totmat, isolut2dred(iseqfrtim), iwork, &
                            ieqn, irhsdlc, first )
         if ( debug ) call sepprintv ( isolut2dred(iseqfrtim), 'u^n+1' )

      end if  ! ( lineartim==3, 4 )
      if ( ierror/=0 ) go to 1000

!     --- Remove temporary arrays

      call seprmsolinfo ( iwork )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepnewmarkstep'

      end if  ! ( debug )
      call erclos ( 'sepnewmarkstep' )

      end
