      subroutine seppjparspar ( projvc, seqblocks, globbloc )
! ======================================================================
!
!        programmer    Fred Vermolen
!        version  1.0  date 01-07-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     In this subroutine the sparse projection vector is made for the
!     present block / processor.
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     projection
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleproj
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer seqblocks(iacnodes), globbloc(iacnodes)
      double precision projvc(npoint*nproj)

!     globbloc       i    local block number of a global block number
!     projvc         o    See sepmoduleproj: projU
!     seqblocks      o    Work array of length nblocks (=nsurfs) which
!                         is used to find the sequence of block numbers
!                         in array blockinfo
!                         seqblocks(iblock) = 0: block iblock not present
!                         seqblocks(iblock) = i: block iblock is the i-th block
!                         in blockinfo
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, numneigh, nneigh, inusol, idxprojv, &
              iblock, sendstart, icontext, neighstart, nnproj1
      double precision, allocatable :: usend(:)

!     i              Loop variable
!     iblock         index of block
!     icontext       sequence number of communication with a neighboring block
!     idxprojv       index of projvc
!     inusol         Position of unknown in array USOL
!     j              Loop variable
!     neighstart     lower bound of index
!     nneigh         number of unknowns at the interface that is shared by
!                    a particular neighbor
!     nnproj1        Number of entries in projvc1
!     numneigh       Number of neighboring blocks
!     sendstart      starting address in array sendinfo
!     usend          Array of partly integer and double precision that
!                    contains index of neighboring block and its value
!                    of the matrix E
!                    Part I:  Integers that indicate the index of the
!                             neighbor
!                    Part II: Double precision that gives its corresponding
!                             entry of E
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INIRECEIVEMPI  Receive information from other processor (mpi)
!     INISENDMPI     Send information to other node (mpi)
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Goal: (1) Creating sparse projection vectors from the constant parts
!               and the average overlaps at the interface nodes
!           (2) Send and receive the interface parts of the projection
!               vectors to / from the neighboring blocks / processors
!
!     Semi PSD: prpjSPAR4
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'seppjparspar')
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppjparspar'
         write(irefwr,1) 'nrusol', nrusol
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      projrows(1) = 0

      numneigh = blockunkinfo(1)

      allocate ( usend(nrusol), stat = error )
      if ( error/=0 ) call eralloc ( error, nrusol, 'usend' )
      if ( ierror/=0 ) go to 1000

      projvc(1:nrusol) = 1d0

      globbloc(1:iacnodes) = 0

      if ( debug ) write(irefwr,1) 'numneigh', numneigh

!     --- Fill projrows(1:numneigh) with number of neighbor nodes in
!         neighbor block
!         Fill projvc(1:n) with number of nodes in block and neighbor blocks

      do i = 1, numneigh

         sendstart = blockunkinfo(2*i+1)
         iblock = blockunkinfo(2*i)
         globbloc(iblock) = i
         nneigh = sendinfo(sendstart)
         projrows(i+1) = projrows(i) + nneigh

         do j = 1, nneigh
            inusol = sendinfo(sendstart+j)
            projvc(inusol) = projvc(inusol)+1d0
         end do

      end do  ! i = 1, numneigh

      if ( debug ) then
         call prinin ( projrows, numneigh+1, 'projrows/1' )
         call prinrl ( projvc, nrusol, 'projvc/1' )
      end if  ! ( debug )

      nnproj1 = nrusol + projrows(numneigh+1)

      projvc(1:nrusol) = 1d0/projvc(1:nrusol)
      if ( debug ) call prinrl ( projvc, nrusol, 'projvc/2' )

!     --- Filling of projrows with degrees of freedom

      idxprojv = 0

      do i = 1,numneigh

         sendstart = blockunkinfo(2*i+1)
         nneigh = sendinfo(sendstart)

         do j = 1, nneigh
            idxprojv = idxprojv + 1
            inusol = sendinfo(sendstart+j)
            projcolstmp(idxprojv) = inusol
            projvc(nrusol+idxprojv) = 0d0
         end do  ! j = 1, nneigh

      end do  ! i = 1,numneigh

      call inichkarlen ( idxprojv, nproj*npoint, 'projcolstmp' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,1) 'idxprojv', idxprojv
         call prinin ( projrows, numneigh+1, 'projrows/3' )
         call prinin ( projcolstmp, idxprojv, 'projcolstmp/3' )
         call prinrl ( projvc, nrusol+idxprojv, 'projvc/3' )
      end if  ! ( debug )

!     --- Prepare to send values of projection vectors on interfaces
!         to neighboring processors
!         The second part of projvc is filled with the values of the neighbors

      numneigh = blockunkinfo(1)

      seqblocks(1:iacnodes) = 0

      do j = 1,numneigh
         iblock = blockunkinfo(2*j)
         seqblocks(iblock) = j
      end do

      do i = 1,numneigh
         iblock = blockunkinfo(2*i)
         sendstart = blockunkinfo(2*i+1)
         nneigh = sendinfo(sendstart)
         do j = 1,nneigh
             inusol = sendinfo(sendstart+j)
             usend(j) = projvc(inusol)
         end do

!        --- Send the information to neighboring processors for each
!            interface on a neighbor.

         icontext = 7
         call inisendmpi( usend,nneigh,SPDOUBLE,iblock,icontext )

      end do

!     --- Receive the information from neighboring processors

      do i = 1,numneigh

         icontext = 7
         nneigh = nrusol
         call inireceivempi( usend,nneigh,SPDOUBLE,iblock,icontext )
         sendstart = blockunkinfo(2*seqblocks(iblock)+1)
         neighstart = projrows(globbloc(iblock))

!        --- Fill the projection vectors with entries that are received
!            from the other processors from each neighboring processor.

         do j = 1, nneigh
             projvc(nrusol+neighstart+j) = usend(j)
         end do

      end do

      call inichkarlen ( numneigh+1+idxprojv, nproj*npoint, 'intprj' )

      deallocate ( usend, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'usend' )

1000  call erclos( 'seppjparspar' )
      if ( debug ) then

!     --- Debug information

         call prinin ( projrows, numneigh+1, 'projrows' )
         call prinin ( projcolstmp, idxprojv, 'projcolstmp' )
         call prinrl ( projvc, nnproj1, 'projvc' )
         write(irefwr,*) 'End seppjparspar'

      end if  ! ( debug )

      end subroutine seppjparspar
