      subroutine seppjsubm ( ishale, nlocdgfd )
! ======================================================================
!
!        programmer     Gjalt Wijma
!        version  1.0  date 14-06-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     In this subroutine the submatrix is assembled.
!     Old subroutine prpjsubm
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulekmesh
      use sepmoduleproj
      use sepmodulekprob
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer :: ishale, nlocdgfd

!     ishale      i     Dummy variable, set number of a shale set
!     nlocdgfd    o     Number of degrees of freedom in the subsystem.
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ilocdgfd, idegfd, inodp, first2, i2, i3, &
              jset, jdegfd, jlocdgfd, searchcl, collmn, ncpos, icpos, &
              mcpos(100), mccol(100), i4, jnodp
      double precision coeff, mccoeff(100)
      type (globmat), pointer :: S
      type (intmatrix), pointer :: IM
      integer, pointer :: rows(:), cols(:), diag(:)
      double precision, pointer :: mat(:)

!     IM             Refers to intmt(intmat)
!     S              Refers to global matrix
!     coeff          Dummy variable, contains a coefficient from the large matrix
!     collmn         Dummy variable, column number in the large matrix.
!     cols           array with column numbers
!     diag           Positions of the diagonal elements of the matrix
!     first2      Start of loop.
!     i2          Loop variable.
!     i3          Loop variable.
!     i4          Loop variable.
!     icpos       Loop variable, index into MCPOS, MCCOL and MCCOEFF.
!     idegfd      Dummy variable, number of a degree of freedom.
!     ilocdgfd    Dummy variable, number of a degree of freedom, local
!                 numbering.
!     inodp       Dummy variable, nodal point number.
!     jdegfd      Dummy variable, degree of freedom.
!     jlocdgfd    Dummy variable, local degree of freedom.
!     jnodp       Dummy variable, nodal point number.
!     jset        Dummy variable, set number.
!     mat            elements of matrix
!     mccoeff     Double precision array containing the coeffcients of the
!                 new row of the subsystem.
!                 MCCOEFF(iCPOS) = coefficient with local column number
!                 MCCOL(iCPOS).
!     mccol       Integer array, contains the column numbbers of the
!                 new row of the submatrix.
!                 MCCOL(iCPOS) = column number of coefficient MCCOEF(iCPOS).
!     mcpos       Integer array containing information on the order of
!                 the entries in the arrays MCCOL, MCPOS and MCCOEFF.
!                 If MCPOS(icpos) = 1 then MCCOL(icpos) is the lowest
!                 column number in MCCOL, etc. This array is used to put the
!                 row entries in the submatrix in the right order.
!     ncpos       Number of entries in MCCOL, MCCOEFF and MCPOS.
!     rows           contains pointers to cols
!     searchcl    Dummy variable, column number.
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPCOMMATREV   Fill arrays irowsnew and icolsnew with upper triangular
!                    part if icolsii and irowsii contain the lower triangular
!                    part and vice versa
!     SEPMATREV      Copy matrix from rowwise to columnwise storage
!     SEPPRINTMATR   print large real matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Goal: assembling the submatrix.
!
!     Note that degrees of freedom in the subsystem are called
!     "local degrees of freedom".
!
!     Note that for both the submatrix and the large matrix
!     sparse symmetrical storage is used.
!
!     Note that the column numbers in the submatrix have to
!     be ascending.
!
!     Semi PSD: seppjsubm
!
!     LOOP over all local degrees of freedom
!        Read the global number of degree of freedom iDEGFD from
!        DMAP and copy the diagonal element from the large matrix
!        in the submatrix.
!        LOOP over all connected degrees of freedom
!           IF the connected node lies in the shale set AND
!           has a local degree of freedom number
!           THEN
!              Read the global number of the conn. degree of freedom, &
!              jDEGFD, from DMAP. The next step is to look for the
!              right matrix element in the large matrix. If jDEGFD
!              is lower than iDEGFD then we have to check all
!              column numbers in row iDEGFD, if not then it is
!              vice versa. Once the matrix element is found it can
!              be inserted in the new row in such a way that
!              the (local) column numbers are ascending. This is
!              done by looping over all local column numbers in
!              the new row. If the smallest larger column number
!              is found then the new column number takes it place
!              and all larger column numbers are shifted one to
!              the right.
!           END IF
!        END LOOP
!        Once all connected degrees of freedom are processed the
!        new row is copied into the submatrix.
!     END LOOP
!
!     --- WARNING: in the declaration of MCPOS, MCCOL and MMCOEFF is assumed
!         that there are no more then 100 entries in a row of the large
!         matrix.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppjsubm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppjsubm'
         write(irefwr,1) 'ishale', ishale
         call sepprintmatr ( iseqmt )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      S => realmt(iseqmt)
      IM => intmt(S%intmatref)

      allocate ( rows(size(irowsii)), cols(size(icolsii)), &
                 diag(size(IM%diag)), mat(size(S%S_ii)) )

!     --- Change to rowwise storage

      call sepcommatrev ( rows, cols, irowsii, icolsii, nrusol )
      call sepmatrev ( rows, mat, irowsii, icolsii, S%S_ii, nrusol )
      diag = rows(2:size(irowsii))

      do ilocdgfd = 1, nlocdgfd

!     --- copying the diagonal element

         idegfd = dmap( nusol+ilocdgfd )
         inodp = kprobhinv(idegfd  )

         coeff = mat( diag(idegfd) )

         locmat(ilocdgfd) = coeff

!        --- There are no entries in the new row yet

         ncpos = 0

!        --- Loop over all connected nodes

         do i2 = kmshe1(inodp)+1, kmshe1(inodp+1)

            jnodp = kmshe2(i2)
            jdegfd = kprobh( jnodp )
            jset = nodeset( jnodp )

            if ( jset==ishale ) then

!           --- The connected node lies in the shale set.

              jlocdgfd = dmap( jdegfd )
              if ( jlocdgfd<ilocdgfd ) then

!              --- A new matrix element is added

                  if ( idegfd<jdegfd ) then
                     i3 = rows( jdegfd )
                     searchcl = idegfd
                  else
                     i3 = rows( idegfd )
                     searchcl = jdegfd
                  end if
                  collmn = 0
100               if ( collmn.ne.searchcl ) then
                     i3 = i3+1
                     collmn = cols(i3)
                     go to 100
                  end if

!                 --- Determine the order of the coefficients in the
!                     new row

                  ncpos = ncpos+1
                  mcpos( ncpos ) = ncpos
                  mccol( ncpos ) = jlocdgfd
                  mccoeff( ncpos ) = mat(i3 )
                  i3 = nusol
                  i4 = ncpos

!                 --- Loop over all columns and their positions in the
!                     new row, except the last one.

                  do icpos = 1, ncpos - 1
                     if ( jlocdgfd<mccol(icpos)) then

!                    --- This column number: MCCOL(icpos) is higher
!                        than jLOCDGFD so its position is shifted one
!                        to the right.

                        if ( mccol(icpos)<i3 ) then

!                       --- This column is the closest larger column
!                           so the new column gets the position of
!                           this one.

                           i3 = mccol(icpos)
                           i4 = mcpos(icpos)
                        end if
                        mcpos(icpos) = mcpos(icpos)+1
                     end if
                  end do
                  mcpos( ncpos ) = i4
               end if
            end if

!           --- Copy the new row into the submatrix

            first2 = locintmt(ilocdgfd)+nlocdgfd
            do icpos = 1, ncpos
               locintmt(first2+mcpos(icpos)+1) = mccol(icpos)
               locmat(first2+mcpos(icpos)) = mccoeff(icpos)
            end do
         end do
      end do
      deallocate ( rows, cols, diag, mat )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( locintmt, nlocdgfd+1, 'rows' )
         call prinin ( locintmt(nlocdgfd+2), locintmt(nlocdgfd+1), 'cols' )
         call prinrl ( locmat, nlocdgfd, 'diag_mat' )
         call prinrl ( locmat(nlocdgfd+1), locintmt(nlocdgfd+1), 'Lmat' )
         write(irefwr,*) 'End seppjsubm'

      end if  ! ( debug )
      call erclos ( 'seppjsubm' )

      end subroutine seppjsubm
