      subroutine septimbouncond ( first )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 11-12-2017 New storage of intmat
!        version  2.0  date 20-08-2016 use sepmoduleinput
!        version  1.0  date 02-11-2015
!
!   copyright (c) 2015-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill boundary conditions in array isol2d and check if the array
!     corresponds to the correct problem (help routine for soltim)
!     This subroutine is only called if FILTIM is not used
!     Old subroutine prtm01
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulekprob
      use sepmodulemain
      use sepmoduleintmat
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical, intent(in) :: first

!     first          i    If true it concerns the first time step in a
!                         time-dependent problem
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ictime, iseqnr, isubr, iproblc
      logical :: skipbouncond

!     i              Counting variable
!     ictime         Indication if information must be read from standard
!                    input file ( 0 ) or that information read before is
!                    used ( 1 )
!     iproblc        Local value of iprob
!     iseqnr         Sequence number with respect to the storage in array ISEPIN
!     isubr          Indicates the type of calling subroutine for er0034 to
!                    isubr = isubrout+100*(ivecsq-1) with
!                    isubrout sequence number of caling subroutine:
!                        1:  CREATE
!                        2:  PRESDF
!                        3:  CREAVC
!                        4:  PRESTM
!                        5:  ERCRET
!                        6:  ERPRES
!                    ivecsq sequence number of first vector
!                    At output isubr contains the maximum sequence number
!                    of the array created
!     skipbouncond   If true there are no essential boundary conditions
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0034         Performs the actual tasks for CREAT and PRESDF
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPLOCTRN      Compute (back-)tranformation in case of local transform
!     SEPPRINTV      Print solution vector or vector of special structure
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     799   Array INTMAT and ISOL correspond to different problems
! **********************************************************************
!
!                       PSEUDO CODE
!
!    if ( ireadbctim=0 ) then
!       Read input when first is true, otherwise use previous input
!    else if ( ireadbctim=-1 ) then
!       Copy isol2d from isolut2dred
!    else if ( ireadbctim<-10 ) then
!       Copy isol2d from isolut2dred special sequence number
!    else ( ireadbctim>0 )
!       Fill boundary conditions: the information has already been read
!    if ( first ) then
!       Check if array isol2d corresponds to the correct problem
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimbouncond' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimbouncond'
         write(irefwr,1) 'ireadbctim, ncopeqtim', ireadbctim, ncopeqtim
         write(irefwr,*) 'first ', first
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- set skipbouncond, i.e. check if there are any
!         essential bcs

      skipbouncond = .true.
      iproblc = iprob
      do i = 1, ncopeqtim

         if ( intmatar(i)==0 ) then
            iprobtim = i
         else
            iseqim = intmatar(i)
            iprobtim = intmt(iseqim)%iprob
         end if  ! ( intmatar(i)==0 )
         iseqkp = kprob
         kprobpart => kp(iseqkp)%kprobsub(iprobtim)
         kprobloc => kprobpart%kprobloc
         if ( kprobloc(64)<kprobloc(9) ) skipbouncond = .false.
         if ( debug ) &
            write(irefwr,1) 'i, iprobtim, kprobloc(10)', &
                             i, iprobtim, kprobloc(10)
      end do  ! i = 1, ncopeqtim

      if ( debug ) write(irefwr,*) 'skipbouncond ', skipbouncond

      if ( skipbouncond ) then

!     --- skipbouncond = true, hence there no essential boundary conditions
!         copy solution from isolut2dred

         do i = 1, ncopeqtim

            call sepcopyvc ( isolut2dred(Teq(i)%iseqfr), isol2d(i) )

         end do
         go to 1000

      end if  ! ( skipbouncond )

      if ( ireadbctim==0 ) then

!     --- ireadbctim = 0, Read input when first is true, otherwise
!                      use previous input

         if ( first ) then
            ictime = 0
         else
            ictime = 1
         end if
         iseqnr = 1
         isubr = -2
         call er0034 ( isubr, ictime, isol2d, iseqnr, isolut2dred, ncopeqtim )

      else if ( ireadbctim==-1 ) then

!     --- ireadbctim = -1, Copy boundary conditions from isolut2dred

         do i = 1, ncopeqtim

            call sepcopyvc ( isolut2dred(Teq(i)%iseqfr), isol2d(i) )

!           --- Next it is transformed from Cartesian coordinates to new
!               coordinates in case of local transformations

            call seploctrn ( 1, isol2d(i) )

         end do

      else if ( ireadbctim<-10 ) then

!     --- ireadbctim<-10, Copy boundary conditions from isolut2dred
!                         special position

         iseqnr = -(ireadbctim+10)
         do i = 1, ncopeqtim
            call sepcopyvc ( isolut2dred(i+iseqnr-1), isol2d(i) )
         end do

      else

!     --- ireadbctim>0, Use input read by SEPSTN when first is true,
!                      otherwise use previous input

         ictime = 0
         isubr = 4
         call er0034 ( isubr, ictime, isol2d, ireadbctim, isolut2dred,&
                       ncopeqtim )

      end if
      if ( ierror/=0 ) go to 1000

      if ( first ) then

!     --- Check if array isol2d corresponds to array intmat

         do i = 1, ncopeqtim
            if ( intmatar(i)==0 ) then
               iprobtim = ks(isol2d(i))%iprob
            else
               iseqim = intmatar(i)
               iprobtim = intmt(iseqim)%iprob
               if ( ks(isol2d(i))%iprob/=iprobtim ) then

!              --- matrix and solution vector correspond to different vectors

                  call errint ( i, 1 )
                  call errint ( ks(isol2d(i))%iprob, 2 )
                  call errint ( iprobtim, 3 )
                  call errsub ( 799, 3, 0, 0 )

               end if
            end if  ! ( intmatar(i)==0 ) then
         end do

      end if
      iprob = iproblc

1000  if ( debug ) then

!     --- Debug information

         call prinin ( isol2d, 5, 'isol2d' )
         call sepprintv ( isol2d, 'u bc' )
         write(irefwr,*) 'End septimbouncond'

      end if  ! ( debug )
      call erclos ( 'septimbouncond' )

      end subroutine septimbouncond
