      subroutine septimbuildmat ( isol, irhsdlc, islold, first, ieqn )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-12-2017 New storage of intmat
!        version  1.0  date 28-10-2015
!
!   copyright (c) 2015-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Build mass matrix, stiffness matrix and right-hand side for
!     one equation and one time level in case of a time-dependent problem
!     Old subroutine prtimbuildmat
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
!     stiffness_matrix
!     mass_matrix
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulemain
      use sepmoduleuser
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isol(*), islold(*), ieqn
      integer, intent(inout) :: irhsdlc(*)
      logical, intent(in) :: first

!     first          i    If true it concerns the first call in a row,
!                         otherwise it concerns a subsequent call
!     ieqn           i    Equation sequence number
!     irhsdlc       i/o   Standard SEPRAN array, containing information of the
!                         large vector
!     islold         i    User input array in which the user puts information
!                         of all preceding solutions
!                         Integer array of length 5 x numold
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: j
      logical :: matlab
      logical, save :: buildmat

!     buildmat       If true the matrix and right-hand side must be built
!                    each time step
!     j              Counting variable
!     matlab         If true matlab input is generated
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     FILTIM         user subroutine to fill the mass matrix, stiffness matrix
!                    and the right-hand side, depending on time
!     PRININ         print 1d integer array
!     PRMATLABMATRIX Print a sepran matrix in a form suitable for reading
!                    in matlab
!     PRMATLABVECTOR Print a sepran vector in a form suitable for reading
!                    in matlab
!     SEPBUILD       Build matrix and right-hand side
!     SEPFILLSMF     Fill all input for subroutine SEPBUILD in SMF
!     SEPPRINTMAT    Print matrix
!     SEPPRINTV      Print solution vector or vector of special structure
!     SEPTIMUSER     Fill arrays iuser and user for a specific
!                    problem in a sequence of problems
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1238   The right-hand side has not been filled correctly in
!           subroutine FILTIM.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimbuildmat' )
      debug = .false.
      matlab = .false.
      if ( first .and. debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimbuildmat'
         write(irefwr,*) 'first ', first
         write(irefwr,1) 'ifilltim, ieqn', ifilltim, ieqn
         call prinin ( iinbldtim, iinbldtim(1), 'iinbldtim' )
  1      format ( a, 1x, (10i10) )

      end if
      if ( ierror/=0 ) go to 1000

      intmat => intmatar(ieqn)

      buildmat = Teq(ieqn)%iconsstiff==0 .or. &
                 Teq(ieqn)%iconsmass==0 .or. &
                 Teq(ieqn)%iconsrhs/=2

      if ( debug ) write(irefwr,*) 'buildmat', buildmat

      iinbldtim(13) = Teq(ieqn)%iseqfr
      if ( debug ) call prinin ( iinbldtim, iinbldtim(1), 'iinbldtim' )
      iprobtim = intmt(intmat)%iprob

!     --- build matrices M , S and vector F on new time level

      if ( ifilltim==1 ) then

!     --- ifilltim=1, call subroutine filtim

         call septimuser ( first, ieqn )
         call filtim ( kmesh, kprob, intmat, isol(ieqn), stiffmat, &
                       massmat, irhsdlc, islold, user, iuser )

         if ( first ) then

!        --- Check the matrices that have been build (only the first time)

            if ( ks(irhsdlc(1))%typevec/=110 ) then
               call errint ( ks(irhsdlc(1))%typevec, 1 )
               call errint ( ieqn, 2 )
               call errsub ( 1238, 2, 0, 0 )
            end if

         end if

      else if ( first .or. buildmat ) then

!     --- Build matrix and right-hand side explicitly

         if ( debug ) write(irefwr,*) 'before sepbuild'
         call sepfillsmf ( iinbldtim, irhsdlc, &
                           isol(ieqn), islold )
         call sepbuild
         if ( ierror/=0 ) go to 1000

         if ( debug ) then

!        --- Debug information

            call sepprintv ( irhsdlc, 'rhsd' )
            write(irefwr,*) 'Stiffness matrix'
            call sepprintmat ( stiffmat )
            write(irefwr,*) 'Mass matrix'
            call sepprintmat ( massmat )

         end if  ! ( debug )

         if ( matlab ) then

!        --- Write results to matlab

            call prmatlabmatrix ( stiffmat, 'stiffmat.dat' )
            call prmatlabmatrix ( massmat, 'massmat.dat' )
            call prmatlabvector ( irhsdlc, 'rhsd.dat' )

         end if  ! ( matlab )

      end if  ! ( iintlc(2)==1 )

1000  if ( first .and. debug ) then

!     --- Debug information

         write(irefwr,*) 'End septimbuildmat'

      end if
      call erclos ( 'septimbuildmat' )

      end subroutine septimbuildmat
