      subroutine septimconvsub ( first, iinnavstokes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve one step of the time-dependent Navier-Stokes equations
!     Use an explicit method for the convection with smaller time steps
!     (Substepping)
!     The method applied is a form of operator splitting
!     Old subroutine prtimconvsub
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     navier_stokes_equation
!     time_integration
!     convection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduletime
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinnavstokes(*)
      logical, intent(inout) :: first

!     first          i    If true it concerns the first time step
!     iinnavstokes   i    Integer input array for Navier-Stokes
!                         See subroutine readnavstokes for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer joutp, jtime, iseqveloc, iun(5), mstep, &
              iinpresscor(3), ieqn, iincopy(2)

!     ieqn           Equation sequence number
!     iincopy        User input array for subroutine COPYVEC
!     iinpresscor    Integer input array for the pressure correction process
!                    See subroutine readpresscor
!     iseqveloc      Sequence number of velocity vector in set of solution
!                    arrays
!     iun            Contains information about the old solution vector un
!     joutp          Help parameter to store the value of ioutp
!     jtime          Help parameter to store the value of itime
!     mstep          Number of substeps per time step
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPCOPYVEC     copy Sepran array
!     SEPPRINTMAT    Print matrix
!     SEPPRINTV      Print solution vector or vector of special structure
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     SEPTIMBOUNCND  Transform solution vector to local coordinates
!                    Fill essential boundary conditions in solution vector
!     SEPTIMCHECK    Perform some checks with respect to time integration input
!                    Fill array iinbldtim
!     SEPTIMCONV     Treat the convective terms in the Navier-Stokes equations
!                    in an explicit way using substepping and operator splitting
!     SEPTIMEULSTEP  Perform one time step for an Euler implicit discretization
!     SEPTIMPRESSMAT Compute mass matrix divided by dt and stiffness matrix
!                    with respect to the predictor step of pressure correction
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The operator splitting applied has the following form
!
!     We have to solve the incompressible Navier-Stokes equations
!     In discretized form:
!
!     M du/dt + S u + N(u)u + L^t p = f
!     L u = 0
!
!     with M the mass matrix, S the stiffness matrix corresponding
!          S the stiffness matrix corresponding to the viscous part
!          N(u)u the discretization of the convection
!          L^t p the discretization of the gradient of p
!          L u = 0 the discretization of the continuity equation
!
!    To solve this system we apply the following operator splitting:
!
!     Given u^n at the previous level
!     Let mstep be the number of substeps
!     Then:
!     dt* = dt/mstep
!     For istep := 1 (1) mstep do
!        M (u*-u^n)/dt* + N(u^n)u^n = 0
!        u^n := u*
!     M  (u^(n+1)-u*)/dt + S u^(n+1) + L^t p^(n+1) = f^(n+1)
!     L u^(n+1) = 0
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimconvsub' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimconvsub'
         write(irefwr,110) 't, tstep', t, tstep
100      format ( a, 1x, (10i6) )
110      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000
      stiffmat => stiffm
      massmat => massmt(1)
      totmat => mattot(1)

!     --- Get constants

      iseqveloc = iinnavstokes(3)
      isolut => isolut2d(iseqveloc)
      complrun = .false.
      if ( debug ) write(irefwr,100) 'iseqveloc', iseqveloc

!     --- set ioutp and itime temporarily

      joutp = ioutp
      ioutp = min(0,ioutp)
      jtime = itime
      itime = 0

!     --- Copy old velocity vector into un

      iun(1) = 0
      call sepcopyvc ( isolut, iun )

      if ( debug ) call sepprintv ( iun, 'un' )

!     --- Define parameters and fill arrays iinbld, rinsol, inpsol

      call septimcheck ( first )
      Teq(1)%iseqfr = iseqveloc

!     --- Raise t, necessary for correct boundary conditions
!         and coefficients
!         In case of keep_t, it is not raised

      if ( keeptim==0 ) t = t + tstep
      if ( debug ) write(irefwr,110) 't_new', t

!     --- Transform the solution in case of local transformations and fill
!         essential boundary conditions at the new time level if necessary
!         Back transformation is done by the linear solver

      call septimbouncnd ( first )

!     --- Build mass matrix, stiffness matrix and right-hand side
!         At this moment always, in the future we should make this
!         dependent on the parameters in iintim

      iinpresscor(1) = iseqveloc ! sequence number of velocity in isolut2d
      iinpresscor(2) = 0         ! not used
      iinpresscor(3) = ireadcftim(1)   ! input block coefficients
      dt = 1d0             ! fake to prevent subdivision of M by dt
      call septimpressmat ( iinpresscor )

      if ( .false. .and. debug ) then

!     --- Debug information

         write(irefwr,*) 'stiffness matrix'
         call sepprintmat ( stiffmat )
         write(irefwr,*) 'mass matrix'
         call sepprintmat ( massmat )
         call sepprintv ( irhsd, 'right-hand side' )

      end if  ! ( debug )

!     --- Start given u^n at previous time level
!         Perform an explicit integration of the convection term by substepping
!         The mass matrix will be created in prtimconv
!         The convective term is created in each substep as right-hand-side
!         vector

      mstep = iinnavstokes(4)     ! number of substeps
      call septimconv ( iun, 1, mstep )
      if ( debug ) then

!     --- Debug information

         call sepprintv ( iun, 'u*' )

      end if  ! ( debug )

!     --- In un we have the intermediate solution vector u*
!         This must be used in the Stokes equation however, provided
!         with the boundary conditions on the new level
!         So we have to copy un into isolut except for the
!         boundary values

      iincopy(1) = 2   ! Number of positions filled
      iincopy(2) = 1   ! Copy only part not corresponding to ess. bc's
      call sepcopyvec ( iun, isolut, iincopy )

!     --- Perform one Stokes step to compute the new velocity

      if ( .false. .and. debug ) then

!     --- Debug information

         write(irefwr,*) 'stiffness matrix'
         call sepprintmat ( stiffmat )
         write(irefwr,*) 'mass matrix'
         call sepprintmat ( massmat )
         call sepprintv ( irhsd, 'right-hand side' )

      end if  ! ( debug )

      ieqn = 1       ! sequence number of equation
      dt = theta*tstep
      call septimeulstep ( first, isolut, isolut2d, irhsd, iun, ieqn )

      if ( debug ) then

!     --- Debug information

         call sepprintv ( isolut, 'u^n+1' )

      end if  ! ( debug )

!     --- reset ioutp and itime

      first = .false.
      ioutp = joutp
      itime = jtime

!     --- resmove arrays

      call seprmsolinfo ( iun(1) )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septimconvsub'

      end if
      call erclos ( 'septimconvsub' )

      end
