      subroutine septimefr03 ( finished, first, isubr, iseqenthalpy, &
                               iinenthalpy, rinenthalpy, rewind, changestep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 15-12-2017 New storage of intmat
!        version  1.0  date 07-11-2015
!
!   copyright (c) 2015-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     solves time-dependent enthalpy problem
!     Input may be from the standard input file or through the parameter list
!     Old subroutine prtimefr03
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulecons
      use sepmoduletime
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: isubr, iseqenthalpy, iinenthalpy(*)
      double precision rinenthalpy(*)
      logical first, finished, rewind, changestep

!     changestep     o    If true the time step may be changed
!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
!     first         i/o   If true it concerns the first step
!     iinenthalpy    i    Integer input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
!     iseqenthalpy   i    Sequence number of input for block enthalpy
!     isubr          i    Indicates the type of calling subroutine
!                         Possible values
!                         1:  timprb
!                         2:  prtimd
!                         3:  timefrbf
!     rewind         i    When true, the current time level is rewinded
!                         to a previous time level
!     rinenthalpy    i    Real input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer solutionmethod
      double precision eps
      double precision :: timebefore(2)

!     eps            Local accuracy
!     solutionmethod Defines type of solution method
!                    1: overrelaxation
!                    2: newton
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERSETTIME      Get the CPU time at start of subroutine
!     PRININ         print 1d integer array
!     SEPTIMENTHAL1  Time integration for enthalpy equation using
!                    overrelaxation
!     SEPTIMENTHAL2  Time integration for enthalpy equation using
!                    newton
!     SEPTIMOUTP     Perform output for subroutine septimprb during a time loop
!     SEPTIMSET      Set some time parameters in the time loop
!     SEPTIMSTART    Start part of subroutine septimprb
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill arrays ibuffr(ipintm) and rintim
!     Fill some constants
!     Write initial condition depending on toutin
!     While t<tend do
!        compute tout
!        call time stepping subroutine and raise t
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimefr03' )
      if ( itime>0 ) call ersettime ( timebefore )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimefr03'
         write(irefwr,1) 'iseqenthalpy', iseqenthalpy
         write(irefwr,2) 't, tout, tend, tstep', t, tout, tend, tstep
         call prinin ( isolut2d, 5, 'isolut2d' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )
         call prinin ( iinenthalpy, 6, 'iinenthalpy' )
         call prinin ( isolut2d, 5, 'isolut2d' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Start part of subroutine septimefr03
!         Fill input arrays and constants and write initial condition

      call septimstart ( first, eps, isubr )
      if ( ierror/=0 ) go to 1000

!     --- Set some parameters in the timing subroutine

      Teq(1)%imas = 1  ! mass matrix is stored as diagonal matrix
      solutionmethod = iinenthalpy(6)
      if ( debug ) write(irefwr,1) 'solutionmethod', solutionmethod

!     --- While loop time-method depending on ibuffr(ipintm) and rintim

200   if ( itimloop>=1 ) then

!     --- First set tstep and tout

         call septimset ( first, eps )
         if ( ierror/=0 ) go to 1000

!        --- Call time integration subroutine

         if ( solutionmethod==1 ) then

!        --- Solutionmethod = 1, use overrelaxation

            call  septimenthal1 ( first, finished, max(1,iseqenthalpy), &
                                  iinenthalpy, rinenthalpy )

         else if ( solutionmethod>=2 ) then

!        --- Solutionmethod = 2, 3, use nedjar or newton

            rewind = .false.
            call septimenthal2 ( first, finished, max(1,iseqenthalpy), &
                                 iinenthalpy, rinenthalpy, &
                                 solutionmethod, rewind, changestep )

         end if  ! ( solutionmethod==1 )

         if ( ierror/=0 ) go to 1000

!        --- Perform output during a time loop

         call septimoutp ( eps )

         if ( ierror/=0 ) go to 1000
         if ( complrun ) go to 200

      end if
      if ( complrun ) itimloop = 0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septimefr03'

      end if
      call erclmn ( 'septimefr03', timebefore )

      end
