      subroutine septimenth1eu ( finished, iseqenthalpy, iinenthalpy, &
                                 rinenthalpy )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 15-12-2017 New storage of intmatar
!        version  1.0  date 12-11-2015
!
!   copyright (c) 2015-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Time integration of a discretized partial differential equation
!    of first order in time with an implicit modified theta method
!              .
!          M * u + S * u = F
!              -       -   -
!    It has been assumed that the matrices M and S are not dependent
!    of the solution vector u. Hence the system is linear . M and
!    S may of course depend on place and time.
!    Solution method used is the modified theta method. The user
!    has to give the stepsize tstep.
!    Special subroutine for enthalpy in combination with overelaxation
!     Old subroutine prtimenthal1eu
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     enthalpy
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduleintmat
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iseqenthalpy, iinenthalpy(*)
      double precision, intent(in) :: rinenthalpy(*)
      logical, intent(inout) :: finished

!     finished      i/o   Indicates if the time loop has been finished (true)
!                         or not  (false)
!     iinenthalpy    i    Integer input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
!     iseqenthalpy   i    Sequence number of input for block enthalpy
!     rinenthalpy    i    Real input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: maxprbloc=10
      integer, target :: isol(maxprbloc)
      integer :: i, ieqn, itemp, isolprec(maxprbloc)
      double precision :: tsteps
      logical :: first, finishtm, linear
      character (len=25) :: text
      save isol, first, isolprec

!     finishtm       Indicates if the time loop has been finished (true)
!                    or not (false)
!     first          If true it concerns the first time step
!     i              Counting variable
!     ieqn           Equation sequence number
!     isol           Standard SEPRAN array containing information of the
!                    solution arrays
!     isolprec       Solution at preceding level
!     itemp          Temporary sepran solution array
!     linear         If true the linear time-iteration is used, if false an
!                    iteration per time step is performed
!     maxprbloc      Maximum number of coupled problems allowed
!     maxprbloc5     5*maxprbloc
!     text           text with respect to writing or plotting
!     tsteps         help variable to save the value of tstep
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRINTHIS       Print time history
!     PRKIRCHBACK    Transform temperature from normalized temperature back to
!                    physical temperature
!     PRKIRCHTRANS   Transform temperature in normalized temperature and compute
!                    the corresponding enthalpy
!     SEPCOPYVEC     copy Sepran array
!     SEPLOCTRN      Compute (back-)transformation in case of local transform
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     SEPTIMBOUNCOND Fill boundary conditions in array isol and check if the
!                    array corresponds to the correct problem
!     SEPTIMIINBLD   Fill some parameters in iinbld and initialize
!                    some other parameters for time integration
!     SEPTIMRESET    reset all arrays with respect to time integration that
!                    are no longer used
!     SEPTIMTHETA1   Start the time integration for suboutine septimtheta
!     SEPTIMTHETA2   Set some parameters during a time loop
!     SEPTIMTHETAFRC Extrapolate the solution from old and intermediate time
!                    step to next time step in case of theta method
!     SEPTIMUSER     Fill arrays iuser and user for a specific
!                    problem in a sequence of problems
!     SEPTMENTHALPY  Perform one step in an Euler implicit scheme for the
!                    enthalpy equation
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Differential equations to be solved:
!        .
!     M  u   +  S  u   =  f
!     Application of Euler
!     -Euler part (predictor):
!       n+o     n
!      u    -  u
!       j       j           n+o      n+o
!    M ----------- +  S  u     =  f          (n is actual time-level)
!          dt               j        j         (n+o is time at t+theta dt)
!    - Extrapolation step
!     n+1     1     n+o            n
!    u    = -----( u   - (1-theta)u )
!           theta
!    or. &
!                 n+o        n         n+o
!   ( M + dt S ) u     =  M u    + dt f
!                 j          j         j
!                             n                 n+1
!   M and S may depend on:   u    k>=j  and  u      k<j (Silechi approach)
!                             k                 k
!   If a stationary solution is required, the process is stopped if
!       k+1    k      1-lambda               k+1
!   || u    - u  ||<-------- (eps_abs + ||u   || eps_rel)
!                      lambda
!                     k+1   k
!                 || u   - u ||
!   with lambda = ------------- &
!                     k   k-1
!                 || u - u ||
!   Check input
!   ifrac := 1
!   stationary := false
!   While ( ifrac>1 or t<tout and not stationary ) do
!      if ( nfrac=1 ) then
!         t := t + theta dt
!      else ( nfrac>1 )
!         t := t + theta_(ifrac+1) dt
!         theta := theta_(ifrac+1) / (theta_ifrac + theta_(ifrac+1) )
!      Copy isolut2d into isol
!      For iprobtim := 1 (1) ncopeq do
!         Build matrices M(iprobtim) and S(iprobtim)
!         Store boundary conditions into usol(iprobtim)
!         Build right-hand side f without the effect of boundary conditions
!            i.e. the part fi
!         help := M un, i.e. Mii ui + Mir ur
!         help := help + dt f
!         if ( icons=0 or t=t0+dt ) then
!            M_help = M + dt S
!         help1 := M_helpir ur (n+1)
!         help  := help - help1
!         Solve M_help isol(iprobtim) = help
!         Copy isol(iprobtim) into isolut2d(iprobtim)
!      If ( extrapolation required ) then
!         extrapolate solution
!         if ( nfrac=1 ) then
!            t := t + (1-theta) dt
!         else
!            t := t + theta_ifrac dt
!            ifrac := modulo(ifrac+2,nfrac)
!      If ( stationary solution required ) then
!         estimate lambda
!         check stopping criterion
!   If ( t>=tend ) then
!      Delete all arrays created
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data isol /maxprbloc*0/, itemp /0/, &
           isolprec/maxprbloc*0/
! ======================================================================
!
      call eropen ( 'septimenth1eu' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimenth1eu'
         write(irefwr,*) 'iseqenthalpy ', iseqenthalpy

      end if
      call inichkarlen ( maxprb, maxprbloc, 'iseqeq' )

!     --- Start of process
!         Set parameters

      ncopeqtim = 2
      Teq(1)%iseqfr = iinenthalpy(2)  ! enthalpy
      Teq(2)%iseqfr = iinenthalpy(3)  ! temperature
      ireadbctim = iinenthalpy(5)
      call  septimtheta1 ( finishtm, first, linear )
      tsteps = tstep
      call sepcopyvec ( isolut2d(iinenthalpy(3)), isol(2), 0 )

!     --- Next it is transformed from Cartesian coordinates to new
!         coordinates in case of local transformations

      call seploctrn ( 1, isol(2) )

!     --- While loop over time-steps

      do while ( .not. finishtm )

!     --- t<tout
!         Copy solution at old time level into isolprec

         do i = 1, ncopeqtim

            call sepcopyvec ( isolut2d(Teq(i)%iseqfr), isolprec(i), 0 )

         end do  ! i = 1, ncopeqtim

!        --- In case of the enthalpy equation we have to transform the
!            temperature to a normalized temperature

         call prkirchtrans ( rinenthalpy, &
                             isolprec, 3 )

!        --- set some parameters, which are used in the time loop

         do i = 1, ncopeqtim
            call septimtheta2 ( tsteps, i )
         end do  ! i = 1, ncopeqtim

!        --- Put boundary conditions into isol

         ncopeqtim = 1
         if ( itimbc<2 .and. (first .or. itimbc==0) ) then

!        --- Either first call, or boundary conditions are time-dependent
!            Mark that for the ncopeq problems only one
!            call to presbc is used
!            The boundary conditions are always given in the transformed form
!            the boundary conditions are only given for the temperature (2)

            Teq(1)%iseqfr = iinenthalpy(3)
            isol2d => isol(2:maxprbloc)
            isolut2dred => isolut2d
            call septimbouncond ( first )

!           --- we have to transform the
!               temperature and fill the enthalpy by the Kirchoff
!               transformation. This is done with the boundary conditions
!               at the new time level

             call prkirchtrans ( rinenthalpy, isol, 0 )

         end if  ! ( itimbc<2 .and. (first .or. itimbc==0) )

!        --- While loop for non-linear iteration
!            Loop over all problems

         Teq(1)%iseqfr = iinenthalpy(2)
         do ieqn = 1, ncopeqtim

!        --- First set the correct parameters in iinbld

            call septimiinbld ( first, ieqn )

!           --- Compute starting addresses for the arrays iuser and user

            call  septimuser ( first, ieqn )

!           --- Perform one step Euler implicit
!               special case of enthalpy equation
!               an adapted version of prtm00 is used
!               iuser and user come from the input file

            call septmenthalpy ( first, isol, isolprec, iseqenthalpy )

!           --- copy isol into isolut2d
!               First save isolut2d in itemp and then copy itemp into isol
!               In this way isol contains the solution at the old level
!               and isolut2d at the new level

            itemp = 0
            call sepcopyvec ( isolut2d(Teq(ieqn)%iseqfr), itemp, 0 )
            call sepcopyvec ( isol(ieqn), &
                             isolut2d(Teq(ieqn)%iseqfr), 0 )
            call sepcopyvec ( itemp, isol(ieqn), 1 )
            call seprmsolinfo ( itemp )

!           --- Special case of enthalpy equation, also the temperature
!               must be copied from isol into isolut2d

            call sepcopyvec ( isol(2), isolut2d(iinenthalpy(3)), 0 )

         end do  ! ieqn = 1, ncopeqtim

         ncopeqtim = 2  ! in the enthalpy case we have
                     ! 2 vectors (temp and enthalpy)

         if ( nfrac>1 .or. abs(1d0-theta)>100d0*epsmac ) then

!        --- extrapolate to final time

            call septimthetafrc ( isolut2d, isolprec )

         end if  ! ( nfrac>1 .or. abs(1d0-theta)>100d0*epsmac )

!        --- transform the temperature back

         call prkirchback ( iinenthalpy, &
                            rinenthalpy, isolut2d )

         if ( nhist>0 ) then

!        --- print time histor. &

            call printhis ( nhist, nodhist, t, ncopeqtim, &
                            isolut2d(iinenthalpy(2)), first )

         end if
         if ( ifrac==1 .and. t>=tout-0.0001d0*tstep .or. &
              .not. complrun ) finishtm = .true.
         first = .false.

      end do

!     --- While loop has been finished, reset quantities

      tstep = tsteps
      iflag = 3

      if ( t>=tend-0.00001*tstep ) then

!     --- t = tend   reset all arrays

         call septimreset ( irhsd, isol, isolprec, finished )

      end if  ! ( t>=tend-0.00001*tstep )

      if ( debug ) write(irefwr,*) 'End septimenth1eu'
      call erclos ( 'septimenth1eu' )

      end subroutine septimenth1eu
