      subroutine septimenth3eu ( iinenthalpy, rinenthalpy, &
                                 finished, rewind )
! ======================================================================
!
!        programmer    John Brusche
!        version  1.1  date 15-12-2017 New storage of intmat
!        version  1.0  date 12-11-2015
!
!   copyright (c) 2015-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Time integration of the enthalpy equation using Newton linearization
!    The method of Fachinotti et al is used
!     Old subroutine prtimenthal3eu
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     enthalpy
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematr
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemain
      use sepmoduleintmat
      use sepmodulesolve
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinenthalpy(*)
      double precision, intent(in) :: rinenthalpy(*)
      logical, intent(inout) :: finished, rewind

!     finished      i/o   Indicates if the time loop has been finished (true)
!                         or not  (false)
!     iinenthalpy    i    Integer input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
!     rewind        i/o   When true, the current time level is rewinded
!     rinenthalpy    i    Real input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ideltatemp, iseqtemp, gradf, irhsdcp, &
                 ichois, iter, ireadcf, normtype, &
                 errortype, ntimesteps, totiter
      double precision, parameter :: stpmx=100d0, tolx=1d-12, tolmin=1d-6
      double precision :: eps, resnorm, stpmax, f, fold, &
                          resnorm1, res2nrm, resinfnrm, &
                          tm, bnorm, meaniter, tstepmin, xhist, yhist, &
                          zhist
      logical :: first, converged, check, checkphase
      save meaniter, ntimesteps, totiter, tstepmin
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     bnorm          Norm of initial residual
!     check          If true, convergence on ideltatemp is checked
!     checkphase     If true, it is checked whether the moving interface
!                    does not skip one or more elements
!     converged      If true, the Newton Raphson process is terminated
!     eps            Required level of accuracy
!     errortype      If equal to 1, the error is measured relative to
!                    the initial error. If equal to 2, the absolute
!                    error is considered.
!     f              0.5d0*|F|^2
!     first          If true it concerns the first step
!     fold           Old value of f
!     gradf          Gradient of f
!     ichois         choice parameter
!     ideltatemp     Search direction vector
!     ireadcf        Sequence number of input for coefficients
!     irhsdcp        Copy of irhsd(1)
!     iseqtemp       Sequence number of temperature in solution vector
!     iter           Counting variable that keeps track of the number of
!                    iterations
!     meaniter       Average number of iterations per time step
!     normtype       Type of norm used. 1: infinity norm 2: l2-nor.
!     ntimesteps     Total number of time steps
!     res2nrm        Residual in 2-nor.
!     resinfnrm      Residual in inf-nor.
!     resnorm        Norm of the residual
!     resnorm1       Norm of the initial residual
!     stpmax         Maximum step size allowed
!     stpmx          Maximum step size allowed
!     tm             Melting temperature
!     tolmin         Tolerance for measuring gradf==0
!     tolx           Maximum tolerated error on the temperature
!     totiter        Total number of iterations
!     tstepmin       Minimum time step size
!     xhist          x-coordinate of point to keep a history of
!     yhist          y-coordinate of point to keep a history of
!     zhist          z-coordinate of point to keep a history of
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      logical :: prtimenthal3eu1, prtimenthal3eu2

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     FINDXY         Find nodal point closest to a specific point
!     INSTOP         Stop the program
!     PRCHECKPHASE   Check whether element skipped phase transition
!     PRCREATEPHASE1 Create vectors that keep track of phase of elements
!     PRTIMENTHAL3EU1 Check for spurious convergence
!     PRTIMENTHAL3EU2 Check for convergence on deltatemp
!     SEPCOPINPSOL   Copy arrays inpsl1 and rinsl1 into inpsol and rinsol
!                    into sepmodulesolve
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPENTHBLDRHS  Build the right-hand side vector R2 for line search
!     SEPENTHLINSEAR Perform line search algorithm to optimize search direction
!     SEPLINSOL      Body of linear solver
!     SEPMAVER       perform matrix vector multiplication
!     SEPRMMATRIX    Remove array with sequence number iseq from realmt
!     SEPRMSOLINFO   Remove array with sequence number ivectlc from ks
!     SEPTIMEENTH3E1 Build matrix and right-hand side for enthalpy equation
!                    solved by Newtons method
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ideltatemp /0/, gradf/0/, irhsdcp /0/
! ======================================================================
!
      call eropen ( 'septimenth3eu' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimenth3eu'
         write(irefwr,2) 't, tout, tend, tstep', t, tout, tend, tstep
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      intmat => intmatar(1)

!     --- Set some parameters

      eps = max(absaccurtim,relaccurtim)
      iseqtemp = iinenthalpy(3) ! sequence number of temperature in isolut2d
      first = t<=t0+0.1d0*tstep ! if true first time step
      iprob = intmt(intmat)%iprob  ! problem number
      ireadcf = iinenthalpy(7)  ! sequence number for reading coefficients
      iprint = iinenthalpy(8)   ! print level
      tm = rinenthalpy(7)       ! melting temperature
      converged = .false.
      checkphase = .true.
      normtype = 1
      errortype = 1
      iter = 0
      rewind = .false.
      temperature => ks(isolut2d(iseqtemp))%sol

!     --- create some vector.

      nusol = size(temperature)
      allocate ( tempold(nusol) )

      if ( debug ) then
         write(irefwr,1) 'maxitertim, iseqtemp, nusol', &
                          maxitertim, iseqtemp, nusol
         write(irefwr,2) 'eps', eps
         write(irefwr,*) 'first', first
      end if  ! ( debug )

!     --- Adapt array inpsoltim

      if ( first ) then

!     --- First call, set some parameters in inpsoltim

         IM => intmt(intmat)
         if ( IM%direct .and. .not. IM%nested .and. .not. IM%mumps ) then

!        --- direct method

            if ( inpsoltim(3)>0 ) inpsoltim(3) = 0

         else

!        --- iterative method
!            use old solution and keep preconditioning if matrix is constant

            if ( inpsoltim(3)==0 ) inpsoltim(3) = 1
            inpsoltim(9)  = 1

         end if ! ( IM%direct .and. .not. IM%nested .and. .not. IM%mumps )
         ntimesteps = 0
         totiter = 0
         meaniter = 0d0
         tstepmin = tstep

      end if ! ( first )

!     --- Create phase arrays, that keep track of the phase of each element
!         Time step size is adapted to ensure that an element never jumps
!         from fully solid to fully liquid or vice versa

      allocate ( phaseold(nelem), phasenew(nelem) )

!     --- Fill phaseold

      call prcreatephase1 ( phaseold, temperature, tm )
      if ( ierror/=0 ) go to 1000

!     --- Init phasenew

      phasenew = 0

!     --- Furthermore we have to make the essential boundary conditions
!         equal to 0. This can be done by setting the whole vector to 0

      call sepcopyvc ( isolut2d(iseqtemp), ideltatemp )
      deltatemp => ks(ideltatemp)%sol
      deltatemp = 0d0

!     --- Test for initial guess being a root. Use more stringent test than
!         simply eps. First we build the right hand side vectors and form the
!         right hand side (residual).
!         Build matrix and right-hand side

      call septimeenth3e1 ( iinenthalpy, 1 )

      rhsd1 => ks(irhsd(1))%sol
      rhsd2 => ks(irhsd(2))%sol
      call sepcopyvc ( irhsd(1), irhsdcp )
      rhsdcp => usoltemp

      rhsdcp = rhsd1

!     --- add both right-hand sides and store residual in irhsd(1)

      rhsd1 = rhsd1+rhsd2

!     --- Initial guess is a root? If so, set converged to true
!         Keep in mind that irhsd(1) contains -1d0*residual !!!
!         Multiply by tstep!

      if (normtype==2) then

         resnorm = norm2 ( rhsd1 ) / size(rhsd1)
         bnorm   = resnorm

      else

         resnorm = max ( maxval(rhsd1), -minval(rhsd1) )
         bnorm   = resnorm

      end if

      resnorm1 = resnorm*tstep
      bnorm = bnorm*tstep
      if (debug) write(irefwr,*) 'resnorm1, bnorm', resnorm1, bnorm

      if (resnorm1<1d-6*eps) converged = .true.
      if (converged) then
         write(irefwr,*) 'Initial guess is a root'
         call instop
      end if

!     --- compute scaled maximum step length, stpmax, allowed in line
!         searches.

      stpmax = norm2 ( temperature ) /size(temperature)
      if ( debug ) write(irefwr,2) 'stpmax/1', stpmax
      stpmax = STPMX*npoint*max(stpmax, 1d0)
      if ( debug ) write(irefwr,2) 'stpmax/2', stpmax

!     --- compute f = 0.5d0*residual*residual for initial T
!         we use a copy of irhsd(1)
!         Keep in mind that irhsd(1) contains -1d0*residual !!!

      rhsdcp = rhsd1*0.25d0
      f = dot_product( rhsdcp, rhsdcp )

!     --- save copy of constant rhsd in res2

      allocate ( res2(nusol) )

!     --- While loop for one time-step
!         First raise time step

      t = t + tstep  ! new time level
      if ( iprint>1 ) then

         write(irefwr,*) 't, tstep', t, tstep
         write(irefwr,*) &
             'Iteration   2-Norm of residual   inf-Norm of residual'

      end if ! ( iprint>1 )

      do while ( .not. converged .and. iter<maxitertim )

         iter = iter + 1

!        --- Build matrix and right-hand side
!            When iter==1, both R1 and R2 are build.
!            For iter>1, only R1 is build

         call septimeenth3e1 ( iinenthalpy, iter )

         if (iter==1) then

!        --- save copy of constant rhsd in res2

            res2 = rhsd2

         else

!        --- retrieve saved R2 and store in irhds(1,2)

            rhsd2 = res2

         end if

!        --- now add both right-hand sides and store in irhsd(1)

         rhsd1 = rhsd1+rhsd2

!        --- compute grad(f) = matrix*right-hand side
!            Keep in mind that irhsd(1) contains -1d0*residual !!!
!            we use a copy of irhsd(1)

         rhsdcp = -rhsd1

         ichois = 5 ! boundary conditions included, positions in y

!        --- get value 0d0

         call sepmaver ( stiffmat, irhsdcp, gradf, ichois )
         grad => ks(gradf)%sol

!        --- Backup old temperature vector

         tempold = temperature

!        --- compute f = 0.5d0*residual*residual for initial T
!            we use a copy of irhsd(1)
!            Keep in mind that irhsd(1) contains -1d0*residual !!!

         rhsdcp = rhsd1*0.25d0
         f = dot_product ( rhsdcp, rhsdcp )

!        --- Store f

         fold = f

!        --- Solve system of equations

         ireadinpsol = -1
         iseqsol = 1
         iseqreac = 0
         iseqintmat = 1
         keepmat = 0
         iseqsolut = 1
         iseqmt = stiffmat(1)
         call sepcopinpsol ( inpsoltim, rinsoltim )
         call seplinsol ( stiffmat, ideltatemp, irhsd )
         if ( ierror/=0 ) go to 1000

!        --- Perform linesearch

         call sepenthlinsear ( fold, f, stpmax, check, &
                               TOLX, iseqtemp, ireadcf, iprint)

!        --- Test for convergence on function values, therefor.
!            Build right-hand side
!            Compute norm and test

         isolut => isolut2d(iseqtemp)
         call sepenthbldrhs ( iprob, ireadcf, irhsd )

!        --- retrieve saved R2 and store in irhds(1,2)

         rhsd2 = res2

!        --- now add both right-hand sides and store in irhsd(1)

         rhsd1 = rhsd1+rhsd2

!        --- actual test
!            Keep in mind that irhsd(1) contains -1d0*residual !!!
!            Multiply by tstep!

         res2nrm = norm2 ( rhsd1 )/size(rhsd1)
         resinfnrm = max ( maxval(rhsd1), - minval(rhsd1) )

         if (normtype==2) then

            resnorm = res2nrm

         else

            resnorm = resinfnrm

         end if

         resnorm = resnorm*tstep
         if ( errortype==1 ) then

            if (resnorm<eps*bnorm ) then

               converged = .true.
               check = .false.

            end if ! (resnorm<eps*bnorm )

         else if ( errortype==1 ) then

            if ( resnorm<eps ) then

               converged = .true.
               check = .false.

            end if ! ( resnorm<eps )

         end if ! ( errortype==1 )

         if ( iprint>1 ) write(irefwr,200) iter, res2nrm, resinfnrm
200      format ( i5, 11x, d12.5, 10x, d12.5 )

!        --- Check whether an element has switched from fully liquid to
!            fully solid, or vice versa. If so, rewind time.

         if (checkphase) then

            call prcheckphase ( tm, rewind )
            if ( rewind ) goto 666

         end if

         if (check) then

!        --- check for gradf zero, i.e. spurious convergence

            check = prtimenthal3eu1 ( grad, temperature, &
                                      npoint, TOLMIN, f)

            if ( check .and. iprint>1 ) &
               write(irefwr,*) 'Convergence to local minimum'

         end if ! (check)

         if (.not. converged) then

!        --- check convergence on delta T

            converged = prtimenthal3eu2 ( temperature, tempold, npoint, &
                                          TOLX)

            if (converged .and. iprint>1) &
                 write(irefwr,*) 'Convergence on delta T'

         end if ! (.not. converged)

666      if (rewind) then

            iter = maxitertim
            write(irefwr,*) 'Element skipped phase transition'

         end if

      end do ! while ( .not. converged .and. iter<maxitertim )

      if ( converged ) then

         if ( rewind ) then

            iter = maxitertim + 1

         else

!        --- Converged = true, print number of iterations

            if ( iprint>0 ) write(irefwr,210) iter, res2nrm, resinfnrm
210         format ( 'Convergence after ', i4, ' iterations'/ &
                     '2-Norm of residual is ', d12.5,/ &
                     'inf-Norm of residual is ', d12.5 )

            if (debug) then

               ntimesteps = ntimesteps + 1
               totiter = totiter + iter
               meaniter = totiter/dble(ntimesteps)
               tstepmin = min(tstepmin, tstep)

               xhist = 0d0
               yhist = 0d0
               zhist = 0d0
               call findxy ( coor, xhist, yhist, zhist, &
                             npoint, nodhist, ndim, iprint )

               if ( iprint>1 ) then

                  print *, 'nodhist', nodhist
                  print *, 't, temphist', t, temperature(nodhist)
                  if (t>=tend-0.00001*tstep) then
                     print *, 'ntimesteps, totiter, meaniter, tstepmin', &
                         ntimesteps, totiter, meaniter, tstepmin
                  end if ! (t>=tend-0.00001*tstep)

               end if ! ( iprint>1 )

            end if

         end if

      end if  ! ( converged )

      if (iter>=maxitertim .and. .not. converged) then

         if ( iprint>0 ) write(irefwr,*) &
            'Maximum number of iterations reached, rewinding'
         rewind = .true.

      end if  ! (iter>=maxitertim)

!     --- While loop has been finished, reset quantities

      iflag = 3

      if ( .not. rewind .and. t>=tend-0.00001*tstep ) then

!     --- t = tend   reset all arrays

         call seprmsolinfo ( ideltatemp )
         call seprmmatrix ( stiffmat )
         call seprmsolinfo ( irhsd(1) )
         call seprmsolinfo ( irhsd(2) )
         finished = .true.

      end if  ! ( t>=tend-0.00001*tstep )
      deallocate ( phasenew, phaseold )

1000  if ( debug ) then
         write(irefwr,*) 'End septimenth3eu'
      end if  ! ( debug )
      call erclos ( 'septimenth3eu' )

      end subroutine septimenth3eu
