      subroutine septimeulexst ( first, isol, irhsdlc, isolprec, ieqn )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform one step in an Euler explicit scheme for one equation.
!     Coefficients may depend on time and previous solutions
!     It is supposed that the time has been raised outside this subroutine
!     Also that boundary conditions have been filled as well as arrays iuser
!     and user
!     This is the basic subroutine with respect to time integration.
!     All decisions are made outside this subroutine.
!     Old subroutine pr1107step
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulematr
      use sepmodulemain
      use sepmoduleintmat
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) ::  irhsdlc, isolprec, ieqn
      integer, intent(inout) :: isol(*)
      logical, intent(in) :: first

!     first          i    If true it concerns the first call in a row, &
!                         otherwise it concerns a subsequent call
!     ieqn           i    Equation sequence number
!     irhsdlc          i    Standard SEPRAN array, containing information of the
!                         large vector
!     isol          i/o   SEPRAN array containing information of the solution
!                         array. At input the boundary condition must have been
!                         filled in isol
!     isolprec       i    Array referring to the previous time-level
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ihelp, ihelp1

!     ihelp          Help array to store information of a help vector
!     ihelp1         Help array to store information of a help vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRCOMPREAC     Compute reaction force
!     PRININ         print 1d integer array
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPMAVER       perform matrix vector multiplication
!     SEPPRINTV      Print solution vector or vector of special structure
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     SEPTIMBUILDMAT Build mass matrix, stiffness matrix and right-hand side
!     SEPTIMSOLVE    Solve system of linear equations
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Differential equations to be solved:
!
!        .
!     M  u   +  S  u   =  f
!
!     Application of Euler
!
!
!       n+1     n
!      u    -  u            n      n
!    M ----------- = -S  u     +  f          (n is actual time-level)
!          dt
!
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimeulexst' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimeulexst'
         write(irefwr,*) 'dt, iseqdt ', dt, iseqdt
         call prinin ( isolprec, 5, 'isolprec' )
         call sepprintv ( isolprec, 'uold' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call septimbuildmat ( isol, irhsdlc, isolut2dred, first, ieqn )

!     --- Set first positions of help arrays to 0 (mm-position)

      ihelp = 0
      ihelp1 = 0
      iseqreactim = Teq(ieqn)%iseqreac

!     --- determine new solution at time level t (which is in fact t+dt)
!         help :=  M * u(old)

      call sepmaver ( massmat, isolprec, ihelp, 5 )
      if ( debug ) call sepprintv ( ihelp, 'M uold' )

!     --- Compute S * u(old)

      call sepmaver ( stiffmat, isolprec, ihelp1, 5 )
      if ( debug ) call sepprintv ( ihelp1, 'S uold' )

!     --- Subtract from ihelp

      help1 => ks(ihelp1)%sol
      help => ks(ihelp)%sol
      help = help - dt*help1
      if ( debug ) call sepprintv ( ihelp, 'M uold - dt*S uold' )

!     --- ihelp := ihelp + dt * f

      if ( Teq(ieqn)%iconsrhs/=2 ) then

!     --- f # 0

         rhsd => ks(irhsdlc)%sol
         help = help + dt*rhsd

      end if
      if ( debug ) &
         call sepprintv ( ihelp, 'M uold  - dt*S uold+ dt f' )

!     --- Build matrices and boundary conditions at new time level

      t = t + dt  ! raise time temporarily

      call septimbuildmat ( isol, irhsdlc, isolut2dred, .false., ieqn )

      t = t - dt  ! reset time

      if ( .not. realmt(massmat)%diagonal ) then

!     --- Compute effect of boundary conditions and add to right-hand side
!         help1 := M_helpir ur (n+1)

        call sepmaver ( massmat, isol(ieqn), ihelp1, 6 )
        if ( debug ) call sepprintv ( ihelp1, 'help1' )

!       --- help := help - help1

        help = help - help1
        if ( debug ) call sepprintv ( ihelp, 'help' )

      end if  ! ( massmat(2)/=110 )

!     --- solve system to obtain isol

      call septimsolve ( massmat, isol, ihelp, ieqn, irhsdlc, first )

      if ( iseqreactim/=0 ) then

!     --- Compute reaction forces, store in isol(ncopeqtim+1)

         call sepcompreac ( isol, stiffmat, ieqn, ncopeqtim+1, irhsdlc )

!        --- Copy reaction forces into isolut2dred

         call sepcopyvc ( isol(ncopeqtim+1), isolut2dred(iseqreactim) )

      end if  ! ( iseqreactim/=0 )

!     --- Delete the arrays ihelp and ihelp1

      call seprmsolinfo ( ihelp1 )
      call seprmsolinfo ( ihelp )

1000  if ( debug ) then

!     --- Debug information

         call sepprintv ( isol, 'sol' )
         write(irefwr,*) 'End septimeulexst'

      end if
      call erclos ( 'septimeulexst' )

      end subroutine septimeulexst
