      subroutine septimeurkfss
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.0  date 14-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    subroutine for the time integration of the discretized partial
!    differential equation
!              .
!          M * u + S * u = F
!              -       -   -
!    It has been assumed that the matrices M and S are not dependent
!    of the solution vector u . Hence the system is linear . M and
!    S may of course depend on place and time .
!    Solution method used is the Explicit Runge-Kutta-Fehlberg method.
!    The routine computes the timestep tstep itself such that the result
!    of the computation has an accuracy of epstim per unit step. The time-
!    step is saved in order to have the possibility to restart directly
!    with a large enough step.
!     Old subroutine pr1103
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulemain
      use sepmodulematr
      use sepmoduleintmat
      use sepmoduleuser
      use sepmoduletime
      use sepmodulesolve
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iresul, isol, iextra, ihelp, &
              itest, istar, irhold, &
              ifail, mdiag, ipoint, idum(1), idummy(1)
      double precision eps, p, q, difmax, told, tnew, tstep2, &
                       epscon, tstepo, a, b
      logical :: first
      save iextra, iresul, istar, isol, irhold, itest, tstepo

!     a
!     b
!     difmax  ?
!     eps            Local accuracy
!     epscon         Accuracy for conjugate gradient matrix
!     first          If true it concerns the first time step
!     idum           General dummy variable
!     idummy         Dummy parameter
!     iextra
!     ifail   ?
!     ihelp          sepran array to store intermediate result
!     ipoint
!     iresul  ?
!     irhold  ?
!     isol    Standard SEPRAN array containing information of the solution
!             array
!     istar   ?
!     itest   ?
!     mdiag   ?
!     p       ?
!     q       ?
!     tnew    ?
!     told    ?
!     tstep2  ?
!     tstepo  Last suggested time step, not adjusted for TOUT
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision sepnorm

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRSUB         Error messages
!     SEPALGEBR      algebraical operations on SEPRAN vectors
!     SEPCOPINPSOL   Copy arrays inpsl1 and rinsl1 into inpsol and rinsol
!                    into sepmodulesolve
!     SEPCOPYMT      Copy matrix
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPLINSOL      Body of linear solver
!     SEPMAVER       perform matrix vector multiplication
!     SEPNORM        Compute norm of vector
!     SEPRMMATRIX    Remove array with sequence number iseq from realm
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     SEPSOLDIR      Actual body of subroutine SOLVE
!     SEPTIMBUILDMAT Build mass matrix, stiffness matrix and right-hand side
!     SEPTIMSOLVE    Solve system of linear equations
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     454            ?
!     623   tout < t
!    1239   rintim(1) too small
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data iresul/0/,isol/0/,iextra/0/, &
           itest/0/,istar/0/,irhold/0/
! ======================================================================
!
      call eropen ( 'septimeurkfss' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimeurkfss'
         write(irefwr,2) 't, tout, tstep, tend', t, tout, tstep, tend
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      intmat => intmatar(1)

!     --- check input

      if ( tout<=t ) then
         call erreal ( t, 1 )
         call erreal ( tout, 2 )
         call errsub ( 623, 0, 2, 0 )
      end if
      if ( epstim< 100d0*epsmac ) then
         call erreal ( epstim, 1 )
         call erreal ( 100d0*epsmac, 2 )
         call errsub ( 1239, 0, 2, 0 )
      end if
      p = 0d0
      q = 0d0
      ipoint = 0
      iprobtim = intmt(intmat)%iprob
      massmat => massmt(iprobtim)
      stiffmat => stiffm(iprobtim:maxprb)
      totmat => mattot(iprobtim)

      first = .false.
      if ( t<=t0+0.1d0*tstep ) first = .true.

!     --- no errors in input
!         copy isolut2dred in isol

      call sepcopyvc ( isolut2dred,isol)

!     --- compute eps

      eps=100d0*epsmac
      epscon = max(eps, epstim*1d-3)
      rinsoltim(1) = epscon

!     --- set number of failures

      ifail=0

!     --- build matrices M , S and right-hand side F

      call septimbuildmat ( isol, irhsd, isolut2dred, first, 1 )
      if ( ierror/=0 ) go to 1000

      if ( realmt(massmat)%diagonal ) then
         mdiag=1
      else
         mdiag=2
      end if

!     --- fill irhold

      call sepcopyvc ( irhsd, irhold )

!     --- determine stepsize tstep

      call sepmaver (stiffmat,isolut2dred,itest,5 )
      a=-1d0
      b=1d0
      call sepalgebr ( 3, 0, itest, irhsd, iresul, &
                       a, b, p, q, ipoint )
      ipos=1
      call sepcopyvc ( isol, istar )
      call sepcopymt ( massmat,totmat )
      call septimsolve ( totmat, istar, iresul, 1, irhsd, first )
      difmax = sepnorm ( 1,3,0,istar,idum, idummy)
      if ( abs( t - t0 )<eps ) then

!     --- initial situation

         if ( difmax>eps ) then
            tstep=0.8d0*sqrt(epstim/(4d0*difmax))
         else
            tstep=tout-t+epsmac
         end if
      else
         tstep = tstepo
         if ( t + tstep>tout ) tstep = tout - t + epsmac
      end if
      iflag=2

!     --- First estimate ; Euler explicit

100   told=t
      tnew=t+tstep

!     --- determine new solution
!         compute  :  M * u(old)

      if ( mdiag==2 ) then
         call sepmaver (massmat,isolut2dred,itest,5 )
      else
         call sepmaver (massmat,isolut2dred,itest, 4 )
      end if

!     --- compute  :  S * u(old)

      call sepmaver ( stiffmat, isolut2dred, iextra, 5 )

!     --- add      :  M * u(old) - dt ( S * u(old) )

      a=1d0
      b=-tstep
      call sepalgebr ( 3, 0, itest, iextra, iresul, &
                       a, b, p, q, ipoint )

!     --- add      : dt * F(old)

      a=1d0
      call sepalgebr ( 3, 0, iresul, irhsd, iextra, &
                       a, tstep, p, q, ipoint )

!     --- compute matrices on new time level

      t=tnew
      call septimbuildmat ( isol, irhsd, isolut2dred, first, 1 )
      if ( mdiag==2 ) then

!     --- compute  :  M * boundaries on new time level , &

           call sepmaver (massmat,isol,itest, 6 )

!          --- add      :  -itest

           a=1d0
           b=-1d0
           call sepalgebr ( 3, 0, iextra, itest, iresul, &
                            a, b, p, q, ipoint )
      else
           call sepcopyvc ( iextra,iresul)
      end if
      ipos = 1

!     --- solve system to obtain isol

      call sepcopymt ( massmat,totmat )
      inpsol(1:ninpsol1,1) = inpsoltim(1:ninpsol1)

      iseqmt = totmat

      if ( .not. realmt(iseqmt)%diagonal ) then

!     --- Non-diagonal matrix

         ireadinpsol = -1
         iseqsol = 1
         iseqreac = 0
         iseqintmat = 1
         keepmat = 0
         iseqsolut = Teq(1)%iseqfr
         call sepcopinpsol ( inpsoltim, rinsoltim )
         call seplinsol ( totmat, isol, iresul )
      else

!     --- Diagonal matrix

         inpsol(2,1) = ipos
         call sepsoldir ( totmat, isol, iresul )

      end if  ! ( .not. realmt(iseqmt)%diagonal )

!     --- Second estimate ; Modified Euler method
!         Compute istar using correction formula

      a=1d0
      b=-1d0
      call sepalgebr ( 3, 0, isol, isolut2dred, iextra, &
                       a, b, p, q, ipoint )
      call sepmaver (stiffmat,iextra,itest,5 )
      tstep2=tstep/2d0
      a=-tstep2
      b=-tstep2
      call sepalgebr ( 3, 0, itest, irhold, iextra, &
                       a, b,  p, q, ipoint )
      a=1d0
      b=tstep2
      call sepalgebr ( 3, 0, iextra, irhsd, iresul, &
                       a, b, p, q, ipoint )

!     --- clear iextra

      a=0d0
      b=0d0
      call sepalgebr ( 3, 0, iresul, irhsd, iextra, &
                       a, b, p, q, ipoint )

!     --- solve system to obtain correction

      iseqmt = totmat
      if ( .not. realmt(iseqmt)%diagonal ) then

!     --- Non-diagonal matrix

         call sepcopinpsol ( inpsoltim, rinsoltim )
         call seplinsol ( totmat, iextra, iresul )

      else

!     --- Diagonal matrix

         inpsol(2,1) = ipos
         call sepsoldir ( totmat, iextra, iresul )

      end if  ! ( .not. realmt(iseqmt)%diagonal )

!     --- add correction to isol

      a=1d0
      b=1d0
      call sepalgebr ( 3, 0, iextra, isol, istar, &
                       a, b, p, q, ipoint )

!     --- determine new stepsize

      difmax = sepnorm ( 0,3,0,istar,isol, idummy)
      alpha=difmax/epstim
      alpha=min(100d0,alpha)
      alpha=max(0.01d0,alpha)
      dt=tstep/(sqrt(alpha))
      if ( alpha<1.0d0 ) then

!     --- accept result

           tstep=0.7d0*dt
           if ( tstep<eps ) tstep=eps
           call sepcopyvc ( istar,isolut2dred )
           call sepcopyvc ( irhsd,irhold )
           call sepcopyvc ( istar,isol )
           iflag=2
           ifail=0
      else

!     --- result cannot be accepted , recompute with smaller step

           tstep=0.5d0*dt
           if ( tstep<eps ) tstep=eps
           ifail=ifail+1
           t=told
           iflag=4
           if ( ifail>5 ) then
              call erreal ( t, 1 )
              call errsub ( 454, 0, 1, 0 )
              go to 1000
           end if
      end if

      if ( debug) &
         write(irefwr,2) 't, tout, tstep, tend/2', t, tout, tstep, tend

      if ( t>=tout-1d-4*tstep ) then
         iflag=3
         call sepcopyvc ( isol,isolut2dred )
         if(t>=tend-0.1*tstep) then

!        --- t = tend   reset all arrays

            call seprmsolinfo ( iextra  )
            call seprmsolinfo ( iresul  )
            call seprmsolinfo ( istar  )
            call seprmsolinfo ( isol  )
            call seprmsolinfo ( irhold  )
            call seprmmatrix ( massmat )
            call seprmmatrix ( stiffmat )
            call seprmmatrix ( totmat )
            call seprmsolinfo ( irhsd  )
            call seprmsolinfo ( itest  )
            itimloop = 0
         end if

         go to 1000

      end if

!     --- Save old timestep

      tstepo = tstep
      if ( t+tstep>tout ) tstep=tout-t+eps
      goto 100

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,2) 't, tout, tstep, tend', t, tout, tstep, tend
         write(irefwr,*) 'End septimeurkfss'

      end if  ! ( debug )
      call erclos ( 'septimeurkfss' )

      end subroutine septimeurkfss
