      subroutine septimpressmat ( iinpresscor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute mass matrix divided by dt and stiffness matrix with respect
!     to the predictor step of pressure correction
!     Also build right-hand side vector, pressure part and convection part
!     Create matrix (M/dt + S)
!     Create new right-hand side F = F_1 + Mu^n/dt
!     Old subroutine prtimpressmat
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     vector
!     time_integration
!     navier_stokes_equation
!     pressure_correction
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmoduleintmat
      use sepmodulemain
      use sepmodulebuild
      use sepmodulekprob
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinpresscor(*)

!     iinpresscor    i    Integer input array for the pressure correction
!                         process
!                         1: iseqveloc
!                         2: - &
!                         3: ireadcf
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iinbldlc(15), iseqveloc, &
                 ireadcf, iconsrhs, isubdivide
      double precision, pointer :: Mass(:)

!     Mass           Mass matrix
!     iconsrhs       if 0 do not build right-hand side, if 1 do
!     iinbldlc       User input array for subroutine BUILD
!     ireadcf        Sequence number of input for coefficients
!     iseqveloc      Sequence number of velocity vector in set of solution
!                    arrays
!     isubdivide     indicates if quadratic elements must be
!                    treated as a conglomerate of linear elements
!                    (1) or as quadratic elements itself (0)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERLINS         Help subroutine to call FILCOF, which fills coefficients in
!                    IUSER and USER
!     EROPEN         Produces concatenated name of local subroutine
!     PRFILLIINBLD   Fill arrays iinbld for building of matrix with defaults and
!                    some parameters from the input file
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPBUILD       Build matrix and right-hand side
!     SEPFILLSMF     Fill all input for subroutine SEPBUILD in SMF
!     SEPPRINTMAT    Print matrix
!     SEPPRINTV      Print solution vector or vector of special structure
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Set constants
!     Compute iuser1(iprob) and iuser2, indicating references to start addresses
!     of iuser and user
!     Build matrices M and S right-hand side f_1
!     Compute right-hand side vectors Gp^n and Convection at
!     previous level
!     Multiply mass matrix by 1/dt
!     Clear end of ibuffr
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimpressmat' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimpressmat'
         call prinin ( iinpresscor, 3, 'iinpresscor' )
         write(irefwr,110) 'dt', dt
110      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      intmat => intmatar(1)

!     --- Set constants

      if ( intmat>0 ) then
         iprob = intmt(intmat)%iprob
      else
         iprob = max(1,iprob)
      end if  ! ( intmat>0 )
      iseqveloc = iinpresscor(1)
      ireadcf = iinpresscor(3)

!     --- Compute iuser1(iprob) and iuser2, indicating references to start
!         addresses of iuser and user

      isubr = 3   ! used by LINPRB type subroutine
      call erlins ( isubr, iprob, ireadcf )
      if ( ierror/=0 ) go to 1000
      iuser => iuser1(iprob)%iuser
      user => iuser1(iprob)%user

!     --- Build matrices M and S right-hand side f_1

      iconsrhs = 1  !  build right-hand side
      imas = 2      !  consistent mass amtrix
      isubdivide = 0 ! no subdivision
      call prfilliinbld ( iinbldlc, iconsrhs, imas, iprob, numoldtim, &
                          iseqveloc, isubdivide )
      if ( debug ) then
         call prinin ( iinbldlc, 15, 'iinbld' )
         call prinin ( iuser, 50, 'iuser' )
         call prinrl ( user, 20, 'user' )
      end if  ! ( debug )

      call sepfillsmf ( iinbldlc, irhsd,isolut2d(iseqveloc), isolut2d )
      call sepbuild
      if ( ierror/=0 ) go to 1000

!     --- Multiply mass matrix by 1/dt

      if ( abs(dt-1d0)>1d-10 ) then
         Mass => realmt(massmt(1))%S_ii
         Mass = Mass/dt
      end if  ! ( abs(dt-1d0)>1d-10 )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Stiffness matrix'
         call sepprintmat ( stiffm )
         write(irefwr,*) 'Mass matrix'
         call sepprintmat ( massmt )
         call sepprintv ( irhsd, 'rhsd' )

      end if  ! ( debug )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septimpressmat'

      end if  ! ( debug )
      call erclos ( 'septimpressmat' )

      end subroutine septimpressmat
