      subroutine septimpresspr ( first, iinpresscor, first_step )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the prediction step in the pressure correction equation for
!     the incompressible Navier-Stokes equations
!     Old subroutine prtimpresspred
! **********************************************************************
!
!                       KEYWORDS
!
!     time_integration
!     navier_stokes_equation
!     pressure_correction
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulematr
      use sepmodulevecs
      use sepmodulemain
      use sepmodulesolve
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinpresscor(*)
      logical, intent(in) :: first
      logical, intent(inout) :: first_step

!     first          i    If true it concerns the first step
!     first_step    i/o   If true it concerns the first time step
!     iinpresscor    i    Integer input array for the pressure correction
!                         process
!                         See subroutine readpresscor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iincopy(2), iseqveloc, un, ihelp, ireadsl, &
                 iseqcf, mstep, unmin1, ihelp1, iinvec(2)
      double precision :: eps, factor, rinvec(2)
      double precision, pointer :: STIFF(:), TOTM(:), MASS(:)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     MASS           Refers to mass matrix
!     STIFF          Refers to stifness matrix
!     TOTM           Refers to combination of mass and stiffness matrix
!     eps            Local accuracy
!     factor         Multiplication factor
!     ihelp          integer work array to store a solution vector temporarily
!     ihelp1         integer work array to store a solution vector temporarily
!     iincopy        User input array for subroutine COPYVEC
!     iinvec         User input array for subroutine MANVEC
!                    For a description see subroutine MANVEC
!     ireadsl        Read sequence number corresponding to the solver
!     iseqcf         Sequence number corresponding to the coefficients
!     iseqveloc      Sequence number of velocity vector in set of solution
!                    arrays
!     mstep          Number of substeps per time step
!     rinvec         Real user input array for subroutine MANVEC
!                    The contents depend on iinvec
!     un             Contains information about the velocity at the previous
!                    time level n
!     unmin1         Contains information about the velocity at
!                    time level n-1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRFILLINPSOL   Fill arrays inpsol and rinsol for linear solver depending
!                    on input in input file or defaults
!     PRTIMCONV      Treat the convective terms in the Navier-Stokes equations
!                    in an explicit way using substepping and operator splitting
!     SEPCOPINPSOL   Copy arrays inpsl1 and rinsl1 into inpsol and rinsol
!                    into sepmodulesolve
!     SEPCOPYMT      Copy matrix
!     SEPCOPYVEC     copy Sepran array
!     SEPLINSOL      Body of linear solver
!     SEPMANVEC      Vector manipulation subroutine
!     SEPMAVER       perform matrix vector multiplication
!     SEPPRINTMAT    Print matrix
!     SEPPRINTV      Print solution vector or vector of special structure
!     SEPRMMATRIX    Remove array with sequence number iseq from realmt
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     SEPTIMBOUNCND  Transform solution vector to local coordinates
!                    Fill essential boundary conditions in solution vector
!     SEPTIMPRESSMAT Compute mass matrix divided by dt and stiffness matrix
!                    with respect to the predictor step of pressure correction
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2846   Time integration method not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The prediction step consists in the solution of one time step
!     using the old pressure but without the incompressiblity constraint
!     At this moment we have only implemented the Euler implicit time
!     integration in combination with an explicit convection term
!
!     Formulas:
!
!     M (u*-u^n) /dt + S u* + N(u^n)u^n - Gp^n = f^n+1
!
!     with M the mass matrix, S the viscosity part, N(u)u the convection
!     term, G the gradient matrix and f the right-hand side
!
!     Method:
!
!     input p^n, u^n
!     Copy u^n to u* &
!     Put new boundary conditions into u* &
!     Build mass matrix M/dt and stiffness matrix S
!     Build right-hand side vector F_1 consisting of f part, &
!     pressure part and convection part
!     Create matrix (M/dt + S)
!     Create new right-hand side F = F_1 + Mu^n/dt
!     Solve system of equations
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data unmin1/0/
! ======================================================================
!
      call eropen ( 'septimpresspr' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimpresspr'
         write(irefwr,2) 't, dt', t, dt
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000
      iseqveloc = iinpresscor(1)
      Teq(1)%iseqfr = iseqveloc     ! first vector is velocity
      eps = 0.01d0  ! temporary
      first_step = t<t0+dt*1.01d0  ! if true first time step
      massmat => massmt(1)

      if ( ichtim/=5 .and. ichtim/=13 ) then

!     --- Time integration method has not been implemented

         call errint ( ichtim, 1 )
         call errsub ( 2846, 1, 0, 0 )
         go to 1000

      end if  ! ( ichtim/=1 .and. ichtim/=13 ) then

      if ( debug ) then
         write(irefwr,1) 'ichtim', ichtim
      end if  ! ( debug )

!     --- Fill array inpsol and rinsol if necessary

      call prfillinpsol ( Teq(1)%ireadsl, inpsoltim, &
                          rinsoltim, ninpsol, nrinsol, eps )
      if ( ierror/=0 ) go to 1000

!     --- Copy u^n to u*

      iincopy(1) = 0
      un = 0
      call sepcopyvec ( isolut2d(iseqveloc), un, iincopy )
      if ( ierror/=0 ) go to 1000

!     --- Transform the solution in case of local transformations and fill
!         essential boundary conditions at the new time level if necessary
!         Back transformation is done by the linear solver

      call septimbouncnd ( first )
      if ( ierror/=0 ) go to 1000

!     --- Build mass matrix M/dt and stiffness matrix S
!         Build right-hand side vector F_1 consisting of f part,
!         pressure part and convection part

      irhsd(1) = 0
      call septimpressmat ( iinpresscor )
      if ( ierror/=0 ) go to 1000

      STIFF => realmt(stiffm(1))%S_ii

      IM => intmt(realmt(stiffm(1))%intmatref)
      MASS => realmt(massmat)%S_ii

      if ( iinpresscor(8)>0 ) then

!     --- The convection is treated by a number of substeps using an
!         operator splitting
!         This requires the mass matrix, the old solution and the
!         number of sub-steps

         mstep = iinpresscor(9)    ! number of sub-steps
         iseqcf = iinpresscor(10)  ! sequence number coefficients for convection
         ireadsl = iinpresscor(11) ! sequence number solver for mass matrix
         call prtimconv ( un, 1, mstep, iseqcf, ireadsl )
         if ( ierror/=0 ) go to 1000

      end if  ! ( iinpresscor(8)>0 )

!     --- Create new right-hand side F = F_1 + Mu^n/dt (Euler implicit)
!         or  F = F_1 + M (2 u^n - 1/2 u^n-1)/dt (Second order Gear)
!         ihelp :=  M * u(old) or M (2 u^n - 1/2 u^n-1)

      ihelp = 0
      if ( first_step .or. ichtim==5 ) then

!     --- First time step or Euler implicit

         call sepmaver ( massmat, un, ihelp, 5 )
         if ( ierror/=0 ) go to 1000

      else

!     --- Next time step and second order Gear
!         First store 2 u^n - 1/2 u^n-1 in ihelp1

         iinvec(1) = 2
         iinvec(2) = 27
         rinvec(1) = 2d0
         rinvec(2) = -0.5d0
         ihelp1 = 0
         call sepmanvec ( iinvec, rinvec, un,unmin1, ihelp1 )
         if ( ierror/=0 ) go to 1000

!        --- Next matrix vector multiplication

         call sepmaver ( massmat, ihelp1, ihelp, 5 )
         if ( ierror/=0 ) go to 1000

!        --- Remove ihelp1

         call seprmsolinfo ( ihelp1 )

      end if  ! ( first_step .or. ichtim==5 )
      if ( ierror/=0 ) go to 1000
      if ( debug ) call sepprintv ( ihelp, 'M uold' )

!     --- irhsd := irhsd + ihelp

      help => ks(ihelp)%sol
      rhsd => ks(irhsd(1))%sol
      rhsd = rhsd + help
      if ( debug ) call sepprintv ( irhsd, 'M uold + f' )

!     --- Create matrix (M/dt + S in case of Euler and 1.5d0*M/dt+S
!         in case of second order Gear

      if ( first_step .or. ichtim==5 ) then

!     --- First time step or Euler implicit

         factor = 1d0

      else

!     --- Next time step and second order Gear

         factor = 1.5d0

      end if  ! ( first_step .or. ichtim==5 )
      call sepcopymt ( stiffm, mattot )
      if ( ierror/=0 ) go to 1000
      TOTM => realmt(mattot(1))%S_ii
      if ( realmt(massmat)%diagonal ) then
         TOTM(IM%diag) = TOTM(IM%diag)+factor*MASS
      else
         TOTM = TOTM+factor*MASS
      end if  ! ( realmt(massmat)%diagonal )
      if ( ierror/=0 ) go to 1000
      if ( debug ) call sepprintmat ( mattot )

      if ( ichtim==13 ) then

!     --- second order Gear, store un in un-1

         call sepcopyvec ( un, unmin1, iincopy )

      end if  ! ( ichtim==13 )
      if ( ierror/=0 ) go to 1000

!     --- Remove un

      call seprmsolinfo ( un )
      if ( ierror/=0 ) go to 1000

!     --- Compute effect of boundary conditions and add to right-hand side
!         ihelp := M_helpir ur (n+1)

      call sepmaver ( mattot, isolut2d, ihelp, 6 )
      if ( ierror/=0 ) go to 1000
      if ( debug ) call sepprintv ( ihelp, 'effect bc' )

!     --- rhsd := rhsd - ihelp

      rhsd = rhsd - help
      if ( debug ) call sepprintv ( irhsd, 'rhsd' )

!     --- Remove ihelp

      call seprmsolinfo ( ihelp )
      if ( ierror/=0 ) go to 1000

!     --- Solve system of equations

      ireadinpsol = -1
      iseqsol = 1
      iseqreac = 0
      iseqintmat = 1
      keepmat = 0
      iseqsolut = 1
      iseqmt = mattot(1)
      call sepcopinpsol ( inpsoltim, rinsoltim )
      call seplinsol ( mattot, isolut2d, irhsd )
      if ( ierror/=0 ) go to 1000
      if ( debug ) call sepprintv ( isolut2d(iseqveloc), 'u*' )

!     --- Remove mattot and irhsd

      call seprmsolinfo ( irhsd(1) )
      call seprmmatrix ( mattot )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septimpresspr'

      end if
      call erclos ( 'septimpresspr' )

      end subroutine septimpresspr
