      subroutine septmenthalpy ( first, isol, isolprec, iseqenthalpy )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 15-12-2017 New storage of intmat
!        version  1.0  date 12-11-2015
!
!   copyright (c) 2015-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Perform one step in an Euler implicit scheme for the enthalpy equation.
!    Coefficients may depend on time and previous solutions
!    It is supposed that the time has been raised outside this subroutine
!    Also that boundary conditions have been filled as well as arrays iuser and
!    user
!    This is the basic subroutine with respect to time integration.
!    All decisions are made outside this subroutine.
!     Old subroutine prtmenthalpy
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulematr
      use sepmodulekprob
      use sepmodulemain
      use sepmoduleintmat
      use sepmodulevecs
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), isolprec, iseqenthalpy
      logical first

!     first          i    If true it concerns the first step
!     iseqenthalpy   i    Sequence number of input for block enthalpy
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution array
!     isolprec       i    Solution at previous time-level
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqlc, ihelp, ihelp1, i, n
      logical buildmat
      save buildmat
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     buildmat       If true the matrix and right-hand side must be built
!                    each time step
!                    they are not
!     i              Counting variable
!     ihelp          Help array to store information of a help vector
!     ihelp1         Help array to store information of a help vector
!     iseqlc           Sequence number of vector in VECOLD/ISLOLD
!     n              number of free degrees of freedom
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRSOLVEOVERRL  Solve system of linear equations by the overrelaxation
!                    method
!     SEPMAVER       perform matrix vector multiplication
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     SEPTIMBUILDMAT Build mass matrix, stiffness matrix and right-hand side
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Differential equations to be solved:
!
!        .
!     M  u   +  S  u   =  f
!
!     Application of Euler
!
!     -Euler part (predictor):
!
!       n+1     n
!      u    -  u            n+1      n
!    M ----------- +  S  u     =  f          (n is actual time-level)
!          dt
!
!
!    or.
!
!                 n+1        n         n+1
!   ( M + dt S ) u     =  M u    + dt f
!
!
!   Effect of boundary conditions:
!
!   We split the matrix into an internal part and a part due to
!   boundary conditions. The Euler part can then be written as:
!
!                        .
!   |  M       M   |  |  u  |      |  S       S   |  |  u  |      |  f  |
!   |   ii      ir |  |   i |      |   ii      ir |  |   i |      |   i |
!   |              |  |  .  |   +  |              |  |     |  =   |     |
!   |  M       M   |  |  u  |      |  S       S   |  |  u  |      |  f  |
!   |   ri      rr |  |   r |      |   ri      rr |  |   r |      |   r |
!
!   The last row is deleted, hence the discretized equations become:
!
!                    n+1          n+1        n+1          n          n+1   n
!   ( M   + dt S  ) u    =  dt ( f    - S   u   )  + M   u   - M  ( u   - u  )
!      ii       ii   i            i      ir  r        ii  i     ir   r     r
!
!
!
!   Store boundary conditions into usol
!   Build matrix M and S
!   Build right-hand side f without the effect of boundary conditions
!      i.e. the part fi
!   help := M un, i.e. Mii ui + Mir ur
!   help := help + dt f
!   M_help = M + dt S
!   help1 := M_helpir ur (n+1)
!   help  := help - help1
!   Solve M_help isol = help
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ihelp /0/ ihelp1 /0/
! ======================================================================
!
      call eropen ( 'septmenthalpy' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septmenthalpy'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call septimbuildmat ( isol, irhsd, isolut2d, first, 1 )
      if ( ierror/=0 ) go to 1000

!     --- determine new solution at time level t (which is in fact t+dt)
!         help :=  M * u(old)

      call sepmaver ( massmat, isolprec, ihelp, 5 )
      if ( debug ) write(irefwr,*) 'after maver/1'

!     --- ihelp := ihelp + dt f

      iprob = realmt(massmat)%iprob
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      n = kprobloc(5)

      if ( debug ) write(irefwr,1) 'n, iconsrhs', n, Teq(1)%iconsrhs

      usolt => ks(ihelp)%sol
      if ( Teq(1)%iconsrhs/=2 ) then

!     --- f # 0

         usoltemp => ks(irhsd(1))%sol
         usolt = usolt+dt*usoltemp

      end if

!     --- Compute effect of boundary conditions and add to right-hand side
!         help1 := M_helpir ur (n+1)

      call sepmaver ( stiffmat, isol(2), ihelp1, 6 )
      if ( debug ) write(irefwr,*) 'after maver/2'

!     --- help := help - help1

      usoltemp => ks(ihelp1)%sol
      usolt = usolt-dt*usoltemp
      if ( debug ) write(irefwr,*) 'after daxpy'

!     --- solve system to obtain isol

      if ( first ) then

!     --- First call, set some parameters in inpsoltim

         IM => intmt(intmat)
         if ( IM%direct .and. .not. IM%nested .and. .not. IM%mumps ) then

!        --- direct method

            if ( inpsoltim(3)>0 ) inpsoltim(3) = 0

         else

!        --- iterative method
!            use old solution and keep preconditioning if matrix is constant

            if ( inpsoltim(3)==0 ) inpsoltim(3) = 1
            inpsoltim(9)  = 1
            if ( Teq(1)%imasstot/=0 ) inpsoltim(10) = 1

         end if ! ( IM%direct .and. .not. IM%nested .and. .not. IM%mumps )

      else

!     --- next call, reuse preconditioning is possible

         if ( Teq(1)%imasstot/=0 ) inpsoltim(10) = 2

      end if

!     --- Give extra information to linear solver

      inpsoltim(17) = 10            ! overrelaxation for enthalpy method
      inpsoltim(39) = iseqenthalpy  ! sequence number of enthalpy input

!     --- Solve (non) linear system

      if ( debug ) write(irefwr,*) 'before solvelbf'
      call prsolveoverrl ( inpsoltim, rinsoltim, ihelp, isol )
      ks(isol(1))%compl = ks(irhsd(1))%compl
      if ( debug ) write(irefwr,*) 'after solvelbf'

!     --- Delete the arrays ihelp and ihelp1

      call seprmsolinfo ( ihelp1 )
      call seprmsolinfo ( ihelp )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septmenthalpy'

      end if  ! ( debug )
      call erclos ( 'septmenthalpy' )

      end subroutine septmenthalpy
