      subroutine toaddsiout55 ( storeinfo1, iwork1, leniwork1, &
                                amatii, amatiout )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-05-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Add information stored in storeinfo1 to matrices amatii and amatiout
!     Row compact storage (petsc)
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     matrix
!     real
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulebuild
      use sepmodulekprob
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: leniwork1
      integer, intent(inout) :: iwork1(leniwork1)
      double precision, intent(in) :: storeinfo1(*)
      double precision, intent(inout) :: amatii(*), amatiout(*)

!     amatii        i/o   Matrix part ii i.e. internal unknowns coupled
!                         to internal unknowns
!     amatiout      i/o   Matrix part i out i.e. part of rows with
!                         column numbers outside the present cpu
!     iwork1        i/o   integer work array to address column numbers
!     leniwork1      i    Declared length of array iwork1
!     storeinfo1     i    Array to store all information that is received
!                         from other blocks
!                         Comparable to storeinfo in subroutine incommatpar3
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, istart, ninterf, irow, istart1, &
                 istart2, icol, k, ilow, ihigh, j, iposout, ncols, l
      double precision :: aval

!     aval           Value of matrix element that is received
!                    they are not
!     i              Counting variable
!     icol           Column sequence number
!     ihigh          largest index of interface unknown+1
!     ilow           smallest index of interface unknown
!     iposout        position in array amatiout
!     irow           Row sequence number
!     istart         First position used at a part
!     istart1        First position used at a part
!     istart2        First position used at a part
!     j              Counting variable
!     k              Counting variable
!     l              Counting variable
!     ncols          number of non-zero columns in row
!     ninterf        Number of unknowns in interf (interface)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toaddsiout55' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toaddsiout55'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ilow = 0
      if ( inodenr>1 ) ilow = nrusollocs(inodenr-1)
      ihigh = nrusollocs(inodenr)

      if ( debug ) write(irefwr,1) 'ilow, ihigh', ilow, ihigh

      do i = 1, iacnodes

         istart = storeinfo(i)  ! start address of neighbor block information

         if ( debug ) write(irefwr,1) 'i, istart', i, istart

         if ( istart>0 ) then

!        --- istart>0, information available

            ninterf = storeinfo(istart)  ! number of interface unknowns
            istart1 = istart+ninterf+1

            if ( debug ) &
               write(irefwr,1) 'ninterf, istart1', ninterf, istart1

            do j = 1, ninterf

               irow = kprobhglobinv(storeinfo(istart+j))  !row number for ii
               istart2 = istart1+storeinfo(istart1+j-1)+ninterf
               ncols = storeinfo(istart1+j)-storeinfo(istart1+j-1)

!              --- Fill iwork with pointers to amatiout

               do k = irowsout(irow)+1, irowsout(irow+1)
                  iwork(icolsout(k)) = k
               end do
               do k = irowsii(irow)+1, irowsii(irow+1)
                  iwork1(icolsii(k)) = k
               end do

               if ( debug ) then
                  write(irefwr,1) 'j, irow, ncols', j, irow, ncols
                  call prinin ( storeinfo(istart2+1), ncols, 'cols' )
               end if  ! ( debug )

               do k = 1, ncols

                  icol = storeinfo(istart2+k)  ! global column number
                  aval = storeinfo1(istart2+k)

                  if ( icol<=ilow .or. icol>ihigh ) then

!                 --- Column number outside present block, add to amatiout

                     iposout = iwork(icol)
                     amatiout(iposout) = amatiout(iposout)+aval

                  else

!                 --- Column number in present block, add to amatii

                     icol = kprobhglobinv(icol)  ! local column number (ii)
                     iposout = iwork1(icol)
                     amatii(iposout) = amatii(iposout)+aval

                  end if  ! ( icol<=ilow .or. icol>ihigh )

               end do  ! k = 1, ncols

            end do  ! j = 1, ninterf

         end if  ! ( istart>0 )

      end do  ! i = 1, iacnodes

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End toaddsiout55'

      end if  ! ( debug )
      call erclos ( 'toaddsiout55' )

      end subroutine toaddsiout55
