      subroutine checkzerlev3d ( ilev )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-07-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check the faces in the zero level set definition
!     All edges that are present less than two times or more than
!     three times are printed
! **********************************************************************
!
!                       KEYWORDS
!
!     check
!     level_set
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ilev

!     ilev           i    Sequence number of level being considered
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kstart, numfaces, i, iface, &
              numedges, j, ipstart, jpstart, num, k, jface, kpstart, &
              num1, iedge
      integer, allocatable :: markedges(:), markfaces(:)

!     i              Counting variable
!     iedge          edge sequence number
!     iface          Face sequence number
!     ipstart        Starting address in array
!     j              Counting variable
!     jface          Face sequence number
!     jpstart        Starting address in array
!     k              Counting variable
!     kpstart        Starting address in array
!     kstart         Start value in obstfaces
!     markedges      array of length nedges indicating how many times
!                    an edge appears in the set of faces in obstfaces
!     markfaces      Array of length nfaces indicating whether a
!                    face is in obstfaces of not
!     num            number of faces connected
!     num1           number of nodes in face
!     numedges       number of edges
!     numfaces       number of faces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPFACEEDGE    Fill nmesh part i with the face numbers of the faces
!                    connected to the edges
!     SEPFILLNMESHA  Fill array nmesha
!     SEPFILLNMESHG  Fill array nmeshg
!     SEPFILLOBFACES Fill kmesh part ad subpart 9: iobstfaces
!     SEPPRINKMESH   Print array KMESH
!     SEPPRINNMESHC  print nmesh part c
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear markedges
!     for all faces in zero level set do
!        raise markedges for edges in face
!     for all faces in zero level set do
!        for all edges in face do
!           if ( markedges(edge)/=1 ) then
!              print edge number, markedges(edge) and face number
!              print all faces not in zero level set connected to edge
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'checkzerlev3d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from checkzerlev3d'
         call sepprinkmesh ( 45 )
         call sepprinnmeshc
         call sepprinkmesh ( 47 )
         call sepprinkmesh ( 64 )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate or create arrays

      if ( nmesh(31)==0 ) call sepfillnmesha
      if ( nmesh(37)==0 ) call sepfillnmeshg
      if ( nmesh(37)==0 ) call sepfaceedge
      if ( kmeshad(9)==0 ) call sepfillobfaces
      if ( ierror/=0 ) go to 1000


      kstart = obstfaces(ilev)
      numfaces = obstfaces(kstart)

      if ( debug ) &
         call prinin ( obstfaces(kstart+1), numfaces, 'obstfaces' )

      allocate ( markedges(nedges), markfaces(nfaces), stat = error )
      if ( error/=0 ) call eralloc ( error, nedges+nfaces, 'markedges' )
      if ( ierror/=0 ) go to 1000

!     --- Clear markedges and markfaces

      markedges = 0
      markfaces = 0

!     --- mark all faces in obstfaces

      markfaces(obstfaces(kstart+1:kstart+numfaces)) = 1

!     for all faces in zero level set do
!        raise markedges for edges in face

      do i = 1, numfaces

         iface = obstfaces(kstart+i)       ! face number
         ipstart = nmeshb(iface)            ! starting address
         numedges = nmeshb(ipstart)         ! number of edges in face
         do j = 1, numedges
            iedge = abs(nmeshb(ipstart+j))
            markedges(iedge) = markedges(iedge)+1
         end do  ! j = 1, numedges

      end do  ! i = 1, numfaces

!     for all faces in zero level set do
!        for all edges in face do
!           if ( markedges(edge)/=2 ) then
!              print edge number, markedges(edge) and face number
!              print all faces not in zero level set connected to edge

      do i = 1, numfaces

         iface = obstfaces(kstart+i)       ! face number
         ipstart = nmeshb(iface)            ! starting address
         numedges = nmeshb(ipstart)         ! number of edges in face
         do j = 1, numedges

            iedge = abs(nmeshb(ipstart+j))
            if ( markedges(iedge)/=2 ) then

!           --- markedges(edge)#1
!               print edge number, markedges(edge) and face number
!               print all faces not in zero level set connected to edge

               jpstart = nmeshc(iedge)
               num = nmeshc(jpstart)
               kpstart = nmeshg(iface)
               write(irefwr,100) iedge, nmeshc(jpstart+1), &
                   nmeshc(jpstart+num), iface, &
                   nmeshg(kpstart+1:kpstart+3), markedges(iedge)
100            format ( 'edge ', i6, ' ( ', i6,' , ', i6, ' )'/ &
                        'in face ', i6, ' ( ', i6,' , ', i6, &
                        ' , ', i6, ' )'/ &
                        'is present in ', i1, ' faces'/ &
                        'The edge belongs to the following faces', &
                        ' with nodes not in obstfaces'/ &
                        '   face           nodes' )

               jpstart = nmeshi(iedge)       ! starting address
               num = nmeshi(jpstart)         ! number of faces connected

               do k = 1, num

                  jface = nmeshi(jpstart+k)
                  if ( markfaces(jface)==0 ) then

!                 --- face is not in obstfaces

                     kpstart = nmeshg(jface)
                     num1 = nmeshg(kpstart)
                     write(irefwr,110) jface, &
                        nmeshg(kpstart+1:kpstart+num1)
110                  format(10i8)

                  end if  ! ( markfaces(jface)/=0 )

               end do  ! k = 1, num

            end if  ! ( markedges(nmeshb(ipstart+j))/=1 )

         end do  ! j = 1, numedges

      end do  ! i = 1, numfaces

      deallocate ( markedges, markfaces, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'markedges' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End checkzerlev3d'

      end if  ! ( debug )
      call erclos ( 'checkzerlev3d' )

      end
