      subroutine incm01 ( iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version 16.0  date 06-01-2016 Use sepmodulematstr
!        version 15.0  date 17-12-2015 New parameter list
!        version 14.3  date 28-07-2014 Extension with jmethod = 55
!
!   copyright (c) 1989-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute first part of contact matrix, i.e. fill array INTMAT part 1.
!     For each internal row it is stored how many degrees of freedom are coupled
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulekmesh
      use sepmodulematstr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(3,*)

!     iwork          o    work array of length 3*npoint to indicate if a point
!                         is periodical or not
!                         If iwork(1,i) = 0 then point is not periodical
!                         For a chain of periodical points, iwork is
!                         filled as follows:
!                         iwork(1,first_node_in_chain) = sequence number in
!                         iwork(2,.)
!                         iwork(1,next_nodes in chain) = -first_node_in_chain
!                         iwork(2,.) contains for each
!                         chain of periodical points the number of points that
!                         are connected to the first point. These numbers are
!                         accumulated
!                         iwork(2,1) = 1
!                         iwork(2,i+1)-iwork(2,i) = number of points connected
!                         to nodal point i
!                         iwork(3,.) contains the nodal points
!                         connected to the first point in a chain in the
!                         sequence given by iwork(2,.)
!                         Hence iwork(3,.) positions iwork(2,i) to
!                         iwork(2,i+1)-1
!                         contain all nodes from the chain starting with node i
!                         iwork is used in subroutine incm02
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical extracentr
      integer i, isum
      integer, allocatable :: icheck(:), istore(:,:)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     extracentr     If true extra fillin for the centres of discontinuous
!                    Navier-Stokes elements is required
!     i              Counting variable
!     icheck         work array of length nusol to indicate if a degree of
!                    is present in array istore (1) or not (0)
!     istore         work array of length nusolx2 to store the
!                    non-prescribed neighbour degrees of freedom of a point
!                    (i,1) contains the degree of freedom
!                    (i,2) contains the position of the unknown in the node
!     isum           Integer variable to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INCM011        Mark positions in array iwork corresponding to periodical
!                    boundary conditions and create chain of nodes
!     INCM012        Fill part of irowsii corresponding to global unknowns
!     INCM01BASIS    Compute first part of contact matrix
!                    For each internal row it is stored how many degrees of
!                    freedom are coupled
!                    Basis subroutine performs the main part
!     INCM01FICT     Fill part of irowsii corresponding to fictitious unknowns
!     INFIL1         Compute space needed for extra fill in if fillin_extra = 1
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2366   The computed length of the matrix is larger than the largest
!           integer
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear arrays irowsii, IWORK and ICHECK
!     Fill periodical points into IWORK
!     Fill essential boundary conditions in ICHECK
!     For node = 1 (1) NUMBER OF NODES do
!        If ( iwork(1,node)>=0 ) then
!           Store free unknows in array IFREE
!           If ( there are free unknowns in node ) then
!              If ( node is not periodical ) then
!                 Store all free unknowns coupled to point in ISTORE
!              else (node is periodical) then
!                 Store all free unknowns coupled to point and to corresponding
!                 periodical nodal points in ISTORE
!              Store number of free unknowns in irowsii
!              Store number of free unknowns in iblocks
!              Raise nblocks
!     Accumulate array irowsii
!     If ( jmethod=11 or 12 ) then
!        Re-arange the results
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incm01' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incm01'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      allocate ( icheck(nusol), istore(nusol,2), stat=error )
      if ( error/=0 ) call eralloc ( error, 3*nusol, 'icheck' )
      if ( ierror/=0 ) go to 1000

!     --- Set constants

      if ( extrafillin<=3 ) then

!     --- Standard situation with respect to fill-in

         extracentr = mod(extrafillin,2)==1

      else

         extracentr = .false.

      end if

!     --- clear irowsii, iwork and icheck

      irowsii(1:nrusol+1) = 0
      icheck(1:nrusol) = 0
      icheck(nrusol+1:nusol) = 1

!     --- Mark positions in iwork corresponding to periodical boundary
!         conditions

      if ( debug ) write(irefwr,1) 'npboun', npboun
      if ( npboun>0 )  call incm011 ( iwork )

!     --- Main part of subroutine, compute information over rows

      if ( debug ) write(irefwr,*) 'before incm01basis'
      call incm01basis ( iwork, icheck, istore )
      if ( ierror/=0 ) go to 1000
      if ( debug ) call prinin ( irowsii, nrusol+1, 'irowsii/1' )

!     --- if indpry>0, there are fictitious elements
!         treat these separately in incm01fict

      if ( indpry>0 ) call incm01fict ( istore, icheck )

!     --- if indprx>0, there are global elements
!         treat these separately in incm012

      if ( indprx>0 ) call incm012 ( istore )

      if ( extracentr ) then

!     --- extracentr = true, fill extra fill in for neighbours of pressure

         call infil1

      end if

!     --- Accumulate irowsii

      irowsii(1) = 0
      do i = 2, nrusol+1
         irowsii(i) = irowsii(i) + irowsii(i-1)
      end do

      if ( debug ) call prinin ( irowsii, nrusol+1, 'irowsii/2' )

!     --- Check on integer overflow

      isum = 0
      do i = 1, nrusol+1
         if ( irowsii(i)<isum ) then

!        --- This is not possible, hence we have integer overflow

            call errsub ( 2366, 0, 0, 0 )
            go to 1000

         end if
         isum = irowsii(i)
      end do

      deallocate ( icheck, istore, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'icheck' )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( irowsii, nrusol+1, 'irowsii' )
         write(irefwr,*) 'End incm01'

      end if  ! ( debug )
      call erclos ( 'incm01' )

      end
