      subroutine incm02fict ( istore )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 06-01-2016 Use sepmodulematstr
!        version  4.0  date 17-12-2015 New parameter list
!        version  3.0  date 12-10-2007 Extra parameter nusol
!
!   copyright (c) 1999-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Fill second part of contact matrix, with respect to fictitious unknowns
!   For each internal row the neighbours are stored.
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     fictitious_domain
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulematstr
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istore(nusol,3)

!     istore              work array of length 2*nusol
!                         The first nusol positions are used to store the
!                         non-prescribed neighbour degrees of freedom of a
!                         point
!                         In the third nusol positions the position of the
!                         last column number stored for each row is put
!                         Hence the next free position for a column in row i
!                         is defined by istore(nusol+i)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isum, knusol, inusol, i1, i2, inpelm, inunkp, &
              node, k, nusolclassic, icurnr, inunkpgr, istoreloc(54), &
              ielemstruct, ikelmcfluid, ikelmcstruct, inpelmfluid, &
              inpelmstruct, ishapelam, icurve1, icurve2, igrp, j, &
              ielemlowstruct, ielemfluid, ielemlowfluid, l, &
              ifluidgrp1, ikelmc, iposkproby, &
              iregionlam, istructgrp, nfictpnt, inunkp_elm, &
              isurnr, ishape_curve

!     i              Counting variable
!     i1             First point of curve in KMESH part m
!     i2             Last point of curve in KMESH part m
!     icurnr         Curve sequence number
!     icurve1        First curve number of a set
!     icurve2        Last curve number of a set
!     ielemfluid     Element sequence number of fluid element
!     ielemlowfluid  Lowest element sequence number of fluid elements
!     ielemlowstruct Lowest element sequence number of structural elements
!     ielemstruct    Element sequence number of structural element
!     ifluidgrp1     First fluid element group number
!     igrp           Boundary element group number read
!     ikelmc         position in kmesh part c
!     ikelmcfluid    Position in kmeshc part c with respect to fluid elements
!     ikelmcstruct   Position in kmeshc part c with respect to structural
!                    elements
!     inpelm         number of points per element
!     inpelmfluid    number of points per fluid element
!     inpelmstruct   number of points per structural element
!     inunkp         Number of degrees of freedom in point
!     inunkp_elm     Number of unknowns per point
!     inunkpgr       Number of global unknowns in a group
!     inusol         Position of unknown in array USOL
!     iposkproby     Position in kprob y
!     iregionlam     type of region where the lambda's are defined:
!                     1:  curves
!                     2:  surfaces
!     ishape_curve   Shape of lambda basis functions
!     ishapelam      Shape of lambda basis functions
!     istoreloc      Local array to store quantities
!                    In this case the nodes in the fluid and structural
!                    element are stored
!     istructgrp     Structural element group
!     isum           Integer variable to compute a sum
!     isurnr         Surface sequence number
!     j              Counting variable
!     k              Counting variable
!     knusol         Position of unknown in array USOL
!     l              General loop variable
!     nfictpnt       Total number of points with fictitious unknowns
!     node           Node number
!     nusolclassic   Number of free classical unknowns, i.e ficitious unknowns
!                    and global unknowns excluded
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!      Initialise parameters
!      For igrp := 1 (1) nfictgrp
!         Get some relevant parameters
!         Compute starting address of structural elements in kmeshc
!         Compute starting address of fluid elements in kmeshc
!         if ( region is of type curves ) then
!            For all curves in region do
!               Find structural and fluid element containing the lambdas
!               For all elements along curve
!                  For all lambda points in element
!                     For all nodes in the structural element
!                        Store the nodes in istoreloc
!                     For all nodes in the fluid element
!                        Store the nodes in istoreloc
!                     For all nodes in istoreloc
!                        For all unknowns in node
!                           if (unknown not prescribed ) then
!                              store information in icolsii
!         else ( region is of type surfaces )
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incm02fict' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incm02fict'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nfictpnt = 0
      nusolclassic = nusol-nglunk-nfictunk

!     --- Loop over fictitious groups

      do igrp = 1, nfictgrp

!     --- Redefine inpelm for multiple fictitious groups

         if ( igrp>1 ) inpelm = npelm

!        --- Clear array istore

         istore(1:nusol,1) = 0

!        --- Get some relevant parameters

         iposkproby = kproby(10+igrp)-1
         inunkpgr   = kproby(iposkproby+2)
         iregionlam = kproby(iposkproby+3)
         ishapelam  = kproby(iposkproby+4)
         icurve1    = kproby(iposkproby+5)
         icurve2    = kproby(iposkproby+6)
         istructgrp = kproby(iposkproby+7)
         ifluidgrp1 = kproby(iposkproby+8)
         inunkp_elm = kproby(iposkproby+10)/inunkpgr

         if ( debug ) then
            write(irefwr,1) 'igrp, iposkproby, inunkpgr, iregionlam', &
                             igrp, iposkproby, inunkpgr, iregionlam
            write(irefwr,1) 'icurve1, icurve2, istructgrp', &
                             icurve1, icurve2, istructgrp
            write(irefwr,1) 'ifluidgrp1, inunkp_elm, ishapelam', &
                             ifluidgrp1, inunkp_elm, ishapelam
         end if  ! ( debug )

!        --- Compute starting address of structural elements in kmeshc

         ikelmcstruct = 0
         ielemlowstruct = 1
         do i = 1, istructgrp-1
            inpelm = sepgetinpelm(i)
            inelem = sepgetinelem(i)
            ikelmcstruct = ikelmcstruct+inelem*inpelm
            ielemlowstruct = ielemlowstruct+inelem
         end do
         inpelmstruct = sepgetinpelm(istructgrp)

!        --- Compute starting address of fluid elements in kmeshc

         ikelmcfluid = 0
         ielemlowfluid = 1
         do i = 1, ifluidgrp1-1
            inpelm = sepgetinpelm(i)
            inelem = sepgetinelem(i)
            ikelmcfluid = ikelmcfluid+inelem*inpelm
            ielemlowfluid = ielemlowfluid+inelem
         end do
         inpelmfluid = sepgetinpelm(ifluidgrp1)

         if ( iregionlam==1 ) then

!        --- region is of type curve
!            ishape := shape number of structural element

            ishape = sepgetshape(istructgrp)
            if ( ishape==3 .or. ishape==5 ) then

!           --- ishape = 3 or 5

               ishape_curve = 1

            else if ( ishape==4 .or. ishape==6 ) then

!           --- ishape = 4 or 6

               ishape_curve = 2

            else

!           --- Shape number not implemented

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               ishape_curve = 1

            end if
            if ( debug ) &
               write(irefwr,1) 'ishape, ishape_curve', &
                                ishape, ishape_curve

!           --- loop over curves

            do icurnr = icurve1, icurve2

               i1 = kmeshm(4+icurnr)
               i2 = kmeshm(5+icurnr)-1
               if ( debug ) &
                  write(irefwr,1) 'icurnr, i1, i2', icurnr, i1, i2

!              --- Find structural and fluid element containing the lambdas
!                  Loop over elements along curve

               do i = i1, i2-1, ishape_curve

!              --- Loop over lambda points in element

                  do j = 1, ishapelam
                     nfictpnt = nfictpnt+1
                     ielemstruct = kprobfict(1,nfictpnt)

!                    --- Loop over the nodes in the structural element
!                        Store the nodes in istoreloc

                     ikelmc = ikelmcstruct+ &
                              (ielemstruct-ielemlowstruct)*inpelmstruct
                     istoreloc(1:inpelmstruct) = &
                        kmeshc(ikelmc+1:ikelmc+inpelmstruct)

!                    --- Loop over the nodes in the fluid element
!                        Store the nodes in istoreloc

                     ielemfluid = kprobfict(2,nfictpnt)
                     ikelmc = ikelmcfluid+ &
                              (ielemfluid-ielemlowfluid)*inpelmfluid
                     do k = 1, inpelmfluid
                        istoreloc(k+inpelmstruct) = kmeshc(ikelmc+k)
                     end do
                     do k = 1, inpelmstruct+inpelmfluid
                        node = istoreloc(k)
                        inusol = sepgetinusol ( node, inunkp )
                        if ( indprr==0 ) then

!                       --- indprr = 0

                           do l = 1, inunkp

                              if ( istore(inusol+l,1)==0 ) then

!                             --- New unknown found

                                 istore(inusol+l,1) = 1

                              end if

                           end do

                        else

!                       --- indprr>0

                           do l = 1, inunkp
                              knusol = kprobr(inusol+l)

                              if ( istore(knusol,1)==0 .and. &
                                   knusol<=nrusol ) then

!                             --- New unknown found

                                 istore(knusol,1) = 1

                              end if

                           end do

                        end if

                     end do

!                    --- Fill array irowsii for all fictitious unknowns
!                        corresponding to ifictgrp

                     do l = 1, inunkpgr
                        knusol = l+nusolclassic
                        if ( indprr>0 ) knusol = kprobr(knusol)
                        isum = irowsii(knusol)
                        do k = 1, nrusol
                           if ( istore(k,1)/=0 ) then
                              isum = isum+1
                              icolsii(isum) = k
                           end if
                        end do
                        do k = 1, l-1
                           isum = isum+1
                           icolsii(isum) = knusol-l+k
                        end do
                     end do
                     nusolclassic = nusolclassic+inunkpgr
                     do l = irowsii(knusol)+1, irowsii(knusol+1)
                        istore(icolsii(l),1) = 0
                     end do

                  end do

               end do

            end do

         else

!        --- region is of type surface
!            loop over surfaces

            do isurnr = icurve1, icurve2

               inelem = kmeshn(nsurfs+isurnr)
               inpelm = kmeshn(isurnr)

!              --- Find structural and fluid element containing the lambdas
!                  Loop over elements along surface

               do i = 1, inelem

!              --- Loop over lambda points in element

                  do j = 1, inunkp_elm
                     nfictpnt = nfictpnt+1
                     ielemstruct = kprobfict(1,nfictpnt)

!                    --- Loop over the nodes in the structural element
!                        Store the nodes in istoreloc

                     ikelmc = ikelmcstruct+ &
                              (ielemstruct-ielemlowstruct)*inpelmstruct
                     do k = 1, inpelmstruct
                        istoreloc(k) = kmeshc(ikelmc+k)
                     end do

!                    --- Loop over the nodes in the fluid element
!                        Store the nodes in istoreloc

                     ielemfluid = kprobfict(2,nfictpnt)
                     ikelmc = ikelmcfluid+ &
                              (ielemfluid-ielemlowfluid)*inpelmfluid
                     do k = 1, inpelmfluid
                        istoreloc(k+inpelmstruct) = kmeshc(ikelmc+k)
                     end do

!                    --- For all nodes in istoreloc

                     do k = 1, inpelmstruct+inpelmfluid
                        node = istoreloc(k)
                        inusol = sepgetinusol ( node, inunkp )
                        if ( indprr==0 ) then

!                       --- indprr = 0

                           do l = 1, inunkp

                              if ( istore(inusol+l,1)==0 ) then

!                             --- New unknown found

                                 istore(inusol+l,1) = 1

                              end if

                           end do

                        else

!                       --- indprr>0

                           do l = 1, inunkp
                              knusol = kprobr(inusol+l)

                              if ( istore(knusol,1)==0 .and. &
                                   knusol<=nrusol ) then

!                             --- New unknown found

                                 istore(knusol,1) = 1

                              end if

                           end do

                        end if

                     end do

!                    --- Fill array irowsii for all fictitious unknowns
!                        corresponding to ifictgrp

                     do l = 1, inunkpgr
                        knusol = l+nusolclassic
                        if ( indprr>0 ) knusol = kprobr(knusol)
                        isum = irowsii(knusol)
                        do k = 1, nrusol
                           if ( istore(k,1)/=0 ) then
                              isum = isum+1
                              icolsii(isum) = k
                           end if
                        end do
                        do k = 1, l-1
                           isum = isum+1
                           icolsii(isum) = knusol-l+k
                        end do
                     end do

                     nusolclassic = nusolclassic+inunkpgr
                     do l = irowsii(knusol)+1, irowsii(knusol+1)
                        istore(icolsii(l),1) = 0
                     end do

                  end do

               end do

            end do

         end if

!        --- Raise parameters

         iposkproby = iposkproby+12

      end do

1000  call erclos ( 'incm02fict' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End incm02fict'

      end if  ! ( debug )

      end
