      subroutine incm05 ( intmt1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.0  date 06-01-2016 Use sepmodulematstr
!        version  5.0  date 17-12-2015 New parameter list
!        version  4.2  date 21-04-2015 New definition intmat
!
!   copyright (c) 1989-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill part of array icolsic with connections corresponding to the
!     "right" part of the periodical unknowns.
!     INCM05 must be preceded by INCM03 and should only be called if
!     ndepen>0.
!     INCM05 is the counterpart of ini011 with respect to periodical
!     boundary conditions. It must be called after ini011 !
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulematstr
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer intmt1(*)

!     intmt1         i    First part of array INTMAT part 2, containing the
!                         number of connections of essential boundary conditions
!                         and "right" periodical boundary conditions accumulated
!                         INTMT1 must have been filled by ini011 and incm04
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer knunkp, ihigh, ilow, inodp, inpelm, inunkp, &
              inusol, nhelp1, nhelp2, j, knusol, nfnbnr, ibr, ineigh, &
              k, ipos, jnusol, &
              ikelmc, ielgrp, itype, iwork(npoint), ihelp(nusol)

!     ibr            Relative neighbour number of node
!     ielgrp         element group number
!     ihelp          Work array in which all connections of a point to
!                    non-prescribed degrees of freedom is stored. The
!                    "left" periodical degrees of freedom are excluded
!     ihigh          Highest position number in USOL with respect to "right"
!                    periodical boundary conditions
!                    ihigh = nusol-nbound
!     ikelmc         position in kmesh part c
!     ilow           Lowest position number in usol with respect to "right"
!                    periodical boundary conditions.
!                    ilow = nusol-nbound-ndepen+1
!     ineigh         Node number of neighbour of inodp
!     inodp          Nodal point number
!     inpelm         number of points per element
!     inunkp         Number of degrees of freedom in point
!     inusol         Position of first unknown in array usol corresponding to
!                    point minus one, before renumbering
!     ipos           Position in array icolsic
!     itype          Type number of element group
!     iwork          Work array of length npoint that will be filled in the
!                    following way
!                    iwork(i)=0   i is not a "right" periodical point
!                    iwork(i)>0   i is a "right" periodical point. The
!                                 value of iwork(i) gives the corresponding
!                                 "left"  nodal point.
!     j              Counting variable
!     jnusol         See inusol, however, with respect to ineigh
!     k              Counting variable
!     knunkp         Number of unknown in node ineigh
!     knusol         Renumbered jnusol or inusol
!     nfnbnr         Help variable to compute neighbour starting position of
!                    neighbour of node inodp in KMSHE2 minus 1
!     nhelp1         Number non-prescribed unknowns in "right" periodical node
!     nhelp2         Number of non-prescribed unknowns connected to unknowns in
!                    "right" periodical node, excluded the corresponding "left"
!                    periodical point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill array iwork
!
!     for inodp := 1 (1) npoint do
!        if ( iwork(inodp)>0 ) then
!           compute inusol and inunkp
!           nhelp1 := 0 ; nhelp2 := 0
!           for j := 1 (1) inunkp
!              compute knusol
!              if (knusol<=ihigh) then
!                 nhelp2 := nhelp2 + 1
!                 ihelp(nhelp2) := knusol
!                 nhelp1 := 1
!           if ( nhelp1>0 ) then
!              for ibr := 1(1) number of neightbours of inodp do
!                 ineigh := ibrth neighbour of inodp
!                 if ( iwork(inodp) # ineigh) then
!                    store all internal degrees of freedom in icolsic
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incm05' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incm05'

      end if

!     --- Set some constants

      knunkp = inunkp
      ihigh  = nusol-nbound
      ilow   = ihigh-npboun+1
      if ( debug ) write(irefwr,*) &
         'npoint,indprf,inunkp,nusol,nbound,ilow, ihigh ', &
          npoint,indprf,inunkp,nusol,nbound,ilow, ihigh

!     --- fill array iwork

      iwork = 0
      ikelmc = 0

!     --- loop over element groups

      do ielgrp = 1, nelgrp

         itype = kprobc(ielgrp)
         inelem = sepgetinelem(ielgrp)
         inpelm = sepgetinpelm(ielgrp)
         if ( debug ) write(irefwr,*) 'ielgrp, itype, inelem, inpelm ', &
             ielgrp, itype, inelem, inpelm
         if ( itype==-1 ) then

!        --- loop over elements

            do inodp=1, inelem
               iwork(kmeshc(ikelmc+2)) = kmeshc(ikelmc+1)
               ikelmc = ikelmc+2
            end do
         else
            ikelmc = ikelmc + inelem*inpelm
         end if

      end do  ! ielgrp = 1, nelgrp

      if ( numpairs>0 ) then

!     --- numpairs (numpairs)>0, hence periodical boundary conditions

         inelem = numpairs
         do inodp = 1, inelem

!        --- loop over pairs

            iwork(kprobper(2,inodp)) = kprobper(1,inodp)

         end do  ! inodp = 1, inelem
         inelem = nelem

      end if  ! ( numpairs>0 )

!     --- loop over nodes

      do inodp = 1, npoint

         if ( iwork(inodp)>0 ) then

!        --- iwork(inodp)>0, so node is "right" periodical point
!            Compute inunkp and inusol

            inusol = sepgetinusol ( inodp, inunkp )
            nhelp1 = 0
            nhelp2 = 0
            if ( debug ) write(irefwr,*) 'inodp, inusol ', inodp, inusol

!           --- loop over unknowns; fill ihelp

            do j = 1, inunkp
               knusol = kprobh(inusol+j)
               if ( knusol<=ihigh ) then
                  nhelp2 = nhelp2+1
                  ihelp(nhelp2) = knusol
                  nhelp1 = 1
               end if
            end do
            if ( debug ) call prinin (ihelp,nhelp2,'ihelp')
            if ( nhelp1>0 ) then

!           --- nhelp1 >0, so there are free degrees of freedom in periodical
!               node

               nfnbnr = kmshe1(inodp)

!              --- loop over neighbours

               do ibr = 1, kmshe1(inodp+1)-nfnbnr
                  ineigh = kmshe2(nfnbnr+ibr)
                  if ( iwork(inodp)/=ineigh) then

!                 --- ineigh # iwork(inodp), i.e. ineigh is not "left"
!                     periodical point
!                     Compute jnusol and knunkp

                     jnusol = sepgetinusol ( ineigh, knunkp )

!                    --- loop over unknowns

                     do j = 1, knunkp
                        knusol = kprobh(jnusol+j)
                        if ( knusol<=ihigh ) then

!                       --- knusol is non-prescribed

                           nhelp2 = nhelp2 + 1
                           ihelp(nhelp2) = knusol
                        end if
                     end do
                  end if
               end do
               if ( debug ) call prinin (ihelp,nhelp2,'ihelp-2')

!              --- unknowns are stored in array ihelp2. Fill icolsic
!                  loop over unknowns

               do j = 1, inunkp
                  knusol = kprobh(inusol+j)
                  if ( knusol>=ilow .and. knusol<=ihigh ) then

!                 --- knusol<=ilow, unknown is free

                     ipos = intmt1(knusol-ilow+1)
                     if ( debug ) &
                        write(irefwr,*) 'ipos, knusol ', ipos, knusol
                     do k = 1, nhelp2
                        icolsic(ipos+k) = ihelp(k)
                     end do
                  end if
               end do
            end if
         end if
      end do

      call erclos ( 'incm05' )
      if ( debug ) write(irefwr,*) 'End incm05'

      end
