      subroutine incmkmeshe ( kmshe1, kmshe2, npoint, kmeshe1new, &
                              kmeshe2new )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-06-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create a new temporary array kmesh part e in case of periodical
!     boundary conditions
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     periodical
!     boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, kmshe1(npoint+1), kmshe2(*), kmeshe1new(npoint+1), &
              kmeshe2new(*)

!     kmeshe1new     o    New array kmesh e part 1 consisting of the old one
!                         extended by the contribution of periodical nodes
!     kmeshe2new     o    New array kmesh e part 2 consisting of the old one
!                         extended by the contribution of periodical nodes
!     kmshe1         i    Contains for each nodal the starting point of the
!                         neighboring points  in array kmshe2 minus one
!                         kmshe1(1) = 0
!                         number of neighbors of node i is
!                         kmshe1(i+1)-kmshe1(i)
!     kmshe2         i    Contains the neighbors of all the nodes
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, nodp, k, neighbor, isave, icount, nneigh, &
              ih, numfilled
      integer, allocatable :: iwork(:), istart(:), kmshe1tmp(:), &
                              kmshe2tmp(:), filled(:)

!     error          Return value of allocate or deallocate
!     filled         Array of size npoint containing the number of nodes that
!                    has been filled in kmshe2tmp
!     i              Counting variable
!     icount         Counter
!     ih             Help variable to store a constant temporarily
!     isave          Help variable to save a parameter temporarily
!     istart         First position used at a part
!     iwork          integer work array of size npoint to mark neighbors of
!                    nodes that already have been stored
!     j              Counting variable
!     k              Counting variable
!     kmshe1tmp      Same definition as kmshe1 but now restricted to periodical
!                    nodes only
!     kmshe2tmp      Same definition as kmshe2 but now restricted to periodical
!                    nodes only
!     neighbor       Neighbor of node
!     nneigh         Number of neighbors of node
!     nodp           Nodal point sequence number
!     numfilled      Number of nodes filled in kmshe2tmp
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PR0012         Print accumulated array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear kmshe1tmp(1:npoint)
!     Fill kmshe1tmp with number of times a node is in kprobper
!     Accumulate  kmshe1tmp(kmshe1tmp(1)=0)
!     Clear iwork(1:npoint)
!     Fill array kmeshe1new(1:npoint) with present number of neighbors
!     For all nodes in kprobper do
!        set iwork to 1 for each neighbor of node
!        if ( iwork(neighbor)==0 ) then
!           raise kmeshe1new(node)
!        clear iwork
!     Accumulate kmeshe1new
!     Fill istart(1:npoint) by copying from kmeshe1new
!     Copy kmeshe2 to kmeshe2new and raise istart
!     For all nodes in kprobper do
!        set iwork to 1 for each neighbor of node
!        if ( iwork(neighbor)==0 ) then
!           fill neighbor in kmeshe2new and raise istart(node)
!        clear iwork
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incmkmeshe' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incmkmeshe'
         write(irefwr,1) 'numpairs, npoint', numpairs, npoint
         call prinin1 ( kprobper, numpairs, 2, 'kprobper' )
         write ( irefwr, '(/a/)') &
               'Original nodal point connected to nodal points'
         call pr0012 ( npoint, kmshe1, kmshe2 )

  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(npoint), istart(npoint), kmshe1tmp(npoint+1), &
                 kmshe2tmp(2*numpairs), filled(npoint), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, 3*npoint+1+2*numpairs, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear kmshe1tmp(1:npoint)

      kmshe1tmp = 0
      filled = 0

!     --- Fill with number of times a node is in kprobper

      do i = 1, numpairs

         do j = 1, 2
            nodp = kprobper(j,i)
            kmshe1tmp(nodp) = kmshe1tmp(nodp)+1
         end do  ! j = 1, 2

      end do  ! i = 1, numpairs

!     --- Accumulate kmshe1tmp (kmshe1tmp(1)=0)

      icount = 0
      do i = 1, npoint

         isave = kmshe1tmp(i)
         kmshe1tmp(i) = icount
         icount = icount+isave

      end do  ! i = 1, npoint
      kmshe1tmp(npoint+1) = icount

!     --- Clear iwork(1:npoint)

      iwork = 0

!     --- Fill array kmeshe1new(1:npoint) with present number of neighbors

      do i = 1, npoint
         kmeshe1new(i) = kmshe1(i+1)-kmshe1(i)
      end do  ! i = 1, npoint

!     --- For all nodes in kprobper do
!           set iwork to 1 for each neighbor of node
!           if ( iwork(neighbor)==0 ) then
!              raise kmeshe1new(node)
!           clear iwork

      do i = 1, numpairs

         do j = 1, 2  ! both nodes in pair

            nodp = kprobper(j,i)  ! node in pair
            k = mod(j,2)+1
            neighbor = kprobper(k,i) ! neighbor in pair

            iwork(kmshe2(kmshe1(nodp)+1:kmshe1(nodp+1))) = 1 ! all neighbors
            numfilled = filled(nodp)
            ih = kmshe1tmp(nodp)

            if ( debug ) write(irefwr,1) 'ih, numfilled', ih, numfilled
            if ( numfilled>0 ) iwork(kmshe2tmp(ih+1:ih+numfilled)) = 1

            if ( debug ) &
               write(irefwr,1) 'i, j, nodp, neighbor, iwork(neighbor)', &
                                i, j, nodp, neighbor, iwork(neighbor)

            if ( iwork(neighbor)==0 ) then
               kmeshe1new(nodp) = kmeshe1new(nodp)+1  ! new neighbor
               filled(nodp) = numfilled+1
               kmshe2tmp(ih+numfilled+1) = neighbor
            end if  ! ( iwork(neighbor)==0 )

            iwork(kmshe2(kmshe1(nodp)+1:kmshe1(nodp+1))) = 0 ! reset
            iwork(kmshe2tmp(ih+1:ih+numfilled)) = 0

         end do  ! j = 1, 2

      end do  ! i = 1, numpairs

!     --- Accumulate kmeshe1new

      icount = 0
      do i = 1, npoint

         isave = kmeshe1new(i)
         kmeshe1new(i) = icount
         icount = icount+isave

      end do  ! i = 1, npoint
      kmeshe1new(npoint+1) = icount

!     --- Fill istart(1:npoint) by copying from kmeshe1new

      istart(1:npoint) = kmeshe1new(1:npoint)

!     --- Copy kmeshe2 to kmeshe2new and raise istart

      isave = 0
      do i = 1, npoint
         nneigh = kmshe1(i+1)-isave
         do j = 1, nneigh
            istart(i) = istart(i)+ 1
            kmeshe2new(istart(i)) = kmshe2(isave+j)
         end do  ! j = 1, nneigh
         isave = kmshe1(i+1)
      end do  ! i = 1, npoint

!     --- For all nodes in kprobper do
!         set iwork to 1 for each neighbor of node
!         if ( iwork(neighbor)==0 ) then
!            fill neighbor in kmeshe2new and raise istart(node)
!         clear iwork

      filled = 0
      do i = 1, numpairs

         do j = 1, 2  ! both nodes in pair

            nodp = kprobper(j,i)  ! node in pair
            k = mod(j,2)+1
            neighbor = kprobper(k,i) ! neighbor in pair

            iwork(kmshe2(kmshe1(nodp)+1:kmshe1(nodp+1))) = 1 ! all neighbors
            numfilled = filled(nodp)
            ih = kmshe1tmp(nodp)
            if ( debug ) write(irefwr,1) 'ih, numfilled', ih, numfilled

            if ( numfilled>0 ) iwork(kmshe2tmp(ih+1:ih+numfilled)) = 1

            if ( debug ) &
               write(irefwr,1) 'i, j, nodp, neighbor, iwork(neighbor)', &
                                i, j, nodp, neighbor, iwork(neighbor)

            if ( iwork(neighbor)==0 ) then
               istart(nodp) = istart(nodp)+1  ! new neighbor
               kmeshe2new(istart(nodp)) = neighbor
               filled(nodp) = numfilled+1
            end if  ! ( iwork(neighbor)==0 ) then

            iwork(kmshe2(kmshe1(nodp)+1:kmshe1(nodp+1))) = 0 ! reset
            iwork(kmshe2tmp(ih+1:ih+numfilled)) = 0

         end do  ! j = 1, 2

      end do  ! i = 1, numpairs

      deallocate ( iwork, istart, kmshe1tmp, kmshe2tmp, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write ( irefwr, '(/a/)') &
               ' nodal point connected to nodal points'
         call pr0012 ( npoint, kmeshe1new, kmeshe2new )
         write(irefwr,*) 'End incmkmeshe'

      end if  ! ( debug )
      call erclos ( 'incmkmeshe' )

      end
