      subroutine incommatcol ( intmatrow1, intmatrow2, nrow, &
                               intmatcol1, intmatcol2, ncol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-02-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make a columnwise copy of the matrix structure, &
!     where the present orientation is row-wise
!     The matrix does not have be square
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nrow, intmatrow1(nrow+1), intmatrow2(*), &
              ncol, intmatcol1(ncol+1), intmatcol2(*)

!     intmatcol1     o    First part containing pointers of matrix storage of
!                         rectangular matrix. Contains number of non-zero
!                         elements per row accumulated (column-wise storage)
!     intmatcol2     o    Second part of matrix storage of rectangular matrix.
!                         Contains column numbers (column-wise storage)
!     intmatrow1     i    First part containing pointers of matrix storage of
!                         rectangular matrix. Contains number of non-zero
!                         elements per row accumulated (row-wise storage)
!     intmatrow2     i    Second part of matrix storage of rectangular matrix.
!                         Contains column numbers (row-wise storage)
!     ncol           i    Number of columns in rectangular matrix
!     nrow           i    Number of rows in rectangular matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer irow, icol, j, isum, jsum, ipos

!     icol           column number
!     ipos           Position in array
!     irow           Row number
!     isum           Integer variable to compute a sum
!     j              Counting variable
!     jsum           Integer variable to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR0004         Print INTMAT corresponding to internal matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array intmatcol1
!     Store number of non-zero rows corresponding to each column in array
!     Accumulate intmatcol1, make first position 0
!     Fill array intmatcol2, while updating intmatcol1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incommatcol' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incommatcol'
         call pr0004 ( 5, nrow, intmatrow1, intmatrow2 )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear array intmatcol1

      intmatcol1 = 0

!     --- Store number of non-zero rows corresponding to each column in array

      do irow = 1, nrow

         do j = intmatrow1(irow)+1, intmatrow1(irow+1)

            icol = intmatrow2(j)
            intmatcol1(icol) = intmatcol1(icol)+1

         end do  ! j = intmatrow1(i)+1, intmatrow1(i+1)

      end do  ! irow = 1, nrow

!     --- Accumulate intmatcol1, make first position 0

      isum = 0
      do icol = 1, ncol
         jsum = isum
         isum = isum + intmatcol1(icol)
         intmatcol1(icol) = jsum
      end do  ! icol = 1, ncol-1

!     --- Fill array intmatcol2, while updating intmatcol1

      do irow = 1, nrow

         do j = intmatrow1(irow)+1, intmatrow1(irow+1)

            icol = intmatrow2(j)
            ipos = intmatcol1(icol)+1
            intmatcol2(ipos) = irow
            intmatcol1(icol) = ipos

         end do  ! j = intmatrow1(i)+1, intmatrow1(i+1)

      end do  ! irow = 1, nrow

!     --- Reorder intmatcol1 back

      do icol = ncol, 1, -1
         intmatcol1(icol+1) = intmatcol1(icol)
      end do  ! icol = ncol, 1, -1
      intmatcol1(1) = 0

1000  if ( debug ) then

!     --- Debug information

         call pr0004 ( 5, ncol, intmatcol1, intmatcol2 )
         write(irefwr,*) 'End incommatcol'

      end if  ! ( debug )
      call erclos ( 'incommatcol' )

      end subroutine incommatcol
