      subroutine init01 ( ishape, ndim, istore, istfl1, inpelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.13 date 26-05-2014 Extension for shape 22
!        version  1.12 date 05-01-2003 Extension for prisms
!        version  1.11 date 18-12-2002 Check on ishape
!
!   copyright (c) 1992-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Fill array ISTORE using the information in ISTFL1 for the special
!   situation of ISHAPE=-1 in file 1
! **********************************************************************
!
!                       KEYWORDS
!
!     file_1
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ishape, ndim, inpelm, istore(inpelm), istfl1(10)

!     inpelm  i   Number of points in the element
!     ishape  i   Shape number of the element as generated by the mesh generator
!     istfl1  i   Contains the information about the storage of the unknowns
!                 as stored on file 1
!                 ISTFL1 is filled as follows:
!                 Pos. 1:  number of unknowns in the vertices
!                      2:  number of unknowns in mid side points
!                      3:  number of unknowns in the centroid
!                      4:  number of unknowns in the centroids of the faces (3D)
!     istore  o   The number of unknowns in each point of the element is
!                 stored in this array sequentially
!     ndim    i   Dimension of the space
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inunkp

!    i         general loop variable
!    inunkp    number of unknwons in a specific point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer init08

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIT08         Compute nunkp as function of inunkp and ndim for
!                    inunkp<-1000
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!   The following table shows the relative nodal point numbers for vertices, &
!   midside points etc. in relation with the shape of the elements
!   shape nr. |       vertices     |     midsides    | centroids|  centroids of
!             |                    |                 |          |  faces
!------------------------------------------------------------------------------ &
!     1       |       all          |        -        |    -     |     - &
!     2       |      1, 3          |        2        |    -     |     - &
!     3       |       all          |        -        |    -     |     - &
!     4       |     1, 3, 5        |      2, 4, 6    |    -     |     - &
!     5       |       all          |        -        |    -     |     - &
!     6       |     1, 3, 5, 7     |    2, 4, 6, 8   |    9     |     - &
!     7       |     1, 3, 5        |      2, 4, 6    |    7     |     - &
!     8       |     1, 4, 7        | 2, 3, 5, 6, 8, 9|   10     |     - &
!     9       |     1, 2, 3, 4     |        -        |    5     |     - &
!    10       |       -            |        -        |    -     |     - &
!    11       |      all           |        -        |    -     |     - &
!    12       |     1, 3, 5, 10    | 2, 4, 6, 7, 8, 9|    -     |     - &
!    13       |      all           |        -        |    -     |     - &
!    14       | 1,3,7,9,19,21,25,27|2,4,6,8,10,12,16,|   14     | 5, 11, 13, &
!             |                    |18, 20, 22,24,26 |          | 15, 17, 23
!    15       |     1, 3, 5, 10    | 2, 4, 6, 7, 8, 9|    -     |11,12,13,14
!    16       |     1, 3, 5, 10    | 2, 4, 6, 7, 8, 9|   15     |11,12,13,14
!    17       |       1-8          |         -       |    9     |      - &
!    18       |       1-4          |         -       |    5     |      - &
!    21       |       all          |         -       |    -     |      - &
!------------------------------------------------------------------------------ &
!    Clear store
!    For i := 1 (1) 4 do
!       if ( istfl1(i) # 0 ) then
!          Fill unknowns for each point cooresponding to the table
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'init01' )

!     --- Clear store

      do i = 1, inpelm
         istore(i) = 0
      end do

!     --- Fill istore for all nodes

      if ( istfl1(1)/=0 ) then

!     --- vertices

         inunkp = init08 ( istfl1(1), ndim, ishape )
         if ( ishape==1 .or. ishape==3 .or. ishape==5 .or. &
              ishape==11 .or. ishape==13 .or. ishape==21 ) then

!        --- Shape numbers 1, 3, 5, 11 and 13: fill all points

            do i = 1, inpelm
               istore(i) = inunkp
            end do

         else if ( ishape==2 .or. ishape==4 .or. ishape==6 .or. &
                   ishape==7 ) then

!        --- Shape numbers 2, 4, 6 and 7: fill odd points

            do i = 1, max(3,inpelm-1), 2
               istore(i) = inunkp
            end do

         else if ( ishape==8 ) then

!        --- Shape number 8, special

            do i = 1, 7, 3
               istore(i) = inunkp
            end do

         else if ( ishape==9 .or. ishape>=17 ) then

!        --- Shape number 9, fill vertices

            do i = 1, inpelm-1
               istore(i) = inunkp
            end do

         else if ( ishape==10 ) then

!        --- Shape number 10, fill vertices

            do i = 1, 3
               istore(i) = inunkp
            end do

         else if ( ishape==12 .or. ishape==15 .or. ishape==16 ) then

!        --- Shape numbers 12, 15 and 16 special

            istore(1) = inunkp
            istore(3) = inunkp
            istore(5) = inunkp
            istore(10) = inunkp

         else if ( ishape==14 ) then

!        --- Shape number 14, special

            istore(1) = inunkp
            istore(3) = inunkp
            istore(7) = inunkp
            istore(9) = inunkp
            istore(19) = inunkp
            istore(21) = inunkp
            istore(25) = inunkp
            istore(27) = inunkp

         else if ( ishape==22 ) then

!        --- Shape number 22, special

            istore(1) = inunkp
            istore(3) = inunkp
            istore(5) = inunkp
            istore(13) = inunkp
            istore(15) = inunkp
            istore(17) = inunkp

         else if ( ishape>0 ) then

!        --- Shape number not implemented

            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )
            go to 1000

         end if

      end if
      if ( istfl1(2)/=0 ) then

!     --- mid side points

         inunkp = init08 ( istfl1(2), ndim, ishape )
         if ( ishape==2 .or. ishape==4 .or. ishape==6 .or. ishape==7 ) then

!        --- Shape numbers 2, 4, 6 and 7: fill even points

            do i = 2, inpelm, 2
               istore(i) = inunkp
            end do
         else if ( ishape==8 ) then

!        --- Shape number 8, special

            istore(2) = inunkp
            istore(3) = inunkp
            istore(5) = inunkp
            istore(6) = inunkp
            istore(8) = inunkp
            istore(9) = inunkp
         else if ( ishape==12 .or. ishape==15 .or. ishape==16 ) then

!        --- Shape numbers 12, 15 and 16 special

            istore(2) = inunkp
            istore(4) = inunkp
            istore(6) = inunkp
            istore(7) = inunkp
            istore(8) = inunkp
            istore(9) = inunkp

         else if ( ishape==14 ) then

!        --- Shape number 14, special

            do i = 2, 12, 2
               istore(i) = inunkp
               istore(i+14) = inunkp
            end do

         else if ( ishape==22 ) then

!        --- Shape number 22, special

            do i = 2, 6, 2
               istore(i) = inunkp
               istore(i+5) = inunkp
               istore(i+12) = inunkp
            end do

         else if ( ishape>18 .and. ishape/=21 ) then

!        --- Shape number not implemented

            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )
            go to 1000

         end if

      end if
      if ( istfl1(3)/=0 ) then

!     --- centroid

         inunkp = init08 ( istfl1(3), ndim, ishape )
         if ( ishape>=6 .and. ishape<=10 .or. ishape>=17 .and. &
              ishape<=18 ) then

!        --- Shape numbers 6, 7, 8, 9, 10, 17 and 18:

            istore(inpelm) = inunkp

         else if ( ishape==16 ) then

!        --- Shape number 16 special

            istore(15) = inunkp

         else if ( ishape==14 ) then

!        --- Shape number 14, special

            istore(14) = inunkp

         else if ( ishape>18 .and. ishape<21 .or. ishape>22 ) then

!        --- Shape number not implemented

            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )
            go to 1000

         end if

      end if
      if ( (ishape>13 .and. ishape<=16 .or. ishape==22) .and. &
           istfl1(4)/=0 ) then

!     --- centroids of faces

         inunkp = init08 ( istfl1(4), ndim, ishape )
         if ( ishape==14 ) then

!        --- Shape number 14, special

            istore(5) = inunkp
            do i = 11, 17, 2
               istore(i) = inunkp
            end do
            istore(23) = inunkp

         else if ( ishape==15 .or. ishape==16 ) then

!        --- Shape number 15, special

            do i = 11, 14
               istore(i) = inunkp
            end do

         else

!        --- Shape number 22, special

            do i = 8, 12, 2
               istore(i) = inunkp
            end do

         end if

      end if

1000  call erclos ( 'init01' )

      end
