      subroutine prmarkoutlevsp ( imark, ielemobst, iobst, nelem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-11-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark array imark for all elements that are completely outside
!     the obstacle
!     It is supposed that imark has been initialized before
! **********************************************************************
!
!                       KEYWORDS
!
!     obstacle
!     element
!     mark
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer imark(*), ielemobst(*), iobst, nelem

!     ielemobst      i    Array containing information about the elements in an
!                         obstacle
!                         The first numobst positions contain the starting
!                         positions of each obstacle sequentially
!                         Per obstacle the first three positions contain
!                         1. The number of elements that are completely inside
!                            the obstacle
!                         2. The number of elements that are not completely
!                            inside the element but with all vertices inside
!                            the element
!                         3. The number of elements that are partly inside
!                            the obstacle
!                            elements that contain only boundary points of the
!                            obstacle are not considered
!                         These positions are followed by the element numbers
!                         in the sequence defined by the first three positions
!     imark         i/o   Marker array of length nelem
!                         imark must have been initialized by 0
!                         imark(i) = 0: element i inside obstacle
!                         imark(i) = 1: element i not inside obstacle
!     iobst          i    Obstacle sequence number
!     nelem          i    Number of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, numinelmobst, i
      integer, allocatable :: iwork(:)

!     error          Return value of allocate or deallocate
!     i              Counting variable
!     istart         position in ielemobst
!     iwork          integer work array
!                    iwork(i) = 0: element is outside obstacle
!                    iwork(i) = 1: element is (partly) inside obstacle
!     numinelmobst   Number of elements that are within the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmarkoutlevsp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmarkoutlevsp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, nelem, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- clear iwork

      iwork = 0

!     --- Set iwork = 1 for all elements (partly) in the obstacle

      istart = ielemobst(iobst)
      numinelmobst = ielemobst(istart)+ielemobst(istart+1)+ &
                     ielemobst(istart+2)
      istart = istart+2
      do i = 1, numinelmobst
         iwork(ielemobst(istart+i)) = 1
      end do

!     --- Set imark = 1 for all elements outside obstacle

      do i = 1, nelem
         if ( iwork(i)==0 ) imark(i) = 1
      end do  ! i = 1, nelem

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmarkoutlevsp'

      end if  ! ( debug )
      call erclos ( 'prmarkoutlevsp' )

      end
