      subroutine prob04sfold ( iinput, iinp, nkbnd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the number of nodal points with prescribed boundary
!     conditions for surface isrf1 to isrf2
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, nkbnd

!     iinp          i/o   last position used in array iinput
!     iinput         i    integer input array
!                         See subroutine prord3
!     nkbnd         i/o   Number of points with essential boundary conditions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, iskipbn, isrf1, &
              isrf2, istart, j, jinp, jnelem, jnpelm, ncurvsact, ivers
      integer, allocatable :: isurfaces(:), iwork(:), icurvind(:)

!     icurvind       Integer work array of length ncurvs that is filled as
!                    follows
!                    icurvind(i) = -1, at the boundary of the surfaces
!                    icurvind(i) = 1, or not in the surfaces icurvind(i)=0
!     inpelm         number of points per element
!     iskipbn        Defines if the outer boundary must be skipped
!                    Possible values:
!                    0:  the outer boundary must not be skipped
!                    1:  the complete outer boundary must be skipped
!                    2:  the inner boundary must be skipped as well as some
!                        explicitly given outer curves
!                    3:  the outer boundary must be skipped except some
!                        explicitly given outer curves
!     isrf1          First surface number
!     isrf2          Last surface number
!     istart         First position used at a part
!     isurfaces      Array containing the surfaces isrf1 to isrf2
!     ivers          Version sequence number
!     iwork          integer work array of length npoint
!                    See subroutine prob04sfels for a description
!     j              Counting variable
!     jinp           last position used in array iinput with respect to
!                    iprec
!     jnelem         number of elements in an element group for one super
!                    element
!     jnpelm         Number of nodes in an element
!     ncurvsact      Actual number of curves to be used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PROB04SFELS    Compute the number of nodal points with prescribed boundary
!                    conditions for a set of nodes corresponding to surfaces
!     PROB09S1       Mark all points on external curves with a -1 in iwork
!     PRTS01         Check isurf1, isurf2
!     SEPEXTKMESHT   Compute starting and ending address of elements in kmeshc
!     SEPOUTCR       Find all curves surrounding a set of surfaces
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     272   number of nodal points in elements of the same group for the
!           generation of essential boundary conditions, not the same.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob04sfold' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob04sfold'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( isurfaces(nsurfs), iwork(npoint), icurvind(ncurvs), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, nsurfs+npoint+ncurvs, 'isurfaces' )
      if ( ierror/=0 ) go to 1000

      ivers  = iinput(15)
      isrf1 = iinput(iinp+2)
      isrf2 = iinput(iinp+3)
      iinp = iinp+3
      if ( isrf2==0 ) isrf2 = isrf1

      call prts01 ( isrf1, isrf2, nsurfs )

!     --- compute inpelm

      if ( kelmn>0 ) then

!     --- kmeshn>0, use kmesh part n

         inpelm = kmeshn(isrf1)
         if ( isrf2>isrf1 ) then

!        --- isrf2>isrf1  check inpelm

            do j = isrf1, isrf2
               if ( inpelm/=kmeshn(j) ) then
                  call errint ( isrf1, 1 )
                  call errint ( isrf2, 2 )
                  call errint ( isrf1, 3 )
                  call errint ( inpelm, 4 )
                  call errint ( j, 5 )
                  call errint ( kmeshn(j), 6 )
                  call errsub ( 272, 6, 0, 0 )
               end if
            end do

         end if

!        --- compute the number of elements  inelem

         inelem = 0
         do j = isrf1, isrf2
            inelem = inelem+kmeshn(nsurfs+j)
         end do

!        --- compute starting position in kmeshn

         istart = 2*nsurfs
         do j = 1, isrf1-1
            jnpelm = kmeshn(j)
            jnelem = kmeshn(nsurfs+j)
            istart = istart+jnpelm*jnelem
         end do

      else

!     --- kmeshn = 0, use kmesh part t and kmeshc
!         compute inpelm and inelem

         call sepextkmesht ( isrf1, isrf2, istart, inpelm )
         istart = istart-1

      end if

!     --- clear array iwork

      iwork = 0

!     --- compute iskipbn

      jinp = iinp+inpelm
      if ( ivers>=6 ) then

!     --- version number>5, get iskipbn from iinput

         jinp = jinp+2
         iskipbn = iinput(jinp-1)
         ncurvsact = iinput(jinp)

      else

!     --- version number<6, no iskipbn

         iskipbn = 0
         ncurvsact = 0

      end if

      if ( iskipbn>=1 ) then

!     --- Boundary must be skipped, &
!         first find curves which correspond to boundary of surfaces

         do j = 1, isrf2-isrf1+1
            isurfaces(j) = isrf1+j-1
         end do  ! j = 1, isrf2-isrf1+1
         call sepoutcr ( icurvind, isurfaces, isrf2-isrf1+1 )

!        --- Next mark all points on the outer curves by -1

         call prob09s1 ( icurvind, iwork, iskipbn, &
                         ncurvsact, iinput(jinp+1) )

      end if

!     --- Compute nkbnd for all nodes in surfaces

      call prob04sfels ( inpelm, istart, iinput, iinp, iwork, nkbnd )

      iinp = jinp+ncurvsact

      deallocate ( isurfaces, iwork, icurvind, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'isurfaces' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob04sfold'

      end if  ! ( debug )
      call erclos ( 'prob04sfold' )

      end
