      subroutine prob04vlmold ( iinput, iinp, nkbnd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the number of nodal points with prescribed boundary
!     conditions for volume ivlm1 to ivlm2
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, nkbnd

!     iinp          i/o   last position used in array iinput
!     iinput         i    integer input array
!                         See subroutine prord3
!     nkbnd         i/o   Number of points with essential boundary conditions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, iskipbn, ivlm1, &
              ivlm2, istart, jinp, nsurfsact, ivers
      integer, allocatable :: iwork(:)

!     inpelm         number of points per element
!     iskipbn        Defines if the outer boundary must be skipped
!                    Possible values:
!                    0:  the outer boundary must not be skipped
!                    1:  the complete outer boundary must be skipped
!                    2:  the inner boundary must be skipped as well as some
!                        explicitly given outer curves
!                    3:  the outer boundary must be skipped except some
!                        explicitly given outer curves
!     istart         First position used at a part
!     ivers          Version sequence number
!     ivlm1          First volume number
!     ivlm2          Last volume number
!     iwork          integer work array of length npoint
!                    See subroutine prob04vlmels for a description
!     jinp           last position used in array iinput with respect to
!                    iprec
!     nsurfsact      Actual number of curves to be used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PROB04VLMELS   Compute the number of nodal points with prescribed boundary
!                    conditions for a set of nodes corresponding to volumes
!     PRTS01         Check isurf1, isurf2
!     SEPEXTKMESHT   Compute starting and ending address of elements in kmeshc
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2281   the option skip surfaces has not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob04vlmold' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob04vlmold'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, nvolms+npoint, 'ivolumes' )
      if ( ierror/=0 ) go to 1000

      ivers  = iinput(15)
      ivlm1 = iinput(iinp+2)
      ivlm2 = iinput(iinp+3)
      iinp = iinp+3
      if ( ivlm2==0 ) ivlm2 = ivlm1

      call prts01 ( ivlm1, ivlm2, nvolms )

!     --- compute inpelm and inelem

      call sepextkmesht ( ivlm1, ivlm2, istart, inpelm )
      istart = istart-1

!     --- clear array iwork

      iwork = 0

!     --- compute iskipbn

      jinp = iinp+inpelm
      if ( ivers>=6 ) then

!     --- version number>5, get iskipbn from iinput

         jinp = jinp+2
         iskipbn = iinput(jinp-1)
         nsurfsact = iinput(jinp)

      else

!     --- version number<6, no iskipbn

         iskipbn = 0
         nsurfsact = 0

      end if

      if ( iskipbn>=1 .or. nsurfsact>0 ) then

!     --- iskipbn>0, or ncurves>0: noy yet implemented

         call errsub ( 2281, 0, 0, 0 )
         go to 1000

      end if  ! ( iskipbn>=1 .or. nsurfsact>0 )

!     --- Compute nkbnd for all nodes in volumes

      call prob04vlmels ( inelem, inpelm, kmeshc, istart, &
                          iinput, iinp, iwork, nkbnd )

      iinp = jinp+nsurfsact

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ivolumes' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob04vlmold'

      end if  ! ( debug )
      call erclos ( 'prob04vlmold' )

      end
