      subroutine prob09surf ( iinp, nlocsf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute npltra and store in kprob with respect to surfaces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     local_transform
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp, nlocsf

!     iinp          i/o   last position used in array iinputkp
!     nlocsf         i    Number of surfaces with local transforms
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iskipbn, istart, ielem, &
              jdgfd1, jdgfd2, isrf1, isrf2, isurnr, itang, jdgfd3, &
              k, nodp, j, jtrans, inpelm, inunkp, jinp, &
              ncurvsact, numcurvs
      integer, allocatable :: isurfaces(:), iwork(:), icurvind(:)

!     i              Counting variable
!     icurvind       Integer work array of length ncurvs that is filled as
!                    follows
!                    icurvind(i) = -1, internal curve
!                    icurvind(i) = 1, boundary curve
!                    icurvind(i) = 0, not in the surfaces
!     ielem          Element sequence number
!     inpelm         Number of nodes per element
!     inunkp         Number of degrees of freedom in first point
!     iskipbn        If 1 the outer boundary of the surfaces must be skipped
!     isrf1          First surface number
!     isrf2          Last surface number
!     istart         First position used at a part
!     isurfaces      Array containing the surfaces from which information
!                    must be found
!     isurnr         Surface sequence number
!     itang          Defines how the first tangential vector must be defined
!                    0: arbitrary
!                    1: By a line through points p1 and p2
!     iwork          integer work array of length npoint
!     j              Counting variable
!     jdgfd1         Degree of freedom
!     jdgfd2         Degree of freedom
!     jdgfd3         Degree of freedom
!     jinp           position used in array iinputkp
!     jtrans         Type of local transformation
!                    1:  standard transformation
!                    2:  symmetric transformation
!                    3:  non-symmetric transformation
!     k              Counting variable
!     ncurvsact      Actual number of curves to be used
!     nodp           Nodal point sequence number
!     numcurvs       Number of curves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRTS01         Check isurf1, isurf2
!     SEPOUTCR       Find all curves surrounding a set of surfaces
!     SEPPROB09S1    Mark all points on external curves with a -1 in iwork
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     335   degree of freedom has wrong value
!     337   at least two degrees of freedom have same value
!    1280   Number of nodes per element in different surfaces not the same
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob09surf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob09surf'
  1      format ( a, 1x, (10i6) )
         write(irefwr,1) 'iinp', iinp
         call prinin ( iinputkp, iinp+30, 'iinputkp' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      maxtrn = max(9,maxtrn)
      if ( debug ) &
         write(irefwr,1) 'nunkp, nsurfs, maxtrn, nlocsf', &
                          nunkp, nsurfs, maxtrn, nlocsf

      allocate ( isurfaces(nsurfs), iwork(npoint), icurvind(ncurvs), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, nsurfs, 'isurfaces' )
      if ( ierror/=0 ) go to 1000

      do i = 1, nlocsf

         if ( debug )write(irefwr,1) 'iinp', iinp
         jdgfd1 = iinputkp(iinp+1)
         jdgfd2 = iinputkp(iinp+2)
         jdgfd3 = iinputkp(iinp+3)
         isrf1 = iinputkp(iinp+4)
         isrf2 = iinputkp(iinp+5)
         if ( debug ) &
            write(irefwr,1) 'i, jdgfd1, jdgfd2, jdgfd3, isrf1, isrf2', &
                             i, jdgfd1, jdgfd2, jdgfd3, isrf1, isrf2
         if ( jdgfd1==0 ) jdgfd1 = 1
         if ( jdgfd2==0 ) jdgfd2 = 2
         if ( jdgfd3==0 ) jdgfd3 = 3
         iinputkp(iinp+1) = jdgfd1
         iinputkp(iinp+2) = jdgfd2
         iinputkp(iinp+3) = jdgfd3
         iinp = iinp+5
         if ( debug ) write(irefwr,1) 'iinp', iinp
         if ( jdgfd1<0 .or. jdgfd1>nunkp ) then
            call errint ( 1, 1 )
            call errint ( jdgfd1, 2 )
            call errint ( nunkp, 3 )
            call errint ( isrf1, 4 )
            call errsub ( 335, 4, 0, 0 )
         end if
         if ( jdgfd2<0 .or. jdgfd2>nunkp ) then
            call errint ( 2, 1 )
            call errint ( jdgfd2, 2 )
            call errint ( nunkp, 3 )
            call errint ( isrf1, 4 )
            call errsub ( 335, 4, 0, 0 )
         end if
         if ( jdgfd3<0 .or. jdgfd3>nunkp ) then
            call errint ( 3, 1 )
            call errint ( jdgfd3, 2 )
            call errint ( nunkp, 3 )
            call errint ( isrf1, 4 )
            call errsub ( 335, 4, 0, 0 )
         end if
         if ( jdgfd1==jdgfd2 .or. jdgfd1==jdgfd3 .or. &
              jdgfd2==jdgfd3 ) then
            call errint ( isrf1, 1 )
            call errsub ( 337, 1, 0, 0 )
         end if
         if ( isrf2==0 ) isrf2 = isrf1
         call prts01 ( isrf1, isrf2, nsurfs )
         iinputkp(iinp) = isrf2
         jtrans = iinputkp(iinp+1)
         iskipbn = mod(iinputkp(iinp+2),10)
         itang = iinputkp(iinp+3)
         inpelm = kmeshn(isrf1)
         if ( itang==10 ) numcurvs = iinputkp(iinp+4+inpelm) ! number of curves
         iinp = iinp+3

         if ( debug ) then
            write(irefwr,1) 'iinp', iinp
            write(irefwr,1) 'jtrans, iskipbn, itang, inpelm', &
                             jtrans, iskipbn, itang, inpelm
            write(irefwr,1) 'numcurvs', numcurvs
         end if  ! ( debug )

         do isurnr = isrf1+1, isrf2
            if ( inpelm/=kmeshn(isurnr) ) then

!           --- Error 1280:number of nodes in various surfaces not the same

               call errint ( isurnr, 1 )
               call errint ( kmeshn(isurnr), 2 )
               call errint ( isrf1, 3 )
               call errsub ( 1280, 3, 0, 0 )

            end if

         end do

!        --- Find all nodes in surfaces with local transformations

         iwork = 0

         ncurvsact = 0
         if ( iskipbn>=1 ) then

!        --- Boundary must be skipped, &
!            first find curves which correspond to boundary of surfaces

            do j = 1, isrf2-isrf1+1
               isurfaces(j) = isrf1+j-1
            end do  ! j = 1, isrf2-isrf1+1
            call sepoutcr ( icurvind, isurfaces, isrf2-isrf1+1 )

            if ( debug ) then
               call prinin ( isurfaces, isrf2-isrf1+1, 'surfaces' )
               call prinin ( icurvind, ncurvs, 'icurvind' )
            end if  ! ( debug )

!           --- Next mark all points on the outer curves by -1

            if ( iskipbn>1 ) then

!           --- iskipbn>1, extra curves required

               jinp = iinp+inpelm
               if ( jtrans==2 ) then
                  jinp = jinp+9
                  maxtrn = max(maxtrn,9)
               else if ( jtrans==3 ) then
                  jinp = jinp+18
                  maxtrn = max(maxtrn,18)
               end if
               if ( itang>0 .and. itang<10 ) then

!              --- 1<=itang<=10, 2 extra positions

                  jinp = jinp+2

               else if ( itang>9 ) then

!              --- itang>=10, 1+numcurvs extra positions

                  jinp = jinp+1+numcurvs

               end if  ! ( itang>0 .and. itang<10 )

               ncurvsact = iinputkp(jinp+1)
               jinp = jinp+2

            else

!           --- iskipbn = 1, no extra curves required

               jinp = iinp

            end if  ! ( iskipbn>1 )
            if ( debug ) write(irefwr,1) 'jinp', jinp

            call sepprob09s1 ( icurvind, iwork, iskipbn, &
                                ncurvsact, iinputkp(jinp) )

         end if

!        --- Find all points in surfaces with iwork(i) = 0 and raise npltra

         istart = 2*nsurfs
         do k = 1, isrf1-1
            istart = istart+kmeshn(k)*kmeshn(nsurfs+k)
         end do

!        --- Loop over surfaces

         do isurnr = isrf1, isrf2

            inelem = kmeshn(nsurfs+isurnr)

            if ( debug ) then
               write(irefwr,1) 'isurnr, inelem, inpelm', &
                                isurnr, inelem, inpelm
               call prinin ( iinputkp(iinp+1), inpelm, 'iinputkp' )
            end if  ! ( debug )

!           --- Loop over elements in surface

            do ielem = 1, inelem

!           --- Loop over nodes in element

               do k = 1, inpelm

                  if ( iinputkp(iinp+k)==1 ) then

!                 --- Node must be taken into account

                     nodp = kmeshn(istart+k)
                     if ( nodp>0 .and. iwork(nodp)==0 ) then
                        if ( indprf>0 ) then

!                       --- indprf>0  kprob part f contains the number
!                           of unknowns

                           inunkp = kprobf(nodp+1)-kprobf(nodp)
                           if ( jdgfd1>inunkp .or. jdgfd2>inunkp &
                                .or. jdgfd3>inunkp ) goto 200

                        end if
                        npltra = npltra+1
200                     iwork(nodp) = 1

                     end if

                  end if

               end do
               istart = istart+inpelm

            end do

         end do
         iinp = iinp+inpelm
         if ( jtrans==2 ) then
            iinp = iinp+9
            maxtrn = max(maxtrn,9)
         else if ( jtrans==3 ) then
            iinp = iinp+18
            maxtrn = max(maxtrn,18)
         end if
         if ( itang>0 .and. itang<10 ) then

!        --- 1<=itang<=9, 2 extra positions

            iinp = iinp+2

         else if ( itang>9 ) then

!        --- itang>=10, numcurvs+1 extra positions

            iinp = iinp+1+numcurvs

         end if  ! ( itang>0 .and. itang<10 )

         if ( ncurvsact>0 ) iinp = iinp+1+ncurvsact
         if ( debug ) then
            write(irefwr,1) 'iinp, jtrans, itang, ncurvsact', &
                             iinp, jtrans, itang, ncurvsact
         end if  ! ( debug )

      end do

      deallocate ( isurfaces, iwork, icurvind, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'isurfaces' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'npltra', npltra
         write(irefwr,*) 'End prob09surf'

      end if  ! ( debug )
      call erclos ( 'prob09surf' )

      end
