      subroutine prob10curv ( iinp, nloccv, mappinglc, iwork2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill kprob part n with degrees of freedom with respect to curves
!     and transformation factors (coortr)
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     local_transform
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulecpack
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nloccv, iinp, mappinglc(*), iwork2(*)

!     iinp          i/o   last position used in array iinputkp
!     iwork2        i/o   integer work array of length npoint
!                         iwork2 is used to mark nodes that have been
!                         transformed already
!                         iwork2(i) contains the number of times a node
!                         is transformed
!     mappinglc        i    contains global node numbers for mesh
!     nloccv         i    Number of curves with local transforms
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, jdgfd1, jdgfd2, jstep, iexclu, ifirst, ilast, iinsign, &
              jtrans, kstep, n, k, kstart, kend, jdgfd3
      integer, allocatable :: iwork(:), iwork1(:)
      double precision normallc(3)
      logical planefound

!     i              Counting variable
!     iexclu         Indicates which points are excluded
!     ifirst         First curve number
!     iinsign        Sign of curve number
!     ilast          Last curve number
!     iwork          integer work array of length npointglob
!     iwork1         integer work array of length npointglob
!     jdgfd1         Degree of freedom
!     jdgfd2         Degree of freedom
!     jdgfd3         Degree of freedom
!     jstep          Step in loop
!     jtrans         Type of local transformation
!                    1:  standard transformation
!                    2:  symmetric transformation
!                    3:  non-symmetric transformation
!     k              Counting variable
!     kend           End value of parameter
!     kstart         Start value of parameter
!     kstep          Step in loop
!     n              number of nodes in curve
!     normallc       Contains normal to plane
!                    This array is only used in case of 3d curves
!     planefound     if true all nodes are in a plane
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     PROB10CV1      fill kprob part n for curves with degrees of freedom
!                    and transformation factors (coortr)
!     PROB10CV2D     fill kprob part n for curves with degrees of freedom
!                    and transformation factors (coortr)
!     PROB10CV3D     fill kprob part n for curves with degrees of freedom
!                    and transformation factors (coortr)
!                    Special case of a 3d curve lying in a plane
!     PROB10CVPLANE  Check if all nodes in curve are in a plane
!                    compute normal on this plane
!     SEPER0026      fill nodal points of curves icrv1 to icrv2 in array iwork
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob10curv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob10curv'
         write(irefwr,1) 'iinp', iinp
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) &
         write(irefwr,1) 'nunkp, maxtrn, nloccv', nunkp, maxtrn, nloccv

!     --- allocate work space

      allocate ( iwork(npointglob), iwork1(npointglob), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*npointglob, 'iwork' )
      if ( ierror/=0 ) go to 1000

      do i = 1, nloccv
         jdgfd1 = iinputkp(iinp+1)
         jdgfd2 = iinputkp(iinp+2)
         if ( ndim==3 ) jdgfd3 = iinputkp(iinp+3)
         jstep = iinputkp(iinp+ndim+1)

         if ( debug ) &
            write(irefwr,1) 'i, jdgfd1, jdgfd2, jstep', &
                             i, jdgfd1, jdgfd2, jstep
         if ( jstep>=0 ) then

!        --- jstep>=  0,  compute iexclu

            iexclu = jstep/100
            jstep = jstep-100*iexclu
         else
            iexclu = 0
         end if
         ifirst = iinputkp(iinp+ndim+2)
         ilast = iinputkp(iinp+ndim+3)
         if ( ifirst<0 ) then
            ifirst = -ifirst
            iinsign = -1
         else
            iinsign = 1
         end if
         iinp = iinp+ndim+4
         jtrans = iinputkp(iinp)
         kstep = abs(jstep)+1

         if ( debug ) &
            write(irefwr,1) 'ifirst, ilast, kstep', ifirst, ilast, kstep

         call seper0026 ( ifirst, ilast, kstep, n, iwork1, .true., kmeshmglob )
         if ( iinsign==-1 ) then

!        --- Reverse sequence in iwork1

            do k = 1, n
               iwork(k) = iwork1(n+1-k)
            end do
            do k = 1, n
               iwork1(k) = iwork(k)
            end do

         end if

         if ( debug ) call prinin ( iwork1, n, 'iwork1' )

         iwork = 0

         if ( jstep>=0 ) then

!        --- jstep>=0   end points of intervals
!            Correct for initial and end point in the case that exclude>0

            if ( iexclu==0 ) then

!           --- iexclu = 0  No excluding of nodal points

               kstart = 1
               kend = n

            else if ( iexclu==1 ) then

!           --- iexclu = 1 Exclude initial point

               kstart = 2
               kend = n

            else if ( iexclu==2 ) then

!           --- iexclu = 2 Exclude end point

               kstart = 1
               kend = n-1

            else if ( iexclu==3 ) then

!           --- iexclu = 3 Exclude initial and end point

               kstart = 2
               kend = n-1

            end if

            if ( debug ) write(irefwr,1) 'kstart, kend', kstart, kend

!           --- In case of 3d we check the special situation
!               of all nodes in a plane
!               This is only possible if there are at least 3 points
!               Also it is necessary that jtrans = 1 or 4

            planefound = ndim==3 .and. (kend-kstart)/kstep>2 .and. &
                         (jtrans==1 .or. jtrans==4 )

            if ( planefound ) then

!           --- Check if nodes in plane and find normal

               call  prob10cvplane ( iwork1, iwork, coorglob, planefound, &
                                     kstart, kend, kstep, normallc )

            end if  ! ( planefound )
            if ( debug ) then
               write(irefwr,*) 'planefound', planefound
               if ( planefound ) call prinrl ( normallc, 3, 'normallc' )
            end if  ! ( debug )

            if ( planefound ) then

!           --- 3d curve in a plane
!               The first normal is defined perpendicular to the curve
!               in the plane, the second (first tangent) perpendicular
!               to the plane, with the largest component positive
!               The third vector is tangent to the curve

               call prob10cv3d ( iwork, iwork1, iwork2, &
                                 kstart, kend, kstep, jdgfd1, jdgfd2, n, &
                                 jdgfd3, ifirst, ilast, normallc, mappinglc )

            else

!           --- 2d or not a plane found, standard case

               call prob10cv2d ( iwork, iwork1, iwork2, &
                                 kstart, kend, kstep, jdgfd1, jdgfd2, &
                                 jtrans, iinp, n, &
                                 jdgfd3, ifirst, ilast, mappinglc )

            end if  ! ( planefound )

         else

!        --- jstep <0      points between intervals

            call prob10cv1 ( n, kstep, iwork1, jdgfd1, &
                             jdgfd2, jdgfd3, iwork, iwork2, &
                             jtrans, ifirst, ilast, &
                             iinp, mappinglc )

         end if
         if ( jtrans==2 ) then
            iinp = iinp+ndim**2
         else if ( jtrans==3 ) then
            iinp = iinp+2*ndim**2
         end if

      end do

      deallocate ( iwork, iwork1, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         call prinin1 ( kprobn, npltra, ndim, 'kprobn' )
         call prinrl1 ( coortr, npltra, maxtrn, 'coortr' )
         write(irefwr,*) 'End prob10curv'

      end if  ! ( debug )
      call erclos ( 'prob10curv' )

      end subroutine prob10curv
