      subroutine prob10cv3d ( iwork, iwork1, iwork2, &
                              kstart, kend, kstep, jdgfd1, jdgfd2, n, &
                              jdgfd3, ifirst, ilast, normallc, mappinglc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 26-11-2007 Give warning in case of double transform
!        version  2.0  date 24-09-2007 Remove array curves
!        version  1.0  date 23-10-2006
!
!   copyright (c) 2006-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill kprob part n for curves with degrees of freedom
!     and transformation factors (coortr)
!     Special case of a 3d curve lying in a plane
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     local_transform
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), iwork1(*), iwork2(*), &
              kstart, kend, kstep, jdgfd1, jdgfd2, jdgfd3,  n, &
              ifirst, ilast, mappinglc(*)
      double precision normallc(3)

!     ifirst         i    First curve number
!     ilast          i    Last curve number
!     iwork         i/o    integer work array of length npoint
!     iwork1         i    integer work array length max(npoint, ncurvs)
!     iwork2        i/o   integer work array of length npoint
!                         iwork2 is used to mark nodes that have been
!                         transformed already
!                         iwork2(i) contains the number of times a node
!                         is transformed
!     jdgfd1         i    Degree of freedom
!     jdgfd2         i    Degree of freedom
!     jdgfd3         i    Degree of freedom
!     kend           i    End value of parameter
!     kstart         i    Start value of parameter
!     kstep          i    Step in loop
!     mappinglc        i    Array of size npointglobal
!                         Contains the local nodal point numbers corresponding
!                         to nodes in the global mesh
!                         If gobal node is not present in local mesh the
!                         corresponding value in mappinglc is 0
!     n              i    number of nodes in curve
!     normallc      i/o   Contains normal to plane
!                         This array is only used in case of curves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, nodp, inusol, inunkp, ideg, iprec, inext, i, nodpglob
      double precision amax, normal1(3), t1(3), component, h

!     amax           Maximum value in absolute value
!     component      Component of vector with maximum value
!     h              Length of vector
!     i              Counting variable
!     ideg           Degree of freedom
!     inext          Next point in curve
!     inunkp         Number of degrees of freedom in point
!     inusol         Position of unknown in array USOL
!     iprec          Previous point in curve
!     k              Counting variable
!     nodp           Nodal point sequence number (local mesh)
!     nodpglob       Nodal point sequence number (global mesh)
!     normal1        Second normal vector
!     t1             First tangential vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELOUTR         Compute outer product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     PRINRL         Print 1d real vector
!     PROB10TR01     Compute the components anx, any and anz of the tangential
!                    vector along a curve
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2909   Double transformation
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob10cv3d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob10cv3d'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check if largest component of normal is positive,
!         if not multiply by -1

      amax = 0d0
      do k = 1, 3
         if ( abs(normallc(k))>amax ) then
            amax = abs(normallc(k))         ! maximum (absolute value)
            component = normallc(k)         ! corresponding component
         end if  ! ( abs(normallc(k))>amax )
      end do  ! k = 1, 3

      if ( component<0d0 ) then

!     --- Largest component<0, multiply by -1

         do k = 1, 3
            normallc(k) = - normallc(k)
         end do  ! k = 1, 3

      end if  ! ( component<0d0 )
      if ( debug ) call prinrl( normallc, 3, 'normallc' )

      do k = kstart, kend, kstep

         if ( ipstkmeshpar>0 ) then

!        --- parallel case, use array mappinglc

            nodpglob = iwork1(k)
            nodp = mappinglc(nodpglob)
            if ( k==1 ) then
               iprec = nodpglob
            else
               iprec = iwork1(k-1)
            end if
            if ( k==n) then
               inext = nodpglob
            else
               inext = iwork1(k+1)
            end if

         else

!        --- serial case

            nodp = iwork1(k)
            nodpglob = nodp
            if ( k==1 ) then
               iprec = nodp
            else
               iprec = iwork1(k-1)
            end if
            if ( k==n) then
               inext = nodp
            else
               inext = iwork1(k+1)
            end if

         end if  ! ( ipstkmeshpar>0 )
         if ( debug ) &
            write(irefwr,1) 'k, nodp, nodpglob, iprec, inext', &
                             k, nodp, nodpglob, iprec, inext

         if ( nodp<=0 ) go to 200  ! skip non-positive nodes

         if ( iwork(nodp)==0 ) then

!        --- node not used in this part

            if ( iwork2(nodp)>0 ) then

!           --- Node has been tranformed before, give warning and skip

               call errint ( nodp, 1 )
               call errwar ( 2909, 1, 0, 0 )
               go to 200

            end if  ! ( iwork2(nodp)>0 )

            if ( indprf>0) then

!           --- indprf>0  kprob part f contains the number of unknowns

               inusol = kprobf(nodp)
               inunkp = kprobf(nodp+1)-inusol
               if ( jdgfd1>inunkp .or. jdgfd2>inunkp.or. &
                    jdgfd3>inunkp ) goto 200

            else

               inusol = (nodp-1)*nunkp

            end if  ! ( indprf>0)

            iwork(nodp) = 1
            iwork2(nodp) = iwork2(nodp)+1

            npltra = npltra+1
            ideg = inusol+jdgfd1
            if ( indprh/=0) ideg = kprobh(ideg)
            kprobn(1,npltra) = ideg
            ideg = inusol+jdgfd2
            if ( indprh/=0) ideg = kprobh(ideg)
            kprobn(2,npltra) = ideg
            ideg = inusol+jdgfd3
            if ( indprh/=0) ideg = kprobh(ideg)
            kprobn(3,npltra) = ideg

!           --- Compute nx and ny from tangential vector
!               Store in t1

            call prob10tr01 ( 3, coorglob, t1(1), t1(2), t1(3), iprec, &
                              inext, nodpglob, ifirst, &
                              ilast, h, k )
            do i = 1, 3
               t1(i) = t1(i)/h
            end do  ! i = 1, 3

!           --- Compute second normal vector

            call eloutr ( normallc, t1, normal1 )
            if ( debug ) then
               call prinrl ( t1, 3, 't1' )
               call prinrl ( normal1, 3, 'normal1' )
            end if  ! ( debug )

!           --- Fill array coortr

            do i = 1, 3
               coortr(3*i-2,npltra) = normal1(i)
               coortr(3*i-1,npltra) = normallc(i)
               coortr(3*i  ,npltra) = t1(i)
            end do

         end if  ! ( nodp>0 .and. iwork(nodp)==0 )

200   end do  ! k = kstart, kend, kstep

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob10cv3d'

      end if  ! ( debug )
      call erclos ( 'prob10cv3d' )

      end
