      subroutine probcurvelm ( icurnr, nelemb, &
                               kprobi, ikprbi, lenkmeshm3 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 14-03-2007 Correction for parallel
!        version  1.2  date 24-11-2004 Fortran 90
!        version  1.1  date 28-06-2004 Correction iw
!
!   copyright (c) 2002-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store line elements along a (compound) curve into array kprobi
!
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     boundary_condition
!     boundary
!     natural
!     line_element
!     compound_curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icurnr, nelemb, kprobi(*), ikprbi, lenkmeshm3

!     icurnr         i    Curve sequence number
!     ikprbi        i/o   last position used in array kprob part i
!     kprobi        i/o   KPROB part i, contains nodal point numbers of
!                         boundary elements
!     lenkmeshm3     i    Length of kmesh m part 3
!     nelemb         o    Number of elements along curve that are created
!                         in processor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i1, i2, nsubcv, ic, j1, j2, npoints, jcurnr, &
              firstnode, lastnode, prevlastnode, iwork(lenkmeshm3), &
              inpelm, istar3, iw, j, k, kcurnr, nelemlc
      logical present

!     firstnode      First node on curve
!     i1             First point of curve in KMESH part m
!     i2             Last point of curve in KMESH part m
!     ic             Relative subcurve number
!     inpelm         number of points per element
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     iw             Address
!     iwork          integer work array
!     j              Counting variable
!     j1             Lower index number along second direction of a face
!     j2             Upper index number along second direction of a face
!     jcurnr         curve sequence number of subcurve
!     k              Counting variable
!     kcurnr         |jcurnr|
!     lastnode       Last node on curve
!     nelemlc        Number of elements along curve
!     npoints        Number of points
!     nsubcv         Number of subcurves of a curve
!     present        If true the element is lying in the processor
!     prevlastnode   Last node on previous curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Extract parameters
!     Compute begin and end position of curve in kmeshm part 3
!     Check if curve is a compound curve
!     Fill elements
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probcurvelm' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probcurvelm'
  1      format ( a, 1x, (10i6) )

      end if

!     --- Extract parameters

      istar3 = kmeshm(2)-1

!     --- Compute begin and end position of curve in kmeshm part 3

      i1 = kmeshm(4+icurnr)
      i2 = kmeshm(5+icurnr)-1
      inpelm = ishape+1
      nsubcv = 0
      nelemb = 0
      if ( kelmq/=0 ) nsubcv = kmeshq(icurnr+1) - kmeshq(icurnr)
      if ( debug ) write(irefwr,1) 'inpelm, ishape', inpelm, ishape

!     --- Check if curve is a compound curve

      if ( nsubcv<=1 ) then

!     --- There are no compound curves
!         Fill nodes in iwork

         nelemlc = (i2-i1)/max(1,ishape)
         if ( ishape==0 ) nelemlc = nelemlc+1
         iw = 1
         do j = i1, i2
            iwork(iw+j-i1) = kmeshm(istar3+j)
         end do

!        --- Fill elements in kprobi

         iw = 0
         do k = 1, nelemlc

            present = .true.  ! indicates whether element is in processor

            do j = 1, inpelm
               if ( iwork(iw+j)<=0 ) present = .false. ! node not in processor
            end do  ! j = 1, inpelm

            if ( present ) then

!           --- Element is completely in processor

               do j = 1, inpelm
                  kprobi(ikprbi+j) = iwork(iw+j)
               end do
               ikprbi = ikprbi+inpelm
               nelemb = nelemb+1

            end if  ! ( present )

            iw = iw+max(1,ishape)

         end do  ! k = 1, nelemlc

      else

!     --- There are compound curves

         nelemlc = 0

!        --- Loop over subcurves

         do ic = 1, nsubcv

!        --- Compute subcurve number, first point last point and number
!            of nodes

            jcurnr = kmeshq( kmeshq(icurnr)+ic )
            kcurnr = abs(jcurnr)
            j1 = kmeshm(4+kcurnr)
            j2 = kmeshm(5+kcurnr)-1
            if ( jcurnr>0 ) then

!           --- jcurnr>0, curve must be used in forward direction

               firstnode = kmeshm(istar3+j1)
               lastnode = kmeshm(istar3+j2)

            else

!           --- jcurnr<0, curve must be used in backward direction

               firstnode = kmeshm(istar3+j2)
               lastnode = kmeshm(istar3+j1)

            end if

            if ( debug ) then
               write(irefwr,1) 'ic, jcurnr, j1, j2', ic, jcurnr, j1, j2
               write(irefwr,1) 'firstnode, lastnode, prevlastnode ', &
                                firstnode, lastnode, prevlastnode
               call prinin ( kmeshm(istar3+j1), j2-j1+1, 'kmeshm' )
            end if

!           --- Check if this curve is the start of a new series
!               or the next in a row

            if ( ic==1 ) then

!           --- First curve, hence always next in a row
!               Set npoints and fill iwork

               npoints = j2-j1
               iw = 1
               if ( jcurnr>0 ) then

!              --- jcurnr>0, curve must be used in forward direction

                  do j = j1, j2
                     iwork(iw+j-j1) = kmeshm(istar3+j)
                  end do

               else

!              --- jcurnr<0, curve must be used in backward direction

                  do j = j2, j1, -1
                     iwork(iw+j2-j) = kmeshm(istar3+j)
                  end do

               end if
               if ( debug ) then
                  write(irefwr,1) 'iw, npoints', iw, npoints
                  call prinin ( iwork(iw), j2-j1+1, 'iwork' )
               end if
               iw = iw+j2-j1+1

            else if ( firstnode==prevlastnode ) then

!           --- first node is equal to previous last node
!               Hence the curve is the next in a row
!               The first point must be stored on the same position as the last
!               one
!               Raise npoints and fill iwork

               npoints = npoints+j2-j1
               iw = iw-1
               if ( jcurnr>0 ) then

!              --- jcurnr>0, curve must be used in forward direction

                  do j = j1, j2
                     iwork(iw+j-j1) = kmeshm(istar3+j)
                  end do

               else

!              --- jcurnr<0, curve must be used in backward direction

                  do j = j2, j1, -1
                     iwork(iw+j2-j) = kmeshm(istar3+j)
                  end do

               end if

               if ( debug ) then
                  write(irefwr,1) 'iw, npoints', iw, npoints
                  call prinin ( iwork(iw), j2-j1+1, 'iwork' )
               end if

               iw = iw+j2-j1+1

               if ( debug )  write(irefwr,1) 'iw', iw

            else

!           --- first node is not equal to previous last node
!               Hence the curve is the start of a new row
!               Copy information of iwork into kprobi

               npoints = npoints/max(1,ishape)
               if ( ishape==0 ) npoints = npoints+1
               nelemlc = nelemlc+npoints
               iw = 0
               do k = 1, npoints

                  present = .true.  ! indicates whether element is in processor

                  do j = 1, inpelm
                     if ( iwork(iw+j)<=0 ) present = .false. ! not in processor
                  end do  ! j = 1, inpelm

                  if ( present ) then

!                 --- Element is completely in processor

                     do j = 1, inpelm
                        kprobi(ikprbi+j) = iwork(iw+j)
                     end do
                     ikprbi = ikprbi+inpelm
                     nelemb = nelemb+1

                  end if  ! ( present )

                  iw = iw+max(1,ishape)

               end do  ! k = 1, npoints

!              --- Reset npoints and iw

               npoints = j2-j1
               iw = 1
               if ( jcurnr>0 ) then

!              --- jcurnr>0, curve must be used in forward direction

                  do j = j1, j2
                     iwork(iw+j-j1) = kmeshm(istar3+j)
                  end do

               else

!              --- jcurnr<0, curve must be used in backward direction

                  do j = j2, j1, -1
                     iwork(iw+j2-j) = kmeshm(istar3+j)
                  end do

               end if
               if ( debug ) then
                  write(irefwr,1) 'iw, npoints', iw, npoints
                  call prinin ( iwork(iw), j2-j1+1, 'iwork' )
               end if
               iw = j2-j1+2
               if ( debug )  write(irefwr,1) 'iw', iw

            end if
            prevlastnode = lastnode

         end do

!        --- Finally last part

         npoints = npoints/max(1,ishape)
         if ( ishape==0 ) npoints = npoints+1
         nelemlc = nelemlc+npoints
         iw = 0
         do k = 1, npoints

            present = .true.  ! indicates whether element is in processor

            do j = 1, inpelm
               if ( iwork(iw+j)<=0 ) present = .false. ! node not in processor
            end do  ! j = 1, inpelm

            if ( present ) then

!           --- Element is completely in processor

               do j = 1, inpelm
                  kprobi(ikprbi+j) = iwork(iw+j)
               end do
               ikprbi = ikprbi+inpelm
               nelemb = nelemb+1

            end if  ! ( present )

            iw = iw+max(1,ishape)

         end do  ! k = 1, npoints

      end if  ! ( nsubcv<=1 )

      call erclos ( 'probcurvelm' )
      end
