      subroutine probdfext ( ichprb, iseqprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.5  date 17-03-2015 extra debug
!        version  3.4  date 06-08-2014 Extra debug
!        version  3.3  date 27-03-2013 New call to probdfloop
!
!   copyright (c) 2005-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     integer information with respect to the problem is computed
!     and filled in array kprob
!     the mesh must have been created before
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     fictitious_domain
!     global_unknown
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulexxx
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulekprob
      use sepmoduletexts
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ichprb, iseqprob

!     ichprb         i    Choice parameter consisting of two parts:
!                         ichoice and kread according to
!                         ichprb = ichoice + kread*100
!                         with
!                         kread     Indication if first record has been read
!                                   (>0) or not (0)
!                                   Possible values:
!                                   0:  first record has not been read
!                                   1:  first record has been read but not the
!                                       problem number
!                                   2:  The default problem number is used
!                                   >10:  first record has been read as well as
!                                       the problem number
!                                       The problem number is equal to kread-10
!                         ichoice    choice parameter.  possibilities:
!                         0    all input is read from the standard input file
!                         1    the input is stored by the user in array iinput
!                         2    the input is read from the standard input file
!                              and stored in array iinput
!                         3    previously read input is reapplied to changed
!                              mesh
!                         4    special case for seppost
!                              iinput(1) must have been filled with the
!                              memory management sequence number of the
!                              previous iinput.
!                              Boundary conditions, local transforms etc. are
!                              skipped
!                         5    special case for seppost
!                              iinput(1) must have been filled with the
!                              memory management sequence number of the previous
!                              iinput.
!                              iinput(2) must have been filled with the
!                              memory management sequence number of the previous
!                              rinput.
!                              Boundary conditions, local transforms etc. are
!                              skipped
!                         10   the input is read from the standard input file
!                              and stored in array iinput.
!                              No further actions are performed, so the only
!                              task is to fill iinput.
!     iseqprob       i    Sequence number of problem input
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenrinp, leniinp, ichoice, kread, ipiinp, iprinp, &
              iprint
      logical newelm
      double precision :: dum

!     dum            dummy
!     ichoice        See ichprb
!     ipiinp         Starting address of array IINPUT into array IBUFFR
!     iprinp         Starting address of array RINPUT into array BUFFER
!     iprint         Level of print information
!     kread          See ichoice
!     leniinp        Length of array iinput that has been used for
!                    the previous problem
!     lenrinp        Length of part of array rinput that has been used for
!                    the previous problem
!     newelm         Parameter indicating if different fluid elements are
!                    found that contain lambda points. Hence, a new call
!                    to commat should be made.
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     PROBDFINPUT    Read arrays iinput and rinput for probdf
!     PROBFICT       Store the fluid elements corresponding to each lambda point
!                    in array kprobfict
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPFILLKMESHE  Fill array kmesh part e
!     SEPFILLKPROB   Fill parameters in module sepmodulekprob
!     SEPPRINKPROB   Print array KPROB
!     SEPPROBLOOP1   information with respect to the problem is computed
!                    and filled in array kprob
! **********************************************************************
!
!                       I/O
!
!    PROBDF reads input from the standard input file (subroutine PROREA)
!    as well as from file 1 (subroutine PR )
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       1   ichoice is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Perform checks on input and initialize
!   Reserve some permanent space
!   if ( ichoice = 0, 2 or 10 ) then
!      Read input from standard input file
!   if ( ichoice # 10 ) then
!      for iprob = 1 (1) nprob do
!         Decrease priorities of all arrays
!         Reset priorities for the permanent arrays
!         Compute starting positions of parts in array IINPUT
!         Compute number of nodes per element group as well as
!         corresponding shape numbers (also for boundary elements)
!         Store at the end of IBUFFR from position ipelshap
!         Fill information concerning standard problems into array KPROB
!         Fill information concerning the natural boundary conditions
!         Fill arrays KPROB part f and e
!            In part f the number of degrees of freedom per point is
!            stored if not constant, in part e information concerning
!            vectors of special structure is stored
!         Compute number of points where essential boundary conditions
!         are given
!         If ( there are physical degrees of freedom ) then
!            Fill prob part p with information about the physical degrees of
!            freedom
!         Fill arrays KPROB part h, r and s
!         Renumber KPROB part p
!            The new sequence of the unknowns as used in the matrix is stored
!            in these parts
!         If ( local transformations are present ) then
!            Store information about local transform in part n
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Initializations

      call eropen ( 'probdfext' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probdfext'
         call ertrace
         write(irefwr,1) 'ichprb, iseqprob', ichprb, iseqprob
  1      format ( a, 1x, (10i10) )

      end if
      if ( ierror/=0 ) go to 1000
      ichoice = mod(ichprb,100)
      kread = ichprb/100
      iprint = 0
      nocallprobdf = .false.

!     --- Tests

      if ( kread<0 .or. kread>2 .and. kread/=11 .or. &
           (ichoice<0 .or.ichoice>5) .and. ichoice/=10 ) then
         call errint ( ichprb, 1 )
         call errsub ( 1, 1, 0, 0 )
      end if
      call sepfillkmesh ( kmesh )
      if ( kelme==0 ) call sepfillkmeshe
      bufchanged = .false.

!     --- read iinput and rinput if necessary

      if ( debug ) write(irefwr,1) 'before probdfinput'

      call probdfinput ( ichprb, iseqprob )
      if ( debug ) &
         write(irefwr,1) 'after probdfinput, iseqprob', iseqprob

!     --- Loop over all problems
!         The reservation of memory takes place in under subroutines instead
!         of PROBDF.
!         In this way PROBDF differs from other standard SEPRAN subroutines
!         This situation should be handled with care!

      if ( ichoice/=10 ) then

!     --- ichoice # 10, carry out the computations
!         Store IINPUT and RINPUT

         ipiinp = 1
         iprinp = 1

         if ( kp(iseqkp)%iinputkp(15)>1 ) iprint = kp(iseqkp)%iinputkp(35)

         do iprob = 1, nprob

!        --- main subroutine

            if ( debug ) write(irefwr,1) 'iprob', iprob
            kprobpart => kp(iseqkp)%kprobsub(iprob)
            kprobloc => kprobpart%kprobloc
            kprobloc(48) = maxcontc
            kprobpart%iinputloc => kp(iseqkp)%iinputkp(ipiinp:)
            kprobpart%rinputloc => kp(iseqkp)%rinputkp(iprinp:)
            iinputkp => kprobpart%iinputloc
            rinputkp => kprobpart%rinputloc
            allocate ( kprobpart%intarprb(20) )
            intarprb => kprobpart%intarprb
            call sepprobloop1 ( ichoice, leniinp, lenrinp )
            if ( debug ) write(irefwr,1) 'after probdfloop'

!           --- Raise ipiinp for next problem

            kprobpart%iinputloc => kp(iseqkp)%iinputkp(ipiinp:ipiinp+leniinp-1)
            if ( iprob<nprob ) then

               ipiinp = ipiinp + leniinp
               iprinp = iprinp + lenrinp  ! raise iprinp with part used
               if ( debug ) &
                  write(irefwr,1) 'ipiinp, iprinp', ipiinp, iprinp

            end if
            if ( ierror/=0 ) go to 1000

         end do  ! iprob = 1, nprob

      end if  ! ( ichoice/=10 )

1000  do iprob = 1, nprob

         kprobloc => kp(iseqkp)%kprobsub(iprob)%kprobloc
         call sepfillkprob
         kmeshloc => kmesh

         if ( kprobloc(51)>0 ) then

!        --- kprob(51)>0, hence there are fictitious unknowns
!            Store fluid elements coupled to fictitious unknowns

            call probfict ( newelm )

         end if

      end do  ! iprob = 1, nprob

      if ( iprint==1 ) then

!     --- iprint = 1, selected output

         call sepprinkprob ( -1 )

      else if ( debug .or. iprint==2 ) then

!     --- iprint = 2, complete output

         call sepprinkprob ( 0 )

      end if
      iprob = 0    ! reset necessary for subroutine ercofs
      bufchanged = .true.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End probdfext'

      end if

      call erclos ( 'probdfext' )

      end subroutine probdfext
