      subroutine probfict01
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 25-11-2002 Reorganization (JdH)
!        version  2.1  date 19-11-2002 Allow 1 lambda point for quadratic
!                                      line element (RvL)
!        version  2.0  date 03-04-2001 Allow 1 lambda point for biquadr.
!                                      surface element (J de Hart)
!
!   copyright (c) 2000-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Store the structural elements corresponding to each fictitious unknown
!
! **********************************************************************
!
!                       KEYWORDS
!
!     fictitious_domain
!     structural_element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i1, i2, icurnr, icurve1, icurve2, igrp, inunkpgr, &
              inunkp_elm, iposkproby, iregionlam, ishape_curve, &
              istructgrp, isurnr, nelemlc, ishapelam, &
              nfictpnt, i, ielem, ielemlow, ielemtop, inpelm, &
              istar3, istart, j, k, kstep, nodp1, nodp2, nvert
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     i1             First point of curve in KMESH part m
!     i2             Last point of curve in KMESH part m
!     icurnr         Curve sequence number
!     icurve1        First curve number of a set
!     icurve2        Last curve number of a set
!     ielem          Element number
!     ielemlow       Smallest element number in group
!     ielemtop       Largest element number in group
!     igrp           Boundary element group number read
!     inpelm         number of points per element
!     inunkp_elm     Number of unknowns per point
!     inunkpgr       Number of global unknowns in a group
!     iposkproby     Position in kprob y
!     iregionlam     type of region where the lambda's are defined:
!                     1:  curves
!                     2:  surfaces
!     ishape_curve   Shape of lambda basis functions
!     ishapelam      Shape of lambda basis functions
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         First position used at a part
!     istructgrp     Structural element group
!     isurnr         Surface sequence number
!     iwork          integer work array of length nelem
!                    iwork(i) is set equal to 1 if element i is connected
!                    to the actual node
!     j              Counting variable
!     k              Counting variable
!     kstep          Step in loop
!     nelemlc        Number of elements
!     nfictpnt       Total number of points with fictitious unknowns
!     nodp1          Node sequence number
!     nodp2          Node sequence number
!     nvert          Used only when jmetod=11. Number of columns in the
!                    scheme.
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
!    2153   shape number of lambda's does not correspond to shape number of
!           structural elements
! **********************************************************************
!
!                       PSEUDO CODE
!
!    nfictgrp := kproby(1)
!    nfictpnt := 0
!    Clear array iwork
!    For igrp := 1 (1) nfictgrp do
!       istructgrp := structural group corresponding to igrp
!       inunkp_elm := number of lambda's in element
!       icurve1 := first curve number for this group
!       icurve2 := last curve number for this group
!       iregionlam := type of region where the lambda's are defined
!       ielemlow := lowest element number of structural group
!       ielemtop := highest element number of structural group
!
!       if ( region is of type curve ) then
!          ishape := shape number of structural element
!          if ( ishape = 3 or 5 ) then
!             ishape_curve := 1
!          else if ( ishape = 4 or 6 ) then
!             ishape_curve := 2
!          for icurnr := icurve1 (1) icurve2 do
!             For all elements ielem along curve icurnr
!                Find structural element containing ielem
!                Sore structural element number into next inunkp_elm positions
!             nfictpnt := nfictpnt+inunkp_elm
!       else if ( region is of type surface ) then
!          ishape := shape number of structural element
!          if ( ishape = 11 ) then
!             ishape_curve := 3
!          else if ( ishape = 12 ) then
!             ishape_curve := 4
!          else if ( ishape = 13 ) then
!             ishape_curve := 5
!          else if ( ishape = 14 ) then
!             ishape_curve := 6
!          for isurnr := icurve1 (1) icurve2 do
!             For all elements ielem along surface number isurnr
!                Find structural element containing ielem
!                Sore structural element number into next inunkp_elm positions
!             nfictpnt := nfictpnt+inunkp_elm
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probfict01' )

!     --- Make some initializations

      nfictgrp = kproby(1)
      nfictpnt = 0

      allocate ( iwork(nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, nelem, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork = 0

!     --- Loop over fictitious groups

      istar3 = kmeshm(2)-1
      do igrp = 1, nfictgrp

!     --- Set some relevant parameters

         iposkproby = kproby(10+igrp)-1
         inunkpgr   = kproby(iposkproby+2)
         iregionlam = kproby(iposkproby+3)
         ishapelam  = kproby(iposkproby+4)
         icurve1    = kproby(iposkproby+5)
         icurve2    = kproby(iposkproby+6)
         istructgrp = kproby(iposkproby+7)
         inunkp_elm = kproby(iposkproby+10)/inunkpgr

         ielemlow = 0
         do i = 1, istructgrp-1
            ielemlow = kmeshg(i)+ielemlow
         end do
         ielemtop = ielemlow + kmeshg(istructgrp)
         ielemlow = ielemlow+1

         if ( iregionlam==1 ) then

!        --- region is of type curve
!            ishape := shape number of structural element

            ishape = kmeshf(istructgrp)
            if ( ishape==3 .or. ishape==5 .or. ishape.eq.9 ) then

!           --- ishape = 3 or 5

               ishape_curve = 1

            else if ( ishape==4 .or. ishape==6 ) then

!           --- ishape = 4 or 6

               ishape_curve = 2

            else

!           --- Shape number not available

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               goto 1000

            end if
            if ( ishapelam/=ishape_curve .and.ishape/=6 ) then

!           --- shape number of lambda's incorrect

               call errint ( igrp, 1 )
               call errint ( ishapelam, 2 )
               call errint ( istructgrp, 3 )
               call errint ( ishape_curve, 4 )
               call errsub ( 2153, 4, 0, 0 )

            end if

!           --- loop over curves

            do icurnr = icurve1, icurve2

               i1 = kmeshm(4+icurnr)
               i2 = kmeshm(5+icurnr)-1

!              --- Find structural element containing ielem

               do i = i1, i2-1, ishape_curve

                  nodp1 = kmeshm(istar3+i)
                  nodp2 = kmeshm(istar3+i+ishape_curve)

!                 --- set iwork to 1 for all elements corresponding to
!                     nodp1

                  do k = kmshd1(nodp1)+1, kmshd1(nodp1+1)
                     ielem = kmshd2(k)
                     if ( ielem>=ielemlow .and. ielem<=ielemtop ) &
                        iwork(ielem) = 1
                  end do

!                 --- find neighbour element if nodp2 that has iwork(ielem)=1

                  do j = kmshd1(nodp2)+1, kmshd1(nodp2+1)
                     ielem = kmshd2(j)
                     if ( iwork(ielem)==1 ) then

!                    --- element found

                        do k = 1, inunkp_elm
                           kprobfict(1,nfictpnt+k) = ielem
                        end do
                        nfictpnt = nfictpnt+inunkp_elm
                        go to 100

                     end if

                  end do

!                 --- Clear iwork

100               do k = kmshd1(nodp1)+1, kmshd1(nodp1+1)
                     iwork(kmshd2(k)) = 0
                  end do

               end do

            end do

         else

!        --- region is of type surface
!            ishape := shape number of structural element

            ishape = kmeshf(istructgrp)
            if ( ishape==11 ) then

!           --- ishape = 11

               ishape_curve = 3
               nvert = 3
               kstep = 1

            else if ( ishape==12 ) then

!           --- ishape = 12

               ishape_curve = 4
               nvert = 3
               kstep = 2

            else if ( ishape==13 ) then

!           --- ishape = 13

               ishape_curve = 5
               nvert = 4
               kstep = 1

            else if ( ishape==14 ) then

!           --- ishape = 14

               ishape_curve = 6
               nvert = 4
               kstep = 2

            else

!           --- error: shape number not yet implemented

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               go to 1000

            end if
            if ( ishapelam/=ishape_curve.and.ishape_curve/=6 ) then

!           --- shape number of lambda's incorrect

               call errint ( igrp, 1 )
               call errint ( ishapelam, 2 )
               call errint ( istructgrp, 3 )
               call errint ( ishape_curve, 4 )
               call errsub ( 2153, 4, 0, 0 )

            end if

!           --- Find starting address in kmesh part n

            istart = 2*nsurfs
            do isurnr = 1, icurve1-1
               istart = istart+kmeshn(nsurfs+isurnr)*kmeshn(isurnr)
            end do

!           --- loop over surfaces

            do isurnr = icurve1, icurve2

               nelemlc = kmeshn(nsurfs+isurnr)
               inpelm = kmeshn(isurnr)

!              --- Find structural element containing ielem

               do i = 1, nelemlc
                  nodp1 = kmeshn(istart+1)

!                 --- set iwork to 1 for all elements corresponding to
!                     nodp1

                  do k = kmshd1(nodp1)+1, kmshd1(nodp1+1)
                     ielem = kmshd2(k)
                     if ( ielem>=ielemlow .and. ielem<=ielemtop ) &
                        iwork(ielem) = 1
                  end do

!                 --- set iwork to 0 for all elements not corresponding to
!                     the other vertices

                  do j = 2, nvert-1
                     nodp2 = kmeshn(istart-kstep+1+j*kstep)
                     do k = kmshd1(nodp2)+1, kmshd1(nodp2+1)
                        ielem = kmshd2(k)
                        if ( iwork(ielem)>0 ) &
                           iwork(ielem) = iwork(ielem)+1
                     end do
                  end do
                  nodp2 = kmeshn(istart-kstep+1+nvert*kstep)
                  do j = kmshd1(nodp2)+1, kmshd1(nodp2+1)
                     ielem = kmshd2(j)
                     if ( iwork(ielem)==nvert-1 ) then

!                    --- element found

                        do k = 1, inunkp_elm
                           kprobfict(1,nfictpnt+k) = ielem
                        end do
                        nfictpnt = nfictpnt+inunkp_elm
                        go to 200

                     end if
                  end do

!                 --- Clear iwork

200               do k = kmshd1(nodp1)+1, kmshd1(nodp1+1)
                     iwork(kmshd2(k)) = 0
                  end do

                  istart = istart + inpelm

               end do

            end do

         end if
         iposkproby = iposkproby+12

      end do

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'probfict01' )

      end
