      subroutine probflkprobg
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.18 date 17-03-2015 extra debug
!        version  2.17 date 13-10-2013 New call to prob04
!        version  2.16 date 15-10-2012 Fill kprob(64)
!
!   copyright (c) 2000-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information with respect to essential boundary conditions in
!     KPROB for one problem
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     boundary_condition
!     essential
!     essential_boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nesintersect
      integer, allocatable :: iskippoint(:), nodes(:,:), ipoint(:)

!     ipoint         Array of length nesintersect containing the
!                    number of nodes for each intersection
!     iskippoint     Array of length nskip+1 containing the points to
!                    be skipped
!                    Position 1 contains nskip (number of points to be
!                               skipped
!                    The next positions contain the node numbers to be
!                    skipped
!     nesintersect   Number of intersections where essential boundary
!                    conditions are given
!     nodes          Array of size nesintersect x npoint
!                    nodes(i,1:ipoint(i)) contains the nodal points
!                    corresponding to intersection i
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PROB04         Compute number of points where essential boundary
!                    conditions are given
!     PROB05         Fill array KPROB part g
!     PROBSKIPPOINT  Detect which points are completely in the set of elements
!                    that are skipped
!     SEPFILLKMESHD  Fill kmesh part d
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Make initializations
!     If ( kprob part aa has been filled ) then
!        Store nodes to be skipped temporarily in iskippoints
!     Activate arrays
!     if ( numessallobst>0 or numessinnobst>0 ) then
!         Fill array kmesh ad with information concerning obstacles
!     Compute starting addresses
!     Compute number of points where essential boundary conditions are given
!     if ( nkbnd>0 ) then
!        Make array KPROB part g permanent
!        Fill array KPROB part g
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probflkprobg' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probflkprobg'
         write(irefwr,1) 'kprob(53)', kprobloc(53)
         call prinin ( iinputkp, 50, 'iinputkp')
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( kprobloc(53)>0 ) then

!     --- kprob part aa has been filled
!         First create or activate kmesh part d

         if ( debug ) write(irefwr,*) 'array iskippoint is used'
         if ( kelmd==0 ) call sepfillkmeshd

!        --- reserve npoint+1 positions to store array iskippoint

         allocate ( iskippoint(npoint), stat = error )
         if ( error/=0 ) call eralloc ( error, npoint+1, 'iskippoint' )
         if ( ierror/=0 ) go to 1000

!        --- Store nodes to be skipped temporarily in iskippoint

         call probskippoint ( iskippoint )

      else

!     --- iskippoint is not needed

         allocate ( iskippoint(1), stat = error )
         if ( error/=0 ) call eralloc ( error, 1, 'iskippoint' )
         if ( ierror/=0 ) go to 1000

      end if  ! ( kprob(53)>0 )

      if ( iinputkp(56)>0 ) then

!     --- iinputkp(56)>0, hence intersection points are required

         nesintersect = iinputkp(56)

         if ( debug ) &
            write(irefwr,1) 'nesintersect, npoint', nesintersect, npoint

!        --- reserve npoint*nesintersect positions to store array nodes
!            and nesintersect positions to store array ipoint

         allocate ( nodes(npoint,nesintersect), ipoint(nesintersect), &
                    stat = error )
         if ( error/=0 ) &
            call eralloc ( error, nesintersect*(npoint+1), 'nodes' )
         if ( ierror/=0 ) go to 1000

      else

!     --- iinputkp(56)=0, arrays are not needed

         nesintersect = 1
         allocate ( nodes(1,1), ipoint(1), stat = error )
         if ( error/=0 ) call eralloc ( error, 2, 'nodes' )
         if ( ierror/=0 ) go to 1000

      end if  ! ( iinputkp(56)>0 )

!     --- Compute number of points where essential boundary conditions
!         are given

      if ( debug ) write(irefwr,1) 'before prob04'
      call prob04 ( iskippoint, nodes, ipoint, nesintersect )

      kprobloc(9) = nkbnd
      if ( debug ) write(irefwr,1) 'nkbnd', nkbnd

      if ( nkbnd>0 ) then

!     --- Make array KPROB part g permanent

         allocate ( kprobpart%kprobg(2,nkbnd) )
         kprobg => kprobpart%kprobg
         kprobloc(20) = nkbnd

!        --- Fill array kprobg

         call prob05 ( iskippoint, nodes, ipoint, nesintersect )

      end if

!     --- remove temporary array iskippoint

      deallocate ( iskippoint, nodes, ipoint, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iskippoint' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nkbnd', nkbnd
         write(irefwr,*) 'End probflkprobg'

      end if
      call erclos ( 'probflkprobg' )

      end
