      subroutine probmap3d2d ( iwork, isol, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-02-2005
!
!   copyright (c) 2005-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map kprob defined on 3d grid onto 2d surface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     map
!     surface
!     2d
!     3d
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), isol(*), nnodes

!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     iwork          i    Contain the local node numbers corresponding to the
!                         global ones
!                         if zero node is not local
!                         length of array iwork is npoint
!     nnodes         i    Number of nodes in 2d mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision dummy(1)
      integer, allocatable :: kprobf2d(:)
      integer, pointer :: kproblc

!     dummy          Dummy parameter
!     i              Counting variable
!     kprobf2d       kprob part f corresponding to 2d mesh
!     kproblc        Contains information of the problem definition
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PROBNUMUNK     Compute number of unknwons per point as well as total
!                    number of unknowns in vector restricted to a set of
!                    surfaces
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPCRPROBINFO  create a new array in kp
!     SEPRMPROBINFO  Remove array with sequence number kproblc from kp
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear arry kproblc
!     Fill first part of kprob
!     Compute number of degrees of freedom per point
!     Fill kprob parts a and f if necessary
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probmap3d2d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probmap3d2d'
100      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      kproblc => kprob2d(maxmesh+1)
      call sepcrprobinfo ( kproblc, 1 )
      kprobpart => kp(iseqkp)%kprobsub(1)
      kprobloc => kprobpart%kprobloc

!     --- Clear arry kproblc, except first position

      if ( kprobloc(2)==101 ) call seprmprobinfo ( kproblc )

!     --- Fill first part of kprob

      kprobloc(2) = 101
      kprobloc(7) = 1         ! ntype
      kprobloc(8) = 1         ! nelgrp
      kprobloc(31) = 2
      kprobloc(40) = 1        ! nprob
      allocate ( kp(iseqkp)%kprobsub(1)%kprobc(1))
      kprobc => kprobpart%kprobc

      kprobc(1) = 1        ! kproblc part c

!     --- Compute number of degrees of freedom per point
!         First activate solution array and corresponding arrays

      call sepactsolbf2 ( isol )
      nusoli = isol(5)

!     --- Make temporary space to store kprobf2d

      allocate ( kprobf2d(nnodes+1), stat = error )
      if ( error/=0 ) call eralloc ( error, nnodes+1, 'kprobf2d' )
      if ( ierror/=0 ) go to 1000

!     --- Compute number of degrees of freedom per point

      call probnumunk ( nunkpi, nusoli, indprfi, kprobfi, &
                        kprobf2d, iwork, npoint, nnodes )

!     --- Fill kproblc part f if necessary

      kprobloc(4) = nunkpi
      kprobloc(5) = nusoli
      kprobloc(33) = nunkpi
      if ( indprfi>0 ) then

!     --- indprf>0, hence number of nodes per point in 2d mesh not constant
!         Make array permanent

         if ( associated(kprobpart%kprobf) ) &
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobf )
         allocate ( kp(iseqkp)%kprobsub(iprob)%kprobf(nnodes+1) )
         kprobloc(19) = nnodes+1
         kprobfo => kprobpart%kprobf

      end if  ! ( indprf>0 ) then

      deallocate ( kprobf2d, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'kprobf2d' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,100) 'nunkp, nusol', nunkpi, nusoli
         call prinin ( kproblc, 61, 'kproblc' )
         write(irefwr,*) 'End probmap3d2d'

      end if
      call erclos ( 'probmap3d2d' )

      end
