      subroutine probparglob2
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-03-2013
!
!   copyright (c) 2013-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute global quantities with respect to problem definition
!     Parallel case (nodes>1)
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: leniwork, lenkprobf, lenkprobh, icontext
      integer, pointer :: send_buffer(:)
      integer, save :: sendbuffersize

!     icontext       Tag for sending and receiving, is used as check
!     iwork          integer work array with the following contents
!                    1:  npoint (local)
!                    2:  nunkp (local)
!                    3:  nrusol (local)
!                    4:  lenkprobf (local)
!                    5:  lenkprobh (local)
!                    followed by array map (length npoint)
!                    followed by array kprobf (length lenkprobf)
!                    followed by array kprobh (length lenkprobh)
!     leniwork       Computed length of array iwork
!     lenkprobf      Length of array kprobf
!     lenkprobh      Length of array kprobh
!     send_buffer    Buffer for sending messages
!     sendbuffersize Size of send_buffer in bytes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INIBSENDMPI    Send information to other node (mpi buffered)
!     INIBUFATTMPI   Attach mpi buffer if it exists
!     INIBUFDETMPI   Detach mpi buffer if it exists
!     INIBUFSIZEMPI  Extend send_buffer_size with mpi overhead
!     PROBPARGLOB11  Send information from node to host with respect to global
!                    vectors of special structure
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2890   Periodical boundary conditions have not been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Send dimension quantities as well as mapping and kprobf/h to block 1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probparglob2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probparglob2'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set parameters

      lenkprobf = 0
      lenkprobh = 0
      if ( indprf>0 ) lenkprobf = size(kprobf)
      if ( indprh>0 ) lenkprobh = size(kprobh)
      leniwork = 5+npoint+lenkprobf+lenkprobh*2

      if ( debug ) &
         write(irefwr,1) 'npoint, lenkprobf, lenkprobh, leniwork', &
                          npoint, lenkprobf, lenkprobh, leniwork

!     --- allocate iwork

      allocate ( iwork(leniwork), stat = error )
      if ( error/=0 ) call eralloc ( error, leniwork, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Fill iwork

      iwork(1) = npoint
      iwork(2) = nunkp
      iwork(3) = kprobloc(66)       ! nrusol
      iwork(4) = lenkprobf       ! indication if kprobf has been filled
      iwork(5) = lenkprobh*2     ! indication if kprobh has been filled

      if ( kprobloc(34)>0 ) then

!     --- Error: periodical boundary conditions not yet implemented

         call errsub ( 2890, 0, 0, 0 )
         go to 1000

      end if  ! ( kprob(34)>0 )

      iwork(6:5+npoint) = mappingpar(1:npoint)  ! copy mapping from local to global
      leniwork = 5+npoint

      if ( lenkprobf>0 ) then

!     --- Array kprob f exists

         iwork(leniwork+1:leniwork+lenkprobf) = kprobf(1:lenkprobf)
         leniwork = leniwork+lenkprobf

      end if  ! ( lenkprobf>0 )

      if ( lenkprobh>0 ) then

!     --- Array kprob h exists

         iwork(leniwork+1:leniwork+lenkprobh) = kprobh
         leniwork = leniwork+lenkprobh
         iwork(leniwork+1:leniwork+lenkprobh) = kprobhinv
         leniwork = leniwork+lenkprobh

      end if  ! ( lenkprobf>0 )

      sendbuffersize = leniwork*4  ! size in bytes

!     --- Compute actual buffer size using mpi information
!         allocate array and attach buffer

      call inibufsizempi ( sendbuffersize, 1 )

      allocate(send_buffer(sendbuffersize))

      call inibufattmpi ( send_buffer, sendbuffersize )

      if ( debug ) write(irefwr,1) 'leniwork', leniwork

!     --- Send array iwork

      icontext = 42
      call inibsendmpi ( iwork, leniwork, SPINT, 1, icontext )

      if (associated(send_buffer)) then

!     --- deattach send_buffer and deallocate

         call inibufdetmpi ( send_buffer, sendbuffersize )
         deallocate(send_buffer)

      end if  ! (associated(send_buffer))

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

      call probparglob11

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End probparglob2'

      end if  ! ( debug )
      call erclos ( 'probparglob2' )

      end
