      subroutine prord3volm ( iinputlc, iinplc, table, tabout, tabmax, &
                              nesvlm, lenvlm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-03-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interprete volume information read by prord3 and store in iinputlc
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     volume
!     essential_boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer  iinputlc(*), iinplc, nesvlm, lenvlm
      double precision table(*), tabout(*), tabmax(*)

!     iinplc        i/o   Last position used in array iinputlc
!     iinputlc      i/o   Work array in which the input read is stored before
!                         copying in iinput
!                         See subroutine prord2
!     lenvlm         i    Length of volumes part
!     nesvlm         i    Number of parts where volume essential boundary
!                         conditions are given
!     table          i    Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmax         i    Contains maximum values allowed in table
!     tabout         i    Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iposut, inum, iskipbn, jnpelm, ncurves, m, iinplcbef

!     iinplcbef      Value of iinplc at start
!     inum           Number of local nodes
!     iposut         Position in array tabout
!     iskipbn        Defines if the outer boundary must be skipped
!                    Possible values:
!                    0:  the outer boundary must not be skipped
!                    1:  the complete outer boundary must be skipped
!                    2:  the inner boundary must be skipped as well as some
!                        explicitly given outer curves
!                    3:  the outer boundary must be skipped except some
!                        explicitly given outer curves
!     jnpelm         Number of nodes in an element
!     m              number of volumes
!     ncurves        Number of curves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     READC301       Check the numrel numbers stored in array relnods
! **********************************************************************
!
!                       I/O
!
!     VOLUMES [i1, i2, i3, ... ] (Vj, Vk, .. ), [SKIP_SURFACES (Si, Sj, ... )]
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prord3volm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord3volm'
         iinplcbef = iinplc
         write(irefwr,1) 'nesvlm', nesvlm
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nesvlm = nesvlm+1
      iinputlc(iinplc+2) = 11
      iposut = nint(table(6))
      inum = nint(tabmax(6))
      m = inum/10000          ! number of volumes
      inum = mod(inum,10000)  ! number of local nodes

      iinputlc(iinplc+4) = m  ! number of volumes
      iinputlc(iinplc+5:iinplc+4+m) = &
         nint(tabout(iposut:iposut+m-1))  ! set of volumes
      jnpelm = sepgetinpelsurf ( iinputlc(iinplc+5), 1 )
      iinputlc(iinplc+3) = jnpelm

      iinplc = iinplc+4+m

      if ( debug ) &
         write(irefwr,1) 'iposut, inum, m, jnpelm', &
                          iposut, inum, m, jnpelm

      if ( inum<=0 ) then

!     --- inum<=0

         iinputlc(iinplc+1:iinplc+jnpelm) = 1

      else

!     --- inum>0,  copy contents from work into iinput

         call readc301 ( inum, jnpelm, tabout(iposut+m-1), work )
         iinputlc(iinplc+1:iinplc+jnpelm) = nint(work(1:jnpelm))

      end if  ! ( inum<=0 )
      if ( debug ) &
         call prinin ( iinputlc(iinplc+1), jnpelm, 'local nodes' )

      iinplc = iinplc+jnpelm

      if ( table(23)>0d0 ) then

!     --- skip surfaces

         iskipbn = 1

      else

!     --- do not skip surfaces

         iskipbn = 0
         ncurves = 0

      end if  ! ( table(23)>0d0 )

      iinputlc(iinplc+1) = iskipbn
      iinputlc(iinplc+2) = ncurves
      iinplc = iinplc+2

      if ( iskipbn>=1 ) then

!     --- iskipbn>0,  copy surfaces from work into iinput

         ncurves = nint(tabmax(23+iskipbn))
         iposut = nint(table(23+iskipbn))
         iinputlc(iinplc+1:iinplc+ncurves) = &
            nint(tabout(iposut:iposut+ncurves))

      end if  ! ( iskipbn>=1 )
      iinplc = iinplc+ncurves
      lenvlm = lenvlm+jnpelm+5+ncurves+m

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iinputlc(iinplcbef+1), iinplc-iinplcbef, &
                       'iinplc' )
         write(irefwr,1) 'iskipbn, iinplc, lenvlm', &
                          iskipbn, iinplc, lenvlm
         write(irefwr,*) 'End prord3volm'

      end if  ! ( debug )
      call erclos ( 'prord3volm' )

      end
