      subroutine prord8 ( iinput, maxlen, iinp, iinputlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 27-09-2010 Remove work arrays
!        version  1.3  date 30-03-2007 Correction error message
!        version  1.2  date 11-01-1999 Adaptation for surfaces in 2D
!
!   copyright (c) 1998-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read cards for global elements
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     global_unknown
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), maxlen, &
              iinp, iinputlc(*)

!     iinp      i/o   Last position used in IINPUT
!     iinput    i/o   integer array to be filled with information concerning
!                     the global elements
!     iinputlc        Work array in which the input read is stored before
!                     copying in iinput
!                     Per record read the following information is stored:
!                     Global element group number, followed by:
!                     Type of record:
!                        1: CURVES
!                        2: SURFACES
!                        3: VOLUMES
!                        4: ALL
!                     Followed by INUM = number of integers to follow, &
!                     followed by the integers itself
!     maxlen     i    Maximum amount of space for array iinput
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: ncodsb=3, nncode1=10
      integer itype1(nncode1), ncode1(nncode1), ncodmx1(nncode1), &
              nscods(ncodsb)
      double precision tabmin1(nncode1), tabmax1(nncode1)
      character (len=19) codesb(ncodsb), code1(nncode1)
      integer inum, iinplc, nglgrp, icurnr, inodp, isurnr, &
              iglgrp, ifirst, ilast, istart, nnodes, &
              i, jinp, numglcrv, numglsrf, &
              iform, istcrv, istsrf, &
              istall, istvlm, numglall, numglvlm
      integer, allocatable :: iwork(:)

!     code1      Table with keywords
!     codesb     Table with subkeywords
!     i          Counting variable
!     icurnr     Curve sequence number
!     ifirst     First curve (surface) number read
!     iform      Type of element
!     iglgrp     global element group number
!     iinplc     Last position used in array iinputlc
!     ilast      Last curve (surface) number read
!     inodp      Nodal point number
!     inum       Number of numbers read in a record
!     istall     Starting address of information about complete domain in iinput
!     istart     First position used at a part
!     istcrv     Starting address of curves in iinput
!     istsrf     Starting address of surfaces in iinput
!     istvlm     Starting address of volumes in iinput
!     isurnr     Surface sequence number
!     itype1     Indicates the type of action corresponding to keywords
!                in code1
!     iwork      work array of length nglgrp
!                iwork(i) = 0, means group i has not been treated yet
!     jinp       Help variable to store the value of iinp
!     ncode1     Table containing the number of significant letters in code1
!     ncodmx1    See ncodmx, now for code1
!     ncodsb     Number of entries in codesb
!     nglgrp     number of element groups corresponding to global unknowns
!     nncode1    Number of entries in code1
!     nnodes     Number of nodes
!     nscods     Number of sub keywords
!     numglall   number of parts where all elements are defined
!                for global elements
!     numglcrv   number of parts where curve elements are defined
!                for global elements
!     numglsrf   number of parts where surface elements are defined
!                for global elements
!     numglvlm   number of parts where volume elements are defined
!                for global elements
!     tabmax1    Contains maximum values allowed in table for code1
!     tabmin1    Contains minimum values allowed in table for code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    input is read from the input file.
!    The following texts are recognized:
!       gelm2 = curves ( [shape = k,] c1 to  c2 )
!       gelm3 = surfaces ( s1 to s2 )
!       gelm4 = volumes ( v1 to v2 )
!       gelm5 = all
!    main keywords ( see prorea )
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      29   Too few space for standard problems in IINPUT
!     152   last curve number has incorrect value
!     155   first surface number has incorrect value
!     156   last surface number has incorrect value
!     166   boundary element group number incorrect
!     296   first volume number has incorrect value
!     297   last volume number has incorrect value
!     334   record with curves contains wrong number of curve numbers
!     407   record with surfaces contains wrong number of surface numbers
!     550   array length too short
!    1943   More than one position information for one global element group
!    2002   The possibility global groups corresponding to the complete region
!           or to volumes has not yet been implemented
!    2004   Not all nodes along curve corresspond to nodal points
!    2005   Not all nodes along surface correspond to nodal points
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Read all input until main keyword is found, store in iinputlc
!    Copy all input in the correct sequence into iinput
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'GELM', 'CURVES', 'SURFACES', 'VOLUMES', &
                   'ALL', 'SHAPE', 'C', 'S', 'V', 'TO' /
      data ncode1  / 3, 5, 6, 6, 3, 4, 1, 1, 1,  2 /
      data ncodmx1 / 4, 6, 8, 7, 3, 5, 1, 1, 1,  2 /
      data itype1  / 0, 2, 2, 2, 2, 0,14,14,14, -1 /
      data tabmin1 / 1d0, 0d0, 0d0, 0d0, 0d0, 1d0, 2d0, 2d0, 2d0, 0d0 /
      data tabmax1 / 1d3, 0d0, 0d0, 0d0, 0d0, 2d0, 0d0, 0d0, 0d0, 0d0 /
      data codesb / 'GLOBAL_ELEMENTS', 'GLGRP', 'TYPE' /
      data nscods /  4, 3, 3 /

!     code1    Table of local keywords
!              1: GELM
!              2: CURVES
!              3: SURFACES
!              4: VOLUMES
!              5: ALL
!              6: SHAPE
!              7: C
!              8: S
!              9: V
!             10: TO
!     codesb   Table of local subkeywords
!              1: GLOBAL_ELEMENTS
!              2: GLGRP
!              3: TYPE
! ======================================================================
!
      call eropen ( 'prord8' )

!     --- Copy information into codesub

      call sepcopycodesub ( codesb, ncodsb, nscods )
      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )
      call sepcopytype ( nncode1, itype1, tabmin1, tabmax1 )
      nmain = nncodemain
      iretur = icode

      ityprd = 1

!     --- Read records for the generation of Global elements

      nglgrp = iinput(29)

      allocate ( iwork(nglgrp), stat = error )
      if ( error/=0 ) call eralloc ( error, nglgrp, 'iwork' )
      if ( ierror/=0 ) go to 1000

      numglcrv = 0
      numglsrf = 0
      numglvlm = 0
      numglall = 0
      iinplc = 0
      jinp = iinp
300   table(1:nncode) = -1d20
      iwork(1:nglgrp) = 0
      tabmax(7) = ncurvs
      tabmax(8) = nsurfs
      tabmax(9) = nvolms
      table(6) = 0d0
      iretur = icode
      call sepreadsetkeyw
      icode = iretur
      if ( icode==0 ) then

!     --- Not a main keyword found, hence GELM = ...

         if ( table(1)>0 ) then
            iglgrp = nint(table(1))
         else
            iglgrp = 0
         end if
         if ( iglgrp<1 .or. iglgrp>nglgrp ) then

!        --- Error global group number out of range

            call errint ( iglgrp, 1 )
            call errint ( nglgrp, 2 )
            call errsub ( 166, 2, 0, 0 )

         end if
         if ( iwork(iglgrp)/=0 ) then

!        --- Group already in use

            call errint ( iglgrp, 1 )
            call errsub ( 1943, 1, 0, 0 )

         end if
         iwork(iglgrp) = 1
         iinputlc(iinplc+1) = iglgrp
         iinplc = iinplc+1
         if ( table(2)>0d0 ) then

!        --- curves card found

            numglcrv = numglcrv+1

            ifirst = 0
            ilast = 0
            ishape = nint(table(6))
            inum = nint(tabmax(7))
            if ( inum==0 ) then
               call errint ( inum, 1 )
               call errint ( 2, 2 )
               call errsub ( 334, 2, 0, 0 )
            else
               iposut = nint(table(7))
               ifirst = nint(tabout(iposut))
               if ( inum>1 ) ilast = nint(tabout(iposut+1))
            end if
            iinputlc(iinplc+1) = 1
            iinputlc(iinplc+2) = ishape
            if ( ilast==0) ilast = ifirst
            if ( ilast<ifirst .or. ilast>ncurvs ) then
               call errint ( ilast, 1 )
               call errint ( ifirst, 2 )
               call errint ( ncurvs, 3 )
               call errsub ( 152, 3, 0, 0 )
            end if

!           --- Check if these curves have been filled

            do icurnr = ifirst, ilast
               istart = kmeshm(icurnr+4)
               nnodes = kmeshm(icurnr+5)-istart
               istart = istart+kmeshm(2)-1
               do i = 0, nnodes-1
                  inodp = kmeshm(istart+i)
                  if ( inodp<=0 ) then

!                 --- Nodal point has not been filled

                     call errint ( icurnr, 1 )
                     call errsub ( 2004, 1, 0, 0 )
                     go to 310

                  end if
               end do
310         end do

            iinputlc(iinplc+3) = ifirst
            iinputlc(iinplc+4) = ilast
            iinplc = iinplc+4

         else if ( table(3)>0d0 ) then

!        --- surface card found

            numglsrf = numglsrf+1
            ifirst = 0
            ilast = 0
            inum = nint(tabmax(8))
            if ( inum==0 ) then

!           --- No surface numbers found

               call errsub ( 407, 0, 0, 0 )

            else
               iposut = nint(table(8))
               ifirst = nint(tabout(iposut))
               if ( inum>1 ) ilast = nint(tabout(iposut+1))
            end if
            if ( ilast==0) ilast = ifirst
            if ( ilast<ifirst ) then
               call errint ( ilast, 1 )
               call errint ( ifirst, 2 )
               call errint ( nsurfs, 2 )
               call errsub ( 155, 2, 0, 0 )
            end if
            if ( ilast<ifirst .or. ilast>nsurfs ) then
               call errint ( ilast, 1 )
               call errint ( ifirst, 2 )
               call errint ( nsurfs, 3 )
               call errsub ( 156, 3, 0, 0 )
            end if
            if ( kelmn>0 ) then

!           --- Check if these surfaces have been filled

               istart = 2*nsurfs
               do isurnr = 1, ifirst-1
                  istart = istart + kmeshn(isurnr)*kmeshn(isurnr+nsurfs)
               end do
               do isurnr = ifirst, ilast
                  nnodes = kmeshn(isurnr)*kmeshn(isurnr+nsurfs)
                  do i = 1, nnodes
                     inodp = kmeshn(istart+i)
                     if ( inodp<=0 ) then

!                    --- Nodal point has not been filled

                        call errint ( isurnr, 1 )
                        call errsub ( 2005, 1, 0, 0 )
                        go to 320

                     end if
                  end do
320            end do
            end if
            iinputlc(iinplc+1) = 2
            iinputlc(iinplc+2) = ifirst
            iinputlc(iinplc+3) = ilast
            iinplc = iinplc+3

         else if ( table(4)>0d0 ) then

!        --- volume card found

            numglvlm = numglvlm+1
            ifirst = 0
            ilast = 0
            inum = nint(tabmax(9))
            if ( inum==0 ) then

!           --- No surface numbers found

               call errsub ( 407, 0, 0, 0 )

            else
               iposut = nint(table(9))
               ifirst = nint(tabout(iposut))
               if ( inum>1 ) ilast = nint(tabout(iposut+1))
            end if
            if ( ilast==0) ilast = ifirst
            if ( ilast<ifirst ) then
               call errint ( ifirst, 1 )
               call errint ( nvolms, 2 )
               call errsub ( 296, 2, 0, 0 )
            end if
            if ( ilast<ifirst ) then
               call errint ( ilast, 1 )
               call errint ( ifirst, 2 )
               call errint ( nvolms, 3 )
               call errsub ( 297, 3, 0, 0 )
            end if
            iinputlc(iinplc+1) = 3
            iinputlc(iinplc+2) = ifirst
            iinputlc(iinplc+3) = ilast
            iinplc = iinplc+3
            call errsub ( 2002, 0, 0, 0 )

         else if ( table(5)>0d0 ) then

!        --- all found

            numglall = numglall + 1
            iinputlc(iinplc+1) = 4
            iinplc = iinplc+1
            call errsub ( 2002, 0, 0, 0 )

         end if

         go to 300

      end if

      if ( iinplc>maxlen ) then

!     --- Too much information, i.e. maxlen too short

         call errchr ( 'iinputlc', 1 )
         call errint ( iinplc, 1 )
         call errint ( maxlen, 2 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

!     --- Copy all input in the correct sequence into iinput

      istcrv = iinp
      istsrf = istcrv + 4*numglcrv
      istvlm = istsrf + 3*numglsrf
      istall = istvlm + 3*numglvlm
      iinp   = istall + numglall
      jinp = 0

400   if ( jinp<iinplc ) then

!     --- The last record has not yet been stored

         iform = iinputlc(jinp+2)
         if ( iform==1 ) then

!        --- Curve record found, 4 positions

            iinput(istcrv+1) = iinputlc(jinp+1)
            do i = 1, 3
               iinput(istcrv+1+i) = iinputlc(jinp+2+i)
            end do
            istcrv = istcrv+4
            jinp = jinp+5

         else if ( iform==2 ) then

!        --- Surface record found, 3 positions

            iinput(istsrf+1) = iinputlc(jinp+1)
            iinput(istsrf+2) = iinputlc(jinp+3)
            iinput(istsrf+3) = iinputlc(jinp+4)
            istsrf = istsrf+3
            jinp = jinp+4

         else if ( iform==3 ) then

!        --- Volume record found, 3 positions

            iinput(istvlm+1) = iinputlc(jinp+1)
            iinput(istvlm+2) = iinputlc(jinp+3)
            iinput(istvlm+3) = iinputlc(jinp+4)
            istvlm = istvlm+3
            jinp = jinp+4

         else if ( iform==4 ) then

!        --- All found, 1 position

            iinput(istall+1) = iinputlc(jinp+1)
            istall = istall+3
            jinp = jinp+2

         end if

         go to 400

      end if

      if ( iinp>=maxlen ) then

!     --- iinp>maxlen, too few space in IINPUT

         call errint ( iinp, 1 )
         call errint ( maxlen, 2 )
         call errsub ( 29, 2, 0, 0 )
         call errcrd

      end if
      iinput(30) = numglcrv
      iinput(31) = numglsrf
      iinput(32) = numglvlm
      iinput(33) = numglall

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'prord8' )

      end
