      subroutine prordfictelm ( iinput, maxlen, iinp, iinputlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 27-09-2010 Remove work arrays
!        version  1.2  date 30-03-2007 Correction error message
!        version  1.1  date 08-12-2000 Extension with obstacles
!        version  1.0  date 07-02-2000
!
!   copyright (c) 2000-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   read cards for fictitious elements
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     fictitious_domain
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), maxlen, iinp, iinputlc(*)

!     iinp          i/o   Last position used in IINPUT
!     iinput        i/o   integer array to be filled with information concerning
!                         the fictitious elements
!     iinputlc            Work array in which the input read is stored before
!                         copying in iinput
!                         Per record read the following information is stored:
!                         fictitious element group number, followed by:
!                         Type of record:
!                            1: CURVES
!                            2: SURFACES
!                         Followed by INUM = number of integers to follow, &
!                         followed by the integers itself
!     maxlen         i    Maximum amount of space for array iinput
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: ncodsb=3, nncode1=11
      integer itype1(nncode1), ncode1(nncode1), ncodmx1(nncode1), &
              nscods(ncodsb)
      double precision :: tabmin1(nncode1), tabmax1(nncode1)
      character (len=19) codesb(ncodsb), code1(nncode1)
      integer inum, iinplc, nfictgrp, icurnr, inodp, isurnr, &
              ifictgrp, ifirst, ilast, istart, nnodes, &
              i, jinp, numfictcrv, numfictsrf, istrucgroup, &
              iform, istcrv, istsrf, ifluid1, ifluid2, iobst, &
              numfictobst, istobst
      integer, allocatable :: iwork(:)

!     code1          Table with keywords
!     codesb         Table with subkeywords
!     i              Counting variable
!     icurnr         Curve sequence number
!     ifictgrp       fictitious element group number
!     ifirst         First curve (surface) number read
!     ifluid1        First element group number with respect to fluid elements
!     ifluid2        Last element group number with respect to fluid elements
!     iform          Shape of element
!     iinplc         Last position used in array iinputlc
!     ilast          Last curve (surface) number read
!     inodp          Nodal point number
!     inum           Number of numbers read in a record
!     iobst          Obstacle sequence number
!     istart         First position used at a part
!     istcrv         Starting address of curves in iinput
!     istobst        Starting address of obstacles in iinput
!     istrucgroup    Element group number with respect to structural elements
!     istsrf         Starting address of surfaces in iinput
!     isurnr         Surface sequence number
!     itype1         Indicates the type of action corresponding to keywords
!                    in code1
!     iwork          work array of length nfictgrp
!                    iwork(i) = 0, means group i has not been treated yet
!     jinp           Help variable to store the value of iinp
!     ncode1         Table containing the number of significant letters in
!                    code1
!     ncodmx1        Table containing the maximal number of  characters in
!                    the keyword table code1
!     ncodsb         Number of entries in codesb
!     nfictgrp       number of element groups corresponding to fictitious
!                    unknowns
!     nncode1        Number of entries in code1
!     nnodes         Number of nodes
!     nscods         Number of sub keywords
!     numfictcrv     number of parts where curve elements are defined
!                    for fictitious elements
!     numfictobst    number of parts where obstacle elements are defined
!     numfictsrf     number of parts where surface elements are defined
!                    for fictitious elements
!     tabmax1        Contains maximum values allowed in table for code1
!     tabmin1        Contains minimum values allowed in table for code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    input is read from the input file.
!    The following texts are recognized:
!       felm2 = curves ( c1 to  c2 ), multiplier_shape = i, &
!               structure_group = j, fluid_groups = i1 to i2
!       felm3 = surfaces ( s1 to s2 ), multiplier_shape = i, &
!               structure_group = j, fluid_groups = i1 to i2
!       felmi = obstacle j, type_exclude = k
!    main keywords ( see prorea )
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      29   Too few space for standard problems in IINPUT
!     152   last curve number has incorrect value
!     155   first surface number has incorrect value
!     156   last surface number has incorrect value
!     334   record with curves contains wrong number of curve numbers
!     407   record with surfaces contains wrong number of surface numbers
!     550   array length too short
!     749   fictitious element group number incorrect
!    1615   No structure_group part found
!    1943   More than one position information for one global element group
!    2004   Not all nodes along curve correspond to nodal points
!    2005   Not all nodes along surface correspond to nodal points
!    2144   No fluid_groups found
!    2215   obstacles and curves or surfaces mutually exclusive for fictitious
!           elements
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Read all input until main keyword is found, store in iinputlc
!    Copy all input in the correct sequence into iinput
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'FELM', 'CURVES', 'SURFACES', 'OBSTACLE', &
                   'MULTIPLIER_SHAPE', 'STRUCTURE_GROUP', &
                   'FLUID_GROUPS', 'TYPE_EXCLUDE', 'C', 'S', 'TO' /
      data ncode1  / 3,  5,  6,  5,  5,    5,  5,  7,  1,  1, &
                     2 /
      data ncodmx1 / 4,  6,  8,  8, 16,   15, 12, 12,  1,  1, &
                     2 /
      data itype1  / 0,  2,  2,  0,  0,    0, 14,  0, 14, 14, &
                    -1 /
      data tabmin1 / 1d0, 0d0, 0d0, 1d0, 1d0,   1d0, 2d0, 1d0, 2d0, 2d0, &
                     0d0 /
      data tabmax1 / 1d3, 0d0, 0d0, 0d0, 1d1,   1d3, 0d0, 0d0, 0d0, 0d0, &
                     0d0 /
      data codesb / 'FICTITIOUS_ELEMENTS', 'FICTGRP', 'TYPE' /
      data nscods /  6, 6, 3 /

!     code1    Table of local keywords
!              1: FELM
!              2: CURVES
!              3: SURFACES
!              4: OBSTACLE
!              5: MULTIPLIER_SHAPE
!              6: STRUCTURE_GROUP
!              7: FLUID_GROUPS
!              8: TYPE_EXCLUDE
!              9: C
!             10: S
!             11: TO
!     codesb   Table of local subkeywords
!              1: FICTITIOUS_ELEMENTS
!              2: FICTGRP
!              3: TYPE
! ======================================================================
!
      call eropen ( 'prordfictelm' )

!     --- Copy information into codesub

      call sepcopycodesub ( codesb, ncodsb, nscods )
      call sepcopycode ( code1, nncode1, ncode1, ncodmx1 )
      call sepcopytype ( nncode1, itype1, tabmin1, tabmax1 )
      nmain = nncodemain
      iretur = icode

      ityprd = 1

!     --- Read records for the generation of fictitious elements

      nfictgrp = iinput(36)

      allocate ( iwork(nfictgrp), stat = error )
      if ( error/=0 ) call eralloc ( error, nfictgrp, 'iwork' )
      if ( ierror/=0 ) go to 1000

      numfictcrv = 0
      numfictsrf = 0
      numfictobst = 0
      iinplc = 0
      jinp = iinp
300   table(1:nncode) = -1d20
      iwork = 0
      tabmax(4) = numobst
      tabmax(6) = nelgrp
      tabmax(7) = nelgrp
      tabmax(9) = ncurvs
      tabmax(10) = nsurfs
      table(5) = 1d0
      call sepreadsetkeyw
      icode = iretur
      if ( icode==0 ) then

!     --- Not a main keyword found, hence felm = ...

         if ( table(1)>0 ) then
            ifictgrp = nint(table(1))
         else
            ifictgrp = 0
         end if
         if ( ifictgrp<1 .or. ifictgrp>nfictgrp ) then

!        --- Error fictitious group number out of range

            call errint ( ifictgrp, 1 )
            call errint ( nfictgrp, 2 )
            call errsub ( 749, 2, 0, 0 )

         end if
         if ( iwork(ifictgrp)/=0 ) then

!        --- Group already in use

            call errint ( ifictgrp, 1 )
            call errsub ( 1943, 1, 0, 0 )

         end if
         iwork(ifictgrp) = 1
         iinputlc(iinplc+1) = ifictgrp
         iinplc = iinplc+1
         if ( table(2)>0d0 ) then

!        --- curves card found

            numfictcrv = numfictcrv+1

            ifirst = 0
            ilast = 0
            inum = nint(tabmax(9))
            if ( inum==0 ) then
               call errint ( inum, 1 )
               call errint ( 2, 2 )
               call errsub ( 334, 2, 0, 0 )
            else
               iposut = nint(table(9))
               ifirst = nint(tabout(iposut))
               if ( inum>1 ) ilast = nint(tabout(iposut+1))
            end if
            iinputlc(iinplc+1) = 1
            if ( ilast==0) ilast = ifirst
            if ( ilast<ifirst .or. ilast>ncurvs ) then
               call errint ( ilast, 1 )
               call errint ( ifirst, 2 )
               call errint ( ncurvs, 3 )
               call errsub ( 152, 3, 0, 0 )
            end if

!           --- Check if these curves have been filled

            do icurnr = ifirst, ilast
               istart = kmeshm(icurnr+4)
               nnodes = kmeshm(icurnr+5)-istart
               istart = istart+kmeshm(2)-1
               do i = 1, nnodes
                  inodp = kmeshm(istart+i)
                  if ( inodp<=0 ) then

!                 --- Nodal point has not been filled

                     call errint ( icurnr, 1 )
                     call errsub ( 2004, 1, 0, 0 )
                     go to 310

                  end if
               end do
310         end do

            iinputlc(iinplc+3) = ifirst
            iinputlc(iinplc+4) = ilast

         else if ( table(3)>0d0 ) then

!        --- surface card found

            numfictsrf = numfictsrf+1
            ifirst = 0
            ilast = 0
            inum = nint(tabmax(10))
            if ( inum==0 ) then

!           --- No surface numbers found

               call errsub ( 407, 0, 0, 0 )

            else
               iposut = nint(table(10))
               ifirst = nint(tabout(iposut))
               if ( inum>1 ) ilast = nint(tabout(iposut+1))
            end if
            if ( ilast==0) ilast = ifirst
            if ( ilast<ifirst ) then
               call errint ( ilast, 1 )
               call errint ( ifirst, 2 )
               call errint ( nsurfs, 2 )
               call errsub ( 155, 2, 0, 0 )
            end if
            if ( ilast<ifirst .or. ilast>nsurfs ) then
               call errint ( ilast, 1 )
               call errint ( ifirst, 2 )
               call errint ( nsurfs, 3 )
               call errsub ( 156, 3, 0, 0 )
            end if
            if ( kelmn>0 ) then

!           --- Check if these surfaces have been filled

               istart = 2*nsurfs
               do isurnr = 1, ifirst-1
                  istart = istart + kmeshn(isurnr)*kmeshn(isurnr+nsurfs)
               end do
               do isurnr = ifirst, ilast
                  nnodes = kmeshn(isurnr)*kmeshn(isurnr+nsurfs)
                  do i = 1, nnodes
                     inodp = kmeshn(istart+i)
                     if ( inodp<=0 ) then

!                    --- Nodal point has not been filled

                        call errint ( isurnr, 1 )
                        call errsub ( 2005, 1, 0, 0 )
                        go to 320

                     end if
                  end do
320            end do
            end if
            iinputlc(iinplc+1) = 2
            iinputlc(iinplc+3) = ifirst
            iinputlc(iinplc+4) = ilast

         else if ( table(4)>0d0 ) then

!        --- obstacle card found

            numfictobst = numfictobst+1
            iobst = nint(table(4))
            if ( table(8)>0d0 ) then
               iform = nint(table(8))
            else
               iform = 1
            end if
            iinputlc(iinplc+1) = 3
            iinputlc(iinplc+3) = iobst
            iinputlc(iinplc+4) = iform
            iinplc = iinplc+4

         end if
         if ( table(2)>0d0 .or. table(3)>0d0 ) then

!        --- curves or surfaces found, now also the part multiplier_shape, &
!            structure_group and fluid_groups

            ishape = nint(table(5))
            if ( table(6)<=0d0 ) then

!           --- No structure group found

               call errint ( ifictgrp, 1 )
               call errsub ( 1615, 1, 0, 0 )
               istrucgroup = 1

            else

!           --- Structure group found

               istrucgroup = nint(table(6))

            end if
            if ( table(7)<=0d0 ) then

!           --- No fluid group found

               call errint ( ifictgrp, 1 )
               call errsub ( 2144, 1, 0, 0 )
               ifluid1 = 1
               ifluid2 = 1

            else

!           --- Fluid group found

               iposut = nint(table(7))
               inum   = nint(tabmax(7))
               ifluid1 = nint(tabout(iposut))
               if ( inum>1 ) then
                  ifluid2 = nint(tabout(iposut+1))
               else
                  ifluid2 = ifluid1
               end if

            end if
            iinputlc(iinplc+2) = ishape
            iinputlc(iinplc+5) = istrucgroup
            iinputlc(iinplc+6) = ifluid1
            iinputlc(iinplc+7) = ifluid2
            iinplc = iinplc+7

         end if

         go to 300

      end if

      if ( iinplc>maxlen ) then

!     --- Too much information, i.e. maxlen too short

         call errchr ( 'iinputlc', 1 )
         call errint ( iinplc, 1 )
         call errint ( maxlen, 2 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

!     --- Copy all input in the correct sequence into iinput

      istcrv = iinp
      istsrf = istcrv + 7*numfictcrv
      istobst = istsrf + 7*numfictsrf
      iinp   = istobst + 4*numfictobst
      jinp = 0

400   if ( jinp<iinplc ) then

!     --- The last record has not yet been stored

         iform = iinputlc(jinp+2)
         if ( iform==1 ) then

!        --- Curve record found, 7 positions

            iinput(istcrv+1) = iinputlc(jinp+1)
            do i = 1, 6
               iinput(istcrv+1+i) = iinputlc(jinp+2+i)
            end do
            istcrv = istcrv+7
            jinp = jinp+8

         else if ( iform==2 ) then

!        --- Surface record found, 7 positions

            iinput(istsrf+1) = iinputlc(jinp+1)
            do i = 1, 6
               iinput(istsrf+1+i) = iinputlc(jinp+2+i)
            end do
            istsrf = istsrf+7
            jinp = jinp+8

         else if ( iform==3 ) then

!        --- Obstacle record found, 4 positions

            iinput(istobst+1) = iinputlc(jinp+1)
            do i = 1, 3
               iinput(istobst+1+i) = iinputlc(jinp+2+i)
            end do
            istobst = istobst+4
            jinp = jinp+5

         end if

         go to 400

      end if

      if ( iinp>=maxlen ) then

!     --- iinp>maxlen, too few space in IINPUT

         call errint ( iinp, 1 )
         call errint ( maxlen, 2 )
         call errsub ( 29, 2, 0, 0 )
         call errcrd

      end if
      iinput(37) = numfictcrv
      iinput(38) = numfictsrf
      iinput(40) = numfictobst
      if ( numfictobst>0 .and. numfictcrv+numfictsrf>0 ) then

!     --- obstacles and curves or surfaces are mutually exclusive at this
!         moment

         call errsub ( 2215, 0, 0, 0 )

      end if

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'prordfictelm' )

      end
