      subroutine prordnatboun ( iinput, iinp, nelgrp, iinputtypes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 12-01-2015 Different types for Reynolds
!        version  1.0  date 03-11-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill type numbers for natural boundary conditions into array iinput
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: iinput(*), iinp
      integer, intent(in) :: nelgrp, iinputtypes(*)

!     iinp          i/o   last position used in array iinput
!     iinput        i/o   integer input array
!                         See subroutine prorea
!                         In this part the type numbers of the natural
!                         boundary conditions are filled
!     iinputtypes    i    Contains type numbers for internal elements
!                         For each element group 20 numbers
!     nelgrp         i    Number of element groups
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ntype, jinp, itype, ielgrp, jtype, k, numnat, icount, &
                 i, numnatbpnt, ktype

!     i              Counting variable
!     icount         Counter
!     ielgrp         element group number
!     itype          Type number
!     jinp           last position used in array iinput
!     jtype          Type number
!     k              Counting variable
!     ktype          Type number
!     ntype          Number of different type numbers to be used for one
!                    group
!     numnat         Number of different boundary groups
!     numnatbpnt     Number of different point boundary groups
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2544   boundary elements for points and curves not yet implemented
!    3204   BOUNELEMENTS found before NATURAL_BOUNDARY_CONDITIONS but
!           not all element types are the same or they are less than 100
!    3205   More than 1 type per element group not implemented
!    3206   Type number not yet implemented in this subroutine
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check if all type numbers are the same and larger than 99
!     Fill information in iinput
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prordnatboun' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prordnatboun'
         call prinin ( iinput, 20, 'iinput' )
         call prinin ( iinputtypes, 20, 'iinputtypes' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check if all type numbers are the same and larger than 99
!         Store type number in itype

      ntype = iinput(1)  ! number of different types
      if ( debug ) write(irefwr,1) 'ntype', ntype

      if ( ntype>1 ) then

!     --- ntype>1, has not been implemented in this subroutine

         call errint ( ntype, 1 )
         call errsub ( 3205, 1, 0, 0 )
         go to 1000

      end if  ! ( ntype>1 )

      jinp = 0   ! position in iinputtypes
      itype = 0  ! type number

      do ielgrp = 1, nelgrp

         ktype = 0
         do k = 1, ntype
            jtype = iinputtypes(jinp+k)
            if ( ktype==0 ) then

!           --- ktype = 0, hence has not been initialized

               ktype = max(jtype,0)

            else if ( jtype<0 ) then

!           --- jtype<0, skip

            else if ( ktype/=jtype .or. jtype<100 .and. jtype/=0 ) then

!           --- Error: jtype#itype or jtype<100

               call errint ( jtype, 1 )
               call errint ( ktype, 2 )
               call errsub ( 3204, 2, 0, 0 )
               go to 1000

            end if  ! ( ktype==0 )

         end do  ! k = 1, ntype
         jinp = jinp+20

         if ( itype==0 ) then

!        --- itype = 0, initialize

            itype = max(itype,ktype)

         else if ( ktype>0 .and. itype/=ktype ) then

!        --- itype # 0, and ktype has different value: error

            call errint ( itype, 1 )
            call errint ( ktype, 2 )
            call errsub ( 3204, 2, 0, 0 )
            go to 1000

         end if  ! ( itype==0 )

         if ( debug ) &
            write(irefwr,1) 'ielgrp, itype, jtype', ielgrp, itype, jtype

         if ( jtype==-1 ) then

!        --- Special case, more input possible

            icount = iinput(jinp+1)
            jinp = jinp+2*icount+1

         end if  ! ( jtype==-1 )

      end do  ! ielgrp = 1, nelgrp

!     --- Find type number of boundary element corresponding to itype

      numnat = iinput(2)      ! number of boundary groups
      numnatbpnt = iinput(3)  ! number of point boundary groups

      select case ( itype )

         case ( 700:703, 705, 800, 803 )
            itype = 801
         case ( 250 )
            itype = 251
         case(325)

            if ( numnatbpnt==numnat ) then

!           --- All boundary elements are point elements

               itype = 304

            else if ( numnatbpnt==0 ) then

!           --- These are no point elements

               itype = 801

            else

!           --- Mixture of point and curve elements (not yet implemented)

               call errsub ( 2544, 0, 0, 0 )
               go to 1000

            end if  ! ( numnatbpnt==numnat )

         case default

            call errint ( itype, 1 )
            call errsub ( 3206, 1, 0, 0 )

      end select  ! case ( itype )

!     --- Fill information in iinput

      do i = 1, numnat
         iinput(iinp+1:iinp+20) = itype
         iinp = iinp+20
      end do  ! i = 1, numnat

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prordnatboun'

      end if  ! ( debug )
      call erclos ( 'prordnatboun' )

      end subroutine prordnatboun
