      subroutine prordperbcfin ( iinput, iinp, numpoints, &
                                 numcurves, numsurfs, iinputpnt, &
                                 iinputcrv, iinputsrf, iinputdeg, &
                                 icountdeg, maxlen, rinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 14-01-2013 correction surfaces
!        version  1.0  date 09-06-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Rearrange input read in sub arrays for periodical boundary conditions
!     and store in array iinput
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     periodical
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, numpoints, numcurves, numsurfs, &
              iinputpnt(2,*), iinputcrv(3,*), iinputsrf(*), &
              iinputdeg(5,*), icountdeg, maxlen
      double precision rinput(*)

!     icountdeg      i    number of parts stored in iinputdeg
!     iinp          i/o   Last position used in IINPUT
!     iinput        i/o   integer array to be filled with information concerning
!                         periodical boundary conditions
!                         See subroutine prordperbc for the details
!     iinputcrv      i    Array of size 3 x maxcurves in which the input read
!                         is stored
!                         See subroutine prordperbc for the details
!     iinputdeg      i    Array of size 5 x maxinpdeg in which the input read
!                         is stored
!                         See subroutine prordperbc for the details
!     iinputpnt      i    Array of size 2 x maxpoints in which the input read
!                         is stored
!                         See subroutine prordperbc for the details
!     iinputsrf      i    Array of size lensurfs in which the input read
!                         is stored
!                         See subroutine prordperbc for the details
!     maxlen         i    Maximum amount of contiguous space in IBUFFR after a
!                         starting address
!     numcurves      i    Number of curves
!     numpoints      i    Number of points
!     numsurfs       i    number of surfaces
!     rinput         i    Real input array
!                         Filling depends on iinput
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jinp, istsrf, i, irepeat, istart, j, numexcl, iinpsav

!     i              Counting variable
!     iinpsav        Help variable to save the value of iinpx
!     irepeat        Number of times input for degrees of freedom is repeated
!     istart         Starting address in array iinput
!     istsrf         Starting address of surfaces in iinputsrf
!     j              Counting variable
!     jinp           last position used in array iinput
!     numexcl        number of curves to be excluded
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINPROPERSUR  Print the contents of array iinputsrf for periodical
!                    boundary conditions
!     PRINPROPRERINP print the contents of array iinput for periodical
!                    boundary conditions
!     PRORDPERDEG1   Rearrange array iinputdeg into a natural sequence
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prordperbcfin' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prordperbcfin'
         write(irefwr,1) 'numpoints, numcurves, numsurfs', &
                          numpoints, numcurves, numsurfs
         call prinin1 ( iinputpnt, numpoints, 2, 'points' )
         call prinin1 ( iinputcrv, numcurves, 3, 'curves' )
         call prinpropersur ( iinputsrf, numsurfs )
         call prinin1 ( iinputdeg, icountdeg, 5, 'iinputdeg' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Rearrange array iinputdeg in the sequence points, curves, surfaces

      call prordperdeg1 ( icountdeg, iinputdeg )

!     --- Fill array iinput (first 3 positions number of points curves etc)

      iinput(iinp+2) = numpoints
      iinput(iinp+3) = numcurves
      iinput(iinp+4) = numsurfs

      jinp = iinp+numpoints+numcurves+numsurfs+4  ! starting address information
      iinpsav = iinp
      istart = iinp+5
      call inichkarlen ( jinp, maxlen, 'iinput' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,1) 'iinp, jinp, istart', iinp, jinp, istart
      end if  ! ( debug )

!     --- Information of point pairs

      icountdeg = 0
      do i = 1, numpoints

         iinput(istart) = jinp-iinpsav-1  ! starting address of point pair i
         istart = istart+1    ! raise istart

         call inichkarlen ( jinp+3, maxlen, 'iinput' )
         if ( ierror/=0 ) go to 1000

         irepeat = iinputdeg(1,icountdeg+1)
         iinput(jinp+1:jinp+2) = iinputpnt(1:2,i)  ! ifirst and ilast
         iinput(jinp+3) = irepeat  ! number of parts with information
         jinp = jinp+3

!        --- Copy information of degrees of freedom

         do j = 1, irepeat

            call inichkarlen ( jinp+3, maxlen, 'iinput' )
            if ( ierror/=0 ) go to 1000

            iinput(jinp+1:jinp+3) = iinputdeg(2:4,icountdeg+j)
            jinp = jinp+3

         end do  ! j = 1, irepeat
         icountdeg = icountdeg+irepeat    ! raise icountdeg

      end do  ! i = 1, numpoints

!     --- Information of curve pairs

      do i = 1, numcurves

         iinput(istart) = jinp-iinpsav  ! starting address of curve pair i
         istart = istart+1    ! raise istart

         if ( debug ) &
             write(irefwr,1) 'i, istart, icountdeg, jinp', &
                              i, istart, icountdeg, jinp
         irepeat = iinputdeg(1,icountdeg+1)
         iinput(jinp+1:jinp+3) = iinputcrv(1:3,i)  ! ifirst, ilast, j
         iinput(jinp+4) = irepeat  ! number of parts with information
         jinp = jinp+4

         if ( debug ) write(irefwr,1) 'irepeat, jinp', irepeat, jinp

         call inichkarlen ( jinp+4, maxlen, 'iinput' )
         if ( ierror/=0 ) go to 1000

!        --- Copy information of degrees of freedom

         do j = 1, irepeat

            call inichkarlen ( jinp+3, maxlen, 'iinput' )
            if ( ierror/=0 ) go to 1000

            iinput(jinp+1:jinp+3) = iinputdeg(2:4,icountdeg+j)
            jinp = jinp+3

         end do  ! j = 1, irepeat
         icountdeg = icountdeg+irepeat    ! raise icountdeg

      end do  ! i = 1, numcurves

!     --- Information of surface pairs

      istsrf = 0
      do i = 1, numsurfs

         iinput(istart) = jinp-iinpsav  ! starting address of surface pair i
         istart = istart+1    ! raise istart

         if ( debug ) write(irefwr,1) 'istart, jinp', istart, jinp

         irepeat = iinputdeg(1,icountdeg+1)
         iinput(jinp+1:jinp+3) = iinputsrf(istsrf+1:istsrf+3)  ! ifirst, ilast
         numexcl = iinputsrf(istsrf+3)   ! number of curves to be excluded

         if ( debug ) call prinin ( iinput(jinp+1), 3, 'iinputsrf' )

         istsrf = istsrf+3
         jinp = jinp+3

         call inichkarlen ( jinp+3+numexcl, maxlen, 'iinput' )
         if ( ierror/=0 ) go to 1000

         if ( numexcl>0 ) then

!        --- There are numexcl curves to bee excluded

            iinput(jinp+1:jinp+numexcl) = &
               iinputsrf(istsrf+1:istsrf+numexcl)
            jinp = jinp+numexcl

         end if  ! ( numexcl>0 )

         iinput(jinp+1) = irepeat  ! number of parts with information
         jinp = jinp+1

!        --- Copy information of degrees of freedom

         do j = 1, irepeat

            call inichkarlen ( jinp+3, maxlen, 'iinput' )
            if ( ierror/=0 ) go to 1000

            iinput(jinp+1:jinp+3) = iinputdeg(2:4,icountdeg+j)
            jinp = jinp+3

         end do  ! j = 1, irepeat
         icountdeg = icountdeg+irepeat    ! raise icountdeg

      end do  ! i = 1, numsurfs
      iinput(iinp+1) = jinp-iinp  ! length of part

1000  call erclos ( 'prordperbcfin' )
      if ( debug ) then

!     --- Debug information

         call prinin ( iinput(iinp+1), jinp-iinp, 'iinput' )
         call prinproprerinp ( iinput(iinp+1), rinput, istart )
         write(irefwr,*) 'End prordperbcfin'

      end if  ! ( debug )

      iinp = jinp  ! raise iinp

      end
