      subroutine prordperbcsrf ( table, code, ncodmx, tabmax, tabout, &
                                 numsurfs, lensurfs, iinputsrf, &
                                 iinpsrf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 14-01-2013 correction surfaces
!        version  1.0  date 06-06-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret records for definition of periodical boundary conditions
!     Part surfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     periodical
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numsurfs, ncodmx(*), lensurfs, iinputsrf(lensurfs), &
              iinpsrf
      double precision table(*), tabmax(*), tabout(*)
      character (len=*) code(*)

!     code           i    Table with keywords, see subroutine prordperbc
!     iinpsrf       i/o   Last position used in iinputsrf
!     iinputsrf     i/o   Array of size lensurfs in which the input read
!                         is stored
!                         See subroutine prordperbc for a description
!     lensurfs       i    Length of array iinputsrf
!     ncodmx         i    Table containing the maximal number of  characters in
!                         the keyword table code
!     numsurfs       i    number of pairs of surfaces stored in iinputsrf
!     table          i    Contains the numbers read for the various keywords
!     tabmax         i    Contains maximum values allowed in table
!     tabout         i    Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inum, iposut, ifirst, ilast

!     ifirst         First surface number read
!     ilast          Last surface number read
!     inum           Number of numbers read in a record
!     iposut         Position in array tabout
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     317   record with pelm contains more than two user points
!    1432   User point connected to itself
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prordperbcsrf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prordperbcsrf'
         write(irefwr,1) 'iinpsrf', iinpsrf
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      inum = nint(tabmax(3))  ! number of user surfaces read (must be 2)
      if ( inum/=2 ) then

!     --- Error, not two user surfs read

         call errint ( inum, 1 )
         call errsub ( 317, 1, 0, 0 )
         call errcrd
         go to 1000

      end if

!     --- First check if keywords are not mutually exclusive

      if ( table(5)>0d0 .and. table(9)>0d0 ) &
         call errkeywexcl ( code, ncodmx, 5, 9 ) ! C and exclude

      numsurfs = numsurfs+1
      call inichkarlen ( iinpsrf+4, lensurfs, 'iinputsrf' ) ! check array len
      if ( ierror/=0 ) go to 1000

      iposut = nint(table(3))  ! position of user surfaces in tabout
      ifirst = nint(tabout(iposut))  ! first user surface
      ilast = nint(tabout(iposut+1)) ! last user surface
      if ( ilast==ifirst ) then

!     --- error, both user surfs are identical

         call errint ( ilast, 1 )
         call errsub ( 1432, 1, 0, 0 )
         call errcrd

      end if

!     --- Store surfs in iinputsrf

      iinputsrf(iinpsrf+1) = ifirst           ! first surface number
      iinputsrf(iinpsrf+2) = ilast            ! last surface number
      iinputsrf(iinpsrf+3) = 0                ! no exclusion
      inum = 0
      if ( table(5)>0d0 ) then

!     --- exclude all found, i.e. all curves

         iinputsrf(iinpsrf+3) = -1            ! exlude all curves

      else if ( table(9)>0d0 ) then

!     --- exclude a set of curves

         inum = nint(tabmax(9))            ! number of curves
         call inichkarlen ( iinpsrf+3+inum, lensurfs, 'iinputsrf' )
         if ( ierror/=0 ) go to 1000

         iposut = nint(table(9))
         iinputsrf(iinpsrf+3) = inum
         iinputsrf(iinpsrf+4:iinpsrf+3+inum) = &
            nint(tabout(iposut:iposut+inum-1))

      end if  ! ( table(5)>0d0 )

      if ( debug ) &
         call prinin ( iinputsrf(iinpsrf+1), 3+inum, 'iinputsrf' )
      iinpsrf = iinpsrf+3+inum

1000  call erclos ( 'prordperbcsrf' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iinpsrf', iinpsrf
         write(irefwr,*) 'End prordperbcsrf'

      end if  ! ( debug )

      end
