      subroutine prordperdegfd ( icountdeg, iinputdeg, itypepos, &
                                 numpoints, numcurves, numsurfs, &
                                 maxinpdeg, table, tabmax, tabout, &
                                 lengrl, rinput, maxrl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 14-01-2013 correction factor
!        version  1.0  date 09-06-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret records for definition of periodical boundary conditions
!     Part degfdi and factors
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     periodical
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icountdeg, maxinpdeg, itypepos, numpoints, numcurves, &
              numsurfs, iinputdeg(5,maxinpdeg), lengrl, maxrl
      double precision table(*), tabmax(*), tabout(*), rinput(*)

!     icountdeg     i/o   number of parts stored in iinputdeg
!     iinputdeg     i/o   Array of size 5 x maxinpdeg in which the input read
!                         is stored
!                         See subroutine prordperbc for a description
!     itypepos       i    Type of last item that has been read
!                         1: user points
!                         2: curves
!                         3: surfaces
!     lengrl        i/o   Number of items stored in rinput
!     maxinpdeg      i    Size of array iinputdeg
!     maxrl          i    Maximum number of reals that are allowed in rinput
!     numcurves      i    number of curve pairs that have been read
!     numpoints      i    Number of point pairs that have been read
!     numsurfs       i    number of surface pairs that have been read
!     rinput        i/o   Real input array
!     table          i    Contains the numbers read for the various keywords
!     tabmax         i    Contains maximum values allowed in table
!     tabout         i    Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inum, iposut, ihelp(20), inunkp, i, ihelp1(20)

!     i              Counting variable
!     ihelp          Work array of length 20 to check if a degree of freedom is
!                    already in use in a point
!                    ihelp(i) = 0, degree of freedom i is not in use
!                    ihelp(i) = 1, degree of freedom i is in use
!     ihelp1         Work array of length 20 to store the degrees of freedom
!     inum           Number of numbers read in a record
!     inunkp         Defines degrees of freedom in bit pattern
!     iposut         Position in array tabout
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inibit

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INIBIT         Translate degrees of freedom into bit pattern
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ1        print 2d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
!     The part that is interpreted concerns
!          constant=c1, factor=c2, degfdi, degfdj , ... (may be repeated)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     839   Degree of freedom has incorrect value
!    2028   Degree of freedom found more than once
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prordperdegfd' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prordperdegfd'
         write(irefwr,1) 'icountdeg, itypepos, lengrl', &
                          icountdeg, itypepos, lengrl
         write(irefwr,2) 'table(6:8)', table(6:8)
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icountdeg = icountdeg+1
      call inichkarlen ( icountdeg, maxinpdeg, 'iinputdeg' )
      if ( ierror/=0 ) go to 1000

      iinputdeg(1,icountdeg) = itypepos

      select case (itypepos )

      case(1)

!     --- Point record

         iinputdeg(2,icountdeg) = numpoints

      case(2)

!     --- Curve record

         iinputdeg(2,icountdeg) = numcurves

      case(3)

!     --- Surface record

         iinputdeg(2,icountdeg) = numsurfs

      end select  ! case (itypepos )

      iinputdeg(3:5,icountdeg) = 0  ! default values

      if ( table(8)>0d0 ) then

!     --- table(8)>0, hence degfd found

         inum = nint(tabmax(8))  ! number of degrees of freedom
         iposut = nint(table(8)) ! position in tabout
         ihelp = 0
         do i = 1, inum
            inunkp = nint(tabout(iposut+i-1))
            if ( inunkp<1 .or. inunkp>20 ) then

!           --- i-th degree of freedom out of range

               call errint ( inunkp, 1 )
               call errint ( 20, 2 )
               call errsub ( 839, 2, 0, 0 )
               call errcrd
               inunkp = 20

            end if  ! ( inunkp<1 .or. inunkp>20 )

            if ( ihelp(inunkp)==0 ) then

!           --- Degree of freedom is new

               ihelp1(i) = inunkp

            else

!           --- Degree of freedom has been found before

               call errint ( inunkp, 1 )
               call errwar ( 2028, 1, 0, 0 )
               call errcrd

            end if  ! ( ihelp(inunkp)==0 )

         end do  ! i = 1, inum
         inunkp = inibit ( ihelp1, inum )
         iinputdeg(3,icountdeg) = inunkp  ! bit pattern

      end if  ! ( table(8)>0d0 )

!     --- Constant and factor

      do i = 0, 1

        if ( table(6+i)>-1d19 ) then

!       --- constant or factor found

           lengrl = lengrl+1
           call inichkarlen ( lengrl, maxrl, 'rinput' )
           if ( ierror/=0 ) go to 1000

           rinput(lengrl) = table(6+i)
           iinputdeg(4+i,icountdeg) = lengrl  ! address of constant or factor

        end if  ! ( table(6+i)>-1d20 )

      end do  ! i = 0, 1

1000  call erclos ( 'prordperdegfd' )
      if ( debug ) then

!     --- Debug information

         call prinin1 ( iinputdeg, icountdeg, 5, 'iinputdeg' )
         call prinrl ( rinput, lengrl, 'rinput' )
         write(irefwr,*) 'End prordperdegfd'

      end if  ! ( debug )

      end
