      subroutine prordr ( iinput, iwork, iseq, iinp, nprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.15 date 03-11-2014 Extra debug
!        version  2.14 date 13-10-2013 Extension with leniinpfp
!        version  2.13 date 06-06-2012 Extension with periodical boundary conds
!
!   copyright (c) 1986-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    rearrange array iinput
!    fill first positions
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     fictitious_domain
!     global_unknown
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iwork(*), iseq(15,2), iinp, nprob

!     iinp           o    Last position used in IINPUT
!     iinput        i/o   integer input array
!     iseq           i    Array containing the start and end addresses of array
!                         iinput for the various parts in the sequence:
!                          1:  type numbers for internal elements
!                          2:  type numbers for boundary elements
!                          3:  information about boundary elements
!                          4:  information about essential boundary conditions
!                          5:  information about local transforms
!                          6:  information about unknown constants
!                          7:  information about renumbering
!                          8:  type numbers for global elements
!                          9:  information about global elements
!                         10:  information about global elements renumbering
!                         11:  type numbers for fictitious elements
!                         12:  information about fictitious elements
!                         13:  information about elements to be skipped
!                         14:  information with respect to level set
!     iwork               integer work array in which iinput is copied
!                         temporarily
!     nprob          i    number of different problems
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, ielgrp, icheck, indtvc, iphys, ncount, iextra, nglgrp, &
              jinp, itype, inunkp, ntype, numnat, inpelm, &
              nfictgrp, icount, leniinpfp

!     icheck         Indication if all type numbers are smaller than 100
!                    (0) or  not (1)
!     icount         Counter
!     ielgrp         element group number
!     iextra         Number of extra vectors of special structure
!     indtvc         Number of vectors of special structure
!     inpelm         Number of nodes in element group
!     inunkp         Number of degrees of freedom in point
!     iphys          Number of physical variables
!     itype          Type number of element
!     jinp           Help variable to get a position in array IINPUT
!     k              Counting variable
!     leniinpfp      Length of first part of iinput for probdf
!     ncount         Counter
!     nfictgrp       number of element groups corresponding to fictitious
!                    unknowns
!     nglgrp         number of element groups corresponding to global
!                    unknowns
!     ntype          Number of different type numbers to be used for one
!                    group
!     numnat         Number of different boundary elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRORDRLEV      Reorganize contents of level set part of iinput for probdf
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     161   No record with types found
!     697   Next record is not of the type BOUNELEMENTS
!    2003   No input for global elements found, whereas there are global
!           element groups
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Fill iinput positions 1 to 5
!   Copy all (reduced) information into iwork in the correct sequence:
!     First positions corresponding to type numbers (at most ntype)
!     next positions corresponding to boundary type numbers (at most ntype)
!     next positions corresponding to boundary elements
!     next positions corresponding to essential boundary conditions
!     next positions corresponding to local transforms
!     next positions corresponding to unknown constants
!     next positions corresponding to renumbering
!     next positions corresponding to skip elements
!     next positions corresponding to level set
!   Copy iwork back to iinput
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prordr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prordr'
         call prinin ( iinput, iinp, 'iinput/in')
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill iinput positions 1 to 5

      ntype    = iinput(1)
      numnat   = iinput(2)
      nglgrp   = iinput(29)
      nfictgrp = iinput(36)
      leniinpfp = 60
      inpelm = npelm

!     --- Set version number

      iinput(15) = 10  ! ivers

!     --- Copy all (reduced) information into iwork in the correct sequence

      iinp = 0
      if ( iseq(1,1)==0 ) then
         call errint ( nprob, 1 )
         call errsub ( 161, 1, 0, 0 )
      end if
      jinp = iseq(1,1)
      do ielgrp = 1, nelgrp
         do k = 1, ntype
            iwork(iinp+k) = iinput(jinp+k)
         end do
         icheck = 0
         iinp = iinp+ntype
         do k = 1, ntype
            itype = iinput(jinp+k)
            if ( itype>=100 ) icheck = 1
         end do
         jinp = jinp+20

         if ( itype==-3 ) then

!        --- itype = -3  special boundary condition, do not store anything

         else if ( itype==-1 ) then

!        --- itype = -1  periodical boundary conditions, stored idegfd

            icount =iinput(jinp+1)
            iinp = iinp+1
            jinp = jinp+1
            iwork(iinp) = iinput(jinp)

            do k = 1, icount

!           --- loop over all the options

               iwork(iinp+1) = iinput(jinp+1)
               iwork(iinp+2) = iinput(jinp+2)
               iinp = iinp+2
               jinp = jinp+2

            end do

         else if ( icheck==0 .and. itype/=0 ) then

!        --- icheck  =  0  user elements

            inunkp = iinput(jinp+1)
            indtvc = iinput(jinp+2)
            iphys = iinput(jinp+3)
            ncount = 3
            inpelm = sepgetinpelm ( ielgrp )
            if ( inunkp==0 ) ncount = ncount+inpelm
            ncount = ncount+(indtvc+iphys)*inpelm
            do k = 1, ncount
               iwork(iinp+k) = iinput(jinp+k)
            end do
            iinp = iinp+ncount
            jinp = jinp+ncount

         else if ( itype>10000 ) then

!        --- itype>10000, extra vectors of special structure

            iextra = itype/10000
            do k = 1, 4*iextra+1
               iwork(iinp+k) = iinput(jinp+k)
            end do
            iinp = iinp+4*iextra+1
            jinp = jinp+4*iextra+1

         end if

      end do

!     --- numnat>0  natural boundary conditions

      jinp = iseq(2,1)
      do ielgrp = 1, numnat

         iwork(iinp+1:iinp+ntype) = iinput(jinp+1:jinp+ntype)
         iinp = iinp+ntype
         icheck = 0
         do k = 1, ntype
            itype = iinput(jinp+k)
            if ( itype>=100 ) icheck = 1
         end do
         jinp = jinp+20
         if ( icheck==0 ) then

            inpelm = iinput(jinp+1)
            iwork(iinp+1) = inpelm
            iinp = iinp+1
            jinp = jinp+1
            inunkp = iinput(jinp+1)
            indtvc = iinput(jinp+2)
            iphys = iinput(jinp+3)
            ncount = 3
            if ( inunkp==0 ) ncount = ncount+inpelm
            ncount = ncount+(indtvc+iphys)*inpelm
            do k = 1, ncount
               iwork(iinp+k) = iinput(jinp+k)
            end do
            iinp = iinp+ncount
            jinp = jinp+ncount

         end if

      end do

!     --- Boundary elements

      if ( numnat>0 ) then

!     --- Boundary groups have been found

         if ( iseq(3,1)==0 ) then
            call errint ( nprob, 1 )
            call errsub ( 697, 1, 0, 0 )
         end if
         jinp = iseq(3,1)
         do k = 1, iseq(3,2)-iseq(3,1)
            iwork(iinp+k) = iinput(jinp+k)
         end do

         if ( debug ) then
            write(irefwr,1) 'iinp, numnat', iinp, numnat
            call prinin ( iwork(iinp+1), iseq(3,2)-iseq(3,1), 'iinput' )
         end if  ! ( debug )

         iinp = iinp + iseq(3,2)-iseq(3,1)

      end if

!     --- nglgrp>0  global element groups

      jinp = iseq(8,1)
      do ielgrp = 1, nglgrp

         iwork(iinp+1) = iinput(jinp+1)
         iinp = iinp+1
         jinp = jinp+1
         iwork(iinp+1) = iinput(jinp+1)
         iinp = iinp+1
         jinp = jinp+1

      end do

!     --- global elements

      if ( nglgrp>0 ) then

!     --- global element groups have been found

         if ( iseq(9,1)==0 ) then
            call errint ( nprob, 1 )
            call errsub ( 2003, 1, 0, 0 )
         end if
         jinp = iseq(9,1)
         do k = 1, iseq(9,2)-iseq(9,1)
            iwork(iinp+k) = iinput(jinp+k)
         end do
         iinp = iinp + iseq(9,2)-iseq(9,1)
         if ( iseq(10,1)==0 ) then
            iwork(iinp+1) = 0
            iinp = iinp+1
         else
            jinp = iseq(10,1)
            do k = 1, iseq(10,2)-iseq(10,1)
               iwork(iinp+k) = iinput(jinp+k)
            end do
            iinp = iinp + iseq(10,2)-iseq(10,1)
         end if

      end if

!     --- nfictgrp>0  fictitious element groups

      jinp = iseq(11,1)
      do ielgrp = 1, nfictgrp

         iwork(iinp+1) = iinput(jinp+1)
         iinp = iinp+1
         jinp = jinp+1
         iwork(iinp+1) = iinput(jinp+1)
         iinp = iinp+1
         jinp = jinp+1

      end do

!     --- fictitious elements

      if ( nfictgrp>0 ) then

!     --- fictitious element groups have been found

         if ( iseq(12,1)==0 ) then
            call errint ( nprob, 1 )
            call errsub ( 2003, 1, 0, 0 )
         end if
         jinp = iseq(12,1)
         do k = 1, iseq(12,2)-iseq(9,1)
            iwork(iinp+k) = iinput(jinp+k)
         end do
         iinp = iinp + iseq(12,2)-iseq(12,1)

      end if

!     --- Essential boundary conditions

      jinp = iseq(4,1)
      do k = 1, iseq(4,2)-iseq(4,1)
         iwork(iinp+k) = iinput(jinp+k)
      end do
      iinp = iinp + iseq(4,2)-iseq(4,1)

!     --- Local transforms

      jinp = iseq(5,1)
      do k = 1, iseq(5,2)-iseq(5,1)
         iwork(iinp+k) = iinput(jinp+k)
      end do
      iinp = iinp + iseq(5,2)-iseq(5,1)
      if ( debug ) call prinin ( iwork, iinp, 'iwork' )

!     --- Boundary conditions of type unknown constant

      jinp = iseq(6,1)
      do k = 1, iseq(6,2)-iseq(6,1)
         iwork(iinp+k) = iinput(jinp+k)
      end do
      iinp = iinp + iseq(6,2)-iseq(6,1)

!     --- Renumbering

      jinp = iseq(7,1)
      do k = 1, iseq(7,2)-iseq(7,1)
         iwork(iinp+k) = iinput(jinp+k)
      end do
      iinp = iinp + iseq(7,2)-iseq(7,1)

!     --- Skip elements

      jinp = iseq(13,1)
      do k = 1, iseq(13,2)-iseq(13,1)
         iwork(iinp+k) = iinput(jinp+k)
      end do
      iinp = iinp + iseq(13,2)-iseq(13,1)

!     --- Level set

      if ( iseq(14,1)>0 .or. iinput(48)>0 ) then

!     --- There is level set information

         jinp = iseq(14,1)
         call prordrlev ( iinput, jinp, iinp, iwork )

      end if  ! ( iseq(14,1)>0 .or. iinput(48)>0 ) then

!     --- Periodical boundary conditions

      jinp = iseq(15,1)
      do k = 1, iseq(15,2)-iseq(15,1)
         iwork(iinp+k) = iinput(jinp+k)
      end do
      iinp = iinp + iseq(15,2)-iseq(15,1)

!     --- Copy iwork back to iinput

      iinput(leniinpfp+1:leniinpfp+iinp) = iwork(1:iinp)
      iinp = iinp+leniinpfp

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iinput, iinp, 'iinput/out' )
         write(irefwr,*) 'End prordr'

      end if  ! ( debug )
      call erclos ( 'prordr' )

      end
