      subroutine sepbounlevset4 ( bndfaces, lenbndfaces )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array bndfaces with faces corresponding to the boundary
!     of the region with elements with levelset>0
!     Old subroutine prbounlevset31
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     face
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lenbndfaces, bndfaces(lenbndfaces)

!     bndfaces       o    contains face sequence numbers of boundary
!                         of levelset
!                         bndfaces(i) contains starting address of level set i
!                         bndfaces(istart) contains the number of faces
!                         in boundary
!                         bndfaces(istart+j) the face sequence numbers
!     lenbndfaces   i/o   Computed length of array iobstfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, ielem, ipstart, nface_elem, node, &
              numinelmobst, numfaces, iface, ilev, istart, kstart, &
              jstart, numpntobst, numinnobst, numbndobst, nnode_face
      integer, allocatable :: iwork(:), iwork1(:)

!     i              Counting variable
!     ielem          Element number
!     iface          face sequence number
!     ilev           Sequence number of level being considered
!     ipstart        Starting address in array
!     istart         First position used at a part
!     iwork          integer work array to mark nodes
!                    iwork(i) contains number of nodes found on boundary for
!                    face i
!     iwork1         integer work array to mark nodes with level set zero
!     j              Counting variable
!     jstart         starting position in array
!     k              Counting variable
!     kstart         Next free position in bndfaces
!     nface_elem     number of faces corresponding to element
!     nnode_face     number of nodes corresponding to face
!     node           Node number
!     numbndobst     Number of nodal points on the boundary of the obstacle
!     numfaces       number of faces
!     numinelmobst   Number of elements that are completely within the obstacle
!     numinnobst     Number of points that are near the boundary of the obstacle
!     numpntobst     Number of nodal points inside the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepbounlevset4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepbounlevset4'
         write(irefwr,1) 'nfaces, npoint', nfaces, npoint
  1      format ( a, 1x, (10i9) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Allocate work space

      allocate ( iwork(nfaces), iwork1(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, nfaces+npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear iwork arrays

      iwork = 0
      iwork1 = 0

!     --- Loop over all levels

      kstart = nlevelsets+1

      do ilev = 1, nlevelsets

         jstart = ipointobst(ilev)
         numpntobst = ipointobst(jstart)
         numinnobst = ipointobst(jstart+1)
         numbndobst = ipointobst(jstart+2)       ! number of nodes in ipointobst
         jstart = jstart+numpntobst+numinnobst+4
         if ( debug ) then
            write(irefwr,1) 'ilev, numpntobst, numinnobst, numbndobst', &
                             ilev, numpntobst, numinnobst, numbndobst
            write(irefwr,1) 'jstart', jstart
            call prinin ( ipointobst(jstart), numbndobst, &
                          'ipointobst/1' )
         end if  ! ( debug )

!        --- Mark nodes of boundary in iwork1

         iwork1(ipointobst(jstart:jstart+numbndobst-1)) = 1

         istart = ielemobst(ilev)
         numinelmobst = ielemobst(istart)+ielemobst(istart+1)
         istart = istart+3

!        --- Mark all faces in inner elements inside the levelset function

         do j = istart, numinelmobst-1+istart

            ielem = ielemobst(j)
            ipstart = nmesha(ielem)  ! starting address of element in nmesha
            nface_elem = nmesha(ipstart)/2  ! number of faces of ielem

            if ( debug ) then
               write(irefwr,1) 'j, ielem, ipstart, nface_elem', &
                                j, ielem, ipstart, nface_elem
               call prinin ( nmesha(ipstart+1), nface_elem, 'faces' )
            end if  ! ( debug )

            do k = 1, nface_elem
               iface = abs(nmesha(ipstart+k))
               iwork(iface) = iwork(iface)+1  ! raise iwork for face
            end do  ! k = 1, nface_elem

         end do  ! j = istart, numinelmobst-1+istart

!         if ( debug ) call prinin ( iwork, nfaces, 'iwork' )

!        --- Find all faces that appear only once.
!            These are the boundary faces

         bndfaces(ilev) = kstart    ! starting address
         numfaces = 0
         do j = 1, nfaces

            if ( iwork(j)==1 ) then

!           --- iwork(j) = 1, hence boundary face found
!               Check if both nodes belong to ipointobst

               ipstart = nmeshg(j)
               nnode_face = nmeshg(ipstart)
               do i = 1, nnode_face
                  node = nmeshg(ipstart+i)
                  if ( iwork1(node)==0 ) go to 100  ! jump face not on bound
               end do  ! j = 1, nnode_face

!              --- If the program reaches this place a face has been found

               numfaces = numfaces+1        ! raise number of faces
               bndfaces(kstart+numfaces) = j     ! store face

            end if  ! ( iwork(j)==1 )

100      end do  ! j = 1, nfaces
         bndfaces(kstart) = numfaces  ! store number of faces
         kstart = kstart+1+numfaces

         if ( ilev<nlevelsets ) then

!        --- ilev<nlevelsets, clear iwork, iwork1

            iwork = 0  ! clear iwork
            do i = 0, numbndobst-1
               iwork1(ipointobst(jstart+i)) = 0   ! clear iwork1
            end do  ! i = 0, numbndobst-1

         end if  ! ( ilev<nlevelsets )

      end do  ! ilev = 1, nlevelsets

!     --- Deallocate work space

      deallocate ( iwork, iwork1, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

      lenbndfaces = kstart-1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'lenbndfaces', lenbndfaces
         do i = 1, nlevelsets
            write(irefwr,1) 'level set sequence number', i
            kstart = bndfaces(i)
            numfaces = bndfaces(kstart)
            call prinin ( bndfaces(kstart+1), numfaces, 'bndfaces' )
         end do  ! i = 1, nlevelsets
         write(irefwr,*) 'End sepbounlevset4'

      end if  ! ( debug )
      call erclos ( 'sepbounlevset4' )

      end
