      subroutine sepcommatcomp ( lenim2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-01-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill structure of matrix in case of a compact matrix
!     Old subroutine incommatcomp
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     iterative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulekmesh
      use sepmodulecpack
      use sepmodulematstr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lenim2

!     lenim2         o    length of the array imat part 2
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenim1
      integer, allocatable :: iwork(:)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     iwork          work array of length 3*npoint to indicate if a point
!                    is periodical or not
!                    If iwork(1,i) = 0 then point is not periodical
!                    For a chain of periodical points, iwork is
!                    filled as follows:
!                    iwork(1,first_node_in_chain) = sequence number in
!                    iwork(2,.)
!                    iwork(1,next_nodes in chain) = -first_node_in_chain
!                    iwork(2,.) contains for each
!                    chain of periodical points the number of points that
!                    are connected to the first point. These numbers are
!                    accumulated
!                    iwork(2,1) = 1
!                    iwork(2,i+1)-iwork(2,i) = number of points connected
!                    to nodal point i
!                    iwork(3,.) contains the nodal points
!                    connected to the first point in a chain in the
!                    sequence given by iwork(2,.)
!                    Hence iwork(3,.) positions iwork(2,i) to
!                    iwork(2,i+1)-1
!                    contain all nodes from the chain starting with node i
!     lenim1         Length of the array imat part 1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INCOMMATCOLS   Fill structure of matrix in case of a compact matrix
!                    Part 1b, columns numbers
!     INCOMMATDIAG   Store position of diagonal elements of array icolsii into
!                    intdia in case of row-compact storag
!     INCOMMATROWS   Fill structure of matrix in case of a compact matrix
!                    Part 1a, number of non-zero columns per row
!     SEPINTMAT      Transform symmetric compact storage scheme to
!                    row-compact storage
!     SEPPRINCOMPSTR print part of array intmat corresponding to internal
!                    unknowns. Compact storage
!     SEPSTRUCSIMPLE Fill matrix structure in case of simple type method
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcommatcomp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcommatcomp'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      if ( nrusol==1 ) then
         allocate ( irowsii(2), icolsii(1) )
         irowsii(1) = 0
         irowsii(2) = 1
         icolsii(1) = 1
         go to 1000
      end if  ! ( nrusol==1 )

!     --- Set constants

      allocate ( iwork(3*npoint), stat=error )
      if ( error/=0 ) call eralloc ( error, 3*npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000
      iwork = 0

!     --- Fill intmat part 1a: number of columns per row

      if ( debug ) write(irefwr,*) 'before incommatrows'
      call incommatrows ( iwork, lenim1, lenim2 )
      if ( ierror/=0 ) go to 1000

      length = lenim1+lenim2

      if ( lenim2==0 ) go to 1000  ! length of part cols=0

      if ( debug ) &
         write(irefwr,1) 'lenim1, lenim2, length, npoint, nrusol', &
                          lenim1, lenim2, length, npoint, nrusol

!     --- Fill intmat part 1b: column numbers of non-zero elements per row

      if ( debug ) write(irefwr,*) 'before incommatcols'
      call incommatcols ( lenim2, iwork )

      if ( debug ) write(irefwr,*) 'before deallocate'
      deallocate ( iwork, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )
      if ( ierror/=0 ) go to 1000

      if ( IM%simple ) then

!     --- storage for Schur_complement method
!         In this case the contents of intmat1 are completely reordered

         length = length+2
         call sepstrucsimple ( length )

      else

!     --- Transform to row-compact structure

         if ( debug ) call sepprincompstr ( nrusol, irowsii, icolsii )
         call sepintmat
         if ( .not. IM%symmetric .or. IM%mumps ) call incommatdiag

      end if  ! ( IM%simple )

1000  if ( debug ) then

!     --- Debug information

         call sepprincompstr ( nrusol, irowsii, icolsii )
         write(irefwr,*) 'End sepcommatcomp'

      end if  ! ( debug )
      call erclos ( 'sepcommatcomp' )

      end subroutine sepcommatcomp
