      subroutine sepdistnod ( viewp, ielems, ipoints, nodes, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all nodes with the largest distance to a given view point
!     Old subroutine erdistnod
! **********************************************************************
!
!                       KEYWORDS
!
!     distance
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ielems(*), ipoints(*), nodes(*), nnodes
      double precision :: viewp(*)

!     ielems         i    Array of length nelem indicating if an element
!                         is still free (0), must be skipped (-1) or is alweady
!                         in use (1)
!     ipoints             Work array of length npoint containing a set of nodes
!     nnodes         o    Number of nodes in nodes
!     nodes          o    Contains the node numbers in the starting set
!     viewp          i    Array containing the first view point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision x1(3), x2(3), xc(3), dist, drhs, drhs1, drhs2, &
                       mindist, largedist, x11, x12, x13, x21, x22, x23, &
                       eps, locnorm, normalvec(3)
      integer i, ielem, inode, ielgrp, ikelmc, inpelm, lsign, &
              startnode

!     dist           Distance between two points
!     drhs           Distance expressed as right-hand side
!     drhs1          Distance expressed as right-hand side
!     drhs2          Distance expressed as right-hand side
!     eps            Local accuracy
!     i              Counting variable
!     ielem          Element number
!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     inode          Node sequence number
!     inpelm         number of points per element
!     largedist      Largest distance
!     locnorm        Local norm
!     lsign          Sign of a scalar
!     mindist        Minimum distance
!     normalvec      Vector of length 3 containing the components of the
!                    normal
!     startnode      Starting node number
!     x1             x-coordinate of first point
!     x11            dummy
!     x12            dummy
!     x13            dummy
!     x2             x-coordinate of last point
!     x21            dummy
!     x22            dummy
!     x23            dummy
!     xc             array with the coordinates of the centre point of the
!                    element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision distance, inprod
      distance ( x11, x12, x13, x21, x22, x23 ) = &
                 (x21-x11)**2+(x22-x12)**2+(x23-x13)**2
      inprod ( x11, x12, x13, x21, x22, x23 ) = &
               x11*x21+x12*x22+x13*x23

!     DISTANCE       Inline subroutine to compute the distance between two
!                    vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INPROD         Inline subroutine to compute the inner product of two
!                    vectors
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Find starting address in array KMESHC
!    Clear array ipoints
!    Mark all points in the range of elements:
!       For ielem := ielemlow (1) ielemtop do
!          For all nodes inode in element ielem do
!             ipoints(inode) := 1
!    Find smallest distance min_dis between nodes in elements
!    eps := 0.5*min_dis
!    Find node i1 (x_1) with largest distance to viewpoint x_0
!    Find all nodes below the plane perpendicular to the line from viewpoint
!    to -viewpoint that goes through x_1 corrected with a distance eps in the
!    direction of x_0
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepdistnod' )

!     --- Clear array ipoints

      ipoints(1:npoint) = 0

!     --- Mark all points in the range of elements:
!            For ielem := ielemlow (1) ielemtop do
!               For all nodes inode in element ielem do
!                  ipoints(inode) := 1
!         Find smallest distance min_dis between nodes in elements

      mindist = rinfin
      ikelmc = 0
      do ielgrp = 1, nelgrp

         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm
         do ielem = 1, inelem
            if ( ielems(i)==0 ) then

!           --- element must not be skipped

               inode = kmeshc(ikelmc+1)
               ipoints(inode) = 1
               x1(1) = coor(1,inode)
               x1(2) = coor(2,inode)
               x1(3) = coor(3,inode)
               do i = 2, inpelm
                  inode = kmeshc(ikelmc+i)
                  ipoints(inode) = 1
                  x2(1) = coor(1,inode)
                  x2(2) = coor(2,inode)
                  x2(3) = coor(3,inode)
                  mindist = min ( mindist , &
                     distance(x1(1), x1(2), x1(3), x2(1), x2(2), x2(3)))
               end do

            end if
            ikelmc = ikelmc + inpelm

         end do

      end do

!     --- eps := 0.9*min_dis

      eps = 0.9d0*sqrt(mindist)

!     --- Find node i1 (x_1) with largest distance to viewpoint x_0

      x1(1) = viewp(1)
      x1(2) = viewp(2)
      x1(3) = viewp(3)
      largedist = 0d0
      do i = 1, npoint

         if ( ipoints(i)==0 ) then

!        --- Point is in the range

            x2(1) = coor(1,i)
            x2(2) = coor(2,i)
            x2(3) = coor(3,i)
            dist = distance(x1(1), x1(2), x1(3), x2(1), x2(2), x2(3))
            if ( dist>largedist ) then

!           --- dist > largedist, update

               largedist = dist
               startnode = i

            end if

         end if

      end do

!     --- Find all nodes below the plane perpendicular to the line from
!         viewpoint to centroid that goes through x_1 corrected with a
!         distance eps in the direction of x_0
!         This set defines the set of starting nodes

      xc(1) = 0d0
      xc(2) = 0d0
      xc(3) = 0d0
      do i = 1, npoint
         xc(1) = xc(1)+coor(1,i)
         xc(2) = xc(2)+coor(2,i)
         xc(3) = xc(3)+coor(3,i)
      end do
      xc(1) = xc(1)/npoint
      xc(2) = xc(2)/npoint
      xc(3) = xc(3)/npoint
      locnorm = 1d0/sqrt &
         (distance(xc(1), xc(2), xc(3), viewp(1), viewp(2), viewp(3)))
      do i = 1, 3
         normalvec(i) = viewp(i)*locnorm
         x2(i) = coor(i,startnode)
      end do
      drhs1 = inprod(normalvec(1),normalvec(2),normalvec(3), &
                     viewp(1),viewp(2),viewp(3))
      drhs2 = inprod(normalvec(1),normalvec(2),normalvec(3), &
                     x2(1),x2(2),x2(3))
      if ( drhs2-drhs1>0d0 ) then

!     --- distance positive

         drhs  = drhs2-eps*(drhs2-drhs1)
         lsign = 1

      else

!     --- distance negative

         drhs  = drhs2+eps*(drhs2-drhs1)
         lsign = -1

      end if
      nnodes = 0
      do i = 1, npoint
         if ( ipoints(i)==0 ) then

!        --- Point is in the range

            x2(1) = coor(1,i)
            x2(2) = coor(2,i)
            x2(3) = coor(3,i)
            dist  = inprod(normalvec(1),normalvec(2),normalvec(3), &
                           x2(1),x2(2),x2(3))
            if ( lsign==1 ) then

!           --- distance positive

               if ( dist>drhs ) then
                  nnodes = nnodes+1
                  nodes(nnodes) = i
               end if

            else

!           --- distance negative

               if ( dist<drhs ) then
                  nnodes = nnodes+1
                  nodes(nnodes) = i
               end if

            end if

         end if

      end do

      call erclos ( 'sepdistnod' )
      end
