      subroutine sepnesteddis
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-05-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Renumbering with nested dissection
!
! **********************************************************************
!
!                       KEYWORDS
!
!     renumbering
!     nested_dissection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nremain, i, sizeset, startnode, levellength, middleset, &
                 lensubset, numsubsets
      integer, pointer :: setP(:), setR(:), subset(:), sizesubsets(:)

!     i              Counting variable
!     lensubset      size of subset
!     levellength    Length of rooted level structure
!     middleset      Sequence number of "middle" set
!     nremain        Number of nodes that have not been renumbered yet
!     numsubsets     Number of subsets
!     setP           Set of renumbered nodes
!     setR           Set of remaining nodes
!     sizeset        size of set
!     sizesubsets    Array with sizes of all subsets
!     startnode      Starting node number
!     subset         Subset of the complete set of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepstartnode

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPCUTHILL     Renumber a set of (possibly disjoint) nodes according to
!                    Cuthill McKee
!     SEPFILLSUBSET  Fill subset with nodes in level middleset that have at
!                    least one node in common with level middleset+1
!     SEPROOTLEVEL   Find rooted level structure corresponding to node
!     SEPSTARTNODE   Find start node in a set of nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     setP := empty; setR := {1,...,npoint}; nremain := npoint
!     sizeset := npoint
!     while ( nremain>0 )
!        set := setR(1:sizeset)
!        startnode := start node in set
!        L1, L2, ... , Lm is rooted level structure in set
!        if ( m==1 ) then
!           set := L1
!        else
!           j = (m+1)/2
!           set := {all nodes in Lj with at least an adjacent node in Lj+1}
!        end_if
!        Renumber set with reversed Cuthill-McKee and put at the end of setP
!        Remove set from setR; nremain := nremain-sizeset
!        Find connected set of nodes in setR starting with setR(1)
!        sizeset := number of connected nodes
!     end_while
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepnesteddis' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepnesteddis'
         write(irefwr,1) 'npoint', npoint
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      allocate ( marknodes(npoint), setP(npoint), setR(npoint), &
                 subset(npoint), levelstruc(npoint), pointlevel(npoint+1), &
                 sizesubsets(npoint) )

!     --- setP := empty; setR := {1,...,npoint}; nremain := npoint

      setP = 0
      marknodes = 0
      nremain = npoint
      do i = 1, npoint
         setR(i) = i
      end do  ! i = 1, npoint

!     --- sizeset := npoint

      sizeset = npoint
      numsubsets = 0

!     --- while ( nremain>0 )

      do while ( nremain>0 )

!     --- set := setR(1:sizeset)

         set => setR(1:sizeset)

         if ( debug ) call prinin ( set, sizeset, 'set' )

         if ( sizeset==1 ) then

!        --- sizeset = 1

            lensubset = 1
            subset(1) = set(1)

         else

!        --- sizeset startnode := start node in set

            startnode = sepstartnode()
            if ( debug ) write(irefwr,1) 'startnode', startnode

!           --- L1, L2, ... , Lm is rooted level structure in set

            call seprootlevel ( startnode, levellength )
            if ( debug ) then
               call prinin ( pointlevel, levellength+1, 'pointlevel' )
               call prinin ( levelstruc, pointlevel(levellength+1)-1, &
                             'levelstruc' )
            end if  ! ( debug )

            if ( levellength==1 ) then

!              m==1
!              subset := L1

               lensubset = pointlevel(2)-1
               subset(1:lensubset) = levelstruc(1:lensubset)

            else

!           --- j = (m+1)/2
!               set := {all nodes in Lj with at least an adjacent node in Lj+1}

               middleset = (levellength+1)/2
               call sepfillsubset ( subset, middleset, lensubset )
               if ( debug ) then
                  write(irefwr,1) 'middleset', middleset
                  call prinin ( subset, lensubset, 'subset' )
               end if  ! ( debug )

            end if  ! ( levellength>=1 )

            if ( lensubset>1 ) then

!           --- Renumber set with reversed Cuthill-McKee and
!               put at the end of setP

               call sepcuthill ( lensubset, subset )
               if ( debug ) then
                  write(irefwr,1) 'middleset', middleset
                  call prinin ( subset, lensubset, 'subset' )
               end if  ! ( debug )

            end if  ! ( lensubset>1 )

         end if  ! ( sizeset==1 )
         numsubsets = numsubsets+1

         setP(nremain-lensubset+1:nremain) = subset(1:lensubset)

!        --- Remove set from setR; nremain := nremain-sizeset

         marknodes(subset(1:lensubset)) = -10
         nremain = 0
         do i = 1, npoint

            if ( marknodes(i)>-10 ) then

 !          --- Node does not belong to setP

               nremain = nremain+1
               setR(nremain) = i

            end if  ! ( marknodes(i)>-10 )

         end do  ! i = 1, npoint
         if ( debug ) then
            call prinin ( setP, npoint, 'setP' )
            call prinin ( setR, nremain, 'setR' )
         end if  ! ( debug )
         sizesubsets(numsubsets) = lensubset

!        --- Find connected set of nodes in setR starting with setR(1)
!            sizeset := number of connected nodes

         if ( nremain>0 ) then
            set => setR(1:nremain)
            call seprootlevel ( setR(1), levellength )
            sizeset = pointlevel(levellength+1)-1
            if ( debug ) then
               call prinin ( pointlevel, levellength+1, 'pointlevel' )
               call prinin ( levelstruc, pointlevel(levellength+1)-1, &
                             'levelstruc' )
            end if  ! ( debug )
         end if  ! ( nremain>0 )
         if ( debug ) write(irefwr,1) 'nremain, sizeset', nremain, sizeset

      end do  ! while ( nremain>0 )

      allocate ( km(iseqkm)%pointlevel(numsubsets+1), &
                 km(iseqkm)%levelstruc(npoint) )
      km(iseqkm)%levelstruc = setP
      km(iseqkm)%pointlevel(1) = 1
      do i = 1, numsubsets
         sizeset = sizesubsets(numsubsets+1-i)
         km(iseqkm)%pointlevel(i+1) = sizeset+km(iseqkm)%pointlevel(i)
      end do  ! i = numsubsets, -1, 1

      if ( .not. associated(km(iseqkm)%rennested ) ) &
         allocate ( km(iseqkm)%rennested(npoint) )
      km(iseqkm)%rennested = km(iseqkm)%levelstruc
      kmeshj => km(iseqkm)%rennested
      kelmj = npoint

      deallocate ( marknodes, setP, setR, subset, levelstruc, pointlevel, &
                   sizesubsets )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( km(iseqkm)%pointlevel, numsubsets+1, 'pointlevel' )
         call prinin ( km(iseqkm)%levelstruc, npoint, 'levelstruc' )
         write(irefwr,*) 'End sepnesteddis'

      end if  ! ( debug )
      call erclos ( 'sepnesteddis' )

      end subroutine sepnesteddis
