      subroutine topbv1ren ( irenum, lenrenum, maxren )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform renumbering if necessary and fill array irenum
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     periodical
!     problem_definition
!     renumbering
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: lenrenum
      integer, intent(out) :: irenum(lenrenum), maxren

!     irenum         o    This array contains the levels of the renumbering of
!                         all unknowns per point
!                         So if iren(i)=j then unknown i is numbered in the
!                         j-th level
!                         Unknowns with sequence number larger than maxren
!                         correspond always to the largest level
!     lenrenum       i    Declared length of array irenum
!     maxren         o    Maximum number of renumbered unknowns
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: jrenum(10), nphysl, iinp7

!     iinp7          first pos. iinputkp definition connected elements
!     jrenum         Parameter jrenum for TOPOL
!     nphysl         Local value of nphys
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     TOPBV1RENUM    Fill array irenum, depending on iinputkp and maxren
!     TOPOL          Fill integer information corresponding to the mesh in KMESH
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1532   More unknowns to be renumbered than physical unknowns
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topbv1ren' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topbv1ren'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      nphysl = max(nphys,nunkp)
      iinp7 = intarprb(7)

      maxren = iinputkp(14)
      if ( debug ) write(irefwr,1) 'maxren/1, kelmk', maxren, kelmk

      if ( maxren<0 .and. kelmk==0 ) then

!     --- Unknowns must be renumbered with respect to levels, but there are
!         no levels
!         In this case renumering is forced by a call to topol

         jrenum(1) = -10  ! 10 entries are used
         jrenum(2) = 1    ! Cuthill-McKee
         jrenum(3) = 0    ! smallest profile
         jrenum(4) = 0    ! Minimal level for printing
         jrenum(5) = 3    ! Priority for vertex nodes
         jrenum(6) = 3    ! Priority for mid point nodes
         jrenum(7) = 3    ! Priority for centroids
         jrenum(8) = 0    ! ilevel
         jrenum(9) = 4    ! Start point given
         jrenum(10) = 1   ! Start with node 1
         call topol ( jrenum )

      end if

      if ( abs(maxren)>nphysl ) then

!     --- nphys too large

         call errint ( abs(maxren), 1 )
         call errint ( nphysl, 2 )
         call errwar ( 1532, 2, 0, 0 )
         if ( maxren>0 ) then
             maxren = nphysl
         else
             maxren = -nphysl
         end if

      end if
      if ( debug ) write(irefwr,1) 'maxren/2', maxren

!     --- Fill array irenum with information with respect to renumbering
!         this is necessary for example in case of plast renumbering

      if ( maxren/=0 ) &
         call topbv1renum ( maxren, irenum, lenrenum, iinputkp(iinp7) )
      if ( ierror/=0 ) go to 1000

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'maxren/3', maxren
         call prinin ( irenum, abs(maxren), 'irenum' )
         write(irefwr,*) 'End topbv1ren'

      end if  ! ( debug )
      call erclos ( 'topbv1ren' )

      end subroutine topbv1ren
