      subroutine topbv1renum ( maxren, irenum, lenrenum, iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 17-04-2009 Remove skipping linear Taylor-Hood elemts
!        version  1.0  date 30-04-2003
!
!   copyright (c) 2003-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array irenum, depending on iinput and maxren
!     If plast ordering is used, fill based on elements
! **********************************************************************
!
!                       KEYWORDS
!
!     renumbering
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer maxren, irenum(*), lenrenum, iinput(*)

!     iinput         i    integer input array for PROBDF
!                         In this case it concerns only the part corresponding
!                         to renumbering
!     irenum         o    This array contains the levels of the renumbering of
!                         all unknowns per point
!                         So if iren(i)=j then unknown i is numbered in the j-th
!                         level
!                         Unknowns with sequence number larger than maxren
!                         correspond always to the largest level
!     lenrenum       i    Declared length of array irenum
!     maxren        i/o   Maximum number of renumbered unknowns
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielgrp, itype
      logical type901, type902, type903

!     i              Counting variable
!     ielgrp         element group number
!     itype          Type number of element
!     type901        If true elements of type 901 have been found
!     type902        If true elements of type 902 have been found
!     type903        If true elements of type 903 have been found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    2440   iinput is incorrect
!    2442   mixture of elements of type 901, 902 and 903
!    2443   Type number can not be used with plast renumbering
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( maxren>1 or iinput(1)>0 ) then
!        Copy iinput into irenum
!     else
!        maxren := nphys
!        for ielgrp := 1 (1) nelgrp do
!           ishape := shape number of elements
!           itype := type number of element
!           if ( shape number corresponds to ndim ) then
!              if ( itype = 261 or 901 ) then
!                 irenum(1:ndim) := 1
!                 irenum(ndim+1) := 2
!              else if ( itype = 262 or 902 ) then
!                 irenum(1:ndim) := 1
!                 irenum(ndim+1:2*ndim+1) := 2
!              else if ( itype = 903 and ishape #3 or 11 ) then
!                 irenum(1:ndim) := 1
!                 irenum(ndim+1) := 2
!              else
!                 error
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topbv1renum' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topbv1renum'
         write(irefwr,1) 'maxren, iinput(1)', maxren, iinput(1)
  1      format ( a, 1x, (10i6) )

      end if
      if ( maxren/=1 .or. iinput(1)>0 ) then

!     --- maxren>1 or iinput(1)>0
!         Copy iinput into irenum

         if ( abs(maxren)>lenrenum ) then

!        --- Array irenum too small

            call errint ( abs(maxren), 1 )
            call errint ( lenrenum, 2 )
            call errchr ( 'irenum', 1 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if

         do i = 1, abs(maxren)
            irenum(i) = iinput(i)
         end do

      else if ( maxren==1 .and. iinput(1)==-1 ) then

!     --- iinput(1) = -1, plast renumbering
!         maxren := nphys

         nphys = max(nphys,nunkp)
         maxren = nphys
         irenum(1:nphys) = 1

         type901 = .false.
         type902 = .false.
         type903 = .false.

!        --- for ielgrp := 1 (1) nelgrp do

         do ielgrp = 1, nelgrp

!        --- ishape := shape number of elements
!            itype := type number of element

            ishape = kmeshf(ielgrp)

            if ( ndim==2 .and. ishape>2 .or. ishape>10 ) then

!           --- shape number corresponds to ndim
!               ndim = 1, does not appear
!               ndim = 2: 3-10
!               ndim = 3:>10

               itype = kprobc(ielgrp)
               if ( itype==261 .or. itype==901 ) then

!              --- itype = 261 or 901
!                  irenum(1:ndim) := 1
!                  irenum(ndim+1) := 2

                  irenum(ndim+1) = 2
                  if ( type902 .or. type903 ) then

!                 --- Mixture of type 901 (261) 902 (262) and 903 not allowed

                     call errsub ( 2442, 0, 0, 0 )

                  end if
                  type901 = .true.

               else if ( itype==262 .or. itype==902 ) then

!              --- itype = 262 or 902
!                  irenum(1:ndim) := 1
!                  irenum(ndim+1) := 2

                  do i = 1, ndim+1
                     irenum(ndim+i) = 2
                  end do
                  if ( type901 .or. type903 ) then

!                 --- Mixture of type 901 (261) 902 (262) and 903 not allowed

                     call errsub ( 2442, 0, 0, 0 )

                  end if
                  type902 = .true.

               else if ( itype==903 ) then

!              --- itype = 903
!                  irenum(1:ndim) := 1
!                  irenum(ndim+1:2*ndim) := 2

                  irenum(ndim+1) = 2
                  if ( type901 .or. type902 ) then

!                 --- Mixture of type 901 (261) 902 (262) and 903 not allowed

                     call errsub ( 2442, 0, 0, 0 )

                  end if
                  type903 = .true.

               else

!              --- error type number not implemented

                  call errint ( itype, 1 )
                  call errsub ( 2443, 1, 0, 0 )

               end if  ! ( itype==261 .or. itype==901 )

            end if  ! ( ndim==2 .and. ishape>2 .or. ishape>10 )

         end do  ! ielgrp = 1, nelgrp

      else

!     --- iinput incorrect

         call errint ( maxren, 1 )
         call errint ( iinput(1), 2 )
         call errsub ( 2440, 2, 0, 0 )

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinin ( irenum, abs(maxren) ,'irenum' )
         write(irefwr,*) 'End topbv1renum'

      end if
      call erclos ( 'topbv1renum' )

      end
