      subroutine topbv5 ( iwork, jumps, factors, istoreleft, lenistore )
! ======================================================================
!
!        programmer    Guus Segal
!        version  9.0  date 15-06-2012 Remove ihelp from parameter list
!        version  8.0  date 26-10-2005 Extra parameter iprinp
!        version  7.1  date 04-09-2002 Extra parameters for topbv01/2
!                                      nunkp<=320 allowed
!
!   copyright (c) 1985-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Nodal points of which the unknowns are connected through periodical
!     boundary conditions (possibly with jump) are marked
!       degrees of freedom corresponding to one of the nodal points
!       are filled in iwork with a negative number referring to the
!       other one
!     Warning: the algorithm used in this subroutine is also used in
!            subroutine incm01
!            If this subroutine is changed, also change incm01
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     periodical
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), istoreleft(*), lenistore
      double precision :: jumps(*), factors(*)

!     factors        o    Array of size nusol containing the multiplication
!                         factors for the periodical boundary conditions
!     istoreleft     o    Contains all periodical right neighbor unknowns
!                         for all left periodical neighbor unknowns
!                         Pos. 1 to nusol+1 contain pointers to the second part
!                         istoreleft(1) contains nusol+1
!                         The number of neighbors of unknown i is stored in
!                         istoreleft(i+1)-istoreleft(i)
!                         The second part contains the neighbors
!     iwork         i/o   Semi-permanent work array of length nusol in which
!                         information about essential boundary conditions and
!                         periodical boundary conditions is stored
!                         iwork is filled as follows:
!                         iwork(i) = 0: means unknown i is free
!                         iwork(i)>0: means unknown i is prescribed
!                                       (essential bc)
!                         iwork(i)<0: means unknown i refers to unknown
!                                       iwork(i), &
!                         which means that unknown i has the same value, &
!                         possibly extended with a jump
!                         Subroutine TOPBV2 has already initialised iwork with
!                         non-negative values
!     jumps          o    Array of size nusol containing the jumps
!                         for the periodical boundary conditions
!     lenistore      i    Estimated length of arrays istoreleft and istoreright
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer itemp(nusol), istoreright(lenistore)
      double precision rjumpleft(nusol), rmultright(nusol), &
                       rjumpright(nusol), rmultleft(nusol)

!     istoreright    Contains all periodical left neighbor unknowns
!                    for all right periodical neighbor unknowns
!                    Pos. 1 to nusol+1 contain pointers to the second part
!                    istoreright(1) contains nusol+1
!                    The number of neighbors of unknown i is stored in
!                    istoreright(i+1)-istoreright(i)
!                    The second part contains the neighbors
!     itemp          Temporary work array of length nusol to indicate
!                    if an unknown corresponds to a "left" periodical
!                    boundary condition (1) or not (0)
!     rjumpleft      Array wih the same length as the second part of
!                    istoreleft containing the jumps w.r.t the right part
!     rjumpright     Array wih the same length as the second part of
!                    istoreright containing the jumps w.r.t the left part
!     rmultleft      Array wih the same length as the second part of
!                    istoreleft containing the factors w.r.t the right part
!     rmultright     Array wih the same length as the second part of
!                    istoreright containing the factors w.r.t the left part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINDIFFARR    Print the contents of an array consisting of pointers
!                    followed by the actual information
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     TOPBV501       Fill number of right neighbor unknowns in istoreleft and
!                    number of left neighbor unknowns in istoreright
!     TOPBV502       Fill rest of arrays istoreleft and istoreright
!                    Fill jumps in array jumpleft and jumpright and
!                    multiplication factors in rmultleft and rmultfact
!     TOPBV703       Nodal points of which the unknowns are connected through
!                    periodical boundary conditions (possibly with jump) are
!                    marked
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    2343   Internal error: nunkp>32
! **********************************************************************
!
!                       PSEUDO CODE
!
!     clear arrays istoreleft, istoreright,ihelp, jumps and factors
!     Fill number of right unknowns in istoreleft and
!          number of left unknowns in istoreright
!     Fill all neighbors in ileft and iright and jumps and factors in work1/2
!     Fill array ihelp
!     nhelp := number of non-zero entries in ihelp
!     While ( nhelp>0 ) do
!        istart := smallest unknown with ihelp(istart) = 1
!        ihelp(istart) := 0, nhelp:=nhelp-1
!        Store all neighbors of istart in istore
!        nstore := number of entries in istore
!        Fill iwork, jumps, factors for all entries in istore
!        Remove all neighbors from ihelp and decrease nhelp
!        While ( nstore>0 ) do
!           inext := first unknown in istore
!           remove inext from istore and decrease nstore
!           For all neighbors ineigh of inext do
!              if ( ihelp(neigh) = 0 ) then
!                 debug jump and factor
!              else
!                 add neighbor to iwork, jumps, factors and istore
!                 nstore := nstore+1
!                 ihelp(ineigh) := 0
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topbv5' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topbv5'

      end if  ! ( debug )
      if ( nunkp>320 ) then

!     --- nunkp too large

         call errint ( nunkp, 1 )
         call errint ( 320, 2 )
         call errsub ( 2343, 2, 0, 0 )
         nunkp = 320

      end if

!     --- Clear arrays ihelp, jumps and factors

      itemp(1:nusol) = 0
      jumps(1:nusol) = 0d0
      factors(1:nusol) = 1d0

!     --- Fill number of right unknowns in istoreleft and
!              number of left unknowns in istoreright

      call topbv501 ( istoreleft, istoreright )
      if ( ierror/=0 ) go to 1000

      if ( istoreleft(nusol+1)>lenistore ) then

!     --- Length of array istoreleft too small

         call errint ( istoreleft(nusol+1), 1 )
         call errint ( lenistore, 2 )
         call errchr ( 'istoreleft', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if
      if ( istoreright(nusol+1)>lenistore ) then

!     --- Length of array istoreright too small

         call errint ( istoreright(nusol+1), 1 )
         call errint ( lenistore, 2 )
         call errchr ( 'istoreright', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

!     --- Fill all neighbors in istoreleft and istoreright and
!         jumps and factors in rjumpleft, rjumpright rmultleft, rmultright

      call topbv502 ( istoreleft, istoreright, rjumpleft, rjumpright, &
                      rmultleft, rmultright, iwork, itemp )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then

!     --- Debug statements

         call prindiffarr ( istoreleft, nusol, 'istoreleft' )
         call prindiffarr ( istoreright, nusol, 'istoreright' )
         call prinrl ( rjumpleft, istoreleft(nusol+1)-nusol-1, &
                       'rjumpleft' )
         call prinrl ( rjumpright, istoreright(nusol+1)-nusol-1, &
                       'rjumpright' )
         call prinrl ( rmultleft, istoreleft(nusol+1)-nusol-1, &
                       'rmultleft' )
         call prinrl ( rmultright, istoreright(nusol+1)-nusol-1, &
                       'rmultright' )

      end if

!     --- Fill final arrays

      call topbv703 ( iwork, itemp, jumps, factors, istoreleft, &
                      nusol, rjumpleft, rmultright, rjumpright, &
                      rmultleft, istoreright )
      if ( debug ) then

!     --- Debug statements

         call prinin (iwork, nusol, 'iwork' )
         call prinrl ( jumps, nusol, 'jumps' )
         call prinrl ( factors, nusol, 'factors' )
         write(irefwr,*) 'End topbv5'

      end if

1000  call erclos ( 'topbv5' )
      end
