      subroutine topbv701 ( istoreleft, istoreright, iprinp, nodepairs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 14-01-2013 correction surfaces
!        version  1.0  date 18-06-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill number of right neighbor unknowns in istoreleft and
!     number of left neighbor unknowns in istoreright
!     New version without the use of connection elements
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     periodical
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istoreleft(*), istoreright(*), iprinp, nodepairs(2,*)

!     iprinp         o    Starting address in array rinputkp
!     istoreleft     o    Contains all periodical right neighbor unknowns
!                         for all left periodical neighbor unknowns
!                         Pos. 1 to nusol+1 contain pointers to the second part
!                         istoreleft(1) contains nusol+1
!                         The number of neighbors of unknown i is stored in
!                         istoreleft(i+1)-istoreleft(i)
!                         The second part contains the neighbors
!                         In this subroutine the first part is stored
!     istoreright    o    Contains all periodical left neighbor unknowns
!                         for all right periodical neighbor unknowns
!                         Pos. 1 to nusol+1 contain pointers to the second part
!                         istoreright(1) contains nusol+1
!                         The number of neighbors of unknown i is stored in
!                         istoreright(i+1)-istoreright(i)
!                         The second part contains the neighbors
!                         In this subroutine the first part is stored
!     nodepairs      o    Work array of size 2*npoint in which the pairs of node
!                         numbers corresponding to the periodical boundary conditions
!                         are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, iinp, iexclu, ifirst, &
              idegper(nunkp), i, iinp15, ilast, ilefst, irigst, &
              iinp1, inpelm,  nodp1, nodp2, istar1, &
              istart, numcrvs, numpnts, numpoints, numsrfs
      integer, allocatable :: inodesleft(:), inodesright(:), iwork(:)
      double precision rjumploc(nunkp), rmultloc(nunkp)

!     i              Counting variable
!     idegper        Array of size nunkp containing a 1 if the
!                    corresponding unknown is periodical and a 0 if not
!     iexclu         Indicates which curves are excluded
!     ifirst         First curve (surface) number read
!     iinp           last position used in array iinputkp
!     iinp1          first pos. iinputkp definition curves
!     iinp15         first pos. iinputkp dummy element groups
!     ilast          Last curve (surface) number read
!     ilefst         Starting point of "left" surface in KMESH part n
!     inodesleft     Contains node numbers at left-hand side of per bc's
!     inodesright    Contains node numbers at right-hand side of per bc's
!     inpelm         number of points per element
!     irigst         Starting point of "right" surface in KMESH part n
!     istar1         Starting position of first curve in KMESHM
!     istart         Starting address
!     iwork          integer work array of length npoint to mark nodes
!                    that must be excluded
!     j              Counting variable
!     nodp1          Node sequence number
!     nodp2          Node sequence number
!     numcrvs        Number of perodical points in curves
!     numpnts        Number of perodical points in user point
!     numpoints      Number of perodical points in surface
!     numsrfs        Number of perodical points
!     rjumploc       Array of size nunkp containing the jumps for each
!                    degree of freedom
!     rmultloc       Array of size nunkp containing the multiplication factors
!                    for each degree of freedom
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCONNECTCURV Store nodes along left-hand side into istoreleft
!                    and along right-hand side into istoreright
!     MSHCONNECTSURF Compute first part for connection of surfaces
!     MSHCONNECTWORK Fill array iwork for connection of points in surface
!                    elements
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     TOPBV5012      Accumulate arrays istoreleft and istoreright
!     TOPBV7011      Fill array idegper with degrees of freedom that are
!                    periodical
!     TOPBV7012      Fill number of right neighbor unknowns in istoreleft and
!                    number of left neighbor unknowns in istoreright
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear arrays istoreleft and istoreright
!     For all element groups do
!        if ( element is periodical ) then
!           store which unknowns are periodical in idegper
!           For all elements of this group do
!              find left and right node
!              Add positions in istoreleft and istoreright
!     Accumulate arrays istoreleft and istoreright
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topbv701' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topbv701'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpelm = npelm
      iinp1 = intarprb(1)
      iinp15= intarprb(19)
      iinp = iinp1
      iprinp = 0

      numpairs = 0

!     --- Clear arrays istoreleft and istoreright

      istoreleft(1:nusol) = 0
      istoreright(1:nusol) = 0

!     --- Get information of periodical boundary conditions

      iinp = iinp15

      numpnts = iinputkp(iinp+1)
      numcrvs = iinputkp(iinp+2)
      numsrfs = iinputkp(iinp+3)
      iinp = iinp+3
      if ( debug ) &
         write(irefwr,1) 'iinp, iinp15, numpnts, numcrvs, numsrfs', &
                          iinp, iinp15, numpnts, numcrvs, numsrfs

!     --- Point periodical boundary conditions

      do i = 1, numpnts

         istart = iinputkp(iinp+i)+iinp15
         ifirst = iinputkp(istart)
         ilast = iinputkp(istart+1)

         nodp1 = kmeshl(ifirst)
         nodp2 = kmeshl(ilast)

         if ( debug ) then
            write(irefwr,1) 'istart, ifirst, ilast', &
                             istart, ifirst, ilast
            write(irefwr,1) 'nodp1, nodp2', nodp1, nodp2
         end if  ! ( debug )

         if ( nodp1>0 .and. nodp2>0 ) then

!        --- Both nodes are present
!            Fill array idegper

            numpairs = numpairs+1
            nodepairs(1,numpairs) = nodp1
            nodepairs(2,numpairs) = nodp2
            call topbv7011 ( iinputkp(istart+2), idegper, rjumploc, &
                             rmultloc )
            if ( ierror/=0 ) go to 1000

!           --- Fill number of right neighbor unknowns in istoreleft and
!               number of left neighbor unknowns in istoreright

            call topbv7012 ( nodp1, nodp2, indprf, kprobf, idegper, &
                             nunkp, istoreleft, istoreright )
            if ( ierror/=0 ) go to 1000

         end if  ! ( nodp1>0 .and. nodp2>0 )

      end do  ! i = 1, numpnts
      iinp = iinp+3*numpnts

!     --- Curve periodical boundary conditions

      do i = 1, numcrvs

         istart = iinputkp(iinp+i)+iinp15
         ifirst = iinputkp(istart)
         ilast = iinputkp(istart+1)
         iexclu = iinputkp(istart+2)

         if ( debug ) &
            write(irefwr,1) 'istart, ifirst, ilast, iexclu', &
                             istart, ifirst, ilast, iexclu

!        --- Fill array idegper

         call topbv7011 ( iinputkp(istart+3), idegper, rjumploc, &
                          rmultloc )
         if ( ierror/=0 ) go to 1000

!        --- Store nodes in inodesleft and inodesright

         length = kmeshm(5+ifirst)-kmeshm(4+ifirst)
         if ( debug ) write(irefwr,1) 'length', length
         allocate ( inodesleft(length), inodesright(length), &
                    stat = error )
         if ( error/=0 ) call eralloc ( error, length, 'inodesleft' )
         if ( ierror/=0 ) go to 1000

         ishape = 0
         call mshconnectcurv ( ifirst, ilast, inodesleft, inodesright, &
                               numpoints, 0, i, iexclu, istar1 )
         if ( ierror/=0 ) go to 1000

         do j = 1, numpoints

            nodp1 = inodesleft(istar1+j)
            nodp2 = inodesright(istar1+j)

            if ( nodp1>0 .and. nodp2>0 ) then

!           --- Both nodes are present
!               Fill number of right neighbor unknowns in istoreleft and
!               number of left neighbor unknowns in istoreright

               numpairs = numpairs+1
               nodepairs(1,numpairs) = nodp1
               nodepairs(2,numpairs) = nodp2
               call topbv7012 ( nodp1, nodp2, indprf, kprobf, idegper, &
                                nunkp, istoreleft, istoreright )

            end if  ! ( nodp1>0 .and. nodp2>0 )

         end do  ! j = 1, numpoints

         deallocate ( inodesleft, inodesright, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'inodesleft' )
         if ( ierror/=0 ) go to 1000

      end do  ! i = 1, numcrvs
      iinp = iinp+3*numcrvs

!     --- Surface periodical boundary conditions

      do i = 1, numsrfs

         istart = iinputkp(iinp+i)+iinp15
         ifirst = iinputkp(istart)
         ilast = iinputkp(istart+1)
         iexclu = iinputkp(istart+2)

         if ( debug ) &
            write(irefwr,1) 'istart, ifirst, ilast, iexclu, iinp15', &
                             istart, ifirst, ilast, iexclu, iinp15

!        --- Compute parameters (first part)

         call mshconnectsurf ( ifirst, ilast, inpelm, ilefst, irigst )
         if ( ierror/=0 ) go to 1000

!        --- Mark nodes in iwork that must be excluded

         allocate ( iwork(npoint), stat = error )
         if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
         if ( ierror/=0 ) go to 1000

         call mshconnectwork ( iwork, ifirst, iexclu, iinputkp(istart+3) )
         if ( ierror/=0 ) go to 1000

!        --- Fill array idegper

         istart = istart+3
         if ( iexclu>0 ) istart = istart+iexclu
         call topbv7011 ( iinputkp(istart), idegper, rjumploc, &
                          rmultloc )
         if ( ierror/=0 ) go to 1000

         do j = 1, inelem*inpelm

            nodp1 = kmeshn(ilefst+j)
            nodp2 = kmeshn(irigst+j)

            if ( debug ) &
               write(irefwr,1) 'j, nodp1, nodp2, iwork()', &
                                j, nodp1, nodp2, iwork(nodp1), &
                                iwork(nodp2)

            if ( nodp1>0 .and. nodp2>0 ) then

!           --- Both nodes are present

               if ( iwork(nodp1)<0 ) then

!              --- iwork(nodp1)<0, the node is excluded

                  iwork(nodp1) = 1

               else if ( iwork(nodp1)==0 ) then

!              --- iwork(nodp1) = 0, the node has not been used yet

                  iwork(nodp1) = 1

!                 --- Fill number of right neighbor unknowns in istoreleft and
!                     number of left neighbor unknowns in istoreright

                  numpairs = numpairs+1
                  nodepairs(1,numpairs) = nodp1
                  nodepairs(2,numpairs) = nodp2
                  call topbv7012 ( nodp1, nodp2, indprf, kprobf, &
                                   idegper, nunkp, istoreleft, &
                                   istoreright )

               end if  ! ( iwork(nodp1)<0 )

            end if  ! ( nodp1>0 .and. nodp2>0 )

         end do  ! j = 1, numpoints

         deallocate ( iwork, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iwork' )
         if ( ierror/=0 ) go to 1000

      end do  ! i = 1, numsrfs
      iinp = iinp+3*numsrfs

!     --- Accumulate arrays istoreleft and istoreright

      call  topbv5012 ( nusol, istoreleft, istoreright )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( istoreleft, nusol+1, 'istoreleft' )
         call prinin ( istoreright, nusol+1,'istoreright' )
         call prinin1 ( nodepairs, numpairs, 2, 'nodepairs' )
         write(irefwr,*) 'End topbv701'

      end if  ! ( debug )
      call erclos ( 'topbv701' )

      end
