      subroutine topfa05skip ( nmeshflc, melpef, lpnmpf, &
                               iskipelem, nelemskip, ishapevolm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 22-05-2014 Extension with shape 22
!        version  3.0  date 11-09-2013 New parameter list
!        version  2.1  date 10-03-2006 Extension with interface elements
!
!   copyright (c) 2002-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     creates nmesh part f, i,e. volume elements connected to faces
!     In this case elements marked in iskipelem are skipped
!     Copy of subroutine topfa05
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     element
!     surface
!     face
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nmeshflc(*), melpef, lpnmpf, iskipelem(*), nelemskip, ishapevolm

!     ishapevolm     o    shape number of volume element
!     iskipelem      i    Array with element numbers that must be skipped
!     lpnmpf        i/o   length of nmeshflc
!                         At input: declared length
!                         At output: computed length
!     melpef         o    maximum number of elements connected with an face
!     nelemskip      i    Number of elements in array iskipelem
!     nmeshflc         o    nmesh part f
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipf, iface, ipa, jfapel, &
              jelpef, ielem, i, ielgrp
      integer, allocatable :: iworkshape(:), iwork(:)

!     i              Counting variable
!     ielem          Element number
!     ielgrp         element group number
!     iface          Face sequence number
!     ipa            pointer in kmeshaaa
!     ipf            pointer in kmeshaaf
!     iwork          integer work array of length nfaces that is used to
!                    store the number of elements coupled to a face
!     iworkshape     integer work array of length nelem
!                    iworkshape(i) = 0: element is not a volume element
!                    iworkshape(i) = 1: element is a volume element
!     jelpef         number of elements on face
!     jfapel         face in an element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!     800   Shape number not available
!    1939   More than 2 elements connected to a face
!    2423   Not all volume elements have the same shape
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork
!     Fill array iworkshape
!     Fill iwork with number of volume elements coupled to each face
!     Fill first part of nmeshflc
!     Clear array iwork
!     Fill rest of nmeshflc
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topfa05skip' )

      allocate ( iworkshape(nelem), iwork(nfaces), stat = error )
      if ( error/=0 ) call eralloc ( error, nelem, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork(1:nfaces) = 0

!     --- Fill array iworkshape

      iworkshape = 1

!     --- Mark all elements that must be skipped

      iworkshape(iskipelem(1:nelemskip)) = 0

      ishapevolm = 0
      if ( nelgrp>1 ) then

!     --- there are several element groups, hence there may be surface elements

         ielem = 0
         do ielgrp = 1, nelgrp
            ishape = kmeshf(ielgrp)
            inelem = kmeshg(ielgrp)
            if ( ishape==19 .or. ishape==20 .or. ishape>22 .and. &
                 ishape<50 .or. ishape>59 .and. &
                 ishape<71 .or. ishape>80 ) then

!           --- Shape number not implemented

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               go to 1000

            end if
            if ( ishape<=10 .or. ishape>=50 ) then

!           --- surface element or interface element found

               do i = 1, inelem
                  iworkshape(ielem+i) = 0
               end do

            else if ( ishapevolm==0 ) then

!           --- Set ishapevolm equal to ishape

               ishapevolm = ishape

            else if ( ishapevolm==21 .or. ishape==21 ) then

!           --- Set ishapevolm equal to ishape

               ishapevolm = 21

            else if ( ishapevolm==22 .or. ishape==22 ) then

!           --- Set ishapevolm equal to ishape

               ishapevolm = 22

            else if ( ishapevolm/=ishape ) then

!           --- Not all volume elements have the same shape

               call errint ( ishapevolm, 1 )
               call errint ( ielgrp, 2 )
               call errint ( ishape, 3 )
               call errsub ( 2423, 3, 0, 0 )

            end if
            ielem = ielem+inelem

         end do

      else

!     --- Only one element group

         ishapevolm = kmeshf(1)

      end if

!     --- Fill iwork with number of elements coupled to each face

      length = 2*nfaces
      do ielem = 1, nelem

         if ( iworkshape(ielem)>0 ) then

!        --- volume element

            ipa = nmesha(ielem)
            nfapel = nmesha(ipa)/2
            do jfapel = 1, nfapel
               iface = abs(nmesha(ipa+jfapel))
               iwork(iface) = iwork(iface)+1
            end do
            length = length+nfapel

         end if

      end do

      if ( length>lpnmpf ) then

!     --- error: length of array nmeshflc too small

         call errint ( length, 1 )
         call errint ( lpnmpf, 2 )
         call errchr ( 'nmeshflc', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if
      lpnmpf = length

!     --- Fill first part of nmeshflc

      ipf = nfaces+1
      melpef = 0
      do iface = 1, nfaces

         nmeshflc(iface) = ipf
         jelpef = iwork(iface)
         nmeshflc(ipf) = jelpef
         ipf = ipf+iwork(iface)+1
         melpef = max(melpef,jelpef)
         if ( jelpef>2 ) then

!        --- More than two elements connected to face

            call errint ( iface, 1 )
            call errint ( jelpef, 2 )
            call errsub ( 1939, 2, 0, 0 )

         end if

      end do

!     --- Clear array iwork

      iwork = 0

!     --- Fill rest of nmeshflc

      do ielem = 1, nelem

         if ( iworkshape(ielem)>0 ) then

!        --- volume element

            ipa = nmesha(ielem)
            nfapel = nmesha(ipa)/2
            do jfapel = 1, nfapel
               iface = abs(nmesha(ipa+jfapel))
               iwork(iface) = iwork(iface)+1
               ipf = nmeshflc(iface)
               nmeshflc(ipf+iwork(iface)) = ielem
            end do

         end if

      end do

      deallocate ( iworkshape, iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iworkshape' )

1000  call erclos ( 'topfa05skip' )

      end
