      subroutine topren ( jrenum, meshsplt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.15 date 12-06-2015 New call to ch0003
!        version  4.14 date 28-08-2013 New call to chtmc6
!        version  4.13 date 15-11-2012 Long integers
!
!   copyright (c) 1993-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Renumber nodal point numbers in order to get a "minimal" profile
!     or a small band width (Either the method of Sloan or Cuthill McKee)
! **********************************************************************
!
!                       KEYWORDS
!
!     renumbering
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      use sepmodulework
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jrenum(*), meshsplt(*)

!     jrenum     i    Array of length 10 with information about the
!                     renumbering.
!                     Array must be filled as follows:
!                      1: if>=0, only position 1 is filled.
!                         This position contains the parameter method according
!                         to the old SEPRAN definition, i.e.
!                         jrenum = 9: Use default depending on mesh
!                         jrenum = 11: Use nested dissection
!                         else:
!                         jrenum = ispecial*100 + method + 3 * ipref
!                         ispecial = 0: standard
!                                    1: ivertx=1, imidp=1, icentr=3
!                         If<0, -jrenum(1) contains the sequence number
!                         of the last entry filled by the user
!                         All other values are filled with defaults
!                      2: method Indication of the renumbering method used
!                                The algorithms are compared to the original
!                                numbering:
!                                0  No renumbering
!                                1  Cuthill-Mckee algorithm
!                                2  Sloan algorithm
!                                3  Both algorithms,  the best is used
!                                9  Choose depending on mesh
!                         Default: 9
!                      3: ipref  Indication of the criterion to be used to find
!                                the best algorithm:
!                                0  Choose the smallest profile
!                                1  Choose the smallest band width
!                                2  Use only the renumbered algorithm (method=1
!                                   or 2)
!                         Default: 0
!                      4: iprint Indicates if information about the renumbering
!                                must always be printed (1) or only depending on
!                                ioutp (0)
!                         Default: 0
!                      5: ivertx Priority for vertex nodes (1-3)
!                         Default: 3
!                      6: imidp  Priority for mid point nodes (1-3)
!                         Default: 3
!                      7: icentr Priority for centroids (1-3)
!                         Default: 3
!                      8: ilevel Indicates if renumbering of specia nodes must
!                                be performed per level (1) or not (0)
!                         Default: 0
!                      9: istmet Indicates how the start of the renumbering
!                                algorithm must be performed
!                                0  Automatic
!                                1  Start user point given
!                                2  Start curve given
!                                3  Start surface given
!                                4  Start node given
!                         Default: 0
!                     10: istval Value of start point, curve f surface
!                         Default: 1
!     meshsplt   o    In this array the largest node number of the various
!                     disconnected parts of the mesh are stored
!                     Length: number of parts (<npoint)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision bwid, prof
      integer (kind=8) :: lenpf1, lenpf2, lenpf3
      integer i, ipref, lenstr, method, nbwid1, nbwid2, nbwid3, &
              ivertx, imidp, &
              icentr, iprint, ilevel, istmet, istval, &
              ifirst, ielgrp, methodsave, lenstrsav, irenum(npoint)
      logical renum, backwd, linear, trian
      integer, allocatable :: itemp(:), levels(:)
      double precision :: timebefore(2)

!     backwd         If true the previous renumbering must be used
!     bwid           Relative gain in band width
!     i              Counting variable
!     icentr         priority number corresponding to centroid
!     ielgrp         element group number
!     ifirst         The nodes from ifirst must be renumbered
!     ilevel         If 1 renumbering must be performed per level, if 0 not
!     imidp          priority number corresponding to mid point
!     ipref          Sequence number of preference
!                    0:   profile
!                    1:   band
!                    2:   best
!     iprint         If 1 information must be printed
!     irenum         work array
!     istmet         Indicates the method for computing the start for
!                    renumbering
!                    0:  default
!                    1:  user point
!                    2:  curve
!                    3:  surface
!     istval         Depending on istmet start value for user point, &
!                    curve or surface
!     itemp          temporary work array
!     ivertx         priority number corresponding to vertex
!     lenpf1         Length of first profile
!     lenpf2         Length of second profile
!     lenpf3         Length of third profile
!     lenstr         Length of start level
!     lenstrsav      Help variable to store the value of lenstr
!     levels         Contains the levels in the mesh
!     linear         Indicates whether the element is a linear simplex
!                    (true) or not (false)
!     method         defines type of renumbering applied
!     methodsave     Help variable to save the value of method
!     nbwid1         First band width
!     nbwid2         Second band width
!     nbwid3         Third band width
!     prof           Relative gain in profile
!     renum          If true the renumbering must be applied
!     trian          If true triangles are found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CH0003         Compute band width and profile
!     CHECKHOLES     Find and print all user points, curves, surfaces and
!                    volumes that have not been connected to the first part of
!                    the mesh
!     CHPSPN         Find start set for renumbering
!     CHSEQ1         Renumber nodes such that vertices, mid points and centroids
!                    get a sequence depending on the priority
!     CHTMC6         Check the number of disconnected parts in the matrix
!     CHTMCK         Renumber nodes using Cuthill-McKee
!     CHTSLO         Renumber nodes using Sloan
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLMN         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     PRININ         print 1d integer array
!     SEPFILLKMESHO  Fill kmesh part o
!     SEPNESTEDDIS   Renumbering with nested dissection
!     SEPPRINKMESH   Print array KMESH
!     TOPBV4         Renumber nodes in case of spectral elements
!     TOPLEV         Fill array levels
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     584   Value of jrenum is incorrect
!     827   The combination ipref=2 and method=3 is not allowed in renumbering.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topren' )
      debug = .false.
      if ( itime>0 ) call ersettime ( timebefore )
      if ( debug ) then

!     --- debug statements

         write(irefwr,*) ' in topren'
         call sepprinkmesh ( 1 )
  1      format ( a, 1x, (10i10) )

      end if

!     --- Compute ipref and method

      ivertx = 3
      imidp  = 3
      icentr = 3
      method = 9
      ipref  = 0
      iprint = 0
      ilevel = 0
      istmet = 0
      istval = 0
      nlevel = 0  ! initialization

      if ( jrenum(1)>0 ) then

!     --- jrenum(1)>0, old situation
!         get parameters from jrenum(1)

         method = mod(jrenum(1),100)
         if ( jrenum(1)>=100 ) then
            ivertx = 1
            imidp  = 1
         end if
         if ( method<9 ) then
            ipref = (jrenum(1)-1)/3
            method = jrenum(1)-3*ipref
         else if ( method==11 ) then
            method = 5
            ipref = 2
         end if

      else

!     --- jrenum(1)<0, new situation
!         get parameters from array jrenum

         if ( jrenum(1)<=-2 ) method = jrenum(2)
         if ( jrenum(1)<=-3 ) ipref  = jrenum(3)
         if ( jrenum(1)<=-4 ) iprint = jrenum(4)
         if ( jrenum(1)<=-5 ) ivertx = jrenum(5)
         if ( jrenum(1)<=-6 ) imidp  = jrenum(6)
         if ( jrenum(1)<=-7 ) icentr = jrenum(7)
         if ( jrenum(1)<=-8 ) ilevel = jrenum(8)
         if ( jrenum(1)<=-9 ) istmet = jrenum(9)
         if ( jrenum(1)<=-10) istval = jrenum(10)

      end if
      if ( ioutp>=1 ) iprint = 1

      if ( debug ) then
         write(irefwr,1) 'nelgrp, npoint, nelem', nelgrp, npoint, nelem
         write(irefwr,1) 'method, ipref, iprint, ivertx, imidp, icentr', &
                          method, ipref, iprint, ivertx, imidp, icentr
         write(irefwr,1) 'ilevel, istmet, istval', &
                          ilevel, istmet, istval
      end if  ! ( debug )

!     --- Determine nspec

      if ( kelmu>0 ) then

         trian = .false.
         do ielgrp = 1, nelgrp
            if ( kmeshf(ielgrp)==3 ) trian = .true.
         end do
         if ( trian ) then
            nspec = 0
         else
            nspec = kmeshu(1)
         end if

      else

         nspec  = 0

      end if

      renum  = .true.
      backwd = .false.
      methodsave = method
      if ( method==9 ) then

!     --- default

         if ( nspec>1 .or. ndim==3 .and. npoint>20000 ) then

!        --- in case of spectral elements, with at least two points between, &
!            a special form of Cuthill-Mckee is used

            method = 1

         else

            method = 2

         end if

      end if

      if ( method<1 .or. method>5 ) then
         call errint ( 2, 1 )
         call errint ( method, 2 )
         call errsub ( 584, 2, 0, 0 )
      end if
      if ( ipref<0 .or. ipref>2 ) then
         call errint ( 3, 1 )
         call errint ( ipref, 2 )
         call errsub ( 584, 2, 0, 0 )
      end if
      if ( method==3 .and. ipref==2 ) then
         call errsub ( 827, 0, 0, 0 )
      end if

!     --- Create kmesh parts e and o if necessay

      if ( nspec>1 ) call sepfillkmesho ! spectral elements

!     --- Compute profile and band width of original numbering

      if ( debug ) write(irefwr,*) 'Before ch0003/1'
      call ch0003 ( .false., lenpf1, nbwid1, irenum )
      if ( debug ) write(irefwr,1) 'lenpf1, nbwid1', lenpf1, nbwid1

!     --- Create permanent array kmesh part j and semi  permanent array itemp
!         The last one is deleted at the end of this subroutine

      allocate ( km(iseqkm)%kmeshj(npoint) )
      kmeshj => km(iseqkm)%kmeshj
      kelmj = npoint
      if ( nspec>1 ) then
         length = max ( npoint, nelem )
      else
         length = npoint
      end if

      if ( method==5 ) then

!     --- method = 5, nested dissection

         call sepnesteddis
         go to 1000

      end if  ! ( method==5 )

!     --- Reserve work space

      allocate ( marknodes(npoint), levels(length), &
                 iwork(length), itemp(npoint), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, length+npoint, 'marknodes' )
      if ( ierror/=0 ) go to 1000

      allocate ( levelstruc(npoint), pointlevel(npoint+1) )

!     --- Check if the mesh consists of sub parts

      if ( debug ) write(irefwr,*) 'Before chtmc6'
      call chtmc6 ( meshsplt )
      if ( methodsave==9 .and. nsplit>1 ) method = 2

      if ( debug ) then

!     --- debug statements

         write(irefwr,*) 'ierror', ierror

      end if

      if ( ierror==1654 ) then

!     --- The mesh consists of parts that are not connected
!         In this case we must give a message of which nodes, curves, surfaces
!         and volumes are not connected

         ierror = 0
         nerror = 0

!        --- Finally the message is given

         if ( debug ) write(irefwr,*) 'Before checkholes'
         call checkholes ( marknodes, meshsplt )
         ierror = 1654
         nerror = 1
         go to 1000

      end if
      if ( ierror/=0 ) go to 1000
      ifirst = 1
      lenstr = 0

      if ( debug ) then

!     --- debug statements

         write(irefwr,*) 'ifirst, method, nspec ', ifirst, method, nspec

      end if

      if ( method==2 .or. nspec<=1 ) then

!     --- Compute starting set for renumbering and store in array istart

         if ( debug ) write(irefwr,*) 'Before chpspn'
         call chpspn ( itemp, lenstr, istmet, istval, &
                       linear, ifirst, meshsplt(1) )
         if ( debug ) &
            call prinin ( itemp, lenstr, 'start set' )

      end if
      lenstrsav = lenstr

      if ( method/=2 ) then

!     --- method = 1 or 3   Cuthill-Mckee algorithm

          if ( nspec<=1 ) then

!         --- Standard case

            if ( debug ) write(irefwr,*) 'Before chtmck'
            call chtmck ( itemp, lenstr, meshsplt )

        else

!       --- Spectral elements with at least two points between each vertex

            call topbv4

         end if

         if ( debug ) then

!        --- debug statements

            write(irefwr,*) 'Cuthill-Mckee renumbering'
            call prinin ( kmeshj, npoint, 'kmeshj in topren' )

         end if

!        --- Store inverse of jrenum in irenum
!            compute renumbered profile and band width

         do i = 1, npoint
            irenum(kmeshj(i)) = i
         end do
         if ( debug ) write(irefwr,*) 'Before ch0003/2'
         call ch0003 ( .true., lenpf2, nbwid2, irenum )
         if ( debug ) write(irefwr,1) 'lenpf2, nbwid2', lenpf2, nbwid2

         prof = lenpf2
         if ( lenpf1>0 ) then

!        --- Standard case, profile may be computed

            prof = prof/lenpf1

         else

!        --- The matrix is too large, profile may not be computed

            prof = 0
            lenpf2 = 0

         end if  ! ( lenpf1>0 )

         bwid = nbwid2
         bwid = bwid/nbwid1
         if ( debug .or. iprint>=1 ) &
            write(irefwr, '(a/a, f8.3, 5x, a, f8.3)') &
            ' Cuthill-Mckee algorithm:', ' reduction in profile', &
            prof, ' reduction in band width', bwid
         if ( method==1 ) then

!        --- method = 1  Cuthill-Mckee algorithm only

            if ( ipref==0 ) then

!           --- ipref = 0  Check length of profile

               if ( lenpf1<lenpf2 .or. lenpf1==lenpf2 .and. &
                    nbwid1<=nbwid2 ) renum = .false.

            else if ( ipref==1 ) then

!           --- ipref = 1  Check band width

               if ( nbwid1<nbwid2 .or. nbwid1==nbwid2 .and. &
                    lenpf1<=lenpf2 ) renum = .false.

            end if

         else

!        --- All methods should be used
!            Copy renum to iwork

            iwork(1:npoint) = kmeshj

         end if

         if ( debug ) then

!        --- debug statements

            write(irefwr,*) 'renum ', renum
            write(irefwr,*) 'Final renumbering'
            call prinin ( kmeshj, npoint, 'kmeshj in topren' )

         end if

      end if

      if ( method==3 .and. lenstrsav>0 ) then

!     --- method = 3, hence both Cuthill-McKee and Sloan are used
!         Set start set back to first part of Cuthill-McKee numbering

         lenstr = lenstrsav
         itemp(1:lenstr) = kmeshj(1:lenstr)

      end if  ! ( method==3 )

      if ( method/=1 ) then

!     --- method = 2 or 3   Sloan algorithm

         if ( debug ) write(irefwr,*) 'Before chtslo'
         call chtslo ( levels, itemp, lenstr, istmet, ifirst, meshsplt )

         if ( debug ) then

!        --- debug statements

            write(irefwr,*) 'Sloan renumbering'
            call prinin ( kmeshj, npoint, 'kmeshj in topren' )

         end if

!        --- Store inverse of jrenum in irenum

         do i = 1, npoint
            irenum(kmeshj(i)) = i
         end do
         if ( debug ) write(irefwr,*) 'Before ch0003/3'
         call ch0003 ( .true., lenpf3, nbwid3, irenum )

         prof = lenpf3
         if ( lenpf1>0 ) then

!        --- Standard case, profile may be computed

            prof = prof/lenpf1

         else

!        --- The matrix is too large, profile may not be computed

            prof = 0
            lenpf3 = 0

         end if  ! ( lenpf1>0 )

         bwid = nbwid3
         bwid = bwid/nbwid1
         if ( debug .or. iprint>=1 ) &
            write(irefwr, '(a/a, f8.3, 5x, a, f8.3)') &
            ' Sloan algorithm:', ' reduction in profile', &
            prof, ' reduction in band width', bwid

         if ( method==2 ) then

!        --- method = 2  Sloan algorithm only

            if ( ipref==0 ) then

!           --- ipref = 0  Check length of profile

               if ( lenpf1<lenpf3 .or. lenpf1==lenpf3 .and. &
                    nbwid1<=nbwid3 ) renum = .false.

            else if ( ipref==1 ) then

!           --- ipref = 1  Check band width

               if ( nbwid1<nbwid3 .or. nbwid1==nbwid3 .and. &
                    lenpf1<=lenpf3 ) renum = .false.

            end if

         else

!        --- All methods should be used

            if ( ipref==0 ) then

!           --- ipref = 0  Check length of profile

               if ( lenpf1>min(lenpf3, lenpf2) ) then

!              --- New profile smaller than original one

                  if ( lenpf2<lenpf3 ) backwd = .true.

               else if ( lenpf1==lenpf2 ) then

!              --- Both profiles have equal length, check on band width

                  if ( nbwid1<=nbwid2 ) then
                     renum = .false.
                  else
                     backwd = .true.
                  end if

               else if ( lenpf1==lenpf3 ) then

!              --- Both profiles have equal length, check on band width

                  if ( nbwid1<=nbwid3 ) renum = .false.

               else

!              --- New profile larger than original one

                  renum = .false.

               end if

            else if ( ipref==1 ) then

!           --- ipref = 1  Check band width

               if ( nbwid1>min(nbwid2, nbwid3) ) then

!              --- New band width smaller than original one

                  if ( nbwid2<=nbwid3 ) backwd = .true.

               else if ( nbwid1==nbwid2 ) then

!              --- Both band widths have equal length, check profile

                  if ( lenpf1<=lenpf2 ) then
                     renum = .false.
                  else
                     backwd = .true.
                  end if

               else if ( nbwid1==nbwid3 ) then

!              --- Both band widths are equal, check profiles

                  if ( lenpf1<=lenpf3 ) renum = .false.

               else

!              --- New band width larger than original one

                  renum = .false.

               end if

            end if

         end if

      end if

      if ( debug ) write(irefwr,*) 'renum, backwd ', renum, backwd

      if ( renum .and. backwd ) then

!     --- backwd = true, use Cuthill McKee

         kmeshj = iwork(1:npoint)

      end if

      if ( debug ) &
         write(irefwr,*) 'linear, ivertx, imidp, icentr', &
                          linear, ivertx, imidp, icentr
      if ( .not. linear .and. (ivertx/=imidp .or. ivertx/=icentr) ) then

!     --- Renumber centroids, mid points and vertices according to
!         priority

         if ( .not. renum ) then

!        --- No renumbering used until now
!            Create and initialize renumbering array

            do i = 1, npoint
               kmeshj(i) = i
            end do
            kelmj = 0

         end if

         if ( ilevel/=0 ) then
            call toplev ( levels )
         end if
         call chseq1 ( imidp, ivertx, icentr, ilevel, levels )

      else if ( .not. renum ) then

!     --- renum false, no renumbering

         deallocate ( km(iseqkm)%kmeshj )
         nullify(kmeshj)
         kelmj = 0

      end if  ! ( .not. linear .and. (ivertx/=imidp .or. ivertx/=icentr) )

      deallocate ( marknodes, levels, iwork, itemp, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'marknodes' )

      deallocate ( levelstruc, pointlevel )

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ilevel, nlevel', ilevel, nlevel
         call prinin ( kmeshj, npoint, 'kmeshj' )
         write(irefwr,*) 'End topren'

      end if

1000  call erclmn ( 'topren', timebefore )
      end subroutine topren
