      subroutine topsurfaces
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 26-01-2009 Remove parameter iwork
!        version  2.0  date 16-05-2004 Extra parameter iwork
!        version  1.0  date 03-03-2004
!
!   copyright (c) 2004-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find the face numbers of all faces stored in the given surfaces
!     Almost the same as topsu02 except that elssurfs is not filled
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     face
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, iface, jface, numfaces, &
              ipkmeshn, istart, inpelm, ielem, idummy(1), &
              isurnr, isufnd, jstart, nelemlc
      integer, allocatable :: iwork(:)
      double precision dummy(1)
      character (len=21) text

!     dummy          Dummy parameter
!     i              Counting variable
!     idummy         Dummy parameter
!     ielem          Element number
!     iface          Face sequence number
!     inpelm         number of points per element
!     ipkmeshn       Starting address in kmeshn
!     istart         First position used at a part in this case in nmeshj
!     isufnd         i'th surface node
!     isurnr         Surface sequence number
!     iwork          integer work array of size nfaces
!     j              Counting variable
!     jface          Face sequence number
!     jstart         starting position in array
!     nelemlc          Number of elements
!     numfaces       number of faces
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PR0009         Print information of surfaces
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     363   No face found
!    2626   Face appears more than once in surface
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork(1:nfaces)
!     Initialize pointer istart
!     For isurnr := 1 (1) nsurfs do
!        extract inpelm and inelem from kmeshn
!        For ielem := 1 (1) inelem do
!           isufnd := first node in element ielem
!           Set iwork(iface) = 1 for all faces iface connected to isufnd
!           For all other nodes inode in element do
!              For all faces iface connected to node do
!                 if ( iwork(iface)>0 ) then
!                    iwork(iface) := iwork(iface)+1
!           iface := position in iwork with iwork(iface) = inpelm
!           Clear iwork
!           Store information in facesurfs
!        Raise istart
!     Check if double faces are present in surface
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topsurfaces' )

      allocate ( iwork(nfaces), stat = error )
      if ( error/=0 ) then

!     --- error#0, so not enough space

         call errint ( nfaces, 1 )
         call errint ( error, 2 )
         call errchr ( 'iwork', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 1000

      end if  ! ( error/= 0 )

      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topsurfaces'
         call  pr0009 ( 0, kmeshn, dummy, idummy, &
                        idummy(1), idummy, nsurfs )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork(1:nfaces)

      do i = 1, nfaces
         iwork(i) = 0
      end do

!     --- Initialize pointer istart

      istart = 0
      ipkmeshn = 2*nsurfs

!     --- Loop over the surfaces

      do isurnr = 1, nsurfs

!     --- extract inpelm and inelem from kmeshn

         inpelm = kmeshn(isurnr)
         inelem = kmeshn(nsurfs+isurnr)

!        --- Loop over the elements

         do ielem = 1, inelem

!        --- isufnd := first node in element ielem

            isufnd = kmeshn(ipkmeshn+1)
            if ( isufnd>0 ) then

!           --- Node connected to mesh
!               Set iwork(iface) = 1 for all faces iface connected to isufnd

               jstart = nmeshk(isufnd)
               numfaces = nmeshk(jstart)
               iwork(nmeshk(jstart+1:jstart+numfaces)) = 1
               jface = 0

!              --- For all other nodes inode in element do

               do j = 2, inpelm

!              --- For all faces iface connected to node do

                  jstart = nmeshk(kmeshn(ipkmeshn+j))
                  numfaces = nmeshk(jstart)
                  do i = 1, numfaces
                     iface = nmeshk(jstart+i)
                     if ( iwork(iface)>0 ) then

!                    --- iwork(iface)>0
!                        iwork(iface) := iwork(iface)+1

                        iwork(iface) = iwork(iface)+1
                        if ( iwork(iface)==inpelm ) jface = iface

                     end if

                  end do

               end do

!              --- iface := position in iwork with iwork(iface) = inpelm

               iface = jface
               if ( iface==0 ) then

!              --- did not find a face, weird

                  call errint ( isurnr, 1 )
                  call errint ( ielem, 2 )
                  call errint ( ielem, 2 )
                  do i = 1, inpelm
                     call errint ( kmeshn(ipkmeshn+i), 2+i )
                  end do
                  call errsub ( 363, 2+inpelm, 0, 0 )

               end if

!              --- Clear iwork

               jstart = nmeshk(isufnd)
               numfaces = nmeshk(jstart)
               iwork(nmeshk(jstart+1:jstart+numfaces)) = 0

!              --- Store information in elssurfs

               istart = istart + 1
               nmeshj(istart) = iface

            else

!           --- Surface element has not been filled

               istart = istart + 1
               nmeshj(istart) = 0

            end if  ! ( isufnd>0 )

            ipkmeshn = ipkmeshn+inpelm

         end do  ! ielem = 1, inelem

      end do  ! isurnr = 1, nsurfs

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Face numbers of surfaces'
         istart = 1

         do  isurnr = 1, nsurfs

            inelem = kmeshn(nsurfs+isurnr)
            write(text,1010) isurnr
1010        format ( 'surface number ', i6 )
            call prinin ( nmeshj(istart), inelem, text )
            istart = istart+inelem

         end do  ! isurnr = 1, nsurfs

      end if

!     --- Check if all face numbers per surface are unique

      iwork = 0

      ipkmeshn = 2*nsurfs
      istart = 0
      nelemlc = 0
      do isurnr = 1, nsurfs

!     --- extract inpelm and inelem from kmeshn

         inpelm = kmeshn(isurnr)
         inelem = kmeshn(nsurfs+isurnr)

         do ielem = 1, inelem

            iface = nmeshj(istart+ielem)
            if ( iwork(iface)/=0 ) then

!           --- Double face found: error

               call errint ( isurnr, 1 )
               call errint ( iface, 2 )
               call errint ( iwork(iface), 3 )
               call errint ( nelemlc+iwork(iface), 4 )
               call errint ( ielem, 5 )
               call errint ( nelemlc+ielem, 6 )
               do i = 1, inpelm
                  call errint ( kmeshn(ipkmeshn+(ielem-1)*inpelm+i), 6+i )
               end do
               call errsub ( 2626, inpelm+6, 0, 0 )

            else

!           --- Face not used yet

               iwork(iface) = ielem

            end if

         end do  ! ielem = 1, inelem

!        --- Clear iwork

         do ielem = 1, inelem
            iwork(nmeshj(istart+ielem)) = 0
         end do  ! ielem = 1, inelem

!        --- Raise starting addresses

         istart = istart+inelem
         nelemlc = nelemlc+inelem
         ipkmeshn = ipkmeshn+inpelm*inelem

      end do  ! isurnr = 1, nsurfs

1000  deallocate ( iwork, stat = error )
      if ( error/=0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'iwork', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/=0 )

      call erclos ( 'topsurfaces' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topsurfaces'

      end if

      end
