      subroutine el0325 ( iuser, user, jelp, ibear, newton, kfirst, &
                          isub, isup, array, iseq )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.7  date 07-01-2015 extension with string for linearization
!        version  3.6  date 05-11-2014 Debug statements
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine ELM325
!     and ELD325 if ICHELD = 22
!     in the case that IFIRST = 0
!     The parameters in the common blocks celint, celiar and celp
!     get a start value
!     Array array is initialized with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     bearing
!     compressible
!     incompressible
!     element_matrix
!     element_vector
!     lubrication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), array(*)
      integer iuser(*), jelp, ibear, newton, isub, isup, kfirst, &
              iseq

!     array   o      Real work array in which the values of the coefficients
!                    in the integration points are stored in the sequence:
!                         1D                2D
!                    1:   h                h
!                    2:   mu               mu
!                    3:   ht               ht
!                    4:   k                k
!                    5:   p0               p0
!                    6:   u_1 1            u_1 1
!                    7:                    u_1 2
!                    8:
!                    9:   rho              rho
!                   10:   RT               RT
!                   11:   f1               f1
!                   12:                    f2
!                   13:   u_2 1            u_2 1
!                   14:                    u_2 2
!                    Each coefficient requires exact m (is number of
!                    integration points) positions
!     ibear    o     Indicates type of bearing.
!                    Possible values:
!                    0:  incompressible bearing
!                    1:  compressible bearing (adiabatic)
!     iseq     o     Sequence number (1)
!     isub     o     Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup     o     Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     iuser    i     User array in which integer information of the
!                    coefficients is stored (See Standard Problems)
!     jelp     o     Indication of the type of basis function subroutine must
!                    be called by ELM325. Possible vaues:
!                    1:  Subroutine ELP610  (Linear line element)
!                    2:  Subroutine ELP611  (Quadratic line element)
!                    3:  Subroutine ELP620  (Linear triangle)
!                    4:  Subroutine ELP622  (Quadratic triangle)
!                    5:  Subroutine ELP621  (Bilinear quadrilateral)
!                    6:  Subroutine ELP623  (Biquadratic quadrilateral)
!     kfirst   o     This parameter indicates the smallest velocity component
!                    that is not equal to zero in the convective terms
!                    For example kfirst = 2, means that the y- &
!                    component of the velocity is unequal to zero
!                    kfirst = 0, means that the first derivative is zero
!     newton   o     Indicates type of linearization in case of compressible
!                    bearing.
!                    Possible values:
!                    0:  Newton linearization
!                    1:  Picard linearization
!     user     i     User array in which real information of the
!                    coefficients is stored (See Standard Problems)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode=3
      integer :: ncode(nncode)
      integer ncoef, i, j, isubcf, mrule, ishift
      character (len=15) :: code (nncode)
      logical :: funchk

!     code           Table with subkeywords for string parameters
!                    corresponding to type_of_bearing
!     funchk         Indicates whether there are coefficients that must be
!                    evaluated by funcc3 (true) or not (false)
!     i              Counting variable
!     ishift         Shift in coefficient sequence number
!     isubcf         Under bound for coefficients check
!     j              Counting variable
!     mrule          Type of integration rule as given by the user
!     ncode          Table containing the number of significant characters
!                    in the keyword table code
!     ncoef          Number of coefficients in the equation
!     nncode         Number of entries in code
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: inirepscal, elgetintcoef, elfilllinear

!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ELFILLLINEAR   Set elfilllinear equal to value following linearization
!     ELGETINTCOEF   Give elgetintcoef the actual integer value corresponding to
!                    iuser(ipos)
!     ELI325NM       Fill the parameters jelp, n, m, nm and jdercn for
!                    bearing element
!     ELUSR3         Extracts information concerning the coefficients from IUSER
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INIREPSCAL     Replace ivalue by scalar if necessary
!     PRININ         print 1d integer array
!     PRINLG         Print and checks length of user arrays
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     See subroutine elm300 for the options
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!    1304   For coefficient only positive constants are allowed
!    1305   Coefficient has wrong value
!    1766   Type of integration rule incorrect
!    1767   Type of co-ordinates incorrect
!    1768   Coefficient is filled that is not used
!    2437   No transformation given
!    2771   ibear = 2 is only allowed in combination with irule = 1
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code  / 'INCOMPRESSIBLE', 'COMPRESSIBLE', 'MASS_CONSERVING' /
      data ncode  / 14,12,15 /
! ======================================================================
!
!     --- Fill some standard values in the common blocks celp and celint

      call eropen ( 'el0325' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0325'
         write(irefwr,1) 'jelp', jelp
         call prinin ( iuser, 40, 'iuser' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call elcomm
      iweigh = 21
      jsecnd = 2
      iseq   = 1
      ishift = 0

!     --- if derivatives must be computed, the weights are not necessary

      if ( icheld>0 ) ndimlc = ndim

!     --- Set parameters depending on the shape numbers

      call eli325nm ( jelp )

      ncoef = 14

!     --- Fill the arrays IND, MST and ARRAY with information of the
!         coefficient

      ipos = iuser ( ielgrp + 5 )
      if ( ipos<7 ) then
         call errint ( ielgrp+5, 1 )
         call errint ( ipos, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 42, 5, 0, 0 )
      end if

      if ( debug ) write(irefwr,1) 'ipos', ipos

      notmas = 1
      nuser = 0

!     --- Compute the parameters ibear and newton

      ibear = elgetintcoef ( iuser, ipos, ishift, code, nncode, ncode, &
                             1, 'type_of_bearing'  )
      if ( ibear<0 .or. ibear>2 ) then
         call errint ( 1, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errint ( ielem, 4 )
         call errint ( ibear, 5 )
         call errchr ( 'Type of bearing', 1 )
         call errsub ( 1305, 5, 0, 1 )
      end if

!     --- Since picard and newton have the reversed sequence of the standard
!         sequence we have to perform a trick to make newton 0 and picard 1
!         standard is newton 2 and picard 1
!         So first we add 1 in case of given text to distinguish with the
!         standard and afterwards we correct

      newton = elfilllinear ( iuser, ipos, ishift, -1 )
      if ( newton>=2 ) newton = -newton+3

      if ( ibear==1 .and. (newton<0 .or. newton>1) ) then
         call errint ( 2, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errint ( ielem, 4 )
         call errint ( newton, 5 )
         call errchr ( 'Type of linearization', 1 )
         call errsub ( 1305, 5, 0, 1 )
      end if

      if ( debug ) &
         write(irefwr,1) 'ibear, ipos, newton', ibear, ipos, newton

!     --- Compute the parameters jcart and irule

      mrule = inirepscal ( iuser(ipos) )
      irule = max ( 1, mrule )
      ipos = ipos + 1
      ishift = ishift+1
      if ( mrule<0 .or. mrule>40 ) then
         call errint ( mrule, 1 )
         call errint ( 3, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 1766, 5, 0, 0 )
      end if

      jcart = inirepscal ( iuser(ipos) )
      if ( jcart>9 .and. ndimlc==3 ) ndimlc = 2
      ipos = ipos + 1
      ishift = ishift+1
      if ( jcart<0 .or. mod(jcart,10)>2 .or. &
           ndim>2 .and. jcart/10>4 ) then
         call errint ( jcart, 1 )
         call errint ( 4, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 1767, 5, 0, 0 )
      end if
      jcart = jcart + 1
      if ( icheld/=0 ) irule = 1
      if ( ndim==3 .and. jcart/10==0 ) then

!     --- Warning standard transformation

         call errwar ( 2437, 0, 0, 0 )

      end if
      if ( debug ) &
         write(irefwr,1) 'mrule, irule, jcart, ndim, ipos', &
                          mrule, irule, jcart, ndim, ipos

      if ( irule/=1 ) then

!     --- irule>1, Gauss rule

         igausp = 1
         iphi = 1
         jdiag = 2
         if ( jelp<=2 ) then
            m = irule - 1
         else if ( jelp==3 .or. jelp==5 ) then
            if ( irule==2 ) then
               m = 1
            else
               m = 4
            end if
         else if ( jelp==4 ) then
            if ( irule==2 ) then
               m = 4
            else
               m = 7
            end if
         else if ( jelp==6 ) then
            if ( irule==2 ) then
               m = 4
            else
               m = 9
            end if
         end if
         mn = m * n

      end if
      ipos = ipos + 1
      ishift = ishift+1

      if ( debug ) write(irefwr,1) 'ipos, ishift, ncoef', ipos, ishift, ncoef

      do i = 1, ncoef-2
         call elusr3 ( i, array(1+(i-1)*m), iuser, user, i+ishift )
      end do
      ipos = ipos + 1
      do i =  ncoef-1, ncoef
         call elusr3 ( i, array(1+(i-1)*m), iuser, user, i+ishift )
      end do
      ishift = ishift+ncoef

      if ( debug ) call prinrl1 ( array, ncoef, m, 'array' )

!     --- Test some parameters

      if ( ind(1)==0 ) then

!     --- h = 0

         call errint ( 6, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errint ( ielem, 4 )
         call erreal ( 0d0, 1 )
         call errchr ( 'Thickness h of the layer', 1 )
         call errsub ( 1304, 4, 1, 1 )

      else if ( ind(1)<0 ) then

!     --- h is constant

         if ( array(1)<=0d0 ) then

!        --- h not positive

            call errint ( 6, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errint ( ielem, 4 )
            call erreal ( array(1), 1 )
            call errchr ( 'Thickness h of the layer', 1 )
            call errsub ( 1304, 4, 1, 1 )

         end if

      end if

      if ( ind(2)==0 ) then

!     --- mu = 0

         call errint ( 7, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errint ( ielem, 4 )
         call erreal ( 0d0, 1 )
         call errchr ( 'Viscosity mu', 1 )
         call errsub ( 1304, 4, 1, 1 )

      else if ( ind(2)<0 ) then

!     --- mu is constant

         if ( array(m+1)<=0d0 ) then

!        --- mu not positive

            call errint ( 7, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errint ( ielem, 4 )
            call erreal ( array(m+1), 1 )
            call errchr ( 'Viscosity mu', 1 )
            call errsub ( 1304, 4, 1, 1 )

         end if

      end if

      if ( ind(11)/=0 .or. ind(12)/=0 ) then

!     --- f # 0, test rho and RT

         if ( ibear/=1 ) then

!        --- incompressible bearing, test rho

            if ( ind(9)==0 ) then

!           --- rho = 0

               call errint ( 14, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errint ( ielem, 4 )
               call erreal ( 0d0, 1 )
               call errchr ( 'Density rho', 1 )
               call errsub ( 1304, 4, 1, 1 )

            else if ( ind(9)<0 ) then

!           --- rho is constant

               if ( array(8*m+1)<=0d0 ) then

!              --- rho not positive

                  call errint ( 14, 1 )
                  call errint ( itype, 2 )
                  call errint ( ielgrp, 3 )
                  call errint ( ielem, 4 )
                  call erreal ( array(8*m+1), 1 )
                  call errchr ( 'Density rho', 1 )
                  call errsub ( 1304, 4, 1, 1 )

               end if

            end if

         else

!        --- compressible bearing, test RT

            if ( ind(10)==0 ) then

!           --- RT = 0

               call errint ( 15, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errint ( ielem, 4 )
               call erreal ( 0d0, 1 )
               call errchr ( 'RT', 1 )
               call errsub ( 1304, 4, 1, 1 )

            else if ( ind(10)<0 ) then

!           --- RT is constant

               if ( array(9*m+1)<=0d0 ) then

!              --- RT not positive

                  call errint ( 15, 1 )
                  call errint ( itype, 2 )
                  call errint ( ielgrp, 3 )
                  call errint ( ielem, 4 )
                  call erreal ( array(9*m+1), 1 )
                  call errchr ( 'RT', 1 )
                  call errsub ( 1304, 4, 1, 1 )

               end if

            end if

         end if

      end if

      if ( ndim==1 ) then

!     --- ndim = 1, test if too many parameters have been filled

         if ( ind(7)/=0 ) then

!        --- u2 has been filled

            call errint ( 12, 1 )
            call errint ( ielem, 2 )
            call errint ( itype, 3 )
            call errint ( ielgrp, 4 )
            call errwar ( 1768, 4, 0, 0 )

         end if

         if ( ind(12)/=0 ) then

!        --- f2 has been filled

            call errint ( 17, 1 )
            call errint ( ielem, 2 )
            call errint ( itype, 3 )
            call errint ( ielgrp, 4 )
            call errwar ( 1768, 4, 0, 0 )

         end if

         if ( ind(14)/=0 ) then

!        --- u2 has been filled

            call errint ( 20, 1 )
            call errint ( ielem, 2 )
            call errint ( itype, 3 )
            call errint ( ielgrp, 4 )
            call errwar ( 1768, 4, 0, 0 )

         end if

      end if

      if ( ibear/=1 ) then

!     --- incompressible bearing

         if ( ind(10)/=0 ) then

!        --- RT has been filled

            call errint ( 15, 1 )
            call errint ( ielem, 2 )
            call errint ( itype, 3 )
            call errint ( ielgrp, 4 )
            call errwar ( 1768, 4, 0, 0 )

         end if

      else if ( ibear==1 ) then

!     --- compressible bearing

         if ( ind(9)/=0 ) then

!        --- rho has been filled

            call errint ( 14, 1 )
            call errint ( ielem, 2 )
            call errint ( itype, 3 )
            call errint ( ielgrp, 4 )
            call errwar ( 1768, 4, 0, 0 )

         end if

      end if

!     --- Set parameters jzero, kfirst, idim, jderiv and jconsf

      if ( ind(4)==0 ) then

!     --- k = 0

         jzero = 0

      else if ( ibear/=1 .and. ind(4)<0 ) then

!     --- k constant and incompressible bearing

         jzero = 3

      else

!     --- k variable or compressible bearing

         jzero = 2

      end if

      if ( ibear/=1 ) then

!     --- incompressible bearing

         kfirst = 0
         idim = 0

      else if ( newton==0 ) then

!     --- Newton linearization

         kfirst = 1
         idim = ndim

      else

!     --- Picard linearization

         if ( ind(6)/=0 .or. ind(11)/=0 .or. ind(13)/=0 ) then
            kfirst = 1
            if ( ind(7)==0 .and. ind(12)==0 .and. ind(14)==0 ) &
               then
               idim = ndim
            else
               idim = 1
            end if
         else if ( ind(7)/=0 .or. ind(12)/=0 .or. ind(14)/=0 ) &
            then
            kfirst = 2
            idim = 1
         else
            kfirst = 0
            idim = 0
         end if

      end if
      if ( icheld>0 .or. ibear==1 .and. newton==0 ) then
         jderiv = 1
      else
         jderiv = 0
      end if

      if ( ibear/=1 .or. ibear==1 .and. newton==1 ) then
         if ( ind(3)==0 .and. (ind(4)==0 .or. ind(5)==0) ) then
            jconsf = 0
         else
            jconsf = 2
         end if
      else
         jconsf = 2
      end if

      if ( ibear==2 .and. irule/=1 ) then

!     --- The combination of irule#1 and ibear=2 has not yet been implemented

         call errint ( irule, 1 )
         call errsub ( 2771, 1, 0, 0 )

      end if  ! ( ibear==2 .and. irule/=1 )

!     --- Test array lengths

      call prinlg ( 'iuser', ipos-1, user, iuser )
      call prinlg ( 'user', -nuser, user, iuser )

!     --- Check which parameters depend on the space co-ordinates
!         Compute isub and isup

      isubcf = 1

      do i = isubcf, ncoef
         isub = i

!        --- If ind(i)>0 then paramater i corresponds to a function

         if ( ind(i)>0 ) goto 170

      end do

      isub = ncoef + 1

170   isup = ncoef
      do i = ncoef, isub , -1
         isup = i
         if ( ind(i)>0 ) goto 190

      end do

!     --- Check for double parameters (functions) to avoid calls for
!         function subroutines
!         ist is used to store the references for the double parameters
!         Set ind(lengthmst)

190   do i = isub, isup
         ist(i) = 0
      end do
      ind(lengthmst) = 0
      if ( ibear==1 .or. icheld>0 ) ind(lengthmst) = 1
      funchk = .false.
      do i = isub, isup
         if ( ind(i)>0 .and. ind(i)<2000 ) then

!        --- only the function calls are checked

            do j = i+1,isup
               if ( ind(j)==ind(i) ) then
                  ist(j) = i
                  ind(j) = 0
               end if
            end do
         end if
         if ( ind(i)>1000 .and. ind(i)<=2000 ) ind(lengthmst) = 1
         if ( ind(i)>10000 ) funchk = .true.
      end do

      if ( funchk ) ind(lengthmst) = ind(lengthmst) + 2

      if ( jelp==4 ) kstep = 2
      jsol = 1

1000  call erclos ( 'el0325' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'idim, jzero, isub, isup, ncoef', &
                          idim, jzero, isub, isup, ncoef
         call prinin ( ind, isup, 'ind' )
         call prinin ( ist, isup, 'ist' )
         call prinrl1 ( array, ncoef, m, 'array' )
         write(irefwr,*) 'End el0325'

      end if  ! ( debug )

      end subroutine el0325
