      subroutine el0601 ( iuser, user, jelp, jboun, isub, isup, array )
! ======================================================================
!
!        programmer    Patrick Anderson
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.4  date 23-08-2001 New call to elusr3
!        version  1.3  date 03-09-2000 Extension for scalars
!
!   copyright (c) 1995-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine ELM601
!     in the case that IFIRST = 0
!     The parameters in the common blocks celint, celiar and celp
!     get a start value
!     Array array is initialized with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), array(*)
      integer iuser(*), jelp, jboun, isub, isup

!     array   o      Real work array in which the values of the coefficients
!                    in the integration points are stored in the sequence:
!                         1D                  2D            3D
!                    1:   sigma            sigma          sigma
!                    2:   h                h              h
!                    3:   alpha 11         alpha 11       alpha 11
!                    4:                    alpha 12       alpha 12
!                    5:                    alpha 22       alpha 13
!                    6:                                   alpha 22
!                    7:                                   alpha 23
!                    8:                                   alpha 33
!                    Each coefficient requires exact m (is number of
!                    integration points) positions
!     isub     o     Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup     o     Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     iuser    i     User array in which integer information of the
!                    coefficients is stored (See Standard Problems)
!     jboun    o     Defines the type of boundary condition.
!                    Possible values:
!                    2:  Standard mixed (Robbins) boundary condition
!                    3:  Boundary condition of type 3
!                    4:  Boundary condition of type 4 (not yet available)
!     jelp     o     Indication of the type of basis function subroutine must
!                    be called by ELM601. Possible vaues:
!                    0:  Point element, no special basis functions
!                    1:  Subroutine ELP610  (Linear line element)
!                    2:  Subroutine ELP611  (Quadratic line element)
!                    3:  Subroutine ELP620  (Linear triangle)
!                    4:  Subroutine ELP622  (Quadratic triangle)
!                    5:  Subroutine ELP621  (Bilinear quadrilateral)
!                    6:  Subroutine ELP623  (Biquadratic quadrilateral)
!     user     i     User array in which real information of the
!                    coefficients is stored (See Standard Problems)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jwcons, ncoef, i, j, isubcf, mrule, ishift
      logical linear, funchk

!     funchk     Indicates whether there are coefficients that must be
!                evaluated by funcc3 (true) or not (false)
!     i          Counting variable
!     ishift     Shift in coefficient sequence number
!     isubcf     Under bound for coefficients check
!     j          Counting variable
!     jwcons     Indication if the weight function is constant (1) or not (2)
!     linear     Indicates whether the element is a linear simplex (true)
!                or not (false)
!     mrule      Type of integration rule as given by the user
!     ncoef      Number of coefficients in the equation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ELUSR3         Extracts information concerning the coefficients from IUSER
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREPSCAL     Replace ivalue by scalar if necessary
!     INSTOP         Stop the program
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     In the case of a call by BUILD:
!     1:  Type of natural boundary condition
!         Possible values:
!         0,2: natural boundary condition of type 2
!         3: natural boundary condition of type 3
!         4: natural boundary condition of type 4
!         5: natural boundary condition of type 5
!     2:  Not yet in use:  set equal to 0
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-orindates
!         1: Axi-symmetric co-orindates
!         2: Polar co-orindates
!     5:  Not yet in use:  set equal to 0
!     6:  sigma
!     7:  h
!     8:  alpha 11  (1D/2D/3D)  (The parameters 8-13 are only used for
!     9:  alpha 12  (2D/3D)      boundary conditions of type 4)
!    10:  alpha 13  (3D)
!    11:  alpha 22  (2D/3D)
!    12:  alpha 23  (3D)
!    13:  alpha 33  (3D)
!    14-15: Not yet in use (must be zero)
!    In case of boundary conditions of type 5:
!     6:  u1
!     7:  u2
!     8:  u3
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!    1546   Boundary conditions of type 5 have only been implemented for
!           line elements
!    1766   Type of integration rule incorrect
!    1767   Type of co-ordinates incorrect
!    1771   Shape number not yet implemented
!    1912   Parameter jboun has incorrect value
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill some standard values in the common blocks celp and celint

      call eropen ( 'el0601' )
      call elcomm
      ishift = 0
      jtrans = 0
      ideriv = 0
      jderiv = 0

!     --- Set parameters depending on the shape numbers

      n = inpelm
      m = inpelm
      mone = int(n**(1d0/ndim)+0.5d0)
      jelp = ishape
      if ( inpelm.eq.1 ) then

         jelp = 0
         linear = .true.

      else if ( jelp.eq.1 .or. jelp.eq.3 ) then

!     --- linear element

         linear = .true.
         mn = m

      else

!     --- Error 1771: this type of element has not yet been implemented

         call errint ( ishape, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errsub ( 1771, 4, 0, 0 )

      end if

!     --- Fill the arrays IND, MST and ARRAY with information of the
!     coefficient

      ipos = iuser ( ielgrp + 5 )
      if ( ipos.lt.7 ) then
         call errint ( ielgrp+5, 1 )
         call errint ( ipos, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 42, 5, 0, 0 )
      end if

      nuser = 0

!     --- Compute the parameters jboun, jcart and irule

      jboun = inirepscal ( iuser(ipos) )
      if ( jboun.eq.0 ) jboun = 2
      ipos = ipos + 1
      ishift = ishift+1
      if ( jboun.lt.2 .or. jboun.gt.5 ) then
         call errint ( jboun, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errsub ( 1912, 4, 0, 0 )
      end if
      if ( jboun.eq.5 .and. ishape.gt.2 ) then
         call errint ( ielem, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 1546, 3, 0, 0 )
      end if
      ipos = ipos + 1
      ishift = ishift+1

      mrule = inirepscal ( iuser(ipos) )
      irule = max ( 1, mrule )
      ipos = ipos + 1
      ishift = ishift+1
      if ( mrule.lt.0 .or. mrule.gt.40 ) then
         call errint ( mrule, 1 )
         call errint ( 3, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 1766, 5, 0, 0 )
      end if

      jcart = inirepscal ( iuser(ipos) )
      ipos = ipos + 1
      ishift = ishift+1
      if ( jcart.lt.0 .or. jcart.gt.2 ) then
         call errint ( jcart, 1 )
         call errint ( 4, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 1767, 5, 0, 0 )
      end if
      jcart = jcart + 1

      if ( linear ) then
         jwcons = min(jcart,2)
      else
         jwcons = 2
      end if

      ipos = ipos + 1
      ishift = ishift+1
      if ( jboun.le.3 ) then
         ncoef = 2
      else if ( jboun.eq.5 ) then
         ncoef = ndim
         icoor = 11
      else
         ncoef = 8
      end if
      do i = 1, ncoef
         call elusr3 ( i, array(1+(i-1)*m), iuser, user, i+ishift )
      end do

!     --- Test array lengths, number of nodes and number of degrees of freedom

      call prinlg ( 'iuser', ipos-1, user, iuser )
      call prinlg ( 'user', -nuser, user, iuser )

!     --- Compute the parameters jzero and jconsf

      if ( jwcons.eq.1 .and. ind(1).lt.0 ) then
         jzero = 1
      else
         jzero = 2
      end if

      if ( jwcons.eq.1 .and. ind(2).lt.0 ) then
         jconsf = 1
      else
         jconsf = 2
      end if

!     --- Check which parameters depend on the space co-ordinates
!         Compute isub and isup

      isubcf = 1

      do i = isubcf, ncoef
         isub = i

!        --- If ind(i) > 0 then paramater i corresponds to a function

         if ( ind(i).gt.0 ) goto 170

      end do

      isub = ncoef + 1

170   isup = ncoef
      do i = ncoef, isub , -1
         isup = i
         if ( ind(i).gt.0 ) goto 190

      end do

!     --- Check for double parameters (functions) to avoid calls for
!         function subroutines
!         ist is used to store the references for the double parameters
!         Set ind(lengthmst)

190   do i = isub, isup
         ist(i) = 0
      end do
      ind(lengthmst) = 0
      funchk = .false.
      do i = isub, isup
         if ( ind(i).gt.0 .and. ind(i).lt.2000 ) then

!        --- only the function calls are checked

            do j = i+1,isup
               if ( ind(j).eq.ind(i) ) then
                  ist(j) = i
                  ind(j) = 0
               end if
            end do
         end if
         if ( ind(i).gt.1000 .and. ind(i).le.2000 ) ind(lengthmst) = 1
         if ( ind(i).gt.10000 ) funchk = .true.
      end do

      if ( ind(1).eq.0 ) notmat = 1
      if ( ind(2).eq.0 .or. jboun.eq.5 ) notvec = 1
      if ( funchk ) ind(lengthmst) = ind(lengthmst) + 2

      if ( ierror.ne.0 ) call instop
      notmas = 1
      call erclos ( 'el0601' )

      end
