      subroutine el0807 ( iuser, user, jelp, isub, isup, array, iseq, &
                          intdiscrule )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.2  date 11-06-2010 Extension with rho_liquid
!        version  1.1  date 07-03-2005 Extension with derivatives
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine ELM807 or
!     ELD807 in the case that IFIRST = 0
!     The parameters in the common blocks celint, celiar and celp
!     get a start value
!     Array array is initialized with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
!     enthalpy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), array(*)
      integer iuser(*), jelp, isub, isup, iseq, intdiscrule

!     array          o    Real work array in which the values of the
!                         coefficients in the integration points are stored
!                         in the sequence:
!                          1:   k_solid
!                          2:   k_fluid
!                          3:   rho
!                          4:   c_solid
!                          5:   c_fluid
!                          6:   Q
!                          7:   L
!                          8:   T_melt
!                         Each coefficient requires exact m (is number of
!                         integration points) positions
!     intdiscrule    o    Numerical integration rule to be used for
!                         discontinuous integration
!                         See eldiscint
!     iseq           i    Sequence number of vector in VECOLD/ISLOLD
!                         In case of a call by ELI807 it is the parameter JDEGFD
!     isub           o    Integer help parameter for do-loops to indicate the
!                         smallest coefficient number that is dependent on space
!     isup           o    Integer help parameter for do-loops to indicate the
!                         largest coefficient number that is dependent on space
!     iuser          i    User array in which integer information of the
!                         coefficients is stored (See Standard Problems)
!     jelp           o    Indication of the type of basis function subroutine
!                         must be called by ELM807.
!     user           i    User array in which real information of the
!                         coefficients is stored (See Standard Problems)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jwcons, ncoef, i, j, isubcf, icoef, kchois, mrule
      logical linear, funchk

!     funchk         Indicates whether there are coefficients that must be
!                    evaluated by funcc3 (true) or not (false)
!     i              Counting variable
!     icoef          Coefficient sequence number
!     isubcf         Under bound for coefficients check
!     j              Counting variable
!     jwcons         Indication if the weight function is constant (1)
!                    or not (2)
!     kchois         Indication what type of parameters must be computed
!                    Possible values:
!                    0:  Standard method, the calling subroutine is ELM807
!                        It is supposed that an element matrix must be computed
!                    1:  Subroutine ELD807 calls this subroutine, only
!                        derivatives must be computed
!                    2:  Subroutine ELD807 or ELI807 calls this subroutine, &
!                        a user function defined as first parameter in
!                        IUSER/USER must be filled in elemvc (icheld=9)
!                    3:  Subroutine ELD807 calls this subroutine, &
!                        alpha times the gradient must be stored in ELEMVC
!                        The first 3 parameters of IUSER/USER are used
!     linear         Indicates whether the element is a linear simplex
!                    (true) or not (false)
!     mrule          Type of integration rule as given by the user
!     ncoef          Number of coefficients in the equation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0800DER      Compute some parameters related to derivatives or
!                    integrals
!     EL0807IN       Compute the number of integration points for elements of
!                    type 807
!     ELUSR3         Extracts information concerning the coefficients from IUSER
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INSTOP         Stop the program
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     In the case of a call by BUILD:
!     1:  Not yet used 0
!     2:  Not yet used 0
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!         jcart:
!         0: Cartesian co-ordinates
!         1: Axi-symmetric co-ordinates
!         2: Polar co-orindates
!     5:  0  not yet used
!     6:  kappa_solid
!     7:  kappa_fluid
!     8:  rho_solid
!     9:  c_solid
!    10:  c_fuid
!    11:  Q              (source term)
!    12:  L              (latent heat)
!    13:  T_sol          (meltig temperature)
!    14:  rho_solid, if zero rho_solid is used
!    15:  Not yet used (must be 0)
!    16:  Not yet used (must be 0)
!    17:  Not yet used (must be 0)
!    18:  Not yet used (must be 0)
!    19:  Not yet used (must be 0)
!    20:  Not yet used (must be 0)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     341   Icheli has wrong value
!     406   Value of icheld not allowed
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill some standard values in the common blocks celp and celint

      call eropen ( 'el0807' )

!     --- Set parameters depending on the shape numbers

      call el0807in ( iuser, kchois, mrule, ncoef, linear, &
                      jelp, jwcons, intdiscrule )

!     --- if derivatives are computed set iseq equal to iseqi1

      iseq = iseqi1

      if ( icheld>0 ) then

!     --- icheld>0, hence compute derivatives
!         check values of icheld

         if ( icheld==4 .or. icheld>=6 .and. icheld<= 10 .or. &
              icheld==14 .or. icheld>=16 .and. icheld<= 30 .or. &
              icheld==34 .or. icheld>=36 .and. icheld<= 44 .or. &
              icheld>=46 ) then

            call errint ( icheld, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 406, 3, 0, 0 )

         end if  ! ( icheld==4 ....... )
         ncoef = 0

      else if ( icheli>0 ) then

         call errint ( -icheld, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 341, 3, 0, 0 )

      end if
      if ( ierror/=0 ) go to 1000

!     --- set some parameters with respect to derivatives

      call el0800der (icheld, iweigh, jderiv, ndimlc, ideriv, &
                      icheli, ix )

      mn = m * n

!     --- Fill the arrays IND, MST and ARRAY with information of the
!         coefficients

      if ( ncoef>0 ) then

         nuser = 0

         isub = 1
         ipos = ipos + 1

         do i = isub, ncoef
            call elusr3 ( i, array(1+(i-1)*m), iuser, user, i )
         end do

         if ( ierror/=0 ) go to 1000

!        --- Test array lengths, number of nodes and number of degrees
!            of freedom

         call prinlg ( 'iuser', ipos-1, user, iuser )
         call prinlg ( 'user', -nuser, user, iuser )

      end if  ! ( ncoef>0 )
      if ( ierror/=0 ) go to 1000

!     --- Next consider all terms carefully in order to get the cheapest
!         computation

      jsecnd = jwcons  ! no tensor, just constant diffusion

      if ( kchois==0 .and. vector .or. kchois==2 ) then

!     --- Vector = true, the element vector must be computed
!         or kchois = 2, the first parameter must be computed
!         Compute jconsf

         if ( kchois==0 ) then

            icoef = 6

         else

            icoef = 1

         end if
         if ( ind(icoef)==0 ) then

!        --- F = 0, no right-hand side

            jconsf = 0

         else if ( ind(icoef)<0 .or. ind(icoef)==2002 ) then

!        --- F is constant (per element)

            jconsf = jwcons

         else

!        --- General

            jconsf = 2

         end if

      end if

!     --- Check which parameters depend on the space co-ordinates
!         Compute isub and isup

      if ( matrix .and. kchois==0 .or. kchois>=2 ) then
         isubcf = max(1,isub)
      else
         isubcf = 11
      end if

      do i = isubcf , ncoef
         isub = i

!        --- If ind(i)>0 then paramater i corresponds to a function

         if ( ind(i)>0 ) goto 170

      end do

      isub = ncoef + 1

170   isup = ncoef
      do i = ncoef, isub , -1
         isup = i
         if ( ind(i)>0 ) goto 190

      end do

!     --- Check for double parameters (functions) to avoid calls for
!         function subroutines
!         ist is used to store the references for the double parameters
!         Set ind(lengthmst)

190   do i = isub, isup
         ist(i) = 0
      end do
      ind(lengthmst) = 1
      funchk = .false.
      do i = isub, isup
         if ( ind(i)>0 .and. ind(i)<2000 ) then

!        --- only the function calls are checked

            do j = i+1,isup
               if ( ind(j)==ind(i) ) then
                  ist(j) = i
                  ind(j) = 0
               end if
            end do
         end if
         if ( ind(i)>1000 .and. ind(i)<=2000 ) ind(lengthmst) = 1
         if ( ind(i)>10000 ) funchk = .true.
      end do

      if ( funchk ) ind(lengthmst) = ind(lengthmst) + 2
      if ( icheld==45 )  then

!     --- Special case of refinement for enthalpy computation
!         using newton (type 807)

         ind(lengthmst) = 2
         iphi = 1
         iweigh = 1

      end if  ! ( icheld==45 )

1000  if ( ierror/=0 ) call instop

      call erclos ( 'el0807' )

      end
