      subroutine el0807in ( iuser, kchois, mrule, ncoef, linear, &
                            jelp, jwcons, intdiscrule )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.2  date 11-06-2010 Extension with rho_liquid
!        version  1.1  date 17-03-2009 Debug information
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the number of integration points for this element group
!     using array iuser
!     Enthalpy equation with Newton
! **********************************************************************
!
!                       KEYWORDS
!
!     elliptic_equation
!     initialization
!     enthalpy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*), kchois, mrule, ncoef, jelp, jwcons, intdiscrule
      logical linear

!     intdiscrule    o    Numerical integration rule to be used for
!                         discontinuous integration
!                         See eldiscint
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     jelp           o    Indication of the type of basis function subroutine
!                         must be called by ELel0807inx.
!     jwcons         o    Indication if the weight function is constant
!                         (1) or not (2)
!     kchois         o    Indication what type of parameters must be computed
!                         Possible values:
!                         0:  Standard method, the calling subroutine is ELM100
!                             It is supposed that an element matrix must be
!                             computed
!                         1:  Subroutine ELD100 calls this subroutine, only
!                             derivatives must be computed
!                         2:  Subroutine ELD100 or ELI100 calls this subroutine, &
!                             a user function defined as first parameter in
!                             IUSER/USER must be filled in elemvc (icheld=9)
!                         3:  Subroutine ELD100 calls this subroutine, &
!                             alpha times the gradient must be stored in ELEMVC
!                             The first 3 parameters of IUSER/USER are used
!     linear         o    Indicates whether the element is a linear simplex
!                         (true) or not (false)
!     mrule          o    Type of integration rule as given by the user
!     ncoef          o    Number of coefficients in the equation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     EL0800INM      Compute the parameter m for an element
!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREPSCAL     Corrects integer value in case of scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!    1766   Type of integration rule incorrect
!    1767   Type of co-ordinates incorrect
!    1771   Shape number not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0807in' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0807in'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call elcomm
      mrule = 0
      iphi = 1
      intdiscrule = 2

!     --- Compute kchois

      if ( icheld==0 ) then

!     --- Call by elm807, all coefficients required

         kchois = 0

      end if

      if ( ierror/=0 ) go to 1000

!     --- Set parameters depending on the shape numbers

      n = inpelm
      m = n

      jelp = ishape
      if ( jelp==1 .or. jelp==3 .or. jelp==11 ) then

!     --- linear element

         linear = .true.
         mn = m
         jdercn = 0

      else

!     --- Error 1771: this type of element has not yet been implemented

         call errint ( ishape, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errsub ( 1771, 4, 0, 0 )

      end if

      if ( kchois==0 ) then

!     --- standard situation

         ncoef = 9  ! One extra, due to rho_liquid

      end if
      if ( ierror/=0 ) go to 1000

!     --- Fill the arrays IND, MST and ARRAY with information of the
!         coefficients

      if ( ncoef>0 ) then

         ipos = iuser ( ielgrp + 5 )
         if ( ipos<7 ) then
            call errint ( ielgrp+5, 1 )
            call errint ( ipos, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 42, 5, 0, 0 )
         end if

         ipos = ipos + 2  ! skip first 2 coefficients

         if ( kchois==0 .or. kchois==2 ) then

!        --- Call by BUILD or INTEGR, Compute the parameters jcart and irule
!            mrule consists of 2 parts: mrule_orig + 100*(intdiscrule-1)

            if ( debug ) &
               write(irefwr,1) 'ipos, iuser(ipos)', ipos, iuser(ipos)
            mrule = inirepscal ( iuser(ipos) )
            if ( debug ) write(irefwr,1) 'mrule', mrule
            if ( mrule>0 ) then

!           --- mrule has been given

               intdiscrule = mrule/100 + 1 ! part with respect to discontinuous
                                          ! integration
               mrule = mod(mrule,100) ! standard integration rule

            else

!           --- mrule has not been given

               intdiscrule = 2

            end if  ! ( mrule>0 )

            if ( mrule==0 ) then
               irule = 1
            else
               irule = mrule
            end if
            ipos = ipos + 1

            if ( mrule<0 .or. mrule>40 .or. intdiscrule>4 ) then
               call errint ( mrule+100*(intdiscrule-1), 1 )
               if ( kchois==0 ) then
                  call errint ( 3, 2 )
               else
                  call errint ( 1, 2 )
               end if
               call errint ( ielem, 3 )
               call errint ( itype, 4 )
               call errint ( ielgrp, 5 )
               call errsub ( 1766, 5, 0, 0 )
            end if

            if ( debug ) &
               write(irefwr,1) 'ipos, iuser(ipos)', ipos, iuser(ipos)
            jcart = inirepscal ( iuser(ipos) )
            if ( debug ) write(irefwr,1) 'jcart', jcart
            ipos = ipos + 1
            if ( jcart<0 .or. mod(jcart,10)>2 .or. &
                 ndim>2 .and. jcart/10>4 ) then
               call errint ( jcart, 1 )
               if ( kchois==0 ) then
                  call errint ( 4, 2 )
               else
                  call errint ( 2, 2 )
               end if
               call errint ( ielem, 3 )
               call errint ( itype, 4 )
               call errint ( ielgrp, 5 )
               call errsub ( 1767, 5, 0, 0 )
            end if
            jcart = jcart + 1

         end if
         if ( ierror/=0 ) go to 1000

         if ( linear ) then
            jwcons = min(jcart,2)
         else
            jwcons = 2
         end if

         if ( irule/=1 ) then

!        --- irule>1, Gauss rule, compute m and n

            call el0800inm ( m, jelp, jwcons, irule, igausp, jdiag, &
                             iphi )
            mn = m * n

         end if

      end if  ! ( ncoef>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el0807in'

      end if  ! ( debug )
      call erclos ( 'el0807in' )

      end
