      subroutine el0815m ( iuser, jelp, kfirst, jheat, isub, &
                           isup, array )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.1  date 31-07-2012 Extra diffusion and smoothing
!        version  1.0  date 20-01-2012
!
!   copyright (c) 2012-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine ELM815
!     The parameters in the common blocks celint, celiar and celp
!     get a start value
!     Array array is initialized with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision array(m,*)
      integer iuser(*), jelp, kfirst, jheat, isub, isup

!     array          o    Real work array in which the values of the
!                         coefficients in the integration points are stored
!                         in the sequence
!                         See elm815
!                         Each coefficient requires exact m (is number of
!                         integration points) positions
!     isub           o    Integer help parameter for do-loops to indicate the
!                         smallest coefficient number that is dependent on space
!     isup           o    Integer help parameter for do-loops to indicate the
!                         largest coefficient number that is dependent on space
!     iuser          i    User array in which integer information of the
!                         coefficients is stored (See Standard Problems)
!     jelp           o    Indication of the type of basis function subroutine
!                         must be called by ELM815
!     jheat          o    Integer parameter indicating if the mass matrix for
!                         the heat equation must be computed
!                         Possible values:
!                         0:  no mass matrix
!                         1:  diagonal mass matrix with constant coefficient
!                         2:  diagonal mass matrix with variable coefficient
!     kfirst         o    This parameter indicates the smallest velocity
!                         component that is not equal to zero in the convective
!                         terms
!                         For example kfirst = 2, means that the y- &
!                         component of the velocity is unequal to zero
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jwcons, ncoef, ishift, mrule, &
              metup, interpol, iqmat, iqmat1
      logical linear

!     interpol       If 0 the standard interpolation is applied, &
!                    if 1 the interpolation is restricted to linear subelements
!     iqmat          Indication whether a transformation has to be performed if
!                    the elements are of a lower dimension than the dimension of
!                    the coordinates.  If 0 no tranformation is performed
!                    (standard case) if 1 then the velocities are mapped to
!                    the dimension of the elment
!     iqmat1         Indication whether the right-hand side has to be
!                    transformed
!     ishift         Shift in coefficient sequence number
!     jwcons         Indication if the weight function is constant (1)
!                    or not (2)
!     linear         Indicates whether the element is a linear simplex (true)
!                    or not (false)
!     metup          Upwind parameter
!     mrule          Type of integration rule as given by the user
!     ncoef          Number of coefficients in the equation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0800INMAT    Compute the number of integration points for this element
!                    group using array iuser
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     See subroutine elm815
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0815m' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0815m'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Initialize arrays ind and ist

      ind = 0
      ist = 0

!     --- Set parameters depending on the shape numbers

      call el0800inmat ( iuser, metup, mrule, linear, &
                         jelp, ishift, interpol, iqmat, iqmat1, &
                         jheat, jwcons )
      if ( ierror/=0 ) go to 1000

      mn = m * n

!     --- Fill the arrays IND, MST and ARRAY with information of the
!         coefficients

      nuser = 0
      isub = 1
      ishift = ishift+1
      ipos = ipos+1
      jderiv = 1  ! compute dphidx

!     --- Next consider all terms carefully in order to get the cheapest
!         computation

      if ( matrix .or. metup>0 ) then

!     --- matrix = true  The element matrix must be computed.
!         or metupw>0:  Upwinding
!                       Compute jsecnd, idim and jzero

         jsecnd = 1   ! diffusion terms

         if ( iuser(5)==0 ) then

!        --- iuser(5)=0, standard case

            kfirst = 1   ! first velocity term
            idim = ndim  ! last velocity term
            ind(7:6+ndim) = 1  ! velocity variable

         else

!        --- iuser(5)>0, no convection and rhs

            kfirst = 0   ! no convection
            idim = 0     ! no convection

         end if  ! ( iuser(5)==0 )
         jzero = 0    ! no constant term

      end if  ! ( matrix .or. metup>0 )
      if ( ierror/=0 ) go to 1000

!     --- Compute jconsf

      if ( vector ) then

!     --- vector = true, fill information about rhs

         if ( iuser(5)==0 ) then

!        --- iuser(5)=0, standard case

            ind(11) = 1  ! right-hand side variable
            jconsf = jwcons  ! element vector constant per element

         else

!        --- iuser(5)>0, no convection and rhs

            ind(11) = 0  ! right-hand side zero
            jconsf = 0
            notvec = 1

         end if  ! ( iuser(5)==0 )

      end if  ! ( vector )

!     --- Compute parameters with respect to time derivative

      ind(12) = -1         ! rho_cp constant
      array(1:m,12) = 1d0  ! rho_cp = 1
      jheat = jwcons

!     --- Check which parameters depend on the space co-ordinates
!         Compute isub and isup
!         adapt kstep and iweigh if necessary

      if ( iuser(5)==0 ) ind(lengthmst) = 1  ! compute both phi and dphidx

!     --- Compute parameters isub and isup defining the lowest and largest
!         coefficient depending on space
!         Adapt some paramaters

      isub = 2  ! no variable parameters
      isup = 1
      ncoef = 0

1000  call erclos ( 'el0815m' )
      if ( debug ) then

!     --- Debug information

         call prinin ( ind, ncoef, 'ind' )
         call prinin ( ist, ncoef, 'ist' )
         write(irefwr,1) 'isub, isup, igausp, m, n, iweigh, iphi', &
                          isub, isup, igausp, m, n, iweigh, iphi
         write(irefwr,1) 'irule, metup', irule, metup
         write(irefwr,1) 'jsecnd', jsecnd
         write(irefwr,*) 'End el0815m'

      end if  ! ( debug )

      end
