      subroutine el1001 ( icheld, ix, dphidx, dudx, elemvc, elemwg, &
                          array, ndim, u )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.5  date 16-04-2014 Minor extension comment
!        version  2.4  date 01-12-1995 Correction d/dxi for quadratic element
!        version  2.3  date 14-01-1995 Extra possibilities icheld
!
!   copyright (c) 1992-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    actual filling of derivatives and weights in elemvc and elemwg
!    with respect to subroutine ELD800
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icheld, ix, ndim
      double precision  dudx(m,ndimlc), dphidx(n,m,ndimlc), u(*), &
                        elemwg(ndimlc,*), elemvc(ndimlc,*), array(m,*)

!     array    i     Array containing the coefficients of the second order
!                    part of the elliptic equation
!                    At this moment array is only used for ICHELD = 6 or 16
!                    Array contains the coefficientsin the integration points
!                    in the sequence:
!                         1D                  2D            3D
!                    1:   alpha 11         alpha 11       alpha 11
!                    2:                    alpha 12       alpha 12
!                    3:                    alpha 22       alpha 13
!                    4:                                   alpha 22
!                    5:                                   alpha 23
!                    6:                                   alpha 33
!                    Array(.,1) contains the coefficient alpha 11
!                    Array(.,2) contains the coefficient alpha 12
!                    etc.
!     dphidx   i     Array of length n x m x ndim  containing the derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = dphidx (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = dphidx (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = dphidx (i,k,3);
!                         i        k
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     dudx     i     Real m x ndim  array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du / dx  (x ) = dudx ( k, j )
!                           j   k
!                    If jdercn = 0 in common block celint, then only
!                    dudx (1,*) is filled because dudx is then a constant
!                    per element
!                    dudx is only filled if jderiv>0
!     elemvc   o     element vector containing the derivatives for subroutine
!                    deriva
!     elemwg   o     element vector containing the weights for subroutine
!                    deriva
!     icheld   i     Choice parameter to distinguish between the various
!                    possibilities of this subroutine
!                    Definitions:
!                    1:   compute the derivative in ix direction
!                         ix = 1    elemvc(1,*) = du/dx
!                         ix = 2    elemvc(1,*) = du/dy
!                         ix = 3    elemvc(1,*) = du/dz
!                    2:   grad u
!                    3:   - grad u
!                    4:   Not yet defined
!                    5:   elemvc(1,*)=du/dy   elemvc(2,*)=-du/dx (2D only)
!                    6:   compute - A grad u, where A is the matrix
!                         corresponding to the second order term of the elliptic
!                         differential equation
!                    7-10: Not yet available
!                   11-20: See 1-10, however, now defined per element
!                   31-40: See 1-10, however, now derivatives are computed
!                         with general weight 1
!     ix       i     Indication of the type of derivative to be computed.
!                    When icheld=1 or 11, du/d ix is computed
!     ndim     i     Dimension of the space.
!     u        i     array of length m containing the values of the vector
!                    of which the derivative must be computed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL10011        fill derivatives and weights in elemvc and elemwg;
!                    icheld=1: d c /d_ix
!     EL100111       fill derivatives in elemvc
!                    icheld=11: d c /d_ix element-wise
!     EL100112       fill derivatives in elemvc
!                    icheld=12, 13: grad c element-wise
!     EL100115       fill derivatives in elemvc
!                    icheld=15: curl 2d element-wise
!     EL100116       fill derivatives in elemvc
!                    icheld=6,16,26: -A grad c
!     EL10012        fill derivatives and weights in elemvc and elemwg;
!                    icheld=2, 3, 5: grad c
!     EL100131       fill derivatives in elemvc
!                    icheld=31: d c /d_ix
!     EL100132       fill derivatives in elemvc
!                    icheld=32, 33, 35: grad c
!     EL10017        fill derivatives in elemvc
!                    icheld=17: div u
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     406   Value of icheld not allowed
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el1001' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el1001'
         write(irefwr,1) 'icheld, ix, ndim, ndimlc, m, n', &
                          icheld, ix, ndim, ndimlc, m, n
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      select case ( icheld )

      case(1)

!     --- icheld = 1, compute derivative du/d ix

         call el10011 ( ix, dphidx, dudx, elemvc, elemwg, ndim )

      case(2, 3, 5)

!     --- icheld = 2,3,5  Compute gradient

         call el10012 ( icheld, dphidx, dudx, elemvc, elemwg, ndim )

      case(7)

!     --- icheld = 7, compute div u

         call el10017 ( dphidx, u, elemvc, ndim )

      case(11)

!     --- icheld = 11, See icheld = 1, however, without weights

         call el100111 ( dudx, elemvc )

      case(12:13)

!     --- icheld = 12, 13, See icheld = 2, 3, however, without weights

         call el100112 ( dudx, elemvc, icheld )

      case(15)

!     --- icheld = 15, See icheld = 5, however, without weights

         call el100115 (  dudx, elemvc )

      case(6,16,36)

!     --- icheld = 6, 16, 26  Compute - A gradient

         call el100116 ( icheld, dudx, elemvc, array, ndim )

      case(31)

!     --- icheld = 31, compute derivative du/d ix

         call el100131 ( dudx, elemvc, ndim )

      case(32, 33, 35)

!     --- icheld = 32,33,35  Compute gradient

         call el100132 ( icheld, dudx, elemvc, ndim )

      case default

         call errint ( icheld, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 406, 3, 0, 0 )

      end select  ! case ( icheld )

1000  call erclos ( 'el1001' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el1001'

      end if  ! ( debug )

      end
