      subroutine el1017 ( elemvc, p, gradp, array, ibear, work, icheld, &
                          ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 07-11-2006 Fortran 90
!        version  2.0  date 28-02-1998 Ron van Ostayen: adaptation for
!                                      cavitation, extra ndim
!        version  1.0  date 05-11-1997
!
!   copyright (c) 1997-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element vector for flow vector and shear stress of Reynolds equation
!     Both compressible and incompressible bearing
!     One and two dimensional elements
! **********************************************************************
!
!                       KEYWORDS
!
!     bearing
!     compressible
!     incompressible
!     element_vector
!     lubrication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibear, icheld, ndim
      double precision array(m,*), p(m), gradp(m,ndimlc), work(m,2), &
                       elemvc(ndim,*)

!     array          array in which the values of the coefficients
!                    in the integration points are stored in the sequence:
!                         1D                2D
!                    1:   h                h
!                    2:   mu               mu
!                    3:   ht               ht
!                    4:   k                k
!                    5:   p0               p0
!                    6:   u_1 1            u_1 1
!                    7:                    u_1 2
!                    8:
!                    9:   rho              rho
!                   10:   RT               RT
!                   11:   f1               f1
!                   12:                    f2
!                   13:   u_2 1            u_2 1
!                   14:                    u_2 2
!                    Each coefficient requires exact m (is number of
!                    integration points) positions
!     elemvc   o     Contains the flow vector for subroutine DERIV
!                    24: the flow vector is computed
!                    25: the shear stress at the bottom is computed
!                    26: the shear stress at the top is computed
!     gradp    i     Real m x ndim array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du / dx  (x ) = gradp ( k, j )
!                           j   k
!     ibear    i     Indicates type of bearing.
!                    Possible values:
!                    0:  incompressible bearing
!                    1:  compressible bearing (adiabatic)
!     icheld   i     Choice parameter to distinguish between the various
!                    possibilities of this subroutine
!     ndim     i     Dimension of space
!     p        i     Contains the pressure p
!     work           Work array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      logical cavitat, cavstart

!     cavitat    If true we have cavitation in the element
!     cavstart   Starting value for cavitat
!     i          Summation variable
!     j          Summation variable
!     k          Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The flow vector is given by:
!    q = -h^3/12 mu (gradp-rho f) +h/2(u1+u2) (incompressible)
!    q = p(-h^3/12 mu (gradp-rho f) +h/2(u1+u2)) (compressible)
!    The shear stress at the bottom:
!    tau = -h/2(grad p -rho f) + mu/h (u2-u1)
!    The shear stress at the top:
!    tau =  h/2(grad p -rho f) + mu/h (u2-u1)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( icheld==23 ) then

!     --- Store H^3/12mu in work(1) en h/2 in work(2)

         do k = 1, m
            work(k,1) = array(k,1)**3/(12d0*array(k,2))
            work(k,2) = array(k,1)*0.5d0
         end do

      else

!     --- Store H/2 in work(1) en mu/h in work(2)

         do k = 1, m
            work(k,1) = array(k,1)*0.5d0
            work(k,2) = array(k,2)/array(k,1)
         end do

      end if

!     --- If Reynolds=2D and ndimlc=3D init component 3

      if ( ndim==3 ) then
         do k = 1, n
            elemvc(3,k) = 0d0
         end do
      end if

      if ( icheld==23 ) then

!     --- Compute flow vector

         do j = 1, min(2,ndimlc)
            do k = 1, n
               elemvc(j,k) = -work(k,1)*(gradp(k,j)- &
                              array(k,9)*array(k,10+j))+ &
                              work(k,2)*(array(k,5+j)+array(k,12+j))
            end do
         end do
         if ( ibear/=0 ) then
            do j = 1, min(2,ndimlc)
               do k = 1, n
                  elemvc(j,k) = p(k)*elemvc(j,k)/array(k,10)
               end do
            end do
         end if

      else if ( icheld==24 ) then

!     --- Compute lower shear stress
!         if pressure==0d0 for all elements asume no cavitation
!         else if pressure==0d0 AND dpdx==0d0 AND dpdy==0d0 then
!           assume cavitation in point and thus no shear stress
!         SHOULD BE REPLACED BY A CORRECT MASS-CONSERVATION ALGORITHM

         cavstart = .false.
         do k = 1, n
            if ( abs(p(k))>1.0d-8 ) then
               cavstart = .true.
            end if
         end do
         do j = 1, min(2,ndimlc)
            do k = 1, n
               elemvc(j,k) = -work(k,1)*(gradp(k,j)- &
                              array(k,9)*array(k,10+j))+ &
                              work(k,2)*(array(k,12+j)-array(k,5+j))
               if ( abs(p(k))<=0d0 ) then
                  cavitat = cavstart
               else
                  cavitat = .false.
               end if
               do i = 1, ndimlc
                  if ( abs(gradp(k,i))>0d0 ) cavitat = .false.
               end do
               if ( cavitat ) elemvc(j,k) = 0d0
            end do
         end do

      else

!     --- Compute upper shear stress
!         if pressure==0d0 for all elements asume no cavitation
!         else if pressure==0d0 AND dpdx==0d0 AND dpdy==0d0 then
!         assume cavitation in point and thus no shear stress

         cavstart = .false.
         do k = 1, n
            if ( abs(p(k))>1.0d-8 ) then
               cavstart = .true.
            end if
         end do
         do j = 1, min(2,ndimlc)
            do k = 1, n
               elemvc(j,k) = work(k,1)*(gradp(k,j)- &
                             array(k,9)*array(k,10+j))+ &
                             work(k,2)*(array(k,12+j)-array(k,5+j))
               if ( abs(p(k))<=0d0 ) then
                  cavitat = cavstart
               else
                  cavitat = .false.
               end if
               do i = 1, ndimlc
                  if ( abs(gradp(k,i))>0d0 ) cavitat = .false.
               end do
               if ( cavitat ) elemvc(j,k) = 0d0
            end do
         end do

      end if
      end
