      subroutine el2010g ( elemmt, phix, phi, rho, w, mconv, u, dudx, &
                           xgauss, elmtsub, u0, du0dx, goertler, kappa )
! ======================================================================
!
!        programmer    Guus Segal/Ralf Biertmpfel
!        version  2.1  date 20-10-2010 new call to el3002
!        version  2.0  date 27-02-2010 Change of parameter list
!        version  1.0  date 28-09-1998
!
!   copyright (c) 1998-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the convective part of the element matrix for the special case of
!     the goertler equations
!     and store in matrix s for 2-d and 3-d elements
!     NOTE: only the cartesian coordinates are modified; only valid for
!           Newton or Picard linearization
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
!     goertler_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mconv
      double precision phix(n,m,ndimlc), rho(m), w(m), xgauss(m,ndimlc), &
                       elemmt(ndimlc*n,ndimlc*n), elmtsub(n,n), &
                       u(m,*), dudx(m,ndimlc,*), phi(n,m), &
                       u0(m,*), du0dx(m,ndimlc,*), goertler, kappa

!     du0dx     i    See dudx, but now for the extar velocity
!     dudx      i    Real m x ndim x ncomp array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du / dx  (x ) = dudx ( k, j, i )
!                      i    j   k
!     elemmt    i    Element matrix
!     elmtsub        Work array in which a sub element matrix is stored
!     goertler  i    Goertler number
!     kappa     i    dimensionless wall curvature
!     mconv     i    Defines the treatment of the convective terms
!                    Possible values:
!                    0: convective terms are skipped (Stokes flow)
!                    1: convective terms are linearized by Picard iteration
!                    2: convective terms are linearized by Newton iteration
!                    3: convective terms are linearized by the incorrect Picard
!                       iteration
!     phi       i    array of lenght n * m, values of shape
!                    functions in integration points
!     phix      i    Array of length n x m x 3  containing the derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = phix (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = phix (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = phix (i,k,3);
!                         i        k
!                    If the element is a linear tetrahedron (n=4), then the
!                    derivatives are constant and only k = 1 is filled.
!     rho       i    density rho
!     u         i    array of length m x ncomp in which the values of
!                    u in the integration points are stored the sequence:
!                    u  (x ) = u(j,i)  j = 1(1)m, i = 1 (1) ncomp
!                     i   j
!     u0        i    See u, but now for the extra velocity
!     w         i    array of length m containing the weights for integration
!     work      i    Work array of length m x 4
!     xgauss    i    array of length m x ndim containing the co-ordinates of
!                    the gauss points.  array xgauss is only filled when gauss
!                    integration is applied.
!                    xg  = xgauss (i,1);  yg  = xgauss (i,2);
!                      i                    i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision sum, gradgr(27,3,3), udum(27,3), rhow(m), &
                       rhowr(m), work(m), rhowu(m), rhowv(m)

!     gradgr  gradient of the main velocities in the sequence:
!                 dU0 / dx  (x ) = gradgr ( k, j, i )
!                    i    j   k
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     rhow           Contains rho w
!     rhowr          Contains rho w /r
!     rhowu          Contains u rho w /r
!     rhowv          Contains v rho w /r
!     sum            Help variable to compute a sum
!     udum           dummy array of length m x ncomp  containing a quantity
!                    of the integration points in the sequence:
!                    udum  (x ) = udum(j,i)  where j=1 to m and i=1 to ncomp
!                        i   j
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3001  Compute integrals of the shape / u(x).grad phi phi (x) dx
!                                            e -            j   i
!     EL3002  Compute integrals of the shape / beta(x) phi (x) dx
!                                            e            i
!     EL3010  Fill small matrix elemmt into larger matrix s
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Computation of the convective matrix results in a 3x3 block matrix, each
!     consisting of nxn matrices of the following shape:
!            |   S       S      S     |
!            |    uu      uv     uw   |
!            |                        |
!            |   S       S      S     |
!     S =    |    vu      vv     vw   |
!            |                        |
!            |   S       S      S     |
!            |    wu      wv     ww   |
!     The submatrices depend on the parameter mconv.
!     The non-linear term has the shape:
!            | u ux     v uy      w uz |
!            |                         |
!       rho  | u vx     v vy      w vz | = rho u . grad u
!            |                         |
!            | u wx     v wy      w wz |
!ralf<
!     If ( mconv=1 ) then           (Picard's linearization)
!         rho u . grad u (n+1)  =  rho u(n) . grad u (n+1)
!         rho u . grad u (n+1)  =  rho u(n) . grad u (n+1)
!     If ( mconv=2 ) then           (Newton's linearization)
!         u . u (n+1)   =   2 u(n) . u(n+1)  -  u(n) . u(n)
!         rho u . grad u (n+1)  =  rho u(n) . grad u (n+1) + &
!                                  rho u(n+1) . grad u (n) - &
!                                  rho u(n) . grad u (n)
!     If ( mconv=3 ) then           (incorrect Picard)
!ralf>
!         rho u . grad u (n+1)  =  rho u(n+1) . grad u (n)
!     The submatrices rho u(n) . grad u (n+1) are defined by:
!     S uu =  rho u(n) . gradphi(j) phi(i) = Svv = Sww
!     The submatrices rho u(n+1) . grad u (n) are defined by:
!     S uu =  rho ux(n) phi(j) phi(i)
!     S uv =  rho uy(n) phi(j) phi(i)
!     S uw =  rho uz(n) phi(j) phi(i)
!     S vu =  rho vx(n) phi(j) phi(i)
!     S vv =  rho vy(n) phi(j) phi(i)
!     S vw =  rho vz(n) phi(j) phi(i)
!     S wu =  rho wx(n) phi(j) phi(i)
!     S wv =  rho wy(n) phi(j) phi(i)
!     S ww =  rho wz(n) phi(j) phi(i)
!    In case of cylinder co-ordinates the convective terms become
!    ur dur/dr   + uphi/r dur/dphi   + uz dur/dz   - uphi**2/r
!    ur duphi/dr + uphi/r duphi/dphi + uz duphi/dz + ur uphi/r
!    ur duz/dr   + uphi/r duz/dphi   + uz duz/dz
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- multiply the weight by rho and store in rhow

      rhow = rho * w

      do k = 1, m

!ralf<
         gradgr(k,1,1) = du0dx(k,1,1)
         gradgr(k,2,1) = du0dx(k,2,1)
         gradgr(k,3,1) = du0dx(k,3,1)
         gradgr(k,1,2) = du0dx(k,1,2)-u0(k,1) * 2d0 * goertler * kappa
         gradgr(k,2,2) = du0dx(k,2,2)
         gradgr(k,3,2) = du0dx(k,3,2)
         gradgr(k,1,3) = du0dx(k,1,3)
         gradgr(k,2,3) = du0dx(k,2,3)
         gradgr(k,3,3) = du0dx(k,3,3)

!ralf>
      end do
      jzero = 2

      if ( jcart>2 ) then

!     --- Cylinder co-ordinates, store w rho/r in rhowr

         rhowr(1:m) = rhow(1:m)/xgauss(1:m,1)

      end if

      if ( jcart<=2 ) then

!     --- jcart = 1, Cartesian co-ordinates
!         or jcart = 2, Axi-symmetrical co-ordinates without swirl
!         --------------------------------------------------------- &

         do i = 1, ndimlc
            do j = 1, ndimlc

               jadd = 0
               call el3002 ( gradgr(1,j,i), elmtsub, rhow, phi, m, n, n)
               jadd = 1
               call el3010 ( elemmt, elmtsub, ndimlc*n, n, j, i, &
                             ndimlc, .false., 1d0 )
            end do
         end do
      end if

!     --- nonlinear terms

!ralf ---- for different linearization methods
!         ---------------------------------------------------------- &

      if ( mconv/=1 ) then

!     --- mconv = 2 or 3, successive substitution part, i.e.
!         rho u(n+1) . grad u (n)

         if ( jcart<=2 ) then

!        --- jcart = 1, Cartesian co-ordinates
!            or jcart = 2, Axi-symmetrical co-ordinates without swirl

            do i = 1, ndimlc
               do j = 1, ndimlc

                  jadd = 0
                  call el3002 ( dudx(1,j,i), elmtsub, rhow, phi, m, n, &
                                n )
                  jadd = 1
                  call el3010 ( elemmt, elmtsub, ndimlc*n, n, j, i, &
                                ndimlc, .false., 1d0 )
               end do
            end do

!ralf<

!ralf       --- put the nonlinear goertler term in stiffness matrix

!ralf           S vu  =   - goertler * kappa* u(n) * u(n+1)

!ralf           incorrect Picard part
            jadd=0
            j=1
            i=2
            do k = 1, m
               udum(k,j) = -goertler * kappa * u(k,j)
            end do
            call el3002 ( udum(1,j), elmtsub, rhow, phi, m, n, n )
            jadd = 1
            call el3010 ( elemmt, elmtsub, ndimlc*n, n, j, i, &
                                ndimlc, .false., 1d0 )

!ralf>

         else if ( jcart==3 ) then

!        --- jcart = 3, Polar co-ordinates
!            ---------------------------------------------------------- &

            do i = 1, 2

               jadd = 0
               call el3002 ( dudx(1,1,i), elmtsub, rhow, phi, m, n, n )
               jadd = 1
               if ( i==2 ) call el3002 ( u(1,1), elmtsub, rhowr, &
                  phi, m, n, n )
               call el3010 ( elemmt, elmtsub, 2*n, n, 1, i, &
                             ndimlc, .false., 1d0 )
            end do

            do i = 1, 2

               jadd = 0
               call el3002 ( dudx(1,2,i), elmtsub, rhowr, phi, m, n, &
                             n )
               jadd = -1
               if ( i==1 ) call el3002 ( u(1,2), elmtsub, rhowr, &
                  phi, m, n, n )
               jadd = 1
               call el3010 ( elemmt, elmtsub, 2*n, n, 2, i, &
                             ndimlc, .false., 1d0 )
            end do

         end if

      end if

      if ( mconv<3 ) then

!     --- mconv = 1 or 2, Picard part, i.e.
!         rho (u(n) + udum(n)) . grad u (n+1)

         if ( jcart<=2 ) then

!        --- jcart = 1, Cartesian co-ordinates
!            or jcart = 2, Axi-symmetrical co-ordinates without swirl

            idim = ndimlc
            jadd = 0

!           --- calculate the sum of the main velocities and the
!               perturbation velocities per point using the function grdgr

            do k = 1, m

!ralf<      --- initialize udum = u + u0

               do i = 1, ncomp
                  udum(k,i) = u(k,i) + u0(k,i)
               end do

!ralf>
            end do

!           --- derive udum . grad u(n+1)

            call el3001 ( udum, elmtsub, rhow, phi, phix, idim, n, &
                          work )

            do j = 1, ndimlc

               jadd = 1
               call el3010 ( elemmt, elmtsub, ndimlc*n, n, j, j, ndimlc, &
                             .false., 1d0 )
            end do

!           --- put the nonlinear goertler term in stiffness matrix

!ralf           S vu  =    goertler * kappa * u(n) * u(n+1)
!               Picard part

            jadd=0
            j=1
            i=2
            do k = 1, m

!ralf<
               udum(k,j) = -goertler * kappa * u(k,j)

!ralf>
            end do
            call el3002 ( udum(1,j), elmtsub, rhow, phi, m, n, n )
            jadd = 1
            call el3010 ( elemmt, elmtsub, ndimlc*n, n, j, i, &
                                ndimlc, .false., 1d0 )

         else if ( jcart==3 ) then

!        --- jcart = 3, Polar co-ordinates

            rhowu(1:m) = rhow(1:m)*u(1:m,1)
            rhowv(1:m) = rhow(1:m)*u(1:m,2)
            do i = 1, n
               do j = 1, n
                  sum = 0d0
                  do k = 1, m
                     sum = sum + ( rhowu(k) * phix(j,k,1) + &
                                   rhowv(k) * phix(j,k,2)) * phi(i,k)
                  end do
                  elmtsub(j,i) = sum
               end do
            end do
            do j = 1, 2
               jadd = 1
               call el3010 ( elemmt, elmtsub, 2*n, n, j, j, 2, &
                             .false., 1d0 )
            end do
            do i = 1, n
               do j = 1, i
                  sum = 0d0
                  do k = 1, m
                     sum = sum + rhowv(k) * phi(j,k) * phi(i,k)
                  end do
                  elmtsub(j,i) = sum
                  elmtsub(i,j) = sum
               end do
            end do
            call el3010 ( elemmt, elmtsub, 2*n, n, 1, 2, 2, &
                          .false., 1d0 )
            call el3010 ( elemmt, elmtsub, 2*n, n, 2, 1, 2, &
                          .false., -1d0 )

         end if

      end if

      end
