      subroutine el2107 ( elemmt, s, r0, r1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 27-02-2010 Debug statements
!        version  1.0  date 28-01-1995 Renamed subroutine el2107
!
!   copyright (c) 1995-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Eliminate the centroid point for the incompressible Navier-Stokes
!     equations in case of an extended quadratic triangle, quadrilateral
!     or 27 node brick
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision s(n*ndimlc,n*ndimlc),r0(ndimlc*(n-1),ndimlc), &
                       elemmt((n-1)*ndimlc,(n-1)*ndimlc), &
                       r1(ndimlc*(n-1),ndimlc)

!     elemmt  o    New element matrix
!     r0      i    Contains the divergence transformation matrix
!     r1      i    Contains the pressure transformation matrix
!     s       i    Element matrix to be changed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, l, ncent
      double precision sum1, sum

!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     l              Counting variable
!     ncent          Number of nt to be eliminated velocity unknowns
!     sum            Help parameter to compute a sum
!     sum1           Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The Navier-Stokes matrix can be written as:
!
!            |                T     T  |  |   |     |    |
!            |   S     S    -L    -L   |  | u |     | f  |   (1)
!            |    11    12    11    21 |  |  1|     |  1 |
!            |                         |  |   |     |    |
!            |                T     T  |  |   |     |    |
!            |   S     S    -L    -L   |  | u |     | f  |   (2)
!            |    21    22    12    22 |  |  2|  =  |  2 |
!            |                         |  |   |     |    |
!            |   D     D     0     0   |  | p |     | fd |
!            |    11    12             |  |  1|     |   1|
!            |                         |  |   |     |    |
!            |   D     D     0     0   |  | p |     | fd |
!            |    21    22             |  |  2|     |   2|
!
!     Here the u1 refers to the velocities outside the centroid, &
!     u2 to the velocities in the centroid p1 to the pressure in the
!     centroid and p2 to the gradient of the pressure.
!     All quantities with index 2 must be eliminated.
!
!     It follows immediately that:
!
!            -1           -1
!     u  = -D   D   u  + D   fd  = R  u  - fd_tr
!      2     22  21  1    22   2    0  1
!
!     Put this in equations (1) and (2):
!
!     (S_11 + S_12 R_0) u_1 - L_11^T p_1 - L_21^T p_2 = f_1 + S_12 fd_tr  (3)
!     (S_21 + S_22 R_0) u_1 - L_12^T p_1 - L_22^T p_2 = f_2 + S_22 fd_tr  (4)
!
!     Multiply (4) by R_1 = - L_21^T L_22^-T and add to (3):
!
!     (S_11 + S_12 R_0 + R_1(S_21 + S_22 R_0) ) u_1 - &
!     (L_11^T + R_1 L_12^T ) p_1 = f_1 + R_1 f_2 + S_12 fd_tr + R_1 S_22 fd_tr
!
!
!     Hence we can solve:
!
!     S u  = F, &
!        1
!                     T                 T
!     with S = S   + R   S   + S  R  + R  S  R
!               11    1   21    12 0    1  22 0
!
!                    T             T
!     and  F = f  + R  f + ( S  + R   S   ) fd_tr , &
!               1    1  2     12   1   22
!
!                  -1
!     where R  = -L   L
!            1     22  21
!
!             new   11     T  21     12          T  22
!     Hence: S   = S   + R1  S    + S   R0   + R1  S  R0   = &
!             ij    ij     ik kj     ik   kj     ik kl  lj
!
!                   11        21     12             22
!                  S   + R1  S    + S   R0   + R1  S  R0   = &
!                   ij     ki kj     ik   kj     ki kl  lj
!
!                   old       old      old               old
!                  S   + R1  S      + S      R0   + R1  S         R0
!                   ij     ki k+12,j   i,k+12  kj     ki k+12,l+12  lj
!
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- MODULES USED

      call eropen ( 'el2107' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el2107'
         call prinrl1 ( s, ndimlc*n, ndimlc*n, 's' )
         call prinrl1 ( r0, ndimlc, ndimlc*(n-1), 'r0' )
         call prinrl1 ( r1, ndimlc, ndimlc*(n-1), 'r1' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ndimlc==2 ) then

!     --- R2

         ncent = ndimlc*(n-1)
         do i = 1, ncent
            do j = 1, ncent
               sum = 0d0
               do k = 1, ndimlc
                  sum1 = 0d0
                  do l = 1, ndimlc
                     sum1 = sum1 + s(l+ncent,k+ncent)*r0(j,l)
                  end do
                  sum = sum + r1(i,k)*(s(j,k+ncent)+sum1) + &
                        s(k+ncent,i)*r0(j,k)
               end do
               elemmt(j,i) = s(j,i) + sum
            end do
         end do

      else

!     --- R3

         ncent = ndimlc*13
         do i = 1, ncent
            do j = 1, ncent
               sum = 0d0
               do k = 1, ndimlc
                  sum1 = 0d0
                  do l = 1, ndimlc
                     sum1 = sum1 + s(l+ncent,k+ncent)*r0(j,l)
                  end do
                  sum = sum + r1(i,k)*(s(j,k+ncent)+sum1) + &
                        s(k+ncent,i)*r0(j,k)
               end do
               elemmt(j,i) = s(j,i) + sum
            end do
         end do
         do i = 43, 81
            do j = 43, 81
               sum = 0d0
               do k = 1, ndimlc
                  sum1 = 0d0
                  do l = 1, ndimlc
                     sum1 = sum1 + s(l+ncent,k+ncent)*r0(j,l)
                  end do
                  sum = sum + r1(i,k)*(s(j,k+ncent)+sum1) + &
                        s(k+ncent,i)*r0(j,k)
               end do
               elemmt(j-3,i-3) = s(j,i) + sum
            end do
         end do

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( elemmt, icount, icount, 'elemmt' )
         write(irefwr,*) 'End el2107'

      end if  ! ( debug )
      call erclos ( 'el2107' )

      end
