      subroutine el3004 ( alpha, s, w, dphidx, mdecl, ndecl, nsrow )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 03-02-2015 extra debug
!        version  1.1  date 10-06-2013 Fortran 90
!        version  1.0  date 02-02-1989
!
!   copyright (c) 1989-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute contribution to element stiffness matrix corresponding to the
!     second derivative part of a two-dimensional equation, i.e.
!        s  = s   + / (alpha grad phi ) . grad phi dx
!         ij   ij   e                j            i
!                                   k             k               k
!           = s   + sum  w  (alpha(x ) grad phi (x ) . grad phi (x )
!              ij    k    k                    j               i
!     EL3004 is called by several types of element subroutines
!            It must be preceded by ELP subroutines of the 6-series
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mdecl, ndecl, nsrow
      double precision alpha(mdecl,3), s(nsrow,nsrow), w(mdecl), &
                       dphidx(ndecl,mdecl,2)

!     alpha    i     array of length m x 3 in which the values of
!                    alpha  , alpha  , alpha
!                         11       12       22
!                    are stored in the sequence:
!                                            k
!                    alpha ( k,i ) = alpha (x )   i = 1(1)3, k=1(1)m
!                                         i
!     dphidx   i     Array of length n x m x 2  containing the derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = dphidx (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = dphidx (i,k,2);
!                         i        k
!                    If the element is a linear triangle (n=3), then the
!                    derivatives are constant and only k = 1 is filled.
!     mdecl    i     Number of integration points (m)
!     ndecl    i     Number of basis functions (n)
!     nsrow    i     Size of the element matrix S
!     s        o     Element stiffness matrix to be filled
!                    Since SEPRAN assumes an one-dimensional storage, S
!                    must be stored transposed.
!                    S has size nsrow x nsrow, but only the part (1:n,1:n)
!                    is filled.
!     w        i     array of length m containing the weights for the
!                    numerical integration
!     work           Work array of length m x 3, to store products of
!                    alpha   x w temporarily
!                         ij
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision a11con, a12con, a22con, work(m,3)
      double precision som
      integer i, j, k

!     a11con         Constant value of alpha_11
!     a12con         Constant value of alpha_12
!     a22con         Constant value of alpha_22
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     som            Help variable to store a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el3004' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el3004'
         write(irefwr,1) 'jsecnd, n, m, nsrow', jsecnd, n, m, nsrow
         call prinrl ( w, m, 'w' )
         call prinrl1 ( dphidx(1,1,1), m, n, 'dphidx' )
         call prinrl1 ( dphidx(1,1,2), m, n, 'dphidy' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( jsecnd==1 ) then

!     --- jsecnd = 1   a12=0; a11=a22  a11 and w constant

         a11con = w(1) * alpha(1,1) * m
         if ( n==3 ) then

!        --- n = 3;  Linear triangle  ( derivatives constant )

            do i = 1,n
               do j = 1,i
                  s(i,j) = a11con * ( dphidx(i,1,1) * dphidx(j,1,1) + &
                                      dphidx(i,1,2) * dphidx(j,1,2) )
               end do  ! j = 1,i
            end do  ! i = 1,n

         else

!        --- n # 3  other cases,  summation over integration points

            do i = 1,n
               do j = 1,i
                  som = 0d0
                  do k = kstep, m, kstep
                     som = som + ( dphidx(i,k,1) * dphidx(j,k,1) + &
                                   dphidx(i,k,2) * dphidx(j,k,2) )
                  end do  ! k = kstep, m, kstep

                  s(i,j) = a11con * som
               end do  ! j = 1,i
            end do  ! i = 1,n

         end if  ! ( n==3 )

      else if ( jsecnd==2 ) then

!     --- jsecnd = 2   a12=0; a11=a22

          if(n==3) then

!         --- n = 3;  Linear triangle  ( derivatives constant )

            a11con = 0d0
            do k = 1,m
               a11con = a11con + w(k) * alpha(k,1)
            end do  ! k = 1,m

            do i = 1,n
               do j = 1,i
                  s(i,j) = a11con * ( dphidx(i,1,1) * dphidx(j,1,1) + &
                                      dphidx(i,1,2) * dphidx(j,1,2) )
               end do  ! j = 1,i
            end do  ! i = 1,n

         else

!        --- n # 3  other cases,  summation over integration points
!              Store w  x alpha  (k) in work(k,1)
!                   k        11

            do k = kstep, m, kstep
               work(k,1) = w(k) * alpha(k,1)
            end do  ! k = kstep, m, kstep

            do i = 1,n
               do j = 1,i
                  som = 0d0
                  do k = kstep, m, kstep
                     som = som + work(k,1)*(dphidx(i,k,1)*dphidx(j,k,1)+ &
                                            dphidx(i,k,2)*dphidx(j,k,2))
                  end do  ! k = kstep, m, kstep
                  s(i,j) = som
               end do  ! j = 1,i
            end do  ! i = 1,n

         end if

      else if ( jsecnd==3 ) then

!     --- jsecnd = 3   a12#0; a11=a22

         if(n==3) then

!        --- n = 3;  Linear triangle  ( derivatives constant )

            a11con = 0d0
            a12con = 0d0
            do k = kstep, m, kstep
               a11con = a11con + w(k) * alpha(k,1)
               a12con = a12con + w(k) * alpha(k,2)
            end do  ! k = kstep, m, kstep

            do i = 1,n
               do j = 1,i
                  s(i,j) = a11con * ( dphidx(i,1,1) * dphidx(j,1,1) + &
                                      dphidx(i,1,2) * dphidx(j,1,2) ) &
                         + a12con * ( dphidx(i,1,1) * dphidx(j,1,2) + &
                                      dphidx(i,1,2) * dphidx(j,1,1) )
               end do  ! j = 1,i
            end do  ! i = 1,n

         else

!        --- n # 3  other cases,  summation over integration points
!              Store w(k)  x alpha  (k) in work(k,ij)
!                               ij

            do k = kstep, m, kstep
               work(k,1) = w(k) * alpha(k,1)
               work(k,2) = w(k) * alpha(k,2)
            end do  ! k = kstep, m, kstep
            do i = 1,n
               do j = 1,i
                  som = 0d0
                  do k = kstep, m, kstep
                     som = som + work(k,1)*(dphidx(i,k,1)*dphidx(j,k,1)+ &
                                            dphidx(i,k,2)*dphidx(j,k,2)) &
                         + work(k,2) * ( dphidx(i,k,1) * dphidx(j,k,2) + &
                                         dphidx(i,k,2) * dphidx(j,k,1) )
                  end do  ! k = kstep, m, kstep

                  s(i,j) = som

               end do  ! j = 1,i
            end do  ! i = 1,n

         end if

      else if ( jsecnd==4 ) then

!     --- jsecnd = 4   general

         if(n==3) then

!        --- n = 3;  Linear triangle  ( derivatives constant )

            a11con = 0d0
            a12con = 0d0
            a22con = 0d0
            do k = kstep, m, kstep
               a11con = a11con + w(k) * alpha(k,1)
               a12con = a12con + w(k) * alpha(k,2)
               a22con = a22con + w(k) * alpha(k,3)
            end do  ! k = kstep, m, kstep
            do i = 1,n
               do j = 1,i
                  s(i,j) = a11con * ( dphidx(i,1,1) * dphidx(j,1,1) ) &
                         + a22con * ( dphidx(i,1,2) * dphidx(j,1,2) ) &
                         + a12con * ( dphidx(i,1,1) * dphidx(j,1,2) + &
                                      dphidx(i,1,2) * dphidx(j,1,1) )
               end do  ! j = 1,i
            end do  ! i = 1,n

         else

!        --- n # 3  other cases,  summation over integration points
!              Store w(k)  x alpha  (k) in work(k,ij)
!                               ij

            do j = 1,3
               do k = 1,m
                  work(k,j) = w(k) * alpha(k,j)
               end do  ! k = 1,m
            end do  ! j = 1,3

            do i = 1,n
               do j = 1,i
                  som = 0d0
                  do k = kstep, m, kstep
                     som = som + work(k,1)*(dphidx(i,k,1)*dphidx(j,k,1)) &
                               + work(k,3)*(dphidx(i,k,2)*dphidx(j,k,2)) &
                         + work(k,2) * ( dphidx(i,k,1) * dphidx(j,k,2) + &
                                         dphidx(i,k,2) * dphidx(j,k,1) )
                  end do  ! k = kstep, m, kstep

                  s(i,j) = som
               end do  ! j = 1,i
            end do  ! i = 1,n

         end if

      end if

!     --- the upper triangle of s is computed by transposition

      do i = 1, n
         s(1:i-1,i) = s(i,1:i-1)
      end do  ! i = 1, n

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( s, n, n, 's' )
         write(irefwr,*) 'End el3004'

      end if  ! ( debug )
      call erclos ( 'el3004' )

      end
