      subroutine el6063 ( ichois, tout, txxn, txyn, tyyn, ux, uy, &
                          vx, vy, w1, det, work, f, &
                          theta, mlab, mone1, m1 )
! ======================================================================
!
!        programmer    Patrick Anderson/Gerard Haagh
!        version  2.0  date 24-05-2001 New parameter list
!        version  1.1  date 04-12-1997 Tensor L set to 0 in gas domains
!        version  1.0  date 16-10-1997 First draft; derived from EL6062
!
!   copyright (c) 1997-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute the right hand side for the elastic Finger tensor evolution
!   equation using the compressible Leonov model. (2D version.)
!   Developed for Gerard Haagh
!   Derived for 2D stress compuations from EL6062. All terms not
!   connected to the xx, xy, or yy components deleted. All txzn, &
!   tyzn, and tzzn replaced by 1d0 (value of the Finger tensor component
!   for 2D case)
!   Calls to EL6039 (3D case) replaced by calls to EL6032 (2D case).
!   Calls to EL6083 (3D case) replaced by calls to EL6053 (2D case).
!   Calls to EL6095 (3D case) replaced by calls to EL6037 (2D case).
! **********************************************************************
!
!                       KEYWORDS
!
!     spectral
!     visco_elastic
!     2d
!     leonov
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mone1, m1, ichois
      double precision tout(m1), txxn(m1),  txyn(m1),  tyyn(m1), &
                       ux(m1), uy(m1), vx(m1), vy(m1)
      double precision w1(mone1),det(mone1,mone1), &
                       work(m1), theta(m1), mlab(m1), f(m1)

!    det       i   Transformation determinant, length m1
!    f         i   Work array
!    ichois    i   Number of the Finger tensor component to be computed
!    m1        i   Number of integration points
!    mlab      i   Material labels
!    mone1     i   Number of integration points in one dimension
!    theta     i   Relaxation time
!    tout      o   Right hand side of Finger tensor evolution equation
!    txxn      i   xx-component of previous Finger tensor solution
!    txyn      i   xy-component of previous Finger tensor solution
!    tyyn      i   yy-component of previous Finger tensor solution
!    ux        i   Velocity gradient du/dx
!    uy        i   Velocity gradient du/dy
!    vx        i   Velocity gradient dv/dx
!    vy        i   Velocity gradient dv/dy
!    w1        i   Indicates the Legendre Gauss-Lobatto weights for
!                  numerical integration in the x-direction corresponding
!                  to the reference element, length mone1
!    work      i   Work array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, jddprv, jzprv
      double precision uxorg(9), uyorg(9), vxorg(9), vyorg(9)

!     i       Loop parameter
!     jddprv  Keep jadd
!     jzprv   Keep jzero
!     uxorg   Original velocity gradient du/dx
!     uyorg   Original velocity gradient du/dy
!     vxorg   Original velocity gradient dv/dx
!     vyorg   Original velocity gradient dv/dy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL6032         compute the spectral element Helmholtz residual
!     EL6053         compute contributions to the spectral element vector
!                    and the spectral element residual
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       1   ICHOIS is incorrect
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'el6063' )

!     --- Keep JADD and JZERO, just not to disturb diff. eqn

      jddprv = jadd
      jzprv  = jzero

!     --- Check value of parameter ICHOIS

      if ( ichois.lt.1 .or. ichois.gt.3 ) then

!     --- ichois has wrong value

         call errint ( ichois, 1 )
         call errsub ( 1, 1, 0, 0 )
         go to 1000
      end if

!     --- For all integration points in air/gas domain:
!         set velocity gradient tensor to null tensor;
!         maintain orginal values in **org arrays.

      if ( m1.gt.9 ) then

!     --- m1 too small
!         Arrays uxorg, uyorg, etc. too short

         call errchr ( 'uxorg', 1 )
         call errint ( 9, 1 )
         call errint ( m1, 2 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000
      end if
      do i = 1, m1
         if( mlab(i).lt.0.5d0 )then
            uxorg(i) = ux(i)
            uyorg(i) = uy(i)
            vxorg(i) = vx(i)
            vyorg(i) = vy(i)
            ux(i) = 0d0
            uy(i) = 0d0
            vx(i) = 0d0
            vy(i) = 0d0
         end if
      end do

!     --- Build F = tr(B) + tr(inv(B))
!         2D case: tzz = 1d0, txz = tyz = 0d0
!         first compute det(B); this should be equal to 1d0

      do i = 1, m1
         f(i) = txxn(i) * tyyn(i) - txyn(i)**2d0
      end do
      do i = i, m1
         if( dabs (f(i) - 1.0d0).gt.1.d-1 )then
            write (*,*) 'EL6063: Error in F : det (B) = ', f(i)
            call instop
         end if
      end do

!     --- compute II(B)

      do i = 1, m1
         f(i) = (txxn(i) * tyyn(i)  + tyyn(i) + txxn(i) - &
                 txyn(i)**2d0 )

!        --- subtract II(B) from tr(B)

         f(i) = txxn(i) + tyyn(i) + 1d0 - f(i)
      end do
      go to ( 100, 200, 400 ) ichois

!     --- Compute XX-component of Finger tensor
!         Build S11 txxn : [ -(u ddx + v ddy) + &
!                            2/3 ( 2 ux - vy ) - 1/(2 theta)
!                            (txxn - 1/3 F) ] txxn
!         Build Helmholtz part

100   do i = 1, m1
         work(i) =  2d0/3d0 * (2d0 * ux(i) - vy(i) ) - &
            (1d0/(2d0 * theta(i)) ) * ( txxn(i) - 1d0/3d0 * f(i) )
      end do

!     --- Initialize TOUT

      jadd = 0

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      call el6032 ( work, tout, txxn, w1, w1, det, mone1, m1 )

!     --- Add (Substract) Convection term to TOUT

      jadd = 1

!     --- Build and add S12 txyn to tout : ( 2 uy - 1/(2 theta) * txyn )txyn
!         Build Helmholtz part

      do i = 1, m1
         work(i) = 2d0 * uy(i) - (1d0/(2d0*theta(i))) * txyn(i)
      end do

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      jadd = 1
      call el6032( work, tout, txyn, w1, w1, det, mone1, m1 )

!     --- Other contributions

      do i = 1, m1
         work(i) = 1d0/(2d0 * theta(i))
      end do
      jadd = 1
      call el6053 ( work, tout, w1, det, mone1, m1 )
      go to 900

!     --- Compute xy-component of Finger tensor
!         Build S21 txxn :  vx * txxn
!         Build Helmholtz part

200   do i = 1, m1
         work(i) = vx(i)
      end do

!     --- Initialize tout

      jadd = 0

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      call el6032( work, tout, txxn, w1, w1, det, mone1, m1 )

!     --- Build and add S22 txyn to tout  :  [ -(u ddx + v ddy ) + &
!                            1/3 ( ux + vy ) - 1/(2 theta)
!                            (txxn - 1/3 F) ] txyn
!         Build Helmholtz part

      do i = 1, m1
         work(i) =  1.0d0/3.0d0 * (ux(i) + vy(i) ) - &
            (1d0/(2d0 * theta(i)) ) * ( txxn(i) - 1d0/3d0 * f(i) )
      end do

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      jadd = 1
      call el6032( work, tout, txyn, w1, w1, det, mone1, m1 )

!     --- Add convection term to TOUT

      jadd = 1

!     --- Build S24 tyyn :  (uy -1/(2 theta) txyn) tyyn
!         Build Helmholtz part

      do i = 1, m1
         work(i) = uy(i) - (1d0/(2d0*theta(i))) * txyn(i)
      end do

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      jadd = 1
      call el6032( work, tout, tyyn, w1, w1, det, mone1, m1 )
      go to 900

!     --- Compute yy-component of Finger tensor
!         Build S42 txyn :  (2 vx- (1/( 2 theta)) txyn ) txyn
!         Build Helmholtz part

400   do i = 1, m1
         work(i) = 2d0 * vx(i) - (1d0/(2d0*theta(i))) * txyn(i)
      end do

!     --- Initialize TOUT

      jadd = 0

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      call el6032( work, tout, txyn, w1, w1, det, mone1, m1 )

!     --- Build S44 tyyn : [ -(u ddx + v ddy ) + &
!                            2/3 ( - ux +2 vy ) - 1/(2 theta)
!                            (tyyn - 1/3 F) ] tyyn
!         Build Helmholtz part

      do i = 1, m1
         work(i) =  2d0/3d0 * (- ux(i) + 2d0*vy(i) ) - &
            (1d0/(2d0 * theta(i)) ) * ( tyyn(i) - 1d0/3d0 * f(i) )
      end do
      jzero = 2
      call el6032( work, tout, tyyn, w1, w1, det, mone1, m1 )

!     --- Add Convection term to TOUT

      jadd = 1

!     --- Convection is NEGLECTED
!         other contributions

      do i = 1, m1
         work(i) = 1d0/(2d0 * theta(i))
      end do
      jadd = 1
      call el6053 ( work, tout, w1, det, mone1, m1 )
      go to 900

!     --- Restore JADD, JZERO and velocity gradients to previous values

900   jadd  = jddprv
      jzero = jzprv
      do i = 1, m1
         if( mlab(i).lt.0.5d0 )then
            ux(i) = uxorg(i)
            uy(i) = uyorg(i)
            vx(i) = vxorg(i)
            vy(i) = vyorg(i)
         end if
      end do
1000  call erclos( 'el6063' )
      end
