      subroutine eld603 ( coorloc, iuser, user, swork )
! ======================================================================
!
!        programmer    Peter Minev
!        version  1.4  date 09-03-2013 Replace elflr7 by elflr71
!        version  1.3  date 12-12-2012 Long integers
!        version  1.2  date 18-09-2002 New call to el3007
!
!   copyright (c) 1995-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill spectral element vector for inhomogeneous Neumann
!     condition
!     ELD603 is a help subroutine for subroutine DERIV
!     it is called through the intermediate subroutine ELD000
!     so:
!     DERIV
!       -  loop over spectral elements
!          -  ELD000
!             -  ELD603
!     itype = 603  :   one dimensional spectral element
!                      (m nodal points)
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*)
      double precision coorloc(*), user(*), swork(*)

!     coorloc     i     array of length npoint containing the coorlocdinates of
!                    the nodal points with respect to the global numbering
!                    x  = coorloc (1,i)
!                     i
!     iuser    i     integer user array to pass user information from
!                    main program to subroutine, see STANDARD PROBLEMS
!     swork          spectral element work array
!                    total length is: 2*m*m + m*(5 + ndimlc)
!     user     i     real user array to pass user information from
!                    main program to subroutine, see STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iparry, ipxref, ipx, ipwref, ipw, ipold, ipwork, ipd, i
      double precision rdum
      save iparry, ipxref, ipx, ipwref, ipw, ipold, ipwork, ipd

!     i         Counting variable
!     iparry    indicates array in which the coefficients are stored
!               length m
!     ipd       indicates the derivative of the basic functions, m*m
!     ipold     indicates the old solution in the nodal points, length
!               m x 1
!     ipw       indicates the corresponding weights in the element
!               length m
!     ipwork    work array, lenght m*m
!     ipwref    indicates the Legendre Gauss-Lobatto weights for
!               numerical integration corresponding to the reference
!               element, length m
!     ipx       indicates the corresponding points in the element, &
!               length m x ndimlc
!     ipxref    indicates the Legendre Gauss-Lobatto points corresponding
!               to the reference element [-1,1], length m
!     rdum      dummy variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0603    initialize parameters for subroutine ELD603
!     EL3007    Fills old solution and derivatives
!     EL6009    compute the boundary integral
!     ELFLR71   Fill variable coefficients in array ARRAY
!     ELP640    compute basisfunctions, derivatives and weights for
!               numerical integration corresponding to the reference
!               element (element independent)
!     ELP644    compute the integration points, weights and derivatives
!               (element dependent)
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
!     the user provides the coefficients through the arrays IUSER and
!     USER in the usual way, the sequence is:
!     1:  alpha, non-zero
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1781   The possibility ifunc>10000 not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('eld603')
      if (ifirst==0) then

!     --- ifirst = 0, compute element independent quantities
!         perform initialisations

         iparry = 1
         call el0603(iuser, user, swork(iparry))

!        --- initialize pointers in the work array

         ipxref = iparry + m
         ipx = ipxref + m
         ipwref = ipx + m*ndimlc
         ipw = ipwref + m
         ipold = ipw + m
         ipwork = ipold + m
         ipd = ipwork + m*m

!        --- compute points and weights for numerical integration
!            corresponding to the reference element

         call elp640(1, swork(ipxref), swork(ipwref), swork(ipd), &
                     swork(ipwork), m)
      end if
      if ( ierror/=0 ) go to 1000
      if ( icheld==10 ) then

!     --- calculate the source term
!         first compute integration points, weights and derivatives

         call elp644 ( coorloc, index1, swork(ipx), swork(ipwref), &
                       swork(ipw), swork(ipd), m)

!        --- fill variable coefficients if necessary
!            first fill old solution if necessary

         if (ind(lengthmst)==1) then
            call el3007 ( swork(ipold), swork(ipwork), rdum, &
                          rdum, m, m, ndimlc, ncomp, index1, &
                          rdum, swork(ipold), jtrans, numold, 1 )
         end if
         if ((ind(1)>0) .or. (ist(1)>0)) then

!        --- coefficient is variable

            if ( ind(1)>10000 ) then
               call errint ( 1, 1 )
               call errsub ( 1781, 1, 0, 0 )
            end if
            call elflr71(1, user, swork(iparry), &
                        swork(ipx), swork(ipx), m, m, rdum, &
                        1, index1, iuser, swork(ipold), swork )
         end if
         call el6009(swork(iparry), elemvc1, swork(ipw), m)
      else

!     --- set the source vector to 0

         do 100 i = 1, m
            elemvc1(i) = 0d0
100      continue
      end if
1000  call erclos('eld603')
      end
