      subroutine eldiscint1iso ( massmt, stiffmt, dldtmt, latentvc, &
                                 dphidx, temperature, coefarr, &
                                 intrule, phi, h, mloc )
! ======================================================================
!
!        programmer    John Brusche
!        version  2.0  date 11-06-2010 Remove parameters m and n
!        version  1.0  date 30-05-2006
!
!   copyright (c) 2005-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Construct element mass matrix, stiffness matrix, derivative latent heat
!   matrix and latent heat vector using discontinous integration.
!   This is the isothermal case.
!   The code differs from that using an artificial mushy region, since the
!   dLdT matrix and L-vector are computed differently.
!   This is the 1D version.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer          intrule, mloc
      double precision temperature(*), coefarr(m,*), &
                       dphidx(n,m), massmt(n,n), h, phi(n,mloc), &
                       stiffmt(n,n), dldtmt(n,n), latentvc(n)

!     coefarr        i    Array containing the coefficients of the PDE
!                         Pos. (.,1)   k_solid
!                         Pos. (.,2)   k_fluid
!                         Pos. (.,3)   rho
!                         Pos. (.,4)   c_solid
!                         Pos. (.,5)   c_fluid
!                         Pos. (.,6)   Q
!                         Pos. (.,7)   L
!                         Pos. (.,8)   T_melt
!     dldtmt         o    Element dldt matrix
!     dphidx         i    derivatives of basis functions
!     intrule        i    Numerical integration rule to be used for
!                         discontinuous integration
!                         1: Newton-Cotes, 1st order, 'substraction approach'
!                         2: Newton-Cotes, 2nd order, 'substraction approach'
!                         3: Newton-Cotes, 1st order, &
!                            'subdivision into 3 subtriangles'
!                         4: Newton-Cotes, 1st order, &
!                            'averaging of heat capacity'
!     latentvc       o    Element latent heat vector
!     massmt         o    Element mass matrix
!     stiffmt        o    Element stiffness matrix
!     temperature    i    temperature vector in integration points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, l
      double precision rho(2), cs, cl, tm, dh, wleft, latentheat, &
                       cright, kl, ks, rhos, rhol, rholeft, rhoright, &
                       kleft, kright, c(2,mloc+1), w(2,mloc+1), &
                       kappa(2,mloc+1), sum, phixs(2), &
                       phicut(n,mloc+1,2), &
                       wright, cleft, dtdx

!     c              specific heat matrix
!     cl             Specific heat of the liquid
!     cleft          Specific heat for 'left' side of the element
!     cright         Specific heat for 'right' side of the element
!     cs             Specific heat of the solid
!     dh             partial length of edge
!     dtdx           Derivitative of temperature w.r.t. x
!     h              width of the element
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     kappa          Diffusivity matrix
!     kl             Diffusivity of the liquid
!     kleft          Diffusivity for 'left' side of the element
!     kright         Diffusivity for 'right' side of the element
!     ks             Diffusivity of the solid
!     l              Counting variable
!     latentheat     latent heat
!     mloc           local copy of m modified for Newton-Cotes
!     phi            basis functions
!     phicut         phi matrix for intersected element
!     phixs          phi values in intersection point xs
!     rho            density rho for 2 phases
!     rhol           rho liquid
!     rholeft        rho in left part
!     rhoright       rho in right part
!     rhos           rho solid
!     sum            Help parameter to compute a sum
!     tm             Melting temperature
!     w              weights matrix
!     wleft          Weight factor for 'left' side of the element
!     wright         Weight factor for 'right' side of the element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eldiscint1iso' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eldiscint1iso'
      end if

!     --- get parameters

      ks  = coefarr(1,1)
      kl  = coefarr(1,2)
      rhol = coefarr(1,3)
      cs  = coefarr(1,4)
      cl  = coefarr(1,5)
      latentheat = coefarr(1,7)
      Tm  = coefarr(1,8)
      if ( ind(9)==0 ) then
         rhos  = rhol
      else
         rhos  = coefarr(1,9)
      end if  ! ( ind(9)==0 )

!     --- compute width of left section of the element (1D)
!         the most basic way is to use linear interpolation
!         assume temperature(1) is the left node, temperature(2) on
!         the right node

      dh = h*(Tm - temperature(1))/(temperature(2) - temperature(1))

!     --- compute the weights for both intervals

      wleft = dh/2d0
      wright = (h-dh)/2d0

!     --- set physical parameters for the subsections of the element

      if (temperature(1)<Tm) then

!     --- left side is solid, right side is liquid

         cleft  = cs
         cright = cl
         kleft  = ks
         kright = kl
         rholeft = rhos
         rhoright = rhol
      else ! otherwise
         cleft  = cl
         cright = cs
         kleft  = kl
         kright = ks
         rholeft = rhol
         rhoright = rhos
      end if

!     --- build c, kappa and weights matrices

      do i = 1, mloc+1
         c(1,i) = cleft
         c(2,i) = cright
         w(1,i) = wleft
         w(2,i) = wright
         kappa(1,i) = kleft
         kappa(2,i) = kright
      end do
      rho(1) = rholeft
      rho(2) = rhoright

!     --- in case of 1D Newton Cotes the values of the basis functions in
!         the intersection point xs are simply either dh or h-dh.
!         we construct a 3 dimensional matrix phicut(i,j,k) as described
!         above.

      phixs(1) = 1d0-dh/h
      phixs(2) = dh/h

      do i = 1, n
         phicut(i, 1, 1) = phi(i, 1)
         phicut(i, 1, 2) = phixs(i)
         phicut(i, 2, 1) = phixs(i)
         phicut(i, 2, 2) = phi(i, 2)
      if (intrule==2) then

!     --- quadratic, in order to get the factor 4, we have multiplied each
!            of the following terms by a factor 2

            phicut(i, 3, 1) = phicut(i,1,1) + phicut(i,2,1)
            phicut(i, 3, 2) = phicut(i,1,2) + phicut(i,2,2)
         end if
      end do

!     --- compute derivative of temperature w.r.t. x

      dtdx = (temperature(2) - temperature(1))/h

!     --- fill element mass matrix

      do i = 1, n            ! basis functions
         do j = 1, n         ! basis functions
            sum = 0d0
            do l = 1, 2      ! phases
               if (intrule==1) then
                  do k = 1, 2      ! integration points
                     sum = sum + &
                      rho(l)*c(l,k)*w(l,k)*phicut(i,k,l)*phicut(j,k,l)
                  end do
               else if (intrule==2) then
                  do k = 1, 3      ! integration points
        ! divide by 3 instead of 6 because of the factor .5 already in w
                     sum = sum + &
                  rho(l)*c(l,k)*w(l,k)*phicut(i,k,l)*phicut(j,k,l)/3d0
                  end do
               end if
            end do
            massmt(i,j) = sum
         end do
      end do

!     --- fill element stiffness matrix

      do i = 1, n            ! basis functions
         do j = 1, n         ! basis functions
            sum = 0d0
            do l = 1, 2      ! phases
               do k = 1, mloc      ! integration points
                  sum = sum + kappa(l,k)*w(l,k)
               end do
            end do
            stiffmt(i,j) = sum*dphidx(i,1)*dphidx(j,1)
         end do
      end do

!     --- fill element dldt matrix

      do i = 1, n
         do j = 1, n
            dldtmt(i,j) = rhol*latentheat*phixs(i)*phixs(j)/ &
                          abs(dtdx)
         end do
      end do

!     --- fill latent heat element vector
!         latentvc = 0 in solid, so restrict to liquid

      if (temperature(1)<Tm) then

!     --- 'right' section is liquid

         do i = 1, n     ! basis functions
            sum = 0d0
            do j = 1, mloc  ! integration points
               sum = sum + rhol*latentheat*wright*phicut(i,j,2)
            end do
            latentvc(i) = sum
         end do

      else

!     --- 'left' section is liquid

         do i = 1, n     ! basis functions
            sum = 0d0
            do j = 1, mloc  ! integration points
               sum = sum + rhol*latentheat*wleft*phicut(i,j,1)
            end do
            latentvc(i) = sum
         end do

      end if

      call erclos ( 'eldiscint1iso' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End eldiscint1iso'

      end if

      end
