      subroutine eldiscint2sl( intrule, rho, c, kappa, latentheat, &
                              phi, dphidx, delta, massmt, stiffmt, &
                              latentvc, nsign, mloc, liquid)
! ======================================================================
!
!        programmer    John Brusche
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 30-05-2006
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Construct element mass matrix, stiffness matrix, derivative latent heat
!   matrix and latent heat vector using discontinous integration.
!   This is the mushy case. Solid or liquid region.
!   The code differs from the isothermal case, since the dLdT matrix and
!   L-vector are computed differently.
!   This is the 1D version.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer          intrule, mloc
      double precision c, kappa, rho, latentheat, &
                       dphidx(n,ndim), massmt(n,n), delta, phi(n,mloc), &
                       stiffmt(n,n), latentvc(n), nsign
      logical          liquid

!     c              i    Specific heat
!     delta          i    Volume of element
!     dphidx         i    derivatives of basis functions
!     intrule        i    Numerical integration rule to be used for
!                         discontinuous integration
!                         1: Newton-Cotes, 1st order, 'substraction approach'
!                         2: Newton-Cotes, 2nd order, 'substraction approach'
!                         3: Newton-Cotes, 1st order, &
!                            'subdivision into 3 subtriangles'
!                         4: Newton-Cotes, 1st order, &
!                            'averaging of heat capacity'
!     kappa          i    Diffusivity
!     latentheat    i    latent heat
!     latentvc      i/o   Element latent heat vector
!     liquid         i    If true, the element is liquid, otherwise solid
!     massmt        i/o   Element mass matrix
!     mloc           i    local copy of m modified for Newton-Cotes
!     nsign          i    determines whether contribution is added or
!                         subtracted
!     phi            i    basis functions
!     rho            i    density
!     stiffmt       i/o   Element stiffness matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision sum, phicut(n,mloc), w

!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     phicut         phi matrix for intersected element
!     sum            Help parameter to compute a sum
!     w              Weight for Newton Cotes integration rule
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL2        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eldiscint2sl' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eldiscint2sl'
         call prinrl2(phi, n, mloc, 'phi s/l')
      end if

      w = delta/6d0

      if (intrule==2) then
         do i = 1, n

            phicut(i, 1) = 0.5d0*(phi(i,1) + phi(i,2))
            phicut(i, 2) = 0.5d0*(phi(i,1) + phi(i,3))
            phicut(i, 3) = 0.5d0*(phi(i,2) + phi(i,3))

         end do
      end if

!     --- fill element mass matrix

      do i = 1, n            ! basis functions
         do j = 1, n         ! basis functions
            sum = 0d0
            if (intrule==1) then
               do k = 1, mloc      ! integration points
                  sum = sum + phi(i,k)*phi(j,k)
               end do
            else if (intrule==2) then
               do k = 1, mloc      ! integration points
                  sum = sum + phicut(i,k)*phicut(j,k)
               end do
            end if
            massmt(i,j) = massmt(i,j) + nsign*rho*c*w*sum
         end do
      end do

!     --- fill element stiffness matrix

      do i = 1, n            ! basis functions
         do j = 1, n         ! basis functions
            sum = 0d0
            do k = 1, mloc      ! integration points
               sum = sum + (dphidx(i,1)*dphidx(j,1)+ &
                                    dphidx(i,2)*dphidx(j,2))
            end do
            stiffmt(i,j) = stiffmt(i,j) + nsign*kappa*w*sum
         end do
      end do

!     --- latent heat element vector is>0 only for liquid subelement

      if ( liquid ) then

         do i = 1, n     ! basis functions
            sum = 0d0
            do j = 1, mloc  ! integration points
               sum = sum + phicut(i,j)
            end do
            latentvc(i) = latentvc(i) + nsign*rho*latentheat*w*sum
         end do

      end if

      call erclos ( 'eldiscint2sl' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End eldiscint2sl'
         call prinrl(latentvc, n, 'latentvc s/l')

      end if

      end
