      subroutine eldiscint3iso ( massmt, stiffmt, dldtmt, latentvc, &
                                 dphidx, temperature, coefarr, &
                                 intrule, phi, mloc, x)
! ======================================================================
!
!        programmer    John Brusche
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.0  date 16-06-2010 New parameter list
!        version  1.1  date 21-12-2005 Added intrule, added NC 2nd order
!                                      NC 1st order revised
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Construct element mass matrix, stiffness matrix, derivative latent heat
!   matrix and latent heat vector using discontinous integration.
!   This is the 3D version.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer          intrule, mloc
      double precision temperature(*), coefarr(m,*), massmt(n,n), &
                       dphidx(n,m,ndim), x(n,ndim), phi(n, mloc), &
                       latentvc(n), stiffmt(n,n), dldtmt(n,n)

!     coefarr        i    Array containing the coefficients of the PDE
!                         Pos. (.,1)   ksolid
!                         Pos. (.,2)   kfluid
!                         Pos. (.,3)   rho
!                         Pos. (.,4)   csolid
!                         Pos. (.,5)   cfluid
!                         Pos. (.,6)   Q
!                         Pos. (.,7)   L
!                         Pos. (.,8)   Tmelt
!     dldtmt         o    Element dldt matrix
!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     intrule       i    Numerical integration rule to be used for
!                         discontinuous integration
!                         1: Newton-Cotes, 1st order, 'substraction approach'
!                         2: Newton-Cotes, 2nd order, 'substraction approach'
!     latentvc       o    Element latent heat vector
!     massmt         o    Element mass matrix
!     stiffmt        o    Element stiffness matrix
!     temperature    i    temperature vector in integration points
!     x              i    x-coordinates of the integration points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, nnegativet, negativet(3), xhat(4), k, l, &
              inttype, index2(2,3), index1(4,6), index4(4), &
              index3(4,2)
      double precision rhol, rhos, cs, cl, latentheat, tm, wtetra1, &
                       ctetra, kl, ks, gradtemp(ndim), nrmgradt, &
                       ktetra1, ktetra, det, det1, wphi1, &
                       sum1, sum2, v(3, mloc), h(4), dh(4), phixs(n,4), &
                       wtetra, ctetra1, xs(4,3), phicut(n,10,6), &
                       detcut(2), &
                       e(4,3), xtetra(4,3,6), deters(6), weights(6), &
                       c(6), kappa(6), wcut(2), fac

!     c              matrix containing local specific heat
!     cl             Specific heat of the liquid
!     cs             Specific heat of the solid
!     ctetra         Specific heat for 'tetra' part of the element
!     ctetra1        Specific heat for 'tetra1' part of the element
!     det            determinant for cumputing weigth factor of whole element
!     det1           determinant for cumputing weigth factor sub element
!     detcut        determinant for computing weigth factor intersection
!                    triangle
!     deters         determinants for computing weigth factors of subdivision
!                    tetrahedra
!     dh             partial lengths of edges
!     e              Work array of length 4 x 3, to store the factors
!                     ij
!                    e.
!     fac            additional scaling factor for weigths when intrule==2
!     gradtemp       gradient of the temperature
!     h              lengths of edges
!     i              Counting variable
!     index1         Integer index matrix
!     index2         Integer index matrix
!     index3         Integer index matrix
!     index4         Integer index vector
!     inttype        indicates whether we cut the tetrahedron in 2 prisms or
!                    a tetrahedron and rest
!     j              Counting variable
!     k              Counting variable
!     kappa          matrix containing local diffusivities
!     kl             Diffusivity of the liquid
!     ks             Diffusivity of the solid
!     ktetra         Diffusivity for 'tetra' part of the element
!     ktetra1        Diffusivity for 'tetra1' part of the element
!     l              General loop variable
!     latentheat    latent heat
!     mloc           local copy of m modified for Newton-Cotes
!     negativet     node numbers of nodes with temperature less than Tm
!     nnegativet    number of nodes with temperature less than Tm
!     nrmgradt       2-norm of the gradient of temperature T
!     phi            array of length n * m, values of shape
!                    functions in integration points in the sequence:
!                    phii(xg^k) = phi (i,k)
!     phicut         phi matrix for intersected element
!     phixs          phi values in intersection point xs
!     rhol           Density of the liquid
!     rhos           Density of the solid
!     sum1           Help parameter to compute a sum
!     sum2           Help parameter to compute a sum
!     tm             Melting temperature
!     v              Help matrix to compute determinants
!     wcut          weight factor of the intersection triangle/trapezium
!     weights        weights of the subdivision tetrahedra
!     wphi1          Help variable for computing integrals
!     wtetra         Weight factor for 'tetra' part of the element
!     wtetra1        Weight factor for 'tetra1' part of the element
!     xhat          Local numbering of nodes
!     xs             coordinates of the intersection points
!     xtetra        coordinates of the subdivision tetrahedra
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELCOMPDET     compute the determinant for a triangle in 3D space
!     ELDISCINT31    Fill array phicut for 2nd order Newton Cotes
!     ELOUTR         Compute outer product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data index2 /1,2,4,1,3,4/, index4 /1,2,3,1/
! ======================================================================
!
      call eropen ( 'eldiscint3iso' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eldiscint3iso'
100      format ( a, 1x, (10i6) )
110      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- get parameters

      ks  = coefarr(1,1)
      kl  = coefarr(1,2)
      rhol = coefarr(1,3)
      if ( ind(9)==0 ) then
         rhos  = rhol
      else
         rhos  = coefarr(1,9)
      end if  ! ( ind(9)==0 )
      cs  = coefarr(1,4)
      cl  = coefarr(1,5)
      latentheat = coefarr(1,7)*rhol
      Tm  = coefarr(1,8)

!     --- Define temperatures<Tm as negative, otherwise as positive
!         For an intersected tetrahedron it holds that either one node
!         has different sign w.r.t. the other nodes, or the number of
!         negative and positive nodes are both equal to 2
!         Two subdivisions of the tetrahedron are possible:
!         Case 1: tetrahedron + rest (inttype = 1)
!         Case 2: 2 prisms           (inttype = 2)
!         First count the number of negative nodes and determine type
!         of intersection

      nnegativet = 0
      do i = 1, n
         if (temperature(i)<Tm) then
            nnegativet = nnegativet + 1
            negativet(nnegativet) = i
         end if
      end do

      if (nnegativet==2) then
         inttype = 2
         if (debug) print *, 'inttype = ', inttype, ' = (2 prisms)'
      else
         inttype = 1
         if (debug) print *, 'inttype = ', inttype, ' = (tetra+rest)'
      end if

      if (inttype==2) then ! 2 prisms

         xhat(1) = negativet(1)
         do i = 2, n
            xhat(i) = mod(xhat(1)+i-2,mloc)+1
         end do

         do i = 2, n
            if (xhat(i)==negativet(2)) then
                xhat(i) = xhat(2)
                xhat(2) = negativet(2)
            end if
         end do

         if (debug) write(irefwr,100) 'xhat =', xhat(1),xhat(2), &
                                       xhat(3),xhat(4)

!        --- compute lengths of intersected edges

         do i = 1, 2
            do j = 1, 2
               h(j+(i-1)*2) = sqrt((x(xhat(i),1)- x(xhat(j+2),1))**2 + &
                     (x(xhat(i),2) - x(xhat(j+2),2))**2 + &
                     (x(xhat(i),3) - x(xhat(j+2),3))**2  )
               if (debug) write(irefwr,110) 'h', h(j+(i-1)*2)
            end do
         end do

!        --- compute estimate intersection positions
!            the most basic way is to use linear interpolation

         do i = 1, 2
            do j = 1, 2
               dh(j+(i-1)*2) = h(j+(i-1)*2)*(Tm- temperature(xhat(i)))/ &
                      (temperature(xhat(j+2)) - temperature(xhat(i)))
               if (debug) write(irefwr,110) 'dh', dh(j+(i-1)*2)
            end do
         end do

!        --- compute coordinates of intersection points

         do i = 1, 2
            do k = 1, 2
               do j = 1, ndim

!              --- if (x(xhat(i),j)==x(xhat(k+2),j)) then
!                     xs(k+(i-1)*2,j) = x(xhat(i),j)
!                  else

                  if (x(xhat(i),j)<x(xhat(k+2),j)) then
                     xs(k+(i-1)*2,j) = x(xhat(i),j) + &
                           abs(x(xhat(i),j)-x(xhat(k+2),j))* &
                                dh(k+(i-1)*2)/h(k+(i-1)*2)
                  else
                     xs(k+(i-1)*2,j) = x(xhat(i),j) - &
                           abs(x(xhat(i),j)-x(xhat(k+2),j))* &
                                dh(k+(i-1)*2)/h(k+(i-1)*2)
                  end if
               end do
            end do
         end do

         if (debug) then
            do i = 1, n
               write(irefwr,*) 'xs ', xs(i,1), xs(i,2), xs(i,3)
            end do
         end if

!        --- divide the liquid and the solid prism into 3 tetrahedra
!            to compute the weigths for the integrals over these 6 tetrahedra
!            the determinant for each tetrahedron is computed and stored in
!            deters. The first three entries correspond to the three solid
!            tetrahedra, the last three with the three liquid ones

         do i = 1, ndim

!        --- tetra 1:

            xtetra(1,i,1) = x(xhat(1),i)
            xtetra(2,i,1) = xs(2,i)
            xtetra(3,i,1) = xs(1,i)
            xtetra(4,i,1) = x(xhat(2),i)

!           --- tetra 2:

            xtetra(1,i,2) = xs(1,i)
            xtetra(2,i,2) = xs(2,i)
            xtetra(3,i,2) = xs(4,i)
            xtetra(4,i,2) = x(xhat(2),i)

!           --- tetra 3

            xtetra(1,i,3) = xs(1,i)
            xtetra(2,i,3) = xs(4,i)
            xtetra(3,i,3) = xs(3,i)
            xtetra(4,i,3) = x(xhat(2),i)

!           --- tetra 4:

            xtetra(1,i,4) = xs(1,i)
            xtetra(2,i,4) = xs(4,i)
            xtetra(3,i,4) = xs(3,i)
            xtetra(4,i,4) = x(xhat(4),i)

!           --- tetra 5:

            xtetra(1,i,5) = xs(1,i)
            xtetra(2,i,5) = xs(2,i)
            xtetra(3,i,5) = xs(4,i)
            xtetra(4,i,5) = x(xhat(4),i)

!           --- tetra 6:

            xtetra(1,i,6) = xs(1,i)
            xtetra(2,i,6) = x(xhat(4),i)
            xtetra(3,i,6) = xs(3,i)
            xtetra(4,i,6) = x(xhat(3),i)
         end do

         do i = 1, 6
            call elcompdet( deters(i), e, xtetra(1,1,i), 0 )
            if (debug) &
               write(irefwr,*) 'determinant of tetra ',i, deters(i)
         end do

!        --- compute the weights for the integrals over the 6 tetrahedra

         do i = 1, 6
            weights(i) = abs(deters(i))/24d0
            if (debug) write(irefwr,*) 'weight for tetra ',i, weights(i)
         end do

!        --- compute detcut such that |detcut| = 2*surface intersection
!            triangle
!            intersection consists of two triangle, for each of which we compute
!            the deteminant

         do i = 1, 3
            v(i,1) = abs(xs(1,i) - xs(2,i))
            v(i,2) = abs(xs(1,i) - xs(4,i))
         end do

         call eloutr(v(1,1), v(1,2), v(1,3))

         detcut(1) = sqrt(v(1,3)**2 + v(2,3)**2 + v(3,3)**2)

         do i = 1, 3
            v(i,1) = abs(xs(4,i) - xs(3,i))
            v(i,2) = abs(xs(4,i) - xs(1,i))
         end do

         call eloutr(v(1,1), v(1,2), v(1,3))

         detcut(2) = sqrt(v(1,3)**2 + v(2,3)**2 + v(3,3)**2)

         if (debug) write(irefwr,*) 'detcut = ', detcut(1), detcut(2)

!        --- compute wcut

         do i = 1, 2
            wcut(i) = abs(detcut(i))/6d0
            if (debug) write(irefwr,*) 'wcut ', wcut(i)
         end do

!        --- fill coefficient matrices
!            first three entries are for the integrals over the solid domain, &
!            last three entries for the integrals over the liquid domain

         c(1:3) = cs*rhos
         c(4:6) = cl*rhol
         kappa(1:3) = ks
         kappa(4:6) = kl

!        --- construct phixs

         phixs(xhat(1),1) = 1d0 - dh(1)/h(1)
         phixs(xhat(2),1) = 0d0
         phixs(xhat(3),1) = dh(1)/h(1)
         phixs(xhat(4),1) = 0d0
         phixs(xhat(1),2) = 1d0 - dh(2)/h(2)
         phixs(xhat(2),2) = 0d0
         phixs(xhat(3),2) = 0d0
         phixs(xhat(4),2) = dh(2)/h(2)
         phixs(xhat(1),3) = 0d0
         phixs(xhat(2),3) = 1d0 - dh(3)/h(3)
         phixs(xhat(3),3) = dh(3)/h(3)
         phixs(xhat(4),3) = 0d0
         phixs(xhat(1),4) = 0d0
         phixs(xhat(2),4) = 1d0 - dh(4)/h(4)
         phixs(xhat(3),4) = 0d0
         phixs(xhat(4),4) = dh(4)/h(4)

!        --- Fill phicut

         do i = 1, n
            phicut(xhat(i),xhat(1),1) = phi(xhat(i),xhat(1))
            phicut(xhat(i),xhat(2),1) = phi(xhat(i),xhat(2))
            phicut(xhat(i),xhat(3),1) = phi(xhat(i),xhat(3))
            phicut(xhat(i),xhat(4),1) = phi(xhat(i),xhat(4))
            phicut(xhat(i),5,1) = phixs(xhat(i),1)
            phicut(xhat(i),6,1) = phixs(xhat(i),2)
            phicut(xhat(i),7,1) = phixs(xhat(i),3)
            phicut(xhat(i),8,1) = phixs(xhat(i),4)
         end do

         do i = 1, n
            do j = 1, 8
              if (debug) write(irefwr,*) 'phicut(',xhat(i),',',j,') ', &
                                         phicut(xhat(i),j,1)
            end do
         end do

!        --- compute gradtemp

         do i = 1, ndim
            sum1 = 0d0
            do j = 1, n
               sum1 = sum1 + temperature(j)*dphidx(j,1,i)
            end do
            gradtemp(i) = sum1
         end do

         nrmgradt = sqrt(gradtemp(1)**2+gradtemp(2)**2+gradtemp(3)**2 )

         if (debug) write(irefwr,*)'gradtemp ',gradtemp(1),gradtemp(2), &
                                        gradtemp(3)
         if (debug) write(irefwr,*) 'nrmgradt ', nrmgradt

!        --- Fill index matrix for individual tetrahedra

         index1(1,1) = xhat(1)
         index1(2,1) = xhat(2)
         index1(3,1) = 5
         index1(4,1) = 6
         index1(1,2) = xhat(2)
         index1(2,2) = 5
         index1(3,2) = 6
         index1(4,2) = 8
         index1(1,3) = xhat(2)
         index1(2,3) = 5
         index1(3,3) = 7
         index1(4,3) = 8
         index1(1,4) = xhat(4)
         index1(2,4) = 5
         index1(3,4) = 7
         index1(4,4) = 8
         index1(1,5) = xhat(4)
         index1(2,5) = 5
         index1(3,5) = 6
         index1(4,5) = 8
         index1(1,6) = xhat(3)
         index1(2,6) = xhat(4)
         index1(3,6) = 5
         index1(4,6) = 7

!        --- If second order Newton Cotes is used, fill phicut for
!            the additional integration points

         if (intrule==2) then
            call eldiscint31(phicut, index1, n, 6)
         end if

!        --- fill element mass matrix

         do i = 1, n            ! basis functions
            do j = 1, n         ! basis functions
               sum1 = 0d0
               do l = 1, 6      ! tetrahedra
                 if (intrule==1) then
                  do k = 1, mloc   ! integration points
                     sum1 = sum1 + c(l)*weights(l)* &
                                phicut(i,index1(k,l),1)* &
                                phicut(j,index1(k,l),1)
                  end do
                 else if ( intrule==2) then
                  do k = 1, 10      ! integration points
                     if (k<=4) then
                        fac = -1d0/5d0
                     else
                        fac = 4d0/5d0
                     end if
                     sum1 = sum1 + c(l)*fac*weights(l)* &
                                phicut(i,k,l)* &
                                phicut(j,k,l)
                  end do
                 end if
               end do
               massmt(i,j) = sum1
            end do
         end do

         if (debug) then
            do i = 1, n
               do j = 1, n
                  write(irefwr,*) 'massmt(',i,',',j,' ) = ', massmt(i,j)
               end do
            end do
         end if

!        --- fill element stiffness matrix

         do i = 1, n            ! basis functions
            do j = 1, n         ! basis functions
               sum1 = 0d0
               do l = 1, 6      ! tetrahedra
                  do k = 1, mloc   ! integration points
                     sum1 = sum1 + &
                        kappa(l)*weights(l)*( &
                        dphidx(i,1,1)*dphidx(j,1,1)+ &
                        dphidx(i,1,2)*dphidx(j,1,2)+ &
                        dphidx(i,1,3)*dphidx(j,1,3))
                  end do
               end do
               stiffmt(i,j) = sum1
            end do
         end do

         if (debug) then
            do i = 1, n
               do j = 1, n
                  write(irefwr,*) 'stiffmt(',i,',',j,' ) = ', &
                           stiffmt(i,j)
               end do
            end do
         end if

!        --- fill element dldt matrix

         do i = 1, n
            do j = 1, n
               sum1 = 0d0
               if (intrule==1) then
                  do l = 1, 2 ! triangle
                     do k = 1, 3 ! integration point
                        sum1 = sum1 + latentheat*wcut(l)* &
                        phixs(i,index2(l,k))* &
                        phixs(j,index2(l,k))/nrmgradt
                     end do
                  end do
               else if (intrule==2) then
                  do l = 1, 2 ! triangle
                     do k = 1, 3 ! integration point
                        sum1 = sum1 + latentheat*wcut(l)* &
                          0.5d0*(phixs(i,index2(l,index4(k)))+ &
                                 phixs(i,index2(l,index4(k+1))))* &
                          0.5d0*(phixs(j,index2(l,index4(k)))+ &
                               phixs(j,index2(l,index4(k+1))))/nrmgradt
                     end do
                  end do
               end if
               dldtmt(i,j) = sum1
            end do
         end do

         if (debug) then
            do i = 1, n
               do j = 1, n
                  write(irefwr,*) 'dldtmt(',i,',',j,' ) = ', dldtmt(i,j)
               end do
            end do
         end if

!        --- fill latent heat element vector
!            latentvc = 0 in solid, so restrict to liquid
!            phicut(i,j,4-6) correspond to liquid tetrahedra

         do i = 1, n     ! basis functions
            sum1 = 0d0
            do l = 4, 6  ! tetrahedra
               do j = 1, mloc  ! integration points
                  sum1 = sum1 + latentheat*weights(l)* &
                         phicut(i,index1(j,l),1)
               end do
            end do
            latentvc(i) = sum1
         end do

         if (debug) then
            do i = 1, n
               write(irefwr,*) 'latentvc(',i,') = ', latentvc(i)
            end do
         end if

!        --- progress bar ------------------------------ &

      else ! tetrahedron + rest

!     --- For case 1 we determine the node that forms the 'top' of the
!         small tetrahedron and store it in xhat(1)

         if (nnegativet==1) then
            xhat(1) = negativet(1)
         else
            xhat(1) = 10 - negativet(1)-negativet(2)-negativet(3)
         end if

         do i = 2, n
            xhat(i) = mod(xhat(1)+i-2,mloc)+1
         end do

         if (debug) write(irefwr,*)'xhat = ',xhat(1),xhat(2), &
                                       xhat(3),xhat(4)

!        --- compute lengths of intersected edges

         do i = 1, 3
            h(i) = sqrt((x(xhat(1),1) - x(xhat(i+1),1))**2 + &
                     (x(xhat(1),2) - x(xhat(i+1),2))**2 + &
                     (x(xhat(1),3) - x(xhat(i+1),3))**2  )
            if (debug) write(irefwr,*) 'h ', h(i)
         end do

!        --- compute estimate intersection positions
!            the most basic way is to use linear interpolation

         do i = 1, 3
            dh(i) = h(i)*(Tm - temperature(xhat(1)))/ &
                      (temperature(xhat(i+1)) - temperature(xhat(1)))
            if (debug) write(irefwr,*) 'dh ', dh(i)
         end do

!        --- compute coordinates of intersection points

         do i = 1, 3
            do j = 1, 3

!           --- if (x(xhat(1),j)==x(xhat(i+1),j)) then
!                  xs(i,j) = x(xhat(1),j)
!               else

               if (x(xhat(1),j)<x(xhat(i+1),j)) then
                  xs(i,j) = x(xhat(1),j) + &
                        abs(x(xhat(1),j)-x(xhat(i+1),j))*dh(i)/h(i)
               else
                  xs(i,j) = x(xhat(1),j) - &
                        abs(x(xhat(1),j)-x(xhat(i+1),j))*dh(i)/h(i)
               end if
            end do
         end do

         if (debug) then
            do i = 1, 3
               write(irefwr,*) 'xs ', xs(i,1), xs(i,2), xs(i,3)
            end do
         end if

!        --- compute detcut such that |detcut| = 2*surface intersection
!            triangle

         do i = 1,3
            v(i,1) = abs(xs(1,i) - xs(2,i))
            v(i,2) = abs(xs(1,i) - xs(3,i))
         end do

         call eloutr(v(1,1), v(1,2), v(1,3))

         detcut(1) = sqrt(v(1,3)**2 + v(2,3)**2 + v(3,3)**2)
         if (debug) write(irefwr,*) 'detcut = ', detcut(1)

!        --- compute wcut

         wcut(1) = abs(detcut(1))/6d0
         if (debug) write(irefwr,*) 'wcut ', wcut(1)

!        --- compute det such that |det| = 6*volume whole element

         call elcompdet ( det, e, x, 0 )
         if (debug) write(irefwr,*) 'det = ', det

!        --- compute det1 such that |det1| = 6*volume of subtetra1

         do i = 1, 3
            xs(4,i) = x(xhat(1),i)
         end do

         call elcompdet ( det1, e, xs, 0 )
         if (debug) write(irefwr,*) 'det1 = ', det1

!        --- compute weights

         wtetra1 = abs(det1)/24d0
         wtetra = abs(det)/24d0

         if (debug) write(irefwr,*) 'wtetra, wtetra1 ', wtetra, wtetra1

!        --- set physical parameters for the subsections of the element

         if (temperature(xhat(1))<Tm) then

!        --- tetra1 is solid, tetra is liquid

            ctetra1 = cs*rhos
            ctetra  = cl*rhol
            ktetra1 = ks
            ktetra  = kl
         else ! otherwise
            ctetra1 = cl*rhol
            ctetra  = cs*rhos
            ktetra1 = kl
            ktetra  = ks
         end if

!        --- construct phixs and phicut

         phixs(xhat(1),1) = 1d0 - dh(1)/h(1)
         phixs(xhat(2),1) = dh(1)/h(1)
         phixs(xhat(3),1) = 0d0
         phixs(xhat(4),1) = 0d0
         phixs(xhat(1),2) = 1d0 - dh(2)/h(2)
         phixs(xhat(2),2) = 0d0
         phixs(xhat(3),2) = dh(2)/h(2)
         phixs(xhat(4),2) = 0d0
         phixs(xhat(1),3) = 1d0 - dh(3)/h(3)
         phixs(xhat(2),3) = 0d0
         phixs(xhat(3),3) = 0d0
         phixs(xhat(4),3) = dh(3)/h(3)

!        --- Copy phi for the vertices of the whole element

         do i = 1, n
            do j = 1, n
               phicut(i,j,1) = phi(i,j)
            end do
         end do

!        --- Add phi's in the intersection points

         do i = 1, n
            phicut(xhat(i),5,1) = phixs(xhat(i),1)
            phicut(xhat(i),6,1) = phixs(xhat(i),2)
            phicut(xhat(i),7,1) = phixs(xhat(i),3)
         end do

         if (debug) then
            do i = 1, n
               do j = 1, 7
                  write(irefwr,*) 'phicut(',xhat(i),',',j,') ', &
             phicut(xhat(i),j,1)
               end do
            end do
         end if

!        --- compute gradtemp

         do i = 1, ndim
            sum1 = 0d0
            do j = 1, n
               sum1 = sum1 + temperature(j)*dphidx(j,1,i)
            end do
            gradtemp(i) = sum1
         end do

         nrmgradt = sqrt(gradtemp(1)**2+gradtemp(2)**2+gradtemp(3)**2 )

         if (debug) write(irefwr,*)'gradtemp ',gradtemp(1),gradtemp(2), &
                                        gradtemp(3)
         if (debug) write(irefwr,*) 'nrmgradt ', nrmgradt

!        --- Fill index array

         index3(1,1) = xhat(1)
         index3(2,1) = xhat(2)
         index3(3,1) = xhat(3)
         index3(4,1) = xhat(4)
         index3(1,2) = xhat(1)
         index3(2,2) = 5
         index3(3,2) = 6
         index3(4,2) = 7

         if (intrule==2) then

!        --- Fill phicut for additional integration points

            call eldiscint31(phicut, index3, n, 2)

         end if

!        --- fill element mass matrix

         do i = 1, n            ! basis functions
            do j = 1, n         ! basis functions
               sum1 = 0d0
               sum2 = 0d0
               if (intrule==1) then
                  do k = 1, mloc      ! integration points
                     wphi1 = wtetra1*phicut(i,index3(k,2),1)* &
                                     phicut(j,index3(k,2),1)
                     sum1 = sum1 + &
                            ctetra*(wtetra*phicut(i,index3(k,1),1)* &
                                         phicut(j,index3(k,1),1)-wphi1)
                     sum2 = sum2 + ctetra1*wphi1
                  end do
               else if ( intrule==2) then
                  do k = 1, 10      ! integration points
                     if (k<=4) then
                        fac = -1d0/5d0
                     else
                        fac = 4d0/5d0
                     end if
                     wphi1 = fac*wtetra1*phicut(i,k,2)* &
                                         phicut(j,k,2)
                     sum1 = sum1 + &
                       ctetra*(fac*wtetra*phicut(i,k,1)* &
                                   phicut(j,k,1)-wphi1)
                     sum2 = sum2 + ctetra1*wphi1
                  end do
               end if
               massmt(i,j) = sum1 + sum2
            end do
         end do

         if (debug) then
            do i = 1, n
               do j = 1, n
                  write(irefwr,*) 'massmt(',i,',',j,' ) = ', massmt(i,j)
               end do
            end do
         end if

!        --- fill element stiffness matrix

         do i = 1, n            ! basis functions
            do j = 1, n         ! basis functions
               sum1 = 0d0
               do k = 1, mloc      ! integration points
                  wphi1 = dphidx(i,1,1)*dphidx(j,1,1)+ &
                          dphidx(i,1,2)*dphidx(j,1,2)+ &
                          dphidx(i,1,3)*dphidx(j,1,3)
                  sum1 = sum1 + wphi1*(ktetra1*wtetra1 + &
                                       ktetra*(wtetra-wtetra1))
               end do
               stiffmt(i,j) = sum1
            end do
         end do

         if (debug) then
            do i = 1, n
               do j = 1, n
                  write(irefwr,*) 'stiffmt(',i,',',j,' ) = ', &
             stiffmt(i,j)
               end do
            end do
         end if

!        --- fill element dldt matrix

         do i = 1, n
            do j = 1, n
               sum1 = 0d0
               do k = 1, 3
                  if (intrule==1) then
                     sum1 = sum1 + latentheat*wcut(1)* &
                                     phixs(i,k)*phixs(j,k)/nrmgradt
                  else if ( intrule==2) then
                     sum1 = sum1 + latentheat*wcut(1)* &
                            0.5d0*(phixs(i,index4(k))+ &
                                   phixs(i,index4(k+1)))* &
                            0.5d0*(phixs(j,index4(k))+ &
                                   phixs(j,index4(k+1)))/nrmgradt
                  end if
               end do
               dldtmt(i,j) = sum1
            end do
         end do

         if (debug) then
            do i = 1, n
               do j = 1, n
                  write(irefwr,*) 'dldtmt(',i,',',j,' ) = ', dldtmt(i,j)
               end do
            end do
         end if

!        --- fill latent heat element vector
!            latentvc = 0 in solid, so restrict to liquid

         if (temperature(xhat(1))<Tm) then

!        --- trapezium is liquid

            if (debug) write(irefwr,*) 'trapezium is liquid'
            do i = 1, n     ! basis functions
               sum1 = 0d0
               do j = 1, mloc  ! integration points
                  if (intrule==1) then
                     wphi1 = wtetra1*phicut(i,index3(j,2),1)
                     sum1 = sum1 + &
                        latentheat*(wtetra*phicut(i,index3(j,1),1) &
                                 -wphi1)
                  else if (intrule==2) then
                     wphi1 = wtetra1*phicut(i,j,2)
                     sum1 = sum1 + &
                        latentheat*(wtetra*phicut(i,j,1) &
                                 -wphi1)
                  end if
               end do
               latentvc(i) = sum1
            end do

         else

!        --- tetra1 is liquid

            if (debug) write(irefwr,*) 'tetra is liquid'

            do i = 1, n     ! basis functions
               sum1 = 0d0
               do j = 1, mloc  ! integration points
                  if (intrule==1) then
                     sum1 = sum1 + latentheat*wtetra1* &
                                     phicut(i,index3(j,2),1)
                  else if (intrule==2) then
                     sum1 = sum1 + latentheat*wtetra1* &
                                     phicut(i,j,2)
                  end if
               end do
               latentvc(i) = sum1
            end do

         end if

         if (debug) then
            do i = 1, n
               write(irefwr,*) 'latentvc(',i,') = ', latentvc(i)
            end do
         end if

      end if

1000  call erclos ( 'eldiscint3iso' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End eldiscint3iso'

      end if

      end
