      subroutine eldiscint3m( intrule, rho, c, kappa, latentheat, &
                             phi, dphidx, delta, temperature, massmt, &
                             stiffmt, dldtmt, latentvc, nsign, mloc)
! ======================================================================
!
!        programmer    John Brusche
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.1  date 20-06-2006 Added intrule = 3
!        version  1.0  date 30-05-2006
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Construct element mass matrix, stiffness matrix, derivative latent heat
!   matrix and latent heat vector using discontinous integration.
!   This is the mushy case. Mushy region.
!   The code differs from the isothermal case, since the dLdT matrix and
!   L-vector are computed differently.
!   This is the 3D version.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmodulemushy
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer          intrule, mloc
      double precision c, kappa, rho, latentheat, temperature(*), &
                       dphidx(n,ndim), massmt(n,n), delta, phi(n,mloc), &
                       stiffmt(n,n), dldtmt(n,n), latentvc(n), nsign

!     c              i    Specific heat
!     delta          i    Volume of element
!     dldtmt        i/o   Element dldt matrix
!     dphidx         i    derivatives of basis functions
!     intrule       i    Numerical integration rule to be used for
!                         discontinuous integration
!                         1: Newton-Cotes, 1st order, 'substraction approach'
!                         2: Newton-Cotes, 2nd order, 'substraction approach'
!                         3: Newton-Cotes, 1st order, &
!                            'subdivision into 3 subtriangles'
!                         4: Newton-Cotes, 1st order, &
!                            'averaging of heat capacity'
!     kappa          i    Diffusivity
!     latentheat     i    latent heat
!     latentvc      i/o   Element latent heat vector
!     massmt        i/o   Element mass matrix
!     mloc           local copy of m modified for Newton-Cotes
!     nsign          i    determines whether contribution is added or
!                         subtracted
!     phi            i    basis functions
!     rho            i    density
!     stiffmt       i/o   Element stiffness matrix
!     temperature    i    temperature vector in integration points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision sum, phicut(n,10), tempcut(10), w, fac

!     fac            extra multiplication factor for integration
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     phicut        phi matrix for intersected element
!     sum            Help parameter to compute a sum
!     tempcut        Temperature in integration points
!     w              Weight for Newton Cotes integration rule
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL2        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eldiscint3m' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eldiscint3m'
         call prinrl(temperature, n, 'temperature mushy')
         call prinrl2(phi, n, mloc, 'phi mushy')
         print *, 'delta',delta

      end if

      w = delta/6d0

      do i = 1, n

         phicut(i, 1) = phi(i,1)
         phicut(i, 2) = phi(i,2)
         phicut(i, 3) = phi(i,3)
         phicut(i, 4) = phi(i,4)

      end do

      tempcut(1) = temperature(1)
      tempcut(2) = temperature(2)
      tempcut(3) = temperature(3)
      tempcut(4) = temperature(4)

      if (intrule==2) then
         do i = 1, n
            phicut(i, 5) = 0.5d0*(phi(i,1) + phi(i,2))
            phicut(i, 6) = 0.5d0*(phi(i,1) + phi(i,3))
            phicut(i, 7) = 0.5d0*(phi(i,1) + phi(i,4))
            phicut(i, 8) = 0.5d0*(phi(i,2) + phi(i,3))
            phicut(i, 9) = 0.5d0*(phi(i,2) + phi(i,4))
            phicut(i,10) = 0.5d0*(phi(i,3) + phi(i,4))
         end do

         tempcut(5) = 0.5d0*(temperature(1) + temperature(2))
         tempcut(6) = 0.5d0*(temperature(1) + temperature(3))
         tempcut(7) = 0.5d0*(temperature(1) + temperature(4))
         tempcut(8) = 0.5d0*(temperature(2) + temperature(3))
         tempcut(9) = 0.5d0*(temperature(2) + temperature(4))
         tempcut(10) = 0.5d0*(temperature(3) + temperature(4))

         if (debug) call prinrl2(phicut, n, 10, 'phicut mush')

      else if (intrule==3) then
         do i = 1, n
            phicut(i,5) = 1/3d0*(phi(i,1) + phi(i,2) + phi(i,3))
            phicut(i,6) = 1/3d0*(phi(i,1) + phi(i,2) + phi(i,4))
            phicut(i,7) = 1/3d0*(phi(i,1) + phi(i,3) + phi(i,4))
            phicut(i,8) = 1/3d0*(phi(i,2) + phi(i,3) + phi(i,4))
         end do
         tempcut(5) = 1/3d0*(temperature(1) + temperature(2) + &
                             temperature(3))
         tempcut(6) = 1/3d0*(temperature(1) + temperature(2) + &
                             temperature(4))
         tempcut(7) = 1/3d0*(temperature(1) + temperature(3) + &
                             temperature(4))
         tempcut(8) = 1/3d0*(temperature(2) + temperature(3) + &
                             temperature(4))

         if (debug) call prinrl2(phicut, n, 8, 'phicut mush')

      end if

!     --- fill element mass matrix

      do i = 1, n            ! basis functions
         do j = 1, n         ! basis functions
            sum = 0d0
            if (intrule==1) then
               do k = 1, mloc      ! integration points
                  sum = sum + 0.25d0*phicut(i,k)*phicut(j,k)
               end do
            else if (intrule==2) then
               do k = 1, 10      ! integration points
                  if (k<=4) then
                     fac = -1d0/2d1
                  else
                     fac = 4d0/2d1
                  end if
                  sum = sum + fac*phicut(i,k)*phicut(j,k)
               end do
            else if (intrule==3) then
               do k = 1, 8      ! integration points
                  if (k<=4) then
                     fac = 1d0/4d1
                  else
                     fac = 9d0/4d1
                  end if
                  sum = sum + fac*phicut(i,k)*phicut(j,k)
               end do
            end if
            massmt(i,j) = massmt(i,j) + nsign*rho*c*w*sum
         end do
      end do

!     --- fill element stiffness matrix

      do i = 1, n            ! basis functions
         do j = 1, n         ! basis functions
            sum = 0d0
            do k = 1, mloc      ! integration points
               sum = sum + dphidx(i,1)*dphidx(j,1)+ &
                           dphidx(i,2)*dphidx(j,2)+ &
                           dphidx(i,3)*dphidx(j,3)
            end do
            stiffmt(i,j) = stiffmt(i,j) + 0.25d0*nsign*kappa*w*sum
         end do
      end do

!     --- fill element dldt matrix

      do i = 1, n
         do j = 1, n
            sum = 0d0
            if (intrule==1) then
               do k = 1, mloc      ! integration points
                  sum = sum + 0.25d0*phicut(i,k)*phicut(j,k)
               end do
            else if (intrule==2) then
               do k = 1, 10      ! integration points
                  if (k<=4) then
                     fac = -1d0/2d1
                  else
                     fac = 4d0/2d1
                  end if
                  sum = sum + fac*phicut(i,k)*phicut(j,k)
               end do
            else if (intrule==3) then
               do k = 1, 8      ! integration points
                  if (k<=4) then
                     fac = 1d0/4d1
                  else
                     fac = 9d0/4d1
                  end if
                  sum = sum + fac*phicut(i,k)*phicut(j,k)
               end do
            end if

            dldtmt(i,j) = dldtmt(i,j) + &
                               nsign*rho*latentheat*w*sum/(tliq - tsol)
         end do
      end do

!     --- fill latent heat element vector

      do i = 1, n     ! basis functions
         sum = 0d0
         if (intrule==1) then
            do k = 1, mloc      ! integration points
               sum = sum + 0.25d0*phi(i,k)*(tempcut(k) - tsol)
            end do
         else if (intrule==2) then
            do k = 1, 10      ! integration points
               if (k<=4) then
                  fac = -1d0/2d1
               else
                  fac = 4d0/2d1
               end if
               sum = sum + fac*phicut(i,k)*(tempcut(k) - tsol)
            end do
         else if (intrule==3) then
            do k = 1, 8      ! integration points
               if (k<=4) then
                  fac = 1d0/4d1
               else
                  fac = 9d0/4d1
               end if
               sum = sum + fac*phicut(i,k)*(tempcut(k) - tsol)
            end do
         end if
         latentvc(i) = latentvc(i) + &
                               nsign*rho*latentheat*w*sum/(tliq - tsol)
      end do

      call erclos ( 'eldiscint3m' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End eldiscint3m'
         call prinrl2(dldtmt, n, n, 'dldtmt m')
         call prinrl(latentvc, n, 'latentvc m')

      end if

      end
