      subroutine eldiscint3mush ( massmt, stiffmt, dldtmt, latentvc, &
                                  dphidx, temperature, coefarr, &
                                  intrule, phi, mloc, x )
! ======================================================================
!
!        programmer    John Brusche
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 30-05-2006
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Construct element mass matrix, stiffness matrix, derivative latent heat
!   matrix and latent heat vector using discontinous integration.
!   This is the mushy case.
!   The code differs from the isothermal case, since the dLdT matrix and
!   L-vector are computed differently.
!   This is the 3D version.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmodulemushy
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer          intrule, mloc
      double precision temperature(*), coefarr(m,*), x(n, ndim), &
                       dphidx(n,ndim), massmt(n,n), phi(n,mloc), &
                       stiffmt(n,n), dldtmt(n,n), latentvc(n)

!     coefarr        i    Array containing the coefficients of the PDE
!                         Pos. (.,1)   ksolid
!                         Pos. (.,2)   kfluid
!                         Pos. (.,3)   rho
!                         Pos. (.,4)   csolid
!                         Pos. (.,5)   cfluid
!                         Pos. (.,6)   Q
!                         Pos. (.,7)   L
!                         Pos. (.,8)   Tmelt
!     dldtmt         o    Element dldt matrix
!     dphidx         i    derivatives of basis functions
!     intrule        i    Numerical integration rule to be used for
!                         discontinuous integration
!                         1: Newton-Cotes, 1st order, 'substraction approach'
!                         2: Newton-Cotes, 2nd order, 'substraction approach'
!                         3: Newton-Cotes, 1st order, &
!                            'subdivision into 3 subtriangles'
!                         4: Newton-Cotes, 1st order, &
!                            'averaging of heat capacity'
!     latentvc       o    Element latent heat vector
!     massmt         o    Element mass matrix
!     stiffmt        o    Element stiffness matrix
!     temperature    i    temperature vector in integration points
!     x              i    x-coordinates of the integration points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, nsol, nliq, nmush, xsol(3), xliq(3), xmush(4), &
              nx(4), inttype
      double precision rho, cs, cl, latentheat, det, &
                       kl, ks, cm, kmushy, h(4), dh(4), &
                       phixs(n,4), xs(4,ndim), phicut(n,6), &
                       tempcut(6), e(4,3), xcut(6,ndim)

!     cl             Specific heat of the liquid
!     cm             Specific heat of the mushy
!     cs             Specific heat of the solid
!     det            determinant
!     dh             partial length of edge
!     e              Work array of length 4 x 3, to store the factors
!                     ij
!                    e.
!     h              width of the element
!     i              Counting variable
!     inttype        type of intersection
!     j              Counting variable
!     k              Counting variable
!     kl             Diffusivity of the liquid
!     kmushy             Diffusivity of the mushy
!     ks             Diffusivity of the solid
!     latentheat     latent heat
!     mloc           local copy of m modified for Newton-Cotes
!     nliq           number of liquid nodes
!     nmush          number of mushy nodes
!     nsol           number of solid nodes
!     nx             new/local numbering of the nodes
!     phi            basis functions
!     phicut         phi matrix for intersected element
!     phixs          phi values in intersection point xs
!     rho            density rho
!     tempcut        local variant of temperature
!     xcut           local variant of coordinates
!     xliq           node numbers of liquid nodes
!     xmush          node numbers of mushy nodes
!     xs             coordinates of intersection points
!     xsol           node numbers of solid nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELCOMPDET       compute the determinant for a triangle in 3D space
!     ELDISCINT3M     3D discontinuous integration, integrals over mushy
!     ELDISCINT3PRISM 3D discontinuous integration, integrals over prism
!                     subelement
!     ELDISCINT3SL    3D discontinuous integration, integrals over solid
!                     or liquid subelement
!     ERCLOS          Resets old name of previous subroutine of higher level
!     EROPEN          Produces concatenated name of local subroutine
!     INSTOP          Stop the program
!     PRINRL          Print 1d real vector
!     PRINRL2         Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eldiscint3mush' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eldiscint3mush'
         write(irefwr,*) 'tsol, tliq', tsol, tliq
         call prinrl(temperature, 4, 'temperature')
110      format ( a, 1x, (5d12.4) )
      end if

!     --- get parameters

      ks  = coefarr(1,1)
      kl  = coefarr(1,2)
      rho = coefarr(1,3)
      cs  = coefarr(1,4)
      cl  = coefarr(1,5)
      latentheat = coefarr(1,7)

      cm = 0.5d0*(cs+cl)
      kmushy = 0.5d0*(ks+kl)

      nsol = 0
      nliq = 0
      nmush = 0

!     --- determine type of intersection and fix node numbering

      do i = 1, mloc
         if (temperature(i)<tsol) then
            nsol = nsol + 1
            xsol(nsol) = i
         else if (temperature(i)>tliq) then
            nliq = nliq + 1
            xliq(nliq) = i
         else
            nmush = nmush + 1
            xmush(nmush) = i
         end if
      end do

      if (nmush==4) then
         inttype = 1 ! fully mushy
         do i = 1, mloc
            nx(i) = i
         end do
      else if (nmush==0) then
         if (nsol==2) then
            inttype = 2 ! 2 liquid nodes, 2 solid nodes
            nx(1) = xsol(1)
            nx(2) = xsol(2)
            nx(3) = xliq(1)
            nx(4) = xliq(2)
         else
            inttype = 3 ! 1 liquid/solid node, 3 solid/liquid nodes
            if (nsol==1) then
               nx(1) = xsol(1)
               nx(2) = xliq(1)
               nx(3) = xliq(2)
               nx(4) = xliq(3)
            else
               nx(1) = xliq(1)
               nx(2) = xsol(1)
               nx(3) = xsol(2)
               nx(4) = xsol(3)
            end if
         end if
      else if (nmush==2) then
         if (nsol==1 ) then
            inttype = 4 ! 1 solid, 2 mushy, 1 liquid node
            nx(1) = xsol(1)
            nx(2) = xmush(1)
            nx(3) = xmush(2)
            nx(4) = xliq(1)
         else
            inttype = 5 ! 2 mushy, 2 solid or liquid
            if (nsol==2) then
               nx(1) = xsol(1)
               nx(2) = xsol(2)
            else
               nx(1) = xliq(1)
               nx(2) = xliq(2)
            end if
            nx(3) = xmush(1)
            nx(4) = xmush(2)
         end if
      else if ( nmush==1 .and. (nsol==1 .or. nliq==1) ) then
         inttype = 6 ! 1 solid/liquid, 1 mushy, 2 liquid/solid
         if (nsol==1) then
            nx(1) = xsol(1)
            nx(3) = xliq(1)
            nx(4) = xliq(2)
         else
            nx(1) = xliq(1)
            nx(3) = xsol(1)
            nx(4) = xsol(2)
         end if
         nx(2) = xmush(1)
      else
         inttype = 7 ! 1 solid/liquid/mushy, 3 mushy/mushy/(solid/liquid)
         if (nmush==1) then
            nx(1) = xmush(1)
            if (nsol==3) then
               nx(2) = xsol(1)
               nx(3) = xsol(2)
               nx(4) = xsol(3)
            else
               nx(2) = xliq(1)
               nx(3) = xliq(2)
               nx(4) = xliq(3)
            end if
         else
            nx(2) = xmush(1)
            nx(3) = xmush(2)
            nx(4) = xmush(3)
            if (nsol==1) then
               nx(1) = xsol(1)
            else if (nliq==1) then
               nx(1) = xliq(1)
            end if
         end if
      end if

      if (debug) then
         print *, 'nx (new numbering)', nx(1), nx(2), nx(3), nx(4)
         print *, 'intersection type', inttype
         print *, 'temperature', temperature(nx(1)), temperature(nx(2)), &
                          temperature(nx(3)), temperature(nx(4))
         print *, 'nsol, nliq, nmush', nsol, nliq, nmush
      end if

!     --- fill phi and temperature using local numbering

      do i = 1, mloc
         do j = 1, mloc
            phicut(nx(i), j) = phi(nx(i), nx(j))
         end do
         tempcut(i) = temperature(nx(i))
      end do

!     --- compute matrices and vector on special element

      if ( inttype==1) then ! fully mushy

!     --- compute det such that |det| = 6*volume whole element

         call elcompdet ( det, e, x, 0 )
         det = abs(det)
         if (debug) write(irefwr,*) 'det = ', det

         call eldiscint3m( intrule, rho, cm, kmushy, latentheat, phicut, &
                          dphidx, det, tempcut, massmt, stiffmt, &
                          dldtmt, latentvc, 1d0, mloc)

      else if ( inttype==2) then ! 2 liquid nodes, 2 solid nodes

         if (debug) call prinrl2(x, n, ndim, 'x mush')

!        --- compute det such that |det| = 6*volume whole element

         call elcompdet ( det, e, x, 0 )
         det = abs(det)
         if (debug) write(irefwr,*) 'det = ', det

         call eldiscint3m( intrule, rho, cm, kmushy, latentheat, phicut, &
                          dphidx, det, tempcut, massmt, stiffmt, &
                          dldtmt, latentvc, 1d0, mloc)

         if (debug) call prinrl2(dldtmt, n,n,'dldtmt 1')

!        --- compute lengths of intersected edges

         do i = 1, 2
            do j = 1, 2
               h(j+(i-1)*2) = sqrt((x(nx(i),1)- x(nx(j+2),1))**2 + &
                     (x(nx(i),2) - x(nx(j+2),2))**2 + &
                     (x(nx(i),3) - x(nx(j+2),3))**2  )
               if (debug) write(irefwr,110) 'h', h(j+(i-1)*2)
            end do
         end do

!        --- compute intersection positions

         do i = 1, 2
            do j = 1, 2
               dh(j+(i-1)*2) = h(j+(i-1)*2)*(tsol - temperature(nx(i)))/ &
                      (temperature(nx(j+2)) - temperature(nx(i)))
               if (debug) write(irefwr,110) 'dh', dh(j+(i-1)*2)
            end do
         end do

!        --- compute coordinates of intersection points

         do i = 1, 2
            do k = 1, 2
               do j = 1, ndim
                  if (x(nx(i),j)<x(nx(k+2),j)) then
                     xs(k+(i-1)*2,j) = x(nx(i),j) + &
                           abs(x(nx(i),j)-x(nx(k+2),j))* &
                                dh(k+(i-1)*2)/h(k+(i-1)*2)
                  else
                     xs(k+(i-1)*2,j) = x(nx(i),j) - &
                           abs(x(nx(i),j)-x(nx(k+2),j))* &
                                dh(k+(i-1)*2)/h(k+(i-1)*2)
                  end if
               end do
            end do
         end do

!        --- construct phixs

         do j = 1, ndim
            do i = 1, 4
               xcut(i,j) = xs(i,j)
            end do
            xcut(5,j) = x(nx(1),j)
            xcut(6,j) = x(nx(2),j)
         end do

!        --- construct phixs

         phixs(nx(1), 1) = 1d0 - dh(1)/h(1)
         phixs(nx(2), 1) = 0d0
         phixs(nx(3), 1) = dh(1)/h(1)
         phixs(nx(4), 1) = 0d0
         phixs(nx(1), 2) = 1d0 - dh(2)/h(2)
         phixs(nx(2), 2) = 0d0
         phixs(nx(3), 2) = 0d0
         phixs(nx(4), 2) = dh(2)/h(2)
         phixs(nx(1), 3) = 0d0
         phixs(nx(2), 3) = 1d0 - dh(3)/h(3)
         phixs(nx(3), 3) = dh(3)/h(3)
         phixs(nx(4), 3) = 0d0
         phixs(nx(1), 4) = 0d0
         phixs(nx(2), 4) = 1d0 - dh(4)/h(4)
         phixs(nx(3), 4) = 0d0
         phixs(nx(4), 4) = dh(4)/h(4)

!        --- Fill phicut

         do i = 1, n
            phicut(nx(i),1) = phixs(nx(i),1)
            phicut(nx(i),2) = phixs(nx(i),2)
            phicut(nx(i),3) = phixs(nx(i),3)
            phicut(nx(i),4) = phixs(nx(i),4)
            phicut(nx(i),5) = phi(nx(i),nx(1))
            phicut(nx(i),6) = phi(nx(i),nx(2))
         end do

!        --- fill tempcut

         do i = 1, 4
            tempcut(i) = tsol
         end do
         tempcut(5) = temperature(nx(1))
         tempcut(6) = temperature(nx(2))

!        --- evaluate integrals over solid prism

         call eldiscint3prism( intrule, coefarr, 0, nx, 1, &
                              phicut, dphidx, xcut, tempcut, massmt, &
                              stiffmt, dldtmt, latentvc, -1d0, mloc)

         call eldiscint3prism( intrule, coefarr, 1, nx, 1, &
                              phicut, dphidx, xcut, tempcut, massmt, &
                              stiffmt, dldtmt, latentvc, 1d0, mloc)

         if (debug) call prinrl2(dldtmt, n,n,'dldtmt 2')

!        --- compute intersection positions

         do i = 1, 2
            do j = 1, 2
               dh(j+(i-1)*2) = h(j+(i-1)*2)*(tliq - temperature(nx(i)))/ &
                      (temperature(nx(j+2)) - temperature(nx(i)))
               if (debug) write(irefwr,110) 'dh', dh(j+(i-1)*2)
            end do
         end do

!        --- compute coordinates of intersection points

         do i = 1, 2
            do k = 1, 2
               do j = 1, ndim
                  if (x(nx(i),j)<x(nx(k+2),j)) then
                     xs(k+(i-1)*2,j) = x(nx(i),j) + &
                           abs(x(nx(i),j)-x(nx(k+2),j))* &
                                dh(k+(i-1)*2)/h(k+(i-1)*2)
                  else
                     xs(k+(i-1)*2,j) = x(nx(i),j) - &
                           abs(x(nx(i),j)-x(nx(k+2),j))* &
                                dh(k+(i-1)*2)/h(k+(i-1)*2)
                  end if
               end do
            end do
         end do

!        --- construct phixs

         do j = 1, ndim
            do i = 1, 4
               xcut(i,j) = xs(i,j)
            end do
            xcut(5,j) = x(nx(3),j)
            xcut(6,j) = x(nx(4),j)
         end do

!        --- construct phixs

         phixs(nx(1), 1) = 1d0 - dh(1)/h(1)
         phixs(nx(2), 1) = 0d0
         phixs(nx(3), 1) = dh(1)/h(1)
         phixs(nx(4), 1) = 0d0
         phixs(nx(1), 2) = 1d0 - dh(2)/h(2)
         phixs(nx(2), 2) = 0d0
         phixs(nx(3), 2) = 0d0
         phixs(nx(4), 2) = dh(2)/h(2)
         phixs(nx(1), 3) = 0d0
         phixs(nx(2), 3) = 1d0 - dh(3)/h(3)
         phixs(nx(3), 3) = dh(3)/h(3)
         phixs(nx(4), 3) = 0d0
         phixs(nx(1), 4) = 0d0
         phixs(nx(2), 4) = 1d0 - dh(4)/h(4)
         phixs(nx(3), 4) = 0d0
         phixs(nx(4), 4) = dh(4)/h(4)

!        --- Fill phicut

         do i = 1, n
            phicut(nx(i),1) = phixs(nx(i),1)
            phicut(nx(i),2) = phixs(nx(i),2)
            phicut(nx(i),3) = phixs(nx(i),3)
            phicut(nx(i),4) = phixs(nx(i),4)
            phicut(nx(i),5) = phi(nx(i),nx(3))
            phicut(nx(i),6) = phi(nx(i),nx(4))
         end do

         if (debug) call prinrl2(phicut, n, 6, 'phicut')

!        --- fill tempcut

         do i = 1, 4
            tempcut(i) = tliq
         end do
         tempcut(5) = temperature(nx(3))
         tempcut(6) = temperature(nx(4))

!        --- evaluate integrals over solid prism

         call eldiscint3prism( intrule, coefarr, 0, nx, 2, &
                              phicut, dphidx, xcut, tempcut, massmt, &
                              stiffmt, dldtmt, latentvc, -1d0, mloc)

         call eldiscint3prism( intrule, coefarr, 2, nx, 2, &
                              phicut, dphidx, xcut, tempcut, massmt, &
                              stiffmt, dldtmt, latentvc, 1d0, mloc)

         if (debug) call prinrl2(dldtmt, n,n,'dldtmt 3')

!        --- call instop

      else if ( inttype==3 ) then ! 1 liquid/solid node, 3 solid/liquid nodes

!     --- compute det such that |det| = 6*volume whole element

         call elcompdet ( det, e, x, 0 )
         det = abs(det)
         if (debug) write(irefwr,*) 'det = ', det

         if (nsol==1) then
            call eldiscint3sl( intrule, rho, cl, kl, latentheat, &
                              phicut, dphidx, det, tempcut, massmt, &
                              stiffmt, latentvc, 1d0, mloc, .true.)
         else
            call eldiscint3sl( intrule, rho, cs, ks, latentheat, &
                              phicut, dphidx, det, tempcut, massmt, &
                              stiffmt, latentvc, 1d0, mloc, .false.)

         end if ! (nsol==1)

!        --- compute lengths of intersected edges

         do i = 1, 3
            h(i) = sqrt((x(nx(1),1) - x(nx(i+1),1))**2 + &
                     (x(nx(1),2) - x(nx(i+1),2))**2 + &
                     (x(nx(1),3) - x(nx(i+1),3))**2  )
            if (debug) write(irefwr,*) 'h ', h(i)
            if (h(i)<0d0) then
               print *, 'h', h(i)
               call instop
            end if
         end do

!        --- compute intersection positions

         do i = 1, 3
            if (nsol==1) then
               dh(i) = h(i)*(tsol - temperature(nx(1)))/ &
                      (temperature(nx(i+1)) - temperature(nx(1)))
            else
               dh(i) = h(i)*(tliq - temperature(nx(1)))/ &
                      (temperature(nx(i+1)) - temperature(nx(1)))
            end if
            if (debug) write(irefwr,*) 'dh ', dh(i)
            if (dh(i)<0d0) then
               print *, 'dh', dh(i)
               call instop
            end if
         end do

!        --- compute coordinates of intersection points

         do i = 1, 3
            do j = 1, ndim
               if (x(nx(1),j)<x(nx(i+1),j)) then
                  xs(i,j) = x(nx(1),j) + &
                        abs(x(nx(1),j)-x(nx(i+1),j))*dh(i)/h(i)
               else
                  xs(i,j) = x(nx(1),j) - &
                        abs(x(nx(1),j)-x(nx(i+1),j))*dh(i)/h(i)
               end if
            end do
         end do

         if (debug) then
            do i = 1, ndim
               write(irefwr,*) 'xs ', xs(i,1), xs(i,2), xs(i,3)
            end do
         end if

!        --- compute det such that |det| = 6*volume of subtetra

         do i = 1, ndim
            xs(4,i) = x(nx(1),i)
         end do

         call elcompdet ( det, e, xs, 0 )
         det = abs(det)
         if (debug) write(irefwr,*) 'det = ', det

!        --- construct phixs

         phixs(nx(1),1) = 1d0 - dh(1)/h(1)
         phixs(nx(2),1) = dh(1)/h(1)
         phixs(nx(3),1) = 0d0
         phixs(nx(4),1) = 0d0
         phixs(nx(1),2) = 1d0 - dh(2)/h(2)
         phixs(nx(2),2) = 0d0
         phixs(nx(3),2) = dh(2)/h(2)
         phixs(nx(4),2) = 0d0
         phixs(nx(1),3) = 1d0 - dh(3)/h(3)
         phixs(nx(2),3) = 0d0
         phixs(nx(3),3) = 0d0
         phixs(nx(4),3) = dh(3)/h(3)

         do i = 1, n
            phicut(nx(i),1) = phixs(nx(i),1)
            phicut(nx(i),2) = phixs(nx(i),2)
            phicut(nx(i),3) = phixs(nx(i),3)
            phicut(nx(i),4) = phi(nx(i),nx(1))
         end do

         if (nsol==1) then

            tempcut(1) = tsol
            tempcut(2) = tsol
            tempcut(3) = tsol
            tempcut(4) = temperature(nx(1))

            call eldiscint3sl( intrule, rho, cs, ks, latentheat, &
                              phicut, dphidx, det, tempcut, massmt, &
                              stiffmt, latentvc, 1d0, mloc, .false.)

         else
            tempcut(1) = tliq
            tempcut(2) = tliq
            tempcut(3) = tliq
            tempcut(4) = temperature(nx(1))

            call eldiscint3sl( intrule, rho, cl, kl, latentheat, &
                              phicut, dphidx, det, tempcut, massmt, &
                              stiffmt, latentvc, 1d0, mloc, .true.)

         end if ! (nsol==1)

         call eldiscint3m( intrule, rho, cm, kmushy, latentheat, &
                          phicut, dphidx, det, tempcut, massmt, &
                          stiffmt, dldtmt, latentvc, -1d0, mloc)

!        --- compute intersection positions

         do i = 1, 3
            if ( nsol==1) then
               dh(i) = h(i)*(tliq - temperature(nx(1)))/ &
                      (temperature(nx(i+1)) - temperature(nx(1)))
            else
               dh(i) = h(i)*(tsol - temperature(nx(1)))/ &
                      (temperature(nx(i+1)) - temperature(nx(1)))
            end if
            if (debug) write(irefwr,*) 'dh ', dh(i)
            if (dh(i)<0d0) then
               print *, 'dh', dh(i)
               call instop
            end if
         end do

!        --- compute coordinates of intersection points

         do i = 1, 3
            do j = 1, ndim

               if (x(nx(1),j)<x(nx(i+1),j)) then
                  xs(i,j) = x(nx(1),j) + &
                        abs(x(nx(1),j)-x(nx(i+1),j))*dh(i)/h(i)
               else
                  xs(i,j) = x(nx(1),j) - &
                        abs(x(nx(1),j)-x(nx(i+1),j))*dh(i)/h(i)
               end if
            end do
         end do

         if (debug) then
            do i = 1, ndim
               write(irefwr,*) 'xs ', xs(i,1), xs(i,2), xs(i,3)
            end do
         end if

!        --- compute det such that |det| = 6*volume of subtetra

         do i = 1, ndim
            xs(4,i) = x(nx(1),i)
         end do

         call elcompdet ( det, e, xs, 0 )
         det = abs(det)
         if (debug) write(irefwr,*) 'det = ', det

!        --- construct phixs

         phixs(nx(1),1) = 1d0 - dh(1)/h(1)
         phixs(nx(2),1) = dh(1)/h(1)
         phixs(nx(3),1) = 0d0
         phixs(nx(4),1) = 0d0
         phixs(nx(1),2) = 1d0 - dh(2)/h(2)
         phixs(nx(2),2) = 0d0
         phixs(nx(3),2) = dh(2)/h(2)
         phixs(nx(4),2) = 0d0
         phixs(nx(1),3) = 1d0 - dh(3)/h(3)
         phixs(nx(2),3) = 0d0
         phixs(nx(3),3) = 0d0
         phixs(nx(4),3) = dh(3)/h(3)

         do i = 1, n
            phicut(nx(i),1) = phixs(nx(i),1)
            phicut(nx(i),2) = phixs(nx(i),2)
            phicut(nx(i),3) = phixs(nx(i),3)
            phicut(nx(i),4) = phi(nx(i),nx(1))
         end do

         if (nsol==1) then

            tempcut(1) = tliq
            tempcut(2) = tliq
            tempcut(3) = tliq
            tempcut(4) = temperature(nx(1))

            call eldiscint3sl( intrule, rho, cl, kl, latentheat, &
                              phicut, dphidx, det, tempcut, massmt, &
                              stiffmt, latentvc, -1d0, mloc, .true.)
         else
            tempcut(1) = tsol
            tempcut(2) = tsol
            tempcut(3) = tsol
            tempcut(4) = temperature(nx(1))

            call eldiscint3sl( intrule, rho, cs, ks, latentheat, &
                              phicut, dphidx, det, tempcut, massmt, &
                              stiffmt, latentvc, -1d0, mloc, .false.)

         end if ! (nsol==1)

         call eldiscint3m( intrule, rho, cm, kmushy, latentheat, phicut, &
                          dphidx, det, tempcut, massmt, stiffmt, &
                          dldtmt, latentvc, 1d0, mloc)

      else if ( inttype==4 ) then ! 1 liquid, 1 solid node, 2 mushy

!     --- compute det such that |det| = 6*volume whole element

         call elcompdet ( det, e, x, 0 )
         det = abs(det)
         if (debug) write(irefwr,*) 'det = ', det

         call eldiscint3m( intrule, rho, cm, kmushy, latentheat, &
                          phicut, dphidx, det, tempcut, massmt, &
                          stiffmt, dldtmt, latentvc, 1d0, mloc)

!        --- compute lengths of intersected edges

         do i = 1, 3
            h(i) = sqrt((x(nx(1),1) - x(nx(i+1),1))**2 + &
                     (x(nx(1),2) - x(nx(i+1),2))**2 + &
                     (x(nx(1),3) - x(nx(i+1),3))**2  )
            if (debug) write(irefwr,*) 'h ', h(i)
         end do

!        --- compute intersection positions

         do i = 1, 3
            dh(i) = h(i)*(tsol - temperature(nx(1)))/ &
                      (temperature(nx(i+1)) - temperature(nx(1)))
            if (debug) write(irefwr,*) 'dh ', dh(i)
         end do

!        --- compute coordinates of intersection points

         do i = 1, 3
            do j = 1, ndim
               if (x(nx(1),j)<x(nx(i+1),j)) then
                  xs(i,j) = x(nx(1),j) + &
                        abs(x(nx(1),j)-x(nx(i+1),j))*dh(i)/h(i)
               else
                  xs(i,j) = x(nx(1),j) - &
                        abs(x(nx(1),j)-x(nx(i+1),j))*dh(i)/h(i)
               end if
            end do
         end do

         if (debug) then
            do i = 1, ndim
               write(irefwr,*) 'xs ', xs(i,1), xs(i,2), xs(i,3)
            end do
         end if

!        --- compute det such that |det| = 6*volume of subtetra

         do i = 1, ndim
            xs(4,i) = x(nx(1),i)
         end do

         call elcompdet ( det, e, xs, 0 )
         det = abs(det)
         if (debug) write(irefwr,*) 'det = ', det

!        --- construct phixs

         phixs(nx(1),1) = 1d0 - dh(1)/h(1)
         phixs(nx(2),1) = dh(1)/h(1)
         phixs(nx(3),1) = 0d0
         phixs(nx(4),1) = 0d0
         phixs(nx(1),2) = 1d0 - dh(2)/h(2)
         phixs(nx(2),2) = 0d0
         phixs(nx(3),2) = dh(2)/h(2)
         phixs(nx(4),2) = 0d0
         phixs(nx(1),3) = 1d0 - dh(3)/h(3)
         phixs(nx(2),3) = 0d0
         phixs(nx(3),3) = 0d0
         phixs(nx(4),3) = dh(3)/h(3)

         do i = 1, n
            phicut(nx(i),1) = phixs(nx(i),1)
            phicut(nx(i),2) = phixs(nx(i),2)
            phicut(nx(i),3) = phixs(nx(i),3)
            phicut(nx(i),4) = phi(nx(i),nx(1))
         end do

         tempcut(1) = tsol
         tempcut(2) = tsol
         tempcut(3) = tsol
         tempcut(4) = temperature(nx(1))

         call eldiscint3sl( intrule, rho, cs, ks, latentheat, &
                           phicut, dphidx, det, tempcut, massmt, &
                           stiffmt, latentvc, 1d0, mloc, .false.)

         call eldiscint3m( intrule, rho, cm, kmushy, latentheat, &
                          phicut, dphidx, det, tempcut, massmt, &
                          stiffmt, dldtmt, latentvc, -1d0, mloc)

!        --- compute lengths of intersected edges

         do i = 1, 3
            h(i) = sqrt((x(nx(4),1) - x(nx(i),1))**2 + &
                     (x(nx(4),2) - x(nx(i),2))**2 + &
                     (x(nx(4),3) - x(nx(i),3))**2  )
            if (debug) write(irefwr,*) 'h ', h(i)
         end do

!        --- compute intersection positions

         do i = 1, 3
            dh(i) = h(i)*(tliq - temperature(nx(4)))/ &
                      (temperature(nx(i)) - temperature(nx(4)))
            if (debug) write(irefwr,*) 'dh ', dh(i)
         end do

!        --- compute coordinates of intersection points

         do i = 1, 3
            do j = 1, ndim
               if (x(nx(4),j)<x(nx(i),j)) then
                  xs(i,j) = x(nx(4),j) + &
                        abs(x(nx(4),j)-x(nx(i),j))*dh(i)/h(i)
               else
                  xs(i,j) = x(nx(4),j) - &
                        abs(x(nx(4),j)-x(nx(i),j))*dh(i)/h(i)
               end if
            end do
         end do

         if (debug) then
            do i = 1, ndim
               write(irefwr,*) 'xs ', xs(i,1), xs(i,2), xs(i,3)
            end do
         end if

!        --- compute det such that |det| = 6*volume of subtetra

         do i = 1, ndim
            xs(4,i) = x(nx(4),i)
         end do

         call elcompdet ( det, e, xs, 0 )
         det = abs(det)
         if (debug) write(irefwr,*) 'det = ', det

!        --- construct phixs

         phixs(nx(1),1) = dh(1)/h(1)
         phixs(nx(2),1) = 0d0
         phixs(nx(3),1) = 0d0
         phixs(nx(4),1) = 1d0 - dh(1)/h(1)
         phixs(nx(1),2) = 0d0
         phixs(nx(2),2) = dh(2)/h(2)
         phixs(nx(3),2) = 0d0
         phixs(nx(4),2) = 1d0 - dh(2)/h(2)
         phixs(nx(1),3) = 0d0
         phixs(nx(2),3) = 0d0
         phixs(nx(3),3) = dh(3)/h(3)
         phixs(nx(4),3) = 1d0 - dh(3)/h(3)

         do i = 1, n
            phicut(nx(i),1) = phixs(nx(i),1)
            phicut(nx(i),2) = phixs(nx(i),2)
            phicut(nx(i),3) = phixs(nx(i),3)
            phicut(nx(i),4) = phi(nx(i),nx(4))
         end do

         tempcut(1) = tliq
         tempcut(2) = tliq
         tempcut(3) = tliq
         tempcut(4) = temperature(nx(4))

         call eldiscint3sl( intrule, rho, cl, kl, latentheat, &
                           phicut, dphidx, det, tempcut, massmt, &
                           stiffmt, latentvc, 1d0, mloc, .true.)

         call eldiscint3m( intrule, rho, cm, kmushy, latentheat, &
                          phicut, dphidx, det, tempcut, massmt, &
                          stiffmt, dldtmt, latentvc, -1d0, mloc)

      else if ( inttype==5) then ! 2 liquid/solid nodes, 2 mushy nodes

!     --- compute det such that |det| = 6*volume whole element

         call elcompdet ( det, e, x, 0 )
         det = abs(det)
         if (debug) write(irefwr,*) 'det = ', det

         call eldiscint3m( intrule, rho, cm, kmushy, latentheat, phicut, &
                          dphidx, det, tempcut, massmt, stiffmt, &
                          dldtmt, latentvc, 1d0, mloc)

!        --- compute lengths of intersected edges

         do i = 1, 2
            do j = 1, 2
               h(j+(i-1)*2) = sqrt((x(nx(i),1)- x(nx(j+2),1))**2 + &
                     (x(nx(i),2) - x(nx(j+2),2))**2 + &
                     (x(nx(i),3) - x(nx(j+2),3))**2  )
               if (debug) write(irefwr,110) 'h', h(j+(i-1)*2)
            end do
         end do

!        --- compute intersection positions

         do i = 1, 2
            do j = 1, 2
               if ( nsol==2 ) then
                  dh(j+(i-1)*2) = h(j+(i-1)*2)* &
                      (tsol - temperature(nx(i)))/ &
                      (temperature(nx(j+2)) - temperature(nx(i)))
               else
                  dh(j+(i-1)*2) = h(j+(i-1)*2)* &
                      (tliq - temperature(nx(i)))/ &
                      (temperature(nx(j+2)) - temperature(nx(i)))
               end if
               if (debug) write(irefwr,110) 'dh', dh(j+(i-1)*2)
            end do
         end do

!        --- compute coordinates of intersection points

         do i = 1, 2
            do k = 1, 2
               do j = 1, ndim
                  if (x(nx(i),j)<x(nx(k+2),j)) then
                     xs(k+(i-1)*2,j) = x(nx(i),j) + &
                           abs(x(nx(i),j)-x(nx(k+2),j))* &
                                dh(k+(i-1)*2)/h(k+(i-1)*2)
                  else
                     xs(k+(i-1)*2,j) = x(nx(i),j) - &
                           abs(x(nx(i),j)-x(nx(k+2),j))* &
                                dh(k+(i-1)*2)/h(k+(i-1)*2)
                  end if
               end do
            end do
         end do

!        --- construct phixs

         do j = 1, ndim
            do i = 1, 4
               xcut(i,j) = xs(i,j)
            end do
            xcut(5,j) = x(nx(1),j)
            xcut(6,j) = x(nx(2),j)
         end do

!        --- construct phixs

         phixs(nx(1), 1) = 1d0 - dh(1)/h(1)
         phixs(nx(2), 1) = 0d0
         phixs(nx(3), 1) = dh(1)/h(1)
         phixs(nx(4), 1) = 0d0
         phixs(nx(1), 2) = 1d0 - dh(2)/h(2)
         phixs(nx(2), 2) = 0d0
         phixs(nx(3), 2) = 0d0
         phixs(nx(4), 2) = dh(2)/h(2)
         phixs(nx(1), 3) = 0d0
         phixs(nx(2), 3) = 1d0 - dh(3)/h(3)
         phixs(nx(3), 3) = dh(3)/h(3)
         phixs(nx(4), 3) = 0d0
         phixs(nx(1), 4) = 0d0
         phixs(nx(2), 4) = 1d0 - dh(4)/h(4)
         phixs(nx(3), 4) = 0d0
         phixs(nx(4), 4) = dh(4)/h(4)

!        --- Fill phicut

         do i = 1, n
            phicut(nx(i),1) = phixs(nx(i),1)
            phicut(nx(i),2) = phixs(nx(i),2)
            phicut(nx(i),3) = phixs(nx(i),3)
            phicut(nx(i),4) = phixs(nx(i),4)
            phicut(nx(i),5) = phi(nx(i),nx(1))
            phicut(nx(i),6) = phi(nx(i),nx(2))
         end do

!        --- fill tempcut

         do i = 1, 4
            if (nsol==2) then
               tempcut(i) = tsol
            else
               tempcut(i) = tliq
            end if
         end do
         tempcut(5) = temperature(nx(1))
         tempcut(6) = temperature(nx(2))

!        --- evaluate integrals over non-mushy prism

         call eldiscint3prism( intrule, coefarr, 0, nx, 1, &
                              phicut, dphidx, xcut, tempcut, massmt, &
                              stiffmt, dldtmt, latentvc, -1d0, mloc)
         if (nsol==2) then
            call eldiscint3prism( intrule, coefarr, 1, nx, 1, &
                                 phicut, dphidx, xcut, tempcut, massmt, &
                                 stiffmt, dldtmt, latentvc, 1d0, mloc)
         else
            call eldiscint3prism( intrule, coefarr, 2, nx, 1, &
                                 phicut, dphidx, xcut, tempcut, massmt, &
                                 stiffmt, dldtmt, latentvc, 1d0, mloc)
         end if

      else if ( inttype==6 ) then ! 1/2 liquid/solid node, 1 mushy

!     --- compute det such that |det| = 6*volume whole element

         call elcompdet ( det, e, x, 0 )
         det = abs(det)
         if (debug) write(irefwr,*) 'det = ', det

         call eldiscint3m( intrule, rho, cm, kmushy, latentheat, &
                          phicut, dphidx, det, tempcut, massmt, &
                          stiffmt, dldtmt, latentvc, 1d0, mloc)

!        --- compute lengths of intersected edges

         do i = 1, 3
            h(i) = sqrt((x(nx(1),1) - x(nx(i+1),1))**2 + &
                     (x(nx(1),2) - x(nx(i+1),2))**2 + &
                     (x(nx(1),3) - x(nx(i+1),3))**2  )
            if (debug) write(irefwr,*) 'h ', h(i)
         end do

!        --- compute intersection positions

         do i = 1, 3
            if (nsol==1) then
               dh(i) = h(i)*(tsol - temperature(nx(1)))/ &
                      (temperature(nx(i+1)) - temperature(nx(1)))
            else
               dh(i) = h(i)*(tliq - temperature(nx(1)))/ &
                      (temperature(nx(i+1)) - temperature(nx(1)))
            end if
            if (debug) write(irefwr,*) 'dh ', dh(i)
         end do

!        --- compute coordinates of intersection points

         do i = 1, 3
            do j = 1, ndim
               if (x(nx(1),j)<x(nx(i+1),j)) then
                  xs(i,j) = x(nx(1),j) + &
                        abs(x(nx(1),j)-x(nx(i+1),j))*dh(i)/h(i)
               else
                  xs(i,j) = x(nx(1),j) - &
                        abs(x(nx(1),j)-x(nx(i+1),j))*dh(i)/h(i)
               end if
            end do
         end do

         if (debug) then
            do i = 1, ndim
               write(irefwr,*) 'xs ', xs(i,1), xs(i,2), xs(i,3)
            end do
         end if

!        --- compute det such that |det| = 6*volume of subtetra

         do i = 1, ndim
            xs(4,i) = x(nx(1),i)
         end do

         call elcompdet ( det, e, xs, 0 )
         det = abs(det)
         if (debug) write(irefwr,*) 'det = ', det

!        --- construct phixs

         phixs(nx(1),1) = 1d0 - dh(1)/h(1)
         phixs(nx(2),1) = dh(1)/h(1)
         phixs(nx(3),1) = 0d0
         phixs(nx(4),1) = 0d0
         phixs(nx(1),2) = 1d0 - dh(2)/h(2)
         phixs(nx(2),2) = 0d0
         phixs(nx(3),2) = dh(2)/h(2)
         phixs(nx(4),2) = 0d0
         phixs(nx(1),3) = 1d0 - dh(3)/h(3)
         phixs(nx(2),3) = 0d0
         phixs(nx(3),3) = 0d0
         phixs(nx(4),3) = dh(3)/h(3)

         do i = 1, n
            phicut(nx(i),1) = phixs(nx(i),1)
            phicut(nx(i),2) = phixs(nx(i),2)
            phicut(nx(i),3) = phixs(nx(i),3)
            phicut(nx(i),4) = phi(nx(i),nx(1))
         end do

         if (nsol==1) then

            tempcut(1) = tsol
            tempcut(2) = tsol
            tempcut(3) = tsol
            tempcut(4) = temperature(nx(1))

            call eldiscint3sl( intrule, rho, cs, ks, latentheat, &
                              phicut, dphidx, det, tempcut, massmt, &
                              stiffmt, latentvc, 1d0, mloc, .false.)

         else
            tempcut(1) = tliq
            tempcut(2) = tliq
            tempcut(3) = tliq
            tempcut(4) = temperature(nx(1))

            call eldiscint3sl( intrule, rho, cl, kl, latentheat, &
                              phicut, dphidx, det, tempcut, massmt, &
                              stiffmt, latentvc, 1d0, mloc, .true.)

         end if ! (nsol==1)

         call eldiscint3m( intrule, rho, cm, kmushy, latentheat, &
                          phicut, dphidx, det, tempcut, massmt, &
                          stiffmt, dldtmt, latentvc, -1d0, mloc)

!        --- compute lengths of intersected edges

         do i = 1, 2
            do j = 1, 2
               h(j+(i-1)*2) = sqrt((x(nx(i),1)- x(nx(j+2),1))**2 + &
                     (x(nx(i),2) - x(nx(j+2),2))**2 + &
                     (x(nx(i),3) - x(nx(j+2),3))**2  )
               if (debug) write(irefwr,110) 'h', h(j+(i-1)*2)
            end do
         end do

!        --- compute intersection positions

         do i = 1, 2
            do j = 1, 2
               if ( nsol==2 ) then
                  dh(j+(i-1)*2) = h(j+(i-1)*2)* &
                      (tsol - temperature(nx(i)))/ &
                      (temperature(nx(j+2)) - temperature(nx(i)))
               else
                  dh(j+(i-1)*2) = h(j+(i-1)*2)* &
                      (tliq - temperature(nx(i)))/ &
                      (temperature(nx(j+2)) - temperature(nx(i)))
               end if
               if (debug) write(irefwr,110) 'dh', dh(j+(i-1)*2)
            end do
         end do

!        --- compute coordinates of intersection points

         do i = 1, 2
            do k = 1, 2
               do j = 1, ndim
                  if (x(nx(i),j)<x(nx(k+2),j)) then
                     xs(k+(i-1)*2,j) = x(nx(i),j) + &
                           abs(x(nx(i),j)-x(nx(k+2),j))* &
                                dh(k+(i-1)*2)/h(k+(i-1)*2)
                  else
                     xs(k+(i-1)*2,j) = x(nx(i),j) - &
                           abs(x(nx(i),j)-x(nx(k+2),j))* &
                                dh(k+(i-1)*2)/h(k+(i-1)*2)
                  end if
               end do
            end do
         end do

!        --- construct phixs

         do j = 1, ndim
            do i = 1, 4
               xcut(i,j) = xs(i,j)
            end do
            xcut(5,j) = x(nx(3),j)
            xcut(6,j) = x(nx(4),j)
         end do

!        --- construct phixs

         phixs(nx(1), 1) = 1d0 - dh(1)/h(1)
         phixs(nx(2), 1) = 0d0
         phixs(nx(3), 1) = dh(1)/h(1)
         phixs(nx(4), 1) = 0d0
         phixs(nx(1), 2) = 1d0 - dh(2)/h(2)
         phixs(nx(2), 2) = 0d0
         phixs(nx(3), 2) = 0d0
         phixs(nx(4), 2) = dh(2)/h(2)
         phixs(nx(1), 3) = 0d0
         phixs(nx(2), 3) = 1d0 - dh(3)/h(3)
         phixs(nx(3), 3) = dh(3)/h(3)
         phixs(nx(4), 3) = 0d0
         phixs(nx(1), 4) = 0d0
         phixs(nx(2), 4) = 1d0 - dh(4)/h(4)
         phixs(nx(3), 4) = 0d0
         phixs(nx(4), 4) = dh(4)/h(4)

!        --- Fill phicut

         do i = 1, n
            phicut(nx(i),1) = phixs(nx(i),1)
            phicut(nx(i),2) = phixs(nx(i),2)
            phicut(nx(i),3) = phixs(nx(i),3)
            phicut(nx(i),4) = phixs(nx(i),4)
            phicut(nx(i),5) = phi(nx(i),nx(3))
            phicut(nx(i),6) = phi(nx(i),nx(4))
         end do

!        --- fill tempcut

         do i = 1, 4
            if (nsol==2) then
               tempcut(i) = tsol
            else
               tempcut(i) = tliq
            end if
         end do
         tempcut(5) = temperature(nx(3))
         tempcut(6) = temperature(nx(4))

!        --- evaluate integrals over non-mushy prism

         call eldiscint3prism( intrule, coefarr, 0, nx, 2, &
                              phicut, dphidx, xcut, tempcut, massmt, &
                              stiffmt, dldtmt, latentvc, -1d0, mloc)
         if (nsol==2) then
            call eldiscint3prism( intrule, coefarr, 1, nx, 2, &
                                 phicut, dphidx, xcut, tempcut, massmt, &
                                 stiffmt, dldtmt, latentvc, 1d0, mloc)
         else
            call eldiscint3prism( intrule, coefarr, 2, nx, 2, &
                                 phicut, dphidx, xcut, tempcut, massmt, &
                                 stiffmt, dldtmt, latentvc, 1d0, mloc)
         end if

      else if ( inttype==7 ) then! 1 liquid/solid node, 3 solid/liquid nodes

!     --- compute det such that |det| = 6*volume whole element

         call elcompdet ( det, e, x, 0 )
         det = abs(det)
         if (debug) write(irefwr,*) 'det = ', det

         if (nsol==3) then
            call eldiscint3sl( intrule, rho, cs, ks, latentheat, &
                              phicut, dphidx, det, tempcut, massmt, &
                              stiffmt, latentvc, 1d0, mloc, .false.)
         else if (nliq==3) then
            call eldiscint3sl( intrule, rho, cl, kl, latentheat, &
                              phicut, dphidx, det, tempcut, massmt, &
                              stiffmt, latentvc, 1d0, mloc, .true.)

         else
            call eldiscint3m( intrule, rho, cm, kmushy, latentheat, &
                             phicut, dphidx, det, tempcut, massmt, &
                             stiffmt, dldtmt, latentvc, 1d0, mloc)
         end if ! (nsol==3)

!        --- compute lengths of intersected edges

         do i = 1, 3
            h(i) = sqrt((x(nx(1),1) - x(nx(i+1),1))**2 + &
                     (x(nx(1),2) - x(nx(i+1),2))**2 + &
                     (x(nx(1),3) - x(nx(i+1),3))**2  )
            if (debug) write(irefwr,*) 'h ', h(i)
         end do

!        --- compute intersection positions

         do i = 1, 3
            if (nsol==1 .or. nsol==3) then
               dh(i) = h(i)*(tsol - temperature(nx(1)))/ &
                         (temperature(nx(i+1)) - temperature(nx(1)))
            else
               dh(i) = h(i)*(tliq - temperature(nx(1)))/ &
                         (temperature(nx(i+1)) - temperature(nx(1)))

            end if ! (nsol==1 .or. nsol==3)
            if (debug) write(irefwr,*) 'dh ', dh(i)
         end do

!        --- compute coordinates of intersection points

         do i = 1, 3
            do j = 1, ndim
               if (x(nx(1),j)<x(nx(i+1),j)) then
                  xs(i,j) = x(nx(1),j) + &
                        abs(x(nx(1),j)-x(nx(i+1),j))*dh(i)/h(i)
               else
                  xs(i,j) = x(nx(1),j) - &
                        abs(x(nx(1),j)-x(nx(i+1),j))*dh(i)/h(i)
               end if
            end do
         end do

         if (debug) then
            do i = 1, ndim
               write(irefwr,*) 'xs ', xs(i,1), xs(i,2), xs(i,3)
            end do
         end if

!        --- compute det such that |det| = 6*volume of subtetra

         do i = 1, ndim
            xs(4,i) = x(nx(1),i)
         end do

         call elcompdet ( det, e, xs, 0 )
         det = abs(det)
         if (debug) write(irefwr,*) 'det = ', det

!        --- construct phixs

         phixs(nx(1),1) = 1d0 - dh(1)/h(1)
         phixs(nx(2),1) = dh(1)/h(1)
         phixs(nx(3),1) = 0d0
         phixs(nx(4),1) = 0d0
         phixs(nx(1),2) = 1d0 - dh(2)/h(2)
         phixs(nx(2),2) = 0d0
         phixs(nx(3),2) = dh(2)/h(2)
         phixs(nx(4),2) = 0d0
         phixs(nx(1),3) = 1d0 - dh(3)/h(3)
         phixs(nx(2),3) = 0d0
         phixs(nx(3),3) = 0d0
         phixs(nx(4),3) = dh(3)/h(3)

         do i = 1, n
            phicut(nx(i),1) = phixs(nx(i),1)
            phicut(nx(i),2) = phixs(nx(i),2)
            phicut(nx(i),3) = phixs(nx(i),3)
            phicut(nx(i),4) = phi(nx(i),nx(1))
         end do

         if (nsol==1) then

            tempcut(1) = tsol
            tempcut(2) = tsol
            tempcut(3) = tsol
            tempcut(4) = temperature(nx(1))

            call eldiscint3sl( intrule, rho, cs, ks, latentheat, &
                              phicut, dphidx, det, tempcut, massmt, &
                              stiffmt, latentvc, 1d0, mloc, .false.)
            call eldiscint3m( intrule, rho, cm, kmushy, latentheat, &
                             phicut, dphidx, det, tempcut, massmt, &
                             stiffmt, dldtmt, latentvc, -1d0, mloc)

         else if (nsol==3) then

            tempcut(1) = tsol
            tempcut(2) = tsol
            tempcut(3) = tsol
            tempcut(4) = temperature(nx(1))

            call eldiscint3sl( intrule, rho, cs, ks, latentheat, &
                              phicut, dphidx, det, tempcut, massmt, &
                              stiffmt, latentvc, -1d0, mloc, .false.)
            call eldiscint3m( intrule, rho, cm, kmushy, latentheat, &
                             phicut, dphidx, det, tempcut, massmt, &
                             stiffmt, dldtmt, latentvc, 1d0, mloc)

         else if (nliq==1) then

            tempcut(1) = tliq
            tempcut(2) = tliq
            tempcut(3) = tliq
            tempcut(4) = temperature(nx(1))

            call eldiscint3sl( intrule, rho, cl, kl, latentheat, &
                              phicut, dphidx, det, tempcut, massmt, &
                              stiffmt, latentvc, 1d0, mloc, .true.)
            call eldiscint3m( intrule, rho, cm, kmushy, latentheat, &
                             phicut, dphidx, det, tempcut, massmt, &
                             stiffmt, dldtmt, latentvc, -1d0, mloc)

         else

            tempcut(1) = tliq
            tempcut(2) = tliq
            tempcut(3) = tliq
            tempcut(4) = temperature(nx(1))
            call eldiscint3sl( intrule, rho, cl, kl, latentheat, &
                              phicut, dphidx, det, tempcut, massmt, &
                              stiffmt, latentvc, -1d0, mloc, .true.)
            call eldiscint3m( intrule, rho, cm, kmushy, latentheat, &
                             phicut, dphidx, det, tempcut, massmt, &
                             stiffmt, dldtmt, latentvc, 1d0, mloc)
         end if ! (nsol==1)

      end if ! ( inttype==1)

      if ( debug ) then
         do i = 1, n
            if ( latentvc(i)<-1d-6 ) then
               print *, 'inttype', inttype
               call prinrl(latentvc,n,'latentvc')
               call instop
            end if
            do j = 1, n
               if ( dldtmt(i, j)<-1d-6 ) then
                  print *, 'inttype', inttype
                  call prinrl2(dldtmt,n,n,'dldtmt')
                  call instop
               end if
            end do
         end do
      end if

      call erclos ( 'eldiscint3mush' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End eldiscint3mush'

      end if

      end
