      subroutine elfillcoeffint ( coeffintpnts, coeffnodes, phi, &
                                  number )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 17-07-2011
!
!   copyright (c) 2011-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill coefficients computed in nodes in integration points
!     jdiag = 2, nodes and integration points differ
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     element
!     fill
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer number
      double precision coeffnodes(*), coeffintpnts(*), phi(n,*)

!     coeffintpnts  i/o   Real array in which the values of the
!                         coefficients in the integration points are stored
!     coeffnodes     i    Real array in which the values of the
!                         coefficients in the nodal points are stored
!     number         i    Sequence number of coefficient
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, interpol, ndegfd
      logical linear

!     i              Counting variable
!     interpol       If 0 the standard interpolation is applied, &
!                    if 1 the interpolation is restricted to linear
!                    subelements
!     j              Counting variable
!     linear         Indicates whether the element is a linear simplex
!                    (true) or not (false)
!     ndegfd         Number of degrees of freedom per point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ddot

!     DDOT           (CDOT)     Inner product of two vectors
!     ELFILLCOEFFIN1 Fill coefficients computed in nodes in integration points
!                    interpolation in subelements must be used
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PJLIMT         Special subroutine for program gobs
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elfillcoeffint' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elfillcoeffint'
         write(irefwr,1) 'irule, ishape', irule
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      interpol = irule/100
      linear = ishape==3 .or. ishape==5 .or. ishape==11 .or. &
               ishape==13 .or. ishape>=17 .or. ishape==9 .or. &
               ishape==10

      if ( debug ) then
         write(irefwr,1) 'interpol', interpol
         write(irefwr,*) 'linear', linear
      end if  ! ( debug )

      if ( ich(number)>1001 .and. ich(number)<2000 ) then

!     --- ndegfd degrees of freedom

         ndegfd = ich(number)-1000

         do j = 1, ndegfd
            do i = 1, m
               coeffintpnts(i+(j-1)*m) = &
                  ddot ( n, coeffnodes(1+(j-1)*n), 1, phi(1,i), 1 )
            end do  ! i = 1, m
         end do  ! j = 1, ndegfd

         if ( debug ) then
            call prinrl1 ( coeffnodes, ndegfd, n, 'coeffnodes' )
            call prinrl1 ( coeffintpnts, ndegfd, m, 'coeffintpnts' )
         end if  ! ( debug )

      else

!     --- 1 degree of freedom

         ndegfd = 1
         do i = 1, m
            coeffintpnts(i) = ddot ( n, coeffnodes, 1, phi(1,i), 1 )
         end do  ! i = 1, m

         if ( debug ) then
            call prinrl ( coeffnodes, n, 'coeffnodes' )
            call prinrl ( coeffintpnts, m, 'coeffintpnts' )
         end if  ! ( debug )

      end if  ! ( ich(number)>1000 .and. ich(number)<=2000 )

      if ( interpol==1 .and. .not. linear ) then

!     --- interpol = 1, use linear interpolation in subelements

         call elfillcoeffin1 ( coeffintpnts, coeffnodes, ndegfd )

      end if  ! ( interpol==0 )

1000  call erclos ( 'elfillcoeffint' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elfillcoeffint'

      end if  ! ( debug )

      end
