      subroutine elfillderiv ( u, dphidx, dudx, n, ncomp, m, jdercn, &
                               lsub, lsup, lstep, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-08-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill the derivatives of the old solution in nodal points
!    It is assumed that the solution itself has been already filled
! **********************************************************************
!
!                       KEYWORDS
!
!     fill
!     fill_solution
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, ncomp, m, jdercn, lsub, lsup, lstep,ndim
      double precision u(n,ncomp), &
                       dudx(m,ndim,ncomp), dphidx(n,m,ndim)

!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dudx          o     Real m x ndim x ncomp array in which the values of the
!                         derivatives of the solution vector in the integration
!                         points are stored in the sequence:
!                         du_i/dx_j(x_k) = dudx ( k, j, i )
!     jdercn         i    Indication if the derivatives of the basis functions
!                         are constant (jdercn=0) or variable (jdercn=1)
!     lstep          i    Step variable for vertices. Usually lstep = 1, &
!                         but when the solution must be computed in the vertices
!                         and the element is quadratic then lstep = 2
!     lsub           i    Under index for the direction of the derivative to be
!                         computed
!     lsup           i    Upper index for the direction of the derivative to be
!                         computed
!     m              i    number of integration points
!     n              i    number of basis functions for each component
!     ncomp          i    array containing lower (1, segnum) and upper (2, &
!                         segnum) components of segment segnum.
!     ndim           i    Dimension of the space.
!     u              i    array of length m containing the values of the
!                         second coefficient in the integration points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, k, l
      double precision sum

!     j              Counting variable
!     k              Counting variable
!     l              General loop variable
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ddot

!     DDOT           (CDOT)     Inner product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elfillderiv' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elfillderiv'
         write(irefwr,*) 'derivconst ', jdercn
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         write(irefwr,1) 'n, m, ndim, lsup', &
                          n, m, ndim, lsup
      end if  ! ( debug )

      if ( jdercn==0 ) then

!     --- Derivatives of basis functions are constant, hence dudx is constant
!                   n
!         du/dx =  sum  u   d phi  / dx
!              k   i=1   i       i     k

         do j = 1, ncomp
            do k = lsub, lsup
               sum = ddot ( n, u(1,j), 1, dphidx(1,1,k), 1 )
               do l = 1, m, lstep
                  dudx(l,k+1-lsub,j) = sum
               end do
            end do
         end do

      else

!     --- dudx is variable

         do j = 1, ncomp
            do k = lsub, lsup
               do l = 1, m, lstep
                  dudx(l,k+1-lsub,j) = ddot ( n, u(1,j), 1, &
                                              dphidx(1,l,k), 1 )
               end do
            end do
         end do

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elfillderiv'

      end if  ! ( debug )
      call erclos ( 'elfillderiv' )

      end
