      subroutine elflr8 ( number, coeff, &
                          work, phi, ndecl, iuser, ifunc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.22 date 10-02-2015 Extension for non-conforming triangle
!        version  2.21 date 10-03-2013 Extra debug
!        version  2.20 date 12-12-2012 Long integers
!
!   copyright (c) 1989-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fills array coeff with a coefficient, where the coefficient is
!     equal to a solution or derivative of a preceding computation.
!     The value is only stored in all nodal points.
!     If the old solution is not available for all nodes, then it is
!     computed in the missing points by interpolation.
!     At this moment it is assumed that the old solution is at least
!     available in all vertex points
!     Number corresponds to the number th coefficient in common block celiar
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmodulecomio
      use sepmoduletypes
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer number, ndecl, iuser(*), ifunc
      double precision coeff(*), work(*), phi(ndecl,*)

!     coeff   o      Double precision output array of length n in which the
!                    values of the coefficient in the nodal points are
!                    stored
!     ifunc   o      Output parameter to be used if ind(number)>4000
!                    It indicates if the parameter which with the function
!                    is multiplied is a function (>0) or a constant (<0)
!     iuser   i      Standard array containing integer information of the
!                    coefficients
!     ndecl   i      Declared value of n
!     number  i      Parameter indicating which coefficient in common block
!                    celiar must be considered
!     phi     i      array containing the values of the basis functions
!                    in the integration points
!     work           Two-dimensional work array of length max(n+1,m)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivec, idegfd, i, ndegfd, interpol, j
      double precision, pointer :: vecoldlc(:)
      double precision som

!     i              Counting variable
!     idegfd         Degree of freedom in VECTOR(IVEC) that must be used
!     interpol       If 0 the standard interpolation is applied, &
!                    if 1 the interpolation is restricted to linear subelements
!     ivec           Sequence number of vector in array vecold.
!                    Hence array vecold(IVEC) is considered
!     j              Counting variable
!     ndegfd         Number of degrees of freedom per point in array
!                    vecold(IVEC)
!     som            Help variable to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inivec
      double precision ddot

!     DDOT           (CDOT)     Inner product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INIVEC         Return with vector number corresponding to ivecin
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2049   unknown not present in all nodes in 3D not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elflr8' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elflr8'
         write(irefwr,1) 'irule,number,inpelm', &
                          irule,number,inpelm
         write(irefwr,1) 'kstep, m, nvert, jsol', kstep, m, nvert, jsol
         write(irefwr,1) 'inpelm, n, jdiag', inpelm, n, jdiag
         write(irefwr,1) 'ind(number)', ind(number)
         write(irefwr,1) 'mst(number)', mst(number)
         write(irefwr,1) 'ich(number)', ich(number)
         write(irefwr,1) 'ivecnums(number)', ivecnums(number)
  1      format ( a, 1x, (10i6) )

      end if

      interpol = irule/100
      if ( ind(number)==2004 ) then

!     --- ind(number)=2004, special situation old solution must be
!                           multiplied by a coefficient
!                           The multiplication is carried out in elflr6

         ipos  = ivecnums(number)
         ifunc = iuser(ipos+2)
         ivec = inivec ( iuser(ipos) )
         if ( debug ) &
            write(irefwr,1) 'ipos, ivec, ifunc', ipos, ivec, ifunc

      else

!     --- ind(number)=2003

         ipos  = ivecnums(number)
         ivec = inivec ( iuser(ipos) )
         if ( debug ) write(irefwr,1) 'ivec', ivec

      end if
      idegfd = max(1,mst(number))

      iseqtype = oldsols(ivec)%iseqtype

      index3n => indoldsols(iseqtype)%index3
      index4n => indoldsols(iseqtype)%index4
      vecoldlc => oldsols(ivec)%oldvecs

      if ( debug ) then
         write(irefwr,1) 'iseqtype', iseqtype
!         call prinrl ( vecoldlc(1),40,' vecoldlc' )
      end if
      if ( ich(number)==0 ) then

!     --- ich(number) = 0, constant number of unknowns per element

         som = vecoldlc(index3n(idegfd))
         do i = kstep, m, kstep
            coeff(i) = som
         end do
         go to 1000

      else if ( ich(number)==1 ) then

!     --- in each point there is exactly one unknown

         do i = 1, inpelm
            work(i) = vecoldlc ( index3n(i) )
         end do

      else if ( ich(number)==2 ) then

!     --- in each point the number of unknowns is constant

         ndegfd = index4n(1)
         do i = 1, inpelm
            work(i) = vecoldlc ( index3n((i-1)*ndegfd+idegfd) )
         end do

      else if ( ich(number)==3 ) then

!     --- There is exactly one unknown in the vertices only

         if ( n==27 ) then

!        --- quadratic hexahedron

            do i = 1, 8
               work(ivertx14(i)) = vecoldlc ( index3n(i) )
            end do

         else

!        --- standard situation

            do i = 1, nvert*2, 2
               work(i) = vecoldlc ( index3n((i+1)/2) )
            end do

         end if

      else if ( ich(number)==4 ) then

!     --- in each vertex the number of unknowns is constant
!         Outside the vertices there are no unknowns

         ndegfd = index4n(1)
         do i = 1, nvert*2 ,2
            work(i) = vecoldlc ( index3n((i-1)*ndegfd+idegfd) )
         end do

      else if ( ich(number)==5 ) then

!     --- The number of unknowns is variable
!         The unknown is present in all nodes

         if ( debug ) then
            write(irefwr,1) 'idegfd', idegfd
            write(irefwr,1) 'index3n(idegfd)', index3n(idegfd)
         end if
         work(1) = vecoldlc ( index3n(idegfd) )
         do i = 2, inpelm
            work(i) = vecoldlc ( index3n(index4n(i-1)+idegfd) )
         end do
         if ( debug ) call prinrl ( work, inpelm, 'work' )

      else if ( ich(number)==7 ) then

!     --- The number of unknowns is variable
!         The unknown is present in all nodes except centroid

         work(1) = vecoldlc ( index3n(idegfd) )
         do i = 2, inpelm-1
            work(i) = vecoldlc ( index3n(index4n(i-1)+idegfd) )
         end do  ! i = 1, inpelm

         if ( inpelm==7 ) then

!        --- inpelm = 7

            work(7) = (4d0*(work(2)+work(4)+work(6))- &
                       work(1)-work(3)-work(5))/9d0

         else

!        --- inpelm # 7

            som = work(1)
            do i = 2, inpelm-1
               som = som + work(i)
            end do
            work(inpelm) = som/inpelm

         end if  ! ( inpelm==7 )

      else if ( ich(number)==8 ) then

!     --- The number of unknowns is variable
!         The unknown is present in all nodes except centroids
!         of tetrahedrons
!         First fill vertices and mid-points

         do i = 1, 10
            work(i) = vecoldlc ( index3n(index4n(i-1)+idegfd) )
         end do  ! i = 1, inpelm

!        --- Next fill the 4 centroids of the faces

         work(11) = (4d0*(work(2)+work(4)+work(6))- &
                     work(1)-work(3)-work(5))/9d0
         work(12) = (4d0*(work(2)+work(8)+work(7))- &
                     work(1)-work(3)-work(10))/9d0
         work(13) = (4d0*(work(4)+work(8)+work(9))- &
                     work(3)-work(5)-work(10))/9d0
         work(14) = (4d0*(work(6)+work(7)+work(9))- &
                     work(1)-work(5)-work(10))/9d0

         if ( inpelm==15 ) then

!        --- inpelm = 15, centroid of element

            work(15) = 0.25d0*(work(11)+work(12)+work(13)+work(14))

         end if  ! ( inpelm==15 )

      else if ( ich(number)>2000 ) then

!     --- Special situation of vector of type 129 with ich(number)-2000
!         degrees of freedom per element

         do i = 1, (ich(number)-2000)*m
            coeff(i) = vecoldlc ( index3n(i) )
         end do

      else if ( ich(number)>1000 ) then

!     --- Special situation of vector of type 126 with ich(number)-1000
!         degrees of freedom per element

         if ( jdiag==1 ) then

!        --- The nodes are also integration points

            ndegfd = ich(number)-1000
            do j = 1, ndegfd
               do i = 1, n
                  coeff(i+(j-1)*m) = &
                     vecoldlc ( index3n(j+(i-1)*ndegfd) )
               end do
            end do

         else

!        --- Gauss integration points, store in work and then compute coeff

            ndegfd = ich(number)-1000
            do j = 1, ndegfd
               do i = 1, n
                  work(i) = vecoldlc ( index3n(j+(i-1)*ndegfd) )
               end do
               do i = 1, m
                  coeff(i+(j-1)*m) = ddot ( n, work, 1, phi(1,i), 1 )
               end do
            end do

         end if

      else if ( ich(number)>100 ) then

!     --- Special situation of vector of type 116 with ich(number)-100
!         degrees of freedom per element

         do i = 1, ich(number)-100
            coeff(i) = vecoldlc ( index3n(i) )
         end do

      else

!     --- The number of unknowns is variable
!         The unknown is present in all vertices, but not in all nodes

         work(1) = vecoldlc ( index3n(idegfd) )

         if ( inpelm==3 ) then

!        --- inpelm = 3:  quadratic one-dimensional elements

            work(3)=vecoldlc( index3n(index4n(4)+idegfd) )

         else if ( inpelm==6 .or. inpelm==7 .or. inpelm==9 ) then

!        --- inpelm = 6, 7 or 9:  quadratic two-dimensional elements

            do i = 3, inpelm-1, 2
               work(i) = vecoldlc ( index3n( index4n(2*i-2)+idegfd))
            end do

         else if ( inpelm==27 ) then

!        --- inpelm = 27:  quadratic hexahedron

            do i = 1, 8
               work(i) = vecoldlc ( index3n(index4n(i)+idegfd) )
            end do

         else

!        --- This possibility has not yet been implemented in ELFLR8

            call errsub ( 2049, 0, 0, 0 )
            go to 1000

         end if

      end if

      if ( ich(number)==3 .or. ich(number)==4 .or. &
           ich(number)==6 ) then

!     --- The coefficients have only been computed in the vertices
!         Interpolate to the other nodes

         if ( n==3 ) then

!        --- n = 3:  quadratic one-dimensional elements

            work(2) = 0.5d0*( work(1) + work(3) )

         else if ( n==6 .or. n==7 .or. n==9 ) then

!        --- n = 6, 7 or 9:  quadratic two-dimensional elements

            work(2*nvert+1) = work(1)
            do i = 1, nvert
               work(2*i) = 0.5d0 * ( work(2*i-1) + work(2*i+1) )
            end do

            if ( n==7 ) then

!           --- n = 7, fill 7 th point

               work(n) = ( work(1) + work(3) + work(5) ) / 3d0

            else if ( n==9 ) then

!           --- n = 9, fill 9 th point

               work(n) = ( work(1) + work(3) + work(5) + work(7) ) * .25d0

            end if

         else if ( n==27 ) then

!        --- n = 27, fill midside points and centroids

            work(10) = 0.5d0*(work(1)+work(19))
            work(12) = 0.5d0*(work(3)+work(21))
            work(16) = 0.5d0*(work(7)+work(25))
            work(18) = 0.5d0*(work(9)+work(27))
            do i = 0, 18, 9
               work(2+i) = 0.5d0*(work(1+i)+work(3+i))
               work(4+i) = 0.5d0*(work(1+i)+work(7+i))
               work(6+i) = 0.5d0*(work(3+i)+work(9+i))
               work(8+i) = 0.5d0*(work(7+i)+work(9+i))
               work(5+i) = 0.25d0*(work(1+i)+work(3+i)+ &
                                   work(7+i)+work(9+i))
               work(2+i) = 0.5d0*(work(1+i)+work(3+i))
               work(4+i) = 0.5d0*(work(1+i)+work(7+i))
               work(6+i) = 0.5d0*(work(3+i)+work(9+i))
               work(8+i) = 0.5d0*(work(7+i)+work(9+i))
               work(5+i) = 0.25d0*(work(1+i)+work(3+i)+ &
                                   work(7+i)+work(9+i))
            end do

         else

!        --- This possibility has not yet been implemented in ELFLR8

            call errsub ( 2049, 0, 0, 0 )
            go to 1000

         end if

      end if

      if ( inpelm<n .and. ich(number)<100 ) then

!     --- inpelm<n, special situation
!                     At this moment only 6 point triangles with dummy
!                     centroid are allowed

         if ( n==7 ) then
            if ( interpol==0 ) then
               work(n) = ( -work(1)-work(3)-work(5) + &
                         4d0 * ( work(2)+work(4)+work(6) ) ) / 9d0
            else
               work(n) = ( work(2)+work(4)+work(6) ) /3d0
            end if
         else if ( n==4 ) then
            work(n) = ( work(1)+work(2)+work(3) ) /3d0
         else if ( n==5 ) then
            work(n) = ( work(1)+work(2)+work(3)+work(4) ) * 0.25d0
         end if

      end if

      if ( inpelm==6 .and. n==3 ) then

!     --- Special case of non-conforming triangle

         do i = 1, 3
            work(i) = work(2*i)
         end do  ! i = 1, 3

      end if  ! ( inpelm==6 .and. n==3 )

      if ( debug ) call prinrl ( work, n, 'work' )

      if ( ich(number)/=0 .and. ich(number)<100 .and. jdiag==2 ) then

!     --- jdiag = 2, compute coefficients in Gauss points
!                    Store result in coeff

         if ( jsol>0 ) then
            if ( interpol==0 ) then
               do i = 1, m
                  coeff(i) = ddot ( n, work, 1, phi(1,i), 1 )
               end do
            else if ( ndimlc==2 .and. n==6 .or. n==7 ) then
               if ( m==4 ) then
                  coeff(1) = ( work(2)+work(4)+work(6) ) /3d0
                  coeff(2) = 0.2d0*work(1)+0.4d0*(work(2)+work(6))
                  coeff(3) = 0.2d0*work(2)+0.4d0*(work(4)+work(2))
                  coeff(4) = 0.2d0*work(3)+0.4d0*(work(6)+work(4))
               else if ( m==7 ) then
                  coeff(1) = ( work(2)+work(4)+work(6) ) /3d0
                  coeff(2) = (alam3(1,2)+alam3(2,2)-alam3(3,2))*work(2)+ &
                             (alam3(2,2)+alam3(3,2)-alam3(1,2))*work(4)+ &
                             (alam3(3,2)+alam3(1,2)-alam3(2,2))*work(6)
                  coeff(3) = (alam3(1,3)+alam3(2,3)-alam3(3,3))*work(2)+ &
                             (alam3(2,3)+alam3(3,3)-alam3(1,3))*work(4)+ &
                             (alam3(3,3)+alam3(1,3)-alam3(2,3))*work(6)
                  coeff(4) = (alam3(1,4)+alam3(2,4)-alam3(3,4))*work(2)+ &
                             (alam3(2,4)+alam3(3,4)-alam3(1,4))*work(4)+ &
                             (alam3(3,4)+alam3(1,4)-alam3(2,4))*work(6)
                  coeff(5) = (alam3(1,5)-alam3(2,5)-alam3(3,5))*work(1)+ &
                             2d0*alam3(2,5)*work(2)+ &
                             2d0*alam3(3,6)*work(6)
                  coeff(6) = (alam3(2,6)-alam3(3,6)-alam3(1,6))*work(3)+ &
                             2d0*alam3(3,6)*work(4)+ &
                             2d0*alam3(1,6)*work(2)
                  coeff(7) = (alam3(3,7)-alam3(1,7)-alam3(2,7))*work(5)+ &
                             2d0*alam3(1,7)*work(6)+ &
                             2d0*alam3(2,7)*work(4)
               else
                  do i = 1, m
                     coeff(i) = ddot ( n, work, 1, phi(1,i), 1 )
                  end do
               end if
            else
               do i = 1, m
                  coeff(i) = ddot ( n, work, 1, phi(1,i), 1 )
               end do
            end if

         else if ( jsol==-1 ) then

!        --- Compute solution in Gauss points
!            Temporary situation, especially for old versions of ELP
!            subroutines, &
!            where the sequence of derivatives of basis functions and basis
!            functions itself differs from the new storage
!                          n
!            coeffg (k) = sum   coeff (i) phi (k)
!                  l      i=1                i

            do i = 1, m
               coeff(i) = ddot ( n, work, 1, phi(i,1), m )
            end do

         end if

      else if ( ich(number)/=0 .and. ich(number)<100 ) then

!     --- jdiag = 1, copy coefficients in coeff

         do i = kstep, m, kstep
            coeff(i) = work(i)
         end do

      end if

1000  if ( debug ) then
         call prinrl ( coeff, m, 'coeff' )
         write(irefwr,*) 'End elflr8'
      end if  ! ( debug )
      call erclos ( 'elflr8' )

      end
