      function elintsub ( jcheli, kdegfd, iuser, user )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.1  date 28-08-2014 New call to elintsubr
!        version  1.0  date 12-12-2012
!
!   copyright (c) 2012-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      User element subroutine to compute integrals to be used by SEPRAN
!      This is the general version, which produces an error message if it is
!      called except in the case of the Numerical Analysis Lab of Delft
!      University of Technology
! **********************************************************************
!
!                       KEYWORDS
!
!     integral
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelem
      use sepmodulemach
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jcheli, kdegfd, iuser(*)
      double precision elintsub, user(*)

!     elintsub       o    Value of the element integral
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     index2         i    Array of length icount containing the positions of the
!                         degrees of freedom in the element with respect to USOL
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     jcheli         i    Defines the type of integral to be computed
!     kdegfd         i    The degree of freedom kdegfd in vectrlc is used
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     vectrlc         i    In this array all preceding solutions are stored, i.e.
!                         all solutions that have been computed before and
!                         have been carried to system or deriva by the parameter
!                         islold in the parameter list of these main subroutines
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenhome, lenarray, i, inpelmcopy, &
              icountcopy, itypecopy, jchelicopy
      parameter ( lenarray=100 )
      double precision x(lenarray), u(lenarray), y(lenarray)
      double precision, pointer :: vecoldlc(:)
      character * 80 filenm
      logical check

!     check       Indicates if the file exist (true) or not
!     filenm      Name of file
!     i           Counting variable
!     icountcopy  Copy of icount
!     inpelmcopy  Copy of inpelm
!     itypecopy   Copy of itype
!     jchelicopy  Copy of jcheli
!     lenarray    Length of arrays u and x
!     lenhome     Number of characters in the name of the SEPRAN home directory
!     sum         Help parameter to compute a sum
!     u           array of length m containing the values of vectrlc1 in the
!                 nodal points of the element
!     x           Contains the x-coordinates of the nodes of the element using the
!                 local node numbering
!     y           Contains the y-coordinates of the nodes of the element using the
!                 local node numbering
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision elintsubr, elint

!     ELINT          Element subroutine for user element
!     ELINTSUBR      Special user element subroutine to be called by ELINT and
!                    meant for the numerical analysis lab at Delft University
!                    of Technology
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!    If ( file sphome/bin/update/wi211 exists ) then
!       prepare call to elintsubr
!       call elintsubr
!       give elint a value
!    else
!       give error message
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elintsub' )

      lenhome = index(sphome,' ')-1
      filenm = sphome(1:lenhome)//'/bin/update/wi211'
      inquire ( file=filenm, exist=check )

      if ( jcheli==999 ) then

!     --- Special case of jcheli = 999, add absolute values of solution

         iseqtype = oldsols(iseqin)%iseqtype
         index3n => indoldsols(iseqtype)%index3
         vecoldlc => oldsols(iseqin)%oldvecs
         u(1:icount) = vecoldlc(index3n(1:icount))
         elintsub = sum(abs(u(1:icount)))

      else if ( check ) then

!     --- wi211 file found, Hence special situation for the "wi211-lab" at
!         Delft University
!         First check array boundaries

         if ( inpelm*ndimlc>lenarray ) then
            call errint ( inpelm*ndimlc, 1 )
            call errint ( lenarray, 2 )
            call errchr ( 'x', 1 )
            call errsub ( 550, 2, 0, 1 )
         end if
         if ( icount>lenarray ) then
            call errint ( icount, 1 )
            call errint ( lenarray, 2 )
            call errchr ( 'u', 1 )
            call errsub ( 550, 2, 0, 1 )
         end if
         if ( ierror/=0 ) go to 1000

!        --- Copy co-ordinates from coor into x

         x(1:inpelm) = coor(1,index1(1:inpelm))
         y(1:inpelm) = coor(2,index1(1:inpelm))

!        --- Copy old solution into u

         iseqtype = oldsols(iseqin)%iseqtype
         index3n => indoldsols(iseqtype)%index3
         vecoldlc => oldsols(iseqin)%oldvecs
         u(1:icount) = vecoldlc(index3n(1:icount))
         inpelmcopy = inpelm
         icountcopy = icount
         itypecopy = itype
         jchelicopy = jcheli

         elintsub = elintsubr ( inpelmcopy, x, y, icountcopy, &
                                u, itypecopy, jchelicopy )

      else

!     --- Standard situation

         elintsub = elint ( jcheli, kdegfd, coor, iuser, user, &
                            vecold, index1, index2 )

      end if

1000  call erclos ( 'elintsub' )
      if ( ierror/=0 ) call instop

      end
