      subroutine elm800upwbase ( metup, jelp, alpha, u, phi, psi, x, &
                                 second, dphidx, dudx, iflipflop, &
                                 dpsidx, w )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.1  date 20-01-2012 Extra debug
!        version  2.0  date 12-08-2011 Extra parameter w
!
!   copyright (c) 2002-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the modified basis functions psi due to upwinding
!     Help subroutine for elm800
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     upwinding
!     basis_function
!     elliptic_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer metup, jelp, iflipflop
      double precision alpha(*), phi(*), x(*), dphidx(*), dudx(*), &
                       u(*), psi(*), dpsidx(*), w(*)
      logical second

!     alpha          i    array of length m x 3 in which the values of
!                         alpha  , alpha  , alpha
!                              11       12       22
!                         are stored in the sequence:
!                                                 k
!                         alpha ( k,i ) = alpha (x )   i = 1(1)3, k=1(1)m
!                                              i
!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dpsidx         i    Array of length n x m x l  containing the derivatives
!                         of the linear basis functions in the sequence:
!                         d psi / dx (xg ) = psix (i,k,1);
!                              i        k
!                         d psi / dy (xg ) = psix (i,k,2);
!                              i        k
!                         d psi / dz (xg ) = psix (i,k,3);
!                              i        k
!     dudx           i    Real m x ndim  array in which the values of the
!                         derivatives of the solution vector in the integration
!                         points are stored in the sequence:
!                         du / dx  (x ) = dudx ( k, j )
!                           j     k
!                         If jdercn = 0 in common block celint, then only
!                         dudx (1,*) is filled because dudx is then a constant
!                         per element
!                         dudx is only filled if jderiv>0
!     iflipflop      i    Work array to prevent flip-flop behaviour in non- &
!                         linear upwind
!                         The status of the element with respect to upwind is
!                         stored in this array
!     jelp           i    Indication of the type of basis function subroutine
!                         must be called by ELelm800upwbasex. Possible values:
!                          1:  Subroutine ELP610  (Linear line element)
!                          2:  Subroutine ELP611  (Quadratic line element)
!                          3:  Subroutine ELP620  (Linear triangle)
!                          4:  Subroutine ELP624  (Extended quadratic triangle)
!                          5:  Subroutine ELP621  (Bilinear quadrilateral)
!                          6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                         11:  Subroutine ELP630  (Linear tetrahedron)
!                         12:  Subroutine ELP631  (Quadratic tetrahedron)
!                         13:  Subroutine ELP632  (Trilinear brick)
!                         14:  Subroutine ELP633  (Triquadratic brick)
!     metup          o    Upwind parameter
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     psi            o    Array of length n x m containing the values of the
!                         modified basis functions in the integration points
!     second        i/o   Indicates if second derivative term is important in
!                         case of upwinding
!     u              i    Contains velocity vector in integration points
!     w              i    array of length m containing the weights for
!                         integration
!     x              i    Contains coordinates of nodes in element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ih, ndimloc

!     ih             Help variable to store a constant temporarily
!     ndimloc        Local dimension
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELPUP0         Compute modified basis functions for upwind elements (2D)
!     ELPUP1         Compute modified basis functions for upwind elements (1D)
!     ELPUP2         Compute modified basis functions for quadrilateral upwind
!                    elements (2D)
!     ELPUPLINMAX    Compute modified basis functions for upwind elements
!                    (2D, linear triangles, non-linear with maximum principle)
!     ELPUPTIMDEP    Compute modified basis functions psi in case of upwind
!                    of type 12
!     ELPUPTIMDEP1   Compute modified basis functions psi in case of upwind
!                    of type 12 (linear 1d elements)
!     ELPUPTIMDEP3D  Compute modified basis functions psi in case of upwind
!                    of type 12 (linear 3d elements)
!     ELPUPWIND3D    Compute modified basis functions for upwind elements
!                    Three dimensional elements
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm800upwbase' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm800upwbase'
         write(irefwr,1) 'jelp, irule, ndim, metupw', &
                          jelp, irule, ndim, metupw
         write(irefwr,*) 'second ', second
         call prinrl1 ( u, ndim, m, 'u' )
         call prinrl ( alpha, m, 'alpha' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ih = metupw
      metupw = metup
      if ( jelp==4 .and. irule==1 ) kstep = 2

      ndimloc = ndim
      if ( jelp<3 ) then
         ndimloc = 1
      else if ( jelp<7 ) then
         ndimloc = 2
      end if  ! ( jelp<3 )

      select case (ndimloc)
      case(1)

!     --- ndim = 1

         if ( mod(metup,20)==12 ) then

!        --- linear elements, metup = 12

            call elpuptimdep1 ( jelp, u, phi, psi, dphidx, w )

         else

!        --- standard case

            call elpup1 ( alpha, u, phi, psi, dphidx, x, second )

         end if  ! ( mod(metup,20)==12 )

      case(2)

!     --- ndim = 2

         select case (jelp)
         case(3)

!        --- jelp = 3: linear triangles

            if ( mod(metup,20)>=9 .and. mod(metup,20)<=11 ) then

!           --- triangles metup = 9, 10 or 11
!               Special non-linear method

              call elpuplinmax ( alpha, u, phi, psi, dphidx, x, &
                                 dudx, iflipflop, mod(metup,20) )
            else if ( mod(metup,20)==12 ) then

!           --- triangles metup = 12

                call elpuptimdep ( jelp, u, phi, psi, dphidx, w )

            else

!           --- ndim = 2, triangles

               call elpup0 ( alpha, u, phi, psi, dphidx, second, dudx, &
                             dpsidx )
            end if  ! ( mod(metup,20)>=9 .and. mod(metup,20)<=11 )

         case(4)

!        --- quadratic triangles

            call elpup0 ( alpha, u, phi, psi, dphidx, second, dudx, &
                          dpsidx )

         case default

!        --- general case

            call elpup2 ( alpha, u, phi, psi, dphidx, x, &
                          inpelm, second, dudx )

         end select  ! case (jelp)

      case(3)

!     --- ndim = 3

         if ( mod(metup,20)==12 ) then

!        --- linear elements, metup = 12

            call elpuptimdep3d ( jelp, u, phi, psi, dphidx, w )

         else

!        --- standard case

            call elpupwind3d ( alpha, u, phi, psi, dphidx, second, &
                               dudx, jelp, x, dpsidx )

         end if  ! ( mod(metup,20)==12 )

      end select  ! case (ndimloc)

      metup = metupw
      metupw = ih

1000  if ( debug ) then

!     --- Debug information

         call prinrl(psi,n*m,'psiupw')
         write(irefwr,*) 'End elm800upwbase'

      end if  ! ( debug )
      call erclos ( 'elm800upwbase' )

      end
