      subroutine elm813 ( vecloc, coeffarrlc, iflipflop )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.0  date 12-08-2013 Extra parameters
!        version  1.2  date 12-12-2012 Long integers
!
!   copyright (c) 2012-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for second order elliptic equations
!     and related heat equation
!     Type 813: special right-hand side
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision vecloc(*), coeffarrlc(*)
      integer iflipflop

!     coeffarrlc            Real work array which must be kept for each element
!                         group
!                         work consists of the following parts
!                         Starting address    length   Name:
!                         1                   13*27    array containing the
!                                                      coefficients
!                         ipdphx              27*27*3  DPHIDX
!                         ippsi               27*27    PSI
!                         ipphixi            3*27*27  phixi
!                         ipphixicp          3*27*27  copy of phixi
!                         array contains the folowing contents:
!                         array in which the values of the coefficients
!                         in the integration points are stored in the sequence:
!                               1D                  2D            3D
!                          1:   alpha 11         alpha 11       alpha 11
!                          2:                    alpha 12       alpha 12
!                          3:                    alpha 22       alpha 13
!                          4:                                   alpha 22
!                          5:                                   alpha 23
!                          6:                                   alpha 33
!                          7:   u 1              u 1            u 1
!                          8:                    u 2            u 2
!                          9:                                   u 3
!                         10:   beta             beta           beta
!                         11:   f                f              f
!                         12:   rho cp           rho cp         rho cp
!                         13:   gamma_1          gamma_1        gamma_1
!                         14:                    gamma_2        gamma_2
!                         15:                                   gamma_3
!                         16:   g_extra          g_extra        g_extra
!                         Each coefficient requires exact m (is number of
!                         integration points) positions
!     iflipflop     i/o   Work array to prevent flip-flop behaviour in non- &
!                         linear upwind
!                         The status of the element with respect to upwind is
!                         stored in this array
!                         See subroutine elpuplinmax
!     vecloc              Work array in which all old solution vectors for the
!                         integration points are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isub, isup, jelp, kfirst, jheat, ipalpha, &
              ipu, ktrans, metup, metupc, ipf, iprho, &
              imapf, ipgextra, ipbeta, ipphixicp, &
              ipend, ipgamma, kderiv, &
              iuserlc(13)
      double precision  u(3), ah(3,3), f, &
                       psiupw(inpelm*27), &
                       dudx(3*3*27), dpsidx(3*27*8),  &
                       wrk1(27*inpelm+inpelm*7), &
                       uold(27), ugauss(27), told(27), tgauss(27), &
                       dtdx(3*3*27), alpha(27,6)
      logical second
      save isub, isup, jelp, kfirst, jheat, u, f, ipalpha, ipu, &
           metup, metupc, ipf, ipgextra, ipbeta, iprho, &
           imapf, ipphixicp, ipgamma

!     ah             Help matrix that stores diffusion tensor
!     alpha          array of length m x 3 in which the values of
!                    alpha  , alpha  , alpha
!                         11       12       22
!                    are stored in the sequence:
!                                            k
!                    alpha ( k,i ) = alpha (x )   i = 1(1)3, k=1(1)m
!                                         i
!     dpsidx         Array of length n x m x l  containing the derivatives
!                    of the linear basis functions in the sequence:
!                    d psi / dx (xg ) = psix (i,k,1);
!                         i        k
!                    d psi / dy (xg ) = psix (i,k,2);
!                         i        k
!                    d psi / dz (xg ) = psix (i,k,3);
!                         i        k
!     dtdx           Real m x ndim  array containing the derivatives of
!                    the temperature in the integration points, see dudx
!     dudx           Real m x ndim  array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du / dx  (x ) = dudx ( k, j )
!                      j     k
!                    If jdercn = 0 in common block celint, then only
!                    dudx (1,*) is filled because dudx is then a constant
!                    per element
!                    dudx is only filled if jderiv>0
!     f              Constant value of rhs if rhs is constant
!     imapf          If 1 the right-hand side is mapped from higher dimension to
!                    lower dimension
!     ipalpha        Starting position of diffusion tensor in coeffarrlc
!     ipbeta         Starting position of parameter beta in coeffarrlc
!     ipend          Last position in array
!     ipf            Starting position of parameter f in coeffarrlc
!     ipgamma        Starting position of parameter gamma in coeffarrlc
!     ipgextra       Starting position of parameter gextra in coeffarrlc
!     ipphixicp     Starting position of parameter phixicp in coeffarrlc
!     iprho          Starting position of parameter rho in coeffarrlc
!     ipu            Starting position of velocity vector in coeffarrlc
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     iuserlc        ?
!     jelp           Choice parameter to distinguish between the types of
!                    element subroutines to be called. Possible values:
!                     1:  Subroutine ELP610  (Linear line element)
!                     2:  Subroutine ELP611  (Quadratic line element)
!                     3:  Subroutine ELP620  (Linear triangle)
!                     4:  Subroutine ELP622  (Quadratic triangle)
!                     5:  Subroutine ELP621  (Bilinear quadrilateral)
!                     6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                    11:  Subroutine ELP630  (Linear tetrahedron)
!                    12:  Subroutine ELP631  (Quadratic tetrahedron)
!                    13:  Subroutine ELP632  (Trilinear brick)
!                    14:  Subroutine ELP633  (Triquadratic  brick)
!     jheat          Integer parameter indicating if the mass matrix for the
!                    heat equation must be computed
!                    Possible values:
!                     0:  no mass matrix
!                     1:  diagonal mass matrix with constant coefficient
!                     2:  diagonal mass matrix with variable coefficient
!                    11:  full mass matrix with constant coefficient
!                    12:  full mass matrix with variable coefficient
!     kderiv         Help parameter to store value of jderiv
!     kfirst         This parameter indicates the smallest velocity component
!                    that is not equal to zero in the convective terms
!                    For example kfirst = 2, means that the y- &
!                    component of the velocity is unequal to zero
!     ktrans         Help variable to save the value of jtrans
!     metup          Upwind parameter
!     metupc         If 0 the complete upwinding must be taken into account, &
!                    otherwise (1) only the convective terms
!     psiupw         Array of size n*m containing the adapted basis functions
!                    for upwind
!     second         Indicates if second derivative term is important in case of
!                    upwinding
!     tgauss         temperature in Gauss points
!     told           temperature in nodes
!     u              Array to store the 3 velocity components if constant
!     ugauss         Preceding solution in Gauss points
!     uold           Array containing the previous iteration (old solution)
!     wrk1           Work array to be used by some under subroutines
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0813M        Initialize parameters for subroutine ELM813
!     EL2005         Compute a local vector and its derivatives from a known
!                    vector
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELM800MAP      Map velocities, right-hand side and diffusion tensor onto
!                    surface
!     ELM800MASSMAT  Fill element mass matrix for elements of type 800, i.e.
!                    general second order elliptic equation with one degree of
!                    freedom per point
!     ELM800MATRIX   Fill element matrix for elements of type 800, i.e.
!                    general second order elliptic equation with one degree of
!                    freedom per point
!     ELM800RESET    Reset the diffusion tensor, velcoity vector and right-hand
!                    side to their original values
!     ELM800RHSD     Fill element vector for elements of type 800, i.e.
!                    general second order elliptic equation with one degree of
!                    freedom per point
!     ELM800UPWBASE  Compute the modified basis functions psi due to upwinding
!     ELM800VECSEC   Compute contribution of second derivative of function to
!                    right-hand side vector
!     ELMFILL815     Fill velocity vector and rhs vector for typ 815
!     ELMFILLCOEFF   Fill the space dependent coefficients for an element
!                    subroutine
!                    Fill the old vectors in the nodes of the element
!     ELMMULTURHO    Multiply the velocity u by rho in case both are constant
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPELBASEFNCH  Define parameters for choice of basis functions
!     SEPELBASEFUNC  Compute information about basis functions
!     SEPELDEALLOC   Deallocate coefficients and basis functions for element
!                    subroutines
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     1:  Indicates if the velocities and the diffusion tensor must be
!         transformed from 3D to a local lower dimensional system (10)
!         or not (0)
!     2:  Extra information about the convection terms:
!         This information consists of the following parts:
!         metupw + 1000*icompress, &
!         with metuwp information about the upwinding applied
!          and icompress information about the form of the convection term
!         Possible values for metupw:
!         0: no upwinding
!         1: Brooks and Hughes: classical upwind scheme
!         2: Brooks and Hughes: Il'in scheme
!         3: Brooks and Hughes: Doubly assymptotic approximation
!         4: Brooks and Hughes: Critical approximation
!         5: Hughes approximation
!         6: Time-dependent approximation
!         7: Hughes et al: Discontinuity capturing 1
!         8: Hughes et al: Discontinuity capturing 2
!         9: Mizukami triangles satisfying maximum principle
!        10: Mizukami triangles satisfying maximum principle, &
!            activate flip-flop detection
!        11: Mizukami triangles satisfying maximum principle, &
!            use flip-flop detection
!        12: Time-dependent approximation according to Tornberg and Enquist
!       21-32: See metupw-20, however, only the convective term
!            is adapted
!       >99: See metupw-100, however the mass matrix is not
!            weighted by the new basis functions
!         Possible values for icompress:
!         0: The convective term has the shape rho u . grad c
!                                                  - &
!         1: The convective term has the shape div ( rho u c )
!                                                        - &
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-ordinates
!         1: Axi-symmetric co-ordinates
!         2: Polar co-ordinates
!     5:  iseq_temperature
!     6:  alpha 11  (1D/2D/3D)
!     7:  alpha 12  (2D/3D)
!     8:  alpha 13  (3D)
!     9:  alpha 22  (2D/3D)
!    10:  alpha 23  (3D)
!    11:  alpha 33  (3D)
!    12:  u 1       (1D/2D/3D)
!    13:  u 2       (2D/3D)
!    14:  u 3       (3D)
!    15:  beta      (1D/2D/3D)
!    16:  f         (1D/2D/3D)
!    17:  rho cp    (1D/2D/3D)
!    18:  lambda
!    19-20:  Not yet used (must be 0)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!       Differential equation:
!    rho cp d phi/d t - div alpha grad phi +rho cp u . grad phi + beta phi = f
!                                                  - &
!        The following formulae are used:
!           S   =  S      +  S   +  S
!                   alpha     u      beta
!           S     (i,j) = / alpha grad phi  . grad phi   de
!            alpha        e               j           i
!           S     (i,j) = / u . grad phi   phi  de
!            u            e -           j     i
!           S     (i,j) = / beta phi   phi  de
!            beta        e          j     i
!           M (i,j) = / rho cp phi  phi  de
!                     e           i    j
!           F_1(i)    =  /  f phi  de
!                      e       i
!           F_2(i)    =  /  gamma . grad phi_i  de
!                      e
!        Each integral is approximated by a numerical rule of the shape:
!        / int(x) dx = sum  w  int(x )
!        e              k    k      k
!        the factors w  form the weights for the numerical integration
!                     k
!        In the axi-symmetric case they include the factor 2 pi r
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm813' )
      debug = .false.

      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         icheld = 0
         ix = 0
         call el0813m ( E%iuser, E%user, jelp, kfirst, jheat, isub, &
                        isup, coeffarrlc )
         imapf = 0
         if ( iweigh>=30 ) then
            iweigh = iweigh - 20
            imapf = 1
         end if
         B%rule = irule
         B%cart = jcart
         E%ishape = jelp
         call sepelbasefnch

         metup = mod(metupw,1000)
         metupc= metup/20
         metup = metup-20*metupc

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( E%islold, numold, isub, isup, E%iuser )

         ipalpha  = 1
         ipu      = 1+6*m
         ipbeta   = 1+9*m
         ipf      = 1+10*m
         iprho    = 1+11*m
         ipgamma  = 1+12*m
         ipgextra = 1+15*m
         ipphixicp = 1+20*m  ! work array to store copy of phixi
                              ! of length 3*27*27
         ipend =  ipphixicp+ndim*n*m
         if ( ipend>12285 ) then

!        --- array wrk1 too small

            call errint ( ipend, 1 )
            call errint ( 12285, 2 )
            call errchr ( 'coeffarrlc', 1 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if

!        --- Store value of velocity in u and multiply u in coeffarrlc by rho
!            This is only done for constant velocity components

         call elmmulturho ( coeffarrlc(ipu), coeffarrlc(iprho), ind(7), &
                            kstep, ndim, m, u, 1, ind(12) )

         f = coeffarrlc(ipf)

      end if
      if ( ierror/=0 ) go to 1000

!     --- compute basis functions and weights for numerical integration

      call sepelbasefunc
      if ( ierror/=0 ) go to 1000

!     --- Fill variable coefficients if necessary
!         First fill uold and dudx if necessary
!         Fill array vecloc with the values of all arrays stored in vecold
!         in the integration points depending on ind(lengthmst)

      call elmfillcoeff ( uold, dudx, B%phi, B%phider, E%nodes, ugauss, iseqin, &
                          vecloc, wrk1, isub, isup, E%user, coeffarrlc, &
                          numold, E%iuser, maxunk, B%xyz, B%xyzg )
      if ( ierror/=0 ) go to 1000

      if ( itype==815 ) then

!     --- itype = 815, fill velocity and rhs vector

         iuserlc = 0
         call elmfill815 ( uold, dudx, coeffarrlc(ipu), coeffarrlc(ipf), &
                           alpha, iuserlc, E%user )

      end if  ! ( itype==815 )

      if ( ind(12)/=0 ) then

!     --- rho cp has been given, multiply velocity by rho cp

         call elmmulturho ( coeffarrlc(ipu), coeffarrlc(iprho), ind(7), &
                            kstep, ndim, m, u, 2, ind(12) )

      end if  ! ( ind(12)/=0 )

!     --- Perform some special actions on coefficients for special
!         element types

      ktrans = jtrans
      kderiv = jderiv
      jtrans = 0
      jderiv = 1
      call el2005 ( told, dtdx, B%phi, B%phider, &
                    E%nodes, tgauss, iseqtemp )
      jtrans = ktrans
      jderiv = kderiv

      if ( iweigh>=10 ) then

!     --- iweigh>=10, transformation of coefficients onto surface

         call elm800map ( coeffarrlc(ipalpha), imapf, B%type_basisfunc, B%qmat, ah )

      end if  ! ( iweigh>=10 )

      if ( ierror/=0 ) go to 1000

!     --- in case of upwinding compute modified basis functions

      second = .false.
      if ( metup>0 ) then

!     --- Upwinding is applied; compute modified basis functions

         call elm800upwbase ( metup, B%type_basisfunc, coeffarrlc(ipalpha), &
                              coeffarrlc(ipu), B%phi, psiupw, &
                              B%xyz, second, B%phider, dudx, iflipflop, &
                              dpsidx, B%w )
         if(debug)then
            write(irefwr,*) 'n,m,jelp ',n,m,B%type_basisfunc
            call prinrl(psiupw,n*m,'psiupw')
         end if

      end if

      if ( ierror/=0 ) go to 1000

!     --- Fill parts of matrix if necessary

      if ( matrix ) then

!     --- matrix = true    compute element matrix

         call elm800matrix ( B%type_basisfunc, coeffarrlc(ipalpha), E%mat, B%w, &
                             B%phider, psiupw, B%phi, coor, &
                             E%nodes, E%iuser, E%user, &
                             vecloc, second, &
                             coeffarrlc(ipphixicp), B%xyz, B%xyzg, &
                             symm, metup, coeffarrlc(ipu), kfirst, metupc, &
                             coeffarrlc(ipbeta), 0, dudx, &
                             idefcr, ndim )

      end if
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,*) 'symm', symm

!     --- Fill element vector if necessary

      if ( vector .and. notvec/=1 ) then

!     --- vector = true, compute element vector

         call elm800rhsd ( 0, wrk1, dudx, coeffarrlc(ipu), &
                           coeffarrlc(ipgextra), coeffarrlc(ipf), ndim, &
                           E%vec, metupc, metup, B%phi, &
                           coeffarrlc(ipalpha), idefcr, B%phider, &
                           psiupw, ishape, B%qmat, B%w, &
                           coeffarrlc(ipgamma) )

!        --- add extra right-hand side part due to d^2T^dx^2

         call elm800vecsec ( E%vec, coeffarrlc(ipgamma), B%phider, &
                             B%w, dtdx )

      end if

      if ( ierror/=0 ) go to 1000

!     --- Fill mass matrix if necessary

      if ( notmas==0 ) then

!     --- notmas = 0,  compute mass matrix

         call elm800massmat ( jheat, metup, metupc, &
                              coeffarrlc(iprho), B%phi, &
                              psiupw, E%mmat, wrk1, B%w )

      end if  ! ( notmas==0 )

      if ( iweigh>=10 ) then

!     --- Reset coefficients to original values

         call elm800reset ( coeffarrlc(ipalpha), coeffarrlc(ipu), u, &
                            coeffarrlc(ipf), f, ah )

      end if

      if ( E%last==1 ) then
         call sepeldealloc
      end if  ! ( E%last==1 )

1000  if ( debug ) then

!     --- debug = true

         write(irefwr,*) 'element ', ielem, '  itype ',itype
         write(irefwr,*) 'jheat ', jheat
         call prinrl(E%vec,icount,'elemvc')
         call prinrl1(E%mat,icount,icount,'elemmt')
         if ( notmas==0 ) then
            if ( imas==1 ) then
               call prinrl(E%mmat,icount,'elemms')
            else
               call prinrl1(E%mmat,icount,icount,'elemms')
            end if  ! ( imas==1 )
         end if  ! ( notmas==0 )

      end if
      call erclos ( 'elm813' )

      end
