      subroutine elm814rhsd ( dudx, elemvc, w, dphidx )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 16-12-2010
!
!   copyright (c) 2010-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix for type 814
!     minimal surface problem (Only in case of Newton)
! **********************************************************************
!
!                       KEYWORDS
!
!     minimal_surface
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision dudx(m,ndim), elemvc(n), w(m), dphidx(n,m,ndim)

!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                              d phi_i/ dx_j(xg_k) = dphidx (i,k,j);
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dudx           i    Real m x ndim  array in which the values of the
!                         derivatives of the solution vector in the integration
!                         points are stored in the sequence:
!                         du / dx  (x ) = dudx ( k, j )
!                           j     k
!                         If jdercn = 0 in common block celint, then only
!                         dudx (1,*) is filled because dudx is then a constant
!                         per element
!                         dudx is only filled if jderiv>0
!     elemvc         o    Element vector
!     w              i    array of length m containing the weights for
!                         integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k
      double precision alpha(m), factor, addsum

!     addsum         Help parameter to compute a sum
!     alpha          work array to store coefficients
!     factor         Multiplication factor
!     i              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The right-hand side in case of Newton is given in elm814matrix
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm814rhsd' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm814rhsd'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill coefficient alpha

      if ( jdercn==0 ) then

!     --- Derivatives constant

         alpha(1) = 1d0/sqrt(1d0+dudx(1,1)**2+dudx(1,2)**2)

      else

!     --- Derivatives variable

         alpha(1:m) = 1d0/sqrt(1d0+dudx(1:m,1)**2+dudx(1:m,2)**2)

      end if  ! ( jdercn==0 )

      if ( jdercn==0 ) then

!     --- Derivatives constant

         factor = -sum(w)*alpha(1)

         do i = 1, n

            elemvc(i) = factor*(dudx(1,1)*dphidx(i,1,1) + &
                                dudx(1,2)*dphidx(i,1,2) )

         end do  ! i = 1, n

      else

!     --- Derivatives variable

         do i = 1, n

            addsum = 0d0

            do k = 1, m
               addsum = addsum+w(k)*alpha(k)* &
                        (dudx(k,1)*dphidx(i,k,1) + &
                         dudx(k,2)*dphidx(i,k,2) )
            end do  ! k = 1, m

            elemvc(i) = -addsum

         end do  ! i = 1, n

      end if  ! ( jdercn==0 )

1000  call erclos ( 'elm814rhsd' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elm814rhsd'

      end if  ! ( debug )

      end
