      subroutine elm815 ( vecloc, coeffarrlc, iflipflop )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.0  date 12-08-2013 Extra parameters
!        version  1.2  date 12-12-2012 Long integers
!
!   copyright (c) 2012-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for second order elliptic equations
!     and related heat equation
!     One, two and three dimensional linear elements
!     Type 815: make phi a distance function
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision vecloc(*), coeffarrlc(m,*)
      integer iflipflop

!     coeffarrlc            Real work array which must be kept for each element
!                         group
!                         work consists of the following parts
!                         Starting address    length   Name:
!                         array contains the folowing contents:
!                         array in which the values of the coefficients
!                         in the integration points are stored in the sequence:
!                               1D                  2D            3D
!                          1-6:  not used
!                          7:   u 1              u 1            u 1
!                          8:                    u 2            u 2
!                          9:                                   u 3
!                         10:   beta             beta           beta
!                         11:   f                f              f
!                         12:   rho cp           rho cp         rho cp
!                         Each coefficient requires exact m (is number of
!                         integration points) positions
!     iflipflop     i/o   Work array to prevent flip-flop behaviour in non- &
!                         linear upwind
!                         The status of the element with respect to upwind is
!                         stored in this array
!                         See subroutine elpuplinmax
!     vecloc              Work array in which all old solution vectors for the
!                         integration points are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isub, isup, jelp, kfirst, jheat, metup, metupc, &
              indf, indbeta, indrho, indu, inddiff
      double precision psiupw(inpelm*27), &
                       dudx(3*3*27), dpsidx(3*27*8), &
                       wrk1(27*inpelm+inpelm*7), &
                       uold(27), ugauss(27)
      logical second
      save isub, isup, jelp, kfirst, jheat, &
           metup, metupc, indf, indbeta, indrho, indu, inddiff

!     dpsidx         Array of length n x m x l  containing the derivatives
!                    of the linear basis functions in the sequence:
!                    d psi / dx (xg ) = psix (i,k,1);
!                         i        k
!                    d psi / dy (xg ) = psix (i,k,2);
!                         i        k
!                    d psi / dz (xg ) = psix (i,k,3);
!                         i        k
!     dudx           Real m x ndim  array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du / dx  (x ) = dudx ( k, j )
!                      j     k
!                    If jdercn = 0 in common block celint, then only
!                    dudx (1,*) is filled because dudx is then a constant
!                    per element
!                    dudx is only filled if jderiv>0
!     indbeta        Sequence number of beta in list of coefficients
!     inddiff        Sequence number of diffusion in list of coefficients
!     indf           Sequence number of f in list of coefficients
!     indrho         Sequence number of rho in list of coefficients
!     indu           Sequence number of velocity in list of coefficients
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     jelp           Choice parameter to distinguish between the types of
!                    element subroutines to be called. Possible values:
!                     1:  Subroutine ELP610  (Linear line element)
!                     2:  Subroutine ELP611  (Quadratic line element)
!                     3:  Subroutine ELP620  (Linear triangle)
!                     4:  Subroutine ELP622  (Quadratic triangle)
!                     5:  Subroutine ELP621  (Bilinear quadrilateral)
!                     6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                    11:  Subroutine ELP630  (Linear tetrahedron)
!                    12:  Subroutine ELP631  (Quadratic tetrahedron)
!                    13:  Subroutine ELP632  (Trilinear brick)
!                    14:  Subroutine ELP633  (Triquadratic  brick)
!     jheat          Integer parameter indicating if the mass matrix for the
!                    heat equation must be computed
!                    Possible values:
!                     0:  no mass matrix
!                     1:  diagonal mass matrix with constant coefficient
!                     2:  diagonal mass matrix with variable coefficient
!                    11:  full mass matrix with constant coefficient
!                    12:  full mass matrix with variable coefficient
!     kfirst         This parameter indicates the smallest velocity component
!                    that is not equal to zero in the convective terms
!                    For example kfirst = 2, means that the y- &
!                    component of the velocity is unequal to zero
!     metup          Upwind parameter
!     metupc         If 0 the complete upwinding must be taken into account,
!                    otherwise (1) only the convective terms
!     psiupw         Array of size n*m containing the adapted basis functions
!                    for upwind
!     second         Indicates if second derivative term is important in case of
!                    upwinding
!     ugauss         Preceding solution in Gauss points
!     uold           Array containing the previous iteration (old solution)
!     wrk1           Work array to be used by some under subroutines
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0815M        Initialize parameters for subroutine ELM815
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELM800MASSMAT  Fill element mass matrix for elements of type 800, i.e.
!                    general second order elliptic equation with one degree of
!                    freedom per point
!     ELM800MATRIX   Fill element matrix for elements of type 800, i.e.
!                    general second order elliptic equation with one degree of
!                    freedom per point
!     ELM800RHSD     Fill element vector for elements of type 800, i.e.
!                    general second order elliptic equation with one degree of
!                    freedom per point
!     ELM800UPWBASE  Compute the modified basis functions psi due to upwinding
!     ELMFILL815     Fill velocity vector and rhs vector for typ 815
!     ELMFILLCOEFF   Fill the space dependent coefficients for an element
!                    subroutine
!                    Fill the old vectors in the nodes of the element
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPELBASEFNCH  Define parameters for choice of basis functions
!     SEPELBASEFUNC  Compute information about basis functions
!     SEPELDEALLOC   Deallocate coefficients and basis functions for element
!                    subroutines
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     1:  Not used
!     2:  Extra information about the convection terms:
!         This information consists of the following parts:
!         metupw + 1000*icompress, &
!         with metuwp information about the upwinding applied
!          and icompress information about the form of the convection term
!         Possible values for metupw:
!         0: no upwinding
!         1: Brooks and Hughes: classical upwind scheme
!         2: Brooks and Hughes: Il'in scheme
!         3: Brooks and Hughes: Doubly assymptotic approximation
!         4: Brooks and Hughes: Critical approximation
!         5: Hughes approximation
!         6: Time-dependent approximation
!         7: Hughes et al: Discontinuity capturing 1
!         8: Hughes et al: Discontinuity capturing 2
!         9: Mizukami triangles satisfying maximum principle
!        10: Mizukami triangles satisfying maximum principle,
!            activate flip-flop detection
!        11: Mizukami triangles satisfying maximum principle,
!            use flip-flop detection
!        12: Time-dependent approximation according to Tornberg and Enquist
!       21-32: See metupw-20, however, only the convective term
!            is adapted
!       >99: See metupw-100, however the mass matrix is not
!            weighted by the new basis functions
!         Possible values for icompress:
!         0: The convective term has the shape rho u . grad c
!                                                  - &
!         1: The convective term has the shape div ( rho u c )
!                                                        - &
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-ordinates
!         1: Axi-symmetric co-ordinates
!         2: Polar co-ordinates
!     5:  Number of smoothing steps
!     6:  Refers to diffusion term
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!       Differential equation:
!       d phi/d t + u . grad phi  = f
!
!        The following formulae are used:
!           S   =   S_u
!           S_u (i,j) = /_e u . grad phi_j   phi_i  de
!           with u = sign(phi) grad(phi)/||grad(phi)||
!
!           F(i)    =  /_e  f phi_i  de
!           with f = sign(phi)
!
!        Each integral is approximated by a numerical rule of the shape:
!        /_e int(x) dx = sum_k  w_k  int(x_k)
!        the factors w_k  form the weights for the numerical integration
!        In the axi-symmetric case they include the factor 2 pi r
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm815' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm815'
         write(irefwr,1) 'n, m', n, m
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         icheld = 0
         ix = 0
         call el0815m ( E%iuser, jelp, kfirst, jheat, isub, &
                        isup, coeffarrlc )
         B%rule = irule
         B%cart = jcart
         E%ishape = jelp
         call sepelbasefnch

         metup = mod(metupw,1000)
         metupc= metup/20
         metup = metup-20*metupc

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( E%islold, numold, isub, isup, E%iuser )

         inddiff   = 1
         indu      = 7
         indbeta   = 10
         indf      = 11
         indrho    = 12

      end if
      if ( ierror/=0 ) go to 1000

!     --- compute basis functions and weights for numerical integration

      call sepelbasefunc
      if ( ierror/=0 ) go to 1000

!     --- Fill variable coefficients if necessary
!         First fill uold and dudx if necessary
!         Fill array vecloc with the values of all arrays stored in vecold
!         in the integration points depending on ind(lengthmst)

      call elmfillcoeff ( uold, dudx, B%phi, B%phider, E%nodes, ugauss, iseqin, &
                          vecloc, wrk1, isub, isup, E%user, coeffarrlc, &
                          numold, E%iuser, maxunk, B%xyz, B%xyzg )
      if ( ierror/=0 ) go to 1000

      call elmfill815 ( uold, dudx, coeffarrlc(1,indu), coeffarrlc(1,indf), &
                        coeffarrlc(1,inddiff), E%iuser, E%user )

!     --- in case of upwinding compute modified basis functions

      second = .false.
      if ( metup>0 ) then

!     --- Upwinding is applied; compute modified basis functions

         call elm800upwbase ( metup, B%type_basisfunc, coeffarrlc, &
                              coeffarrlc(1,indu), B%phi, psiupw, &
                              B%xyz, second, B%phider, dudx, iflipflop, &
                              dpsidx, B%w )
         if(debug)then
            write(irefwr,*) 'n,m,jelp ',n,m,B%type_basisfunc
            call prinrl(psiupw,n*m,'psiupw')
         end if

      end if

      if ( ierror/=0 ) go to 1000

!     --- Fill parts of matrix if necessary

      if ( matrix ) then

!     --- matrix = true    compute element matrix

         call elm800matrix ( B%type_basisfunc, coeffarrlc, E%mat, B%w, &
                             B%phider, psiupw, B%phi, coor, &
                             E%nodes, E%iuser, E%user, vecloc, second, &
                             wrk1, B%xyz, B%xyzg, symm, metup, &
                             coeffarrlc(1,indu), kfirst, metupc, &
                             coeffarrlc(1,indbeta), 0, dudx, &
                             idefcr, ndim )

      end if
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,*) 'symm', symm

!     --- Fill element vector if necessary

      if ( vector .and. notvec/=1 ) then

!     --- vector = true, compute element vector

         call elm800rhsd ( 0, wrk1, dudx, coeffarrlc(1,indu), &
                           coeffarrlc, coeffarrlc(1,indf), &
                           ndim, E%vec, metupc, metup, B%phi, &
                           coeffarrlc, idefcr, B%phider, &
                           psiupw, ishape, B%qmat, B%w, coeffarrlc )

      end if

      if ( ierror/=0 ) go to 1000

!     --- Fill mass matrix if necessary

      if ( notmas==0 ) then

!     --- notmas = 0,  compute mass matrix

         call elm800massmat ( jheat, metup, metupc, &
                              coeffarrlc(1,indrho), B%phi, &
                              psiupw, E%mmat, wrk1, B%w )

      end if  ! ( notmas==0 )

      if ( E%last==1 ) then
         call sepeldealloc
      end if  ! ( E%last==1 )

1000  if ( debug ) then

!     --- debug = true

         write(irefwr,*) 'element ', ielem, '  itype ',itype
         write(irefwr,*) 'jheat ', jheat
         call prinrl(E%vec,icount,'elemvc')
         call prinrl1(E%mat,icount,icount,'elemmt')
         if ( notmas==0 ) then
            if ( imas==1 ) then
               call prinrl(E%mmat,icount,'elemms')
            else
               call prinrl1(E%mmat,icount,icount,'elemms')
            end if  ! ( imas==1 )
         end if  ! ( notmas==0 )
         write(irefwr,*) 'End elm815'

      end if
      call erclos ( 'elm815' )

      end
