      subroutine elm913 ( coor, elemmt, elemvc, iuser, user, index1, &
                          ishape, index4 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.6  date 12-08-2013 Combine dphi/dxi and dphideta
!        version  3.5  date 15-02-2004 New call to elm910basefn
!        version  3.4  date 21-05-2003 Call to elm910basefn
!
!   copyright (c) 1998-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill element matrix and element vector with contribution for given
!    mass flow
!    The option global unknowns is used
!    The element matrix contains only one row, which must be added both to the
!    row and column of the large matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     mass_flow
!     boundary_element
!     element_matrix
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/celwrkf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*), index1(*), ishape, index4(*)
      double precision elemmt(*), elemvc(*), coor(*), user(*)

!     coor       i    array of length ndim x npointcac containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!                     x  = coor (1,i);  y  = coor (2,i);  z  = coor (3,i);
!                      i                 i                 i
!     elemmt     o    Element matrix to be computed
!     elemvc     o    Element vector to be computed
!     index1     i    Array of length inpelm containing the point numbers
!                     of the nodal points in the element
!     index4     i    Array of length inpelm+1 containing the starting
!                     positions of the nodes in array index2 minus 1.
!                     index4(1) = 0, index4(inpelm) = icount
!     ishape     i    Shape number of elements
!     iuser      i    Integer user array to pass user information from
!                     main program to subroutine. See STANDARD PROBLEMS
!     user       i    Real user array to pass user information from
!                     main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, idegfd, nunkloc, ishift, ipphi, ippsi, &
              ipdphidxi, ipdphideta, ipend
      double precision massflux(1), w(9), dpsidx(36*3), tang(27), &
                       xgauss(27), x(27), dphidx(243), qmat(81)
      save massflux, nunkloc, idegfd, ipphi, ippsi, ipdphidxi, &
           ipdphideta

!     dphidx         Array of length n x m x ndim  containing the
!                    derivative
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = dphidx (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = dphidx (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = dphidx (i,k,3);
!                         i        k
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     dpsidx         Array of length n x m x l  containing the derivatives
!                    of the linear basis functions in the sequence:
!                    d psi / dx (xg ) = psix (i,k,1);
!                         i        k
!                    d psi / dy (xg ) = psix (i,k,2);
!                         i        k
!                    d psi / dz (xg ) = psix (i,k,3);
!                         i        k
!     i              Counting variable
!     idegfd         Degree of freedom sequence number
!     ipdphideta     Starting address of array dphideta
!     ipdphidxi      Starting address of array dphidxi
!     ipend          Last position in array
!     ipphi          Starting address of array phi
!     ippsi          Starting address in array work of psi
!     ishift         Shift in coefficient sequence number
!     massflux       Given mass flux
!     nunkloc        Number of unknowns per point for this local element
!     qmat           tdummy
!     tang           In this array the tangential vector is stored
!                    tang(j,i) contains the j th component of the
!                    tangential vector in the i-th integration point
!     w              array of length m containing the weights for integration
!     x              array containing the co-ordinates of the points
!     xgauss         dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3003         Compute integrals of the shape / f(x) phi (x) dx
!                                                   e         i
!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ELM910BASEFN   Compute the basis functions phi, its derivatives and other
!                    related quantities for various element shapes
!                    (boundary elements)
!     ELUSR3         Extracts information concerning the coefficients from IUSER
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!
!     The sequence is for the given mass-flow element is:
!
!     1:  - &
!     2:  - &
!     3:  - &
!     4:  Type of co-ordinate system
!         0 = Cartesian
!         1 = axi-symmetric
!     5:  Sequence number of degree of freedom that corresponds to normal
!         component
!         If u is the normal component: 1
!         If v is the normal component: 2
!         If w is the normal component: 3
!         Use local co-ordinates if the boundary is not in one of these
!         directions
!     6:  Amount of mass through boundary, i.e. / u.n ds
!     7-10:  - &
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!     550   array length too short
!     838   Penalty parameter is not constant
!    1070   Degree of freedom incorrect
!    1766   Type of integration rule incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The given mass flux relates to an equation of the shape:
!
!    / u.n ds = m, where the integral must be taken over the inflow (or outflow)
!                  boundary and m is a given constant
!
!    This equation is discretized assuming that the normal component is one
!    of the unknowns
!    This results in an extra equation of the shape:  L u = m, &
!    where L is a nx1 matrix and m a scalar
!    This matrix L is stored as extra row in the matrix
!    Also an extra unknown c is introduced, which is considered as a global
!    unknown.
!    This constant c updates the matrix in a symmetric way by an extra
!    column, which is the transpose of the extra row
!    Hence the global system of equations becomes:
!
!    | S  L^T | |u|     |f|
!    | L   0  | |c|  =  |m|
!
!     In this subroutine the part of the element matrix corresponding to
!     one element is computed and if it is the first element, also
!     the right-hand side is computed
!     For all other elements elemvc = 0
!     The contribution in elemmt must be added to both the column and the
!     row of the matrix
!
!    The contribution to the element matrix is zero for the components that are
!    not the normal components and equal to the area of the integral over
!    the basis functions for the normal component
!    This is exactly the weight of the integration in case of a Newton Cotes
!    rule
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm913' )

      ishift = 0
      if ( ifirst==0 ) then

!     --- First element in a group

         call elcomm
         ideriv = 0

!        --- Get information from arrays iuser and user

         ipos = iuser ( ielgrp + 5 )
         if ( ipos<7 ) then
            call errint ( ielgrp+5, 1 )
            call errint ( ipos, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 42, 5, 0, 0 )
         end if
         ipos = ipos + 2
         ishift = ishift+2
         irule = iuser(ipos)
         if ( irule==0 ) irule = 1
         ipos = ipos + 1
         ishift = ishift+1
         if ( irule<1 .or. irule>4 ) then
            call errint ( irule, 1 )
            call errint ( 5, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 1766, 5, 0, 0 )
         end if
         jcart = iuser(ipos)
         ipos = ipos + 1
         ishift = ishift+1
         if ( jcart<0 .or. jcart>1 ) then
            call errint ( jcart, 1 )
            call errint ( 4, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 1766, 5, 0, 0 )
         end if
         jcart = jcart+1
         idegfd = iuser(ipos)
         if ( idegfd==0 ) idegfd = 1
         ipos = ipos + 1
         ishift = ishift+1

!        --- Get mass flux

         m = 1
         ishift = ishift+1
         call elusr3 ( 1, massflux, iuser, user, ishift )
         if ( ind(1)>0 ) then
            call errint ( itype, 1 )
            call errint ( 6, 2 )
            call errint ( ielgrp, 3 )
            call errint ( ielem, 4 )
            call errsub ( 838, 4, 0, 0 )
         end if
         n = inpelm
         m = inpelm
         do i = 1, inpelm
            nunkloc = index4(i+1)-index4(i)
            if ( idegfd<1 .or. idegfd>nunkloc ) then
               call errint ( idegfd, 1 )
               call errint ( 4, 2 )
               call errint ( ielem, 3 )
               call errint ( itype, 4 )
               call errint ( ielgrp, 5 )
               call errsub ( 1070, 5, 0, 0 )
            end if
         end do
         if ( irule>1 ) then

!        --- irule>1, Gauss rule

            igausp = 1
            iphi = 1
            jdiag = 2
            if ( ishape<=2 ) then
               if ( ishape==2 .and. irule==2 ) irule = 3
               m = irule - 1
            else if ( ishape==3 .or. ishape==5 ) then
               if ( irule==2 ) then
                  m = 1
               else
                  m = 4
               end if
            else if ( ishape==4 ) then
               if ( irule==2 ) then
                  m = 4
               else
                  m = 7
               end if
            else if ( ishape==6 ) then
               if ( irule==2 ) then
                  m = 4
               else
                  m = 9
               end if
            end if
         end if
         ipphi = 1+m
         ippsi = ipphi+n*m
         ipdphidxi = ippsi+n*m
         ipdphideta = ipdphidxi+n*m*3
         ipend = ipdphideta+n*m*3

         if ( ipend>1500 ) then

!        --- Array work too small

            call errint ( ipend, 1 )
            call errint ( 1500, 2 )
            call errchr ( 'work', 1 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if

      end if

!     --- Compute basis functions and related quantities

      call elm910basefn ( coor, x, w, dphidx, xgauss, work(ipphi), &
                          index1, qmat, tang, ishape, work(ippsi), &
                          dpsidx, work(ipdphidxi) )
      if ( ierror/=0 ) go to 1000

!     --- Compute the element matrix and vector

      do i = 1, icount
         elemmt(i) = 0d0
      end do

      if ( irule==1 ) then

!     --- Newton Cotes rule

         do i = 1, m
            elemmt(index4(i)+idegfd) = w(i)
         end do

      else

!     --- Gauss rule

         jconsf = 2
         jadd = 0
         do i = 1, m
            qmat(i) = 1d0
         end do
         call el3003 ( qmat, work, w, work(ipphi), m, n )
         do i = 1, inpelm
            elemmt(index4(i)+idegfd) = work(i)
         end do

      end if

      if ( irelem==1 ) then
         elemvc(1) = massflux(1)
      else
         elemvc(1) = 0d0
      end if

1000  call erclos ( 'elm913' )

      end
