      subroutine elm921 ( coor2d, elemmt, index1struct, index1fluid, &
                          inforelement, xgauss, igaussp, weight, &
                          philambda, nlambda )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.2  date 14-03-2003 Extension with extended quadr.
!                                      tetrahedrons
!        version  2.1  date 20-11-2002 Extension with extended quadr. triangles
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute element matrix for fictitious domain method
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     element_group
!     fictitious_domain
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1struct(*), index1fluid(*), nlambda, &
              inforelement(2,*), igaussp
      double precision elemmt(nlambda*ndimlc,*), coor2d(ndim,*), &
                       xgauss(m,3), weight, philambda(nlambda,m)

!     coor2d           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     elemmt         o    Element matrix
!     igaussp        i    sequence number of Gauss point
!     index1fluid    i    Contains the node numbers of the fluid element
!     index1struct   i    Contains the node numbers of the structural element
!     inforelement   i    Array of size 2 x 2 containing the following
!                         information
!                         (1,.) refers to the structural element
!                         (2,.) refers to the fluid element
!                         (.,1) contains the shape number of the element
!                         (.,2) contains the number of points in the element
!     nlambda        i    number of lambda points in the lambda element
!     philambda      i    Contains the values of the lambda basis function
!                         in the gauss point
!     weight         i    Contains the weight for the integration
!     xgauss         i    contains the coordinates of the specific Gauss point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, inpelmfluid, inpelmstruct, ishapestruct, ishapefluid
      double precision sloc(12,54), phistruct(27), phifluid(27), wh, &
                       eta0, xsi0, zeta0, xcoord, ycoord, zcoord

!     eta0           Value of eta in previous iteration (dummy)
!     i              Counting variable
!     inpelmfluid    number of points per fluid element
!     inpelmstruct   number of points per structural element
!     ishapefluid    shape number for fluid element
!     ishapestruct   shape number for structural element
!     j              Counting variable
!     phifluid       Contains values of basis functions for fluid element
!     phistruct      Contains values of basis functions for structural element
!     sloc           Local element matrix
!     wh             multiplication factor
!     xcoord         x-coordinate of current Gauss point
!     xsi0           Value of xsi in previous iteration (dummy)
!     ycoord         x-coordinate of current Gauss point
!     zcoord         x-coordinate of current Gauss point
!     zeta0          Value of zeta in previous iteration
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELP620INTPOL   Compute the values of the basis functions in a given
!                    point for a linear triangle
!     ELP621INTPOL   Compute the values of the basis functions in a given
!                    point for a bi-linear quadrilateral
!     ELP622INTPOL   Compute the values of the basis functions in a given
!                    point for a quadratic triangle
!     ELP623INTPOL   Compute the values of the basis functions in a given
!                    point for a bi-quadratic quadrilateral
!     ELP624INTPOL   Compute the values of the basis function in a specific
!                    point for an extended quadratic triangle
!     ELP630INTPOL   Compute the values of the basis functions in a given
!                    point for a linear tetrahedron
!     ELP631INTPOL   Compute the values of the basis functions in a given
!                    point for a quadratic tetrahedron
!     ELP632INTPOL   Compute the values of the basis functions in a given
!                    point for a tri-linear hexahedron
!     ELP633INTPOL   Compute the values of the basis functions in a given
!                    point for a tri-quadratic hexahedron
!     ELP637INTPOL   Compute the values of the basis functions in a given
!                    point for an extended quadratic tetrahedron
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2187   Structural elements of this shape have not yet been implemented
!    2188   Fluid elements of this shape have not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!     In the ficitious domain method the velocity of the fluid is made equal
!     to the velocity of the structural element
!     This is done by introducing a Lagrangian multiplier lambda
!     Let the shape functions of the Lagrangian multiplier e phi_lambda
!     Then we have the following equality:
!
!     /_gamma phi_lambda (u_struct - u_fluid) d Gamma = 0
!
!     Gamma is the boundary of the structural element
!     u_struct is the structural velocity and u_fluid the fluid velocity
!
!     The integral is approximated by a Gaussian integration rule:
!
!     sum w(k) phi_lambda(x_k) (u_struct - u_fluid)(x_k)
!
!     and for u_struct and u_fluid we take the approximation per element
!
!     In this subroutine we evaluate the integral for one specific Gauss point, &
!     but for all lambda's in the element
!
!     We get an element matrix of size n_lambda x ( n_fluid+n_struct)
!     with n_lamda the number of lambda's in the boundary element, &
!     n_fluid the number of fluid unknowns in a fluid element and
!     n_struct the number of structural unknowns in a structural element
!
!     The elements of the matrix are equal to:
!
!     s(i,j) = w phi_lambda(i) phi_struct_j(xgauss) (j=1(1)n_struct)
!     and
!     s(i,j) =-w phi_lambda(i) phi_fluid_(j-n_struct)(xgauss) (j=1(1)n_fluid)
!
!     w is a scalar (the weight in the integration point)
!     phi_lambda is an array of size n_lamda containing the values of the
!     lambda basis functions in the integration point
!     phi_fluid is the basis function for the specific fluid point
!     and phi_struct for the structural point
!
!     Actually the notation above is for the case of one unknown per point
!     Since the number of unknowns per point is ndim the total matrix has the
!     following shape for 2d
!
!         s(1,1)  0    s(1,2)   0    s(1,3)    0, ...
!            0  s(1,1)   0    s(1,2)   0    s(1,3), ...
!         s(2,1)  0    s(2,2)   0    s(2,3)    0, ...
!           ......
!
!     In this subroutine w, phi_lamda and xgauss are input parameters
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm921' )

!     --- First compute the value of the basis functions for the structural
!         element in the Gauss point

      ishapestruct = inforelement(1,1)
      inpelmstruct = inforelement(1,2)
      if ( ndim.eq.2 ) then

!     --- Store coordinates of Gauss point in (xcoord,ycoord)

         xcoord = xgauss(igaussp,1)
         ycoord = xgauss(igaussp,2)

!        --- two dimensional elements

         if ( ishapestruct.lt.3 .or. ishapestruct.gt.6 ) then

!        --- Structural element not yet implemented

            call errint ( ishapestruct, 1 )
            call errsub ( 2187, 1, 0, 0 )
            go to 1000

         end if
         go to ( 130, 140, 150, 160 ), ishapestruct-2

!        --- ishapestruct = 3, linear triangle

130      call elp620intpol ( coor2d, index1struct, phistruct, &
                             xcoord, ycoord )
         go to 200

!        --- ishapestruct = 4, quadratic triangle

140      call elp622intpol ( coor2d, index1struct, phistruct, &
                             xcoord, ycoord )
         go to 200

!        --- ishapestruct = 5, bi-linear quadrilateral

150      call elp621intpol ( coor2d, index1struct, phistruct, &
                             xcoord, ycoord, &
                             xsi0, eta0, .true. )
         go to 200

!        --- ishapestruct = 6, bi-quadratic quadrilateral

160      call elp623intpol ( coor2d, index1struct, phistruct, &
                             xcoord, ycoord )
         go to 200

!        --- Fluid part

200      ishapefluid = inforelement(2,1)
         inpelmfluid = inforelement(2,2)

         if ( ishapefluid.lt.3 .or. ishapefluid.gt.10 .or. &
              ishapefluid.eq.8 ) then

!        --- fluid element not yet implemented

            call errint ( ishapefluid, 1 )
            call errsub ( 2188, 1, 0, 0 )
            go to 1000

         end if
         go to ( 230, 240, 250, 260, 270,1000, 250, 230 ), ishapefluid-2

!        --- ishapefluid = 3, linear triangle

230      call elp620intpol ( coor2d, index1fluid, phifluid, &
                             xcoord, ycoord )
         go to 500

!        --- ishapefluid = 4, quadratic triangle

240      call elp622intpol ( coor2d, index1fluid, phifluid, &
                             xcoord, ycoord )
         go to 500

!        --- ishapefluid = 5, bi-linear quadrilateral

250      call elp621intpol ( coor2d, index1fluid, phifluid, &
                             xcoord, ycoord, &
                             xsi0, eta0, .true. )
         go to 500

!        --- ishapefluid = 6, bi-quadratic quadrilateral

260      call elp623intpol ( coor2d, index1fluid, phifluid, &
                             xcoord, ycoord )
         go to 500

!        --- ishapefluid = 7, extended quadratic triangle

270      call elp624intpol ( coor2d, index1fluid, phifluid, &
                             xcoord, ycoord, xsi0, eta0 )
         go to 500

      else

!     --- Store coordinates of Gauss point in (xcoord,ycoord,zcoord)

         xcoord = xgauss(igaussp,1)
         ycoord = xgauss(igaussp,2)
         zcoord = xgauss(igaussp,3)

!        --- three dimensional elements

         if ( ishapestruct.lt.11 .or. ishapestruct.gt.14 .or. &
              ishapestruct.eq.12 ) then

!        --- Structural element not yet implemented

            call errint ( ishapestruct, 1 )
            call errsub ( 2187, 1, 0, 0 )
            go to 1000

         end if
         go to ( 330, 340, 350, 360, 1000, 370 ), ishapestruct-10

!        --- ishapestruct = 11, linear tetrahedron

330      call elp630intpol ( coor2d, index1struct, phistruct, &
                             xcoord, ycoord, zcoord )
         go to 400

!        --- ishapestruct = 12, quadratic tetrahedron

340      call elp631intpol ( coor2d, index1struct, phistruct, &
                             xcoord, ycoord, zcoord )
         go to 400

!        --- ishapestruct = 13, tri-linear quadrilateral

350      call elp632intpol ( coor2d, index1struct, phistruct, &
                             xcoord, ycoord, zcoord, &
                             xsi0, eta0, zeta0 )
         go to 400

!        --- ishapestruct = 14, tri-quadratic quadrilateral

360      call elp633intpol ( coor2d, index1struct, phistruct, &
                             xcoord, ycoord, zcoord )
         go to 400

!        --- ishapestruct = 16, extended tri-quadratic tetrahedron

370      call elp637intpol ( coor2d, index1struct, phistruct, &
                             xcoord, ycoord, zcoord, &
                             xsi0, eta0, zeta0 )
         go to 400

!        --- Fluid part

400      ishapefluid = inforelement(2,1)
         inpelmfluid = inforelement(2,2)

         if ( ishapefluid.lt.11 .or. ishapefluid.eq.15 .or. &
              ishapefluid.eq.12 ) then

!        --- fluid element not yet implemented

            call errint ( ishapefluid, 1 )
            call errsub ( 2188, 1, 0, 0 )
            go to 1000

         end if
         go to ( 430, 440, 450, 460, 1000, 470 ), ishapefluid-10

!        --- ishapefluid = 11, linear tetrahedron

430      call elp630intpol ( coor2d, index1fluid, phifluid, &
                             xcoord, ycoord, zcoord )
         go to 500

!        --- ishapefluid = 12, quadratic tetrahedron

440      call elp631intpol ( coor2d, index1fluid, phifluid, &
                             xcoord, ycoord, zcoord )
         go to 500

!        --- ishapefluid = 13, tri-linear quadrilateral

450      call elp632intpol ( coor2d, index1fluid, phifluid, &
                             xcoord, ycoord, zcoord, &
                             xsi0, eta0, zeta0 )
         go to 500

!        --- ishapefluid = 14, tri-quadratic quadrilateral

460      call elp633intpol ( coor2d, index1fluid, phifluid, &
                             xcoord, ycoord, zcoord )
         go to 500

!        --- ishapefluid = 16, extended tri-quadratic tetrahedron

470      call elp637intpol ( coor2d, index1fluid, phifluid, &
                             xcoord, ycoord, zcoord, &
                             xsi0, eta0, zeta0 )
         go to 500

      end if

!     --- Compute element matrix, first the local one

500   do i = 1, nlambda
         wh = weight*philambda(i,igaussp)
         do j = 1, inpelmstruct
            sloc(i,j) = wh*phistruct(j)
         end do
         do j = 1, inpelmfluid
            sloc(i,j+inpelmstruct) = -wh*phifluid(j)
         end do
      end do

!     --- Finally the complete matrix
!         Is is assumed that the element matrix has been cleared before

      if ( ndimlc.eq.1 ) then

!     --- ndim = 1

         do i = 1, nlambda
            do j = 1, inpelmstruct+inpelmfluid
               elemmt(i,j) = sloc(i,j)
            end do
         end do

      else if ( ndimlc.eq.2 ) then

!     --- ndim = 2

         do i = 1, nlambda
            do j = 1, inpelmstruct+inpelmfluid
               elemmt(2*i-1,2*j-1) = sloc(i,j)
               elemmt(2*i,2*j) = sloc(i,j)
            end do
         end do

      else

!     --- ndim = 3

         do i = 1, nlambda
            do j = 1, inpelmstruct+inpelmfluid
               elemmt(3*i-2,3*j-2) = sloc(i,j)
               elemmt(3*i-1,3*j-1) = sloc(i,j)
               elemmt(3*i,3*j) = sloc(i,j)
            end do
         end do

      end if
1000  call erclos ( 'elm921' )

      end
