      subroutine elmfill815 ( uold, dudx, uvel, frhs, alpha, iuser, &
                              user )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.2  date 31-07-2012 Extra diffusion and smoothing
!        version  1.1  date 22-03-2012 Correction normgradphi
!
!   copyright (c) 2012-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill right-hand side and velocity vector as function of
!     phi and gradphi
!     Part of computation of distance function
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*)
      double precision uold(n), dudx(m,ndim), uvel(m,ndim), frhs(m), &
                       alpha(m,6), user(*)

!     alpha          o    array of length m x 3 in which the values of
!                         alpha  , alpha  , alpha
!                              11       12       22
!                         are stored in the sequence:
!                                                 k
!                         alpha ( k,i ) = alpha (x )   i = 1(1)3, k=1(1)m
!                                              i
!     dudx           i    Real m x ndim  array in which the values of the
!                         derivatives of the solution vector in the integration
!                         points are stored in the sequence:
!                         du / dx  (x ) = dudx ( k, j )
!                           j     k
!                         If jdercn = 0 in common block celint, then only
!                         dudx (1,*) is filled because dudx is then a constant
!                         per element
!                         dudx is only filled if jderiv>0
!     frhs           o    Contains the rhs vector in the integration points
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     uold           i    Array containing the previous iteration (old solution)
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     uvel           o    Contains the velocity vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision normgradphi, signphi, eps

!     eps            Diffusion parameter
!     normgradphi    Norm of gradient of phi
!     signphi        sign of phi in element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elmfill815' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elmfill815'
         write(irefwr,1) 'n, m, ndim', n, m, ndim
         call prinrl ( uold, n, 'uold' )
         call prinrl1 ( dudx, ndim, m, 'dudx' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iuser(5)>0 ) then

!     --- iuser(5)>0, no convection and rhs

         uvel = 0d0
         frhs = 0d0

      else

!     --- iuser(5)=0, standard case

         if ( minval(uold)<0d0 ) then

!        --- phi<0

            signphi = -1d0

         else

!        --- phi>0

            signphi = 1d0

         end if  ! ( minval(uold)<0d0 )
         normgradphi = sqrt(dot_product(dudx(1,:),dudx(1,:)))

         if ( normgradphi>1d-10 ) then
            normgradphi = signphi/normgradphi
         else
            signphi = 0d0
            normgradphi = 0d0
         end if  ! ( normgradphi>1d-10 )

         uvel(1:m,1:ndim) = normgradphi*dudx(1:m,1:ndim)
         frhs = signphi

      end if  ! ( iuser(5)>0 )

!     --- Set diffusion term

      alpha = 0d0
      eps = 0d0
      if ( iuser(13)>0 ) eps = user(iuser(13))  ! diffusion term
      alpha(:,1) = eps
      alpha(:,4) = eps
      alpha(:,6) = eps

1000  call erclos ( 'elmfill815' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'normgradphi, signphi', normgradphi, signphi
         call prinrl1 ( uvel, ndim, m, 'uvel' )
         write(irefwr,*) 'End elmfill815'

      end if  ! ( debug )

      end
