      subroutine elp3dto2ddelta ( x, dphidxi, dxdxi, delta, index1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 09-08-2013
!
!   copyright (c) 2013-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the jacobian delta and dx/dxi for a 2d element in 3d
!
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     2d
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(n)
      double precision x(n,ndim), dphidxi(n,m,2), dxdxi(m,3,3), delta(m)

!     delta          o    contains the jacobian of the element in the
!                         integration points
!     dphidxi        i    Array containing dphi_i/dxi in the integration points
!     dxdxi          o    array of length mx3x3 to store the values of
!                         dx/dxi, dx/deta, dy/dxi and dy/deta according to:
!                         dx/dxi(x_k) = dxdxi(k,1,1)
!                         dx/deta(x_k) = dxdxi(k,1,2)
!                         dy/dxi(x_k) = dxdxi(k,2,1)
!                         dy/deta(x_k) = dxdxi(k,2,2) etc.
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     x              i    contains the coordinates of the nodes of the element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k

!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     230   Jacobian of element is zero or has changing sign
!    1540   Jacobian zero or changing sign
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp3dto2ddelta' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp3dto2ddelta'
         write(irefwr,1) 'ndim, kstep, m, n', ndim, kstep, m, n
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do j = 1, ndim
         do i = kstep, m, kstep
            dxdxi(i,j,1) = dot_product ( x(1:n,j), dphidxi(1:n,i,1) )
            dxdxi(i,j,2) = dot_product ( x(1:n,j), dphidxi(1:n,i,2) )
         end do
      end do

!     --- fill jacobian in delta

      do i = 1, m
         dxdxi(i,1,3) = dxdxi(i,2,1)*dxdxi(i,3,2)- &
                        dxdxi(i,3,1)*dxdxi(i,2,2)
         dxdxi(i,2,3) = dxdxi(i,3,1)*dxdxi(i,1,2)- &
                        dxdxi(i,1,1)*dxdxi(i,3,2)
         dxdxi(i,3,3) = dxdxi(i,1,1)*dxdxi(i,2,2)- &
                        dxdxi(i,2,1)*dxdxi(i,1,2)
      end do

      do k = kstep, m, kstep
         delta(k) = sqrt(dot_product (dxdxi(k,:,3),dxdxi(k,:,3)))
         if ( delta(k)<=0d0 ) then
            if ( isubr==0 ) then
               call errint ( ielem, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 230, 3, 0, 0 )
            else
               call errint ( ielgrp, 1 )
               call errint ( ielem, 2 )
               call errchr ( 'surface', 1 )
               call errsub ( 1540, 2, 0, 1 )
            end if
            call prinin(index1,n,'node numbers')
            call prinrl2(x,n,3,'co-ordinates')
            go to 1000
         end if
      end do

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( delta, m, 'delta' )
         write(irefwr,*) 'End elp3dto2ddelta'

      end if  ! ( debug )
      call erclos ( 'elp3dto2ddelta' )

      end
