      subroutine elp3dto2dderiv ( dxdxi, delta, dphidx, dphidxi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 09-08-2013
!
!   copyright (c) 2013-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the derivatives dphi/dx for a 2d element in 3d
!
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     2d
!     3d
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision dphidx(n,m,ndim), dphidxi(n,m,2), &
                       dxdxi(m,3,3), delta(m)

!     delta          i    contains the jacobian of the element in the
!                         integration points
!     dphidx         o    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                              d phi_i/ dx_j(xg_k) = dphidx (i,k,j);
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dphidxi        i    Array containing dphi_i/dxi in the integration points
!     dxdxi          i    array of length mx3x3 to store the values of
!                         dx/dxi, dx/deta, dy/dxi and dy/deta according to:
!                         dx/dxi(x_k) = dxdxi(k,1,1)
!                         dx/deta(x_k) = dxdxi(k,1,2)
!                         dy/dxi(x_k) = dxdxi(k,2,1)
!                         dy/deta(x_k) = dxdxi(k,2,2) etc.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision dxidx(m,2,3)

!     dxidx          Contains the derivatives d xi/ dx
!                    dxidx(k,i,j) = dxi_i/dx_j (x^k)
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp3dto2dderiv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp3dto2dderiv'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do k = kstep, m, kstep
         dxdxi(k,:,3) = dxdxi(k,:,3)/delta(k)
      end do

      do k = kstep, m, kstep
         dxidx(k,1,1) = (dxdxi(k,2,2)*dxdxi(k,3,3) - &
                         dxdxi(k,2,3)*dxdxi(k,3,2))/delta(k)
         dxidx(k,1,2) = (dxdxi(k,2,3)*dxdxi(k,3,1) - &
                         dxdxi(k,2,1)*dxdxi(k,3,3))/delta(k)
         dxidx(k,1,3) = (dxdxi(k,2,1)*dxdxi(k,3,2) - &
                         dxdxi(k,2,2)*dxdxi(k,3,1))/delta(k)
         dxidx(k,2,1) = (dxdxi(k,3,2)*dxdxi(k,1,3) - &
                         dxdxi(k,3,3)*dxdxi(k,1,2))/delta(k)
         dxidx(k,2,2) = (dxdxi(k,3,3)*dxdxi(k,1,1) - &
                         dxdxi(k,3,1)*dxdxi(k,1,3))/delta(k)
         dxidx(k,2,3) = (dxdxi(k,3,1)*dxdxi(k,1,2) - &
                         dxdxi(k,3,2)*dxdxi(k,1,1))/delta(k)
      end do

      do j = 1, 3
         do k = kstep, m, kstep
            do i = 1, n
               dphidx(i,k,j) = dphidxi(i,k,1)*dxidx(k,1,j) + &
                               dphidxi(i,k,2)*dxidx(k,2,j)
            end do
         end do
      end do

1000  call erclos ( 'elp3dto2dderiv' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp3dto2dderiv'

      end if  ! ( debug )

      end
