      subroutine elp621jacob ( x, wh, isubr, index1, &
                               xg, intrule, dxidx, kderiv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-07-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the Jacobian of a quadrilateral
!     Also the factors e^ij, necessary for the derivatives are computed
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     determinant
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isubr, index1(n), intrule, kderiv
      double precision x(n,2), wh(*), xg(m,2), dxidx(m,2,2)

!     dxidx          o    Contains the derivatives d xi/ dx
!                         dxidx(k,i,j) = dxi_i/dx_j (x^k)
!     kderiv         i    If 1 the derivatives of the basis functions
!                         must be computed
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     intrule        i    Actual integration rule
!     isubr          i    Indicates the type of calling subroutine
!     wh             o    Array of length m containing the values of the
!                         Jacobian in the vertices or for the Gauss integration
!                         rules work array to store some help variables
!     x              i    contains coordinates of nodes of the element
!     xg             i    Array of length mx2 containing the co-ordinates of the
!                         Gauss points in the reference element for a four point
!                         Gauss rule
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k
      double precision a(3,2), h

!     a              Contains the coefficients of the bi-linear polynomial
!                    except the constant part a0
!     h              Help variable to store a constant temporarily
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELP621DET      Compute the Jacobian determinant of the bilinear
!                    quadrilateral
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Let the reference element be given by (0,0), (1,0), (1,1), (0,1)
!     Then (in vector form)
!     x = a_0 + a_1 xsi + a_2 eta + a_3 xsi eta  (a_i = ai,bi)
!     From x_i = xi follows
!     a_0 = x_1
!     a_1 = x_2-x_1
!     a_2 = x_4-x_1
!     a_3 = x_3-(a_0+a_1+a_2) = x_3-x_2-a_2
!     d x / d xsi = a_1 + a_3 eta
!     d x / d eta = a_2 + a_3 xsi
!
!     The Jacobian matrix is given by
!
!     |  x_xsi  x_eta  |
!     |  y_xsi  y_eta  |
!
!     The determinant is given by J = x_xsi y_eta - y_xsi x_eta
!     which is equal to
!        (a1+a3 eta) (b2+b3 xsi) - (a2+a3 xsi) (b1+b3 eta)
!        a1 b2 - a2 b1 + (a1 b3 - a3 b1) xsi + (a3 b2 - a2 b3) eta
!     The inverse of the Jacobian matrix defines d xsi /dx etc by
!
!     |  xsi_x  xsi_y  |        |  y_eta  -x_eta  |
!     |  eta_x  eta_y  | = 1/J  | -y_xsi   x_xsi  |
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp621jacob' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp621jacob'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute the quantities  a_1 until a_3, store in a(1:3,1:2)

      a(1,:) = x(2,:)-x(1,:)
      a(2,:) = x(4,:)-x(1,:)
      a(3,:) = x(3,:)-x(2,:)-a(2,:)

      if ( debug ) call prinrl2 ( a, 3, 2, 'a' )

!     --- Compute the determinant J in the vertices and store in wh
!         a1 b2 - a2 b1 + (a1 b3 - a3 b1) xsi + (a3 b2 - a2 b3) eta

      call elp621det ( isubr, a, wh, index1, x, xg, intrule )

      if ( kderiv==1 ) then

!     --- kderiv = 1, compute dxidx

         if ( intrule==1 ) then

!        --- intrule = 1, only compute dxidx
!            use the fact the xsi and eta are only 1 or 0

            dxidx(1,1,1) = a(2,2)/wh(1)
            dxidx(2:3,1,1) = (a(2,2)+a(3,2))/wh(2:3)
            dxidx(4,1,1) = a(2,2)/wh(4)

            dxidx(1,1,2) = -a(2,1)/wh(1)
            dxidx(2:3,1,2) = -(a(2,1)+a(3,1))/wh(2:3)
            dxidx(4,1,2) = -a(2,1)/wh(4)

            dxidx(1:2,2,1) = -a(1,2)/wh(1:2)
            dxidx(3:4,2,1) = -(a(1,2)+a(3,2))/wh(3:4)

            dxidx(1:2,2,2) = a(1,1)/wh(1:2)
            dxidx(3:4,2,2) = (a(1,1)+a(3,1))/wh(3:4)

         else

!        --- intrule>1, Gauss rule

            do k = 1, m
               h = 1d0 / wh(k)
               dxidx(k,1,1) = (a(2,2)+a(3,2)*xg(k,1))*h  ! dxsi/dx
               dxidx(k,1,2) =-(a(2,1)+a(3,1)*xg(k,1))*h  ! dxsi/dy
               dxidx(k,2,1) =-(a(1,2)+a(3,2)*xg(k,2))*h  ! deta/dx
               dxidx(k,2,2) = (a(1,1)+a(3,1)*xg(k,2))*h  ! deta/dy
            end do  ! k = 1, m

         end if  ! ( intrule==1 )

      end if  ! ( kderiv==1 )

1000  call erclos ( 'elp621jacob' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp621jacob'

      end if  ! ( debug )

      end
