      subroutine elp622straight ( iweigo, x, index1, intrule, dphidx, &
                                  dpsidx, comppsi, delta, lambda )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 27-02-2009
!
!   copyright (c) 2009-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute basis functions and related quantities for quadratic
!     triangle (straight element)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iweigo, index1(6), intrule
      double precision x(6,2), dphidx(6,m,ndim), &
                       dpsidx(3,m,2), delta, lambda(3,m)
      logical comppsi

!     comppsi        i    if true the linear basis functions must be evaluated
!     delta          o    jacobian of element
!     dphidx         o    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!     dpsidx         o    Array of length n x m x l  containing the derivatives
!                         of the linear basis functions in the sequence:
!                         d psi / dx (xg ) = psix (i,k,1);
!                              i        k
!                         d psi / dy (xg ) = psix (i,k,2);
!                              i        k
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     intrule        i    Actual integration rule
!     iweigo         i    if 1 the weights must be filled
!     lambda         i    Array containing the linear basis functions in the
!                        integration points
!     x              o    array of length 9 x ndim containing the x- and
!                         y-coordinates of the nodal points with respect to the
!                         local numbering in the sequence:
!                         x_i = x (i,1);  y_i = x (i,2);
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, j
      double precision e(3,2)

!     e              work array of length 3 x 2, to store the factors
!                     ij
!                    e.        see method
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELP622STRDELTA Fill arrays e and dpsidx as well as Jacobian delta
!                    for a straight quadratic triangle
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!        to compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!         ij        i+1    i+2    i+1
!        e   =  (-1)    ( x    - x    )   cyclic: i=1,2,3 ;   j=1,2
!                          j      j
!                    32  11    12 31
!        delta   =  e   e   - e  e
!                             ij
!        d lambda  / dx   =  e   /  delta
!                i     j
!        phi  =  lambda  (2 lambda - 1)        i=1,2,3   (basis functions:1,2,3)
!           i          i          i
!        phi  = 4  lambda   lambda             1<=i<j<=3 (basis functions:4,5,6)
!           ij           i        j
!        grad phi  = grad lambda  ( 4 lambda  - 1 )
!                i              i           i
!        grad phi  = 4 ( grad lambda  lambda  + grad lambda  lambda  )
!                ij                 i       j              j       i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp622straight' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp622straight'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill array e, dpsidx and delta

      call elp622strdelta ( iweigo, e, x, delta, dpsidx, comppsi, &
                            index1 )

      if ( ideriv==1 ) then

!     --- ideriv = 1   fill derivatives

         select case (intrule)

         case(1)

!        --- intrule  =  1  newton-cotes rule

            do k = 1, 2
               dphidx(1,2,k) =  e(1,k)
               dphidx(1,4,k) = -e(1,k)
               dphidx(1,6,k) =  e(1,k)
               dphidx(3,2,k) =  e(2,k)
               dphidx(3,4,k) =  e(2,k)
               dphidx(3,6,k) = -e(2,k)
               dphidx(5,2,k) = -e(3,k)
               dphidx(5,4,k) =  e(3,k)
               dphidx(5,6,k) =  e(3,k)
               dphidx(2,2,k) = -2d0*e(3,k)
               dphidx(2,4,k) =  2d0*e(1,k)
               dphidx(2,6,k) =  2d0*e(2,k)
               dphidx(4,2,k) =  2d0*e(3,k)
               dphidx(4,4,k) = -2d0*e(1,k)
               dphidx(4,6,k) =  2d0*e(2,k)
               dphidx(6,2,k) =  2d0*e(3,k)
               dphidx(6,4,k) =  2d0*e(1,k)
               dphidx(6,6,k) = -2d0*e(2,k)
            end do

            if ( m>3 ) then

!           --- m = 6:  fill derivatives in vertices

               do k = 1, 2
                  dphidx(1,1,k) = 3d0*e(1,k)
                  dphidx(1,3,k) =    -e(1,k)
                  dphidx(1,5,k) =    -e(1,k)
                  dphidx(3,1,k) =    -e(2,k)
                  dphidx(3,3,k) = 3d0*e(2,k)
                  dphidx(3,5,k) =    -e(2,k)
                  dphidx(5,1,k) =    -e(3,k)
                  dphidx(5,3,k) =    -e(3,k)
                  dphidx(5,5,k) = 3d0*e(3,k)
                  dphidx(2,1,k) = 4d0*e(2,k)
                  dphidx(2,3,k) = 4d0*e(1,k)
                  dphidx(2,5,k) = 0d0
                  dphidx(4,1,k) = 0d0
                  dphidx(4,3,k) = 4d0*e(3,k)
                  dphidx(4,5,k) = 4d0*e(2,k)
                  dphidx(6,1,k) = 4d0*e(3,k)
                  dphidx(6,3,k) = 0d0
                  dphidx(6,5,k) = 4d0*e(1,k)
               end do

            end if

         case(2, 3)

!        --- intrule  =  2  gauss 4-point rule

            do j = 1, 2
               do i = 1, 3
                  do k = 1, m
                     dphidx(2*i-1,k,j) = e(i,j)*(4d0*lambda(i,k)-1d0)
                  end do
               end do
               do k = 1, m
                  dphidx(2,k,j) = 4d0*(lambda(1,k)*e(2,j)+ &
                                  lambda(2,k)*e(1,j))
                  dphidx(4,k,j) = 4d0*(lambda(2,k)*e(3,j)+ &
                                  lambda(3,k)*e(2,j))
                  dphidx(6,k,j) = 4d0*(lambda(3,k)*e(1,j)+ &
                                  lambda(1,k)*e(3,j))
               end do
            end do

         end select  ! case (intrule)

      end if

1000  call erclos ( 'elp622straight' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp622straight'

      end if  ! ( debug )

      end
