      subroutine elp622strdelta ( iweigo, e, x, delta, dpsidx, comppsi, &
                                  index1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 21-08-2013
!
!   copyright (c) 2013-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays e and dpsidx as well as Jacobian delta
!     for a straight quadratic triangle
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmoduledebug
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iweigo, index1(n)
      double precision e(3,2), x(n,2), delta, dpsidx(3,m,2)
      logical comppsi

!     comppsi        i    if true the linear basis functions must be evaluated
!     delta          o    jacobian of element
!     dpsidx         o    Array of length n x m x l  containing the derivatives
!                         of the linear basis functions in the sequence:
!                         d psi / dx (xg ) = psix (i,k,1);
!                              i        k
!                         d psi / dy (xg ) = psix (i,k,2);
!                              i        k
!                         d psi / dz (xg ) = psix (i,k,3);
!                              i        k
!     e              o     work array of length 3 x 2, to store the factors
!                          e^ij
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     iweigo         i    if 1 the weights must be filled
!     x              i    array of length 9 x ndim containing the x- and
!                         y-coordinates of the nodal points with respect to the
!                         local numbering in the sequence:
!                         x_i = x (i,1);  y_i = x (i,2);
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, k

!     j              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     105   Jacobian of element is zero
!    1540   Jacobian zero or changing sign
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp622strdelta' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp622strdelta'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iweigo==1 .or. ideriv==1 ) then

!     --- ideriv = 1  compute derivatives or iweigo = 1: weights
!                                   ij
!         compute the quantities  e   and delta
!                                    ij
!         store d lambda  / d x  in e
!                       i      j

         e(1,1) = x(3,2) - x(5,2)
         e(2,1) = x(5,2) - x(1,2)
         e(3,1) = x(1,2) - x(3,2)
         e(1,2) = x(5,1) - x(3,1)
         e(2,2) = x(1,1) - x(5,1)
         e(3,2) = x(3,1) - x(1,1)

         delta = e(3,1)*e(1,2) - e(1,1)*e(3,2)
         if ( checksign .and. delta<0d0 ) then
            write(irefwr,*) 'area of element ', ielem, ' is negative'
         end if

!        --- check if delta is non-zero

         if ( abs(delta)<=0d0 ) then
            if ( isubr==0 ) then
               call errint ( ielem, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 105, 3, 0, 0 )
            else
               call errint ( ielgrp, 1 )
               call errint ( ielem, 2 )
               call errchr ( 'surface', 1 )
               call errsub ( 1540, 2, 0, 1 )
            end if
            call prinin(index1,6,'node numbers')
            call prinrl2(x,6,2,'co-ordinates')
            go to 1000
         end if

         e = e / delta

      end if

      if ( comppsi .and. ideriv==1 ) then

!     --- ideriv = 1   fill derivatives

         if ( comppsi ) then

!        --- Compute linear basis functions

            do j = 1, m
               do k = 1,2
                  dpsidx(1,j,k) = e(1,k)
                  dpsidx(3,j,k) = e(3,k)
                  dpsidx(2,j,k) = -e(1,k) - e(3,k)
               end do
            end do

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp622strdelta'

      end if  ! ( debug )
      call erclos ( 'elp622strdelta' )

      end
