      subroutine elp623first ( comppsi, xg, dphidxi, &
                               phi, psi, psixi, intrule, weight )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 12-08-2013 New parameter list
!        version  1.0  date 25-02-2009
!
!   copyright (c) 2009-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element independent quantities for a bi-quadratic quadrilateral
!
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
!     quadrilateral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer intrule
      double precision xg(m,2), psixi(4,9,2), weight(m), &
                       phi(9,m), dphidxi(9,9,2), psi(4,m)
      logical comppsi

!     comppsi        o    if true the linear basis functions must be evaluated
!     dphidxi        o    Array containing dphi_i/dxi in a point
!     intrule        i    Actual integration rule
!     phi            o    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     psi            o    Array of length n x m containing the values of the
!                         modified basis functions in the integration points
!     psixi          o    Array of length 9 x m x 2 containing the values of the
!                         derivatives of the basis functions in the Gauss
!                         points in the reference element
!     weight        i/o   Contains the weights in the reference element
!     xg            i/o   Array of length 4x2 containing the co-ordinates of the
!                         Gauss points in the reference element for a four point
!                         Gauss rule
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer indpks(9), indpet(9), i, k, indxi(9), indeta(9)
      double precision theta(3,3), dtetds(3,3), s, h
      character (len=25) text

!     dtetds         d theta_i / ds in integration points
!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     indeta         eta index for node
!     indpet         etha index for integration point
!     indpks         xi index for integration point
!     indxi          ?
!     k              Counting variable
!     s              Mean value of derivative in element
!     text           String variable containing some text
!     theta          Array of length 3x3 containing the one-dimensional
!                    basis functions in the one-dimensional integration points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data text / 'Biquadratic quadrilateral' /
! ======================================================================
!
      call eropen ( 'elp623first' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp623first'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      indxi = 1
      indxi(2) = 2
      indxi(3:5) = 3
      indxi(6) = 2
      indxi(9) = 2
      indeta = 1
      indeta(4) = 2
      indeta(5:7) = 3
      indeta(8:9) = 2

      comppsi = jpsi==1
      if ( jcart>2 ) then

!     --- Error 1769: Co-ordinate system not available

         call errorelvar ( jcart-1, 1769, text, 0 )

      end if
      if ( intrule<1 .or. intrule>3 ) then

!     --- Error 1770: Integration rule not available

         call errorelvar ( intrule, 1770, text, 1 )
         intrule = 1

      end if
      n = 9
      if ( intrule==1 .and. m/=9 .or. &
           intrule==2 .and. m/=4 .or. &
           intrule==3 .and. m/=9 ) then

!     --- Error 1424: m and integration rule do not match

         call errorelvar2 ( m, intrule, 1424, text, 0 )

      end if

!     --- Fill arrays theta and dtetds in integration points

      select case (intrule)

      case(1)

!     --- intrule = 1  Newton Cotes rule

         theta = 0d0
         do i = 1, 3
            theta(i,i) = 1d0
            s = (i-1) * 0.5d0
            dtetds(1,i) = 4d0 * s - 3d0
            dtetds(2,i) =-8d0 * s + 4d0
            dtetds(3,i) = 4d0 * s - 1d0

         end do

         indpks = indxi
         indpet = indeta
         xg(:,1) = (indpks-1)*0.5d0
         xg(:,2) = (indpet-1)*0.5d0

         h = 1d0 / 36d0
         do k = 1, 7, 2
            weight(k) = h
         end do
         h = 4d0*h
         do k = 2, 8, 2
            weight(k) = h
         end do
         weight(9) = 4d0*h

      case(2)

!     --- intrule = 2  Four point Gauss rule

         do k = 1, 2
            theta(1,k) = 2d0 * (alam4(k)-0.5d0) * (alam4(k)-1d0)
            theta(2,k) =-4d0 * alam4(k) * (alam4(k)-1d0)
            theta(3,k) = 2d0 * alam4(k) * (alam4(k)-0.5d0)
            dtetds(1,k) = 4d0 * alam4(k) - 3d0
            dtetds(2,k) =-8d0 * alam4(k) + 4d0
            dtetds(3,k) = 4d0 * alam4(k) - 1d0
            indpks(k) = k
            indpks(k+2) = k
            indpet(k) = 1
            indpet(k+2) = 2
            xg(k,1) = alam4(k)
            xg(k+2,1) = alam4(k)
            xg(k,2) = alam4(1)
            xg(k+2,2) = alam4(2)
         end do
         weight(1:m) = 0.25d0

      case(3)

!     --- intrule = 3  Nine point Gauss rule

         do k = 1, 3
            theta(1,k) = 2d0 * (alam5(k)-0.5d0) * (alam5(k)-1d0)
            theta(2,k) =-4d0 * alam5(k) * (alam5(k)-1d0)
            theta(3,k) = 2d0 * alam5(k) * (alam5(k)-0.5d0)
            dtetds(1,k) = 4d0 * alam5(k) - 3d0
            dtetds(2,k) =-8d0 * alam5(k) + 4d0
            dtetds(3,k) = 4d0 * alam5(k) - 1d0
         end do
         do k = 1, 9
            indpks(k) = indxi(k)
            indpet(k) = indeta(k)
            xg(k,1) = alam5(indpks(k))
            xg(k,2) = alam5(indpet(k))
         end do
         weight = wgs5(indxi) * wgs5(indeta)

      end select  ! case (intrule)

!     --- Fill the arrays dphi/dxi and dphi/deta in the integration points

      do k = 1, m
         do i = 1, 9
            dphidxi(i,k,2) = dtetds(indeta(i),indpet(k))* &
                             theta(indxi(i),indpks(k))
            dphidxi(i,k,1) = dtetds(indxi(i),indpks(k))* &
                             theta(indeta(i),indpet(k))
         end do
      end do

!     --- Fill array phi

      select case (intrule)

      case(1)

!     --- intrule = 1  Newton Cotes rule

         phi = 0d0
         do i = 1, 9
            phi(i,i) = 1d0
         end do
         if ( comppsi ) then

!        --- Compute linear basis functions

               psi = 0d0
            do i = 1, 4
               psi(i,(2*i-1)) = 1d0
               psi(i,2*i) = 0.5d0
               psi(i,9) = 0.25d0
            end do
            psi(1,8) = 0.5d0
            psi(2,2) = 0.5d0
            psi(3,4) = 0.5d0
            psi(4,6) = 0.5d0
            do k = 1, m
               psixi(1,k,1) = -(1d0-xg(k,2))
               psixi(2,k,1) = 1d0-xg(k,2)
               psixi(3,k,1) = xg(k,2)
               psixi(4,k,1) = -xg(k,2)
               psixi(1,k,2) = -(1d0-xg(k,1))
               psixi(2,k,2) = -xg(k,1)
               psixi(3,k,2) = xg(k,1)
               psixi(4,k,2) = 1d0-xg(k,1)
            end do

         end if

      case(2,3)

!     --- intrule = 2  Four point Gauss rule or
!         intrule = 3  Nine point Gauss rule

         do k = 1, m
            do i = 1, 9
               phi(i,k) = theta(indxi(i),indpks(k))* &
                          theta(indeta(i),indpet(k))
            end do
         end do
         if ( comppsi ) then

!        --- Compute linear basis functions

            do k = 1, m
               psi(1,k) = ( 1d0-xg(k,1) ) * ( 1d0-xg(k,2) )
               psi(2,k) = xg(k,1) * ( 1d0-xg(k,2) )
               psi(3,k) = xg(k,1) * xg(k,2)
               psi(4,k) = ( 1d0-xg(k,1) ) * xg(k,2)
               psixi(1,k,1) = -(1d0-xg(k,2))
               psixi(2,k,1) = 1d0-xg(k,2)
               psixi(3,k,1) = xg(k,2)
               psixi(4,k,1) = -xg(k,2)
               psixi(1,k,2) = -(1d0-xg(k,1))
               psixi(2,k,2) = -xg(k,1)
               psixi(3,k,2) = xg(k,1)
               psixi(4,k,2) = 1d0-xg(k,1)
            end do

         end if

      end select  ! case (intrule)

1000  call erclos ( 'elp623first' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp623first'

      end if  ! ( debug )

      end
