      subroutine elp623intpol ( coor, index1, phi, xcoord, ycoord )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 19-11-2002 Fixed a bug (RvL)
!        version  1.1  date 10-08-2000 Adaptation error message
!        version  1.0  date 05-04-2000
!
!   copyright (c) 2000-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute the values of the basis function in a specific point
!    Bi-quadratic quadrilateral
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     interpolation
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(2,*), phi(*), xcoord, ycoord
      integer index1(*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     phi            o    array of length 4 containing the values of the basis
!                         functions in the point xcoord, ycoord
!     xcoord         i    x-coordinate of point
!     ycoord         i    y-coordinate of point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision x(9), y(9), determinant, xsi1, dphidxsi(9), &
                       gradfmin1(2,2), distance, fnewton(2), gradf(2,2), &
                       eta1, xsi0, eta0, thetaxsi(3), dphideta(9), &
                       sumx, sumy, thetaeta(3), a0, a1, a2, a3, &
                       b0, b1, b2, b3, dthetadxsi(3), dthetadeta(3)
      integer i, j, iter, indxsi(9), indeta(9)

!     a0             constant
!     a1             constant
!     a2             constant
!     a3             constant
!     b0             constant
!     b1             constant
!     b2             constant
!     b3             constant
!     determinant    jacobian of element
!     distance       distance between two quantities
!     dphideta       Array containing dphi_i/deta in a point
!     dphidxsi       Array containing dphi_i/dxsi in a point
!     dthetadeta     Derivative of one-dimensional basis function in eta
!                    direction
!     dthetadxsi     Derivative of one-dimensional basis function in xsi
!                    direction
!     eta0           Value of eta in previous iteration
!     eta1           New value of eta
!     fnewton        Contains the values of the funcion on which the Newton
!                    process is carried out
!     gradf          Gradient of function in Newton process
!     gradfmin1      Inverse of gradf
!     i              Counting variable
!     indeta         eta index for node
!     indxsi         xsi index for node
!     iter           Iteration sequence number
!     j              Counting variable
!     sumx           x-sum
!     sumy           y-sum
!     thetaeta       one-dimensional basis function in eta direction
!     thetaxsi       one-dimensional basis function in xsi direction
!     x              array containing the x-coordinates of the nodes
!     xsi0           Value of xsi in previous iteration
!     xsi1           New value of xsi
!     y              array containing the y-coordinates of the nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ddot

!     DDOT           (CDOT)     Inner product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL2        Print a real two-diemsional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     105   Jacobian of element is zero
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The six nodes define six quadratic basis functions phi_i(x,y)
!    In order to compute these basis functions a transformation to
!    a standard triangle is performed (see elp623)
!
!    The quadratic interpolation is defined by:
!
!    x = sum x_i phi_i(xsi,eta)
!    y = sum y_i phi_i(xsi,eta)
!
!    Hence in point (x,y) we have
!         f(1) =  x - sum x_i phi_i(xsi,eta) = 0
!         f(2) =  y - sum y_i phi_i(xsi,eta) = 0
!    This is a non-linear system of equations that can be solved by Newton:
!
!    F' ^(-1) (xsi^k+1 - xsi^k) = -f(1)
!             (eta^k+1 - eta^k) = -f(2)
!
!    With F' = ( df(1)/dxsi, df(2)/dxsi )  =  (a1 + a3 eta, b1 + b3 eta )
!              ( df(1)/deta, df(2)/deta )  =  (a2 + a3 xsi, b2 + b3 xsi )
!
!         F' ^(-1) = 1/det ( df(2)/deta,-df(2)/dxsi )
!                          (-df(1)/deta, df(1)/dxsi )
!
!    The starting value is found by computing the xsi eta for the bi-linear
!    basis functions corresponding to the quadrilateral, see elp621intpol
!
!    The basis functions and its derivatives are defined in subroutine elp623
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data indxsi / 1, 2, 3, 3, 3, 2, 1, 1, 2 /, &
           indeta / 1, 1, 1, 2, 3, 3, 3, 2, 2 /
! ======================================================================
!
      do j = 1, 9
         x(j) = coor(1,index1(j))
         y(j) = coor(2,index1(j))
      end do

!     --- First estimate for xsi and eta, using the bi-linear interpolation

      a0 = x(1)-xcoord
      b0 = y(1)-ycoord
      a1 = x(3)-x(1)
      b1 = y(3)-y(1)
      a2 = x(7)-x(1)
      b2 = y(7)-y(1)
      a3 = x(1)-x(3)+x(5)-x(7)
      b3 = y(1)-y(3)+y(5)-y(7)

!     --- Perform Newton iteration

      xsi0 = 0.5d0
      eta0 = 0.5d0
      distance = 1d0
      iter = 0
      do while ( distance.gt.1d-6 .and. iter.le.5 )

!     --- Stop if the distance is less than 1d-3 or number of
!         iterations is 5

         iter = iter+1
         fnewton(1) = a0+a1*xsi0+a2*eta0+a3*xsi0*eta0
         fnewton(2) = b0+b1*xsi0+b2*eta0+b3*xsi0*eta0
         gradf(1,1) = a1+a3*eta0
         gradf(1,2) = b1+b3*eta0
         gradf(2,1) = a2+a3*xsi0
         gradf(2,2) = b2+b3*xsi0
         determinant = (gradf(1,1)*gradf(2,2)- &
                        gradf(1,2)*gradf(2,1))
         determinant = 1d0/determinant
         if ( abs(determinant).le.0d0 ) then
            call eropen ( 'elp623intpol' )
            call errint ( ielem, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 105, 3, 0, 0 )
            call prinin(index1,9,'node numbers')
            call prinrl2(x,9,2,'co-ordinates')
            call erclos ( 'elp623intpol' )
            go to 1000
         end if
         gradfmin1(1,1) = determinant*gradf(2,2)
         gradfmin1(1,2) =-determinant*gradf(1,2)
         gradfmin1(2,1) =-determinant*gradf(2,1)
         gradfmin1(2,2) = determinant*gradf(1,1)
         xsi1 = xsi0 - gradfmin1(1,1)*fnewton(1) - &
                       gradfmin1(2,1)*fnewton(2)
         eta1 = eta0 - gradfmin1(1,2)*fnewton(1) - &
                       gradfmin1(2,2)*fnewton(2)
         distance = (xsi1-xsi0)**2 + (eta1-eta0)**2
         xsi0 = xsi1
         eta0 = eta1

      end do

!     --- Perform Newton iteration in curved quadrilateral

      distance = 1d0
      iter = 0
      do while ( distance.gt.1d-6 .and. iter.le.5 )

!     --- Stop if the distance is less than 1d-6 or number of
!         iterations is 5

         iter = iter+1
         thetaxsi(1) = (1d0-xsi0)*(2d0*(1d0-xsi0)-1d0)
         thetaxsi(2) = 4d0*(1d0-xsi0)*xsi0
         thetaxsi(3) = xsi0*(2d0*xsi0-1d0)
         thetaeta(1) = (1d0-eta0)*(2d0*(1d0-eta0)-1d0)
         thetaeta(2) = 4d0*(1d0-eta0)*eta0
         thetaeta(3) = eta0*(2d0*eta0-1d0)
         dthetadxsi(1) = -(4d0*(1d0-xsi0)-1d0)
         dthetadxsi(2) = 4d0*(1d0-2d0*xsi0)
         dthetadxsi(3) = 4d0*xsi0-1d0
         dthetadeta(1) = -(4d0*(1d0-eta0)-1d0)
         dthetadeta(2) = 4d0*(1d0-2d0*eta0)
         dthetadeta(3) = 4d0*eta0-1d0
         do i = 1, 9
            phi(i) = thetaxsi(indxsi(i))*thetaeta(indeta(i))
            dphidxsi(i) = dthetadxsi(indxsi(i))*thetaeta(indeta(i))
            dphideta(i) = thetaxsi(indxsi(i))*dthetadeta(indeta(i))
         end do
         sumx = ddot ( 9, phi, 1, x, 1 )
         sumy = ddot ( 9, phi, 1, y, 1 )
         fnewton(1) = xcoord-sumx
         fnewton(2) = ycoord-sumy
         gradf(1,1) = -ddot ( 9, dphidxsi, 1, x, 1 )
         gradf(2,1) = -ddot ( 9, dphidxsi, 1, y, 1 )
         gradf(1,2) = -ddot ( 9, dphideta, 1, x, 1 )
         gradf(2,2) = -ddot ( 9, dphideta, 1, y, 1 )
         determinant = (gradf(1,1)*gradf(2,2)- &
                        gradf(1,2)*gradf(2,1))
         determinant = 1d0/determinant
         if ( abs(determinant).le.0d0 ) then
            call eropen ( 'elp623intpol' )
            call errint ( ielem, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 105, 3, 0, 0 )
            call prinin(index1,9,'node numbers')
            call prinrl2(x,9,2,'co-ordinates')
            go to 1000
         end if
         gradfmin1(1,1) = determinant*gradf(2,2)
         gradfmin1(1,2) =-determinant*gradf(2,1)
         gradfmin1(2,1) =-determinant*gradf(1,2)
         gradfmin1(2,2) = determinant*gradf(1,1)
         xsi1 = xsi0 - gradfmin1(1,1)*fnewton(1) - &
                       gradfmin1(2,1)*fnewton(2)
         eta1 = eta0 - gradfmin1(1,2)*fnewton(1) - &
                       gradfmin1(2,2)*fnewton(2)
         distance = (xsi1-xsi0)**2 + (eta1-eta0)**2
         xsi0 = xsi1
         eta0 = eta1
      end do

1000  end
