      subroutine elp625 ( coor2d, x, w, dphidx, xgauss, phi, index1, &
                          qmat, psi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.0  date 10-06-2013 Use subroutines
!        version  1.7  date 08-03-2010 Change for irule = 1
!
!   copyright (c) 1996-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute basis functions and related quantities for extended
!     linear triangle
!     computed are:     weights for numerical integration
!                       values of derivatives of basis functions
!                       values of basis functions in integration points
!     elp625 is called by several types of element subroutines
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmoduleconsts
      use sepmoduledebug
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(*)
      double precision coor2d(2,*), x(4,2), w(m+n), dphidx(4,m,2), &
                       xgauss(m,2), phi(4,m), qmat(3,*), psi(3,m)

!     coor2d     i     array of length 2 x number of nodal points containing
!                    the co-ordinates of the nodal points with respect to
!                    the global numbering
!                    x  = coor2d (1,i);  y  = coor2d (2,i);
!                     i                 i
!     dphidx   o     array of length 4 x m x 2  containing the derivatives
!                    of the basis functions in the sequence:
!                                 k
!                    d phi / dx (x )= dphidx (i,k,1);
!                         i
!                                 k
!                    d phi / dy (x )= dphidx (i,k,2);
!                         i
!                    if a newton cotes rule is used and m=3 then the derivatives
!                    are filled in the midside points, else if m=6 the
!                    derivatives are filled in all nodes
!     index1   i     array of length 4 containing the nodal point numbers
!                    of the element
!     phi      o     array of length 4 x m containing the values of the
!                    basis functions in the gauss points in the sequence:
!                           k
!                    phi (xg ) = phi (i,k)
!                       i
!                    array phi is only filled when ifirst = 0 and gauss
!                    integration is applied
!     psi      o     array of length 3 x m containing the values of the linear
!                    basis functions in the gauss points in the sequence:
!                           k
!                    psi (xg ) = psi (i,k)
!                       i
!                    array psi is only filled when ifirst = 0 and gauss
!                    integration is applied
!     qmat     o     transformation matrix global -> local coordinates
!                    in case of a 3D element
!     w        o     array of length m containing the weights for the
!                    numerical integration
!     x       i/o    array of length 4 x 2 containing the x- and
!                    y-coordinates of the nodal points with respect to the
!                    local numbering in the sequence:
!                    x  = x (i,1);  y  = x (i,2);
!                     i              i
!     xgauss   o     array of length m x 2 containing the co-ordinates of
!                    the gauss points.  array xgauss is only filled when gauss
!                    integration is applied.
!                    xg  = xgauss (i,1);  yg  = xgauss (i,2);
!                      i                    i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision som, gradc, phicen, area, &
                       delta, e(3,2)
      integer i, j, k, iweigo, intrule
      character (len=24) text
      save iweigo

!     area           Area of element
!     delta          Jacobian
!     e              work array of length 3 x 2, to store the factors
!                     ij
!                    e.        see method
!     gradc          Gradient of lambda_1 * lambda_2 * lambda_3
!     i              Counting variable
!     intrule        Actual intergration rule
!     iweigo         if 1 the weights must be filled
!     j              Counting variable
!     k              Counting variable
!     phicen         Value of alam1*alam2*alam3 in integration point
!     som            Help parameter to compute a sum
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     PRINRL2        Print a real two-diemsional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     105   Jacobian of element is zero
! **********************************************************************
!
!                       PSEUDO CODE
!
!        to compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!         ij        i+1    i+2    i+1
!        e   =  (-1)    ( x    - x    )   cyclic: i=1,2,3 ;   j=1,2
!                          j      j
!                    32  11    12 31
!        delta   =  e   e   - e  e
!                             ij
!        d lambda  / dx   =  e   /  delta
!                i     j
!        lamc = lambda lambda lambda
!                     1      2      3
!        phi  =  lambda  - 9 lamc  i=1,2,3   (basis functions:1,2,3)
!           i          i
!        phi  = 27 lamc
!           4
!        grad phi  = grad lambda  - 9 grad lamc
!                i              i
!        grad phi    = 27 grad lamc
!                4
!        grad lamc = lambda lambda grad lambda  + lambda lambda grad lambda + &
!                          2      3           1         3      1           2
!                    lambda lambda grad lambda
!                          1      2           3
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data text / 'Extended linear triangle' /
! ======================================================================
!
      call eropen ( 'elp625' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp625'
         write(irefwr,1) 'irule, iweigh, jcart', irule, iweigh, jcart
         write(irefwr,1) 'icoor, ndim', icoor, ndim
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1500

      intrule = mod(irule,100)
      if ( ifirst==0 ) then

!     --- ifirst = 0, make some initializations

         iweigo = mod(iweigh,10)

         if ( jcart<1 .or. jcart>4 ) then

!        --- Error 1769: Co-ordinate system not available

            call errorelvar ( jcart-1, 1769, text, 0 )

         end if
         if ( intrule<1 .or. intrule>3 ) then

!        --- Error 1770: Integration rule not available

            call errorelvar ( intrule, 1770, text, 1 )
            intrule = 2

         end if
         if ( intrule==1 .and. intrule==2 .and. m/=4 .or. &
              intrule==3 .and. m/=7 ) then

!        --- Error 1424: m and integration rule do not match

            call errorelvar2 ( m, intrule, 1424, text, 0 )

         end if

         if ( ideriv>0 ) then

!        --- ideriv = 1, set grad phi equal to 0

            dphidx = 0d0

         end if

      end if

      if ( icoor==1 ) then

!     --- icoor = 1   fill co-ordinates of nodes in array x

         do i = 1, inpelm
            x(i,1:2) = coor2d(1:2,index1(i))
         end do
         if ( inpelm==3 ) then
            x(4,1:2) = ( x(1,1:2)+x(2,1:2)+x(3,1:2) ) / 3d0
         end if

         if ( debug ) call prinrl1 ( x, 2, 4, 'x' )

      end if

!     --- check if element is straight or curved

      if ( ifirst==0 .and. (iphi==1 .or. intrule>=2) ) then

!     --- ifirst = 0  first call of element subroutine, and iphi = 1
!         fill array phi

         if ( intrule==1 ) then

!        --- intrule = 1  Newton Cotes rule

            phi = 0d0
            psi = 0d0
            do i = 1, n
               phi(i,i) = 1d0
            end do
            do i = 1, 3
               psi(i,i) = 1d0
            end do
            psi(1:3,m) = 1d0/3d0

         else if ( intrule==2 ) then

!        --- intrule = 2  4-points gauss rule

            do k = 1, 4
               phicen = alam2(1,k)*alam2(2,k)*alam2(3,k)
               phi(4,k) = 27d0 * phicen
               do i = 1, 3
                  phi(i,k) = alam2(i,k) - 9d0*phicen
                  psi(i,k) = alam2(i,k)
               end do
            end do

         else if ( intrule==3 ) then

!        --- intrule = 3  7 -points gauss rule

            do k = 1, 7
               phicen = alam3(1,k)*alam3(2,k)*alam3(3,k)
               phi(4,k) = 27d0 * phicen
               do i = 1, 3
                  phi(i,k) = alam3(i,k) - 9d0*phicen
                  psi(i,k) = alam3(i,k)
               end do
            end do

         end if
         if ( debug ) then
            call prinrl1 ( phi, m, 4, 'phi' )
            call prinrl1 ( psi, m, 3, 'psi' )
         end if  ! ( debug )

      end if

      if ( iweigo==1 .or. ideriv==1 ) then

!     --- ideriv = 1  compute derivatives or iweigo = 1: weights
!                                  ij
!         compute the quantities  e   and delta
!                                    ij
!         store d lambda  / d x  in e
!                       i      j

         e(1,1) = x(2,2) - x(3,2)
         e(2,1) = x(3,2) - x(1,2)
         e(3,1) = x(1,2) - x(2,2)
         e(1,2) = x(3,1) - x(2,1)
         e(2,2) = x(1,1) - x(3,1)
         e(3,2) = x(2,1) - x(1,1)

         delta = e(3,1)*e(1,2) - e(1,1)*e(3,2)
         area = delta
         if ( checksign .and. delta<0d0 ) then
            print *,'area of element ', ielem, ' is negative'
         end if

!        --- check if delta is non-zero

         if( abs(delta)<=0d0 ) then
            call errint( ielem, 1 )
            call errint( itype, 2 )
            call errint( ielgrp, 3 )
            call errsub ( 105, 3, 0, 0 )
            call prinin(index1,4,'node numbers')
            call prinrl2(x,4,2,'co-ordinates')
            go to 1500
         end if

         e = e / delta

      end if

      if ( igausp==1 .or. intrule>=2 .and. jcart>=2 ) then

!     --- gauss points must be filled

         if ( intrule==1 ) then

!        --- intrule = 1  Newton Cotes rule

            xgauss = x

         else if ( intrule==2 ) then

!        --- intrule = 2  four-point gauss rule

            do j = 1, 2
               do k = 1, 4
                  som = 0d0
                  do i = 1, 3
                     som = som + alam2(i,k) * x(i,j)
                  end do
                  xgauss(k,j) = som
               end do
            end do

         else if ( intrule==3 ) then

!        --- intrule = 3  seven-point gauss rule

            do j = 1, 2
               do k = 1, 7
                  som = 0d0
                  do i = 1, 3
                     som = som + alam3(i,k) * x(i,j)
                  end do
                  xgauss(k,j) = som
               end do
            end do

         end if
         if ( debug ) call prinrl1 ( xgauss, 2, m, 'xgauss' )

      end if
      if ( iweigo==1 ) then

!     --- iweigo = 1  fill weights

         if ( intrule==1 ) then

!        --- intrule = 1  newton cotes rule

            w(1:3) = abs(delta)*11d0/120d0
            w(4) = abs(delta)*27d0/120d0

         else if ( intrule==2 ) then

!        --- intrule = 2  four points gauss rule

            w(1:m) = wgs2 * abs(delta)

         else if ( intrule==3 ) then

!        --- intrule = 3  seven points gauss rule

            w(1:m) = wgs3 * abs(delta)

         end if

         if ( debug ) call prinrl ( w, m, 'w/1' )
         if ( jcart==2 .or. jcart==4 ) then

!        --- jcart = 2, 4 axi-symmetric co-ordinates

            do i = 1, m
               w(i) = w(i) * xgauss(i,1) * pi2
            end do

         else if ( jcart==3 ) then

!        --- jcart = 3  polar co-ordinates

            do i = 1, m
               w(i) = w(i) * xgauss(i,1)
            end do

         end if
         if ( debug ) call prinrl ( w, m, 'w' )

      end if

      if ( ideriv==1 ) then

!     --- ideriv = 1   fill derivatives

         if ( intrule==1 ) then

!        --- Newton Cotes rule

            do j = 1, 2
               do k = 1, 4
                  do i = 1, 3
                     dphidx(i,k,j) = e(i,j)
                  end do
                  dphidx(4,k,j) = 0d0
               end do
            end do

         else if ( intrule==2 ) then

!        --- four-point Gauss rule

            do j = 1, 2
               do k = 1, m
                  gradc = alam2(2,k)*alam2(3,k)*e(1,j) + &
                          alam2(3,k)*alam2(1,k)*e(2,j) + &
                          alam2(1,k)*alam2(2,k)*e(3,j)
                  do i = 1, 3
                     dphidx(i,k,j) = e(i,j)-9d0*gradc
                  end do
                  dphidx(4,k,j) = 27d0*gradc
               end do
            end do

         else

!        --- seven-point Gauss rule

            do j = 1, 2
               do k = 1, m
                  gradc = alam3(2,k)*alam3(3,k)*e(1,j) + &
                          alam3(3,k)*alam3(1,k)*e(2,j) + &
                          alam3(1,k)*alam3(2,k)*e(3,j)
                  do i = 1, 3
                     dphidx(i,k,j) = e(i,j)-9d0*gradc
                  end do
                  dphidx(4,k,j) = 27d0*gradc
               end do
            end do

         end if

      end if
      if ( iweigo==1 ) then
         delta = 0.5d0*area
      end if

      if ( itype>=902 .and. itype<=903 ) then

!     --- Special case of Navier-Stokes element
!         w(1..m ) contains the weights
!         w(m+1 .. m+n) the diagonal of the mass matrix
!         In this case it is sufficient to use the integration rule available

         do i = 1, n
            som = 0d0
            do k = 1, m
               som = som+w(k)*phi(i,k)**2
            end do  ! k = 1, m
            w(m+i)= som
         end do  ! i = 1, n

      end if  ! ( itype>=902 .and. itype<=903 )

1500  call erclos ( 'elp625' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp625'

      end if  ! ( debug )

      end
