      subroutine elp626 ( coor2d, x, w, dphidx, xgauss, phi, index1, &
                          qmat, psi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.0  date 10-06-2013 Use subroutines
!        version  1.5  date 09-03-2013 Correction declaration
!
!   copyright (c) 1996-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute basis functions and related quantities for extended
!     bilinear quadrilateral
!     Computed are:     weights for numerical integration
!                       values of derivatives of basis functions
!                       values of basis functions in integration points
!     ELP626 is called by several types of element subroutines
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmoduleconsts
      use sepmoduledebug
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(5)
      double precision coor2d(ndim,*), x(5,ndim), w(m), &
                       dphidx(5,m,ndim), &
                       xgauss(m,ndim), phi(5,m), qmat(3,3), psi(4,m)

!     coor2d     i     array of length 2 x number of nodal points containing
!                    the co-ordinates of the nodal points with respect to
!                    the global numbering
!                    x  = coor2d (1,i);  y  = coor2d (2,i);
!                     i                 i
!     dphidx   o     Array of length 5 x m x 2  containing the derivatives
!                    of the basis functions in the sequence:
!                                 k
!                    d phi / dx (x )= dphidx (i,k,1);
!                         i
!                                 k
!                    d phi / dy (x )= dphidx (i,k,2);
!                         i
!     index1   i     Array of length 5 containing the nodal point numbers
!                    of the element
!     phi      o     Array of length 5 x m containing the values of the
!                    basis functions in the Gauss points in the sequence:
!                           k
!                    phi (xg ) = phi (i,k)
!                       i
!                    Array phi is only filled when ifirst = 0 and gauss
!                    integration is applied
!     psi      o     array of length 4 x m containing the values of the linear
!                    basis functions in the gauss points in the sequence:
!                           k
!                    psi (xg ) = psi (i,k)
!                       i
!                    array psi is only filled when ifirst = 0 and gauss
!                    integration is applied
!     qmat     o     transformation matrix global -> local coordinates
!                    in case of a 3D element
!     w        o     array of length m containing the weights for the
!                    numerical integration
!     x       i/o    array of length 5 x 2 containing the x- and
!                    y-coordinates of the nodal points with respect to the
!                    local numbering in the sequence:
!                    x  = x (i,1);  y  = x (i,2);
!                     i              i
!     xgauss   o     Array of length m x 2 containing the co-ordinates of
!                    the gauss points.  Array xgauss is only filled when Gauss
!                    integration is applied.
!                    xg  = xgauss (i,1);  yg  = xgauss (i,2);
!                      i                    i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical allneg, allpos
      double precision A, B, J0, J1, J2, xi(2), eta(2), h, h1, h2, &
                       som, dpdxic, dpdetac, phic, &
                       e(4,2), wh(9), xg(9,2), &
                       xloc(4,2), v1work(3), v2work(3), v3work(3), &
                       dpdxi(5,9), dpdeta(5,9), dxdxi(9,3,3), &
                       dxidx(9,2,3), delta(9)
      integer i, j, k, indlc(4), iweigo, iqmat, intrule
      character (len=32) text
      save iweigo, iqmat, xg, dpdxi, dpdeta

!     a              Factor for the computation of the Jacobian (See method)
!     allneg         Indication the values of the Jacobian in the vertices
!                    are all negative (true) or not (false)
!     allpos         Indication the values of the Jacobian in the vertices
!                    are all positive (true) or not (false)
!     b              Factor for the computation of the Jacobian (See method)
!     delta          work array of length 4 to store the values of the jacobian
!                    in the nodes
!     dpdeta         contains derivatives in reference element for gauss rule
!                                                   k
!                    dpdeta(i,k) = d phi  / d eta (x )
!                                       i
!     dpdetac        Value of dphi_c/deta
!     dpdxi          contains derivatives in reference element for gauss rule
!                                                   k
!                    dpdxi(i,k) = d phi  / d xi (x )
!                                      i
!     dpdxic         Value of dphi_c/dxi
!     dxdxi          work array of length 7x3x3 to store the values of dx/dxi, &
!                    dx/deta, dy/dxi and dy/deta according to:
!                            k
!                    dx/dxi(x ) = dxdxi(k,1,1)
!                             k
!                    dx/deta(x ) = dxdxi(k,1,2)
!                            k
!                    dy/dxi(x ) = dxdxi(k,2,1)
!                             k
!                    dy/deta(x ) = dxdxi(k,2,2) etc.
!     dxidx          work array of length 7x2x3 to store the values of dxi/dx, &
!                    dxi/dy, dxi/dz and deta/dx ... according to:
!                            k                             k
!                    dxi/dx(x ) = dxdxi(k,1,1)   dxi/dy(x ) = dxdxi(k,1,2)
!     e              Work array of length 4 x 2, to store the factors
!                     ij
!                    e.        See method
!     eta            Array of length 2 to store the values of eta in a point
!     h              Help variable to store a quantity temporarily
!     h1             Help variable to store a quantity temporarily
!     h2             Help variable to store a quantity temporarily
!     i              Counting variable
!     ind            Integer array containing the cyclic permutation of 1,2,3,4
!     intrule        Actual intergration rule
!     iqmat          if 1 the qmat matrix must be filled
!     iweigo         if 1 the weights must be filled
!     j              Counting variable
!     j0             Factors for the computation of the Jacobian (See method)
!     j1             Factors for the computation of the Jacobian (See method)
!     j2             Factors for the computation of the Jacobian (See method)
!     k              Counting variable
!     phic           Value of phi_c
!     som            help variable to compute a sum
!     text           String variable containing some text
!     v1work         First help vector to compute transformation
!     v2work         Second help vector to compute transformation
!     v3work         Third help vector to compute transformation
!     wh             Array of length 4 containing the values of the Jacobian in
!                    the vertices or for the Gauss integration rules work array
!                    to store some help variables
!     xg             Array of length 4x2 containing the co-ordinates of the
!                    Gauss points in the reference element for a four point
!                    Gauss rule
!     xi             Array of length 2 to store the values of xi in a point
!     xloc           Local co-ordinates
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ddot

!     DDOT           (CDOT)     Inner product of two vectors
!     ELOUTR         Compute outer product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
!     ERRSUB         Error messages
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     230   Jacobian of element is zero or has changing sign
!    1540   Jacobian zero or changing sign
! **********************************************************************
!
!                       PSEUDO CODE
!
!        To compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!         ij       j     i+1    i
!        e   = (-1)  (  x    - x    )   cyclic: i=1,2,3,4 ;   j=1,2
!                        j+1    j+1
!              1     2    1          4    1          3    2    1    4
!        x  = x  + (x  - x ) xi + (x  - x ) eta + (x  - x  + x  - x ) xi eta
!         i    i     i    i          i    i          i    i    i    i
!                                   i = 1,2
!                  1    12        42
!        So: x  = x  + e   xi - e   eta + A xi eta
!             1    1
!                  1    11        41
!            x  = x  - e   xi + e   eta + B xi eta
!             2    2
!            |   2    1                  4    1          |
!            |  x  - x  + A eta         x  - x  + A xi  |
!            |   1    1                  1    1          |
!        J  =|                                           |
!            |   2    1                  4    1          |
!            |  x  - x  + B eta         x  - x  + B xi  |
!            |   2    2                  2    2          |
!             1    2    3    4        12    32
!        A = x  - x  + x  - x    = - e   - e
!             1    1    1    1
!             3    4    2    1       11    31
!        B = x  - x  - x  + x    =  e   + e
!             2    2    2    2
!       |J| = J  + J  xi + J  eta
!              0    1        2
!              41   12    11   42
!        J  = e  * e   - e  * e
!         0
!                   12         11
!        J  = B  * e   - A  * e
!         1
!                   41         42
!        J  = A  * e   + B  * e
!         2
!        dphi / dx  = dphi / dxi dxi/dx  + dphi / deta deta/dx
!                 j                      j                      j
!                       41
!        dxi/dx  = [  e   + B xi ] / |J|
!               1
!                       42
!        dxi/dx  = [  e   - A xi ] / |J|
!               2
!                       11
!        deta/dx  = [  e   - B eta ] / |J|
!               1
!                       12
!        deta/dx  = [  e   + A eta ] / |J|
!               2
!     In case of 3D elements we use the following formulae:
!     x = sum phi (xi,eta) x    dx/dxi = sum dphi /dxi x   dx/deta=...
!     -    i     i          -i    -         i      i      -i
!             a                      a                         a
!     dphi /dx = dphi / dxi  dxi/dx   +  dphi / deta  deta/dx
!         i          i                         i
!                                                                        -1
!     | dxi/dx   deta/dx   dzeta/dx  |   | dx/dxi   dy/dxi   dz/dxi |
!     | dxi/dy   deta/dy   dzeta/dy  | = | dx/deta   dy/deta   dz/deta |
!     | dxi/dz   deta/dz   dzeta/dz  |   | dx/dzeta  dy/dzeta  dz/dzeta|
!     zeta is choosen perpendicular to the xi-eta direction, which implies
!     that we may choose:
!     dx / dzeta = dx / dxi times  dx / deta / || dx/dxi x dx/deta ||
!      -            -                -              -         - &
!     where times or x is the outer product
!     The Jacobian of the dx/dxi matrix is given by:
!     J = dx/dzeta ( dy/dxi dz/deta - dy/deta dz/dxi )
!       + dy/dzeta ( dz/dxi dx/deta - dz/deta dx/dxi )
!       + dz/dzeta ( dx/dxi dy/deta - dx/deta dy/dxi )
!     and since dx/dzeta before normalization is given by
!         dx/dzeta = dy/dxi dz/deta - dy/deta dz/dxi
!         dy/dzeta = dz/dxi dx/deta - dz/deta dx/dxi
!         dz/dzeta = dx/dxi dy/deta - dx/deta dy/dxi
!     we see that the normalization factor is equal to
!        sqrt(     ( dy/dxi dz/deta - dy/deta dz/dxi )^2
!                + ( dz/dxi dx/deta - dz/deta dx/dxi )^2
!                + ( dx/dxi dy/deta - dx/deta dy/dxi )^2 ) = J
!      The derivatives dxi/dx and so on follow from the inverse, &
!      which in this 3x3 case leads to:
!      dxi/dx = ( dy/deta  dz/dzeta - dy/dzeta dz/deta  ) / J
!      dxi/dy = ( dy/dzeta dz/dxi  - dy/dxi  dz/dzeta ) / J
!      dxi/dz = ( dy/dxi  dz/deta  - dy/deta  dz/dxi  ) / J
!      deta/dx = ( dz/deta  dx/dzeta - dz/dzeta dx/deta  ) / J
!      deta/dy = ( dz/dzeta dx/dxi  - dz/dxi  dx/dzeta ) / J
!      deta/dz = ( dz/dxi  dx/deta  - dz/deta  dx/dxi  ) / J
!    The basis functions at the standard unit square are defined by:
!    phi_1 = lambda_1(xi) lambda_1(eta) - 4 phi_c
!    phi_2 = lambda_2(xi) lambda_1(eta) - 4 phi_c
!    phi_3 = lambda_2(xi) lambda_2(eta) - 4 phi_c
!    phi_4 = lambda_1(xi) lambda_2(eta) - 4 phi_c
!    phi_5 = 16 phi_c
!    phi_c = lambda_1(xi) lambda_2(xi)lambda_1(eta) lambda_2(eta)
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data indlc / 2, 3, 4, 1 /
      data text / 'Extendend Bilinear quadrilateral' /
! ======================================================================
!
      iweigo = mod(iweigh,10)
      intrule = mod(irule,100)
      iqmat  = iweigh/10
      if ( ifirst==0 ) then

!     --- ifirst = 0  (First call)

         call eropen ( 'elp626' )
         if ( jcart==2 ) then

!        --- jcart = 2  (Axi-symmetric co-ordinates)

         else if ( jcart/=1 ) then

!        --- Error 1769: Co-ordinate system not available

            call errorelvar ( jcart-1, 1769, text, 0 )

         end if
         if ( intrule/=1 .and. (intrule<3 .or. intrule>4) ) then

!        --- Error 1770: Integration rule not available

            call errorelvar ( intrule, 1770, text, 1 )
            intrule = 3

         end if
         if ( intrule==1 .and. m/=5 .or. &
              intrule==3 .and. m/=4 .or. &
              intrule==4 .and. m/=9 ) then

!        --- Error 1424: m and integration rule do not match

            call errorelvar2 ( m, intrule, 1424, text, 0 )

         end if
         call erclos ( 'elp626' )

         if ( intrule==1 ) then

!        --- intrule = 1  Four point Newton Cotes rule, Fill xsi and eta for
!                         reference element

            xg(1,1) = 0d0
            xg(2,1) = 1d0
            xg(3,1) = 1d0
            xg(4,1) = 0d0
            xg(5,1) = 0.5d0
            xg(1,2) = 0d0
            xg(2,2) = 0d0
            xg(3,2) = 1d0
            xg(4,2) = 1d0
            xg(5,2) = 0.5d0

         else if ( intrule==3 ) then

!        --- intrule = 3  Four point Gauss rule,  Fill Gauss points for
!                                                 reference element

            xg(1,1) = alam4(1)
            xg(2,1) = alam4(2)
            xg(3,1) = alam4(2)
            xg(4,1) = alam4(1)
            xg(1,2) = alam4(1)
            xg(2,2) = alam4(1)
            xg(3,2) = alam4(2)
            xg(4,2) = alam4(2)

         else

!        --- intrule = 4  Nine point Gauss rule,  Fill Gauss points for
!                                                 reference element

            do i = 1, 3
               do j = 1, 3
                  xg((i-1)*3+j,1) = alam5(i)
                  xg((i-1)*3+j,2) = alam5(j)
               end do
            end do
         end if
         do k = 1, m
            dpdxic = xg(k,2)*(1d0-xg(k,2))*(1d0-2d0*xg(k,1))
            h = 4d0*dpdxic
            dpdxi(1,k) = -(1d0-xg(k,2)) - h
            dpdxi(2,k) =   1d0-xg(k,2) - h
            dpdxi(3,k) =       xg(k,2) - h
            dpdxi(4,k) =      -xg(k,2) - h
            dpdxi(5,k) = 4d0*h
            dpdetac = xg(k,1)*(1d0-xg(k,1))*(1d0-2d0*xg(k,2))
            h = 4d0*dpdetac
            dpdeta(1,k) = -(1d0-xg(k,1)) - h
            dpdeta(2,k) =      -xg(k,1) - h
            dpdeta(3,k) =       xg(k,1) - h
            dpdeta(4,k) =   1d0-xg(k,1) - h
            dpdeta(5,k) = 4d0*h
         end do

         if ( iphi==1 .or. igausp==1 ) then

!        --- Fill arrays phi and psi

            do k = 1, m
               phic = (1d0-xg(k,1))*xg(k,1)*(1d0-xg(k,2))*xg(k,2)
               h = 4d0*phic
               psi(1,k) = ( 1d0-xg(k,1) ) * ( 1d0-xg(k,2) )
               psi(2,k) = xg(k,1) * ( 1d0-xg(k,2) )
               psi(3,k) = xg(k,1) * xg(k,2)
               psi(4,k) = ( 1d0-xg(k,1) ) * xg(k,2)
               phi(1,k) = psi(1,k) - h
               phi(2,k) = psi(2,k) - h
               phi(3,k) = psi(3,k) - h
               phi(4,k) = psi(4,k) - h
               phi(5,k) = 4d0*h
            end do

         end if

      end if

      if ( icoor==1 ) then

!     --- icoor = 1   Fill co-ordinates of nodes in array x

         do k = 1, ndim
            do i = 1, inpelm
               x(i,k) = coor2d(k,index1(i))
            end do
            if ( inpelm==5 ) then
               x(5,k) = ( x(1,k)+x(2,k)+x(3,k)+x(4,k) ) * 0.25d0
            end if
         end do

         if ( ndim>2 ) then

!        --- Quadrilateral in three-dimensional space
!            transform co-ordinates

            if ( iqmat==2 ) then

!           --- at this moment only the linear transformation has been
!               implemented

               do i = 1, 3
                  v1work(i) = x(2,i)-x(1,i)
                  v2work(i) = x(4,i)-x(1,i)
               end do
               call eloutr ( v1work, v2work, v3work )
               h1 = 1d0/sqrt(v1work(1)**2+v1work(2)**2+v1work(3)**2)
               h2 = 1d0/sqrt(v3work(1)**2+v3work(2)**2+v3work(3)**2)
               do i = 1, 3
                  v1work(i) = v1work(i)*h1
                  v3work(i) = v3work(i)*h2
               end do
               call eloutr ( v1work, v3work, v2work )
               h1 = 1d0/sqrt(v2work(1)**2+v2work(2)**2+v2work(3)**2)
               do i = 1, 3
                  qmat(i,2) = v1work(i)
                  qmat(i,3) = v2work(i)*h1
                  qmat(i,1) = v3work(i)
               end do
               do k = 1, 2
                  do i = 1, 4
                     xloc(i,k) = qmat(2,k)*(x(i,1)-x(1,1)) + &
                                 qmat(3,k)*(x(i,2)-x(1,2)) + &
                                 qmat(1,k)*(x(i,3)-x(1,3))
                  end do
               end do
               do k = 1, 2
                  do i = 1, 4
                     x(i,k) = xloc(i,k)
                  end do
               end do

            end if

         end if

      end if

      if ( ndim==2 .or. iqmat>0 ) then

!     --- standard situation

         if ( iweigo==1 .or. ideriv==1 ) then

!        --- ideriv = 1  Compute derivatives or iweigo = 1: weights
!                                     ij
!            Compute the quantities  e

            do i = 1, 4
               e(i,1) = x(i,2) - x(indlc(i),2)
               e(i,2) = x(indlc(i),1) - x(i,1)
            end do

            A = - e(1,2) - e(3,2)
            B =   e(1,1) + e(3,1)
            J0 =  e(1,2)*e(4,1) - e(4,2)*e(1,1)
            J1 =  B * e(1,2) + A * e(1,1)
            J2 =  A * e(4,1) + B * e(4,2)

!           --- Compute the Jacobian J in the vertices

            wh(1) = J0
            wh(2) = J0 + J1
            wh(3) = J0 + J1 + J2
            wh(4) = J0 + J2

!           --- Check whether Jacobian has constant sign

            allneg=( wh(1)<0 .and. wh(2)<0 .and. wh(3)<0 &
                                .and. wh(4)<0 )
            allpos=( wh(1)>0 .and. wh(2)>0 .and. wh(3)>0 &
                                .and. wh(4)>0 )
            if ( checksign .and. allneg ) then
               print *,'area of element ', ielem, ' is negative'
            end if
            if( .not. allneg .and. .not. allpos ) then
               call eropen ( 'elp626' )
               if ( isubr==0 ) then
                  call errint ( ielem, 1 )
                  call errint ( itype, 2 )
                  call errint ( ielgrp, 3 )
                  call errsub ( 230, 3, 0, 0 )
               else
                  call errint ( ielgrp, 1 )
                  call errint ( ielem, 2 )
                  call errchr ( 'surface', 1 )
                  call errsub ( 1540, 2, 0, 1 )
               end if
               write ( irefwr, 320 ) ((x(k,j),j=1,2), k=1, 5)
320            format ( ( 1x, ss, 2(1pe14.5e2) ) )
               call erclos ( 'elp626' )
               go to 1000
            end if
         end if

         if ( igausp==1 ) then

!        --- gauss points must be filled

            if ( intrule==1 ) then

!           --- intrule = 1  Newton Cotes rule

               do k = 1, ndim
                  do i = 1, m
                     xgauss(i,k)=x(i,k)
                  end do
               end do

            else

!           --- intrule>1  Gauss rule

               do k = 1, ndim
                  do i = 1, m
                     xgauss(i,k) = phi(1,i)*x(1,k)+phi(2,i)*x(2,k)+ &
                                   phi(3,i)*x(3,k)+phi(4,i)*x(4,k)+ &
                                   phi(5,i)*x(5,k)
                  end do
               end do

            end if

         end if

         do k = 1, m
            wh(k) = J0+J1*xg(k,1)+J2*xg(k,2)
         end do

         if ( ideriv==1 ) then

!        --- ideriv = 1   Fill derivatives

            do k = 1, m
               xi(1)=(e(4,1)+B*xg(k,1))/wh(k)
               xi(2)=(e(4,2)-A*xg(k,1))/wh(k)
               eta(1)=(e(1,1)-B*xg(k,2))/wh(k)
               eta(2)=(e(1,2)+A*xg(k,2))/wh(k)
               do i = 1, 2
                  do j = 1, 5
                     dphidx(j,k,i) = dpdxi(j,k)*xi(i)- &
                                     dpdeta(j,k)*eta(i)
                  end do
               end do

            end do

         end if

         if ( iweigo==1 ) then

!        --- iweigo = 1  Fill weights

            if ( intrule==1 ) then
               do k = 1, 4
                  w(k) = -5d0/12d0 * abs(wh(k))
               end do
               w(5) = 8d0/3d0 * abs(wh(5))
            else if ( intrule==3 ) then
               do k = 1, 4
                  w(k) = .25d0 * abs(wh(k))
               end do
            else
               do i = 1, 3
                  do j = 1, 3
                     w((i-1)*3+j) = wgs5(i)*wgs5(j)*abs(wh((i-1)*3+j))
                  end do
               end do
            end if
            if ( jcart==2 ) then

!           --- axisymmetric co-ordinates

               if ( igausp/=1 ) then

!              --- gauss points have not been filled

                  do i = 1, m
                     xgauss(i,1) = phi(1,i)*x(1,1)+phi(2,i)*x(2,1)+ &
                                   phi(3,i)*x(3,1)+phi(4,i)*x(4,1)+ &
                                   phi(5,i)*x(5,1)
                  end do

               end if
               do k = 1, m
                  w(k) = w(k) * xgauss(k,1) * pi2
               end do
            end if
         end if
         if ( iqmat==2 ) then
            do k = 1, ndim
               do i = 1, 5
                  x(i,k) = coor2d(k,index1(i))
               end do
            end do
         end if

      else

!     --- ndim = 3

         if ( iweigo==1 .or. ideriv==1 ) then

!        --- ideriv = 1  compute derivatives or iweigo = 1: weights
!            compute dx/dxi, dy/dxi, dx/deta, dy/deta;  store in dxdxi

            do j = 1, ndim
               do i = 1, m
                  dxdxi(i,j,1) = ddot(4, x(1,j), 1, dpdxi(1,i), 1)
                  dxdxi(i,j,2) = ddot(4, x(1,j), 1, dpdeta(1,i), 1)
               end do
            end do

!           --- fill jacobian in delta

            do i = 1, m
               dxdxi(i,1,3) = dxdxi(i,2,1)*dxdxi(i,3,2)- &
                               dxdxi(i,3,1)*dxdxi(i,2,2)
               dxdxi(i,2,3) = dxdxi(i,3,1)*dxdxi(i,1,2)- &
                               dxdxi(i,1,1)*dxdxi(i,3,2)
               dxdxi(i,3,3) = dxdxi(i,1,1)*dxdxi(i,2,2)- &
                               dxdxi(i,2,1)*dxdxi(i,1,2)
            end do
            do k = 1, m
               delta(k) = sqrt( dxdxi(k,1,3)**2 + &
                                dxdxi(k,2,3)**2 + &
                                dxdxi(k,3,3)**2 )
               if ( delta(k)<=0d0 ) then
                  call eropen ( 'elp626' )
                  if ( isubr==0 ) then
                     call errint ( ielem, 1 )
                     call errint ( itype, 2 )
                     call errint ( ielgrp, 3 )
                     call errsub ( 230, 3, 0, 0 )
                  else
                     call errint ( ielgrp, 1 )
                     call errint ( ielem, 2 )
                     call errchr ( 'surface', 1 )
                     call errsub ( 1540, 2, 0, 1 )
                  end if
                  write(irefwr, 570 ) ((x(i,j),j=1,3), i=1, 4)
570               format ( ( 1x, ss, 3(1pe14.5e2) ) )
                  call erclos ( 'elp626' )
                  go to 1000
               end if
            end do

         end if

!        --- gauss points must be filled

         do j = 1, ndim
            do k = 1, m
               som = 0d0
               do i = 1, 4
                  som = som + phi(i,k) * x(i,j)
               end do
               xgauss(k,j) = som
            end do
         end do

         if ( iweigo==1 ) then

!        --- iweigo = 1  fill weights

            do i = 1, m
               w(i) = 0.25d0*delta(i)
            end do

         end if

         if ( ideriv==1 ) then

!        --- ideriv = 1   fill derivatives

            do k = 1, m
               dxdxi(k,1,3) = dxdxi(k,1,3)/delta(k)
               dxdxi(k,2,3) = dxdxi(k,2,3)/delta(k)
               dxdxi(k,3,3) = dxdxi(k,3,3)/delta(k)
            end do
            do k = 1, m
               dxidx(k,1,1) = (dxdxi(k,2,2)*dxdxi(k,3,3) - &
                               dxdxi(k,2,3)*dxdxi(k,3,2))/delta(k)
               dxidx(k,1,2) = (dxdxi(k,2,3)*dxdxi(k,3,1) - &
                               dxdxi(k,2,1)*dxdxi(k,3,3))/delta(k)
               dxidx(k,1,3) = (dxdxi(k,2,1)*dxdxi(k,3,2) - &
                               dxdxi(k,2,2)*dxdxi(k,3,1))/delta(k)
               dxidx(k,2,1) = (dxdxi(k,3,2)*dxdxi(k,1,3) - &
                               dxdxi(k,3,3)*dxdxi(k,1,2))/delta(k)
               dxidx(k,2,2) = (dxdxi(k,3,3)*dxdxi(k,1,1) - &
                               dxdxi(k,3,1)*dxdxi(k,1,3))/delta(k)
               dxidx(k,2,3) = (dxdxi(k,3,1)*dxdxi(k,1,2) - &
                               dxdxi(k,3,2)*dxdxi(k,1,1))/delta(k)
            end do
            do j = 1, 3
               do k = 1, m
                  do i = 1, 4
                     dphidx(i,k,j) = dpdxi(i,k)*dxidx(k,1,j) + &
                                     dpdeta(i,k)*dxidx(k,2,j)
                  end do
               end do
            end do

         end if

      end if

1000  if ( ierror/=0 ) call instop

      end
