      subroutine elp631curved ( x, index1, w, wh, comppsi, dpsidx, &
                                dphidx, psiksi, phiksi, idx, mvert )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 19-06-2013
!
!   copyright (c) 2013-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute derivatives of basis functions for quadratic tetrahedron
!     Curved element
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     3d
!     quadratic
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmoduledebug
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(n), idx(nvert), mvert
      double precision x(n,3), dpsidx(nvert,mvert,3), dphidx(n,m,3), &
                       phiksi(n,m,3), wh(m), psiksi(nvert,mvert,3)
      logical comppsi

!     comppsi        i    if true the linear basis functions must be evaluated
!     dphidx         o    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                              d phi_i/ dx_j(xg_k) = dphidx (i,k,j);
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dpsidx         o    Array of length n x m x l  containing the derivatives
!                         of the linear basis functions in the sequence:
!                         d psi / dx (xg ) = psix (i,k,1);
!                              i        k
!                         d psi / dy (xg ) = psix (i,k,2);
!                              i        k
!                         d psi / dz (xg ) = psix (i,k,3);
!                              i        k
!     idx            i    Array containing the sequence of the vertices
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     mvert          i    Number of integration points for dpsidx
!                         In case of a tetrahedron: 1
!     phiksi         i    array of length n x m to store the values of dphi/dxsi
!                         according to:
!                                     k
!                         dphi /dxsi(x ) = phiksi(i,k)
!                             i
!     psiksi         i    Array of length 8 x m x 3 containing the values of the
!                         derivatives of the basis functions in the Gauss points in
!                         the reference element
!     w              o    array of length m containing the weights for
!                         integration
!     wh             i    Array of length 4 containing the values of the
!                         Jacobian in the vertices or for the Gauss integration
!                         rules work array to store some help variables
!     x              i    array of length 3 x 10 containing the x- y- and
!                         z-coordinates of the nodal points with respect to the
!                         local numbering in the sequence:
!                         x  = x (i,1);  y  = x (i,2);  z  = x (i,3);
!                          i              i              i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, i1, i2, indlc(4), isign, jsign, j1, j2, k, ni
      double precision w(m), dxdksi(m,3,3), detinv(m,3,3)
      logical check

!     check          Logical variable that is used to indicate if the Jacobian
!                    is zero (true) or not (false)
!     detinv         Work array of length m x 3 x 3, to store the inverse
!                    of the jacobian matrix as follows:
!                     -1    k
!                    J   ( x ) =  detinv ( k, i, j )
!                     ij
!     dxdksi         d x_i / d xsi integration points
!     i              Counting variable
!     i1             help parameter in computation
!     i2             help parameter in computation
!     indlc          Integer array containing the cyclic permutation of
!                    1,2,3
!     isign          Help parameters to check the sign of the Jacobian in
!                    the integration points
!     j              Counting variable
!     j1             help parameter in computation
!     j2             help parameter in computation
!     jsign          Help parameter to compute sign of the Jacobian
!     k              Counting variable
!     ni             Upper boundary in the computation for the loop
!                    variable i
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     105   Jacobian of element is zero
!     230   Jacobian of element is zero or has changing sign
!    1540   Jacobian zero or changing sign
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp631curved' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp631curved'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      indlc(1) = 2
      indlc(2) = 3
      indlc(3) = 1
      indlc(4) = 2

!     --- First compute dx/dksi
!         The first component refers to the integration point
!         The second one to the derivative
!         The third one to the coordinate direction

      do i = 1, 3
         do j = 1, 3
            do k = 1, m
               dxdksi(k,i,j) = dot_product ( x(1:n,j), phiksi(1:n,k,i) )
            end do  ! k = 1, m
         end do  ! j = 1, 3
      end do  ! i = 1, 3

      if ( ideriv==1 ) then

!     --- ideriv = 1,  compute all factors e^(ij)

         ni = 3

      else

!     --- ideriv = 0,  compute only the factors e^(1j)

         ni = 1

      end if

!     --- Compute determinant in integration points
!         Store temporarily in array w
!         compute inverse of Jacobian

      do i = 1, ni

         i1 = indlc(i)
         i2 = indlc(i+1)
         do k = 1, m
            do j = 1, 3
               j1 = indlc(j)
               j2 = indlc(j+1)
               detinv(k,j,i) = dxdksi(k,j1,i1)*dxdksi(k,j2,i2)- &
                               dxdksi(k,j2,i1)*dxdksi(k,j1,i2)
            end do  ! j = 1, 3
         end do  ! k = 1, m

      end do  ! i = 1, ni

      do k = 1, m
         w(k) = dxdksi(k,1,1)*detinv(k,1,1)+dxdksi(k,2,1)* &
                detinv(k,2,1)+dxdksi(k,3,1)*detinv(k,3,1)
      end do  ! k = 1, m

!     --- check sign of jacobian

      isign = nint( sign (1d0, w(1)) )
      if ( checksign .and. isign<0 ) then
         write(irefwr,*) 'area of element ', ielem, ' is negative'
         write(irefwr,*) 'area = ', w(1)
         call prinin ( index1, n, 'nodes' )
         call prinrl2 ( x, n, 3, 'coordinates' )
      end if
      check = .false.

      do i = 1, m
         if ( abs(w(i))<=0d0 ) check = .true.
         jsign = nint(sign(1d0, w(i)))
         if( jsign/=isign ) then

            if ( isubr==0 ) then
               call errint ( ielem, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 230, 3, 0, 0 )
            else
               call errint ( ielgrp, 1 )
               call errint ( ielem, 2 )
               call errchr ( 'volume', 1 )
               call errsub ( 1540, 2, 0, 1 )
            end if
            call prinin(index1,n,'node numbers')
            call prinrl2(x,n,3,'co-ordinates_1')
            call prinrl ( w, m, 'w' )
            go to 1000
         end if

      end do  ! i = 1, m

      if ( check ) then

         if ( isubr==0 ) then
            call errint ( ielem, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 105, 3, 0, 0 )
         else
            call errint ( ielgrp, 1 )
            call errint ( ielem, 2 )
            call errchr ( 'volume', 1 )
            call errsub ( 1540, 2, 0, 1 )
         end if
         call prinin(index1,27,'node numbers')
         call prinrl2(x,27,3,'co-ordinates_2')
         call prinrl ( w, m, 'w' )

         go to 1000

      end if  ! ( check )

      if ( ideriv==1 ) then

!     --- ideriv = 1   Fill derivatives

         do i = 1, n
            do k = 1, m
               do j = 1, 3
                  dphidx(i,k,j) = dot_product(phiksi(i,k,1:3), &
                                              detinv(k,1:3,j))/w(k)
               end do  ! j = 1, 3
            end do  ! k = 1, m
         end do  ! i = 1, n

         if ( comppsi ) then

            do i = 1, nvert
               do k = 1, mvert
                  do j = 1, 3
                     dpsidx(idx(i),k,j) = dot_product(psiksi(i,k,1:3), &
                                                  detinv(k,1:3,j))/w(k)
                  end do  ! j = 1, 3
               end do  ! k = 1, m
            end do  ! i = 1, n

         end if  ! ( comppsi )

      end if  ! ( ideriv==1 )

      w(1:m) = abs( w(1:m) ) * wh(1:m)

1000  call erclos ( 'elp631curved' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp631curved'

      end if  ! ( debug )

      end
