      subroutine elp635 ( coor2d, x, w, dphidx, xgauss, phi, index1, &
                          psi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.0  date 10-06-2013 Use subroutines
!        version  1.9  date 09-03-2013 Correction declaration
!
!   copyright (c) 1996-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute basis functions and related quantities for extended
!     linear tetrahedron
!     Computed are:     weights for numerical integration
!                       values of derivatives of basis functions
!                       values of basis functions in integration points
!     ELP635 is called by several types of element subroutines
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmoduledebug
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(*)
      double precision coor2d(3,*), x(5,3), w(*), dphidx(5,m,3), &
                       xgauss(m,3), phi(5,m), psi(4,m)

!     coor2d     o     array of length ndim x npointcac containing the co-ordinates
!                    of the nodal points with respect to the global numbering
!                    x  = coor2d (1,i);  y  = coor2d (2,i);  z  = coor2d (3,i)
!                     i                 i                 i
!     dphidx   o     Array of length n x m x ndim  containing the derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = dphidx (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = dphidx (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = dphidx (i,k,3);
!                         i        k
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     index1   i     Array of length inpelm containing the point numbers
!                    of the nodal points in the element
!     phi      o     Array of length 5 x m containing the values of the
!                    basis functions in the Gauss points in the sequence:
!                           k
!                    phi (xg ) = phi (i,k)
!                       i
!                    Array phi is only filled when ifirst = 0 and gauss
!                    integration is applied
!     psi      o     array of length 4 x m containing the values of the linear
!                    basis functions in the gauss points in the sequence:
!                           k
!                    psi (xg ) = psi (i,k)
!                       i
!                    array psi is only filled when ifirst = 0 and gauss
!                    integration is applied
!     w        o     array of length m containing the weights for integration
!     x       i/o    array of length 3 x 5 containing the x- y- and
!                    z-coordinates of the nodal points with respect to the
!                    local numbering in the sequence:
!                    x  = x (i,1);  y  = x (i,2);  z  = x (i,3);
!                     i              i              i
!     xgauss   o     Double precision array of length m x ndim in which the
!                    co-ordinates of the integration points are stored
!                    according to:
!                    x  = xg(i,1);  y  = xg(i,2);  z  = xg(i,3)
!                     i              i              i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision djacob, som, gradc, phicen, delta(4,3), e(4,3)
      integer i, j, k, iven, ni, indlc(5), ind1(4), iweigo, intrule
      character (len=27) text
      save iweigo

!     delta          Work array of length 4 x 3, to store the factors
!                         ij
!                    delta.    See method
!     djacob         jacobian of element
!     e              Work array of length 4 x 3, to store the factors
!                     ij
!                    e.        See method
!     gradc          Gradient of lambda_1 * lambda_2 * lambda_3
!     i              Counting variable
!     ind1           Array of length 4 to store the cyclic permutation of 1 - 3
!     indlc          Integer array containing the cyclic permutation of
!                    1,2,3,4,5
!     intrule        Actual quadrature rule
!     iven           help parameter to compute (-1)
!     iweigo         if 1 the weights must be filled
!     j              Counting variable
!     k              Counting variable
!     ni             Upper boundary in the computation for the loop variable i
!     phicen         Value of alam1*alam2*alam3 in integration point
!     som            Help variable to compute a sum
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     105   Jacobian of element is zero
!    1540   Jacobian zero or changing sign
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!        To compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!             ij     i+1     i
!        delta   =  x    -  x       cyclic;  i = 1,2,3,4;  j =  1,2,3
!                    j       j
!         ij        i        i+1,j+1     i+2,j+2        i+1,j+2     i+1,j+2
!        e   =  (-1)  [ delta       delta        - delta       delta        ]
!                        11  11        12  12       13  13
!        djacob = - delta   e   - delta   e  - delta   e
!                             ij
!        d lambda  / dx   =  e   /  djacob
!                i     j
!        lamc = lambda lambda lambda lambda
!                     1      2      3      4
!        phi  =  lambda  - 64 lamc  i=1,2,3,4   (basis functions:1,2,3,4)
!           i          i
!        phi  = 256 lamc
!           5
!        grad phi  = grad lambda  - 64 grad lamc
!                i              i
!        grad phi    = 256 grad lamc
!                5
!        grad lamc = lambda_2 lambda_3 lambda_4 grad lambda_1 + &
!                    lambda_3 lambda_4 lambda_1 grad lambda_2 + &
!                    lambda_4 lambda_1 lambda_2 grad lambda_3 + &
!                    lambda_1 lambda_2 lambda_3 grad lambda_4
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data indlc / 2,3,4,1,2 /, ind1 / 2,3,1,2 /
      data text / 'Extended linear tetrahedron' /
! ======================================================================
!
      call eropen ( 'elp635' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp635'
         write(irefwr,1) 'irule, ifirst', irule, ifirst
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      intrule = mod(irule,100)
      if ( ifirst==0 ) then

!     --- ifirst = 0  (First call)

         iweigo = mod(iweigh,10)
         if ( jcart/=1 ) then

!        --- Error 1769: Co-ordinate system not available

            call errorelvar ( jcart-1, 1769, text, 0 )

         end if
         if ( intrule/=1 .and. intrule<3.or. intrule>4 ) then

!        --- Error 1770: Integration rule not available

            call errorelvar ( intrule, 1770, text, 1 )
            intrule = 3

         end if
         if ( intrule==3 .and. m/=4 .or. &
              intrule==4 .and. m/=5) then

!        --- Error 1424: m and integration rule do not match

            call errorelvar2 ( m, intrule, 1424, text, 0 )

         end if

      end if

      if ( icoor==1 ) then

!     --- icoor = 1   Fill co-ordinates of nodes in array x

         do k = 1, 3
            do i = 1, inpelm
               x(i,k) = coor2d(k,index1(i))
            end do
            if ( inpelm==4 ) then
               x(5,k) = ( x(1,k)+x(2,k)+x(3,k)+x(4,k) ) * 0.25d0
            end if
         end do

      end if

      if ( iweigo==1 .or. ideriv==1 ) then

!     --- ideriv = 1  Compute derivatives or iweigo = 1: weights
!         Compute the quantities delta^(ij) ,  e^(ij) and djacob

         do k = 1, 3
            do i = 1, 4
               delta(i,k) = x(indlc(i),k) - x(i,k)
            end do
         end do

         if ( ideriv==1 ) then

!        --- ideriv = 1,  compute all factors e^(ij)

            ni = 4

         else

!        --- ideriv = 0,  compute only the factors e^(1j)

            ni = 1

         end if

         iven = -1
         do i = 1, ni
            do k = 1, 3
               e (i,k) = iven * ( delta ( indlc(i), ind1(k) ) * &
                         delta ( indlc(i+1), ind1(k+1) ) - &
                         delta ( indlc(i), ind1(k+1) ) * &
                         delta ( indlc(i+1), ind1(k) ) )
            end do
            iven = - iven
         end do

         djacob = - delta(1,1) * e(1,1) - delta(1,2) * e(1,2) &
                  - delta(1,3) * e(1,3)

!        --- Check if djacob is non-zero

         if ( checksign .and. djacob<0d0 ) then
            write(irefwr,*) 'area of element ', ielem, ' is negative'
         end if
         if ( abs(djacob)<=0d0 ) then

            if ( isubr==0 ) then
               call errint ( ielem, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 105, 3, 0, 0 )
            else
               call errint ( ielgrp, 1 )
               call errint ( ielem, 2 )
               call errchr ( 'volume', 1 )
               call errsub ( 1540, 2, 0, 1 )
            end if
            write ( irefwr, 215 ) ((x(k,j),j=1,3), k=1,5)
215         format ( ( 1x, ss, 3(1pe14.5e2) ) )

            go to 1000
         end if
      end if

      if ( iweigo==1 ) then

!     --- iweigo = 1  Fill weights

         if ( intrule==1 ) then

!        --- intrule = 1  Newton Cotes rule

            som = abs(djacob)/24d0
            phicen = abs(djacob)/5040d0
            do k = 1, 4
               w(k) = som-64d0*phicen
            end do
            w(5) = 256d0*phicen

         else if ( intrule==3 ) then

!        --- intrule = 3  Four-point Gauss rule

            do k = 1, 4
               w(k) = abs(djacob) * wgs7(k)
            end do

         else

!        --- intrule = 4  Five-point Gauss rule

            do k = 1, 5
               w(k) = abs(djacob) * wgs8(k)
            end do

         end if

      end if

      if ( ideriv==1 ) then

!     --- ideriv = 1   Fill derivatives

         if ( intrule==1 ) then

!        --- Newton Cotes rule

            do j = 1, 3
               do k = 1, m
                  do i = 1, 4
                     dphidx(i,k,j) = e(i,j)/djacob
                  end do
                  dphidx(5,k,j) = 0d0
               end do
            end do

         else if ( intrule==2 ) then

!        --- four-point Gauss rule

            do j = 1, 3
               do k = 1, m
                  gradc = alam7(2,k)*alam7(3,k)*alam7(4,k)*e(1,j) + &
                          alam7(3,k)*alam7(4,k)*alam7(1,k)*e(2,j) + &
                          alam7(4,k)*alam7(1,k)*alam7(2,k)*e(3,j) + &
                          alam7(1,k)*alam7(2,k)*alam7(3,k)*e(4,j)
                  do i = 1, 4
                     dphidx(i,k,j) = (e(i,j)-64d0*gradc)/djacob
                  end do
                  dphidx(5,k,j) = 256d0*gradc/djacob
               end do
            end do

         else

!        --- five-point Gauss rule

            do j = 1, 3
               do k = 1, m
                  gradc = alam8(2,k)*alam8(3,k)*alam8(4,k)*e(1,j) + &
                          alam8(3,k)*alam8(4,k)*alam8(1,k)*e(2,j) + &
                          alam8(4,k)*alam8(1,k)*alam8(2,k)*e(3,j) + &
                          alam8(1,k)*alam8(2,k)*alam8(3,k)*e(4,j)
                  do i = 1, 4
                     dphidx(i,k,j) = (e(i,j)-64d0*gradc)/djacob
                  end do
                  dphidx(5,k,j) = 256d0*gradc/djacob
               end do
            end do

         end if

      end if

      if ( ifirst==0 .and. iphi==1 ) then

!     --- ifirst = 0  First call of element subroutine, and iphi = 1
!         Fill array phi

         if ( intrule==1 ) then

!        --- intrule = 1  Newton Cotes rule

            do k = 1, 4
               do i = 1, 4
                  psi(i,k) = 0d0
               end do
               psi(k,k) = 1d0
            end do

            do k = 1, 5
               do i = 1, 5
                  phi(i,k) = 0d0
               end do
               phi(k,k) = 1d0
            end do

         else if ( intrule==3 ) then

!        --- intrule = 3  four-point Gauss rule

            do k = 1, 4
               phicen = alam7(1,k)*alam7(2,k)*alam7(3,k)*alam7(4,k)
               phi(5,k) = 256d0 * phicen
               do i = 1, 4
                  phi(i,k) = alam7(i,k) - 64d0*phicen
                  psi(i,k) = alam7(i,k)
               end do
            end do

         else

!        --- intrule = 4  five-point Gauss rule

            do k = 1, 5
               phicen = alam8(1,k)*alam8(2,k)*alam8(3,k)*alam8(4,k)
               phi(5,k) = 256d0 * phicen
               do i = 1, 4
                  phi(i,k) = alam8(i,k) - 64d0*phicen
                  psi(i,k) = alam8(i,k)
               end do
            end do

         end if

      end if

      if ( igausp==1 ) then

!     --- gauss points must be filled

         if ( intrule==1 ) then

!        --- intrule = 1  Newton Cotes rule

               do k = 1, 3
                  do i = 1, 5
                     xgauss(i,k)=x(i,k)
                  end do
               end do

         else if ( intrule==3 ) then

!        --- intrule = 3  four-point Gauss rule

            do j = 1, 3
               do k = 1, 4
                  som = 0d0
                  do i = 1, 4
                     som = som + alam7(i,k) * x(i,j)
                  end do
                  xgauss(k,j) = som
               end do
            end do

         else

!        --- intrule = 4  five-point Gauss rule

            do j = 1, 3
               do k = 1, 5
                  som = 0d0
                  do i = 1, 4
                     som = som + alam8(i,k) * x(i,j)
                  end do
                  xgauss(k,j) = som
               end do
            end do

         end if

      end if

      if ( itype>=902 .and. itype<=903 ) then

!     --- Special case of Navier-Stokes element
!         w(1..m ) contains the weights
!         w(m+1 .. m+n) the diagonal of the mass matrix
!         In this case it is sufficient to use the integration rule available

         do i = 1, n
            som = 0d0
            do k = 1, m
               som = som+w(k)*phi(i,k)**2
            end do  ! k = 1, m
            w(m+i)= som
         end do  ! i = 1, n

      end if  ! ( itype>=902 .and. itype<=903 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp635'

      end if  ! ( debug )
      call erclos ( 'elp635' )

      end
