      subroutine elp639 ( coor2d, x, w, dphidx, xgauss, phi, index1, &
                          phiksi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 22-05-2014
!
!   copyright (c) 2014-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute basis functions and related quantities for triquadratic wedge
!     Computed are:     weights for numerical integration
!                       values of derivatives of basis functions
!                       values of basis functions in integration points
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     wedge
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmoduledebug
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(*)
      double precision coor2d(3,*), x(18,3), w(m), dphidx(18,m,3), &
                       xgauss(m,3), phi(18,m), phiksi(18,m,3)

!     coor2d     i     array of length 3 x number of nodal points containing
!                    the co-ordinates of the nodal points with respect to
!                    the global numbering
!                    x  = coor2d (1,i);  y  = coor2d (2,i);  z  = coor2d (3,i)
!                     i                 i                 i
!     dphidx   o     Array of length 6 x m x 3  containing the derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = dphidx (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = dphidx (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = dphidx (i,k,3);
!                         i        k
!     index1   i     Array of length 6 containing the nodal point numbers
!                    of the element
!     phi      o     Array of length 6 x m containing the values of the
!                    basis functions in the Gauss points in the sequence:
!                    phi (xg ) = phi (i,k)
!                       i   k
!                    Array phi is only filled when ifirst = 0 and gauss
!                    integration is applied
!     phiksi   o     Array of length 6 x m x 3 containing the values of the
!                    derivatives of the basis functions in the Gauss points in
!                    the reference element.
!                    Array phi is only filled when ifirst = 0 and must be kept
!                    by the program
!     w        o     array of length m containing the weights for the
!                    numerical integration
!                    During the computation w is temporarily used to store
!                    the Jacobian in the integration points, so even
!                    if there is no need to compute the weights for the
!                    integration, still w should be declared
!     x       i/o    array of length 3 x 6 containing the x- y- and
!                    z-coordinates of the nodal points with respect to the
!                    local numbering in the sequence:
!                    x  = x (i,1);  y  = x (i,2);  z  = x (i,3);
!                     i              i              i
!     xgauss   o     Array of length m x 3 containing the co-ordinates of
!                    the gauss points.  Array xgauss is only filled when Gauss
!                    integration is applied.
!                    xg  = xgauss (i,1);  yg  = xgauss (i,2);
!                      i                    i
!                    zg  = xgauss (i,3);
!                      i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, isign, jsign, iweigo, intrule, istep
      double precision weight(18), dxdxi(m,3,3), detinv(m,3,3)
      logical :: check
      save weight

!     check          Logical variable that is used to indicate if the Jacobian
!                    is zero (true) or not (false)
!     detinv         inverse of determinant in each integration point
!     dxdxi          dx_i/dxi_j in each integration point
!     i              Counting variable
!     intrule        Actual integration rule
!     isign          Help parameters to check the sign of the Jacobian in
!                    the integration points
!     istep          Step size of weights (newton cotes 2, else 1)
!     iweigo         if 1 the weights must be filled
!     j              Counting variable
!     jsign          Help parameter to compute sign of the Jacobian
!     k              Counting variable
!     weight         Help array to store the weights in the reference element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELP639FIRST    Compute basis functions and weights for triquadratic wedge
!     ELPCOOR        fill co-ordinates of nodes in array x
!     ELPGAUSS       Fill Gauss coordinates
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     105   Jacobian of element is zero
!     230   Jacobian of element is zero or has changing sign
!    1540   Jacobian zero or changing sign
! **********************************************************************
!
!                       PSEUDO CODE
!
!     To compute the derivatives of the basis functions, the jacobian
!     of the element and the weights, the following definitions are used:
!     The lower triangle is defined by the points 1..6, the upper
!     triangle by 13..18
!     The basis functions are mapped on a unit wedge in the (xi, eta ,zeta )
!     space with basis functions:
!
!     phi_1..6 = psi_1..6(xi,eta) chi_1(zeta)
!     phi_7..12 = psi_1..6(xi,eta) chi_2(zeta)
!     phi_13..18 = psi_1..6(xi,eta) chi_3(zeta)
!
!     with psi_1..6 the basis functions for a quadratic triangle
!     and ch_1..3 the basis functions for a quadratic line
!
!     Hence the triangles are in the xi,eta planes and the upwards direction
!     is defined as zeta
!
!     The weights for the numerical integration rule are given as follows:
!     Let J be the Jacobian in the integration points then:
!
!     intrule = 1: (Newton Cotes rule)  w = 1/12 J(x_k)
!     intrule = 2: (Four point Gauss rule in xi-eta dir and 2 point
!                   Gauss rule in zeta dir)  w = 1/2 J(x_k)
!     intrule = 3: (Seven point Gauss rule in xi-eta dir and 2 point
!                   Gauss rule in zeta dir)  w = 1/2 J(x_k)
!
!
!     dphi_dxi_j can be computed immediately from the standard basis functions
!
!     dx/dxi_j = sum_k x_k dphi_k/dxi_j
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elp639' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp639'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get some constants

      intrule = mod(irule,100)
      iweigo = mod(iweigh,10)

      if ( debug ) write(irefwr,1) 'intrule, iweigo', intrule, iweigo

      if ( ifirst==0 .or. ifirst==2 ) then

!     --- ifirst = 0  (First call) or
!         ifirst = 2, first call to store dphidx in the nodes
!                     phiksi is filled differently from the previous
!                     values, hence use another array
!                     This value of ifirst should be used in combination with
!                     ifirst = 3 and is meant for elup04 only

         call elp639first ( phi, phiksi, weight )

         if ( debug ) then
            call prinrl2 ( phi, n, m, 'phi' )
            do j = 1, 3
               call prinrl2 ( phiksi(1,1,j), n, m, 'phiksi' )
            end do  ! j = 1, 3
            call prinrl ( w, m, 'w' )
         end if  ! ( debug )

      end if

!     --- fill co-ordinates of nodes in array x

      call elpcoor ( icoor, coor2d, index1, x, n, ndim, n )

      if ( debug ) call prinrl1 ( x, n, 3, 'x' )

!     --- First store dx / dksi etc. in array dxdxi

      do j = 1, ndim
         do i = 1, m
            do k = 1, 3
               dxdxi(i,j,k) = &
                  dot_product ( x(1:n,k), phiksi(1:n,i,j) )
            end do  ! k = 1, 3
         end do  ! i = 1, m
      end do  ! j = 1, ndim

      if ( debug ) then
         do j = 1, 3
            call prinrl1 ( dxdxi(1,1,j), n, 3, 'dxdxi' )
         end do  ! j = 1, 3
      end if  ! ( debug )

!     --- Fill gauss points

      call elpgauss ( ndim, intrule, x, xgauss, phi )

      if ( debug ) call prinrl1 ( xgauss, m, 3, 'xgauss' )

      if ( iweigo==1 .or. ideriv==1 ) then

!     --- ideriv = 1  Compute derivatives or iweigo = 1: weights
!                     Compute determinant in integration points
!                     Store temporarily in array w
!                     compute inverse of Jacobian

         do k = 1, m
            detinv(k,1,1) = dxdxi(k,2,2)*dxdxi(k,3,3)- &
                            dxdxi(k,3,2)*dxdxi(k,2,3)
            detinv(k,2,1) = dxdxi(k,3,2)*dxdxi(k,1,3)- &
                            dxdxi(k,1,2)*dxdxi(k,3,3)
            detinv(k,3,1) = dxdxi(k,1,2)*dxdxi(k,2,3)- &
                            dxdxi(k,2,2)*dxdxi(k,1,3)
            w(k) = dxdxi(k,1,1)*detinv(k,1,1)+dxdxi(k,2,1)* &
                   detinv(k,2,1)+dxdxi(k,3,1)*detinv(k,3,1)
         end do

         if ( debug ) then
            call prinrl ( w, m, 'w' )
         end if  ! ( debug )

         if ( ideriv==1 ) then

!        --- ideriv = 1,  compute all factors of inverse

            do k = 1,m
               detinv(k,1,2) = dxdxi(k,2,3)*dxdxi(k,3,1)- &
                               dxdxi(k,3,3)*dxdxi(k,2,1)
               detinv(k,2,2) = dxdxi(k,3,3)*dxdxi(k,1,1)- &
                               dxdxi(k,1,3)*dxdxi(k,3,1)
               detinv(k,3,2) = dxdxi(k,1,3)*dxdxi(k,2,1)- &
                               dxdxi(k,2,3)*dxdxi(k,1,1)
               detinv(k,1,3) = dxdxi(k,2,1)*dxdxi(k,3,2)- &
                               dxdxi(k,3,1)*dxdxi(k,2,2)
               detinv(k,2,3) = dxdxi(k,3,1)*dxdxi(k,1,2)- &
                               dxdxi(k,1,1)*dxdxi(k,3,2)
               detinv(k,3,3) = dxdxi(k,1,1)*dxdxi(k,2,2)- &
                               dxdxi(k,2,1)*dxdxi(k,1,2)
            end do

         end if

!        --- check sign of jacobian

         isign = nint( sign (1d0, w(2)) )
         istep = 1
         if ( intrule==1 ) istep = 2
         if ( checksign .and. isign<0 ) then
            write(irefwr,*) 'area of element ', ielem, ' is negative'
         end if
         check = .false.
         do i = istep, m, istep
            if ( abs(w(i))<=0d0 ) check = .true.
            jsign = nint(sign(1d0, w(i)))
            if( jsign/=isign ) then
               if ( isubr==0 ) then
                  call errint ( ielem, 1 )
                  call errint ( itype, 2 )
                  call errint ( ielgrp, 3 )
                  call errsub ( 230, 3, 0, 0 )
               else
                  call errint ( ielgrp, 1 )
                  call errint ( ielem, 2 )
                  call errchr ( 'volume', 1 )
                  call errsub ( 1540, 2, 0, 1 )
               end if
               call prinin(index1,n,'node numbers')
               call prinrl2(x,n,3,'co-ordinates')
               go to 1000
            end if
         end do
         if ( check ) then
            if ( isubr==0 ) then
               call errint ( ielem, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 105, 3, 0, 0 )
            else
               call errint ( ielgrp, 1 )
               call errint ( ielem, 2 )
               call errchr ( 'volume', 1 )
               call errsub ( 1540, 2, 0, 1 )
            end if
            call prinin(index1,n,'node numbers')
            call prinrl2(x,n,3,'co-ordinates')
            go to 1000
         end if

      end if

      if ( ideriv==1 ) then

!     --- ideriv = 1   Fill derivatives

         do i = 1, n
            do k = 1, m
               dphidx(i,k,1:3) = (phiksi(i,k,1)*detinv(k,1,1:3)+ &
                                  phiksi(i,k,2)*detinv(k,2,1:3)+ &
                                  phiksi(i,k,3)*detinv(k,3,1:3))/w(k)
            end do
         end do

      end if

      if ( iweigo==1 ) w(1:m) = abs( w(1:m) ) * weight(1:m)

1000  call erclos ( 'elp639' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( w, m, 'w' )
         write(irefwr,*) 'End elp639'

      end if  ! ( debug )

      end
