      subroutine elp640(ichois, xref, wref, d1ref, d2ref, m)
!
!***********************************************************************
!
!     programmer Luc Timmermans
!     version 4.0    date  18-09-1992  extension 2nd order derivatives
!     version 3.0    date  30-07-1992  change in parameterlist
!     version 2.1    date  21-01-1992  New definition of dref
!
!***********************************************************************
!
!     compute quantities for one-dimensional spectral reference
!     element [-1,1]
!
!     computed are:  Gauss-Lobatto-Legendre points
!                    Gauss-Lobatto-Legendre weights for numerical
!                    integration
!                    values of the derivatives of the basis functions
!                    in the GLL-points
!
!     ELP640 is called by the subroutine ELM600
!
!***********************************************************************
!
!     INPUT/OUTPUT PARAMETERS

      implicit none
      integer ichois, m
      double precision xref(m), wref(m), d1ref(m,m), d2ref(m,m)
!
!     ichois     i     parameter to indicate whether the derivatives
!                      must be computed (1,2) or  not (0)
!                      ichois = 0 : only points & weights
!                      ichois = 1 : points & weights + 1st order derivs
!                      ichois = 2 : points & weights + 1st + 2nd order derivs
!     m          i     number of integration/interpolation points in
!                      the element
!     xref       o     array of length m containing the coordinates of
!                      Legendre-Gauss-Lobatto points
!     wref       o     array of length m containing the Legendre
!                      Gauss-Lobatto weights for numerical integration
!     d1ref      o     array of length m x m containing the values of
!                      the derivatives of the basis functions in
!                      the GLL-points, &
!     d2ref      o     array of length m x m containing the values of
!                      the 2nd order derivatives of the basis functions in
!                      the GLL-points, &
!
!                      d1ref(k,j) = dphi(j)/dxref(k)
!                      d2ref(k,j) = d2phi(j)/dxref2(k)
!
!***********************************************************************
!
!     LOCAL PARAMETERS

      integer i, j, npm
      double precision rm, rn, xi, xj, poly0, poly1, poly2, rdum0, &
                       rdum1, rdum2, rdum3, rdum4
!
!     i         counting variable
!     j         counting variable
!                th
!     npm       (-1)**m
!     rm        double precision value of m
!     rn        rm - 1d0
!     xi        i   Gauss-Lobatto point
!                th
!     xj        j   Gauss-Lobatto point
!     poly0     value of Legendre polynomial of order m-1 in a
!               Gauss-Lobatto point
!     poly1     likewise
!     poly2     likewise
!     rdum0     dummy variable
!     rdum1     dummy variable
!     rdum2     dummy variable
!     rdum3     dummy variable
!     rdum4     dummy variable
!
!***********************************************************************
!
!     SUBROUTINES CALLED
!
!     ELLGLP    compute the Legendre Gauss-Lobatto points
!     ELLEGP    compute the Legendre polynomials
!
!***********************************************************************

!     * compute Legendre Gauss-Lobatto points

      call ellglp(xref, m)
      rm = dble(m)

!     * compute Legendre Gauss-Lobatto weights

      do 100 i = 1, m
!        * compute value of the Legendre polynomial of order m-1 in
!               th
!          the i   Gauss-Lobatto point

         xi = xref(i)
         call ellegp(m-1, poly0, rdum0, rdum1, rdum2, rdum3, rdum4, xi)

!        * compute weight

         wref(i) = 2d0/((rm - 1d0)*rm*poly0*poly0)
100   continue

!     * check whether the derivatives must be computed

      if (ichois.eq.0) return

!     * compute values of the derivatives of the basisfunctions
!       in the Gauss-Lobatto points

      do 300 i = 1, m
!        * compute value of the Legendre polynomial of order m-1 in
!               th
!          the i   Gauss-Lobatto point

         xi = xref(i)
         call ellegp(m-1, poly1, rdum0, rdum1, rdum2, rdum3, rdum4, xi)

         do 200 j = 1, m
!           * loop over number of basisfunctions

            if (i.eq.j) then

               d1ref(i,j) = 0d0
            else
!              * compute value of the Legendre polynomial of order
!                            th
!                m-1 in the j   Gauss-Lobatto point

               xj = xref(j)
               call ellegp(m-1, poly2, rdum0, rdum1, rdum2, rdum3, &
                           rdum4, xj)
               d1ref(i,j) = poly1/(poly2*(xi - xj))
            end if

200      continue
300   continue

!     * adjust values of derivatives

      d1ref(1,1) = - ((rm - 1d0)*rm)/4d0
      d1ref(m,m) = ((rm - 1d0)*rm)/4d0

      if ( ichois.eq.1 ) return

!     * compute values of the 2nd order derivatives of the basisfunctions
!       in the Gauss-Lobatto points
!
      rn = rm - 1d0
      npm = -1
      if (mod(m-1,2).eq.0) npm = 1

      do 500 i = 1, m
!        * compute value of the Legendre polynomial of order m-1 in
!               th
!          the i   Gauss-Lobatto point

         xi = xref(i)
         call ellegp(m-1, poly1, rdum0, rdum1, rdum2, rdum3, rdum4, xi)

         do 400 j = 1, m
!           * loop over number of basisfunctions

            if (i.eq.j) then
               xj = xref(j)
               if ( j.ne.1 .and. j.ne.m ) then
                  d2ref(i,j) = -rn*(rn+1d0)/(3d0*(1d0-xj*xj))
               else
                  d2ref(i,j) = (rn+2d0)*(rn+1d0)*rn*(rn-1d0)/24d0
               end if
            else
               xj = xref(j)
               call ellegp(m-1, poly2, rdum0, rdum1, rdum2, rdum3, &
                           rdum4, xj)
               if (i.ne.1 .and. i.ne.m) then
                  d2ref(i,j) = -2d0*poly1/(poly2*(xi - xj)*(xi - xj))
               else if (i.eq.1) then
                  d2ref(i,j) = -dble(npm)*rn*(rn+1d0)/ &
                       (2d0*poly2*(-1d0-xj)) - &
                       2d0*dble(npm)/(poly2*(-1d0-xj)*(-1d0-xj))
               else if (i.eq.m) then
                  d2ref(i,j) = rn*(rn+1d0)/(2d0*poly2*(1d0-xj)) - &
                       2d0/(poly2*(1d0-xj)*(1d0-xj))
               end if
            end if

400      continue
500   continue

      end
