      subroutine elpderiv2d ( dxdxi, x, dphidxi, delta, index1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 22-05-2013
!
!   copyright (c) 2013-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the Jacobian of an element and store in delta
!     It is assumed that x, dphi/dxi are given
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     jacobian
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(n)
      double precision x(n,ndim), dphidxi(n,m,2), &
                       dxdxi(m,3,3), delta(m)

!     delta          o    jacobian of element in integration points
!     dphidxi        i    contains derivatives in reference element for gauss
!                         rule.
!                         dphidxi(i,k,l) = d phi_i/ d xi_l (x^k)
!     dxdxi          o    work array of length mx3x3 to store the values of
!                         dx/dxi, dx/deta, dy/dxi and dy/deta according to:
!                         dx/dxi(x_k) = dxdxi(k,1,1)
!                         dx/deta(x_k) = dxdxi(k,1,2)
!                         dy/dxi(x_k) = dxdxi(k,2,1)
!                         dy/deta(x_k) = dxdxi(k,2,2) etc.
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     x              i    array of length n x 2 containing the x- and
!                         y-coordinates of the nodal points with respect to the
!                         local numbering in the sequence:
!                         x  = x (i,1);  y  = x (i,2);
!                          i              i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, j, i, ksign, jsign

!     i              Counting variable
!     j              Counting variable
!     jsign          Help parameter to compute sign of the Jacobian
!     k              Counting variable
!     ksign          Help parameter to compute sign of the Jacobian
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     230   Jacobian of element is zero or has changing sign
!    1540   Jacobian zero or changing sign
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elpderiv2d' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elpderiv2d'
         call prinrl2 ( x, n, ndim, 'x' )
         call prinrl2 ( dphidxi(1,1,1), n, m, 'dphi/dxi' )
         call prinrl2 ( dphidxi(1,1,2), n, m, 'dphi/deta' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do j = 1, ndim
         do i = kstep, m, kstep
            dxdxi(i,j,1) = dot_product(x(1:n,j),dphidxi(1:n,i,1))
            dxdxi(i,j,2) = dot_product(x(1:n,j),dphidxi(1:n,i,2))
         end do
      end do
      do k = kstep, m, kstep
         delta(k) = dxdxi(k,1,1)*dxdxi(k,2,2) - &
                    dxdxi(k,1,2)*dxdxi(k,2,1)
      end do

!     --- check sign of jacobian

      ksign = nint(sign ( 1d0,delta(kstep) ))
      do k = 2*kstep, m, kstep
         jsign = nint(sign ( 1d0, delta(k) ))
         if ( ksign/=jsign ) then
            if ( isubr==0 ) then
               call errint ( ielem, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 230, 3, 0, 0 )
            else
               call errint ( ielgrp, 1 )
               call errint ( ielem, 2 )
               call errchr ( 'surface', 1 )
               call errsub ( 1540, 2, 0, 1 )
            end if
            call prinin(index1,n,'node numbers')
            call prinrl2(x,n,2,'co-ordinates')
            go to 1000
         end if
      end do

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( delta, m, 'delta' )
         write(irefwr,*) 'End elpderiv2d'

      end if  ! ( debug )
      call erclos ( 'elpderiv2d' )

      end
