      subroutine elpuplinmax ( alpha, u, phi, psi, dphidx, x, dcdx, &
                               iflipflop, metupwind )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-08-2000
!
!   copyright (c) 2000-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute modified basis functions for upwind elements
!    Two dimensional elements (triangles)
!    The method of Mizukami and Hughes described in
!    A Petrov-Galerkin finite element method for convection-dominated flows:
!    an accurate upwinding technique for satisfying the maximum principle
!    Comp. Methods in Applied Mechanics and Engineering 50 (1985), pp 181-193
!    is applied
!    To prevent flip-flop during the non-linear ietartion an extra flipflop
!    array has been introduced
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     upwinding
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision alpha(3), u(2), phi(n), psi(n), &
                       dphidx(n,2), dcdx(ndimlc), x(3,2)
      integer iflipflop, metupwind

!     alpha          i    array of length m x 3 in which the values of
!                         alpha  , alpha  , alpha
!                              11       12       22
!                         are stored in the sequence:
!                                                 k
!                         alpha ( k,i ) = alpha (x )   i = 1(1)3, k=1(1)m
!                                              i
!     dcdx           i    Contains the values of dc/dx in the integration points
!     dphidx         i    Array of length n x m x ndim  containing the
!     derivative
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     iflipflop      i    Work array to prevent flip-flop behaviour in non- &
!                         linear upwind
!                         The status of the element with respect to upwind is
!                         stored in this array
!                         here only the position with respect to this element
!                         is given
!     metupwind      i    Type of upwinding method
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     psi            o    Array of length n x m containing the values of the
!                         modified basis functions in the integration points
!     u              i    Array of length m x 2 containing the velocity in the
!                         integration points
!     x              i    x co-ordinate of point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, kpos, jpos, indloc(5), npos
      double precision phimax, beta, a(2), b(2), c(3), &
                       unorm, eps, h, factor, fact1, ugradphi(3), &
                       uparallel(2), ap, bp, normgradc, ugradc, tau

!     a              Vector pointing from point 1 to 2 (cyclic)
!     ap             u . grad c grad c . a
!     b              Vector pointing from point 1 to 3 (cyclic)
!     beta           Parameter beta in method
!     bp             u . grad c grad c . b
!     c              real, coefficient of z
!     eps            Local accuracy
!     fact1          Multiplication factor
!     factor         Multiplication factor
!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     ind            Integer array containing the cyclic permutation of
!                    1,2,3
!     jpos           Last negative inner product u . grad phi found
!     kpos           Last positive inner product u . grad phi found
!     normgradc      length of vector grad c
!     npos           Number of positive inner products u . grad phi
!     phimax         Max j ( |u_i. grad phi_j| )
!     tau            Work array of length n to store the relative distances
!                    between points
!     ugradc         u . grad c
!     ugradphi       u . grad phi
!     unorm          Length of velocity vector
!     uparallel      u . grad c grad c
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!                                                       new
!       The basis functions are modified such that:  phi    = phi + p
!                                                       i        i   i
!       or:
!          new  k          k        k
!       phi   (x ) = phi (x ) + p (x )
!          i            i        i
!       The basis funtions phi are assumed to be filled before
!
!       To compute p_i we follow the method of Mizukmai and Hughes
!
!       p_i is in their notation c_i
!
!       First we detect in which vertex or edge zone the velocity points
!       If in vertex zone then only one of the inner products u . grad phi
!       is positive and p_i is given by ( 2/3, -1/3, -1/3 ) cyclic
!       If in edge zone then two of the inner products u . grad phi
!       are positive
!       This is the difficult case described in the article
!       If u . grad phi = 0 we use the standard upwind
!       else the method depends on the vectors a, b and their inner products
!       with u and also of grad c
!       In this case grad c may flip flop and therefore it is detected if
!       one of the psi's is equal to 1, if so this situation is kept
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data indloc /1, 2, 3, 1, 2 /
! ======================================================================
!
!     --- Fill norm of velocity in vertices into array unorm

      unorm = sqrt ( u(1)**2 + u(2)**2 )

!     --- Store component of alpha tensor in u-direction into eps

      if ( jsecnd.eq.0 ) then

!     --- jsecnd = 0,   eps = 0

         eps = 0d0

      else if ( jsecnd.le.2 ) then

!     --- jsecnd = 1,   eps = alpha = a22  (constant)  a12 = 0

         eps = alpha(1)

      else

!     --- jsecnd > 2,   General case                         T
!                      eps is defined by the formula  eps = u  alpha u
!                                                           -        - &

         if ( unorm.gt.0d0 ) then
            eps = ( u(1)**2*alpha(1)+2d0*u(1)* alpha(2)* u(2) + &
                    u(2)**2*alpha(3) ) / unorm
         else
            eps = alpha(1)
         end if

      end if

!     --- Compute the distance h in the centroid

      phimax = max ( abs(u(1)*dphidx(1,1)+u(2)*dphidx(1,2)), &
                     abs(u(1)*dphidx(2,1)+u(2)*dphidx(2,2)), &
                     abs(u(1)*dphidx(3,1)+u(2)*dphidx(3,2)) )
      if ( phimax.gt.epsmac ) then
         h = unorm/phimax
      else
         h = 0d0
      end if

!     --- Compute beta and factor

      if ( eps.gt.1d-15 ) then

!     --- eps > 1d-10,  hence second order term plays a role

         beta = unorm*h/eps
         if ( beta.le.2d0 ) then

            do i = 1, n
               c(i) = 0d0
            end do
            go to 1000

         end if
         factor = 1d0-2d0/beta

      else

         beta = 0d0
         factor = 1d0

      end if

!     --- Determine the inner products of u and grad phi
!         Compute number of positive elements

      npos = 0
      do i = 1, n
         ugradphi(i) = u(1)*dphidx(i,1)+u(2)*dphidx(i,2)
         if ( ugradphi(i).gt.0d0 ) then

!        --- inner product of u and grad phi positive

            npos = npos+1
            kpos = i

         else if ( ugradphi(i).lt.0d0 ) then

!        --- inner product of u and grad phi negative

            jpos = i

         end if

      end do

!     --- Compute updates c

      fact1 = 1d0/3d0
      do i = 1, n
         c(i) = -fact1
      end do
      if ( npos.eq.1 ) then

!     --- npos = 1,   velocity directs into vertex part i

         c(kpos) = 2d0*fact1

      else

!     --- npos = 2,   velocity directs into edge part i

         ugradc = u(1)*dcdx(1)+u(2)*dcdx(2)
         normgradc = dcdx(1)**2+dcdx(2)**2
         if ( abs(ugradc).le.1d-5*sqrt(normgradc)*unorm ) then

!        --- u . grad c = 0

            tau = fact1/abs(ugradphi(jpos))
            do i = 1, n
               c(i) = tau*ugradphi(i)
            end do

         else

!        --- u . grad c # 0

            ugradc = ugradc/normgradc
            uparallel(1) = ugradc*dcdx(1)
            uparallel(2) = ugradc*dcdx(2)
            a(1) = x(indloc(jpos+1),1)-x(jpos,1)
            a(2) = x(indloc(jpos+1),2)-x(jpos,2)
            b(1) = x(indloc(jpos+2),1)-x(jpos,1)
            b(2) = x(indloc(jpos+2),2)-x(jpos,2)
            ap = a(1)*uparallel(1)+a(2)*uparallel(2)
            bp = b(1)*uparallel(1)+b(2)*uparallel(2)
            if ( ap.gt.0d0 .and. bp.gt.0d0 ) then

!           --- Both inner products positive

               tau = fact1/abs(ugradphi(jpos))
               do i = 1, n
                  c(i) = tau*ugradphi(i)
               end do

            else if ( ap.gt.0d0 ) then

!           --- Inner product with a positive

               c(indloc(jpos+1)) = 2d0*fact1

            else

!           --- Inner product with b positive

               c(indloc(jpos+2)) = 2d0*fact1

            end if

         end if

      end if

      if ( metupwind.eq.10 ) then

!     --- metupwind = 10, fill flipflop array

         jpos = 4
         do i = 1, 3
            if(abs(c(i)).gt.0.66666d0) jpos=i
         end do
         iflipflop = jpos

      else if ( metupwind.eq.11 ) then

!     --- metupwind = 11, use flipflop array

         if ( iflipflop.eq.4 ) then

            jpos = 4
            do i = 1, 3
               if ( abs(c(i)).gt.0.66666d0 ) jpos = i
            end do
            iflipflop=jpos

         else

            do i = 1, n
               c(i) = -fact1
            end do
            c(iflipflop)=2d0/3d0

         end if

      end if

1000  do i = 1, n
         psi(i) = phi(i)+c(i)*factor
      end do
      end
