      subroutine elpupwdist ( u, dphidx, h, unorm, jelp, x, nx, dlamdx )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 27-05-2014 Extension for irule = 22
!        version  2.0  date 04-07-2003 Extra parameter dlamdx
!        version  1.0  date 15-01-2003
!
!   copyright (c) 2003-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute distance h for modified basis functions for upwind elements
!     Three dimensional elements
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     upwinding
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jelp, nx
      double precision h(*), u(m,3), unorm(*), dphidx(n,m,3), x(nx,3), &
                       dlamdx(*)

!     dlamdx         i    Contains derivatives of linear basis functions
!     dphidx         i    Array of length n x m x 3 containing the values of
!                         the derivatives of the basis functions in the
!                         integration points
!     h              o    Contains the approximation of the distance
!                         h in the integration points
!     jelp           i    Indication of the type of basis function subroutine
!                         must be called by ELxxxx. Possible values:
!     nx             i    number of nodes
!     u              i    Array of length m x 3 containing the velocity in the
!                         integration points
!     unorm          i    Length of velocity vector in the integration points
!     x              i    Contains the coordinates of the nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision xcent(6,3)
      integer j

!     j              Counting variable
!     xcent          Contains the coordinates of the vertices
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELPUP3         Compute modified basis functions for tetrahedral upwind
!                    elements (3D)
!     ELPUP31        Compute distance h for modified basis functions for upwind
!                    element
!                    Three dimensional elements (quadratic tetrahedrons)
!     ELPUP4         Compute modified basis functions for hexahedral upwind
!                    elements (3D)
!     ELPUP5         Compute distance h for modified basis functions for upwind
!                    element
!                    Three dimensional elements, linear prisms
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elpupwdist' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elpupwdist'

      end if

      if ( jelp==11 ) then

!     --- Linear tetrahedron

         call elpup3 ( u, dphidx, h, unorm )

      else if ( jelp==12 ) then

!     --- Quadratic tetrahedron

         call elpup31 ( u, h, unorm, dlamdx )

      else if ( jelp==13 ) then

!     --- Trilinear hexahedron

         do j = 1, 3
            xcent(1,j) = 0.25d0 * ( x(1,j)+x(2,j)+x(3,j)+x(4,j) )
            xcent(2,j) = 0.25d0 * ( x(5,j)+x(6,j)+x(7,j)+x(8,j) )
            xcent(3,j) = 0.25d0 * ( x(1,j)+x(2,j)+x(6,j)+x(5,j) )
            xcent(4,j) = 0.25d0 * ( x(4,j)+x(3,j)+x(7,j)+x(8,j) )
            xcent(5,j) = 0.25d0 * ( x(1,j)+x(4,j)+x(8,j)+x(5,j) )
            xcent(6,j) = 0.25d0 * ( x(2,j)+x(3,j)+x(7,j)+x(6,j) )
         end do
         call elpup4 ( u, xcent, h, unorm )

      else if ( jelp==14 ) then

!     --- Triquadratic hexahedron

         do j = 1, 3
            xcent(1,j) = x(5,j)
            xcent(2,j) = x(23,j)
            xcent(3,j) = x(11,j)
            xcent(4,j) = x(17,j)
            xcent(5,j) = x(13,j)
            xcent(6,j) = x(15,j)
         end do
         call elpup4 ( u, xcent, h, unorm )

      else if ( jelp==21 ) then

!     --- Trilinear prism

         call elpup5 ( u, h, x, unorm )

      else if ( jelp==22 ) then

!     --- Triquadratic prism

         do j = 1, 3
            xcent(1,j) = x(1,j)
            xcent(2,j) = x(3,j)
            xcent(3,j) = x(5,j)
            xcent(4,j) = x(13,j)
            xcent(5,j) = x(15,j)
            xcent(6,j) = x(17,j)
         end do
         call elpup5 ( u, h, xcent, unorm )

      else

!     --- Value of jelp not yet implemented

         call errint ( jelp, 1 )
         call errsub ( 800, 1, 0, 0 )
         go to 1000

      end if

1000  call erclos ( 'elpupwdist' )
      if ( debug ) write(irefwr,*) 'End elpupwdist'

      end
