      subroutine elrhsdenthal ( elemvc, vecloc, coefarr, phi, w, itypeeq )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.0  date 22-06-2005 Extra parameter itypeeq
!        version  1.3  date 15-03-2005 Correction dimension tau
!                                      (J.H. Brusche)
!                                      of elemvc (J.H. Brusche)
!
!   copyright (c) 2004-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the special part of the right-hand side of the enthalpy
!     equation in case of a newton linearization
!     Computed is -M H - mu M ( T- tau(H)) element-wise per integration point
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
!     enthalpy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer          itypeeq
      double precision elemvc(*), vecloc(numold,maxunk,*), &
                       coefarr(m,*), phi(n,*), w(*)

!     coefarr        i    Array containing the coefficients of the PDE
!                         Pos. (.,11)  rho
!                         Pos. (.,12)  capacity_s
!                         Pos. (.,13)  capacity_l
!                         Pos. (.,14)  latent_heat
!                         Pos. (.,15)  melt_temp
!     elemvc         o    Element vector, contains enthalpy in integration
!                         nodes
!     itypeeq        i    if 0 the standard enthalpy equation is solved
!                         if 1 the equation is restricted to a solid temperature
!                         equation
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     vecloc         i    Work array in which all old solution vectors for the
!                         integration points are stored
!     w              i    array of length m containing the weights for
!                         integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision tau(m), rho, cs, cl, L, Tm, mu, temperature(m), &
                       enthalpy(m), sum

!     cl             Specific heat of the liquid
!     cs             Specific heat of the solid
!     enthalpy       enthalpy vector in integration points
!     i              Counting variable
!     j              Counting variable
!     l              Latent heat
!     mu             relaxation parameter
!     rho            density rho
!     sum            Help parameter to compute a sum
!     tau            Temperature as function of the enthalpy
!     temperature    temperature vector in integration points
!     tm             Melting temperature
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elrhsdenthal' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elrhsdenthal'
         write(irefwr,100) 'iseqtemp, iseqenth', iseqtemp, iseqenth
100      format ( a, 1x, (10i6) )
110      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- get parameters

      rho = coefarr(1,12)
      cs  = coefarr(1,3)
      cl  = coefarr(1,4)
      L   = coefarr(1,5)
      Tm  = coefarr(1,6)
      if ( debug ) &
         write(irefwr,110) 'rho, cs, cl, L, Tm', rho, cs, cl, L, Tm

!     --- Copy temperature and enthalpy

      do j = 1, m
         temperature(j) = vecloc(iseqtemp,1,j)
         enthalpy(j) = vecloc(iseqenth,1,j)
      end do  ! j = 1, m

!     --- Determine mu

      mu = rho * min(cs, cl)

!     --- Compute tau(H)

      if ( itypeeq==1 ) then
         do i = 1, m
            tau(i) = Tm + enthalpy(i)/(rho*cs)
         end do
      else
         do i = 1, m
            if (enthalpy(i)<=0) then
               tau(i) = Tm + enthalpy(i)/(rho*cs)
            else if (enthalpy(i)>=rho*L) then
               tau(i) = Tm + (enthalpy(i) - rho*L)/(rho*cl)
            else
               tau(i) = Tm
            end if
         end do
      end if  ! ( itypeeq==1 )

!     --- Compute -MH - muM(T - tau(H))

      do i = 1, n
         sum = 0d0
         do j = 1, m
            sum = sum-w(j)*phi(i,j)*(enthalpy(j) + &
                      mu*(temperature(j)-tau(j)))
         end do
         elemvc(i) = sum
      end do

1000  call erclos ( 'elrhsdenthal' )
      if ( debug ) then

!     --- Debug information

         if ( itypeeq==0 ) then
            call prinrl ( temperature, m, 'temperature' )
            call prinrl ( enthalpy, m, 'enthalpy' )
            call prinrl ( tau, m, 'tau' )
            call prinrl1 ( phi, m, n, 'phi' )
            call prinrl ( w, m, 'w' )
         end if  ! ( itypeq==0 )
         call prinrl ( elemvc, n, 'elemvc' )
         write(irefwr,*) 'End elrhsdenthal'

      end if

      end
