      subroutine eltrans3dquad ( iqmat, x, intrule, qmat, phiksi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 29-07-2013 Use phiksi
!        version  1.0  date 17-01-2003
!
!   copyright (c) 2003-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map coordinates from a bilinear quadrilateral surface in R^3 onto a
!     two-dimensional representation
!     Only meant for 4 point quads
!     The new 2d coordinates overwrite the old 3d ones
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     2d
!     transformation
!     element
!     element
!     basis_function
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iqmat, intrule
      double precision x(n,3), qmat(3,3,*), phiksi(n,m,2)

!     intrule        i    Actual integration rule
!     iqmat          i    Defines how the transformation matrix must be
!                         filled
!                           0: do not fill
!                           1:
!                           2:
!                           3:
!                         4-6: the first basis vector is in the e_i direction
!                              along the surface with i = iqmat-3
!     phiksi         i    contains derivatives in reference element for gauss
!                         rule.
!                         phiksi(i,k,l) = d phi_i/ d xi_l (x^k)
!     qmat           o    transformation matrix global -> local coordinates
!                         in case of a 3D element
!     x             i/o   Array of size 4 x 3 containing the coordinates of
!                         the vertices
!                         At input: 4x3 Cartesian coordinates
!                         At output: 4x2 xi-eta coordinates
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, j
      double precision dxdxi(4,3,2), xloc(4,2), dxdxiint(m,3,2)

!     dxdxi          work array of length 4x3x3 to store the values of
!                    dx/dxi, dx/deta, dy/dxi and dy/deta according to:
!                    dx/dxi(x_k) = dxdxi(k,1,1)
!                    dx/deta(x_k) = dxdxi(k,1,2)
!                    dy/dxi(x_k) = dxdxi(k,2,1)
!                    dy/deta(x_k) = dxdxi(k,2,2) etc.
!     dxdxiint       See dxdxi but now for the integration points
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     xloc           co-ordinates of vertices in local (xi-zeta) coordinate
!                    system
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELFILLQMAT     Fill the transformation matrix qmat for the mapping of a
!                    3d surface onto a 2d surface
!     ELLOCCOORGEN   Compute local coordinates of a surface element in 3d
!                    given the transformation by base vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eltrans3dquad' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eltrans3dquad'

      end if

!     --- First fill arrays dxdxi with dx/dxi and dx/deta
!         in the vertices

      do k = 1, 3
         dxdxi(1:2,k,1) = x(2,k)-x(1,k)
         dxdxi(3:4,k,1) = x(3,k)-x(4,k)
         dxdxi(1,k,2) = x(4,k)-x(1,k)
         dxdxi(2,k,2) = x(3,k)-x(2,k)
         dxdxi(3,k,2) = x(3,k)-x(2,k)
         dxdxi(4,k,2) = dxdxi(1,k,2)
      end do  ! k = 1, 3

!     --- Next compute lamina system and store in qmat

      if ( intrule/=1 ) then

!     --- Gauss integration rule, &
!         compute dx/dxi and dx/deta in integration points
!         in order to store qmat

         do j = 1, 3
            do i = 1, m
               do k = 1, 2
                  dxdxiint(i,j,k) = &
                     dot_product ( x(1:n,j), phiksi(1:n,i,k) )
               end do  ! k = 1, 2
            end do  ! i = 1, m
         end do  ! j = 1, 3

         call elfillqmat ( iqmat, dxdxiint, m, 1, 2, qmat )

      else

!     --- Compute lamina system and store in qmat

         call elfillqmat ( iqmat, dxdxi, n, 1, 2, qmat )

      end if  ! ( intrule/=1 )

!     --- Finally compute coordinates in local system

      call elloccoorgen ( x, xloc, qmat, n )

!     --- Copy local coordinates to x

      do k = 1, n
         x(k,1) = xloc(k,1)
         x(k,2) = xloc(k,2)
      end do

      call erclos ( 'eltrans3dquad' )
      if ( debug ) write(irefwr,*) 'End eltrans3dquad'

      end
