      subroutine sepel0250cf
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill real coefficients for elements of type 250
!
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
!     coefficient
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, igprob, ne, nu, numcf
      double precision :: anu, elas

!     anu            Poisson parameter nu
!     elas           Elasticity Modulus E
!     i              Counting variable
!     igprob         Type of stress-strain relations
!                    igprob(here) = igprob(manual) + 1
!     ne             Number of E coefficients
!     nu             Number of nu coefficients
!     numcf          number of coefficients in D matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPELALLOCCF   Allocate coeffs and related arrays for element
!     SEPELLOWUPP    Find lower and upper bound of variable coefficients
!                    Fill oldvectors if necessary
!     SEPFILLDMAT    Fill D-matrix for linear elasticity problem
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     554   Elasticity modulus E (or E1) has wrong value
!     555   Poisson ratio nu (or nu1) has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel0250cf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel0250cf'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%indsource = 23
      E%indrho = 38

      if ( debug ) &
         write(irefwr,1) 'indsource, indrho', E%indsource, E%indrho

      call sepelalloccf ( 0, 0 ) ! allocate coefficient arrays

      if ( E%ind(22)==0 ) then

!     --- Thickness has not been filled, set to 1

         E%coeffs(:,22) = 1d0
         E%ind(22) = -1

      end if  ! ( E%ind(22)==0 )

!     --- Check coefficients
!         First Elasticity

      igprob = E%stress_strain+1+10*E%modelelastnl
      if ( igprob<=3 ) then
         ne = 1
      else if ( igprob<=6 ) then
         ne = 2
      else
         ne = 0
      end if

      do i = 1, ne

         if ( E%ind(i)==0 ) then

!        --- ind(i)=0, hence Ei = 0

            call errint ( ielem, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call erreal ( 0d0, 1 )
            call errsub ( 554, 3, 1, 0 )

         else if ( E%ind(i)<0 ) then

!        --- ind(1)<0, hence E is constant

            elas=E%coeffs(1,1)
            if ( elas<=0d0 ) then
               call errint ( ielem, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call erreal ( elas, 1 )
               call errsub ( 554, 3, 1, 0 )
            end if

         end if

      end do

      if ( igprob<=3 ) then
         nu = 1
      else if ( igprob<=6 ) then
         nu = 3
      else
         nu = 0
      end if

      do i = 1, nu

         if ( E%ind(ne+i)==0 ) then

!        --- ind(ne+i)=0, hence nu(i) = 0

            call errint ( ielem, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call erreal ( 0d0, 1 )
            call errsub ( 555, 3, 1, 0 )

         else if ( E%ind(ne+i)<0 ) then

!        --- ind(ne+i)<0, hence nu1 is constant

            anu=E%coeffs(1,ne+i)
            if ( i==3 ) anu = min(.45d0, anu)
            if ( anu<=0d0 .or. anu>=.5d0 ) then
               call errint ( ielem, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call erreal ( anu, 1 )
               call errsub ( 555, 3, 1, 0 )
            end if

         end if

      end do

!     --- Define rho

      if ( E%ind(E%indrho)==0 ) then

!     --- rho = 0, make it equal to 1

         E%coeffs(:,E%indrho) = 1d0
         E%ind(E%indrho) = -1

      end if  ! ( E%ind(E%indrho)==0 )

      do i = 1, E%ncoef
         E%coeffsnode(1:B%n,i) = E%coeffs(1,i)
      end do  ! i = 1, E%ncoef

!     --- Find lower and upper bound for space dependent coefficients
!         Fill old vectors in locvecs if necessary

      call sepellowupp

!     --- Set initial_strain and initial_stress

      E%initial_strain = .false.
      E%initial_stress = .false.
      do i = 1, 6
         if ( E%ind(25+i)/=0 ) E%initial_strain = .true.
         if ( E%ind(31+i)/=0 ) E%initial_stress = .true.
      end do  ! i = 1, 6

!     --- Check if matrix D is constant or not

      numcf = 4
      select case ( E%stress_strain )
         case (0:2)
            numcf = 2
         case (3:5)
            numcf = 5
         case (6)
            numcf = 6
         case (7)
            numcf = 10
         case (8)
            numcf = 21
      end select  ! case ( E%stress_strain )

      E%dmatconst = E%ind(1)<=0
      do i = 2, numcf
         E%dmatconst = E%dmatconst .and. E%ind(i)<=0
      end do

      if ( E%ndim==2 .and. E%initial_stress .and. E%stress_strain<=1 ) then

!     --- initial stress and isotropic material,
!         move fourth component to third position

         E%ind(34) = E%ind(35)
         E%ich(34) = E%ich(35)
         E%ist(34) = E%ist(35)
         E%mst(34) = E%mst(35)
         E%ind(35) = 0
         E%ich(35) = 0
         E%ist(35) = 0
         E%mst(35) = 0
         if ( E%ncoef==37 ) E%ncoef = 34

         E%coeffs(:,34) = E%coeffs(:,35)

      end if  ! ( E%ndim==2 .and. E%initial_stress .and. E%stress_strain<=1 )

      if ( E%ndim==2 .and. E%initial_strain .and. E%stress_strain<=1 ) then

!     --- initial strain and isotropic material,
!         move fourth component to third position

         E%ind(28) = E%ind(29)
         E%ich(28) = E%ich(29)
         E%ist(28) = E%ist(29)
         E%mst(28) = E%mst(29)
         E%ind(29) = 0
         E%ich(29) = 0
         E%ist(29) = 0
         E%mst(29) = 0
         if ( E%ncoef==31 ) E%ncoef = 28

         E%coeffs(:,28) = E%coeffs(:,29)

      end if  ! ( E%ndim==2 .and. E%initial_strain .and. E%stress_strain<=1 )

!     --- Fill D-matrix if constant

      if ( E%dmatconst ) call sepfilldmat

!     --- If right-hand side is 0, skip computation of element element_vector

      if ( E%ind(23)==0 .and. E%ind(24)==0 .and. E%ind(25)==0 .and. &
           .not. E%initial_strain .and. .not. E%initial_stress ) &
         E%notvec = 1

1000  if ( debug ) then

!     --- Debug information

         call prinin ( E%ich, E%ncoef, 'ich' )
         call prinin ( E%ind, E%ncoef, 'ind' )
         write(irefwr,*) 'End sepel0250cf'

      end if  ! ( debug )
      call erclos ( 'sepel0250cf' )

      end subroutine sepel0250cf
