      subroutine sepelconvect4
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix for compressible convection equation
!     It is assumed that convection coefficients are stored in coefficients
!     1 to 3
!     The derivatives of the basis functions are variable
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     convection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: h1, hx, hy, hz, workx(B%m), worky(B%m), workz(B%m)
      double precision, pointer :: phix(:,:), phiy(:,:), phiz(:,:)
      double precision, pointer :: u(:), v(:), w(:), phi(:,:)
      integer :: i, j, k, m, n, kstep, nd, jdiag, ind(3)

!     h1             help parameter to store element of convection matrix
!     hx             help parameter to store w*u_x
!     hy             help parameter to store w*u_y
!     hz             help parameter to store w*u_z
!     i              General loop variable
!     ind            Contains the indices of the velocity components
!                    in array coeffs
!     j              General loop variable
!     jdiag          if 1 the basis functions are 0 or 1 in the quadrature
!                    points
!     k              Loop variable (quadrature points)
!     kstep          Step in loop
!     m              number of quadrature points
!     n              number of basis functions for each component
!     nd             Formal dimension corresponding to n
!     phi            Contains the values of the basis functions in the
!                    quadrature points
!     phix           x derivative of basis function phi
!     phiy           y derivative of basis function phi
!     phiz           z derivative of basis function phi
!     u              u-component of velocity
!     v              v-component of velocity
!     w              w-component of velocity
!     workx          Work array to store w*u_x
!     worky          Work array to store w*u_y
!     workz          Work array to store w*u_z
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The convection matrix is given by
!
!     / u . grad(phi_j) phi_i with u the velocity vector
!
!     The elements of the element matrix are
!
!     s_ij = sum_k ( w_k  (ux_k dphidx_j(x_k) + uy_k dphidy_j(x_k) +
!                          uz_k dphidz_j(x_k) ) * phi_i(x_k) )
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelconvect4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelconvect4'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      phix => B%phider(:,:,1)
      phiy => B%phider(:,:,2)
      phiz => B%phider(:,:,3)
      phi => B%phi

      m = B%m
      n = B%n
      kstep = B%kstep
      nd = B%nd
      jdiag = B%jdiag
      ind(1) = E%indvel
      ind(2) = E%indvel+1
      ind(3) = E%indvel+2

      u => E%coeffs(:,ind(1))
      v => E%coeffs(:,ind(2))
      w => E%coeffs(:,ind(3))

      if ( jdiag==1 ) then

!     --- basis functions satisfy phi_i(x_j) = delta_ij

         select case ( nd )

         case(1)

!        --- 1d dimensional element

            do i = 1, n
               hx = B%w(i)*u(i)
               do j = 1, n
                  E%mat(i,j) = E%mat(i,j)-hx*phix(j,i)
               end do  ! j = 1, n
            end do  ! i = 1, n

         case(2)

!        --- 2d dimensional element

            do i = 1, n
               if ( ind(1)/=0 ) hx = B%w(i)*u(i)
               if ( ind(2)/=0 ) hy = B%w(i)*v(i)
               do j = 1, n
                  h1 = 0d0
                  if ( ind(1)/=0 ) h1 = h1+hx*phix(j,i)
                  if ( ind(2)/=0 ) h1 = h1+hy*phiy(j,i)
                  E%mat(i,j) = E%mat(i,j)-h1
               end do  ! j = 1, n
            end do  ! i = 1, n

         case(3)

!        --- 3d dimensional element

            do i = 1, n
               if ( ind(1)/=0 ) hx = B%w(i)*u(i)
               if ( ind(2)/=0 ) hy = B%w(i)*v(i)
               if ( ind(3)/=0 ) hz = B%w(i)*w(i)
               do j = 1, n
                  h1 = 0d0
                  if ( ind(1)/=0 ) h1 = h1+hx*phix(j,i)
                  if ( ind(2)/=0 ) h1 = h1+hy*phiy(j,i)
                  if ( ind(3)/=0 ) h1 = h1+hz*phiz(j,i)
                  E%mat(i,j) = E%mat(i,j)-h1
               end do  ! j = 1, n
            end do  ! i = 1, n

         end select  ! case ( nd )

      else

!     --- Gauss rule  phi_i(x_j) # delta_ij

         if ( ind(1)/=0 ) then
            do k = kstep, m, kstep
               workx(k) = B%w(k) * u(k)
            end do
         end if  ! ( ind(1)/=0 )

         if ( ind(2)/=0 ) then
            do k = kstep, m, kstep
               worky(k) = B%w(k) * v(k)
            end do
         end if  ! ( ind(2)/=0 )

         if ( ind(3)/=0 ) then
            do k = kstep, m, kstep
               workz(k) = B%w(k) * w(k)
            end do
         end if  ! ( ind(2)/=0 )

         select case ( nd )

         case(1)

!        --- 1d dimensional element

            if ( ind(1)/=0 ) then
               do i = 1, n
                  do j = 1, n
                     h1 = 0d0
                     do k = kstep, m, kstep
                        h1 = h1+workx(k)*phix(j,k)*phi(i,k)
                     end do  ! k = kstep, m, kstep
                     E%mat(i,j) = E%mat(i,j)-h1
                  end do  ! j = 1, n
               end do  ! i = 1, n
            end if  ! ( ind(1)/=0 )

         case(2)

!        --- 2d dimensional element

            do i = 1, n
               do j = 1, n
                  h1 = 0d0
                  if ( ind(1)/=0 ) then
                     do k = kstep, m, kstep
                        h1 = h1+workx(k)*phix(j,k)*phi(i,k)
                     end do  ! k = kstep, m, kstep
                  end if  ! ( ind(1)/=0 )
                  if ( ind(2)/=0 ) then
                     do k = kstep, m, kstep
                        h1 = h1+worky(k)*phiy(j,k)*phi(i,k)
                     end do  ! k = kstep, m, kstep
                  end if  ! ( ind(2)/=0 )
                  E%mat(i,j) = E%mat(i,j)-h1
               end do  ! j = 1, n
            end do  ! i = 1, n

         case(3)

!        --- 3d dimensional element

            do i = 1, n
               do j = 1, n
                  h1 = 0d0
                  if ( ind(1)/=0 ) then
                     do k = kstep, m, kstep
                        h1 = h1+workx(k)*phix(j,k)*phi(i,k)
                     end do  ! k = kstep, m, kstep
                  end if  ! ( ind(1)/=0 )
                  if ( ind(2)/=0 ) then
                     do k = kstep, m, kstep
                        h1 = h1+worky(k)*phiy(j,k)*phi(i,k)
                     end do  ! k = kstep, m, kstep
                  end if  ! ( ind(2)/=0 )
                  if ( ind(3)/=0 ) then
                     do k = kstep, m, kstep
                        h1 = h1+workz(k)*phiz(j,k)*phi(i,k)
                     end do  ! k = kstep, m, kstep
                  end if  ! ( ind(3)/=0 )
                  E%mat(i,j) = E%mat(i,j)-h1
               end do  ! j = 1, n
            end do  ! i = 1, n

         end select  ! case ( nd )

      end if  ! ( jdiag==1 )

1000  call erclos ( 'sepelconvect4' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelconvect4'

      end if  ! ( debug )

      end subroutine sepelconvect4
