      subroutine sepeldifftens ( coeffs, ind )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix for diffusion equation
!     The diffusion is a tensor
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in), target :: coeffs(B%m,6)
      integer, intent(in) :: ind(6)

!     coeffs         i    contains the diffusion coefficients in the quadrature
!                         points:
!                         (.,1): D_xx
!                         (.,2): D_xy
!                         (.,3): D_xz
!                         (.,4): D_yy
!                         (.,5): D_yz
!                         (.,6): D_zz
!     ind            i    Indicates if coefficients are 0, constant or variable
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k
      double precision :: hx, h1, hy, hz, hxy, hxz, hyz, &
                          workx(B%m), worky(B%m), workz(B%m), workxy(B%m), &
                          workxz(B%m), workyz(B%m)
      double precision, pointer :: phix(:,:), phiy(:,:), phiz(:,:), mat(:,:)

!     h1             help parameter to store element of diffusion matrix
!     hx             help parameter to store w*diff_xx
!     hxy            help parameter to store w*diff_xy
!     hxz            help parameter to store w*diff_xx
!     hy             help parameter to store w*diff_yy
!     hyz            help parameter to store w*diff_yz
!     hz             help parameter to store w*diff_zz
!     i              General loop variable
!     j              General loop variable
!     k              Loop variable (quadrature points)
!     mat            sub element matrix
!     phix           x derivative of basis function phi
!     phiy           y derivative of basis function phi
!     phiz           z derivative of basis function phi
!     workx          help array to store w*diff_xx
!     workxy         help array to store w*diff_xy
!     workxz         help array to store w*diff_xz
!     worky          help array to store w*diff_yy
!     workyz         help array to store w*diff_yz
!     workz          help array to store w*diff_zz
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The diffusion matrix is given by
!
!     / D grad(phi_i) grad(phi_j) with D a matrix
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeldifftens' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeldifftens'
         write(irefwr,*) 'derivconst ', B%derivconst
         write(irefwr,1) 'n, m, nd', B%n, B%m, B%nd
  1      format ( a, 1x, (10i6) )
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      phix => B%phider(:,:,1)
      phiy => B%phider(:,:,2)
      phiz => B%phider(:,:,3)
      mat => E%submat

      if ( B%derivconst ) then

!     --- Derivatives of basis functions constant

         if ( ind(1)/=0 ) then
            hx = sum ( B%w*coeffs(:,1) )
         else
            hx = 0d0
         end if  ! ( ind(1)/=0 )
         select case ( B%nd )

         case(1)

!        --- 1d dimensional element

            do i = 1, B%n
               do j = 1, i
                  h1 = hx*phix(i,1)*phix(j,1)
                  mat(i,j) = mat(i,j)+h1
                  if ( j/=i ) mat(j,i) = mat(j,i)+h1
               end do  ! j = 1, i
            end do  ! i = 1, B%n

         case(2)

!        --- 2d dimensional element

            if ( ind(2)/=0 ) hxy = sum ( B%w*coeffs(:,2) )
            if ( ind(4)/=0 ) hy = sum ( B%w*coeffs(:,4) )
            do i = 1, B%n
               do j = 1, i
                  h1 = 0d0
                  if ( ind(1)/=0 ) h1 = h1+hx*phix(i,1)*phix(j,1)
                  if ( ind(2)/=0 ) h1 = h1+hxy*&
                       (phix(i,1)*phiy(j,1)+phiy(i,1)*phix(j,1))
                  if ( ind(4)/=0 ) h1 = h1+hy*phiy(i,1)*phiy(j,1)
                  mat(i,j) = mat(i,j)+h1
                  if ( j/=i ) mat(j,i) = mat(j,i)+h1
               end do  ! j = 1, i
            end do  ! i = 1, B%n

         case(3)

!        --- 3d dimensional element

            if ( ind(2)/=0 ) hxy = sum ( B%w*coeffs(:,2) )
            if ( ind(3)/=0 ) hxz = sum ( B%w*coeffs(:,3) )
            if ( ind(4)/=0 ) hy = sum ( B%w*coeffs(:,4) )
            if ( ind(5)/=0 ) hyz = sum ( B%w*coeffs(:,5) )
            if ( ind(6)/=0 ) hz = sum ( B%w*coeffs(:,6) )

            do i = 1, B%n
               do j = 1, i
                  h1 = 0d0
                  if ( ind(1)/=0 ) h1 = h1+hx*phix(i,1)*phix(j,1)
                  if ( ind(2)/=0 ) h1 = h1+hxy*&
                       (phix(i,1)*phiy(j,1)+phiy(i,1)*phix(j,1))
                  if ( ind(3)/=0 ) h1 = h1+hxz*&
                       (phix(i,1)*phiz(j,1)+phiz(i,1)*phix(j,1))
                  if ( ind(4)/=0 ) h1 = h1+hy*phiy(i,1)*phiy(j,1)
                  if ( ind(5)/=0 ) h1 = h1+hyz*&
                       (phiy(i,1)*phiz(j,1)+phiz(i,1)*phiy(j,1))
                  if ( ind(6)/=0 ) h1 = h1+hz*phiz(i,1)*phiz(j,1)
                  mat(i,j) = mat(i,j)+h1
                  if ( j/=i ) mat(j,i) = mat(j,i)+h1
               end do  ! j = 1, i
            end do  ! i = 1, B%n

         end select  ! case ( B%nd )

      else

!     --- Derivatives of basis functions are variable

         if ( ind(1)/=0 ) then
            do k = B%kstep, B%m, B%kstep
               workx(k) = B%w(k)*coeffs(k,1)
            end do  ! k = B%kstep, B%m, B%kstep
         else
            workx = 0d0
         end if  ! ( ind(1)/=0 )
         select case ( B%nd )

         case(1)

!        --- 1d dimensional element

            do i = 1, B%n
               do j = 1, i
                  h1 = sum ( workx(:)*phix(i,:)*phix(j,:) )
                  mat(i,j) = mat(i,j)+h1
                  if ( j/=i ) mat(j,i) = mat(j,i)+h1
               end do  ! j = 1, i
            end do  ! i = 1, B%n

         case(2)

!        --- 2d dimensional element

            if ( B%kstep==1 ) then

!           --- Standard case, all quadrature points are used

               if ( ind(2)/=0 ) workxy = B%w*coeffs(:,2)
               if ( ind(4)/=0 ) worky = B%w*coeffs(:,4)
               do i = 1, B%n
                  do j = 1, i
                     h1 = 0d0
                     if ( ind(1)/=0 ) &
                        h1 = h1+sum ( workx(:)*phix(i,:)*phix(j,:) )
                     if ( ind(2)/=0 ) &
                        h1 = h1+sum ( workxy(:)*(phix(i,1)*phiy(j,1)+&
                                      phiy(i,1)*phix(j,1)) )
                     if ( ind(4)/=0 ) &
                        h1 = h1+sum ( worky(:)*phiy(i,:)*phiy(j,:) )
                     mat(i,j) = mat(i,j)+h1
                     if ( j/=i ) mat(j,i) = mat(j,i)+h1
                  end do  ! j = 1, i
               end do  ! i = 1, B%n

            else

!           --- Special case of quadratic triangle, where only the mid-points
!               are used

               if ( ind(2)/=0 ) then
                  do k = B%kstep, B%m, B%kstep
                     workxy(k) = B%w(k)*coeffs(k,2)
                  end do  ! k = B%kstep, B%m, B%kstep
               end if  ! ( ind(2)/=0 )
               if ( ind(4)/=0 ) then
                  do k = B%kstep, B%m, B%kstep
                     worky(k) = B%w(k)*coeffs(k,4)
                  end do  ! k = B%kstep, B%m, B%kstep
               end if  ! ( ind(4)/=0 )
               do i = 1, B%n
                  do j = 1, i
                     h1 = 0d0
                     if ( ind(1)/=0 ) then
                        do k = B%kstep, B%m, B%kstep
                           h1 = h1+workx(k)*phix(i,k)*phix(j,k)
                        end do  ! k = B%kstep, B%m, B%kstep
                     end if  ! ( ind(1)/=0 )
                     if ( ind(2)/=0 ) then
                        do k = B%kstep, B%m, B%kstep
                           h1 = h1+workxy(k)*(phix(i,k)*phiy(j,k)+&
                                              phiy(i,k)*phix(j,k))
                        end do  ! k = B%kstep, B%m, B%kstep
                     end if  ! ( ind(2)/=0 )
                     if ( ind(4)/=0 ) then
                        do k = B%kstep, B%m, B%kstep
                           h1 = h1+worky(k)*phiy(i,k)*phiy(j,k)
                        end do  ! k = B%kstep, B%m, B%kstep
                     end if  ! ( ind(4)/=0 )
                     mat(i,j) = mat(i,j)+h1
                     if ( j/=i ) mat(j,i) = mat(j,i)+h1
                  end do  ! j = 1, i
               end do  ! i = 1, B%n

            end if  ! ( B%kstep==1 )

         case(3)

!        --- 3d dimensional element

            if ( ind(2)/=0 ) workxy = B%w*coeffs(:,2)
            if ( ind(3)/=0 ) workxz = B%w*coeffs(:,3)
            if ( ind(4)/=0 ) worky = B%w*coeffs(:,4)
            if ( ind(5)/=0 ) workyz = B%w*coeffs(:,5)
            if ( ind(6)/=0 ) workz = B%w*coeffs(:,6)
            do i = 1, B%n
               do j = 1, i
                  h1 = 0d0
                  if ( ind(1)/=0 ) &
                     h1 = h1+sum ( workx(:)*phix(i,:)*phix(j,:) )
                  if ( ind(2)/=0 ) &
                     h1 = h1+sum ( workxy(:)*(phix(i,1)*phiy(j,1)+&
                                              phiy(i,1)*phix(j,1)) )
                  if ( ind(3)/=0 ) &
                     h1 = h1+sum ( workxz(:)*(phix(i,1)*phiz(j,1)+&
                                              phiz(i,1)*phix(j,1)) )
                  if ( ind(4)/=0 ) &
                     h1 = h1+sum ( worky(:)*phiy(i,:)*phiy(j,:) )
                  if ( ind(5)/=0 ) &
                     h1 = h1+sum ( workyz(:)*(phiy(i,1)*phiz(j,1)+&
                                              phiz(i,1)*phiy(j,1)) )
                  if ( ind(6)/=0 ) &
                     h1 = h1+sum ( workz(:)*phiz(i,:)*phiz(j,:) )
                  mat(i,j) = mat(i,j)+h1
                  if ( j/=i ) mat(j,i) = mat(j,i)+h1
               end do  ! j = 1, i
            end do  ! i = 1, B%n

         end select  ! case ( B%nd )

      end if  ! ( B%derivconst )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeldifftens'

      end if  ! ( debug )
      call erclos ( 'sepeldifftens' )

      end subroutine sepeldifftens
