      subroutine sepelloccoorgn
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-02-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute local coordinates of a surface element in 3d given the
!     transformation by base vectors
!     Subroutine meant for general element
!     Old subroutine elloccoorgen
! **********************************************************************
!
!                       KEYWORDS
!
!     coordinate
!     element
!     element
!     basis_function
!     transformation
!     3d
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, k
      double precision :: s11, s12, s21, s22, f1, f2, det, sinv11, sinv12, &
                          sinv21, sinv22, exi(3), ezeta(3)
      double precision :: xloc(B%n,2)
      double precision, pointer :: qmat(:,:,:), x(:,:)

!                    they are not
!     det            determinant of matrix S
!     exi            base vector in xi direction
!     ezeta          base vector in zeta direction
!     f1             first component of right-hand side vector
!     f2             second component of right-hand side vector
!     i              Counting variable
!     k              Counting variable
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     s11            11 element of matrix
!     s12            12 element of matrix
!     s21            21 element of matrix
!     s22            22 element of matrix
!     sinv11         11 element of inverse of matrix
!     sinv12         12 element of inverse of matrix
!     sinv21         21 element of inverse of matrix
!     sinv22         22 element of inverse of matrix
!     x              co-ordinates of nodes
!     xloc           co-ordinates of nodes in local (xi-zeta) coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The local coordinates are defined by:
!     x = x_1 + alpha e_zeta + beta e_xi
!     Hence x_i = x_1 + alpha_i e_zeta + beta_i e_xi
!     and alpha_i, beta_i are the solution of
!     (e_zeta e_xi)^T (e_zeta e_xi) (alpha_i beta_i)^T = &
!        (e_zeta e_xi)^T (x_i - x_1)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelloccoorgn' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelloccoorgn'

      end if

      qmat => B%qmat
      x => B%xyz

      xloc = 0d0

      do i = 2, B%n

         ezeta = qmat(2,:,i)
         exi = qmat(3,:,i)

!        --- Compute least squares matrix

         s11 = dot_product(ezeta,ezeta)
         s12 = dot_product(ezeta,exi)
         s21 = s12
         s22 = dot_product(exi,exi)

!        --- Compute determinant

         det = 1d0/(s11*s22-s12*s21)

!        --- Compute inverse matrix

         sinv11 = s22*det
         sinv12 = -s12*det
         sinv21 = sinv12
         sinv22 = s11*det

!        --- Compute least squares right-hand side and solve system of equations

         f1 = ezeta(1)*(x(i,1)-x(1,1))+ &
              ezeta(2)*(x(i,2)-x(1,2))+ &
              ezeta(3)*(x(i,3)-x(1,3))
         f2 = exi(1)*(x(i,1)-x(1,1))+ &
              exi(2)*(x(i,2)-x(1,2))+ &
              exi(3)*(x(i,3)-x(1,3))
         xloc(i,1) = sinv11*f1+sinv12*f2
         xloc(i,2) = sinv21*f1+sinv22*f2

      end do

      x(:,1:2) = xloc(:,1:2)

      call erclos ( 'sepelloccoorgn' )
      if ( debug ) write(irefwr,*) 'End sepelloccoorgn'

      end
