      subroutine sepelm150
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for the complex elliptic equation
!     one unknown per point
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: j
      double complex, pointer :: source(:), beta(:)

!     beta           coefficient for zeroth order term
!     j              general loop variable
!     source         Contains the source in quadrature points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPEL0150      Fill element independent quantities for element type 150
!     SEPELBASEFUNC  Compute information about basis functions
!     SEPELCCONVECT  Compute the complex element matrix for the
!                    convection equation
!     SEPELCDIFFUS   Compute the complex element matrix for the
!                    diffusion equation
!     SEPELCLOAD     Fill the element vector with a load
!     SEPELCVECGAMMA Add term div( gamma) to complex element vector
!     SEPELDEALLOC   Deallocate coefficients and basis functions for element
!                    subroutines
!     SEPELM150CF    Fill coefficients for complex general elliptic equation
!     SEPELMASSMAT   Fill the element mass matrix; one unknown per node
!     SEPELMASSMATC  Fill the complex element mass matrix; one unknown per node
!                    Consistent mass matrix
!     SEPELOLDCVECS  Fill old complex vectors for element
!     SEPPRINELCMAT  Print complex element matrix and vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The following coefficients are used (complex)
!
!     icoef 1 = -
!     icoef 2 = -
!     icoef 3 = irule (numerical quadrature)
!     icoef 4 = jcart (type of coordinate system)
!     icoef 5 = -
!     coef  6 = D_11
!     coef  7 = D_12
!     coef  8 = D_13
!     coef  9 = D_22
!     coef 10 = D_23
!     coef 11 = D_33
!     coef 12 = U_1
!     coef 13 = U_2
!     coef 14 = U_3
!     coef 15 = beta
!     coef 16 = source
!     coef 17 = density (rho)
!     coef 18 = heat_capacity (cp)
!     coef 19 = gamma_1
!     coef 20 = gamma_2
!     coef 21 = gamma_3
!     coef 22 = rho cp
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm150' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm150'
         write(irefwr,1) 'ielem', E%ielem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%cmat = 0d0
      E%cvec = 0d0

!     --- Define element independent quantities
!         only for first element

      if ( E%first==0 ) call sepel0150

!     --- Map relation between coefficients and specific names

      beta => E%ccoeffs(:,E%indbeta)
      source => E%ccoeffs(:,E%indsource)

!     --- Fill basis functions

      call sepelbasefunc

!     --- Fill old vectors in quadrature points if necessary

      if ( E%compold>0 ) call sepeloldcvecs

      if ( debug ) then
         write(irefwr,1) 'isupcfvar, isubcfvar', E%isupcfvar, E%isubcfvar
         write(irefwr,*) 'B%fillqmat ', B%fillqmat
      end if  ! ( debug )

!     --- Fill all coefficients

      call sepelm150cf

      if ( matrix ) then

!     --- Fill element matrix

         if ( debug ) &
            write(irefwr,*) 'diffusion, convection, zeroterm', &
                             E%diffusion, E%convection, E%zeroterm

         if ( E%diffusion ) call sepelcdiffus  ! diffusion
         if ( E%convection ) call sepelcconvect   ! convection
         if ( E%zeroterm ) call sepelmassmatc ( beta ) ! zero term

      end if  ! ( matrix )

      if ( vector .and. notvec==0 ) then

!     --- Fill element vector

         if ( E%sourceterm ) &
            call sepelcload ( source )
         if ( E%gammaterm ) call sepelcvecgamma

      end if  ! ( vector )

!     --- Fill element mass matrix if necessary

      if ( E%imas>0 ) call sepelmassmat

!     --- Reset velocity if necessary

      if ( E%ind(E%indrhocp)/=0 ) &
         E%ccoeffs(:,E%indvel:E%indvel+2) = E%ccoeffs(:,E%ncoef+1:E%ncoef+3)

!     --- Remove basis functions and coefficients in case of last call

      if ( debug ) write(irefwr,1) 'last', E%last
      if ( E%last==1 ) call sepeldealloc

1000  if ( debug ) then

!     --- Debug information

         call sepprinelcmat
         write(irefwr,*) 'End sepelm150'

      end if  ! ( debug )
      call erclos ( 'sepelm150' )

      end subroutine sepelm150
