      subroutine sepelm152
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for boundary conditions for
!     second order complex elliptic equations
!     Old subroutine elm152
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     parabolic_equation
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double complex, pointer :: sigma(:), flux(:)

!     flux           Refers to the the right-hand side of the boundary cond
!     sigma          parameter sigma for boundary conditions of the type 2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINCOMPL      Print 1d complex vector
!     SEPEL0152      ?
!     SEPELBASEFUNC  Compute information about basis functions
!     SEPELCLOAD     Fill the element vector with a load
!     SEPELCONVECT   Compute the element matrix for the convection equation
!     SEPELDEALLOC   Deallocate coefficients and basis functions for element
!                    subroutines
!     SEPELFILLCVAR  Fill complex space dependent coefficients
!     SEPELMASSMATC  Fill the complex element mass matrix; one unknown per node
!                    Consistent mass matrix
!     SEPELOLDCVECS  Fill old complex vectors for element
!     SEPPRINELCMAT  Print complex element matrix and vector
! **********************************************************************
!
!                       I/O
!
!     The following coefficients are used
!
!     icoef 1 = typenatbnd  Type of natural boundary condition
!         Possible values:
!         0,2: natural boundary condition of type 2
!         3: natural boundary condition of type 3
!         4: natural boundary condition of type 4
!         5: natural boundary condition of type 5
!         6: natural boundary condition of type 6
!     icoef 3 = irule (numerical quadrature)
!     icoef 4 = jcart (type of coordinate system)
!     coef 6:  sigma
!     coef 7:  h
!     coef 8:  alpha 11  (1D/2D/3D)  (The parameters 8-13 are only used for
!     coef 9:  alpha 12  (2D/3D)      boundary conditions of type 4)
!     icoef 0:  alpha 13  (3D)
!     coef 11:  alpha 22  (2D/3D)
!     coef 12:  alpha 23  (3D)
!     coef 13:  alpha 33  (3D)
!     coef 14:  dc/dx (bc of type 6)
!     coef 15:  dc/dy (bc of type 6)
!     coef 16:  u1 (bc of type 5)
!     coef 17:  u2 (bc of type 5)
!     coef 18:  u3 (bc of type 5)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3216   Type of natural boundary condition not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!       Boundary conditions:
!       Type 2:   sum  sum alpha   dc/dx n  + sigma c = h
!                  i    j       ij      j i
!       Type 3:   sum  sum alpha   dc/dx n  + sigma dc/dt = h
!                  i    j       ij      j i
!       Type 4:   dc/dn + sigma dc/dt = h
!       Type 5:   special for convective terms of compressible type
!                 See 2: h=0, sigma = u.n
!       Type 6:   grad c given, hence A grad c . n must be computed
!        The following formulae are used:
!       Type 2:  S(i,j) = /  sigma phi  phi  de
!                         e           i    j
!                f(i)   = /  h phi  de
!                         e       i
!        Each integral is approximated by a numerical rule of the shape:
!        / int(x) dx = sum  w  int(x )
!        e              k    k      k
!        the factors w  form the weights for the numerical integration
!                     k
!        In the axi-symmetric case they include the factor 2 pi r
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm152' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm152'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%cmat = 0d0
      E%cvec = 0d0

!     --- Define element independent quantities
!         only for first element

      if ( E%first==0 ) call sepel0152

      if ( debug ) &
         write(irefwr,1) 'E%typenatbnd, B%m, B%n, B%rule', &
                          E%typenatbnd, B%m, B%n, B%rule

!     --- Map relation between coefficients and specific names

      sigma => E%ccoeffs(:,1)
      if ( E%typenatbnd==3 ) then
         flux => E%ccoeffs(:,12)
      else
         flux => E%ccoeffs(:,2)
      end if  ! ( E%typenatbnd==3 )

!     --- Fill basis functions

      call sepelbasefunc

!     --- Fill old vectors in quadrature points if necessary

      if ( E%compold>0 ) call sepeloldcvecs

!     --- Fill variable coefficients if necessary

      if ( E%isupcfvar>=E%isubcfvar )  call sepelfillcvar

      if ( debug ) then
         call princompl ( sigma, B%m, 'sigma' )
         call princompl ( flux, B%m, 'flux' )
      end if  ! ( debug )

!     --- Fill element matrix

      if ( E%notmat==0 ) then

         select case ( E%typenatbnd )

         case(2)

!        --- Boundary conditions of type 2

            call sepelmassmatc ( sigma ) ! zero term

         case default

!        --- Boundary conditions not yet implemented

            call errint ( E%typenatbnd, 1 )
            call errsub ( 3216, 1, 0, 0 )
            go to 1000

            call sepelconvect

         end select  ! case ( E%typenatbnd )

      end if  ! ( E%notmat==0 )

!     --- Fill element vector

      if ( E%notvec==0 ) then

         if ( E%typenatbnd==6 ) then

!        --- Option not yet implemented

            call errint ( E%typenatbnd, 1 )
            call errsub ( 3216, 1, 0, 0 )
            go to 1000

         end if  ! ( E%typenatbnd==6 )

         call sepelcload ( flux ) ! source term

      end if  ! ( E%notvec==0 )

!     --- Remove basis functions and coefficients in case of last call

      if ( debug ) write(irefwr,1) 'last', E%last
      if ( E%last==1 ) call sepeldealloc

1000  if ( debug ) then

!     --- Debug information

         call sepprinelcmat
         write(irefwr,*) 'End sepelm152'

      end if  ! ( debug )
      call erclos ( 'sepelm152' )

      end subroutine sepelm152
