      subroutine sepelm250
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-06-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for linear elastic problems
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elasticity_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: thickness(:)
      logical :: plane

!     plane          If true we are dealing with plane stress
!     thickness      Thickness in quadrature points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPEL0250      Performs initialization for subroutine sepelm250
!     SEPELBASEFUNC  Compute information about basis functions
!     SEPELDEALLOC   Deallocate coefficients and basis functions for element
!                    subroutines
!     SEPELFILLVAR   Fill space dependent coefficients
!     SEPELMASELAS   Fill the element mass matrix for linear elasticity problems
!     SEPELMLINELAS  Fill element matrix for linear elasticity problem
!     SEPELOLDVECS   Fill old vectors for element
!     SEPELVLINELAS  Fill element vector for linear elasticity problem
!     SEPPRINELMAT   Print element matrix and vector
! **********************************************************************
!
!                       I/O
!
!     The following coefficients are used
!
!     1:  -
!     2:  Type of stress-strain relations. This parameter is equal to IGPROB-1
!         Possible values:
!                    igprob = irelat + 10*inonlin + 100*ispecial
!                    Possible values:
!            ------------------------------------------------------------
!            inonlin = 0
!            ------------------------------------------------------------
!            irelat  0: plane stress, isotropic material
!                       or 3D stress, isotropic material (Cartesian)
!                    1: plane strain, isotropic material (Cartesian)
!                    2: axi-symmetric stress, isotropic material
!                                                        (axi-symmetric)
!                    3: plane stress, orthotropic material (Cartesian)
!                    4: plane strain, orthotropic material (Cartesian)
!                    5: axi-symmetric stress, orthotropic material
!                       (axi-symmetric)
!                    6: User defined elasticity matrix (3x3) (Cartesian)
!                    7: User defined elasticity matrix (4x4) (axi-symmetric)
!                    8: User defined elasticity matrix (6x6) (3d)
!            -------------------------------------------------------------
!            inonlin>0
!            -------------------------------------------------------------
!            inonlin 1: Geometrical and physical nonlinear behaviour (3D)
!                    2: 1 + reduced integration of compressibility term
!                    3: nonlin + variable anisotropy
!                    4: 3 + reduced integration of compressibility term
!            irelat  0: 3D
!                    2: 2D axi symmetric
!            -------------------------------------------------------------
!            ispecial = 1: muscle contraction, requires ELMUSC
!                          contact TUE/WFW for info
!                       2: mass matrix is filled with linear stiffness matrix
!                          defined by the parameters mu_1 and mu_2
!            -------------------------------------------------------------
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Not yet used (must be 0) -> passed to fnmateri
!     5:  Not yet used (must be 0) -> test switch
!     6-26: Information about the elasticity matrix depending on the
!           value of IGPROB
!         IGPROB = 1, 2 or 3:
!            6:  E
!            7:  nu
!         IGPROB = 4 or 5:
!            6:  E1
!            7:  E2
!            8:  nu1 (This parameter is not used for IGPROB=4, put here nu2)
!            9:  nu2
!           10:  G2
!         IGPROB = 7:
!            6:  D11
!            7:  D21
!            8:  D22
!            9:  D31
!           10:  D32
!           11:  D33
!         IGPROB = 8:
!            6:  D11
!            7:  D21
!            8:  D22
!            9:  D31
!           10:  D32
!           11:  D33
!           12:  D41
!           13:  D42
!           14:  D43
!           15:  D44
!         IGPROB = 9:
!           6-26: 3d case
!         IGPROB = 10,11,12,20: 21 user defined parameters
!    27:  h (thickness of the plate: 2D Cartesian)
!         Default value 1
!    28:  F1  (Body force in 1-direction)
!    29:  F2  (Body force in 2-direction)
!    30:  F3  (Body force in 3-direction)
!    31:  epsx 0  (or epsr 0)             initial strain
!    32:  epsy 0  (or epsz 0)
!    33:  epsz 0  (or epstheta 0)
!    34:  gammaxy 0  (or gammarz 0)
!    35:  gammayz 0
!    36:  gammazx 0
!    37:  sigmax 0  (or sigmar 0)         initial stress
!    38:  sigmay 0  (or sigmaz 0)
!    39:  sigmaz 0  (or sigmatheta 0)
!    40:  tauxy 0   (or taurz 0)
!    41:  tauyz 0
!    42:  tauzx 0
!    43:  rho (parameter for time-derivative)
!         Default: 1
!    44:  mu_1
!    45:  mu_2
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Equilibrium equation:
!                         T
!    -div SIGMA = F, or -B  SIGMA = F
!     with:
!                   T
!     2D Cart: SIGMA = sigma , sigma , tau
!                           x       y     xy
!                   T
!     3D Cart: SIGMA = sigma , sigma , sigma , tau  , tau  , tau
!                           x       y       z     xy     yz     zx
!                            T
!     3D Axi-symmetrix: SIGMA = sigma , sigma , sigma     , tau
!                                    r       z       theta     rz
!               T
!     2D Cart: B = | d/dx   0  d/dy |
!                  |  0   d/dy d/dx |
!               T
!     3D Cart: B = | d/dx   0   0   d/dy  0   d/dz |
!                  |  0   d/dy  0   d/dx d/dz  0   |
!                  |  0    0   d/dz  0   d/dy d/dx |
!                        T
!     3D Axi-symmetric: B = | d/dr+1/r  0   -1/r    d/dz  |
!                           |  0       d/dz   0  d/dr+1/r |
!    Stress-strain relations
!    EPS = B U
!    Constitutive relations
!    SIGMA = D EPS
!        Each integral is approximated by a numerical rule of the shape:
!        / int(x) dx = sum  w  int(x )
!        e              k    k      k
!        the factors w  form the weights for the numerical integration
!                     k
!        In the axi-symmetric case they include the factor 2 pi r
!   IN CASE OF NON-LINEAR MATERIAL:
!        Equilibrium is solved in an iterative way. Shape changes are
!        taken into account via a Total-Lagrange approach.
!        METHOD:
!          c    *        4             c  c   c            c  *    c
!        Dw : ( P . Du +  N : (Du.F + F.Du ).F ) = fext -Dw : P . F
!        Dw     derivatives of weightfunctions
!        P      stress estimate
!        N      incremental stress/strain matrix
!        F      deformationgradient of estimate
!        Du     derivatives of displacement
!        F (and P) is updated util fext-fint is small enough.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm250' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm250'
         write(irefwr,*) 'matrix, vector', matrix, vector
         write(irefwr,1) 'ielem, ifirst', E%ielem, E%first
         write(irefwr,1) 'notvec, notmat', notvec, notmat
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%mat = 0d0
      E%vec = 0d0

!     --- Define element independent quantities
!         only for first element

      if ( E%first==0 ) call sepel0250
      if ( ierror/=0 ) go to 1000

!     --- Map relation between coefficients and specific names

      thickness => E%coeffs(:,22)

!     --- Fill basis functions

      call sepelbasefunc
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         call prinrl1 ( B%xyz, E%ndim, B%n, 'x' )
         call prinrl1 ( B%xyzg, E%ndim, B%m, 'xgauss' )
      end if  ! ( debug )

!     --- Fill old vectors in quadrature points if necessary

      if ( E%compold>0 ) call sepeloldvecs

      if ( debug ) then
         write(irefwr,1) 'isupcfvar, isubcfvar', E%isupcfvar, E%isubcfvar
         write(irefwr,*) 'B%fillqmat ', B%fillqmat
      end if  ! ( debug )

!     --- Fill variable coefficients if necessary

      if ( E%isupcfvar>=E%isubcfvar ) call sepelfillvar

!     --- Correct weights for thickness of plate for plane stress/strain

      if ( E%plane ) B%w = B%w*thickness

      if ( debug ) call prinrl ( B%w, B%m, 'w' )

      if ( E%modelelastnl>0 ) then

!      --- Nonlinear case, compute some special quantities

         call sepelm250nl

      end if  ! ( E%modelelastnl>0 )
      if ( matrix ) then

!     --- Fill element matrix

         if ( E%modelelastnl==0) then

!        --- Linear case

            call sepelmlinelas

         else

!        --- Non-linear case

            call sepelmnlinelas

         end if  ! ( E%modelelastnl==0)

      end if  ! ( matrix )

!     --- Fill element vector

      if ( vector .and. E%notvec==0 ) call sepelvlinelas

!     --- Fill element vector non-linear part

      if ( E%modelelastnl>0 )  call sepelvnlinelas

!     --- Fill element mass matrix if necessary

      if ( E%imas>0 ) call sepelmaselas

!     --- Remove basis functions and coefficients in case of last call

      if ( debug ) write(irefwr,1) 'last', E%last
      if ( E%last==1 ) then
         call sepeldealloc
         if ( E%b2filled ) then
            B => B2
            call sepeldealloc
         end if  ! ( E%b2filled )
         if ( E%modelelastnl>0 ) deallocate( E%detf0, E%gradred )
         deallocate ( E%Dmat, E%Bmat, E%wr, E%detf, E%detfr, &
                      E%bl, E%blr, E%pksest, E%pksr, E%ser )
      end if  ! ( E%last==1 )

1000  if ( debug ) then

!     --- Debug information

         call sepprinelmat
         write(irefwr,*) 'End sepelm250'

      end if  ! ( debug )
      call erclos ( 'sepelm250' )

      end subroutine sepelm250
