      subroutine sepelm251
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for boundary conditions for
!     linear elastic problem
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elasticity_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduleoldvecs
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: thickness(:)

!     thickness      Thickness in quadrature points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPEL0251      Fill element independent quantities for element type 251
!     SEPELBASEFUNC  Compute information about basis functions
!     SEPELDEALLOC   Deallocate coefficients and basis functions for element
!                    subroutines
!     SEPELFILLVAR   Fill space dependent coefficients
!     SEPELMLINELASB Fill element matrix for boundary element corresponding to
!                    linear elastic problem
!     SEPELOLDVECS   Fill old vectors for element
!     SEPELVLINELASB Fill element vector for boundary element corresponding to
!                    linear elastic problem
!     SEPPRINELMAT   Print element matrix and vector
! **********************************************************************
!
!                       I/O
!
!     The following coefficients are used
!
!     1:  Type of boundary condition
!         Possible values:
!       0,2: Prescribed load (in direction of local co-ordinates)
!         3: Prescribed load (T and theta given, 2D)
!         4: Prescribed load in outer normal and tangential direction
!            The direction of the element defines the outer normal
!     2:  Type of stress-strain relations. This parameter is equal to IGPROB-1
!         Possible values:
!         0: plane stress, isotropic material
!            or 3D stress, isotropic material (Cartesian)
!         1: plane strain, isotropic material (Cartesian)
!         2: axi-symmetric stress, isotropic material (axi-symmetric)
!         3: plane stress, orthotropic material (Cartesian)
!         4: plane strain, orthotropic material (Cartesian)
!         5: axi-symmetric stress, anisotropic material (axi-symmetric)
!         6: User defined elasticity matrix (3x3) (Cartesian)
!         7: User defined elasticity matrix (4x4) (axi-symmetric)
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Not yet in use:  set equal to 0
!     5:  Not yet in use:  set equal to 0
!     6:  Tx (x component of load or 1-component in local co-ordinate system)
!     7:  Ty (y component of load or 2-component in local co-ordinate system)
!     8:  Tz (z component of load or 3-component in local co-ordinate system)
!     9:  h  (thickness h, 2D only)
!    10:  alpha 11
!    11:  alpha 12
!    12:  alpha 13
!    13:  alpha 22
!    14:  alpha 23
!    15:  alpha 33
!    16-25: Not yet in use (must be zero)
!    Remark:
!    If iload = 3, T must be stored as coefficient 6 and theta as coeff 7
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm251' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm251'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%mat = 0d0
      E%vec = 0d0

!     --- Define element independent quantities
!         only for first element

      if ( E%first==0 ) call sepel0251

!     --- Map relation between coefficients and specific names

      thickness => E%coeffs(:,4)

!     --- compute basis functions and weights for numerical integration

      call sepelbasefunc
      if ( ierror/=0 ) go to 1000

!     --- Fill old vectors in quadrature points if necessary

      if ( E%compold>0 ) call sepeloldvecs

      if ( debug ) &
        write(irefwr,1) 'isupcfvar, isubcfvar', E%isupcfvar, E%isubcfvar

!     --- Fill variable coefficients if necessary

      if ( E%isupcfvar>=E%isubcfvar ) call sepelfillvar

!     --- Correct weights for thickness of plate for plane stress/strain

      if ( E%plane ) B%w = B%w*thickness

!     --- Fill parts of matrix if necessary

      if ( matrix .and. notmat/=1 ) then

!     --- matrix = true    compute element matrix

         call sepelmlinelasb

      end if

!     --- Fill element vector if necessary

      if ( vector .and. notvec/=1 ) then

!     --- vector = true, compute element vector

          call sepelvlinelasb

      end if
      if ( E%last==1 ) call sepeldealloc

1000  if ( debug ) then

!     --- Debug information

         call sepprinelmat
         write(irefwr,*) 'End sepelm251'

      end if  ! ( debug )
      call erclos ( 'sepelm251' )

      end subroutine sepelm251
