      subroutine sepelmassmatc ( rho )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the element mass matrix in case of one unknown per node
!     Consistent mass matrix
!     Complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     mass_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulebuild
      use sepmoduleelm
      use sepmodulebasefn
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double complex, intent(in) :: rho(B%m)

!     rho            i    density rho cp
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j
      double complex :: work(B%m), selem(B%n,B%n)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     i              General loop variable
!     j              General loop variable
!     selem          Work array to store the local element matrix
!     work           work array to store rho*w
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The element mass matrix has elements
!
!     / rho phi_j psi_i d Omega
!
!     Hence m_ij = sum_k rho(x_k) w_k phi_j(x_k) psi_i(x_k)
!
!     At this moment the transpose of the matrix is stored
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelmassmatc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelmassmatc'
         write(irefwr,1) 'm, n', B%m, B%n
         call prinrl ( rho, B%m, 'rho' )
         call prinrl ( B%w, B%m, 'w' )
         call prinrl1 ( B%phi, B%m, B%n, 'phi' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(iseqim)

      work = rho*B%w
      kstep = B%kstep

      selem = 0d0

      if ( debug) call prinrl ( work, B%m, 'work' )

      if ( B%jdiag==1 ) then

!     --- jdiag=1, phi_ik = delta_ik


         do i = kstep, B%n, kstep
            selem(i,i) = work(i)
         end do  ! i =

      else

!     --- jdiag#1, phi general

         do i = 1, B%n
            do j = 1, B%n
               selem(j,i) = sum ( work*B%phi(j,:)*B%phi(i,:) )
            end do  ! j = 1, B%n
         end do  ! i = 1, B%n

      end if  ! ( B%jdiag==1 )

      E%cmat = E%cmat+selem

      if ( IM%lapshift ) then

!     --- IM%lapshift


         if ( B%jdiag==1 ) then
            do i = 1, B%n
               E%cmat2(i,1) = selem(i,i)
            end do  ! i = 1, n
         else
            do i = 1, B%n
               E%cmat2(i,1) = sum(selem(i,1:B%n))
            end do
         end if  ! ( B%jdiag==1 )

      end if  ! ( IM%lapshift )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelmassmatc'

      end if  ! ( debug )
      call erclos ( 'sepelmassmatc' )

      end subroutine sepelmassmatc
