      subroutine sepelmat302
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix for type 302
!
! **********************************************************************
!
!                       KEYWORDS
!
!     bearing
!     elasticity_equation
!     element_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduletime
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j
      double precision, pointer :: dphidx(:), w(:), u(:)
      double precision, pointer :: eta(:), k0(:)
      double precision :: c, s(6,6)

!     c              variable to store a constant temporarily
!     dphidx         Array of length n x m x ndim  containing the
!                    derivatives
!                    of the basis functions in the sequence:
!                         d phi_i/ dx_j(xg_k) = dphidx (i,k,j);
!     eta            viscosity in quadrature points
!     i              Counting variable
!     j              Counting variable
!     k0             Constant k in the equations
!     s              Element matrix to be filled
!     u              velocity in quadrature points
!     w              quadrature weights
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    store element matrix in the following form:
!          |  0      0      S    |
!          |                 vp  |
!          |  S      0      S    |
!          |   wv            wp  |
!          |  S      0      S    |
!          |   pv            pp  |
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelmat302' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelmat302'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dphidx => B%phider(:,1,1)
      w => B%w
      eta => E%coeffs(:,2)
      k0 => E%coeffs(:,3)
      u => E%coeffs(:,5)

      s = 0d0

!     --- Picard part
!         S_vp  (i,j) = -Kronecker delta_ij w_i

      s(1,5)=-E%waxi(1)
      s(2,6)=-E%waxi(2)

!     --- S_wp(i,j) = -1/2 dphi_j /dz_i h (x_i ) w

      if ( E%tangstress==0 ) then
         do i=1,n
            do j=1,n
               s(2+i,j+4)=-0.5d0*dphidx(j)*E%hn(i)*E%waxi(i)
            end do  ! j=1,n
         end do  ! i=1,n
      end if  ! ( E%tangstress==0 )

!     --- S_pv (i,j) = -1/2 U(x_i) dphi_i /dz w_j

      do i=1,n
         do j=1,n
            s(4+i,j)=-0.5d0*u(j)*dphidx(i)*w(j)
         end do  ! j=1,n
      end do  ! i=1,n

!     --- S_pp (i,j) = sum_k w_k h(x_k )/(12eta(x_k )) dphi j/dz dphi i/dz +
!               kappa(x_i)Kronecker delta_ij w_i

      c=(w(1)*E%hn(1)**3/eta(1)+ w(2)*E%hn(2)**3/eta(2))/12d0
      do i=1,n
         do j=1,n
            s(4+i,4+j)=c*dphidx(j)*dphidx(i)
          end do  ! j=1,n
      end do  ! i=1,n
      if ( E%ind(3)/=0 ) then
         s(5,5)=s(5,5)+w(1)*k0(1)
         s(6,6)=s(6,6)+w(2)*k0(2)
      end if  ! ( E%ind(3)/=0 )

      if ( E%newton ) then

!        --- newton = true    add terms for Newton linearisation
!                                                   n               n 2
!            S  (i,j) = -Kronecker delta   w [1/2 dp /dz - eta U /(h ) ]
!             wv                        ij  i                 i i   i

         if ( E%tangstress==0 ) then
            s(3,1)=-(0.5d0*E%dpndz+eta(1)*u(1)/ E%hn(1)**2)*E%waxi(1)
            s(4,2)=-(0.5d0*E%dpndz+eta(2)*u(2)/ E%hn(2)**2)*E%waxi(2)
         end if

!     -- S_pv (i,j) = (h_j ) / (4 eta_j ) dp /dz dphi_i/dz w_j

         do i=1,n
            do j=1,n
               s(4+i,j)=s(4+i,j)+E%hn(j)**2/(4d0*eta(j))*E%dpndz*dphidx(i)*w(j)
              end do  ! j=1,n
         end do  ! i=1,n

      end if  ! ( E%newton )
      if ( E%timedep ) then

!     --- timdep = true    add terms for Time dependent part
!
!         S_pv (i,j) = Kronecker delta_ij w_i / (theta deta t)

         s(5,1)=s(5,1)+w(1)/(tstep*theta)
         s(6,2)=s(6,2)+w(2)/(tstep*theta)

      end if

!     --- rearrange element matrix

      do i = 1, 3
        do j = 1, 3
           E%mat(i,j) = s(2*j-1, 2*i-1)
           E%mat(i+3,j) = s(2*j-1, 2*i)
           E%mat(i,j+3) = s(2*j, 2*i-1)
           E%mat(i+3,j+3) = s(2*j, 2*i)
        end do  ! j = 1, 3
      end do  ! i = 1, 3

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelmat302'

      end if  ! ( debug )
      call erclos ( 'sepelmat302' )

      end subroutine sepelmat302
