      subroutine sepeloldcvecs4 ( work, vertex, centroid, j, ivec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array work in all nodes for degree of freedom j
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     coefficient
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: j, ivec
      double complex, intent(out) :: work(inpelm)
      logical, intent(in) :: centroid, vertex

!     centroid       i    If true the function is available in the centroid
!     ivec           i    Sequence number of vector in set of old vectors
!     j              i    Sequence number of degree of freedom
!     vertex         i    If true not only the vertices have been filled
!     work           o    work array of length n in which one component of
!                         one vector in all nodes is stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k, istart, lenindex3

!     istart         Starting position of node in index3n
!     k              General loop variable (over integration points)
!     lenindex3      Length of array index3n
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINCOMPL      Print 1d complex vector
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1774   Only constant numbers of degrees of freedom permitted
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeloldcvecs4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeloldcvecs4'
         write(irefwr,1) 'j, itype, npelm', j, E%itype, E%npelm
         write(irefwr,1) 'n, ielem', B%n, E%ielem
         write(irefwr,*) 'vertex, centroid', vertex, centroid
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      lenindex3 = size(index3n)
      if ( lenindex3<=1 ) then

!     --- lenindex3<= 1, even not all vertices have been filled
!         Probably this concerns an element ot type 0 for this problem

         work = 0d0  ! clear work and return
         go to 1000

      end if

      if ( .not. vertex ) then

!     --- unknown only available in the vertices

         if ( E%ishape==2 ) then

!        --- quadratic line element

            istart = 0
            do k = 1, E%npelm, 2
               work(k) = cvecold(index3n(istart+j))
               istart = index4n(2)
            end do  ! k = 1, E%npelm, 2
            work(2) = 0.5d0 * ( work(1)+work(3) )

         else if ( E%ishape<=7 ) then

!        --- quadratic surface element

            istart = 0
            do k = 1, E%npelm-1, 2
               work(k) = cvecold(index3n(istart+j))
               istart = index4n(k+1)
            end do  ! k = 1, E%npelm-1, 2
            do k = 2, E%npelm-2, 2
               work(k) = 0.5d0 * ( work(k-1)+work(k+1) )
            end do  ! k = 2, E%npelm-2, 2
            if ( E%ishape==6 ) then
               work(8) = 0.5d0 * ( work(1)+work(7) )
               work(9) = 0.25d0*(work(2)+work(4)+work(6)+work(8))
            else if ( E%ishape==4 ) then
               work(6) = 0.5d0 * ( work(1)+work(5) )
            else
               work(6) = 0.5d0 * ( work(1)+work(5) )
               work(7) = (work(1)+work(3)+work(5))/3d0
            end if  ! ( E%ishape==6 )

         else if ( E%ishape==12 ) then

!        --- quadratic tetrahedron

            istart = 0
            do k = 1, 5, 2
               work(k) = cvecold(index3n(istart+j))
               istart = index4n(k+1)
            end do  ! k = 1, 5, 2
            istart = index4n(9)
            work(10) = cvecold(index3n(istart+j))
            work(2) = 0.5d0 * ( work(1)+work(3) )
            work(4) = 0.5d0 * ( work(3)+work(5) )
            work(6) = 0.5d0 * ( work(1)+work(5) )
            work(7) = 0.5d0 * ( work(1)+work(10) )
            work(8) = 0.5d0 * ( work(3)+work(10) )
            work(9) = 0.5d0 * ( work(5)+work(10) )

         else if ( E%ishape==14 ) then

!        --- quadratic hexahedron

            istart = 0
            work(1) = cvecold(index3n(istart+j))
            istart = index4n(2)
            work(3) = cvecold(index3n(istart+j))
            istart = index4n(6)
            work(7) = cvecold(index3n(istart+j))
            istart = index4n(8)
            work(9) = cvecold(index3n(istart+j))
            istart = index4n(18)
            work(19) = cvecold(index3n(istart+j))
            istart = index4n(20)
            work(21) = cvecold(index3n(istart+j))
            istart = index4n(24)
            work(25) = cvecold(index3n(istart+j))
            istart = index4n(26)
            work(27) = cvecold(index3n(istart+j))
            work(2) = 0.5d0 * ( work(1)+work(3) )
            work(4) = 0.5d0 * ( work(1)+work(7) )
            work(6) = 0.5d0 * ( work(3)+work(9) )
            work(8) = 0.5d0 * ( work(7)+work(9) )
            work(5) = 0.25d0*(work(2)+work(4)+work(6)+work(8))
            work(20) = 0.5d0 * ( work(19)+work(21) )
            work(22) = 0.5d0 * ( work(19)+work(25) )
            work(24) = 0.5d0 * ( work(21)+work(27) )
            work(26) = 0.5d0 * ( work(25)+work(27) )
            work(23) =0.25d0*(work(20)+work(22)+work(24)+work(26))
            work(10) = 0.5d0 * ( work(1)+work(19) )
            work(12) = 0.5d0 * ( work(3)+work(21) )
            work(16) = 0.5d0 * ( work(7)+work(25) )
            work(18) = 0.5d0 * ( work(9)+work(27) )
            work(11) = 0.5d0 * ( work(10)+work(12) )
            work(13) = 0.5d0 * ( work(10)+work(16) )
            work(15) = 0.5d0 * ( work(12)+work(18) )
            work(17) = 0.5d0 * ( work(16)+work(18) )
            work(14) =0.25d0*(work(11)+work(13)+work(15)+work(17))

         else

!        --- Error 1774:  Only constant number of unknowns per point
!                         are permitted

            call errint ( ivec, 1 )
            call errsub ( 1774, 1, 0, 0 )
            go to 1000

         end if  ! ( E%ishape==2 )

      else if ( .not. centroid ) then

!     --- unknown available in all points except centroid

         if ( E%ishape/=14 ) then

!        --- ishape # 14

            istart = 0
            do k = 1, E%npelm-1
               work(k) = cvecold(index3n(istart+j))
               istart = index4n(k)
            end do  ! k = 1, E%npelm-1
            if ( E%npelm==4 ) then
               work(4) = (work(1)+work(2)+work(3))/3d0
            else if ( E%npelm==5 ) then
               work(5) = 0.25d0*(work(1)+work(2)+work(3)+work(4))
            else if ( E%npelm==7 ) then
               work(7) = (-work(1)-work(3)-work(5) + &
                         4d0*(work(2)+work(4)+work(6)) )/ 9d0
            else if ( E%ishape==6 ) then
               work(9) = 0.25d0*(work(2)+work(4)+work(6)+work(8))
            else if ( E%ishape==17 ) then
               work(9) = 0.125d0*(work(1)+work(2)+work(3)+work(4)+&
                                  work(5)+work(6)+work(7)+work(8))
            else

!           --- Error 1774:  Only constant number of unknowns per point
!                            are permitted

               call errint ( ivec, 1 )
               call errsub ( 1774, 1, 0, 0 )
               go to 1000

            end if  ! ( E%npelm==4 )

         else

!        --- ishape = 14

            istart = 0
            do k = 1, E%npelm
               work(k) = cvecold(index3n(istart+j))
               istart = index4n(k)
            end do  ! k = 1, E%npelm
            work(14) = 0.125d0*(work(1)+work(3)+work(7)+work(9)+ &
                       work(19)+work(21)+work(25)+work(27))
         end if  ! ( E%ishape/=14 )

      else

!     --- Standard case, unknown available in all points

         istart = 0

         if ( debug ) then
            call prinin ( index4n, n, 'index4n' )
            call prinin ( index3n, lenindex3, 'index3n' )
         end if  ! ( debug )

         n = B%n
         if ( n>E%npelm ) n = E%npelm
         work(1:n) = 0d0
         do k = 1, n
            if ( index4n(k)-istart>=j ) &
               work(k) = cvecold(index3n(istart+j))
            istart = index4n(k)
         end do  ! k = 1, n

         if ( debug ) call princompl ( work, n, 'work' )

      end if  ! ( .not. vertex )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeloldcvecs4'

      end if  ! ( debug )
      call erclos ( 'sepeloldcvecs4' )

      end subroutine sepeloldcvecs4
