      subroutine sepeloldvecsk ( ivec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill old vectors for element (only one vector)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     coefficient
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleelm
      use sepmoduleoldvecs
      use sepmodulebasefn
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ivec

!     ivec           i    Sequence number of vector in set of old vectors
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ndegfd, i, j, k, itypvc, &
                 lenindex3, lenoldvec
      double precision :: save1, work(inpelm+1)
      double precision, pointer :: vecloc(:,:), veclocnode(:,:)
      logical :: centroid, vertex

!     centroid       If true the function is available in the centroid
!     i              General loop variable
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     j              General loop variable (over degrees of freedom)
!     k              General loop variable (over integration points)
!     lenindex3      Length of array index3n
!     lenoldvec      length of array oldvecs
!     ndegfd         Number of degrees of freedom per point
!     save1          Help variable to store a constant
!     vecloc         Local vector defined per element
!     veclocnode     Local velocity vector defined per node
!     vertex         If true not only the vertices have been filled
!     work           work array of length n in which one component of
!                    one vector in all nodes is stored
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL2        Print a real two-dimensional array
!     SEPELOLDVECS1  Fill array vecloc in case of vectors of type 126
!     SEPELOLDVECS2  Fill array vecloc in case of vectors of type 129
!     SEPELOLDVECS3  Compute centroid and vertex
!     SEPELOLDVECS4  Fill array work in all nodes for degree of freedom j
!     SEPELOLDVECS5  Compute ndegfd
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1774   Only constant numbers of degrees of freedom permitted
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeloldvecsk' )
      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeloldvecsk'
         call ertrace
         write(irefwr,1) 'ivec, maxunk, npelm', ivec, E%maxunk, E%npelm
         write(irefwr,1) 'm ,n', m, n
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      centroid = .true.
      vertex = .true.

      iseqtype = oldsols(ivec)%iseqtype
      if ( debug ) write(irefwr,1) 'iseqtype', iseqtype
      lenindex3 = indoldsols(iseqtype)%lenindex3
      index3n => indoldsols(iseqtype)%index3(1:lenindex3)
      index4n => indoldsols(iseqtype)%index4(1:E%npelm+1)
      lenoldvec = oldsols(ivec)%lenoldvec
      vecold => oldsols(ivec)%oldvecs(1:lenoldvec)
      vecloc => oldsols(ivec)%locvecs(1:m,1:E%maxunk)
      veclocnode => oldsols(ivec)%locvecsnode(1:n,1:E%maxunk)
      itypvc = index4n(E%npelm+1)
      if ( debug ) then
         write(irefwr,1) 'ivec, iseqtype', ivec, iseqtype
         call prinin ( index4n, inpelm, 'index4n' )
         write(irefwr,1) 'index4n(inpelm+1)', itypvc
      end if

      select case ( itypvc )

      case ( -2 )

!     --- Vector has not been filled: skip

         ndegfd = 0

      case ( -1 )

!     --- Number of unknowns per point not constant
!         At this moment only the centroid may have a different number
!         of unknowns per point
!         Compute centroid and vertex and fill ndegfd

         call sepeloldvecs5 ( ndegfd, ivec )

      case ( 1 )

!     --- index4n(inpelm+1) = 1, one unknown in each point

         ndegfd = 1

      case ( 2 )

!     --- index4n(inpelm+1) = 2, constant number of unknowns in each
!         point

         ndegfd = index4n(1)

      case ( 110 )

!     --- vector of type 110

         ndegfd = index4n(1)
         do i = 1, inpelm-1
            if ( index4n(i+1)-index4n(i)/=ndegfd ) then

!           --- Number of degrees of freedom not constant

               call errint ( ivec, 1 )
               call errsub ( 1774, 1, 0, 0 )
               go to 1000

            end if  ! ( index4n(i+1)-index4n(i)/=ndegfd )
         end do  ! i = 2, inpelm

      case ( 116 )

!     --- Vector of type 116

         ndegfd = index4n(1)
         do j = 1, ndegfd
            save1 = vecold(index3n(j))
            vecloc(1:m,j) = save1
         end do  ! j = 1, ndegfd
         ndegfd = 0

      case ( 126 )

!     --- Vector of type 126
!         Fill vecloc and make ndegfd 0

         call sepeloldvecs1 ( ndegfd, vecloc )

      case ( 129 )

!     --- Vector of type 129
!         Fill vecloc and make ndegfd 0

         call sepeloldvecs2 ( ndegfd, vecloc )

      case default

!     --- Error 1774:  Only constant number of unknowns per point are
!                      permitted

         call errint ( ivec, 1 )
         call errsub ( 1774, 1, 0, 0 )
         go to 1000

      end select  ! case ( itypvc )

!        --- Loop over degrees of freedom

      if ( debug ) write(irefwr,*) 'ndegfd, vertex, centroid ', &
                                    ndegfd, vertex, centroid
      do j = 1, ndegfd

!     --- Fill array work with j^th degree of freedom for all nodes

         if ( itypvc==-1 ) &
            call sepeloldvecs3 ( j, centroid, vertex, ivec )

         call sepeloldvecs4 ( work, vertex, centroid, j, ivec )

         if ( debug ) call prinrl ( work, n, 'work' )

!        --- Copy information from array work into vecloc

         if ( B%jdiag==1 ) then

!        --- jdiag = 1, integration points are nodal points

            vecloc(1:n,j) = work(1:n)
            if ( n==E%npelm+1 ) then

               if ( n==7 ) then
                  work(7) = (-work(1)-work(3)-work(5) + &
                             4d0*(work(2)+work(4)+work(6)) )/ 9d0

!                 --- in case of overshoot or undershoot we use
!                     linear interpolation

                  if ( work(7)<minval(work(1:6))*0.999d0 .or. &
                       work(7)>maxval(work(1:6))*1.001d0 ) &
                       work(7) = (work(2)+work(4)+work(6))/3d0
                  vecloc(7,j) = work(7)

               else if ( n==4 ) then
                  vecloc(4,j) = (work(1)+work(2)+work(3))/3d0
               else if ( n==5 ) then
                  vecloc(5,j) = &
                     (work(1)+work(2)+work(3)+work(4))*0.25d0
               end if  ! ( n==7 )

            end if  ! ( n==E%npelm+1 )
            veclocnode = vecloc

         else

!        --- jdiag = 2, integration points and nodal points are different

            if ( n==E%npelm+1 ) then
               if ( n==7 ) then
                  work(7) = (-work(1)-work(3)-work(5) + &
                             4d0*(work(2)+work(4)+work(6)) )/ 9d0
               else if ( n==4 ) then
                  work(4) = (work(1)+work(2)+work(3))/3d0
               else if ( n==5 ) then
                  work(5) = (work(1)+work(2)+work(3)+work(4))*0.25d0
               end if  ! ( n==7 )
            end if  ! ( n==E%npelm+1 )

            if ( debug ) call prinrl ( work, E%npelm, 'work/1' )

            if ( B%type_basisfunc==30 ) then

!           --- hermite polynomials, use lambda

               do k = 1, m
                  vecloc(k,j) = &
                     dot_product (work(1:E%npelm), B%lam(1:E%npelm,k))
               end do  ! k = 1, m
               veclocnode(1:E%npelm,j) = work(1:E%npelm)

            else

!           --- standard case

               do k = 1, m
                  vecloc(k,j) = dot_product (work(1:n), B%phi(1:n,k))
               end do  ! k = 1, m
               veclocnode(1:n,j) = work(1:n)

            end if  ! ( B%type_basisfunc==30 )

         end if  ! ( B%jdiag==1 )

      end do  ! j = 1, ndegfd

1000  if ( debug ) then

!     --- Debug information

         call prinrl2 ( vecloc, m, ndegfd, 'vecloc' )
         call prinrl2 ( veclocnode, n, ndegfd, 'veclocnode' )
         write(irefwr,*) 'End sepeloldvecsk'

      end if  ! ( debug )
      call erclos ( 'sepeloldvecsk' )

      end subroutine sepeloldvecsk
