      subroutine sepelp01qmat ( t1, h )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array qmat with transformation to local coordinates
!     Linear 1-dimensional elements
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleelm
      use sepmodulebasefn
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(inout) :: t1(E%ndim)
      double precision, intent(in) :: h

!     h              i    Length of element
!     t1             i    Tangential vector along element (x2-x1)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k
      double precision, pointer :: qmat(:,:)
      double precision :: hloc, t2(E%ndim), normal(E%ndim)

!     hloc           Help parameter tp compute quantities
!     k              General loop variable
!     normal         Normal vector to t1
!     t2             vector perpendicular to t1 and normal, hence second normal
!     qmat           contains transformation matrix of global to local
!                    coordinates
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELOUTR         Compute cross product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp01qmat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp01qmat'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Associate pointers with arrays in modules

      qmat => B%qmat(:,:,1)

!     --- Normalize tangential vector

      hloc = 1d0/h
      t1 =t1*hloc

      if ( E%ndim==2 ) then

!     --- Simple 2d case

         normal(1) = t1(2)
         normal(2) = -t1(1)
         t2 = 0d0

      else

!     --- 3d case
!         Compute index of smallest component of |t2|

         k = maxloc(abs(t2),1)

!        --- Set t2 equal to ek

         t2 = 0d0
         t2(k) = 1d0

!        --- normal = (qmat(.,3) x ej) / ||qmat(.,3) x ej||

         call eloutr ( t1, t2, normal )
         hloc = 1d0/sqrt(dot_product(normal,normal))
         t2 = hloc*normal

!        --- normal = t1 x t2

         call eloutr ( t1, t2, normal )

      end if  ! ( E%ndim==2 )

      qmat(1,:) = normal
      qmat(2,:) = t1
      qmat(3,:) = t2

!     --- Copy to all other quadrature points

      do k = 2, B%m
         B%qmat(1:3,1:E%ndim,k) = B%qmat(1:3,1:E%ndim,1)
      end do  ! k = 2, B%m

1000  call erclos ( 'sepelp01qmat' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( qmat, E%ndim, 3, 'qmat' )
         write(irefwr,*) 'End sepelp01qmat'

      end if  ! ( debug )

      end subroutine sepelp01qmat
