      subroutine sepelp05det ( a, jac, det )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the Jacobian determinant of the bilinear quadrilateral
!     Old subroutine: elp621det
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadrilateral
!     linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmoduleelm
      use sepmodulecomio
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: a(3,2)
      double precision, intent(out) :: jac(B%m), det(4)

!     a              i    Contains the coefficients of the bi-linear polynomial
!                         except the constant part a0
!     det            o    Jacobian in nodes
!     jac            o    Array of length m containing the values of the
!                         Jacobian in the vertices or for the Gauss integration
!                         rules work array to store some help variables
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: j0, j1, j2
      logical :: allneg, allpos

!     allneg         Indication the values of the Jacobian in the vertices
!                    are all negative (true) or not (false)
!     allpos         Indication the values of the Jacobian in the vertices
!                    are all positive (true) or not (false)
!     j0             Constant part of jacobian
!     j1             Linear part of jacobian
!     j2             Linear part of jacobian
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     230   Jacobian of element is zero or has changing sign
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp05det' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp05det'
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute the determinant J in the vertices and store in det
!         a1 b2 - a2 b1 + (a1 b3 - a3 b1) xsi + (a3 b2 - a2 b3) eta

      J0 = a(1,1) * a(2,2) - a(2,1) * a(1,2)
      J1 = a(1,1) * a(3,2) - a(3,1) * a(1,2)
      J2 = a(3,1) * a(2,2) - a(2,1) * a(3,2)

      if ( debug ) write(irefwr,2) 'J', J0, J1, J2

      det(1) = J0
      det(2) = J0 + J1
      det(3) = J0 + J1 + J2
      det(4) = J0 + J2
      if ( debug ) call prinrl ( det, 4, 'det' )

      allneg = det(1)<0 .and. det(2)<0 .and. det(3)<0 .and. det(4)<0
      allpos = det(1)>0 .and. det(2)>0 .and. det(3)>0 .and. det(4)>0
      if ( checksign .and. allneg ) then
         write(irefwr,*) 'area of element ', E%ielem, ' is negative'
      end if

      if( .not. allneg .and. .not. allpos ) then

         call errint ( E%ielem, 1 )
         call errint ( E%itype, 2 )
         call errint ( E%ielgrp, 3 )
         call errsub ( 230, 3, 0, 0 )
         call prinin(E%nodes,B%n,'node numbers')
         call prinrl2(B%xyz,B%n,2,'co-ordinates')

      end if

      if ( B%rule>1 .or. B%m/=4 ) then

!     --- Gauss rule, recompute jac in integration points

         jac(:) = J0+J1*B%xyzgc(:,1)+J2*B%xyzgc(:,2)

      else

!     --- Newton Cotes rule

         jac = det

      end if  ! ( B%rule>1 )

1000  call erclos ( 'sepelp05det' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp05det'

      end if  ! ( debug )

      end subroutine sepelp05det
