      subroutine sepelp05jacob
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute Jacobian and derivatives of basis functions
!     Bi-linear quadrilateral (4 nodes)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k, n, m, i
      double precision :: a(3,2), h, det(4)
      double precision, pointer :: dxidx(:,:,:), xyzg(:,:), xyz(:,:)

!     a              Contains the coefficients of the bi-linear polynomial
!                    except the constant part a0
!     det            Contains the Jacobian for the nodes
!     dxidx          work array of length 9x2x3 to store the values of
!                    dxi/dx, dxi/dy, dxi/dz and deta/dx ... according to:
!                    dxi/dx(x^k) = dxdxi(k,1,1)  dxi/dy(x^k) = dxdxi(k,1,2)
!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     k              Counting variable
!     m              number of quadrature points
!     n              number of basis functions
!     xyz            coordinates of nodes of elements
!     xyzg           coordinates of Gauss points of elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL2        Print a real two-dimensional array
!     SEPELP05DET    Compute the Jacobian determinant of the bilinear
!                    quadrilateral
!     SEPELP05NC     Compute the derivatives of the basis functions in the nodes
!                    of an bi-linear quadrilateral (4 nodes)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp05jacob' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp05jacob'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      m = B%m
      n = B%n

      dxidx => B%dxidx
      xyzg => B%xyzgc
      xyz => B%xyz

!     --- Compute the quantities  a_1 until a_3, store in a(1:3,1:2)

      a(1,:) = xyz(2,1:2)-xyz(1,1:2)
      a(2,:) = xyz(4,1:2)-xyz(1,1:2)
      a(3,:) = xyz(3,1:2)-xyz(2,1:2)-a(2,:)

      if ( debug ) call prinrl2 ( a, 3, 2, 'a' )

!     --- Compute the determinant J in the vertices and store in jac
!         a1 b2 - a2 b1 + (a1 b3 - a3 b1) xsi + (a3 b2 - a2 b3) eta

      call sepelp05det ( a, B%jac, det )

!     --- compute derivatives in reference element

      if ( B%fillderiv ) then

!     --- fillderiv = true, fill first derivatives

         if ( B%rule==1 .and. n/=5 .or. B%fillderivn ) then

!        --- rule = 1, only compute dxsi/dx
!            use the fact the xsi and eta are only 1 or 0

            dxidx(1,1,1) = a(2,2)/det(1)
            dxidx(2:3,1,1) = (a(2,2)+a(3,2))/det(2:3)
            dxidx(4,1,1) = a(2,2)/det(4)

            dxidx(1,1,2) = -a(2,1)/det(1)
            dxidx(2:3,1,2) = -(a(2,1)+a(3,1))/det(2:3)
            dxidx(4,1,2) = -a(2,1)/det(4)

            dxidx(1:2,2,1) = -a(1,2)/det(1:2)
            dxidx(3:4,2,1) = -(a(1,2)+a(3,2))/det(3:4)

            dxidx(1:2,2,2) = a(1,1)/det(1:2)
            dxidx(3:4,2,2) = (a(1,1)+a(3,1))/det(3:4)

            if ( B%rule==1 ) then

!           --- Rule = 1, fill array phider

               call sepelp05nc ( B%phider )
               if ( B%fillderivn ) B%phidern = B%phider

            else if ( B%fillderivn ) then

!           --- Rule > 1, fill array phidern

               call sepelp05nc ( B%phidern )

            end if  ! ( B%rule==1 )

         end if  ! ( B%rule==1 .and. n/=5 .or. B%fillderivn )

         if ( B%rule>1 .or. n==5 ) then

!        --- rule>1, Gauss rule

            do k = 1, m
               h = 1d0 / B%jac(k)
               dxidx(k,1,1) = (a(2,2)+a(3,2)*xyzg(k,1))*h  ! dxsi/dx
               dxidx(k,1,2) =-(a(2,1)+a(3,1)*xyzg(k,1))*h  ! dxsi/dy
               dxidx(k,2,1) =-(a(1,2)+a(3,2)*xyzg(k,2))*h  ! deta/dx
               dxidx(k,2,2) = (a(1,1)+a(3,1)*xyzg(k,2))*h  ! deta/dy
            end do  ! k = 1, m
            do i = 1, n

               do k = 1, m

                  B%phider(i,k,1:2) = B%phiderc(i,k,1)*dxidx(k,1,1:2) +&
                                      B%phiderc(i,k,2)*dxidx(k,2,1:2)

               end do  ! k = 1, m

            end do  ! i = 1, n

         end if  ! ( B%rule>1 .or. n=5 )

      end if  ! ( B%fillderiv )

1000  call erclos ( 'sepelp05jacob' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp05jacob'

      end if  ! ( debug )

      end subroutine sepelp05jacob
