      subroutine sepelp06jacstr ( jac )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute basis functions and related quantities for quadratic
!     quadrilateral (straight element)
!     See elp623straight
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
!     quadrilateral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision :: jac(*)

!     jac            o    Array of length m containing the values of the
!                         Jacobian in the vertices or for the Gauss integration
!                         rules work array to store some help variables
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, k, m, n
      double precision, pointer :: x(:,:), dphidx(:,:,:), xg(:,:), &
                                   dphidxi(:,:,:)
      double precision, pointer :: psixi(:,:,:), dpsidx(:,:,:)
      double precision :: a(3,2), dxidx(B%m,3,3), h, det(9), dxidxn(B%n,3,3)

!     a              Help variable to store quantity
!     det            Contains the Jacobian for the nodes
!     dphidx         Contains values of dphi/dx in quadrature points
!     dphidxi        Contains values of dphi/dxi in quadrature points
!     dpsidx         Contains values of dpsi/dx in quadrature points,
!                    with psi the linear basis functions
!     dxidx          work array of length 9x2x3 to store the values of
!                    dxi/dx, dxi/dy, dxi/dz and deta/dx ... according to:
!                    dxi/dx(x^k) = dxdxi(k,1,1)  dxi/dy(x^k) = dxdxi(k,1,2)
!     dxidxn         Contains dxidx in the nodes
!     h              1/jac(.)
!     i              Counting variable
!     k              Counting variable
!     m              number of quadrature points
!     n              number of basis functions
!     psixi          Contains the values of the derivatives of the
!                    linear basis functions in the Gauss points in
!                    the reference element.
!     x              coordinates of nodes
!     xg             coordinates of Gauss points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     PRINRL2        Print a real two-dimensional array
!     SEPELP05DET    Compute the Jacobian determinant of the bilinear
!                    quadrilateral
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       To compute the derivatives of the basis functions, the jacobian
!       of the element and the weights, the following definitions are used:
!
!
!     Straight boundaries:
!
!         ij       j     i+1    i
!        e   = (-1)  (  x    - x    )   cyclic: i=1,2,3,4 ;   j=1,2
!                        j+1    j+1
!
!              1     2    1          4    1          3    2    1    4
!        x  = x  + (x  - x ) xi + (x  - x ) eta + (x  - x  + x  - x ) xi eta
!         i    i     i    i          i    i          i    i    i    i
!
!                                   i = 1,2
!                  1    12        42
!        So: x  = x  + e   xi - e   eta + A xi eta
!             1    1
!
!                  1    11        41
!            x  = x  - e   xi + e   eta + B xi eta
!             2    2
!
!
!            |   2    1                  4    1          |
!            |  x  - x  + A eta         x  - x  + A xi  |
!            |   1    1                  1    1          |
!        J  =|                                           |
!            |   2    1                  4    1          |
!            |  x  - x  + B eta         x  - x  + B xi  |
!            |   2    2                  2    2          |
!
!
!             1    2    3    4        12    32
!        A = x  - x  + x  - x    = - e   - e
!             1    1    1    1
!
!
!             3    4    2    1       11    31
!             2    2    2    2
!
!
!       |J| = J  + J  xi + J  eta
!              0    1        2
!
!              41   12    11   42
!        J  = e  * e   - e  * e
!         0
!
!                   12         11
!        J  = B  * e   - A  * e
!         1
!
!                   41         42
!        J  = A  * e   + B  * e
!         2
!
!
!        dphi / dx  = dphi / dxi dxi/dx  + dphi / deta deta/dx
!                 j                      j                      j
!
!                       41
!        dxi/dx  = [  e   + B xi ] / |J|
!               1
!
!                       42
!        dxi/dx  = [  e   - A xi ] / |J|
!               2
!
!                       11
!        deta/dx  = [  e   - B eta ] / |J|
!               1
!
!                       12
!        deta/dx  = [  e   + A eta ] / |J|
!               2
!
!
!                                           9          k      k
!        Newton Cotes rule:  / f(x) dx =   sum [ w  f(x ) |J(x )| ]
!                            e             k=1    k
!
!        with w =w =w =w =1/36; w = w = w = w =4/36;   w = 16/36
!              1  3  7  9        2   4   6   8          5
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp06jacstr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp06jacstr'
         write(irefwr,*) 'fillderiv ', B%fillderiv

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      x => B%xyz
      xg => B%xyzgc
      dphidx => B%phider
      dphidxi => B%phiderc
      psixi => B%lamderc
      dpsidx => B%lamder

      m = B%m
      n = B%n

      if ( debug ) call prinrl1 ( xg, 2, m, 'xgauss' )

!     --- weights, or derivatives must be computed
!         Compute Jacobian in the Gauss points
!         Compute the quantities  a_1 until a_3, store in a(1:3,1:2)

      a(1,:) = x(3,:)-x(1,:)
      a(2,:) = x(7,:)-x(1,:)
      a(3,:) = x(5,:)-x(3,:)-a(2,:)

      if ( debug ) call prinrl2 ( a, 3, 2, 'a' )

      call sepelp05det ( a, jac, det )

      if ( B%rule==1 .or. B%fillderivn ) then

!     --- Recompute jac

         do k = 3, 1, -1
            det(2*k+1) = det(k+1)
         end do  ! k = 3, 1, -1
         do k = 1, 3
            det(2*k) = (det(2*k-1)+det(2*k+1))*0.5d0
         end do  ! k = 1, 3
         det(8) = 0.5d0 * ( det(7) + det(1) )
         det(9) = 0.25d0 * ( det(2) + det(4) + det(6) + det(8) )

         if ( B%rule==1 ) jac(1:9) = det(1:9)

      end if  ! ( B%rule==1 )

      if ( debug ) call prinrl ( jac, B%m, 'jac' )

      if ( B%fillderiv ) then

!     --- fillderiv = true, fill first derivatives
!         store dxi/dx in dxidx

         do k = 1, m
            h = 1d0 / jac(k)
            dxidx(k,1,1) = (a(2,2)+a(3,2)*xg(k,1))*h  ! dxsi/dx
            dxidx(k,1,2) =-(a(2,1)+a(3,1)*xg(k,1))*h  ! dxsi/dy
            dxidx(k,2,1) =-(a(1,2)+a(3,2)*xg(k,2))*h  ! deta/dx
            dxidx(k,2,2) = (a(1,1)+a(3,1)*xg(k,2))*h  ! deta/dy
         end do  ! k = 1, m

         if ( debug ) then
            call prinrl ( dxidx(1,1,1), m, 'dxsi/dx' )
            call prinrl ( dxidx(1,1,2), m, 'dxsi/dy' )
            call prinrl ( dxidx(1,2,1), m, 'deta/dx' )
            call prinrl ( dxidx(1,2,2), m, 'deta/dy' )
         end if  ! ( debug )

!        --- ideriv = 1   Fill derivatives

         do k = 1, m
            do i = 1, n
               dphidx(i,k,1:2) = dxidx(k,1,1:2)*dphidxi(i,k,1)+ &
                                 dxidx(k,2,1:2)*dphidxi(i,k,2)
            end do
         end do

      end if  ! ( B%fillderiv )

      if ( B%fillderivn .and. B%rule>1 ) then

!     --- fillderivn = true, fill first derivatives in nodes
!         store dxi/dx in dxidx

         xg => B%xyzgcn
         dphidx => B%phidern
         dphidxi => B%phidercn
         do k = 1, n
            h = 1d0 / det(k)
            dxidxn(k,1,1) = (a(2,2)+a(3,2)*xg(k,1))*h  ! dxsi/dx
            dxidxn(k,1,2) =-(a(2,1)+a(3,1)*xg(k,1))*h  ! dxsi/dy
            dxidxn(k,2,1) =-(a(1,2)+a(3,2)*xg(k,2))*h  ! deta/dx
            dxidxn(k,2,2) = (a(1,1)+a(3,1)*xg(k,2))*h  ! deta/dy
         end do  ! k = 1, n

         if ( debug ) then
            call prinrl ( dxidxn(1,1,1), n, 'dxsi/dx' )
            call prinrl ( dxidxn(1,1,2), n, 'dxsi/dy' )
            call prinrl ( dxidxn(1,2,1), n, 'deta/dx' )
            call prinrl ( dxidxn(1,2,2), n, 'deta/dy' )
         end if  ! ( debug )

!        --- ideriv = 1   Fill derivatives

         do k = 1, n
            do i = 1, n
               dphidx(i,k,1:2) = dxidxn(k,1,1:2)*dphidxi(i,k,1)+ &
                                 dxidxn(k,2,1:2)*dphidxi(i,k,2)
            end do
         end do

      else if ( B%fillderivn ) then

!     --- fillderivn = true, fill first derivatives in nodes

         B%phidern = B%phider

      end if  ! ( B%fillderiv )

      if ( B%comppsi ) then

!     --- comppsi true, compute derivatives of linear basis functions

         do k = 1, m
            do i = 1, 4
               dpsidx(i,k,1:2) = dxidx(k,1,1:2)*psixi(i,k,1)+ &
                                 dxidx(k,2,1:2)*psixi(i,k,2)
            end do
         end do

      end if  ! ( B%comppsi )

1000  call erclos ( 'sepelp06jacstr' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( dphidx(1,1,1), m, n, 'dphidx' )
         call prinrl1 ( dphidx(1,1,2), m, n, 'dphidy' )
         write(irefwr,*) 'End sepelp06jacstr'

      end if  ! ( debug )

      end subroutine sepelp06jacstr
