      subroutine sepelp07
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute information of basis functions
!     Quadratic triangle (7 nodes)
!     Old routine: elp624
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduleconsts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: xg(:), jac(:)
      integer :: i

!     jac            Jacobian in the quadrature points
!     xg             x-coordinates of Gauss points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPELP07FIRST  compute some element independent quantities
!                    Extended quadratic triangle (7 nodes)
!     SEPELP07JACOB  Compute the Jacobian of an element and store in jac
!                    Extended quadratic triangle (7 nodes)
!     SEPELPGAUSS    Fill coordinates of Gauss points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!        to compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!         ij        i+1    i+2    i+1
!        e   =  (-1)    ( x    - x    )   cyclic: i=1,2,3 ;   j=1,2
!                          j      j
!                    32  11    12 31
!        delta   =  e   e   - e  e
!                             ij
!        d lambda  / dx   =  e   /  delta
!                i     j
!        lamc = lambda lambda lambda
!                     1      2      3
!        phi  =  lambda (2 lambda -1)+3 lamc  i=1,2,3   (basis functions:1,2,3)
!           i          i         i
!        phi  = 4  lambda   lambda - 12 lamc   1<=i<j<=3 (basis functions:4,5,6)
!           ij           i        j
!        phi   = 27 lamc
!           123
!        grad phi  = grad lambda  ( 4 lambda  - 1 ) + 3 grad lamc
!                i              i           i
!        grad phi  = 4 ( grad lambda  lambda  + grad lambda  lambda  ) - &
!                ij                 i       j              j       i
!                   12 grad lamc
!        grad phi    = 27 grad lamc
!                123
!        grad lamc = lambda lambda grad lambda  + lambda lambda grad lambda + &
!                          2      3           1         3      1           2
!                    lambda lambda grad lambda
!                          1      2           3
!       curved boundaries:
!            |  x          y    |
!            |   xi        xi |
!        j  =|                  |
!            |  x          y    |
!            |   eta        eta |
!                         7    k
!        d x / dxi   =  sum  x    d phi / d xi
!           i      j     k=1   i        k       j
!                          -1
!        grad    phi   =  j   grad        phi
!            xy     i             xi eta    i
!                                          7       k      k
!        newton cotes rule:  / f(x) dx =  sum [ f(x ) |j(x )| w ]
!                            e            k=1                  k
!        w  = 1/15     w   = 1/40    w    = 0.225
!         i             ij            123
!
!      To compute qmat, we use the lamina coordinate system as introduced by
!      Hughes:
!
!      Define e_xi = x,xi / ||x,xi||
!      Define e_eta = x,eta / ||x,eta||
!      e_3(l) = e_xi x e_eta / ||e_xi x e_eta||
!      e_alpha = (e_xi + e_eta)/2 / || (e_xi + e_eta)/2 ||
!      e_beta = e_3(l) x e_alpha / || e_3(l) x e_alpha ||
!      e_1(l) = srqt(2)/2 ( e_alpha - e_beta )
!      e_2(l) = srqt(2)/2 ( e_alpha + e_beta )
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp07' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp07'
         write(irefwr,1) 'ifirst, n, m', E%first, B%n, B%m
         write(irefwr,1) 'inpelm, irule', E%npelm, B%rule
         write(irefwr,1) 'kstep', B%kstep
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      xg => B%xyzg(:,1)
      jac => B%jac

      if ( E%first==0 ) call sepelp07first

!     --- Compute Jacobian and derivatives of basis functions

      call sepelp07jacob

!     --- Fill Gauss points

      call sepelpgauss

!     --- Fill weights

      if ( B%cart==1 ) then

!     --- B%cart = 1  Cartesian co-ordinates

         B%w = B%wc*abs(jac(1:B%m))

      else if ( B%cart>=2 ) then

!     --- B%cart = 2, 3  Axi-symmetric or polar co-ordinates

         B%w = abs(jac)*xg*B%wc
         if ( B%cart==2 ) B%w = B%w*pi2

      end if  ! ( B%cart==1 )

      if ( itype>=902 .and. itype<=903 ) then

!     --- Special case of Navier-Stokes element
!         w contains the weights
!         w2 the diagonal of the mass matrix

         do i = 1, B%n
            B%w2(i) = sum(B%w*B%phi(i,:)**2)
         end do  ! i = 1, B%n

      end if  ! ( itype>=902 .and. itype<=903 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( B%xyz, E%ndim, 7, 'x' )
         call prinrl1 ( B%xyzg, E%ndim, B%m, 'xgauss' )
         call prinrl2 ( B%phi, 7, B%m, 'phi' )
         call prinrl ( B%w, B%m, 'w' )
         call prinrl2 ( B%phider(1,1,1), B%n, B%m, 'dphidx' )
         call prinrl2 ( B%phider(1,1,2), B%n, B%m, 'dphidy' )
         write(irefwr,*) 'End sepelp07'

      end if  ! ( debug )
      call erclos ( 'sepelp07' )

      end subroutine sepelp07
