      subroutine sepelp07jacstr
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute basis functions and related quantities for quadratic
!     triangle (straight element)
!     Old subroutine  elp624straight
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: m, k, i
      double precision, pointer :: dphidx(:,:,:), lambda(:,:), dpsidx(:,:,:)
      double precision, pointer :: jac(:)
      double precision :: e1(3,2), h

!     dphidx         Contains the values of dphi/dx
!     dpsidx         Contains values of dpsi/dx in quadrature points,
!                    with psi the linear basis functions
!     e1             Work array of length 3 x 2, to store the factors
!                    e^ij See method
!     h              help variable to store a constant
!     i              Counting variable
!     jac            Jacobian in the quadrature points
!     k              Counting variable
!     lambda         Contains values of lambda in quadrature points
!     m              Number of quadrature points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL2        Print a real two-dimensional array
!     SEPELP03JACOB  Compute Jacobian and derivatives of basis functions
!                    Linear triangle (3 nodes)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp07jacstr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp07jacstr'
         write(irefwr,1) 'rule, n, m', B%rule, B%n, B%m
         write(irefwr,*) 'fillderiv, fillderivn', B%fillderiv, B%fillderivn
  1      format ( a, 1x, (10i8) )
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dphidx => B%phider
      lambda => B%lam
      dpsidx => B%lamder
      jac => B%jac
      m = B%m

!     --- First compute jacobian and e1

      B%xloc => B%xyz
      call sepelp03jacob ( e1, jac )
      jac(2:B%m) = jac(1)

      if ( debug ) then
         call prinrl ( jac, B%m, 'jacobian' )
         call prinrl2 ( e1, 3, 2, 'e1' )
      end if  ! ( debug )

      if ( B%fillderiv ) then

!     --- fillderiv = true, fill first derivatives

         if ( B%rule==1 .or. B%fillderivn ) then

            if ( B%rule>1 ) dphidx => B%phidern

            h = 1d0/3d0
            dphidx(1,1,:)=3*e1(1,:)
            dphidx(1,2,:)=e1(1,:)+0.75d0*e1(3,:)
            dphidx(1,3,:)=-e1(1,:)
            dphidx(1,4,:)=-e1(1,:)+0.75d0*e1(1,:)
            dphidx(1,5,:)=-e1(1,:)
            dphidx(1,6,:)=e1(1,:)+0.75d0*e1(2,:)
            dphidx(1,7,:)=e1(1,:)*h
            dphidx(2,1,:)=4*e1(2,:)
            dphidx(2,2,:)=-2*e1(3,:)-3*e1(3,:)
            dphidx(2,3,:)=4*e1(1,:)
            dphidx(2,4,:)=2*e1(1,:)-3*e1(1,:)
            dphidx(2,5,:)=0.0d0
            dphidx(2,6,:)=2*e1(2,:)-3*e1(2,:)
            dphidx(2,7,:)=-4*e1(3,:)*h
            dphidx(3,1,:)=-e1(2,:)
            dphidx(3,2,:)=e1(2,:)+0.75d0*e1(3,:)
            dphidx(3,3,:)=3*e1(2,:)
            dphidx(3,4,:)=e1(2,:)+0.75d0*e1(1,:)
            dphidx(3,5,:)=-e1(2,:)
            dphidx(3,6,:)=-e1(2,:)+0.75d0*e1(2,:)
            dphidx(3,7,:)=e1(2,:)*h
            dphidx(4,1,:)=0.0d0
            dphidx(4,2,:)=2*e1(3,:)-3*e1(3,:)
            dphidx(4,3,:)=4*e1(3,:)
            dphidx(4,4,:)=-2*e1(1,:)-3*e1(1,:)
            dphidx(4,5,:)=4*e1(2,:)
            dphidx(4,6,:)=2*e1(2,:)-3*e1(2,:)
            dphidx(4,7,:)=-4*e1(1,:)*h
            dphidx(5,1,:)=-e1(3,:)
            dphidx(5,2,:)=-e1(3,:)+0.75d0*e1(3,:)
            dphidx(5,3,:)=-e1(3,:)
            dphidx(5,4,:)=e1(3,:)+0.75d0*e1(1,:)
            dphidx(5,5,:)=3*e1(3,:)
            dphidx(5,6,:)=e1(3,:)+0.75d0*e1(2,:)
            dphidx(5,7,:)=e1(3,:)*h
            dphidx(6,1,:)=4*e1(3,:)
            dphidx(6,2,:)=2*e1(3,:)-3*e1(3,:)
            dphidx(6,3,:)=0.0d0
            dphidx(6,4,:)=2*e1(1,:)-3*e1(1,:)
            dphidx(6,5,:)=4*e1(1,:)
            dphidx(6,6,:)=-2*e1(2,:)-3*e1(2,:)
            dphidx(6,7,:)=-4*e1(2,:)*h
            dphidx(7,1,:)=0.0d0
            dphidx(7,2,:)=6.75d0*e1(3,:)
            dphidx(7,3,:)=0.0d0
            dphidx(7,4,:)=6.75d0*e1(1,:)
            dphidx(7,5,:)=0.0d0
            dphidx(7,6,:)=6.75d0*e1(2,:)
            dphidx(7,7,:)=0.0d0

         end if  ! ( B%rule==1 .or. B%fillderivn )
         dphidx => B%phider

         select case (B%rule)

         case(1)

!        ---rule  =  1  newton-cotes rule

            if ( B%fillderivn ) B%phidern = B%phider

         case(2, 3)

!        ---rule>1  gauss rule

            do k = 1, m
               dphidx(7,k,:) = lambda(2,k)*lambda(3,k)*e1(1,:) + &
                               lambda(3,k)*lambda(1,k)*e1(2,:) + &
                               lambda(1,k)*lambda(2,k)*e1(3,:)
            end do

            do i = 1, 3
               do k = 1, m
                  dphidx(2*i-1,k,:) = e1(i,:)*(4d0*lambda(i,k)-1d0)+ &
                                      3d0*dphidx(7,k,:)
               end do
            end do
            do k = 1, m
               dphidx(2,k,:) = 4d0*(lambda(1,k)*e1(2,:)+ &
                               lambda(2,k)*e1(1,:))- &
                               12d0*dphidx(7,k,:)
               dphidx(4,k,:) = 4d0*(lambda(2,k)*e1(3,:)+ &
                               lambda(3,k)*e1(2,:))- &
                               12d0*dphidx(7,k,:)
               dphidx(6,k,:) = 4d0*(lambda(3,k)*e1(1,:)+ &
                               lambda(1,k)*e1(3,:))- &
                               12d0*dphidx(7,k,:)
               dphidx(7,k,:) = 27d0*dphidx(7,k,:)
            end do

         end select  ! case (B%rule)

      end if  ! ( B%fillderiv )

      if ( B%comppsi ) then

!     --- Compute linear basis functions

         do k = 1, 2
            dpsidx(1,:,k) = e1(1,k)
            dpsidx(3,:,k) = e1(3,k)
            dpsidx(2,:,k) = -e1(1,k) - e1(3,k)
         end do

      end if  ! ( B%comppsi )

1000  if ( debug ) then

!     --- Debug information

         call prinrl2 ( B%phider(1,1,1), 7, m, 'dphidx' )
         call prinrl2 ( B%phider(1,1,2), 7, m, 'dphidy' )

         if ( B%fillderivn ) then
            call prinrl2 ( B%phidern(1,1,1), B%n, 7, 'dphidxn' )
            call prinrl2 ( B%phidern(1,1,2), B%n, 7, 'dphidyn' )
         end if  ! ( B%fillderivn )
         write(irefwr,*) 'End sepelp07jacstr'

      end if  ! ( debug )
      call erclos ( 'sepelp07jacstr' )

      end subroutine sepelp07jacstr
