      subroutine sepelp09
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute information of basis functions
!     Extended bi-linear quadrilateral (5 nodes)
!     Old routine: elp626
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduleconsts
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k, i
      double precision :: h
      double precision, pointer :: x(:), xg(:)

!     h              Help variable to store part of a weight
!     i              Counting variable
!     k              Loop variable
!     x              Contains x co-ordinates of nodes
!     xg             x-coordinates of Gauss points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPELP05JACOB  Compute Jacobian and derivatives of basis functions
!                    Bi-linear quadrilateral (4 nodes)
!     SEPELP09FIRST  compute some element independent quantities
!                    Bi-linear quadrilateral (4 nodes)
!     SEPELPGAUSS    Fill coordinates of Gauss points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!        To compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!         ij       j     i+1    i
!        e   = (-1)  (  x    - x    )   cyclic: i=1,2,3,4 ;   j=1,2
!                        j+1    j+1
!              1     2    1          4    1          3    2    1    4
!        x  = x  + (x  - x ) xi + (x  - x ) eta + (x  - x  + x  - x ) xi eta
!         i    i     i    i          i    i          i    i    i    i
!                                   i = 1,2
!                  1    12        42
!        So: x  = x  + e   xi - e   eta + A xi eta
!             1    1
!                  1    11        41
!            x  = x  - e   xi + e   eta + B xi eta
!             2    2
!            |   2    1                  4    1          |
!            |  x  - x  + A eta         x  - x  + A xi  |
!            |   1    1                  1    1          |
!        J  =|                                           |
!            |   2    1                  4    1          |
!            |  x  - x  + B eta         x  - x  + B xi  |
!            |   2    2                  2    2          |
!             1    2    3    4        12    32
!        A = x  - x  + x  - x    = - e   - e
!             1    1    1    1
!             3    4    2    1       11    31
!             2    2    2    2
!       |J| = J  + J  xi + J  eta
!              0    1        2
!              41   12    11   42
!        J  = e  * e   - e  * e
!         0
!                   12         11
!        J  = B  * e   - A  * e
!         1
!                   41         42
!        J  = A  * e   + B  * e
!         2
!        dphi / dx  = dphi / dxi dxi/dx  + dphi / deta deta/dx
!                 j                      j                      j
!                       41
!        dxi/dx  = [  e   + B xi ] / |J|
!               1
!                       42
!        dxi/dx  = [  e   - A xi ] / |J|
!               2
!                       11
!        deta/dx  = [  e   - B eta ] / |J|
!               1
!                       12
!        deta/dx  = [  e   + A eta ] / |J|
!               2
!     In case of 3D elements we use the following formulae:
!     x = sum phi (xi,eta) x    dx/dxi = sum dphi /dxi x   dx/deta=...
!     -    i     i          -i    -         i      i      -i
!             a                      a                         a
!     dphi /dx = dphi / dxi  dxi/dx   +  dphi / deta  deta/dx
!         i          i                         i
!                                                                        -1
!     | dxi/dx   deta/dx   dzeta/dx  |   | dx/dxi   dy/dxi   dz/dxi |
!     | dxi/dy   deta/dy   dzeta/dy  | = | dx/deta   dy/deta   dz/deta |
!     | dxi/dz   deta/dz   dzeta/dz  |   | dx/dzeta  dy/dzeta  dz/dzeta|
!     zeta is choosen perpendicular to the xi-eta direction, which implies
!     that we may choose:
!     dx / dzeta = dx / dxi times  dx / deta / || dx/dxi x dx/deta ||
!      -            -                -              -         -
!     where times or x is the outer product
!     The Jacobian of the dx/dxi matrix is given by:
!     J = dx/dzeta ( dy/dxi dz/deta - dy/deta dz/dxi )
!       + dy/dzeta ( dz/dxi dx/deta - dz/deta dx/dxi )
!       + dz/dzeta ( dx/dxi dy/deta - dx/deta dy/dxi )
!     and since dx/dzeta before normalization is given by
!         dx/dzeta = dy/dxi dz/deta - dy/deta dz/dxi
!         dy/dzeta = dz/dxi dx/deta - dz/deta dx/dxi
!         dz/dzeta = dx/dxi dy/deta - dx/deta dy/dxi
!     we see that the normalization factor is equal to
!        sqrt(     ( dy/dxi dz/deta - dy/deta dz/dxi )^2
!                + ( dz/dxi dx/deta - dz/deta dx/dxi )^2
!                + ( dx/dxi dy/deta - dx/deta dy/dxi )^2 ) = J
!      The derivatives dxi/dx and so on follow from the inverse,
!      which in this 3x3 case leads to:
!      dxi/dx = ( dy/deta  dz/dzeta - dy/dzeta dz/deta  ) / J
!      dxi/dy = ( dy/dzeta dz/dxi  - dy/dxi  dz/dzeta ) / J
!      dxi/dz = ( dy/dxi  dz/deta  - dy/deta  dz/dxi  ) / J
!      deta/dx = ( dz/deta  dx/dzeta - dz/dzeta dx/deta  ) / J
!      deta/dy = ( dz/dzeta dx/dxi  - dz/dxi  dx/dzeta ) / J
!      deta/dz = ( dz/dxi  dx/deta  - dz/deta  dx/dxi  ) / J
!    The basis functions at the standard unit square are defined by:
!    phi_1 = lambda_1(xi) lambda_1(eta) - 4 phi_c
!    phi_2 = lambda_2(xi) lambda_1(eta) - 4 phi_c
!    phi_3 = lambda_2(xi) lambda_2(eta) - 4 phi_c
!    phi_4 = lambda_1(xi) lambda_2(eta) - 4 phi_c
!    phi_5 = 16 phi_c
!    phi_c = lambda_1(xi) lambda_2(xi)lambda_1(eta) lambda_2(eta)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp09' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp09'
         write(irefwr,1) 'ifirst, n, m', E%first, B%n, B%m
         write(irefwr,1) 'inpelm', E%npelm
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      x => B%xyz(:,1)
      xg => B%xyzg(:,1)

      if ( E%first==0 ) call sepelp09first

!     --- Fill Gauss points

      call sepelpgauss

!     --- Compute Jacobian and derivatives of basis functions

      call sepelp05jacob

!     --- Fill weights

      if ( B%cart==1 ) then

!     --- B%cart = 1  Cartesian co-ordinates

         B%w = B%wc*abs(B%jac(1:B%m))

      else if ( B%cart>=2 ) then

!     --- B%cart = 2  Axi-symmetric co-ordinates

         B%w = abs(B%jac)*xg*B%wc
         if ( B%cart==2 ) B%w = B%w*pi2

      end if  ! ( B%cart==1 )

      if ( debug ) call prinrl ( B%w, B%m, 'weights' )

      if ( B%fillderiv ) then

!     --- fillderiv = true, fill first derivatives

         do i = 1, B%n

            do k = 1, B%m

               B%phider(i,k,1:2) = B%phiderc(i,k,1)*B%dxidx(k,1,1:2) +&
                                   B%phiderc(i,k,2)*B%dxidx(k,2,1:2)

            end do  ! k = 1, B%m

         end do  ! i = 1, B%n

      end if  ! ( B%fillderiv )

1000  call erclos ( 'sepelp09' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( B%phi, B%m, 5, 'phi' )
         call prinrl1 ( B%phider(1,1,1), B%m, 5, 'dphi/dx' )
         call prinrl1 ( B%phider(1,1,2), B%m, 5, 'dphi/dy' )
         call prinrl ( B%w, B%m, 'w' )
         write(irefwr,*) 'End sepelp09'

      end if  ! ( debug )

      end subroutine sepelp09
