      subroutine sepelp11
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute information of basis functions
!     Linear tetrahedron (4 nodes)
!     Old subroutine elp630
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: e1(4,3), djacob, h
      integer :: k, i

!     djacob         Jacobian of the element
!     e1             Work array of length 3 x 2, to store the factors
!                    e^ij See method
!     h              Help variable to store part of a weight
!     i              Counting variable
!     k              Loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPELP11FIRST  compute some element independent quantities
!                    Linear tetrahedron (4 nodes)
!     SEPELP11JACOB  Compute Jacobian and derivatives of basis functions
!                    Linear tetrahedrons
!     SEPELPGAUSS    Fill coordinates of Gauss points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!        To compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!             ij     i+1     i
!        delta   =  x    -  x       cyclic;  i = 1,2,3,4;  j =  1,2,3
!                    j       j
!         ij        i        i+1,j+1     i+2,j+2        i+1,j+2     i+1,j+2
!        e   =  (-1)  [ delta       delta        - delta       delta        ]
!                        11  11        12  12       13  13
!        djacob = - delta   e   - delta   e  - delta   e
!                          ij
!        d phi  / dx   =  e   /  djacob
!             i     j
!                                        |djacob|      1     2     3     4
!        Newton Cotes rule:  / f(x) dx = -------- [ f(x )+f(x )+f(x )+f(x ) ]
!                            e             24
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp11' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp11'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( E%first==0 ) call sepelp11first

!     --- Fill Gauss points

      call sepelpgauss

!     --- Compute Jacobian and derivatives of basis functions

      call sepelp11jacob ( e1, djacob )

!     --- Fill weights

      B%w = abs(djacob)*B%wc
      if ( debug ) call prinrl ( B%w, B%m, 'weights' )

      if ( B%fillderiv ) then

!     --- fillderiv = true, fill first derivatives

         do k = 1, 3
            do i = 1, 4
               B%phider(i,:,k) = e1(i,k)
            end do
         end do

      end if  ! ( B%fillderiv )

      if ( itype>=902 .and. itype<=903 ) then

!     --- Special case of Navier-Stokes element
!         w contains the weights
!         w2 the diagonal of the mass matrix

         do i = 1, B%n
            B%w2(i) = sum(B%w*B%phi(i,:)**2)
         end do  ! i = 1, B%n

      end if  ! ( itype>=902 .and. itype<=903 )

1000  call erclos ( 'sepelp11' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp11'

      end if  ! ( debug )

      end subroutine sepelp11
