      function sepelp12cur ( intern, imidp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if quadratic tetrahedron is curved
!     Old subroutine elp631cur
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     3d
!     quadratic
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical :: sepelp12cur
      integer, intent(in) :: intern(10), imidp(6,2)

!     imidp          i    Contains the i,j numbering of the mid side points in
!                         the sequence of intern
!                         So basis function
!                         phi(intern(4+i)) = 4 lambda(imidp(i,1))
!                                              lambda(imidp(i,2))
!     intern         i    Internal numbering of the nodes, vertices first
!     sepelp12cur    o    Indicates if element is curved (true) or not
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, i1, i2
      double precision :: xmid(3), eps, dist
      double precision, pointer :: x(:,:)
      logical :: curved

!     curved         indication if the element is curved (true) or not
!                    (false)
!     dist           Distance between mid point and centre of line
!     eps            Local accuracy
!     i              Counting variable
!     i1             Local sequence number of first end node
!     i2             Local sequence number of first end node
!     x              coordinates of nodes
!     xmid           Contains difference between mid point and centre of line
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3145   curvature of boundary too large
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp12cur' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp12cur'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      x => B%xyz

      curved = .false.
      eps = (maxval(abs(x))*sqreps)**2

      if ( debug ) write(irefwr,2) 'eps', eps

      do i = 1, 6

         i1 = intern(imidp(i,1))
         i2 = intern(imidp(i,2))
         xmid(:) = 0.5d0*(x(i1,:)+x(i2,:))-x(intern(i+4),:)

         if ( debug ) then
            write(irefwr,1) 'i, i1, i2, imid', i, i1, i2, intern(i+4)
            call prinrl ( xmid, 3, 'xmid' )
         end if  ! ( debug )

         dist = dot_product(xmid,xmid)
         if ( dist>eps ) then

!        --- mid point not in the middle of vertices
!            check if mid-side points nodes are close to the centre of the line

            curved = .true.

            if ( dist>1d11*eps ) then

!           --- error distance>1d11*eps

               call errint ( E%nodes(intern(i+4)), 1 )
               call errint ( E%nodes(i1), 2 )
               call errint ( E%nodes(i2), 3 )
               call errint ( E%ielem, 4 )
               call erreal ( dist, 1 )
               call erreal ( 1d11*eps, 2 )
               call errsub ( 3145, 4, 2, 0 )
               call prinin ( E%nodes, 10, 'node numbers')
               call prinrl ( xmid, 3, 'coordinates of diff vector' )

            end if  ! ( dist>1d3*eps )

            exit  ! jump out of loop

         end if  ! ( dot_product(xmid,xmid)>eps )

      end do  ! i = 1, 6

      sepelp12cur = curved

1000  call erclos ( 'sepelp12cur' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp12cur'

      end if  ! ( debug )

      end function sepelp12cur
