      subroutine sepelp12jacstr ( w, intern, imidp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute derivatives of basis functions for quadratic tetrahedron
!     Straight element
!     Old subroutine: elp631straight
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     3d
!     quadratic
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: intern(10), imidp(6,2)
      double precision :: w(B%m)

!     imidp          i    Contains the i,j numbering of the mid side points in
!                         the sequence of intern
!                         So basis function
!                         phi(intern(4+i)) = 4 lambda(imidp(i,1))
!                                              lambda(imidp(i,2))
!     intern         i    Internal numbering of the nodes, vertices first
!     w              o    Array of length m containing the values of the
!                         Jacobian in the vertices or for the Gauss integration
!                         rules work array to store some help variables
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, i1, i2, index1cp(4)
      double precision xvert(4,3), djacob, e1(4,3)
      double precision, pointer :: dphidx(:,:,:), lambda(:,:), x(:,:)

!     djacob         jacobian of element
!     dphidx         Contains values of dphi/dx in quadrature points
!     e1             Work array of length 4 x 3, to store the factors
!                     ij
!                    e.        See method
!     i              Counting variable
!     i1             i-index of mid side point ij
!     i2             j-index of mid side point ij
!     index1cp       Copy of index1 with respect to the vertices only
!     j              Counting variable
!     lambda         Array containing the values of lambda in the integration
!                    points  lambda(i,k) = lambda_i(xk)
!     x              coordinates of nodes
!     xvert          Contains the coordinates of the vertices
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELP11JACOB  Compute Jacobian and derivatives of basis functions
!                    Linear tetrahedrons
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp12jacstr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp12jacstr'
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dphidx => B%phider
      lambda => B%lam
      x => B%xyz

!     --- Copy coordinates of vertices into xvert

      xvert(1:4,:) = x(1:4,:)          ! save first 4 nodes
      x(1:4,:) = x(intern(1:4),:)
      index1cp(1:4) = E%nodes(1:4)     ! save first 4 nodes
      E%nodes(1:4) = E%nodes(intern(1:4))
      B%n = 4

      call sepelp11jacob ( e1, djacob )
      E%nodes(1:4) = index1cp(1:4)     ! reset
      B%n = 10
      x(1:4,:)= xvert(1:4,:)

      do i = 1, B%m
         B%lamder(1:4,i,1:3) = e1(1:4,1:3)
      end do  ! i = 1, m

      if ( ierror/=0 ) go to 1000

      w = djacob

      if ( debug ) write(irefwr,2) 'djacob', djacob

      if ( B%fillderiv ) then

!     --- Compute derivatives

         do j = 1, 3
            do i = 1, 4
               dphidx(intern(i),:,j) = e1(i,j)*(4d0*lambda(i,:)-1d0)
            end do
         end do

         do j = 1, 3
            do i = 1, 6
               i1 = imidp(i,1)
               i2 = imidp(i,2)
               dphidx(intern(i+4),:,j) = 4d0*(e1(i1,j)*lambda(i2,:)+ &
                                              e1(i2,j)*lambda(i1,:))
            end do
         end do

      end if  ! ( B%fillderiv )

      if ( B%fillderivn .and. B%rule>1 ) then

!     --- fillderivn = true, fill first derivatives in nodes

         dphidx => B%phidern
         lambda => B%lamn

         do j = 1, 3
            do i = 1, 4
               dphidx(intern(i),:,j) = e1(i,j)*(4d0*lambda(i,:)-1d0)
            end do
         end do

         do j = 1, 3
            do i = 1, 6
               i1 = imidp(i,1)
               i2 = imidp(i,2)
               dphidx(intern(i+4),:,j) = 4d0*(e1(i1,j)*lambda(i2,:)+ &
                                              e1(i2,j)*lambda(i1,:))
            end do
         end do

      else if ( B%fillderivn ) then

!     --- fillderivn = true, fill first derivatives in nodes

         B%phidern = B%phider

      end if  ! ( B%fillderiv )

1000  call erclos ( 'sepelp12jacstr' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp12jacstr'

      end if  ! ( debug )

      end subroutine sepelp12jacstr
