      subroutine sepelp13
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute information of basis functions
!     3d Tri-linear hexahedron (8 nodes)
!     Old subroutine elp632
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     hexahedron
!     linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: jac(B%m)
      integer :: i

!     jac            Array of length m containing the values of the
!                    Jacobian in the vertices or for the Gauss integration
!                    rules work array to store some help variables
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELP13FIRST  compute some element independent quantities
!                    3d Tri-linear hexahedron (8 nodes)
!     SEPELP13JACOB  Compute Jacobian and derivatives of basis functions
!                    3d Tri-linear hexahedron (8 nodes)
!     SEPELPGAUSS   Fill coordinates of Gauss points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!        To compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!         -1
!        J   = ( dx   / dalpha     dx   / dalpha    -
!         ij       i+1        i+1    i+2        i+2
!                dx   / dalpha     dx   / dalpha    ) / det(J)
!                  i+2        i+1    i+1        i+2
!               cyclic;  i = 1,2,3;  j =  1,2,3
!                   3
!        det (J) = sum   dx  / dalpha   ( dx   / dalpha     dx   / dalpha    -
!                  i=1     1         i      i+1        i+1    i+2        i+2
!                                         dx   / dalpha     dx   / dalpha    )
!                                           i+2        i+1    i+1        i+2
!                        k     3    -1   k                       k
!        d phi  / d x  (x ) = sum  J   (x )  d phi  / d alpha  (x )
!             i      l        j=1   lj            i          j
!        For the Newton-Cotes rule we have the following expressions
!        to compute dxdksi and dphidx:
!                        2k-1                    2k
!        dx  / dalpha  (x    ) = dx  / dalpha  (x  ) =
!          i         j             i         j
!         ialpha(2,k,j)      ialpha(1,k,j)
!        x               -  x
!         i                  i
!                                         ialpha(1,i,l)
!         d phi              / d alpha  (x             )  =  - 1
!              ialpha(1,i,l)          l
!                                         ialpha(2,i,l)
!         d phi              / d alpha  (x             )  =  - 1
!              ialpha(1,i,l)          l
!                                         ialpha(1,i,l)
!         d phi              / d alpha  (x             )  =  1
!              ialpha(2,i,l)          l
!                                         ialpha(2,i,l)
!         d phi              / d alpha  (x             )  =  1
!              ialpha(2,i,l)          l
!         Therefore the following algorithm may be used to compute dphidx:
!         - Set dphidx equal to zero
!         - Summation:  i = 1 (1) 4; j = 1 (1) 3; l = 1 (1) 3
!           d phi                d phi
!                ialpha(1,i,l)        ialpha(1,i,l)    -1      ialpha(1,i,l)
!           ------------------ = ------------------ - J      (x             )
!           d x                  d x                   jl
!              j                    j
!           d phi                d phi
!                ialpha(1,i,l)        ialpha(1,i,l)    -1      ialpha(2,i,l)
!           ------------------ = ------------------ - J      (x             )
!           d x                  d x                   jl
!              j                    j
!           d phi                d phi
!                ialpha(2,i,l)        ialpha(2,i,l)    -1      ialpha(1,i,l)
!           ------------------ = ------------------ + J      (x             )
!           d x                  d x                   jl
!              j                    j
!           d phi                d phi
!                ialpha(2,i,l)        ialpha(2,i,l)    -1      ialpha(2,i,l)
!           ------------------ = ------------------ + J      (x             )
!           d x                  d x                   jl
!              j                    j
!                                        1   8      k      k
!        Newton Cotes rule:  / f(x) dx = -  sum  f(x ) |J(x )|
!                            e           8  k=1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp13' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp13'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( E%first==0 ) call sepelp13first

!     --- Fill Gauss points

      call sepelpgauss

!     --- Compute Jacobian and derivatives of basis functions

      call sepelp13jacob ( jac )

!     --- Fill weights

      B%w = B%wc*abs(jac(1:B%m))

      if ( itype>=902 .and. itype<=903 ) then

!     --- Special case of Navier-Stokes element
!         w contains the weights
!         w2 the diagonal of the mass matrix

         do i = 1, B%n
            B%w2(i) = sum(B%w*B%phi(i,:)**2)
         end do  ! i = 1, B%n

      end if  ! ( itype>=902 .and. itype<=903 )

1000  call erclos ( 'sepelp13' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp13'

      end if  ! ( debug )

      end subroutine sepelp13
