      subroutine sepelp21first
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute some element independent quantities
!     linear wedge element
!     Old subroutine elp638first
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     wedge
!     linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, k, m
      double precision :: c, xi(8), eta(8), zeta(8)
      double precision :: lambda2d(3,B%m)
      double precision, target :: lambda1d(2,B%m)
      double precision, pointer :: alam1(:), weight(:)
      character (len=15), parameter :: text = 'Trilinear wedge'

!     alam1          same as lambda1d
!     c              1/6
!     eta            array that contains the values of eta in the
!                    integration points
!     i              Counting variable
!     k              Loop variable
!     lambda1d       Contains the 1-dimensional basis function in quadrature
!                    points
!     lambda2d       Contains the 2-dimensional basis function in quadrature
!                    points
!     m              Number of quadrature points
!     text           String variable containing some text
!     weight         Contains weights in reference element
!     xi             array that contains the values of xi in the
!                    integration points
!     zeta           array that contains the values of xeta in the
!                    integration points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
!     ERRSUB         Error messages
!     SEPELP21PHIXI  Compute dphi/dxi in quadrature points
!                    linear wedge element
!     SEPELPLAMBDA1D Fill the one-dimensional linear basis functions in case of
!                    a non-Newton Cotes rule
!     SEPELPLAMBDA2D Fill the two-dimensional linear basis functions in case of
!                    a non-Newton Cotes rule
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1497   Type of integration rule not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     To compute the derivatives of the basis functions, the jacobian
!     of the element and the weights, the following definitions are used:
!     The lower triangle is defined by the points 1,2,3, the upper
!     triangle by 4,5,6
!     The basis functions are mapped on a unit wedge in the (xi, eta ,zeta )
!     space with basis functions:
!
!     phi_1 = lambda_1(xi,eta) lambda_1(zeta)
!     phi_2 = lambda_2(xi,eta) lambda_1(zeta)
!     phi_3 = lambda_3(xi,eta) lambda_1(zeta)
!     phi_4 = lambda_1(xi,eta) lambda_2(zeta)
!     phi_5 = lambda_2(xi,eta) lambda_2(zeta)
!     phi_6 = lambda_3(xi,eta) lambda_2(zeta)
!
!     lambda_1(xi,eta) = 1 -xi - eta
!     lambda_2(xi,eta) = xi
!     lambda_3(xi,eta) = eta
!
!     lambda_1(zeta) = 1-zeta
!     lambda_2(zeta) = zeta
!
!     Hence the triangles are in the xi,eta planes and the upwards direction
!     is defined as zeta
!
!     The weights for the numerical integration rule are given as follows:
!     Let J be the Jacobian in the integration points then:
!
!     intrule = 1: (Newton Cotes rule)  w = 1/12 J(x_k)
!     intrule = 2: (One point Gauss rule)  w = 1/2 J(x_k)
!     intrule = 3: (Eight point Gauss rule)  w = 1/2 wgs2(k3) J(x_k)
!        (k3 is k modulo 3)
!
!     The integration points for the numerical integration rule are given as
!     follows:
!
!     intrule = 1: (Newton Cotes rule)  6 vertices
!     intrule = 2: (One point Gauss rule)  1 centroid
!     intrule = 3: (Eight point Gauss rule)
!       (xi_k,eta_k,zeta_k) = lambda2d(k3) wgs2(kz)
!       (kz = (k-1)/3+1)
!
!     dphi_dxi_j can be computed immediately from the standard basis functions:
!
!     dphi_i/dxi_j        xi              eta            zeta
!         1           -(1-zeta)        -(1-zeta)        -(1-xi-eta)
!         2             1-zeta             0               -xi
!         3                0             1-zeta            -eta
!         4              -zeta            zeta            1-xi-eta
!         5               zeta             0                xi
!         6                0              zeta              eta
!
!     dx/dxi_j = sum_k x_k dphi_k/dxi_j
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp21first' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp21first'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( B%cart>1 ) call errorelvar ( B%cart-1, 1769, text, 0 )

      weight => B%wc
      m = B%m

      xi = 0d0
      eta = 0d0
      zeta = 0d0

      select case (B%rule)

      case (1)

!     --- Newton Cotes rule

         if ( B%m/=6 ) call errorelvar2 ( B%m, B%rule, 1424, text, 0 )
         if ( ierror/=0 ) go to 1000

            B%phi = 0d0
            do i = 1, 6
               B%phi(i,i) = 1d0
            end do
            c = 1d0/12d0
            weight = c

            xi(2) = 1d0
            xi(5) = 1d0

            eta(3) = 1d0
            eta(6) = 1d0

            zeta(4:6) = 1d0

      case(2)

!     --- intrule = 2  one-point Gauss rule

         c = 1d0/6d0
         B%phi(:,1) = c
         weight(1) = 0.5d0

         xi(1) = 1d0/3d0
         eta(1) = xi(1)
         zeta(1) = 0.5d0

      case(3)

!     --- 8 point Gauss rule phi_ij = lambda2d * lambda1d

         call sepelplambda1d ( lambda1d, text, 3, 2, weight )
         call sepelplambda2d ( lambda2d, text, 3, 4, weight )
         alam1 => lambda1d(1:2,1)
         do i = 1, 3
            B%phi(i+3,1:4) = lambda2d(i,1:4) * alam1(1)
            B%phi(i,1:4) = lambda2d(i,1:4) * (1d0-alam1(1))
            B%phi(i+3,5:8) = lambda2d(i,1:4) * alam1(2)
            B%phi(i,5:8) = lambda2d(i,1:4) * (1d0-alam1(2))
         end do
         weight(1:4) = weight(1:4) * 0.5d0
         weight(5:8) = weight(1:4)

         xi(1:4) = lambda2d(2,1:4)
         xi(5:8) = xi(1:4)
         eta(1:4) = lambda2d(3,1:4)
         eta(5:8) = eta(1:4)
         zeta(1:4) = alam1(1)
         zeta(5:8) = alam1(2)

      case default

!     --- Quadrature not available

         call errint ( B%rule, 1 )
         call errsub ( 1497, 1, 0, 0)
         go to 1000

      end select  ! case (B%rule)

      B%phic = B%phi

!     --- Compute dphi/dxi, store in phiderc

      call sepelp21phixi ( B%m, B%phiderc, xi, eta, zeta )
      B%lamderc => B%phiderc
      B%lamder => B%phider

      if ( B%fillderivn ) then

!     --- Fill also phidercn

         if ( B%rule==1 ) then

!        --- Newton Cotes rule

            B%phidercn = B%phiderc

         else

!        --- Gauss rule, use formulas from Newton Cotes

            xi = 0d0
            eta = 0d0
            zeta = 0d0

            xi(2) = 1d0
            xi(5) = 1d0

            eta(3) = 1d0
            eta(6) = 1d0

            zeta(4:6) = 1d0

            call sepelp21phixi ( 6, B%phidercn, xi, eta, zeta )

         end if  ! ( B%rule==1 )

      end if  ! ( B%fillderivn )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp21first'

      end if  ! ( debug )
      call erclos ( 'sepelp21first' )

      end subroutine sepelp21first
