      subroutine sepelp30
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute information of basis functions
!     Third order 1d hermite element (2 nodes)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduleconsts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: h, h1, h2(B%m)
      double precision, pointer :: phixx(:,:), phix(:,:), xg(:), x(:)

!     h              Step size (x1-x1)
!     h1             1/h
!     h2             Help array to store some values
!     phix           First derivative of basis function phi
!     phixx          second derivative of basis function phi
!     x              x-coordinates of nodal points
!     xg             x-coordinates of Gauss points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELPGAUSS   Fill coordinates of Gauss points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1497   Type of integration rule not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The basis functions are defined as follows:
!     third degree polynomials
!     phi_1(x1) = 1, phi_1(x2) = 0, dphi_1/dx(x1) = 0, dphi_1/dx(x2) = 0
!     phi_3(x1) = 0, phi_3(x2) = 1, dphi_3/dx(x1) = 0, dphi_3/dx(x2) = 0
!     phi_2(x1) = 0, phi_2(x2) = 0, dphi_2/dx(x1) = 1, dphi_2/dx(x2) = 0
!     phi_4(x1) = 0, phi_4(x2) = 0, dphi_4/dx(x1) = 0, dphi_4/dx(x2) = 0
!
!     Let lambda be the linear basis functions and h = x2-x1
!
!     From these demands one easily derives:
!
!     phi_2 = h lamda_1**2 lambda_2
!     phi_4 = -h lamda_1 lambda_2**2
!     phi_1 = lambda_1+(phi_2+phi_4)/h
!     phi_2 = lambda_2-(phi_2+phi_4)/h
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp30' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp30'
         write(irefwr,*) 'E%first, B%fillderiv, B%fillderiv2 ', &
                          E%first, B%fillderiv, B%fillderiv2

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      phixx => B%phider2(:,:,1)
      phix => B%phider(:,:,1)
      x => B%xyz(:,1)
      xg => B%xyzg(:,1)

!     --- First call: compute some element independent quantities

      if ( E%first==0 ) call sepelp30first

!     --- Fill basis functions in quadrature points

      h = x(2)-x(1)
      h1 = 1d0/h
      B%phi(2,:) = B%phic(2,:)*h
      B%phi(4,:) = -B%phic(4,:)*h
      h2 = (B%phi(2,:)+B%phi(4,:))*h1
      B%phi(1,:) = B%lam(1,:)+h2
      B%phi(3,:) = B%lam(2,:)-h2

!     --- Fill weights

      B%w = h*B%wc

      if ( B%fillderiv ) then

!     --- fillderiv = true, fill first derivatives

         phix(2,:) = B%lam(1,:)*(B%lam(1,:)-2d0*B%lam(2,:))
         phix(4,:) = B%lam(2,:)*(B%lam(2,:)-2d0*B%lam(1,:))
         phix(1,:) = -h1+(phix(2,:)+phix(4,:))*h1
         phix(3,:) = -phix(1,:)

      end if  ! ( B%fillderiv )

      if ( B%fillderiv2 ) then

!     --- fillderiv2 = true, fill second derivatives

         phixx(2,:) = (2d0*B%lam(2,:)-4d0*B%lam(1,:))/h
         phixx(4,:) = (-2d0*B%lam(1,:)+4d0*B%lam(2,:))/h
         phixx(1,:) = (phixx(2,:)+phixx(4,:))*h1
         phixx(3,:) = -phixx(1,:)

      end if  ! ( B%fillderiv )

!     --- Fill Gauss points

      call sepelpgauss ( B%lam, E%npelm )

      if ( B%cart>=2 ) B%w = pi2*B%w*xg  ! axi-symmetric coordinates

1000  call erclos ( 'sepelp30' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp30'

      end if  ! ( debug )

      end subroutine sepelp30
