      subroutine sepelp3d2djac
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-02-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the jacobian delta and dx/dxi for a 2d element in 3d
!     Old subroutine elp3dto2ddelta
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     2d
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmoduleelm
      use sepmodulebasefn
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k
      double precision, pointer :: dxdxi(:,:,:), dphidxi(:,:,:), x(:,:)

!     dphidxi        Contains values of dphi/dxi in quadrature points
!     dxdxi          Contains the derivatives d x/ dxi
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     x              coordinates of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     230   Jacobian of element is zero or has changing sign
!    1540   Jacobian zero or changing sign
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp3d2djac' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp3d2djac'
         write(irefwr,1) 'ndim, kstep, m, n', ndim, kstep, m, n
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dxdxi => B%dxdxig
      x => B%xyz
      dphidxi => B%phiderc

      m = B%m
      n = B%n
      kstep = B%kstep

      do j = 1, E%ndim
         do i = kstep, m, kstep
            dxdxi(i,j,1) = dot_product ( x(1:n,j), dphidxi(1:n,i,1) )
            dxdxi(i,j,2) = dot_product ( x(1:n,j), dphidxi(1:n,i,2) )
         end do
      end do

!     --- fill jacobian in delta

      do i = 1, m
         dxdxi(i,1,3) = dxdxi(i,2,1)*dxdxi(i,3,2)- &
                        dxdxi(i,3,1)*dxdxi(i,2,2)
         dxdxi(i,2,3) = dxdxi(i,3,1)*dxdxi(i,1,2)- &
                        dxdxi(i,1,1)*dxdxi(i,3,2)
         dxdxi(i,3,3) = dxdxi(i,1,1)*dxdxi(i,2,2)- &
                        dxdxi(i,2,1)*dxdxi(i,1,2)
      end do

      do k = kstep, m, kstep
         B%Jac(k) = sqrt(dot_product (dxdxi(k,:,3),dxdxi(k,:,3)))
         if ( B%Jac(k)<=0d0 ) then
            if ( isubr==0 ) then
               call errint ( E%ielem, 1 )
               call errint ( E%itype, 2 )
               call errint ( E%ielgrp, 3 )
               call errsub ( 230, 3, 0, 0 )
            else
               call errint ( E%ielgrp, 1 )
               call errint ( E%ielem, 2 )
               call errchr ( 'surface', 1 )
               call errsub ( 1540, 2, 0, 1 )
            end if
            call prinin(E%nodes,n,'node numbers')
            call prinrl2(x,n,3,'co-ordinates')
            go to 1000
         end if
      end do

1000  call erclos ( 'sepelp3d2djac' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( B%Jac, m, 'B%Jac' )
         write(irefwr,*) 'End sepelp3d2djac'

      end if  ! ( debug )

      end
