      subroutine sepelpup4 ( xcent, h, unorm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute distance h for modified basis functions for upwind elements
!     Three dimensional elements (trilinear hexahedrons)
!     Old subroutine elpup4
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     upwind
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision h(B%m), unorm(B%m), xcent(6,3)

!     h              o    Contains the approximation of the distance
!                         h in the integration points
!     unorm          i    Length of velocity vector in the integration points
!     xcent          i    Contains the coordinates of the centers of the faces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i
      double precision :: ueta, uxi, uzeta, heta, hxi, hzeta
      double precision, pointer :: u(:,:)

!     heta           Distance in eta direction
!     hxi            Distance in ksi direction
!     hzeta          Distance in zeta direction
!     i              Counting variable
!     u              Velocities in quadrature points
!     ueta           Velocity component in eta direction
!     uxi            Velocity component in ksi direction
!     uzeta          Velocity component in zeta direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       h = max distance in element in the direction of u
!
!       h is computed as follows:
!       h = ( |u   | h    + |u   | h     + |u   | h     ) / ||u||
!               ksi   ksi     eta   eta      zeta  zeta       -
!       h ksi = || xf6 - xf5 ||      h eta = || xf4 - xf3 ||
!       h zeta = || xf2 - xf1 ||
!       xf1 = (x1 + x2 + x3 + x4 )/4     etc.
!              -    -    -    -
!         8 +-------------+ 7
!           |             |             f1 = under face    (1234)
!           |             |             f2 = upper face    (5678)
!     5 +------------+6   |             f3 = forward face  (1265)
!       |   |        |    |             f4 = backward face (4378)
!       |   +--------|----+             f5 = left face     (1485)
!       |   4        |    3             f6 = right face    (2376)
!       +------------+
!       1            2
!       u ksi = ux ( xf6-xf5) + uy ( yf6-yf5) + uz ( zf6-zf5)
!       u eta = ux ( xf4-xf3) + uy ( yf4-yf3) + uz ( zf4-zf3)
!       u zeta= ux ( xf2-xf1) + uy ( yf2-yf1) + uz ( zf2-zf1)
!       First tau and h are computed in the vertices and afterwards tau is
!       interpolated to the integration points
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelpup4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelpup4'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      u => E%veloc

!     --- Compute the distance h in the faces

      do i = 1, B%m
         if ( unorm(i).gt.epsmac ) then
            uxi  = u(i,1) * ( xcent(6,1) - xcent(5,1) ) + &
                   u(i,2) * ( xcent(6,2) - xcent(5,2) ) + &
                   u(i,3) * ( xcent(6,3) - xcent(5,3) )
            ueta = u(i,1) * ( xcent(4,1) - xcent(3,1) ) + &
                   u(i,2) * ( xcent(4,2) - xcent(3,2) ) + &
                   u(i,3) * ( xcent(4,3) - xcent(3,3) )
            uzeta= u(i,1) * ( xcent(2,1) - xcent(1,1) ) + &
                   u(i,2) * ( xcent(2,2) - xcent(1,2) ) + &
                   u(i,3) * ( xcent(2,3) - xcent(1,3) )
            h(i) = ( abs(uxi) + abs(ueta) + abs(uzeta) ) / unorm(i)
         else
            hxi  = sqrt ( (xcent(6,1)-xcent(5,1))**2 + &
                          (xcent(6,2)-xcent(5,2))**2 + &
                          (xcent(6,3)-xcent(5,3))**2 )
            heta = sqrt ( (xcent(4,1)-xcent(3,1))**2 + &
                          (xcent(4,2)-xcent(3,2))**2 + &
                          (xcent(4,3)-xcent(3,3))**2 )
            hzeta= sqrt ( (xcent(2,1)-xcent(1,1))**2 + &
                          (xcent(2,2)-xcent(1,2))**2 + &
                          (xcent(2,3)-xcent(1,3))**2 )
            h(i) = ( hxi + heta + hzeta ) / 3d0
         end if
      end do

1000  call erclos ( 'sepelpup4' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelpup4'

      end if  ! ( debug )

      end subroutine sepelpup4
