      subroutine sepelpupw03
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
! !   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute modified basis functions psi in case of upwind
!     Two dimensional elements (triangles)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     upwind
!     triangle
!     linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecomio
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, k, jstep, methodupw
      double precision :: xi, phimax, beta, cgnorm, tau1(B%m), u1(B%m), &
                          u2(B%m), tau(B%m), h(B%m), unorm(B%m), eps(B%m), two
      double precision, pointer :: uvel(:), vvel(:), phix(:,:), phiy(:,:)
      double precision, pointer :: lamx(:), lamy(:)
      double precision, pointer :: dcdx(:,:)

!     beta           Parameter beta in method
!     cgnorm         Norm of dphidx in integration point for specific value of i
!     dcdx           Gradient of old solution
!     eps            Array of length 9 to store the value of the tensor alpha
!                    in the direction of the velocity in the vertices
!     h              Array of length 9 to store the values of the diameter h
!                    in the vertices
!     i              Counting variable
!     jstep          Step in loop
!     k              Counting variable
!     lamx           x-derivative of lambda
!     lamy           y-derivative of lambda
!     methodupw      Type of upwind scheme
!     phimax         Max ( |u  + grad phi | )
!                     j      i           j
!     phix           Contains x-derivatives of basis functions in quadrature
!                    points
!     phiy           Contains y-derivatives of basis functions in quadrature
!                    points
!     tau            Array of length 9 to store the values of tau in the
!                    integration points
!     tau1           Array of length 9 to store the values of tau parallel in
!                    the integration points
!     two            Double precision number 2
!     u1             Parallel velocity
!     u2             Parallel velocity
!     unorm          array of length 9 to store the norm of the velocity in the
!                    vertex
!     uvel           u-component velocity
!     vvel           v-component velocity
!     xi             Factor xi, see method
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPELPUPW1     Compute modified basis functions for upwind elements
!                    One-dimensional elements
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelpupw03' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelpupw03'
         write(irefwr,1) 'jdiag, m, n, irule', B%jdiag, B%m, B%n, B%rule
         write(irefwr,1) 'methodupw', E%methodupw
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      uvel => E%veloc(:,1)
      vvel => E%veloc(:,2)
      phix => B%phider(:,:,1)
      phiy => B%phider(:,:,2)
      lamx => B%lamder(:,1,1)
      lamy => B%lamder(:,1,2)
      if ( E%filloldderiv ) dcdx => gradoldvec(1:B%m,1:E%ndim,1)

      if ( debug ) then
         call prinrl(uvel,B%m,'u')
         call prinrl(vvel,B%m,'v')
      end if  ! ( debug )

      methodupw = E%methodupw

!     --- Store component of alpha tensor in u-direction into eps

      call sepelpupw1 ( eps, unorm, uvel, vvel )

      if ( debug ) then
         write(irefwr,1) 'n, m, B%jdiag', B%n, B%m, B%jdiag
         call prinrl(unorm,B%m,'unorm')
         call prinrl ( eps,B%m,'eps' )
      end if

!     --- Compute the distance h in the vertices

      if ( B%jdiag==1 ) then

!     --- newton cotes integration rules

         if ( B%n==3 ) then

!        --- n = 3,  linear element

            do i = 1, 3
               phimax = max ( abs(uvel(i)*phix(1,1)+vvel(i)*phiy(1,1)), &
                        abs(uvel(i)*phix(2,1)+vvel(i)*phiy(2,1)), &
                        abs(uvel(i)*phix(3,1)+vvel(i)*phiy(3,1)) )
               if ( phimax>sqreps ) then
                  h(i) = unorm(i)/phimax
               else
                  h(i) = 0d0
               end if
            end do

         else

!        --- n = 6,  quadratic element

            do i = 1, 5, 2
               phimax = max(abs(uvel(i)*lamy(1)+vvel(i)*lamx(1)), &
                            abs(uvel(i)*lamy(2)+vvel(i)*lamx(2)), &
                            abs(uvel(i)*lamy(3)+vvel(i)*lamx(3)))
               if ( phimax>sqreps ) then
                  h(i) = unorm(i)/phimax
               else
                  h(i) = 0d0
               end if
            end do

         end if

      else

!     --- gauss integration rules

         if ( B%n==3 ) then

!        --- n = 3,  linear element

            do i = 1, B%m
               phimax = max ( abs(uvel(i)*phix(1,1)+vvel(i)*phiy(1,1)), &
                        abs(uvel(i)*phix(2,1)+vvel(i)*phiy(2,1)), &
                        abs(uvel(i)*phix(3,1)+vvel(i)*phiy(3,1)) )
               if ( phimax>epsmac ) then
                  h(i) = unorm(i)/phimax
               else
                  h(i) = 0d0
               end if
            end do

         else

            do i = 1, B%m
               phimax = max(abs(uvel(i)*lamx(1)+vvel(i)*lamy(1)), &
                            abs(uvel(i)*lamx(2)+vvel(i)*lamy(2)), &
                            abs(uvel(i)*lamx(3)+vvel(i)*lamy(3)) )
               if ( phimax>epsmac ) then
                  h(i) = unorm(i)/phimax
               else
                  h(i) = 0d0
               end if
            end do

         end if

      end if
      if ( debug ) then
         call prinrl ( h, B%m, 'h' )
      end if

!     --- Compute tau in the vertices
!         for quadratic element with newton cotes integration use linear
!         tau field

      jstep = 1
      if ( B%jdiag==1 .and. B%n==6 ) jstep = 2

      do i = 1, B%m, jstep

         if ( eps(i)>1d-15 .and. methodupw<=8 .and. methodupw/=6 ) then

!        --- eps(i)>1d-10,  hence second order term plays a role
!            Due to an error in the HP compiler we have to repace 2d0
!            by a variable

            two = 2d0
            beta = unorm(i)*h(i)/(two*eps(i))
            if ( beta<1d-15 ) then
               xi = 0d0
            else if ( methodupw==1 ) then

!           --- methodupw = 1   classical upwind scheme

               xi = 1d0

            else if ( methodupw==2 ) then

!           --- methodupw = 2   il'in scheme

               xi = 1d0/tanh(beta)-1d0/beta

            else if ( methodupw==3 .or. methodupw==7 .or. methodupw==8 ) &
               then

!           --- methodupw = 3, 7 or 8    doubly assymptotic approximation

               if ( abs(beta)>3d0 ) then
                  xi=sign(1d0,beta)
               else
                  xi = beta / 3d0
               end if

            else if ( methodupw==4 ) then

!           --- methodupw = 4    critical approximation

               if ( beta<-1d0 ) then
                  xi = -1d0 - 1d0 / beta
               else if ( beta>1d0 ) then
                  xi = 1d0 - 1d0 / beta
               else
                  xi = 0d0
               end if

            else if ( methodupw==5 ) then

!           --- methodupw = 5

               if ( unorm(i)<=1d-5*eps(i) ) then
                  xi = 0d0
               else
                  beta = beta*beta
                  xi = sqrt(beta/(9d0+beta))
               end if
            end if

         else if (methodupw==6 ) then

!        --- methodupw = 6

            if ( h(i)>epsmac ) then
               beta = (2d0*unorm(i)/h(i))**2 + (4d0*eps(i)/h(i)**2)**2
            else
               beta = 0d0
            end if
            if ( tstep>0d0 ) beta = beta + 4d0/(tstep*tstep)
            if ( beta>0d0 ) then
               tau(i) = 1d0/sqrt(beta)
            else
               tau(i) = 0d0
            end if  ! ( beta>0d0 )

         else
            xi = 1d0

         end if

         if ( methodupw<=8 .and. methodupw/=6 ) then
            if ( unorm(i)>epsmac ) then
               tau(i) = h(i)*xi/(2d0*unorm(i))
            else
               tau(i) = 0d0
            end if
         end if

      end do

!     --- interpolate tau in integration points for quadratic element

      if ( jstep==2 ) then

!     --- n = 6   Quadratic element with Newton Cotes integration

         tau(2) = 0.5d0 * (tau(1) + tau(3) )
         tau(4) = 0.5d0 * (tau(3) + tau(5) )
         tau(6) = 0.5d0 * (tau(5) + tau(1) )

      end if
      if ( debug ) call prinrl ( tau, B%m, 'tau' )

!     --- compute new basis functions, it has been assumed that the old
!         basis functions are already stored in array phi

      if ( B%derivconst ) then

!     --- jdercn = 0,  constant derivatives

         do k = 1, B%m
            do i = 1, B%n
               B%psi(i,k) = tau(k) * ( uvel(k)*phix(i,1)+ &
                                       vvel(k)*phiy(i,1) ) + B%phi(i,k)
            end do
         end do

      else

!     --- jdercn = 1,  variable derivatives

         do k = 1, B%m
            do i = 1, B%n
               B%psi(i,k) = tau(k) * ( uvel(k)*phix(i,k)+ &
                                       vvel(k)*phiy(i,k) ) + B%phi(i,k)
            end do
         end do
      end if
      if ( B%cart==1 ) then
         do k = 1, B%m
            if ( abs(unorm(k))>0d0 .and. &
                 abs(eps(k))*1d2>unorm(k) ) B%secondupw = .true.
         end do
      end if
      if ( methodupw==7 .or. methodupw==8 ) then

!     --- Discontinuity capturing
!         First compute u
!                        ||

         do k = 1, B%m
            cgnorm = dcdx(k,1)**2 + dcdx(k,2)**2
            if ( cgnorm<epsmac ) then
               u1(k) = 0d0
               u2(k) = 0d0
            else
               xi = (uvel(k)*dcdx(k,1) + vvel(k)*dcdx(k,2))/cgnorm
               u1(k) = xi*dcdx(k,1)
               u2(k) = xi*dcdx(k,2)
            end if
         end do

         call sepelpupw1 ( eps, unorm, u1, u2 )

!        --- Compute the distance h in the vertices

         if ( B%jdiag==1 ) then

!        --- newton cotes integration rules

            if ( B%n==3 ) then

!           --- n = 3,  linear element

               do i = 1, 3
                  phimax = max ( abs(u1(i)*phix(1,1)+u2(i)*phiy(1,1)), &
                                 abs(u1(i)*phix(2,1)+u2(i)*phiy(2,1)), &
                                 abs(u1(i)*phix(3,1)+u2(i)*phiy(3,1)) )
                  if ( phimax>epsmac ) then
                     h(i) = unorm(i)/phimax
                  else
                     h(i) = 0d0
                  end if
               end do

            else

!           --- n = 6,  quadratic element

               do i = 1, 5, 2
                  phimax = max(abs(u1(i)*lamx(1)+u2(i)*lamy(1)), &
                               abs(u1(i)*lamx(2)+u2(i)*lamy(2)), &
                               abs(u1(i)*lamx(3)+u2(i)*lamy(3)) )
                  if ( phimax>epsmac ) then
                     h(i) = unorm(i)/phimax
                  else
                     h(i) = 0d0
                  end if
               end do

            end if

         else

!        --- gauss integration rules

            if ( B%n==3 ) then

!           --- n = 3,  linear element

               do i = 1, B%m
                  phimax = max ( abs(u1(i)*phix(1,1)+u2(i)*phiy(1,1)), &
                                 abs(u1(i)*phix(2,1)+u2(i)*phiy(2,1)), &
                                 abs(u1(i)*phix(3,1)+u2(i)*phiy(3,1)) )
                  if ( phimax>epsmac ) then
                     h(i) = unorm(i)/phimax
                  else
                     h(i) = 0d0
                  end if
               end do
            else

               do i = 1, B%m
                  phimax = max(abs(u1(i)*lamx(1)+u2(i)*lamy(1)), &
                               abs(u1(i)*lamx(2)+u2(i)*lamy(2)), &
                               abs(u1(i)*lamx(3)+u2(i)*lamy(3)) )
                  if ( phimax>epsmac ) then
                     h(i) = unorm(i)/phimax
                  else
                     h(i) = 0d0
                  end if
               end do
            end if
         end if

!        --- Compute tau in the vertices
!            for quadratic element with newton cotes integration use linear
!            tau field

         jstep = 1
         if ( B%jdiag==1 .and. B%n==6 ) jstep = 2

         do i = 1, B%m, jstep

            if ( eps(i)>1d-15 .and. methodupw<=8 .and. methodupw/=6 ) then

!           --- eps(i)>1d-15,  hence second order term plays a role

               beta = unorm(i)*h(i)/(2d0*eps(i))
               if ( beta<1d-15 ) then
                  xi = 0d0
               else

!              --- methodupw = 3, 7 or 8    doubly assymptotic approximation

                  if ( abs(beta)>3d0 ) then
                     xi=sign(1d0,beta)
                  else
                     xi = beta / 3d0
                  end if
               end if
               if ( unorm(i)>epsmac ) then
                  tau1(i) = h(i)*xi/(2d0*unorm(i))
               else
                  tau1(i) = 0d0
               end if

            end if  ! ( eps(i)>1d-15 )

         end do  ! i = 1, B%m, jstep

!        --- interpolate tau in integration points for quadratic element

         if ( jstep==2 ) then

!        --- n = 6   Quadratic element with Newton Cotes integration

            tau(2) = 0.5d0 * (tau(1) + tau(3) )
            tau(4) = 0.5d0 * (tau(3) + tau(5) )
            tau(6) = 0.5d0 * (tau(5) + tau(1) )

         end if
         if ( debug ) call prinrl ( tau, B%m, 'tau' )

         if ( methodupw==8 ) then
            do k = 1, B%m
               tau1(k) = max(0d0,tau1(k)-tau(k))
            end do
         end if
         do i = 1, B%n
            do k = 1, B%m
               B%psi(i,k) = tau1(k) * ( u1(k)*phix(i,k)+ &
                                      u2(k)*phiy(i,k) ) + B%psi(i,k)
            end do
         end do
      end if

      do i = 1, B%m
         B%lamder(:,i,:) = B%lamder(:,1,:)
      end do  ! i = 1, B%m

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( B%psi, B%m, B%n, 'psiupw')
         write(irefwr,*) 'End sepelpupw03'

      end if  ! ( debug )
      call erclos ( 'sepelpupw03' )

      end subroutine sepelpupw03
