      subroutine sepelpupw11tim
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute modified basis functions psi in case of upwind of type 12
!     Three dimensional elements (tetrahedrons)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     upwind
!     tetrahedron
!     linear
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecomio
      use sepmoduleelm
      use sepmodulebasefn
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k, m, n
      double precision :: normu(B%m), beta(B%m), delta
      double precision, pointer :: u(:,:), psi(:,:), phi(:,:), w(:)
      double precision, pointer :: dphidx(:,:,:)

!     beta           array of length m containing the values of the
!                    multiplication factor
!     delta          jacobian of element
!     dphidx         derivatives of the basis functions
!     k              Counting variable
!     m              Number of quadrature points
!     n              Number of basis functions
!     normu          norm of u-vector
!     phi            Contains values of the basis functions in the
!                    integration points
!     psi            Contains values of the modified basis functions in the
!                    integration points
!     u              velocity vector
!     w              weights for the integration
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
!    3082   This type of upwind has not been implemented for this shape
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The upwind basis function is given by
!
!     psi_i = phi_i + beta u . grad phi_i
!
!     with beta = 1 / 2 sqrt(Delta t^-2 + h^-2 ||u||^2)
!     instead of h^2 we take the Jacobian of the element |delta|^2/3
!     which is equal to 6 times the volume of the tetrahedron
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelpupw11tim' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelpupw11tim'
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      psi => B%psi
      phi => B%phi
      u => E%veloc
      w => B%w
      dphidx => B%phider

      m = B%m
      n = B%n

      if ( B%type_basisfunc/=11 ) then

!     --- Error: shape # 11

         call errint ( 12, 1 )
         call errint ( B%type_basisfunc, 2 )
         call errsub ( 3082, 2, 0, 0 )
         go to 1000

      end if  ! ( jelp/=11 )

!     --- Compute delta

      delta = 6d0*sum(w)
      delta = delta**(2d0/3d0)  ! to make it h^2
      if ( debug ) write(irefwr,2) 'delta', delta

!     --- Compute beta
!         beta = 1 / 2 sqrt(Delta t^-2 + h^-2 ||u||^2)

      normu = u(:,1)**2+u(:,2)**2+u(:,3)**2
      beta = 1d0 / (2d0*sqrt(1d0/tstep**2+normu/delta))
      if ( debug ) write(irefwr,2) 'normu, beta', normu, beta

!     --- Compute psi
!         psi_i = phi_i + beta u . grad phi_i

      do k = 1, m
         psi(1:n,k) = phi(1:n,k) + beta(k) * &
                      ( u(k,1)*dphidx(1:n,k,1)+u(k,2)*dphidx(1:n,k,2)+ &
                        u(k,3)*dphidx(1:n,k,3) )
      end do  ! k = 1, m

1000  call erclos ( 'sepelpupw11tim' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelpupw11tim'

      end if  ! ( debug )

      end subroutine sepelpupw11tim
