      subroutine sepelvarcofs1 ( iseqlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array ich in common block celiar for coefficients
!     that depend on a prior solution
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     element_matrix
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmoduleelm
      use sepmodulebuild
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iseqlc

!     iseqlc         i    Sequence number of coefficient
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: idegfd, itypvc, ndegfd, k, inunkp, klast, ivec, jnunkp
      logical :: variab, allpnt

!     allpnt         Indication if all points contain the degree of freedom
!                    (true) or not (false)
!     idegfd         Degree of freedom in VECTOR(IVEC) that must be used
!     inunkp         Number of degrees of freedom in first point
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector in array VECOLD.
!                    Hence array VECOLD(IVEC) is considered
!     jnunkp         Number of degrees of freedom in second point
!     k              Loop variable
!     klast          Last relative node index for which a variable is present
!     ndegfd         Number of degrees of freedom in VECTOR(IVEC)
!     variab         Indication if the number of degrees of freedom per point is
!                    constant (false) or not (true)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inivec

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     INIVEC         Return with vector number corresponding to ivecin
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      787  Coefficient refers to prior solution vector that
!           does not contain the required degree of freedom in any point
!      788  Coefficient refers to prior solution vector but
!           the degree of freedom is not present in the first point
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelvarcofs1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelvarcofs1'
         call ertrace
         write(irefwr,1) 'iseqlc', iseqlc
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ipos = E%ivecnums(iseqlc)
      if ( debug ) write(irefwr,1) 'ipos', ipos

      ivec = inivec(E%iuser(ipos))
      if ( debug ) write(irefwr,1) 'ivec', ivec

      idegfd = max(1,E%mst(iseqlc))
      iseqtype = oldsols(ivec)%iseqtype
      if ( iseqtype==0 ) go to 1000

      itypvc = oldsols(ivec)%itypvc


      if ( debug ) write(irefwr,1) 'itypvc, iseqtype', itypvc, iseqtype

      index4n => indoldsols(iseqtype)%index4(1:E%npelm+1)

      n = min(B%n, E%npelm)

      if ( debug ) write(irefwr,1) 'itypvc, iseqtype, n', itypvc, iseqtype, n
      if ( itypvc==116 ) then

!     --- itypvc = 116, vector of special structure defined per element

         ndegfd = oldsols(ivec)%ndegfd
         if ( ndegfd==n .and. E%mst(iseqlc)==0 ) then
            E%ich(iseqlc) = 1
         else if ( ndegfd==1 .or. E%mst(iseqlc)/=0 ) then
            E%ich(iseqlc) = 0
         else
            E%ich(iseqlc) = 100+ndegfd
         end if

      else if ( itypvc==126 ) then

!     --- itypvc = 126, vector of special structure defined per element
!         per node

         ndegfd = oldsols(ivec)%ndegfd
         E%ich(iseqlc) = 1000+ndegfd

      else if ( itypvc==129 ) then

!     --- itypvc = 129, vector of special structure defined per element
!         per integration point

         ndegfd = oldsols(ivec)%ndegfd
         E%ich(iseqlc) = 2000+ndegfd

      else

!     --- itypvc = 110 or 115
!         Check number of degrees of freedom per point

         variab = .false.
         inunkp = index4n(1)
         if ( debug ) write(irefwr,1) 'inunkp', inunkp

         do k = 2, n
            if ( index4n(k) - index4n(k-1)/=inunkp ) then

!           --- variable degrees of freedom per point

               variab = .true.
               exit

            end if
         end do

         if ( debug ) write(irefwr,*) 'variab ', variab

         if ( .not. variab ) then

!        --- constant number of degrees of freedom

            if ( idegfd>inunkp ) then
               call errint ( iseqlc, 1 )
               call errint ( idegfd, 2 )
               call errint ( inunkp, 3 )
               call errint ( E%ielgrp, 4 )
               call errsub ( 787, 4, 0, 0 )
            end if
            if ( inunkp==1 ) then
               E%ich(iseqlc) = 1
            else
               E%ich(iseqlc) = 2
            end if

         else

!        --- Variable number of degrees of freedom per point

            if ( idegfd>inunkp ) then
               call errint ( iseqlc, 1 )
               call errint ( idegfd, 2 )
               call errint ( inunkp, 3 )
               call errint ( E%ielgrp, 4 )
               call errsub ( 788, 4, 0, 0 )
            end if
            allpnt = .true.
            klast = n
            do k = 2, n
               if ( index4n(k) - index4n(k-1) < idegfd ) then

!              --- Degree of freedom idegfd not in this point

                  allpnt = .false.
                  klast = k-1
                  go to 130

               end if
            end do

130         if ( allpnt ) then

!           --- allpnt = true,  degree of freedom in all nodes

               E%ich(iseqlc) = 5

            else

!           --- allpnt = false,  degree of freedom not in all nodes

               jnunkp = index4n(2) - index4n(1)
               if ( jnunkp==0 ) then

!              --- Degree of freedom in vertices, but not in
!                  the mid points

                  if ( inunkp==1 ) then
                     E%ich(iseqlc) = 3
                  else
                     E%ich(iseqlc) = 4
                  end if

               else if ( klast==n-1 .and. &
                         (E%ishape==7 .or. E%ishape==9) ) then

!              --- Special situation, variable not in centroid

                  E%ich(iseqlc) = 7

               else if ( klast==10 .and. &
                         (E%ishape==15 .or. E%ishape==16) ) then

!              --- Special situation, variable not in centroids

                  E%ich(iseqlc) = 8

               else

!              --- Variable situation

                  E%ich(iseqlc) = 6

               end if

            end if

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iseqlc, ich(iseqlc)', iseqlc, E%ich(iseqlc)
         write(irefwr,*) 'End sepelvarcofs1'

      end if  ! ( debug )
      call erclos ( 'sepelvarcofs1' )

      end subroutine sepelvarcofs1
