      subroutine el0900flcoeffs ( ncoef, modelv, mcont, mcoefconv, &
                                  coeffintpnts, jtime, jcheld, cn, &
                                  clamb, kchois, iuser, modelrhsd, &
                                  mconv, imesh, isub, isup, &
                                  mdiv, user, ishift, coeffnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.2  date 24-07-2015 Correction coeffnodes(1,7)
!        version  3.1  date 06-03-2013 Extra debug
!
!   copyright (c) 2006-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Initialize coefficients for elements of type 900-902 and 916-918
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncoef, modelv, mcont, mcoefconv, jtime, jcheld, kchois, &
              iuser(*), modelrhsd, mconv, imesh, &
              isub, isup, mdiv, ishift
      double precision coeffintpnts(m,*), cn, clamb, user(*), &
                       coeffnodes(n,*)

!     clamb          o    Parameter lambda with respect to viscosity model
!     cn             o    Power in power law model
!     coeffintpnts  i/o   Real array in which the values of the
!                         coefficients in the integration points are stored
!                         See subroutine el0900
!     coeffnodes    i/o   Real array in which the values of the
!                         coefficients in the nodal points are stored
!     imesh          o    Sequence number of mesh velocity in input vector
!                         If 0 no mesh velocity is present
!     ishift        i/o   Shift in coefficient sequence number
!     isub           o    Integer help parameter for do-loops to indicate the
!                         smallest coefficient number that is dependent on space
!     isup           o    Integer help parameter for do-loops to indicate the
!                         largest coefficient number that is dependent on space
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     jcheld         i    Value of icheld corrected for different storage
!                         schemes
!     jtime          o    Integer parameter indicating if the mass matrix for
!                         the time-dependent equation must be computed (>0)
!                         or not (=0)
!                         Possible values:
!                           0:  no mass matrix
!                           1:  diagonal mass matrix with constant coefficient
!                           2:  diagonal mass matrix with variable coefficient
!                          11:  full mass matrix with constant coefficient
!                          12:  full mass matrix with variable coefficient
!                         101:  refers to the special possibility of the thetha
!                               method directly applied
!                         102:  refers to the special possibility of the thetha
!                               method directly applied with variable rho
!     kchois         i    Indication what type of parameters must be computed
!                         Possible values:
!                         0:  Standard method, the calling subroutine is ELM900
!                             It is supposed that an element matrix must be
!                             computed
!                         1:  Subroutine ELD900 calls this subroutine, only
!                             derivatives must be computed
!                         2:  Subroutine ELI900 calls this subroutine, only
!                             integrals must be computed
!     mcoefconv      o    Defines how the convetive term is treated:
!                         0: standard case
!                         1: special case: the convective term is defined as
!                               c_conv_1 u du/dx + c_conv_2 v du/dy
!                               c_conv_3 u dv/dx + c_conv_4 v dv/dy
!     mcont          o    Defines the type of continuity equation
!                         Possible values:
!                         0: incompressible flow
!                         1: compressible flow: div (rho u ) = 0
!                         2: Goertler equations
!                         4: the continuity equation contains a pressure term
!     mconv          o    Defines the treatment of the convective terms
!                         Possible values:
!                         0: convective terms are skipped (Stokes flow)
!                         1: convective terms are linearized by Picard iteration
!                         2: convective terms are linearized by Newton iteration
!                         3: convective terms are linearized by the incorrect
!                            Picard iteration
!                         4: convective terms are linearized by successive
!                            substitution
!     mdiv           o    Defines if divergence right-hand side is used (1)
!                         or not (0)
!     modelrhsd      o    Defines how the stress tensor is treated in the rhs
!                         Possible values:
!                         0:    if there is a given stress tensor the reference
!                               to this tensor is stored in position 19
!                         1:    It is supposed that there are 6 given stress
!                               tensor components stored in positions 19-24
!     modelv         o    Defines the type of viscosity model
!                         Possible values:
!                           1:  newtonian liquid
!                           2:  power law liquid
!                           3:  carreau liquid
!                           4:  plastico viscous liquid
!                           5:    special case: the viscous term is defined as
!                                 c_visc_1 d^2 u /dx^2 + c_visc_2 d^2 u /dy^2
!                                 c_visc_3 d^2 v /dx^2 + c_visc_4 d^2 v /dy^2
!                          10:  simplified newtonian liquid, &
!                               with constant viscosity
!                         100:  user provided model depending on grad u
!                         101:  user provided model depending on the second
!                               invariant
!                         102:  user provided model depending on the second
!                               invariant
!                               the co-ordinates and the velocity
!                         103:  user provided model depending on the second
!                               invariant
!                               the co-ordinates, the velocity, and old
!                               solutions
!     ncoef          i    Number of coefficients in the equation
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, inunkp, isubcf
      double precision etha
      logical funchk

!     etha            Viscosity parameter etha
!     funchk         Indicates whether there are coefficients that must be
!                    evaluated by funcc3 (true) or not (false)
!     i              Counting variable
!     inunkp         Number of degrees of freedom in point
!     isubcf         Under bound for coefficients check
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     ELUSR4         Extracts information concerning the coefficients from IUSER
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREPSCAL     Corrects integer value in case of scalars
!     PRININ         print 1d integer array
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     238   Omega unequal to zero, whereas axi symmetric elements are used
!    1014   Sequence number of mesh velocity out of range
!    1015   Sequence number of mesh velocity identical to velocity
!    1016   Structure of mesh velocity vector and velocity vector different
!    1303   Only constants are allowed for this coefficient
!    1306   Coefficient has wrong value
!    1430   The viscosity parameter etha is non-positive
!    2024   Option has not been implemented for 3d
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0900flcoeffs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0900flcoeffs'
         write(irefwr,1) 'ncoef, ipos, ishift', ncoef, ipos, ishift
         call prinin ( ind, ncoef, 'ind' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( ncoef>1 .and. (modelv==5 .or. mcont==3 .or. &
           mcoefconv>0) ) ncoef = 32

!     --- Clear array coeffintpnts

      coeffintpnts(1:m,1:ncoef) = 0d0

      do i = 1, min(6,ncoef)
         call elusr4 ( i, coeffintpnts(1,i), iuser, user, ishift+i, &
                       coeffnodes(1,i) )
      end do
      ishift = ishift+ncoef

      if ( debug ) write(irefwr,1) 'ipos/2', ipos

      if ( ncoef==32 ) then

!     --- Special case of non-dimensional parameters
!         skip 3 positions in iuser

         ipos = ipos+7
         do i = 11, 32
            call elusr4 ( i, coeffintpnts(1,i), iuser, user, i+9, &
                          coeffnodes(1,i) )
         end do

      end if

!     --- Test coefficients

      if ( ind(1)==0 ) then

!     --- rho has not been given a value, set rho equal to 1

         coeffintpnts(1:m,1) = 1d0
         coeffnodes(1:n,1) = 1d0

      else if ( ind(1)>0 .and. icheld==0 ) then

!     --- rho not constant

         if ( jtime>0 ) jtime = jtime + 1

      end if

      if ( jcart==2 .and. ind(2)/=0 ) then

!     --- omega#0, whereas axi-symmetric elements are used

         call errint ( ielgrp, 1 )
         call errint ( ielem, 2 )
         call errsub ( 238, 2, 0, 0 )
         ind(2) = 0

      end if

      if ( ind(6)<=0 .and. (icheld==0 .or. jcheld==6 .or. &
           jcheld==11) .and. modelv<100 ) &
         then

!     --- etha is constant, test whether etha is positive

         etha = coeffintpnts(1,6)
         if ( etha<=0d0 ) then

!        --- etha<=0

            call erreal ( etha, 1 )
            call errint ( itype, 1 )
            call errint ( 12, 2 )
            call errint ( ielgrp, 3 )
            call errint ( ielem, 4 )
            call errsub ( 1430, 4, 1, 0 )

         end if

      end if

      if ( debug ) write(irefwr,1) 'modelv', modelv

!     --- Read extra parameters depending on modelv

      if ( ncoef>=6 .and. ncoef<=20 .and. modelv>1 .and. &
           modelv<5 ) then

!     --- power law type fluid

         if ( ncoef<10 ) &
            call elusr4 ( 7, coeffintpnts(1,7), iuser, user, ishift, &
                          coeffnodes(1,7) )
         if ( ind(7)>0 ) then

!        --- cn is given as function instead of coefficient

            call errint ( 13, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errint ( ielem, 4 )
            call errchr ( 'cn', 1 )
            call errsub ( 1303, 4, 0, 1 )

         end if
         cn = coeffintpnts(1,7)
         if ( cn<0d0 .or. cn>5.0d0 ) then

!        --- cn out of range

            call errint ( 13, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errint ( ielem, 4 )
            call erreal ( cn, 1 )
            call errchr ( 'cn', 1 )
            call errsub ( 1306, 4, 1, 1 )

         end if

      else if ( kchois/=2 .and. ncoef<=20 ) then

         ipos = ipos+1

      end if

      if ( ncoef>=6 .and. ncoef<=20 .and. modelv>2 .and. &
           modelv<5 ) then

!     --- power law type fluid with two parameters

         ishift = ishift+1
         call elusr4 ( 7, coeffintpnts(1,7), iuser, user, ishift, &
                       coeffnodes(1,7) )
         if ( ind(7)>0 ) then

!        --- clamb is given as function instead of coefficient

            call errint ( 14, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errint ( ielem, 4 )
            if ( modelv==3 ) then
               call errchr ( 'lambda', 1 )
            else
               call errchr ( 's', 1 )
            end if
            call errsub ( 1303, 4, 0, 1 )

         end if
         clamb = coeffintpnts(1,7)
         if ( clamb<0d0 ) then

!        --- clamb out of range

            call errint ( 13, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errint ( ielem, 4 )
            call erreal ( clamb, 1 )
            if ( modelv==3 ) then
               call errchr ( 'lambda', 1 )
            else
               call errchr ( 's', 1 )
            end if
            call errsub ( 1306, 4, 1, 1 )

         end if

      else if ( kchois/=2 .and. ncoef<=20 ) then

         ipos = ipos+1

      end if
      if ( debug ) &
         write(irefwr,1) 'ipos, iuser(ipos)', ipos, iuser(ipos)
      if ( itype/=903 .and. ncoef>=6 .and. ncoef<=20 ) then
        imesh = inirepscal ( iuser(ipos) )
      else
        imesh = 0
      end if

      if ( debug ) write(irefwr,1) 'imesh', imesh

      if ( kchois/=2 .and. ncoef<=20 ) ipos = ipos+1

      if ( imesh<0 .or. imesh>numold ) then

!     --- imesh out of range

         call errint ( imesh, 1 )
         call errint ( numold, 2 )
         call errsub ( 1014, 2, 0, 0 )

      end if
      if ( imesh==iseqin .and. imesh>0 ) then

!     --- imesh has same value as solution vector (not allowed)

         call errint ( imesh, 1 )
         call errsub ( 1015, 1, 0, 0 )

      end if
      if ( imesh>0 ) then

!     --- Check if imesh  has at least ndim degrees of freedom per point

         iseqtype = oldsols(imesh)%iseqtype
         index4n => indoldsols(iseqtype)%index4
         j = 0
         do i = 1, inpelm
            inunkp = index4n(i)-j
            if ( inunkp<ndim ) then
               call errint ( imesh, 1 )
               call errint ( inunkp, 2 )
               call errint ( i, 3 )
               call errint ( ndim, 4 )
               call errchr ( namesol(imesh), 1 )
               call errsub ( 1016, 4, 0, 1 )
            end if
            j = index4n(i)
         end do

      end if
      if ( mcont==2 .or. mcont==4 ) then

!     --- Goertler equations or continuity equation with pressure term

         ishift = ishift+4
         do i = 7, 8
            call elusr4 ( i, coeffintpnts(1,i), iuser, user, &
                          ishift-7+i, coeffnodes(1,i) )
         end do
         ncoef = 8

      else if ( kchois/=2 .and. ncoef<=20 ) then

         ipos = ipos+2

      end if

!     --- Right-hand side for continuity equation

      if ( ncoef>=6 .and. ncoef<=20 ) then
         ishift = ishift+2
         i = 9
         call elusr4 ( i, coeffintpnts(1,i), iuser, user, ishift, &
                       coeffnodes(1,i) )
      end if

      if ( ind(9)/=0 ) then

!     --- The right-hand side for the continuity equation is not equal to zero

         mdiv = 1
         ncoef = 9

      else

         mdiv = 0

      end if

!     --- Given stress tensor for the right-hand side

      if ( ncoef>=6 .and. ncoef<=20 ) then

         if ( modelrhsd>=1 ) then
            isup = 15
         else
            isup = 10
         end if
         do i = 10, isup

            call elusr4 ( i, coeffintpnts(1,i), iuser, user, 9+i, &
                          coeffnodes(1,i) )

            if ( ind(i)/=0 ) then

!           --- Given stress tensor for the right-hand side

               ncoef = i
               if ( ndim==3 .or. jcart>2 ) then

!              --- Not yet implemented

                  call errsub ( 2024, 0, 0, 0 )

               end if

            end if

         end do  ! i = 10, isup

      end if

      if ( ncoef>0 ) then

!     --- Test array lengths, number of nodes and number of degrees of freedom

         call prinlg ( 'iuser', ipos-1, user, iuser )
         call prinlg ( 'user', -nuser, user, iuser )

      end if

!     --- Check which parameters depend on the space co-ordinates
!         Compute isub and isup

      isubcf = 1
      if ( icheld>0 .and. .not. ( ndim==2 .and. &
           jcheld==7 ) ) &
          isubcf = 6

      do i = isubcf, ncoef
         isub = i

!        --- If ind(i)>0 then paramater i corresponds to a function

         if ( ind(i)>0 ) goto 170

      end do

      isub = ncoef + 1

170   isup = ncoef
      do i = ncoef, isub , -1
         isup = i
         if ( ind(i)>0 ) goto 190
      end do

!     --- Check for double parameters (functions) to avoid calls for
!         function subroutines
!         ist is used to store the references for the double parameters
!         Set ind(lengthmst)

190   do i = isub, isup
         ist(i) = 0
      end do

      if ( icheld>0 ) then

!     --- Compute derivative quantities

         ind(lengthmst) = 1
         if ( ndim==3 .and. jcheld==7 ) then
            jderiv = 0
         else
            jderiv = 1
         end if

      else if ( icheld==-1 .or. icheld==-2 .or. icheld==-6 ) then

!     --- icheld = -1 or -2, do not compute derivatives

         ideriv = 0
         jderiv = 0

      else if ( icheld<-2 ) then

!     --- icheld<-2, derivatives should be computed

         jderiv = -icheld+3

      else if ( mconv>0 .or. modelv>1 .and. modelv/=10 .or. &
                jtime>0 ) then

!     --- the convective terms must be taken into account and/or the
!         viscosity model depends on the gradient of the velocity and/or
!         the thetha method must be applied

         ind(lengthmst) = 1
         jderiv = 1

      else

!     --- Stokes flow with Newtonian viscosity model

         ind(lengthmst) = 0

      end if

      funchk = .false.
      do i = isub, isup
         if ( ind(i)>0 .and. ind(i)<2000 ) then

!        --- only the function calls are checked

            do j = i+1,isup
               if ( ind(j)==ind(i) ) then
                  ist(j) = i
                  ind(j) = 0
               end if
             end do
            igausp = 1
         end if
         if ( ind(i)>1000 .and. ind(i)<=2000 ) ind(lengthmst) = 1
         if ( ind(i)>10000 ) funchk = .true.
         if ( ind(i)>1000 .and. ind(i)<=2000 .or. ind(i)>10000 ) &
              igausp=1
      end do

      if ( icheld<-1 .or. icheld>0 ) ind(lengthmst) = 1
      if ( funchk .or. modelv==103 .or. imesh>0 ) &
          ind(lengthmst) = ind(lengthmst)+2

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el0900flcoeffs'

      end if  ! ( debug )
      call erclos ( 'el0900flcoeffs' )

      end
