      subroutine el0900init ( icheld, ncoef, compress, itype, jcheld, &
                              ielgrp, ix, ndim, jdegfd, ncomp, jderiv, &
                              kchois, icount, inpelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 07-02-2013 Correction error message
!        version  1.0  date 08-06-2005
!
!   copyright (c) 2005-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make some initialization for subroutines el0900/3/5
!     depending on icheld
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icheld, ncoef, itype, jcheld, ielgrp, ix, ndim, jdegfd, &
              ncomp, jderiv, kchois, icount, inpelm
      logical compress

!     compress       i    If true compressible has been read
!     icheld         i    Choice parameter to distinguish between the various
!                         possibilities of this subroutine
!                         0:   Call by ELMxxx
!                         >0:  Call by ELDxxx
!                         <0:  Call by ELIxxx
!                         See the calling subroutines
!     icount         i    Number of unknowns in element
!     ielgrp         i    element group number
!     inpelm         i    number of points per element
!     itype          i    Type number for equation
!     ix             i    indication what type of derivative must be computed
!                         See DERIVA
!     jcheld         o    Value of icheld corrected for different storage
!                         schemes
!     jdegfd         i    The derivative du jdegfd / d ix is computed
!     jderiv         o    Indication if the value of the derivatives must be
!                         computed.
!     kchois         o    Indication what type of parameters must be computed
!                         Possible values:
!                         0:  Standard method, the calling subroutine is ELM900
!                             It is supposed that an element matrix must be
!                             computed
!                         1:  Subroutine ELD900 calls this subroutine, only
!                             derivatives must be computed
!                         2:  Subroutine ELI900 calls this subroutine, only
!                             integrals must be computed
!     ncoef          o    Number of coefficients in the equation
!     ncomp          o    Number of velocity components
!     ndim           i    Dimension of the space.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      real dum
      logical icheldallowed

!     dum            General dummy variable
!     icheldallowed  if true the value of icheld is allowed
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     262   ix has wrong value
!     264   jdegfd has wrong value
!     406   Value of icheld not allowed
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0900init' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0900init'

      end if
      if ( ierror/=0 ) go to 1000
      if ( icheld==0 ) then

!     --- Call by elm900/3/5, all coefficients required

         ncoef = 6
         compress = .true.

      else if ( icheld>0 ) then

!     --- Call by eld900/3/5, check parameter icheld

         icheldallowed = .false.
         if ( itype>=900 .and. itype<=902 .or. &
              itype>=916 .and. itype<=918 ) then
            icheldallowed = icheld<=31 .or. icheld>41 .and. icheld<=53
         else if ( itype==903 .or. itype==919 ) then
            icheldallowed = icheld<=31 .or. icheld>41 .and. icheld<=53
         else if ( itype==905 ) then
            icheldallowed = icheld<=31 .or. icheld>41 .and. icheld<=55
            if ( icheld==7 .or. icheld==20 .or. icheld==38 .or. &
                 icheld==49 ) icheldallowed = .false.
         end if  ! ( itype>=900 .and. itype<=902 ... )

         if ( .not. icheldallowed ) then

!        --- icheld out of range

            call errint ( icheld, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 406, 3, 0, 0 )

         end if
         if ( icheld<=11 ) then
            jcheld = icheld
         else if ( icheld<=16 ) then
            jcheld = icheld-10
         else if ( icheld==17 ) then
            jcheld = 11
         else if ( icheld<=19 ) then
            jcheld = icheld-10
         else if ( icheld==20 ) then
            jcheld = 7
         else if ( icheld<=31 ) then
            jcheld = icheld-20
         else if ( icheld<=42 ) then
            jcheld = icheld-31
         else
            jcheld = icheld-42
            compress = .true.
         end if
         compress = compress .or. jcheld==7
         if ( jcheld>=6 .and. jcheld/=7 .and. jcheld<=11 ) then
            ncoef = 6
         else if ( jcheld==7 ) then
            if ( ndim==2 .and. inpelm>=6 .and. &
                 itype/=902 .and. itype/=918 ) then
               ncoef = 6
            else
               ncoef = 1
            end if
         else
            ncoef = 0
            kchois = 1
         end if

         if ( jcheld==1 ) then

            if ( ix<=0 .or. ix>ndim ) ix = 1
            if ( jdegfd<=0 .or. jdegfd>ndim+1 ) jdegfd = 1

         end if

!        --- Compute ncomp

         if ( itype<916 ) then

!        --- itype = 900-903, standard case

            ncomp  = ndim

         else

!        --- itype = 916-919, 2d axi-symmetric with swirl

            ncomp  = 3

         end if  ! ( itype<916 )
         jderiv = 1

      else if ( icheld<0 ) then

!     --- call by eli900, one (new) coefficient required

         kchois = 2
         ncoef  = 1
         if ( itype==903 .or. itype==905 ) then

!        --- Types 903, 905

            ncomp = ndim

         else

!        --- Types 900-902

            dum = icount
            ncomp = nint(dum/inpelm)

         end if  ! ( itype==905 )

         if ( jdegfd<1 .or. jdegfd>ncomp ) then
            call errint ( jdegfd, 1 )
            call errint ( ncomp, 2 )
            call errsub ( 264, 2, 0, 0 )
         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el0900init'

      end if
      call erclos ( 'el0900init' )

      end
