      subroutine el0900rule ( mrule, irule, compress, ishape, icheld, &
                              jtype, n, m, jtrans, jelp, nveloc, ndim, &
                              iphi, igausp, jdiag, nvert, &
                              ndimlc, jcart, mn, inpelm, ielem, ielgrp, &
                              itype )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 10-02-2015 Extension for jelp = 8
!        version  1.2  date 13-05-2009 Jump in case of error
!        version  1.1  date 03-01-2007 Extension with type 903
!
!   copyright (c) 2006-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Set integration parameters with respect to elements of type 900-902
!     Depending on shape of elements
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mrule, irule, ishape, icheld, jtype, n, m, jtrans, jelp, &
              nveloc, ndim, iphi, igausp, jdiag, nvert, ndimlc, &
              jcart, mn, inpelm, ielem, ielgrp, itype
      logical compress

!     compress       i    If true the pressure must be computed
!     icheld         i    Choice parameter to distinguish between the various
!                         possibilities of this subroutine
!     ielem          i    Element number
!     ielgrp         i    element group number
!     igausp         i    Indication if Gauss points must be computed (1)
!                         or not (0)
!     inpelm         i    number of points per element
!     iphi           i    Indication if array phi must be filled (1)
!                         or not (0)
!     irule          i    type of numerical integration
!     ishape         i    Shape number of elements
!     itype          i    Type number of standard element
!     jcart          i    indication whether the element is a triangle with
!                         Cartesian coordinates (1)
!                         or a circular triangle with cylinder symmetric
!                         coordinates (2)
!     jdiag          i    Indicates if the basis functions satisfy phii(j = 0, &
!                         if i#j (jdiag=1) or not (jdiag=2)
!     jelp           o    Indication of the type of basis function subroutine
!                         must be called by ELxxxx.
!     jtrans         i    See common celint
!     jtype          i    Help parameter to store reduced type number
!     m              o    number of integration points
!     mn             o    n*m
!     mrule          i    Type of integration rule as given by the user
!     n              o    number of basis functions for each component
!     ndim           i    Dimension of the space.
!     ndimlc         i    Local dimension of space
!     nveloc         o    Number of velocity parameters
!     nvert          o    Used only when jmetod=11. Number of columns in the
!                         scheme.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jwcons

!     jwcons         Indication if the weight function is constant
!                    (1) or not (2)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0800INM      Compute the parameter m for an element
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1771   Shape number not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0900rule' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0900rule'
         write(irefwr,1) 'mrule, icheld, jelp, n, m', &
                          mrule, icheld, jelp, n, m
         write(irefwr,*) 'compress', compress
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( mrule>0 ) then

!     --- integration rule given by user

         irule = mrule

      else if ( .not. compress ) then

!     --- no compressibility types 900-902

         irule = 1

      else

!     --- compressibility

         irule = 3

      end if  ! ( mrule>0 )

      if ( ishape/=16 .and. icheld>0 .and. icheld<43 .and. &
           .not. compress .or.icheld<-2 ) irule = 1

      if ( jtype==900 .and. inpelm==6 .and. compress ) then
         n = 7
         jtrans = 1
         jelp = 7
         nveloc = 12
      else if ( jtype==901 .and. inpelm==7 ) then
         if ( icheld<0 ) then
            n = 6
            jelp = 4
         else
            n = 7
            jtrans = 1
            jelp = 7
         end if
         nveloc = 12
      else if ( jtype==901 .and. inpelm==9 ) then
         n = 9
         jtrans = 1
         jelp = 6
         nveloc = 16
      else if ( jtype==904 ) then
         n = 3
         jtrans = 0
         jelp = 8     ! linear non-conforming element
         nveloc = 6
      else if ( jtype>900 .and. inpelm==5 ) then
         n = 4
         jelp = 5
         nveloc = 8
      else if ( jtype>900 .and. ishape==17 ) then
         n = 8
         jelp = 13
         nveloc = 24
      else if ( jtype==902 .and. (inpelm==27 .or. ishape==6 .or. &
                ishape==16) .or. inpelm==7 ) then
         n = inpelm
         jtrans = 0
         jelp = ishape
         if ( icheld<0 .and. inpelm==7 ) then
            n = 6
            jelp = 4
         end if
         nveloc = ndim*n

      else if ( jtype==900 .and. (ishape>=4 .and. ishape<7 .or. &
                ishape==13 ) .or. jtype==901 .and. &
                ishape==7 .or. ishape==14 ) then
         n = inpelm
         jelp = ishape
         nveloc = ndimlc*n

      else if ( jtype/=903 ) then

         call errint ( ishape, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errsub ( 1771, 4, 0, 0 )
         go to 1000

      end if
      if ( jcart>=2 .and. irule==1 .and. compress ) irule = 3

      iphi=1
      igausp = 1
      jwcons = 2
      m = n
      if ( irule/=1 ) then

!     --- irule#1, compute m

         call el0800inm ( m, jelp, jwcons, irule, igausp, jdiag, iphi )

      end if  ! ( irule/=1 )

      mn = m * n
      if ( inpelm==27 ) then
         nvert  = 8
      else if ( inpelm==6 .or. inpelm==7 ) then
         nvert = 3
      else if ( ishape==6 .or. ishape==16 ) then
         nvert = 4
      else
         nvert = n
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el0900rule'

      end if  ! ( debug )
      call erclos ( 'el0900rule' )

      end
