      subroutine el0903rule ( mrule, irule, compress, ishape, icheld, &
                              n, m, jtrans, jelp, nveloc, ndim, &
                              iphi, igausp, jdiag, nvert, ncoef, &
                              jcart, mn, jdercn, ielem, ielgrp, &
                              itype, kchois, imas, iseqvel, iseqpres, &
                              npsi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 30-09-2010 Correction irule=2/jelp=12
!        version  1.1  date 08-03-2010 Error message for linear + NC
!        version  1.0  date 03-01-2007
!
!   copyright (c) 2007-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Set integration parameters with respect to elements of type 903
!     Depending on shape of elements
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mrule, irule, ishape, icheld, n, m, jtrans, jelp, &
              nveloc, ndim, iphi, igausp, jdiag, nvert, &
              jcart, mn, jdercn, ielem, ielgrp, itype, kchois, imas, &
              iseqvel(*), iseqpres(*), ncoef, npsi
      logical compress

!     compress       i    If true the pressure must be computed
!     icheld         i    Choice parameter to distinguish between the various
!                         possibilities of this subroutine
!     ielem          i    Element number
!     ielgrp         i    element group number
!     igausp         i    Indication if Gauss points must be computed (1)
!                         or not (0)
!     imas           i    Indicates the type of mass matrix (1=diagonal, &
!                         2=consistent)
!     iphi           i    Indication if array phi must be filled (1)
!                         or not (0)
!     irule          i    type of numerical integration
!     iseqpres       o     Contains the sequence numbers of the pressure in the
!                          element vector
!     iseqvel        o     Array containing the positions of the velocities in
!                          the element vector in the sequence required by el2005
!     ishape         i    Shape number of elements
!     itype          i    Type number of standard element
!     jcart          i    indication whether the element is a triangle with
!                         Cartesian coordinates (1)
!                         or a circular triangle with cylinder symmetric
!                         coordinates (2)
!     jdercn         i    Indication if the derivatives of the basis functions
!                         are constant (jdercn=0) or variable (jdercn=1)
!     jdiag          i    Indicates if the basis functions satisfy phii(j = 0, &
!                         if i#j (jdiag=1) or not (jdiag=2)
!     jelp           o    Indication of the type of basis function subroutine
!                         must be called by ELxxxx.
!     jtrans         i    See common celint
!     kchois         i    Indication what type of parameters must be computed
!     m              o    number of integration points
!     mn             o    n*m
!     mrule          i    Type of integration rule as given by the user
!     n              o    number of basis functions for each component
!     ncoef          i    Number of coefficients in the equation
!     ndim           i    Dimension of the space.
!     npsi           o    Number of pressure basis functions
!     nveloc         o    Number of velocity parameters
!     nvert          o    Used only when jmetod=11. Number of columns in the
!                         scheme.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jwcons, i, j

!     i              Counting variable
!     j              Counting variable
!     jwcons         Indication if the weight function is constant
!                    (1) or not (2)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0800INM      Compute the parameter m for an element
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1541   The combination imas = 1 and ishape=4 is not permitted
!    1771   Shape number not yet implemented
!    2859   integration rule is not newton cotes rule
!    3045   Newton Cotes rule not allowed for mini element
!    3046   Newton Cotes rule not allowed in combination with cylinder
!           coordinates
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0903rule' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0903rule'
  1      format ( a, 1x, (10i6) )
         write(irefwr,1) 'mrule, kchois, icheld, jelp, n', &
                          mrule, kchois, icheld, jelp, n

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( mrule>0 ) then

!     --- integration rule given by user

         irule = mrule
         if ( irule==1 ) then
            if ( ishape==3 .and. icheld==0 .or. ishape==10 ) then

!           --- Error: newton Cotes rule may not be applied

               call errsub ( 3045, 0, 0, 0 )
               go to 1000

            else if ( jcart>1 .and. ishape==4 .and. compress .and. &
                      icheld==0 ) then

!           --- Error: newton Cotes rule may not be applied

               call errsub ( 3046, 0, 0, 0 )
               go to 1000

            end if  ! ( ishape==3 .and. icheld==0 .or. ishape==10 )

         end if  ! ( irule==1 )

      else if ( kchois==1 .and. (icheld<43 .or. icheld>53) ) then

!     --- Call by derivative, set irule = 1
!         Only when not required in integration points

         irule = 1

      else if ( ishape==3 .and. icheld==0 .or. ishape==10 ) then
         irule = 2
      else if ( ishape==5 .and. icheld==0  .or. ishape==9 .or. &
                ishape==13 .or. ishape==17 .or. &
                ishape==4 .and. imas>1 ) then
         irule = 3
      else if ( ishape==11 .and. icheld==0 .or. ishape==18 .or. &
                ishape==12 .and. imas>1 ) then
         irule = 4
      else if ( ncoef==0 ) then
         irule = 1
      else if ( jcart>1 .and. compress ) then
         irule = 3
      else
         irule = 1
      end if

      if ( icheld>0 .and. irule/=1 ) then

!     --- Derivatives must be computed
!         in this case it is necessary to use a Newton Cotes rule

         call errint ( irule, 1 )
         call errwar ( 2859, 1, 0, 0 )
         irule = 1

      end if  ! ( icheld>0 .and. irule/=1 )

      if ( debug ) then
         write(irefwr,1) 'irule, n, m', irule, n, m
         write(irefwr,1) 'kchois, imas, ishape', kchois, imas, ishape
      end if  ! ( debug )

      if ( kchois==0 .and. imas>0 .and. irule==1 .and. &
           (ishape==4 .or. ishape==12) ) then

!     --- error ishape=4, 12 may not be combined with a diagonal mass
!         matrix, i.e. irule = 1

         call errint ( ishape, 1 )
         call errint ( irule, 2 )
         call errint ( imas, 3 )
         call errsub ( 1541, 3, 0, 0 )

      end if  ! ( imas==2 .and. irule==1 .and. .. )

      if ( irule/=1 .or. (ishape==3 .or. ishape==11 .or. &
           ishape==13) .and. icheld==0 .or. &
           ishape>=17 .or. ishape==10 ) jdiag=2

      if ( ishape==3 .or. ishape==10 ) then

!     --- extended linear triangle

         if ( icheld==0 ) then
            n = 4
            if ( ishape==3 ) jtrans = 1
            jelp = 10
         else
            jdercn = 0
         end if
         do i = 1, 4
            do j = 1, 2
               iseqvel((i-1)*2+j) = (i-1)*3+j
            end do
         end do
         do i = 1, 3
            iseqpres(i) = 3*i
         end do

      else if ( ishape==5 .or. ishape==9 ) then

!     --- extended linear quadrilateral

         if ( icheld==0 ) then
            n = 5
            if ( ishape==5 ) jtrans = 1
            jelp = 9
         else
            jdercn = 0
         end if
         do i = 1, 5
            do j = 1, 2
               iseqvel((i-1)*2+j) = (i-1)*3+j
            end do
         end do
         do i = 1, 4
            iseqpres(i) = 3*i
         end do
         call errint ( ishape, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errsub ( 1771, 4, 0, 0 )

      else if ( ishape==4 ) then

!     --- quadratic triangle

         do j = 1, 2
            do i = 1, 3
               iseqvel((i-1)*4+j) = (i-1)*5+j
               iseqvel((i-1)*4+2+j) = (i-1)*5+3+j
            end do
         end do
         iseqpres(1) = 3
         iseqpres(2) = 8
         iseqpres(3) = 13

      else if ( ishape==6 ) then

!     --- quadratic quadrilateral

         do j = 1, 2
            do i = 1, 4
               iseqvel((i-1)*4+j) = (i-1)*5+j
               iseqvel((i-1)*4+2+j) = (i-1)*5+3+j
            end do
            iseqvel(17) = 21
            iseqvel(18) = 22
         end do
         iseqpres(1) = 3
         iseqpres(2) = 8
         iseqpres(3) = 13
         iseqpres(4) = 18

      else if ( ishape==11 .or. ishape==18 ) then

!     --- extended linear tetrahedron

         if ( icheld==0 ) then
            n = 5
            if ( ishape==11 ) jtrans = 1
            jelp = 18
         else
            jdercn = 0
         end if
         do i = 1, 5
            do j = 1, 3
               iseqvel((i-1)*3+j) = (i-1)*4+j
            end do
         end do
         do i = 1, 4
            iseqpres(i) = 4*i
         end do
         if ( icheld==0 .and. irule<3 ) then

!        --- Integration rule<3 not allowed

            call errint ( irule, 1 )
            call errwar ( 2859, 1, 0, 0 )
            irule = 4  ! Default rule

         end if  ! ( icheld==0 .and. irule<3 )

      else if ( ishape==13 .or. ishape==17 ) then

!     --- extended linear hexahedron

         if ( icheld==0 ) then
            n = 9
            if ( ishape==13 ) jtrans = 1
            jelp = 17
         else
            jdercn = 0
         end if
         do i = 1, 9
            do j = 1, 3
               iseqvel((i-1)*3+j) = (i-1)*4+j
            end do
         end do
         do i = 1, 8
            iseqpres(i) = 4*i
         end do
         call errint ( ishape, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errsub ( 1771, 4, 0, 0 )

      else if ( ishape==12 ) then

!     --- quadratic tetrahedron

         do j = 1, 3
            do i = 1, 3
               iseqvel((i-1)*6+j) = (i-1)*7+j
               iseqvel((i-1)*6+3+j) = (i-1)*7+4+j
            end do
         end do
         do i = 19, 30
            iseqvel(i) = i+3
         end do
         iseqpres(1) = 4
         iseqpres(2) = 11
         iseqpres(3) = 18
         iseqpres(4) = 34
         if ( irule==2 ) then

!        --- Integration rule<3 not allowed

            call errint ( irule, 1 )
            call errwar ( 2859, 1, 0, 0 )
            if ( icheld==0 ) then
               irule = 4  ! Default rule
            else
               irule = 1  ! Newton Cotes
            end if  ! ( icheld==0 )

         end if  ! ( irule==2 )

      else if ( ishape==14 ) then

!     --- quadratic hexahedron

         do j = 1, 3
            do i = 1, 2
               iseqvel((i-1)*6+j) = (i-1)*7+j
               iseqvel((i-1)*6+18+j) = (i-1)*7+20+j
               iseqvel((i-1)*6+54+j) = (i-1)*7+58+j
               iseqvel((i-1)*6+72+j) = (i-1)*7+78+j
            end do
            iseqvel(3+j) = 4+j
            iseqvel(21+j) = 24+j
            iseqvel(57+j) = 62+j
            iseqvel(75+j) = 82+j
         end do
         do i = 10, 18
            iseqvel(i) = i+2
         end do
         do i = 28, 54
            iseqvel(i) = i+4
         end do
         do i = 64, 72
            iseqvel(i) = i+6
         end do
         do i = 1, 2
            iseqpres(1+(i-1)*4) = 4 +58*(i-1)
            iseqpres(2+(i-1)*4) = 11+58*(i-1)
            iseqpres(3+(i-1)*4) = 24+58*(i-1)
            iseqpres(4+(i-1)*4) = 31+58*(i-1)
         end do

      else if ( ishape==21 .and. icheld>0 ) then

!     --- Linear prism in case of derivatives

         do i = 1, 6
            do j = 1, 3
               iseqvel((i-1)*3+j) = (i-1)*4+j
            end do
         end do
         do i = 1, 6
            iseqpres(i) = 4*i
         end do

      else

!     --- shape number not yet available

         call errint ( ishape, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errsub ( 1771, 4, 0, 0 )

      end if

!     --- Compute m as function of irule and jelp

      if ( debug ) write(irefwr,1) 'n, jelp, irule', n, jelp, irule
      m = n
      if ( irule/=1 ) then

!     --- irule>1, Gauss rule

         jwcons = 2
         call el0800inm ( m, jelp, jwcons, irule, igausp, jdiag, &
                       iphi )
         if ( debug ) &
            write(irefwr,1) 'n, m, jelp, irule', n, m, jelp, irule

      end if  ! ( irule/=1 ) then

      mn = m * n
      iphi = 1
      igausp = 1
      if ( ishape==4 .or. ishape==10 ) then
         nvert = 3
      else if ( ishape==6 .or. ishape==9 .or. ishape==12 .or. &
                ishape==18 ) then
         nvert = 4
      else if ( ishape==14 .or. ishape==17 ) then
         nvert = 8
      end if
      nveloc = n*ndim
      npsi = nvert

1000  call erclos ( 'el0903rule' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el0903rule'

      end if  ! ( debug )

      end
