      subroutine el0907 ( iuser, user, jelp, jboun, isub, isup, array, &
                          ind1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 29-12-2006
!
!   copyright (c) 2006-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine ELM907
!     in the case that IFIRST = 0
!     The parameters in the common blocks celint, celiar and celp
!     get a start value
!     Array array is initialized with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     pressure_correction
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), array(*)
      integer iuser(*), jelp, jboun, isub, isup, ind1(*)

!     array   o      Real work array in which the values of the coefficients
!                    in the integration points are stored in the sequence:
!                         1D                  2D            3D
!                    1:   sigma            sigma          sigma
!                    2:   h                h              h
!                    3:   alpha 11         alpha 11       alpha 11
!                    4:                    alpha 12       alpha 12
!                    5:                    alpha 22       alpha 13
!                    6:                                   alpha 22
!                    7:                                   alpha 23
!                    8:                                   alpha 33
!                    Each coefficient requires exact m (is number of
!                    integration points) positions
!     ind1     o     Array containing the relative position of the vertices
!                    in index1
!     isub     o     Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup     o     Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     iuser    i     User array in which integer information of the
!                    coefficients is stored (See Standard Problems)
!     jboun    o     Defines the type of boundary condition.
!                    Possible values:
!                    2:  Standard mixed (Robbins) boundary condition
!                    3:  Boundary condition of type 3
!                    4:  Boundary condition of type 4 (not yet available)
!     jelp     o     Indication of the type of basis function subroutine must
!                    be called by ELM801. Possible vaues:
!                    0:  Point element, no special basis functions
!                    1:  Subroutine ELP610  (Linear line element)
!                    2:  Subroutine ELP611  (Quadratic line element)
!                    3:  Subroutine ELP620  (Linear triangle)
!                    4:  Subroutine ELP622  (Quadratic triangle)
!                    5:  Subroutine ELP621  (Bilinear quadrilateral)
!                    6:  Subroutine ELP623  (Biquadratic quadrilateral)
!     user     i     User array in which real information of the
!                    coefficients is stored (See Standard Problems)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jwcons, ncoef, mrule, ishift, i
      logical linear

!     i              Counting variable
!     ishift         Shift in coefficient sequence number
!     jwcons         Indication if the weight function is constant (1) or
!                    not (2)
!     linear         Indicates whether the element is a linear simplex (true)
!                    or not (false)
!     mrule          Type of integration rule as given by the user
!     ncoef          Number of coefficients in the equation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0801COEFF    Initialize coefficients for elements of type 801 and 907
!     EL0801FLCOEFFS Initialize coefficients for elements of type 801 and 907
!     EL0801RULE     Set integration parameters with respect to elements
!                    of type 801, 907
!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1771   Shape number not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0907' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0907'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call elcomm
      jtrans = 0
      ideriv = 0
      jderiv = 0
      ishift = 0

!     --- Set parameters depending on the shape numbers

      select case ( ishape )
         case ( :1,3,5,7: )

!        --- Error 1771: this type of element has not yet been implemented

            call errint ( ishape, 1 )
            call errint ( ielem, 2 )
            call errint ( itype, 3 )
            call errint ( ielgrp, 4 )
            call errsub ( 1771, 4, 0, 0 )

         case (2)

!        --- Quadratic line elements, will be treated as linear

            n = 2
            jelp = 1

         case (4)

!        --- Quadratic triangles, will be treated as linear

            n = 3
            jelp = 3

         case (6)

!        --- Bi-quadratic quadrilaterals, will be treated as bi-linear

            n = 4
            jelp = 5

      end select  ! case ( ishape )

      do i = 1, n
         ind1(i) = 2*i-1
      end do  ! i = 1, n

      m = n
      if ( jelp==5 ) then

!     --- jelp = 5, quadrilateral

         linear = .false.
         jdercn = 1
         mn = m * n

      else

!     --- jelp = 1, 3, linear element

         linear = .true.
         mn = m

      end if  ! ( jelp==5 )

!     --- Fill the arrays IND, MST and ARRAY with information of the
!         coefficient

      call el0801coeff ( iuser, linear, jboun, mrule, jwcons, &
                         ishift )
      if ( ierror/=0 ) go to 1000

!     --- Set parameters depending on the shape numbers

      call el0801rule ( irule, igausp, iphi, jdiag, jwcons, m, &
                        jelp, mn, kstep, n, jcart )
      if ( ierror/=0 ) go to 1000

!     --- Fill coefficients in array and do some checks

      call el0801flcoeffs ( ncoef, ishift, iuser, user, &
                            array, jboun, jwcons, isub, isup )
      notmas = 1

1000  call erclos ( 'el0907' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el0907'

      end if  ! ( debug )

      end
