      subroutine el1000sp ( sp, div, phix, w, x, xgauss, rhsdiv, &
                            fdiv, mdiv, ishape, coeffgrad, coeffdiv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 27-02-2010 Extra parameters
!        version  1.0  date 26-03-2001
!
!   copyright (c) 2001-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the divergence and the pressure gradient matrix for the
!     incompressible Navier-Stokes equations
!     The divergence matrix is stored in array DIV
!     The transpose of the pressure gradient matrix in SP
!     mcont = 3, i.e. special divergence and graident matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision phix(n,m,ndimlc), w(m), sp(ndimlc*n,*), &
                       xgauss(m,*), x(n,*), &
                       div(ndimlc*n,*), rhsdiv(*), fdiv(m), &
                       coeffgrad(m), coeffdiv(m,ndimlc)
      integer mdiv, ishape

!     coeffdiv       i    Array of size m*ndim containing the coefficients
!                         for the partial derivatives in the pseudo divergence
!                         operators for the m integration points
!                         Hence coeffdiv(k,i) is the coefficient for du/dx_i
!                         in integration point x^k
!     coeffgrad      i    Array of size m containing the multiplication factor
!                         for the gradient in the integration points
!     div            o    Divergence matrix
!                         div(i,k) corresponds to the k-th row and i-th column
!                         of the matrix, The k refers to the basis functions of
!                         the pressure, i to the basis functions of the velocity
!                         in the sequence of the solution vector
!     fdiv           i    Contains values of divergence right-hand side to be
!                         integrated (if mdiv=1)
!     ishape         i    Shape number of elements
!     mdiv           i    Defines if divergence right-hand side is used (1)
!                         or not (0)
!     phix           i    Array of length n x m x ndim  containing the
!                         derivatives of the basis functions in the sequence:
!                         d phi / dx (xg ) = phix (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = phix (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = phix (i,k,3);
!                              i        k
!                         If the element is a linear tetrahedron (n=4), then
!                         the derivatives are constant and only k = 1 is filled.
!     rhsdiv         o    Divergence right-hand side (if mdiv=1)
!     sp             o    Gradient matrix
!                         SP(i,k) corresponds to the k-th row and i-th column
!                         of the matrix, The k refers to the basis functions of
!                         the pressure, i to the basis functions of the velocity
!                         in the sequence of the solution vector
!     w              i    array of length m containing the weights for
!                         integration
!     work                Work array of length 2m+n
!     x              i    array containing the coordinates of the nodal points
!                         according to:
!                         x  = x (i,1);  y  = x (i,2);  z  = x (i,3)
!                          i              i              i
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, icent, jdiags
      double precision sum, work(2*n+m)
      save icent

!     i       Counting variable
!     icent   Index of centroid
!     j       Counting variable
!     jdiags  Save value of jdiag
!     k       Counting variable
!     sum     help variable to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3003  Compute integrals of the shape / fdiv(x) phi_i(x) dx
!                                            e
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2248   Coorindate system not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The gradient matrix is given by:
!     SP = |  SPu  SPv  SPw  |
!     with:
!     SPu(i,k) = / c_grad psi(i) dphidx(k) dx
!                x
!     SPv(i,k) = / c_grad psi(i) dphidy(k) dx
!                x
!     SPw(i,k) = / c_grad psi(i) dphidz(k) dx
!                x
!     The divergence matrix is defined by:
!     DIVu(i,k) = / c_div(1) psi(i) dphidx(k) dx
!                x
!     DIv(i,k) = / c_div(2) psi(i) dphidy(k) dx
!                x
!     DIw(i,k) = / c_div(3) psi(i) dphidz(k) dx
!                x
!     For a linear approximation of the pressure (np=4), psi is given by
!     psi(1)  = 1
!     psi(2)  = x - x14
!     psi(3)  = y - y14
!     psi(4)  = z - z14
!     The matrix SP is organized in the sequence of the velocity vector
!     i.e. u1, v1, w1, u2, v2 ....
!     In case of axi-symmetric co-ordinates the matrix is given by:
!     SPu(i,k) = / psi(i) (dphidx(k) + phi(k)/x) dx
!                x
!     SPv(i,k) = / psi(i) dphidy(k) dx
!                x
!     In case of polar co-ordinates the matrix is given by:
!     SPu(i,k) = / psi(i) (dphidx(k) + phi(k)/x) dx
!                x
!     SPv(i,k) = / psi(i) dphidy(k)/x dx
!                x
!     The right-hand side is defined by:
!     rhsdiv(i) = / psi(i) f dx
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( ifirst==0 ) then

!     --- ifirst = 0, compute icent as function of inpelm

         if ( inpelm==27 ) then
            icent = 14
         else
            icent = n
         end if

      end if

      if ( jcart==1 ) then

!     --- Cartesian co-ordinates
!         psi = 1

         jadd = 0
         jconsf = 2
         jdiags = jdiag
         jdiag = 2
         do i = 1, ndimlc
            call el3003 ( coeffgrad, work, w, phix(1,1,i), m, n )
            do k = 1, n
               sp(ndimlc*(k-1)+i,1) = work(k)
            end do
         end do

!        --- fill the divergence matrix

         do i = 1, ndimlc
            call el3003 ( coeffdiv(1,i), work, w, phix(1,1,i), m, n )
            do k = 1, n
               div(ndimlc*(k-1)+i,1) = work(k)
            end do
         end do

         if ( mdiv==1 ) then

!        --- mdiv = 1, divergence right-hand side

            sum = 0d0
            do k = 1, m
               sum = sum + w(k)*fdiv(k)
            end do
            rhsdiv(1) = sum

         end if

         if ( ishape==4 .or. ishape==6 .or. ishape==7 .or. &
              ishape==14 ) then

!        --- psi = x  - xcent
!                   j        j

            do j = 1, ndimlc
               do k = 1, m
                  work(k+n) = (xgauss(k,j) - x(icent,j))*coeffgrad(k)
               end do
               do i = 1, ndimlc
                  call el3003 ( work(n+1), work, w, phix(1,1,i), m, n )
                  do k = 1, n
                     sp(ndimlc*(k-1)+i,j+1) = work(k)
                  end do
               end do

!              --- fill divergence

               do k = 1, m
                  work(k+n) = (xgauss(k,j) - x(icent,j))*coeffdiv(k,j)
               end do
               do i = 1, ndimlc
                  call el3003 ( work(n+1), work, w, phix(1,1,i), m, n )
                  do k = 1, n
                     div(ndimlc*(k-1)+i,j+1) = work(k)
                  end do
               end do

               if ( mdiv==1 ) then

!              --- mdiv = 1, divergence right-hand side

                  sum = 0d0
                  do k = 1, m
                     sum = sum + w(k)*fdiv(k)*work(k+n)
                  end do
                  rhsdiv(j+1) = sum

               end if

            end do
            jdiag = jdiags

         end if

      else

!     --- error 2248: jcart # 1, not yet implemented

         call eropen ( 'el1000sp' )
         call errint ( jcart, 1 )
         call errint ( 1, 2 )
         call errint ( 1, 3 )
         call errint ( ielem, 4 )
         call errint ( itype, 5 )
         call errint ( ielgrp, 6 )
         call errsub ( 2248, 6, 0, 0 )
         call erclos ( 'el1000sp' )

      end if

      end
