      subroutine el1008 ( elemmt, s, sp, div, elemvc, f, iseqvel, &
                          iseqpres, pp, mcont, rhsdiv, signdiv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.5  date 07-02-2013 Extra debug
!        version  2.4  date 13-08-2012 New call to elmatinv
!        version  2.3  date 17-09-2008 Extension with pressure mass matrix
!
!   copyright (c) 1996-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill element matrix from the velocity matrix, the gradient (pressure)
!    matrix and the divergence matrix (Taylor-Hood)
!    Fill also the element vector
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision elemmt(icount,icount), s(ndimlc*n,ndimlc*n), &
                       sp(ndimlc*n,npsi), div(ndimlc*n,npsi), &
                       elemvc(icount), f(ndimlc*n), pp(npsi,npsi), &
                       rhsdiv(npsi)
      integer iseqvel(*), iseqpres(*), mcont, signdiv

!     div            i     Divergence matrix
!                          div(i,k) corresponds to the k-th row and i-th column
!                          of the matrix, The k refers to the basis functions of
!                          the pressure, i to the basis functions of the
!                          velocity in the sequence of the solution vector
!     elemmt         o     Element matrix
!     elemvc         o     Element vector
!     f              i     Element vector with respect to velocity
!     iseqpres       i     Contains the sequence numbers of the pressure in the
!                          element vector
!     iseqvel        i     Array containing the positions of the velocities in
!                          the element vector in the sequence required by el2005
!     mcont          i     Defines the type of continuity equation
!                          Possible values:
!                          0: incompressible flow
!                          1: compressible flow: div (rho u ) = 0
!                          4: the continuity equation contains a pressure term
!     pp             i     Pressure matrix
!     rhsdiv         i     Divergence right-hand side (if mdiv=1)
!     s              i     Element matrix with respect to velocity
!     signdiv        i    Multiplication sign of "continuity equations"
!                         if -1 the stokes matrix is symmetric
!                         if 1 all eigenvalues are in the positive half plane
!     sp             i     Gradient of pressure matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, nact, nveloc, nvelrd, isigndiv
      double precision sinv(3,3), det
      double precision, allocatable :: eltmp(:,:), elvec(:,:)
      logical singular
      save nact, nveloc, nvelrd

!     det            Determinant of matrix
!     eltmp          Help array to store the element matrix before deleting the
!                    centroid
!     elvec          Help array to store a pasrt of the matrix matrix
!                    multiplication
!     error          Return value of allocate or deallocate
!     i              Counting variable
!     isigndiv       signdiv except for mcont = 5, then 1
!     j              Counting variable
!     nact           Actual number of points with velocity unknowns
!     nveloc         number of velocity unknowns
!     nvelrd         reduced number of velocity unknowns
!     singular       if false the matrix inverse subroutine stops in case the
!                    matrix is singular
!     sinv           Contains the inverse of the matrix Scc
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELMATINV       Compute inverse of 3 x 3 matrix
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3121   determinant is zero
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The Navier-Stokes matrix has the shape:
!            |               T   |
!            |   S        -SP    |    u
!     S   =  |    uu             |
!            |                   |
!            |   DIV        0    |    p
!   Actually the velocity and pressure are intermixed.
!   For Taylor Hood the sequence is:
!   Linear extended triangle:
!     u_1, v_1, p_1, u_2, v_2, p_2, u_3, v_3, p_3,u_4,v_4
!   quadratic triangle:
!     u_1, v_1, p_1, u_2, v_2, u_3, v_3, p_3, u_4,v_4, p_4, u_5, v_5, p_5, &
!     u_6, v_6
!   if ( matrix ) then
!      Fill velocity matrix
!      Fill divergence matrix
!   else ( vector)
!      Fill velocity vector
!   If the centroid must be eliminated the procedure is as follows:
!   The unknowns are splitted in three parts u_i, u_c and p, &
!   where u_c efers to the centroid velocities
!   The matrix has the structure:
!   |  S_ii  S_ic    -SP_i  |  | u_i |    | f_i |
!   |  S_ci  S_cc    -SP_c  |  | u_c | =  | f_c |
!   |  D_ci  D_cc       P   |  | p   |    | g   |
!   From S_ci u_i + S_cc u_c + SP_c p = f_c, we get
!   u_c = ( f_c - S_ci u_i - SP_c p ) / S_cc  (inversion)
!   Hence the system of equations becomes:
!   | S_ii - S_ic / S_cc S_ci   - SP_i + S_ic / S_cc SP_c | | u_i |
!   | D_ci - D_cc / S_cc S_ci     P  - D_cc / S_cc SP_c   | | p   | = &
!   | f_i - S_ic / S_cc f_c |
!   |  g  - D_cc / S_cc f_c  |
!   In 2d we have:
!                        1               |  SC_22  -SC_12 |
!   1/S_cc = ------------------------- |                |
!            SC_11 SC_22 - SC_21 SC_12   | -SC_21   SC_11 |
!   Mark that the storage of the two-dimensional arrays is transposed
!   with respect to the standard sequence as used by SEPRAN
!   Hence  S_ij = S(j,i) and so on
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el1008' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el1008'
         write(irefwr,1) 'n, ndimlc, jtrans', n, ndimlc, jtrans
         write(irefwr,1) 'mcont', mcont
         call prinrl1 ( s, npsi, ndimlc*n, 's' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         nact = n-jtrans
         nveloc = n*ndimlc
         nvelrd = nact*ndimlc

      end if

!     --- initialize isigndiv
!         Must be 1 in case of mcont = 5

      isigndiv = signdiv
      if ( mcont==5 ) isigndiv = 1

      if ( debug ) &
         write(irefwr,1) 'nact, nveloc, nvelrd, isigndiv, npsi', &
                          nact, nveloc, nvelrd, isigndiv, npsi

      if ( jtrans==0 ) then

!     --- No transformation

         if ( matrix ) then

!        --- Fill velocity matrix

            do i = 1, nact*ndimlc
               do j = 1, nact*ndimlc
                  elemmt(iseqvel(i),iseqvel(j)) = s(i,j)
               end do
            end do

!           --- Divergence matrix

            do i = 1, npsi
               do j = 1, nact*ndimlc
                  elemmt(iseqvel(j),iseqpres(i)) = signdiv*div(j,i)
               end do
            end do

!           --- gradient matrix

            do j = 1, npsi
               do i = 1, nact*ndimlc
                  elemmt(iseqpres(j),iseqvel(i)) = -sp(i,j)
               end do
            end do

            if ( mcont==4 .or. mcont==5 ) then

!           --- pressure matrix

               do j = 1, npsi
                  do i = 1, npsi
                     elemmt(iseqpres(j),iseqpres(i)) = isigndiv*pp(i,j)
                  end do
               end do

            else

!           --- clear pressure matrix

               do j = 1, npsi
                  do i = 1, npsi
                     elemmt(iseqpres(j),iseqpres(i)) = 0d0
                  end do
               end do

            end if

         end if

         if ( vector ) then

!        --- Fill velocity vector

            do i = 1, nact*ndimlc
               elemvc(iseqvel(i)) = f(i)
            end do

         end if

      else if ( jtrans==1 ) then

!     --- jtrans = 1, the centroid must be eliminated
!         Fill velocity matrix in standard sequence
!         After the elimination it is transposed

         allocate ( eltmp(npsi+nveloc,npsi+nveloc), &
                    elvec(1:ndimlc,1:nvelrd), stat = error )
         if ( error/=0 ) &
            call eralloc ( error, (npsi+nveloc)**2+ndimlc*nvelrd, &
                           'eltmp' )
         if ( ierror/=0 ) go to 1000

         do i = 1, nveloc
            eltmp(i,1:nveloc) = s(1:nveloc,i)
         end do

!        --- Divergence matrix

         do i = 1, npsi
            eltmp(i+nveloc,1:nveloc) = signdiv*div(1:nveloc,i)
         end do

!        --- gradient matrix

         do j = 1, npsi
            eltmp(1:nveloc,j+nveloc) = -sp(1:nveloc,j)
         end do

         if ( mcont==4 .or. mcont==5 ) then

!        --- pressure matrix

            do j = 1, npsi
               do i = 1, npsi
                  eltmp(i+nveloc,j+nveloc) = isigndiv*pp(j,i)
               end do
            end do

         else

!        --- clear pressure matrix

            eltmp(1+nveloc:npsi+nveloc,1+nveloc:npsi+nveloc) = 0d0

         end if

         if ( debug ) &
            call prinrl1 ( eltmp, nveloc+npsi, nveloc+npsi, 'eltmp' )

         if ( ndimlc==2 ) then

!        --- ndim = 2
!            Compute the inverse of the matrix S_cc

            det = eltmp(nvelrd+1,nvelrd+1)*eltmp(nvelrd+2,nvelrd+2)- &
                  eltmp(nvelrd+1,nvelrd+2)*eltmp(nvelrd+2,nvelrd+1)

            if ( debug ) write(irefwr,2) 'det', det

            if ( abs(det)<=0d0 ) then

!           --- determinant zero

               call errint ( itype, 1 )
               call errsub ( 3121, 1, 0, 0 )
               go to 1000

            end if  ! ( abs(det)<=0d0 )

            det = 1d0/det
            sinv(1,1) = det*eltmp(nvelrd+2,nvelrd+2)
            sinv(1,2) = -det*eltmp(nvelrd+1,nvelrd+2)
            sinv(2,1) = -det*eltmp(nvelrd+2,nvelrd+1)
            sinv(2,2) = det*eltmp(nvelrd+1,nvelrd+1)

            if ( matrix ) then

!           --- Compute S_cc^(-1) S_ci

               do i = 1, 2
                  do j = 1, nvelrd
                     elvec(i,j) = sinv(i,1)*eltmp(nvelrd+1,j)+ &
                                  sinv(i,2)*eltmp(nvelrd+2,j)
                  end do
               end do

!              --- Fill velocity matrix

               do i = 1, nvelrd
                  do j = 1, nvelrd
                     elemmt(iseqvel(j),iseqvel(i)) = eltmp(i,j)- &
                        eltmp(i,1+nvelrd)*elvec(1,j) - &
                        eltmp(i,2+nvelrd)*elvec(2,j)
                  end do
               end do

!              --- Divergence matrix

               do i = 1, npsi
                  do j = 1, nvelrd
                     elemmt(iseqvel(j),iseqpres(i)) = eltmp(i+nveloc,j)- &
                        eltmp(i+nveloc,nvelrd+1)*elvec(1,j)- &
                        eltmp(i+nveloc,nvelrd+2)*elvec(2,j)
                  end do
               end do

!              --- Compute S_cc^(-1) SP_c

               do i = 1, 2
                  do j = 1, npsi
                     elvec(i,j) = sinv(i,1)*eltmp(1+nvelrd,j+nveloc) + &
                                  sinv(i,2)*eltmp(2+nvelrd,j+nveloc)
                  end do
               end do

!              --- gradient matrix

               do i = 1, nvelrd
                  do j = 1, npsi
                     elemmt(iseqpres(j),iseqvel(i)) = eltmp(i,j+nveloc)- &
                        eltmp(i,nvelrd+1)*elvec(1,j) - &
                        eltmp(i,nvelrd+2)*elvec(2,j)
                  end do
               end do

!              --- Pressure matrix

               do i = 1, npsi
                  do j = 1, npsi
                     elemmt(iseqpres(j),iseqpres(i)) = &
                        eltmp(i+nveloc,j+nveloc) - &
                        eltmp(i+nveloc,nvelrd+1)*elvec(1,j) - &
                        eltmp(i+nveloc,nvelrd+2)*elvec(2,j)
                  end do
               end do

            end if

            if ( vector ) then

!           --- Fill velocity vector

               do i = 1, 2
                  elvec(i,1) = sinv(i,1)*f(1+nvelrd)+ &
                               sinv(i,2)*f(2+nvelrd)
               end do
               do i = 1, nvelrd
                  elemvc(iseqvel(i)) = f(i) - &
                     eltmp(i,1+nvelrd)*elvec(1,1) - &
                     eltmp(i,2+nvelrd)*elvec(2,1)
               end do
               do i = 1, npsi
                  elemvc(iseqpres(i)) = &
                     signdiv*rhsdiv(i) - &
                     eltmp(i+nveloc,1+nvelrd)*elvec(1,1) - &
                     eltmp(i+nveloc,2+nvelrd)*elvec(2,1)
               end do

            end if

         else

!        --- ndim = 3

            do i = 1, 3
               do j = 1, 3
                  elvec(i,j) = eltmp(nvelrd+i,nvelrd+j)
               end do
            end do
            singular= .false.
            call elmatinv ( elvec, sinv, singular )

            if ( matrix ) then

!           --- Compute S_cc^(-1) S_ci

               do i = 1, 3
                  do j = 1, nvelrd
                     elvec(i,j) = sinv(i,1)*eltmp(nvelrd+1,j)+ &
                                  sinv(i,2)*eltmp(nvelrd+2,j)+ &
                                  sinv(i,3)*eltmp(nvelrd+3,j)
                  end do
               end do

!              --- Fill velocity matrix

               do i = 1, nvelrd
                  do j = 1, nvelrd
                     elemmt(iseqvel(j),iseqvel(i)) = eltmp(i,j)- &
                        eltmp(i,1+nvelrd)*elvec(1,j) - &
                        eltmp(i,2+nvelrd)*elvec(2,j) - &
                        eltmp(i,3+nvelrd)*elvec(3,j)
                  end do
               end do

!              --- Divergence matrix

               do i = 1, npsi
                  do j = 1, nvelrd
                     elemmt(iseqvel(j),iseqpres(i)) = eltmp(i+nveloc,j)- &
                        eltmp(i+nveloc,nvelrd+1)*elvec(1,j)- &
                        eltmp(i+nveloc,nvelrd+2)*elvec(2,j)- &
                        eltmp(i+nveloc,nvelrd+3)*elvec(3,j)
                  end do
               end do

!              --- Compute S_cc^(-1) SP_c

               do i = 1, 3
                  do j = 1, npsi
                     elvec(i,j) = sinv(i,1)*eltmp(1+nvelrd,j+nveloc) + &
                                  sinv(i,2)*eltmp(2+nvelrd,j+nveloc) + &
                                  sinv(i,3)*eltmp(3+nvelrd,j+nveloc)
                  end do
               end do

!              --- gradient matrix

               do i = 1, nvelrd
                  do j = 1, npsi
                     elemmt(iseqpres(j),iseqvel(i)) = eltmp(i,j+nveloc)- &
                        eltmp(i,nvelrd+1)*elvec(1,j) - &
                        eltmp(i,nvelrd+2)*elvec(2,j) - &
                        eltmp(i,nvelrd+3)*elvec(3,j)
                  end do
               end do

!              --- Pressure matrix

               do i = 1, npsi
                  do j = 1, npsi
                     elemmt(iseqpres(j),iseqpres(i)) = &
                        eltmp(i+nveloc,j+nveloc) - &
                        eltmp(i+nveloc,nvelrd+1)*elvec(1,j) - &
                        eltmp(i+nveloc,nvelrd+2)*elvec(2,j) - &
                        eltmp(i+nveloc,nvelrd+3)*elvec(3,j)
                  end do
               end do

            end if

            if ( vector ) then

!           --- Fill velocity vector

               do i = 1, 3
                  elvec(i,1) = sinv(i,1)*f(1+nvelrd)+ &
                               sinv(i,2)*f(2+nvelrd)+ &
                               sinv(i,3)*f(3+nvelrd)
               end do
               do i = 1, nvelrd
                  elemvc(iseqvel(i)) = f(i) - &
                     eltmp(i,1+nvelrd)*elvec(1,1) - &
                     eltmp(i,2+nvelrd)*elvec(2,1) - &
                     eltmp(i,3+nvelrd)*elvec(3,1)
               end do
               do i = 1, npsi
                  elemvc(iseqpres(i)) = &
                     rhsdiv(i) - &
                     eltmp(i+nveloc,1+nvelrd)*elvec(1,1) - &
                     eltmp(i+nveloc,2+nvelrd)*elvec(2,1) - &
                     eltmp(i+nveloc,3+nvelrd)*elvec(3,1)
               end do

            end if

         end if

         deallocate ( eltmp, elvec, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'eltmp' )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el1008'

      end if  ! ( debug )
      call erclos ( 'el1008' )

      end
