      subroutine el2600 ( phi0, phin, phi, dphidx0, dphidxn, &
                          psi, x, xg, u, ug, Fd, Jd, Fn, Jn, p, pn, usrvec, &
                          iseqnr, iusrvc, isequ, iseqp, npres, &
                          xg0, xgn, maxusr )
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  5.0  date 06-01-2016 Use sepmoduleelem
!        version  4.1  date 12-12-2012 Long integers
!        version  4.0  date 07-07-2007 Extra parameter maxusr
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Get old displacement vector and deformation gradient for
!    general nonlinear solid. If necessary the initial stress is
!    retrieved.
! **********************************************************************
!
!                       KEYWORDS
!
!     mechanics
!     non_linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqnr, iusrvc, npres, isequ(*), &
              iseqp(*), maxusr
      double precision phi0(n,m),  phin(n,m), phi(n,m), &
                       dphidx0(n,m,ndim),  dphidxn(n,m,ndim), &
                       psi(npres,m), x(n,ndim), xg(m,ndim), &
                       u(n,ndim), ug(m,ndim), fd(m,ndim,ndim), jd(m), &
                       fn(m,ndim,ndim), jn(m), pn(m), p(m), &
                       usrvec(m,*), xg0(m,ndim), xgn(m,ndim)

!     dphidx0        i    Derivatives of the shape functions in the initial
!                         configuration
!     dphidxn        i    Derivatives of the shape functions in the reference
!                         configuration
!     fd             o    Deformation gradient F(k,j,i) = F_ij(x^k)
!     fn             o    Deformation gradient at incr. n Fn(k,j,i) = F_ij(x^k)
!     iseqnr         i    Sequence number with respect to the storage in array
!                         ISEPIN
!     iseqp          i    Sequence for pressure dof's
!     isequ          i    Sequence for displacement dof's
!     iusrvc         i    if>0, a user defined vector is used
!     jd             o    Determinant of deformation gradient
!     jn             o    Determinant of deformation gradient at incr. n
!     maxusr         i    Maximum number of degrees of freedom in usrvec
!     npres          i    Number of pressure parameters per element
!     p              i    Array containing the pressure
!                         p(1) contains the pressure in the centroid
!                         p(2-4) contains the gradient of the pressure in the
!                         centroid
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied. These are the shape functions in the current
!                         configuration
!     phi0           i    Similar to array phi but now in the initial
!                         configuration
!     phin           i    Similar to array phi but now in the reference
!                         configuration
!     pn             i    The pressures in the reference configuration
!     psi            o    Array of length n x m containing the values of the
!                         modified basis functions in the integration points
!                         Basis functions for the pressure
!     u              o    Real n x ncomp array in which the values of the
!                         solution vector in the nodal points of the element
!                         are stored in the sequence:
!                         u  (x ) = u(j,i)  j = 1(1)n, i = 1 (1) ncomp
!                          i   j
!                         Array u is always filled
!     ug             o    array of length m x ncomp in which the values of
!                         u in the integration points are stored the sequence:
!                         ug (x ) = u(j,i)  j = 1(1)m, i = 1 (1) ncomp
!                           i   j
!                         ugauss is only filled if jdiag>1 in common celint
!     usrvec         i    user defined vector
!     x              i    array of length inpelm x 3 containing the x, y and
!                         z-coordinates of the nodal points with respect to the
!                         local numbering in the sequence:
!                         x  = x (i,1);  y  = x (i,2); z  = x (i,3)
!                          i              i             i
!     xg             i    Array of length m x 3 containing the co-ordinates of
!                         the gauss points in the current configuration.
!                         Array xgauss is only filled when Gauss integration is
!                         applied.
!                         xg  = xg(i,1);  yg  = xg(i,2);  zg  = xg(i,3);
!                           i               i               i
!     xg0            i    Array of length m x 3 containing the co-ordinates of
!                         the gauss points in the initial configuration.
!                         Array xgauss is only filled when Gauss integration is
!                         applied.
!                         xg  = xg(i,1);  yg  = xg(i,2);  zg  = xg(i,3);
!                           i               i               i
!     xgn            i    Array of length m x 3 containing the co-ordinates of
!                         the gauss points in the reference configuration.
!                         Array xgauss is only filled when Gauss integration is
!                         applied.
!                         xg  = xg(i,1);  yg  = xg(i,2);  zg  = xg(i,3);
!                           i               i               i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, ncompsav
      double precision, pointer :: vecoldlc(:)
      double precision pn_tmp(npres), pd_tmp(npres), rdum(1), &
                       un(n,ndim), usr_tmp(n,maxusr)

!     i              Counting variable
!     j              Counting variable
!     ncompsav       Help variable to save the value of ncomp
!     pd_tmp         Help array to store the pressures
!     pn_tmp         Help array to store the pressures
!     rdum           dummy variable
!     un             normal component of velocity
!     usr_tmp        ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL2005         Compute a local vector and its derivatives from a known
!                    vector
!     EL2700         Compute deformation gradient
!     EL3007         Fills old solution and derivatives
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el2600' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el2600'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get the displacements and derivatives on increment n
!         always jind = 0, because we use isequ to look up the
!         correct degree of freedom

      jderiv = 1
      jsol = 1
      jind = 0
      call el2005 ( un, Fn, phi0, dphidx0, &
                    isequ, ug, iseqnr+1 )

!     --- Compute deformation gradient from displacement gradient

      call el2700 (Fn, Jn)

!     --- Get the delta displacements and derivatives
!         always jind = 0, because we use isequ to look up the
!         correct degree of freedom

      jderiv = 1
      jsol = 1
      jind = 0
      call el2005 ( u, Fd, phin, dphidxn, &
                    isequ, ug, iseqnr )

!     --- Compute deformation gradient from displacement gradient

      call el2700 (Fd, Jd)

      if (jcart==2) then
         do i=1,m
            Jn(i)=Jn(i)*xgn(i,1)/xg0(i,1)
            Jd(i)=Jd(i)*xg(i,1)/xgn(i,1)
         end do
      end if

!     --- Get pressure degrees of freedom for TYPE = 201/202

      if (itype>200) then
         iseqtype = oldsols(iseqnr+1)%iseqtype
         index3n => indoldsols(iseqtype)%index3
         vecoldlc => oldsols(iseqnr+1)%oldvecs
         pn_tmp(1:npres) = vecoldlc(index3n(iseqp(1:npres)))
         iseqtype = oldsols(iseqnr)%iseqtype
         index3n => indoldsols(iseqtype)%index3
         vecoldlc => oldsols(iseqnr)%oldvecs
         pd_tmp(1:npres) = vecoldlc(index3n(iseqp(1:npres)))
      end if

!     --- Check whether to retrieve the user defined vector

      if (iusrvc>=1) then

         jderiv = 0
         jsol = 1
         jind = 1
         ncompsav = ncomp
         ncomp = iusrvc

!        --- only the values in the integration points are returned

         call el3007 ( usr_tmp, rdum, phi, rdum, m, n, &
                       ndim, iusrvc, ind, rdum, usrvec, &
                       jtrans, numold, iseqnr+2 )
         ncomp = ncompsav

      end if

!     --- Compute pressure basis functions in integration points
!         for TYPE=201 (Crouzeix-Raviart). This was NOT carried out
!         in the ELP routine!

      if (itype==201) then
         do i = 1, m
            psi(1,i) = 1d0
         end do
         do j = 1, ndim
            do i = 1, m
               if (ishape==14) then
                  psi(j+1,i) = xg(i,j)-x(14,j)
               else if (ishape==16) then
                  psi(j+1,i) = xg(i,j)-x(15,j)
               else if (ishape==6) then
                  psi(j+1,i) = xg(i,j)-x(9,j)
               else if (ishape==7) then
                  psi(j+1,i) = xg(i,j)-x(7,j)
               end if
            end do
         end do
      end if

!     --- Compute pressure in each integration point
!         the current pressure is computed by p = pn + dp

      if (itype>200) then
         do i = 1, m
            p(i)  = 0d0
            pn(i) = 0d0
            do j = 1, npres
               p(i)  = p(i) + (pn_tmp(j)+pd_tmp(j))*psi(j,i)
               pn(i) = pn(i) + pn_tmp(j) * psi(j,i)
            end do
         end do
      end if

1000  call erclos ( 'el2600' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( usrvec, 3, m, 'uservec(.,1-3)' )
         write(irefwr,*) 'End el2600'

      end if  ! ( debug )

      end
