      subroutine el2603 ( phi0, phin, phi, dphidx0, dphidxn, psi, &
                          u, ug, Fd, Jd, Fn, Jn, &
                          p, p8, pn, pn8, usrvec, &
                          iseqnr, iusrvc, isequ, iseqp, npres )
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.3  date 12-12-2012 Long integers
!        version  2.2  date 28-05-2003 adaptation basis functions
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Get old displacement vector and deformation gradient for
!    general nonlinear solid. If necessary the initial stress is
!    retrieved.
! **********************************************************************
!
!                       KEYWORDS
!
!     mechanics
!     non_linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqnr, iusrvc, npres, isequ(*), iseqp(*)
      double precision phi0(m,m), phin(m,m), phi(n,m), &
                       dphidx0(m,m,ndim), dphidxn(m,m,ndim), &
                       psi(npres,m), u(n,ndim), ug(m,ndim), &
                       Fd(m,ndim,ndim), Jd(*), Fn(m,ndim,ndim), Jn(*), &
                       pn8(*), p(*), pn(*), p8(*), usrvec(m,*)

!     dphidx0        i    ?
!     dphidxn        i    ?
!     fd             o    Deformation gradient F(k,j,i) = F_ij(x^k)
!     fn             o    Deformation gradient at incr. n Fn(k,j,i) = F_ij(x^k)
!     iseqnr         i    Sequence number with respect to the storage in array
!                         ISEPIN
!     iseqp          i    Sequence for pressure dof's
!     isequ          i    Sequence for displacement dof's
!     iusrvc         i    if>0, a user defined vector is used
!     jd             o    Determinant of deformation gradient
!     jn             o    Determinant of deformation gradient at incr. n
!     npres          i    Number of pressure parameters per element
!     p              o    Pressure in nodal points
!     p8             i    ?
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phi0           i    ?
!     phin           i    ?
!     pn             i    ?
!     pn8            i    ?
!     psi            o    Array of length n x m containing the values of the
!                         modified basis functions in the integration points
!                         Basis functions for the pressure
!     u              o    Real n x ncomp array in which the values of the
!                         solution vector in the nodal points of the element are
!                         stored in the sequence:
!                         u  (x ) = u(j,i)  j = 1(1)n, i = 1 (1) ncomp
!                          i   j
!                         Array u is always filled
!     ug             o    array of length m x ncomp in which the values of
!                         u in the integration points are stored the sequence:
!                         ug (x ) = u(j,i)  j = 1(1)m, i = 1 (1) ncomp
!                           i   j
!                         ugauss is only filled if jdiag>1 in common celint
!     usrvec         i    user defined vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, idx(8), ncompsav
      double precision pd_tmp(npres), pn_tmp(npres), &
                       rdum(1), un(m,ndim)
      double precision, pointer :: vecoldlc(:)

!     i              Counting variable
!     idx            ?
!     j              Counting variable
!     ncompsav       Help variable to save the value of ncomp
!     pd_tmp         ?
!     pn_tmp         ?
!     rdum           dummy variable
!     un             normal component of velocity
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL2005         Compute a local vector and its derivatives from a known
!                    vector
!     EL2700         Compute deformation gradient
!     EL3007         Fills old solution and derivatives
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data idx / 1, 3, 7, 9, 19, 21, 25, 27 /
! ======================================================================
!
!     --- Get the displacements and derivatives on increment n
!         always jind = 0, because we use isequ to look up the
!         correct degree of freedom

      jderiv = 1
      jsol = 1
      jind = 0
      call el2005 ( un, Fn, phi0, dphidx0, isequ, ug, iseqnr+1 )

!     --- Compute deformation gradient from displacement gradient

      call el2700 (Fn, Jn)

!     --- Get the delta displacements and derivatives
!         always jind = 0, because we use isequ to look up the
!         correct degree of freedom

      jderiv = 1
      jsol = 1
      jind = 0
      call el2005 ( u, Fd, phin, dphidxn, isequ, ug, iseqnr )

!     --- Compute deformation gradient from displacement gradient

      call el2700 (Fd, Jd)

!     --- Get pressure degrees of freedom

      iseqtype = oldsols(iseqnr+1)%iseqtype
      index3n => indoldsols(iseqtype)%index3
      vecoldlc => oldsols(iseqnr+1)%oldvecs
      pn_tmp(1:npres) = vecoldlc(index3n(iseqp(1:npres)))
      iseqtype = oldsols(iseqnr)%iseqtype
      index3n => indoldsols(iseqtype)%index3
      vecoldlc => oldsols(iseqnr)%oldvecs
      pd_tmp(1:npres) = vecoldlc(index3n(iseqp(1:npres)))
!      do i = 1, npres
!         pn_tmp(i) = vecold(index3(iseqnr+1,iseqp(i)))
!         pd_tmp(i) = vecold(index3(iseqnr,iseqp(i)))
!      end do

!     --- Check whether to retrieve the user defined vector

      if (iusrvc>=1) then

         jderiv = 0
         jsol = 1
         jind = 1
         ncompsav = ncomp
         ncomp = iusrvc

!        --- only the values in the integration points are returned

         call el3007 ( usrvec, rdum, phi, rdum, m, n, &
                       ndim, iusrvc, ind, rdum, usrvec, &
                       jtrans, numold, iseqnr+2 )
         ncomp = ncompsav

      end if

!     --- Compute pressure and old pressure in each integration point
!         the current pressure is computed by p = pn + dp

      do i = 1, m
         p(i)  = 0d0
         pn(i) = 0d0
         do j = 1, npres
            p(i)  = p(i) + (pn_tmp(j)+pd_tmp(j))*psi(j,i)
            pn(i) = pn(i) + pn_tmp(j)*psi(j,i)
         end do
      end do

!     --- Compute pressure and old pressure in integration points
!         on the vertices

      if (ishape==4.or.ishape==6.or.ishape==7) then
         idx(1)=1
         idx(2)=3
         idx(3)=5
         idx(4)=7
      end if

      do i = 1, npres
         p8(i) = p(idx(i))
         pn8(i) = pn(idx(i))
      end do

      end
