      subroutine el4026(ichois,itcoor,ip,mloc,ndim, &
                        modelv,etha,cn,cl,param, &
                        xg,u,dudx,viscos,secinv,gradv,derivq)
!
! ======================================================================
!
!   programmer   jaap van der zanden
!
!   version 3.4  date   12-12-1995 (add modelv=5,6,7)
!   version 3.3  date   22-12-1992 (improve error report)
!   version 3.2  date   21-11-1992 (remove name; replace ertrap)
!
!
!   copyright (c) 1984-1995  "ingenieursbureau sepra"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "ingenieursbureau sepra".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!   DESCRIPTION
!
!     this subroutine tests and computes model-dependent
!     quantities for a generalized newtonian liquid
!                    and plastico-viscous liquids
!                    in nodal point ip (.le.m)
!     available are :  modelv=1, newtonian model
!                      modelv=2, power law model
!                      modelv=3, 3 parameter carreau model
!                      modelv=4, 3 parameter plastico-viscous liquid
!                               cn=1 : bingham model
!                               cn=2 : casson model
!                      modelv=5, Papanastasiou plastico-viscous model
!                      modelv=6, bi-viscosity plastico-viscous model
!                      modelv=7, Carreau-Yasuda model
!                      modelv=100 , user defined effective viscosity
!                                   based on gradv
!                      modelv=101 , user defined effective viscosity
!                                   based on second invariant
!                      modelv=102 , user defined effective viscosity
!                                   based on x,u,second invariant
!
!    REMARK   the order of computions is such that the gradv and derivq
!             may use the same memory locations (see the call in el4038)
!
! **********************************************************************
!
!   KEYWORDS
!
!   newtonian_fluid
!   power_law_model
!   carreau_model
!   plastico_viscous
! **********************************************************************
!
!   INPUT / OUTPUT  PARAMETERS
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
      double precision cn, cl, viscos, secinv, etha(*), param(*), &
             xg(*), u(*), dudx(*), gradv(*), derivq(*)
      integer ichois,itcoor,ip,mloc,ndim,modelv

!     cl        carreau model parameter lambda, or
!               bingham (plastico-viscous) model parameter s
!     cn        power (law model) parameter n
!     derivq    ichois=1 computed stress tensor in ip
!                     =3 elongation in ip
!                     =4 shear rate in ip (vector in 3-dim or swirl)
!     dudx      derivatives of velocity
!     etha      viscosity parameter etha0 in integration points
!     gradv     gradient of velocity tensor in ip
!               ichois=1-4  gradient of velocity is computed
!     ichois    choice parameter
!     ip        point number
!     itcoor    type of co-ordinates
!     m         number of inetgration points
!     modelv    type of viscosity model
!     ndim      dimension of space
!     param     viscosity model parameters (modelv=6,7)
!     secinv    computed second invariant of tensor a1
!     u         velocity
!     viscos    computed effective viscosity
!     xg        integration points
!
! **********************************************************************
!
!    COMMON  BLOCKS
!
! **********************************************************************
!
!   LOCAL  PARAMETERS
!
      double precision el4027,fnv000,fnv001,fnv002, &
                       u1,u2,u3,work,x1,x2,x3,xgk
      integer m2,m3,m4,m5,m6,m7,m8
      save m2,m3,m4,m5,m6,m7,m8
!
!     m2      ?
!     m3      ?
!     m4      ?
!     m5      ?
!     m6      ?
!     m7      ?
!     m8      ?
!     u1      ?
!     u2      Second component of velocity vector
!     u3      Third component of velocity vector
!     work    Work array of length m x 3, to store products of
!             alpha   x w temporarily
!             ij
!     x1      x-coordinate of first point in curve
!     x2      x-coordinate of last point in curve
!     x3      Third component of co-ordinate vector
!     xgk     ?
! **********************************************************************
!
!   SUBROUTINES  CALLED
!
!   EL4027  generalized newtonian viscosity models
!   ERCLOS  reset old name of calling tree
!   EROPEN  add name of routine to calling tree
!   ERREAL  Put real in error message
!   ERRINT  Put integer in error message
!   ERRSUB  report error
!   FNV000  used-defined viscosity (modelv=100)
!   FNV001  used-defined viscosity (modelv=101)
!   FNV002  used-defined viscosity (modelv=102)
!
! **********************************************************************
!
!   I / O
!
!   none
!
! **********************************************************************
!
!   ERROR  MESSAGES
!
! **********************************************************************
!
!   ERROR  MESSAGES
!
!   234     viscosity not positive
!
! **********************************************************************
!
!   PSEUDO  CODE
!
!   trivial
!
! ======================================================================
!
!     --- computation of grad(v) for all values of ichois

      if (ifirst.eq.0) then
         m2=mloc+mloc
         m3=m2+mloc
         if (itcoor.eq.3) then
            m4=m3+mloc
            m5=m4+mloc
         else if (ndim.eq.3) then
            m4=m3+mloc
            m5=m4+mloc
            m6=m5+mloc
            m7=m6+mloc
            m8=m7+mloc
         end if
      end if
      if ( itcoor.eq.0 .and. ndim.eq.2 ) then

!     --- 2-dim cartesian coordinates

         gradv(1)=dudx(ip)
         gradv(2)=dudx(m2+ip)
         gradv(3)=dudx(mloc+ip)
         gradv(4)=dudx(m3+ip)
         if ( ichois.eq.1 .or. ichois.eq.5 ) &
            secinv=2*(gradv(1)*gradv(1)+gradv(4)*gradv(4)) &
                     +(gradv(2)+gradv(3))*(gradv(2)+gradv(3))
      else if ( itcoor.eq.1 .and. ndim.eq.2 ) then

!     --- axisymmetrical coordinates (no swirl)

         gradv(1)=dudx(ip)
         gradv(2)=dudx(m2+ip)
         gradv(3)=dudx(mloc+ip)
         gradv(4)=dudx(m3+ip)
         xgk=xg(ip)
         if (abs(xgk).gt.epsmac) then
            gradv(5)=u(ip)/xgk
         else

!        --- for r=0 apply the div u = 0 condition

            gradv(5)=dudx(ip)
         end if
         if ( ichois.eq.1 .or. ichois.eq.5 ) &
            secinv=2*(gradv(1)*gradv(1)+gradv(4)*gradv(4) &
                     +gradv(5)*gradv(5)) &
                    +(gradv(2)+gradv(3))*(gradv(2)+gradv(3))
      else if ( itcoor.eq.2 .and. ndim.eq.2 ) then

!     --- 2-dim polar coordinates

         xgk=xg(ip)
         gradv(1)=dudx(ip)
         gradv(2)=dudx(m2+ip)
         if (abs(xgk).gt.epsmac) then
            gradv(3)=(dudx(mloc+ip)-u(mloc+ip))/xgk
            gradv(4)=(dudx(m3+ip)+u(ip))/xgk
         else

!        --- for r=0 apply the div u = 0  condition

            gradv(3)=-dudx(m2+ip)
            gradv(4)=dudx(ip)
         end if
         if ( ichois.eq.1 .or. ichois.eq.5 ) &
               secinv=2*(gradv(1)*gradv(1)+gradv(4)*gradv(4)) &
                         +(gradv(2)+gradv(3))*(gradv(2)+gradv(3))
      else if ( itcoor.eq.3 .and. ndim.eq.2 ) then

!     --- axisymmetrical coordinates (with swirl)

         xgk=xg(ip)
         gradv(1)=dudx(ip)
         gradv(2)=dudx(m4+ip)
         gradv(3)=dudx(m2+ip)
         if (abs(xgk).gt.epsmac) then
            gradv(4)=-u(m2+ip)/xgk
            gradv(5)=u(ip)/xgk
         else

!        --- for r=0 apply the div u = 0  condition

            gradv(4)=-dudx(m4+ip)
            gradv(5)=dudx(ip)
         end if
         gradv(6)=0.0d0
         gradv(7)=dudx(mloc+ip)
         gradv(8)=dudx(m5+ip)
         gradv(9)=dudx(m3+ip)
         if ( ichois.eq.1 .or. ichois.eq.5 ) &
            secinv=2*(gradv(1)*gradv(1)+gradv(5)*gradv(5) &
                     +gradv(9)*gradv(9)) &
                    +(gradv(2)+gradv(4))*(gradv(2)+gradv(4)) &
                    +(gradv(3)+gradv(7))*(gradv(3)+gradv(7)) &
                    + gradv(8)*gradv(8)
      else if ( itcoor.eq.0 .and. ndim.eq.3 ) then

!     --- 3-dim cartesian coordinates

         gradv(1)=dudx(ip)
         gradv(2)=dudx(m3+ip)
         gradv(3)=dudx(m6+ip)
         gradv(4)=dudx(mloc+ip)
         gradv(5)=dudx(m4+ip)
         gradv(6)=dudx(m7+ip)
         gradv(7)=dudx(m2+ip)
         gradv(8)=dudx(m5+ip)
         gradv(9)=dudx(m8+ip)
         if ( ichois.eq.1 .or. ichois.eq.5 ) &
            secinv=2*(gradv(1)*gradv(1)+gradv(5)*gradv(5) &
                     +gradv(9)*gradv(9)) &
                    +(gradv(2)+gradv(4))*(gradv(2)+gradv(4)) &
                    +(gradv(3)+gradv(7))*(gradv(3)+gradv(7)) &
                    +(gradv(6)+gradv(8))*(gradv(6)+gradv(8))
      else if ( itcoor.eq.4 .and. ndim.eq.3 ) then

!     --- 3-dim cylindrical coordinates

         xgk=xg(ip)
         gradv(1)=dudx(ip)
         gradv(2)=dudx(m3+ip)
         gradv(3)=dudx(m6+ip)
         if (abs(xgk).gt.epsmac) then
            gradv(4)=(dudx(mloc+ip)-u(mloc+ip))/xgk
            gradv(5)=(dudx(m4+ip)+u(ip))/xgk
            gradv(6)=dudx(m7+ip)/xgk
         else

!        --- for r=0 apply the div u = 0  condition

            gradv(4)=-dudx(m3+ip)
            gradv(5)=dudx(ip)
            gradv(6)=0.0d0
         end if
         gradv(7)=dudx(m2+ip)
         gradv(8)=dudx(m5+ip)
         gradv(9)=dudx(m8+ip)
         if ( ichois.eq.1 .or. ichois.eq.5 ) &
            secinv=2*(gradv(1)*gradv(1)+gradv(5)*gradv(5) &
                     +gradv(9)*gradv(9)) &
                    +(gradv(2)+gradv(4))*(gradv(2)+gradv(4)) &
                    +(gradv(3)+gradv(7))*(gradv(3)+gradv(7)) &
                    +(gradv(6)+gradv(8))*(gradv(6)+gradv(8))
      end if
      if (ichois.eq.1) then

!     --- computation of effective viscosity

         if (modelv.eq.1) then

!        --- newtonian shear-independent viscosity

            viscos=etha(ip)
         else if (modelv.eq.100) then
            viscos=fnv000(gradv)
            if (viscos.le.0) then
               call eropen('el4026')
               call erreal(viscos,1)
               call errint(ielem,1)
               call errint(itype,2)
               call errint(ielgrp,3)
               call errsub ( 234, 3, 1, 0)
               call erclos('el4026')
            end if
         else if (modelv.eq.101) then
            viscos=fnv001(secinv)
            if (viscos.le.0) then
               call eropen('el4026')
               call erreal(viscos,1)
               call errint(ielem,1)
               call errint(itype,2)
               call errint(ielgrp,3)
               call errsub ( 234, 3, 1, 0)
               call erclos('el4026')
            end if
         else if (modelv.eq.102) then
            x1=xg(ip)
            x2=xg(ip+mloc)
            u1=u(ip)
            u2=u(ip+mloc)
            if (ndim.eq.3) then
                x3=xg(ip+m2)
                u3=u(ip+m2)
            else if ( ndim.eq.2 .and. itcoor.eq.3 ) then
                u2=u(ip+m2)
                u3=u(ip+mloc)
            end if
            viscos=fnv002(x1,x2,x3,u1,u2,u3,secinv)
            if (viscos.le.0) then
               call eropen('el4026')
               call erreal(viscos,1)
               call errint(ielem,1)
               call errint(itype,2)
               call errint(ielgrp,3)
               call errsub ( 234, 3, 1, 0)
               call erclos('el4026')
            end if
         else if ( modelv.ge.2 .and. modelv.le.7 ) then
            work=etha(ip)
            viscos=el4027(modelv,work,cn,cl,param,secinv)
         end if

!        --- compute stress tensor in ip

         if ( (itcoor.eq.0 .or. itcoor.eq.2) .and. ndim.eq.2 ) then
            derivq(1)=2*viscos*gradv(1)
            derivq(2)=viscos*(gradv(2)+gradv(3))
            derivq(3)=2*viscos*gradv(4)
         else if ( itcoor.eq.1 .and. ndim.eq.2 ) then
            derivq(1)=2*viscos*gradv(1)
            derivq(2)=viscos*(gradv(2)+gradv(3))
            derivq(3)=2*viscos*gradv(4)
            derivq(4)=2*viscos*gradv(5)
         else if ( (itcoor.eq.3 .and. ndim.eq.2) .or. ndim.eq.3 ) then
            derivq(1)=2*viscos*gradv(1)
            derivq(2)=viscos*(gradv(2)+gradv(4))
            derivq(3)=viscos*(gradv(3)+gradv(7))
            derivq(4)=2*viscos*gradv(5)
            derivq(5)=viscos*(gradv(6)+gradv(8))
            derivq(6)=2*viscos*gradv(9)
         end if
      else if (ichois.eq.3) then

!     --- elongation

         u1=u(ip)
         u2=u(ip+mloc)
         if ( ndim.eq.2 .and. itcoor.ne.3 ) then
            if ( abs(u1).le.sqreps .and. abs(u2).le.sqreps ) then
               derivq(1)=0.0d0
            else
               derivq(1)=(u1*gradv(1)*u1 &
                         +u1*(gradv(2)+gradv(3))*u2 &
                         +u2*gradv(4)*u2)/(u1*u1+u2*u2)
            end if
         else if ( ndim.eq.3 .or. itcoor.eq.3 ) then
            if (ndim.eq.3) then
               u3=u(ip+m2)
            else if ( ndim.eq.2 .and. itcoor.eq.3 ) then
               u2=u(ip+m2)
               u3=u(ip+mloc)
            end if
            if ( abs(u1).le.sqreps .and. abs(u2).le.sqreps .and. &
                 abs(u3).le.sqreps ) then
               derivq(1)=0.0d0
            else
               derivq(1)=(u1*gradv(1)*u1 &
                         +u1*(gradv(2)+gradv(4))*u2 &
                         +u1*(gradv(3)+gradv(7))*u3 &
                         +u2*gradv(5)*u2 &
                         +u2*(gradv(6)+gradv(8))*u3 &
                         +u3*gradv(9)*u3)/(u1*u1+u2*u2+u3*u3)
            end if
         end if
      else if (ichois.eq.4) then

!     --- deformation

         if ( ndim.eq.2 .and. itcoor.ne.3 ) then
            u1=u(ip)
            u2=u(ip+mloc)
            if ( abs(u1).lt.sqreps .and. abs(u2).lt.sqreps ) then

!           --- zero velocity, take velocity in centroid for directions

               u1=u(mloc)
               u2=u(mloc+mloc)
               if ( abs(u1).lt.sqreps .and. abs(u2).lt.sqreps ) then
                  derivq(1)=0.0d0
               else
                  derivq(1)=(2*u1*u2*(-gradv(1)+gradv(4)) &
                            +(u1*u1-u2*u2)*(gradv(2)+gradv(3))) &
                            /(u1*u1+u2*u2)
               end if
            else
               derivq(1)=(2*u1*u2*(-gradv(1)+gradv(4)) &
                         +(u1*u1-u2*u2)*(gradv(2)+gradv(3))) &
                         /(u1*u1+u2*u2)
            end if
         else if ( ndim.eq.3 .or. itcoor.eq.3 ) then
            derivq(1)=gradv(2)+gradv(4)
            derivq(2)=gradv(3)+gradv(7)
            derivq(3)=gradv(6)+gradv(8)
         end if
      else if (ichois.eq.5) then
         derivq(1)=sqrt(secinv)
      end if
      end
