      subroutine el4040(ichois,itcoor,ip,mloc,ndim, &
                        modelv,etha,cn,cl,ct,temp1,visprm, &
                        xg,u,dudx,viscos,gradv,secinv,detdsc)
!
! ====================================================================
!
!        programmer   jaap van der zanden
!
!  version 1.6  date   25-01-1995 .lt.secndc --> .le.secndc
!  version 1.5  date   18-12-1995 add modelv=5,6,7
!  version 1.4  date   23-12-1992 improve err. rep.
!
!   copyright (c) 1986-1995  "ingenieursbureau sepra"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "ingenieursbureau sepra".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ********************************************************************
!
!   DESCRIPTION
!
!     this subroutine computes model-dependent viscosity
!     for a generalized newtonian liquid with thermal effects
!     in all points 1,..,m or in one point ip (.le.mloc)
!     available are :  modelv=1, newtonian model
!                      modelv=2, power law model
!                      modelv=3, 3 parameter carreau model
!                      modelv=4, 3 parameter plastico-viscous liquid
!                               cn=1 : bingham model
!                               cn=2 : casson model
!                      modelv=5, 3 parameter Papanastasiou model
!                      modelv=6, 4 parameter bi-viscosity  model
!                      modelv=7, 5 parameter Carreau-Yasuda model
!                      modelv=100 , user defined effective viscosity
!                                   based on gradv
!                      modelv=101 , user defined effective viscosity
!                                   based on second invariant
!                      modelv=102 , user defined effective viscosity
!                                   based on x,u,second invariant
!
! ********************************************************************
!
!   INPUT / OUTPUT  PARAMETERS
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
      integer ichois, itcoor, ip, mloc, ndim, modelv
      double precision cn, cl, ct, temp1
      double precision etha(*), visprm(*), xg(*), u(*), dudx(*), &
             viscos(*), gradv(*), secinv(*), detdsc(*)

!     cl         i    carreau model parameter lambda, or
!                     bingham (plastico-viscous) model parameter s
!     cn         i    power (law model) parameter n
!     ct         i    coefficient for temperature dependence
!     detdsc     i
!     dudx       i    Real m x ndim  array in which the values of the
!                     derivatives of the solution vector in the integration
!                     points are stored in the sequence:
!                     du / dx  (x ) = dudx ( k, j )
!                       j     k
!                     If jdercn = 0 in common block celint, then only
!                     dudx (1,*) is filled because dudx is then a constant
!                     per element
!                     dudx is only filled if jderiv > 0
!     etha      i    viscosity parameter etha0 in integration points
!     gradv     i    gradient of velocity tensor in ip
!     ichois    i    =0  computations of viscos in all points 1,..,m
!                    =1  computations in ip only, but also gradv in ip
!                    =2  derivative of viscos with respect to secinv
!                         (for modelv=2)
!                        9 < ichois < 20 : like ichois-10
!                                          compute gradv, secinv, viscos
!                        9 < ichois < 30 : like ichois-20
!                                          compute gradv and secinv, not viscos
!     ip         i    nodal point numbers of line to be considered
!     itcoor     i
!     m          i    number of integration points
!     modelv     i    Defines the type of viscosity model
!                     Possible values:
!                       1:  newtonian liquid
!                       2:  power law liquid
!                       3:  carreau liquid
!                       4:  plastico viscous liquid
!                     100:  user provided model depending on grad u
!                     101:  user provided model depending on the second inv
!                     102:  user provided model depending on the second inv
!                           the co-ordinates and the velocity
!     ndim       i    Dimension of the space.
!     secinv     i    computed second invariant of tensor a1
!     temp1      i    reference temperature
!     u
!     viscos     o    computed effective viscosity
!     visprm     i    viscosity parameters (modelv=6,7)
!     xg         i    Array of length 4x2 containing the co-ordinates of the
!                     Gauss points in the reference element for a four point
!                     Gauss rule
! ********************************************************************
!
!   COMMON    BLOCKS
!
! **********************************************************************
!
!   ERROR MESSAGES
!
!     234   Computed effective viscosity is less than or equal to zero
!
! **********************************************************************
!
!   SUBROUTINES  CALLED
!
!     EL4039    Compute gradient of velocity, second invariant
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERREAL    Put real in error message
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     FNVT00    User-defined viscosity, based on grad v, Temp
!     FNVT01    User-defined viscosity, based on II    , Temp
!     FNVT02    User-defined viscosity, based on x,y,z,u,v,w,II,Temp
! **********************************************************************
!
!   LOCAL  PARAMETERS
!
!
      integer ich39, ip1, ip2, ipj, mv
      double precision expn, fnvt00, fnvt01, fnvt02, sech, temp, term, &
                       u1, u2, u3, x1, x2, x3, &
                       factor, secndc

!     expn       Value of an exponent
!     factor     Multiplication factor
!     ich39     ?
!     ip1       ?
!     ip2       ?
!     ipj       ?
!     mv        ?
!     sech      ?
!     secndc    ?
!     temp      ?
!     term       Value of a term in an expression
!     u1        ?
!     u2         Second component of velocity vector
!     u3         Third component of velocity vector
!     x1         x-coordinate of first point in curve
!     x2         x-coordinate of last point in curve
!     x3         Third component of co-ordinate vector
! **********************************************************************
!
!   PSEUDO CODE
! ====================================================================
!
!     --- computation of grad(v) for all values of ichois

      if (ndim.eq.2) then
         if (itcoor.eq.3) then
            mv=mloc+mloc+mloc
         else
            mv=mloc+mloc
         end if
      else if (ndim.eq.3) then
         mv=mloc+mloc+mloc
      end if
      ich39=1
      if (((modelv.eq.1).or.(modelv.eq.100)).and.(ichois.lt.10)) ich39=0
      if ((ichois.eq.1).or.(ichois.eq.11).or.(ichois.eq.21)) then
         if ((ip.lt.1).or.(ip.gt.mloc)) return
         ip1=ip
         ip2=ip
      else
         ip1=1
         ip2=mloc
      end if
      do 10 ipj=ip1,ip2
         temp=u(mv+ipj)
         if ((modelv.ne.1).or.(ichois.ge.10)) then
            call el4039(ich39,itcoor,ipj,mloc,ndim,xg,u,dudx, &
                     sech,gradv)
            secinv(ipj)=sech
         end if
         if (ichois.ge.20) return
         if (modelv.eq.1) then
            viscos(ipj)=etha(ipj)*exp(-ct*(temp-temp1))
         else if (modelv.eq.2) then
            expn=(cn-1.0d0)/2.0d0
            if (expn.le.0) then
               if (secinv(ipj).lt.epsmac) secinv(ipj)=epsmac
            end if
            viscos(ipj)=etha(ipj)*exp(-ct*(temp-temp1)) &
                       *(secinv(ipj)**expn)
            if (ichois.eq.2) then
               expn=(cn-3.0d0)/2.0d0
               if (expn.le.0) then
                  if (secinv(ipj).lt.epsmac) secinv(ipj)=epsmac
               end if
               detdsc(ipj)=etha(ipj)*exp(-ct*(temp-temp1)) &
                          *(cn-1.0d0)*(secinv(ipj)**expn)
            end if
         else if (modelv.eq.3) then
            viscos(ipj)=etha(ipj)*exp(-ct*(temp-temp1))* &
                        ((1.0d0+cl*secinv(ipj))**((cn-1.0d0)/2.0d0))
         else if (modelv.eq.4) then

!        --- 3 parameter plastico-viscous liquid

            expn=0.5d0/cn
            factor=1.0d7

!           --- Below a critical value of the rate of deformation a Newtonian
!               approximation is used with etha_0 = factor * etha_pv
!               according to:
!               C.R. Beverly, R.I. Tanner, &
!               Numerical analysis of three-dimensional Bingham plastic flow, &
!               J. non-Newt. Fluid Mech., vol 42 (1992) 85-115
!               A factor of 1000 appears to be optimal for channel and pipe
!               flow.
!               (this is confirmed by my test-problems)
!               A factor of 10000 appears to be better for outflow in open
!               vessels
!               especially for large values of the yiels stress.
!               For an ideal Bingham model the factor is as high as possible
!               secndc is the square of the critical rate of deformation where
!               the stress t=etha_0 (grad v + (grad v)^T) is equal to the stress
!               according to the model

            secndc=( cl/(etha(ipj)*(factor**(1d0/cn)-1d0)**cn) )**2
            if (secinv(ipj).le.secndc) then
               viscos(ipj)=factor*etha(ipj)*exp(-ct*(temp-temp1))
            else
               term=(cl*cl/(etha(ipj)*etha(ipj)*secinv(ipj)))**expn
               viscos(ipj)=etha(ipj)*exp(-ct*(temp-temp1))* &
                           (1.0d0+term)**cn
            end if
         else if (modelv.eq.5) then

!        --- 3 parameter Papanastasiou plastico-viscous liquid
!            T.C. Papanastasiou, Flow of materials with yield, &
!            J. Rheology, vol 31, pp 385-404 (1987).

            term=sqrt(secinv(ipj))
            if (term.gt.epsmac) then
               viscos(ipj)=(etha(ipj)+cl*(1.d0-exp(-cn*term))/term)* &
                            exp(-ct*(temp-temp1))
            else
               viscos(ipj)=(etha(ipj)+cl*cn)*exp(-ct*(temp-temp1))
            end if
         else if (modelv.eq.6) then

!        --- 4 parameter bi-viscosity plastico-viscous model

            expn=0.5d0/cn
            factor=visprm(1)
            secndc=( cl/(etha(ipj) * (factor**(1d0/cn)-1d0)**cn) )**2
            if (secinv(ipj).le.secndc) then
               viscos(ipj)=factor*etha(ipj)*exp(-ct*(temp-temp1))
            else
               term=(cl*cl/(etha(ipj)*etha(ipj)*secinv(ipj)))**expn
               viscos(ipj)=etha(ipj)*exp(-ct*(temp-temp1)) &
                            *(1.0d0+term)**cn
            end if
         else if (modelv.eq.7) then

!        --- 5 parameter Carreau-Yasuda model
!              see R.B. Bird, R.C. Armstrong, O. Hassager, &
!              Dynamics of polymeric liquids, Vol 1, &
!              2^nd ed., Wiley, 1987, pp 171-172.

            expn=(cn-1.0d0)/visprm(2)
            term=(1.0d0+(cl*sqrt(secinv(ipj)))**visprm(2))**expn
            viscos(ipj)=(visprm(1)+(etha(ipj)-visprm(1))*term)* &
                         exp(-ct*(temp-temp1))
         else if (modelv.eq.100) then
            viscos(ipj)=fnvt00(gradv,temp)
         else if (modelv.eq.101) then
            viscos(ipj)=fnvt01(secinv(ipj),temp)
         else if (modelv.eq.102) then
            x1=xg(ipj)
            x2=xg(ipj+mloc)
            if (ndim.eq.3) x3=xg(ipj+mloc+mloc)
            u1=u(ipj)
            u2=u(ipj+mloc)
            if ((ndim.eq.3).or.(itcoor.eq.3)) u3=u(ipj+mloc+mloc)
            viscos(ipj)=fnvt02(x1,x2,x3,u1,u2,u3,secinv(ipj),temp)
         end if
         if (viscos(ipj).le.0) then
            call eropen('el4040')
            call erreal(viscos(ipj),1)
            call errint(ielem,1)
            call errint(itype,2)
            call errint(ielgrp,3)
            call errsub ( 234, 3, 1, 0)
            call erclos('el4040')
         end if
10    continue
      end
