      subroutine el4057 ( elemvc, sp, w, x, xgauss, eps, u, p, icheld )
! ======================================================================
!
!        programmer    Guus Segal/ Jaap van der Zanden
!        version  3.3  date 29-09-2000 Adapt array bound
!        version  3.2  date 14-03-1999 Use integration points
!        version  3.1  date 05-02-1996 Extension for trilinear hexahedron
!        version  3.0  date 09-01-1996 Extension with parameter icheld
!        version  2.3  date 28-12-1995 Extension for 5-node elements
!        version  2.2  date 20-08-1994 Extension for 3D
!        version  2.1  date 16-03-1994 Extension for 2D
!        version  2.0  date 16-07-1993 Complete revision
!        version  1.1  date 28-10-1991 add ip for choice of dpdx comp.
!        version  1.0  date 28-10-1991 replace el4034,el4035 for 3-dim brick
!
!   copyright (c) 1991-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the pressure for the incompressible Navier-Stokes equations
!     in the nodal points and store in array elemvc
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision w(m), sp(ndimlc*n,*), u(n,ndimlc), &
                       xgauss(m,*), x(n,*), eps, p(*), &
                       elemvc(*)
      integer icheld

!     elemvc  i    Element vector
!     eps     i    Parameter epsilon for the penalty method
!     icheld  i    Choice parameter to distinguish between the various
!                  possibilities of eld900
!     p      i/o   Array containing the computed pressure
!                  p(1) contains the pressure in the centroid
!                  p(2-4) contains the gradient of the pressure in the centroid
!                  If itype=902 p should have been filled before
!     sp      i    Divergence matrix
!                  SP(i,k) corresponds to the k-th row and i-th column
!                  of the matrix, The k refers to the basis functions of the
!                  pressure, i to the basis functions of the velocity in the
!                  sequence of the solution vector
!     u       i    array of length m containing the values of the
!                  array of length m x ncomp in which the values of
!                  u in the integration points are stored the sequence:
!                  u  (x ) = u(j,i)  j = 1(1)m, i = 1 (1) ncomp
!                   i   j
!     w       i    array of length m containing the weights for integration
!     x       i    array containing the coordinates of the nodal points
!                  according to:
!                  x  = x (i,1);  y  = x (i,2);  z  = x (i,3)
!                   i              i              i
!     xgauss  i    Double precision array of length m x ndimlc in which the
!                  co-ordinates of the integration points are stored
!                  according to:
!                  x  = x (i,1);  y  = x (i,2);  z  = x (i,3)
!                   i              i              i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, icent, npres, ipres
      double precision diag(4), sum

!     diag    Diagonal of pressure mass matrix
!     i       Counting variable
!     icent   Index of centroid
!     ipres   Extra pressure points
!     j       Counting variable
!     k       Counting variable
!     npres   Number of pressure parameters per element
!     sum     Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The Navier-Stokes matrix has the shape:
!            |               T   |
!            |   S        -SP    |    u
!     S   =  |    uu             |
!            |                   |
!            |   SP         0    |    p
!     In the penalty method the zero matrix is replaced by eps M
!                     T
!     Hence Suu u - SP p = f
!           SP  u        = 0
!     is approximated by
!           Suu u -    SP p  = f
!           SP  u + eps M p  = 0
!                       -1
!    or:  p = - 1/eps  M   SP u
!    The matrix M is exactly the same as the one used in the building of the
!    element matrix
!    The pressure is equal to the pressure and the gradient of the pressure
!    in the centroid.
!    It must be evaluated in the nodal points by a linear polynomial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      ipres = 0
      if ( ndimlc.eq.2 ) then
         icent = n
         if ( n.eq.4 .and. inpelm.eq.5 ) ipres = 1
         if ( n.eq.4 ) then
            npres = 1
         else
            npres = ndimlc+1
         end if
      else
         icent = 14
         if ( n.eq.8 ) then
            npres = 1
         else
            npres = ndimlc+1
         end if
      end if
      if ( itype.le.900 ) then

!     --- Compute the diagonal of the mass matrix

         do i = 1, npres
            diag(i) = 0d0
         end do
         do k = 1, m
            diag(1) = diag(1) + w(k)
         end do
         do i = 1, npres-1
            do k = 1, m
               diag(i+1) = diag(i+1) + w(k)*(xgauss(k,i)-x(icent,i))**2
            end do
         end do

!        --- store the inverse divided by eps in diag

         do i = 1, npres
            diag(i) = 1d0 / ( diag(i) * eps )
         end do

!        --- multiply the velocity by the divergence matrix and minus the
!            inverse of the mass matrix divided by eps

         do k = 1, npres
            sum = 0d0
            do j = 1, ndimlc
               do i = 1, n
                  sum = sum + sp((i-1)*ndimlc+j,k) * u(i,j)
               end do
            end do
            p(k) = - diag(k) * sum
         end do

      end if

!     --- Compute the pressure in the nodal points

      if ( icheld.eq.20 ) then

!     --- icheld = 20, special case

         elemvc(1) = p(1)

      else if ( icheld.le.42 ) then

!     --- icheld < 43, standard case

         if ( ndimlc.eq.2 ) then

!        --- ndim = 2

            if ( npres.eq.1 ) then
               do i = 1, n+ipres
                  elemvc(i) = p(1)
               end do
            else
               do i = 1, n
                  elemvc(i) = p(1) + p(2)*(x(i,1)-x(icent,1)) + &
                                     p(3)*(x(i,2)-x(icent,2))
               end do
            end if

         else

!        --- ndim = 3

            if ( npres.eq.1 ) then
               do i = 1, n
                  elemvc(i) = p(1)
               end do
            else
               do i = 1, n
                  elemvc(i) = p(1) + p(2)*(x(i,1)-x(icent,1)) + &
                                     p(3)*(x(i,2)-x(icent,2)) + &
                                     p(4)*(x(i,3)-x(icent,3))
               end do
            end if

         end if

      else

!     --- icheld > 42, fill per integration point

         if ( ndimlc.eq.2 ) then

!        --- ndim = 2

            if ( npres.eq.1 ) then
               do i = 1, m
                  elemvc(i) = p(1)
               end do
            else
               do i = 1, m
                  elemvc(i) = p(1) + p(2)*(xgauss(i,1)-x(icent,1)) + &
                                     p(3)*(xgauss(i,2)-x(icent,2))
               end do
            end if

         else

!        --- ndim = 3

            if ( npres.eq.1 ) then
               do i = 1, m
                  elemvc(i) = p(1)
               end do
            else
               do i = 1, m
                  elemvc(i) = p(1) + p(2)*(xgauss(i,1)-x(icent,1)) + &
                                     p(3)*(xgauss(i,2)-x(icent,2)) + &
                                     p(4)*(xgauss(i,3)-x(icent,3))
               end do
            end if

         end if

      end if

      end
