      subroutine el4402( u, dudx, phi, dphidx, mloc, nloc, &
                     ndimloc, ncomploc, indlc, trans, ugauss, &
                     jtranslc, numoldlc, numveclc, nunkp, &
                     jdercnlc,jderivlc,jdiaglc,jindlc,jsollc )
! ======================================================================
!
!        programmer    Jaap van der Zanden / Guus Segal
!        version  2.2  date 12-12-2012 Long integers
!        version  2.1  date 4-12-1992 order of decl.
!        version  2.0  date 28-03-1992 ncomploc=velocity part, nunkp=vector
!
!   copyright (c) 1991-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!Compute a local vector and its derivatives from a known vector.
!  the values are evaluated in the integration points or in
!  the nodal points
!  el4402 is called by several types of element subroutines
!     It must be preceded by ELP subroutines of the 6-series
!     although it still works for the 2-series
!     (jsollc=-1,jderivlc=-1)
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mloc, nloc, ndimloc, ncomploc, numoldlc, nunkp, &
          indlc(nunkp,nloc), numveclc, jtranslc, jdercnlc, jderivlc, &
          jdiaglc, jindlc, jsollc
      double precision  u(nloc,nunkp), phi(nloc,mloc), &
                    dudx(m,ndimloc,nunkp), dphidx(nloc,mloc, &
                    ndimloc), trans(n-jtranslc,ncomploc*ncomploc), &
                    ugauss(m,nunkp)

!  u    i/o   Real n x nunkp array in which the values of the solution
!             vector in the nodal points of the element are stored in
!             the sequence:
!             u  (x ) = u(j,i)  j = 1(1)n, i = 1 (1) nunkp
!              i   j
!             Array u is always filled, unless numveclc=0
!  dudx   o   Real m x ndimloc x nunkp array in which the values of the
!             derivatives of the solution vector in the integration
!             points are stored in the sequence:
!             du / dx  (x ) = dudx ( k, j, i )
!               i    j   k
!             If jdercnlc = 0 , then only
!             dudx (1,*,*) is filled because dudx is then a constant
!             per element
!             dudx is only filled if jderivlc/=0
!  phi  i     Array of length n x m containing the values of the basis
!             functions in the integration points if a Gauss rule is
!             used.
!             phi  (xg ) = phi (i,k)
!                i    k
!  dphidx   i   Array of length n x m x 3  containing the derivatives
!             of the basis functions in the sequence:
!             d phi / dx (xg ) = dphidx (i,k,1);
!                  i        k
!             d phi / dy (xg ) = dphidx (i,k,2);
!                  i        k
!             d phi / dz (xg ) = dphidx (i,k,3);
!                  i        k
!             If the element is a linear tetrahedron (n=4), then the
!             derivatives are constant and only k = 1 is filled.
!  mloc i     Number of integration points
!  nloc i     Number of basis functions
!  ndimloc   i   Dimension of the space. Indicates the number of
!             derivatives of the basis functions implicitly
!  ncomploc  i   Number of velocity components of the solution vector u
!  indlc  i     Index array that is used if jindlc<=0, to store the
!             way the vector u must be extracted from vecold. See jindlc
!  trans  i   Array of length nloc-jtranslc x ncomploc**2 to store the
!             transformation matrix with which the n - jtranslc to n
!             components of the vector u must be computed. Is only
!             used if jtranslc>0
!  ugauss  o   array of length m x nunkp in which the values of
!             u in the integration points are stored the sequence:
!             u  (x ) = u(j,i)  j = 1(1)m, i = 1 (1) nunkp
!              i   j
!             ugauss is only filled if jdiaglc>1
!  jtranslc  i  Indication if the n   point is computed with the aid
!             of array TRANS (1) (ndimloc = 2 or 3 only) or not (0)
!  numoldlc  i  Number of old vectors that are stored in vecold
!  numveclc  i  The value of numveclc indicates which of the solution
!             vectors must be used (0<=numveclc<=numoldlc)
!             If numveclc = 0, then u is already filled.
!  nunkp   i  Maximal number of degrees of freedom per point
!  jdercnlc  i  Indication if the derivatives of the basis functions
!             are constant (jdercnlc=0) or variable (jdercnlc=1)
!  jderivlc  i  Indication if the value of the derivatives must be
!             computed.  Possibilities:
!              0   No derivatives
!              1   The gradient is computed in all integration points
!                  in the sequence given by dudx
!              2   dudx in the vertices
!              3   dudy in the vertices
!              4   dudz in the vertices
!              5   The gradient is computed in the vertices
!                  in the sequence given by dudx
!              6   dudx in all nodes
!              7   dudy in all nodes
!              8   dudz in all nodes
!              jderivlc=-jderivlc:  elp of the 2-series have been used
!  jdiaglc   i   Indication if the basis functions satisfy:
!                      k                                         k
!              phi i (x ) = delta    for each integration point x
!                                ik
!              True:  jdiaglc = 1,  false jdiaglc = 2
!  jindlc  i     indication of how the local vector u must be found
!              from vecold. possibilities:
!              0:  the value of u in the nodal points must be computed
!                  with the aid of array ind of length n x nunkp
!             -1:  in each nodal point there are the same number of
!                  components of u. the components ind(1), ind(2), ...
!                  ind(nunkp) are stored
!             >0:  in each nodal point there are the same number of
!                  components of u. the components jindlc, jindlc+1,...
!                  jindlc+nunkp-1 must be stored in u.
!  jsollc  i     Indication if the values of the local vector in the nodal
!              points or the integration points must be computed (1)
!              or not (0)
!              jsollc=-1 : elp of the 2-series have been used.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: vecoldlc(:)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DDOT   (CDOT)     Inner product of two vectors
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- LOCAL  VARIABLES
!          i, j, k, l    Counting variables
!          lstep         Step variable for vertices. Usually lstep = 1, &
!                        but when the solution must be computed in the vertices
!                        and the element is quadratic then lstep = 2
!          lsub          Under index for the direction of the derivative to be
!                        computed
!          lsup          Upper index for the direction of the derivative to be
!                        computed

      integer i, j, k, l, lstep, lsub, lsup
      integer kderiv
      save lstep, lsub, lsup
      double precision ddot

!     --- ====================================================================
!              first set some constants

      kderiv = abs(jderivlc)
      if ( kderiv==1 .or. kderiv==5 ) then

!     --- kderiv = 1 or 5:  The gradient must be computed
!                           Components 1 to ndimloc

         lsub=1
         lsup=ndimloc
      else if ( kderiv>1 ) then

!     --- kderiv 2,3,4, 6, 7 or 8:  Component kderiv-1 must be computed

         lsub=kderiv-1
         if ( kderiv>5 ) lsub = lsub-4
         lsup=lsub
      end if
      if ( kderiv>1 .and. kderiv<6 .and. ndimloc==2 .and. n>=6 ) &
           then

!     --- kderiv>1, hence computation in vertices
!                     Furthermore quadratic 2D element
!                     Skip points between

         lstep=2
      else
         lstep=1
      end if
      if ( kderiv>1 .and. ndimloc==3 .and. (n/=4 .and. n/=8) ) &
         then
            write(irefwr,10)
 10         format(' This possibility has not yet been implemented', &
                   ' in el4402')
      end if

!     --- Fill array u with solution in nodes

      iseqtype = oldsols(numveclc)%iseqtype

      index3n => indoldsols(iseqtype)%index3
      vecoldlc => oldsols(numveclc)%oldvecs

      if ( jindlc==0 .and. numveclc>0 ) then

!     --- jindlc = 0, use array ind to find unknowns

         do j = 1, nunkp
         do i = 1, nloc - jtranslc
            u ( i,j ) = vecoldlc(index3n( indlc ( j,i ) ) )
         end do
         end do
      else if ( jindlc==-1 .and. numveclc>0 ) then

!     --- jindlc = -1, use array indlc partly to find unknowns

         if ( nunkp==1 ) then

!        --- nunkp = 1

            do i = 1, nloc - jtranslc
               u ( i,1 ) = vecoldlc(index3n( indlc (1,1)+i-1 ) )
            end do
         else if ( nunkp >0 ) then

!        --- nunkp>1

            do j = 1, nunkp
            do i = 1, nloc - jtranslc
               u ( i,j ) = vecoldlc(index3n(indlc (j,1)-nunkp + i*nunkp ) )
            end do
            end do
         end if
      else if (numveclc>0) then

!     --- jindlc = >0, use jindlc to find unknowns

         if ( nunkp==1 ) then

!        --- nunkp = 1

            do i = 1, nloc - jtranslc
               u ( i,1 ) = vecoldlc(index3n( i ) )
            end do
         else if ( nunkp >0 ) then

!        --- nunkp>1

            do j = 1, nunkp
            do i = 1, nloc - jtranslc
               u ( i,j ) = vecoldlc(index3n(jindlc-1+j-nunkp + i*nunkp ) )
            end do
            end do
         end if
      end if
      if ( jtranslc==1 ) then

!     --- jtranslc = 1  Compute extra unknowns by transformation
!                  nloc-1
!         u (n) =  sum  trans (l)  u (l)
!          k       l=1       k      k

         do j = 1, ncomploc
            u ( nloc,j ) = 0d0
            do i = 1, ncomploc
               u(nloc,j) = u(nloc,j) + ddot ( nloc-jtranslc, &
                        trans(1,i+(j-1)*ncomploc), 1, u(1,i), 1 )
            end do
         end do
      end if
      if ( jsollc==1 .and. jdiaglc/=1 ) then

!     --- Compute solution in Gauss points
!                   n
!         ug (k) = sum   u (i) phi (k)
!           l      i=1    l       i

         do j = 1, nunkp
            do i = 1, nloc
               ugauss ( i,j ) = ddot ( nloc, u(1,j), 1, phi(1,i), 1 )
            end do
         end do
      else if ( jsollc==-1 .and. jdiaglc/=1 ) then

!     --- Compute solution in Gauss points
!         Temporary situation, especcially for old versions of ELP subroutines, &
!         where the sequence of derivatives of basis functions and basis
!         functions itself differs from the new storage
!                         n
!               ug (k) = sum   u (i) phi (k)
!                 l      i=1    l       i

         do j = 1, nunkp
            do i = 1, mloc
               ugauss ( i,j ) = ddot ( nloc, u(1,j), 1, phi(i,1), mloc )
            end do
         end do
      else if ( jsollc/=0 ) then

!     --- jsollc = 1,  Copy u into ugauss

         do j = 1, nunkp
            do i = 1, nloc
               ugauss(i,j) = u(i,j)
            end do
         end do
      end if
      if ( jderivlc>0 ) then

!     --- jderivlc>0,  compute derivatives

         if ( jdercnlc==0 ) then

!        --- Derivatives of basis functions are constant, hence dudx is constant
!                               n
!                     du/dx =  sum  u   d phi  / dx
!                          k   i=1   i       i     k

            do j = 1, nunkp
               do k = lsub, lsup
                  dudx ( 1, k+1-lsub, j ) = ddot ( nloc, u(1,j), 1, &
                                            dphidx(1,1,k), 1 )
               end do
            end do
         else

!        --- dudx is variable

            do j = 1, nunkp
               do k = lsub, lsup
                  do l = 1, mloc, lstep
                     dudx ( l, k+1-lsub, j ) = ddot ( nloc, u(1,j), 1, &
                                               dphidx(1,l,k), 1 )
                  end do
               end do
            end do
         end if
      else if ( jderivlc<0 ) then

!     --- jderivlc<0,  compute derivatives in a special way
!         This possibility is temporarily and is only meant to be used in
!         combination with old basis functions subroutines elp...
!         In these subroutines the sequence of dphidx is different from the
!         new one in the sense that we have:
!               dphidx ( k,i,l );  k=1(1)m, i=1(1)n, l=1(1)ndimloc
!               This possibility works only in the case m=n!!!

         do j = 1, nunkp
            do k = lsub, lsup
               do l = 1, mloc, lstep
                  dudx ( l, k+1-lsub, j ) = ddot ( nloc, u(1,j), 1, &
                                            dphidx(l,1,k), mloc )
               end do
            end do
         end do
      end if
      end
