      subroutine el8212 (f, w, s, dpsidx, Fd, Jd, p, pn, work, ncoef, &
                         npres)
! ======================================================================
!
!        programmer    Raoul van Loon/Chris van Oijen
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.1  date 28-05-2003 extension ndim
!        version  1.0  date 29-05-2000
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the part of the element matrix, that is correlated to
!     the fluid flux out and into the element and its right hand side
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncoef, npres
      double precision f(npres), s(npres,npres), w(*), &
                       dpsidx(npres,m,ndim), Fd(m,ndim,ndim), Jd(*), &
                       p(npres), pn(npres), work(*)

!     dpsidx         i    Array of length 8 x m x 3  containing the derivatives
!                         of the linear basis functions in the sequence:
!                         d psi / dx (xg ) = psix (i,k,1);
!                                 i        k
!                         d psi / dy (xg ) = psix (i,k,2);
!                                 i        k
!                         d psi / dz (xg ) = psix (i,k,3);
!                                 i        k
!     f              i    Deformation gradient in integration points
!     fd             i    Deformation gradient F(k,j,i) = F_ij(x^k)
!     jd             i    Determinant of deformation gradient
!     ncoef          i    Number of coefficients in the equation
!     npres          i    Number of pressure parameters per element
!     p              i    Array containing the pressure
!                         p(1) contains the pressure in the centroid
!                         p(2-4) contains the gradient of the pressure in the
!                         centroid
!     pn             i    ?
!     s              i    Element matrix to be filled
!     w              i    array of length m containing the weights for
!                         integration
!     work           i    Work array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, l, q, r, maxcoef
      parameter (maxcoef=50)
      double precision f1(npres), f2(npres), S_wrk(npres,npres), &
                       coef(maxcoef), &
                       delta_t, theta, K_wrk(ndim,ndim)

!     coef           element coefficients
!     delta_t        ?
!     f1
!     f2             ?
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     k_wrk          ?
!     l              General loop variable
!     maxcoef        ?
!     q              ?
!     r              ?
!     s_wrk          ?
!     theta          Constant, responsible for the extent to which
!                    the old solution is taken into account
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Clear the matrices

      do i = 1, npres
         f1(i) = 0d0
         f2(i) = 0d0
      end do
      do i = 1, npres
         do j = 1, npres
            S_wrk(i,j) = 0d0
         end do
      end do
      do i = 1, ndim
         do j = 1, ndim
            K_wrk(i,j) = 0d0
         end do
      end do

!     --- First term     T
!                      Bp  * K * Bp * p

      do i = 1, npres
         do j = 1, npres
            f1(i) = f1(i) + S(i,j)*p(j)
         end do
      end do

!     --- Second term    T            T
!                      Bp  * F * K * F * Bp * (1/J)

!         --- Loop over the integration points

      do k = 1, m

!     --- get coefficients

         do i = 1, ncoef
            coef(i) = work(1+m*(i-1))
         end do
         delta_t = coef(26-5)
         theta = coef(27-5)

!        --- get K-matrix

         do i = 1, ndim
            K_wrk(i,i) = coef(20-5+i-1)
         end do

         if (ndim.eq.2) then

           K_wrk(1,2) = coef(23-5)
           K_wrk(2,1) = coef(23-5)

         else if (ndim.eq.3) then

           K_wrk(1,2) = coef(23-5)
           K_wrk(2,3) = coef(24-5)
           K_wrk(3,1) = coef(25-5)
           K_wrk(2,1) = coef(23-5)
           K_wrk(3,2) = coef(24-5)
           K_wrk(1,3) = coef(25-5)

         end if

!        --- assemble second term

         do i = 1, npres
            do j = 1, npres
               do l = 1, ndim
                  do q = 1, ndim
                     do r = 1, ndim
                        S_wrk(i,j) = S_wrk(i,j) + (1d0-theta)* &
                                     w(k)*(1d0/Jd(k))* &
                                     dpsidx(i,k,l)*Fd(k,l,q)*K_wrk(q,q)* &
                                     Fd(k,r,q)*dpsidx(j,k,r)*delta_t
                     end do
                  end do
               end do
            end do
         end do
      end do

      do i = 1, npres
         do j = 1, npres
            f2(i) = f2(i) + S_wrk(i,j)*pn(j)
         end do
      end do

!     --- Assemble total vector

      do i = 1, npres
         f(i) = f1(i) + f2(i)
      end do

      end
