      subroutine el915mat ( u, p, mloc, nloc, tang, elemmt, phi, dphidx, &
                            rho, mu, sigma, w, dudx )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-09-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute element matrix for quadratic line element corresponding to
!     type 915: element for update of free boundary according to the
!     method described in
!     N.P. Kruyt, C. Cuvelier, A. Segal, J. van der Zanden, &
!     A total linearisation method for solving viscous free boundary flow
!     problems by the finite element method,<BR>
!     Int. J. for Num. Methods in Fluids, Vol. 8, pp. 351-363, 1988
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     element_matrix
!     free_boundary
!     free_surface
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mloc, nloc
      double precision u(mloc,2), p(mloc), tang(mloc,2), elemmt(icount,icount), &
                       phi(nloc,mloc), dphidx(nloc,mloc), rho(mloc), mu(mloc), sigma(mloc), &
                       w(mloc), dudx(mloc,2)

!     dphidx         i    Array of length n x m x ndim  containing the
!     derivative
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dudx           i    Real m x ndim  array in which the values of the
!                         derivatives of the solution vector in the integration
!                         points are stored in the sequence:
!                         du / dx  (x ) = dudx ( k, j )
!                           j     k
!                         If jdercn = 0 in common block celint, then only
!                         dudx (1,*) is filled because dudx is then a constant
!                         per element
!                         dudx is only filled if jderiv > 0
!     elemmt         o    Element matrix
!     m              i    number of integration points
!     mu             i    viscosity
!     n              i    number of basis functions for each component
!     p              i    Array containing the pressure in the integration
!     node
!     phi            i    array of length n * mloc, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     rho            i    density rho
!     sigma          i    surface tension in integration points
!     tang           i    In this array the tangential vector is stored
!                         tang(j,i) contains the j th component of the
!                         tangential vector in the i-th integration point
!     u              i    contains the velocity in the integration points
!                         u(i,k): i-th component, k-th integration point
!     w              i    array of length m containing the weights for
!                         integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, l, i, j
      double precision utang(mloc), work(mloc), norm(mloc,2), sum

!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     l              General loop variable
!     norm           contains the normal
!     sum            Help parameter to compute a sum
!     utang          tangential velocity in integration points
!     work           Work array of length m
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute tangential velocity
!     Compute norm
!     Compute element matrix corresponding to density
!     Compute element matrix corresponding to surface tension
!     Compute element matrix corresponding to viscosity
!     Compute element matrix corresponding to normal
!     Compute element matrix corresponding to u_t
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el915mat' )

!     --- Compute tangential velocity

      do k = 1, mloc
         utang(k) = u(k,1)*tang(k,1)+u(k,2)*tang(k,2)
      end do

!     --- Compute norm

      do k = 1, 3
         norm(k,1) = -tang(k,2)
         norm(k,2) =  tang(k,1)
      end do

!     --- Compute element matrix corresponding to density

      do l = 1, 2
         do k = 1, 3
            work(k) = rho(k)*utang(k)*dudx(k,l)
         end do
         do i = 1, 3
            do j = 1, 3
               sum = 0d0
               do k = 1, 3
                  sum = sum + w(k)*phi(i,k)*phi(j,k)*work(k)
               end do
               elemmt(3*j,l+3*(i-1)) = sum
            end do
         end do
      end do

!     --- Compute element matrix corresponding to surface tension

      do l = 1, 2
         do i = 1, 3
            do j = 1, 3
               sum = 0d0
               do k = 1, 3
                  sum = sum - &
                        w(k)*norm(k,l)*dphidx(i,k)*dphidx(j,k)*sigma(k)
               end do
               elemmt(3*j,l+3*(i-1)) = elemmt(3*j,l+3*(i-1))+sum
            end do
         end do
      end do

!     --- Compute element matrix corresponding to viscosity

      do k = 1, 3
         work(k) = -p(k)+2d0*mu(k)* &
                   (tang(k,1)*dudx(k,1)+tang(k,2)*dudx(k,2))
      end do
      do l = 1, 2
         do i = 1, 3
            do j = 1, 3
               sum = 0d0
               do k = 1, 3
                  sum = sum+w(k)*dphidx(i,k)*phi(j,k)*work(k)*tang(k,l)
               end do
               elemmt(3*j,l+3*(i-1)) = elemmt(3*j,l+3*(i-1))+sum
            end do
         end do
      end do

!     --- Compute element matrix corresponding to normal

      do k = 1, 3
         norm(k,1) = -tang(k,2)
         norm(k,2) =  tang(k,1)
      end do
      do l = 1, 2
         do i = 1, 3
            do j = 1, 3
               sum = 0d0
               do k = 1, 3
                  sum = sum-w(k)*phi(i,k)*phi(j,k)*norm(k,l)
               end do
               elemmt(l+3*(j-1),3*i) = sum
            end do
         end do
      end do

!     --- Compute element matrix corresponding to u_t

      do i = 1, 3
         do j = 1, 3
            sum = 0d0
            do k = 1, 3
               sum = sum+w(k)*dphidx(i,k)*phi(j,k)*utang(k)
            end do
            elemmt(3*j,3*i) = sum
         end do
      end do

      call erclos ( 'el915mat' )
      end
