      subroutine eld400 ( iuser, user, vecpre, islold, work )
! ======================================================================
!
!        programmer    Jaap van der Zanden / Guus Segal
!        version  5.16 date 09-03-2013 Replace elflr7 by elflr71
!        version  5.15 date 12-12-2012 Long integers
!        version  5.14 date 20-09-2002 Debug statements
!
!   copyright (c) 1984-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element vector to compute pressure and other derived quantities
!     for two-dimensional Navier-Stokes equations
!     extended quadratic triangle for velocities (itype=400,402,404)
!              quadratic triangle for swirl
!     ELD400 is a help subroutine for subroutine DERIVA, &
!     it is called through the intermediate subroutine ELDNS2
!     So:
!     DERIVA
!       -  Loop over elements
!          -  ELDNS2
!             - ELD400
!     itype=400 :    cartesian coordinates   7-point triangle
!                    Penalty function approach
!     itype=402 :    polar coordinates       7-point triangle
!                    Penalty function approach
!     itype=404 :    axisymmetric coordinates  7-point triangle
!                    Penalty function approach
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduletime
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      double precision x, w, dphidx, xgauss, phi, array, amasps, &
                       u, ugauss, un, dudx, trans, transl, qphixx, &
                       etaeff, secinv, dvisd2, omega, sigmalc, rho, cn, &
                       clam, f, eij, dlamdx, psi, dpsidx, dum, worksp, &
                       penal, amas, visprm
      common /celwrk/ x(14), w(7), dphidx(98), xgauss(14), phi(49), &
                      array(56), u(21), ugauss(21), un(21), dudx(42), &
                      trans(24), transl(1), qphixx(147), etaeff(7), &
                      secinv(7), dvisd2(7), omega, sigmalc, rho, &
                      cn, clam, psi(42), f(21), dpsidx(84), amasps(36), &
                      penal(14), amas(49), worksp(210), visprm(3), &
                      eij(6), dlamdx(6), dum(488)
      save   /celwrk/

!                     /celwrk/
!          Work common block to store local arrays
!          CELWRK contains exactly 1500 real positions, the last 500
!          are reserved for the ELP subroutines
!          The following parameters are used in CELWRK
!     amas           Mass matrix corresponding to basis funtions phi
!     amasps         Mass matrix corresponding to basis funtions psi
!     array          8 x m array to store the coefficients for each
!                    integration point
!                    Contents:
!                    Pos 1-m:       - &
!                    Pos m+1-2m:    rho
!                    Pos 2m+1-3m:   - &
!                    Pos 3m+1-4m:   omega
!                    Pos 4m+1-5m:   f 1
!                    Pos 5m+1-6m:   f 2
!                    Pos 6m+1-7m:   f 3
!                    Pos 7m+1-8m:   eta
!     clam           Parameter lambda for Generalised Newtonian fluids
!     cn             Parameter n for Generalised Newtonian fluids
!     dlamdx         Array of 6 positions containing the derivatives of the
!                    barycentric co-ordinates
!     dphidx         Array of length n x m x 2  containing the derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx = dphidx (i,k,1);  d phi / dy = dphidx (i,k,2);
!                         i                             i
!                      in node k
!     dpsidx         Array of length n-1 x m x 2  containing the derivatives
!                    of the special basis functions corresponding to the
!                    nodal points in the sequence:
!                    d psi / dx = dpsidx (i,k,1);  d psi / dy = dpsidx (i,k,2);
!                         i                             i
!                      in node k
!     dudx           Array of length 4 x m containing the derivatives of the
!                    preceding velocity in the integration points in the
!                    sequence:
!                    1    - m:  du/dx
!                    m+1  - 2m: du/dy
!                    2m+1 - 3m: dv/dx
!                    3m+1 - 4m: dv/dy
!     dum            488 positions reserved for the ELP subroutines
!     worksp         work space free  At least 9 positions are used
!     dvisd2         Array of length m to store the derivative of the viscosity
!                    with respect to the second invariant of the
!                    stress tensor in the integration points
!     eij            Array of 6 positions containing the factors e
!     etaeff         Array of length m to store the effective viscosity in
!                    the integration points
!     f              Array of length 3n to store the right-hand side vector
!                    temporarily
!     omega          Angular velocity of rotating system
!     penal          Help array to store the penalty vector
!     phi      o     Array of length n x m containing the values of the
!                    basis functions in the Gauss points in the sequence:
!                           k
!                    phi (xg ) = phi (i, k)
!                       i
!                    Array phi is only filled when ifirst = 0 and gauss
!                    integration is applied
!     psi            Array of length n-1 x m containing the values of the
!                    basis functions corresponding to the nodal points
!                    in the Gauss points in the sequence:
!                           k
!                    psi (xg ) = psi (i, k)
!                       i
!                    Array psi is only filled when ifirst = 0 and gauss
!                    integration is applied
!     qphixx         Array of length 3 x n x n to store the integrals
!                    corresponding to the second derivative terms on a
!                    reference element
!     rho            Density of the fluid
!     secinv         Array of length m to store the second invariant of the
!                    stress tensor in the integration points
!     sigmalc          Inverse of parameter epsilon in Penalty function method
!     trans          Transformation array for the velocity in the centroid
!                    length: 2 * inpelm
!     transl         Not yet used
!     x              array of length n x 2 containing the x and
!                    y-coordinates of the nodal points with respect to the
!                    local numbering in the sequence:
!                    x  = x (i, 1);  y  = x (i, 2);
!                     i              i
!     xgauss   o     Array of length m x 2 containing the co-ordinates of
!                    the gauss points.  Array xgauss is only filled when Gauss
!                    integration is applied.
!                    xg  = xgauss (i, 1);  yg  = xgauss (i, 2);
!                      i                    i
!     u              Array of length 3n to store the preceding velocity in
!                    the nodal points
!     ugauss         Array of length 3m to store the preceding velocity in
!                    the integration points
!     un             Array of length 3n to store the velocity at a preceding
!                    time level in the nodal points
!     visprm         Viscosity model parameters for modelv=6,7
!     w              array of length m containing the weights for the
!                    numerical integration
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), vecpre(*), work(*)
      integer iuser(*), islold(numold)

!     icheld   i     1  pressure
!                    2  du(j)/dx(i) (physical derivative)
!                    3  div v
!                    4  rot v
!                    5  stress11, stress12, stress13, stress22, ...(itcoor=3)
!                    6  grad . (tensor)
!                    7  elongation
!                    8  deformation
!                    9  sqrt(secinv)
!                    icheld = icheld + 10 similar, but derived quantities
!                             in all nodal points instead of in the vertices
!                             only
!                          (icheld=11) not yet ready
!     islold   i     User input array in which the user puts information
!                    of all preceding solutions
!                    Integer array of length 5 x numold
!     iuser    i     Integer user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
!     user     i     Real user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
!     vecpre   i     Solution at previous time (un)
!     work           Work array to be used by this subroutine
!                    At this moment a working space of 800 suffices
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer method, itcoor, npuv, npuv2, nstart, &
              jtime, mconv, irotat, modelv, &
              ichint, icurve, ichiv, i, j, ksign, &
              ipwork, isub, isup, jelp, kderiv, iaddp, ichl
      double precision delta, pc, rhoh(1)
      save method, itcoor, npuv, npuv2, nstart, ipwork, ichiv, &
           jtime, mconv, irotat, modelv, isub, isup, iaddp, ichl
      save rhoh

!                they are not
!     delta      jacobian of element
!     i          Counting variable
!     iaddp      ?
!     ichint     ?
!     ichiv      ?
!     ichl       ?
!     icurve     Type of curve
!     ipwork     Starting address for right-hand side part in array work
!                The first part of array work is reserved for the element
!                matrix
!     irotat     Parameter indicating if the co-ordinate system rotates (10
!                or not (0)
!     isub       Integer help parameter for do-loops to indicate the
!                smallest coefficient number that is dependent on space
!     isup       Integer help parameter for do-loops to indicate the
!                largest coefficient number that is dependent on space
!     itcoor     ?
!     j          Counting variable
!     jelp       Indication of the type of basis function subroutine must
!                be called by ELxxxx. Possible vaues:
!                 1:  Subroutine ELP220  (Seven-point extended triangle)
!     jtime        Parameter indicating the type of time-dependence
!                  Possible values:
!                  0:  Steady state
!                  1:  Instationary equations theta method
!                +-2:  Pressure correction scheme
!                  3:  Mass matrix only
!                  4:  Iterative instationary equations theta method
!     kderiv     ?
!     ksign      Help parameter to compute sign of the Jacobian
!     mconv      With this parameter the type of linearization of the
!                convective terms is indicated
!                Possible values:
!                0:  No convective terms (Stokes)
!                1:  Standard Picard linearization
!                2:  Newton linearization
!                3:  Picard linearization in the wrong way
!                    (small convergence region, slow convergence)
!                4:  Picard linearization (only effect on right-hand side)
!     method     Indicates the type of method that is used to solve the
!                Navier-Stokes equations
!                Possible values:
!                1:   Penalty function method
!     modelv     Defines the type of viscosity model
!                Possible values:
!                  1:  newtonian liquid
!                  2:  power law liquid
!                  3:  carreau liquid
!                  4:  plastico-viscous liquid
!                  5:  Papanastasiou plastico-viscous model
!                  6:  bi-viscosity plastico-viscous model
!                  7:  Carreau-Yasuda model
!                100:  user provided model depending on grad u
!                101:  user provided model depending on the second invariant
!                102:  user provided model depending on the second invariant, &
!                      the co-ordinates and the velocity
!     npuv       ?
!     npuv2      ?
!     nstart
!     pc         ?
!     rhoh       ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0400         Compute element independent quantities
!     EL2002         Compute part of matrix corresponding to zeroth order
!                    derivative
!     EL2015         Compute matrix vector multiplication
!     EL3007         Fills old solution and derivatives
!     EL4022         Compute viscous matrix for 2-dim incompressible flow
!     EL4025         Compute (transformed) penalty vector
!     EL4028         Compute  viscosity for a generalized Newtonian liquid
!     EL4031         Compute (driving) external forces for RHS vector
!     EL4032         Compute Coriolis acceleration and centrifugal force
!     EL4033         Compute convective acceleration for RHS vector
!     EL4034         Compute pressure in centroid for penalty method
!     EL4035         Compute gradient of pressure and fill elemvc1 and elemwg
!     EL4038         Compute derived quantities of a vector
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELP220         Shape functions for 7-point triangle (7 integration points)
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
!   none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1291   Warning for pressure for plastico-viscous model
!    1304   For coefficient only positive constants are allowed
!    1781   The possibility ifunc>10000 not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eld400' )
      debug = .false.

      if ( ifirst==0 ) then

!     --- ifirst = 0, compute element independent quantities

         call el0400 ( iuser, user, jelp, isub, isup, method, mconv, &
                       jtime, modelv, irotat )
         ipwork = ( (inpelm+1)*2 ) ** 2 + 1

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( islold, numold, isub, isup, iuser )

!        --- Set jsol temporarily equal to -1 in order to indicate that the
!            old versions of the ELP subroutines are used

        jsol = -jsol

!       --- Set some constants, these correspond to the old version
!           npuv    number of points for velocity u1, u2 including centroid
!           nstart  number of test function to start with
!           lrow    total number of degrees of freedom in element

         npuv   = inpelm+1
         npuv2  = npuv+npuv
         lrow   = npuv2
         nstart = npuv
         itcoor = jcart - 1

         if(jtime==1 .or. jtime==4) then
            rhoh(1) = rho/(theta*tstep)
         else if(jtime==3) then
            rhoh(1) = rho
         end if
         ichl = itcoor+30
         if (method==0) ichl = ichl+40

!        --- Gauss integration for Cartesian coordinates
!            (reason: Newton-Cotes integration inaccurate on grids with
!               only 6 significant digits of the co-ordinates;
!               like e.g.due to formatted-write of non-equidistant grids)

         if (itcoor==0) then
            ichl = itcoor+30+3
         else
            ichl = itcoor+30
         end if
         if ( icheld/=1 .and. icheld/=11 ) ichl = 30
         if ( modelv>=4 .and. modelv<=6 .and. &
             ( icheld==1 .or. icheld==11 ) ) &
            call errwar ( 1291, 0, 0, 0 )
         if ( (icheld==1 .or.  icheld==11) .and. &
                mconv==0 .and. modelv==1 ) then
            jderiv = 0
         else
            jderiv = 1
         end if
         ichiv = 1

      end if

!     --- Compute basis functions and weights for numerical integration

      call elp220 ( ichl, ichint, coor, index1, ksign, &
                    icurve, delta, x, xgauss, eij, dlamdx, w, phi, &
                    dphidx, trans, transl, qphixx )
      if ( ierror/=0 ) go to 3000
      jdiag=ichint+1

!     --- Fill variable coefficients if necessary
!         First fill uold if necessary

      if ( (jtime==1 .or. jtime==4) .and. &
           (icheld==1 .or. icheld==11) ) then

!     --- jtime = 1, 4, time-dependent method
!                       Both un and u n+1 * are necessary

         kderiv = jderiv
         jderiv = 0
         call el3007 ( un, work, phi, dphidx, m, n, ndimlc, &
                       ncomp, index1, trans, ugauss, jtrans, &
                       numold, 1 )
         jderiv = -kderiv

         call el3007 ( u, dudx, phi, dphidx, m, n, ndimlc, &
                       ncomp, index1, trans, ugauss, jtrans, &
                       numold, 1 )

         jderiv=-jderiv

!        --- calculate massmatrix (only first part)

         call el2002 ( -npuv, ichint+1, rhoh, amas, w, phi, npuv, npuv, &
                       npuv, work(ipwork), work, delta, .true. )

      else

         jderiv = -jderiv
         call el3007 ( u, dudx, phi, dphidx, m, n, ndimlc, &
                       ncomp, index1, trans, ugauss, jtrans, numold, 1 )
         jderiv = -jderiv
      if(debug) then
      call prinin(index1,inpelm,'index1')
      call prinrl(u,14,'u')
      call prinrl(dudx,28,'dudx')
      end if

      end if

      if ( icheld== 1 .or. icheld== 5  .or. &
           icheld==11 .or. icheld==15 ) then

!     --- icheld = 1 or 5, 11 or 15, functions must be computed

         if ( ind(7)/=0 ) ichiv = ichint
         do i = isub, isup

            if ( ind(i)>0 .or. ist(i)>0 ) then

!           --- Coefficient is variable

               if ( ind(i)>10000 ) then
                  call errint ( i, 1 )
                  call errsub ( 1781, 1, 0, 0 )
               end if
               call elflr71 ( i, user, array, x, &
                             xgauss, m, n, phi, 1, index1, &
                             iuser, ugauss, work )
               if ( i==7 ) then

!              --- Coefficient 7, i.e. eta
!                  Test if eta is positive

                   do j = 1, m
                      if ( array(j+(i-1)*m)<=0 ) then

!                     --- Error 1304: Eta is not positive

                         call errint ( i, 1 )
                         call errint ( itype, 2 )
                         call errint ( ielgrp, 3 )
                         call errint ( ielem, 4 )
                         call erreal ( array(j+(i-1)*m), 1 )
                         call errchr ( 'eta (0)', 1 )
                         call errsub ( 1304, 4, 1, 1 )

                      end if

                   end do

                end if

            end if

         end do

      end if
      if (icheld==1 .or. icheld==11) then

!     --- compute effective viscosity

         call el4028 ( 1, itcoor, npuv, ndimlc, &
                       modelv, array(1+6*m), cn, clam, visprm, &
                       work(ipwork), xgauss, ugauss, dudx, &
                       etaeff, secinv, dvisd2 )

!        --- compute derived quantities

      else
         call el4038 ( icheld, ix, jdegfd, itcoor, npuv, &
                       ndimlc, modelv, array(1+6*m), cn, clam, &
                       visprm, x, u, dudx, elemvc1, elemwg, worksp)
         go to 3000
      end if

!     --- compute ndim rows of viscous matrix

      call el4022 ( ichiv, itcoor, etaeff, work, w, xgauss, phi, dphidx, &
                    nstart, npuv, npuv, lrow, phi, dphidx, inpelm, &
                    dlamdx, qphixx, delta, work(ipwork) )

!     --- compute (transformed) penalty vector if required

      iaddp = 0
      if (method==1) &
          call el4025 ( itcoor, ichint, iaddp, work, penal, sigmalc, &
                        ksign, delta, eij, w, xgauss, phi, dphidx, &
                        npuv, npuv, ncomp, lrow )

!     --- compute right-hand side vector
!         (driving) external forces

      call el4031 ( ichint, array(1+4*m), phi, rho, w, work(ipwork), &
                    nstart, npuv, npuv, lrow )

!     --- compute influence of convective acceleration for mconv>0

      if (mconv>0) &
          call el4033 ( ichint, itcoor, -rho, ugauss, dudx, &
                        w, xgauss, phi, work(ipwork), &
                        nstart, npuv, npuv, ndimlc, phi, inpelm )

!     --- compute coriolis acceleration and centrifugal force if required

      if (irotat==1) &
          call el4032 ( ichint, itcoor, phi, rho, omega, ugauss, w, &
                        work(ipwork), nstart, npuv, npuv, phi, &
                        inpelm )

!     --- subtract m(u-uold)

      if (jtime==1) then
         do i = 1, npuv2
            un(i) = un(i)-u(i)
         end do
         call el2015 ( 2, ichint+1, amas, un, 1d0, work(ipwork), npuv, &
                       npuv, npuv )
         call el2015 ( 2, ichint+1, amas, un(1+npuv), 1d0, &
                       work(ipwork+npuv), npuv, npuv, npuv )
      end if

!     --- compute pressure in centroid for penalty method

      call el4034 ( pc, sigmalc, delta, penal, u, npuv, ndimlc )

!     --- compute gradient of pressure and fill elemvc1 and elemwg

      call el4035 ( ichint, itcoor, elemvc1, elemwg, pc, phi, xgauss, &
                    x, u, work, work(ipwork), w, npuv, lrow, ndimlc )
3000  if ( debug ) then

!     --- debug = true

         write(irefwr,*) 'element ', ielem, '  itype ',itype
         call prinrl(elemvc1,3,'elemvc1')
         call prinrl(elemwg,3,'elemwg')

      end if
      call erclos ( 'eld400' )
      end
