      subroutine eld900coeffs ( dphidx, phi, ugauss, dudx, uold, &
                                isub, isup, iuser, user, &
                                vecloc, x, xgauss, p, nveloc, &
                                coeffnodes, coeffintpnts )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.2  date 15-11-2012  Long integers
!        version  2.1  date 17-07-2011 Compute old solution only in nodes
!
!   copyright (c) 2006-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill variable coefficients in coefficients array
!     Fill velocity from prior iteration if necessary
!     Help subroutine for eld900
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     navier_stokes_equation
!     initialization
!     coefficient
!     velocity
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isub, isup, iuser(*), nveloc
      double precision dphidx(*), p(*), uold(*), &
                       user(*), vecloc(*), x(*), xgauss(m,ndim), &
                       phi(*), coeffnodes(n,*), coeffintpnts(m,*), &
                       ugauss(*), dudx(*)

!     coeffintpnts  i/o   Real array in which the values of the
!                         coefficients in the integration points are stored
!     coeffnodes    i/o   Real array in which the values of the
!                         coefficients in the nodal points are stored
!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     dudx           i    Real m x ndim  array in which the values of the
!                         derivatives of the solution vector in the integration
!                         points are stored in the sequence:
!                         du / dx  (x ) = dudx ( k, j )
!                           j     k
!                         If jdercn = 0 in common block celint, then only
!                         dudx (1,*) is filled because dudx is then a constant
!                         per element
!                         dudx is only filled if jderiv>0
!     isub           i    Integer help parameter for do-loops to indicate the
!                         smallest coefficient number that is dependent on space
!     isup           i    Integer help parameter for do-loops to indicate the
!                         largest coefficient number that is dependent on space
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     nveloc         i    Number of velocity parameters
!     p              o    Array containing the pressure
!                         p(1) contains the pressure in the centroid
!                         p(2-4) contains the gradient of the pressure in the
!                         centroid
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     ugauss         i    Preceding solution in Gauss points
!     uold           i    Array containing the previous iteration (old solution)
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     vecloc         i    Work array in which all old solution vectors for the
!                         integration points are stored
!     x              i    Contains coordinates of vertices
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ktrans, jdiagsav, interpol
      double precision work(n)
      double precision, pointer :: vecoldlc(:)

!     i              Counting variable
!     interpol       If 0 the standard interpolation is applied, &
!                    if 1 the interpolation is restricted to linear
!                    subelements
!     jdiagsav       Help parameter to save the value of jdiag
!     ktrans         Help variable to store the value of jtrans
!     work           Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL2005         Compute a local vector and its derivatives from a known
!                    vector
!     EL3009         Fill series of old solutions in array vecloc
!     ELFILLCOEFF    Fills function values in array coeffnodes for the nodal
!                    points and in coeffintpnts for the integration points
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eld900coeffs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eld900coeffs'

      end if
      if ( ierror/=0 ) go to 1000

      interpol = irule/100
      iseqin = iseqi1

      if ( ind(lengthmst)==1 .or. ind(lengthmst)==3 ) then
         ktrans = jtrans
         if ( jtrans==1 ) jtrans = 2
         call el2005 ( uold, dudx, phi, dphidx, &
                       index1, ugauss, iseqin )
         jtrans = ktrans
         if ( itype==901 .or. inpelm==5 ) then

!        --- itype = 901, put pressure in p

            iseqtype = oldsols(iseqin)%iseqtype
            index3n => indoldsols(iseqtype)%index3
            vecoldlc => oldsols(iseqin)%oldvecs
            p(1) = vecoldlc(index3n(nveloc+1))

         else if ( itype==902 ) then

!        --- itype = 902, put pressure and pressure gradient in p

            iseqtype = oldsols(iseqin)%iseqtype
            index3n => indoldsols(iseqtype)%index3
            vecoldlc => oldsols(iseqin)%oldvecs
            if ( inpelm==27 ) then
               do i = 1, 4
                  p(i) = vecoldlc(index3n(14*3+i))
               end do
            else
               do i = 1, ndim+1
                  p(i) = vecoldlc(index3n(nveloc+i))
               end do
            end if

         end if

      end if

      if ( ind(lengthmst)==2 .or. ind(lengthmst)==3 ) then

!     --- ind(lengthmst) = 2 or 3, hence vecloc must be filled
!         This array is only filled in the nodal points
!         hence jdiag is set to 1 temporarily

         jdiagsav = jdiag  ! save value of jdiag
         if ( interpol==1 ) jdiag = 1
         call el3009 ( vecloc, phi, n )
         jdiag = jdiagsav  ! reset jdiag

      end if  ! ( ind(lengthmst)==2 .or. ind(lengthmst)==3 )

!     --- Fill variable coefficients if necessary

      do i = isub, isup

         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            call elfillcoeff ( i, user, coeffnodes(1,i), x, &
                               xgauss, phi, 1, index1, &
                               iuser, uold, vecloc, coeffintpnts(1,i) )

         end if

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End eld900coeffs'

      end if
      call erclos ( 'eld900coeffs' )

      end
