      subroutine elm400 ( work, vecloc, elemmt1d )
! ======================================================================
!
!        programmer    Jaap van der Zanden / Guus Segal
!        version 12.0  date 06-01-2016 Use sepmoduleelem
!        version 11.11 date 09-03-2013 Replace elflr7 by elflr71
!        version 11.10 date 12-12-2012 Long integers
!
!   copyright (c) 1984-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector two-dimensional Navier-Stokes equations
!     ELM400 is a help subroutine for subroutine SYSTEM, &
!     it is called through the intermediate subroutine ELMNS2
!     So:
!     SYSTEM
!       -  Loop over elements
!          -  ELMNS2
!             - ELM400
!     itype=400 :    cartesian coordinates   7-point triangle
!                    Penalty function approach
!     itype=402 :    polar coordinates       7-point triangle
!                    Penalty function approach
!     itype=404 :    axisymmetric coordinates  7-point triangle
!                    Penalty function approach
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     flow
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduletime
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      double precision x, w, dphidx, xgauss, phi, array, amasps, &
                       u, ugauss, un, dudx, trans, transl, qphixx, &
                       etaeff, secinv, dvisd2, omega, sigmalc, rho, cn, &
                       clam, f, eij, dlamdx, psi, dpsidx, dum, dum1, &
                       penal, amas, visprm
      common /celwrk/ x(14), w(7), dphidx(98), xgauss(14), phi(49), &
                      array(56), u(21), ugauss(21), un(21), dudx(42), &
                      trans(24), transl(1), qphixx(147), etaeff(7), &
                      secinv(7), dvisd2(7), omega, sigmalc, rho, &
                      cn, clam, psi(42), f(21), dpsidx(84), amasps(36), &
                      penal(14), amas(49), dum1(210), visprm(3), &
                      eij(6), dlamdx(6), dum(488)
      save  /celwrk/

!                     /celwrk/
!          Work common block to store local arrays
!          CELWRK contains exactly 1500 real positions, the last 500
!          are reserved for the ELP subroutines
!          The following parameters are used in CELWRK
!     amas           Mass matrix corresponding to basis funtions phi
!     amasps         Mass matrix corresponding to basis funtions psi
!     array          8 x m array to store the coefficients for each
!                    integration point
!                    Contents:
!                    Pos 1-m:       - &
!                    Pos m+1-2m:    rho
!                    Pos 2m+1-3m:   - &
!                    Pos 3m+1-4m:   omega
!                    Pos 4m+1-5m:   f 1
!                    Pos 5m+1-6m:   f 2
!                    Pos 6m+1-7m:   f 3
!                    Pos 7m+1-8m:   eta
!     clam           Parameter lambda for Generalised Newtonian fluids
!     cn             Parameter n for Generalised Newtonian fluids
!     dlamdx         Array of 6 positions containing the derivatives of the
!                    barycentric co-ordinates
!     dphidx         Array of length n x m x 2  containing the derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx = dphidx(i,k,1); d phi / dy = dphidx(i,k,2);
!                         i                           i
!                      in node k
!     dpsidx         Array of length n-1 x m x 2  containing the derivatives
!                    of the special basis functions corresponding to the
!                    nodal points in the sequence:
!                    d psi / dx = dpsidx(i,k,1); d psi / dy = dpsidx(i,k,2);
!                         i                           i
!                      in node k
!     dudx           Array of length 4 x m containing the derivatives of the
!                    preceding velocity in the integration points in the
!                    sequence:
!                    1    - m:  du/dx
!                    m+1  - 2m: du/dy
!                    2m+1 - 3m: dv/dx
!                    3m+1 - 4m: dv/dy
!     dum            488 positions reserved for the ELP subroutines
!     dum1           work space free for later use
!     dvisd2         Array of length m to store the derivative of the viscosity
!                    with respect to the second invariant of the
!                    stress tensor in the integration points
!     eij            Array of 6 positions containing the factors e
!     etaeff         Array of length m to store the effective viscosity in
!                    the integration points
!     f              Array of length 3n to store the right-hand side vector
!                    temporarily
!     omega          Angular velocity of rotating system
!     penal          Help array to store the penalty vector
!     phi            Array of length n x m containing the values of the
!                    basis functions in the Gauss points in the sequence:
!                           k
!                    phi (xg ) = phi (i, k)
!                       i
!                    Array phi is only filled when ifirst = 0 and gauss
!                    integration is applied
!     psi            Array of length n-1 x m containing the values of the
!                    basis functions corresponding to the nodal points
!                    in the Gauss points in the sequence:
!                           k
!                    psi (xg ) = psi (i, k)
!                       i
!                    Array psi is only filled when ifirst = 0 and gauss
!                    integration is applied
!     qphixx         Array of length 3 x n x n to store the integrals
!                    corresponding to the second derivative terms on a
!                    reference element
!     rho            Density of the fluid
!     secinv         Array of length m to store the second invariant of the
!                    stress tensor in the integration points
!     sigmalc          Inverse of parameter epsilon in Penalty function method
!     trans          Transformation array for the velocity in the centroid
!                    length: 2 * inpelm
!     transl         Not yet used
!     x              array of length n x 2 containing the x and
!                    y-coordinates of the nodal points with respect to the
!                    local numbering in the sequence:
!                    x  = x (i, 1);  y  = x (i, 2);
!                     i              i
!     xgauss         Array of length m x 2 containing the co-ordinates of
!                    the gauss points.  Array xgauss is only filled when Gauss
!                    integration is applied.
!                    xg  = xgauss (i, 1);  yg  = xgauss (i, 2);
!                      i                    i
!     u              Array of length 3n to store the preceding velocity in
!                    the nodal points
!     ugauss         Array of length 3m to store the preceding velocity in
!                    the integration points
!     un             Array of length 3n to store the velocity at a preceding
!                    time level in the nodal points
!     visprm         Viscosity model parameters for modelv=6,7
!     w              array of length m containing the weights for the
!                    numerical integration
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision vecloc(*), work(*), elemmt1d(*)

!     vecloc         Work array in which all old solution vectors for the
!                    integration points are stored
!     work           Work space for this subroutine
!                    At this moment 2*196 positions work space is sufficient
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer induv(12)
      integer method, npuv, npuv2, nstart, ltime, mconv, &
              irotat, modelv, iche, itcoor, jelp, i, j, &
              ichint, ichiv, ichl, icurve, iptim, ksign, &
              isub, isup, kderiv, ipwork
      double precision delta, rhoh(1)
      save method, itcoor, npuv, npuv2, nstart, iptim, ipwork, &
           induv, ltime, mconv, irotat, modelv, isub, isup, &
           rhoh, iche, ichl
      data induv/1, 7, 2, 8, 3, 9, 4, 10, 5, 11, 6, 12/

!     delta      jacobian of element
!     i          Counting variable
!     iche       ?
!     ichint     ?
!     ichiv      ?
!     ichl       ?
!     icurve     Type of curve
!     induv      ?
!     iptim      ?
!     ipwork     Starting address in array work of work array
!     irotat
!     isub       Integer help parameter for do-loops to indicate the
!                smallest coefficient number that is dependent on space
!     isup       Integer help parameter for do-loops to indicate the
!                largest coefficient number that is dependent on space
!     itcoor     ?
!     j          Counting variable
!     jelp       Indication of the type of basis function subroutine must
!                be called by ELxxxx. Possible vaues:
!                 1:  Subroutine ELP622
!     kderiv     ?
!     ksign      Help parameter to compute sign of the Jacobian
!     ltime      ?
!     mconv
!     method
!     modelv     Defines the type of viscosity model
!                Possible values:
!                  1:  newtonian liquid
!                  2:  power law liquid
!                  3:  carreau liquid
!                  4:  plastico-viscous liquid
!                  5:  Papanastasiou plastico-viscous model
!                  6:  bi-viscosity plastico-viscous model
!                  7:  Carreau-Yasuda model
!                100:  user provided model depending on grad u
!                101:  user provided model depending on the second invariant
!                102:  user provided model depending on the second invariant, &
!                      the co-ordinates and the velocity
!     npuv       ?
!     npuv2      ?
!     nstart
!     rhoh       ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DAXPY   Overwrite double precision dy with double precision da*dx + dy
!     DCOPY   Copy double precision dx to double precision dy
!     EL0400  Compute element independent quantities
!     EL2002  Compute part of matrix corresponding to zeroth order derivatives
!     EL2008  Add matrix sh to matrix s, where sh is of smaller size than s
!     EL2009  Reorder the contents of the element matrix or element vector
!     EL2015  Compute matrix vector multiplication
!     EL2016  Compute sum of to matrices
!     EL3007  Fills old solution and derivatives
!     EL3009  Fill series of old solutions in array vecloc
!     EL4022  Compute viscous matrix for 2-dim incompressible flow
!     EL4023  Compute convective and/or coriolis acceleration (matrix)
!     EL4024  Eliminate velocity from centroid (uc, vc)
!     EL4025  Compute (transformed) penalty vector
!     EL4028  Compute  viscosity for a generalized newtonian liquid
!     EL4031  Compute (driving) external forces for RHS vector
!     EL4033  Compute convective acceleration for RHS vector
!     EL4036  Compute matrix due to newton linearization of viscosity
!     EL4037  Compute vector due to newton linearization of viscosity
!     ELFLR71 Fill variable coefficients in array ARRAY
!     ELIND0  Fill array ICH in COMMON CELIAR for preceding solutions
!     ELP220  Shape functions for 7-point triangle (7 integration points)
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRCHR  Put character in error message
!     ERREAL  Put real in error message
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1304   For coefficient only positive constants are allowed
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm400' )

      if ( ifirst==0 ) then

!     --- ifirst = 0, compute element independent quantities

         ltime = 0
         call el0400 ( E%iuser, E%user, jelp, isub, isup, method, mconv, &
                       ltime, modelv, irotat )
         ipwork = ( (inpelm+1)*2 ) ** 2 + 1

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( E%islold, numold, isub, isup, E%iuser )

!        --- Set jsol temporarily equal to -1 in order to indicate that the
!            old versions of the ELP subroutines are used

        jsol = -jsol

!       --- Set some constants, these correspond to the old version
!           npuv    number of points for velocity u1, u2 including centroid
!           nstart  number of test function to start with
!           lrow    total number of degrees of freedom in element

         npuv=inpelm+1
         npuv2=npuv+npuv
         lrow=npuv2
         nstart=1
         itcoor = jcart - 1
         iptim = npuv*npuv+1

         if(ltime==1 .or. ltime==4) then
            rhoh(1) = rho/(theta*tstep)
         else if(ltime==3) then
            rhoh(1) = rho
         end if
         if ( modelv==2 .and. cn>1d0 ) then
            iche = 2
         else
            iche = 1
         end if

!        --- Gauss integration for Cartesian coordinates
!            (reason: Newton-Cotes integration inaccurate on grids with
!             only 6 significant digits of the co-ordinates;
!             like e.g.due to formatted-write of non-equidistant grids)

         if (itcoor==0) then
            ichl = itcoor+30+3
         else
            ichl = itcoor+30
         end if
         if (method==0) ichl = ichl+40

      end if

!     --- Compute basis functions and weights for numerical integration

      call elp220 ( ichl, ichint, coor, index1, ksign, &
                    icurve, delta, x, xgauss, eij, dlamdx, w, phi, &
                    dphidx, trans, transl, qphixx )

      jdiag = ichint+1

!     --- Fill variable coefficients if necessary
!         First fill uold if necessary

      if ( ltime==4 ) then

!     --- ltime = 4, Iterative time-dependent method
!                    Both un and u n+1 * are necessary

         kderiv = jderiv
         jderiv = 0
         call el3007 ( un, work, phi, dphidx, m, n, ndimlc, &
                       ncomp, index1, trans, ugauss, jtrans, &
                       numold, 1 )
         jderiv = -kderiv

         call el3007 ( u, dudx, phi, dphidx, m, n, ndimlc, &
                       ncomp, index1, trans, ugauss, jtrans, &
                       numold, 2 )
         jderiv = kderiv

      else if ( ind(lengthmst)==1 ) then

         jderiv = -jderiv
         call el3007 ( u, dudx, phi, dphidx, m, n, ndimlc, &
                       ncomp, index1, trans, ugauss, jtrans, &
                       numold, 1 )
         jderiv = -jderiv

      end if

      if ( ind(lengthmst)==2 .or. ind(lengthmst)==3 ) &
         call el3009 ( vecloc, phi, n )

      do i = isub, isup

         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            call elflr71 ( i, E%user, array, x, xgauss, &
                          m, n, phi, 1, index1, &
                          E%iuser, ugauss, vecloc )
            if ( i==7 ) then

!           --- Coefficient 7, i.e. eta
!               Test if eta is positive

                do j = 1, m
                   if ( array(j+(i-1)*m)<=0 ) then

!                  --- Error 1304: Eta is not positive

                      call errint ( i, 1 )
                      call errint ( itype, 2 )
                      call errint ( ielgrp, 3 )
                      call errint ( ielem, 4 )
                      call erreal ( array(j+(i-1)*m), 1 )
                      call errchr ( 'eta (0)', 1 )
                      call errsub ( 1304, 4, 1, 1 )

                   end if

                end do

             end if

         end if

      end do

      if (ind(7)/=0) then
         ichiv = 1
      else
         ichiv = ichint
      end if

!     --- compute effective viscosity

      if (matrix .or. modelv>1 .and. modelv<7 .and. ltime/=3) &
         call el4028 ( iche, itcoor, npuv, ndimlc, &
                       modelv, array(1+6*m), cn, clam, visprm, &
                       work(ipwork), xgauss, ugauss, dudx, &
                       etaeff, secinv, dvisd2 )
      if(ltime>=1) then
         call el2002(6, ichint+1, rhoh, E%mat, w, phi, npuv, &
                npuv, npuv, work(ipwork), work, delta, .true.)
         if(ltime==3) goto 1100

!        --- compute mass matrix * uold  : for ltime = 4 use 'un' else use 'u'

         if(ltime==4) then
           call el2015(1, ichint+1, E%mat, un, 1d0, elemmt1d(iptim), &
                   npuv, npuv, 1)
           call el2015(1, ichint+1, E%mat, un(1+npuv), 1d0, elemmt1d &
                 (iptim+npuv), npuv, npuv, 1)
         else
           call el2015(1, ichint+1, E%mat, u, 1d0, elemmt1d(iptim), &
                   npuv, npuv, 1)
           call el2015(1, ichint+1, E%mat, u(1+npuv), 1d0, elemmt1d &
                 (iptim+npuv), npuv, npuv, 1)
         end if

!        --- copy m uold in array from pos. ipu

         call dcopy(npuv2, elemmt1d(iptim), 1, u, 1)
      end if

!     --- if (matrix) compute element matrix

      if (.not.matrix) goto 2000

!     --- compute viscous stresses and initialize element matrix
!             s

      call el4022 ( ichiv, itcoor, etaeff, work, w, xgauss, phi, &
                    dphidx, nstart, npuv, npuv, lrow, phi, dphidx, &
                    inpelm, dlamdx, qphixx, delta, f )

!     --- compute terms due to newton linearization of viscosity
!             s for special cases only

      if ( modelv==2 .and. cn>1d0 ) then
         call el4036 ( itcoor, w, w(1+npuv), xgauss, phi, dphidx, &
                       ugauss, dudx, dvisd2, work(ipwork), ndimlc, npuv, &
                       npuv, phi, dphidx, inpelm )
         call el2008(work, work(ipwork), npuv2, lrow)
      end if

!     --- compute penalty matrix if required, and add it

      if (method==1) call el4025 ( itcoor, ichint, 1, work, &
                       work(ipwork), sigmalc, ksign, delta, eij, w, &
                       xgauss, phi, dphidx, npuv, npuv, ncomp, lrow )

!     --- compute convective and/or coriolis acceleration if required

      if (mconv>0 .and. mconv/=4 .or. irotat==1 ) then
         call el4023 ( ichint, itcoor, mconv, rho, irotat, omega, &
                       ugauss, dudx, w, xgauss, phi, dphidx, work, &
                       npuv, npuv, lrow, phi, dphidx, inpelm )
      end if
1100  if(ltime==1 .or. ltime==4 .and. theta>0d0) then
         call el2016 ( 1, ichint+1, work, E%mat, 1d0, 1d0, npuv, lrow )

         call el2016 ( 1, ichint+1, work(1+npuv*lrow+npuv), E%mat, 1d0, &
                       1d0, npuv, lrow )
      else if (ltime==3) then
         do i = 1, lrow*lrow
            work(i) = 0d0
         end do
         call el2016 ( 1, ichint+1, work, E%mat, 0d0, 1d0, npuv, lrow )
         call el2016 ( 1, ichint+1, work(1+npuv*lrow+npuv), E%mat, 0d0, &
                       1d0, npuv, lrow )
      end if

!     --- eliminate velocity from centroid (uc, vc) in point npuv
!               i.e. transform matrix : r(transp)*smatrix*r

      call el4024(1, work, work(ipwork), trans, work(ipwork), &
                  ndimlc, npuv, lrow)

!     --- reorder the contents of the element matrix and put it into E%mat

      call el2009( 1, E%mat, elemvc, work, work(ipwork), &
                   induv, icount, 2 )

!     --- compute element vector

2000  if (.not.vector) go to 3000

!     --- fill (driving) external forces into vector (initialize it)

      call el4031 ( ichint, array(1+4*m), phi, rho, w, work(ipwork), &
                    nstart, npuv, npuv, lrow )

!     --- add terms due to newton linearization of viscosity
!             f

      if ( modelv==2 .and. cn>1d0 ) &
         call el4037 ( itcoor, w, work, xgauss, phi, dphidx, ugauss, &
                       dudx, secinv, dvisd2, work(ipwork), ndimlc, npuv, &
                       npuv, phi, dphidx, inpelm )

!     --- compute influence of convective acceleration for mconv = 2

      if (mconv==2 .or. mconv==4) &
         call el4033 ( ichint, itcoor, rho, ugauss, dudx, w, xgauss, &
                       phi, work(ipwork), nstart, npuv, npuv, ndimlc, &
                       phi, inpelm )

!     --- add contribution of mass matrix * uold

      if(ltime==1 .or. ltime==4) call daxpy(npuv2, 1d0, u, 1, &
          work(ipwork), 1 )

!     --- eliminate velocity basis functions from centroid from vector

      call el4024 ( 2, work, work(ipwork), trans, work, ndimlc, npuv, &
                    lrow )

!     --- reorder the contents of the element vector and put it into elemvc

      call el2009(2, E%mat, elemvc, work, work(ipwork), induv, icount, &
                  2)
3000  call erclos ( 'elm400' )

      end
