      subroutine elm922 ( rcrosselem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.3  date 24-07-2015 Debug statements
!        version  1.2  date 23-08-2001 New call to elusr3
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill element matrix and element vector with respect to elements of type 922
!    It concerns a special element that defines the velocities in the
!    cross-section of an obstacle and a mesh
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     element
!     element_matrix
!     element_vector
!     intersection
!     fictitious_domain
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision rcrosselem(*)

!     rcrosselem     i    Contains the factors defining the intersection point
!                         per element
!                         The intersection point is defined by x1 + fac *(x2-x1)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision u, v, w, factor
      integer nveloc, i, j, ishift
      save u, v, w, nveloc

!     factor         Multiplication factor with respect to the velocities
!                    Defines the relative distance between intersection point
!                    and point in element outside obstacle
!     i              Counting variable
!     ishift         Shift in coefficient sequence number
!     j              Counting variable
!     nveloc         Number of velocity parameters (actually non-Lagrangian
!                    parameters)
!     u              u-velocity in intersection point
!     v              v-velocity in intersection point
!     w              w-velocity in intersection point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELUSR3         Extracts information concerning the coefficients from IUSER
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!     419   NDIM # 2
!    1303   Only constants are allowed for this coefficient
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The intersection point is at relative distance f from the outer point x_1
!     hence the velocity in the intersection point is defined as
!
!     u = (1-f) u_1 + f u_2
!
!     This defines 2 or 3 rows which can be considered as constraints
!     The Lagrangian multipliers imply that we need to make the matrix
!     symmetrical
!
!     This subroutine must be checked carefully because I have my doubts
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm922' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm922'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ishift = 0
      if ( ifirst.eq.0 )  then

!     --- ifirst = 0,  compute element independent quantities

         if ( ndim.lt.2 .or. ndim.gt.3 ) then

!        --- ndim = 3, has not yet been implemented

            call errint ( ndim, 1 )
            call errint ( 2, 2 )
            call errint ( 3, 3 )
            call errsub ( 419, 3, 0, 0 )
            go to 1000

         end if

!        --- Get information of coefficients u and v

         ipos = E%iuser ( ielgrp + 5 )
         if ( ipos.lt.7 ) then

!        --- ipos too small

            call errint ( ielgrp+5, 1 )
            call errint ( ipos, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 42, 5, 0, 0 )

         end if
         ipos = ipos+5
         ishift = ishift+5

         m = 1
         ishift = ishift+1
         call elusr3 ( 1, u, E%iuser, E%user, ishift )
         ishift = ishift+1
         call elusr3 ( 2, v, E%iuser, E%user, ishift )
         if ( ind(6).gt.0 ) then

!        --- u not constant

            call errint ( 6, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errint ( ielem, 4 )
            call errchr ( 'u', 1 )
            call errsub ( 1303, 4, 0, 1 )

         end if
         if ( ind(7).gt.0 ) then

!        --- v not constant

            call errint ( 7, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errint ( ielem, 4 )
            call errchr ( 'v', 1 )
            call errsub ( 1303, 4, 0, 1 )

         end if
         if ( ndim.eq.3 ) then

!        --- ndim = 3

            ishift = ishift+1
            call elusr3 ( 3, w, E%iuser, E%user, ishift )
            if ( ind(8).gt.0 ) then

!           --- w not constant

               call errint ( 8, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errint ( ielem, 4 )
               call errchr ( 'w', 1 )
               call errsub ( 1303, 4, 0, 1 )

            end if

         end if
         if ( ierror.ne.0 ) go to 1000

!        --- Compute number of unknowns per point except the Lagrange
!            multipliers

         nveloc = index4(iseqin,1)-ndim

!        --- Clear E%mat and E%vec

         E%mat = 0d0
         E%vec(1:icount) = 0d0

      end if

      factor = rcrosselem(irelem)

      if ( debug ) then
         write(irefwr,1) 'icount, nveloc, ndim, irelem', &
                          icount, nveloc, ndim, irelem
         write(irefwr,2) 'factor', factor
      end if  ! ( debug )

!     --- Fill E%mat and E%vec

      if ( ndim.eq.2 ) then

!     --- ndim = 2
!         u velocity part of matrix
!         first the row

         E%mat(1,nveloc+1) = 1d0-factor
         E%mat(nveloc+3,nveloc+1) = factor

!        --- next the columns

         E%mat(nveloc+1,1) = 1d0-factor
         E%mat(nveloc+1,nveloc+3) = factor

!        --- v velocity part of matrix
!            first the row

         E%mat(2,nveloc+2) = 1d0-factor
         E%mat(nveloc+4,nveloc+2) = factor

!        --- next the columns

         E%mat(nveloc+2,2) = 1d0-factor
         E%mat(nveloc+2,nveloc+4) = factor

      else

!     --- ndim = 3
!         u velocity part of matrix
!         first the row

         E%mat(1,nveloc+1) = 1d0-factor
         E%mat(nveloc+4,nveloc+1) = factor

!        --- next the columns

         E%mat(nveloc+1,1) = 1d0-factor
         E%mat(nveloc+1,nveloc+4) = factor

!        --- v velocity part of matrix
!            first the row

         E%mat(2,nveloc+2) = 1d0-factor
         E%mat(nveloc+5,nveloc+2) = factor

!        --- next the columns

         E%mat(nveloc+2,2) = 1d0-factor
         E%mat(nveloc+2,nveloc+5) = factor

!        --- w velocity part of matrix
!            first the row

         E%mat(3,nveloc+3) = 1d0-factor
         E%mat(nveloc+6,nveloc+3) = factor

!        --- next the columns

         E%mat(nveloc+3,3) = 1d0-factor
         E%mat(nveloc+3,nveloc+6) = factor

      end if

!     --- element vector

      E%vec(nveloc+1) = u
      E%vec(nveloc+2) = v
      if ( ndim==3 ) E%vec(nveloc+6) = w

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( E%vec, icount, 'elemvc')
         call prinrl1 ( E%mat, icount, icount, 'elemmt')

         write(irefwr,*) 'End elm922'

      end if  ! ( debug )
      call erclos ( 'elm922' )

      end subroutine elm922
