      subroutine sepel900conv1 ( u, submat )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill part of convection tensor for Navier-Stokes into matrix S
!     Old subroutine el3001
! **********************************************************************
!
!                       KEYWORDS
!
!     convection
!     element_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleoldvecs
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision :: u(m,E%ndim)
      double precision, target :: submat(n,n)

!     submat         o    submatrix
!     u              i    array of length m containing the values of the
!                         velocity
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k
      double precision :: h1, h2, h3
      double precision, pointer :: phi(:,:), dphidx(:,:,:), dudx(:,:,:)
      double precision, pointer :: s(:,:)

!     dphidx         derivatives of the basis functions
!     dudx           derivatives of the solution vector in the quadrature
!                    point
!     h1             Help variable to compute the element matrix
!     h2             Help variable to compute the element matrix
!     h3             Help variable to compute the element matrix
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     phi            values of shape functions in quadrature points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!        s  = s   + / (u . grad phi )  phi dx
!         ij   ij   e              j      i
!                               k                k         k
!           = s   + sum  w   u(x ) . grad phi (x )  phi (x )
!              ij    k    k                  j          i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel900conv1' )

      dudx => gradoldvec
      dphidx => B%phider

      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel900conv1'
         write(irefwr,1) 'jdiag, kstep', B%jdiag, kstep
         write(irefwr,1) 'metupw', E%methodupw
         call prinrl1 ( u, E%ndim, m, 'u' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      s => submat
      s = 0d0

      if ( E%methodupw==0 ) then
         phi => B%phi
      else
         phi => B%psi
      end if  ! ( E%methodupw==0 )

      if ( E%methodupw<1000 ) then

!     --- Standard convection

         if ( .not. B%derivconst ) then

!        --- jdercn = 1   derivatives of basis functions are variable

            if ( B%jdiag==1 ) then

!           --- B%jdiag = 1, Newton-Cotes rule  phi (x ) = delta
!                                                i  j         ij

               if ( E%ndim==1 .or. jtrans==31 ) then

!              --- E%ndim = 1, only one component of the velocity vector

                  do i = 1, m, kstep
                     h1 = u(i,1)
                     do j = 1, n
                        s(j,i) = h1 * dphidx(j,i,1)
                     end do
                  end do

               else if ( E%ndim==2 .or. jtrans==32 ) then

!              --- E%ndim = 2, two components of the velocity vector

                  do i = 1, m, kstep
                     h1 = u(i,1)
                     h2 = u(i,2)
                     do j = 1, n
                        s(j,i) = h1 * dphidx(j,i,1) + h2 * dphidx(j,i,2)
                     end do
                  end do
               else

!              --- E%ndim = 3, three components of the velocity vector

                  do i = 1, m, kstep
                     h1 = u(i,1)
                     h2 = u(i,2)
                     h3 = u(i,3)
                     do j = 1, n
                        s(j,i) = h1 * dphidx(j,i,1) + &
                                 h2 * dphidx(j,i,2) + &
                                 h3 * dphidx(j,i,3)
                     end do
                  end do
               end if
            else

!           --- B%jdiag = 2, Gauss rule  phi (x ) # delta
!                                         i  j         ij
               if ( E%ndim==1 .or. jtrans==31 ) then

!              --- E%ndim = 1, only one component of the velocity vector

                  do i = 1, n
                     do j = 1, n
                        s(j,i) = sum ( u(:,1) * dphidx(j,:,1) * phi(i,:) )
                     end do
                  end do
               else if ( E%ndim==2 .or. jtrans==32 ) then

!              --- E%ndim = 2, two components of the velocity vector

                  do i = 1, n
                     do j = 1, n
                        h1 = 0d0
                        do k = kstep, m, kstep
                           h1 = h1 + ( u(k,1) * dphidx(j,k,1) + &
                                       u(k,2) * dphidx(j,k,2) ) &
                                                 * phi(i,k)
                        end do
                        s(j,i) = h1
                     end do
                  end do
               else

!              --- E%ndim = 3, three components of the velocity vector

                  do i = 1, n
                     do j = 1, n
                        s(j,i) = sum ( ( u(:,1) * dphidx(j,:,1) + &
                                     u(:,2) * dphidx(j,:,2) + &
                                     u(:,3) * dphidx(j,:,3) ) * phi(i,:) )
                     end do
                  end do
               end if
            end if
         else

!        --- jdercn = 0   derivatives of basis functions are constant

            if ( B%jdiag==1 ) then

!           --- B%jdiag = 1, Newton-Cotes rule  phi (x ) = delta
!                                                i  j         ij

               if ( E%ndim==1 .or. jtrans==31 ) then

!              --- E%ndim = 1, only one component of the velocity vector

                  do i = 1, m
                     h1 = u(i,1)
                     do j = 1, n
                        s(j,i) = h1 * dphidx(j,1,1)
                     end do
                  end do
               else if ( E%ndim==2 .or. jtrans==32 ) then

!              --- E%ndim = 2, two components of the velocity vector

                  do i = 1, m
                     h1 = u(i,1)
                     h2 = u(i,2)
                     do j = 1, n
                        s(j,i) = h1 * dphidx(j,1,1) + &
                                 h2 * dphidx(j,1,2)
                     end do
                  end do
               else

!              --- E%ndim = 3, three components of the velocity vector

                  do i = 1, m
                     h1 = u(i,1)
                     h2 = u(i,2)
                     h3 = u(i,3)
                     do j = 1, n
                        s(j,i) = h1 * dphidx(j,1,1) + &
                                 h2 * dphidx(j,1,2) + &
                                 h3 * dphidx(j,1,3)
                     end do
                  end do
               end if
            else

!           --- B%jdiag = 2, Gauss rule  phi (x ) # delta
!                                         i  j         ij

               if ( E%ndim==1 .or. jtrans==31 ) then

!              --- E%ndim = 1, only one component of the velocity vector

                  do i = 1, n
                     h1 = sum ( u(:,1) * phi(i,:) )
                     do j = 1, n
                        s(j,i) = h1 * dphidx(j,1,1)
                     end do
                  end do
               else if ( E%ndim==2 .or. jtrans==32 ) then

!              --- E%ndim = 2, two components of the velocity vector

                  do i = 1, n
                     h1 = sum ( u(:,1) * phi(i,:) )
                     h2 = sum ( u(:,2) * phi(i,:) )
                     do j = 1, n
                        s(j,i) = h1 * dphidx(j,1,1) + &
                                 h2 * dphidx(j,1,2)
                     end do
                  end do
               else

!              --- E%ndim = 3, three components of the velocity vector

                  do i = 1, n
                     h1 = sum ( u(:,1) * phi(i,:) )
                     h2 = sum ( u(:,2) * phi(i,:) )
                     h3 = sum ( u(:,3) * phi(i,:) )
                     do j = 1, n
                        s(j,i) = h1 * dphidx(j,1,1) + &
                                 h2 * dphidx(j,1,2) + &
                                 h3 * dphidx(j,1,3)
                     end do
                  end do
               end if
            end if
         end if
      else

!     --- Compressible convection

         if ( .not. B%derivconst ) then

!        --- jdercn = 1   derivatives of basis functions are variable

            if ( B%jdiag==1 ) then

!           --- B%jdiag = 1, Newton-Cotes rule  phi (x ) = delta
!                                                i  j         ij

               if ( E%ndim==1 .or. jtrans==31 ) then

!              --- E%ndim = 1, only one component of the velocity vector

                  do i = 1, m, kstep
                     h1 = u(i,1)
                     do j = 1, n
                        s(i,j) = - h1 * dphidx(j,i,1)
                     end do
                  end do
               else if ( E%ndim==2 .or. jtrans==32 ) then

!              --- E%ndim = 2, two components of the velocity vector

                  do i = 1, m, kstep
                     h1 = u(i,1)
                     h2 = u(i,2)
                     do j = 1, n
                        s(i,j) = - h1 * dphidx(j,i,1) &
                                 - h2 * dphidx(j,i,2)
                     end do
                  end do
               else

!              --- E%ndim = 3, three components of the velocity vector

                  do i = 1, m, kstep
                     h1 = u(i,1)
                     h2 = u(i,2)
                     h3 = u(i,3)
                     do j = 1, n
                        s(i,j) = - h1 * dphidx(j,i,1) &
                                 - h2 * dphidx(j,i,2) &
                                 - h3 * dphidx(j,i,3)
                     end do
                  end do
               end if
            else

!           --- B%jdiag = 2, Gauss rule  phi (x ) # delta
!                                         i  j         ij
               if ( E%ndim==1 .or. jtrans==31 ) then

!              --- E%ndim = 1, only one component of the velocity vector

                  do i = 1, n
                     do j = 1, n
                        s(i,j) = - sum( u(:,1) * dphidx(j,:,1) * phi(i,:) )
                     end do
                  end do
               else if ( E%ndim==2 .or. jtrans==32 ) then

!              --- E%ndim = 2, two components of the velocity vector

                  do i = 1, n
                     do j = 1, n
                        h1 = 0d0
                        do k = kstep, m, kstep
                           h1 = h1 + ( u(k,1) * dphidx(j,k,1) + &
                                       u(k,2) * dphidx(j,k,2) ) &
                                              * phi(i,k)
                        end do
                        s(i,j) = - h1
                     end do
                  end do
               else

!              --- E%ndim = 3, three components of the velocity vector

                  do i = 1, n
                     do j = 1, n
                        s(i,j) = - sum( (u(:,1) * dphidx(j,:,1) + &
                                         u(:,2) * dphidx(j,:,2) + &
                                         u(:,3) * dphidx(j,:,3) ) &
                                                * phi(i,:) )
                     end do
                  end do
               end if
            end if
         else

!        --- jdercn = 0   derivatives of basis functions are constant

            if ( B%jdiag==1 ) then

!           --- B%jdiag = 1, Newton-Cotes rule  phi (x ) = delta
!                                                i  j         ij

               if ( E%ndim==1 .or. jtrans==31 ) then

!              --- E%ndim = 1, only one component of the velocity vector

                  do i = 1, m
                     h1 = u(i,1)
                     do j = 1, n
                        s(i,j) = - h1 * dphidx(j,1,1)
                     end do
                  end do
               else if ( E%ndim==2 .or. jtrans==32 ) then

!              --- E%ndim = 2, two components of the velocity vector

                  do i = 1, m
                     h1 = u(i,1)
                     h2 = u(i,2)
                     do j = 1, n
                        s(i,j) = - h1 * dphidx(j,1,1) &
                                        - h2 * dphidx(j,1,2)
                     end do
                  end do
               else

!              --- E%ndim = 3, three components of the velocity vector

                  do i = 1, m
                     h1 = u(i,1)
                     h2 = u(i,2)
                     h3 = u(i,3)
                     do j = 1, n
                        s(i,j) = - h1 * dphidx(j,1,1) &
                                        - h2 * dphidx(j,1,2) &
                                        - h3 * dphidx(j,1,3)
                     end do
                  end do
               end if
            else

!           --- B%jdiag = 2, Gauss rule  phi (x ) # delta
!                                         i  j         ij
               if ( E%ndim==1 .or. jtrans==31 ) then

!              --- E%ndim = 1, only one component of the velocity vector

                  do i = 1, n
                     h1 = sum(u(:,1) * phi(i,:))
                     do j = 1, n
                        s(i,j) = - h1 * dphidx(j,1,1)
                     end do
                  end do
               else if ( E%ndim==2 .or. jtrans==32 ) then

!              --- E%ndim = 2, two components of the velocity vector

                  do i = 1, n
                     h1 = sum(u(:,1) * phi(i,:))
                     h2 = sum(u(:,2) * phi(i,:))
                     do j = 1, n
                        s(i,j) = - h1 * dphidx(j,1,1) &
                                        - h2 * dphidx(j,1,2)
                     end do
                  end do
               else

!              --- E%ndim = 3, three components of the velocity vector

                  do i = 1, n
                     h1 = sum(u(:,1) * phi(i,:))
                     h2 = sum(u(:,2) * phi(i,:))
                     h3 = sum(u(:,3) * phi(i,:))
                     do j = 1, n
                        s(i,j) = - h1 * dphidx(j,1,1) &
                                        - h2 * dphidx(j,1,2) &
                                        - h3 * dphidx(j,1,3)
                     end do
                  end do
               end if
            end if
         end if
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepel900conv1'

      end if  ! ( debug )
      call erclos ( 'sepel900conv1' )

      end subroutine sepel900conv1
