      subroutine sepel900convsp
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-08-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the convective part of the element matrix for navier-stokes
!     and store in matrix s for 2-d and 3-d elements
!     In this case the special form of the Navier-Stokes equations is used
!     based on the given coefficients for the various terms
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleoldvecs
      use sepmodulebasefn
      use sepmoduleelm
      use sepmodulecomio
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision rhow(m), work(m), rhou(m,E%ndim), elmtsub(n,n)
      integer i, j, k, k1
      double precision, pointer :: s(:,:)

!     elmtsub        Work array in which a sub element matrix is stored
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     k1             First point of curve in KMESH part m
!     rhou           Contains u rho
!     rhow           Contains rho w
!     s              Element matrix to be filled
!     work           Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     SEPEL900CONV1  Fill part of convection tensor for Navier-Stokes into
!                    matrix S
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Computation of the convective matrix results in a 3x3 block matrix, each
!     consisting of nxn matrices of the following shape:
!            |   S       S      S     |
!            |    uu      uv     uw   |
!            |                        |
!            |   S       S      S     |
!     S =    |    vu      vv     vw   |
!            |                        |
!            |   S       S      S     |
!            |    wu      wv     ww   |
!     The submatrices depend on the parameter E%modelconv.
!     The non-linear term has the shape:
!            | u ux     v uy      w uz |
!            |                         |
!       rho  | u vx     v vy      w vz | = rho u . grad u
!            |                         |
!            | u wx     v wy      w wz |
!     If ( E%modelconv=1 ) then
!      rho u . grad u (n+1)  =  rho u(n) . grad u (n+1)
!     If ( E%modelconv=2 ) then
!      rho u . grad u (n+1)  =  rho u(n) . grad u (n+1) + &
!                               rho u(n+1) . grad u (n) - &
!                               rho u(n) . grad u (n)
!     If ( E%modelconv=3 ) then
!      rho u . grad u (n+1)  =  rho u(n+1) . grad u (n)
!     The submatrices rho u(n) . grad u (n+1) are defined by:
!     S uu =  rho u(n) . gradphi(j) phi(i) = Svv = Sww
!     The submatrices rho u(n+1) . grad u (n) are defined by:
!     S uu =  rho ux(n) phi(j) phi(i)
!     S uv =  rho uy(n) phi(j) phi(i)
!     S uw =  rho uz(n) phi(j) phi(i)
!     S vu =  rho vx(n) phi(j) phi(i)
!     S vv =  rho vy(n) phi(j) phi(i)
!     S vw =  rho vz(n) phi(j) phi(i)
!     S wu =  rho wx(n) phi(j) phi(i)
!     S wv =  rho wy(n) phi(j) phi(i)
!     S ww =  rho wz(n) phi(j) phi(i)
!    In case of cylinder co-ordinates the convective terms become
!    ur dur/dr   + uphi/r dur/dphi   + uz dur/dz   - uphi**2/r
!    ur duphi/dr + uphi/r duphi/dphi + uz duphi/dz + ur uphi/r
!    ur duz/dr   + uphi/r duz/dphi   + uz duz/dz
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      jzero = 2
      if ( E%ndim>2 ) then
         write(irefwr,*) 'Array work too small in sepel900convsp'
         stop
      end if

      if ( E%modelconv/=1 ) then

!     --- E%modelconv = 2 or 3, successive substitution part, i.e.
!         rho u(n+1) . grad u (n)

         if ( jcart<=2 ) then

!        --- jcart = 1, Cartesian co-ordinates
!            or jcart = 2, Axi-symmetrical co-ordinates without swirl

            do i = 1, E%ndim
               do j = 1, E%ndim

!              --- multiply the weight by rho and store in rhow

                  rhow(:) = B%w(:)*E%cconv(:,j+(i-1)*E%ndim)*gradoldvec(:,j,i)
                  if ( B%jdiag==1 ) then
                     elmtsub = 0d0
                     do k = 1, B%n
                        elmtsub(k,k) = rhow(k)
                     end do   ! k = 1, n
                  else
                     do k = 1, B%n
                        do k1 = 1, k
                           elmtsub(k,k1) = sum ( work*B%phi(j,:)*B%phi(i,:) )
                           elmtsub(k1,k) = elmtsub(k,k1)
                        end do   ! k1 = 1, n
                     end do   ! k = 1, n
                  end if  ! ( B%jdiag==1 )
                  s => E%submat(i:B%n*E%ndim:E%ndim,i:B%n*E%ndim:E%ndim)
                  s = s + elmtsub(1:B%n,1:B%n)
 !                               E%ndim, .false., 1d0 )
               end do
            end do

         end if

      end if

      if ( E%modelconv<3 ) then

!     --- E%modelconv = 1 or 2, Picard part, i.e.
!         rho u(n) . grad u (n+1)

         if ( jcart<=2 ) then

!        --- jcart = 1, Cartesian co-ordinates
!            or jcart = 2, Axi-symmetrical co-ordinates without swirl

            idim = E%ndim
            jadd = 0

            do i = 1, E%ndim

!           --- multiply u by rho and store in rhou

               do j = 1, E%ndim
                  rhou(:,j) = E%ugaussloc(:,j)*E%cconv(:,j+(i-1)*E%ndim)
               end do
               call sepel900conv1 ( rhou, elmtsub )

               s => E%submat(i:B%n*E%ndim:E%ndim,i:B%n*E%ndim:E%ndim)
               s = s + elmtsub(1:B%n,1:B%n)

            end do

         end if

      end if

      end
