      subroutine sepel900rhs1 ( acof )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the convective part of the element vector for navier-stokes
!     and add in consisting element vector
!     Old subroutine el2011
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     navier_stokes_equation
!     convection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleoldvecs
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision :: acof

!     acof           i    multiplication coefficient
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, kdiag
      double precision :: rhow(m), work(m), elvcsub(n)
      double precision, pointer :: u(:,:), phi(:,:), dphidx(:,:,:), &
                                   dudx(:,:,:), xg(:), vec(:)

!     dphidx         derivatives of the basis functions
!     dudx           derivatives of the solution vector in the quadrature
!                    point
!     elvcsub        sub element vector
!     i              Counting variable
!     kdiag          Indicates if integration points are equal to nodal
!                    points (1) or not (2)
!     phi            values of shape functions in quadrature points
!     rhow           Contains rho w
!     u              velcoity
!     vec            sub element vector
!     work           Work array
!     xg             r-coorindates in Gauss points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPELLOAD2     Fill the element vector with a load
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Computation of the convective vector results in a 3x1 block vector, each
!     consisting of nx1 vectors of the following shape:
!
!            |   F    |
!            |    u   |
!            |        |
!            |   F    |
!     F =    |    v   |
!            |        |
!            |   f    |
!            |    w   |
!
!
!    The sub vectors are defined by:
!
!    Fu(i) = rho ( u du/dx + v du/dy + w du/dz ) phi (i)
!
!    Fv(i) = rho ( u dv/dx + v dv/dy + w dv/dz ) phi (i)
!
!    Fw(i) = rho ( u dw/dx + v dw/dy + w dw/dz ) phi (i)
!
!    For the expression in cylinder co-ordinates see subroutine el2010
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel900rhs1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel900rhs1'
         write(irefwr,*) 'Debug information from el2011'
         write(irefwr,1) 'jcart, m, n, ndimlc, jdiag', &
                          B%cart, m, n, E%ndim, B%jdiag
         write(irefwr,2) 'acof', acof
         call prinrl ( E%rho, m, 'rho' )
         call prinrl ( B%w, m, 'w' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      u => E%ugauss
      dudx => gradoldvec
      dphidx => B%phider

      if ( E%methodupw==0 ) then
         phi => B%phi
         kdiag = B%jdiag
      else
         phi => B%psi
         kdiag = 2
      end if  ! ( E%methodupw==0 )

!     --- multiply the weight by rho and store in rhow

      rhow = E%rho * B%w * acof

      if ( B%cart<=2 ) then

!     --- jcart = 1, Cartesian co-ordinates
!         or jcart = 2, Axi-symmetrical co-ordinates without swirl

         do i = 1, E%ndim

!        --- Put u dui/dx + v dui/dy + w dui/dz in work

            work = u(:,1) * dudx(:,1,i) + u(:,2) * dudx(:,2,i)
            if ( E%ndim==3 ) work = work + u(:,3) * dudx(:,3,i)
            if ( debug ) call prinrl ( work, m, 'work' )

!           --- Compute contribution to one element vector and store in elvcsub

            work = work*rhow
            elvcsub = 0d0
            call sepelload2 ( work, kdiag, phi, elvcsub )
            if ( debug ) call prinrl ( elvcsub, n, 'elvcsub' )

!           --- Add contribution to element vector

            vec => E%subvec(i:E%ndim*n:E%ndim)
            vec = vec+elvcsub

         end do

      else

!     --- jcart = 3, Polar co-ordinates
!         Put u du/dr + v/r du/dy - v**2/r in work(k)

         xg => B%xyzg(:,1)
         work = u(:,1)*dudx(:,1,1) + (u(:,2)*dudx(:,2,1)-u(:,2)**2)/xg
         work = work*rhow
         elvcsub = 0d0
         call sepelload2 ( work, kdiag, phi, elvcsub )

!        --- Add contribution to element vector

         vec => E%subvec(1:E%ndim*n:E%ndim)
         vec = vec+elvcsub

!        --- Put u dv/dr + v/r dv/dy + u*v/r in work(k)

         work = u(:,1)*dudx(:,1,2) + &
                        (u(:,2)*dudx(:,2,2)+u(:,1)*u(:,2))/xg
         work = work*rhow
         elvcsub = 0d0
         call sepelload2 ( work, kdiag, phi, elvcsub )

!        --- Add contribution to element vector

         vec => E%subvec(2:E%ndim*n:E%ndim)
         vec = vec+elvcsub

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepel900rhs1'

      end if  ! ( debug )
      call erclos ( 'sepel900rhs1' )

      end subroutine sepel900rhs1
