      subroutine sepeldivgrad
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the divergence and gradient matrix for Navier-Stokes
!     Incompressible equations
!     Also the pressure matrix is computed
!     Type 900: penalty function approach
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
     integer :: i, j
     double precision :: work(m)

!     i              Counting variable
!     j              Counting variable
!     work           Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL2        Print a real two-dimensional array
!     SEPELDIVGRAD03 compute the divergence and the pressure gradient matrix
!                    for the incompressible Navier-Stokes equations
!                    mcont = 3, i.e. special divergence and gradient matrix
!     SEPELFILLDIV   Fill div matrix, in case of slightly compressible flow
!                    with respect to Crouzeix Raviart elements
!     SEPELGRAD      Fill sp matrix, ie. the gradient of the pressure matrix
!                    with respect to Crouzeix Raviart elements
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeldivgrad' )
      debug = .false.! .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeldivgrad'
         write(irefwr,1) 'mcont, mdiv, npsi', E%typecontinuity, E%mdiv, npsi
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First compute the pressure matrix (grad p)

      if ( E%typecontinuity==3 ) then

!     --- typecontinuity=3, special pressure gradient

         call sepeldivgrad03

      else

!     --- typecontinuity#3, standard pressure gradient

         call sepelgrad

         if ( E%mdiv/=0 ) then

!        --- mdiv#0 Fill rhsdiv

            do i = 1, npsi
               E%rhsdiv(i) = sum(B%w*E%coeffs(:,E%indrhscomp)*B%phipress(i,:))
            end do  ! i = 1, npsi

         end if  ! ( E%mdiv/=0 )

!        --- Perform some extra actions depending on E%typecontinuity

         select case ( E%typecontinuity )
         case(1, 5)

!        --- mcont = 1, 5 fill also the divergence matrix

            call sepelfilldiv

         case(2)

!        --- mcont = 2, fill also the divergence matrix
!                    clear part of sp matrix

            E%divmat = E%gradmat
            E%pressmat(1:E%ndim*n:E%ndim,:) = 0d0

         case(4)

!        --- Fill the values of the basis functions in the Gauss points
!            according to:
!            psi = x  - xcent
!            Fill the pressure matrix according to
!            PP(i,j) = / kappa * psi(i) * psi(j) dx
!                      x

            do j = 1, npsi

               work = B%w*E%coeffs(:,E%indkappa)*B%phipress(j,:)
               do i = 1, npsi
                  E%pressmat(i,j) = sum(work*B%phipress(i,:))
               end do  ! i = 1, npsi

            end do

         end select  ! case ( E%typecontinuity )

      end if  ! ( E%typecontinuity==3 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl2 ( E%gradmat, E%ndim*n, npsi, 'sp' )
         call prinrl2 ( E%divmat, E%ndim*n, npsi, 'div' )
         if ( E%typecontinuity==2 .or. E%typecontinuity==4 ) &
            call prinrl2 ( E%pressmat, npsi, npsi, 'pressmat' )
         write(irefwr,*) 'End sepeldivgrad'

      end if  ! ( debug )
      call erclos ( 'sepeldivgrad' )

      end subroutine sepeldivgrad
