      subroutine sepeleffvisc1
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     computes the effective viscosity depending on the
!     viscosity model used
! **********************************************************************
!
!                       KEYWORDS
!
!     viscosity
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmoduleoldvecs
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k, nunkp, j
      double precision :: term, expn, epsbng, x3, u2, u3, cn, clamb
      double precision, pointer :: eta(:), etaeff(:), secinv(:), u(:,:), &
                                   gradv(:,:), dudx(:,:,:), xgauss(:,:), &
                                   detdsc(:), vecloc(:,:)

!     clamb          Parameter lambda with respect to viscosity model
!     cn             Power in power law model
!     detdsc         Derivative of second invariant per quadrature point
!     dudx           derivatives of the solution vector in the quadrature
!                    point
!     epsbng         Lower threshold value for the value of a parameter
!     eta            viscosity
!     etaeff         Effective viscosity
!     expn           Value of an exponent
!     gradv          Array containing the gradient vector in
!                    the quadrature points
!     j              Counting variable
!     k              Counting variable
!     nunkp          Maximum number of unknowns in a point
!     secinv         Array containing the second invariant in the
!                    quadrature points
!     term           Value of a term in an expression
!     u              velocity
!     u2             Second component of velocity vector
!     u3             Third component of velocity vector
!     vecloc         Work array in which all old solution vectors for the
!                    quadrature points are stored
!     x3             Third component of co-ordinate vector
!     xgauss         gauss co-ordinates
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision fnv000, fnv001, fnv002, fnv003

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     FNV000         User-defined viscosity model (modelv=100)
!     FNV001         User-defined viscosity model (modelv=101)
!     FNV002         User-defined viscosity model (modelv=102)
!     FNV003         User-defined viscosity model (modelv=103)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     234   Computed effective viscosity is less than or equal to zero
!    1430   The viscosity parameter eta is non-positive
!    3143   parameter out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeleffvisc1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeleffvisc1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nunkp = oldsols(iseqprevsol)%nunkp
      eta => E%coeffs(:,E%indeta)
      etaeff => E%etaeff
      gradv => E%gradv
      secinv => E%secinv
      detdsc => E%detdsc
      u => E%ugauss
      dudx => gradoldvec
      xgauss => B%xyzg
      cn = E%coeffs(1,E%indpower)
      clamb = E%coeffs(1,E%indlambda)

      if ( E%ind(E%indeta)>0 ) then

!     --- eta variable, check positivity

         do k = 1, m

            if ( eta(k)<0d0 ) then

               call erreal ( eta(k), 1 )
               call errint ( itype, 1 )
               call errint ( 12, 2 )
               call errint ( ielgrp, 3 )
               call errint ( ielem, 4 )
               call errsub ( 1430, 4, 1, 0 )

            end if

         end do

      end if  ! ( E%indeta>0 )

      select case (E%modelvisc)

      case(1, 5, 10)

!      modelv = 1, 10: newtonian fluid
!      copy eta into etaeff

         etaeff = eta

      case(2)

!     --- E%modelvisc = 2, power law model

         expn = 0.5d0 * ( cn-1d0 )
         if ( expn<=0 ) secinv = max ( secinv, epsmac )

         etaeff = eta * secinv ** expn

         if ( cn>1d0 ) then

!        --- cn>1, use Newton linearization

            expn = 0.5d0 * ( cn-3d0 )
            if ( expn<=0 ) secinv = max ( secinv, epsmac )

            detdsc = eta * (cn-1d0) * secinv ** expn
            etaeff = etaeff + detdsc*secinv

         end if

      case(3)

!     --- E%modelvisc = 3, three parameter carreau liquid

         expn = 0.5d0 * ( cn-1d0 )
         etaeff = eta * (1d0+clamb*secinv) ** expn

      case(4)

!     --- E%modelvisc = 3, three parameter plastico-viscous liquid

         expn = 0.5d0/cn
         epsbng = 100d0 * epsmac
         do k = 1, m
            if ( secinv(k)<epsbng ) then

!           --- secinv nearly zero

               if ( secinv(k)>2d0*epsmac ) then
                  secinv(k) = ( secinv(k)-epsmac ) ** 2 / &
                              ( 2d0 * (epsbng-epsmac) )
               else
                  secinv(k) = epsmac
               end if

            end if
            term = ( clamb**2/(eta(k)**2*secinv(k)) ) ** expn
            etaeff(k) = eta(k) * ( 1d0 + term ) ** cn
         end do

      case(100)

!     --- E%modelvisc = 100, user function fnv000

         do k = 1, m
            etaeff(k) = fnv000 ( gradv(1,k) )
         end do

      case(101)

!     --- E%modelvisc = 101, user function fnv001

         do k = 1, m
            etaeff(k) = fnv001 ( secinv(k) )
         end do

      case(102)

!     --- E%modelvisc = 102, user function fnv002

         x3 = 0d0
         u3 = 0d0
         do k = 1, m
            if ( ndimlc==3 ) then

!           --- ndimlc = 3, fill third component of x and u

               x3 = xgauss(k,3)
               u2 = u(k,2)
               u3 = u(k,3)

            else if ( jcart==4 ) then

!           --- jcart = 4, cylindrical co-ordinates with swirl

               u2 = u(k,3)
               u3 = u(k,2)

            else

               u2 = u(k,2)

            end if
            etaeff(k) = fnv002 ( xgauss(k,1), xgauss(k,2), x3, &
                                 u(k,1), u2, u3, secinv(k) )
         end do

      case(103)

!     --- E%modelvisc = 103, user function fnv003

         allocate ( vecloc(numold,E%maxunk))

         x3 = 0d0
         u3 = 0d0
         do k = 1, m
            if ( ndimlc==3 ) then

!           --- ndimlc = 3, fill third component of x and u

               x3 = xgauss(k,3)
               u2 = u(k,2)
               u3 = u(k,3)

            else if ( jcart==4 ) then

!           --- jcart = 4, cylindrical co-ordinates with swirl

               u2 = u(k,3)
               u3 = u(k,2)

            else

               u2 = u(k,2)

            end if

            do j = 1, numold
               if ( indvector(j)>0 .and. oldsols(j)%iseqtype>0 ) then
                  vecloc(j,:) = oldsols(j)%locvecs(k,1:E%maxunk)
               end if  ! ( indvector(j)>0 .and. oldsols(j)%iseqtype>0 )
            end do  ! j = 1, numold
            etaeff(k) = fnv003 ( xgauss(k,1), xgauss(k,2), x3, &
                                 u(k,1), u2, u3, secinv(k), &
                                 numold, E%maxunk, vecloc )
         end do

         deallocate ( vecloc )

      case default

!     --- other values of E%modelvisc not implemented

         call errint ( E%modelvisc, 1 )
         call errchr ( 'E%modelvisc', 1 )
         call errsub ( 3143, 1, 0, 1 )
         go to 1000

      end select  ! case (E%modelvisc)
      if ( E%modelvisc>=100 ) then

!     --- Check if etaeff is positive

         do k = 1, m

            if ( etaeff(k)<=0d0 ) then

               call erreal ( etaeff(k), 1 )
               call errint ( ielem, 1 )
               call errint ( itype, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 234, 3, 1, 0 )

            end if

         end do

      end if  ! ( E%modelvisc>=100 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeleffvisc1'

      end if  ! ( debug )
      call erclos ( 'sepeleffvisc1' )

      end subroutine sepeleffvisc1
