      subroutine sepelm900mass
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-07-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element mass matrix for the incompressible Navier-Stokes equations
!     Old subroutine elm900massmat
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
!     mass_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmoduleoldvecs
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, iseqvel
      double precision :: adams, work(B%n,B%n), work1(B%m), work2(B%n)
      double precision :: thetdt
      double precision, pointer :: uprevm(:,:), goertler(:), kappa(:)
      double precision, pointer :: s(:,:), f(:)

!     adams          Multiplication factor for the Adams-Bashfort method
!     f              element sub vector
!     goertler       Goertler number
!     i              Counting variable
!     iseqvel        Array containing the positions of the velocities in
!                    the element vector in the sequence required by el2005
!     kappa          Diffusion coefficient
!     s              Element matrix to be filled
!     thetdt         1/(theta dt) for the case of the theta method
!                    immediately applied
!     uprevm         prior solution
!     work           Work array
!     work1          work array
!     work2          work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL2107         Eliminate the centroid point for the incompressible
!                    Navier- Stokes equations in case of an extended quadratic
!                    triangle
!     EL21071        See EL2107 for elements of type 901
!     EL3007         Fills old solution and derivatives
!     EL3013         Fill small matrix elemmtloc of size nesmall x nesmall
!                    into larger matrix elemmt of size nelarge x nelarge
!     ELMASS27       Fill the element mass matrix for the special case of 27
!                    node brick with 3 unknowns per point except for the
!                    centroid, where there are 7 unknowns due to the presence
!                    of the pressure
!     ELMASSDIAG     Copy components-wise defined diagonal element mass matrix
!                    into diagonal element mass matrix
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPEL900RHSG   compute convective terms of Goertler equations
!                    for newton iteration   (k-d)  (right-hand side)
!     SEPELLOAD2     Fill the element vector with a load
!     SEPELMASSMAT4  Fill the element mass matrix in case of one unknown per
!                    node. Consistent mass matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2873   upwind not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm900mass' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm900mass'
         write(irefwr,1) 'imas, m, n, ndim, jtrans, itype', &
                          imas, m, n, ndim, jtrans, itype
         write(irefwr,2) 'theta, tstep', theta, tstep
         call prinrl ( E%rho, m, 'E%rho' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( E%methodupw>0 ) then

!     --- Upwind not yet implemented

         call errchr ( 'Mass matrix', 1 )
         call errsub ( 2873, 0, 0, 1 )
         go to 1000

      end if  ! ( E%methodupw>0 )

      thetdt = 1d0 / (theta * tstep)
      if ( E%seq_second_old_sol>0 ) &
         uprevm => oldsols(E%seq_second_old_sol)%locvecsnode ! (1:n,1:E%maxunk)
      goertler => E%coeffs(:,E%indgoertler)
      kappa => E%coeffs(:,E%indkappa)

      if ( jtrans==1 ) E%submass = 0

      jadd = 0
      if ( imas==1 ) then

!     --- diagonal mass matrix

         jconsf = E%typetime
         work1 = B%w*E%rho
         call sepelload2 ( work1, B%jdiag, B%phi, work2 )
         call elmassdiag ( E%mmat, work2, iseqvel )

      else

!     --- non-diagonal mass matrix

         work1 = B%w*E%rho
         call  sepelmassmat4 ( work1, work )
         if ( debug ) call prinrl1 ( work, n, n, 'work' )

         jadd = 0
         if ( ndim==3 .and. inpelm==27 .and. &
              (itype==902 .or. itype==201) ) then

!        --- Special case, because centroid is not the last point

            call elmass27 ( E%mmat, work )

         else

!        --- Standard case

            do i = 1, ndim

               if ( jtrans==1 ) then

!              --- jtrans = 1, unknowns are eliminated by static condensation

                  s => E%submass(i:B%n*E%ndim:E%ndim,i:B%n*E%ndim:E%ndim)
                  s = work(1:B%n,1:B%n)

               else if ( itype==903 .or. itype==202 ) then

!              --- Type 202, 903, (Taylor-Hood element) use iseqvel

                  call el3013 ( E%mmat, work, icount, n, &
                                iseqvel, i, .false., 1d0 )

               else

!              --- Standard case

                  s => E%mmat(i:icount:E%ndim,i:icount:E%ndim)
                  s(1:B%n,1:B%n) = work(1:B%n,1:B%n)

               end if  ! ( jtrans==1 )

            end do

         end if

      end if

      if ( E%typetime>100 ) then

!     --- E%typetime>100, explicit theta method
!         Update matrix, i.e. add 1/(theta dt) M to the stiffness matrix

         do i = 1,ndim
            s => E%submat(i:B%n*E%ndim:E%ndim,i:B%n*E%ndim:E%ndim)
            s = s+thetdt*work(1:B%n,1:B%n)
         end do

!        --- Compute 1/(theta dt) M u(n) and add to right-hand side

         jadd = 1
         do i = 1,ndim

            if (E%seq_second_old_sol/=0) then

!           --- iterative procedure adopted for dynamic analysis:
!               use velocity vector at end of previous time step

               f => E%subvec(i:B%n*E%ndim:E%ndim)
               f = f + thetdt*matmul(work,uprevm(:,i))

            else

!           --- standard case, no iterations: uprev = uprevm

               f => E%subvec(i:B%n*E%ndim:E%ndim)
               f = f + thetdt*matmul(work,uprev(:,i))

            end if

         end do

      end if

      if ( jtrans==1 .and. E%imas==2 ) then

!     --- Eliminate centroid in case of 2D quadratic elements
!         The resulting matrix is stored in E%mmat

          E%tmpsmat => E%submass
          E%tmpmat => E%mmat
          call sepelm900cent

      end if

1000  if ( debug ) then

!     --- Debug information

         if ( imas==1 ) then

!        --- imas = 1, diagonal mass matrix

           call prinrl(E%mmat,icount,'E%mmat')

         else

!        --- imas = 2, non-diagonal mass matrix

            call prinrl1(E%mmat,icount,icount,'E%mmat')

         end if
         write(irefwr,*) 'End sepelm900mass'

      end if  ! ( debug )
      call erclos ( 'sepelm900mass' )
      end
