      subroutine sepelm900mati
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-07-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create element matrix for Navier-Stokes equations in case of integrated
!     method using separate submatrices
!     Old subroutine elm900mat
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: elemtemp(:,:), elemmtcp(:,:)
      integer :: signdiv, ih

!     elemmtcp       Work array to store a copy of the element matrix
!     elemtemp       Work array to store a copy of the element matrix
!     ih             Help variable to store a constant temporarily
!     signdiv        Multiplication sign of "continuity equations"
!                    if -1 the stokes matrix is symmetric
!                    if 1 all eigenvalues are in the positive half plane
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
!     SEPEL900ST27   Put stiffness matrix and part due to gradient and
!                    divergence into element matrix for 27-point
!                    Navier-Stokes equation
!     SEPELM900ELI   Eliminate the centroid point for the incompressible
!                    Navier-Stokes equations in case of an extended quadratic
!                    triangle, quadrilateral or 27 node brick
!     SEPELM900PUTPR Put gradient and divergence matrix into large matrix for
!                    the Navier-Stokes equations
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm900mati' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm900mati'
         write(irefwr,1) 'jtrans, inpelm', jtrans, inpelm
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      signdiv = -1

      if ( jtrans==1 ) then

!     --- jtrans = 1, eliminate centroid and new element matrix into
!                     elemmtcp

         ih = (n-1)*E%ndim
         E%tmpmat => E%submat(1:ih,1:ih)
         E%tmpsmat => E%submat
 
         call sepelm900cent

!        --- Copy back in elemtemp

         elemtemp => E%tmpmat

      else

!     --- jtrans = 0, no transformation, just copy

         elemtemp => E%submat

      end if


      if ( inpelm==27 ) then

!     --- inpelm = 27, special case since centroid is not the end point

         call sepel900st27 ( elemtemp, signdiv )

      else

!     --- inpelm # 27, standard situation
!         First put the velocity stiffness matrix into the large matrix

         if ( debug ) &
            call prinrl1 ( elemtemp, size(elemtemp,1), size(elemtemp,1), &
                           'elemtemp' )
         ih = E%ndim*(n-jtrans)
         E%mat(1:ih,1:ih) = elemtemp
         if ( debug ) &
            call prinrl1(E%mat, icount, icount, 'E%mat/1' )

!        --- Next put gradient, divergence and pressure matrix into
!            E%mat

         call sepelm900putpr ( signdiv )
         if ( debug ) &
            call prinrl1(E%mat, icount, icount, 'E%mat/2' )

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( E%mat, icount, icount, 'E%mat' )
         write(irefwr,*) 'End sepelm900mati'

      end if  ! ( debug )
      call erclos ( 'sepelm900mati' )

      end subroutine sepelm900mati
