      subroutine sepelm900putpr ( signdiv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-07-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Put gradient and divergence matrix into large matrix for the
!     Navier-Stokes equations
!     Old subroutine el4921
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmoduleelm
      use sepmodulecactl
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: signdiv

!     signdiv        i    Multiplication sign of "continuity equations"
!                         if -1 the stokes matrix is symmetric
!                         if 1 all eigenvalues are in the positive half plane
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, isigndiv
      double precision diag(E%npress)

!     diag           Diagonal of the pressure mass matrix
!     i              Counting variable
!     isigndiv       signdiv except for E%typecontinuity = 5, then 1
!     j              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The Navier-Stokes matrix has the shape:
!            |               T   |
!            |   S        -SP    |    u
!     S   =  |    uu             |
!            |                   |
!            |   DIV       PP    |    p
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm900putpr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm900putpr'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialize isigndiv
!         Must be 1 in case of E%typecontinuity = 5

      isigndiv = signdiv
      if ( E%typecontinuity==5 ) isigndiv = 1

!     --- divergence matrix, mark that the matrix E%mat must be used
!         in transposed form

      if ( ndimlc==2 ) then

!     --- ndim = 2

         do j = 1, E%npress
            do i = 1, E%nveloc
               E%mat(i,icount-E%npress+j) = signdiv*E%divmat(i,j)
            end do
         end do

!        --- gradient matrix

         do j = 1, E%npress
            do i = 1, E%nveloc
               E%mat(icount-E%npress+j,i) = -E%gradmat(i,j)
            end do
         end do

!        --- pressure matrix

         if( E%typecontinuity==4 .or. E%typecontinuity==5 )then

!        --- E%typecontinuity = 4 or 5, add pressure mass matrix

            do j = 1, E%npress
               do i = 1, E%npress
                  E%mat(icount-E%npress+j,icount-E%npress+i) = &
                     isigndiv*E%pressmat(i,j)
               end do
            end do
         end if

      else

!     --- ndim = 3

         do j = 1, E%npress
            do i = 1, E%nveloc
               E%mat(i,icount-E%npress+j) = signdiv*E%divmat(i,j)
            end do
         end do

!        --- gradient matrix

         do j = 1, E%npress
            do i = 1, E%nveloc
               E%mat(icount-E%npress+j,i) = -E%gradmat(i,j)
            end do
         end do

!        --- pressure matrix

         if( E%typecontinuity==4 .or. E%typecontinuity==5 )then

!        --- E%typecontinuity = 4 or 5, add pressure mass matrix

            do j = 1, E%npress
               do i = 1, E%npress
                  E%mat(icount-E%npress+j,icount-E%npress+i) = &
                     isigndiv*E%pressmat(i,j)
               end do
            end do

         end if

      end if

!     --- Add penalty parameter to diagonal of pressure part

      if ( E%penalp>0d0 ) then

!     --- E%penalp>0, there is a penalty contribution

         do i = 1, E%npress
            diag(i) = 0d0
         end do
         do k = 1, m
            diag(1) = diag(1) + B%w(k)
         end do

         do i = 1, E%npress-1
            do k = 1, m
               diag(i+1) = diag(i+1) + B%w(k)*(B%xyzg(k,i)-B%xyz(n,i))**2
            end do
         end do

!        --- store the diagonal multiplied by E%penalp in diag

         do i = 1, E%npress
            diag(i) = diag(i) * E%penalp
         end do

         do j = 1, E%npress
            E%mat(icount-E%npress+j,icount-E%npress+j) = &
               E%mat(icount-E%npress+j,icount-E%npress+j) + diag(j)*signdiv
         end do

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelm900putpr'

      end if  ! ( debug )
      call erclos ( 'sepelm900putpr' )

      end subroutine sepelm900putpr
