      subroutine sepelm900vecdi
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the penalty right-hand side for the incompressible Navier-Stokes
!     equations and add to the existing element vector
!     Old subroutine el4900
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, icent
      double precision :: diag(4), signdiv
      double precision, pointer :: rhsdivtr(:), sp(:,:), rhsdiv(:)

!     diag           Diagonal of the large matrix
!     i              Counting variable
!     icent          Index of centroid
!     j              Counting variable
!     rhsdiv         Divergence right-hand side (if mdiv=1)
!     rhsdivtr       Transformed divergence right-hand side (if mdiv=1)
!     signdiv        Multiplication sign of "continuity equations"
!                    if -1 the stokes matrix is symmetric
!                    if 1 all eigenvalues are in the positive half plane
!     sp             Gradient of pressure matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The Navier-Stokes matrix has the shape:
!            |               T   |
!            |   S        -SP    |    u
!     S   =  |    uu             |
!            |                   |
!            |   DIV        0    |    p
!     In the penalty method the zero matrix is repalced by eps M
!                     T
!     Hence Suu u - SP p = f
!           SP  u        = fd
!     is approximated by
!           Suu u -    SP p  = f
!           DIV u + eps M p  = fd
!                    -1
!    or:  p = 1/eps M   ( fd - DIV u )
!                       T  -1                -1   T
!       ( Suu + 1/eps SP  M   DIV ) u = f + M   SP  fd /eps
!    The matrix M is arbitrary but should approximate the mass matrix for
!    the pressure (related to the pressure basis functions) and must be easily
!    invertable
!    M is chosen to be a diagonal matrix containing the diagonal of the
!    pressure mass matrix, so
!    M(i,i) = / psi  psi  dx
!             x    i    i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm900vecdi' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm900vecdi'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      signdiv = -1
      rhsdiv => E%rhsdiv

      if ( itype==900 ) then

!     --- Type 900

         rhsdivtr => E%rhsdivtr
         sp => E%gradmat
         if ( jtrans==1 ) then

!        --- 2D case, at this moment restricted to 7 point extended quadratic
!                     triangle
!                     The centroid will be eliminated, hence only the constant
!                     part of the pressure is taken into account
!                     Store multiplication factor in sum1

            diag(1) = 1d0/(E%penalp*sum(B%w)*2d0)

!           --- Add contribution of penalty right-hand side

            do i = 1, (n-1)*E%ndim
               do j = 1, E%ndim
                  E%vec(i) = E%vec(i) + sp(i,j) * diag(1) * rhsdivtr(j)
               end do
            end do

         else

!        --- 2D and 3D case without elimination
!            Compute the diagonal of the mass matrix

            if ( E%ndim==2 ) then
               icent = n
            else
               if ( E%ishape==14 ) then
                  icent = 14
               else
                  icent = n
               end if
            end if

            diag(1) = sum(B%w)

            do i = 1, E%npress-1
               diag(i+1) = sum(B%w*(B%xyzg(:,i)-B%xyz(icent,i))**2)
            end do  ! i = 1, E%npress-1

!           --- store the inverse divided by eps in diag

            do i = 1, E%npress
               diag(i) = 1d0 / ( diag(i) * E%penalp )
            end do  ! i = 1, E%npress-1

!           --- compute the penalty matrix and add to the element matrix

            do i = 1, n*E%ndim
               E%vec(i) =E%vec(i)  + sum(sp(i,:) * diag * rhsdivtr)
            end do  ! i = 1, E%npress-1

         end if  ! ( jtrans==1 )

      else if ( E%npelm==27 ) then

!     --- 27-point element

         do i = 1, E%npress
            E%vec(i+E%ndim*13) = rhsdiv(i)*signdiv
         end do

      else

!     --- Standard case

         do i = 1, E%npress
            E%vec(icount-E%npress+i) = rhsdiv(i)*signdiv
         end do

      end if  ! ( itype==900 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelm900vecdi'

      end if  ! ( debug )
      call erclos ( 'sepelm900vecdi' )

      end subroutine sepelm900vecdi
