      subroutine sepelnsa1 ( gradv, a1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-09-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the A1 tensor, defined by grad v + grad v^T
!     Navier-Stokes equations
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     gradient
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: gradv(9,m)
      double precision, intent(out) :: a1(9,m)

!     a1             o    A1 tensor defined by gradv + gradv^T
!     gradv          i    Array containing the gradient vector in
!                         the quadrature points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The A1 tensor is defined as grad v + grad v^T
!     This tensor is symmetric
!     The sequence is the same as for the stress tensor, hence
!     2d cartesian:  A1_xx, A1_yy, 0, A1_xy
!     3d cartesian:  A1_xx, A1_yy, A1_zz, A1_xy, A1_yz, A1_zx
!     Axisymmetric coordinates: A1_rr, A1_zz, A1_phi_phi, A1_rz,
!                               A1_z_phi, A1_phi_r
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelnsa1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelnsa1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ndimlc==2 .and. (jcart==1 .or. jcart==3) ) then

!     --- ndimlc = 2, Cartesian or Polar co-ordinates

         a1(1,:) = 2d0*gradv(1,:)
         a1(2,:) = 2d0*gradv(4,:)
         a1(4,:) = gradv(2,:)+gradv(3,:)

      else if ( ndimlc==2 .and. jcart==2 ) then

!     --- ndimlc = 2, Cylinder co-ordinates

         a1(1,:) = 2d0*gradv(1,:)
         a1(2,:) = 2d0*gradv(4,:)
         a1(3,:) = 2d0*gradv(5,:)
         a1(4,:) = gradv(2,:)+gradv(3,:)

      else

!     --- ndimlc = 3

         if ( itype==410 ) then

!        --- itype = 410, the sequence of the stress tensor is defined
!            different from usual

            a1(1,:) = 2d0*gradv(1,:)
            a1(4,:) = 2d0*gradv(5,:)
            a1(6,:) = 2d0*gradv(9,:)
            a1(2,:) = gradv(2,:)+gradv(4,:)
            a1(3,:) = gradv(3,:)+gradv(7,:)
            a1(5,:) = gradv(6,:)+gradv(8,:)

         else

!        --- standard situation

            a1(1,:) = 2d0*gradv(1,:)
            a1(2,:) = 2d0*gradv(5,:)
            a1(3,:) = 2d0*gradv(9,:)
            a1(4,:) = gradv(2,:)+gradv(4,:)
            a1(5,:) = gradv(6,:)+gradv(8,:)
            a1(6,:) = gradv(3,:)+gradv(7,:)

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelnsa1'

      end if  ! ( debug )
      call erclos ( 'sepelnsa1' )

      end subroutine sepelnsa1
