      subroutine sepelnsderiv ( icheld, modelv, eta, etaeff, cn, clamb, x, u, &
                                dudx, gradv, ndcomp, elemvc, nelemvc, &
                                vecloc, maxunk, numold, mdecl, mcont )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-09-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute some derivatives for the Navier-Stokes equations
!     Old subroutine el4917
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     navier_stokes_equation
!     viscosity
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: modelv, ndcomp, icheld, nelemvc, numold, maxunk, mdecl, &
                 mcont
      double precision :: eta(m), etaeff(m), cn, clamb, x(mdecl,*), &
                          u(mdecl,*), dudx(mdecl,ndimlc,*), &
                          gradv(9,m), elemvc(ndcomp,nelemvc), &
                          vecloc(numold,maxunk,*)

!     clamb          i    Parameter lambda with respect to viscosity model
!     cn             i    Power in power law model
!     dudx           i    Real m x ndim  array in which the values of the
!                         derivatives of the solution vector in the integration
!                         points are stored in the sequence:
!                         du / dx  (x ) = dudx ( k, j )
!                           j     k
!                         If jdercn = 0 in common block celint, then only
!                         dudx (1,*) is filled because dudx is then a constant
!                         per element
!     elemvc         o    element vector containing the derivatives for
!                         subroutine deriva
!     eta           i    Array containing the viscosity in the integration
!                         points
!     etaeff         o    Array containing the computed effective viscosity in
!                         the integration points
!     gradv          o    Array containing the gradient vector in
!                         the integration points
!     icheld         i    Choice parameter to distinguish between the various
!                         possibilities of this subroutine
!                         Definitions:
!                          6:   compute the tensor t: t=eta(grad(u)+gradT(u))
!                               t11 = 2 eta gradu(1,1)
!                               t22 = 2 eta gradu(2,2)
!                               t33 = 2 eta gradu(3,3)
!                               t12 =   eta(gradu(1,2)+gradu(2,1))
!                               t23 =   eta(gradu(2,3)+gradu(3,2))
!                               t31 =   eta(gradu(3,1)+gradu(1,3))
!                          8:   compute the rate of elongation
!                               ndimlc = 2, jcart # 4:
!                               e = (u(1)**2*gradu(1,1) + u(2)**2*gradu(2,2) + &
!                                    u(1)*u(2)*(gradu(1,2)+gradu(2,1)))/||u||*2
!                               ndimlc = 3 or jcart = 4:
!                               e = (u(1)**2*gradu(1,1) + u(2)**2*gradu(2,2) + &
!                                    u(3)**2*gradu(3,3) + u(1)*u(2)*(gradu(1,2)+ &
!                                    gradu(2,1)) + &
!                                    u(1)*u(3)*(gradu(1,3)+gradu(3,1))+ &
!                                    u(2)*u(3)*(gradu(2,3)+gradu(3,2)))/||u||*2
!                          9:   compute the shear rate gamma
!                               ndimlc = 2, jcart # 4:
!                               g = (2u(1)u(2)(-gradu(1,1)+gradu(2,2)) + &
!                                    u(1)**2-u(2)**2(gradu(1,2)+gradu(2,1)))/
!                                   ||u||*2
!                               ndimlc = 3 or jcart = 4:
!                               g1 = gradu(1,2) + gradu(2,1)
!                               g2 = gradu(2,3) + gradu(3,2)
!                               g3 = gradu(3,1) + gradu(1,3)
!                         10:   compute the square root of the second invariant
!                         11:   compute the viscous dissipation: 1/2 t:A1
!                               A1 = grad v + grad T v
!                               So: viscous dissipation = eta II
!     maxunk         i    Maximum number of unknowns per point
!     mcont          i    Defines the type of continuity equation
!                         Possible values:
!                         0: incompressible flow
!                         1: compressible flow: div (rho u ) = 0
!     mdecl          i    Formal declaration parameter for the arrays (=m)
!     modelv         i    Defines the type of viscosity model
!                         Possible values:
!                           1:  newtonian liquid
!                           2:  power law liquid
!                           3:  carreau liquid
!                           4:  plastico viscous liquid
!                         100:  user provided model depending on grad u
!                         101:  user provided model depending on the second
!                               invariant
!                         102:  user provided model depending on the second
!                               invariant the co-ordinates and the velocity
!     ndcomp         i    Indicates the number of quantities that must be filled
!                         in elemvc and elemwg per nodal point
!     nelemvc        i    Length parameter for elemvc
!     numold         i    Number of vectors that are stored in ISLOLD
!     u              i    array of length m x ncomp in which the values of
!                         u in the integration points are stored the sequence:
!                         u  (x ) = u(j,i)  j = 1(1)m, i = 1 (1) ncomp
!                          i   j
!     vecloc         i    In this array the values of the function in the
!                         integration points is stored.
!                         vecloc(i,j,k) corresponds to the kth integration
!                         point, the ith vector and the jth unkwon corresponding
!                         to that vector
!     x              i    array containing the coordinates of the nodal points
!                         according to:
!                         x  = x (i,1);  y  = x (i,2);  z  = x (i,3)
!                          i              i              i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: secinv(m), detdsc(m), a1(9,m), t(9,m)
      logical :: compsc, compa1

!     a1             A1 tensor defined by gradv + gradv^T
!     compa1         Indicates if the A1 tensor must be computed or not
!     compsc         Indicates if the second invariant must be computed (true)
!                    or not
!                    they are not
!     detdsc         Derivative of second invariant per quadrature point
!     secinv         Array containing the second invariant in the
!                    integration nodes
!     t              Contains stress tensor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPELNSA1      Compute the A1 tensor, defined by grad v + grad v^T
!     SEPELNSDEFOR   Compute the deformation for the Navier-Stokes equations
!     SEPELNSELONG   Compute the elongation for the Navier-Stokes equations
!     SEPELNSGRAD    Compute gradient tensor in case of Navier-Stokes
!     SEPELNSSECINV  Compute second inverse in case of Navier-Stokes
!     SEPELNSSTRTENS Compute element stress tensor for Navier-Stokes
!     SEPELNSVISDISP compute viscous dissipation for Navier-Stokes
!     SEPELVISC      Compute effective viscosity depending on the viscosity
!                    model
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     406   Value of icheld not allowed
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelnsderiv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelnsderiv'
         write(irefwr,*) 'Debug information from el4917'
         write(irefwr,1) 'icheld, modelv, jcart, ndimlc', &
                          icheld, modelv, jcart, ndimlc
         write(irefwr,1) 'ndcomp, nelemvc', ndcomp, nelemvc
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute the gradient tensor and store in gradv

      call sepelnsgrad ( gradv, x, u, dudx, mdecl )

!     --- If necessary also the second inverse

      compsc = icheld==10 .or. icheld==11 .or. &
               icheld==6 .and. modelv/=100 .and. modelv>1 .and. &
               modelv/=5 .and. modelv/=10
      compa1 = compsc .or. icheld==6 .or. icheld==8
      if ( debug ) write(irefwr,*) 'compsc', compsc

      if ( compa1 ) call sepelnsa1 ( gradv, a1 )
      if ( compsc ) call sepelnssecinv ( a1, secinv )


      select case(icheld)

      case(6,11)

!     --- icheld = 6, 11 fill etaeff and compute stress tensor or viscous
!                        dissipation

         elemvc = 0d0

         call sepelvisc ( modelv, eta, etaeff, cn, clamb, x, u, &
                          dudx, secinv, detdsc, gradv, vecloc, maxunk, &
                          numold )
         if ( ierror/=0 ) go to 1000

         if ( icheld==6 ) then

!        --- icheld = 6, Fill array elemvc with the stress tensor

            call sepelnsstrtens ( elemvc, etaeff, gradv, x, u, a1, &
                                  nelemvc, mcont, ndcomp, mdecl )

         else

!        --- icheld = 11, compute viscous dissipation

            call sepelnsstrtens ( t, etaeff, gradv, x, u, a1, &
                                  m, mcont, 9, mdecl )
            call sepelnsvisdisp ( elemvc, t, a1, nelemvc, mcont, ndcomp )

         end if

      case(8)

!     --- icheld = 8, compute the elongation

         call sepelnselong ( elemvc, a1, u, nelemvc, ndcomp, mdecl )

      case(9)

!     --- icheld = 9, compute the deformation

         call sepelnsdefor ( elemvc, gradv, u, nelemvc, ndcomp, mdecl )

      case(10)

!     --- icheld = 10, compute the sqrt(secinv)

         elemvc(1,:) = sqrt(secinv)

      case default

!     --- Value of icheld not yet implemented

         call errint ( icheld, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 406, 3, 0, 0 )

      end select  ! case(icheld)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelnsderiv'

      end if  ! ( debug )
      call erclos ( 'sepelnsderiv' )

      end subroutine sepelnsderiv
