      subroutine sepelnsstraxi ( s, phix, phi, xgauss, eta, w, mcont )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-09-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill stress tensor for Navier-Stokes into matrix S
!     Axisymmetric coordinates
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision phix(n,m,ndimlc), phi(n,m), eta(m), w(m), &
                       s(ndimlc*n,ndimlc*n), xgauss(m,ndimlc)
      integer mcont

!     eta            i    Array containing the viscosity in the integration
!                         points
!     mcont          i    Defines the type of continuity equation
!                         See elm900
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phix           i    Array of length n x m x ndim  containing the
!                         derivatives of the basis functions in the sequence:
!                         d phi / dx (xg ) = phix (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = phix (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = phix (i,k,3);
!                              i        k
!                         If the element is a linear tetrahedron (n=4), then
!                         the derivatives are constant and only k = 1 is filled.
!     s             i/o   Element matrix to be filled
!     w              i    array of length m containing the weights for
!                         integration
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j
      double precision :: sw (n,n), work(m), work1(m), h

!     h              Help variable to store a constant
!     i              Counting variable
!     j              Counting variable
!     sw             Work array in which a sub element matrix is stored
!     work           Work array to store w eta
!     work1          Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Computation of the stiffness matrix results in a 2 x 2 block matrix, each
!     consisting of nxn matrices of the following shape:
!            |   S       S     |
!            |    uu      uv   |
!     S =    |                 |
!            |   S       S     |
!            |    vu      vv   |
!     The submatrices are defined by:
!              T
!     S    =  S
!      xy      yx
!     2D Cylinder co-ordinates (r, z) without swirl
!     S uu =  2 eta phir phir + eta phiz phiz + 2 eta phi phi /r**2
!     S uv =    eta phir(j) phiz(i)
!     S vu =    eta phiz(j) phir(i)
!     S vv =  2 eta phiz phiz + eta phir phir
!
!    In case of compressible flow the stress term is changed from
!
!    T = eta ( grad v + grad v^T ) into
!
!    T = eta ( grad v + grad v^T ) - 2/3 eta div v I
!
!    with v the velocity
!
!    This means that the following extra terms are added:
!
!    -2/3 eta div(phi(j)) div(phi(i))
!
!    div (v) = d(r v_r)/rdr + dv_z/dz = 1/r*dv_r/dr + v_r + dv_z/dz
!
!
!             | G_j G_i      phi_jy G_i     |
!   -2/3 eta  |                             |
!             | G_j phi_iy   phi_jy phi_iy  |
!
!   with G = phi_ix + phi_i/r
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelnsstraxi' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelnsstraxi'
         write(irefwr,1) 'mcont', mcont
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- 2D Cylinder co-ordinates
!         eta phir phir

      work = w*eta

      do i = 1, n
         do j = 1, i
            sw(i,j) = sum(work*phix(i,1:m,1)*phix(j,1:m,1))
         end do
      end do

!     --- Put in S_rr and S_zz

      do i = 1, n
         do j = 1, i
            s(2*i-1,2*j-1) = 2d0*sw(i,j)
            s(2*i,2*j) = sw(i,j)
         end do
      end do

!     --- 2 eta phi phi / r**2

      work1 = 2d0*work/xgauss(:,1)**2
      do i = 1, n
         do j = 1, i
            sw(i,j) = sum(work1*phi(i,:)*phi(j,:))
         end do
      end do

      do i = 1, n
         do j = 1, i
            s(2*i-1,2*j-1) = s(2*i-1,2*j-1)+sw(i,j)
         end do
      end do

!     --- eta phiz phiz

      if ( mcont==1 ) then
         h = 4d0/3d0
      else
         h = 2d0
      end if  ! ( mcont==1 )

      do i = 1, n
         do j = 1, i
            sw(i,j) = sum(work*phix(i,1:m,2)*phix(j,1:m,2))
         end do
      end do

      do i = 1, n
         do j = 1, i
            s(2*i-1,2*j-1) = s(2*i-1,2*j-1)+sw(i,j)
            s(2*i,2*j) = s(2*i,2*j)+h*sw(i,j)
         end do
      end do

!     --- S uv =  eta phir phiz

      do i = 1, n
         do j = 1, n
            sw(j,i) = sum(work*phix(i,1:m,1)*phix(j,1:m,2))
         end do
      end do

!     --- Put in S_uv

      do i = 1, n
         do j = 1, i
            s(2*i,2*j-1) = sw(j,i)
            s(2*i-1,2*j) = sw(i,j)
         end do
      end do

      if ( mcont==1 ) then

!     --- compressible flow -2/3 eta div U div V
!
!             | G_j G_i      phi_jy G_i     |
!   -2/3 eta  |                             |
!             | G_j phi_iy   phi_jy phi_iy  |
!
!   with G = phi_ix + phi_i/r

         work = -2d0/3d0*work
         do i = 1, n
            do j = 1, i
               sw(j,i) = sum(work*(phix(j,1:m,1)+phi(j,:)/xgauss(:,1))*&
                                  (phix(i,1:m,1)+phi(i,:)/xgauss(:,1)) )
            end do
         end do
         do i = 1, n
            do j = 1, i
               s(2*i-1,2*j-1) = s(2*i-1,2*j-1)+sw(j,i)
            end do
         end do

         do i = 1, n
            do j = 1, n
               sw(j,i) = sum(work*(phix(j,1:m,1)+phi(j,:)/xgauss(:,1))*&
                                   phix(i,:,2) )
            end do
         end do

         do i = 1, n
            do j = 1, i
               s(2*i,2*j-1) = s(2*i,2*j-1)+sw(j,i)
               s(2*i-1,2*j) = s(2*i-1,2*j)+sw(i,j)
            end do
         end do

      end if  ! ( mcont==1 )

!     --- Fill upper half of matrix

      do i = 1, 2*n
         do j = 1, i
            s(j,i) = s(i,j)
         end do  ! j = 1, i
      end do  ! i = 1, 2*n

1000  call erclos ( 'sepelnsstraxi' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelnsstraxi'

      end if  ! ( debug )

      end subroutine sepelnsstraxi
