      subroutine sepelstrrhs ( stress, elemvec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-07-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute right-hand side contribution of a given initial stress tensor
!     Ad-hoc test version, only meant for two-dimensional Cartesian Navier-
!     Stokes
!     Old subroutine elstrrhs
! **********************************************************************
!
!                       KEYWORDS
!
!     stress
!     element_vector
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmoduleelm
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision stress(B%n,6), elemvec(E%ndim,B%n)

!     elemvec        o    element vector stored as two-dimensional vector
!     stress         i    Array of size 6 x nnodes containing the stress tensor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision dpidxint, dpidyint, strssg(6,B%m), stressloc(3)
      integer i, k, j, ivec, number
      double precision, pointer :: vecoldlc(:)

!     dpidxint       Contains the integral of dphi/dx_i over the element
!     dpidyint       Contains the integral of dphi/dy_i over the element
!     i              Counting variable
!     ivec           Sequence number of vector
!     j              Counting variable
!     k              Counting variable
!     number         Sequence number of stress vector
!     stressloc      Local stress vector
!     strssg         Stress in Gauss integration points
!     vecoldlc       Vector corresponding to ivec
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: inivec

!     INIVEC         Return with vector number corresponding to ivecin
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( E%modelrhsd==0 ) then

!     --- Modelrhsd = 0, stress is stored as 3 constants per element

      number = E%indstress
      ipos = E%ivecnums(number)
      ivec = inivec ( E%iuser(ipos) )
      iseqtype = oldsols(ivec)%iseqtype
      index3n => indoldsols(iseqtype)%index3
      vecoldlc => oldsols(ivec)%oldvecs
      stressloc(1:3) = vecoldlc ( index3n(1:3) )
         do i = 1, B%n
            dpidxint = 0d0
            dpidyint = 0d0
            do  k = 1, B%m
               dpidxint = dpidxint + B%w(k)*B%phider(i,k,1)
               dpidyint = dpidyint + B%w(k)*B%phider(i,k,2)
            end do
            elemvec(1,i) = elemvec(1,i) - &
                          stressloc(1)*dpidxint-stressloc(2)*dpidyint
            elemvec(2,i) = elemvec(2,i) - &
                          stressloc(2)*dpidxint-stressloc(3)*dpidyint
         end do

      else

!     --- Modelrhsd = 1, stress is stored completely

         if ( B%cart==1 ) then

            if ( E%ndim==2 ) then

!           --- Interpolate stress vector from nodal points to
!               integration points

               do j = 1, 6
                  do k = 1, B%m
                     strssg(j,k) = 0.d0
                     do i = 1, B%n
                        strssg(j,k) = strssg(j,k) + stress(i,j)*B%phi(i,k)
                     end do
                  end do
               end do

!              --- For the "definition" of the divergence of the stress tensor, &
!                  see subroutine EL2006

               do i = 1, B%n
                  do k = 1, B%m

!                 --- Fill positions corresponding to x-velocities

                     elemvec(1,i) = elemvec(1,i) &
                                   + strssg(1,k)*B%w(k)*B%phider(i,k,1) &
                                   + strssg(4,k)*B%w(k)*B%phider(i,k,2)

!                    --- Fill positions corresponding to y-velocities

                     elemvec(2,i) = elemvec(2,i) &
                                   + strssg(4,k)*B%w(k)*B%phider(i,k,1) &
                                   + strssg(2,k)*B%w(k)*B%phider(i,k,2)
                  end do
               end do

            else if ( E%ndim==3 ) then

!           --- Interpolate stress vector from nodal points to
!               integration points

               do j = 1, 6
                  do k = 1, B%m
                     strssg(j,k) = 0.d0
                     do i = 1, B%n
                        strssg(j,k) = strssg(j,k) + stress(i,j)*B%phi(i,k)
                     end do
                  end do
               end do

!              --- For the "definition" of the divergence of the stress tensor, &
!                  see subroutine EL2006

               do i = 1, B%n
                  do k = 1, B%m

!                 --- Fill positions corresponding to x-velocities

                     elemvec(1,i) = elemvec(1,i) &
                                   + strssg(1,k)*B%w(k)*B%phider(i,k,1) &
                                   + strssg(4,k)*B%w(k)*B%phider(i,k,2) &
                                   + strssg(6,k)*B%w(k)*B%phider(i,k,3)

!                    --- Fill positions corresponding to y-velocities

                     elemvec(2,i) = elemvec(2,i) &
                                   + strssg(4,k)*B%w(k)*B%phider(i,k,1) &
                                   + strssg(2,k)*B%w(k)*B%phider(i,k,2) &
                                   + strssg(5,k)*B%w(k)*B%phider(i,k,3)

!                    --- Fill positions corresponding to z-velocities

                     E%subvec(3+(i-1)*E%ndim) = E%subvec(3+(i-1)*E%ndim) &
                                   + strssg(6,k)*B%w(k)*B%phider(i,k,1) &
                                   + strssg(5,k)*B%w(k)*B%phider(i,k,2) &
                                   + strssg(3,k)*B%w(k)*B%phider(i,k,3)
                  end do
               end do
            else
               write(*,*) 'ERROR: value of E%ndim = ', E%ndim, &
                          'is incorrect.'
               call instop
            end if

         else if ( B%cart==2 ) then

!        --- 2D axisymmetric
!            Interpolate stress vector from nodal points to
!            integration points

            do j = 1, 6
               do k = 1, B%m
                  strssg(j,k) = 0.d0
                  do i = 1, B%n
                     strssg(j,k) = strssg(j,k) + stress(i,j)*B%phi(i,k)
                  end do
               end do
            end do

!           --- For the "definition" of the divergence of the stress tensor, &
!               see subroutine EL2006

            do i = 1, B%n
               do k = 1, B%m

!              --- Fill positions corresponding to r-velocities

                  elemvec(1,i) = elemvec(1,i) &
                                + strssg(1,k)*B%w(k)*B%phider(i,k,1) &
                                + strssg(3,k)*B%w(k)*B%phi(i,k)/B%xyzg(k,1) &
                                + strssg(4,k)*B%w(k)*B%phider(i,k,2)

!                 --- Fill positions corresponding to z-velocities

                  elemvec(2,i) = elemvec(2,i) &
                                + strssg(4,k)*B%w(k)*B%phider(i,k,1) &
                                + strssg(2,k)*B%w(k)*B%phider(i,k,2)
               end do
            end do

         end if

      end if

      end
