      subroutine sepeltrans ( mdiv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the transformation matrix to be used in the elimination
!     process of the centroid point for incompressible Navier-Stokes
!     Old subroutine el2000
! **********************************************************************
!
!                       KEYWORDS
!
!     transformation
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: mdiv

!     mdiv           i    Defines if divergence right-hand side is used (#0)
!                         or not (0)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, n12
      double precision :: rinmat(E%ndim,E%ndim), det, sum
      double precision, pointer ::  sp(:,:)

!     det            Determinant of L22
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     n12            Number of nt to be eliminated velocity unknowns
!     rinmat         Inverse of L22, see method
!     sp             Divergence matrix
!                    SP(i,k) corresponds to the k-th row and i-th column
!                    of the matrix, The k refers to the basis functions of
!                    the pressure, i to the basis functions of the
!                    velocity in the sequence of the solution vector
!     sum            Help variable to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The Navier-Stokes matrix can be written as:
!
!            |                T     T  |  |   |     |    |
!            |   S     S    -L    -L   |  | u |     | f  |
!            |    11    12    11    21 |  |  1|     |  1 |
!            |                         |  |   |     |    |
!            |                T     T  |  |   |     |    |
!            |   S     S    -L    -L   |  | u |     | f  |
!            |    21    22    12    22 |  |  2|  =  |  2 |
!            |                         |  |   |     |    |
!            |   D     D     0     0   |  | p |     | fd |
!            |    11    12             |  |  1|     |   1|
!            |                         |  |   |     |    |
!            |   D     D     0     0   |  | p |     | fd |
!            |    21    22             |  |  2|     |   2|
!
!     Here the u1 refers to the velocities outside the centroid,
!     u2 to the velocities in the centroid p1 to the pressure in the
!     centroid and p2 to the gradient of the pressure.
!     All quantities with index 2 must be eliminated.
!
!     It follows immediately that:
!
!            -1           -1
!     u  = -D   D   u  + D   fd  = R  u  - fd_tr
!      2     22  21  1    22   2    0  1
!
!     In this subroutine the matrix R0 is computed as well as the vector fd_tr
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeltrans' )
      debug = .false.! .and. ioutp>=0

      sp => E%divgrmat
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeltrans'
         write(irefwr,1) 'E%ndim, n', E%ndim, n
         call prinrl1 ( sp, E%ndim+1, E%ndim*n, 'sp' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( E%ndim==2 ) then

!     --- ndim = 2

         n12 = E%ndim*(n-1)
         det = sp(n12+1,2)*sp(n12+2,3)-sp(n12+2,2)*sp(n12+1,3)
         if ( debug ) write(irefwr,2) 'det', det
         rinmat(1,1) = sp(n12+2,3)/det
         rinmat(2,2) = sp(n12+1,2)/det
         rinmat(1,2) = -sp(n12+1,3)/det
         rinmat(2,1) = -sp(n12+2,2)/det
         if ( debug ) call prinrl1 ( rinmat, 2, 2, 'rinmat' )
         do i = 1, n12
            do j = 1, E%ndim
               E%trans(i,j) = -sum(rinmat(1:E%ndim,j)*sp(i,2:E%ndim+1))
            end do
         end do

         if ( mdiv/=0 ) then

!        --- mdiv = 1, compute right-hand side

            E%rhsdivtr(1) = -rinmat(1,1)*E%rhsdiv(2)-rinmat(1,2)*E%rhsdiv(3)
            E%rhsdivtr(2) = -rinmat(2,1)*E%rhsdiv(2)-rinmat(2,2)*E%rhsdiv(3)

         end if

      else

!     --- ndim = 3

         n12 = E%ndim*13
         det = sp(n12+1,2)*sp(n12+2,3)-sp(n12+2,2)*sp(n12+1,3)
         rinmat(1,1) = sp(n12+2,3)*sp(n12+3,4)-sp(n12+2,4)*sp(n12+3,3)
         rinmat(2,1) = sp(n12+2,4)*sp(n12+3,2)-sp(n12+2,2)*sp(n12+3,4)
         rinmat(3,1) = sp(n12+2,2)*sp(n12+3,3)-sp(n12+2,3)*sp(n12+3,2)
         rinmat(1,2) = sp(n12+3,3)*sp(n12+1,4)-sp(n12+3,4)*sp(n12+1,3)
         rinmat(2,2) = sp(n12+3,4)*sp(n12+1,2)-sp(n12+3,2)*sp(n12+1,4)
         rinmat(3,2) = sp(n12+3,2)*sp(n12+1,3)-sp(n12+3,3)*sp(n12+1,2)
         rinmat(1,3) = sp(n12+1,3)*sp(n12+2,4)-sp(n12+1,4)*sp(n12+2,3)
         rinmat(2,3) = sp(n12+1,4)*sp(n12+2,2)-sp(n12+1,2)*sp(n12+2,4)
         rinmat(3,3) = sp(n12+1,2)*sp(n12+2,3)-sp(n12+1,3)*sp(n12+2,2)
         det = sp(n12+1,2)*rinmat(1,1)+ &
               sp(n12+1,3)*rinmat(2,1)+ &
               sp(n12+1,4)*rinmat(3,1)
         det = 1d0/det
         rinmat = det*rinmat
         do i = 1, n12
            do j = 1, E%ndim
               E%trans(i,j) = -sum(rinmat(1:E%ndim,j)*sp(i,2:E%ndim+1))
            end do
         end do
         do i = 43, 81
            do j = 1, E%ndim
               E%trans(i,j) = -sum(rinmat(1:E%ndim,j)*sp(i,2:E%ndim+1))
            end do
         end do

         if ( mdiv/=0 ) then

!        --- mdiv = 1, compute right-hand side

            E%rhsdivtr(1) = -rinmat(1,1)*E%rhsdiv(2)- &
                           rinmat(1,2)*E%rhsdiv(3)- &
                           rinmat(1,3)*E%rhsdiv(4)
            E%rhsdivtr(2) = -rinmat(2,1)*E%rhsdiv(2)- &
                           rinmat(2,2)*E%rhsdiv(3)- &
                           rinmat(2,3)*E%rhsdiv(4)
            E%rhsdivtr(3) = -rinmat(3,1)*E%rhsdiv(2)- &
                           rinmat(3,2)*E%rhsdiv(3)- &
                           rinmat(3,3)*E%rhsdiv(4)

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( E%trans, E%ndim, E%ndim*(n-1), 'trans' )
         write(irefwr,*) 'End sepeltrans'

      end if  ! ( debug )
      call erclos ( 'sepeltrans' )

      end subroutine sepeltrans
