      subroutine el3024 ( g, coor2d, gamma, Kxx, Kxg, Kgx, Kgg, &
                          w, phi, dphidxi, ni, dni )
! ======================================================================
!
!        programmer    Martijn Cox & Niels Driessen
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.0  date 13-08-2013 Remove dphideta
!        version  1.1  date 12-12-2012 Long integers
!
!   copyright (c) 2004-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute contribution to element matrix, resulting from
!   contact conditions for general nonlinear solids.
!   At this moment, the subroutine has been tested only for
!   quadrilaterals and extended triangles.
!   Here it is assumed that the quantities (e.g. gamma, g, ni and dni) are
!   stored in the integration points. However, actually, they are
!   stored in the nodal points. This will not cause inconsistencies
!   because Newton-Cotes integration is used.
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision g(m), coor2d(ndim,n), &
                       gamma(m), w(m), phi(n,m), dphidxi(n,m,2), &
                       Kxx(n*ndim,n*ndim), &
                       ni(m,ndim), Kgx(n,n*ndim), Kxg(n*ndim,n), &
                       Kgg(n,n), dni(m,ndim,ndim)

!     coor2d         i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering in current configuration
!     dni            i    Change of indentor normal in integration points
!     dphidxi        i    Array of length n x m containing the
!                         derivatives of the basis functions with respect to
!                         local coor2ddinates
!     g              i    array of length m in which the values of
!                         g in the integration points are stored
!     gamma          i    array of length m with the contact pressures
!     kgg           i/o   Element matrix to be filled
!     kgx           i/o   Element matrix to be filled
!     kxg           i/o   Element matrix to be filled
!     kxx           i/o   Element matrix to be filled
!     ni             i    Normal of indentor in integration points
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     w              i    array of length m containing the weights for
!                         integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision matrC(ndim,n*ndim), normlc, &
                       matrE(ndim,n*ndim), &
                       CE(n*ndim,n*ndim), t1crosst2(ndim), &
                       normallc(ndim), nC(n*ndim), nE(n*ndim), &
                       matrdni(ndim,n*ndim), Kgxa(n,n*ndim), &
                       Kgxs(n,n*ndim), Kxx1(n*ndim,n*ndim), &
                       Kxx2(n*ndim,n*ndim), dxdxi(m,ndim,ndim)
      integer i, j, k, l, icont(m)
      double precision, pointer :: vecoldlc(:)

!     ce             Matrix for product of C and E matrices
!     dxdxi          Derivatives of glocal to local coor2ddinates
!     i              Counting variable
!     icont          Help array for contact
!     j              Counting variable
!     k              Counting variable
!     kgxa           Part of matrix Kgx
!     kgxs           Part of matrix Kgx
!     kxx1           Matrix
!     kxx2           Matrix
!     l              General loop variable
!     matrc          Matrix containing the shape functions
!     matrdni        Matrix
!     matre          Matrix with derivatives of coor2ddinates and shape functions
!     nc             Matrix
!     ne             Matrix
!     normlc           Norm for coor2ddinate transformation
!     normallc         Normal of the plane
!                    This array is only used in case of curves
!     t1crosst2      Cross product of tangential directions
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Note: the nodal points and integration points coincide because
!          Newton-Cotes integration is used
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- set matrices to zero

      do i = 1, n*ndim
         do j = 1, n*ndim
            Kxx(i,j) = 0d0
         end do
      end do
      do i = 1, n
         do j = 1, n*ndim
            Kxg(j,i) = 0d0
            Kgx(i,j) = 0d0
            Kgxa(i,j) = 0d0
            Kgxs(i,j) = 0d0
         end do
      end do
      do i = 1, n
         do j = 1, n
            Kgg(i,j) = 0d0
         end do
      end do

!     --- Fill icont, based on vecold

      iseqtype = oldsols(4)%iseqtype
      index3n => indoldsols(iseqtype)%index3
      vecoldlc => oldsols(4)%oldvecs
      do i = 1, n
         icont(i) = int(vecoldlc(index3n(2*i-1)))
      end do

!     --- calculate dxdxi = sum( dphidxi * coor2d ), in each integration point

      do i = 1, m
         do j = 1, ndim
            dxdxi(i,j,1) = 0d0
            dxdxi(i,j,2) = 0d0
            do k = 1, n
               dxdxi(i,j,1) = dxdxi(i,j,1) + dphidxi(k,i,1)*coor2d(j,k)
               dxdxi(i,j,2) = dxdxi(i,j,2) + dphidxi(k,i,2)*coor2d(j,k)
            end do
          end do
      end do

!     --- compute the integral

      do i = 1, m

!     --- clear matrices

         do j = 1, ndim
            do k = 1, n*ndim
               matrC(j,k) = 0d0
               matrE(j,k) = 0d0
            end do
         end do

!        --- fill in the C matrix with shape functions in current integration
!            --- point

         do j = 1, n
            do k= 1, ndim
               matrC(k,(j-1)*ndim + k) = phi(j,i)
            end do
         end do

         if ( ndim==2 ) then

            do j = 1, n
               matrE(1,(j-1)*ndim+2) =-dphidxi(j,i,1)
               matrE(2,(j-1)*ndim+1) =dphidxi(j,i,1)
            end do

            normlc =  dsqrt(dxdxi(i,1,1)**2 + dxdxi(i,2,1)**2)

            do j = 1, ndim
               normallc(j) = dxdxi(i,j,1)/normlc
            end do

         else if ( ndim==3 ) then

!        --- fill in the E matrix with derivatives of coor2ddinates and shape
!            --- functions

            do j = 1, n

!           --- first row

               matrE(1,(j-1)*3 + 1) = 0d0
               matrE(1,(j-1)*3 + 2) = dxdxi(i,3,2)*dphidxi(j,i,1) - &
                                      dxdxi(i,3,1)*dphidxi(j,i,2)
               matrE(1,(j-1)*3 + 3) = dxdxi(i,2,1)*dphidxi(j,i,2) - &
                                      dxdxi(i,2,2)*dphidxi(j,i,1)

!              --- second row

               matrE(2,(j-1)*3 + 1) = dxdxi(i,3,1)*dphidxi(j,i,2) - &
                                      dxdxi(i,3,2)*dphidxi(j,i,1)
               matrE(2,(j-1)*3 + 2) = 0d0
               matrE(2,(j-1)*3 + 3) = dxdxi(i,1,2)*dphidxi(j,i,1) - &
                                      dxdxi(i,1,1)*dphidxi(j,i,2)

!              --- third row

               matrE(3,(j-1)*3 + 1) = dxdxi(i,2,2)*dphidxi(j,i,1) - &
                                      dxdxi(i,2,1)*dphidxi(j,i,2)
               matrE(3,(j-1)*3 + 2) = dxdxi(i,1,1)*dphidxi(j,i,2) - &
                                      dxdxi(i,1,2)*dphidxi(j,i,1)
               matrE(3,(j-1)*3 + 3) = 0d0
            end do

!           --- calculate cross-product of t1 and t2 in current integration
!               --- point

            t1crosst2(1) = dxdxi(i,2,1)*dxdxi(i,3,2) - &
                           dxdxi(i,3,1)*dxdxi(i,2,2)
            t1crosst2(2) = dxdxi(i,3,1)*dxdxi(i,1,2) - &
                           dxdxi(i,1,1)*dxdxi(i,3,2)
            t1crosst2(3) = dxdxi(i,1,1)*dxdxi(i,2,2) - &
                           dxdxi(i,2,1)*dxdxi(i,1,2)

!           --- calculate the corresponding norm for coor2ddinate transformation

            normlc = dsqrt( t1crosst2(1)**2d0 + &
                          t1crosst2(2)**2d0 + &
                          t1crosst2(3)**2d0 )

!           --- and store the results in normal

            do j = 1, ndim
               normallc(j) = t1crosst2(j)/normlc
            end do

         end if

!        --- calculate matrdni = C' * dni

         do j = 1, ndim*n
            do k = 1, ndim
               matrdni(k,j) = 0d0
               do l = 1, ndim
                  matrdni(k,j) = matrdni(k,j) + dni(i,k,l) * matrC(l,j)
               end do
            end do
         end do

!        --- calculate nC = ni' * C

         do j = 1, n*ndim
            nC(j) = 0d0
            do k = 1, ndim
                  nC(j) = nC(j) + ni(i,k)*matrC(k,j)
            end do
         end do

!        --- calculate nE = nb' * E

         do j = 1, n*ndim
            nE(j) = 0d0
            do k = 1, ndim
               nE(j) = nE(j) + normallc(k) * matrE(k,j)
            end do
         end do

!        --- calculate C' * ni * nb' * E = (nC)' * nE

         do j = 1, n*ndim
            do k = 1, n*ndim
               CE(j,k) = nC(j) * nE(k)
            end do
         end do

!        --- calculate Kxx (both for contact and for no contact)

         do j = 1, n*ndim
            do k = 1, n*ndim
               Kxx1(j,k) = (w(i)/normlc) * gamma(i) * CE(j,k)
               Kxx2(j,k) = 0d0
               do l = 1, ndim
                  Kxx2(j,k) = Kxx2(j,k) + &
                          w(i) * gamma(i) * matrC(l,j) * matrdni(l,k)
               end do
            end do
         end do

!        --- add Kxx1 and Kxx2

         do j = 1, n*ndim
            do k = 1, n*ndim
               Kxx(j,k) = Kxx(j,k) + Kxx1(j,k) + Kxx2(j,k)
            end do
         end do

!        --- calculate Kxg (both for contact and for no contact)

         do j = 1, n
            do k = 1, n*ndim
               Kxg(k,j) = Kxg(k,j) + w(i) * phi(j,i) * nC(k)
            end do
         end do

!        --- calculate Kgxs, which is different for contact and no contact

         do j = 1, n
            do k = 1, n*ndim

!           --- contact

               if (icont(i)==1) then
                  Kgxs(j,k) = Kgxs(j,k) + w(i) * phi(j,i) * nC(k)

!                 --- no contact (do not add matrix to integral)

               else if (icont(i)==0) then
                  Kgxs(j,k) = Kgxs(j,k)
               end if
            end do
         end do

!        --- calculate Kgg
!            Kgg is different for contact and no contact

         do j = 1, n
            do k = 1, n

!           --- contact (do not add matrix to integral)

               if (icont(i)==1) then
                  Kgg(j,k) = Kgg(j,k)

!                 --- no contact

               else if (icont(i)==0) then
                  Kgg(j,k) = Kgg(j,k) + w(i) * phi(j,i) * phi(k,i)
               end if
            end do
         end do

!        --- calculate Kgxa, which is different for contact and no contact

         do j = 1, n
            do k = 1, n*ndim

!           --- contact

               if (icont(i)==1) then
                  Kgxa(j,k) = Kgxa(j,k) + &
                              (w(i)/normlc) * phi(j,i) * nE(k) * g(i)

!                 --- no contact

               else if (icont(i)==0) then
                  Kgxa(j,k) = Kgxa(j,k) + &
                              (w(i)/normlc) * phi(j,i) * nE(k) * gamma(i)
               end if
            end do
         end do

      end do

!     --- finally, store Kgx = Kgxa + Kgxs

      do j = 1, n
         do k = 1, n*ndim
            Kgx(j,k) = Kgxa(j,k) + Kgxs(j,k)
         end do
      end do

      end
