      subroutine sepelm255
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for plate problems
!     Two-dimensional elements
!     Old subroutine elm255
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elasticity_equation
!     parabolic_equation
!     thick_plate
!     plate
!     bending
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: thickness(:)

!     thickness      Thickness in nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPEL0255      Fill element independent quantities for element type 255
!     SEPELBASEFUNC  Compute information about basis functions
!     SEPELDEALLOC   Deallocate coefficients and basis functions for element
!                    subroutines
!     SEPELFILLVAR   Fill space dependent coefficients
!     SEPELMPLATE    Fill element matrix for plate problems
!     SEPELOLDVECS   Fill old vectors for element
!     SEPELVPLATE    Fill element vector for plate problems
!     SEPFILLBBMAT   Fill matrix B^b for plate problem
!     SEPFILLBSMAT   Fill matrix B^s for plate problem
!     SEPFILLDMAT    Fill D-matrix for linear elasticity problem
!     SEPFILLDMATS   ill elasticity matrix D^s for bending problem, depending on
!                    the values of the elasticity E and the poisson ratio nu
!                    The matrix D^s is the elasticity matrix for shear stiffness
!     SEPPRINELMAT   Print element matrix and vector
! **********************************************************************
!
!                       I/O
!
!     The following coefficients are used
!
!     1:  Not yet used (must be 0)
!     2:  Type of stress-strain relations. This parameter is equal to IGPROB-1
!         Possible values:
!         0: isotropic material
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element (2x2 Gauss)
!     4:  Not yet used (must be 0)
!     5:  Not yet used (must be 0)
!     6-26: Information about the elasticity matrix depending on the
!           value of IGPROB
!         IGPROB = 1
!            6:  E
!            7:  nu
!    27:  h (thickness of the plate: 2D Cartesian)
!         Default value 1
!    28:  F1  Distributed load at the surface
!    29:
!    30:
!
!    31:  epsx 0  (or epsr 0)             initial strain (not yet used)
!    32:  epsy 0  (or epsz 0)
!    33:  epsz 0  (or epstheta 0)
!    34:  gammaxy 0  (or gammarz 0)
!    35:  gammayz 0
!    36:  gammazx 0
!
!    37:  sigmax 0  (or sigmar 0)         initial stress (not yet used)
!    38:  sigmay 0  (or sigmaz 0)
!    39:  sigmaz 0  (or sigmatheta 0)
!    40:  tauxy 0   (or taurz 0)
!    41:  tauyz 0
!    42:  tauzx 0
!    43:  rho (parameter for time-derivative)
!         Default: 1
!    44:  dF1/du
!    45:  Not yet used (must be 0)
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Each point contains three unknowns:
!
!     w:        transverse displacement
!     theta_1:  first component of rotation vector (in x-direction)
!     theta_2:  second component of rotation vector (in y-direction)
!
!     According to the plate theory we have the following relations:
!
!     u = z theta_2
!     v =-z theta_1
!
!     Mark that this a right-handed co-ordinate system and differs from
!     the definition as used by Hughes in his book.
!     However, it is in correspondence with the shell theory of Hughes.
!
!     The stiffness matrix consists of two parts:
!
!         k^b: bending stiffness
!         k^s: shear stiffness
!
!     These matrices are defined by the relations:
!
!        k^b = (B^b)^T D^b B^b
!        k^s = (B^s)^T D^s B^s
!
!     The submatrices are given by:
!
!           | 0     0    -dphi/dx |
!     B^b = | 0  dphi/dy     0    |
!           | 0  dphi/dx -dphi/dy |
!
!           | dphi/dx    0   phi |
!     B^s = | dphi/dy   -phi   0 |
!
!     where phi are the basis functions
!
!     The matrices D^b and D^s follow from integration over z of the classical
!     stiffness matriix and the sigma_{alpha 3} term respectively
!
!     For an isotropic material we have
!
!                            |   1   nu    0      |
!     D^b = E h^3/12(1-nu^2) |  nu    1    0      |
!                            |   0    0  (1-nu)/2 |
!
!                         | 1   0 |
!     D^s = E h k/2(1+nu) | 0   1 |
!
!     k is a constant to account for the actual nonuniformity of the
!     shearing stress
!     Following Hughes we set k=5/6
!
!     In the T1 element B^s is replaced by an adapted B^s as described in
!     T.J.R. Hughes and T.E. Tezduyar, Finite elements based upon Mindlin
!     plate theory with particular reference to the four-node bilinear
!     isoparametric element, Journal of applied Mechanics, sept 1981, vol 48, &
!     pages 587-596
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm255' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm255'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%mat = 0d0
      E%vec = 0d0

!     --- Define element independent quantities
!         only for first element

      if ( E%first==0 ) call sepel0255
      if ( ierror/=0 ) go to 1000

!     --- Map relation between coefficients and specific names

      thickness => E%coeffs(:,22)

!     --- Fill basis functions

      call sepelbasefunc
      if ( ierror/=0 ) go to 1000

!     --- Fill old vectors in quadrature points if necessary

      if ( E%compold>0 ) call sepeloldvecs

      if ( debug ) then
         write(irefwr,1) 'isupcfvar, isubcfvar', E%isupcfvar, E%isubcfvar
         write(irefwr,*) 'B%fillqmat ', B%fillqmat
      end if  ! ( debug )

!     --- Fill variable coefficients if necessary

      if ( E%isupcfvar>=E%isubcfvar ) call sepelfillvar

      if ( E%fillbmat ) then

!     --- Compute matrix B^b phi i and B^s phi i

         call sepfillbbmat
         call sepfillbsmat

      end if  ! ( E%fillbmat )

      if ( .not. E%dmatconst ) then

!     --- Compute matrix D^b and D^s, since they are variable

         call sepfilldmat  ! Dmat
         call sepfilldmats    ! Dmats

      end if  ! ( .not. E%dmatconst )

!     --- Fill element matrix

      if ( matrix ) call sepelmplate

!     --- Fill element vector

      if ( vector .and. E%notvec==0 ) call sepelvplate

!     --- Remove basis functions and coefficients in case of last call

      if ( debug ) write(irefwr,1) 'last', E%last
      if ( E%last==1 ) then
         call sepeldealloc
         deallocate ( E%Dmat, E%Dmats, E%Bb, E%Bs )
      end if  ! ( E%last==1 )

1000  if ( debug ) then

!     --- Debug information

         call sepprinelmat
         write(irefwr,*) 'End sepelm255'

      end if  ! ( debug )
      call erclos ( 'sepelm255' )

      end subroutine sepelm255
