      subroutine sepelm260 ( vecloc, workar )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.0  date 12-08-2013 Extra parameters
!        version  2.11 date 13-08-2013 Debug information
!
!   copyright (c) 1994-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix and vector for rubber elements, i.e.
!     (nearly) incompressible stress elements
!    The subroutine is able to handle the following element shapes and cases:
!    2D:  4 node quadrilateral (Type 260)
!         penalty method
!         5 node quadrilateral (types 261)
!         integrated method
!         6 node triangle (Type 260)
!         penalty method
!         This element is treated internally as a seven-point element
!         7 node triangle (Type 261)
!         integrated method
!         9 node quadrilateral (Type 260, penalty method)
!    3D: 8 node or 27 node brick (Type 260)
!         penalty method
!        27 node brick (Type 262)
!         integrated method
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     rubber_element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/celwrkf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision vecloc(*), workar(*)

!     vecloc         i    Work array in which all old solution vectors for the
!                         integration points are stored
!     workar              work array of length 163 + 238m**3 + 6m + 15n**2 + 10n
!                         contents:
!                         Starting address    length   Name:
!                         1                   6*27     ARRAY
!                         ipugs               3*27     UGAUSS
!                         ipdudx              9*27     DUDX
!                         ipdpdx              27*27*3  DPHIDX
!                         ipphix              27*27*3  PHIKSI for elp633
!                         ipdiv               81*4     SP_matrix (Gradient P)
!                         ipwork              27*31    Work space
!                         ipelem              81*81    Copy of part of el matrix
!                         ipelvc              81*3     Copy of part of el vector
!                         iptran              12*2     Transformation matrix
!                                                      based on div
!                                  (2D only, does not contribute
!                                   to total length, since this
!                                   length is maximized for 3d)
!                         Total length: 13257
!                         The subarrays contain the following contents:
!                         ARRAY:
!                         array in which the values of the coefficients
!                         in the integration points are stored in the sequence:
!                              2D                  3D
!                         1:   rho              rho
!                         2:   nu               nu
!                         3:   E                E
!                         4:   f1               f1
!                         5:   f2               f2
!                         6:                    f3
!                         Each coefficient requires exact m (is number of
!                         integration points) positions
!                         DPHIDX:
!                         Array of length inpelm x m x ndim containing the
!                         derivatives of the basis functions in the sequence:
!                         d phi_i/dx = dphidx(i,k,1); dphi_i/dy = dphidx(i,k,2)
!                         d phi_i/dz = dphidx(i,k,3) in node k
!                         WORK
!                         General work space
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isub, isup, jelp, ipdpdx, ipugs, ipwork, &
              ipdudx, ipdiv, ipelem, ipelvc, iptran, ipelcp, &
              iprho, nveloc, npres, ipdum, signdiv, icent
      double precision rhsdiv(1), rhsdivtr(1), &
                       uold(81), psipres(4*27)
      save isub, isup, jelp, ipdpdx, ipwork, ipugs, npres, &
           ipdudx, ipelem, ipelvc, ipdiv, iptran, ipelcp, iprho, &
           nveloc, ipdum, icent

!     i              Counting variable
!     icent          Index of centroid
!     ipdiv          Starting address of div
!     ipdpdx         Starting address of dphidx
!     ipdudx         Starting address of dudx
!     ipdum          Starting address of array dummy
!     ipelcp         Starting address of copy of element matrix
!     ipelem         Starting address of copy of element matrix
!     ipelvc         Starting address of copy of element vector
!     iprho          Starting address of rho
!     iptran         Starting address of trans with respect to div
!     ipugs          Starting address of ugauss
!     ipwork         Starting address of work array
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     jelp           Indication of the type of basis function subroutine must
!                    be called by ELxxx. Possible vaues:
!                     1:  Subroutine ELP610  (Linear line element)
!                     2:  Subroutine ELP611  (Quadratic line element)
!                     3:  Subroutine ELP620  (Linear triangle)
!                     4:  Subroutine ELP624  (Extended quadratic triangle)
!                     5:  Subroutine ELP621  (Bilinear quadrilateral)
!                     6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                    11:  Subroutine ELP630  (Linear tetrahedron)
!                    12:  Subroutine ELP631  (Quadratic tetrahedron)
!                    13:  Subroutine ELP632  (Trilinear brick)
!                    14:  Subroutine ELP633  (Triquadratic brick)
!     npres          Number of pressure parameters
!     nveloc         Number of velocity parameters
!     psipres        Contains the pressure basis functions
!                    psipres(i,k) refers to i-th basis function in
!                    integration point k
!     rhsdiv         Dummy
!     rhsdivtr       Dummy
!     signdiv        Multiplication sign of "continuity equations"
!                    if -1 the stokes matrix is symmetric
!                    if 1 all eigenvalues are in the positive half plane
!     uold           Array to store the old solution in the nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0260         Initialize parameters for subroutines ELD260/ELI260/sepelm260
!     EL1000         Compute the divergence matrix for Navier-Stokes
!     EL1006         Fill element vector for vertices only
!     EL2000         Compute the transformation matrix to be used in the
!                    elimination process of the centroid point for
!                    incompressible Navier-Stokes
!     EL2005         Compute a local vector and its derivatives from a known
!                    vector
!     EL2107         Eliminate the centroid point for the incompressible
!                    Navier- Stokes equations in case of an extended quadratic
!                    triangle
!     EL2201         Eliminate the centroid point for the incompressible
!                    Navier-Stokes equations in case of an
!                    extended quadratic triangle
!     EL2203         Put stiffness matrix and part due to gradient and
!                    divergence into element matrix for 27-point
!                    Navier-Stokes equation
!     EL3002         Compute integrals of the shape / beta(x) phi (x) phi (x) dx
!                                                   e            i       j
!     EL3003         Compute integrals of the shape / f(x) phi (x) dx
!                                                   e         i
!     EL3009         Fill series of old solutions in array vecloc
!     EL3010         Fill small matrix elemmt into larger matrix s
!     EL4921         Put gradient and divergence matrix into large matrix
!     EL8001         compute the stress part of the element matrix for rubber
!                    elements
!     EL8002         compute the penalty matrix for rubber elements
!     ELFILLPSIPRES  Fill the pressure basis functions in the integration points
!                    Crouzeix Raviart elements
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELM800BASEFN   Compute the basis functions B%phi, its derivatives and other
!                    related quantities for various element shapes
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays iuser and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     1:  information about the time-integration
!         Possible values:
!         0: if imas in BUILD=0 then time-independent flow
!            else the mass matrix is built according to imas
!     2:  - &
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-ordinates
!     5:  - &
!     6-20: Information about the equations and solution method
!     6:  density rho
!     7:  nu  (Poisson ratio)
!     8:  E   (Elasticity modulus)
!     9:  F1  (Body force in 1-direction)
!    10:  F2  (Body force in 2-direction)
!    11:  F3  (Body force in 3-direction)
!    12:  Penalty parameter eps
!    13-20: not yet used
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    if ( first call ) then
!       initialize standard parameters
!    compute basis functions
!    fill old solutions if necessary
!    if ( matrix ) then
!       Compute the stress matrix
!       Compute the divergence matrix
!       Compute the penalty contribution
!    if ( vector ) then
!       Compute contribution of body forces
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm260' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm260'
         write(irefwr,1) 'itype, ielem', itype, ielem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ifirst==0 )  then

!     --- ifirst = 0,  compute element independent quantities

         icheld = 0
         call sepel0260 ( E%iuser, E%user, jelp, isub, isup, workar, &
                          nveloc, npres )
         if ( inpelm==27 ) then
            icent = 14
         else
            icent = n
         end if

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( E%islold, numold, isub, isup, E%iuser )

         iprho  = 1
         ipugs  = 1      + 6*m+n
         ipdudx = ipugs  + ncomp*m
         ipdpdx = ipdudx + ncomp**2*m
         ipdiv = ipdpdx + ndim*m*n
         ipwork = ipdiv  + m*ndim*(ndim+1)
         ipelem = ipwork + n*(n+1+ndim)
         if ( jtrans==1 ) then
            ipelcp = ipelem + (n*ndim)**2
         else
            ipelcp = ipelem
         end if
         ipelvc = ipelcp + (n*ndim)**2
         iptran = ipelvc + n*ndim
         ipdum = 730

      end if
      signdiv = -1

!     --- Fill basis functions

      if ( debug ) write(irefwr,*) 'before sepelbasefunc'
      call sepelbasefunc
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,*) 'after sepelbasefunc'

!     --- Fill variable coefficients if necessary
!         First fill uold if necessary

      if ( ind(lengthmst)==1 .or. ind(lengthmst)==3 ) &
         call el2005 ( uold, workar(ipdudx), B%phi, &
                       B%phider, index1, workar(ipugs), iseqin )

      if ( ind(lengthmst)==2 .or. ind(lengthmst)==3 ) &
         call el3009 ( vecloc, B%phi, B%n )

      do i = isub, isup

         if ( ind(i)>0 .or. ist(i)>0 ) then

!        --- Coefficient is variable

            call elflr71 ( i, E%user, workar, B%xyz, B%xyzg, &
                          B%m, B%n, B%phi, 1, &
                          index1,  E%iuser, workar(ipugs), vecloc )

         end if

      end do

!     --- Compute the divergence and gradient matrix

      if ( debug ) write(irefwr,*) 'before elfillpsipres'
      call elfillpsipres ( psipres, npres, B%xyzg, B%xyz, icent )
      if ( debug ) write(irefwr,*) 'after elfillpsipres'
      call el1000 ( workar(ipdiv), workar(ipdiv), B%phider, &
                    workar, B%phi, B%w, B%xyz, B%xyzg, 0, workar(iprho), &
                    workar, rhsdiv, workar(1+8*m), 0, ishape, psipres, &
                    icent, work )

      if ( jtrans==1 ) then

!     --- Compute the transformation matrix
!         Compute the velocity in the centroid and the derivative of u

         call el2000 ( workar(iptran), workar(ipdiv), 0, rhsdivtr, &
                       rhsdiv )

      end if

!     --- Fill parts of matrix if necessary

      if ( debug ) write(irefwr,*) 'before matrix'
      if ( matrix ) then

!     --- matrix = true    compute element matrix
!         Compute the stiffness part

         call el8001 ( workar(ipelem), B%phider, B%phi, &
                       B%xyzg, workar(1+2*m), B%w, workar(ipwork), &
                       workar(ipwork+n*n), workar(1+m) )

      end if

      if ( debug ) write(irefwr,*) 'before vector'
      if ( vector .and. notvec==0 ) then

!     --- vector = true
!         First set element vector equal to zero

         do i = 1, ndim*n
            workar(ipelvc+i-1)=0d0
         end do

!        --- Next compute contribution due to external forces
!            Multiply weights by density rho and store in work from
!            position ipdiv

         do i = 1, m
            workar(ipdiv+i-1) = B%w(i) * workar(i)
         end do

!        --- First component

         if ( ind(4)/=0 ) then

!        --- f1 # 0  store contribution temporary in array work

            jadd = 0
            call el3003 ( workar(3*m+1), workar(ipwork), workar(ipdiv), &
                          B%phi, B%m, B%n )
            do i = 1, n
               workar(ipelvc+ndim*(i-1)) = workar(ipwork+i-1)
            end do

         end if

!        --- Second component

         if ( ind(5)/=0 ) then

!        --- f2 # 0  store contribution temporary in array work

            jadd = 0
            call el3003 ( workar(4*m+1), workar(ipwork), workar(ipdiv), &
                          B%phi, B%m, B%n )
            do i = 1, n
               workar(ipelvc+ndim*(i-1)+1) = workar(ipwork+i-1)
            end do

         end if

!        --- Third component

         if ( ind(6)/=0 ) then

!        --- f3 # 0  store contribution temporary in array work

            jadd = 0
            call el3003 ( workar(5*m+1), workar(ipwork), workar(ipdiv), &
                          B%phi, B%m, B%n )
            do i = 1, n
               workar(ipelvc+ndim*(i-1)+2) = workar(ipwork+i-1)
            end do

         end if

      end if

      if ( debug ) write(irefwr,*) 'before mass matrix, imas ', imas

      if ( imas>0 ) then

!     --- imas>0, i.e. the mass matrix must be computed
!         First compute the mass matrix for one velocity component
!         The non-diagonal mass matrix for this component is stored in
!         work positions ipwork - ipwork+n*n-1
!         The diagonal matrix in work pos. ipwork+n*n - ipwork+n*n+n-1

         jadd = 0
         if ( imas>1 ) then

!        --- non-diagonal mass matrix

            jzero = 2
            call el3002 ( workar, workar(ipwork), B%w, B%phi, B%m, &
                          B%n, B%n )

         else

!        --- diagonal mass matrix

            jconsf = imas
            call el3003 ( workar, workar(ipwork+n*n), B%w, B%phi, &
                          B%m, B%n )

         end if

         if ( imas==1 ) then

!        --- imas = 1, build diagonal mass matrix

            do i = 1, n
               E%mmat(1,i) = workar(ipwork+n*n+i-1)
               E%mmat(2,i) = workar(ipwork+n*n+i-1)
               E%mmat(3,i) = workar(ipwork+n*n+i-1)
            end do

         else

!        --- imas = 2, build non-diagonal mass matrix

             jadd = 0
             call el3010 ( E%mmat, workar(ipwork), 3*n, B%n, 1, 1, 3, &
                           .false., 1d0 )
             call el3010 ( E%mmat, workar(ipwork), 3*n, B%n, 2, 2, 3, &
                           .false., 1d0 )
             call el3010 ( E%mmat, workar(ipwork), 3*n, B%n, 3, 3, 3, &
                           .false., 1d0 )

         end if

      end if

      if ( debug ) write(irefwr,*) 'before divergence'

      if ( matrix ) then

!     --- Put divergence and gradient part into matrix
!         Eliminate centroid if necessary
!         Compute penalty contribution if necessary

         if ( itype==261 .or. itype==262 ) then

!        --- itype = 261 or 262 integrated method

            if ( jtrans==1 ) then

!           --- jtrans = 1, eliminate centroid and new element matrix into
!                           workar(ipelcp)

               call el2107 ( workar(ipelcp), workar(ipelem), &
                             workar(iptran), workar(iptran) )

            end if

!           --- Compute the complete matrix

            if ( inpelm==27 ) then

!           --- inpelm = 27, special case since centroid is not the end point

               call el2203 ( elemmt, workar(ipelcp), workar(ipdiv), &
                             workar(ipdiv), workar, 0d0, 0, &
                             B%w, B%xyz, B%xyzg, signdiv )

            else

!           --- inpelm # 27, standard situation
!               First put the velocity stiffness matrix into the large matrix

               jadd = 0
               call el3010 ( elemmt, workar(ipelcp), icount, &
                             ndim*(B%n-jtrans), 1, 1, 1, .false., 1d0 )

!              --- Next put gradient and divergence matrix into elemmt

               call el4921 ( elemmt, workar(ipdiv), workar(ipdiv), &
                             workar, nveloc, npres, 0d0, 0, &
                             B%w, B%xyz, B%xyzg, signdiv )

            end if

         else

!        --- Compute the penalty contribution
!            In case of 2D extended 6-point quadratic triangles the result is
!            stored in workar(ipelem) otherwise in elemmt

            if ( debug ) write(irefwr,*) 'before el8002'
            call el8002 ( elemmt, workar(ipelem), workar(ipdiv), B%w, B%xyz, &
                          B%xyzg, workar(1+2*m), workar(1+m) )
            if ( debug ) write(irefwr,1) 'jtrans', jtrans
            if ( jtrans==1 ) &
               call el2107 ( elemmt, workar(ipelem), workar(iptran), &
                             workar(iptran) )

         end if

      end if

      if ( debug ) write(irefwr,*) 'before Eliminate'
      if ( itype/=261 .and. matrix .and. jtrans==1 ) then

!     --- Eliminate centroid in case of 2D quadratic elements
!         The resulting matrix is stored in elemmt

         call el2107 ( elemmt, workar(ipelem), workar(iptran), &
                       workar(iptran) )

      end if

      if ( vector ) then

!     --- Copy element vector in elemvc

         if ( jtrans==1 ) then

!        --- Eliminate centroid in case of 2D quadratic elements
!            The resulting vector is stored in elemvc

            call el2201 ( elemvc, workar(ipelvc), workar(iptran), 0, &
                          elemmt, rhsdivtr )

         else

!        --- Copy vector

            if ( itype==262 .and. inpelm==27 ) then

!           --- 27-point element, integrated method

               call el1006 ( elemvc, workar(ipelvc) )

            else

!           --- Standard situation

               do i = 1, ndim*n
                  elemvc(i) = workar(ipelvc+i-1)
               end do

            end if

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( elemmt, icount, icount, 'elemmt' )
         call prinrl ( elemvc, icount, 'elemvc' )
         write(irefwr,*) 'End sepelm260'

      end if  ! ( debug )
      call erclos ( 'sepelm260' )

      end
