      subroutine readsl ( ivectr, iref )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 29-05-2014 remove call to time subroutine
!        version  1.2  date 15-11-2012 Long integers
!        version  1.1  date 12-08-1999 New call to erclmn
!
!   copyright (c) 1999-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   read real/complex  sepran solution array except the large matrix
!   only solution vectors of type 110, 115 and 116 are allowed
!   in case of formatted files the standard format 5d16.9 is used
!   This subroutine is meant to be used in combination with subroutine
!   prinsl.
!   prinsl writes a solution in character format to a file, &
!   readsl reads the solution
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemain
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivectr, iref

!     iref    i   Reference number of file to which solution must be written
!                 possibilities:  see user manual
!                 if iref<0, the file is read unformatted, otherwise it is
!                 supposed to be formatted
!     ivectr  o   integer array denoting the array to be readed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical compl
      integer ihelp(8), ios, i, itypvc, ivec, k

!     compl          Indication if the solution vector is complex (true) or
!                    not (false)
!     i              General loop variable
!     ihelp          integer work array of length 8 to store
!                    quantities corresponding to ivectr
!                    1: compl
!                    2: iprob
!                    3: ivec
!                    4: jcart
!                    5: nphys
!                    6: nusol
!                    7: typevec
!                    8: iseqkp
!     ios            IOSTAT variable
!     itypvc         Indicates type of vector 110 or 115
!     ivec           Sequence number of vector of special structure
!     k              General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0045         Actual reading and filling of real vector
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
! **********************************************************************
!
!                       I/O
!
!    The solution vector or vector of special structure is read from the
!    file with reference number iref74
!    If iref74>0, the file is formatted, otherwise it is unformatted
!    The first record contain the 5 integers that must be stored into array
!    IVECTR, the second record contains the real values
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      60   Array to be read is of wrong type
!     747   End of file reached
!     748   Error in reading from file
!    2023   The mesh does not correspond to the vectors read
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readsl' )
      if ( ierror/=0 )  go to 1000
      kmesh2d(1) = kmesh
      kmesh => kmesh2d(1)
      call sepfillkmesh(kmesh)

      if ( iref>0 ) then
         read ( iref, 10, iostat=ios ) ihelp
 10      format(5i6)
      else
         read ( -iref, iostat=ios ) ihelp
      end if
      if ( ios<0 ) then

!     --- ios<0, end of file reached

         call errsub ( 747, 0, 0, 0 )
         go to 1000

      else if ( ios>0 ) then

!     --- ios>0, Read error

         call errint ( ios, 1 )
         call errsub ( 748, 1, 0, 0 )
         go to 1000
      end if
!                         1: compl
!                         2: iprob
!                         3: ivec
!                         4: jcart
!                         5: nphys
!                         6: nusol
!                         7: typevec
!                         8: iseqkp

      itypvc = ihelp(7)
      ivec = ihelp(3)
      nusol = ihelp(6)
      if(ihelp(1)==0) then
         compl = .false.
      else
         compl = .true.
      end if
      if ( itypvc/=110 .and. itypvc/=115  .and. itypvc/=116 ) then
         call errint ( itypvc, 1 )
         call errsub ( 60, 1, 0, 0 )
      end if
      iprob  = ihelp(2)
      call sepcreasolbf1 ( ivectr, itypvc, ivec, 'vector', ihelp(1) )

      if ( ierror/=0 )  go to 1000
      length = size ( usoltemp )
      if ( compl ) length = length/2
      if ( length/=nusol ) then

!     --- Error: incorrect length, probably incorrect mesh

         call errint ( nusol, 1 )
         call errint ( length, 2 )
         call errsub ( 2023, 2, 0, 0 )
         go to 1000

      end if

      call er0045 ( usoltemp, nusolo, compl, iref )
1000  call erclos ( 'readsl' )
      end
