      subroutine sepaddmat1 ( kproblc, matr1, matr2, matr3, &
                              matr4, intmat1, intmat2, alpha, beta, gamma )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-01-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute M4 = alpha M1 + beta M2 + gamma M3, where alpha, beta and
!     gamma are complex numbers, M1, M2 and M3 real matrices
!     corresponding to matr1/2/3 and M4 is complex matrix corresponding
!     to matr4
!     Old subroutine addmt1
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     multiplication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleintmat
      use sepmodulekprob
      use sepmodulemain
      use sepmodulematr
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double complex, intent(in) :: alpha, beta, gamma
      integer, intent(in) :: kproblc, matr1, matr2, matr3, intmat1
      integer, intent(out) :: intmat2, matr4

!     alpha          i    multiplication factor
!     beta           i    multiplication factor
!     gamma          i    multiplication factor
!     intmat1        i    Refers to structure of matrices matr1/2/3
!     intmat2        o    Refers to structure of matrix matr4
!     kproblc        i    Refers to the problem definition
!     matr1          i    Refers to matrix M1
!     matr2          i    Refers to matrix M2
!     matr3          i    Refers to matrix M3
!     matr4          o    Refers to matrix M4
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, lengmt
      logical :: matrexist
      double precision, pointer :: S3_ii(:), S4_ii(:)
      type (intmatrix), pointer :: IM1, IM2
      double precision :: timebefore(2)

!     IM1            Refers to intmt(intmat1)
!     IM2            Refers to intmt(intmat2)
!     S3_ii          part of matrix M3
!     S4_ii          part of matrix M4
!     i              Counting variable
!     lengmt         Length of matrix
!     matrexist      if true the matrix already exists
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0050         Perform the operations on the matrices
!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     SEPCOPINTMAT   Copy contents of structure intmat1 into intmat2
!     SEPCRMATRIX    create a new array in realmt
!     SEPFILLMAT     Fill information of matrix with sequence number iseqmt
!                    structure of type mat
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepaddmat1' )
      if ( itime>0 ) call ersettime ( timebefore )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepaddmat1'
         write(irefwr,1) 'kproblc', kproblc
         write(irefwr,1) 'intmat1, intmat2', intmat1, intmat2
         write(irefwr,1) 'matr1, matr2, matr3, matr4', &
                          matr1, matr2, matr3, matr4
         write(irefwr,2) 'alpha, beta, gamma', alpha, beta, gamma
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      IM1 => intmt(intmat1)
      iprob  = IM1%iprob

!      call sepcopintmat ( intmat1, intmat2 )
!      IM2 => intmt(intmat2)
!      IM2%jmethod = IM1%jmethod+2
!      IM2%complexm = .true.
!      IM2%direct = IM1%direct

      if ( debug ) write(irefwr,1) 'iprob', iprob
      if ( iprob/=1 ) call errsub ( 1234, 0, 0, 0)
      if ( ierror/=0) go to 1000
      kprob2d(1) = kproblc
      kprob => kprob2d(1)

      npboun=kprobloc(10)

      if ( debug ) write(irefwr,1) 'npboun', npboun

!     --- Reserve space for arrays
!         First the internal part

      S1_ii => realmt(matr1)%S_ii
      S2_ii => realmt(matr2)%S_ii
      S3_ii => realmt(matr3)%S_ii

      lengmt = size(S1_ii)

      matrexist = matr4>0 .and. matr4<=lastmatrix

      if ( matrexist ) then

!     --- matrix 4 might already exist

         if ( size(realmt(matr4)%S_ii)/=lengmt*2 ) matrexist = .false.

      end if  ! ( matrexist )
      if ( debug ) write(irefwr,*) 'matrexist ', matrexist

      if ( .not. matrexist ) then

!     --- matrix is new: create

         iseqkp = IM1%iseqkp
         iseqkm = kp(iseqkp)%iseqkm
         if ( intmat2==0 ) &
            call commat( IM1%jmethod+2 , iseqkm , iseqkp , intmat2 )
         intmat = intmat2
         call sepcrmatrix ( matr4 )
         IM1%complexm =  .false.

      else
         intmat2 = realmt(matr4)%intmatref
         intmat = intmat2
      end if  ! ( .not. matrexist )
      if ( ierror/=0 ) go to 1000

      S4_ii => realmt(matr4)%S_ii

!     --- Compute the new matrix

      call er0050 ( S1_ii, S2_ii,S3_ii, &
                    S4_ii, lengmt, alpha, beta, gamma )

      realmt(matr4)%complexmat = .true.
      realmt(matr4)%decomp = .true.
      iseqmt = matr4
      call sepfillmat

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepaddmat1'

      end if  ! ( debug )
      call erclmn ( 'sepaddmat1', timebefore )

      end subroutine sepaddmat1
