      subroutine sepmapcom ( iinmaplc, intmat1, intmat2, matr1, matr2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-01-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Maps either intmat1 with into intmat2 of matr1 in matr2
!     The first structure must have been filled completely with jmethod 1 or 5
!     The second structure must exist and jmethod gets the value 9
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulemain
      use sepmodulematr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinmaplc(*), intmat1, matr1
      integer, intent(inout) :: intmat2, matr2

!     iinmaplc       i    Integer input array in which the user defines the
!                         actions to be taken by this subroutine
!                         IINMAP must be filled as follows:
!                         Pos. 1:  number of positions filled by the user
!                                  If IINMAP(1) = 0, only defaults are used
!                         Pos. 2:  ICHOIS, defines the type of action to be
!                                  taken.
!                                  Possible values:
!                                  1:  Map INTMT1 onto INTMT2
!                                  2:  Map MATR1 onto MATR2
!                                  Default value: 1
!     intmat1        i    Refers to matrix structure 1
!     intmat2       i/o   Refers to matrix structure 2
!     matr1          i    Refers to matrix 1
!     matr2         i/o   Refers to matrix 2
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jmethod1, jmethod2, ichois, iprob1, iprob2, intmatref
      integer, pointer :: irows1(:), icols1(:), &
                          irows2(:), icols2(:)
      logical permit
      type (intmatrix), pointer :: IM1, IM2

!     IM1            Refers to intmt(intmat1)
!     IM2            Refers to intmt(intmat2)
!     ichois         Choice parameter, see IINMAP
!     icols1         Contains column numbers of matrix 1
!     icols2         Contains column numbers of matrix 2
!     intmatref      refers to array intmat corresponding to matrices
!     iprob1         Problem number of matrix 1
!     iprob2         Problem number of matrix 2
!     irows1         Contains number of non-zero entries per row of matrix 1
!                    accumulated
!     irows2         Contains number of non-zero entries per row of matrix 2
!                    accumulated
!     jmethod1       jmethod for matrix 1
!     jmethod2       jmethod for matrix 2
!     permit         Logical parameter indicating if a situation is permitted
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INCPCM         Translate contents of array INTMT1 part 1 into
!                    INTMT2 part 1
!     INCPMT         Decrease priorities of arrays in IBUFFR
!     PRININ         print 1d integer array
!     SEPCRMATRIX    create a new array in realmt
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       1   ICHOIS is incorrect
!    1230   Array INTMT1 is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmapcom' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmapcom'
         call prinin ( iinmaplc, 2, 'iinmap' )
         write(irefwr,1) 'matr1, matr2', matr1, matr2
         write(irefwr,1) 'intmat1, intmat2', intmat1, intmat2
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check input

      ichois = 1
      if ( iinmaplc(1)>1 ) then
         ichois = iinmaplc(2)
         if ( ichois<1 .or. ichois>2 ) then
            call errint ( ichois, 1 )
            call errsub ( 1, 1, 0, 0 )
         end if
      end if

      if ( debug ) write(irefwr,1) 'ichois', ichois

      if ( ierror/= 0 ) go to 1000

      if ( ichois==1 ) then

!     --- ichois = 1, Fill INTMT2
!         Activate arrays and reserve space for new and temporary array

         IM1 => intmt(intmat1)
         IM2 => intmt(intmat2)
         irows1 => IM1%rowsii
         icols1 => IM1%colsii

         irows2 => IM2%rowsii
         icols2 => IM2%colsii
         iprob = IM1%iprob
         iseqkp = IM1%iseqkp
         kprobloc => kp(iseqkp)%kprobsub(iprob)%kprobloc

         nbound = kprobloc(10) + kprobloc(34)
         nrusol = kprobloc(66)
         jmethod1 = IM1%jmethod
         if ( jmethod1/=5 ) then
            call errint ( 102, 1 )
            call errint ( jmethod1, 2 )
            call errint ( 5, 3 )
            call errsub ( 1230, 3, 0, 0 )
            go to 1000
         end if

         if ( debug ) &
            write(irefwr,1) 'jmethod1, jmethod2', jmethod1, IM2%jmethod

!        --- Fill first part of array INTMT2

         call incpcm ( nrusol, irows1, icols1, irows2, icols2 )
         length = irows2(1+nrusol) + nrusol

!        --- Copy second part of INTMT1 into INTMT2

         if ( nbound>0 ) then

!        --- Nbound>0, INTMT2 part 2
!            Decrease priorities

            irows1 => IM1%rowsip
            icols1 => IM1%colsip
            if ( associated(IM2%rowsip) ) deallocate(IM2%rowsip)
            allocate(IM2%rowsip(size(irows1)) )
            IM2%colsip = icols1
            if ( associated(IM2%colsip) ) deallocate(IM2%colsip)
            allocate(IM2%colsip(size(icols1)) )
            IM2%colsip = icols1

            if ( associated(IM1%rowspp) ) then

               irows1 => IM1%rowspp
               icols1 => IM1%colspp
               if ( associated(IM2%rowspp) ) deallocate(IM2%rowspp)
               allocate(IM2%rowspp(size(irows1)) )
               IM2%colspp = icols1
               if ( associated(IM2%colspp) ) deallocate(IM2%colspp)
               allocate(IM2%colspp(size(icols1)) )
               IM2%colspp = icols1

            end if  ! ( associated(IM1%rowspp) )

         end if

      else

!     --- ichois = 2, Fill MATR2
!         Activate arrays and reserve space for new and temporary array

         IM1 => intmt(intmat1)
         iprob = IM1%iprob
         iseqkp = IM1%iseqkp
         kprobloc => kp(iseqkp)%kprobsub(iprob)%kprobloc

         nbound = kprobloc(10) + kprobloc(34)
         nrusol = kprobloc(66)

         if ( debug ) &
            write(irefwr,1) 'iprob, iseqkp, nbound, nrusol', &
                             iprob, iseqkp, nbound, nrusol

         jmethod1 = IM1%jmethod

         if ( debug ) write(irefwr,1) 'jmethod1', jmethod1
         if ( jmethod1/=1 .and. jmethod1/=5 ) then
            call errint ( 102, 1 )
            call errint ( jmethod1, 2 )
            call errint ( 5, 3 )
            call errsub ( 1230, 3, 0, 0 )
            go to 1000
         end if
         S1_ii => realmt(matr1)%S_ii

         irows1 => IM1%rowsii
         icols1 => IM1%colsii

         if ( debug ) then
            call prinin ( irows1, 10, 'irows1' )
            call prinin ( icols1, 10, 'icols1' )
            write(irefwr,1) 'sizes', length, size(S2_ii)
         end if  ! ( debug )

         IM2 => intmt(intmat2)

         irows2 => IM2%rowsii
         icols2 => IM2%colsii
         length = size(icols2)

         call sepcrmatrix ( matr2 )
         S2_ii => realmt(matr2)%S_ii

         if ( debug ) then
            call prinin ( irows2, 10, 'irows2' )
            call prinin ( icols2, 10, 'isols2' )
         end if  ! ( debug )

!        --- Fill first part of array MATR2

         call incpmt ( nrusol, irows1, icols1, irows2, icols2, jmethod1, &
                       S1_ii, S2_ii )

!        --- Copy second part of MATR1 into MATR2

         if ( nbound>0 ) then

!        --- Nbound>0, INTMT2 part 2
!            Decrease priority

            realmt(matr2)%S_ip = realmt(matr1)%S_ip
            realmt(matr2)%S_pi = realmt(matr1)%S_pi
            realmt(matr2)%S_pp = realmt(matr1)%S_pp

         end if

         if ( npboun>0 ) realmt(matr2)%S_ic = realmt(matr1)%S_ic

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmapcom'

      end if  ! ( debug )
      call erclos ( 'sepmapcom' )

      end subroutine sepmapcom
