      subroutine mshmetis ( nprocessors, epart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-03-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide a region into blocks using the metis subroutines
!
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     mesh
!     metis
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nprocessors
      integer, intent(out) :: epart(*)

!     epart          o    Array of size nelem containing the processor
!                         sequence numbers at output
!     nprocessors    i    Number of processors that is to be used for
!                         parallel processing
!                         Defines also the number of submeshes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: objval, ikelmc
      integer, allocatable :: kmeshcpnts(:), npart(:)
      integer, pointer :: vwgt(:) => null()
      integer, pointer :: vsize(:) => null()
      double precision, pointer :: tpwgts(:) => null()
      integer, pointer :: options(:) => null()

!     ikelmc         position in kmesh part c
!     kmeshcpnts     Array of size nelem+1
!                    Contains pointers to elements in kmeshc
!                    kmeshcpnts(1) = 0 and kmeshcpnts(nelem+1) contains
!                    length of kmeshc
!     npart          array of size npoint containing the processor sequence
!                    numbers for each node (not used in the rest)
!     objval         output parameter of METIS_PartMeshNodal
!     options        options array for METIS_PartMeshNodal
!                    at this moment only defaults are used
!     tpwgts         weight array for elements (default)
!     vsize          dummy
!     vwgt           weight array for nodes (default)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     METIS_PARTMESHNODAL  Calls metis subdivide algorithm
!     MSHMETIS1      Fill pointers to elements in kmeshcpnts and adapt kmeshc
!                    by decreasing by 1
!     MSHMETIS2      Reset kmeshc from c-style to fortran style
!                    Translate epart from c-numbering to fortran numbering
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmetis' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshmetis'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Allocate work arrays

      allocate ( kmeshcpnts(nelem+1), npart(npoint), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, nelem+1+npoint, 'kmeshcpnts' )
      if ( ierror/=0 ) go to 1000

!     --- Fill pointers to elements in kmeshcpnts and adapt kmeshc
!         by decreasing by 1 (c-style, i.e. numbering from 0)

      call mshmetis1 ( kmeshcpnts, ikelmc )

      if ( debug ) call prinin ( kmeshcpnts, nelem+1, 'kmeshcpnts' )

!     --- actual call of metis subroutine

      call METIS_PartMeshNodal ( nelem, npoint, kmeshcpnts, kmeshc, &
                                 vwgt, vsize, nprocessors, tpwgts, &
                                 options, objval, epart, npart )

!     --- Reset kmeshc to original value by adding 1 and change
!         epart from c-numbering to fortran numbering

      call mshmetis2 ( kmeshc, ikelmc, epart, nelem )

      if ( debug ) then
          call prinin ( epart, nelem, 'epart' )
          call prinin ( npart, npoint, 'npart' )
      end if  ! ( debug )

      deallocate ( kmeshcpnts, npart, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'kmeshcpnts' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshmetis'

      end if  ! ( debug )
      call erclos ( 'mshmetis' )

      end subroutine mshmetis
