      subroutine writbsbf ( ibuffr, buffer, ichois, inum, numarr, &
                            namevc, iarray, ihelp )
! ====================================================================== &
!
!        programmer    Niek Praagman
!        version  1.3  date 04-06-2014 Allow -102 in intmat(2)
!        version  1.2  date 21-11-2012 Long integers
!        version  1.1  date 18-03-2002 Reset narbuf
!
!   copyright (c) 1999-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     writbsbf writes arrays to  background storage
!
! **********************************************************************
!
!                       KEYWORDS
!
!     backing_storage
!     write
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cfile2f90'
      include 'SPcommon/cbuffrf90'
      include 'SPcommon/cdebugf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character *(*) namevc
      integer ibuffr(*), ichois, inum, numarr, iarray(*), ihelp(*)
      double precision buffer(*)

!     buffer    i/o   Buffer array
!     iarray     i    Array to be written on backing storage.
!                     IARRAY may be both a real or an integer array.
!                     SEPRAN arrays are always integer arrays.
!     ibuffr    i/o   Integer buffer array in which all large data is stored
!     ichois     i    Choice parameter. This parameter indicates whether the
!                     array is a SEPRAN array (ICHOIS = 0) or not (ICHOIS # 0)
!                     When IARRAY corresponds to a user array, the value of
!                     |ICHOIS| denotes the length of this array, and the sign of
!                     ICHOIS denotes whether it is a real array (ICHOIS<0) or an
!                     integer array (ICHOIS>0).
!     ihelp      i    integer extra array needed.
!                     Extra arrays necessary for SEPRAN arrays:
!                       IARRAY is KMESH:   IHELP is dummy
!                       IARRAY is KPROB:   IHELP is KMESH
!                       IARRAY is ISOL:    IHELP is KPROB
!                       IARRAY is INTMAT:  IHELP is KPROB
!                       IARRAY is MATR:    IHELP is dummy
!     inum       i    Choice parameter. Possibilities:
!                     When INUM<=0 the array is written on the next free
!                     positions on backing storage.
!                     At most 500 arrays can be written on the permanent file.
!                     When INUM = -1 the subroutine prints a text of the
!                     following form
!                     Array "NAME" is stored as array number "NUMARR"
!                     where "NAME" denotes the value of NAME and "NUMARR"
!                     denotes the value of NUMARR.
!                     When INUM>0  the array is written on positions of a
!                     preceding written array, with array number INUM.
!                     See also NUMARR.
!                     The length of the new and old array may be different.
!                     This possibility may also be used when there is no
!                     preceding array with this number.
!     namevc     i    Name of the array to be written to backing storage.
!                     Name is a character constant of at most 10 characters.
!                     NAME is only used for printing purposes, when INUM = -1.
!     numarr     o    Each array that is written by writbsbf gets a sequence
!                     number, the first array gets number 1, the second number 2
!                     etc.
!                     At output NUMARR gets the value of this sequence number.
!                     Therefore NUMARR must be a variable and not a constant.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) nbufol, ipwork
      integer jchois, narbufsav

!     ipwork         Starting address of work array of length NREC2
!                    This array is used as write buffer
!     jchois         Absolute value of ichois
!     narbufsav      Help variable to store the value of narbuf
!     nbufol         help variable to store the old value of nbuffr
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     INI069  Reserve space at the end of IBUFFR for a semi-permanent array
!     WRITB2  Write array KMESH
!     WRITB3  Write array KPROB
!     WRITB4  Write array solution type arrays
!     WRITB5  Write array INTMAT
!     WRITB6  Write array MATR
!     WRITB7  Write array integer user array
!     WRITB8  Write array real user array
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1242   Array IHELP is not of the type KPROB
!    1714   Array IHELP is not of the type KMESH
!    1715   The array to be written is of the wrong type
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'writbsbf' )

!     --- Reserve space for buffer array iwork

      narbufsav = narbuf
      call ini069 ( ibuffr, ipwork, nrec2, nbufol )

!     --- consider different possibilities

      jchois = abs(ichois)

      if ( ichois>0 ) then

!     --- ichois>0 , integer array of user

         call writb7 ( inum, jchois, numarr, namevc, iarray, &
                       ibuffr(ipwork) )

      else if ( ichois<0 ) then

!     --- ichois<0 , real array of user

         call writb8 ( inum, jchois, numarr, namevc, iarray, &
                       ibuffr(ipwork) )

      else if ( iarray(2)<100 .or. iarray(2)>120) then

!     --- error : iarray(2) has an undefined value

         call errint ( iarray(2), 1 )
         call errsub ( 1715, 1, 0, 0)

      else if ( iarray(2)==100 ) then

!     --- iarray(2) = 100 , iarray is of kmesh-type

         call writb2 ( ibuffr, inum, numarr, namevc, iarray, &
                       ibuffr(ipwork) )

      else if ( iarray(2)==101 ) then

!     --- iarray(2) = 101 , iarray is of kprob-type

         if ( ihelp(2)/=100 ) then

!        --- Array IHELP does not correspond to KMESH

            call errint ( ihelp(2), 1 )
            call errsub ( 1714, 1, 0, 0 )

         else

            call writb3 ( ibuffr, inum, numarr, namevc, iarray, ihelp, &
                          ibuffr(ipwork) )

         end if

      else if ( iarray(2)==110 .or. iarray(2)==120 .or. &
                iarray(2)>=115 .and. iarray(2)<=118 ) then

!     --- iarray(2) = 110, 115, 116, 117, 118, 120, iarray is of i-sol type

         if ( iarray(2)==110 .and. ihelp(2)/=101 ) then

!        --- Array IHELP does not correspond to KPROB

            call errint ( ihelp(2), 1 )
            call errsub ( 1242, 1, 0, 0 )

         else

            call writb4 ( ibuffr, buffer, inum, numarr, namevc, iarray, &
                          ibuffr(ipwork), ihelp )

         end if

      else if ( abs(iarray(2))==102 ) then

!     --- iarray(2) = 102 , iarray is of intmat-type

         if ( ihelp(2)/=101 ) then

!        --- Array IHELP does not correspond to kprob

            call errint ( ihelp(2), 1 )
            call errsub ( 1242, 1, 0, 0 )

         else

            call writb5 ( ibuffr, inum, numarr, namevc, iarray, ihelp, &
                          ibuffr(ipwork) )

         end if

      else if ( iarray(2)==103 ) then

!     --- iarray(2) = 103 , iarray is of matr-type

         if ( ihelp(2)/=101 ) then

!        --- Array IHELP does not correspond to KPROB

            call errint ( ihelp(2), 1 )
            call errsub ( 1242, 1, 0, 0 )

         else

            call writb6 ( ibuffr, inum, numarr, namevc, iarray, ihelp, &
                          ibuffr(ipwork) )

         end if

      end if

!     --- end:  reset nbuffr

      nbuffr = nbufol
      narbuf = narbufsav
      call erclos ( 'writbsbf' )
      end
